/*
  Test program for bitmap.
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "BitmapTest.hpp"

// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( BitmapTest );



void
BitmapTest::testInit0()
{
  const girl::POINT_TYPE minWidth = 1;
  const girl::POINT_TYPE maxWidth = 5;
  const girl::POINT_TYPE minHeight = 1;
  const girl::POINT_TYPE maxHeight = 5;

  for (girl::POINT_TYPE h = minHeight; h<=maxHeight; ++h) {
    for (girl::POINT_TYPE w = minWidth; w<=maxWidth; ++w) {
      girl::bitmap b(w, h);
      CPPUNIT_ASSERT(b.checkConsistency());
    }
  }
  
}

void
BitmapTest::testSwap0()
{
  girl::POINT_TYPE w1=5, h1=7;
  girl::POINT_TYPE w2=9, h2=13;

  girl::bitmap b1(w1, h1);
  girl::bitmap b2(w2, h2);
  girl::point p1(-1, -1);
  girl::point p2(w1-1, -1);
  girl::point p3(w1-1, h1-1);
  girl::point p4(-1, h1-1);
  girl::point p5(w1/2, h1/2);

  b1.setNode(p1);
  b1.setNode(p2);
  b1.setNode(p3);
  b1.setNode(p4);
  b1.setNode(p5);

  CPPUNIT_ASSERT(b1.width() == w1);
  CPPUNIT_ASSERT(b1.height() == h1);
  CPPUNIT_ASSERT(b2.width() == w2);
  CPPUNIT_ASSERT(b2.height() == h2);
  CPPUNIT_ASSERT(b1.isNode(p1));
  CPPUNIT_ASSERT(b1.isNode(p2));
  CPPUNIT_ASSERT(b1.isNode(p3));
  CPPUNIT_ASSERT(b1.isNode(p4));
  CPPUNIT_ASSERT(b1.isNode(p5));

  std::swap(b1, b2);

  CPPUNIT_ASSERT(b1.width() == w2);
  CPPUNIT_ASSERT(b1.height() == h2);
  CPPUNIT_ASSERT(b2.width() == w1);
  CPPUNIT_ASSERT(b2.height() == h1);
  CPPUNIT_ASSERT(b2.isNode(p1));
  CPPUNIT_ASSERT(b2.isNode(p2));
  CPPUNIT_ASSERT(b2.isNode(p3));
  CPPUNIT_ASSERT(b2.isNode(p4));
  CPPUNIT_ASSERT(b2.isNode(p5));
  CPPUNIT_ASSERT(! b1.isNode(p5));
}
