/*
  Test program for common boundary between regions computation.
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef CHECK_COMMONBOUNDARY_HPP
#define CHECK_COMMONBOUNDARY_HPP

#include <cppunit/extensions/HelperMacros.h>

#include <girl/girlToger.hpp>



class CommonBoundaryTest : public CppUnit::TestFixture
{
  CPPUNIT_TEST_SUITE( CommonBoundaryTest );

  CPPUNIT_TEST( testCommonBoundary1 );
  CPPUNIT_TEST( testCommonBoundary2 );
  CPPUNIT_TEST( testCommonBoundary3 );

  CPPUNIT_TEST( testChessboardCommonBoundary );
  CPPUNIT_TEST( testHorizontalCommonBoundary );
  CPPUNIT_TEST( testVerticalCommonBoundary );
  CPPUNIT_TEST( testOddsCommonBoundary );
  CPPUNIT_TEST( testConcentricCommonBoundary );
  CPPUNIT_TEST( testDiagonalCommonBoundary );

  CPPUNIT_TEST_SUITE_END();

public:
  void setUp();
  void tearDown();

  void testCommonBoundary1();
  void testCommonBoundary2();
  void testCommonBoundary3();

  void testChessboardCommonBoundary();
  void testHorizontalCommonBoundary();
  void testVerticalCommonBoundary();
  void testOddsCommonBoundary();
  void testConcentricCommonBoundary();
  void testDiagonalCommonBoundary();

protected:

  void testCommonBoundaryFunc(girl::toger::sameRegion &func, 
			      girl::POINT_TYPE width);
  
  bool checkCommonBoundary(const girl::toger &tog, 
			   girl::region r1, girl::region r2);

  bool checkPathBelongToContours(const girl::path &p,
				 const girl::contourDeque &contours);

  

};





#endif /* ! CHECK_COMMONBOUNDARY_HPP */
