/*
  Test program for closed contour class.
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "ContourTest.hpp"


// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( ContourTest );


void 
ContourTest::setUp()
{
}


void 
ContourTest::tearDown()
{
}

//closed contour, around one pixel, from left-up corner, clockwise.
// length == 4
static 
void
P_init1(std::deque<girl::direction> &dirs)
{
  dirs.clear();
  dirs.push_back(girl::RIGHT);
  dirs.push_back(girl::DOWN);
  dirs.push_back(girl::LEFT);
  dirs.push_back(girl::UP);
}

//closed contour, around one pixel, from left-up corner, counter-clockwise.
// length == 4
static 
void
P_init2(std::deque<girl::direction> &dirs)
{
  dirs.clear();
  dirs.push_back(girl::DOWN);
  dirs.push_back(girl::RIGHT);
  dirs.push_back(girl::UP);
  dirs.push_back(girl::LEFT);
}

//closed contour, cross around fiwe pixels, from left-up corner, clockwise.
// length == 12
static 
void
P_init3(std::deque<girl::direction> &dirs)
{
  dirs.clear();
  dirs.push_back(girl::RIGHT);
  dirs.push_back(girl::DOWN);
  dirs.push_back(girl::RIGHT);
  dirs.push_back(girl::DOWN);
  dirs.push_back(girl::LEFT);
  dirs.push_back(girl::DOWN);
  dirs.push_back(girl::LEFT);
  dirs.push_back(girl::UP);
  dirs.push_back(girl::LEFT);
  dirs.push_back(girl::UP);
  dirs.push_back(girl::RIGHT);
  dirs.push_back(girl::UP);
}

//closed contour, cross around fiwe pixels, from left-up corner, counter-clockwise.
// length == 12
static 
void
P_init4(std::deque<girl::direction> &dirs)
{
  dirs.clear();
  dirs.push_back(girl::DOWN);
  dirs.push_back(girl::LEFT);
  dirs.push_back(girl::DOWN);
  dirs.push_back(girl::RIGHT);
  dirs.push_back(girl::DOWN);
  dirs.push_back(girl::RIGHT);
  dirs.push_back(girl::UP);
  dirs.push_back(girl::RIGHT);
  dirs.push_back(girl::UP);
  dirs.push_back(girl::LEFT);
  dirs.push_back(girl::UP);
  dirs.push_back(girl::LEFT);
}

static const int LARGE = 200;

//closed contour, enclosing LARG*LARGE pixels, clockwise.
// length == 4*LARGE
static 
void
P_init5(std::deque<girl::direction> &dirs)
{
  dirs.clear();
  for (int i=0; i<LARGE; ++i) 
    dirs.push_back(girl::RIGHT);
  for (int i=0; i<LARGE; ++i) 
    dirs.push_back(girl::DOWN);
  for (int i=0; i<LARGE; ++i) 
    dirs.push_back(girl::LEFT);
  for (int i=0; i<LARGE; ++i) 
    dirs.push_back(girl::UP);
}

//closed contour, enclosing LARG*LARGE pixels, counter-clockwise.
// length == 4*LARGE
static 
void
P_init6(std::deque<girl::direction> &dirs)
{
  dirs.clear();
  for (int i=0; i<LARGE; ++i) 
    dirs.push_back(girl::DOWN);
  for (int i=0; i<LARGE; ++i) 
    dirs.push_back(girl::RIGHT);
  for (int i=0; i<LARGE; ++i) 
    dirs.push_back(girl::UP);
  for (int i=0; i<LARGE; ++i) 
    dirs.push_back(girl::LEFT);
}

static 
void
P_init7(std::deque<girl::direction> &dirs)
{
  dirs.clear();
  dirs.push_back(girl::RIGHT);
  dirs.push_back(girl::DOWN);
  dirs.push_back(girl::RIGHT);
  dirs.push_back(girl::UP);
  dirs.push_back(girl::LEFT);
  dirs.push_back(girl::UP);
  dirs.push_back(girl::LEFT);
  dirs.push_back(girl::DOWN);
}


void
ContourTest::testEqual0()
{
  //test equality between empty contours
  girl::contour c1;
  CPPUNIT_ASSERT(c == c1);
}

void
ContourTest::testEqual1()
{
  //test equality between contours initialized during construction and afterwards.

  P_init4(dirs);
  const girl::point startPt(0, 0);
  c.set(startPt, dirs);
  CPPUNIT_ASSERT(c.startPt() == startPt);

  girl::contour c1(girl::point(0, 0), dirs);
  CPPUNIT_ASSERT(c == c1);
  CPPUNIT_ASSERT(c.startPt() == c1.startPt());
  CPPUNIT_ASSERT(c.length() == c1.length());
}

void
ContourTest::testEqual2()
{
  //test equality after assignment

  P_init4(dirs);
  c.set(girl::point(0, 0), dirs);
  
  girl::contour c1 = c;
  CPPUNIT_ASSERT(c == c1);
}

void
ContourTest::testEqual3()
{
  //test equality after different assignments

  P_init4(dirs);
  const girl::point startPt(-1, 0);
  c.set(startPt, dirs);
  
  P_init5(dirs);
  girl::contour c1(startPt, dirs);

  CPPUNIT_ASSERT(c != c1);
  CPPUNIT_ASSERT(c.startPt() == c1.startPt());
}

static
void
P_testDirs(const girl::contour &c,
	   const std::deque<girl::direction> &dirs)
{
   //test that we have the same directions in c and in dirs.

  CPPUNIT_ASSERT(c.length() == dirs.size());

  std::deque<girl::direction>::const_iterator itD = dirs.begin();
  std::deque<girl::direction>::const_iterator itDEnd = dirs.end();

  girl::contour::const_dir_iterator it = c.begin();
  const girl::contour::const_dir_iterator itEnd = c.end();
  for ( ; it != itEnd; ++it) {
    CPPUNIT_ASSERT(itD != itDEnd);
    CPPUNIT_ASSERT(*it == *itD);
    ++itD;
  } 
}

void
ContourTest::testGet() const
{
  P_testDirs(c, dirs);
}

void
ContourTest::testSetGet0()
{
  //test what we set is what we get on empty contour
  testGet();
}

void
ContourTest::testSetGet1()
{
  //test what we set is what we get
  P_init1(dirs);
  c.set(girl::point(0, 0), dirs);
  testGet();
}

void
ContourTest::testSetGet2()
{
  //test what we set is what we get
  P_init1(dirs);
  c.set(girl::point(0, 0), dirs);
  testGet();
}

void
ContourTest::testSetGet3()
{
  //test what we set is what we get
  P_init5(dirs);
  c.set(girl::point(0, 0), dirs);
  testGet();
}

void 
ContourTest::testPositiveOrientation0()
{
  //test empty contour is positively oriented !
  CPPUNIT_ASSERT(c.isPositivelyOriented());
  
}

void 
ContourTest::testPositiveOrientation1()
{
  P_init1(dirs);
  c.set(girl::point(0, 0), dirs);
  CPPUNIT_ASSERT(c.isPositivelyOriented());

}

void 
ContourTest::testPositiveOrientation2()
{
  P_init2(dirs);
  c.set(girl::point(0, 0), dirs);
  CPPUNIT_ASSERT(! c.isPositivelyOriented());
  
}

void 
ContourTest::testPositiveOrientation3()
{
  P_init3(dirs);
  c.set(girl::point(0, 0), dirs);
  CPPUNIT_ASSERT(c.isPositivelyOriented());

}

void 
ContourTest::testPositiveOrientation4()
{
  P_init4(dirs);
  c.set(girl::point(0, 0), dirs);
  CPPUNIT_ASSERT(! c.isPositivelyOriented());
}

void 
ContourTest::testPositiveOrientation5()
{
  P_init5(dirs);
  c.set(girl::point(0, 0), dirs);
  CPPUNIT_ASSERT(c.isPositivelyOriented());
}

void 
ContourTest::testPositiveOrientation6()
{
  P_init7(dirs);
  c.set(girl::point(0, 0), dirs);
  CPPUNIT_ASSERT(! c.isPositivelyOriented());
}

void 
ContourTest::testLength0()
{
  //test empty contour is of length 0
  CPPUNIT_ASSERT(c.length() == 0);
}

void 
ContourTest::testLength1()
{
  P_init4(dirs);
  c.set(girl::point(0, 0), dirs);
  CPPUNIT_ASSERT(c.length() == 12);
}

void 
ContourTest::testLength2()
{
  //test "move" correctly changes start and end point for an empty path

  P_init6(dirs);
  c.set(girl::point(0, 0), dirs);
  CPPUNIT_ASSERT(c.length() == 4*LARGE);
}


void 
ContourTest::testMove1()
{
  //test "move" correctly changes start and end point for an empty contour

  const girl::point newStartPt(211, -1);
  c.move(newStartPt);
  CPPUNIT_ASSERT(c.startPt() == newStartPt);
}

void 
ContourTest::testMove2()
{
  //test "move" correctly changes start and end point for a non-empty contour

  P_init4(dirs);
  c.set(girl::point(1, -1), dirs);

  const girl::point newStartPt(211, -1);
  c.move(newStartPt);

  CPPUNIT_ASSERT(c.startPt() == newStartPt);
}


void
ContourTest::testInvert0()
{
  //test inversion is ok on empty contour
  girl::contour c1 = c;
  c1.invertOrientation();
  CPPUNIT_ASSERT(c == c1);
}

void
ContourTest::testInvert1()
{
  //test inversion is ok 
  P_init1(dirs);
  c.set(girl::point(0, 0), dirs);  
  c.invertOrientation();
  P_init2(dirs);
  testGet();
}

void
ContourTest::testInvert2()
{
  //test inversion is ok 
  P_init3(dirs);
  c.set(girl::point(0, 0), dirs);  
  c.invertOrientation();
  P_init4(dirs);
  testGet();
}

void
ContourTest::testInvert3()
{
  //test inversion is ok 
  P_init5(dirs);
  c.set(girl::point(0, 0), dirs);  
  c.invertOrientation();
  P_init6(dirs);
  testGet();
}

void
ContourTest::testInvert4()
{
  //test double inversion give initial contour
  P_init4(dirs);
  c.set(girl::point(0, 0), dirs);  
  c.invertOrientation();
  c.invertOrientation();
  testGet();
}

static
void
P_fill(std::vector<girl::contour> &vec, std::deque<girl::direction> &dirs)
{
  P_init4(dirs);
  vec[0].set(girl::point(0, 0), dirs);
}

void
ContourTest::testCopy()
{
  std::vector<girl::contour> vec;
  vec.push_back(girl::contour());
  P_fill(vec, dirs);
  c = vec[0];
  testGet();
}

static
void
P_fill2(std::vector<girl::contour> &vec, std::deque<girl::direction> &dirs)
{
  P_init4(dirs);
  vec[0].set(girl::point(0, 0), dirs);
  const size_t cap = vec.capacity()*2;
  for (size_t i=0; i<cap; ++i)
    vec.push_back(vec[0]);
}

void
ContourTest::testCopy2()
{
  std::vector<girl::contour> vec;
  vec.push_back(girl::contour());
  P_fill2(vec, dirs);
  c = vec[vec.size()-1];
  testGet();
}


static
void
P_testBelongs_aux(girl::contour &c,
		  std::deque<girl::direction> &dirs,
		  void (*initFunc)(std::deque<girl::direction> &dirs),
		girl::point startPt)
{
  dirs.clear();
  initFunc(dirs);
  girl::point pt = startPt;
  c.set(pt, dirs);  
  
  CPPUNIT_ASSERT(c.belongs(c.startPt()));
  CPPUNIT_ASSERT(c.belongs(c.endPt()));

  CPPUNIT_ASSERT(c.belongs(pt));
  std::deque<girl::direction>::const_iterator itD = dirs.begin();
  std::deque<girl::direction>::const_iterator itDEnd = dirs.end();
  for ( ; itD != itDEnd; ++itD) {
    pt.move(*itD);
    CPPUNIT_ASSERT(c.belongs(pt));
  }
}
		


void
ContourTest::testBelongs()
{
  P_testBelongs_aux(c, dirs, P_init4, girl::point(-1, -1));

  P_testBelongs_aux(c, dirs, P_init2, girl::point(1, -1));

  P_testBelongs_aux(c, dirs, P_init7, girl::point(123, 0));
}

void
ContourTest::testSwap1()
{
  //test swap method 
  // two calls to swap should lead to same contours
  // whatever the order of the calls.

  P_init1(dirs);
  c.set(girl::point(0, 0), dirs);

  std::deque<girl::direction> dirs2;
  girl::contour c2;
  P_init2(dirs2);
  c2.set(girl::point(7,-1), dirs2);

  c.swap(c2);
  c.swap(c2);

  P_testDirs(c, dirs);
  P_testDirs(c2, dirs2);

  c2.swap(c);
  c2.swap(c);

  P_testDirs(c, dirs);
  P_testDirs(c2, dirs2);
  
  c2.swap(c);
  c.swap(c2);

  P_testDirs(c, dirs);
  P_testDirs(c2, dirs2);
  
  c.swap(c2);
  c2.swap(c);

  P_testDirs(c, dirs);
  P_testDirs(c2, dirs2);
}

void
ContourTest::testSwap2()
{
  //test swap method 
  // similar to testSwap1 but using std::swap instead.

  P_init1(dirs);
  c.set(girl::point(0, 0), dirs);

  std::deque<girl::direction> dirs2;
  girl::contour c2;
  P_init2(dirs2);
  c2.set(girl::point(7,-1), dirs2);

  std::swap(c, c2);
  std::swap(c, c2);

  P_testDirs(c, dirs);
  P_testDirs(c2, dirs2);

  std::swap(c2, c);
  std::swap(c2, c);

  P_testDirs(c, dirs);
  P_testDirs(c2, dirs2);
  
  std::swap(c2, c);
  std::swap(c, c2);

  P_testDirs(c, dirs);
  P_testDirs(c2, dirs2);
  
  std::swap(c, c2);
  std::swap(c2, c);

  P_testDirs(c, dirs);
  P_testDirs(c2, dirs2);
}
