/*
  Test program for crop of toger.
  Copyright (C) 2010 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#include "CropTest.hpp"

// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( CropTest );

#include "same_regions.hpp"

#include <iostream>//DEBUG

void 
CropTest::setUp()
{
}


void 
CropTest::tearDown()
{

}

static
void
testCropEmptyToger_aux(girl::POINT_TYPE togerWidth, girl::POINT_TYPE togerHeight,
		       const girl::point &cropStartPt,
		       girl::POINT_TYPE newWidth, girl::POINT_TYPE newHeight)
{

  girl::toger tog(togerWidth, togerHeight);
  
  girl::point p2(cropStartPt.x()+newWidth-1, cropStartPt.y()+newHeight-1);

  tog.crop(cropStartPt, p2);

  CPPUNIT_ASSERT(tog.width() == newWidth);
  CPPUNIT_ASSERT(tog.height() == newHeight);  
  CPPUNIT_ASSERT(tog.checkConsistency());  

}


void
CropTest::testCrop1()
{
  testCropEmptyToger_aux(11, 9,
			 girl::point(2, 2),
			 7, 5);
}

//idem testCrop1 except that cropStartPt.x() is now odd.
void
CropTest::testCrop1b() 
{
  testCropEmptyToger_aux(11, 9,
			 girl::point(3, 2),
			 7, 5);
}

void
CropTest::testCrop2()
{
  //crop only one dimension : only height is reduced

  testCropEmptyToger_aux(11, 9,
			 girl::point(0, 2),
			 11, 5);
}

void
CropTest::testCrop2c()
{
  //crop only one dimension : only height is reduced
  //crop from up border (from point (0, 0))

  testCropEmptyToger_aux(11, 9,
			 girl::point(0, 0),
			 11, 5);
}

void
CropTest::testCrop2d()
{
  //crop only one dimension : only height is reduced
  //crop from down border 

  testCropEmptyToger_aux(11, 9,
			 girl::point(0, 4),
			 11, 5);
}

void
CropTest::testCrop3()
{
  //crop only one dimension : only width is reduced

  testCropEmptyToger_aux(11, 9,
			 girl::point(2, 0),
			 7, 9);
}

void
CropTest::testCrop3b()
{
  //crop only one dimension : only width is reduced
  //idem testCrop3 except that cropStartPt.x() is now odd.

  testCropEmptyToger_aux(11, 9,
			 girl::point(3, 0),
			 7, 9);
}

void
CropTest::testCrop3c()
{
  //crop only one dimension : only width is reduced
  //crop from up border (from point (0, 0))

  testCropEmptyToger_aux(11, 9,
			 girl::point(0, 0),
			 7, 9);
}

void
CropTest::testCrop3d()
{
  //crop only one dimension : only width is reduced
  //crop from down border

  testCropEmptyToger_aux(11, 9,
			 girl::point(4, 0),
			 7, 9);
}



static
void
P_initRectangle(girl::POINT_TYPE width, girl::POINT_TYPE height, 
		std::deque<girl::direction> &dirs)
{
  dirs.clear();
  for (girl::POINT_TYPE i=0; i<width; ++i) 
    dirs.push_back(girl::RIGHT);
  for (girl::POINT_TYPE i=0; i<height; ++i) 
    dirs.push_back(girl::DOWN);
  for (girl::POINT_TYPE i=0; i<width; ++i) 
    dirs.push_back(girl::LEFT);
  for (girl::POINT_TYPE i=0; i<height; ++i) 
    dirs.push_back(girl::UP);

}

static
void
P_testConcentricCrops(girl::toger &tog, int inc = 1)
{
  girl::point p1(inc, inc);
  girl::point p2(tog.width()-1-inc, tog.height()-1-inc);

  const size_t s = (std::min(tog.width(), tog.height()) -1)/(inc*2);

  for (size_t i=0; i<s; ++i) {

    CPPUNIT_ASSERT(p1.x() <= p2.x());
    CPPUNIT_ASSERT(p1.y() <= p2.y());
    CPPUNIT_ASSERT(p2.x() <= tog.width()-1);
    CPPUNIT_ASSERT(p2.y() <= tog.height()-1);
   
    tog.crop(p1, p2);
    
    p2 = girl::point(tog.width()-1-inc, tog.height()-1-inc);

    CPPUNIT_ASSERT(tog.checkConsistency());  

  }

}

void
CropTest::testCropRectangles0()
{
  //- initialization : 1 inner rectangle
  // test crop on toger with one infinite face in the middle

  const girl::POINT_TYPE togerWidth = 14;
  const girl::POINT_TYPE togerHeight = togerWidth;

  girl::toger tog(togerWidth, togerHeight);

  const girl::POINT_TYPE rectWidth = 4;
  const girl::POINT_TYPE rectHeight = rectWidth;

  std::deque<girl::direction> dirs;
  P_initRectangle(rectWidth, rectHeight, dirs);
  
  const girl::point pt(3, 3);

  girl::path pth(pt, dirs);
    
  girl::pathDeque paths;
  paths.push_back(pth);
  
  girl::regionDeque newRegions;
  girl::regionDeque updatedRegions;
  const bool res = tog.insertPaths(paths, newRegions, updatedRegions);
  
  CPPUNIT_ASSERT(res);
  
  //-test crops
  P_testConcentricCrops(tog, 1);

}



void
CropTest::testCropRectangles1()
{
  //- initialization : 4 inner rectangles + 1 rectangle intersecting the 4 others. 

  const girl::POINT_TYPE togerWidth = 14;
  const girl::POINT_TYPE togerHeight = togerWidth;

  girl::toger tog(togerWidth, togerHeight);

  const girl::POINT_TYPE rectWidth = 4;
  const girl::POINT_TYPE rectHeight = rectWidth;

  std::deque<girl::direction> dirs;
  P_initRectangle(rectWidth, rectHeight, dirs);
  
  typedef std::deque<girl::point> PtDeque;
  PtDeque rectStartPts;
  rectStartPts.push_back(girl::point(1, 1));
  rectStartPts.push_back(girl::point(7, 1));
  rectStartPts.push_back(girl::point(3, 3));
  rectStartPts.push_back(girl::point(1, 7));
  rectStartPts.push_back(girl::point(7, 7));

  for (PtDeque::const_iterator it=rectStartPts.begin();
       it != rectStartPts.end(); ++it) {
  
    const girl::point &pt = *it;

    girl::path pth(pt, dirs);
    
    girl::pathDeque paths;
    paths.push_back(pth);
    
    girl::regionDeque newRegions;
    girl::regionDeque updatedRegions;
    const bool res = tog.insertPaths(paths, newRegions, updatedRegions);
    
    CPPUNIT_ASSERT(res);
  }

  girl::toger tog2;

  //-test crops
  tog2 = tog;
  P_testConcentricCrops(tog2, 1);

  tog2 = tog;
  P_testConcentricCrops(tog2, 3);
}

void
CropTest::testCropRectangles1b()
{
  //- initialization : 4 inner rectangles + 1 rectangle intersecting the 4 others. 
  // same as testCropRectangles1(), except that inner rectangle is not at the same position.

  const girl::POINT_TYPE togerWidth = 14;
  const girl::POINT_TYPE togerHeight = togerWidth;

  girl::toger tog(togerWidth, togerHeight);

  const girl::POINT_TYPE rectWidth = 4;
  const girl::POINT_TYPE rectHeight = rectWidth;

  std::deque<girl::direction> dirs;
  P_initRectangle(rectWidth, rectHeight, dirs);
  
  typedef std::deque<girl::point> PtDeque;
  PtDeque rectStartPts;
  rectStartPts.push_back(girl::point(1, 1));
  rectStartPts.push_back(girl::point(7, 1));
  rectStartPts.push_back(girl::point(4, 4));
  rectStartPts.push_back(girl::point(1, 7));
  rectStartPts.push_back(girl::point(7, 7));

  for (PtDeque::const_iterator it=rectStartPts.begin();
       it != rectStartPts.end(); ++it) {
  
    const girl::point &pt = *it;

    girl::path pth(pt, dirs);
    
    girl::pathDeque paths;
    paths.push_back(pth);
    
    girl::regionDeque newRegions;
    girl::regionDeque updatedRegions;
    const bool res = tog.insertPaths(paths, newRegions, updatedRegions);
    
    CPPUNIT_ASSERT(res);
  }

  girl::toger tog2;

  //-test crops
  tog2 = tog;
  P_testConcentricCrops(tog2, 1);

  tog2 = tog;
  P_testConcentricCrops(tog2, 3);
}

#define DEFAULT_WIDTH 61

static
void
P_testCropRectanglesF(girl::toger::sameRegion &f)
{
  const girl::POINT_TYPE width = DEFAULT_WIDTH;
  const girl::POINT_TYPE height = DEFAULT_WIDTH;
  
  girl::toger tog(width, height);
  
  girl::regionDeque regs;
  tog.splitRegion(1, f, regs);
  
  girl::toger tog2 = tog;

  //-test crops
  P_testConcentricCrops(tog2, 1);

  // we test also smaller crops to have cases where we remove infinite faces.

  tog2 = tog;
  P_testConcentricCrops(tog2, width/7);
  
  tog2 = tog;
  P_testConcentricCrops(tog2, width/3);
}

void
CropTest::testCropRectanglesChessboard()
{
  sameRegion_chessboard f;
  P_testCropRectanglesF(f);
}

void
CropTest::testCropRectanglesHorizontal()
{
  sameRegion_horizontal f;
  P_testCropRectanglesF(f);
}

void
CropTest::testCropRectanglesVertical()
{
  sameRegion_vertical f;
  P_testCropRectanglesF(f);
}

void
CropTest::testCropRectanglesOdds()
{
  sameRegion_odds f(DEFAULT_WIDTH, DEFAULT_WIDTH);
  P_testCropRectanglesF(f);
}

void
CropTest::testCropRectanglesConcentric()
{
  sameRegion_concentric f(DEFAULT_WIDTH, DEFAULT_WIDTH);
  P_testCropRectanglesF(f);
}
void
CropTest::testCropRectanglesDiagonals()
{
  sameRegion_diagonals f;
  P_testCropRectanglesF(f);
}
