/*
  Test program for direction class.
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "DirectionTest.hpp"

// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( DirectionTest );

void 
DirectionTest::setUp()
{
}


void 
DirectionTest::tearDown()
{
}

void 
DirectionTest::testDoubleOpposite()
{
  for (int i=0; i<4; ++i) {
    girl::direction d( static_cast<girl::eDirection>(i) );
    CPPUNIT_ASSERT(d == d.opposite().opposite());
  }
}

void 
DirectionTest::testNextPrevious()
{
  for (int i=0; i<4; ++i) {
    girl::direction d( static_cast<girl::eDirection>(i) );
    CPPUNIT_ASSERT(d.next().previous() == d);
    CPPUNIT_ASSERT(d.previous().next() == d);
  }
}

void
DirectionTest::testCycleNext()
{
  for (int i=0; i<4; ++i) {
    girl::direction d( static_cast<girl::eDirection>(i) );
    girl::direction d2 = d;
    for (int j=0; j<4; ++j)
      d2 = d2.next();
    CPPUNIT_ASSERT(d2 == d);
  }
}

void
DirectionTest::testCyclePrevious()
{
  for (int i=0; i<4; ++i) {
    girl::direction d( static_cast<girl::eDirection>(i) );
    girl::direction d2 = d;
    for (int j=0; j<4; ++j)
      d2 = d2.previous();
    CPPUNIT_ASSERT(d2 == d);
  }
}

void
DirectionTest::testIsVerticalHorizontal()
{

  const girl::direction du = girl::UP;
  CPPUNIT_ASSERT(du.isVertical());

  const girl::direction dd = girl::DOWN;
  CPPUNIT_ASSERT(dd.isVertical());
  
  const girl::direction dr = girl::RIGHT;
  CPPUNIT_ASSERT(dr.isHorizontal());

  const girl::direction dl = girl::LEFT;
  CPPUNIT_ASSERT(dl.isHorizontal());
}
