/*
  Test program for some empty toger properties.
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#include "EmptyTogerTest.hpp"

// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( EmptyTogerTest );

#include <cstdlib> //rand
#include <iostream> //DEBUG

#define DEFAULT_WIDTH 103
#define DEFAULT_HEIGHT 97

/**
 * gives a random value in [min; max]
 *
 * @warning call srandom once before this function.
 */
template <class T>
T
P_bounded_rand(T min, T max)
{
  return static_cast<T>(min + (static_cast<float>( rand() ) / RAND_MAX * (max - min + 1) ));
}


void 
EmptyTogerTest::setUp()
{
}


void 
EmptyTogerTest::tearDown()
{
}

void 
EmptyTogerTest::testConstructor1()
{
  const girl::POINT_TYPE w = DEFAULT_WIDTH;
  const girl::POINT_TYPE h = DEFAULT_HEIGHT;
  
  girl::toger tog(w, h);

  CPPUNIT_ASSERT(tog.checkConsistency());
  CPPUNIT_ASSERT(tog.width() == w);
  CPPUNIT_ASSERT(tog.height() == h);

  girl::regionDeque regs;
  tog.allRegions(regs);
  CPPUNIT_ASSERT(regs.size() == 1);
}

void 
EmptyTogerTest::testBelongsTo1()
{
  const girl::POINT_TYPE w = DEFAULT_WIDTH;
  const girl::POINT_TYPE h = DEFAULT_HEIGHT;
  
  girl::toger tog(w, h);

  const girl::point p(0, 0);
  CPPUNIT_ASSERT(tog.belongsTo(p, 1));

  const girl::point p1(w-1, 0);
  CPPUNIT_ASSERT(tog.belongsTo(p1, 1));

  const girl::point p2(w-1, h-1);
  CPPUNIT_ASSERT(tog.belongsTo(p2, 1));

  const girl::point p3(0, h-1);
  CPPUNIT_ASSERT(tog.belongsTo(p3, 1));
}

void 
EmptyTogerTest::testBelongsTo2()
{
  girl::toger tog(DEFAULT_WIDTH, DEFAULT_WIDTH);
  
  const girl::point p(P_bounded_rand(0, DEFAULT_WIDTH-1), 
		      P_bounded_rand(0, DEFAULT_WIDTH-1));

  CPPUNIT_ASSERT(p.x()>=0);
  CPPUNIT_ASSERT(p.x()<=DEFAULT_WIDTH-1);
  CPPUNIT_ASSERT(p.y()>=0);
  CPPUNIT_ASSERT(p.y()<=DEFAULT_WIDTH-1);

  CPPUNIT_ASSERT(tog.belongsTo(p, 1));
}

void 
EmptyTogerTest::testBelongsTo3()
{
  girl::toger tog(DEFAULT_WIDTH, DEFAULT_WIDTH);
  
  const girl::point p = tog.anyPoint(1);

  CPPUNIT_ASSERT(tog.belongsTo(p, 1));
}


void 
EmptyTogerTest::testFindRegion1()
{
  const girl::POINT_TYPE w = DEFAULT_WIDTH;
  const girl::POINT_TYPE h = DEFAULT_HEIGHT;
  
  girl::toger tog(w, h);

  const girl::point p(0, 0);
  CPPUNIT_ASSERT(tog.findRegion(p) == 1);

  const girl::point p2(w-1, 0);
  CPPUNIT_ASSERT(tog.findRegion(p2) == 1);

  const girl::point p3(w-1, h-1);
  CPPUNIT_ASSERT(tog.findRegion(p3) == 1);

  const girl::point p4(0, h-1);
  CPPUNIT_ASSERT(tog.findRegion(p4) == 1);

}

void 
EmptyTogerTest::testFindRegion1b()
{
  const girl::POINT_TYPE w = 9;
  const girl::POINT_TYPE h = 11;
  
  girl::toger tog(w, h);


  for (girl::POINT_TYPE y=0; y<h; ++y) {
    for (girl::POINT_TYPE x=0; x<w; ++x) {

      const girl::point p(x, y);
      CPPUNIT_ASSERT(tog.findRegion(p) == 1);
    }
  }
}

void 
EmptyTogerTest::testFindRegion2() 
{
  girl::toger tog(DEFAULT_WIDTH, DEFAULT_WIDTH);
  
  const girl::point p(P_bounded_rand(0, DEFAULT_WIDTH-1), 
		      P_bounded_rand(0, DEFAULT_WIDTH-1));

  CPPUNIT_ASSERT(tog.findRegion(p) == 1);  
}

void 
EmptyTogerTest::testFindRegion3() 
{
  girl::toger tog(DEFAULT_WIDTH, DEFAULT_WIDTH);
  
  const girl::point p = tog.anyPoint(1);

  CPPUNIT_ASSERT(tog.findRegion(p) == 1);  
}

void 
EmptyTogerTest::testFindRegions1()
{
  const girl::POINT_TYPE w = 9;
  const girl::POINT_TYPE h = 11;
  
  girl::toger tog(w, h);

  const girl::point p1(0, 0);
  const girl::point p2(w-1, h-1);
  girl::regionDeque regions;

  regions.clear();
  tog.findRegions(p1, p2, regions);
  CPPUNIT_ASSERT(regions.size() == 1);  
  CPPUNIT_ASSERT(regions.back() == 1);
  
  regions.clear();
  tog.findRegions(p1, p1, regions);
  CPPUNIT_ASSERT(regions.size() == 1);  
  CPPUNIT_ASSERT(regions.back() == 1);

  regions.clear();
  tog.findRegions(p2, p2, regions);
  CPPUNIT_ASSERT(regions.size() == 1);  
  CPPUNIT_ASSERT(regions.back() == 1);
}

void 
EmptyTogerTest::testFindRegions2()
{
  const girl::POINT_TYPE w = DEFAULT_WIDTH;
  const girl::POINT_TYPE h = DEFAULT_HEIGHT;
  CPPUNIT_ASSERT(w > 1);
  CPPUNIT_ASSERT(h > 1);

  girl::toger tog(w, h);

  girl::point p1, p2;
  girl::regionDeque regions;


  p1.set(0, 0);
  p2.set(1, 1);
  regions.clear();
  tog.findRegions(p1, p2, regions);
  CPPUNIT_ASSERT(regions.size() == 1);  
  CPPUNIT_ASSERT(regions.back() == 1);  

  p1.set(w-2, 0);
  p2.set(w-1, 1);
  regions.clear();
  tog.findRegions(p1, p2, regions);
  CPPUNIT_ASSERT(regions.size() == 1);  
  CPPUNIT_ASSERT(regions.back() == 1);  

  p2.set(w-2, h-2);
  p2.set(w-1, h-1);
  regions.clear();
  tog.findRegions(p1, p2, regions);
  CPPUNIT_ASSERT(regions.size() == 1);  
  CPPUNIT_ASSERT(regions.back() == 1);  

  p1.set(0, h-2);
  p2.set(1, h-1);
  regions.clear();
  tog.findRegions(p1, p2, regions);
  CPPUNIT_ASSERT(regions.size() == 1);  
  CPPUNIT_ASSERT(regions.back() == 1);  
}


void 
EmptyTogerTest::testAreNeighbours()
{
  //FALSE !!! -1 is not a region !!!

  girl::toger tog(DEFAULT_WIDTH, DEFAULT_WIDTH);
  
#ifndef NDEBUG
  std::cerr<<"1 & -1 are DIRECT neighbours"<<tog.areNeighbours(1, -1, girl::toger::NEIGHBOUR_DIRECT, girl::toger::MARK_NONE)<<std::endl;
  std::cerr<<"1 & -1 are INNER neighbours"<<tog.areNeighbours(1, -1, girl::toger::NEIGHBOUR_INNER, girl::toger::MARK_NONE)<<std::endl;
  std::cerr<<"1 & -1 are OUTER neighbours"<<tog.areNeighbours(1, -1, girl::toger::NEIGHBOUR_OUTER, girl::toger::MARK_NONE)<<std::endl;
#endif

  CPPUNIT_ASSERT(! tog.areNeighbours(1, -1, girl::toger::NEIGHBOUR_ANY, girl::toger::MARK_NONE));
}

void
EmptyTogerTest::testFindDart()
{
    girl::toger tog(DEFAULT_WIDTH, DEFAULT_WIDTH);

    const girl::point p(P_bounded_rand(0, DEFAULT_WIDTH-1), 
			P_bounded_rand(0, DEFAULT_WIDTH-1));
    
    
    CPPUNIT_ASSERT(tog.lambda(tog.findDart(p)) == 1);  
}

void
EmptyTogerTest::testRegionNeighbourhood()
{
  girl::toger tog(DEFAULT_WIDTH, DEFAULT_WIDTH);

  girl::regionDeque regs;
  tog.regionNeighbourhood(1, girl::toger::NEIGHBOUR_ANY, girl::toger::MARK_NONE, regs);
  CPPUNIT_ASSERT(regs.size() == 0);
}

void
EmptyTogerTest::testAnyRegionNeighbour()
{
  girl::toger tog(DEFAULT_WIDTH, DEFAULT_WIDTH);

  const girl::region r = tog.anyRegionNeighbour(1, girl::toger::NEIGHBOUR_ANY, girl::toger::MARK_NONE);
  CPPUNIT_ASSERT(r == 0);
}

void
EmptyTogerTest::testAllRegions()
{
  girl::toger tog(DEFAULT_WIDTH, DEFAULT_WIDTH);

  girl::regionDeque regs;
  tog.allRegions(regs);
  CPPUNIT_ASSERT(regs.size() == 1);
  CPPUNIT_ASSERT(regs.back() == 1);
  CPPUNIT_ASSERT(regs.size() == tog.numRegions());
}

void
EmptyTogerTest::testInnerRegions()
{
    girl::toger tog(DEFAULT_WIDTH, DEFAULT_WIDTH);

    girl::regionDeque regs;
    tog.innerRegions(1, regs);
    CPPUNIT_ASSERT(regs.size() == 0);
}

void
EmptyTogerTest::testFirstChild()
{
    girl::toger tog(DEFAULT_WIDTH, DEFAULT_WIDTH);

    girl::region c = tog.firstChild(1);
    CPPUNIT_ASSERT(c == 0);
}

void
EmptyTogerTest::testParent()
{
    girl::toger tog(DEFAULT_WIDTH, DEFAULT_WIDTH);

    const girl::face p = tog.parent(1);

    CPPUNIT_ASSERT(p == 0);
}

void
EmptyTogerTest::testArtificialNode1()
{
  girl::toger tog(DEFAULT_WIDTH, DEFAULT_WIDTH);

  girl::dart d0 = tog.beta(1);
  CPPUNIT_ASSERT(tog.phi(d0) == d0);
}

void
EmptyTogerTest::testArtificialNode2()
{
  girl::toger tog(DEFAULT_WIDTH, DEFAULT_WIDTH);

  girl::dart d0 = tog.beta(-1);
  CPPUNIT_ASSERT(tog.phi(d0) == d0);
}

void
EmptyTogerTest::testArtificialNode3()
{
  girl::toger tog(DEFAULT_WIDTH, DEFAULT_WIDTH);

  girl::dart d0 = tog.beta(1);
  CPPUNIT_ASSERT(tog.sigma(tog.sigma(d0)) == d0);
}

void
EmptyTogerTest::testArtificialNode4()
{
  girl::toger tog(DEFAULT_WIDTH, DEFAULT_WIDTH);

  girl::dart d0 = tog.beta(-1);
  CPPUNIT_ASSERT(tog.sigma(tog.sigma(d0)) == d0);
}

void
EmptyTogerTest::testNoMark()
{
  girl::toger tog(DEFAULT_WIDTH, DEFAULT_WIDTH);

  CPPUNIT_ASSERT(! tog.isRegionMarked(1, girl::toger::MARK_ANY));
  CPPUNIT_ASSERT(! tog.isRegionOuterBoundaryMarked(1, girl::toger::MARK_ANY));

  CPPUNIT_ASSERT(! tog.isFaceMarked(-1, girl::toger::MARK_ANY));
}

void
EmptyTogerTest::testRegionDomain()
{
  const girl::POINT_TYPE w = DEFAULT_WIDTH;
  const girl::POINT_TYPE h = DEFAULT_HEIGHT;
  
  girl::toger tog(w, h);

  std::vector<girl::point> scan;
  tog.regionDomain(1, scan);

  size_t area = 0;
  for (size_t i=0; i<scan.size(); i+=2) {
    CPPUNIT_ASSERT(scan[i].y() == scan[i+1].y());
    area += scan[i+1].x()-scan[i].x()+1;
  }

  const size_t AREA = w*h;
  
  CPPUNIT_ASSERT(area == AREA);
}

void
EmptyTogerTest::testRegionBoundary()
{
  const girl::POINT_TYPE w = DEFAULT_WIDTH;
  const girl::POINT_TYPE h = DEFAULT_HEIGHT;
  
  girl::toger tog(w, h);
  
  girl::contourDeque cd;
  tog.regionBoundary(1, cd);
  
  CPPUNIT_ASSERT(cd.size() == 1);
  
  girl::contour c;
  tog.regionOuterBoundary(1, c);
  
  CPPUNIT_ASSERT(cd.back() == c);
  
  const size_t perimeter = 2*w + 2*h;
  
  CPPUNIT_ASSERT(c.length() == perimeter);
}

void
EmptyTogerTest::testDartSegment()
{
  const girl::POINT_TYPE w = DEFAULT_WIDTH;
  const girl::POINT_TYPE h = DEFAULT_HEIGHT;
  
  girl::toger tog(w, h);
  
  girl::dart cd = tog.beta(1);
  girl::dart acd = tog.alpha(cd);
  
  girl::path p;
  tog.segment(cd, p);
  
  CPPUNIT_ASSERT(p.length() == tog.segmentLength(cd));
  
  girl::path pa;
  tog.segment(acd, pa);
  
  CPPUNIT_ASSERT(pa.length() == tog.segmentLength(acd));
  
  CPPUNIT_ASSERT(p.length() == pa.length());
  
  const size_t perimeter = 2*w + 2*h;
  
  CPPUNIT_ASSERT(p.length() == perimeter);
  
  pa.invertOrientation();
  
  CPPUNIT_ASSERT(p == pa);
}


class AreaComputer : public girl::toger::pointIterator
{
public:
  AreaComputer()
    : girl::toger::pointIterator(),
      m_area(0)
  {
  }

  virtual void processPoint(const girl::point &)
  {
    ++m_area;
  }

  size_t area()
  {
    return m_area;
  }

protected:
  size_t m_area;
};


void
EmptyTogerTest::testIterateOnDomain()
{
  const girl::POINT_TYPE w = DEFAULT_WIDTH;
  const girl::POINT_TYPE h = DEFAULT_HEIGHT;
  
  girl::toger tog(w, h);

  AreaComputer ac;

  tog.iterateOnRegionDomain(1, ac);

  const size_t area = w*h;

  CPPUNIT_ASSERT(area == ac.area());
}

class PerimeterComputer : public girl::toger::directionIterator
{
public:
  PerimeterComputer()
    : girl::toger::directionIterator(),
      m_perimeter(0)
  {
  }

  virtual void processDirection(girl::direction)
  {
    ++m_perimeter;
  }

  size_t perimeter()
  {
    return m_perimeter;
  }

protected:
  size_t m_perimeter;
};


void
EmptyTogerTest::testIterateOnBoundary()
{
  const girl::POINT_TYPE w = DEFAULT_WIDTH;
  const girl::POINT_TYPE h = DEFAULT_HEIGHT;
  
  girl::toger tog(w, h);

  PerimeterComputer pc;

  tog.iterateOnFaceBoundary(1, pc);

  const size_t perimeter = 2*w + 2*h;

  CPPUNIT_ASSERT(perimeter == pc.perimeter());
}


class SegmentLengthComputer : public girl::toger::directionIterator
{
public:
  SegmentLengthComputer()
    : girl::toger::directionIterator(),
      m_length(0)
  {
  }


  virtual void processDirection(girl::direction)
  {
    ++m_length;
  }


  size_t length()
  {
    return m_length;
  }

protected:
  size_t m_length;
};

void
EmptyTogerTest::testIterateOnSegment()
{
  const girl::POINT_TYPE w = DEFAULT_WIDTH;
  const girl::POINT_TYPE h = DEFAULT_HEIGHT;
  
  girl::toger tog(w, h);

  girl::dart cd = tog.beta(1);

  SegmentLengthComputer slc;

  tog.iterateOnSegment(cd, slc);

  const size_t perimeter = 2*w + 2*h;

  CPPUNIT_ASSERT(perimeter == slc.length());
  
  CPPUNIT_ASSERT(slc.length() == tog.segmentLength(cd));
}


void
EmptyTogerTest::testShrink()
{
  const girl::POINT_TYPE w = DEFAULT_WIDTH;
  const girl::POINT_TYPE h = DEFAULT_HEIGHT;
  
  girl::toger tog(w, h);

  tog.shrink();

  CPPUNIT_ASSERT(tog.checkConsistency());
  
  girl::regionDeque regs;
  tog.allRegions(regs);

  CPPUNIT_ASSERT(regs.size() == 1);
 
}

void
EmptyTogerTest::testSwap()
{
  girl::POINT_TYPE w1=5, h1=7;
  girl::POINT_TYPE w2=9, h2=13;

  girl::toger t1(w1, h1);
  girl::toger t2(w2, h2);
  CPPUNIT_ASSERT(t1.width() == w1);
  CPPUNIT_ASSERT(t1.height() == h1);
  CPPUNIT_ASSERT(t2.width() == w2);
  CPPUNIT_ASSERT(t2.height() == h2);
  
  std::swap(t1, t2);

  CPPUNIT_ASSERT(t1.width() == w2);
  CPPUNIT_ASSERT(t1.height() == h2);
  CPPUNIT_ASSERT(t2.width() == w1);
  CPPUNIT_ASSERT(t2.height() == h1);  
}


void
EmptyTogerTest::testPimpl1a()
{
  const girl::POINT_TYPE w = DEFAULT_WIDTH;
  const girl::POINT_TYPE h = DEFAULT_HEIGHT;
  
  girl::toger tog(w, h);

  girl::toger tog2 = tog;

  tog.xMarkRegionBoundary(1, girl::toger::MARK_ANY);
  CPPUNIT_ASSERT(tog.isRegionMarked(1, girl::toger::MARK_ANY));
  CPPUNIT_ASSERT(! tog2.isRegionMarked(1, girl::toger::MARK_ANY));
}

void
EmptyTogerTest::testPimpl1b()
{
  const girl::POINT_TYPE w = DEFAULT_WIDTH;
  const girl::POINT_TYPE h = DEFAULT_HEIGHT;
  
  girl::toger tog(w, h);

  girl::toger tog2 = tog;

  tog2.xMarkRegionBoundary(1, girl::toger::MARK_ANY);
  CPPUNIT_ASSERT(! tog.isRegionMarked(1, girl::toger::MARK_ANY));
  CPPUNIT_ASSERT(tog2.isRegionMarked(1, girl::toger::MARK_ANY));
}

void
EmptyTogerTest::testPimpl2a()
{
  const girl::POINT_TYPE w = DEFAULT_WIDTH;
  const girl::POINT_TYPE h = DEFAULT_HEIGHT;
  
  girl::toger tog(w, h);

  girl::toger tog2 = tog;

  tog.xMarkOuterBoundary(1, girl::toger::MARK_ANY);
  CPPUNIT_ASSERT(tog.isRegionOuterBoundaryMarked(1, girl::toger::MARK_ANY));
  CPPUNIT_ASSERT(! tog2.isRegionOuterBoundaryMarked(1, girl::toger::MARK_ANY));
}

void
EmptyTogerTest::testPimpl2b()
{
  const girl::POINT_TYPE w = DEFAULT_WIDTH;
  const girl::POINT_TYPE h = DEFAULT_HEIGHT;
  
  girl::toger tog(w, h);

  girl::toger tog2 = tog;

  tog2.xMarkOuterBoundary(1, girl::toger::MARK_ANY);
  CPPUNIT_ASSERT(! tog.isRegionOuterBoundaryMarked(1, girl::toger::MARK_ANY));
  CPPUNIT_ASSERT(tog2.isRegionOuterBoundaryMarked(1, girl::toger::MARK_ANY));
}

void
EmptyTogerTest::testPimpl3a()
{
  const girl::POINT_TYPE w = DEFAULT_WIDTH;
  const girl::POINT_TYPE h = DEFAULT_HEIGHT;
  
  girl::toger tog(w, h);

  girl::toger tog2 = tog;

  tog.markAllDarts(girl::toger::MARK_ANY);
  CPPUNIT_ASSERT(tog.isRegionMarked(1, girl::toger::MARK_ANY));
  CPPUNIT_ASSERT(! tog2.isRegionMarked(1, girl::toger::MARK_ANY));
}

void
EmptyTogerTest::testPimpl3b()
{
  const girl::POINT_TYPE w = DEFAULT_WIDTH;
  const girl::POINT_TYPE h = DEFAULT_HEIGHT;
  
  girl::toger tog(w, h);

  girl::toger tog2 = tog;

  tog2.markAllDarts(girl::toger::MARK_ANY);
  CPPUNIT_ASSERT(! tog.isRegionMarked(1, girl::toger::MARK_ANY));
  CPPUNIT_ASSERT(tog2.isRegionMarked(1, girl::toger::MARK_ANY));  
}

void
EmptyTogerTest::testPimpl4a()
{
  const girl::POINT_TYPE w = DEFAULT_WIDTH;
  const girl::POINT_TYPE h = DEFAULT_HEIGHT;
  
  girl::toger tog(w, h);

  tog.xMarkRegionBoundary(1, girl::toger::MARK_ANY);

  girl::toger tog2 = tog;

  tog2.unmarkRegionBoundary(1, girl::toger::MARK_ANY);
  CPPUNIT_ASSERT(! tog2.isRegionMarked(1, girl::toger::MARK_ANY));
  CPPUNIT_ASSERT(tog.isRegionMarked(1, girl::toger::MARK_ANY));
}

void
EmptyTogerTest::testPimpl4b()
{
  const girl::POINT_TYPE w = DEFAULT_WIDTH;
  const girl::POINT_TYPE h = DEFAULT_HEIGHT;
  
  girl::toger tog(w, h);

  tog.xMarkRegionBoundary(1, girl::toger::MARK_ANY);

  girl::toger tog2 = tog;

  tog.unmarkRegionBoundary(1, girl::toger::MARK_ANY);
  CPPUNIT_ASSERT(! tog.isRegionMarked(1, girl::toger::MARK_ANY));
  CPPUNIT_ASSERT(tog2.isRegionMarked(1, girl::toger::MARK_ANY));
}

void
EmptyTogerTest::testPimpl5a()
{
  const girl::POINT_TYPE w = DEFAULT_WIDTH;
  const girl::POINT_TYPE h = DEFAULT_HEIGHT;
  
  girl::toger tog(w, h);

  tog.xMarkOuterBoundary(1, girl::toger::MARK_ANY);

  girl::toger tog2 = tog;

  tog2.unmarkOuterBoundary(1, girl::toger::MARK_ANY);
  CPPUNIT_ASSERT(! tog2.isRegionMarked(1, girl::toger::MARK_ANY));
  CPPUNIT_ASSERT(tog.isRegionMarked(1, girl::toger::MARK_ANY));
}

void
EmptyTogerTest::testPimpl5b()
{
  const girl::POINT_TYPE w = DEFAULT_WIDTH;
  const girl::POINT_TYPE h = DEFAULT_HEIGHT;
  
  girl::toger tog(w, h);

  tog.xMarkOuterBoundary(1, girl::toger::MARK_ANY);

  girl::toger tog2 = tog;

  tog.unmarkOuterBoundary(1, girl::toger::MARK_ANY);
  CPPUNIT_ASSERT(! tog.isRegionMarked(1, girl::toger::MARK_ANY));
  CPPUNIT_ASSERT(tog2.isRegionMarked(1, girl::toger::MARK_ANY));
}

void
EmptyTogerTest::testPimpl5c()
{
  const girl::POINT_TYPE w = DEFAULT_WIDTH;
  const girl::POINT_TYPE h = DEFAULT_HEIGHT;
  
  girl::toger tog(w, h);

  tog.xMarkOuterBoundary(1, girl::toger::MARK_ANY);

  girl::toger tog2;
  tog2 = tog;

  tog.unmarkOuterBoundary(1, girl::toger::MARK_ANY);
  CPPUNIT_ASSERT(! tog.isRegionMarked(1, girl::toger::MARK_ANY));
  CPPUNIT_ASSERT(tog2.isRegionMarked(1, girl::toger::MARK_ANY));
}

void
EmptyTogerTest::testPimpl6a()
{
  const girl::POINT_TYPE w = DEFAULT_WIDTH;
  const girl::POINT_TYPE h = DEFAULT_HEIGHT;
  
  girl::toger tog(w, h);

  tog.xMarkOuterBoundary(1, girl::toger::MARK_ANY);

  girl::toger tog2 = tog;

  tog2.unmarkAllDarts(girl::toger::MARK_ANY);
  CPPUNIT_ASSERT(! tog2.isRegionMarked(1, girl::toger::MARK_ANY));
  CPPUNIT_ASSERT(tog.isRegionMarked(1, girl::toger::MARK_ANY));
}

void
EmptyTogerTest::testPimpl6b()
{
  const girl::POINT_TYPE w = DEFAULT_WIDTH;
  const girl::POINT_TYPE h = DEFAULT_HEIGHT;
  
  girl::toger tog(w, h);

  tog.xMarkOuterBoundary(1, girl::toger::MARK_ANY);

  girl::toger tog2 = tog;

  tog.unmarkAllDarts(girl::toger::MARK_ANY);
  CPPUNIT_ASSERT(tog2.isRegionMarked(1, girl::toger::MARK_ANY));
  CPPUNIT_ASSERT(! tog.isRegionMarked(1, girl::toger::MARK_ANY));
}

void
EmptyTogerTest::testPimpl7a()
{
  const girl::POINT_TYPE w = DEFAULT_WIDTH;
  const girl::POINT_TYPE h = DEFAULT_HEIGHT;
  
  girl::toger tog(w, h);
  girl::toger tog2 = tog;

  tog.shrink();

  CPPUNIT_ASSERT(tog2.checkConsistency());
  //TODO: we can not easily check that shrink did not apply to tog2...
}

