/*
  Test program for euclidean paths.
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "EuclideanPathTest.hpp"


// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( EuclideanPathTest );


void 
EuclideanPathTest::setUp()
{
}


void 
EuclideanPathTest::tearDown()
{
}

static
void
P_init1(std::deque<girl::direction> &dirs)
{
  dirs.push_back(girl::RIGHT);
  dirs.push_back(girl::DOWN);
  dirs.push_back(girl::LEFT);
  dirs.push_back(girl::UP);
}

static
void
P_init2(std::deque<girl::direction> &dirs)
{
  //internal loop
  // with start on loop

  dirs.push_back(girl::DOWN);
  dirs.push_back(girl::RIGHT);
  dirs.push_back(girl::UP);
  dirs.push_back(girl::LEFT);
  dirs.push_back(girl::UP);
  dirs.push_back(girl::RIGHT);
  dirs.push_back(girl::RIGHT);
  dirs.push_back(girl::DOWN);
  dirs.push_back(girl::DOWN);
  dirs.push_back(girl::DOWN);
  dirs.push_back(girl::LEFT);
  dirs.push_back(girl::LEFT);
  dirs.push_back(girl::LEFT);
  dirs.push_back(girl::UP);
  dirs.push_back(girl::UP);
  dirs.push_back(girl::RIGHT);
}


void 
EuclideanPathTest::testIterator(girl::point startPt,
			       const std::deque<girl::direction> &dirs)
{
  girl::contour c(startPt, dirs);
  CPPUNIT_ASSERT(c.isValid());
  CPPUNIT_ASSERT(c.isClosed());

  girl::euclideanPath ep(c);

  CPPUNIT_ASSERT(c.length() == ep.length());
  
  girl::point pt = startPt;
  girl::euclideanPath::const_iterator it = ep.begin();
  const girl::euclideanPath::const_iterator itEnd = ep.end();
  std::deque<girl::direction>::const_iterator itD = dirs.begin();
  //const std::deque<girl::direction>::const_iterator itDEnd = dirs.end();
  for ( ; it != itEnd; ++it, ++itD) {
    
    girl::point ep_pt = it.getPoint();
    girl::direction dirN = it.getNextDir();
    CPPUNIT_ASSERT(ep_pt == pt);
    CPPUNIT_ASSERT(dirN == *itD);
    pt.move(dirN);
  }
}


void 
EuclideanPathTest::testIterator1()
{
  const girl::point startPt(1, 1);
  std::deque<girl::direction> dirs;
  P_init1(dirs);
  
  testIterator(startPt, dirs);

}

void 
EuclideanPathTest::testIterator2()
{
  const girl::point startPt(1, -1);
  std::deque<girl::direction> dirs;
  P_init2(dirs);
  
  testIterator(startPt, dirs);

}
