/*
  Test program for geometric dart class.
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#include "GeoDartTest.hpp"

// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( GeoDartTest );

void 
GeoDartTest::setUp()
{
}


void 
GeoDartTest::tearDown()
{
}

void 
GeoDartTest::testConstructor1()
{
  girl::POINT_TYPE x = -1;
  girl::POINT_TYPE y = -1;
  girl::direction dir = girl::RIGHT;

  girl::geoDart u0(x, y, dir);

  CPPUNIT_ASSERT(u0.x() == x);
  CPPUNIT_ASSERT(u0.y() == y);
  CPPUNIT_ASSERT(u0.dir() == dir);
}

void 
GeoDartTest::testConstructor2()
{
  girl::POINT_TYPE x = -1;
  girl::POINT_TYPE y = -1;
  girl::direction dir = girl::RIGHT;

  girl::geoDart u0(girl::point(x, y), dir);

  CPPUNIT_ASSERT(u0.x() == x);
  CPPUNIT_ASSERT(u0.y() == y);
  CPPUNIT_ASSERT(u0.dir() == dir);
}

void 
GeoDartTest::testSet()
{

  girl::POINT_TYPE x = -1;
  girl::POINT_TYPE y = -1;
  girl::direction dir = girl::RIGHT;

  girl::geoDart u0(girl::point(x, y), dir);
  
  girl::POINT_TYPE x2 = -1;
  girl::POINT_TYPE y2 = -1;
  girl::direction dir2 = girl::LEFT;
  
  u0.setX(x2);
  u0.setY(y2);
  u0.setDir(dir2);

  CPPUNIT_ASSERT(u0.x() == x2);
  CPPUNIT_ASSERT(u0.y() == y2);
  CPPUNIT_ASSERT(u0.dir() == dir2);

  u0.setPoint(girl::point(x, y));

  CPPUNIT_ASSERT(u0.x() == x);
  CPPUNIT_ASSERT(u0.y() == y);
  CPPUNIT_ASSERT(u0.dir() == dir2);

  u0.set(x2, y2, dir);
  
  CPPUNIT_ASSERT(u0.x() == x2);
  CPPUNIT_ASSERT(u0.y() == y2);
  CPPUNIT_ASSERT(u0.dir() == dir);
}


void 
GeoDartTest::testEquality()
{
  girl::POINT_TYPE x = -1;
  girl::POINT_TYPE y = 1;
  girl::direction dir = girl::UP;

  girl::geoDart u0(girl::point(x, y), dir);
  girl::geoDart u1(x, y, dir);
  
  CPPUNIT_ASSERT(u0 == u1);

  u1 = u0;

  CPPUNIT_ASSERT(u0 == u1);
  
  u1.setX(u0.x()+1);

  CPPUNIT_ASSERT(u0 != u1);
  
  u1 = u0;
  u1.setY(u0.y()+1);
  
  CPPUNIT_ASSERT(u0 != u1);

  u1 = u0;
  u1.setDir(u0.dir().next());
  
  CPPUNIT_ASSERT(u0 != u1);
}


