/*
  Test program for path insertion in toger.
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#ifndef CHECK_INSERT_HPP
#define CHECK_INSERT_HPP

#include <cppunit/extensions/HelperMacros.h>

#include <girl/girlToger.hpp>



class InsertTest : public CppUnit::TestFixture
{
  CPPUNIT_TEST_SUITE( InsertTest );

  CPPUNIT_TEST( testEmptySplit1 );
  CPPUNIT_TEST( testEmptyToger1 );
  CPPUNIT_TEST( testEmptyToger2 );
  CPPUNIT_TEST( testEmptyToger3 );
  //CPPUNIT_TEST( testEmptyToger4 );
  CPPUNIT_TEST( testEmptyToger5 );

  CPPUNIT_TEST( testOverlap1 );

  CPPUNIT_TEST( testIterateOnDomain );

  CPPUNIT_TEST( testPendingEdge1 );
  CPPUNIT_TEST( testInsert1 );
  CPPUNIT_TEST( testInsert1b );
  CPPUNIT_TEST( testInsert2 );
  CPPUNIT_TEST( testInsert3 );
  CPPUNIT_TEST( testInsert4 );
  CPPUNIT_TEST( testInsert5 );
  CPPUNIT_TEST( testInsert6 );
  CPPUNIT_TEST( testInsert7 );

  CPPUNIT_TEST( testInsertTangent1 );
  CPPUNIT_TEST( testInsertTangent2 );
  CPPUNIT_TEST( testInsertTangent3 );
  CPPUNIT_TEST( testInsertTangent4 );
  CPPUNIT_TEST( testInsertTangent5 );
  CPPUNIT_TEST( testInsertTangent6 );
  CPPUNIT_TEST( testInsertTangent7 );
  CPPUNIT_TEST( testInsertTangent8 );

  CPPUNIT_TEST( testInsertPending1 );

  CPPUNIT_TEST( testInsertLoopTangent1 );
  CPPUNIT_TEST( testInsertLoopTangent2 );
  CPPUNIT_TEST( testInsertLoopTangent3 );
  CPPUNIT_TEST( testInsertLoopTangent5 );
  CPPUNIT_TEST( testInsertLoopTangent6 );
  CPPUNIT_TEST( testInsertLoopTangent7 );

  CPPUNIT_TEST( testInsertOverlapRectangle1 );
  CPPUNIT_TEST( testInsertOverlapRectangle1b );
  CPPUNIT_TEST( testInsertOverlapRectangle2 );

  /*
  CPPUNIT_TEST( testInsertOverlap1 );
  CPPUNIT_TEST( testInsertOverlap2 );
  CPPUNIT_TEST( testInsertOverlap3 );
  */

  CPPUNIT_TEST( testInsertCopy );

  //CPPUNIT_TEST( testChessboardInsert );
  CPPUNIT_TEST( testHorizontalInsert );
  //CPPUNIT_TEST( testVerticalInsert );
  //CPPUNIT_TEST( testOddsInsert );
  //CPPUNIT_TEST( testConcentricInsert );
  //CPPUNIT_TEST( testDiagonalInsert );

  CPPUNIT_TEST( testContours );

  CPPUNIT_TEST( testHook1 );
  CPPUNIT_TEST( testHook2 );
  CPPUNIT_TEST( testHook3 );

  CPPUNIT_TEST_SUITE_END();

public:
  void setUp();
  void tearDown();

  void testEmptySplit1();

  void testEmptyToger1();
  void testEmptyToger2();
  void testEmptyToger3();
  void testEmptyToger4();
  void testEmptyToger5();

  void testOverlap1();

  void testIterateOnDomain();

  void testPendingEdge1();

  void testInsert1();
  void testInsert1b();
  void testInsert2();
  void testInsert3();
  void testInsert4();
  void testInsert5();
  void testInsert6();
  void testInsert7();

  void testInsertCopy();

  void testInsertTangent1();
  void testInsertTangent2();
  void testInsertTangent3();
  void testInsertTangent4();
  void testInsertTangent5();
  void testInsertTangent6();
  void testInsertTangent7();
  void testInsertTangent8();

  void testInsertPending1();

  void testInsertLoopTangent1();
  void testInsertLoopTangent2();
  void testInsertLoopTangent3();
  void testInsertLoopTangent5();
  void testInsertLoopTangent6();
  void testInsertLoopTangent7();

  void testInsertOverlapRectangle1();
  void testInsertOverlapRectangle1b();
  void testInsertOverlapRectangle2();

  /*
  void testInsertOverlap1();
  void testInsertOverlap2();
  void testInsertOverlap3();
  */

  //void testChessboardInsert();
  void testHorizontalInsert();
  //void testVerticalInsert();
  //void testOddsInsert();
  //void testConcentricInsert();
  //void testDiagonalInsert();

  void testContours();

  void testHook1();
  void testHook2();
  void testHook3();

protected:

  void testFunc(girl::toger::sameRegion &func, girl::POINT_TYPE width);
  //void testFunc_aux(girl::toger::sameRegion &func, girl::toger::sameRegion &func2, girl::POINT_TYPE width);

};





#endif /* ! CHECK_INSERT_HPP */
