/*
  Test program for path class.
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#include "PathTest.hpp"


// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( PathTest );


void 
PathTest::setUp()
{
}


void 
PathTest::tearDown()
{
}

//simple contour with only one direction
static
void
P_init0(std::deque<girl::direction> &dirs)
{
  dirs.clear();
  dirs.push_back(girl::DOWN);
}

//closed contour, around one pixel, from left-up corner, clockwise.
// length == 4
static 
void
P_init1(std::deque<girl::direction> &dirs)
{
  dirs.clear();
  dirs.push_back(girl::RIGHT);
  dirs.push_back(girl::DOWN);
  dirs.push_back(girl::LEFT);
  dirs.push_back(girl::UP);
}

// not closed
// endPt.x = startPt.x + 3; endPt.y = startPt.y;
//length == 5
static
void
P_init2(std::deque<girl::direction> &dirs)
{
  dirs.clear();
  dirs.push_back(girl::RIGHT);
  dirs.push_back(girl::DOWN);
  dirs.push_back(girl::RIGHT);
  dirs.push_back(girl::UP);
  dirs.push_back(girl::RIGHT);
}


//closed contour, cross around fiwe pixels, from left-up corner, clockwise.
// length == 12
static 
void
P_init3(std::deque<girl::direction> &dirs)
{
  dirs.clear();
  dirs.push_back(girl::RIGHT);
  dirs.push_back(girl::DOWN);
  dirs.push_back(girl::RIGHT);
  dirs.push_back(girl::DOWN);
  dirs.push_back(girl::LEFT);
  dirs.push_back(girl::DOWN);
  dirs.push_back(girl::LEFT);
  dirs.push_back(girl::UP);
  dirs.push_back(girl::LEFT);
  dirs.push_back(girl::UP);
  dirs.push_back(girl::RIGHT);
  dirs.push_back(girl::UP);
}

//not closed
//endPt.x = startPt.x; endPt.y = startPt.y-1; 
//length == 7
static
void
P_init4(std::deque<girl::direction> &dirs)
{
  dirs.clear();
  dirs.push_back(girl::RIGHT);
  dirs.push_back(girl::DOWN);
  dirs.push_back(girl::RIGHT);
  dirs.push_back(girl::UP);
  dirs.push_back(girl::LEFT);
  dirs.push_back(girl::UP);
  dirs.push_back(girl::LEFT);
}

//inversion of P_init4
void
P_init5(std::deque<girl::direction> &dirs)
{
  dirs.clear();
  dirs.push_back(girl::RIGHT);
  dirs.push_back(girl::DOWN);
  dirs.push_back(girl::RIGHT);
  dirs.push_back(girl::DOWN);
  dirs.push_back(girl::LEFT);
  dirs.push_back(girl::UP);
  dirs.push_back(girl::LEFT);
}

void
P_init6(std::deque<girl::direction> &dirs)
{
  dirs.clear();
  dirs.push_back(girl::UP);
  dirs.push_back(girl::LEFT);
  dirs.push_back(girl::UP);
} 


void 
PathTest::testEqual0()
{
  //test equality between empty paths
  girl::path p1;
  CPPUNIT_ASSERT(p == p1);
}

void
PathTest::testEqual1()
{
  //test equality between paths initialized during construction and afterwards.

  P_init4(dirs);
  const girl::point startPt(0, 0);
  p.set(startPt, dirs);
  CPPUNIT_ASSERT(p.startPt() == startPt);

  girl::path p1(girl::point(0, 0), dirs);
  CPPUNIT_ASSERT(p == p1);
}

void
PathTest::testEqual2()
{
  //test equality after assignment

  P_init2(dirs);
  p.set(girl::point(0, 0), dirs);
  
  girl::path p1 = p;
  CPPUNIT_ASSERT(p == p1);
}

void
PathTest::testEqual3()
{
  //test equality after different assignments

  P_init1(dirs);
  p.set(girl::point(0, 0), dirs);
  
  P_init3(dirs);
  girl::path p1(girl::point(0, 0), dirs);

  CPPUNIT_ASSERT(p != p1);
}

void
PathTest::testIterator0()
{
  //test iterator on empty path

  CPPUNIT_ASSERT(p.begin() == p.end());
}

void
PathTest::testIterator1()
{
  P_init0(dirs);
  p.set(girl::point(0, 0), dirs);
 
  CPPUNIT_ASSERT(p.begin() != p.end());
  CPPUNIT_ASSERT(++(p.begin()) == p.end());
}

void
PathTest::testIterator2()
{
  P_init5(dirs);
  p.set(girl::point(-1, 0), dirs);
 
  size_t len = 0;
  girl::path::const_dir_iterator it = p.begin();
  const girl::path::const_dir_iterator itEnd = p.end();
  for ( ; it != itEnd; ++it)
    ++len;
  CPPUNIT_ASSERT(len == p.length());
}

void
PathTest::testIteratorPlusN()
{
  dirs.clear();
  dirs.push_back(girl::RIGHT);
  dirs.push_back(girl::DOWN);
  dirs.push_back(girl::RIGHT);
  dirs.push_back(girl::DOWN);
  dirs.push_back(girl::RIGHT);
  dirs.push_back(girl::UP);
  dirs.push_back(girl::RIGHT);
  dirs.push_back(girl::UP);
  dirs.push_back(girl::RIGHT);
  dirs.push_back(girl::UP);
  dirs.push_back(girl::LEFT);
  dirs.push_back(girl::UP);
  dirs.push_back(girl::LEFT);
  dirs.push_back(girl::UP);
  dirs.push_back(girl::LEFT);
  dirs.push_back(girl::UP);
  dirs.push_back(girl::LEFT);
  p.set(girl::point(-1, 0), dirs);
 
  for (size_t i=0; i<dirs.size(); ++i) {
    CPPUNIT_ASSERT( (*(p.begin() + i)) == (*(dirs.begin() + i)) );
  }
}

void
PathTest::testReverseIterator0()
{
  //test reverse iterator on empty path

  CPPUNIT_ASSERT(p.rbegin() == p.rend());
}

void
PathTest::testReverseIterator1()
{
  P_init0(dirs);
  p.set(girl::point(0, 0), dirs);
 
  CPPUNIT_ASSERT(p.rbegin() != p.rend());
  CPPUNIT_ASSERT(++(p.rbegin()) == p.rend());
}

void
PathTest::testReverseIterator2()
{
  P_init5(dirs);
  p.set(girl::point(-1, 0), dirs);
 
  size_t len = 0;
  girl::path::const_reverse_dir_iterator it = p.rbegin();
  const girl::path::const_reverse_dir_iterator itEnd = p.rend();
  for ( ; it != itEnd; ++it)
    ++len;
  CPPUNIT_ASSERT(len == p.length());
}


static
void
P_testDirs(const girl::path &p,
	   const std::deque<girl::direction> &dirs)
{
  CPPUNIT_ASSERT(p.length() == dirs.size());

  std::deque<girl::direction>::const_iterator itD = dirs.begin();
  std::deque<girl::direction>::const_iterator itDEnd = dirs.end();

  int i = 0;
  girl::path::const_dir_iterator it = p.begin();
  const girl::path::const_dir_iterator itEnd = p.end();
  for ( ; it != itEnd; ++it) {
    CPPUNIT_ASSERT(itD != itDEnd);
    CPPUNIT_ASSERT(*it == *itD);
    CPPUNIT_ASSERT(p[i] == *it);
    ++itD;
    ++i;
  }  
}


void
PathTest::testGet() const
{
  //test that we have the same directions in p and in dirs.

  P_testDirs(p, dirs);
}

void
PathTest::testSetGet0()
{
  //test what we set is what we get on empty path
  testGet();
}

void
PathTest::testSetGet1()
{
  //test what we set is what we get
  P_init1(dirs);
  p.set(girl::point(0, 0), dirs);
  testGet();
}

void
PathTest::testSetGet2()
{
  //test what we set is what we get
  P_init1(dirs);
  p.set(girl::point(0,-1), dirs);
  testGet();
}

void
PathTest::testSetGet3()
{
  //test what we set is what we get
  P_init4(dirs);
  p.set(girl::point(-1, 0), dirs);
  testGet();
}


void
PathTest::testSetGet6()
{
  //test what we set is what we get
  P_init6(dirs);
  p.set(girl::point(0, 0), dirs);
  testGet();
}


void 
PathTest::testLength0()
{
  //test empty path is of length 0
  CPPUNIT_ASSERT(p.length() == 0);
}

void 
PathTest::testLength1()
{
  P_init1(dirs);
  p.set(girl::point(0, 0), dirs);
  CPPUNIT_ASSERT(p.length() == 4);
}

void 
PathTest::testLength2()
{
  P_init4(dirs);
  p.set(girl::point(0, 0), dirs);
  CPPUNIT_ASSERT(p.length() == 7);
}

void
PathTest::testEndPt0()
{
  //startPt==endPt on empty path
  CPPUNIT_ASSERT(p.startPt() == p.endPt());
}

void
PathTest::testEndPt1()
{
  P_init1(dirs);
  p.set(girl::point(0, 0), dirs);
  CPPUNIT_ASSERT(p.startPt() == p.endPt()); 
}

void
PathTest::testEndPt2()
{
  P_init3(dirs);
  p.set(girl::point(0, 0), dirs);
  CPPUNIT_ASSERT(p.startPt() == p.endPt());   
}

void
PathTest::testEndPt3()
{
  P_init2(dirs);
  girl::point startPt(-1, -1);
  p.set(startPt, dirs);
  girl::point endPt(startPt.x()+3, startPt.y());
  CPPUNIT_ASSERT(endPt == p.endPt());   
}

void
PathTest::testEndPt4()
{
  P_init4(dirs);
  girl::point startPt(-1, 7);
  p.set(startPt, dirs);
  girl::point endPt(startPt.x(), startPt.y()-1);
  CPPUNIT_ASSERT(endPt == p.endPt());   
}


void
PathTest::testInvert0()
{
  //test inversion is ok on empty path
  girl::path p1 = p;
  p1.invertOrientation();
  CPPUNIT_ASSERT(p == p1);
}


void
PathTest::testInvert2()
{
  //test inversion is ok 
  P_init5(dirs);
  p.set(girl::point(0, 0), dirs);  
  p.invertOrientation();
  CPPUNIT_ASSERT(p.isValid());
  P_init4(dirs);
  testGet();
}

void
PathTest::testInvert3()
{
  //test inversion is ok 
  P_init4(dirs);
  p.set(girl::point(0, 0), dirs);  
  p.invertOrientation();
  CPPUNIT_ASSERT(p.isValid());
  P_init5(dirs);
  testGet();
}

void
PathTest::testInvert4()
{
  //test endPt'==startPt
  P_init4(dirs);
  girl::point startPt(7, 300);
  p.set(startPt, dirs);
  girl::point endPt = p.endPt();
  p.invertOrientation();
  CPPUNIT_ASSERT(p.endPt() == startPt); 
  CPPUNIT_ASSERT(p.startPt() == endPt); 
  CPPUNIT_ASSERT(p.isValid());
}

void
PathTest::testInvert5()
{
  //test double inversion give initial path
  P_init4(dirs);
  p.set(girl::point(0, 0), dirs);  
  p.invertOrientation();
  CPPUNIT_ASSERT(p.isValid());
  p.invertOrientation();
  CPPUNIT_ASSERT(p.isValid());
  testGet();
}

void
PathTest::testAdd1()
{
  //test "add" gives a correct length from an empty path

  const size_t nbAdds = 27;
  for (size_t i=0; i<nbAdds; ++i) {
    CPPUNIT_ASSERT(p.length() == i);
    p.add(girl::RIGHT);
  }
  CPPUNIT_ASSERT(p.length() == nbAdds);
}

void
PathTest::testAdd2()
{
  //test "add" gives a correct length from a non-empty path

  P_init4(dirs);
  p.set(girl::point(0, 0), dirs);  
  
  const size_t originalLength = p.length();
  const size_t nbAdds = 25;
  for (size_t i=0; i<nbAdds; ++i) {
    CPPUNIT_ASSERT(p.length() == originalLength+i);
    p.add(girl::LEFT);
  }
  CPPUNIT_ASSERT(p.length() == originalLength+nbAdds);
}

void
PathTest::testAdd3()
{
  //test "add" correctly changes start and end point for an empty path

  const girl::point originalStartPt = p.startPt();
  const girl::point originalEndPt = p.endPt();
  CPPUNIT_ASSERT(p.startPt() == p.endPt());
  
  const size_t nbAdds = 29;
  for (size_t i=0; i<nbAdds; ++i) {
    p.add(girl::RIGHT);
  }
  const girl::point newEndPt(originalEndPt.x()+nbAdds, originalEndPt.y());

  CPPUNIT_ASSERT(p.startPt() == originalStartPt);
  CPPUNIT_ASSERT(p.endPt() == newEndPt);
}

void
PathTest::testAdd4()
{
  //test "add" correctly changes start and end point for a non-empty path

  P_init4(dirs);
  p.set(girl::point(7,-1), dirs);  
  
  const girl::point originalStartPt = p.startPt();
  const girl::point originalEndPt = p.endPt();
  
  const size_t nbAdds = 31;
  for (size_t i=0; i<nbAdds; ++i) {
    p.add(girl::LEFT);
  }
  const girl::point newEndPt(originalEndPt.x()-nbAdds, originalEndPt.y());

  CPPUNIT_ASSERT(p.startPt() == originalStartPt);
  CPPUNIT_ASSERT(p.endPt() == newEndPt);
}


void
PathTest::testMove0()
{
  //test "move" on empty path

  const girl::point newStartPt(21, -1);

  p.move(newStartPt);

  girl::path p1(newStartPt);

  CPPUNIT_ASSERT(p == p1);
}

void
PathTest::testMove1()
{
  //test "move" correctly changes start and end point for an empty path

  //const girl::point originalStartPt = p.startPt();
  //const girl::point originalEndPt = p.endPt();
  CPPUNIT_ASSERT(p.startPt() == p.endPt());

  const girl::point newStartPt(211, -1);
  p.move(newStartPt);
  CPPUNIT_ASSERT(p.startPt() == newStartPt);
  CPPUNIT_ASSERT(p.startPt() == p.endPt());
}

void
PathTest::testMove2()
{
  //test "move" correctly changes start and end point for a non-empty path

  P_init5(dirs);
  p.set(girl::point(65, 3200), dirs);  

  const girl::point originalStartPt = p.startPt();
  const girl::point originalEndPt = p.endPt();

  const girl::point newStartPt(-1, -1);
  p.move(newStartPt);

  const girl::point newEndPt(newStartPt.x()+originalEndPt.x()-originalStartPt.x(),
			     newStartPt.y()+originalEndPt.y()-originalStartPt.y());

  CPPUNIT_ASSERT(p.startPt() == newStartPt);
  CPPUNIT_ASSERT(p.endPt() == newEndPt);
  
}

void
PathTest::testAppend0()
{
  //test "append" of an empty path on an empty path correctly changes start, end points and length 

  CPPUNIT_ASSERT(p.isValid());

  const girl::point pStartPt = p.startPt();
  const girl::point pEndPt = p.endPt();
  CPPUNIT_ASSERT(p.length() == 0);
  
  girl::path p2;
  
  p.append(p2);

  CPPUNIT_ASSERT(p.startPt() == pStartPt);
  CPPUNIT_ASSERT(p.endPt() == pEndPt);
  CPPUNIT_ASSERT(p.length() == 0);
}

void
PathTest::testAppend1()
{
  //test "append" on an empty path correctly changes start, end points and length 

  CPPUNIT_ASSERT(p.isValid());

  const girl::point pStartPt = p.startPt();
  const girl::point pEndPt = p.endPt();
  const size_t pLength = p.length();

  P_init5(dirs);
  girl::path p2(girl::point(65, 3200), dirs);

  const girl::point p2StartPt = p2.startPt();
  const girl::point p2EndPt = p2.endPt();
  const size_t p2Length = p2.length();

  p.append(p2);

  const girl::point newEndPt(pEndPt.x()+p2EndPt.x()-p2StartPt.x(),
			     pEndPt.y()+p2EndPt.y()-p2StartPt.y());

  CPPUNIT_ASSERT(p.isValid());
  CPPUNIT_ASSERT(p.startPt() == pStartPt);
  CPPUNIT_ASSERT(p.endPt() == newEndPt);
  CPPUNIT_ASSERT(p.length() == pLength+p2Length);
}

void
PathTest::testAppend1b()
{
  //test "append" of an empty path on an non-empty path correctly changes start, end points and length 

  CPPUNIT_ASSERT(p.isValid());

  P_init5(dirs);
  girl::path p2(girl::point(65, 3200), dirs);

  p = p2;

  const girl::point pStartPt = p.startPt();
  const girl::point pEndPt = p.endPt();
  const size_t pLength = p.length();

  girl::path p3;

  p.append(p3);
  
  CPPUNIT_ASSERT(p.isValid());
  CPPUNIT_ASSERT(p.startPt() == pStartPt);
  CPPUNIT_ASSERT(p.endPt() == pEndPt);
  CPPUNIT_ASSERT(p.length() == pLength);
}




void
PathTest::testAppend2()
{
  //test "append" correctly changes start, end points and length on a non-empty path

  P_init4(dirs);
  p.set(girl::point(7,-1), dirs);

  const girl::point pStartPt = p.startPt();
  const girl::point pEndPt = p.endPt();
  const size_t pLength = p.length();

  P_init6(dirs);
  girl::path p2(girl::point(65, 3200), dirs);
  
  const girl::point p2StartPt = p2.startPt();
  const girl::point p2EndPt = p2.endPt();
  const size_t p2Length = p2.length();

  p.append(p2);

  const girl::point newEndPt(pEndPt.x()+p2EndPt.x()-p2StartPt.x(),
			     pEndPt.y()+p2EndPt.y()-p2StartPt.y());

  CPPUNIT_ASSERT(p.startPt() == pStartPt);
  CPPUNIT_ASSERT(p.endPt() == newEndPt);
  CPPUNIT_ASSERT(p.length() == pLength+p2Length);
  CPPUNIT_ASSERT(p.isValid());
}

static
void
P_appendDeques(std::deque<girl::direction> &dirs, const std::deque<girl::direction> &dirs2)
{
  for (std::deque<girl::direction>::const_iterator it = dirs2.begin();
       it != dirs2.end(); ++it)
    dirs.push_back(*it);
}

void
PathTest::testAppend3()
{
  //test "append" gives correct directions

  P_init4(dirs);
  p.set(girl::point(7,-1), dirs);

  std::deque<girl::direction> dirs2;
  P_init6(dirs2);
  
  girl::path p2(girl::point(26, 28), dirs2);
  
  P_appendDeques(dirs, dirs2);
  p.append(p2);

  testGet();
}

static
void
P_testBelongs_aux(girl::path &p,
		  std::deque<girl::direction> &dirs,
		  void (*initFunc)(std::deque<girl::direction> &dirs),
		  girl::point startPt)
{
  dirs.clear();
  initFunc(dirs);
  girl::point pt = startPt;
  p.set(pt, dirs);  
  
  CPPUNIT_ASSERT(p.belongs(p.startPt()));
  CPPUNIT_ASSERT(p.belongs(p.endPt()));

  CPPUNIT_ASSERT(p.belongs(pt));
  std::deque<girl::direction>::const_iterator itD = dirs.begin();
  std::deque<girl::direction>::const_iterator itDEnd = dirs.end();
  for ( ; itD != itDEnd; ++itD) {
    pt.move(*itD);
    CPPUNIT_ASSERT(p.belongs(pt));
  }
}
		
void
PathTest::testBelongs()
{
  P_testBelongs_aux(p, dirs, P_init4, girl::point(-1, -1));

  P_testBelongs_aux(p, dirs, P_init5, girl::point(1, -1));

  P_testBelongs_aux(p, dirs, P_init6, girl::point(23, 1900));
}

void
PathTest::testClosed()
{
  //test that a closed path is reported as closed
  // and stay closed after invertion

  P_init1(dirs);
  p.set(girl::point(0, 0), dirs);
  CPPUNIT_ASSERT(p.startPt() == p.endPt());
  CPPUNIT_ASSERT(p.isClosed());

  p.invertOrientation();

  CPPUNIT_ASSERT(p.isClosed());
}

void
PathTest::testCrop0()
{
  //test crop of nothing on empty path does nothing

  CPPUNIT_ASSERT(p.isValid());
  const girl::point pStartPt = p.startPt();
  const girl::point pEndPt = p.endPt();
  const size_t pLength = p.length();
  CPPUNIT_ASSERT(p.length() == 0);

  p.crop(0, pLength);
  
  CPPUNIT_ASSERT(p.isValid());
  CPPUNIT_ASSERT(pStartPt == p.startPt());
  CPPUNIT_ASSERT(pEndPt == p.endPt());
  CPPUNIT_ASSERT(pLength == p.length());
}

void
PathTest::testCrop1()
{
  //test crop of nothing on non-empty path does nothing

  P_init4(dirs);
  p.set(girl::point(7,-1), dirs);

  CPPUNIT_ASSERT(p.isValid());
  const girl::point pStartPt = p.startPt();
  const girl::point pEndPt = p.endPt();
  const size_t pLength = p.length();

  p.crop(0, pLength);
  
  CPPUNIT_ASSERT(p.isValid());
  CPPUNIT_ASSERT(pStartPt == p.startPt());
  CPPUNIT_ASSERT(pEndPt == p.endPt());
  CPPUNIT_ASSERT(pLength == p.length());
}

void
PathTest::testCrop1b()
{
  //test crop of all on no-empty path creates an empty path

  P_init4(dirs);
  p.set(girl::point(7,-1), dirs);

  CPPUNIT_ASSERT(p.isValid());
  const girl::point pStartPt = p.startPt();

  p.crop(0, 0);
  
  CPPUNIT_ASSERT(p.isValid());
  CPPUNIT_ASSERT(pStartPt == p.startPt());
  CPPUNIT_ASSERT(p.length() == 0);
  CPPUNIT_ASSERT(p.startPt() == p.endPt());
}

static
void
P_testCrop_end(girl::path &p,
	       std::deque<girl::direction> &dirs,
	       void (*initFunc)(std::deque<girl::direction> &dirs),
	       girl::point startPt)
{
  //test crop of the end of path

  dirs.clear();
  initFunc(dirs);
  girl::point pt = startPt;
  p.set(pt, dirs);  
  CPPUNIT_ASSERT(p.isValid());

  for (size_t i=p.length(); i>0; --i) {
      
    girl::path p2 = p;
    p2.crop(0, i);
    CPPUNIT_ASSERT(p2.isValid());
    CPPUNIT_ASSERT(p2.startPt() == p.startPt());

    girl::point endPt = p2.startPt();
    std::deque<girl::direction>::const_iterator itD = dirs.begin();
    //std::deque<girl::direction>::const_iterator itDEnd = dirs.end();
    for (size_t j=0 ; j<i; ++j, ++itD) {
      endPt.move(*itD);
    }
    CPPUNIT_ASSERT(p2.endPt() == endPt);
  }
}

void
PathTest::testCrop2()
{
  //test crop of the end

  P_testCrop_end(p, dirs, P_init4, girl::point(-1, -1));

  P_testCrop_end(p, dirs, P_init5, girl::point(-1, 1));

  P_testCrop_end(p, dirs, P_init6, girl::point(11, 6));
}




static
void
P_testCrop_end_rit(girl::path &p,
	       std::deque<girl::direction> &dirs,
	       void (*initFunc)(std::deque<girl::direction> &dirs),
	       girl::point startPt)
{
  //test crop of the end of path and valid reverse iterator

  dirs.clear();
  initFunc(dirs);
  girl::point pt = startPt;
  p.set(pt, dirs);  
  CPPUNIT_ASSERT(p.isValid());

  size_t len = p.length();

  while (! dirs.empty()) {

    CPPUNIT_ASSERT(*p.rbegin() == dirs.back());
    --len;
    p.crop(0, len);
    dirs.pop_back();
  }
}


void
PathTest::testCrop2b()
{
  //test crop of end of path and reverse iterator 

  P_testCrop_end_rit(p, dirs, P_init4, girl::point(-1, -1));

  P_testCrop_end_rit(p, dirs, P_init5, girl::point(-1, 1));

  P_testCrop_end_rit(p, dirs, P_init6, girl::point(11, 6));
}

#include <iostream> //DEBUG

static
void
P_testCrop_start(girl::path &p,
		 std::deque<girl::direction> &dirs,
		 void (*initFunc)(std::deque<girl::direction> &dirs),
		 girl::point startPt)
{
  //test crop from the start of path

  dirs.clear();
  initFunc(dirs);
  girl::point pt = startPt;
  p.set(pt, dirs);  
  CPPUNIT_ASSERT(p.isValid());

  const size_t pLength = p.length();

  for (size_t i=0; i<pLength; ++i) {
      
    const size_t newLength = pLength-i;

    girl::path p2 = p;
    p2.crop(i, newLength);
    if (! p2.isValid()) {
      std::cerr<<"p.startPt()=("<<p.startPt().x()<<"; "<<p.startPt().y()<<") p.endPt()=("<<p.endPt().x()<<"; "<<p.endPt().y()<<") length="<<p.length()<<std::endl;
      std::cerr<<"p.crop("<<i<<", "<<newLength<<")"<<std::endl;
      std::cerr<<"p2.startPt()=("<<p2.startPt().x()<<"; "<<p2.startPt().y()<<") p2.endPt()=("<<p2.endPt().x()<<"; "<<p2.endPt().y()<<") length="<<p2.length()<<std::endl;
      std::cerr<<"dirs: ";
      girl::path::const_dir_iterator it = p2.begin();
      const girl::path::const_dir_iterator itEnd = p2.end();
      for ( ; it != itEnd; ++it)
	std::cerr<<*it<<" ";
      std::cerr<<std::endl;

    }

    CPPUNIT_ASSERT(p2.isValid());
    CPPUNIT_ASSERT(p2.endPt() == p.endPt());

    girl::point lStartPt = p.startPt();
    std::deque<girl::direction>::const_iterator itD = dirs.begin();
    //std::deque<girl::direction>::const_iterator itDEnd = dirs.end();
    for (size_t j=0 ; j<i; ++j, ++itD) {
      lStartPt.move(*itD);
    }
    CPPUNIT_ASSERT(p2.startPt() == lStartPt);
  }
}

void
PathTest::testCrop3()
{
  //test crop of the start

  P_testCrop_start(p, dirs, P_init4, girl::point(-1, -1));

  P_testCrop_start(p, dirs, P_init5, girl::point(-1, 1));

  P_testCrop_start(p, dirs, P_init6, girl::point(11, 6));
}

void
PathTest::testCrop4()
{
  //test append after crop

  dirs.clear();
  dirs.push_back(girl::RIGHT);
  dirs.push_back(girl::RIGHT);
  dirs.push_back(girl::DOWN);
  dirs.push_back(girl::RIGHT);
  dirs.push_back(girl::RIGHT);
  dirs.push_back(girl::DOWN);


  girl::point pt(7, 11);;
  p.set(pt, dirs);  
  CPPUNIT_ASSERT(p.isValid());


  std::deque<girl::direction> dirs2;
  dirs2.push_back(girl::UP);
  dirs2.push_back(girl::UP);
  dirs2.push_back(girl::RIGHT);
  
  girl::point pt2(37, 41);;
  girl::path p2(pt2, dirs2);
  CPPUNIT_ASSERT(p2.isValid());
  
  CPPUNIT_ASSERT(*(p.rbegin()) == (*p2.begin()).opposite());

  p.crop(0, p.length()-1);
  p2.crop(1, p2.length()-1);

  CPPUNIT_ASSERT(*(p.rbegin()) != (*p2.begin()).opposite());
  CPPUNIT_ASSERT(p.isValid());
  CPPUNIT_ASSERT(p2.isValid());

  dirs.pop_back();
  dirs2.pop_front();


  {
    {
      girl::path::const_dir_iterator it = p.begin();
      const girl::path::const_dir_iterator itEnd = p.end();

      size_t i=0;

      std::deque<girl::direction>::iterator itD = dirs.begin();
      const std::deque<girl::direction>::iterator itDEnd = dirs.end();
      for ( ; itD != itDEnd; ++itD, ++it) {
	if (it == itEnd)
	  std::cerr<<"end prematuree 1"<<std::endl;
	if (*it != *itD) {
	  std::cerr<<i<<"   *it="<<*it<<" *itD="<<*itD<<std::endl;
	}
	++i;
    

	CPPUNIT_ASSERT(it != itEnd);
	CPPUNIT_ASSERT(*it == *itD);
      }
      CPPUNIT_ASSERT(p.length() == dirs.size());
      
    }

    {
      size_t i = 0;

      girl::path::const_dir_iterator it2 = p2.begin();
      const girl::path::const_dir_iterator it2End = p.end();

      std::deque<girl::direction>::iterator it2D = dirs2.begin();
      const std::deque<girl::direction>::iterator it2DEnd = dirs2.end();
      for ( ; it2D != it2DEnd; ++it2D, ++it2) {
	if (it2 == it2End)
	  std::cerr<<"end prematuree 2"<<std::endl;
	if (*it2 != *it2D) {
	  std::cerr<<i<<"   *it2="<<*it2<<" *it2D="<<*it2D<<std::endl;
	}
	++i;
    

	CPPUNIT_ASSERT(it2 != it2End);
	CPPUNIT_ASSERT(*it2 == *it2D);
      }
      CPPUNIT_ASSERT(p2.length() == dirs2.size());
    }
  }



  const size_t pLength = p.length();
  const size_t p2Length = p2.length();

  p.append(p2);


  {
    girl::path::const_dir_iterator it = p.begin();
    const girl::path::const_dir_iterator itEnd = p.end();

    size_t i=0;

    std::deque<girl::direction>::iterator itD = dirs.begin();
    const std::deque<girl::direction>::iterator itDEnd = dirs.end();
    for ( ; itD != itDEnd; ++itD, ++it) {
      if (it == itEnd)
	std::cerr<<"end prematuree 1"<<std::endl;
      if (*it != *itD) {
	std::cerr<<i<<"   *it="<<*it<<" *itD="<<*itD<<std::endl;
      }
      ++i;
    

      CPPUNIT_ASSERT(it != itEnd);
      CPPUNIT_ASSERT(*it == *itD);
    }

    std::deque<girl::direction>::iterator it2D = dirs2.begin();
    const std::deque<girl::direction>::iterator it2DEnd = dirs2.end();
    for ( ; it2D != it2DEnd; ++it2D, ++it) {
      if (it == itEnd)
	std::cerr<<"end prematuree 2"<<std::endl;
      if (*it != *it2D) {
	std::cerr<<i<<"   *it="<<*it<<" *it2D="<<*it2D<<std::endl;
      }
      ++i;
    

      CPPUNIT_ASSERT(it != itEnd);
      CPPUNIT_ASSERT(*it == *it2D);
    }

    CPPUNIT_ASSERT(p.length() == dirs.size()+dirs2.size());
    CPPUNIT_ASSERT(p.length() == pLength+p2Length);
  }

}

void
PathTest::testSwap1()
{
  //test swap method 
  // two calls to swap should lead to same paths
  // whatever the order of the calls.

  P_init1(dirs);
  p.set(girl::point(0, 0), dirs);

  std::deque<girl::direction> dirs2;
  girl::path p2;
  P_init2(dirs2);
  p2.set(girl::point(7,-1), dirs2);

  p.swap(p2);
  p.swap(p2);

  P_testDirs(p, dirs);
  P_testDirs(p2, dirs2);

  p2.swap(p);
  p2.swap(p);

  P_testDirs(p, dirs);
  P_testDirs(p2, dirs2);
  
  p2.swap(p);
  p.swap(p2);

  P_testDirs(p, dirs);
  P_testDirs(p2, dirs2);
  
  p.swap(p2);
  p2.swap(p);

  P_testDirs(p, dirs);
  P_testDirs(p2, dirs2);
}

void
PathTest::testSwap2()
{
  //test swap method 
  // similar to testSwap1 but using std::swap instead.

  P_init1(dirs);
  p.set(girl::point(0, 0), dirs);

  std::deque<girl::direction> dirs2;
  girl::path p2;
  P_init2(dirs2);
  p2.set(girl::point(7,-1), dirs2);

  std::swap(p, p2);
  std::swap(p, p2);

  P_testDirs(p, dirs);
  P_testDirs(p2, dirs2);

  std::swap(p2, p);
  std::swap(p2, p);

  P_testDirs(p, dirs);
  P_testDirs(p2, dirs2);
  
  std::swap(p2, p);
  std::swap(p, p2);

  P_testDirs(p, dirs);
  P_testDirs(p2, dirs2);
  
  std::swap(p, p2);
  std::swap(p2, p);

  P_testDirs(p, dirs);
  P_testDirs(p2, dirs2);
}
