/*
  Test program for pending edge finder.
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#include "PendingEdgeFinderTest.hpp"


// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( PendingEdgeFinderTest );


void 
PendingEdgeFinderTest::setUp()
{
  m_edgeList.clear();
}


void 
PendingEdgeFinderTest::tearDown()
{

}

static
void
P_hasNot1(std::deque<girl::edge> &edgeList)
{
  //simple cycle
  //-> no pending edge

  edgeList.push_back(girl::edge(1, 7));
  edgeList.push_back(girl::edge(7, 9));
  edgeList.push_back(girl::edge(9, 1));
}

static
void
P_has1(std::deque<girl::edge> &edgeList)
{
  //-> pending edge

  edgeList.push_back(girl::edge(1, 7));
  edgeList.push_back(girl::edge(9, 2));
  edgeList.push_back(girl::edge(7, 9));
  edgeList.push_back(girl::edge(9, 2));
  edgeList.push_back(girl::edge(7, 1));
  edgeList.push_back(girl::edge(2, 9));
}

static
void
P_has2(std::deque<girl::edge> &edgeList)
{
  //simple cycle+pending edge
  //-> pending edge
  
  edgeList.push_back(girl::edge(1, 7));
  edgeList.push_back(girl::edge(7, 9));
  edgeList.push_back(girl::edge(9, 1));
  edgeList.push_back(girl::edge(7, 8));
}

static
void
P_hasNot2(std::deque<girl::edge> &edgeList)
{
  //double cycle
  //-> no pending edge

  edgeList.push_back(girl::edge(1, 7));
  edgeList.push_back(girl::edge(1, 7));
  edgeList.push_back(girl::edge(7, 9));
  edgeList.push_back(girl::edge(9, 1));
  edgeList.push_back(girl::edge(9, 1));
  edgeList.push_back(girl::edge(9, 7));
  edgeList.push_back(girl::edge(9, 1));
  edgeList.push_back(girl::edge(9, 7));
}

static
void
P_hasNot3(std::deque<girl::edge> &edgeList)
{
  //-> no pending edge

  edgeList.push_back(girl::edge(10, 7));
  edgeList.push_back(girl::edge(11, 9));
  edgeList.push_back(girl::edge(4, 11));
  edgeList.push_back(girl::edge(4, 11));
  edgeList.push_back(girl::edge(9, 3));
  edgeList.push_back(girl::edge(3, 9));
  edgeList.push_back(girl::edge(4, 3));
  edgeList.push_back(girl::edge(4, 7));
  edgeList.push_back(girl::edge(4, 7));
  edgeList.push_back(girl::edge(3, 7));
  edgeList.push_back(girl::edge(10, 7));
}
  
static
void
P_has3(std::deque<girl::edge> &edgeList)
{
  //-> pending edge

  edgeList.push_back(girl::edge(1, 7));
  edgeList.push_back(girl::edge(7, 9));
  edgeList.push_back(girl::edge(9, 2));
  edgeList.push_back(girl::edge(4, 8));
  edgeList.push_back(girl::edge(5, 1));
  edgeList.push_back(girl::edge(7, 9));
  edgeList.push_back(girl::edge(6, 2));
  edgeList.push_back(girl::edge(2, 9));
  edgeList.push_back(girl::edge(9, 6));
  edgeList.push_back(girl::edge(3, 9));
  edgeList.push_back(girl::edge(4, 5));
  edgeList.push_back(girl::edge(5, 2));
  edgeList.push_back(girl::edge(5, 4));
  edgeList.push_back(girl::edge(4, 3));
  edgeList.push_back(girl::edge(7, 1));
  edgeList.push_back(girl::edge(5, 2));
}

static
void
P_hasNot4(std::deque<girl::edge> &edgeList)
{
  //-> no pending edge

  edgeList.push_back(girl::edge(1, 2));
  edgeList.push_back(girl::edge(2, 3));
  edgeList.push_back(girl::edge(3, 4));
  edgeList.push_back(girl::edge(4, 5));
  edgeList.push_back(girl::edge(5, 4));
  edgeList.push_back(girl::edge(4, 2));
  edgeList.push_back(girl::edge(2, 1));
}

static
void
P_has4(std::deque<girl::edge> &edgeList)
{
  //-> pending edge

  edgeList.push_back(girl::edge(1, 2));
  edgeList.push_back(girl::edge(2, 3));
  edgeList.push_back(girl::edge(3, 4));
  edgeList.push_back(girl::edge(4, 5));
  edgeList.push_back(girl::edge(5, 3));
  edgeList.push_back(girl::edge(3, 6));
  edgeList.push_back(girl::edge(3, 4));
  edgeList.push_back(girl::edge(6, 7));
  edgeList.push_back(girl::edge(7, 3));
  edgeList.push_back(girl::edge(3, 6));
  edgeList.push_back(girl::edge(2, 8));
  edgeList.push_back(girl::edge(8, 9));
  edgeList.push_back(girl::edge(2, 10));
  edgeList.push_back(girl::edge(1, 2));
  edgeList.push_back(girl::edge(9, 2));
  edgeList.push_back(girl::edge(2, 9));
  edgeList.push_back(girl::edge(1, 10));
}


static
void
P_hasNot5(std::deque<girl::edge> &edgeList)
{
  //-> no pending edge
  // (only triangles, no double edges)

  /*
    //simpler version
  edgeList.push_back(girl::edge(1, 2));
  edgeList.push_back(girl::edge(2, 3));
  edgeList.push_back(girl::edge(3, 4));
  edgeList.push_back(girl::edge(1, 3));
  edgeList.push_back(girl::edge(1, 4));
  edgeList.push_back(girl::edge(4, 5));
  edgeList.push_back(girl::edge(5, 1));
  edgeList.push_back(girl::edge(1, 14));
  edgeList.push_back(girl::edge(14, 5));
  edgeList.push_back(girl::edge(14, 13));
  edgeList.push_back(girl::edge(13, 1));
  edgeList.push_back(girl::edge(12, 2));
  edgeList.push_back(girl::edge(12, 13));
  */
  
  edgeList.push_back(girl::edge(1, 2));
  edgeList.push_back(girl::edge(2, 3));
  edgeList.push_back(girl::edge(3, 10));
  edgeList.push_back(girl::edge(3, 8));
  edgeList.push_back(girl::edge(3, 9));
  edgeList.push_back(girl::edge(3, 4));
  edgeList.push_back(girl::edge(10, 11));
  edgeList.push_back(girl::edge(10, 2));
  edgeList.push_back(girl::edge(1, 3));
  edgeList.push_back(girl::edge(1, 4));
  edgeList.push_back(girl::edge(4, 5));
  edgeList.push_back(girl::edge(5, 1));
  edgeList.push_back(girl::edge(1, 14));
  edgeList.push_back(girl::edge(14, 5));
  edgeList.push_back(girl::edge(4, 6));
  edgeList.push_back(girl::edge(5, 6));
  edgeList.push_back(girl::edge(4, 8));
  edgeList.push_back(girl::edge(8, 7));
  edgeList.push_back(girl::edge(7, 4));
  edgeList.push_back(girl::edge(14, 13));
  edgeList.push_back(girl::edge(13, 1));
  edgeList.push_back(girl::edge(8, 9));
  edgeList.push_back(girl::edge(9, 10));
  edgeList.push_back(girl::edge(11, 2));
  edgeList.push_back(girl::edge(12, 2));
  edgeList.push_back(girl::edge(12, 13));
  
}

static
void
P_has5(std::deque<girl::edge> &edgeList)
{
  //-> pending edge
  // (only triangles, no double edges, but one incomplete triangle)

  edgeList.push_back(girl::edge(1, 2));
  edgeList.push_back(girl::edge(2, 3));
  edgeList.push_back(girl::edge(3, 10));
  edgeList.push_back(girl::edge(3, 8));
  edgeList.push_back(girl::edge(3, 4));
  edgeList.push_back(girl::edge(10, 11));
  edgeList.push_back(girl::edge(10, 2));
  edgeList.push_back(girl::edge(1, 3));
  edgeList.push_back(girl::edge(1, 4));
  edgeList.push_back(girl::edge(4, 5));
  edgeList.push_back(girl::edge(5, 1));
  edgeList.push_back(girl::edge(1, 14));
  edgeList.push_back(girl::edge(14, 5));
  edgeList.push_back(girl::edge(4, 6));
  edgeList.push_back(girl::edge(5, 6));
  edgeList.push_back(girl::edge(4, 8));
  edgeList.push_back(girl::edge(8, 7));
  edgeList.push_back(girl::edge(7, 4));
  edgeList.push_back(girl::edge(14, 13));
  edgeList.push_back(girl::edge(13, 1));
  edgeList.push_back(girl::edge(9, 10));
  edgeList.push_back(girl::edge(11, 2));
  edgeList.push_back(girl::edge(12, 2));
  edgeList.push_back(girl::edge(12, 13));
}

static
void
P_hasNot6(std::deque<girl::edge> &edgeList)
{
  //-> no pending edge
  // (only triangles, only double edges)

  edgeList.push_back(girl::edge(1, 2));
  edgeList.push_back(girl::edge(2, 3));
  edgeList.push_back(girl::edge(3, 10));
  edgeList.push_back(girl::edge(3, 8));
  edgeList.push_back(girl::edge(3, 9));
  edgeList.push_back(girl::edge(3, 4));
  edgeList.push_back(girl::edge(10, 11));
  edgeList.push_back(girl::edge(10, 2));
  edgeList.push_back(girl::edge(1, 3));
  edgeList.push_back(girl::edge(1, 4));
  edgeList.push_back(girl::edge(4, 5));
  edgeList.push_back(girl::edge(5, 1));
  edgeList.push_back(girl::edge(1, 14));
  edgeList.push_back(girl::edge(14, 5));
  edgeList.push_back(girl::edge(4, 6));
  edgeList.push_back(girl::edge(5, 6));
  edgeList.push_back(girl::edge(4, 8));
  edgeList.push_back(girl::edge(8, 7));
  edgeList.push_back(girl::edge(7, 4));
  edgeList.push_back(girl::edge(14, 13));
  edgeList.push_back(girl::edge(13, 1));
  edgeList.push_back(girl::edge(8, 9));
  edgeList.push_back(girl::edge(9, 10));
  edgeList.push_back(girl::edge(11, 2));
  edgeList.push_back(girl::edge(12, 2));
  edgeList.push_back(girl::edge(12, 13));

  edgeList.push_back(girl::edge(10, 11));
  edgeList.push_back(girl::edge(10, 2));
  edgeList.push_back(girl::edge(1, 3));
  edgeList.push_back(girl::edge(1, 4));
  edgeList.push_back(girl::edge(4, 5));
  edgeList.push_back(girl::edge(5, 1));
  edgeList.push_back(girl::edge(1, 14));
  edgeList.push_back(girl::edge(14, 5));
  edgeList.push_back(girl::edge(4, 6));
  edgeList.push_back(girl::edge(5, 6));
  edgeList.push_back(girl::edge(4, 8));
  edgeList.push_back(girl::edge(8, 7));
  edgeList.push_back(girl::edge(7, 4));
  edgeList.push_back(girl::edge(14, 13));
  edgeList.push_back(girl::edge(13, 1));
  edgeList.push_back(girl::edge(8, 9));
  edgeList.push_back(girl::edge(9, 10));
  edgeList.push_back(girl::edge(11, 2));
  edgeList.push_back(girl::edge(12, 2));
  edgeList.push_back(girl::edge(12, 13));
  edgeList.push_back(girl::edge(1, 2));
  edgeList.push_back(girl::edge(2, 3));
  edgeList.push_back(girl::edge(3, 10));
  edgeList.push_back(girl::edge(3, 8));
  edgeList.push_back(girl::edge(3, 9));
  edgeList.push_back(girl::edge(3, 4));
}

static
void
P_has6(std::deque<girl::edge> &edgeList)
{
  //-> pending edge
  // (only triangles, only double edges but pending edge)

  edgeList.push_back(girl::edge(1, 2));
  edgeList.push_back(girl::edge(2, 3));
  edgeList.push_back(girl::edge(3, 10));
  edgeList.push_back(girl::edge(3, 8));
  edgeList.push_back(girl::edge(3, 9));
  edgeList.push_back(girl::edge(3, 4));
  edgeList.push_back(girl::edge(10, 11));
  edgeList.push_back(girl::edge(10, 2));
  edgeList.push_back(girl::edge(1, 3));
  edgeList.push_back(girl::edge(1, 4));
  edgeList.push_back(girl::edge(4, 5));
  edgeList.push_back(girl::edge(5, 1));
  edgeList.push_back(girl::edge(4, 6));
  edgeList.push_back(girl::edge(5, 6));
  edgeList.push_back(girl::edge(4, 8));
  edgeList.push_back(girl::edge(8, 7));
  edgeList.push_back(girl::edge(7, 4));
  edgeList.push_back(girl::edge(14, 13));
  edgeList.push_back(girl::edge(13, 1));
  edgeList.push_back(girl::edge(8, 9));
  edgeList.push_back(girl::edge(9, 10));
  edgeList.push_back(girl::edge(11, 2));
  edgeList.push_back(girl::edge(12, 2));
  edgeList.push_back(girl::edge(12, 13));

  edgeList.push_back(girl::edge(10, 11));
  edgeList.push_back(girl::edge(10, 2));
  edgeList.push_back(girl::edge(1, 3));
  edgeList.push_back(girl::edge(1, 4));
  edgeList.push_back(girl::edge(4, 5));
  edgeList.push_back(girl::edge(5, 1));
  edgeList.push_back(girl::edge(4, 6));
  edgeList.push_back(girl::edge(5, 6));
  edgeList.push_back(girl::edge(4, 8));
  edgeList.push_back(girl::edge(8, 7));
  edgeList.push_back(girl::edge(7, 4));
  edgeList.push_back(girl::edge(13, 1));
  edgeList.push_back(girl::edge(8, 9));
  edgeList.push_back(girl::edge(9, 10));
  edgeList.push_back(girl::edge(11, 2));
  edgeList.push_back(girl::edge(12, 2));
  edgeList.push_back(girl::edge(12, 13));
  edgeList.push_back(girl::edge(1, 2));
  edgeList.push_back(girl::edge(2, 3));
  edgeList.push_back(girl::edge(3, 10));
  edgeList.push_back(girl::edge(3, 8));
  edgeList.push_back(girl::edge(3, 9));
  edgeList.push_back(girl::edge(3, 4));
}


static
void
P_hasNot7(std::deque<girl::edge> &edgeList)
{
  //-> no pending edge
  //(4 branches cross)

  edgeList.push_back(girl::edge(1, 2));
  edgeList.push_back(girl::edge(2, 3));
  edgeList.push_back(girl::edge(3, 6));
  edgeList.push_back(girl::edge(5, 6));
  edgeList.push_back(girl::edge(9, 5));
  edgeList.push_back(girl::edge(5, 4));
  edgeList.push_back(girl::edge(4, 8));
  edgeList.push_back(girl::edge(9, 8));
  edgeList.push_back(girl::edge(4, 7));
  edgeList.push_back(girl::edge(1, 4));
  edgeList.push_back(girl::edge(1, 10));
  edgeList.push_back(girl::edge(10, 7));
  edgeList.push_back(girl::edge(1, 11));
  edgeList.push_back(girl::edge(11, 12));
  edgeList.push_back(girl::edge(12, 2));
}
  
static
void
P_has7(std::deque<girl::edge> &edgeList)
{
  //-> pending edge
  //(4 branches cross + pending edge)

  edgeList.push_back(girl::edge(1, 2));
  edgeList.push_back(girl::edge(2, 3));
  edgeList.push_back(girl::edge(3, 6));
  edgeList.push_back(girl::edge(5, 6));
  edgeList.push_back(girl::edge(9, 5));
  edgeList.push_back(girl::edge(5, 4));
  edgeList.push_back(girl::edge(4, 8));
  edgeList.push_back(girl::edge(9, 8));
  edgeList.push_back(girl::edge(4, 7));
  edgeList.push_back(girl::edge(1, 4));
  edgeList.push_back(girl::edge(1, 10));
  edgeList.push_back(girl::edge(7, 13));
  edgeList.push_back(girl::edge(10, 7));
  edgeList.push_back(girl::edge(1, 11));
  edgeList.push_back(girl::edge(11, 12));
  edgeList.push_back(girl::edge(12, 2));
  edgeList.push_back(girl::edge(14, 13));
  edgeList.push_back(girl::edge(14, 15));
}
  
static
void
P_has8(std::deque<girl::edge> &edgeList)
{
  //-> pending edge

  edgeList.push_back(girl::edge(1, 2));
  edgeList.push_back(girl::edge(2, 3));
  edgeList.push_back(girl::edge(1, 4));
  edgeList.push_back(girl::edge(4, 5));
  edgeList.push_back(girl::edge(1, 6));
  edgeList.push_back(girl::edge(6, 7));
  edgeList.push_back(girl::edge(1, 8));
  edgeList.push_back(girl::edge(8, 9));
}
  
static
void
P_hasNot8(std::deque<girl::edge> &edgeList)
{
  //-> no pending edge

  edgeList.push_back(girl::edge(1, 2));
  edgeList.push_back(girl::edge(2, 3));
  edgeList.push_back(girl::edge(1, 4));
  edgeList.push_back(girl::edge(4, 5));
  edgeList.push_back(girl::edge(1, 6));
  edgeList.push_back(girl::edge(6, 7));
  edgeList.push_back(girl::edge(1, 8));
  edgeList.push_back(girl::edge(8, 9));

  edgeList.push_back(girl::edge(3, 9));
  edgeList.push_back(girl::edge(5, 7));
}
  
static
void
P_hasNot9(std::deque<girl::edge> &edgeList)
{
  //-> no pending edge

  edgeList.push_back(girl::edge(11, 9));
  edgeList.push_back(girl::edge(6, 9));
  edgeList.push_back(girl::edge(10, 9));
  edgeList.push_back(girl::edge(6, 5));
  edgeList.push_back(girl::edge(5, 4));
  edgeList.push_back(girl::edge(4, 3));
  edgeList.push_back(girl::edge(4, 8));
  edgeList.push_back(girl::edge(6, 7));
  edgeList.push_back(girl::edge(7, 13));
  edgeList.push_back(girl::edge(7, 5));
  edgeList.push_back(girl::edge(13, 7));
  edgeList.push_back(girl::edge(3, 8));
  edgeList.push_back(girl::edge(13, 6));
  edgeList.push_back(girl::edge(3, 2));
  edgeList.push_back(girl::edge(14, 2));
  edgeList.push_back(girl::edge(14, 12));
  edgeList.push_back(girl::edge(1, 12));
  edgeList.push_back(girl::edge(1, 11));
  edgeList.push_back(girl::edge(11, 10));
  edgeList.push_back(girl::edge(1, 2));
  edgeList.push_back(girl::edge(5, 4));
  edgeList.push_back(girl::edge(11, 10));
}

static
void
P_has9(std::deque<girl::edge> &edgeList)
{
  //-> pending edge  

  edgeList.push_back(girl::edge(3, 9));
  edgeList.push_back(girl::edge(3, 7));
  edgeList.push_back(girl::edge(3, 2));
  edgeList.push_back(girl::edge(2, 1));
  edgeList.push_back(girl::edge(1, 4));
  edgeList.push_back(girl::edge(11, 4));
  edgeList.push_back(girl::edge(6, 8));
  edgeList.push_back(girl::edge(8, 13));
  edgeList.push_back(girl::edge(10, 13));
  edgeList.push_back(girl::edge(10, 6));
  edgeList.push_back(girl::edge(2, 6));
  edgeList.push_back(girl::edge(12, 9));
  edgeList.push_back(girl::edge(12, 7));
  edgeList.push_back(girl::edge(11, 5));
  edgeList.push_back(girl::edge(1, 5));
}

static
void
P_hasNot10(std::deque<girl::edge> &edgeList)
{
  //-> no pending edge
  // (several connex components)
  
  edgeList.push_back(girl::edge(11, 9));
  edgeList.push_back(girl::edge(5, 7));
  edgeList.push_back(girl::edge(4, 6));
  edgeList.push_back(girl::edge(8, 12));
  edgeList.push_back(girl::edge(12, 1));
  edgeList.push_back(girl::edge(2, 3));
  edgeList.push_back(girl::edge(5, 11));
  edgeList.push_back(girl::edge(6, 2));
  edgeList.push_back(girl::edge(1, 8));
  edgeList.push_back(girl::edge(1, 8));
  edgeList.push_back(girl::edge(9, 7));
  edgeList.push_back(girl::edge(4, 3));

}

static
void
P_has10(std::deque<girl::edge> &edgeList)
{
  //-> pending edge
  // (several separated edges)

  edgeList.push_back(girl::edge(11, 9));
  edgeList.push_back(girl::edge(5, 7));
  edgeList.push_back(girl::edge(4, 6));
}

static
void
P_hasNot11(std::deque<girl::edge> &edgeList)
{
  //-> no pending edge
  //(3*3 4 edges faces)

  edgeList.push_back(girl::edge(12, 13));
  edgeList.push_back(girl::edge(13, 4));
  edgeList.push_back(girl::edge(13, 1));
  edgeList.push_back(girl::edge(1, 16));
  edgeList.push_back(girl::edge(15, 16));
  edgeList.push_back(girl::edge(15, 14));
  edgeList.push_back(girl::edge(14, 3));
  edgeList.push_back(girl::edge(3, 2));
  edgeList.push_back(girl::edge(11, 2));
  edgeList.push_back(girl::edge(11, 8));
  edgeList.push_back(girl::edge(8, 3));
  edgeList.push_back(girl::edge(14, 9));
  edgeList.push_back(girl::edge(6, 9));
  edgeList.push_back(girl::edge(10, 6));
  edgeList.push_back(girl::edge(10, 5));
  edgeList.push_back(girl::edge(5, 4));
  edgeList.push_back(girl::edge(4, 16));
  edgeList.push_back(girl::edge(16, 10));
  edgeList.push_back(girl::edge(15, 6));
  edgeList.push_back(girl::edge(15, 2));
  edgeList.push_back(girl::edge(2, 1));
  edgeList.push_back(girl::edge(1, 7));
  edgeList.push_back(girl::edge(7, 12));
  edgeList.push_back(girl::edge(11, 7));
}

static
void
P_has11(std::deque<girl::edge> &edgeList)
{
  //-> pending edge

  edgeList.push_back(girl::edge(1, 2));
  edgeList.push_back(girl::edge(2, 3));
  edgeList.push_back(girl::edge(3, 4));
  edgeList.push_back(girl::edge(3, 5));
  edgeList.push_back(girl::edge(1, 5));
  edgeList.push_back(girl::edge(2, 5));
  edgeList.push_back(girl::edge(6, 7));
  edgeList.push_back(girl::edge(8, 7));
  edgeList.push_back(girl::edge(8, 9));
  edgeList.push_back(girl::edge(6, 5));
  edgeList.push_back(girl::edge(6, 9));
}

static
void
P_hasNot12(std::deque<girl::edge> &edgeList)
{
  //-> no pending edge

  edgeList.push_back(girl::edge(1, 2));
  edgeList.push_back(girl::edge(1, 5));
  edgeList.push_back(girl::edge(2, 5));
  edgeList.push_back(girl::edge(2, 3));
  edgeList.push_back(girl::edge(3, 4));
  edgeList.push_back(girl::edge(3, 5));
  edgeList.push_back(girl::edge(4, 5));
  edgeList.push_back(girl::edge(3, 5));
  edgeList.push_back(girl::edge(4, 1));
  edgeList.push_back(girl::edge(1, 5));
  edgeList.push_back(girl::edge(2, 5));
  edgeList.push_back(girl::edge(4, 5));
}

static
void
P_has12(std::deque<girl::edge> &edgeList)
{
  //-> pending edge

  edgeList.push_back(girl::edge(1, 2));
  edgeList.push_back(girl::edge(2, 3));
  edgeList.push_back(girl::edge(1, 2));
  edgeList.push_back(girl::edge(4, 3));
  edgeList.push_back(girl::edge(1, 2));
  edgeList.push_back(girl::edge(1, 2));
  edgeList.push_back(girl::edge(4, 5));
  edgeList.push_back(girl::edge(6, 5));
  edgeList.push_back(girl::edge(6, 5));
  edgeList.push_back(girl::edge(6, 5));
  edgeList.push_back(girl::edge(6, 7));
}

void
PendingEdgeFinderTest::testHas(void (*initFct) (std::deque<girl::edge> &))
{
  initFct(m_edgeList);
  const bool hasPendingEdge = m_finder.find(m_edgeList);
  CPPUNIT_ASSERT(hasPendingEdge); 
}

void
PendingEdgeFinderTest::testHasNot(void (*initFct) (std::deque<girl::edge> &))
{
  initFct(m_edgeList);
  const bool hasPendingEdge = m_finder.find(m_edgeList);
  CPPUNIT_ASSERT(! hasPendingEdge); 
}


void 
PendingEdgeFinderTest::testHas1()
{
  testHas(P_has1);
}

void 
PendingEdgeFinderTest::testHasNot1()
{
  testHasNot(P_hasNot1);
}

void 
PendingEdgeFinderTest::testHas2()
{
  testHas(P_has2);
}

void 
PendingEdgeFinderTest::testHasNot2()
{
  testHasNot(P_hasNot2);
}

void 
PendingEdgeFinderTest::testHas3()
{
  testHas(P_has3);
}

void 
PendingEdgeFinderTest::testHasNot3()
{
  testHasNot(P_hasNot3);
}

void 
PendingEdgeFinderTest::testHas4()
{
  testHas(P_has4);
}

void 
PendingEdgeFinderTest::testHasNot4()
{
  testHasNot(P_hasNot4);
}

void 
PendingEdgeFinderTest::testHas5()
{
  testHas(P_has5);
}

void 
PendingEdgeFinderTest::testHasNot5()
{
  testHasNot(P_hasNot5);
}

void 
PendingEdgeFinderTest::testHas6()
{
  testHas(P_has6);
}

void 
PendingEdgeFinderTest::testHasNot6()
{
  testHasNot(P_hasNot6);
}

void 
PendingEdgeFinderTest::testHas7()
{
  testHas(P_has7);
}

void 
PendingEdgeFinderTest::testHasNot7()
{
  testHasNot(P_hasNot7);
}

void 
PendingEdgeFinderTest::testHas8()
{
  testHas(P_has8);
}

void 
PendingEdgeFinderTest::testHasNot8()
{
  testHasNot(P_hasNot8);
}

void 
PendingEdgeFinderTest::testHas9()
{
  testHas(P_has9);
}

void 
PendingEdgeFinderTest::testHasNot9()
{
  testHasNot(P_hasNot9);
}

void 
PendingEdgeFinderTest::testHas10()
{
  testHas(P_has10);
}

void 
PendingEdgeFinderTest::testHasNot10()
{
  testHasNot(P_hasNot10);
}

void 
PendingEdgeFinderTest::testHas11()
{
  testHas(P_has11);
}

void 
PendingEdgeFinderTest::testHasNot11()
{
  testHasNot(P_hasNot11);
}

void 
PendingEdgeFinderTest::testHas12()
{
  testHas(P_has12);
}

void 
PendingEdgeFinderTest::testHasNot12()
{
  testHasNot(P_hasNot12);
}











