/*
  Test program for 2D point class.
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#include "PointTest.hpp"

// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( PointTest );

void 
PointTest::setUp()
{
}


void 
PointTest::tearDown()
{
}

void 
PointTest::testSet()
{
  const int val1 = 10;

  girl::point p1(val1, val1);
  CPPUNIT_ASSERT(p1.x() == val1 && p1.y() == val1);

  const int val2 = -1;

  p1.set(val2, val2);
  CPPUNIT_ASSERT(p1.x() == val2 && p1.y() == val2);

  p1.setX(val1);
  p1.setY(val1);
  CPPUNIT_ASSERT(p1.x() == val1 && p1.y() == val1);

  girl::point p2 = p1;
  CPPUNIT_ASSERT(p2.x() == val1 && p2.y() == val1);
}

void 
PointTest::testEquality()
{
  const int val1 = 10;
  const int val2 = -1;
  
  girl::point p1(val1, val1);
  girl::point p2(val1, val1);
  CPPUNIT_ASSERT(p1.x() == p2.x() && p1.y() == p2.y());
  CPPUNIT_ASSERT(p1 == p2);
  
  p2 = girl::point(val2, val1);
  CPPUNIT_ASSERT(p1.x() != p2.x() && p1.y() == p2.y());
  CPPUNIT_ASSERT(p1 != p2);
  
  p2 = girl::point(val1, val2);
  CPPUNIT_ASSERT(p1.x() == p2.x() && p1.y() != p2.y());
  CPPUNIT_ASSERT(p1 != p2);
}

void
PointTest::testMove()
{
  const int valX = -1;
  const int valY = 0;
  girl::point p(valX, valY);

  for (int i=0; i<4; ++i) {
    girl::direction d( static_cast<girl::eDirection>(i) );
    CPPUNIT_ASSERT(d.opposite().opposite() == d);

    girl::point p2 = p;
    p2.move(d);
    p2.move(d.opposite());
    CPPUNIT_ASSERT(p2 == p);
    p2.move(d.opposite());
    p2.move(d);
    CPPUNIT_ASSERT(p2 == p);
  }
}

void
PointTest::testDirection()
{
  const int valX = -1;
  const int valY = 0;
  girl::point p(valX, valY);

  for (int i=0; i<4; ++i) {
    girl::direction d( static_cast<girl::eDirection>(i) );
    
    girl::point p2 = p;
    p2.move(d);
    CPPUNIT_ASSERT(p.directionTo(p2) == d);
    CPPUNIT_ASSERT(p2.directionTo(p) == d.opposite());

    for (int j=0; j<4; ++j) {
      girl::direction d2( static_cast<girl::eDirection>(i) );
      if (d2 != d.opposite()) {
	girl::point p3 = p;
	p3.move(d);
	p3.move(d2);
	CPPUNIT_ASSERT(p.directionTo(p3) == girl::INVALID);
	CPPUNIT_ASSERT(p3.directionTo(p) == girl::INVALID);
      }
    }
  }

}

