/*
  Test program for swap operation in toger.
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#include "SwapTest.hpp"

#include "same_regions.hpp"

// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( SwapTest );

#define DEFAULT_WIDTH 31



void
SwapTest::testEmptyToger()
{
  //girl::toger tog(width, height);
  
  //TODO: all pts non swapable ???
}

void
SwapTest::testChessboard()
{
  sameRegion_chessboard f;

  const girl::POINT_TYPE width = DEFAULT_WIDTH;
  const girl::POINT_TYPE height = DEFAULT_WIDTH;
  
  girl::toger tog(width, height);

  girl::regionDeque regs;
  tog.splitRegion(1, f, regs);

  for (girl::POINT_TYPE y=0; y<height-1; ++y)
    for (girl::POINT_TYPE x=0; x<width-1; ++x) {
      CPPUNIT_ASSERT(! tog.swapable(girl::point(x, y)));
    }
  
}

void
SwapTest::testSimpleLoop()
{
    
  const girl::POINT_TYPE width = 10;

  girl::toger tog(width, width);
  
  girl::point startPt(1, 1);

  std::deque<girl::direction> dirs;
  dirs.push_back(girl::RIGHT);
  dirs.push_back(girl::DOWN);
  dirs.push_back(girl::LEFT);
  dirs.push_back(girl::UP);
  
  girl::path p(startPt, dirs);

  girl::pathDeque paths;
  paths.push_back(p);

  girl::regionDeque newRegions;
  girl::regionDeque updatedRegions;
  const bool res = tog.insertPaths(paths, newRegions, updatedRegions);

  CPPUNIT_ASSERT(res);
  CPPUNIT_ASSERT(newRegions.size() == 1);
  CPPUNIT_ASSERT(updatedRegions.size() == 1);
  const girl::face f = tog.firstChild(1);
  CPPUNIT_ASSERT(f != 0);

  //const girl::region r2 = tog.lambda(tog.alpha(tog.beta(f)));

  girl::point pt;
  
  pt.set(startPt.x()+1, startPt.y()+1);
  CPPUNIT_ASSERT(! tog.swapable(pt));
  
  //first check 

  pt.set(startPt.x()+1, startPt.y());
  CPPUNIT_ASSERT(tog.swapable(pt));
    
  pt.set(startPt.x()+1, startPt.y()+2);
  CPPUNIT_ASSERT(tog.swapable(pt));

  pt.set(startPt.x(), startPt.y()+1);
  CPPUNIT_ASSERT(tog.swapable(pt));

  pt.set(startPt.x()+2, startPt.y()+1);
  CPPUNIT_ASSERT(tog.swapable(pt));

  //- 1st swap 

  pt.set(startPt.x()+1, startPt.y());
  tog.swap(pt);
  
  pt.set(startPt.x()+1, startPt.y()+2);
  tog.swap(pt);
    
  pt.set(startPt.x(), startPt.y()+1);
  tog.swap(pt);

  pt.set(startPt.x()+2, startPt.y()+1);
  tog.swap(pt);

  //second check 

  pt.set(startPt.x()+1, startPt.y());
  CPPUNIT_ASSERT(tog.swapable(pt));
    
  pt.set(startPt.x()+1, startPt.y()+2);
  CPPUNIT_ASSERT(tog.swapable(pt));

  pt.set(startPt.x(), startPt.y()+1);
  CPPUNIT_ASSERT(tog.swapable(pt));

  pt.set(startPt.x()+2, startPt.y()+1);
  CPPUNIT_ASSERT(tog.swapable(pt));

  //- 2nd swap 

  pt.set(startPt.x()+1, startPt.y());
  tog.swap(pt);
  
  pt.set(startPt.x()+1, startPt.y()+2);
  tog.swap(pt);
    
  pt.set(startPt.x(), startPt.y()+1);
  tog.swap(pt);

  pt.set(startPt.x()+2, startPt.y()+1);
  tog.swap(pt);



  pt.set(startPt.x()+1, startPt.y()+1);
  CPPUNIT_ASSERT(! tog.swapable(pt));

}
