/*
  Functions to compute information on regions of a toger, related to OpenCV.
  Copyright (C) 2004-2011 Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include <opencv2/core/core.hpp>
#include <girl/girlToger.hpp>
#include <girlTools/girlToolsAABBox.hpp>

namespace girl {

  namespace tools {

    /**
     * @brief Compute Axis-Aligned Bounding Box of mask @a mask.
     *
     * Pixels in mask are non-zero.
     *
     * @warning if all pixels are zero in mask, return an invalid bounding box.
     * @warning @a mask must be of type CV_8U and have only one channel.
     */
    extern GIRL_API girl::tools::AABBox getAABBox(const cv::Mat &mask);

    /**
     * @brief Get regions @a regions in toger @a tog enclosed in mask @a mask.
     *
     * @warning @a mask must be of type CV_8U and have only one channel.
     */
    extern GIRL_API void getRegionsInMask(const girl::toger &tog, const cv::Mat &mask, girl::regionDeque &regions);

  }

}

