/*
  Functions to load/save toger to disk.
  Copyright (C) 2004-2009 Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "girlToolsDiskBackup.hpp"

#include <exception>

#include <iostream>

using namespace girl::tools;


oDiskBackup::oDiskBackup(const std::string &filename)
  : m_file(filename.c_str(), std::ios_base::out | std::ios_base::binary)
{
  if (! m_file) {
    std::cerr<<"Unable to open file: "<<filename<<std::endl;
    throw std::exception();
  }

}

girl::io::writeFlags
oDiskBackup::flags() const
{
  return girl::io::DEFAULT;
}



size_t
oDiskBackup::write(const void *p, size_t size)
{
  m_file.write(static_cast<const char *>(p), size);
  if (m_file.fail()) {
    //std::cerr<<"oDiskBackup::write() of "<<size<<" bytes failed !"<<std::endl;
    return 0;
  }
  return size;
}
  
bool
oDiskBackup::end()
{
  m_file.flush();
  if (m_file.fail()) {
    //std::cerr<<"oDiskBackup::end() flush of file failed !"<<std::endl;
    return false;
  }
  return true;
}



iDiskBackup::iDiskBackup(const std::string &filename)
  : m_file(filename.c_str(), std::ios_base::in | std::ios_base::binary)
{
  if (! m_file)
    throw std::exception();
}

girl::io::writeFlags
iDiskBackup::flags() const
{
  return girl::io::DEFAULT;
}

size_t
iDiskBackup::read(void *p, size_t size)
{
  m_file.read(static_cast<char *>(p), size);
  return m_file.gcount();
}
