/*
  Painter of labels on image.
  Copyright (C) 2004-2009 Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "girlToolsLabelImagePainter.hpp"

using namespace girl::tools;

namespace {
  class RegionFiller : public girl::toger::pointIterator
  {
  public:

    RegionFiller(girl::tools::Image &img, const girl::tools::Pixel &color)
      : m_img(img), 
	m_color(color)
    {
    }
  
    void processPoint(const girl::point &pt)
    {
      m_img.setPixel(pt.x(), pt.y(), m_color);
    }

  protected:
    girl::tools::Image &m_img;
    girl::tools::Pixel m_color;
  };
}

void
LabelImagePainter::fillRegion(const girl::toger &tog, 
			 girl::region r,
			 girl::tools::Pixel color,
			 girl::tools::Image &img)
{
  RegionFiller rf(img, color);
  tog.iterateOnRegionDomain(r, rf); 
}

void
LabelImagePainter::drawTogerInFalseColors(const girl::toger &tog, girl::tools::Image &img)
{
  girl::regionDeque regions;
  tog.allRegions(regions);

  girl::regionDeque::const_iterator it = regions.begin();
  const girl::regionDeque::const_iterator itEnd = regions.end(); 
  int label = 1;
  girl::tools::Pixel color;
  for ( ; it != itEnd; ++it) {
    setLabel(label, color);
    fillRegion(tog, *it, color, img);
    ++label;
  }
}

void
LabelImagePainter::setLabel(int label, girl::tools::Pixel &p)
{
  const unsigned char r = ((label >> 0) & 0xffff);
  const unsigned char g = ((label >> 8) & 0xffff);
  const unsigned char b = ((label >> 16) & 0xffff);
  const unsigned char a = ((label >> 24) & 0xffff);
  
  p = girl::tools::Pixel(r, g, b, a);
}

int
LabelImagePainter::getLabel(const girl::tools::Pixel &p)
{
  const int res = ( (p.a()) | (p.b()<<8) | (p.c()<<16) | (p.d()<<24) );
  return res;
}
