/*
  Painter of labels on image.
  Copyright (C) 2004-2009 Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef GIRLTOOLS_LABELIMAGEPAINTER_HPP
#define GIRLTOOLS_LABELIMAGEPAINTER_HPP

#include <girl/girlCommon.hpp>
#include <girl/girlToger.hpp>
#include <girlTools/girlToolsImage.hpp>


namespace girl {

  namespace tools {

    class GIRL_API LabelImagePainter
    {
      public:

      /**
       * @brief Fill all regions of toger @tog in image @a img
       * with a label.
       *
       * It is then possible to retrieve label from a pixel with @a getLabel.
       *
       * @sa fillRegion.
       * @sa setLabel.
       * @sa getLabel.
       */
      void drawTogerInFalseColors(const girl::toger &tog, 
				  girl::tools::Image &img);
      
      /**
       * @brief Fill region @a r of toger @a tog with color @a color
       * in image @a img.
       */
      void fillRegion(const girl::toger &tog, 
		      girl::region r, 
		      girl::tools::Pixel color,
		      girl::tools::Image &img);
      
      /**
       * @brief Set label @a label in pixel @p.
       */
      void setLabel(int label, girl::tools::Pixel &p);

      /**
       * @brief Retrieve label from a pixel @a p.
       */
      int getLabel(const girl::tools::Pixel &p);

      protected:

    };

  }
}

#endif /* ! GIRLTOOLS_LABELIMAGEPAINTER_HPP */
