/*
  Functions to load/save toger compressed with zlib.
  Copyright (C) 2004-2009 Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ZBACKUP_HPP
#define ZBACKUP_HPP

#ifdef _WIN32
#define NOMINMAX
#define ZLIB_WINAPI
#endif //_WIN32
#include <zlib.h>
#include <girl/girlToger.hpp>

namespace girl {

  namespace tools {

    class GIRL_API oZBackup : public girl::io::Writer
    {
    public:

      /**
       * @brief zlib compressed writer.
       *
       * It will use @a w to write data.
       * @a level is the compression level. It must be in [0; 9]
       * Lower compression levels result in faster execution, but less compression. 
       * Higher levels result in greater compression, but slower execution. 
       */
      oZBackup(girl::io::Writer &w, int level=6);

      /**
       * @brief Destructor.
       */
      ~oZBackup();

      /**
       * @brief tells if endianess must be taken into account.
       */
      virtual girl::io::writeFlags flags() const;

      /**
       * @brief write @a size bytes from @a p.
       *
       * @return number of written bytes.
       * If returned number is different than @a size, there was an error.
       */
      virtual size_t write(const void *p, size_t size);
  
      /** 
       * @warning must be called to flush write calls.
       *
       * @return true if everything was ok, else false.
       */
      virtual bool end();

    protected:

      size_t doWrite(int zflag);

    private:
      oZBackup(const oZBackup &);
      oZBackup &operator=(const oZBackup &);
  
    protected:

      girl::io::Writer &m_w;

      unsigned char *m_in;
      unsigned char *m_out;
      
      z_stream m_strm;

      unsigned int m_inSize;

      unsigned int m_start;
      unsigned int m_end;
  
      unsigned int m_outSize;
 
 
      bool m_flushed;
    };


    class GIRL_API iZBackup : public girl::io::Reader
    {
    public:

      explicit iZBackup(girl::io::Reader &r);

      ~iZBackup();

      virtual girl::io::writeFlags flags() const;

      void end();

      virtual size_t read(void *p, size_t size);

    private:
      iZBackup(const iZBackup &);
      iZBackup &operator=(const iZBackup &);
  
    protected:
  
      girl::io::Reader &m_r;

      z_stream m_strm;
  
  
      unsigned char *m_in;
      unsigned int m_inSize;

      unsigned int m_start;
      unsigned int m_end;
  
      unsigned char *m_out;
      unsigned int m_outSize;
  
      bool m_flushed;
    };

  }

}


#endif //ZBACKUP_HPP
