/*
  Test program for functions providing information on regions of toger, related to OpenCV.
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#include "CvUtilsTest.hpp"

#include <girlTools/girlToolsCvUtils.hpp>

#include "same_regions.hpp"

// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( CvUtilsTest );

#define DEFAULT_WIDTH 7

void 
CvUtilsTest::setUp()
{
}


void 
CvUtilsTest::tearDown()
{
}

void
CvUtilsTest::testAABBoxNone()
{
  const girl::POINT_TYPE w = DEFAULT_WIDTH;
  const girl::POINT_TYPE h = DEFAULT_WIDTH;

  cv::Mat mask = cv::Mat::zeros(h, w, CV_8U);
 
  girl::tools::AABBox bb = girl::tools::getAABBox(mask);
  
  CPPUNIT_ASSERT( ! bb.isValid() );
}

void
CvUtilsTest::testAABBoxAll()
{
  const girl::POINT_TYPE w = DEFAULT_WIDTH;
  const girl::POINT_TYPE h = DEFAULT_WIDTH;

  cv::Mat mask = cv::Mat::ones(h, w, CV_8U);
 
  girl::tools::AABBox bb = girl::tools::getAABBox(mask);
  
  CPPUNIT_ASSERT( bb.isValid() );
  CPPUNIT_ASSERT( bb.xMin() == 0 && bb.yMin() == 0 && bb.xMax() == w-1 && bb.yMax() == h-1  );
  
}

void
CvUtilsTest::testAABBox1()
{
  
  const girl::POINT_TYPE w = DEFAULT_WIDTH;
  const girl::POINT_TYPE h = DEFAULT_WIDTH;

  cv::Mat mask = cv::Mat::zeros(h, w, CV_8U);
  mask.at<uchar>(h/2, w/2) = 1;
 
  girl::tools::AABBox bb = girl::tools::getAABBox(mask);
  
  CPPUNIT_ASSERT( bb.isValid() );
  CPPUNIT_ASSERT( bb.xMin() == bb.xMax() && bb.yMin() == bb.yMax() && bb.xMin() == w/2 && bb.yMin() == h/2  );

}

void
CvUtilsTest::testFindRegionsInMaskNone()
{
  const girl::POINT_TYPE w = DEFAULT_WIDTH;
  const girl::POINT_TYPE h = DEFAULT_WIDTH;

  girl::toger tog(w, h);

  cv::Mat mask = cv::Mat::zeros(h, w, CV_8U);
 
  girl::regionDeque regs;
  girl::tools::getRegionsInMask(tog, mask, regs);
   
  CPPUNIT_ASSERT( regs.empty() );
}

void
CvUtilsTest::testFindRegionsInMaskAll()
{
  const girl::POINT_TYPE w = DEFAULT_WIDTH;
  const girl::POINT_TYPE h = DEFAULT_WIDTH;

  girl::toger tog(w, h);

  cv::Mat mask = cv::Mat::ones(h, w, CV_8U);
 
  girl::regionDeque regs;
  girl::tools::getRegionsInMask(tog, mask, regs);
  
  CPPUNIT_ASSERT( regs.size() == 1 );
}

void
CvUtilsTest::testFindRegionsInMaskAll_aux(girl::toger::sameRegion &func, int width, int height)
{
  girl::toger tog(width, height);
  
  girl::regionDeque regs;
  tog.splitRegion(1, func, regs);

  regs.clear();
  tog.allRegions(regs);

  cv::Mat mask = cv::Mat::ones(height, width, CV_8U);

  girl::regionDeque regs2;
  girl::tools::getRegionsInMask(tog, mask, regs2);
  
  CPPUNIT_ASSERT( regs.size() == regs2.size() );
  //TODO: check that all regions in regs are in regs2 and all regions in regs2 are in regs.
}
  
void
CvUtilsTest::testFindRegionsInMaskAllb()
{
  sameRegion_chessboard f;
  testFindRegionsInMaskAll_aux(f, DEFAULT_WIDTH, DEFAULT_WIDTH);


}

void
CvUtilsTest::testFindRegionsInMask1()
{
  const girl::POINT_TYPE w = DEFAULT_WIDTH;
  const girl::POINT_TYPE h = DEFAULT_WIDTH;

  cv::Mat mask = cv::Mat::zeros(h, w, CV_8U);
  mask.at<uchar>(h/2, w/2) = 1;
 
  girl::toger tog(w, h);

  girl::regionDeque regs;
  girl::tools::getRegionsInMask(tog, mask, regs);
  
  CPPUNIT_ASSERT( regs.size() == 1 );
  CPPUNIT_ASSERT( regs.back() == 1 );
}
