/*
  Test program for saving/loading toger on disk.
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#include "DiskBackupTest.hpp"

#include <time.h> //time
#include <cstdlib> //rand

#include "same_regions.hpp"

#include <girlTools/girlToolsDiskBackup.hpp>


// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( DiskBackupTest );

#define DEFAULT_WIDTH 23



std::string DiskBackupTest::ms_path;

void
DiskBackupTest::initPath()
{
  srand(time(NULL));

  const char *basePath = "/tmp/toger.bin.";
  
  std::stringstream ss;
  ss<<basePath<<rand();
  
  ms_path = ss.str();
}


void 
DiskBackupTest::setUp()
{
}


void 
DiskBackupTest::tearDown()
{
}

#ifdef VERBOSE
#include <iostream>
#endif

void
DiskBackupTest::benchToger(girl::toger::sameRegion &func, int width, int height)
{

  girl::toger tog(width, height);
  CPPUNIT_ASSERT(width > 0);
  CPPUNIT_ASSERT(height > 0);
  
  girl::regionDeque regs;
  tog.splitRegion(1, func, regs);

  CPPUNIT_ASSERT(tog.checkConsistency());

  if (ms_path.empty())
    DiskBackupTest::initPath();
  CPPUNIT_ASSERT(! ms_path.empty());

#ifdef VERBOSE
  std::cerr<<"DiskBackupTest::benchToger will save"<<std::endl;
#endif

  girl::tools::oDiskBackup w(ms_path);
  tog.write(w);
  w.end();
  
#ifdef VERBOSE
  std::cerr<<"DiskBackupTest::benchToger will load"<<std::endl;
#endif
  girl::tools::iDiskBackup r(ms_path);
  CPPUNIT_ASSERT( tog.read(r) );

#ifdef VERBOSE
  std::cerr<<"DiskBackupTest::benchToger save/load seems ok..."<<std::endl;
#endif

  CPPUNIT_ASSERT(tog.checkConsistency());
}

void 
DiskBackupTest::testChessboardBackup()
{
  sameRegion_chessboard f;
  benchToger(f, DEFAULT_WIDTH, DEFAULT_WIDTH);
}

void 
DiskBackupTest::testImageBackup()
{
  sameRegion_image f;
  const std::string filename = "lenna.pnm";

  CPPUNIT_ASSERT(f.loadImage(filename));

  benchToger(f, f.width(), f.height());
}

