/*
  Test program for image class.
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#include "ImageTest.hpp"

#include <girlTools/girlToolsImage.hpp>

#include <string.h> //memcmp

// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( ImageTest );


using namespace girl::tools;

void 
ImageTest::setUp()
{
}


void 
ImageTest::tearDown()
{
}

void 
ImageTest::testConstructor1()
{
  Image i1;
  CPPUNIT_ASSERT(i1.isNull());
  CPPUNIT_ASSERT(i1.width() == 0);
  CPPUNIT_ASSERT(i1.height() == 0);
  CPPUNIT_ASSERT(i1.bytesPerLine() == 0);
  CPPUNIT_ASSERT(i1.bits() == NULL);
  CPPUNIT_ASSERT(i1.scanLine(10) == NULL);
}

void 
ImageTest::testSetGet1()
{
  Image i1(200, 200);
  Pixel p(8,8,8,8);
  i1.setPixel(10, 10, p); 

  Pixel p2 = i1.getPixel(10, 10);

  CPPUNIT_ASSERT(p2 == p);
}

void
ImageTest::testCopy1()
{
  Image i1(200, 200);
  Image i2 = i1;
  CPPUNIT_ASSERT(! i1.isNull());
  CPPUNIT_ASSERT(! i2.isNull());
  CPPUNIT_ASSERT(i1.width() == i2.width());
  CPPUNIT_ASSERT(i1.height() == i2.height());
  CPPUNIT_ASSERT(i1.bytesPerLine() == i2.bytesPerLine());
 
}

void 
ImageTest::testCOW1()
{
  Image i1(200, 200);
  Pixel p(8,8,8,8);
  i1.setPixel(10, 10, p); 

  Image i2 = i1;
  Pixel p2(9,9,9,9);
  i2.setPixel(10, 10, p2);
  
  Pixel p3 = i1.getPixel(10, 10);

  CPPUNIT_ASSERT(p3 == p);
}
/*
//B: temporary
#include <iostream>
#include <girlTools/girlToolsLabelImagePainter.hpp>
#include <girlTools/girlToolsDiskBackup.hpp>
#include <girlTools/girlToolsRegionInfos.hpp>
union u_color {
  struct s_color { unsigned char b, g, r, a; } uc_color;
  unsigned int i_color;
};
*/
void 
ImageTest::testSaveLoad()
{
  const int w = 7;
  const int h = 11;
  const int nbChannels = 3;

  Image img(w, h, nbChannels);
  CPPUNIT_ASSERT(! img.isNull());
  CPPUNIT_ASSERT(img.channels() == 3);
  CPPUNIT_ASSERT(img.width() == w);
  CPPUNIT_ASSERT(img.height() == h);
  
  const girl::tools::Pixel p(255, 0, 0);
  for (int j=0; j<h; ++j)
    for (int i=0; i<w; ++i)
      img.setPixel(i, j, p);
    
  const std::string tmpFilename = "/tmp/tmpImg.png";
  const bool writeOk = img.save(tmpFilename);
  CPPUNIT_ASSERT(writeOk);
  
  Image img2(tmpFilename);
  CPPUNIT_ASSERT(! img2.isNull());
  CPPUNIT_ASSERT(img2.channels() == 3);
  
  CPPUNIT_ASSERT(img2.width() == img.width());
  CPPUNIT_ASSERT(img2.height() == img.height());
  CPPUNIT_ASSERT(img2.channels() == img.channels());
  CPPUNIT_ASSERT(img2.bytesPerLine() == img.bytesPerLine());

  const size_t bpl = img.bytesPerLine();
  for (int j=0; j<img.height(); ++j) {
    unsigned char *buf1 = (unsigned char *)img.scanLine(j);
    unsigned char *buf2 = (unsigned char *)img2.scanLine(j);
    CPPUNIT_ASSERT(memcmp(buf1, buf2, bpl) == 0);
  }

}
 
void 
ImageTest::testLoadSaveLoad()
{
  /*
   {//B: temporary
     float sigma = 0.5;
     int k = 32;
     int min_size = 100;

     const std::string togF = "/tmp/90261src_graphseg_0.5_32_100.seg";
     const std::string imgF = "/tmp/90261src.png";

     girl::tools::Image img(imgF);
     girl::tools::Image img2(imgF);

     girl::tools::iDiskBackup in(togF);
     girl::toger tog;
     bool readOk = tog.read(in);
     if (readOk) {
       const std::string savName = imgF.substr(0, imgF.rfind('.')) + "_falsecolorB.png";
       std::cerr<<"save temporary image: "<<savName<<"  numRegions="<<tog.numRegions()<<"\n";
       //girl::tools::LabelImagePainter().drawTogerInFalseColors(tog, img);

       girl::regionDeque regions;
       tog.allRegions(regions);
       size_t i=0;
       for (girl::regionDeque::const_iterator it=regions.begin();
	    it != regions.end();
	    ++it, ++i) {
	 u_color col;
	 const unsigned int rgb = 0xffffff + 1;
	 col.i_color = (unsigned int)( rgb * (i / (double)regions.size()));
	 girl::tools::Pixel color = girl::tools::Pixel(col.uc_color.b, col.uc_color.g, col.uc_color.r, 255);
 
	 girl::tools::LabelImagePainter().fillRegion(tog, *it, color, img);

       }

       girl::tools::Pixel color = girl::tools::Pixel(0, 255, 0, 255);
       girl::tools::paintRegionBorders(img2, tog, regions, color);
       const std::string savName2 = imgF.substr(0, imgF.rfind('.')) + "_borders.png";
       

       img.save(savName);
       img2.save(savName2);
       
     
     }
     else {
       std::cerr<<"Error: unable to read: "<<togF<<std::endl;
     }


   }
  */

  Image img("lenna.pnm");
  CPPUNIT_ASSERT(! img.isNull());
  CPPUNIT_ASSERT(img.channels() == 3);

  const std::string tmpFilename = "/tmp/lenna.png";
  const bool writeOk = img.save(tmpFilename);
  CPPUNIT_ASSERT(writeOk);
  
  Image img2(tmpFilename);
  CPPUNIT_ASSERT(! img2.isNull());
  CPPUNIT_ASSERT(img2.channels() == 3);
  
  CPPUNIT_ASSERT(img2.width() == img.width());
  CPPUNIT_ASSERT(img2.height() == img.height());
  CPPUNIT_ASSERT(img2.channels() == img.channels());
  CPPUNIT_ASSERT(img2.bytesPerLine() == img.bytesPerLine());

  const size_t bpl = img.bytesPerLine();
  for (int j=0; j<img.height(); ++j) {
    unsigned char *buf1 = (unsigned char *)img.scanLine(j);
    unsigned char *buf2 = (unsigned char *)img2.scanLine(j);
    CPPUNIT_ASSERT(memcmp(buf1, buf2, bpl) == 0);
  }




}
 
