/*
  Test program for saving/loading toger in memory.
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#include "MemBackupTest.hpp"

#include "same_regions.hpp"

#include <girl/girlSharedPtr.hpp>
#include <girlTools/girlToolsMemoryBackup.hpp>


// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( MemBackupTest );

#define DEFAULT_WIDTH 43


void 
MemBackupTest::setUp()
{
}


void 
MemBackupTest::tearDown()
{
}


void
MemBackupTest::benchToger(girl::toger::sameRegion &func, int width, int height)
{

  girl::toger tog(width, height);
  
  girl::regionDeque regs;
  tog.splitRegion(1, func, regs);

  regs.clear();
  tog.allRegions(regs);


  girl::tools::oMemoryBackup w;
  tog.write(w);
  w.end();
  
  girl::shared_ptr<girl::tools::MemoryBuffer> mb = w.get();

  girl::tools::iMemoryBackup r(mb);
  CPPUNIT_ASSERT( tog.read(r) );

  CPPUNIT_ASSERT(tog.checkConsistency());

  girl::regionDeque regs2;
  tog.allRegions(regs2);

  CPPUNIT_ASSERT(regs2.size() == regs.size());
  while (regs2.size() != 0) {
    CPPUNIT_ASSERT(regs2.back() == regs.back());
    regs.pop_back();
    regs2.pop_back();
  }

}

void 
MemBackupTest::testChessboardBackup()
{
  sameRegion_chessboard f;
  benchToger(f, DEFAULT_WIDTH, DEFAULT_WIDTH);
}

void 
MemBackupTest::testImageBackup()
{
  sameRegion_image f;
  const std::string filename = "lenna.pnm";

  CPPUNIT_ASSERT(f.loadImage(filename));

  benchToger(f, f.width(), f.height());
}

