/*
  Test program for saving/loading toger with zlib compression.
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#include "ZBackupTest.hpp"

#include "same_regions.hpp"

#include <girl/girlSharedPtr.hpp>
#include <girlTools/girlToolsDiskBackup.hpp>
#include <girlTools/girlToolsMemoryBackup.hpp>
#include <girlTools/girlToolsZBackup.hpp>

// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( ZBackupTest );

#define DEFAULT_WIDTH 27


void 
ZBackupTest::setUp()
{
}


void 
ZBackupTest::tearDown()
{
}

#ifdef VERBOSE
#include <iostream>
#endif

void
ZBackupTest::benchTogerDiskBackup(girl::toger::sameRegion &func, 
				  int width, int height)
{
  girl::toger tog(width, height);
  
  girl::regionDeque regs;
  tog.splitRegion(1, func, regs);

  CPPUNIT_ASSERT(tog.checkConsistency());

  const std::string filename = "/tmp/toger.bin";

  girl::tools::oDiskBackup w(filename);
  girl::tools::oZBackup wz(w);

  tog.write(wz);
  wz.end();
  
  girl::tools::iDiskBackup r(filename);
  girl::tools::iZBackup rz(r);
  
  CPPUNIT_ASSERT( tog.read(rz) );

  CPPUNIT_ASSERT(tog.checkConsistency());
}

void
ZBackupTest::benchTogerMemBackup(girl::toger::sameRegion &func, 
				  int width, int height)
{
  girl::toger tog(width, height);
  
  girl::regionDeque regs;
  tog.splitRegion(1, func, regs);

  CPPUNIT_ASSERT(tog.checkConsistency());

  girl::tools::oMemoryBackup w;
  girl::tools::oZBackup wz(w);
  tog.write(wz);
  wz.end();

#ifdef VERBOSE
  std::cerr<<" write done; will read"<<std::endl;
#endif
  
  girl::shared_ptr<girl::tools::MemoryBuffer> mb = w.get();

  girl::tools::iMemoryBackup r(mb);
  girl::tools::iZBackup rz(r);
  
  CPPUNIT_ASSERT( tog.read(rz) );

  CPPUNIT_ASSERT(tog.checkConsistency());
}

void 
ZBackupTest::testChessboardDiskBackup()
{
#ifdef VERBOSE
  std::cerr<<std::endl<<"ZBackupTest::testChessboardDiskBackup "<<std::endl;
#endif
  sameRegion_chessboard f;
  benchTogerDiskBackup(f, DEFAULT_WIDTH, DEFAULT_WIDTH);
}

void 
ZBackupTest::testImageDiskBackup()
{
#ifdef VERBOSE
  std::cerr<<std::endl<<"ZBackupTest::testImageDiskBackup() "<<std::endl;
#endif
  sameRegion_image f;
  const std::string filename = "lenna.pnm";

  CPPUNIT_ASSERT(f.loadImage(filename));

  benchTogerDiskBackup(f, f.width(), f.height());
}

void 
ZBackupTest::testChessboardMemBackup()
{
#ifdef VERBOSE
  std::cerr<<std::endl<<"ZBackupTest::testChessboardMemBackup() "<<std::endl;
#endif
  sameRegion_chessboard f;
  benchTogerMemBackup(f, DEFAULT_WIDTH, DEFAULT_WIDTH);
}

void 
ZBackupTest::testImageMemBackup()
{
#ifdef VERBOSE
  std::cerr<<std::endl<<"ZBackupTest::testImageMemBackup() "<<std::endl;
#endif
  sameRegion_image f;
  const std::string filename = "lenna.pnm";

  CPPUNIT_ASSERT(f.loadImage(filename));

  benchTogerMemBackup(f, f.width(), f.height());
}

