/* 
 * Example of use of GIRL & ISE
 * Copyright (C) 1996 Luc Brun and Jean-Philippe Domenger
 * Copyright (C) 2005, 2006 Boris Mansencal  boris.mansencal@labri.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation (Inc., 59
 * Temple Place - Suite 330, Boston, MA 02111-1307, USA); either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/time.h> //gettimeofday

#include <girl/girlToger.h>
#include <bcl/bcl.h>

#include <ise/iseRegionSet.h>
#include <ise/iseRegionTools.h>


#include <ise/iseSegmentationContext.h>
#include <ise/iseQuantificationCAG.h>
#include <ise/iseRecursiveSplitAlgorithm.h>



static bclImage img;
static bclImage imgDst;
static int width;
static int height;
static BCLIMAGE_CHANNEL_TYPE *image_start;//B
static BCLIMAGE_CHANNEL_TYPE *imgDst_start;//B
static girlToger tog;
static girlRegion *regs;

static char *filename = NULL;
//static char *output_name = NULL;


static iseRegionSet regSet = NULL;


static iseSegmentationContext segContext = NULL;
static iseQuantification quantif = NULL;
static iseSplitAlgorithm splitAlgo = NULL;

static double maxVariation = 0.3;


static const BCLIMAGE_CHANNEL_TYPE gColor[BCLIMAGE_NUM_CHANNELS] = {255, 0, 0}; 


static void L_new_region_hook(girlRegion regId, void * data GIRL_unused)
{
  const iseRegionColorimetricFeatures currentRegion = iseCreateRegionFeatures(tog, img, regId);

  const iseRegionColorimetricFeatures oldRegion = iseRegionSet_giveRegionFeatures(regSet, regId);
  if (oldRegion != NULL)
    iseRegionColorimetricFeatures_delete(oldRegion);

  iseRegionSet_insertRegion(regSet, currentRegion, regId);
}

static void L_delete_region_hook(girlRegion regId, void * data GIRL_unused)
{
  const iseRegionColorimetricFeatures currentRegion = iseRegionSet_giveRegionFeatures(regSet, regId);

  iseRegionSet_removeRegion(regSet, regId);

  iseRegionColorimetricFeatures_delete(currentRegion);
}

#ifndef NDEBUG
static bool
checkRegSet(girlToger togr, iseRegionSet pRegSet)
{
  bool consistent = true;
  girlRegion *lRegions;
  int lNum_regions = girlToger_allRegions(togr, &lRegions);
  for (int i=0; i<lNum_regions; ++i)
    {
      iseRegionColorimetricFeatures r = iseRegionSet_giveRegionFeatures(pRegSet, lRegions[i]);
      if (r == NULL)
	{
	  printf("!!!!! Error: region %d is NULL in regionSet  (%d-th region on %d)\n", lRegions[i], i, lNum_regions);
	  consistent = false;
	}

    }
  if (iseRegionSet_giveCardinal(pRegSet) != lNum_regions)
    {
      printf("!!!!! Error: cardinal of regSet is %d and there are %d regions\n", iseRegionSet_giveCardinal(pRegSet), lNum_regions);
      consistent = false;
    }
  bclMemory_free(lRegions);

  return consistent;
}
#endif

/*
static void
saveTogerBitmapToFile(const char *output_name, const char *extension, girlToger togr)
{
  if (output_name != NULL)
    {
      int ext_length = strlen(extension);

      char non_simpl_name[1024]; 
      strncpy(non_simpl_name, output_name, 1024 - ext_length - 1);
      strncat(non_simpl_name, extension, ext_length);
      non_simpl_name[1023] = '\0';

      FILE *output_file = fopen(non_simpl_name, "w");
      girlToger_saveBitmap(togr, output_file);
      fclose(output_file);
    }
}
*/

static void
usage(const char *s)
{
  fprintf(stderr, "Usage: %s  image_filename\n", s);
  exit(EXIT_FAILURE);
}

static void
processArgs(int argc, char *argv[])
{
  if (argc != 2 )
    {
      usage(argv[0]);
    }
  filename = argv[argc-1];
}

/*
int
regionLess(const void *r1, const void* r2)
{
  return *(girlRegion*)r2-*(girlRegion*)r1;
}
*/


//return number of milliseconds between t0 and t1
static inline
unsigned long
P_computeTime(struct timeval t0, struct timeval t1)
{
  unsigned long t = 0;
  if (t1.tv_sec>t0.tv_sec)
    t = (t1.tv_sec-t0.tv_sec)*1000 + (t1.tv_usec-t0.tv_usec)/1000;
  else if (t1.tv_usec>t0.tv_usec)
    t = (t1.tv_usec-t0.tv_usec)/1000;
  return t;
}


static 
void
P_saveImage(void)
{
  bclPnm image = bclPnm_new(width, height, BclPnmRawPpm);
  bclPnm_set_uchar_rgb_image(image, imgDst_start);
  bclPnm_save(image, BclPnmRawPpm, "output.ppm");
}

static 
void
P_color(const girlPoint *pt, void *userData GIRL_unused)
{
  if (bclImage_getPixel(imgDst, pt->y, pt->x)[0] != 0
      || bclImage_getPixel(imgDst, pt->y, pt->x)[1] != 0
      || bclImage_getPixel(imgDst, pt->y, pt->x)[2] != 0)
    {
      fprintf(stderr, "Error: pixel (%d ; %d) already set !\n", pt->x, pt->y);
      P_saveImage();
      exit(EXIT_FAILURE);
    }

  bclImage_setPixel(imgDst, pt->y, pt->x, gColor);//warning (y,x) and not (x,y)
}


int 
main(int argc, char *argv[])
{
  /* program arguments procesing */
  processArgs(argc, argv);
  

  bclMemory_init();

  
  /* image loading */
  bclPnm loadedImage = NULL;
#ifdef __cplusplus
  loadedImage = bclPnm_load(filename);
#else
  BCL_HANDLE(bclAny, loadedImage = bclPnm_load(filename));
  if (BCLEXCEPTION_RAISED(bclAny))
    {
      fprintf(stderr, "Unable to load %s\n", filename);
      exit(EXIT_FAILURE);
    }
#endif

  width = bclPnm_get_width(loadedImage);
  height = bclPnm_get_height(loadedImage);
  if (BCLIMAGE_NUM_CHANNELS != 3 || sizeof(BCLIMAGE_CHANNEL_TYPE)!=sizeof(unsigned char))
    {
      fprintf(stderr, "unsupported image format for file : %s\n", filename);
      exit(EXIT_FAILURE);
    }
  img = bclImage_new(width, height);
  image_start = bclImage_getBuffer(img);
  bclPnm_make_uchar_rgb_image(loadedImage, image_start);
  //bclPnm_delete(loadedImage);

  imgDst = bclImage_new(width, height);
  imgDst_start = bclImage_getBuffer(imgDst);
  memset(imgDst_start, 0, sizeof(BCLIMAGE_CHANNEL_TYPE)*BCLIMAGE_NUM_CHANNELS*width*height);

  /* Initialization of representation */
  tog    = girlToger_new(width, height);
  regSet = iseRegionSet_new();
  L_new_region_hook(1, NULL);

  girlToger_setNewRegionHook(tog, L_new_region_hook,    NULL);
  girlToger_setDeleteRegionHook(tog, L_delete_region_hook, NULL);

  
  /* Initialization of segmentation algorithm */
  quantif = iseQuantificationCAG_new(iseConversion_changeSpace(ISE_RGB_to_K1K2K3));

  segContext = iseSegmentationContext_new(tog, regSet, img, NULL);

  girlRegion firstRegion = 1;

  splitAlgo = iseRecursiveSplitAlgorithm_new(segContext, quantif, NULL);

  /* segmentation */

  int num_regions = iseSplitAlgorithm_apply(splitAlgo, &firstRegion, 1, &regs, &maxVariation);
  fprintf(stdout, "found %d regions\n", num_regions);


  bclMemory_free(regs); regs = NULL;
  int num_regionsT = girlToger_allRegions(tog, &regs);
  fprintf (stdout, "total number of regions after first partition = %d\n", num_regionsT);


  for (int i=0; i<num_regionsT; ++i) {
    girlToger_iterateOnRegion(tog, regs[i], GIRL_ITERATE_DOMAIN, P_color, NULL);
  }

  
  for (int j=0; j<height; ++j) {
    for (int i=0; i<width; ++i) {

      assert(i<width);
      assert(j<height);

      if (bclImage_getPixel(imgDst, j, i)[0] != gColor[0]
	  || bclImage_getPixel(imgDst, j, i)[1] != gColor[1]
	  || bclImage_getPixel(imgDst, j, i)[2] != gColor[2])
	{
	  fprintf(stderr, "Error: pixel (%d ; %d) not set !\n", i, j);
	  P_saveImage();
	  exit(EXIT_FAILURE);
	}

    }
  }

  P_saveImage();

  bclPnm_delete(loadedImage);

  bclMemory_free(regs);

  assert(girlToger_checkConsistency(tog));
  
  assert(checkRegSet(tog, regSet));


  //saveTogerBitmapToFile(output_name, "", tog);

  iseSplitAlgorithm_delete(splitAlgo);
  iseQuantification_delete(quantif);
  iseSegmentationContext_delete(segContext);

  iseRegionSet_delete(regSet);
  girlToger_delete(tog);

  bclImage_delete(img);

  bclMemory_terminate();

  exit(EXIT_SUCCESS);
}
