/* 
 * Example of use of GIRL
 * Copyright (C) 1996 Luc Brun and Jean-Philippe Domenger
 * Copyright (C) 2005 Boris Mansencal  boris.mansencal@labri.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation (Inc., 59
 * Temple Place - Suite 330, Boston, MA 02111-1307, USA); either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include <getopt.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/time.h> //gettimeofday

#include <girl/girlToger.h>
#include <bcl/bcl.h>

#include <ise/iseRegionSet.h>
#include <ise/iseRegionTools.h>

#include <ise/iseDescriptorTools.h>

#include <ise/iseSegmentationContext.h>
#include <ise/iseMergeRemoveDetailsAlgorithm.h>
#include <ise/iseMergeAlgorithm.h>
#include <ise/iseMergeAlgorithmXX.h>
#include <ise/iseQuantification.h>
#include <ise/iseUniformQuantification.h>
#include <ise/iseQuantificationCAG.h>
#include <ise/iseSplitAlgorithm.h>
#include <ise/iseRecursiveSplitAlgorithm.h>
#include <ise/iseSplitMergeAlgorithm.h>

#include <ise/iseDominantColorExtraction.h>
#include <ise/iseColorLayoutExtraction.h>
#include <ise/iseColorStructureExtraction.h>


#include <ise/iseQuantificationSC.h>
#include <ise/iseQuantificationDC.h>

#include <ise/iseTemporaryConversion.h>

#include "bench.h"



static bclImage img;
static int width;
static int height;
static BCLIMAGE_CHANNEL_TYPE *image_start;//B
static girlToger tog;
static girlRegion *regs;

static char *inFilename = NULL;
static char *outFilename = NULL;

static iseRegionSet regSet = NULL;

typedef enum splitIdType {SIMPLE_SPLIT=0, RECURSIVE_SPLIT, SPLIT_N_MERGE} splitIdType;
typedef enum quantifIdType {QUANTIF_UNIFORM=0, QUANTIF_CAG, QUANTIF_SCAL, QUANTIF_DC} quantifIdType;

static splitIdType splitId = RECURSIVE_SPLIT;
static quantifIdType quantifId = QUANTIF_CAG;

static iseSegmentationContext segContext = NULL;
static iseMergeAlgorithm mergeAlgo = NULL;
static iseQuantification quantif = NULL;
static iseSplitAlgorithm splitAlgo = NULL;

static double maxVariation = 0.3;



static void L_new_region_hook(girlRegion regId, void * data )
{
  const iseRegionColorimetricFeatures currentRegion = iseCreateRegionFeatures(tog, img, regId);

  const iseRegionColorimetricFeatures oldRegion = iseRegionSet_giveRegionFeatures(regSet, regId);
  if (oldRegion != NULL)
    iseRegionColorimetricFeatures_delete(oldRegion);

  iseRegionSet_insertRegion(regSet, currentRegion, regId);
}

static void L_delete_region_hook(girlRegion regId, void * data )
{
  const iseRegionColorimetricFeatures currentRegion = iseRegionSet_giveRegionFeatures(regSet, regId);

  iseRegionSet_removeRegion(regSet, regId);

  iseRegionColorimetricFeatures_delete(currentRegion);
}



static bool
checkRegSet(girlToger togr, iseRegionSet pRegSet)
{
  bool consistent = true;
  girlRegion *lRegions;
  int lNum_regions = girlToger_allRegions(togr, &lRegions);
  for (int i=0; i<lNum_regions; ++i)
    {
      iseRegionColorimetricFeatures r = iseRegionSet_giveRegionFeatures(pRegSet, lRegions[i]);
      if (r == NULL)
	{
	  printf("!!!!! Error: region %d is NULL in regionSet  (%d-th region on %d)\n", lRegions[i], i, lNum_regions);
	  consistent = false;
	}

    }
  if (iseRegionSet_giveCardinal(pRegSet) != lNum_regions)
    {
      printf("!!!!! Error: cardinal of regSet is %d and there are %d regions\n", iseRegionSet_giveCardinal(pRegSet), lNum_regions);
      consistent = false;
    }
  bclMemory_free(lRegions);

  return consistent;
}


static void
saveTogerBitmapToFile(const char *output_name, const char *extension, girlToger togr)
{
  if (output_name != NULL)
    {
      int ext_length = strlen(extension);

      char non_simpl_name[1024]; 
      strncpy(non_simpl_name, output_name, 1024 - ext_length - 1);
      strncat(non_simpl_name, extension, ext_length);
      non_simpl_name[1023] = '\0';

      girlToger_saveBitmap(togr, non_simpl_name);
    }
}


static
void
P_chooseQuantifAlgo(quantifIdType quantifId)
{
  switch (quantifId) {
  case QUANTIF_UNIFORM:
    {
      int nbCluster = 64;
      quantif = iseUniformQuantification_new(&nbCluster);
    }
    break;
  case QUANTIF_CAG:
    quantif = iseQuantificationCAG_new(iseConversion_changeSpace(ISE_RGB_to_K1K2K3));
    break;
  case QUANTIF_SCAL:
    quantif = iseQuantificationSC_new();
    break;
  case QUANTIF_DC:
    {
      fprintf(stdout, " \n ****** Begin DCL (quantif) ****** \n");
      iseDominantColorList dcl = iseDominantColor_newlist(img, RGB, 1, tog);
      iseDominantColorExtraction(dcl);
      
      iseDominantColor_writeList(dcl);
      quantif = iseQuantificationDC_new(dcl);
      fprintf(stdout, " \n ****** End DCL (quantif) ****** \n");
    }
    break;
  default:
    quantif = iseQuantificationCAG_new(iseConversion_changeSpace(ISE_RGB_to_K1K2K3));
    break;
  }
}

 static
 void
 P_chooseSplitAlgo(splitIdType splitId)
 {
   assert(segContext);
   assert(quantif);
   assert(splitAlgo == NULL);

   switch (splitId) {
   case SIMPLE_SPLIT:
     splitAlgo = iseSplitAlgorithm_new(segContext, quantif, NULL);
     break;
   case RECURSIVE_SPLIT:
     splitAlgo = iseRecursiveSplitAlgorithm_new(segContext, quantif, NULL);
     break;
   case SPLIT_N_MERGE:
     assert(mergeAlgo);
     splitAlgo = iseSplitMergeAlgorithm_new(mergeAlgo, segContext, quantif, NULL);
     break;
   default:
     splitAlgo = iseSplitAlgorithm_new(segContext, quantif, NULL);
     break;
   }
 }



 static void
 usage(char *s)
 {
   fprintf(stderr, "Usage: %s [-a splitAlgo] [-h] [-o maxVariation] [-q quantificationalgo] [-s savefilename] -i filename\n", s);
   fprintf(stderr, "  -a splitAlgo : choose algo : 0 for simple split, 1 for recursive split, 2 for split and merge.\n");
   fprintf(stderr, "  -h : print this help and exit.\n");
   fprintf(stderr, "  -i filename : image file \n");  
   fprintf(stderr, "  -o : set max variation, must be a float in [0.0; 1.0].\n");
   fprintf(stderr, "  -q quantificationAlgo : choose algo : 0 for uniform quantification, 1 for CAG quantification, 2 for SC, 3 for DC\n");
   fprintf(stderr, "  -s saveFilename : save image of segmentation to specified filename.\n"); 
 }

 static void
 processArgs(int argc, char *argv[])
 {
   for (bool end=false; !end; )
     switch (getopt(argc, argv, "a:hi:o:q:s:")) {
     case 'a':
       splitId = (splitIdType)atoi(optarg);
       break;
     case 'h':
       usage(argv[0]);
       exit(EXIT_SUCCESS);
       break;
     case 'i':
       inFilename = optarg;
       break;
     case 'o':
       {
	 float f = atof(optarg);
	 if (f>=0.0f && f<=1.0f)
	   maxVariation = f;
       }
       break;
     case 'q':
       quantifId = (quantifIdType)atoi(optarg);
       break;
     case 's':
       outFilename = optarg;
       break;
     case -1: // not more option
       end = true;
       break;
     default: // unknown option or missing argument
       exit(EXIT_FAILURE);
       break;
     }

   if (inFilename == NULL) {
     usage(argv[0]);
     exit(EXIT_FAILURE);
   }


 }

 //return number of milliseconds between t0 and t1
 static inline
 unsigned long
 P_computeTime(struct timeval t0, struct timeval t1)
 {
   unsigned long t = 0;
   if (t1.tv_sec>t0.tv_sec)
     t = (t1.tv_sec-t0.tv_sec)*1000 + (t1.tv_usec-t0.tv_usec)/1000;
   else if (t1.tv_usec>t0.tv_usec)
     t = (t1.tv_usec-t0.tv_usec)/1000;
   return t;
 }


 void computePixelNumber(const girlPoint * pt, void * userData)
 {
   int * i = (int *) (intptr_t) userData;

   (*i)+=1;
   //fprintf(stderr, "(%d,%d) \t",girlPoint_y(pt), girlPoint_x(pt));
 }

 int 
 main(int argc, char *argv[])
 {
   struct timeval t0, t1;

   /* program arguments procesing */
   processArgs(argc, argv);

   bclMemory_init();


   /* image loading */
   bclPnm loadedImage = NULL;
#ifdef __cplusplus
  loadedImage = bclPnm_load(inFilename);
#else
  BCL_HANDLE(bclAny, loadedImage = bclPnm_load(inFilename));
  if (BCLEXCEPTION_RAISED(bclAny))
    {
      fprintf(stderr, "Unable to load %s\n", inFilename);
      exit(EXIT_FAILURE);
    }
#endif

  width = bclPnm_get_width(loadedImage);
  height = bclPnm_get_height(loadedImage);
  if (BCLIMAGE_NUM_CHANNELS != 3 || sizeof(BCLIMAGE_CHANNEL_TYPE)!=sizeof(unsigned char))
    {
      fprintf(stderr, "unsupported image format for file : %s\n", inFilename);
      exit(EXIT_FAILURE);
    }
  img = bclImage_new(width, height);
  image_start = bclImage_getBuffer(img);
  bclPnm_make_uchar_rgb_image(loadedImage, image_start);
  bclPnm_delete(loadedImage);


/*   /\* iseScalableColor sc; *\/ */
/* /\*   sc = iseScalableColor_new(256,0,RGB,img); *\/ */

/* /\*   iseScalableColorExtraction(sc); *\/ */

/* /\*   iseScalableColor_write(sc); *\/ */
/* /\*   iseScalableColor_delete(sc); *\/ */

  
/* /\*   sc = iseScalableColor_new(64,3,RGB,img); *\/ */

/* /\*   iseScalableColorExtraction(sc); *\/ */

/* /\*   iseScalableColor_write(sc); *\/ */
/* /\*   fprintf(stdout, "\n %p \n", sc); *\/ */
/* /\*   iseScalableColor_delete(sc); *\/ */
/*   //quantif = iseQuantificationDC_new(dcl); */

  /* Initialization of representation */
  tog    = girlToger_new(width, height);
  regSet = iseRegionSet_new();
  L_new_region_hook(1, NULL);

  girlToger_setNewRegionHook(tog, L_new_region_hook,    NULL);
  girlToger_setDeleteRegionHook(tog, L_delete_region_hook, NULL);


  girlRegion firstRegion = 1;
  int regionsize = 0;
  girlToger_iterateOnRegion(tog,firstRegion, GIRL_ITERATE_DOMAIN, computePixelNumber,&regionsize);
  fprintf(stdout, "region size: %d \n ", regionsize);



  fprintf(stdout, " \n ****** Begin Color Structure  ****** \n"); 
  ++total_encode_times;
  start_timer();
  iseColorStructure cs = iseColorStructure_new(Q256, RGB, img, tog, 1);
  iseColorStructureExtraction(cs);
  stop_timer();
  total_encode_time += time_passed();
  fprintf(stdout, "TIMER %g \n", total_encode_time);
/*   FILE *output_file = fopen("histoColorStructure.mws", "w"); */
/*   fprintf(output_file," with(stats):\n"); */
/*   fprintf(output_file," with(stats[statplots]):\n"); */
/*   fprintf(output_file,"csdata:=["); */
/*   for(int s = 0 ; s < 255 ; s++) */
/*     { */
/*       fprintf(output_file,"Weight(%d..%d,%d),",s,s, iseColorStructure_getHistoValue(cs,s)); */
/*     } */
/*   fprintf(output_file,"Weight(%d..%d,%d)]:\n", 255,255,iseColorStructure_getHistoValue(cs,255)); */
/*   fprintf(output_file,"histogram(csdata);"); */
/*   fclose(output_file); */
  iseColorStructure_write(cs);
  iseColorStructure_delete(cs);
  total_encode_times++;
  total_encode_time = 0;
  start_timer();
  cs = iseColorStructure_new(Q32,RGB,img,tog,1);
  iseColorStructureExtraction(cs);
  stop_timer();
  total_encode_time+=time_passed();
  fprintf(stdout, "TIMER %g \n", total_encode_time);

  iseColorStructure_write(cs);
  bclPnm cspnm = iseColorStructure_createPnmFromColorStructure(cs);
  bclPnm_save(cspnm, BclPnmRawPpm, "./csmain.ppm");
  bclPnm_delete(cspnm);

  iseColorStructure_delete(cs);
  fprintf(stdout, " \n ****** End Color Structure ****** \n"); 



  fprintf(stdout, " \n ****** Begin Color Layout  ****** \n"); 
  iseColorLayout cl ;
  

/*   cl = iseColorLayout_new(64,64,RGB,img,tog,1,iseColorLayout_dominantComputation); */
/*   iseColorLayoutExtraction(cl); */
/*   iseColorLayout_write(cl); */

/*   iseColorLayout_delete(cl); */

  total_encode_times++;
  total_encode_time = 0;
  start_timer();
  cl = iseColorLayout_new(64,64,RGB,img,tog,1,iseColorLayout_averageComputation);
  iseColorLayoutExtraction(cl);
  stop_timer();
  total_encode_time+=time_passed();
  fprintf(stdout, "TIMER %g \n", total_encode_time);

  iseColorLayout_write(cl);

  bclPnm psi = iseColorLayout_createPnmFromSmallImage(cl);
  bclPnm_delete(psi);

  bclPnm pcl = iseColorLayout_createPnmFromColorLayout(cl);
  bclPnm_delete(pcl);
  

  iseColorLayout_delete(cl);

  fprintf(stdout, " \n ****** End Color Layout ****** \n");   
  


  fprintf(stdout, " \n ****** Begin Dominant Color ****** \n"); 
  
  total_encode_time = 0;
  total_encode_times++;
  start_timer();
  iseDominantColorList dcl = iseDominantColor_newlist(img, RGB,1,tog); 
  iseDominantColorExtraction(dcl);
  stop_timer();
  total_encode_time+=time_passed();
  fprintf(stdout, "TIMER %g \n", total_encode_time);

  iseDominantColor_writeList(dcl);

  fprintf(stdout, "\n");
/* /\*   iseDominantColorList dclclone = iseDominantColor_clonelist(dcl); *\/ */
/* /\*   iseDominantColor_writeList(dclclone); *\/ */
  
  bclPnm dompnm = iseDominantColor_createPnmFromDominantColor(dcl);

  bclPnm_save(dompnm,BclPnmRawPpm,"./dominantmain.ppm");
  bclPnm_delete(dompnm);

   iseDominantColor_deletelist(dcl); 
/*   //  iseDominantColor_deletelist(dclclone); */

   fprintf(stdout, " \n ****** End Dominant Color ****** \n"); 



   fprintf(stdout, " \n ****** Begin Scalable Color  ****** \n"); 
   total_encode_time = 0;
   total_encode_times++;
   start_timer();
   iseScalableColor sc = iseScalableColor_new(64, 3, RGB, img, tog, 1);

   iseScalableColorExtraction(sc);

   stop_timer();
   total_encode_time+=time_passed();
   fprintf(stdout, "TIMER %g \n", total_encode_time);

   iseScalableColor_write(sc);
   
   iseScalableColor_delete(sc);
   
   fprintf(stdout, " \n ****** End Scalable Color  ****** \n"); 


  /* cleaning */


  iseRegionSet_delete(regSet);
  girlToger_delete(tog);

  bclImage_delete(img);


  bclMemory_terminate();


  return EXIT_SUCCESS;
}

