/*
  Example program for ise.
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/


#include <getopt.h>
#include <cstdio>
#include <cstdlib>
//#include <cstring>
#include <sys/time.h> //gettimeofday

#include <iostream>

#include <girl/girlToger.hpp>
#include <girlTools/girlToolsImage.hpp>
#include <girlTools/girlToolsPnmCodec.hpp>
#include <ise/iseRegionSet.hpp>
#include <ise/iseRegionTools.hpp>
#include <ise/iseSegmentationContext.hpp>
//#include <ise/iseMergeRemoveDetailsAlgorithm.hpp>
//#include <ise/iseMergeDefaultAlgorithm.hpp>
//#include <ise/iseMergeAlgorithmXX.hpp>
//#include <ise/iseMergeRemoveAlgorithm.hpp>
//#include <ise/iseMergeRemoveDetailsAlgorithm.hpp>
#include <ise/iseQuantification.hpp>
#include <ise/iseQuantificationUniform.hpp>
#include <ise/iseQuantificationCAG.hpp>
#include <ise/iseSplitAlgorithmDefault.hpp>
#include <ise/iseSplitAlgorithmBFRecursive.hpp>
#include <ise/iseSplitAlgorithmDFRecursive.hpp>

//#include <ise/iseSplitMergeAlgorithm.hpp>
//#include <ise/iseSplitDriverDefault.hpp>

#include <ise/iseSplitDriverRecursiveDefault.hpp>
#include <ise/iseArrayHistogram.hpp>

#include <ise/iseMergeAlgorithmDefault.hpp>
#include <ise/iseMergeDriverSizeFiltering.hpp>


static girl::tools::Image img;
static int width;
static int height;

static girl::toger *tog;

static std::string inFilename = "lenna.pnm";
static std::string outFilename = "toto.pnm";

static ise::RegionSet *regSet = NULL;

typedef enum splitIdType {SIMPLE_SPLIT=0, RECURSIVE_SPLIT, SPLIT_N_MERGE} splitIdType;
typedef enum quantifIdType {QUANTIF_UNIFORM=0, QUANTIF_CAG} quantifIdType;
typedef enum mergeIdType {DEFAULT_MERGE, XX_MERGE, REMOVE_MERGE, REMOVE_DETAILS_MERGE} mergeIdType;

static const splitIdType default_splitId = RECURSIVE_SPLIT;
static const quantifIdType default_quantifId = QUANTIF_CAG;
static const mergeIdType default_mergeId = REMOVE_DETAILS_MERGE;

static splitIdType splitId;
static quantifIdType quantifId;
static mergeIdType mergeId;

static ise::SegmentationContext *segContext = NULL;
//static iseMergeAlgorithm *mergeAlgo = NULL;
static ise::Quantification *quantif = NULL;
static ise::SplitAlgorithm *splitAlgo = NULL;

static double maxVariation = 0.3;


/*
static
void
L_new_region_hook(girl::region regId, void * data GIRL_unused)
{
  ise::RegionColorimetricFeatures *currentRegion = iseRegionTools_createRegionFeatures(*tog, img, regId);

  regSet->replaceRegion(currentRegion, regId);
}

static
void
L_delete_region_hook(girl::region regId, void * data GIRL_unused)
{
  const ise::RegionColorimetricFeatures *currentRegion = regSet->getRegionFeatures(regId);

  regSet->removeRegion(regId);

  delete currentRegion;
}

static 
void 
L_merge_regions_hook(girl::region keptRegionId, girl::region delRegionId, void * data GIRL_unused)
{
  ise::RegionColorimetricFeatures *keptRegion = regSet->getRegionFeatures(keptRegionId);
  ise::RegionColorimetricFeatures *delRegion = regSet->getRegionFeatures(delRegionId);
  
  keptRegion->merge2RegionFeatures(delRegion);
}

*/

static 
void
saveTogerBitmapToFile(const std::string &output_name, 
		      const std::string &extension, 
		      const girl::toger &togr)
{
  if (! output_name.empty()) {
    std::string filename = output_name + extension;
    
    togr.saveBitmap(filename);
  }
}



int 
main(int argc, char *argv[])
{

  if (argc >= 2) 
    inFilename = argv[1];



  /* image loading */
  girl::tools::PnmCodec codec;
  const bool loadOk = codec.load(inFilename, img);
  if (! loadOk) {
    std::cerr<<"Unable to load "<<inFilename<<std::endl;
    exit(EXIT_FAILURE);
  }
  width = img.width();
  height = img.height();

  /* Initialization of representation */
  tog = new girl::toger(width, height);
  regSet = new ise::RegionSet();
  /*
  L_new_region_hook(1, NULL);
  tog->setNewRegionHook(L_new_region_hook,    NULL);
  tog->setDeleteRegionHook(L_delete_region_hook, NULL);
  tog->setMergeRegionsHook(L_merge_regions_hook, NULL);
  */
  
  segContext = new ise::SegmentationContext(tog, regSet, &img, NULL);

//   quantif = new ise::QuantificationUniform(segContext,
// 					   0,
// 					   5, 5, 5);


  ise::Histogram *histo = new ise::ArrayHistogram(0, 0, 0);

  quantif = new ise::QuantificationCAG(segContext, histo, 0);
  static_cast<ise::QuantificationCAG *>(quantif)->setThreshold(0.3);


//   splitAlgo = new ise::SplitAlgorithmDefault(segContext);
//   ise::SplitDriver *driver = new ise::SplitDriverDefault(segContext);
  
  splitAlgo = new ise::SplitAlgorithmBFRecursive(segContext);
  //ise::SplitDriver *driver = new ise::SplitDriverRecursiveDefault(segContext);
  ise::SplitDriver *driver = new ise::SplitDriverRecursiveDefault(segContext, 0.3);

  ise::MergeDriverSizeFiltering *driverMerge = new ise::MergeDriverSizeFiltering(segContext);
  ise::MergeAlgorithmDefault  *mergeAlgo = new ise::MergeAlgorithmDefault(segContext);

  girl::regionDeque regions2split;
  regions2split.push_back(1);
  girl::regionDeque createdRegions;

  splitAlgo->apply(regions2split, *driver, *quantif, 0, createdRegions);  

  std::cerr<<createdRegions.size()<<" regions"<<std::endl;

  
  girl::regionDeque keptRegions;
  
  mergeAlgo->apply(createdRegions, *driverMerge, 1, keptRegions);
  
  std::cerr<<keptRegions.size()<<" regions"<<std::endl;


  /* segmentation backup */
  if (! outFilename.empty())
    saveTogerBitmapToFile(outFilename, "", *tog);

  /* afficher les caractristiques */
  //region_features_extract(tog);

 
  /* cleaning */

//   if (splitId == SPLIT_N_MERGE) {
//     iseMergeAlgorithm_delete(mergeAlgo);
//   }

  delete driver;
  //delete histo;

  delete splitAlgo;
  delete quantif;
  delete segContext;
  
  delete regSet;
  delete tog;


  return EXIT_SUCCESS;
}

