/* 
 * Example of use of girl/ise
 * Copyright (C) 2005-2011 Boris Mansencal boris.mansencal@labri.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation (Inc., 59
 * Temple Place - Suite 330, Boston, MA 02111-1307, USA); either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
#include <getopt.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/time.h> //gettimeofday
#include <math.h>

#include <girl/girlToger.h>
#include <bcl/bcl.h>



#include <ise/iseRegionSet.h>
#include <ise/iseRegionTools.h>

#include <ise/iseSegmentationContext.h>
#include <ise/iseMergeRemoveDetailsAlgorithm.h>
#include <ise/iseMergeAlgorithm.h>
#include <ise/iseMergeAlgorithmXX.h>
#include <ise/iseQuantification.h>
#include <ise/iseUniformQuantification.h>
#include <ise/iseQuantificationCAG.h>
#include <ise/iseSplitAlgorithm.h>
#include <ise/iseRecursiveSplitAlgorithm.h>
#include <ise/iseSplitMergeAlgorithm.h>
#include <ise/iseContourFourierDescriptors.h>
#include <ise/iseRegionGeometricFeatures.h>
#include <ise/iseRegionMoments.h>
#include <ise/iseRegionARTExtraction.h>
#include <ise/iseCurvatureScaleSpaceExtraction.h>
#include <ise/iseEdgeHistogramExtraction.h>

#include "bench.h"


static bclImage img;
static int width;
static int height;
static BCLIMAGE_CHANNEL_TYPE *image_start;//B
static girlToger tog;
static girlRegion *regs;

static char *inFilename = NULL;
static char *outFilename = NULL;

static iseRegionSet regSet = NULL;

typedef enum splitIdType {SIMPLE_SPLIT=0, RECURSIVE_SPLIT, SPLIT_N_MERGE} splitIdType;
typedef enum quantifIdType {QUANTIF_UNIFORM=0, QUANTIF_CAG} quantifIdType;

static splitIdType splitId = RECURSIVE_SPLIT;
static quantifIdType quantifId = QUANTIF_CAG;

static iseSegmentationContext segContext = NULL;
static iseMergeAlgorithm mergeAlgo = NULL;
static iseQuantification quantif = NULL;
static iseSplitAlgorithm splitAlgo = NULL;

static double maxVariation = 0.3;


static bool recursive;
static bool verbose;

static void L_new_region_hook(girlRegion regId, void * data )
{
  const iseRegionColorimetricFeatures currentRegion = iseCreateRegionFeatures(tog, img, regId);

  const iseRegionColorimetricFeatures oldRegion = iseRegionSet_giveRegionFeatures(regSet, regId);
  if (oldRegion != NULL)
    iseRegionColorimetricFeatures_delete(oldRegion);

  iseRegionSet_insertRegion(regSet, currentRegion, regId);
}

static void L_delete_region_hook(girlRegion regId, void * data )
{
  const iseRegionColorimetricFeatures currentRegion = iseRegionSet_giveRegionFeatures(regSet, regId);

  iseRegionSet_removeRegion(regSet, regId);

  iseRegionColorimetricFeatures_delete(currentRegion);
}



static 
bool
checkRegSet(girlToger togr, iseRegionSet pRegSet)
{
  bool consistent = true;
  girlRegion *lRegions;
  int lNum_regions = girlToger_allRegions(togr, &lRegions);
  for (int i=0; i<lNum_regions; ++i)
    {
      iseRegionColorimetricFeatures r = iseRegionSet_giveRegionFeatures(pRegSet, lRegions[i]);
      if (r == NULL)
	{
	  printf("!!!!! Error: region %d is NULL in regionSet  (%d-th region on %d)\n", lRegions[i], i, lNum_regions);
	  consistent = false;
	}

    }
  if (iseRegionSet_giveCardinal(pRegSet) != lNum_regions)
    {
      printf("!!!!! Error: cardinal of regSet is %d and there are %d regions\n", iseRegionSet_giveCardinal(pRegSet), lNum_regions);
      consistent = false;
    }
  bclMemory_free(lRegions);

  return consistent;
}


static 
void
saveTogerBitmapToFile(const char *output_name, const char *extension, girlToger togr)
{
  if (output_name != NULL)
    {
      int ext_length = strlen(extension);

      char non_simpl_name[1024]; 
      strncpy(non_simpl_name, output_name, 1024 - ext_length - 1);
      strncat(non_simpl_name, extension, ext_length);
      non_simpl_name[1023] = '\0';

      girlToger_saveBitmap(togr, non_simpl_name);
    }
}


static
void
P_chooseQuantifAlgo(quantifIdType quantifId)
{
  switch (quantifId) {
  case QUANTIF_UNIFORM:
    //quantif = iseUniformQuantification_new();
    break;
  default:
  case QUANTIF_CAG:
    quantif = iseQuantificationCAG_new(iseConversion_changeSpace(ISE_RGB_to_K1K2K3));
    break;
  }
}

static
void
P_chooseSplitAlgo(splitIdType splitId)
{
  assert(segContext);
  assert(quantif);
  assert(splitAlgo == NULL);

  switch (splitId) {
  case SIMPLE_SPLIT:
    splitAlgo = iseSplitAlgorithm_new(segContext, quantif, NULL);
    break;
  case RECURSIVE_SPLIT:
    splitAlgo = iseRecursiveSplitAlgorithm_new(segContext, quantif, NULL);
    break;
  case SPLIT_N_MERGE:
    assert(mergeAlgo);
    splitAlgo = iseSplitMergeAlgorithm_new(mergeAlgo, segContext, quantif, NULL);
    break;
  default:
    splitAlgo = iseSplitAlgorithm_new(segContext, quantif, NULL);
    break;
  }
}



static 
void
usage(char *s)
{
  fprintf(stderr, "Usage: %s [-a splitAlgo] [-h] [-o maxVariation] [-q quantificationalgo] [-s savefilename] -i filename\n", s);
  fprintf(stderr, "  -a splitAlgo : choose algo : 0 for simple split, 1 for recursive split, 2 for split and merge.\n");
  fprintf(stderr, "  -h : print this help and exit.\n");
  fprintf(stderr, "  -i filename : image file \n");  
  fprintf(stderr, "  -o : set max variation, must be a float in [0.0; 1.0].\n");
  fprintf(stderr, "  -q quantificationAlgo : choose algo : 0 for uniform quantification, 1 for CAG quantification.\n");
  fprintf(stderr, "  -r : compute shape/texture information for segmented regions\n");
  fprintf(stderr, "  -s saveFilename : save image of segmentation to specified filename.\n"); 
  fprintf(stderr, "  -v : verbose mode : print details for each reagion.\n"); 
}

static 
void
processArgs(int argc, char *argv[])
{
  for (bool end=false; !end; )
    switch (getopt(argc, argv, "a:hi:o:q:rs:v")) {
    case 'a':
      splitId = (splitIdType)atoi(optarg);
      break;
    case 'h':
      usage(argv[0]);
      exit(EXIT_SUCCESS);
      break;
    case 'i':
      inFilename = optarg;
      break;
    case 'o':
      {
	float f = atof(optarg);
	if (f>=0.0f && f<=1.0f)
	  maxVariation = f;
      }
      break;
    case 'q':
      quantifId = (quantifIdType)atoi(optarg);
      break;
    case 'r':
      recursive = true;
      break;
    case 's':
      outFilename = optarg;
      break;
    case 'v':
      verbose = true;
      break;
    case -1: // not more option
      end = true;
      break;
    default: // unknown option or missing argument
      exit(EXIT_FAILURE);
      break;
    }
     
  if (inFilename == NULL) {
    usage(argv[0]);
    exit(EXIT_FAILURE);
  }
    

}

//return number of milliseconds between t0 and t1
static inline
unsigned long
P_computeTime(struct timeval t0, struct timeval t1)
{
  unsigned long t = 0;
  if (t1.tv_sec>t0.tv_sec)
    t = (t1.tv_sec-t0.tv_sec)*1000 + (t1.tv_usec-t0.tv_usec)/1000;
  else if (t1.tv_usec>t0.tv_usec)
    t = (t1.tv_usec-t0.tv_usec)/1000;
  return t;
}

/*
 * Return time in seconds, for curvature scale space descriptor computation for regions.
 */
float
P_curvatureScaleSpaceDescriptor(girlToger tog, girlRegion r, bool verbose)
{
  start_timer();
  girlContour c = girlToger_regionOuterBoundary(tog, r);
  iseCurvatureScaleSpaceDescriptor css = iseCurvatureScaleSpaceDescriptor_new(c);
  iseCurvatureScaleSpaceDescriptorExtraction(css);
  stop_timer();
  const float lTime = time_passed();
  if (verbose) {
    fprintf(stdout, "size of countour of region %d : %d\n", r, girlContour_length(c));
    fprintf(stdout, "CSS descriptor for region %d\n ", r);
    fprintf(stdout, "TIME %g \n", lTime);
    iseCurvatureScaleSpaceDescriptor_write(css);
  }
  iseCurvatureScaleSpaceDescriptor_delete(css);
  girlContour_delete(c);
  return lTime;
}

static
void 
P_computePixelNumber(const girlPoint *pt, void *userData)
{
  int *i = (int *)userData;
  (*i)+=1;
  //fprintf(stderr, "(%d,%d) \t",girlPoint_y(pt), girlPoint_x(pt));
}

/*
 * Return time in seconds, for ART descriptor computation for regions.
 */
float
P_ARTDescriptor(girlToger tog, girlRegion r, bool verbose)
{
  start_timer();
  int sizereg = 0;
  girlToger_iterateOnRegion(tog, r, GIRL_ITERATE_DOMAIN, P_computePixelNumber, &sizereg );
  iseRegionARTDescriptor art = iseRegionARTDescriptor_new(tog, r);
  iseRegionARTDescriptorExtraction(art);
  stop_timer();
  const float lTime = time_passed();
  if (verbose) {
    fprintf(stdout, "ART descriptor for region %d\n", r);
    fprintf(stdout, "TIME %g for area=%d \n", lTime, sizereg);
    iseRegionARTDescriptor_write(art);
  }
  iseRegionARTDescriptor_delete(art);
  return lTime;
}

float
P_edgeHisto(girlToger tog, girlRegion r, bool verbose)
{
  start_timer();
  iseEdgeHistogram eh = iseEdgeHistogram_new(img, tog, r);
  iseEdgeHistogramExtraction(eh);
  stop_timer();
  const float lTime = time_passed();
  if (verbose) {
    fprintf(stdout, "Edge Histo for region %d\n", r);
    fprintf(stdout, "TIME %g \n", lTime);
    iseEdgeHistogram_write(eh);
  }
  iseEdgeHistogram_delete(eh);
  return lTime;
}

int 
main(int argc, char *argv[])
{
  /* program arguments procesing */
  processArgs(argc, argv);
  
  bclMemory_init();


  /* image loading */
  bclPnm loadedImage = NULL;
#ifdef __cplusplus
  loadedImage = bclPnm_load(inFilename);
#else
  BCL_HANDLE(bclAny, loadedImage = bclPnm_load(inFilename));
  if (BCLEXCEPTION_RAISED(bclAny))
    {
      fprintf(stderr, "Unable to load %s\n", inFilename);
      exit(EXIT_FAILURE);
    }
#endif

  width = bclPnm_get_width(loadedImage);
  height = bclPnm_get_height(loadedImage);
  if (BCLIMAGE_NUM_CHANNELS != 3 || sizeof(BCLIMAGE_CHANNEL_TYPE)!=sizeof(unsigned char))
    {
      fprintf(stderr, "unsupported image format for file : %s\n", inFilename);
      exit(EXIT_FAILURE);
    }
  img = bclImage_new(width, height);
  image_start = bclImage_getBuffer(img);
  bclPnm_make_uchar_rgb_image(loadedImage, image_start);
  bclPnm_delete(loadedImage);

  /* Initialization of representation */
  tog    = girlToger_new(width, height);
  regSet = iseRegionSet_new();
  L_new_region_hook(1, NULL);

  girlToger_setNewRegionHook(tog, L_new_region_hook,    NULL);
  girlToger_setDeleteRegionHook(tog, L_delete_region_hook, NULL);

 
  /* Initialization of algorithms */
  P_chooseQuantifAlgo(quantifId);

  segContext = iseSegmentationContext_new(tog, regSet, img, NULL);

  if (splitId == SPLIT_N_MERGE) {
    mergeAlgo = iseMergeRemoveDetailsAlgorithm_new(segContext);
  }
  P_chooseSplitAlgo(splitId);
  


  girlRegion firstRegion = 1;

  P_curvatureScaleSpaceDescriptor(tog, firstRegion, verbose);
  
  P_ARTDescriptor(tog, firstRegion, verbose);
  
  P_edgeHisto(tog, firstRegion, verbose);
   

  if (recursive) {

    /* segmentation */

    int num_regions = iseSplitAlgorithm_apply(splitAlgo, &firstRegion, 1, &regs, &maxVariation);
  
    assert(girlToger_checkConsistency(tog));
    assert(checkRegSet(tog, regSet));


    /****** sa ***/


#if 0
    int petitcpt;

    girlContour co;  

    //B: compute minimum contour length
    //TODO:OPTIM: use iterateRegionBoundary ! 

    int compteurreg ; 
    int pluspetitcontour = girlContour_length(girlToger_regionOuterBoundary(tog, regs[0]));

    for (compteurreg = 1; compteurreg < num_regions; compteurreg++){
      if(girlContour_length(girlToger_regionOuterBoundary(tog, regs[compteurreg])) < pluspetitcontour)
	pluspetitcontour = girlContour_length(girlToger_regionOuterBoundary(tog, regs[compteurreg]));
    }

    int numberSamples = pluspetitcontour / 2;
    int nbFDs = 200;
    if (numberSamples < 200)
      nbFDs = numberSamples;

    iseContourFourierDescriptors ** fdtab;
    fdtab = bclMemory_alloc(num_regions *sizeof(iseContourFourierDescriptors*));
    int nbangles = 360;  
    /*   iseRegionMoments * momentlist = bclMemory_alloc(nb * sizeof(iseRegionMoments)); */
    /*   iseRegionMoments * centralmomentlist = bclMemory_alloc(nb * sizeof(iseRegionMoments)); */
    /*   iseHuMomentInvariants * hu = bclMemory_alloc(nb * sizeof(iseHuMomentInvariants)); */
#endif

    bclMemory_free(regs);
    num_regions = girlToger_allRegions(tog, &regs);


    float totalTimeCSS = 0.0;
    float totalTimeART = 0.0;
    float totalTimeEdgeHisto = 0.0;
  
    for (int i = 0; i < num_regions; ++i) {
    
      const girlRegion r = regs[i];
    
      fprintf(stdout,"\nregion %d (%d/%d)\n", r, i, num_regions);
    

      /* /\*     fprintf(stdout, "calcul des moments \n"); *\/ */
      /* /\*     momentlist[compteurreg] = iseRegionMoments_new(tog,regions[compteurreg],4); *\/ */
      /* /\*     centralmomentlist[compteurreg] = iseRegionCentralMoments_new(tog,regions[compteurreg],4); *\/ */
      /* /\*     fprintf(stdout, "DONE \n"); *\/ */
      /* /\*     fprintf(stdout, "calcul des invariants de Hu \n"); *\/ */
      /* /\*     hu[compteurreg] = iseRegionMoments_computeHuInvariants(tog,regions[compteurreg]); *\/ */
      /* /\*     fprintf(stdout, "DONE \n"); *\/ */

      /*     fdtab[compteurreg] = bclMemory_alloc(18 * sizeof(iseContourFourierDescriptors)); */

      /* //B
	 girlContour * listeco;
	 const int nbcont = girlToger_regionBoundary(tog, regions[compteurreg], &listeco);
	 // // // // fprintf(stdout," contours %d \n ", nbcont); 
     
	 int cptcontour = 0; 

	 //B:TODO:OPTIM: it seems that only outer boundary is used ! 
	 // so we should call girlToger_regionOuterBoundary !!!
	 */

    
      /*     // // // // fprintf(stdout, "Contour %d de taille %d \n", cptcontour, girlContour_length(listeco[cptcontour]) ); */
      /*     // // // // fprintf(stdout, "nombre de samples %d \n", numberSamples); */
      /*     // // // // fprintf(stdout, "nombre de descripteurs %d \n", nbFDs); */
    
      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseComplexShapeSignature avec iseEqualPointsContourSampling \n"); */
      /*     fdtab[compteurreg][0] = iseContourFourierDescriptor_new(listeco[cptcontour],numberSamples,nbFDs,iseEqualPointsContourSampling,iseComplexShapeSignature); */
      /*     //	iseTestAccuracy(listeco[cptcontour],numberSamples,nbFDs,iseEqualArclengthContourSampling,iseComplexShapeSignature); */
      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseComplexShapeSignature avec iseEqualPointsContourSampling done \n"); */
      /*     // iseWriteContourFourierDescriptors(fdtab[compteurreg][0]); */

      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseComplexShapeSignature avec iseEqualPointsContourSamplingWithFloatCoordinates \n"); */
      /*     fdtab[compteurreg][1] = iseContourFourierDescriptor_new(listeco[cptcontour],numberSamples,nbFDs,iseEqualPointsContourSamplingWithFloatCoordinates,iseComplexShapeSignature); */
      /*     //	iseTestAccuracy(listeco[cptcontour],numberSamples,nbFDs,iseEqualArclengthContourSampling,iseComplexShapeSignature); */
      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseComplexShapeSignature avec iseEqualPointsContourSamplingWithFloatCoordinates done \n"); */
      /*     // iseWriteContourFourierDescriptors(fdtab[compteurreg][0]); */


      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseComplexShapeSignature avec iseEqualArclengthContourSampling \n"); */
      /*     fdtab[compteurreg][2] = iseContourFourierDescriptor_new(listeco[cptcontour],numberSamples,nbFDs,iseEqualArclengthContourSampling,iseComplexShapeSignature); */
      /*     //	iseTestAccuracy(listeco[cptcontour],numberSamples,nbFDs,iseEqualArclengthContourSampling,iseComplexShapeSignature); */
      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseComplexShapeSignature avec iseEqualArclengthContourSampling done \n"); */
      /*     // iseWriteContourFourierDescriptors(fdtab[compteurreg][0]); */

      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseComplexShapeSignature avec iseEqualAnglesContourSampling \n");	 */
      /*     fdtab[compteurreg][3] =  iseContourFourierDescriptor_new(listeco[cptcontour],nbangles,nbFDs,iseEqualAnglesContourSampling,iseComplexShapeSignature); */
      /*     //fdtab[compteurreg][1] =  iseContourFourierDescriptor_new(listeco[cptcontour],numberSamples,nbFDs,iseEqualPointsContourSamplingWithFloatCoordinates,iseCentroidShapeSignature); */
      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseComplexShapeSignature avec iseEqualAnglesContourSampling done \n"); */
      /*     // iseWriteContourFourierDescriptors(fdtab[compteurreg][1]); */
      

      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseCentroidShapeSignature / iseEqualPointsContourSampling \n"); */
      /*     fdtab[compteurreg][4]= iseContourFourierDescriptor_new(listeco[cptcontour],numberSamples,nbFDs,iseEqualPointsContourSampling,iseCentroidShapeSignature); */
      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseCentroidShapeSignature / iseEqualPointsContourSampling done \n"); */
      /*     // iseWriteContourFourierDescriptors(fdtab[compteurreg][3]); */
	      


      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseCentroidShapeSignature / iseEqualPointsContourSamplingWithFloatCoordinates \n"); */
      /*     fdtab[compteurreg][5]= iseContourFourierDescriptor_new(listeco[cptcontour],numberSamples,nbFDs,iseEqualPointsContourSamplingWithFloatCoordinates,iseCentroidShapeSignature); */
      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseCentroidShapeSignature / iseEqualPointsContourSamplingWithFloatCoordinates done \n"); */
      /*     // iseWriteContourFourierDescriptors(fdtab[compteurreg][3]); */
	      


	
      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseCentroidShapeSignature / iseEqualArclengthContourSampling \n"); */
      /*     fdtab[compteurreg][6] = iseContourFourierDescriptor_new(listeco[cptcontour],numberSamples,nbFDs,iseEqualArclengthContourSampling,iseCentroidShapeSignature); */
      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseCentroidShapeSignature / iseEqualArclengthContourSampling done \n"); */
      /*     //	iseWriteContourFourierDescriptors(fdtab[compteurreg][2]); */
      /*     //iseTestAccuracy(listeco[cptcontour],numberSamples,nbFDs,iseEqualArclengthContourSampling,iseCentroidShapeSignature); */
      


	
      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseCentroidShapeSignature / iseEqualAnglesContourSampling \n"); */
      /*     fdtab[compteurreg][7] = iseContourFourierDescriptor_new(listeco[cptcontour],nbangles,nbFDs,iseEqualAnglesContourSampling,iseCentroidShapeSignature); */
      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseCentroidShapeSignature / iseEqualArclengthContourSampling done \n"); */
      /*     //	iseWriteContourFourierDescriptors(fdtab[compteurreg][2]); */
      /*     //iseTestAccuracy(listeco[cptcontour],numberSamples,nbFDs,iseEqualArclengthContourSampling,iseCentroidShapeSignature); */
      

      /* /\*     fprintf(stdout, "*******************************\n"); *\/ */
      /* /\*     fprintf(stdout, "iseCurvatureShapeSignature avec iseEqualPointsContourSampling \n"); *\/ */
      /* /\*     fdtab[compteurreg][4] = iseContourFourierDescriptor_new(listeco[cptcontour],numberSamples,nbFDs,iseEqualPointsContourSampling,iseCurvatureShapeSignature); *\/ */
      /* /\*     fprintf(stdout, "*******************************\n"); *\/ */
      /* /\*     fprintf(stdout, "iseCurvatureShapeSignature avec iseEqualPointsContourSampling done \n"); *\/ */
      /* 	// iseWriteContourFourierDescriptors(fdtab[compteurreg][4]); */


      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseCurvatureShapeSignature avec iseEqualPointsContourSamplingWithFloatCoordinates \n"); */
      /*     fdtab[compteurreg][8] = iseContourFourierDescriptor_new(listeco[cptcontour],numberSamples,nbFDs,iseEqualPointsContourSamplingWithFloatCoordinates,iseCurvatureShapeSignature); */
      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseCurvatureShapeSignature avec iseEqualPointsContourSamplingWithFloatCoordinates done \n"); */
      /* 	// iseWriteContourFourierDescriptors(fdtab[compteurreg][4]); */

    
    
      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseCurvatureShapeSignature avec equalarclength \n"); */
      /*     fdtab[compteurreg][9] = iseContourFourierDescriptor_new(listeco[cptcontour],numberSamples,nbFDs,iseEqualArclengthContourSampling,iseCurvatureShapeSignature); */
      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseCurvatureShapeSignature avec equalarclength done \n"); */
      /* 	// iseWriteContourFourierDescriptors(fdtab[compteurreg][4]); */


      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseCurvatureShapeSignature avec equalangles \n"); */
      /*     fdtab[compteurreg][10]= iseContourFourierDescriptor_new(listeco[cptcontour],nbangles,nbFDs,iseEqualAnglesContourSampling,iseCurvatureShapeSignature); */
      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseCurvatureShapeSignature avec equalangles done \n"); */
      /*     // iseWriteContourFourierDescriptors(fdtab[compteurreg][7]); */
	

      /* /\*     fprintf(stdout, "*******************************\n"); *\/ */
      /* /\*     fprintf(stdout, "iseCumulativeAngularShapeSignature avec iseEqualPointsContourSampling \n"); *\/ */
      /* /\*     fdtab[compteurreg][11]= iseContourFourierDescriptor_new(listeco[cptcontour],numberSamples,nbFDs,iseEqualPointsContourSampling,iseCumulativeAngularShapeSignature); *\/ */
      /* /\*     fprintf(stdout, "*******************************\n"); *\/ */
      /* /\*     fprintf(stdout, "iseCumulativeAngularShapeSignature avec iseEqualPointsContourSampling done \n"); *\/ */
      /* /\*     // iseWriteContourFourierDescriptors(fdtab[compteurreg][5]); *\/ */

      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseCumulativeAngularShapeSignature avec iseEqualPointsContourSamplingWithFloatCoordinates \n"); */
      /*     fdtab[compteurreg][11]= iseContourFourierDescriptor_new(listeco[cptcontour],numberSamples,nbFDs,iseEqualPointsContourSamplingWithFloatCoordinates,iseCumulativeAngularShapeSignature); */
      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseCumulativeAngularShapeSignature avec iseEqualPointsContourSamplingWithFloatCoordinates done \n"); */
      /*     // iseWriteContourFourierDescriptors(fdtab[compteurreg][5]); */

	
      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseCumulativeAngularShapeSignature avec equalarclength \n"); */
      /*     fdtab[compteurreg][12]= iseContourFourierDescriptor_new(listeco[cptcontour],numberSamples,nbFDs,iseEqualArclengthContourSampling,iseCumulativeAngularShapeSignature); */
      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseCumulativeAngularShapeSignature avec equalarclength done \n"); */
      /*     // iseWriteContourFourierDescriptors(fdtab[compteurreg][5]); */
	
      /*     //	// // // // fprintf(stdout, "*******************************\n"); */

      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseCumulativeAngularShapeSignature avec equalangles \n"); */
      /*     fdtab[compteurreg][13]= iseContourFourierDescriptor_new(listeco[cptcontour],nbangles,nbFDs,iseEqualAnglesContourSampling,iseCumulativeAngularShapeSignature); */
      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseCumulativeAngularShapeSignature avec equalangles done \n"); */
      /*     // iseWriteContourFourierDescriptors(fdtab[compteurreg][6]); */

      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseAreaShapeSignature avec iseEqualPointsContourSampling \n"); */
      /*     fdtab[compteurreg][14]= iseContourFourierDescriptor_new(listeco[cptcontour],numberSamples,nbFDs,iseEqualPointsContourSampling,iseAreaShapeSignature); */
      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseAreaShapeSignature avec iseEqualPointsContourSampling done \n"); */

    
      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseAreaShapeSignature avec equalpointswithfloat \n"); */
      /*     fdtab[compteurreg][15]= iseContourFourierDescriptor_new(listeco[cptcontour],numberSamples,nbFDs,iseEqualPointsContourSamplingWithFloatCoordinates,iseAreaShapeSignature); */
      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseAreaShapeSignature avec equalpointswithfloat done \n"); */
    
      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseAreaShapeSignature avec equalarclength \n"); */
      /*     fdtab[compteurreg][16]= iseContourFourierDescriptor_new(listeco[cptcontour],numberSamples,nbFDs,iseEqualArclengthContourSampling,iseAreaShapeSignature); */
      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseAreaShapeSignature avec equalarclength done \n"); */
    

      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseAreaShapeSignature avec equalangle \n"); */
      /*     fdtab[compteurreg][17]= iseContourFourierDescriptor_new(listeco[cptcontour],nbangles,nbFDs,iseEqualAnglesContourSampling,iseAreaShapeSignature); */
      /*     // // // // fprintf(stdout, "*******************************\n"); */
      /*     // // // // fprintf(stdout, "iseAreaShapeSignature avec equalangle done \n"); */
    


      // fprintf(stdout, "*******************************\n");
    
      totalTimeCSS += P_curvatureScaleSpaceDescriptor(tog, r, verbose);
    
      totalTimeART += P_ARTDescriptor(tog, r, verbose);
    
      totalTimeEdgeHisto += P_edgeHisto(tog, r, verbose);
    
    }
  
    /*   int compteurfd = 0; */
    /*   int pos = 0; */
    /*  for (compteurreg = 0; compteurreg < nb; compteurreg++){  */
    /*    fprintf( stdout, "region %d \n", regions[compteurreg]); */

    /*    fprintf(stdout, "*************************************"); */
    /*    for (pos=0; pos < 18 ; pos++){ */
    /*      fprintf(stdout, " methode %d \n ", pos);  */
    /*      iseWriteContourFourierDescriptors(fdtab[compteurreg][pos]); */
    /*      fprintf(stdout, "\n"); */
    /*    } */
    /*    fprintf(stdout, "*************************************"); */
    /* /\*    fprintf(stdout, "Moments \n"); *\/ */
    /* /\*    iseWriteRegionMoments(momentlist[compteurreg]); *\/ */
    /* /\*    fprintf(stdout, "Central Moments \n"); *\/ */
    /* /\*    iseWriteRegionMoments(centralmomentlist[compteurreg]); *\/ */
    /* /\*    fprintf(stdout, "Hu invariants \n"); *\/ */
    /* /\*    iseWriteRegionHuInvariants(hu[compteurreg]); *\/ */


    /* /\*    fprintf(stdout, "Verif \n mu[2][0] = %f \n ", iseRegionMoments_getMoment(momentlist[compteurreg],2,0) - iseRegionMoments_getMoment(momentlist[compteurreg],0,0) * BCL_ELM(iseRegionMoments_getCentroid(momentlist[compteurreg]), 0) * BCL_ELM(iseRegionMoments_getCentroid(momentlist[compteurreg]), 0)); *\/ */
    /* /\*     fprintf(stdout, "mu[1][1] = %f \n ", iseRegionMoments_getMoment(momentlist[compteurreg],1,1) - iseRegionMoments_getMoment(momentlist[compteurreg],0,0) * BCL_ELM(iseRegionMoments_getCentroid(momentlist[compteurreg]), 0) * BCL_ELM(iseRegionMoments_getCentroid(momentlist[compteurreg]), 1)); *\/ */
    /* /\*    fprintf(stdout, "mu[0][2] = %f \n ", iseRegionMoments_getMoment(momentlist[compteurreg],0,2) - iseRegionMoments_getMoment(momentlist[compteurreg],0,0) * BCL_ELM(iseRegionMoments_getCentroid(momentlist[compteurreg]), 1) * BCL_ELM(iseRegionMoments_getCentroid(momentlist[compteurreg]), 1)); *\/ */


    /*    fprintf(stdout, "Descripteurs de Fourier \n"); */


    /*    for(compteurfd = 0 ; compteurfd < nb ; compteurfd ++){ */
     
    /*      for (pos=0; pos < 18 ; pos++){ */
    /*        fprintf(stdout, "distance %f ", iseDistanceFourierDescriptors(fdtab[compteurreg][pos], fdtab[compteurfd][pos])); */
    /*        fprintf( stdout, "entre region %d et region %d pour methode %d \n", regions[compteurreg], regions[compteurfd], pos); */
    /*        //pos = 2; */
    /* 	//  fprintf(stdout, "distance %f ", iseDistanceFourierDescriptors(fdtab[compteurreg][pos], fdtab[compteurfd][pos])); */
    /* 	//fprintf( stdout, "entre region %d et region %d pour algo %d \n", regions[compteurreg], regions[compteurfd], pos); */
    /*      } */
    /*    } */
    /*    //iseRegionMoments_delete(momentlist[compteurreg]); */
    /*  } */


 

    /*  bclMemory_free(momentlist); */
    /*  bclMemory_free(centralmomentlist); */
    /*  bclMemory_free(hu); */
    /*   bclFlt eps = sqrt(0.01 * numberSamples); */
    /*   fprintf(stdout, "Bilan avec epsilon = %f \n", eps); */
    /*   for (compteurreg = 0; compteurreg < nb - 1; compteurreg++){ */
    /*     fprintf( stdout, "region %d \n", regions[compteurreg]); */
    /*     for(compteurfd = compteurreg + 1; compteurfd < nb - 1; compteurfd ++){ */
    /*       int pos; */
    /*       for (pos=0; pos < 8 ; pos++){ */
    /* 	//	fprintf( stdout, "descripteurs regions %d et %d pour la methode %d \n", regions[compteurreg],  regions[compteurfd], pos); */
    /* 	//iseWriteContourFourierDescriptors(fdtab[compteurreg][pos]); */
    /* 	//iseWriteContourFourierDescriptors(fdtab[compteurfd][pos]); */
    /* 	if (iseSimilarFourierDescriptors(fdtab[compteurreg][pos], fdtab[compteurfd][pos],eps)){ */
    /* 	  fprintf(stdout, "distance %f ", iseDistanceFourierDescriptors(fdtab[compteurreg][pos], fdtab[compteurfd][pos])); */
    /* 	  fprintf( stdout, "region %d similaire region %d pour algo %d \n", regions[compteurreg], regions[compteurfd], pos); */
    /* 	} */
    /* 	//	else */
    /* 	//	  fprintf( stdout, "region %d NON similaire region %d pour algo %d \n", regions[compteurreg], regions[compteurfd], pos); */
    /*       } */
    /*     } */
    
    /*   } */

    /*   eps = sqrt(0.0004 * numberSamples); */
    /*   fprintf(stdout, "Bilan avec epsilon = %f \n", eps); */
    /*   for (compteurreg = 0; compteurreg < nb - 1; compteurreg++){ */
    /*     fprintf( stdout, "region %d \n", regions[compteurreg]); */
    /*     for(compteurfd = compteurreg + 1; compteurfd < nb - 1; compteurfd ++){ */
    /*       int pos; */
    /*       for (pos=0; pos < 8 ; pos++){ */
    /* 	//	fprintf( stdout, "descripteurs regions %d et %d pour la methode %d \n", regions[compteurreg],  regions[compteurfd], pos); */
    /* 	//iseWriteContourFourierDescriptors(fdtab[compteurreg][pos]); */
    /* 	//iseWriteContourFourierDescriptors(fdtab[compteurfd][pos]); */
    /* 	if (iseSimilarFourierDescriptors(fdtab[compteurreg][pos], fdtab[compteurfd][pos],eps)){ */
    /* 	  fprintf(stdout, "distance %f ", iseDistanceFourierDescriptors(fdtab[compteurreg][pos], fdtab[compteurfd][pos])); */
    /* 	  fprintf( stdout, "region %d similaire region %d pour algo %d \n", regions[compteurreg], regions[compteurfd], pos); */
    /* 	} */
    /* 	//	else */
    /* 	//	  fprintf( stdout, "region %d NON similaire region %d pour algo %d \n", regions[compteurreg], regions[compteurfd], pos); */
    /*       } */
    /*     } */
    /*   } */

    /*   eps = sqrt(0.001 * numberSamples); */
    /*   fprintf(stdout, "Bilan avec epsilon = %f \n", eps); */
    /*   for (compteurreg = 0; compteurreg < nb - 1; compteurreg++){ */
    /*     fprintf( stdout, "region %d \n", regions[compteurreg]); */
    /*     for(compteurfd = compteurreg + 1; compteurfd < nb - 1; compteurfd ++){ */
    /*       int pos; */
    /*       for (pos=0; pos < 8 ; pos++){ */
    /* 	//	fprintf( stdout, "descripteurs regions %d et %d pour la methode %d \n", regions[compteurreg],  regions[compteurfd], pos); */
    /* 	//iseWriteContourFourierDescriptors(fdtab[compteurreg][pos]); */
    /* 	//iseWriteContourFourierDescriptors(fdtab[compteurfd][pos]); */
    /* 	if (iseSimilarFourierDescriptors(fdtab[compteurreg][pos], fdtab[compteurfd][pos],eps)){ */
    /* 	  fprintf(stdout, "distance %f ", iseDistanceFourierDescriptors(fdtab[compteurreg][pos], fdtab[compteurfd][pos])); */
    /* 	  fprintf( stdout, "region %d similaire region %d pour algo %d \n", regions[compteurreg], regions[compteurfd], pos); */
    /* 	} */
    /* 	//	else */
    /* 	//	  fprintf( stdout, "region %d NON similaire region %d pour algo %d \n", regions[compteurreg], regions[compteurfd], pos); */
    /*       } */
    /*     } */
    /*     //    fprintf(stdout,"************************ \n" ); */
    /*   } */

    /*   eps = sqrt(0.001 * numberSamples); */
    /*   fprintf(stdout, "Bilan avec epsilon = %f \n", eps); */
    /*   for (compteurreg = 0; compteurreg < nb - 1; compteurreg++){ */
    /*     fprintf( stdout, "region %d \n", regions[compteurreg]); */
    /*     for(compteurfd = compteurreg + 1; compteurfd < nb - 1; compteurfd ++){ */
    /*       int pos; */
    /*       for (pos=0; pos < 8 ; pos++){ */
    /* 	//	fprintf( stdout, "descripteurs regions %d et %d pour la methode %d \n", regions[compteurreg],  regions[compteurfd], pos); */
    /* 	//iseWriteContourFourierDescriptors(fdtab[compteurreg][pos]); */
    /* 	//iseWriteContourFourierDescriptors(fdtab[compteurfd][pos]); */
    /* 	if (iseSimilarFourierDescriptors(fdtab[compteurreg][pos], fdtab[compteurfd][pos],eps)){ */
    /* 	  fprintf(stdout, "distance %f ", iseDistanceFourierDescriptors(fdtab[compteurreg][pos], fdtab[compteurfd][pos])); */
    /* 	  fprintf( stdout, "region %d similaire region %d pour algo %d \n", regions[compteurreg], regions[compteurfd], pos); */
    /* 	} */
    /* 	//	else */
    /* 	//	  fprintf( stdout, "region %d NON similaire region %d pour algo %d \n", regions[compteurreg], regions[compteurfd], pos); */
    /* 	//	fprintf(stdout, "\n"); */
    /* 	//	fprintf(stdout,"************************ \n" ); */
    /*       } */
    /*     } */
    /*     //    fprintf(stdout,"************************ \n" ); */
    /*   } */

    /*   eps = 0.2; */
    /*   fprintf(stdout, "Bilan avec epsilon = %f \n", eps); */
    /*   for (compteurreg = 0; compteurreg < nb - 1; compteurreg++){ */
    /*     fprintf( stdout, "region %d \n", regions[compteurreg]); */
    /*     for(compteurfd = compteurreg + 1; compteurfd < nb - 1; compteurfd ++){ */
    /*       int pos; */
    /*       for (pos=0; pos < 8 ; pos++){ */
    /* 	//	fprintf( stdout, "descripteurs regions %d et %d pour la methode %d \n", regions[compteurreg],  regions[compteurfd], pos); */
    /* 	//iseWriteContourFourierDescriptors(fdtab[compteurreg][pos]); */
    /* 	//iseWriteContourFourierDescriptors(fdtab[compteurfd][pos]); */
    /* 	if (iseSimilarFourierDescriptors(fdtab[compteurreg][pos], fdtab[compteurfd][pos],eps)){ */
    /* 	  fprintf(stdout, "distance %f ", iseDistanceFourierDescriptors(fdtab[compteurreg][pos], fdtab[compteurfd][pos])); */
    /* 	  fprintf( stdout, "region %d similaire region %d pour algo %d \n", regions[compteurreg], regions[compteurfd], pos); */
    /* 	} */
    /* 	//	fprintf(stdout, "\n"); */
    /* 	//	else */
    /* 	//	  fprintf( stdout, "region %d NON similaire region %d pour algo %d \n", regions[compteurreg], regions[compteurfd], pos); */
    /* 	//	fprintf(stdout,"************************ \n" ); */
    /*       } */
    /*     } */
    /*     //    fprintf(stdout,"************************ \n" ); */
    /*   } */


    /*  for (compteurreg = 0; compteurreg < nb ; compteurreg++){ */
    /*    for (pos=0; pos < 18 ; pos++){ */
    /*      //     fprintf(stdout, "(%d,%d) = %#x \t", compteurreg, pos, fdtab[compteurreg][pos]); */
    /*      iseContourFourierDescriptors_delete(fdtab[compteurreg][pos]); */
    /*    } */
    /*    // fprintf(stdout, "(%d) = %#x \t", compteurreg,  fdtab[compteurreg]); */
    /*    bclMemory_free(fdtab[compteurreg]); */
    /*    fdtab[compteurreg] = NULL; */
    /*  } */
 
    /*  // fprintf(stdout, "\n fdtab :  %#x \n", fdtab); */
    /*  bclMemory_free(fdtab); */
    /*  fdtab = NULL; */
    /************* fin sa/  */

    /*   fprintf (stdout, "total number of found regions = %d\n", num_regions); */
    /*   fprintf(stdout, "total time for segmentation : %ld ms\n", (t1.tv_sec-t0.tv_sec)*1000 + (t1.tv_usec-t0.tv_usec)/1000 ); */

    fprintf(stdout, "nb regions : %d\n", num_regions);
    fprintf(stdout, "total time for CSS : %g\n", totalTimeCSS);
    fprintf(stdout, "total time for ART : %g\n", totalTimeART);
    fprintf(stdout, "total time for Edge Histogram : %g\n", totalTimeEdgeHisto);

    bclMemory_free(regs);

    /* segmentation backup */
    if (outFilename)
      saveTogerBitmapToFile(outFilename, "", tog);
  
  }
  /* cleaning */

  if (splitId == SPLIT_N_MERGE) {
    iseMergeAlgorithm_delete(mergeAlgo);
  }
  iseSplitAlgorithm_delete(splitAlgo);
  iseQuantification_delete(quantif);
  iseSegmentationContext_delete(segContext);

  iseRegionSet_delete(regSet);
  girlToger_delete(tog);

  bclImage_delete(img);


  bclMemory_terminate();


  return EXIT_SUCCESS;
}

