/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef CONFUSIONMATRIX_HPP_
#define CONFUSIONMATRIX_HPP_
#include <girl/girlCommon.hpp>
#include <vector>
#include <cassert>
#include <iostream>

class GIRL_API ConfusionMatrix
{
public:
  explicit ConfusionMatrix(size_t numberOfConcepts);
  virtual ~ConfusionMatrix();

  void increment(size_t trueClass, size_t inferredClass);

  double nbTrueElem(size_t concept) const;

  double nbInferedElem(size_t concept) const;

  double getValue(size_t trueClass, size_t inferredClass) const;

  double recall(size_t concept) const;

  double recall(size_t trueClass, size_t inferredClass) const;

  double precision(size_t concept) const;

  double precision(size_t trueClass, size_t inferredClass) const;

  double fmeasure(size_t concept, double alpha = 1.0) const;

  double fmeasure(size_t trueClass, size_t inferredClass, double alpha = 1.0) const;

  double averageRecall() const;

  double averagePrecision() const;

  void print(std::ostream &) const;

private:

  bool isPresent(size_t concept) const;

  size_t m_numberOfConcepts;

  std::vector<std::vector<double> > m_matrix;

 };

#endif /* CONFUSIONMATRIX_HPP_ */
