/*
  Copyright (C) 2005-2011 Remi Vieux, Boris Mansencal, remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
#ifndef ENCLOSING_CIRCLE_HPP
#define ENCLOSING_CIRCLE_HPP

#include "EnclosingShape.hpp"

class GIRL_API EnclosingCircle : public EnclosingShape
{

public:

  EnclosingCircle(const cv::Point2f &pt, float r) : m_center(pt), m_radius(r) {}
  EnclosingCircle() : m_center(cv::Point2f()), m_radius(0) {}

  ~EnclosingCircle() {}

  virtual void draw(cv::Mat &,
		    const cv::Scalar &color = ENCLOSINGSHAPE_DEFAULT_COLOR,
		    int lineWidth = ENCLOSINGSHAPE_DEFAULT_LINEWIDTH) const;

  virtual void compute(const cv::Mat &);

  virtual double accept (ise::EnclosingShapeFeatureVisitor &visitor) ;

  cv::Rect boundingRect() const
  {
    // warning, min and max could be outside the image range, it is client
    // responsibility to check that if needded
    int minX = static_cast<int>(m_center.x - m_radius - 0.5);
    int minY = static_cast<int>(m_center.y - m_radius - 0.5);
    int maxX = static_cast<int>(m_center.x + m_radius + 0.5);
    int maxY = static_cast<int>(m_center.y + m_radius + 0.5);
    cv::Point tl(minX, minY);
    cv::Rect rect(minX, minY, maxX - minX + 1, maxY - minY + 1);
    return rect;
  }
  cv::Point2f getCenter();

  float getRadius();

protected:
  cv::Point2f m_center;
  float m_radius;
};


#endif // ENCLOSING_CIRCLE_HPP
