/*
  Copyright (C) 2005-2011 Remi Vieux, Boris Mansencal, remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
#ifndef ENCLOSINGCONNECTEDCOMPONENTS_HPP_
#define ENCLOSINGCONNECTEDCOMPONENTS_HPP_

#include <vector>
#include <limits>

#include "EnclosingShape.hpp"
/**
 * The shape is the list of pixels in the connected component
 */
class GIRL_API EnclosingConnectedComponents : public EnclosingShape
{
public:
  EnclosingConnectedComponents() : m_points() {}

  virtual ~EnclosingConnectedComponents() {}

  void
  draw(cv::Mat &, const cv::Scalar&, int) const;

  void
  compute(const cv::Mat &points);

  virtual double accept(ise::EnclosingShapeFeatureVisitor &)
  {
    // TODO
    //assert(false);
    return 0.0;
  }

  cv::Rect boundingRect() const
  {
    int minX = std::numeric_limits<int>::max();
    int minY = std::numeric_limits<int>::max();
    int maxX = std::numeric_limits<int>::min();
    int maxY = std::numeric_limits<int>::min();
    std::vector<cv::Point>::const_iterator it = m_points.begin();
    const std::vector<cv::Point>::const_iterator itEnd = m_points.end();
    for ( ; it != itEnd; ++it) {
        if (it->x < minX)
          minX = it->x;
        if (it->y < minY)
          minY = it->y;
        if (it->x > maxX)
          maxX = it->x;
        if (it->y > maxY)
          maxY = it->y;
    }
    return cv::Rect(minX, minY, maxX - minX + 1, maxY - minY + 1);
  }

protected:

  std::vector<cv::Point> m_points;
};

#endif /* ENCLOSINGCONNECTEDCOMPONENTS_HPP_ */
