/*
  Copyright (C) 2005-2011 Remi Vieux, Boris Mansencal, remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
#include "EnclosingEllipse.hpp"
#include <iostream>

void
EnclosingEllipse::draw(cv::Mat &mat, const cv::Scalar &color, int lineWidth) const
{
  if (m_rect.size.width >= 2 && m_rect.size.height >= 2) {
    cv::ellipse(mat, m_rect, color, lineWidth);
    //cv::ellipse(mat, m_rect.center, cv::Size(m_rect.size.width/2.0, m_rect.size.height/2.0), m_rect.angle, 0, 360, color, lineWidth);
  }
}

void
EnclosingEllipse::compute(const cv::Mat &points)
{
  if (points.rows >= 6) {
    //cv::fitEllipse needs at least 6 points !!!
    m_rect = cv::fitEllipse(points);
  }
  else {
    //B:TODO: what do we do then ???
    
    //here, we compute axis aligned bounding rect
    
    cv::Rect rect = cv::boundingRect(points);
    
    //REM: consider point at center of pixels...

    const cv::Point2f center(rect.x + (rect.width-1)/2.0f,
			     rect.y + (rect.height-1)/2.0f);

    const cv::Size2f size(static_cast<float>(rect.width), 
			  static_cast<float>(rect.height));

    m_rect = cv::RotatedRect(center, size, 0);
    
  }

}

double EnclosingEllipse::accept(ise::EnclosingShapeFeatureVisitor &visitor)
{
    return visitor.visit(this);
}

cv::RotatedRect EnclosingEllipse::getRotatedRect()
{
    return m_rect;
}
