/*
  Copyright (C) 2005-2011 Remi Vieux, Boris Mansencal, remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
#ifndef ENCLOSING_ROTATED_RECT_HPP
#define ENCLOSING_ROTATED_RECT_HPP

#include "EnclosingShape.hpp"

/**
 * Minimal area bounding rectangle
 */
class GIRL_API EnclosingRotatedRect : public EnclosingShape
{

 public:

  explicit EnclosingRotatedRect(const cv::RotatedRect &rect) : m_rect(rect) {}

  EnclosingRotatedRect(const cv::Point2f &center = cv::Point2f(0., 0.),
		       const cv::Size2f &size = cv::Size2f(0., 0.),
		       float angle = 0.)
    : m_rect(center, size, angle) {}

  ~EnclosingRotatedRect() {}

  virtual void draw(cv::Mat &,
		    const cv::Scalar &color = ENCLOSINGSHAPE_DEFAULT_COLOR,
		    int lineWidth = ENCLOSINGSHAPE_DEFAULT_LINEWIDTH) const;

  virtual void compute(const cv::Mat &);

  virtual double accept (ise::EnclosingShapeFeatureVisitor &visitor);

  cv::RotatedRect getRect();

  cv::Rect boundingRect() const
  {
    return m_rect.boundingRect();
  }

 protected:
  cv::RotatedRect m_rect;
};

#endif // ENCLOSING_ROTATED_RECT_HPP
