/*
  Copyright (C) 2005-2011 Remi Vieux, Boris Mansencal, remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
#ifndef ENCLOSING_SHAPE_HPP
#define ENCLOSING_SHAPE_HPP


#include <opencv2/core/core.hpp>
#include <opencv2/imgproc/imgproc.hpp>

#include <girl/girlCommon.hpp>
#include <ise/iseEnclosingShapeFeatureVisitor.hpp>

/**
 * Defines the minimum operations required to launch the segmentation
 * by aggregation of connected components. 
 */

#define ENCLOSINGSHAPE_DEFAULT_COLOR CV_RGB(128, 128, 128)
#define ENCLOSINGSHAPE_DEFAULT_LINEWIDTH 0

class GIRL_API EnclosingShape {

  //  public EnclosingShape
  
public:
  

  virtual ~EnclosingShape() {}

  /**
   * Paint itself in the given image @mat with color @color and linewidth @lineWidth
   */
  virtual void draw(cv::Mat & mat,
		    const cv::Scalar &color = ENCLOSINGSHAPE_DEFAULT_COLOR,
		    int lineWidth = ENCLOSINGSHAPE_DEFAULT_LINEWIDTH) const = 0;

  /**
   * Compute the enclosing shape of the list of points given in parameter.
   * @points The input 2D point set, represented by CV 32SC2 or CV 32FC2 matrix,
   * or by vector<Point> or vector<Point2f> converted to the matrix using 
   * Mat(const vector<T>&) constructor.
   */
  virtual void compute(const cv::Mat & points) = 0;

  /**
   * Returns the minimal up-right rectangle that contains the shape
   */
  virtual cv::Rect boundingRect() const = 0;


/**
  * Applies the given visitor.
  * \see iseEnclosingShapeFeatureVisitor.hpp
  */
  virtual double accept (ise::EnclosingShapeFeatureVisitor &visitor) = 0;

};

#endif // ENCLOSING_SHAPE_HPP
