/*
  Merge driver with threshold on colors distance.
  Copyright (C) 2005-2009 Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISEMERGEDRIVERCOLORDISTANCE_HPP
#define ISEMERGEDRIVERCOLORDISTANCE_HPP


#include <girl/girlCommon.hpp>
#include "iseMergeDriverDefault.hpp"
#include "iseRegionFeatureTools.hpp"
namespace ise {

  class SegmentationContext;
  
  class GIRL_API MergeDriverColorDistance : public MergeDriverDefault
  {
  public :
    
    MergeDriverColorDistance(ise::SegmentationContext *segContext = NULL, double threshold = 0.5);
    
    virtual ~MergeDriverColorDistance() {}

    
    virtual void setParameters(double threshold,
                               const std::string &regSetUri/*,
			       int colorSpaceConversion,
			       const std::vector<double> &channelsCoeffs*/);

    virtual void getParameters(double &threshold,
                               std::string &regSetUri/*,
			       int &colorSpaceConversion,
			       std::vector<double> &channelsCoeffs*/);

    
    virtual void init(const girl::regionDeque &regionsToMerge, int level);
    
    //virtual bool less(girl::region r1, girl::region r2);

    //virtual void filter(girl::regionDeque &regionsToMerge, int level);
    
    virtual double distance(girl::region reg1Id, girl::region reg2Id);
    
    virtual bool allowed(girl::region reg1Id, girl::region reg2Id, int level);
        
    //virtual girl::region selectCandidate(girl::region regId, int level);
    
    //virtual girl::region selectCandidate(girl::region regId, const girl::regionDeque &regions, int level);
    
    //virtual girl::region execute(girl::region reg1Id, girl::region reg2Id);
    

  public:

    class MergeDriverColorDistanceMemento : public MergeDriver::MergeDriverMemento
    {
      friend class MergeDriverColorDistance;
      
    public:
      MergeDriverColorDistanceMemento() : MergeDriverMemento(),
      m_threshold(0.5),
      m_colorMaxDistance(0) {}

      virtual ~MergeDriverColorDistanceMemento() {}

    protected:
      virtual void getParameters(const Mementable &m)
      {
	const MergeDriverColorDistance &md = reinterpret_cast<const MergeDriverColorDistance &>(m);
	MergeDriverMemento::getParameters(md);
	m_threshold = md.m_threshold;
//	m_conversion = md.m_conversion;
//	m_weightedChannelsCoeffs = md.m_weightedChannelsCoeffs;
	m_colorMaxDistance = md.m_colorMaxDistance;
      }

      virtual void setParameters(Mementable &m) const
      {
	MergeDriverColorDistance &md = reinterpret_cast<MergeDriverColorDistance &>(m);
	MergeDriverMemento::setParameters(md);
	md.m_threshold = m_threshold;
//	md.m_conversion = m_conversion;
//	md.m_weightedChannelsCoeffs = m_weightedChannelsCoeffs;
	md.m_colorMaxDistance = m_colorMaxDistance;
      }

    protected:
      double m_threshold;
//      int m_conversion;
//      std::vector<double> m_weightedChannelsCoeffs;
//
      double m_colorMaxDistance;
      
    };

  protected:

    virtual MergeDriverMemento *createMemento() const
    {
      return new MergeDriverColorDistanceMemento;
    }

  protected:
    //    std::string m_regSetUri;
    double m_threshold;
//    int m_conversion;
//    std::vector<double> m_weightedChannelsCoeffs;

    double m_colorMaxDistance;

  };
  
  
}

#endif /* ! ISEMERGEDRIVERCOLORDISTANCE_HPP */
