/*
  Merge driver with threshold on colors homogeneity.
  Copyright (C) 2005-2009 Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
#include "iseMergeDriverColorHomogeneity.hpp"

#include <algorithm>
#include <cmath>
#include <limits>

//#include "iseRegionColorimetricFeatures.hpp"
//#include "iseRegionSet.hpp"
//#include "iseRegionTools.hpp"
#include "iseImageFeatureColorSpace.hpp"
#include "iseRegionFeatureMomentComputer.hpp"
#include "iseSegmentationContext.hpp"
#include "iseStat.hpp"



using namespace ise;



static const int DEFAULT_CONVERSION = ISE_RGB_to_K1K2K3;


MergeDriverColorHomogeneity::MergeDriverColorHomogeneity(ise::SegmentationContext *segContext,
    double threshold)
: MergeDriverDefault(segContext),
  m_threshold(threshold),
//  m_conversion(DEFAULT_CONVERSION),
//  m_weightedChannelsCoeffs(3, 1.0/3.0),  //TODO : better initialization !!!
  m_colorHomogeneityMaxDistance(0)
{

}


void 
MergeDriverColorHomogeneity::setParameters(double threshold,
    const std::string &regSetUri
    /*int colorSpaceConversion,
    const std::vector<double> &channelsCoeffs*/)
{
  m_threshold  = threshold;
  m_regSetUris.clear();
  m_regSetUris.push_back(regSetUri);
//  m_conversion = colorSpaceConversion;
//  m_weightedChannelsCoeffs = channelsCoeffs;

} 

void 
MergeDriverColorHomogeneity::getParameters(double &threshold,
    std::string &regSetUri
    /*int &colorSpaceConversion,
    std::vector<double> &channelsCoeffs*/)
{
  threshold  = m_threshold;
  if (m_regSetUris.size() >= 1) {
    assert(m_regSetUris.size() == 1);
    regSetUri = m_regSetUris[0];
  }
//  colorSpaceConversion = m_conversion;
//  channelsCoeffs = m_weightedChannelsCoeffs;
} 

void 
MergeDriverColorHomogeneity::init(const girl::regionDeque &/*regionsToMerge*/, int /* level */)
{
  //  assert(m_weightedChannelsCoeffs.size() == 3);
  //
  //  const girl::tools::Flt3 cMin(ise::conversion::absoluteMin(m_conversion, 0)*m_weightedChannelsCoeffs[0],
  //			       ise::conversion::absoluteMin(m_conversion, 1)*m_weightedChannelsCoeffs[1],
  //			       ise::conversion::absoluteMin(m_conversion, 2)*m_weightedChannelsCoeffs[2]*m_weightedChannelsCoeffs[2]);
  //
  //  const girl::tools::Flt3 cMax(ise::conversion::absoluteMax(m_conversion, 0)*m_weightedChannelsCoeffs[0],
  //			       ise::conversion::absoluteMax(m_conversion, 1)*m_weightedChannelsCoeffs[1],
  //			       ise::conversion::absoluteMax(m_conversion, 2)*m_weightedChannelsCoeffs[2]*m_weightedChannelsCoeffs[2]);
  //
  assert(m_regSetUris.size() == 1);
  const RegionSetInterface *regSet = m_segContext->getRegionSet(m_regSetUris[0]);
  assert(regSet);
  const RegionFeatureComputer &featureComputer = regSet->getFeatureComputer();
  try {
    const RegionFeatureComputerMoment& momentComputer = static_cast<const RegionFeatureComputerMoment&>(featureComputer);
    std::string imageFeatureURI;
    momentComputer.getParameters(imageFeatureURI);
    const ImageFeatureColorSpace* imgFeatureComputer = static_cast<const ise::ImageFeatureColorSpace*>(m_segContext->getImageFeatureComputer(imageFeatureURI));
    assert(imgFeatureComputer);
    int colorSpace;
    std::vector<double> weights;
    imgFeatureComputer->getParameters(colorSpace, weights);

    const girl::tools::Flt3 cMin(weights[0]*ise::conversion::absoluteMin(colorSpace, 0),
                                 weights[1]*ise::conversion::absoluteMin(colorSpace, 1),
                                 weights[2]*weights[2]*ise::conversion::absoluteMin(colorSpace, 2));

    const girl::tools::Flt3 cMax(weights[0]*ise::conversion::absoluteMax(colorSpace, 0),
                                 weights[1]*ise::conversion::absoluteMax(colorSpace, 1),
                                 weights[2] *weights[2]*ise::conversion::absoluteMax(colorSpace, 2));



    const double distMax = iseFlt3_dist(cMin, cMax);

    assert(distMax >= 0);

    m_colorHomogeneityMaxDistance = sqrt(distMax)/2;
  } catch(std::bad_cast &bc) {
    std::cerr << "Cannot convert region feature computer to a moment feature computer !!"
        << __FILE__ << ":" << __LINE__ << std::endl;
    assert(false);
  }

  //m_segContext->addRegionSet(m_conversion);

}


bool
MergeDriverColorHomogeneity::allowed(girl::region reg1Id, girl::region reg2Id, int /* level */)
{
  return distance(reg1Id, reg2Id) < m_threshold;
}


double
MergeDriverColorHomogeneity::distance(girl::region reg1Id, girl::region reg2Id)
{

  // Get the Region Colorimetric Features
  assert(m_regSetUris.size() == 1);
  const RegionSetInterface *regSet = m_segContext->getRegionSet(m_regSetUris[0]);
  assert(regSet);

  const ise::RegionFeature* region1 = regSet->getRegionFeatures(reg1Id);
  const ise::RegionFeature* region2 = regSet->getRegionFeatures(reg2Id);
  assert(region1);
  assert(region2);

  size_t size    = regSet->getFeatureDim();
  size_t tabSize = (size - 1)/2;
  // Since we know we must be working on RegionColorimetricFeatures, hence 3 channels, we can raise the following assert
  assert(tabSize == 3);

  //  ise::RegionSet *regSet = m_segContext->getRegionSet(m_conversion);
  //  assert(regSet);
  //
  //  ise::RegionColorimetricFeatures *region1 = regSet->getRegionFeatures(reg1Id);
  //  assert(region1 != NULL);
  //
  //  ise::RegionColorimetricFeatures *region2 = regSet->getRegionFeatures(reg2Id);
  //  assert(region2 != NULL);
  //
  //  assert(region1->getSize() == region2->getSize() );
  //
  //  size_t size = region1->getSize();

  //compute homogeneity of union of two regions

  double *mergedMoments = new double[size];
  for (size_t i = 0; i < size; ++i) {
    mergedMoments[i] = (*region1)[i] + (*region2)[i];
  }
  const double variance = iseStat_iseComputeVariance(mergedMoments[0],
      &(mergedMoments[1]),
      &(mergedMoments[tabSize + 1]),
      tabSize);


  //  double m0u;
  //  double *m1u = new double[size];
  //  memset(m1u, 0, size*sizeof(double));
  //  double *m2u = new double[size];
  //  memset(m2u, 0, size*sizeof(double));
  //
  //  m0u = region1->getArea() + region2->getArea();
  //  iseDouble_set(m1u, region1->getMoment1(), size);
  //  iseDouble_addition(m1u, region2->getMoment1(), size);
  //  iseDouble_set(m2u, region1->getMoment2(), size);
  //  iseDouble_addition(m2u, region2->getMoment2(), size);
  //
  //  const double variance = iseStat_iseComputeVariance(m0u, m1u, m2u, region1->getSize());

  if (variance <= std::numeric_limits<double>::epsilon()) { //variance almost zero => union is considered homogeneous.
    homogenity = 1.0;
  }
  else {
    assert(variance <= m_colorHomogeneityMaxDistance*m_colorHomogeneityMaxDistance);
    const double homogenity = 1 - sqrt(variance) / m_colorHomogeneityMaxDistance;
  }
  assert(homogenity>=0.0 && homogenity<=1.0);

  //  delete[] m1u;
  //  delete[] m2u;
  delete [] mergedMoments;
  return homogenity;
}


double
MergeDriverColorHomogeneity::value(girl::region regId) const
{
  //  ise::RegionSet *regSet = m_segContext->getRegionSet(m_conversion);
  //  assert(regSet);
  assert(m_regSetUris.size() == 1);
  const RegionSetInterface *regSet = m_segContext->getRegionSet(m_regSetUris[0]);
  assert(regSet);
  size_t tabSize = (regSet->getFeatureDim() - 1)/2;
  //  ise::RegionColorimetricFeatures *regionF = regSet->getRegionFeatures(regId);
  //  assert(regionF != NULL);
  const ise::RegionFeature* regionF = regSet->getRegionFeatures(regId);
  assert(regionF);

  // REMI: Adapt code with new interface, but generalizing to N dimensions is not trivial
  // due to the normalization factor which is only known for colorspaces.
  // REMI: Do not use the weightChannelsCoeffs but assume the imageFeature was already computed
  // using the channel weights.
  //  assert(false); //B
  //  //B:TODO: This code must be adapted to N dimensions !!!!!!
  //
  //  double r_m0 = regionF->getArea();
  //  const double *r_m1 = regionF->getMoment1();
  //  const double *r_m2 = regionF->getMoment2();
  //
  //  double m0 = r_m0;
  //  girl::tools::Flt3 m1, m2;
  //  for (size_t i=0; i<3; ++i) { //B:B:TODO: change 3 to N !?
  //    m1[i] = r_m1[i] * m_weightedChannelsCoeffs[i];
  //    m2[i] = r_m2[i] * m_weightedChannelsCoeffs[i]*m_weightedChannelsCoeffs[i];
  //  }

  //const double variance = iseStat_iseComputeVariance(regionF->getArea(), regionF->getMoment1(), regionF->getMoment2());
  // const double variance = iseStat_iseComputeVariance(m0, m1, m2);

  const double *data = regionF->getData();
  const double variance = iseStat_iseComputeVariance(data[0],
      &(data[1]),
      &(data[tabSize + 1]),
      tabSize);


  if (variance < std::numeric_limits<double>::epsilon())
    return 1.0;

  assert(variance <= m_colorHomogeneityMaxDistance*m_colorHomogeneityMaxDistance);

  const double homogenity = 1 - sqrt(variance) / m_colorHomogeneityMaxDistance;
  assert(homogenity>=0.0 && homogenity<=1.0);
  return homogenity;
}
