/*
  Merge driver with threshold on colors homogeneity.
  Copyright (C) 2005-2009 Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISEMERGEDRIVERCOLORHOMOGENEITY_HPP
#define ISEMERGEDRIVERCOLORHOMOGENEITY_HPP


#include <girl/girlCommon.hpp>
#include "iseMergeDriverDefault.hpp"

namespace ise {

  class SegmentationContext;
  
  class GIRL_API MergeDriverColorHomogeneity : public MergeDriverDefault
  {
  public :
    
    MergeDriverColorHomogeneity(ise::SegmentationContext *segContext = NULL, double threshold = 0.5);
    
    virtual ~MergeDriverColorHomogeneity() {}

    
    virtual void setParameters(double threshold,
                               const std::string &regSetUri
			       /*int colorSpaceConversion,
			       const std::vector<double> &channelsCoeffs*/);

    virtual void getParameters(double &threshold,
			       std::string &regSetUri
			       /*int &colorSpaceConversion,
			       std::vector<double> &channelsCoeffs*/);

    
    virtual void init(const girl::regionDeque &regionsToMerge, int level);
    
    //virtual bool less(girl::region r1, girl::region r2);

    //virtual void filter(girl::regionDeque &regionsToMerge, int level);
    
    virtual double distance(girl::region reg1Id, girl::region reg2Id);
    
    virtual bool allowed(girl::region reg1Id, girl::region reg2Id, int level);
        
    //virtual girl::region selectCandidate(girl::region regId, int level);
    
    //virtual girl::region selectCandidate(girl::region regId, const girl::regionDeque &regions, int level);
    
    //virtual girl::region execute(girl::region reg1Id, girl::region reg2Id);
    
    
    double value(girl::region regId) const;


  public:
    
    class MergeDriverColorHomogeneityMemento : public MergeDriver::MergeDriverMemento
    {
      friend class MergeDriverColorHomogeneity;
      
    public:
      MergeDriverColorHomogeneityMemento() : MergeDriverMemento(),
      m_threshold(0.5),
      m_colorHomogeneityMaxDistance(0) {}

      virtual ~MergeDriverColorHomogeneityMemento() {}

    protected:
      virtual void getParameters(const ise::Mementable &m)
      {
	const MergeDriverColorHomogeneity &md = reinterpret_cast<const MergeDriverColorHomogeneity &>(m);
	MergeDriverMemento::getParameters(md);
	m_threshold = md.m_threshold;
//	m_conversion = md.m_conversion;
//	m_weightedChannelsCoeffs = md.m_weightedChannelsCoeffs;
	m_colorHomogeneityMaxDistance = md.m_colorHomogeneityMaxDistance;
      }

      virtual void setParameters(ise::Mementable &m) const
      {
	MergeDriverColorHomogeneity &md = reinterpret_cast<MergeDriverColorHomogeneity &>(m);
	MergeDriverMemento::setParameters(md);
	md.m_threshold = m_threshold;
//	md.m_conversion = m_conversion;
//	md.m_weightedChannelsCoeffs = m_weightedChannelsCoeffs;
	md.m_colorHomogeneityMaxDistance = m_colorHomogeneityMaxDistance;
      }

    protected:
      double m_threshold;
//      int m_conversion;
//      std::vector<double> m_weightedChannelsCoeffs;
      
      double m_colorHomogeneityMaxDistance;
      
    };

  protected:

    virtual MergeDriverMemento *createMemento() const
    {
      return new MergeDriverColorHomogeneityMemento;
    }


  protected:
    double m_threshold;
//    int m_conversion;
//    std::vector<double> m_weightedChannelsCoeffs;

    double m_colorHomogeneityMaxDistance;

  };
  
  
}

#endif /* ! ISEMERGEDRIVERCOLORHOMOGENEITY_HPP */
