/*
  Merge driver using threshold on variance of tangent of regions contour.
  Copyright (C) 2005-2009 Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "iseMergeDriverTangentVariance.hpp"

#include <cmath>
#include <algorithm>
#include <limits>

#include <girl/girlCommon.hpp>

#include "iseRegionSet.hpp"
#include "iseSegmentationContext.hpp"
#include "iseRegionColorimetricFeatures.hpp"
#include "iseRegionTools.hpp"
#include "iseStat.hpp"

#include "iseVector.hpp"


using namespace ise;



MergeDriverTangentVariance::MergeDriverTangentVariance(ise::SegmentationContext *segContext,
						       double threshold)
  : MergeDriverDefault(segContext),
    m_threshold(threshold)
{

}


void 
MergeDriverTangentVariance::setParameters(double threshold)
{
  m_threshold  = threshold; 
} 

void 
MergeDriverTangentVariance::getParameters(double &threshold)
{
  threshold  = m_threshold; 
} 

void 
MergeDriverTangentVariance::init(const girl::regionDeque & /* regionsToMerge */, int /* level */)
{


}


bool
MergeDriverTangentVariance::allowed(girl::region reg1Id, girl::region reg2Id, int /* level */)
{
  return distance(reg1Id, reg2Id) > m_threshold; // * M_PI;
}


double
MergeDriverTangentVariance::distance(girl::region reg1Id, girl::region reg2Id)
{
  const girl::toger &tog = *(m_segContext->getToger());

  const double variance = ise::RegionTools_computeTangentVarianceOnCommonBoundary(tog, reg1Id, reg2Id);

  double d = 0;
  if (variance >= std::numeric_limits<double>::epsilon())
    d = sqrt(variance) / M_PI;

  assert(d>=0);
  assert(d<=1.0);

  return d;
}

double
MergeDriverTangentVariance::value(girl::region regId) const
{
  const girl::toger &tog = *(m_segContext->getToger());

  const double variance = ise::RegionTools_computeTangentVarianceOnOuterBoundary(tog, regId);
  
  if (variance < std::numeric_limits<double>::epsilon())
    return 0;
  
  assert(variance > 0);
  
  return sqrt(variance) / M_PI;
}
