/*
  Merge driver using 2 thresholds  on mean color.
  Copyright (C) 2005-2009 Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "iseMergeDriverThresholds.hpp"

#include <cmath>
#include <limits>
#include "iseRegionSet.hpp"
#include "iseSegmentationContext.hpp"
#include "iseRegionColorimetricFeatures.hpp"
#include "iseStat.hpp"

using namespace ise;
using girl::tools::Flt;

MergeDriverThresholds::MergeDriverThresholds(ise::SegmentationContext *segContext)
  : MergeDriverDefault(segContext),
    m_threshold1(0),
    m_threshold2(0)
{

}

MergeDriverThresholds::MergeDriverThresholds(ise::SegmentationContext *segContext,
					     int th1, int th2)
  : MergeDriverDefault(segContext),
    m_threshold1(th1),
    m_threshold2(th2)
{

}

void
MergeDriverThresholds::setThresholds(int th1, int th2)
{
  m_threshold1 = th1;
  m_threshold2 = th2;
}
void
MergeDriverThresholds::getThresholds(int &th1, int &th2)
{
  th1 = m_threshold1;
  th2 = m_threshold2;
}



double
MergeDriverThresholds::distance(girl::region reg1Id, girl::region reg2Id)
{
  // ise::RegionSet *regSet = m_segContext->getRegionSet();
  assert(m_regSetUris.size() == 1);
  const RegionSetInterface *regSet = m_segContext->getRegionSet(m_regSetUris[0]);

//  ise::RegionColorimetricFeatures *region1, *region2;
//
//  region1 = regSet->getRegionFeatures(reg1Id);
//  assert(region1 != NULL);
//
//  region2 = regSet->getRegionFeatures(reg2Id);
//  assert(region2 != NULL);
  const ise::RegionFeature* region1 = regSet->getRegionFeatures(reg1Id);
  const ise::RegionFeature* region2 = regSet->getRegionFeatures(reg2Id);
  assert(region1);
  assert(region2);

  girl::toger *tog = m_segContext->getToger();
//  if ( tog->areNeighbours(reg1Id, reg2Id, girl::toger::NEIGHBOUR_INNER, m_neighbourhoodMark) //B??? GIRL_NEIGHBOUR_INNER or m_neighbourhoodMode
//       && ( (region1->getArea() < region2->getArea()*0.0001)  //B: what is this constant ?
//	    || region1->getArea() *0.0001 > region2->getArea() ) )
//    return 0;

  if ( tog->areNeighbours(reg1Id, reg2Id, girl::toger::NEIGHBOUR_INNER, m_neighbourhoodMark) //B??? GIRL_NEIGHBOUR_INNER or m_neighbourhoodMode
       && ( ((*region1)[0] < (*region2)[0]*0.0001)  //B: what is this constant ?
            || (*region1)[0] *0.0001 > (*region2)[0] ) )
    return 0;

//  assert(region1->getSize() == region2->getSize());
//
//  size_t size = region1->getSize();
  size_t size = (regSet->getFeatureDim() - 1)/2;

  double *r1Mean = new double[size];
  double *r2Mean = new double[size];
//  region1->getMean(r1Mean);
//  region2->getMean(r2Mean);
  for (size_t i = 0; i < size; ++i) {
     r1Mean[i] = (*region1)[i + 1]/(*region1)[0];
     r2Mean[i] = (*region2)[i + 1]/(*region2)[0];
   }

  const int t1 = getLevel(static_cast<int>(r1Mean[0]));
  const int t2 = getLevel(static_cast<int>(r2Mean[0]));

  const Flt td1 = static_cast<Flt>(t1);
  const Flt td2 = static_cast<Flt>(t2);
  const Flt r1Mean0 = r1Mean[0];
  const Flt r2Mean0 = r2Mean[0];

  delete[] r1Mean;
  delete[] r2Mean;

  if (t1 == 1 && t2 != 1) {
    const Flt d1 = r1Mean0 - td1;
    const Flt d2 = td2 - r1Mean0;

    if (t2 == 0)
      if (d1 < d2)
	return d1;
    if (t2 == 2)
      if (d1 > d2)
	return d2;

    return std::numeric_limits<double>::max();
  }
  if (t1 != 1 && t2 == 1) {
    const Flt d1 = r2Mean0 - td1;
    const Flt d2 = td2 - r2Mean0;

    if (t1 == 0)
      if (d1 < d2)
	return d1;
    if (t1 == 2)
      if (d1 > d2)
	return d2;

    
    return std::numeric_limits<double>::max();
  }

  if (t1 == t2)
    return 0;
  
  return std::numeric_limits<double>::max();
}

int
MergeDriverThresholds::getLevel(int v)
{
  //B: ??? test >=0 necessary ?

  if (v>=0 && v<=m_threshold1)
    return 0;
  if (m_threshold1 < v && v < m_threshold2)
    return 1;
  return 2;
}
