/*
  Merge driver using 2 thresholds  on mean color.
  Copyright (C) 2005-2009 Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISEMERGEDRIVERTHRESHOLDS_HPP
#define ISEMERGEDRIVERTHRESHOLDS_HPP


#include <girl/girlCommon.hpp>
#include "iseMergeDriverDefault.hpp"

namespace ise {

  class SegmentationContext;
  
  class GIRL_API MergeDriverThresholds : public MergeDriverDefault
  {
  public :
    explicit MergeDriverThresholds(ise::SegmentationContext *segContext = NULL);

    MergeDriverThresholds(ise::SegmentationContext *segContext, int th0, int th1);
  
    void setThresholds(int th1, int th2);
    void getThresholds(int &th1, int &th2);

    virtual double distance(girl::region reg1Id, girl::region reg2Id);

  protected:
    int getLevel(int v);


  public:

    class MergeDriverThresholdsMemento : public MergeDriver::MergeDriverMemento
    {
      friend class MergeDriverThresholds;
      
    public:

      MergeDriverThresholdsMemento() : MergeDriverMemento(),
      m_threshold1(0),
      m_threshold2(0) {}

      virtual ~MergeDriverThresholdsMemento() {}

    protected:
      virtual void getParameters(const Mementable &m)
      {
	const MergeDriverThresholds &md = reinterpret_cast<const MergeDriverThresholds &>(m);
	MergeDriverMemento::getParameters(md);
	m_threshold1 = md.m_threshold1;
	m_threshold2 = md.m_threshold2;
      }

      virtual void setParameters(Mementable &m) const
      {
	MergeDriverThresholds &md = reinterpret_cast<MergeDriverThresholds &>(m);
	MergeDriverMemento::setParameters(md);
	md.m_threshold1 = m_threshold1;
	md.m_threshold2 = m_threshold2;
      }

    protected:
      int m_threshold1;
      int m_threshold2;
    };

  protected:

    virtual MergeDriverMemento *createMemento() const
    {
      return new MergeDriverThresholdsMemento;
    }


  protected:
    int m_threshold1;
    int m_threshold2;
    
  };

}


#endif /* ! ISEMERGEDRIVERTHRESHOLDS_HPP */
