#ifndef NODE_HPP
#define NODE_HPP

#include <cstdlib>
#include <vector>

#include "Types.hpp"

class Node
{
  
private:

  typedef std::vector<std::size_t> NeigboursContainer;
  typedef std::vector<data_type> WeightsContainer;

public:

  typedef NeigboursContainer::const_iterator NeighboursConstIterator;
  typedef WeightsContainer::const_iterator WeightsConstIterator;
  
  /**
   * Node does not take ownership of data
   * Client must deallocate the array
   */
  explicit Node(data_type *data = NULL);
  ~Node();

  void setData(data_type *data);

   data_type&
  operator[](std::size_t i);

  NeighboursConstIterator
  neighboursBegin() const;

  NeighboursConstIterator
  neighboursEnd() const;

  WeightsConstIterator
  weightsBegin() const;

  WeightsConstIterator
  weightsEnd() const;

  void
  addNeighbour(std::size_t index, data_type weight);
  
private:
  
  data_type *m_data;
 
  NeigboursContainer m_neighbours;
  WeightsContainer   m_weights;
  
};



#endif /* NODE_HPP */
