#ifndef RELAXATION_LABELING_HPP
#define RELAXATION_LABELING_HPP

#include <cstdlib>
#include <vector>
//#include "Types.hpp"
#include "AffinityFunctor.hpp"

#include <Eigen/Dense>

class Node;

template <typename DataType = double>
class RelaxationLabeling
{

  typedef Eigen::Matrix<DataType, Eigen::Dynamic, Eigen::Dynamic, Eigen::RowMajor> CoeffMatrixType;


public:

  RelaxationLabeling(size_t nbLabels,
		     AffinityFunctor<DataType> &affinityComputer,
		     size_t nbNodes);

  ~RelaxationLabeling();

  // void
  // addNode(DataType *probs);

  void
  setNodeProbs(size_t i, DataType *probs);

  void
  addEdge(size_t nodeIndex1, size_t nodeIndex2, DataType weight);

  // void
  // setAffinityCoeffs(const std::vector<DataType> &coeffs);


  void
  doRelax();

private:

  void
  updateProbabilities(size_t nodeIndex, bool affinityUpdate = false);

  void
  updateAffinityCoeffs(size_t nodeIndex1, size_t nodeIndex2);

  bool
  isStable(size_t nodeIndex);

  void
  applyUpdate();

private:

  size_t m_dim;
  AffinityFunctor<DataType> &m_affinityComputer;
  CoeffMatrixType m_coeffs;
  CoeffMatrixType m_initProbs;
  CoeffMatrixType m_updatedProbs;
  

  typedef std::vector<Node> NodeContainer;
  NodeContainer m_nodes;


 #ifdef WITH_TBB
   friend struct TbbRelaxationIteration;
#endif




};

#include "RelaxationLabeling.cpp"


#endif /* RELAXATION_LABELING_HPP */
