/*
  Copyright (C) 2005-2012 Aurore Godiche, Remi Vieux, Jean-Philippe Domenger, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "dscc.hpp"

#define INFINI 50000

#define T1 45

#define T2 15

#define UP 0
#define DOWN 1
#define LEFT 2
#define RIGHT 3

#ifdef WITH_TBB
struct ConnectionRunLength
{
  ConnectionRunLength(const DequeRunLength &runLengths, 
		      const ise::RunLengthExtractor::Orientation oriRL,
		      //std::deque<int> &
		      std::vector<int>* singleConnectionRL1,
		      //std::deque<int> 
		      std::vector<int>* singleConnectionRL2)
    : m_runLengths(runLengths),
      m_oriRL(oriRL),
      m_singleConnectionRL1(singleConnectionRL1),
      m_singleConnectionRL2(singleConnectionRL2)
  {}
  
  void
  operator()(const tbb::blocked_range<int> &r) const
  {
    if (m_oriRL == ise::RunLengthExtractor::VERTICAL) {
      ise::BitmapRunLength::RunLength rl;
      for (int i = r.begin(); i < r.end(); ++i) {
	rl = m_runLengths.at(i);
	int cmptR = 0;
	int cmptL = 0;
	int rl_x = rl.m_start.x();
	    
	for (int j = i + 1; j < m_runLengths.size() && (m_runLengths.at(j).m_start.x() < (rl_x + 2)); ++j) {
	  ise::BitmapRunLength::RunLength rltmp = m_runLengths.at(j);
	  int rltmp_x = rltmp.m_start.x();
	  if (rl_x + 1 == rltmp_x) {
	    if (isNeighboor(rl, rltmp, m_oriRL)) {
	      ++cmptR;
	      m_singleConnectionRL1->at(j)++;
	    }
	  }
		
	  if (rltmp_x == rl_x - 1) {
	    if (isNeighboor(rltmp, rl, m_oriRL)) {
	      ++cmptL;
	    }
	  }
	}
	m_singleConnectionRL1->at(i) += cmptL;
	m_singleConnectionRL2->at(i) += cmptR;
      }
    }
    else
      {
	ise::BitmapRunLength::RunLength rl;
	for (int i = r.begin(); i < r.end(); ++i) {
	  rl = m_runLengths.at(i);
	  int cmptR = 0;
	  int cmptL = 0;
	  int rl_y = rl.m_start.y();
	  
	  for (int j = i + 1; j < m_runLengths.size() && (m_runLengths.at(j).m_start.y() < (rl_y + 2)); ++j) {
	    ise::BitmapRunLength::RunLength rltmp = m_runLengths.at(j);
	    int rltmp_y = rltmp.m_start.y();
	    if (rl_y + 1 == rltmp_y)
	      if (isNeighboor(rl, rltmp, m_oriRL)) {
		++cmptR;
		m_singleConnectionRL1->at(j)++;
	      }
	      
	    if (rltmp_y == rl_y - 1)
	      if (isNeighboor(rltmp, rl, m_oriRL)) {
		++cmptL;
	      }
	  }

	  m_singleConnectionRL1->at(i) += cmptL;
	  m_singleConnectionRL2->at(i) += cmptR;
	}
      }
    
  }
    
  const DequeRunLength m_runLengths;
  const ise::RunLengthExtractor::Orientation m_oriRL;
  std::vector<int>* m_singleConnectionRL1;
  std::vector<int> *m_singleConnectionRL2;
  
};

#endif

DSCC::DSCC(int minLineSizeHoriz, int minLineSizeVerti)
  : m_indice(0),
    m_dscc_c(),
    m_orientation(HORIZONTAL),
    m_minLineSizeHoriz(minLineSizeHoriz),
    m_minLineSizeVerti(minLineSizeVerti)
{

}

DSCC::DSCC(const DSCC &c)
  : m_indice(c.m_indice),
    m_dscc_c(c.m_dscc_c),
    m_orientation(c.m_orientation),
    m_minLineSizeHoriz(c.m_minLineSizeHoriz),
    m_minLineSizeVerti(c.m_minLineSizeVerti)
{
}


DSCC::DSCC(int indice, const DSCC_c &c, Orientation ori)
  : m_indice(indice),
    m_dscc_c(c),
    m_orientation(ori),
    m_minLineSizeHoriz(0),
    m_minLineSizeVerti(0)
{// ????
}

DSCC::~DSCC()
{
}


const DSCC_c &DSCC::getDSCC_c(const DSCC &c) const
{
  return c.m_dscc_c;
}

int DSCC::getIndice(const DSCC &c) const
{
  return c.m_indice;
}

Orientation DSCC::getOrientation(const DSCC &c) const
{
  return c.m_orientation;
}

void DSCC::setIndice(int indice)
{
  m_indice = indice;
}

void DSCC::setDSCC_c(const DSCC_c &dscc_c)
{
  m_dscc_c = dscc_c;
}

void DSCC::setOrientation(Orientation ori)
{
  m_orientation = ori;
}


void DSCC::printDSCCList(std::list<DSCC> dscc_list) const
{
  std::list<DSCC> dscc_list_tmp = dscc_list;
  for (size_t i = 0; i < dscc_list.size(); ++i) {
    const DSCC &ch = dscc_list_tmp.front();
    dscc_list_tmp.pop_front();
    const int indice = getIndice(ch);
    DSCC_c dscc_ch = getDSCC_c(ch);
    const size_t size_dscc = dscc_ch.size();
    for (size_t j = 0; j < size_dscc; ++j) {
      ise::BitmapRunLength::RunLength r = dscc_ch.front();
      dscc_ch.pop_front();
      std::cout << "indice : " << indice << " , ";
      std::cout << "x : " << r.m_start.x() << " , ";
      std::cout << "y : " << r.m_start.y() << " , ";
      std::cout << "size : " << r.m_size << std::endl;
    }
  }
  std::cout << "nombre d'éléments dans la liste : " << dscc_list.size() << std::endl;
}


ise::BitmapRunLength::RunLength DSCC::getRunLength(const DSCC_c &dscc_c, int indice) const
{
  return dscc_c.at(indice);
}

ise::BitmapRunLength::RunLength DSCC::getFirstRunLength(const DSCC_c &dscc_c) const
{
  return dscc_c.front();
}

ise::BitmapRunLength::RunLength DSCC::getLastRunLength(const DSCC_c &dscc_c) const
{
  return dscc_c.back();
}


void DSCC::init(cv::Mat binary, std::list<DSCC> &dscc_list_h, std::list<DSCC> &dscc_list_v)
{
  ise::Histogram1D histo;

  // Horizontal DSCC
  dscc_list_h = findDSCC(HORIZONTAL, histo, binary);
  dscc_list_h = DSCCMergenceRecursive(dscc_list_h);
  dscc_list_v = findDSCC(VERTICAL, histo, binary);
  dscc_list_v = DSCCMergenceRecursive(dscc_list_v);


}

DequeRunLength DSCC::findRunLength(VectorRunLength &runLength, 
				   std::vector<int> &labels, 
				   cv::Mat binary, 
				   ise::RunLengthExtractor::Orientation oriRL)
{
  cv::Mat binaryInt;
  binary.convertTo(binaryInt, CV_32S);
  ise::BitmapRunLength brl(binaryInt, oriRL);
  brl.getRunLength(runLength, labels);

  DequeRunLength runLengths;
  for (size_t i = 0; i < runLength.size(); ++i) {
    if (labels[i] == 255) {
      runLengths.push_back(runLength[i]);
    }
  }
  return runLengths;
}


void DSCC::havingConnectedRunLength(const DequeRunLength &runLengths, 
				    const ise::RunLengthExtractor::Orientation &oriRL,
				    std::deque<int> &singleConnectionRL1,
				    std::deque<int> &singleConnectionRL2)
{
#ifdef WITH_TBB
  std::vector<int> nbConnectionRLL(runLengths.size(), 0);
  std::vector<int> nbConnectionRLR(runLengths.size(), 0);
  ConnectionRunLength connection(runLengths, oriRL, &nbConnectionRLL, &nbConnectionRLR);
  //  ConnectionRunLength connection(runLengths, oriRL, &singleConnectionRL1[0], &singleConnectionRL2[0]);
  tbb::parallel_for(tbb::blocked_range<int>(0, runLengths.size()), connection);

  std::cout << "using tbb" << std::endl;
#else
  if (oriRL == ise::RunLengthExtractor::VERTICAL) {
    ise::BitmapRunLength::RunLength r;
    for (size_t i = 0; i < runLengths.size(); ++i) {
      r = runLengths[i];
      int cmptR = 0;
      int cmptL = 0;
      int r_x = r.m_start.x();
	  
      for (size_t j = i + 1; j < runLengths.size() && (runLengths[j].m_start.x() < (r_x + 2)); ++j) {
	ise::BitmapRunLength::RunLength rtmp = runLengths[j];
	int rtmp_x = rtmp.m_start.x();
	if (r_x + 1 == rtmp_x) {
	  if (isNeighboor(r, rtmp, oriRL)) {
	    ++cmptR;
	    singleConnectionRL1.at(j)++;
	  }
	}
	      
	if (rtmp_x == r_x - 1) {
	  if (isNeighboor(rtmp, r, oriRL)) {
	    ++cmptL;
	  }
	}
      }
      singleConnectionRL1.at(i) += cmptL;
      singleConnectionRL2.at(i) += cmptR;
    }
  }

  else {
    ise::BitmapRunLength::RunLength r;
    for (size_t i = 0; i < runLengths.size(); ++i) {
      r = runLengths[i];
      int cmptR = 0;
      int cmptL = 0;
      int r_y = r.m_start.y();
	  
      for (size_t j = i + 1; j < runLengths.size() && (runLengths[j].m_start.y() < (r_y + 2)); ++j) {
	ise::BitmapRunLength::RunLength rtmp = runLengths[j];
	int rtmp_y = rtmp.m_start.y();
	if (r_y + 1 == rtmp_y)
	  if (isNeighboor(r, rtmp, oriRL)) {
	    ++cmptR;
	    singleConnectionRL1.at(j)++;
	  }
	      
	if (rtmp_y == r_y - 1)
	  if (isNeighboor(rtmp, r, oriRL)) {
	    ++cmptL;
	  }
      }
      singleConnectionRL1.at(i) += cmptL;
      singleConnectionRL2.at(i) += cmptR;
    }
  }
#endif

}

std::list<DSCC>
DSCC::findDSCC(Orientation oriDSCC, ise::Histogram1D &histo, cv::Mat binary)
{
  ise::RunLengthExtractor::Orientation oriRL = ise::RunLengthExtractor::VERTICAL;
  switch(oriDSCC)
    {
    case HORIZONTAL:
      {
	oriRL = ise::RunLengthExtractor::VERTICAL;
	break;
      }
    case VERTICAL:
      {
	oriRL = ise::RunLengthExtractor::HORIZONTAL;
	break;
      }

    default:
      assert(false);
      break;
    }

  VectorRunLength runLength;
  std::vector<int> labels;
  DequeRunLength runLengths = findRunLength(runLength, labels, binary, oriRL);

  std::deque<int> singleConnectedRL1(runLengths.size(), 0);
  std::deque<int> singleConnectedRL2(runLengths.size(), 0);

  havingConnectedRunLength(runLengths, oriRL, singleConnectedRL1, singleConnectedRL2);


  std::list<DSCC> dscc_list;
 
  //int size = runLengths.size();
  int indice_dscc = 0;

  
  /// dscc computation
  if (oriRL == ise::RunLengthExtractor::VERTICAL) {
    while (! runLengths.empty()) {
      DSCC_c dscc_ci;
      ise::BitmapRunLength::RunLength r1 = runLengths[0];
      int sc_r1 = singleConnectedRL2[0];

      dscc_ci.push_back(r1);      
      bool search = true;
      size_t j = 1;
      int r1_x = r1.m_start.x();
      while (sc_r1 < 2 && search && runLengths.size() > j) {
	ise::BitmapRunLength::RunLength r2 = runLengths[j];
	int r2_x = r2.m_start.x();
	      
	if (r1_x + 1 == r2_x) {
	  if (isNeighboor(r1, r2, oriRL) && singleConnectedRL1[j] < 2) {
	    dscc_ci.push_back(r2);
	    r1 = r2;
	    sc_r1 = singleConnectedRL2[j];
	    r1_x = r2_x;
	    runLengths.erase(runLengths.begin() + j);
	    singleConnectedRL1.erase(singleConnectedRL1.begin() + j);
	    singleConnectedRL2.erase(singleConnectedRL2.begin() + j);
	    --j;
	  }
	}
	      
	else if (r2_x > (r1_x + 1))
	  search = false;
	      
	++j;
      }
      if((int)dscc_ci.size() > m_minLineSizeHoriz) {
	DSCC ci(indice_dscc, dscc_ci, oriDSCC);
	dscc_list.push_back(ci);
	++indice_dscc;
      }
      runLengths.erase(runLengths.begin());
      singleConnectedRL1.erase(singleConnectedRL1.begin());
      singleConnectedRL2.erase(singleConnectedRL2.begin());
    }
  }

  else {
    while (! runLengths.empty()) {
      DSCC_c dscc_ci;
      ise::BitmapRunLength::RunLength r1 = runLengths[0];
      int sc_r1 = singleConnectedRL2[0];

      dscc_ci.push_back(r1);
      bool search = true;
      size_t j = 1;
      int r1_y = r1.m_start.y();
      while (sc_r1 < 2 && search && runLengths.size() > j) {
	ise::BitmapRunLength::RunLength r2 = runLengths[j];
	int r2_y = r2.m_start.y();

	if (r1_y + 1 == r2_y) {
	  if (isNeighboor(r1, r2, oriRL) && singleConnectedRL1[j] < 2) {
	    dscc_ci.push_back(r2);
	    r1 = r2;
	    sc_r1 = singleConnectedRL2[j];
	    r1_y = r2_y;
	    runLengths.erase(runLengths.begin() + j);
	    singleConnectedRL1.erase(singleConnectedRL1.begin() + j);
	    singleConnectedRL2.erase(singleConnectedRL2.begin() + j);
	    --j;
	  }
	}
	      
	else if (r2_y > (r1_y + 1))
	  search = false;
	      
	++j;
      }
      
      if ((int)dscc_ci.size() > m_minLineSizeVerti) {
	DSCC ci(indice_dscc, dscc_ci, oriDSCC);
	dscc_list.push_back(ci);
	++indice_dscc;
      }
      runLengths.erase(runLengths.begin());
      singleConnectedRL1.erase(singleConnectedRL1.begin());
      singleConnectedRL2.erase(singleConnectedRL2.begin());
    }
  }
  ///
  
  return dscc_list;
}


bool isNeighboor(ise::BitmapRunLength::RunLength r1, 
		 ise::BitmapRunLength::RunLength r2, 
		 ise::RunLengthExtractor::Orientation oriRL)
{
  if (oriRL == ise::RunLengthExtractor::VERTICAL) {
    int r1_x = r1.m_start.x();
    int r1_y = r1.m_start.y();
    int size_r1 = r1.m_size;

    int r2_x = r2.m_start.x();
    int r2_y = r2.m_start.y();
    int size_r2 = r2.m_size;
      
    if (r1_x + 1 != r2_x)
      return false;
      
    for (int i = r1_y; i < (r1_y + size_r1); ++i) {
      for (int j = r2_y; j < (r2_y + size_r2); ++j) {
	if (i == j)
	  return true;
      }
    }
  }
  else {
    int r1_x = r1.m_start.x();
    int r1_y = r1.m_start.y();
    int size_r1 = r1.m_size;

    int r2_x = r2.m_start.x();
    int r2_y = r2.m_start.y();
    int size_r2 = r2.m_size;

    if (r1_y + 1 != r2_y)
      return false;

    for (int i = r1_x; i < (r1_x + size_r1); ++i) {
      for (int j = r2_x; j < (r2_x + size_r2); ++j) {
	if (i == j)
	  return true;
      }
    }
  }
  return false;
}


int DSCC::computeDistanceX(const DSCC &c1, const DSCC &c2)
{
  const DSCC_c &dscc_c1 = getDSCC_c(c1);
  const DSCC_c &dscc_c2 = getDSCC_c(c2);

  int max, min;
  int x1, x2, xn, xm;
  x1 = dscc_c1.at(0).m_start.x();
  x2 = dscc_c2.at(0).m_start.x();
  xn = dscc_c1.back().m_start.x();
  xm = dscc_c2.back().m_start.x();
  max = x1 > x2 ? x1 : x2;
  min = xn < xm ? xn : xm;
  
  return (max - min);
}


int DSCC::computeDistanceY(const DSCC &c1, const DSCC &c2)
{
  const DSCC_c &dscc_c1 = getDSCC_c(c1);
  const DSCC_c &dscc_c2 = getDSCC_c(c2);

  int max, min;
  int y1, y2, yn, ym;
  y1 = dscc_c1.at(0).m_start.y();
  y2 = dscc_c2.at(0).m_start.y();
  yn = dscc_c1.back().m_start.y();
  ym = dscc_c2.back().m_start.y();
  max = y1 > y2 ? y1 : y2;
  min = yn < ym ? yn : ym;

  return (max - min);
}


double DSCC::computeDistanceC1C2(const DSCC &c1, const DSCC &c2)
{ 
  int m = computeNumberOfNormalRunLength(c2);
  ParamEquationDroite p = computeEquationLine(c2);
  const DSCC_c &dscc_c2 = getDSCC_c(c2);

  double sum = 0;
  if (m_orientation == VERTICAL)
    {
      int distance_y = computeDistanceY(c1, c2);
      if (distance_y <= 0)
	return INFINI;

      int x_c2;
      int nb_run_lengths = dscc_c2.size();
      for (int i = 0; i < nb_run_lengths; ++i) {
	int size = dscc_c2.at(i).m_size;
	x_c2 = findPointLine(p, i);
	sum += (size / 2 - x_c2)*(size / 2 - x_c2);
      }
      return (distance_y + sum / m);
    }

  else {
    int distance_x = computeDistanceX(c1, c2);
    if (distance_x <= 0)
      return INFINI;

    int y_c2;
    int nb_run_lengths = dscc_c2.size();
    for (int i = 0; i < nb_run_lengths; ++i) {
      int size = dscc_c2.at(i).m_size;
      y_c2 = findPointLine(p, i);
      sum += (size / 2  - y_c2)*(size / 2  - y_c2);
    }
    return (distance_x + sum / m);
  }
}


double DSCC::computeAverageWidth(const DSCC &c)
{
  const DSCC c_tmp = c;
  double average = 0;
  const DSCC_c &dscc_c_tmp = getDSCC_c(c_tmp);
  double size = dscc_c_tmp.size();
  for (size_t i = 0; i < size; ++i) {
    const DSCC_c &dscc_c = getDSCC_c(c);
    average += dscc_c.at(i).m_size;
  }
  return (average / size);
}


double DSCC::computeAverageHeight(const DSCC &c)
{
  const DSCC c_tmp = c;
  double average = 0;
  const DSCC_c &dscc_c_tmp = getDSCC_c(c_tmp);
  double size = dscc_c_tmp.size();
  for (size_t i = 0; i < size; ++i) {
    const DSCC_c &dscc_c = getDSCC_c(c);
    average += dscc_c.at(i).m_size;
  }
  return (average / size);
}


int DSCC::computeNumberOfNormalRunLength(const DSCC &c)
{
  DSCC c_tmp = c;
  const DSCC_c &dscc_c_tmp = getDSCC_c(c_tmp);
  double length;
  int m = 0;

  if (m_orientation == VERTICAL) {
    int h = computeAverageHeight(c);
    for (size_t i = 0; i < dscc_c_tmp.size(); ++i) {
      length = dscc_c_tmp.at(i).m_size;
      if (length < (2 * h))
	++m;
    }
  }
  else {
    int w = computeAverageWidth(c);
    for (size_t i = 0; i < dscc_c_tmp.size(); ++i) {
      length = dscc_c_tmp.at(i).m_size;
      if (length < (2 * w))
	++m;
    }
  }
  return m;
}


ParamEquationDroite DSCC::computeEquationLine(const DSCC &ch)
{
  const DSCC_c &dscc_c_ch = getDSCC_c(ch);
  int v1_x = dscc_c_ch.at(0).m_start.x();
  int v2_x = dscc_c_ch.back().m_start.x();
  int v1_y = dscc_c_ch.at(0).m_start.y() + dscc_c_ch.at(0).m_size / 2;
  int v2_y = dscc_c_ch.back().m_start.y() + dscc_c_ch.back().m_size / 2;
  int v_x = v2_x - v1_x;
  int v_y = v2_y - v1_y;

  // Equation de la droite delta du type -> ax + by + c = 0 :
  double a =  v_y;
  double b = -v_x;
  
  // Constante -> c = - (ax + by) ; x et y étant les composantes
  // d'un des deux point de la droite delta :
  double c = -((a * v1_x) + (b * v1_y));    
  
  ParamEquationDroite p;
  p.a = a;
  p.b = b;
  p.c = c;

  return p;
}


int DSCC::findPointLine(ParamEquationDroite p, int val)
{
  if (m_orientation == VERTICAL) {
    if (p.a != 0)
      return (int) (-(p.b * val + p.c) / p.a);
  }
  else {
    if (p.b != 0)
      return (int) (-(p.a * val + p.c) / p.b);
  }
  return 0; //B:Added to silence a warning. Right value???
}


bool DSCC::DSCCMergenceConditions(DSCC &c_seed, DSCC &c_k, double distance_cseed_ck)
{
  if (m_orientation == VERTICAL) {
    // 1. Linear extending condition
    const double h1 = computeAverageHeight(c_seed);
    const int distance_y = computeDistanceY(c_seed, c_k);
    if (sqrt(distance_cseed_ck - distance_y) >= h1)
      return false; 

    // 2. Gap condition:
    // (a) Empty
    if (distance_y == T1)
      return true;
      
    // (b) Exist another chain, and its width is less than two times of the average width of chain C
    const double h2 = computeAverageHeight(c_k);
    if (h2 < 2 * h1)
      return true;
      
    // (c) Exist another chain, and its width is larger than two times of the average width of chain C
    if (h2 > 2 * h1)
      if(distance_y == T2)
	return true;

    return false;
  }
  else {
    // 1. Linear extending condition
    const double w1 = computeAverageWidth(c_seed);
    const int distance_x = computeDistanceX(c_seed, c_k);
    if (sqrt(distance_cseed_ck - distance_x) >= w1)
      return false; 

    // 2. Gap condition:
    // (a) Empty
    if (distance_x == T1)
      return true;
      
    // (b) Exist another chain, and its width is less than two times of the average width of chain C
    const double w2 = computeAverageWidth(c_k);
    if (w2 < 2 * w1)
      return true;
      
    // (c) Exist another chain, and its width is larger than two times of the average width of chain C
    if (w2 > 2 * w1)
      if(distance_x == T2)
	return true;

    return false;
  }
  return false;
}


std::list<DSCC> DSCC::DSCCMergence(std::list<DSCC> &dscc_list)
{  
  PairListBool list_chains_not_processed;
  list_chains_not_processed = initPairListChainsNotProcessed(dscc_list);
  
  int nb_iter = dscc_list.size();

  int nb_chains_remaining_processed = dscc_list.size();
  std::cout << "nb chains : " << nb_chains_remaining_processed << std::endl;
  int size = dscc_list.size();
  int size_tmp = dscc_list.size();
  int nb_tmp = 0;

  do {
    size = size_tmp;
    // step 1
    // The longest chain not processed yet is selected as a seed chain Cs.
    DSCC c_seed = getSeed(list_chains_not_processed);
    int indice_seed = getIndice(c_seed);

    // step 2
    // On one side of Cs, sort chains Ci (i = 1, 2, ..., n) with the co-line distance ascending	       
    short side1=LEFT, side2=RIGHT; //initialization for m_orientation==HORIZONTAL
    if (m_orientation == VERTICAL) {
      side1 = UP;
      side2 = DOWN;
    }

    PairListDouble pair_list_chain;
    if (dscc_list.size() > 1)
      pair_list_chain = sortChain(c_seed, side1, dscc_list);

    // step 3
    // Among the first M (M = 3) chains, if we can find chain C'k with the smallest co-line distance
    // and fill the two merging conditions listed above, then merge Cs and C'k.
    if (pair_list_chain.size() > 1) {
      PairListDouble::iterator iter = pair_list_chain.begin();
      DSCC c_k = (*iter).second;
      int indice_k = getIndice(c_k);
      double distance_c_seed_ck = (*iter).first;

      if (indice_seed != indice_k)
	{
	  if (distance_c_seed_ck < INFINI)
	    {
	      if (DSCCMergenceConditions(c_seed, c_k, distance_c_seed_ck))
		{
		  DSCCMerge(c_seed, c_k);
		  removeChain(c_k, dscc_list, list_chains_not_processed);
		  --nb_iter;
		}
	    }
	}
    }

    // step 4
    // Repeat the step 2 and 3 on two sides of Cs.
    if (dscc_list.size() > 1)
      pair_list_chain = sortChain(c_seed, side2, dscc_list);

    if (pair_list_chain.size() > 1) {
      PairListDouble::iterator iter = pair_list_chain.begin();
      DSCC c_k = (*iter).second;
      int indice_k = getIndice(c_k);
      double distance_c_seed_ck = (*iter).first;

      if (indice_seed != indice_k) {
	if (distance_c_seed_ck != INFINI) {
	  if (DSCCMergenceConditions(c_seed, c_k, distance_c_seed_ck)) {
	    DSCCMerge(c_seed, c_k);
	    removeChain(c_k, dscc_list, list_chains_not_processed);		      
	    --nb_iter;
	  }
	}
      }
    }

    // step 5
    // If all chains are processed then end mergence, else go to step 1.
    chainProcessed(c_seed, list_chains_not_processed, dscc_list);
    nb_chains_remaining_processed -= 3;
    --nb_iter;

    size_tmp = dscc_list.size();
    if (size_tmp == size)
      ++nb_tmp;
    else
      nb_tmp = 0;

  } while ((nb_iter > 1 && nb_iter < size && nb_tmp < 2));

  PairListBool::iterator iter = list_chains_not_processed.begin();
  int size2 =  list_chains_not_processed.size();

  return dscc_list;
}


std::list<DSCC> DSCC::DSCCMergenceRecursive(std::list<DSCC> &dscc_list)
{
  int size_current = dscc_list.size();
  int size_tmp = dscc_list.size();;

  do {
    size_current = size_tmp;
    dscc_list = DSCCMergence(dscc_list);
    size_tmp = dscc_list.size();
  } while (size_current != size_tmp && size_tmp > 1);

  return dscc_list;
}


void DSCC::DSCCMerge(DSCC &c_seed, DSCC &c_k)
{
  DSCC_c dscc_cseed = getDSCC_c(c_seed);
  DSCC_c dscc_ck = getDSCC_c(c_k);
  const size_t size_dscc = dscc_ck.size();

  if (m_orientation == VERTICAL) {
    if (dscc_cseed.back().m_start.y() < dscc_ck.at(0).m_start.y()) {
      for (size_t i = 0; i < size_dscc; ++i) {
	ise::BitmapRunLength::RunLength r_tmp = getRunLength(dscc_ck, i);
	dscc_cseed.push_back(r_tmp);
      }
    }
    else {
      for (size_t i = (size_dscc - 1); i < size_dscc; --i) {
	ise::BitmapRunLength::RunLength r_tmp = getRunLength(dscc_ck, i);
	dscc_cseed.push_front(r_tmp);
      }
    }
  }
  else {
    if (dscc_cseed.back().m_start.x() < dscc_ck.at(0).m_start.x()) {
      for (size_t i = 0; i < size_dscc; ++i) {
	ise::BitmapRunLength::RunLength r_tmp = getRunLength(dscc_ck, i);
	dscc_cseed.push_back(r_tmp);
      }
    }
    else {
      for (size_t i = (size_dscc - 1); i < size_dscc; --i) {
	ise::BitmapRunLength::RunLength r_tmp = getRunLength(dscc_ck, i);
	dscc_cseed.push_front(r_tmp);
      }
    }
  }
  c_seed.setDSCC_c(dscc_cseed);
}


DSCC DSCC::getSeed(std::list< std::pair<bool, DSCC> > list_chains_not_processed)
{ 
  DSCC c_seed(0, 0);
  PairListBool::iterator iter = list_chains_not_processed.begin();
  
  if (iter != list_chains_not_processed.end()) { //B

    while (iter !=  list_chains_not_processed.end() && (*iter).first) {
      ++iter;
    }
  
    c_seed = (*iter).second;
    DSCC_c dscc_c_seed = getDSCC_c(c_seed);
    size_t size_seed = dscc_c_seed.size();
    ++iter;
    int indice_seed = getIndice(c_seed);
  
    while (iter !=  list_chains_not_processed.end()) {
      if(!(*iter).first) {
	DSCC c_tmp = (*iter).second;
	DSCC_c dscc_c_tmp = getDSCC_c(c_tmp);
	size_t size_tmp = dscc_c_tmp.size();

	if (size_tmp > size_seed) {
	  size_seed = size_tmp;
	  c_seed = c_tmp;
	  dscc_c_seed = getDSCC_c(c_seed);
	  indice_seed = getIndice(c_seed);
	}
	else if (size_tmp == 1 && size_seed == 1) {
	  ise::BitmapRunLength::RunLength r_tmp = dscc_c_tmp.front();
	  ise::BitmapRunLength::RunLength r_seed = dscc_c_seed.front();
	  if (r_tmp.m_size > r_seed.m_size) {
	    size_seed = size_tmp;
	    c_seed = c_tmp;
	    dscc_c_seed = getDSCC_c(c_seed);
	    indice_seed = getIndice(c_seed);
	  }
	}
      }
      ++iter;
    }

  }

  return c_seed;
}


PairListDouble DSCC::sortChain(const DSCC &c_seed, short side, const std::list<DSCC> &dscc_list)
{
  std::list <DSCC> list_dscc_tmp = dscc_list;

  // pair list contenant une DSCC et sa co-line distance par rapport à c_seed.
  PairListDouble list_pair_chain; 
  PairListDouble::iterator iter = list_pair_chain.begin();

  int indice_seed = getIndice(c_seed);

  double co_line_distance;
  bool notSort = true;

  if (side == LEFT || side == UP) {
    assert(! list_dscc_tmp.empty());
    DSCC tmp_dscc = list_dscc_tmp.front();
    list_dscc_tmp.pop_front();
    int indice_tmp = getIndice(tmp_dscc);

    co_line_distance = computeDistanceC1C2(c_seed, tmp_dscc);
    list_pair_chain.push_back(make_pair_double(co_line_distance, tmp_dscc));
    tmp_dscc = list_dscc_tmp.front();
    list_dscc_tmp.pop_front();
    indice_tmp = getIndice(tmp_dscc);

    int nb = 0;
    while (indice_tmp < indice_seed) {
      iter = list_pair_chain.end();
      co_line_distance = computeDistanceC1C2(c_seed, tmp_dscc);
      while (notSort) {
	if (co_line_distance > (*iter).first && iter != list_pair_chain.begin()) {
	  --iter;
	}
	else {
	  list_pair_chain.insert(iter, make_pair_double(co_line_distance, tmp_dscc));
	  notSort = false;
	}
      }
      notSort = true;
      tmp_dscc = list_dscc_tmp.front();
      list_dscc_tmp.pop_front();
      indice_tmp = getIndice(tmp_dscc);
      ++nb;
    }
  }
  else {
    assert(! list_dscc_tmp.empty());
    DSCC tmp_dscc = list_dscc_tmp.back();
    list_dscc_tmp.pop_back();
    int indice_tmp = getIndice(tmp_dscc);

    co_line_distance = computeDistanceC1C2(c_seed, tmp_dscc);
    list_pair_chain.push_back(make_pair_double(co_line_distance, tmp_dscc));
    assert(! list_dscc_tmp.empty());
    tmp_dscc = list_dscc_tmp.back();
    list_dscc_tmp.pop_back();
    indice_tmp = getIndice(tmp_dscc);

    int nb = 0;
    while (indice_tmp > indice_seed && ! list_dscc_tmp.empty()) {
      iter = list_pair_chain.begin();

      if (iter != list_pair_chain.end()) { //B

	co_line_distance = computeDistanceC1C2(c_seed, tmp_dscc);

	while (notSort) {
	  if (iter != list_pair_chain.end() && co_line_distance > (*iter).first) {
	    ++iter;
	  }
	  else
	    {
	      list_pair_chain.insert(iter, make_pair_double(co_line_distance, tmp_dscc));
	      notSort = false;
	    }
	}
	notSort = true;
	tmp_dscc = list_dscc_tmp.back();
	list_dscc_tmp.pop_back();
	indice_tmp = getIndice(tmp_dscc);
	++nb;

      }
    }
  }

  return list_pair_chain;
}


PairListBool DSCC::initPairListChainsNotProcessed(const std::list<DSCC> &dscc_list)
{
  PairListBool list_chains_not_processed;
  int size = dscc_list.size();
  std::list <DSCC> dscc_list_tmp = dscc_list;

  DSCC tmp_dscc = dscc_list_tmp.front();
  dscc_list_tmp.pop_front();
  bool processed = false;
  list_chains_not_processed.push_back(make_pair_bool(processed, tmp_dscc));

  for (int i = 1; i < size; ++i) {
    tmp_dscc = dscc_list_tmp.front();
    dscc_list_tmp.pop_front();
    list_chains_not_processed.push_back(make_pair_bool(processed, tmp_dscc));
  }

  return list_chains_not_processed;
}


void DSCC::removeChain(const DSCC &c, std::list<DSCC> &dscc_list, PairListBool &list_chains_not_processed)
{
  int indice_c = getIndice(c);
  size_t size = dscc_list.size();

  std::list<DSCC> dscc_list_tmp;
  PairListBool list_chains_tmp;

  for (size_t j = 0; j < size; ++j) {
    DSCC c_tmp = dscc_list.front();
    int indice_tmp = getIndice(c_tmp);
    std::pair<bool, DSCC> pair_tmp = list_chains_not_processed.front();
    dscc_list.pop_front();
    list_chains_not_processed.pop_front();
    if (indice_tmp != indice_c) {
      dscc_list_tmp.push_back(c_tmp);
      list_chains_tmp.push_back(pair_tmp);
    }
  }
  dscc_list = dscc_list_tmp;
  list_chains_not_processed = list_chains_tmp;
}


void DSCC::chainProcessed(DSCC &c_seed, PairListBool &list_chains_not_processed, std::list<DSCC> &dscc_list)
{
  int indice_seed = getIndice(c_seed);
  PairListBool list_chains_tmp;
  std::list<DSCC> dscc_list_tmp;

  size_t size = list_chains_not_processed.size();
  for (size_t j = 0; j < size; ++j) {
    DSCC c_tmp = dscc_list.front();
    int indice_tmp = getIndice(c_tmp);
    std::pair<bool, DSCC> pair_tmp = list_chains_not_processed.front();
    dscc_list.pop_front();
    list_chains_not_processed.pop_front();
    if (indice_tmp != indice_seed) {
      dscc_list_tmp.push_back(c_tmp);
      list_chains_tmp.push_back(pair_tmp);
    }
    else {
      dscc_list_tmp.push_back(c_seed);
      list_chains_tmp.push_back(make_pair_bool(true, c_seed));
    }
  }
  dscc_list = dscc_list_tmp;
  list_chains_not_processed = list_chains_tmp;
}

void DSCC::histoCCRL(const DequeRunLength &runLengths, ise::Histogram1D &histo)
{
  DequeRunLength runLengthsTmp = runLengths;
 
  int max = std::numeric_limits<int>::min();
  int min = std::numeric_limits<int>::max();

  while (! runLengthsTmp.empty()) {
    ise::BitmapRunLength::RunLength r = runLengthsTmp.back();
    int size = r.m_size;
    if (size > max)
      max = size;
    if (size < min)
      min = size;
    runLengthsTmp.pop_back();
  }
  if (max > 100)
    max = 100;

  histo.reset(min, max);
  runLengthsTmp = runLengths;
  while (! runLengthsTmp.empty()) {
    ise::BitmapRunLength::RunLength r = runLengthsTmp.back();
    int size = r.m_size;
      
    if (size <= 100)
      histo[histo.keyToIndex(size)]++;
    runLengthsTmp.pop_back();
  }
}


void DSCC::subHistoCCRL(const DequeRunLength &runLengths, ise::Histogram1D &histo, int init)
{
  DequeRunLength runLengthsTmp = runLengths;
 
  int max = std::numeric_limits<int>::min();
  int min = std::numeric_limits<int>::max();

  while (! runLengthsTmp.empty()) {
    ise::BitmapRunLength::RunLength r = runLengthsTmp.back();
    int size = r.m_size;
    if (size > max)
      max = size;
    runLengthsTmp.pop_back();
  }
  if (max > 100)
    max = 100;
  min = init;

  histo.reset(min, max);
  runLengthsTmp = runLengths;
  while (! runLengthsTmp.empty()) {
    ise::BitmapRunLength::RunLength r = runLengthsTmp.back();
    int size = r.m_size;
    if (size >= init && size <= 100)
      histo[histo.keyToIndex(size)]++;
    runLengthsTmp.pop_back();
  }
}


