/*
  3D Histogram base class.
  Copyright (C) 2004-2009 Jean-Philippe Domenger, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISEABSTRACTHISTOGRAM_HPP
#define ISEABSTRACTHISTOGRAM_HPP

#include <deque>

#include "iseHistogram.hpp"

namespace ise {

  class AbstractHistogramIterator;

  class GIRL_API AbstractHistogram : public virtual Histogram 
  {

  public:

    AbstractHistogram(unsigned char c1Offset, 
		      unsigned char c2Offset, 
		      unsigned char c3Offset)
      : m_colorVect()
    {
      m_offsets[0] = c1Offset; 
      m_offsets[1] = c2Offset; 
      m_offsets[2] = c3Offset;     
    }

    inline unsigned char getOffset(int i) const { return m_offsets[i]; }

    inline int getSize() const { return m_colorVect.size(); }

    inline void clear() { m_colorVect.clear(); }

  protected:

    inline void setOffsets(unsigned char c1Offset, unsigned char c2Offset, unsigned char c3Offset) 
    { 
      m_offsets[0] = c1Offset; 
      m_offsets[1] = c2Offset; 
      m_offsets[2] = c3Offset; 
    };

  protected:

    friend class AbstractHistogramIterator;

    typedef std::deque<size_t> ColorCollection;
    ColorCollection m_colorVect;
    unsigned char m_offsets[3]; 

  };

}


#endif /* ! ISEABSTRACTHISTOGRAM_HPP */
