/*
  3D Histogram base class iterator.
  Copyright (C) 2004-2009 Jean-Philippe Domenger, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISEABSTRACTHISTOGRAMITERATOR_HPP
#define ISEABSTRACTHISTOGRAMITERATOR_HPP

//#include <stddef.h>
#include <cstddef>
#include <cassert>

#include <girl/girlCommon.hpp>
#include "iseHistogramIterator.hpp"
#include "iseAbstractHistogram.hpp"
#include "iseHistoInfo.hpp"

namespace ise {

  class GIRL_API AbstractHistogramIterator : public virtual HistogramIterator
  {
    
  public:
    
    explicit AbstractHistogramIterator(AbstractHistogram *h = NULL)
      : m_histo(h), m_it(h->m_colorVect.begin())
    {}

    AbstractHistogramIterator(const AbstractHistogramIterator &o)
      : HistogramIterator(o), m_histo(o.m_histo), m_it(o.m_it)
    {
    }

    AbstractHistogramIterator &operator=(const AbstractHistogramIterator &o)
    {
      m_histo  = o.m_histo;
      m_it = o.m_it;
      return *this;
    }

    virtual void get(HistoInfo *info);
    
    inline void begin() { assert(m_histo); m_it = m_histo->m_colorVect.begin(); }
    
    inline void next() { assert(! atEnd()); ++m_it; }
    
    inline bool atEnd() { return (m_it == m_histo->m_colorVect.end()); }
    
  protected:
    
    AbstractHistogram *m_histo;
    AbstractHistogram::ColorCollection::iterator m_it;
    
  };
  
}

#endif /* ! ISEABSTRACTHISTOGRAMITERATOR_HPP */
