/*
  Copyright (C) 2010-2011 Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
#include "iseAreaEnclosingShapeFeatureVisitor.hpp"

#include "EnclosingBox.hpp"
#include "EnclosingConvexHull.hpp"
#include "EnclosingCircle.hpp"
#include "EnclosingEllipse.hpp"
#include "EnclosingRotatedRect.hpp"

#include <cassert>
#include <cmath>
#include <limits>//DEBUG

/// \todo remove ios
#include <iostream>

#ifndef M_PI
//Visual C++ does not define these constants in <cmath>
#define M_PI 3.14159265358979323846
#endif 

using namespace ise;

AreaEnclosingShapeFeatureVisitor::AreaEnclosingShapeFeatureVisitor()
{
}

double
AreaEnclosingShapeFeatureVisitor::visit(EnclosingBox *box)
{
  assert(box);
  const double area = box->getRect().area();
#ifndef _MSC_VER
  assert(std::isfinite(area));
#endif
  return area;
}

double
AreaEnclosingShapeFeatureVisitor::visit(EnclosingConvexHull *box)
{
  assert(box);

  std::vector<cv::Point> points = box->getPoints();

  if (points.empty()) {
    assert(false);
    throw "not enough points to compute area";
  }

#if 0

  if (points.size() == 1) {
    //reduced to a single point/pixel
    return 1.0;
  }
  if (points.size() == 2) {
    //reduced to a line
    //we do not necessarily have :  (points[0].x==points[1].x || points[0].y==points[1].y);
    return ??? 
  }

#else
  
  if (points.size() < 3) {
    return 0;
  }

#endif

  // ConvexHull to triangles
  cv::Point p0 = points[0];
  cv::Point p1;
  cv::Point p2;

  double area = 0.0;
  size_t i = 2;
  while (i < points.size()) {

    p1 = points[i-1];
    p2 = points[i];

    //const double det = p0.x*(p2.y - p1.y) + p1.x*(p0.y - p2.y) + p2.x*(p1.y - p0.y); //3mul, 5add
    const double det = (p1.x - p0.x)*(p2.y - p0.y) - (p2.x - p0.x)*(p1.y - p0.y);//2mul, 5add
#ifndef _MSC_VER
    assert(std::isfinite(det));
#endif
    area += 0.5 * fabs(det);

    ++i;
  }
#ifndef _MSC_VER
  assert(std::isfinite(area));
#endif
  assert(area > 0);  

  return area;
}

double ise::AreaEnclosingShapeFeatureVisitor::visit(EnclosingCircle *box)
{
    return M_PI*box->getRadius()*box->getRadius();
}

double ise::AreaEnclosingShapeFeatureVisitor::visit(EnclosingEllipse *box)
{
    double a = box->getRotatedRect().size.height/2;
    double b = box->getRotatedRect().size.width/2;

    return M_PI*a*b;
}

double ise::AreaEnclosingShapeFeatureVisitor::visit(EnclosingRotatedRect *box)
{
    return box->getRect().size.area();
}
