/*
  3D Histogram stored as array.
  Copyright (C) 2004-2009 Jean-Philippe Domenger, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "iseArrayHistogram.hpp"
#include "iseArrayHistogramIterator.hpp"

#include <cstring> //memset

using namespace ise;

ArrayHistogram::ArrayHistogram(unsigned char c1Offset, unsigned char c2Offset, unsigned char c3Offset)
  : AbstractHistogram(c1Offset, c2Offset, c3Offset), m_histoData(NULL)
{
  const size_t size = dataSize();
  m_histoData = new unsigned int[size];
  clear();
}

ArrayHistogram::~ArrayHistogram()
{
  delete [] m_histoData;    
}

void 
ArrayHistogram::addColor(unsigned char r, 
			 unsigned char g,
			 unsigned char b)
{
  unsigned int c0 = r >> m_offsets[0];
  unsigned int c1 = g >> m_offsets[1];
  unsigned int c2 = b >> m_offsets[2];

  const size_t index = c0 + c1 * (256 >> m_offsets[0]) + c2 * (256 >> m_offsets[0]) * (256 >> m_offsets[1]);

  if (m_histoData[index] == 0)
    m_colorVect.push_back(index);
  
  m_histoData[index]++;
}

void
ArrayHistogram::clear()
{
  AbstractHistogram::clear();
  const size_t size = dataSize();
  memset(m_histoData, 0, size*sizeof(unsigned int)); 
}

HistogramIterator * 
ArrayHistogram::getIterator()
{
  return new ArrayHistogramIterator(this);
}
