/*
  Copyright (C) 2005-2011 Remi Vieux, Boris Mansencal  remi.vieux@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "iseBinarizerCannyOpenCv.hpp"

#include <iostream>

#include <girlTools/girlToolsImage.hpp>

#include <opencv2/core/core.hpp>
#include <opencv2/imgproc/imgproc.hpp>
#include <opencv2/highgui/highgui.hpp>

namespace ise
{

  const char* g_wndName = "Canny Threshold Setup";

  int g_thresh1 = 50;
  int g_thresh2 = 450;
  cv::Mat g_gray;


  void
  changeThresh(int, void* data)
  {
    cv::Mat *output = static_cast<cv::Mat*>(data);
    cv::Canny(g_gray, *output, g_thresh1, g_thresh2, 3, true);
    cv::imshow(g_wndName, *output);
  }

  BinarizerCannyOpenCv::BinarizerCannyOpenCv()
  {
  }

  BinarizerCannyOpenCv::~BinarizerCannyOpenCv()
  {
  }

  void
  BinarizerCannyOpenCv::binarize(const girl::tools::Image *input, cv::Mat &output)
  {
    assert(input);
    const cv::Mat* orig = input->cvMat();
    if (orig->channels() == 4)
      cv::cvtColor(*orig, g_gray, cv::COLOR_BGRA2GRAY);
    else
      cv::cvtColor(*orig, g_gray, cv::COLOR_RGB2GRAY);
    cv::convertScaleAbs(g_gray, g_gray);

    cv::namedWindow(g_wndName, 0);
    cv::createTrackbar("Canny threshold 1", g_wndName, &g_thresh1, 2000, changeThresh, &output);
    cv::createTrackbar("Canny threshold 2", g_wndName, &g_thresh2, 2000, changeThresh,  &output);
    changeThresh(0, &output);
    std::cout << "Choose Canny Threshold, press any key when finished" << std::endl;
    cv::waitKey(0);
    std::cout << "Chosen Canny Threshold min:" << std::min(g_thresh1, g_thresh2) << ", max:" << std::max(g_thresh1, g_thresh2) << std::endl;
#if CV_MAJOR_VERSION*100+CV_MINOR_VERSION*10+CV_SUBMINOR_VERSION >= 220
    cv::destroyWindow(g_wndName);
#endif
  }
}
