/*
  Copyright (C) 2005-2011 Remi Vieux, Boris Mansencal  remi.vieux@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#include "iseBinarizerQuantificationThresholder.hpp"

#include "iseQuantificationThresholder.hpp"
#include "iseHistogram1D.hpp"
#include "iseImageFeatureColorSpace.hpp"

static
int
P_nearestInt(double d)
{
  if (d >= 0)
    return static_cast<int>(d+0.5);
  return static_cast<int>(d-0.5);
}

namespace ise
{

  BinarizerQuantificationThresholder::BinarizerQuantificationThresholder(girl::shared_ptr<QuantificationThresholder> thresholder)
  : m_thresholder(thresholder),
    m_name()
  {

  }

  void
  BinarizerQuantificationThresholder::binarize(const girl::tools::Image *input, cv::Mat &output)
  {
    assert(input);
    output = cv::Mat::zeros(input->height(), input->width(), CV_8U);
    ise::Histogram1D histo(*input, ISE_RGB_to_GRAY, 0);
    std::vector<double> thresholds;
    m_thresholder->getThresholds(histo, thresholds);
    // TODO:optim compute conversion several times (+ new image allocated)
    const int width = input->width();
    const int height = input->height();
    const int threshold = P_nearestInt(thresholds[0]);
    
    ise::ImageFeatureColorSpace computer(NULL);
    computer.setParameters(ISE_RGB_to_GRAY);
    computer.compute(*input);
    girl::shared_ptr<girl::tools::ImageFeature> img = computer.getImageFeature();

    //ise::GenericImage<unsigned char, 1> img(*input, ISE_RGB_to_GRAY);
    for (int i = 0; i < height; ++i) {
      girl::tools::ImageFeature::type *buf = img->scanLine(i);
      for (int j = 0; j < width; ++j) {
	//ise::GenericImage<unsigned char, 1>::pixel_t pixel = img.getPixel(j, i);
	if (buf[0]/*pixel[0]*/ < threshold) {
	  output.at<unsigned char>(i, j) = 255;
	}
      }
    }
  }
}
