/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "iseBinarizerThresholder.hpp"

#include "iseHistogram1D.hpp"
#include "iseImageFeatureColorSpace.hpp"

namespace ise
{
  void
  BinarizerThresholder::binarize(const girl::tools::Image *input, cv::Mat &output)
  {
    assert(input);
    assert(m_levels.size());
    assert(m_channel < 3);
    const int width = input->width();
    const int height = input->height();
    const int threshold = m_levels[0];
    output = cv::Mat::zeros(height, width, CV_8U);
    //ise::Histogram1D histo(*input, m_conversion, m_channel);
    // TODO:optim compute conversion several times (+ new image allocated)
    //B:TODO:OPTIM: should be parallelized !

    ise::ImageFeatureColorSpace computer(NULL);
    computer.setParameters(m_conversion);
    computer.compute(*input);
    girl::shared_ptr<girl::tools::ImageFeature> img = computer.getImageFeature();
    
    //const ise::GenericImage<unsigned char, 3> img(*input, m_conversion);
    for (int i = 0; i < height; ++i) {
      girl::tools::ImageFeature::type *buf = img->scanLine(i);

      for (int j = 0; j < width; ++j) {
	//const ise::GenericImage<unsigned char, 3>::pixel_t pixel = img.getPixel(j, i);
	if (buf[m_channel] /*pixel[m_channel]*/ < threshold) {
	  output.at<unsigned char>(i, j) = 255;
	}
	buf += img->formatSize();
      }
    }
  }
}
