/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISEBINARIZERTHRESHOLDER_HPP_
#define ISEBINARIZERTHRESHOLDER_HPP_

#include <vector>
#include <girl/girlCommon.hpp>

#include "iseConversion.hpp"
#include "iseBinarizer.hpp"
/**
 * Wrapper for a binarizer which thresholds the image with the given threshold.
 * This class is very similar to iseBinarizerQuantificationThresholder, except
 * that instead of having the pointer to the thresholder directly, we just use
 * the threshold values. Hence, from within the interface, we can launch the
 * thresholders interfaces and set up the threshold manually, or at least have
 * a clue about what thresholds are the thresholders giving.
 */
namespace ise
{

  class GIRL_API BinarizerThresholder : public Binarizer
  {
  public:

    BinarizerThresholder()
    : m_levels(),
      m_conversion(ISE_RGB_to_GRAY),
      m_channel(0)
    {}

    virtual
    ~BinarizerThresholder() {}

    const std::string name() const
    {
      return "Launch Thresholders";
    }

    void
    binarize(const girl::tools::Image* input, cv::Mat &output);

    void
    setParameters(const std::vector<int> &levels, int conversion, int channel)
    {
      m_levels = levels;
      m_conversion = conversion;
      m_channel = channel;
    }

    void
    setLevels(const std::vector<int> &levels)
    {
      m_levels = levels;
    }

    void
    setConversion(int conversion)
    {
      m_conversion = conversion;
    }

    void
    setChannel(int channel)
    {
      m_channel = channel;
    }

  protected:

    std::vector<int> m_levels;
    int m_conversion;
    int m_channel;

  };

}

#endif /* ISEBINARIZERTHRESHOLDER_HPP_ */
