#ifndef ISEBITMAPRUNLENGTH_HPP
#define ISEBITMAPRUNLENGTH_HPP

#include <opencv2/core/core.hpp>

#include <girl/girlCommon.hpp>
#include <girlTools/girlToolsAABBox.hpp>

#include "iseRunLengthExtractor.hpp"
namespace ise
{
  class GIRL_API BitmapRunLength
  {
  public:
    
    BitmapRunLength() :
      m_orientation(ise::RunLengthExtractor::HORIZONTAL)
      {}
    
    BitmapRunLength(const cv::Mat &bitmap, ise::RunLengthExtractor::Orientation ori);
    
    /**
     * return the bitmap run length of the specified ROI
     */
    BitmapRunLength(const BitmapRunLength& b, const cv::Rect &rect);
  
    
    ~BitmapRunLength()
      {}
    
    
    BitmapRunLength&
      operator=(const BitmapRunLength& b);
    
    
    void
      setBitmap(const cv::Mat &bitmap)
    {
      reset();
      m_bitmap = bitmap;
      compute();
    }
    
    
    //  BitmapRunLength&
    //  operator()(const girl::tools::AABBox) const;
    
    
    
    typedef std::vector<std::deque<size_t> > RunLengthType;
    typedef std::vector<std::deque<int>    > RunLengthLabelsType;
    
    void
      getRunLength(RunLengthType &runLength, RunLengthLabelsType &labels) const;
    
    typedef struct RunLength {
      girl::point m_start;
      size_t m_size;
    } RunLength;
    
    void
      getRunLength(std::vector<RunLength> &runLength, std::vector<int> &labels) const;
    
    
    /**
     * Fill @boxes with the bounding box of the connected components, indexed by
     * their labels. WARNING: for non used labels, the vector is filled with "empty"
     * boxes
     */
    void
      getConnectedComponentsAABBox(std::vector<girl::tools::AABBox> &boxes) const;
    
    void
      getConnectedComponentsAreas(std::vector<size_t> &areas) const;
    
    /**
     * Return the highest index of a connected components in the object.
     */
    int
      getMaxElement() const;
    
    struct SpacingInfo {
      int a; // label of region 1
      int b; // label of region 2
      size_t dist; // distance btw 1 and 2 in pixels
    };
    
    void
      getSpacings(const girl::unorderedSet<int> &backgroundLabels, std::vector<SpacingInfo> &spacings) const;
    

    /**
     * Fill spacing with the number of pixels separating two regions
     * If allowSelfSpacing is true, the number of pixels in a hole will be also considered
     */
    void
      getSpacings(std::vector<SpacingInfo> &spacings, bool allowSelfSpacing = false) const;
    
  protected:

    void
      reset()
    {
      ZBufferType::iterator it = m_buffer.begin();
      const ZBufferType::const_iterator itEnd = m_buffer.end();
      for (; it != itEnd; ++it) {
	it->clear();
      }
      m_buffer.clear();
    }
    
    void
      compute();
    
    void
      initBuffers(size_t i, const int **start, const int **end, size_t &shift) const;

    size_t
      getShift() const;
    
    size_t
      nbElem() const;
    
    
  protected:
    
    cv::Mat m_bitmap;
    
    typedef std::deque<const int*> IntPtrDeque;
    typedef std::vector<IntPtrDeque> ZBufferType;
    ZBufferType m_buffer;
    
    ise::RunLengthExtractor::Orientation m_orientation;
    
  };
  

}
#endif // ISEBITMAPRUNLENGTH_HPP
