/*
  Cluster.
  Copyright (C) 2004-2009 Jean-Philippe Domenger, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include <cassert>
#include <limits>

#include "iseStat.hpp"
#include "iseCluster.hpp"

using namespace ise;


Cluster::Cluster() :
  mean(0.0, 0.0, 0.0), 
  m2(0.0, 0.0, 0.0),
  m1(0.0, 0.0, 0.0),
  m0(0.0),
  sqe(0.0),
  canonic(0),
  exist(false)
{

}

void Cluster::addColor(girl::tools::Int3 color, long frequency)
{
  girl::tools::Flt3 c(static_cast<double>(color[0]), 
		  static_cast<double>(color[1]), 
		  static_cast<double>(color[2]));

  const double freq = static_cast<double>(frequency);
  m0 += freq;
  m1 += c * freq;
  m2 += girl::tools::Flt3(c[0]*c[0], c[1]*c[1], c[2]*c[2]) * freq;
  
  const double inv_m0 = 1 / m0;
  mean = m1 * inv_m0;

  sqe = iseStat_iseComputeSQE(m0, m1, m2);
}

void Cluster::getMoments(double *M0, girl::tools::Flt3 *M1, girl::tools::Flt3 *M2) const 
{
  *M0 = this->m0;
  *M1 = this->m1;
  *M2 = this->m2;
}

void Cluster::setMoments(double M0, girl::tools::Flt3 M1, girl::tools::Flt3 M2)
{
  assert(M0 != 0.0);

  exist = true;

  this->m0 = M0;
  this->m1 = M1;
  this->m2 = M2;

  sqe = iseStat_iseComputeSQE(m0, m1, m2);
  mean = m1 * (1.0 / m0);
}

void Cluster::move(Cluster *copy)
{
  copy->exist = true;

  copy->m0 = m0;
  copy->m1 = m1;
  copy->m2 = m2;

  if (m0 > std::numeric_limits<double>::epsilon())
    copy->mean = m1 * (1.0 / m0);
  else
    copy->mean = girl::tools::Flt3(0.0, 0.0, 0.0);
  
  copy->sqe = sqe;
}

