/*
  Cluster set.
  Copyright (C) 2004-2009 Jean-Philippe Domenger, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISECLUSTERSET_HPP
#define ISECLUSTERSET_HPP

#include <vector>

#include <girlTools/girlToolsImage.hpp>

#include "iseConversion.hpp"

namespace girl {
  namespace tools {
    class Flt3;
  }
}

namespace ise {

  class Cag;
  class Cluster;
  class Partition;

  class ClusterSet 
  {

  public:

    ClusterSet(ise::conversion::PFConversion *convert, 
	       const std::vector<double> &weigthedChannelsCoeffs, 
	       int nbCluster = 2048);
  
    ~ClusterSet();
  
    void reduction(double);
  
    void addColor(const girl::tools::Pixel &color, long frequency);
  
    short areEquivalentColor(const girl::tools::Pixel &RGBcolor1, 
			     const girl::tools::Pixel &RGBcolor2);
  
    double getDist(int, int) const;
  
    double getColorDist(int, int) const;
  
    void mergeCluster(int clus1, int clus2);
  
    double getSqe(int clus1) const;
  
    double getColorVariance() const;
  
    double getDistMean(int clus1, int clus2) const;
  
    int getCanonicCluster(int clus) const;
  
    int getInitialClusterNumber() const;
  
    ise::conversion::PFConversion *getConversion() const;

  protected:

    void getTable(girl::tools::Flt3 *means) const;
  
    void initPlaneCoeff();
  
    double merge(Cag *graph);

    void mergeEnd(Cag *graph, double threshold);
  
    void compress();

    void setCanonic();
  
    short sameClusterCriterionAndSupTwo(const girl::tools::Pixel &C1, 
					const girl::tools::Pixel &C2);

  private:

    ClusterSet(const ClusterSet &);

    ClusterSet &operator=(const ClusterSet &);


  protected:

    Partition *m_part;
    ise::conversion::PFConversion *m_convert;

    std::vector<double> m_weigthedChannelsCoeffs;

    int *m_equiv;
    Cluster **m_clusters;

    double m_planeCoeff[4];

    double m_sqeSum;
    double m_globalSqe;

    int m_nbClusterInitial;
    int m_nbCluster;
    int m_nbColor;

  };

}
#endif /* ISECLUSTERSET_HPP */
