/*
  Copyright (C) 2005-2009 Julian Roigt, Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef ISECLUSTERERCMEAN_HPP
#define ISECLUSTERERCMEAN_HPP

#include "iseClusterer.hpp"
#include "iseClustererFactoryTemplate.hxx"

namespace ise {
  
  class GIRL_API ClustererCMean : public Clusterer
  {
  public:

    static const double ERROR_DISS;

  public:

    explicit ClustererCMean(size_t numberClasses = 0);

    virtual ~ClustererCMean() {}

    virtual const std::string uri() const;

    virtual void compute(const cv::Mat &, cv::Mat &labels);

    virtual void compute(const std::vector<double*> &, size_t); //TODO:REMOVE 

    virtual ise::Clusterer::LabelType getCluster(const double*, size_t) const;

    void setParameters(double weightingExponent, double tolerence);

    void setParameters(double weightingExponent);


    const cv::Mat& getCentroid() const { return m_centroid; }

  protected:

    void buildClusterer(const cv::Mat &samples);

    void initData(int nbPoints);

    void computeCentroid(const cv::Mat &samples);

    void computeMemberShip(const cv::Mat &samples);

    double computeDissimilarity(const cv::Mat &samples) const;

    void getClusters(const cv::Mat &samples, cv::Mat &labels);

  private:

    double m_weightingExponent;
    double m_tolerence;

    //std::vector<const double*> m_sample;
    size_t m_dim;

    cv::Mat m_centroid;
    cv::Mat m_data;

    //cache
    cv::Mat m_sumD;
    
    mutable cv::Mat m_distances;

  };

  typedef ClustererFactoryTemplate<ClustererCMean> ClustererCMeanFactory;

}



#endif /* ! ISECLUSTERERCMEAN_HPP */
