/*
  Copyright (C) 2005-2011 Remi vieux, Jean-Philippe Domenger, Achille Braquelaire, Jenny Benois-Pineau, Julian Roigt  remi.vieux@labri.fr

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
/*
  This work was (partly) funded by the X-Media project, sponsored by
  the European Commission as part of the Information Society Technologies
  (IST) program under EC grant number IST-FP6-026978.
*/

#include "iseClustererEM.hpp"

#include <cstdlib>
#include <sstream>



using namespace ise;

ClustererEM::ClustererEM(unsigned int numberOfClusters)
  : ClustererExplicit(numberOfClusters)
{
  std::cerr<<"Constructor: numberOfClusters="<<numberOfClusters<<"\n";
}

const
std::string
ClustererEM::uri() const
{
  std::stringstream ss(std::ios::in|std::ios::out);
  ss << __FILE__ << "_" << numberClasses();
  std::string s = ss.str();
  return s;
}

void
ClustererEM::compute(const cv::Mat &instances, cv::Mat &output)
{
#if CV_MAJOR_VERSION*100+CV_MINOR_VERSION*10+CV_SUBMINOR_VERSION < 300
#if CV_MAJOR_VERSION*100+CV_MINOR_VERSION*10+CV_SUBMINOR_VERSION >= 240
  m_em_model = cv::EM(numberClasses());
  //m_em_model.train(instances, cv::Mat(), output);
  m_em_model.train(instances, cv::noArray(), output); 
  //With 2.4.6, here cv::noArray(), but not cv::Mat
#else 
  CvEMParams params(numberClasses());
  m_em_model.train(instances, cv::Mat(), params, &output);
#endif
#else
  m_em_model = cv::ml::EM::create();
  m_em_model->trainEM(instances, cv::noArray(), output); 
#endif
}

Clusterer::LabelType
ClustererEM::getCluster(const double *instance, size_t dim) const
{
  cv::Mat sample(1, dim, CV_64FC1, const_cast<double *>(instance));

#if CV_MAJOR_VERSION*100+CV_MINOR_VERSION*10+CV_SUBMINOR_VERSION < 300
  cv::Vec2d prediction = m_em_model.predict(sample);
#else
  cv::Vec2d prediction = m_em_model->predict(sample);
#endif

  return static_cast<ise::Clusterer::LabelType>(prediction[1]);
}
