/*
  Copyright (C) 2005-2011 Remi vieux, Jean-Philippe Domenger, Achille Braquelaire, Jenny Benois-Pineau, Julian Roigt  remi.vieux@labri.fr

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
/*
  This work was (partly) funded by the X-Media project, sponsored by
  the European Commission as part of the Information Society Technologies
  (IST) program under EC grant number IST-FP6-026978.
*/

#ifndef ISECLUSTEREREM_HPP
#define ISECLUSTEREREM_HPP

#include <opencv2/ml/ml.hpp>


#include "iseClustererExplicit.hpp"
#include "iseClustererFactoryTemplate.hxx"

namespace ise {

  class GIRL_API ClustererEM : public ClustererExplicit
  {
  public:
    explicit ClustererEM(unsigned int numberOfClusters = 0);

    virtual ~ClustererEM() {}

    const std::string uri() const;

    virtual void compute(const cv::Mat &instances, cv::Mat &output);

    virtual ise::Clusterer::LabelType getCluster(const double *instance, size_t dim) const;

  private:

#if CV_MAJOR_VERSION*100+CV_MINOR_VERSION*10+CV_SUBMINOR_VERSION < 300
#if CV_MAJOR_VERSION*100+CV_MINOR_VERSION*10+CV_SUBMINOR_VERSION >= 240
    cv::EM m_em_model;
#else
    CvEM m_em_model;
#endif
#else
    cv::Ptr<cv::ml::EM> m_em_model;
#endif 
  };

  struct EMFactory {
    typedef ClustererFactoryTemplate<ClustererEM > ClustererEMFactory;
  };

}

#endif /* ! ISECLUSTEREREM_HPP */
