/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
#include "iseClustererMeanShift.hpp"
#include <map>

#include <opencv2/highgui/highgui.hpp>//DEBUG
#include <unistd.h> //DEBUG


using namespace ise;

ClustererMeanShift::ClustererMeanShift()
  : Clusterer()
  , m_speedThreshold(0.1)
  , m_pointCount(0)
  , m_msRawData()
  , m_modes()
  , m_sigmaR(7)
  , m_weight()
  , m_dim()
  , m_nb(0)
  , m_sdata()
  , m_modeTable()
  , m_dataIndex()
  , m_indexCv(NULL)
  , m_Mh()
  , m_pointList()
  , m_ytemp()
  , m_indices()
  , m_dist()
  , m_query()
  , m_queryIndices(1, -1)
  , m_queryDist(1, 0)

{
  //B
  m_sMins[0] = m_sMins[1] = m_sMins[2] = 0;
  m_sMaxs[0] = m_sMaxs[1] = m_sMaxs[2] = 0;
  
}

const std::string
ClustererMeanShift::uri() const
{
  std::stringstream ss(std::ios::in|std::ios::out);
  ss << __FILE__ << m_sigmaR;
  std::string s = ss.str();
  return s;
}

void
ClustererMeanShift::compute(const cv::Mat &instances, cv::Mat &labels)
{

  assert(instances.channels() == 1);

  m_dim = instances.cols;

  m_nb = instances.rows;

  {
    std::cerr<<"sigmaR="<<m_sigmaR<<"\n";
    std::cerr<<"instances.rows="<<instances.rows<<" instances.cols="<<instances.cols<<"\n";
    // float inv_sigmaR = 1.0 / m_sigmaR;
    // for (int y = 0 ; y < std::min(20, instances.rows) ; ++y) {
    //   const float *buf = instances.ptr<float>(y);
    //   for (int x = 0 ; x < instances.cols ; ++x)
    // 	std::cerr<<buf[x]<<" ("<<buf[x] * inv_sigmaR <<"), ";
    //   std::cerr<<"\n";
    // }
  }

  //m_sdata = std::vector<double>(m_dim * m_nb, 0);
  m_sdata.resize(m_dim * m_nb);
  std::fill(m_sdata.begin(), m_sdata.end(), 0);
  
  if (m_weight.size() == 0) {
    //m_weight = std::vector<double>(m_nb, 1);
    m_weight.resize(m_nb);
    std::fill(m_weight.begin(), m_weight.end(), 1);
  }

  assert(m_weight.size() == static_cast<size_t>(m_nb));

  // copy data in m_sdata = instances / sigmaR
  int idxs = 0;
  float inv_sigmaR = 1.0 / m_sigmaR;
  //TODO:OPTIM: isContinuous()
  for (int y = 0 ; y < m_nb ; ++y) {
    const float *buf = instances.ptr<float>(y);
    for (int x = 0 ; x < m_dim ; ++x) {
      m_sdata[idxs++] = buf[x] * inv_sigmaR;
    }
  }

  // copy data from m_sdata to index flann in float
  m_dataIndex.create(m_nb, m_dim, CV_32F);

  //TODO:OPTIM: isContinuous()
  for (int i = 0 ; i < m_nb ; ++i) {
    float *f = m_dataIndex.ptr<float>(i);
    for (int c = 0 ; c < m_dim ; ++c) {
      f[c] = static_cast<float>(m_sdata[i*m_dim+c]);
      //std::cout<<f[c]<<" ";
    }
    //std::cout<<std::endl;
  }

  m_indexCv = new cv::flann::Index(m_dataIndex, cv::flann::KDTreeIndexParams(1));
  std::cout<<"index construit"<<std::endl;

  segment();

assert(m_msRawData.size() == (size_t)instances.rows);
  labels = cv::Mat::zeros(instances.rows, 1, CV_32S);
  for (int i = 0; i < instances.rows; ++i) {
    labels.at<int>(i, 0) = m_msRawData[i];
  }
}

void
ClustererMeanShift::compute(const std::vector<double*> &instances, size_t dim)
{
  const size_t height = instances.size();
  cv::Mat mat(instances.size() , dim, CV_32F);
  for (size_t y = 0 ; y < height ; ++y) {
    float *buf = mat.ptr<float>(y);
    for (size_t x = 0 ; x < dim ; ++x) {
      buf[x] = instances[y][x];
    }
  }

  // TODO:REMOVE or fix
  cv::Mat labels;
  compute(mat, labels);
}

void
ClustererMeanShift::setParameters(double *weight, size_t dim)
{
  if (weight != NULL) {
    //m_weight = std::vector<double>(weight, weight + dim);
    m_weight.resize(dim);
    for (size_t i=0; i<dim; ++i)
      m_weight[i] = weight[i];
  }
}

void 
ClustererMeanShift::segment()
{
  if (m_sigmaR <= 0)
    return;

  if (m_dim <= 0)
    return;

  std::cerr<<"m_dim="<<m_dim<<" m_nb="<<m_nb<<"\n";

  //m_msRawData = std::vector<int>(m_nb, -1);
  m_msRawData.resize(m_nb);
  std::fill(m_msRawData.begin(), m_msRawData.end(), -1);

  //m_modeTable = std::vector<int>(m_nb, 0);
  m_modeTable.resize(m_nb);
  std::fill(m_modeTable.begin(), m_modeTable.end(), 0);


  std::vector<float> yk(m_dim, 0);


  //B:OPTIM: see doMeanShift()
  m_Mh.resize(m_dim);
  m_pointList.resize(m_nb);
  m_ytemp.resize(m_dim);
  m_indices.resize(m_nb);
  m_dist.resize(m_nb);

  m_query.resize(m_dim);

  int idxs = 0;

  for (int i = 0 ; i < m_nb ; ++i) {
  
    if (m_modeTable[i] == 1)
      continue;
    
    m_pointCount = 0;
    
    idxs = i*m_dim;

    for (int j = 0 ; j < m_dim ; ++j)
      yk[j] = m_sdata[idxs+j];
    
    doMeanShift(yk);
  }

  std::cout<<"meanShift : done"<<std::endl;

  return;
}

void
ClustererMeanShift::doMeanShift(std::vector<float> &yk)
{
  int modeCandidate_i = 0;

  int idxd = 0;
  int idxs = 0;

  int pos_i = getIndex(yk);
  if ( pos_i == -1) {
    std::cout<<yk[0]<<" "<<yk[1]<<" "<<yk[2]<<std::endl;
    std::cout<<"erreur fatal"<<std::endl;
  }
  
  

  //B:OPTIM: we do not want to allocate these vector each time / for each call to doMeanShift
  // we keep them in the class and just reset them here
  // (but would be a problem for parallelization)
  //std::vector<float> Mh(m_dim, 0);
  //std::vector<int> pointList(m_nb, 0);// point traveled by MS
  assert(m_Mh.size() == (size_t)m_dim);
  assert(m_pointList.size() == (size_t)m_nb);
  float *Mh = &m_Mh[0];
  int *pointList = &m_pointList[0];
  memset(Mh, 0, sizeof(float)*m_Mh.size()); 
  memset(pointList, 0, sizeof(int)*m_pointList.size());


  MSvector(m_Mh, yk, m_pointList);
  
  double mvAbs = magnitudeSq(m_Mh);

  //int iterationCount = 1;
  //int iterationConvergence = 0;
  //int limitConvergence = 20;
  /*
  while ((mvAbs >= EPSILON2)&&(iterationCount < 100)) {
    for ( int j = 0 ; j < m_dim ; ++j)
      yk[j] += Mh[j];

    if (mvAbs < 0.01)
      ++iterationConvergence;
    if (iterationConvergence>20)
      break;

    MSvector(Mh, yk, pointList);
      
    mvAbs = magnitudeSq(Mh);
      
    ++iterationCount;
  }
  */

  //B:OPTIM: idem
  //std::vector<double> ytemp;
  double *ytemp = &m_ytemp[0];

  while ((mvAbs >= EPSILON2)) {// && (iterationCount < LIMIT2)) {
    //ytemp.resize(m_dim);
    assert(m_ytemp.size() == (size_t)m_dim);
      
    for ( int j = 0 ; j < m_dim ; ++j) {
      ytemp[j] = yk[j];
      yk[j] += Mh[j];
    }
    
    idxd = getIndex(yk);
      
    if (idxd >= 0) {
      if ( ( m_modeTable[idxd] != 2) && ( idxd != pos_i)) {
	double diff = 0;
	idxs = m_dim*idxd;

	for ( int k = 0 ; k < m_dim ; ++k) {
	  const double el = m_sdata[idxs+k] - yk[k]; 
	  diff += el*el;
	}
	  
	if (diff < m_speedThreshold) {
	  if (m_modeTable[idxd] == 0) {
	    pointList[m_pointCount++] = idxd;
	    m_modeTable[idxd] = 2;
	  } else {

	    yk = m_modes[m_msRawData[idxd]];
	      
	    m_modeTable[pos_i] = 1;
	      
	    mvAbs = -1;
	      
	    break;
	  }
	}
      }
    }
    else {
      double diff = 0;
      for ( int k = 0 ; k < m_dim ; ++k) { 
	const double el = ytemp[k] - yk[k];
	diff += el*el;
      }
      if ( diff < m_speedThreshold) {
	m_modeTable[pos_i] = 1;
	mvAbs = -1;
	break;
      }
    }
     
    MSvector(m_Mh, yk, m_pointList);
      
    mvAbs = magnitudeSq(m_Mh);
    
    //++iterationCount;
  }
  
  if (mvAbs >= 0) {
    for (int j = 0 ; j < m_dim ; ++j)
      yk[j] += Mh[j];
    
    m_modeTable[pos_i] = 1;
  }
    
  int index = addMode(yk);

  for (int j = 0 ; j < m_pointCount ; ++j) {
    modeCandidate_i = pointList[j];
    if ( modeCandidate_i != -1) {
      m_modeTable[modeCandidate_i] = 1;
	
      m_msRawData[modeCandidate_i] = index;
      
    }
  }
  
  m_msRawData[pos_i] = index;
}

  
double
ClustererMeanShift::magnitudeSq(const std::vector<float> &Mh)
{
  double mvAbs = 0;
  if ( m_dim == 3 ) {
    mvAbs += (Mh[0]*Mh[0]+Mh[1]*Mh[1]+Mh[2]*Mh[2])*m_sigmaR*m_sigmaR;
  }else if ( m_dim == 1) {
    mvAbs += Mh[0]*Mh[0]*m_sigmaR*m_sigmaR;
  }else {
    for (int i = 0 ; i < m_dim ; ++i)
      mvAbs += Mh[i]*Mh[i]*m_sigmaR*m_sigmaR;
  }
  return mvAbs;
}

void
ClustererMeanShift::MSvector(std::vector<float> &Mh, const std::vector<float> &yk, std::vector<int> &pointList)
{
  double wsuml = 0.0;
  double weight = 0.0;

  int idxd = 0;
  int idxs = 0;

  //B:OPTIM: cached, to avoid re-allocation
  //std::vector<int> indices(m_nb,-1);
  //std::vector<float> dist(m_nb,-1);
  assert(m_indices.size() == (size_t)m_nb);
  assert(m_dist.size() == (size_t)m_nb);
  std::fill(m_indices.begin(), m_indices.end(), -1);
  std::fill(m_dist.begin(), m_dist.end(), -1);
  int *indices = &m_indices[0];
  
#if CV_MAJOR_VERSION*100+CV_MINOR_VERSION*10+CV_SUBMINOR_VERSION < 231
  const int taille = m_indexCv->radiusSearch(yk, m_indices, m_dist, 1, cv::flann::SearchParams(m_nb));
#else
  const int maxResults = 1; //B:not documented. We need at least 1. I seems to be a minimum.
  //                         std::numeric_limits<int>::max() is a really bad idea: it will reallocate m_indices & m_dist each time 
  const int taille = m_indexCv->radiusSearch(yk, m_indices, m_dist, 1, maxResults, cv::flann::SearchParams(m_nb));
#endif

  //std::cerr<<"taille="<<taille<<"\n";
  
  for (int i = 0 ; i < taille; ++i) {
    
    assert((size_t)i < m_indices.size());
    if ( indices[i] == -1)
      continue;
    
    double diff = 0;
    idxd = indices[i];
    idxs = m_dim * idxd;
    
    for ( int j = 0 ; j < m_dim ; ++j) {
      assert((size_t)idxs<m_sdata.size() && ! yk.empty());
      const double el = m_sdata[idxs]-yk[0]; //B: bug ??? not dependant of j !!!
      diff += el*el;
    }
    
    assert((size_t)idxd < m_weight.size());
    weight = m_weight[idxd];

    assert(Mh.size() >= (size_t)m_dim);
    for ( int k = 0 ; k < m_dim ; ++k) {
      assert((size_t)(idxs+k) < m_sdata.size());
      Mh[k] += weight * m_sdata[idxs+k];
    }
    wsuml += weight;
    
    if (diff < m_speedThreshold) {
      assert((size_t)idxd < m_modeTable.size());
      if (m_modeTable[idxd] == 0) {
	assert((size_t)m_pointCount < m_pointList.size());
	pointList[m_pointCount++] = idxd;
	m_modeTable[idxd] = 2;
      }
    }
  }
  
  if (wsuml > 0.0) {
    const float inv_wsuml = 1.0 / wsuml;
    assert(m_Mh.size() >= (size_t)m_dim);
    assert(yk.size() >= (size_t)m_dim);
    for (int j = 0; j < m_dim ; ++j)
      Mh[j] = Mh[j] * inv_wsuml - yk[j];
  }
  else {
    assert(m_Mh.size() >= (size_t)m_dim);
    for (int i = 0 ; i < m_dim ; ++i)
      Mh[i] = 0;
  }

}

int
ClustererMeanShift::getIndex(const std::vector<float> &yk) const
{
  //OPTIM: cached
  //std::vector<int> queryIndices(1, -1);
  //std::vector<float> queryDist(1, 0);
  assert(m_queryIndices.size() == 1);
  assert(m_queryDist.size() == 1);
  m_queryIndices[0] = -1;
  m_queryDist[0] = 0;

  m_indexCv->knnSearch(yk, m_queryIndices, m_queryDist, 1, cv::flann::SearchParams(m_nb));
  
  if ( m_queryDist[0] < EPSILON2)
    return m_queryIndices[0];
  else {
    return -1;
  }
}

int
ClustererMeanShift::addMode(const std::vector<float> &yk)
{
  size_t size = m_modes.size();
  bool ret = false;
  for (size_t i = 0 ; i < size ; ++i) {
    ret = true;
    size_t s = yk .size();
    for (size_t j = 0 ; j < s ; ++j) {
      if (fabs(yk[j] - m_modes[i][j] ) > std::numeric_limits<float>::epsilon()) {
	ret = false;
	break;
      }	
    }
    if (ret)
      return i;
  }
  if (!ret) {
    m_modes.push_back(yk);
    //std::cout<<"ClustererMeanShift::addMode() : push vector<float> of size="<<yk.size()<<" in m_modes"<<std::endl;
  }
  return m_modes.size() - 1;
}


ise::Clusterer::LabelType
ClustererMeanShift::getCluster(const double* instance, size_t dim) const
{
  assert(dim == static_cast<size_t>(m_dim));
  //OPTIM: cached
  //std::vector<float> query(m_dim);
  assert(m_query.size() == dim);

  const float inv_sigmaR = (m_sigmaR > 0 ? 1.0 / m_sigmaR : 0.0f);
  for (size_t i = 0 ; i < dim ; ++i) {
    m_query[i] = static_cast<float>(instance[i]) * inv_sigmaR;
  }

  const int idxd = getIndex(m_query);
  if (idxd == -1) 
    std::cout<<"erreur query don't present in index"<<std::endl;
  
  return m_msRawData[idxd];
}

