/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
/**
 * @file iseClustererMeanShift.hpp
 * @brief 
 */

#ifndef ISECLUSTERERMEANSHIFT_HPP
#define ISECLUSTERERMEANSHIFT_HPP

#include "iseSegmentationContext.hpp"

#include <girlTools/girlToolsImageFeature.hpp>
#include <ise/iseConversion.hpp>

#include "iseClusterer.hpp"
#include "iseClustererFactoryTemplate.hxx"

#include <cmath>

#include <opencv2/core/core.hpp>
#include <opencv2/flann/flann.hpp>


#define EPSILON2 0.000001
#define LIMIT2 100

namespace ise {

  class GIRL_API ClustererMeanShift : public Clusterer {
  public:
    /**
     * @brief Constructor
     */
    ClustererMeanShift();
    
    /**
     * @brief Destructor
     */
    virtual ~ClustererMeanShift()
    {
      delete m_indexCv;
    }

    const std::string uri() const;
    
    virtual void compute(const cv::Mat &, cv::Mat &labels);

    virtual void compute(const std::vector<double*>&, size_t);

    virtual ise::Clusterer::LabelType getCluster(const double*, size_t) const;

    void setParameters(double* weight = NULL, size_t dim = 0);

    float sigmaR() const { return m_sigmaR; }

    void setSigmaR(float val) { m_sigmaR = val; }

//    /**
//     * return the modes
//     */
//    const std::vector<double>&
//    getRawData() const
//    {
//      return m_msRawData;
//    }

  private:
    void segment();

    double magnitudeSq(const std::vector<float> &Mh);
    void MSvector(std::vector<float> &Mh, const std::vector<float> &yk, std::vector<int> &pointList);

    int getIndex(const std::vector<float> &yk) const;
    void doMeanShift(std::vector<float> &yk);

    int addMode(const std::vector<float> &yk);
    
  private:

    float m_speedThreshold;

    int m_pointCount;
    
    std::vector<int> m_msRawData;
    std::vector<std::vector<float> > m_modes;

    float m_sigmaR;

    std::vector<double> m_weight;

    int m_dim;
    int m_nb;
    double m_sMins[3];
    double m_sMaxs[3];

    std::vector<double> m_sdata;
    std::vector<int> m_modeTable;

    cv::Mat m_dataIndex;
    cv::flann::Index *m_indexCv;

    cv::SparseMat_<int>  m_index;

    //B:OPTIM: cache
    std::vector<float> m_Mh;
    std::vector<int> m_pointList;
    std::vector<double> m_ytemp;
    std::vector<int> m_indices;
    std::vector<float> m_dist;

    mutable std::vector<float> m_query;
    mutable std::vector<int> m_queryIndices;
    mutable std::vector<float> m_queryDist;

  };

  typedef ClustererFactoryTemplate<ClustererMeanShift> ClustererMeanShiftFactory;
}

#endif /* ISECLUSTERERMEANSHIFT_HPP */
