/*
  3D Color Histogram class.
  Copyright (C) 2010-2011 Cédric Aguerre  aguerre@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISECOLORHISTOGRAM_HPP
#define ISECOLORHISTOGRAM_HPP
  
#include <cassert>
#include <vector>
#include <girl/girlCommon.hpp>
#include <girl/girlToger.hpp>
#include <girlTools/girlToolsImage.hpp>
#include <ise/iseConversion.hpp>
#include <opencv2/core/core.hpp>

namespace ICH { class RegionHistoComputer; }

namespace ise {

  /*
   * Warning: Only use dense histogram (type == CV_HIST_ARRAY)
   *          Sparse histograms is not considered (TODO...)
   */

  class GIRL_API ColorHistogram
  {
    friend class ICH::RegionHistoComputer;
    friend class ColorHistogramIterator;
  public:
    
    ColorHistogram();

    ColorHistogram(int *sizes, int type = CV_HIST_ARRAY, float **ranges = NULL);

    ColorHistogram(const ColorHistogram &ch);

    ~ColorHistogram();

    ColorHistogram &operator=(const ColorHistogram &ch);

    // access (i,j,k)-th element of 3D histogram
    float operator()(const int i, const int j, const int k);
    
    // access i-th element of histogram of channel chan
    float operator()(const int chan, const int i);

    void setRanges(float **ranges);

    void setColorSpaceConversion(int colorSpaceConversion);

    void clear();

    //void populate(const girl::tools::Image &img);

    void populate(const girl::tools::Image &img,
		  const girl::toger &tog,
		  const girl::regionDeque &regions);

    //size_t size() const { return m_frequencies.size(); } // nb of bins
    size_t nbPoints() const { return m_nbPoints; }

    inline void getSizes(int sizes[]) { for (int i = 0; i < 3; ++i) sizes[i] = m_sizes[i]; }

    // compare 3D histograms if chan == -1; else compare histograms of channel chan
    // see cvCompareHist to obtain more details on method
    double compareTo(const ColorHistogram &ch, int method, int chan = -1);

    void normalize();

  protected:

    CvHistogram *m_histo3D; // 3D histogram
    CvHistogram *m_chanHisto[3]; // histogram of each channel
    int m_sizes[3];
    int m_type;
    float m_ranges[3][2];
    bool m_rangesSet; // true if m_ranges has been initialized
    int m_conversion;

    size_t m_nbPoints;

  private:
    void copy(const ColorHistogram &ch);

  };

}

#endif /* ! ISECOLORHISTOGRAM_HPP */
