/*
  3D Color Histogram Iterator class.
  Copyright (C) 2010-2011 Cédric Aguerre  aguerre@labri.fr
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "iseColorHistogramIterator.hpp"
using namespace ise;


ColorHistogramIterator::ColorHistogramIterator(const ColorHistogram &ch1, int chan)
  : m_chan(chan)
{
  cv::Mat planes[1];
  if (chan >= 0 && chan <= 2) {
    cv::Mat h1((const CvMatND *) ch1.m_chanHisto[chan]->bins);
    const cv::Mat *arrays[] = { &h1, 0 };
    it = cv::NAryMatIterator(arrays, planes);
  }
  else {
    cv::Mat h1((const CvMatND *) ch1.m_histo3D->bins);
    const cv::Mat *arrays[] = { &h1, 0 };
    it = cv::NAryMatIterator(arrays, planes);
  }

  plane = 0;
  len = it.planes[0].rows * it.planes[0].cols;
  dataPos = 0;
  data1 = (float *) it.planes[0].data;
  data2 = NULL;
}

ColorHistogramIterator::ColorHistogramIterator(const ColorHistogram &ch1, const ColorHistogram &ch2, int chan)
  : m_chan(chan)
{
  cv::Mat planes[2];
  if (chan >= 0 && chan <= 2) {
    cv::Mat h1((const CvMatND *) ch1.m_chanHisto[chan]->bins);
    cv::Mat h2((const CvMatND *) ch2.m_chanHisto[chan]->bins);
    const cv::Mat *arrays[] = { &h1, &h2, 0 };
    it = cv::NAryMatIterator(arrays, planes);
  }
  else {
    cv::Mat h1((const CvMatND *) ch1.m_histo3D->bins);
    cv::Mat h2((const CvMatND *) ch2.m_histo3D->bins);
    const cv::Mat *arrays[] = { &h1, &h2, 0 };
    it = cv::NAryMatIterator(arrays, planes);
  }

  plane = 0;
  len = it.planes[0].rows * it.planes[0].cols;
  dataPos = 0;
  data1 = (float *) it.planes[0].data;
  data2 = (float *) it.planes[1].data;
}

ColorHistogramIterator &
ColorHistogramIterator::operator++() // ++iterator
{
  if (atEnd()) return *this;
 
  if (dataPos < len-1) 
    ++dataPos;
  else {
    dataPos = 0;
    ++plane;
    ++it;
    data1 = (float *) it.planes[0].data;
    if (data2) 
      data2 = (float *) it.planes[1].data;
  }
  return *this;
}

ColorHistogramIterator
ColorHistogramIterator::operator++(int) // iterator++
{
  ColorHistogramIterator itr = *this;
  ++*this; // call operator++()
  return itr;
}

float &
ColorHistogramIterator::operator()(int i) const
{
  if (i == 1) 
    return data2[dataPos];
  return data1[dataPos];
}

bool
ColorHistogramIterator::atEnd()
{
  return (size_t(plane+1) == it.nplanes && dataPos+1 == len);
}
