/*
  3D Color Histogram Iterator class.
  Copyright (C) 2010-2011 Cédric Aguerre  aguerre@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISECOLORHISTOGRAMITERATOR_HPP
#define ISECOLORHISTOGRAMITERATOR_HPP

#include <opencv2/core/core.hpp>

#include "iseColorHistogram.hpp"


namespace ise {


  class GIRL_API ColorHistogramIterator
  {
  public:

    ColorHistogramIterator();

    // Iterate on ch1.
    // Iterate on 3D elements if chan == -1; else iterate on the specified channel
    ColorHistogramIterator(const ColorHistogram &ch1, int chan = -1);

    // Iterate simultaneously on ch1 and ch2.
    // Iterate on 3D elements if chan == -1; else iterate on the specified channel
    ColorHistogramIterator(const ColorHistogram &ch1, const ColorHistogram &ch2, int chan = -1);

    ColorHistogramIterator &operator++();
    ColorHistogramIterator operator++(int);

    float &operator*() const { return (*this)(0); }
    float &operator()(int i) const;

    bool atEnd();

  private:
    int m_chan; // channel to iterate on; if -1 iterate on 3D elements
    cv::NAryMatIterator it;
    int plane; // current plane in MatND
    int len; // size of a plane
    int dataPos; // in [0;len[
    float *data1, *data2; // planes data
  };

}

#endif /* ! ISECOLORHISTOGRAMITERATOR_HPP */
