/*
  Segmentation clustering: Watershed.
  Copyright (C) 2005-2009 Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "iseCvSobel.hpp"

#include <opencv2/imgproc/imgproc.hpp>

using namespace ise;

CvSobel::CvSobel(girl::shared_ptr<girl::tools::ImageFeature> imageFeature)
  : m_imageFeature(imageFeature)
{
 
}

void
CvSobel::compute(girl::shared_ptr<girl::tools::ImageFeature> &dest, int xorder, int yorder) 
{
  assert( xorder != yorder);
  const int width = m_imageFeature->width();
  const int height = m_imageFeature->height();
  const int channels = m_imageFeature->channels();
  dest = girl::shared_ptr<girl::tools::ImageFeature>(new girl::tools::ImageFeature(width, height, channels));
  
  girl::shared_ptr<cv::Mat> src = m_imageFeature->cvMat();
  girl::shared_ptr<cv::Mat> dst = dest->cvMat();

  cv::Sobel(*(src.get()), *(dst.get()), -1, xorder, yorder);
}
