/*
  Split and Merge algorithm default implementation.
  Copyright (C) 2005-2009 Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "iseDeleteSelectionAndSplitAlgorithm.hpp"

#include "iseSplitDriver.hpp"
#include "iseMergeDriver.hpp"
#include "iseSegmentationClustering.hpp"
#include "iseSegmentationContext.hpp"
#include "iseMergeAlgorithm.hpp"
#include "iseSplitAlgorithm.hpp"

#include <girl/girlToger.hpp>
#include <girlTools/girlToolsRegionInfos.hpp>

using namespace ise;




void 
DeleteSelectionAndSplitAlgorithm::apply(const girl::regionDeque &regionsToSplitMerge,
				  ise::MergeDriver &driverMerge,
				  ise::SplitDriver &driverSplit,
				  ise::SegmentationClustering &clust,
				  int level,
				  girl::regionDeque &createdRegions)
{
  
  assert(m_splitAlgo);
  assert(m_mergeAlgo);
  assert(m_segContext);

  clust.init(regionsToSplitMerge, 0);

  girl::toger *toger = m_segContext->getToger();

  girl::regionDeque copyQueue;

  //TODO
  //m_splitAlgo->init(Queue, );

  girl::regionDeque::const_iterator it = regionsToSplitMerge.begin();
  const girl::regionDeque::const_iterator itEnd = regionsToSplitMerge.end();
  for ( ; it != itEnd; ++it) { 
    toger->xMarkRegionBoundary(*it, girl::toger::MARK_7);
    copyQueue.push_back(*it);
  }

  girl::regionDeque motherQueue;
  girl::tools::getMotherRegions(*toger, regionsToSplitMerge, motherQueue);


  girl::regionDeque::const_iterator it2 = motherQueue.begin();
  const girl::regionDeque::const_iterator it2End = motherQueue.end();
  for ( ; it2 != it2End; ++it2) 
    {
      toger->xMarkRegionBoundary(*it2, girl::toger::MARK_7);
      copyQueue.push_back(*it2);
    }

  driverMerge.setNeighbourhood(girl::toger::NEIGHBOUR_ANY, girl::toger::MARK_7);

  size_t numRegions, prevNumRegions;
  numRegions = toger->numRegions();


  do {

    prevNumRegions = numRegions;

    createdRegions.clear();

    m_mergeAlgo->apply(copyQueue,
		       driverMerge,
		       level,
		       createdRegions);

    std::swap(copyQueue, createdRegions);

    numRegions = toger->numRegions();
    
     std::cerr<<"copyQueue.size()="<<copyQueue.size()<<" createdRegions.size()="<<createdRegions.size()<<" toger->numRegions()="<<toger->numRegions()<<std::endl;

  }
  while (numRegions != prevNumRegions);
  
  
  //std::swap(copyQueue, createdRegions);

  
  toger->unmarkAllDarts(girl::toger::MARK_7);

  m_splitAlgo->apply(copyQueue, 
		     driverSplit,
		     clust,
		     level,
		     createdRegions);


  //std::swap(copyQueue, createdRegions);

}
