#ifndef ISEENCLOSINGSHAPEFEATUREVISITOR_HPP
#define ISEENCLOSINGSHAPEFEATUREVISITOR_HPP


#include <girl/girlCommon.hpp>

class EnclosingBox;
class EnclosingConvexHull;
class EnclosingShape;
class EnclosingEllipse;
class EnclosingCircle;
class EnclosingRotatedRect;

namespace ise {

/**
  * Computes a feature (area, perimeter, ...) on an EnclosingShape.
  * \code
      EnclosingConvexHull hull(points);
      ise::PerimeterEnclosingShapeFeatureVisitor perimeterComputer;
      double area = hull.accept (perimeterComputer) ;
  * \endcode
  * \example tests/testEnclosingShapeFeatureVisitor.cpp
  */
class GIRL_API EnclosingShapeFeatureVisitor
{

public:
    EnclosingShapeFeatureVisitor() {}
    virtual ~EnclosingShapeFeatureVisitor() {}

    virtual double visit(EnclosingBox *box) = 0;
    virtual double visit(EnclosingConvexHull *box) = 0;
    virtual double visit(EnclosingEllipse *box) = 0;
    virtual double visit(EnclosingCircle *box) = 0;
    virtual double visit(EnclosingRotatedRect *box) = 0;
};

}

#endif // ISEENCLOSINGSHAPEFEATUREVISITOR_HPP
