#include "iseEnclosingShapeFeatureVisitorRegistry.hpp"
#include <cassert>
#include <ise/EnclosingShape.hpp>

using namespace ise;

EnclosingShapeFeatureVisitorRegistry *EnclosingShapeFeatureVisitorRegistry::m_instance = NULL;

EnclosingShapeFeatureVisitorRegistry::EnclosingShapeFeatureVisitorRegistry()
{

}

ise::EnclosingShapeFeatureVisitorRegistry::~EnclosingShapeFeatureVisitorRegistry()
{
  m_visitorFactories.erase(m_visitorFactories.begin(), m_visitorFactories.end());
}

double
ise::EnclosingShapeFeatureVisitorRegistry::applyEnclosingShapeFeatureVisitor(const std::string &name, EnclosingShape &enclosingShape)
{
  EnclosingShapeFeatureVisitor *feature = visitorByName(name);
  if (feature == NULL) {
    assert(false);
    throw "missing feature";
  }

  const double featureValue = enclosingShape.accept(*feature);
  delete feature;

  return featureValue;
}

void
ise::EnclosingShapeFeatureVisitorRegistry::addVisitorFactory(ise::EnclosingShapeVisitorFactory *visitor)
{
  m_visitorFactories[visitor->name()] = visitor;
}

ise::EnclosingShapeFeatureVisitor *
EnclosingShapeFeatureVisitorRegistry::visitorByName(const std::string &name)
{
  return m_visitorFactories[name]->create();
}

ise::EnclosingShapeFeatureVisitorRegistry *
EnclosingShapeFeatureVisitorRegistry::instance()
{
  if (m_instance == NULL)
    m_instance = new EnclosingShapeFeatureVisitorRegistry;
  return m_instance;
}

std::vector<std::string>
ise::EnclosingShapeFeatureVisitorRegistry::registeredFeatureVisitors()
{
  std::vector<std::string> visitors;
  MapFactories::const_iterator end = m_visitorFactories.end();
  for (MapFactories::const_iterator it = m_visitorFactories.begin(); it != end; ++it) {
    visitors.push_back(it->first);
  }
  return visitors;
}
