#include "iseEnclosingShapeVisitorOnDistribFilter.hpp"

#include <cassert>
#include <cmath>

#include <ise/iseEnclosingShapeFeatureVisitorRegistry.hpp>

using namespace ise;


/// \attention anonymous class ! Should be moved
/// \todo move the class :-S
class IterateOnShapeListWthVisitorName
{
public :
  virtual ~IterateOnShapeListWthVisitorName() {}

  void iterate(EnclosingShapeVisitorOnDistribFilter::ShapeList &shapeList, const std::string &visitorName)
  {
    EnclosingShapeFeatureVisitorRegistry *registry = EnclosingShapeFeatureVisitorRegistry::instance();
    assert(registry);

    EnclosingShapeVisitorOnDistribFilter::ShapeList::iterator it = shapeList.begin();
    for ( ; it != shapeList.end(); ++it)
      {
	EnclosingShapeFactory::EnclosingShapePtr enclosingShapePtr = *(it);
	assert(enclosingShapePtr);

	const double value = registry->applyEnclosingShapeFeatureVisitor(visitorName, *(enclosingShapePtr));
#ifndef _MSC_VER
	assert(std::isfinite(value));
#endif
	action(value);
      }
  }
protected :
  virtual void action(double value) = 0;
};



class PopulateHistogram : public IterateOnShapeListWthVisitorName
{
public :

  explicit PopulateHistogram(ise::SparseHistogram1DD  &histo)
    : m_histo(histo)
  {}

protected :

  void action(double value)
  {
#ifndef _MSC_VER
	  assert(std::isfinite(value));
#endif
	  m_histo.add(value, 1);
  }
  
  ise::SparseHistogram1DD &m_histo;
};

class FilterShapes : public IterateOnShapeListWthVisitorName
{
public :
  /// \attention filteredShapes should be initialised with false
  FilterShapes(std::vector<bool> &filteredShapes, double threshold)
    : m_filteredShapes(filteredShapes)
    , m_threshold (threshold)
    , m_currentShapeIndex (0)
  {}

protected :

  void action(double value)
  {
    // on garde si <
    if (value > m_threshold )
      m_filteredShapes[m_currentShapeIndex] = true;

    ++m_currentShapeIndex;
  }

  std::vector<bool> &m_filteredShapes;
  double m_threshold;
  int m_currentShapeIndex;
};



EnclosingShapeVisitorOnDistribFilter::EnclosingShapeVisitorOnDistribFilter(ShapeList &list)
  : m_shapeList (list)
{}


void
EnclosingShapeVisitorOnDistribFilter::populateHistogram(ise::SparseHistogram1DD &histo, 
							const std::string &visitorName)
{
  histo.clear();
  PopulateHistogram populator(histo);
  populator.iterate (m_shapeList, visitorName);
}

void
EnclosingShapeVisitorOnDistribFilter::filter(std::vector<bool> &filteredShapes, 
					     double threshold, 
					     const std::string &visitorName)
{
  FilterShapes filterer (filteredShapes, threshold);
  filterer.iterate (m_shapeList, visitorName);
}

