/*
  Copyright (C) 2005-2011 Remi Vieux, Boris Mansencal, remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
#include "iseForegroundBackgroundClassifierColorArea.hpp"

#include <cassert>

#include <girlTools/girlToolsRegionInfos.hpp>

#include "iseConversion.hpp"
#include "iseHistogram1D.hpp"
#include "iseQuantificationThresholderOtsu1T.hpp"
#include "iseSegmentationContext.hpp"
namespace ise
{

  ForegroundBackgroundClassifierColorArea::ForegroundBackgroundClassifierColorArea(ise::SegmentationContext *segContext)
  : ForegroundBackgroundClassifier(segContext)
  {

  }

  ForegroundBackgroundClassifierColorArea::~ForegroundBackgroundClassifierColorArea()
  {
  }



  void
  ForegroundBackgroundClassifierColorArea::classify(const girl::regionDeque &input, girl::regionDeque &foreground, girl::regionDeque &background)
  {

    assert(m_segContext->getImage());
    assert(m_segContext->getToger());
    foreground.clear();
    background.clear();
    ise::Histogram1D meanHisto(0, 255);
    std::vector<double> mean(input.size(), 0);
    girl::regionDeque::const_iterator inputIt = input.begin();
    const girl::regionDeque::const_iterator inputItEnd = input.end();
    std::vector<double>::iterator it = mean.begin();
    for ( ; inputIt != inputItEnd; ++inputIt, ++it) {

        girl::regionDeque reg;
        reg.push_back(*inputIt);
        ise::Histogram1D histo(*m_segContext->getImage(),
            *m_segContext->getToger(),
            ISE_RGB_to_GRAY,
            0,
            reg);
        const double val = histo.getMean();
        meanHisto[histo.keyToIndex(val)]++;
        *it = val;
    }

    ise::QuantificationThresholderOtsu1T otsu;
    std::vector<double> thresholds;
    otsu.getThresholds(meanHisto, thresholds);
    const double threshold = thresholds[0];
    for (it = mean.begin(), inputIt = input.begin();
        inputIt != inputItEnd; ++it, ++inputIt) {
        // most of the time fg regions are darker, that is < threshold
        if (*it < threshold) {
            foreground.push_back(*inputIt);
        } else {
          background.push_back(*inputIt);
        }
    }

    // classification: assume foreground regions have area inferior than bg regions
    // hence area(fg) < area(bg). Otherwise, swap fg/bg

    size_t darkArea  = girl::tools::computeArea(*m_segContext->getToger(), foreground);
    size_t lightArea = girl::tools::computeArea(*m_segContext->getToger(), background);
    // std::cout << "Area regions sombre: " << darkArea << " Area regions claires: " << lightArea << std::endl;
    if (darkArea > lightArea)
      std::swap(foreground, background);

    assert(input.size() == (foreground.size() + background.size()));

  }

}
