/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISEFOREGROUNDBACKGROUNDCLASSIFIERCOLORAREA_HPP_
#define ISEFOREGROUNDBACKGROUNDCLASSIFIERCOLORAREA_HPP_

#include "iseFactoryTemplate.hxx"
#include "iseForegroundBackgroundClassifier.hpp"
namespace ise
{
  class SegmentationContext;

  /**
   * A fg/bg classifier which:
   * sort regions into 2 classes according to their gray levels
   * (mean gray level, otsu thresholding)
   * assign fg/bg label according to the total area: assume that bg area is higher than fg area
   *
   *
   * NB: This classifier is suitable for Connected Components, where all the pixels in the region
   * have approximately the same color. It is NOT suitable for Shapes (Box, Convex Hulls) since
   * a lot of pixels in the region are actually from the background, hence the mean gray levels
   * are highly noisy
   */

  class GIRL_API ForegroundBackgroundClassifierColorArea : public ForegroundBackgroundClassifier
  {
  public:
    explicit ForegroundBackgroundClassifierColorArea(ise::SegmentationContext *segContext);
    virtual
    ~ForegroundBackgroundClassifierColorArea();

    virtual void
    classify(const girl::regionDeque &input, girl::regionDeque &foreground, girl::regionDeque &background);
  };

  typedef FactoryTemplate<ForegroundBackgroundClassifier, ForegroundBackgroundClassifierColorArea> ForegroundBackgroundClassifierColorAreaFactory;

}

#endif /* ISEFOREGROUNDBACKGROUNDCLASSIFIERCOLORAREA_HPP_ */
