/*
  Copyright (C) 2005-2011 Roigt Julian, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "iseGaussianPyramid.hpp"

#include <opencv2/imgproc/imgproc.hpp>

#ifndef M_PI
//Visual C++ does not define these constants in <cmath>
#define M_PI 3.14159265358979323846
#endif 

static const int KERNEL_SIZE = 5;

using namespace ise;

GaussianPyramid::GaussianPyramid()
  : m_levels()
{
}

void
GaussianPyramid::setParameters(int levels, int divider)
{
  init(levels, divider);
}

void
GaussianPyramid::getParameters(int &levels, int &divider) const
{
  levels = this->level();
  divider = this->divider();
}


void 
GaussianPyramid::compute(const cv::Mat &image)
{
  const cv::Mat m = image;
  
  cv::Mat tmp(image.rows, image.cols, CV_8UC3);

  m.convertTo(tmp, CV_8UC3);

  cv::buildPyramid(tmp, m_levels, this->level());
}

void
GaussianPyramid::compute(const girl::tools::Image &image, std::vector<girl::tools::Image> &res)
{
  const cv::Mat* m = image.cvMat();
  
  cv::Mat tmp(image.width(), image.height(), CV_8UC3);

  m->convertTo(tmp, CV_8UC3);

  std::vector<cv::Mat> v;
  cv::buildPyramid(tmp, v, this->level());
  
  for (int i = 0 ; i < this->level(); ++i)
    res.push_back(girl::tools::Image(v[i]));
}
  
void
GaussianPyramid::gaussian(size_t N, std::vector<std::vector<double> > &vect)
{
  vect.resize(N, std::vector<double>(N, 0));
  int size = N/2;
  float s = 0.75;
  for ( int x = -size ; x <= size ; ++x) {
    for ( int y = -size ; y <= size ; ++y) {
      vect[x+size][y+size] = ( 1 / ( s * s * 2 * M_PI ) *  exp( - (x*x + y*y) / (2*s*s)) ); 
    }
  }
}
