/*
  Templated Image class.
  Copyright (C) 2007-2009 Hugo Boujut, Jean-Philippe Domenger, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISEGENERICIMAGE_HPP
#define ISEGENERICIMAGE_HPP

#include <girlTools/girlToolsImage.hpp>
#include <girlTools/girlToolsImageFeature.hpp>

#include "iseImageValuation.hpp"
#include "iseConversion.hpp"
#include "iseVector.hpp"

#include <iostream> //DEBUG

namespace ise
{
  template<typename T, std::size_t CHANNELS>
  class GIRL_API GenericImage: public ValuatedImage<girlExt::vector<T, CHANNELS> >
  {
  public:
    typedef girlExt::vector<T, CHANNELS> pixel_t;

    GenericImage(int width, int height)
      : ValuatedImage<girlExt::vector<T, CHANNELS> >(width, height)
        {}

    GenericImage(const GenericImage<T, CHANNELS> &image)
      : ValuatedImage<girlExt::vector<T, CHANNELS> >(image.width(), image.height())
      {
	for (int y = 0; y < image.height(); ++y) 
          for (int x = 0; x < image.width(); ++x) {
            pixel_t pixel = image.getPixel(x, y);
            ValuatedImage<pixel_t>::setPixel(x, y, pixel);
	  }

      }


    explicit GenericImage(const girl::tools::Image &image)
      : ValuatedImage<girlExt::vector<T, CHANNELS> >(image.width(), image.height())
      {
        for (int y = 0; y < image.height(); ++y) {
          for (int x = 0; x < image.width(); ++x) {
            pixel_t pixel;
            for (size_t c = 0; c < CHANNELS; ++c)
              pixel[c] = image.getPixel(x, y)[c];
            ValuatedImage<pixel_t>::setPixel(x, y, pixel);
          }
        }
      }

    explicit GenericImage(const girl::tools::ImageFeature &imageFeature)
      : ValuatedImage<girlExt::vector<T, CHANNELS> >(imageFeature.width(), imageFeature.height())
      {
	for (int y = 0; y < imageFeature.height(); ++y) {
	  const girl::tools::ImageFeature::type *buf = imageFeature.scanLine(y);
          for (int x = 0; x < imageFeature.width(); ++x) {
            pixel_t pixel;
            for (size_t c = 0; c < CHANNELS; ++c)
              pixel[c] = buf[c];
            ValuatedImage<pixel_t>::setPixel(x, y, pixel);

	    buf += imageFeature.formatSize();
          }
        }
      }

    GenericImage(const girl::tools::Image &image, int colorSpace)
      : ValuatedImage<girlExt::vector<T, CHANNELS> >(image.width(), image.height())
      {
        for (int y = 0; y < image.height(); ++y) {
            for (int x = 0; x < image.width(); ++x) {
                pixel_t newPixel;
                const girl::tools::PixelFromImage<girl::tools::Image::type> rgbPixel = image.getPixel(x, y);
                for (size_t c = 0; c < CHANNELS; ++c)
                  newPixel[c] = conversion::changeSpace(colorSpace)[c](rgbPixel[0], rgbPixel[1], rgbPixel[2]);
		
                ValuatedImage<pixel_t>::setPixel(x, y, newPixel);
              }
          }
      }

    void toGirlToolsImage(girl::tools::Image &image)
    {
      assert(CHANNELS <= static_cast<size_t>(image.channels()) );
      assert(ValuatedImage<pixel_t>::width() == image.width() && ValuatedImage<pixel_t>::height() == image.height());
      int min = (CHANNELS < static_cast<size_t>(image.channels()) ? static_cast<int>(CHANNELS) : image.channels());

      for (int i = 0 ; i < ValuatedImage<pixel_t>::width() ; ++i) {
	for (int j = 0 ; j < ValuatedImage<pixel_t>::height() ; ++j) {
	  std::vector<girl::tools::Image::type> v(4, 0);
	  for (int c = 0 ; c < min ; ++c)
	    v[c] = ValuatedImage<pixel_t>::getPixel(i, j)[c];
	  girl::tools::PixelStandAlone<girl::tools::Image::type> p(v);
	  image.setPixel(i, j, p);
	}
      }
    }
  };
}

#endif /* ISEGENERICIMAGE_HPP */
