/*
  Copyright (C) 2013 Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "iseImageFeatureFromCvMat.hpp"

//parallel_for
#if CV_MAJOR_VERSION*100+CV_MINOR_VERSION*10+CV_SUBMINOR_VERSION >= 243
#include <opencv2/core/core.hpp>
#elif CV_MAJOR_VERSION*100+CV_MINOR_VERSION*10+CV_SUBMINOR_VERSION >= 240
#include <opencv2/core/internal.hpp>
#endif



using namespace ise;


ImageFeatureFromCvMat::ImageFeatureFromCvMat(ise::SegmentationContext *segC)
  : ImageFeatureColorimetricComputer(segC)
{
}

const std::string
ImageFeatureFromCvMat::baseUri() const
{
  return "Mat";
}

const std::string
ImageFeatureFromCvMat::uri() const
{
  std::stringstream ss(std::ios::in|std::ios::out);
  ss<<baseUri()<< IMAGE_FEATURE_URI_SEPARATOR;
  return ss.str();
}

void
ImageFeatureFromCvMat::compute()
{

}


template<typename T>
void
transfer(const cv::Mat &m, girl::tools::ImageFeature &imgF)
{
  const int h = m.rows;
  const int w = m.cols;

  const int nChan = m.channels();
  assert(nChan == imgF.channels());

  for (int i=0; i<h; ++i) {
    const T *mp = m.ptr<T>(i);
    girl::tools::ImageFeature::type *imgFp = imgF.scanLine(i);
    for (int j=0; j<w; ++j) {
      for (int k=0; k<nChan; ++k) {
	*imgFp = static_cast<girl::tools::ImageFeature::type>(*mp);
	++mp;
	++imgFp;
      }
    }
  }
    
}

size_t
ImageFeatureFromCvMat::getFeatureDim() const
{
  if (! m_imgF)
    return 0;
  return m_imgF->channels();
}

void
ImageFeatureFromCvMat::compute(const cv::Mat &m)
{
  int width = m.cols;
  int height = m.rows;
  int dim = m.channels();

  m_imgF = girl::shared_ptr<girl::tools::ImageFeature>(new girl::tools::ImageFeature(width, height, dim));
  
  girl::tools::ImageFeature &imgF = *(m_imgF.get());

  int depth = m.depth();
  switch (depth) {
  case CV_8U:
    transfer<unsigned char>(m, imgF);
    break;
  case CV_8S:
    transfer<char>(m, imgF);
    break;
  case CV_16U:
    transfer<unsigned short>(m, imgF);
    break;
  case CV_16S:
    transfer<short>(m, imgF);
    break;
  case CV_32S:
    transfer<int>(m, imgF);
    break;
  case CV_32F:
    transfer<float>(m, imgF);
    break;
  case CV_64F:
    transfer<double>(m, imgF);
    break;
  default:
    std::cerr<<"ERROR: ImageFeatureFromCvMat::compute input matrix type is not handled \n";
    break;
  }

}

/*
void
ImageFeatureFromCvMat::setParameters(int colorSpace)
{
  m_colorSpace = colorSpace;
}

void
ImageFeatureFromCvMat::getParameters(int &colorSpace) const
{
  colorSpace = m_colorSpace;
}
*/

void 
ImageFeatureFromCvMat::getDimDescription(std::vector<std::string> &dimDesc) const
{
  dimDesc.clear();
  for (size_t i = 0 ; i < getFeatureDim() ; ++i) {
    std::stringstream ss;
    ss<<"Mat "<<i;
    dimDesc.push_back(ss.str());
  }
}

