/*
  Copyright (C) 2013 Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISEIMAGEFEATURECOLORSPACE_HPP
#define ISEIMAGEFEATURECOLORSPACE_HPP

#include <vector>

#include <girl/girlCommon.hpp>
#include <girlTools/girlToolsImageFeature.hpp>

#include "iseFactoryTemplate.hxx"
#include "iseImageFeatureColorimetricComputer.hpp"
#include "iseSegmentationContext.hpp"


namespace ise {

  class GIRL_API ImageFeatureFromCvMat : public ise::ImageFeatureColorimetricComputer
  {
  public:

    explicit ImageFeatureFromCvMat(ise::SegmentationContext *segC);
    
    virtual ~ImageFeatureFromCvMat()
    {
    }

    virtual void compute();
    
    void compute(const cv::Mat &m);

    //void setParameters(int colorSpace);

    //void getParameters(int &colorSpace) const;

    virtual const std::string uri() const;

    virtual const std::string baseUri() const;

    /**
     * @brief return a description of feature : "Mat channel"
     * ex : "Mat 1"
     */ 
    void getDimDescription(std::vector<std::string> &dimDesc) const;

    size_t getFeatureDim() const;

    void getRequiredComputers(FeatureComputerListType&) const {}

  private:
    ImageFeatureFromCvMat(const ImageFeatureFromCvMat&);
    ImageFeatureFromCvMat &operator=(const ImageFeatureFromCvMat&);

  private:
    
  };

  typedef FactoryTemplate<ImageFeatureComputer, ImageFeatureFromCvMat> ImageFeatureFromCvMatFactory;

}
#endif /* ! ISEIMAGEFEATURECOLORSPACE_HPP */
