#ifndef ISEIMAGEFEATUREGABORFILTERS_HPP
#define ISEIMAGEFEATUREGABORFILTERS_HPP

#include <vector>

#include <girl/girlCommon.hpp>
#include <girlTools/girlToolsImageFeature.hpp>

#include "iseFactoryTemplate.hxx"
#include "iseImageFeatureTextureComputer.hpp"
#include "iseSegmentationContext.hpp"

namespace ise
{
/**
  * Compute the
  */
class GIRL_API ImageFeatureGaborFilters : public ise::ImageFeatureTextureComputer
{
protected:
    static const double DEFAULT_SIGMA [5];
    static const double DEFAULT_THETA [6];
    static const double DEFAULT_LAMBDA [1];
    static const double DEFAULT_GAMMA [1];
    static const double DEFAULT_PSI [1];


    static const int DEFAULT_FILTER_SIZE;
    static const bool DEFAULT_IMAGE_NORMALIZATION;
    static const bool DEFAULT_KERNEL_NORMALIZATION;
    static const bool DEFAULT_CONTRAST_NORMALIZATION;

 public:

    static void
    getDefaultParameters(int &size,
			 std::vector<double> &sigmas,
			 std::vector<double> &thetas,
			 std::vector<double> &lambas,
			 std::vector<double> &gammas,
			 std::vector<double> &psis,
			 bool &normalizeImage,
			 bool &normalizeKernel,
			 bool &normalizeContrast);
    

public:
    explicit ImageFeatureGaborFilters(ise::SegmentationContext *segContext);

    ~ImageFeatureGaborFilters();

    void
    compute();

    void
    setParameters(const int size,
                  const std::vector<double> &sigmas,
                  const std::vector<double> &thetas,
                  const std::vector<double> &lambas,
                  const std::vector<double> &gammas,
                  const std::vector<double> &psis,
                  bool normalizeImage,
                  bool normalizeKernel,
                  bool normalizeContrast);

    void
    getParameters(int &size,
                  std::vector<double> &sigmas,
                  std::vector<double> &thetas,
                  std::vector<double> &lambas,
                  std::vector<double> &gammas,
                  std::vector<double> &psis,
                  bool &normalizeImage,
                  bool &normalizeKernel,
                  bool &normalizeContrast);

    const std::string
    baseUri() const;

    const std::string
    uri() const;

    void
    getDimDescription(std::vector<std::string> &dimDesc) const;

    size_t
    getFeatureDim() const;

    void
    getRequiredComputers(FeatureComputerListType &computers) const;


protected:

    void
    getGaborKernels(std::vector<cv::Mat> &kernels);

    void
    normalizeImage(const cv::Mat &image, cv::Mat &normalized);

    void
    filterImage(const cv::Mat &image, const std::vector<cv::Mat> &filterBank, cv::Mat &output);

    cv::Mat
    getGaborKernel(cv::Size ksize, double sigma, double theta,
                   double lambd, double gamma, double psi, int ktype=CV_64F);

    void
    normalizeKernel(cv::Mat &kernel);

    void
    weberNormalization(double* buffer, int dim);

private:

    typedef std::vector<double> GaborParametersContainer;
    GaborParametersContainer m_sigmas;
    GaborParametersContainer m_thetas;
    GaborParametersContainer m_lambdas;
    GaborParametersContainer m_gammas;
    GaborParametersContainer m_psis;

    int m_filterSize;
    bool m_normalizeImage;
    bool m_normalizeKernel;
    bool m_contrastNormalisation;

};

typedef FactoryTemplate<ImageFeatureComputer, ImageFeatureGaborFilters> ImageFeatureGaborFiltersFactory;
}

#endif // ISEIMAGEFEATUREGABORFILTERS_HPP
