/*
  Copyright (C) 2005-2010 Vieux Remi, Roigt Julian, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "iseImageFeatureGlobal.hpp"
#include <cstring>
#include "iseGaussianPyramid.hpp"

#define SWAP(a, b) tempr=(a); (a)=(b); (b)=tempr; 

using namespace ise;

#define NB_FEATURE 5
#define WINDOW_SIZE 32

ImageFeatureGlobal::ImageFeatureGlobal(ise::SegmentationContext *segC)
  : ImageFeatureCompositionComputer(segC)
{
}

const std::string
ImageFeatureGlobal::baseUri() const
{
  return "Global";
}

const std::string
ImageFeatureGlobal::uri() const 
{
  std::stringstream ss(std::ios::in|std::ios::out);
  ss << baseUri() << IMAGE_FEATURE_URI_SEPARATOR;
  size_t size = m_algos.size();
  for (size_t i = 0 ; i < size ; ++i)
    ss <<  &m_algos[i] << IMAGE_FEATURE_URI_SEPARATOR;
  return ss.str();
}

void
ImageFeatureGlobal::compute()
{
  assert(m_segContext);
  
  const int width = m_segContext->getImage()->width();
  const int height = m_segContext->getImage()->height();

  std::vector<girl::shared_ptr<girl::tools::ImageFeature> > imgFs;
  size_t size = m_algos.size();

  size_t channels = 0;

  for (size_t i = 0 ; i < size ; ++i) {
    if ( m_algos[i] ) {
      imgFs.push_back(m_algos[i]->getImageFeature());
      channels += imgFs[i]->channels();
    }
  }
  
  m_imgF = girl::shared_ptr<girl::tools::ImageFeature>(new girl::tools::ImageFeature(width, height, channels));

  int pos = 0;
  int globalPos = 0;
  channels = 0;
  girl::tools::ImageFeature::type *bufDest;
  const girl::tools::ImageFeature::type *bufSrc;
  for (size_t i = 0 ; i < size ; ++i) {
    channels = imgFs[i]->channels();

    for (int y = 0 ; y < height ; ++y) {

      bufDest = m_imgF->scanLine(y);
      bufSrc = imgFs[i]->scanLine(y);

      for (int x = 0 ; x < width ; ++x) {
	pos  = globalPos;

	for (size_t k = 0 ; k < channels ; ++k, ++pos) {
	  bufDest[pos] = bufSrc[k];
	}

	bufDest += m_imgF->formatSize();
	bufSrc += imgFs[i]->formatSize();
      }
    }
    
    globalPos += channels;
  }

}

void
ImageFeatureGlobal::setParameters(const std::vector<girl::shared_ptr<ImageFeatureComputer> > &algos)
{
  m_algos = algos;
}

void
ImageFeatureGlobal::getParameters(std::vector<girl::shared_ptr<ImageFeatureComputer> > &algos) const
{
  algos = m_algos;
}

void
ImageFeatureGlobal::getDimDescription(std::vector<std::string> &dimDesc) const
{
  std::vector<std::string> dims;
  dimDesc.clear();
  for (size_t i = 0 ; i < m_algos.size() ; ++i) {
    assert(m_algos[i].get() != NULL);
    m_algos[i]->getDimDescription(dims);
    size_t size = dims.size();
    for (size_t j = 0 ; j < size ; ++j)
      dimDesc.push_back(dims[j]);
  }
}

size_t
ImageFeatureGlobal::getFeatureDim() const
{
  int dim = 0;
  for (size_t i = 0 ; i < m_algos.size() ; ++i) {
     dim += m_algos[i]->getFeatureDim();
  }
  return dim;
}
