/*
  Copyright (C) 2005-2010 Vieux Remi, Roigt Julian, Boris Mansencal  boris.mansencal@labri.fr

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef ISEIMAGEFEATUREGLOBAL_HPP
#define ISEIMAGEFEATUREGLOBAL_HPP

#include <girl/girlCommon.hpp>
#include <girlTools/girlToolsImage.hpp>
#include <girlTools/girlToolsImageFeature.hpp>

#include "iseFactoryTemplate.hxx"
#include "iseSegmentationContext.hpp"
#include "iseImageFeatureCompositionComputer.hpp"


namespace ise {

  class GIRL_API ImageFeatureGlobal : public ImageFeatureCompositionComputer {
  public:
    explicit ImageFeatureGlobal(ise::SegmentationContext *segC);

    virtual ~ImageFeatureGlobal()
    {

    }

    virtual void compute();

    virtual void setParameters(const std::vector<girl::shared_ptr<ImageFeatureComputer> > &algos);

    virtual void getParameters(std::vector<girl::shared_ptr<ImageFeatureComputer> > &algos) const;

    virtual const std::string baseUri() const;

    virtual const std::string uri() const;

    virtual const std::string imageFeatureComputerUri() const {
      return "Global";
    }

    virtual void getDimDescription(std::vector<std::string> &dimDesc) const;

    size_t
    getFeatureDim() const;

    void
    getRequiredComputers(FeatureComputer::FeatureComputerListType &req) const
    {
      std::vector<girl::shared_ptr<ImageFeatureComputer> >::const_iterator it = m_algos.begin();
      const std::vector<girl::shared_ptr<ImageFeatureComputer> >::const_iterator itEnd = m_algos.end();
      for (; it != itEnd; ++it)
        req.insert( (*it)->uri() );
    }

  };

  typedef FactoryTemplate<ImageFeatureComputer, ImageFeatureGlobal> ImageFeatureGlobalFactory;
}
#endif /* ! ISEIMAGEFEATUREGLOBAL_HPP */
