/*
  Copyright (C) 2005-2011 Vieux Remi, Roigt Julian, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "iseImageFeatureGradient.hpp"

#include <cstring>
#include "iseGaussianPyramid.hpp"

#include <opencv2/imgproc/imgproc.hpp>

using namespace ise;

ImageFeatureGradient::ImageFeatureGradient(ise::SegmentationContext *segC)
  : ImageFeatureGeometricComputer(segC)
  , m_algo()
  , m_orientation(0)
  , m_weightedChannelsCoeffs()
{
}

const std::string
ImageFeatureGradient::baseUri() const
{
  return "Gradient";
}

const std::string
ImageFeatureGradient::uri() const 
{
  std::stringstream ss(std::ios::in|std::ios::out);
  ss<<baseUri()<<IMAGE_FEATURE_URI_SEPARATOR<<&m_algo<<IMAGE_FEATURE_URI_SEPARATOR<<m_orientation;
  return ss.str();
}

void
ImageFeatureGradient::compute()
{
  assert(m_segContext);
  assert(m_weightedChannelsCoeffs.size() == m_algo->getFeatureDim());//m_segContext->getImageFeature(m_uri)->channels());
  //assert(m_segContext->getImageFeature(m_uri) != NULL);

  const int width = m_segContext->getImage()->width();
  const int height = m_segContext->getImage()->height();

  const girl::shared_ptr<girl::tools::ImageFeature> imgF = m_algo->getImageFeature();//m_segContext->getImageFeature(m_uri);

  m_imgF = girl::shared_ptr<girl::tools::ImageFeature>(new girl::tools::ImageFeature(width, height, imgF->channels()));

  if (m_orientation == 0)
    cv::Sobel(*(imgF->cvMat()), *(m_imgF->cvMat()), -1, 1, 0);
  else
    cv::Sobel(*(imgF->cvMat()), *(m_imgF->cvMat()), -1, 0, 1);
  
  for (int y = 0 ; y < height ; ++y) {
    girl::tools::ImageFeature::type *buf = m_imgF->scanLine(y);
    for (int x = 0 ; x < width ; ++x) {
      
      for (int c = 0 ; c < m_imgF->channels() ; ++c) {
  	buf[c] *= m_weightedChannelsCoeffs[c];
      }

      buf += m_imgF->formatSize();
    }
  }
    
}

void
ImageFeatureGradient::setParameters(girl::shared_ptr<ise::ImageFeatureComputer> algo, const std::vector<double> &weightedChannelsCoeffs, int orientation)
{
  m_algo = algo;
  m_orientation = orientation;
  m_weightedChannelsCoeffs = weightedChannelsCoeffs;
}

void
ImageFeatureGradient::getParameters(girl::shared_ptr<ise::ImageFeatureComputer> algo, std::vector<double> &weightedChannelsCoeffs, int &orientation) const
{
  algo = m_algo;
  orientation = m_orientation;
  weightedChannelsCoeffs = m_weightedChannelsCoeffs;
}

void 
ImageFeatureGradient::getDimDescription(std::vector<std::string> &dimDesc) const
{
  dimDesc.clear();
  for (size_t i = 0 ; i < getFeatureDim() ; ++i) {
    std::stringstream ss;
    ss <<"Gradient ";
    if ( m_orientation == 0) 
      ss << "X ";
    else
      ss << "Y ";
    ss << i;
    dimDesc.push_back(ss.str());
  }
}
