/*
  Copyright (C) 2005-2010 Vieux Remi, Roigt Julian, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef ISEIMAGEFEATUREGRADIENTMODULE_HPP
#define ISEIMAGEFEATUREGRADIENTMODULE_HPP

#include <girl/girlCommon.hpp>
#include <girlTools/girlToolsImage.hpp>
#include <girlTools/girlToolsImageFeature.hpp>

#include "iseFactoryTemplate.hxx"
#include "iseSegmentationContext.hpp"
#include "iseImageFeatureGeometricComputer.hpp"

namespace ise {
  
  class GIRL_API ImageFeatureGradientModule : public ImageFeatureGeometricComputer {
    
    static const size_t FEATURE_DIMENSION = 1;

  public:
    
    explicit ImageFeatureGradientModule(ise::SegmentationContext *segC);
    
    virtual ~ImageFeatureGradientModule()
    {
    }

    virtual void compute();
    
    virtual void setParameters(girl::shared_ptr<ise::ImageFeatureComputer> algo, const std::vector<double> &weightedChannelsCoeffs);

    virtual void getParameters(girl::shared_ptr<ise::ImageFeatureComputer> algo, std::vector<double> &weightedChannelsCoeffs) const;

    virtual const std::string baseUri() const;

    virtual const std::string uri() const;

    virtual void getDimDescription(std::vector<std::string> &dimDesc) const;

    size_t
    getFeatureDim() const { return FEATURE_DIMENSION; }

    void
    getRequiredComputers(FeatureComputer::FeatureComputerListType &req) const
    {
      req.insert(m_algo->uri());
    }


  private:
    
    girl::shared_ptr<ise::ImageFeatureComputer> m_algo;
    std::vector<double> m_weightedChannelsCoeffs;
  };

  typedef FactoryTemplate<ImageFeatureComputer, ImageFeatureGradientModule> ImageFeatureGradientModuleFactory;
}
#endif /* ! ISEIMAGEFEATUREGRADIENTMODULE_HPP */
