/*
  Copyright (C) 2005-2010 Vieux Remi, Roigt Julian, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "iseImageFeatureLinearCombination.hpp"
#include <cstring>
#include "iseGaussianPyramid.hpp"

using namespace ise;

ImageFeatureLinearCombination::ImageFeatureLinearCombination(ise::SegmentationContext *segC)
  : ImageFeatureCompositionComputer(segC)
  , m_weight(), m_dim(0)
{
}

const std::string
ImageFeatureLinearCombination::baseUri() const
{
  return "LinearCombination";
}

const std::string
ImageFeatureLinearCombination::uri() const 
{
  std::stringstream ss(std::ios::in|std::ios::out);
  ss << baseUri()<<IMAGE_FEATURE_URI_SEPARATOR<<m_dim;
  for (size_t i = 0 ; i < m_algos.size() ; ++i)
    ss<<IMAGE_FEATURE_URI_SEPARATOR<<&m_algos[i]<<IMAGE_FEATURE_URI_SEPARATOR<<m_weight[i];
  return ss.str();
}

void
ImageFeatureLinearCombination::compute()
{
  assert(m_segContext);
  
  const int width = m_segContext->getImage()->width();
  const int height = m_segContext->getImage()->height();
  const size_t sizeUri = m_algos.size();

  std::vector<girl::shared_ptr<girl::tools::ImageFeature> > vimgF;
  for (size_t i = 0 ; i < sizeUri ; ++i) 
    vimgF.push_back(m_algos[i]->getImageFeature());
  

  m_imgF = girl::shared_ptr<girl::tools::ImageFeature>(new girl::tools::ImageFeature(width, height, m_dim));
  
  girl::tools::ImageFeature::type *bufDest;
  std::vector<const girl::tools::ImageFeature::type*> bufsSrc(sizeUri);
  for (int y = 0 ; y < height ; ++y) {
    
    bufDest = m_imgF->scanLine(y);
    for (size_t i = 0 ; i < sizeUri ; ++i)
      bufsSrc[i] = vimgF[i]->scanLine(y);

    for (int x = 0 ; x < width ; ++x) {
      for (size_t j = 0 ; j < m_dim; ++j) {
	double sum = 0.0;
	for (size_t i = 0 ; i < sizeUri ; ++i) {
	  sum += bufsSrc[i][j] * m_weight[i];
	}
	bufDest[j] = sum;
      }
      bufDest += m_imgF->formatSize();
      for (size_t i = 0 ; i < sizeUri ; ++i) 
	bufsSrc[i] += vimgF[i]->formatSize();
    }
  }
}

void
ImageFeatureLinearCombination::setParameters(const std::vector<girl::shared_ptr<ImageFeatureComputer> > &algos, const std::vector<double> &weight, int dim)
{
  m_algos = algos;
  m_weight = weight;
  m_dim = dim;
}

void
ImageFeatureLinearCombination::getParameters(std::vector<girl::shared_ptr<ImageFeatureComputer> > &algos, std::vector<double> &weight, int &dim) const
{
  algos = m_algos;
  weight = m_weight;
  dim = m_dim;
}

void
ImageFeatureLinearCombination::getDimDescription(std::vector<std::string> &dimDesc) const
{
  std::vector<std::string> dims;
  assert(m_algos.size() > 0);

  assert(m_algos[0].get() != NULL);
  m_algos[0]->getDimDescription(dims);
  size_t size = dims.size();
  
  for (size_t j = 0 ; j < size ; ++j)
    dimDesc.push_back("LinearCombi");
}
