/*
  Copyright (C) 2005-2010 Vieux Remi, Roigt Julian, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef ISEIMAGEFEATURELINEARCOMBINATION_HPP
#define ISEIMAGEFEATURELINEARCOMBINATION_HPP

#include <girl/girlCommon.hpp>
#include <girlTools/girlToolsImage.hpp>
#include <girlTools/girlToolsImageFeature.hpp>

#include "iseFactoryTemplate.hxx"
#include "iseSegmentationContext.hpp"
#include "iseImageFeatureCompositionComputer.hpp"

namespace ise {

  class GIRL_API ImageFeatureLinearCombination : public ImageFeatureCompositionComputer {
  public:
    explicit ImageFeatureLinearCombination(ise::SegmentationContext *segC);
    
    virtual ~ImageFeatureLinearCombination()
    {

    }

    virtual void compute();
    
    virtual void setParameters(const std::vector<girl::shared_ptr<ImageFeatureComputer> > &titles, const std::vector<double> &weight, int dim);

    virtual void getParameters(std::vector<girl::shared_ptr<ImageFeatureComputer> > &titles, std::vector<double> &weight, int &dim) const;

    virtual const std::string baseUri() const;

    virtual const std::string uri() const;

    virtual const std::string imageFeatureComputerUri() const {
      return "LinearCombination";
    }
    
    virtual void getDimDescription(std::vector<std::string> &dimDesc) const;

   size_t
   getFeatureDim() const
    {
      return m_dim;
    }

   void
   getRequiredComputers(FeatureComputer::FeatureComputerListType &req) const
   {
     std::vector<girl::shared_ptr<ImageFeatureComputer> >::const_iterator it = m_algos.begin();
     const std::vector<girl::shared_ptr<ImageFeatureComputer> >::const_iterator itEnd = m_algos.end();
     for (; it != itEnd; ++it)
       req.insert((*it)->uri());
   }


  private:
    
    std::vector<double> m_weight;
    size_t m_dim;
  };

  typedef FactoryTemplate<ImageFeatureComputer, ImageFeatureLinearCombination> ImageFeatureLinearCombinationFactory;
}
#endif /* ! ISEIMAGEFEATURELINEARCOMBINATION_HPP */
