/*
  Copyright (C) 2011 Remi Vieux  remi.vieux@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef ISEIMAGEFEATURELOCALBINARYPATTERNS_HPP_
#define ISEIMAGEFEATURELOCALBINARYPATTERNS_HPP_

#include <vector>

#include <girl/girlCommon.hpp>
#include <girl/girlSharedPtr.hpp>

#include <girlTools/girlToolsImageFeature.hpp>

#include <opencv2/core/core.hpp>

#include "iseFactoryTemplate.hxx"
#include "iseImageFeatureTextureComputer.hpp"


/**
 * Implements
 *  title = {Multiresolution Gray-Scale and Rotation Invariant Texture Classification
 *  with Local Binary Patterns},
 * journal = {IEEE PAMI},
 * year = {2002},
 * volume = {24},
 * pages = {971-987},
 */
namespace ise
{

  class GIRL_API ImageFeatureLocalBinaryPatterns : public ImageFeatureTextureComputer
  {

    typedef girl::shared_ptr<girl::tools::ImageFeature> ImageFeaturePtr;

  public:

    static const size_t FEATURE_SIZE   = 1;
    static const double DEFAULT_RADIUS;
    static const size_t DEFAULT_NB_PTS = 8;
    static const size_t DEFAULT_CHAN   = 0;

  public:
    explicit ImageFeatureLocalBinaryPatterns(ise::SegmentationContext *segContext);
    virtual
    ~ImageFeatureLocalBinaryPatterns();

    void
    compute();

    void
    getDimDescription(std::vector<std::string> &dimDesc) const;

    size_t
    getFeatureDim() const;

    const std::string
    help() const;

    const std::string
    uri() const;

    void
    setParameters(ImageFeaturePtr img, size_t chan, double radius, size_t nbPoints);

    void
    getParameters(ImageFeaturePtr &img, size_t &chan, double &radius, size_t &nbPoints) const;

    void
    getRequiredComputers(FeatureComputerListType &) const
    {
      // TODO:UNSURE We actually rely on another ImageFeature but I passed directly
      // the girl::tools::ImageFeature instead of the ImageFeatureComputer as parameter
      // so the dependancy is not achieved.
    }

    size_t
    getNbDifferentPatterns() const;


    // TODO:UNSURE What is baseUri for?? Is it even used??
    const std::string
    baseUri() const
    {
      return "";
    }


  private:

    void
    computePointCoordinates();

    size_t
    computeUniformity(const std::vector<bool> &pattern, size_t &nonZeros) const;


  private:

    double m_radius;
    size_t m_nbPoints;
    size_t m_chan;

    std::vector<cv::Point2d> m_ptsCoordinates;

    ImageFeaturePtr m_img;

  };

  typedef FactoryTemplate<ImageFeatureComputer, ImageFeatureLocalBinaryPatterns> ImageFeatureLocalBinaryPatternsFactory;
}

#endif /* ISEIMAGEFEATURELOCALBINARYPATTERNS_HPP_ */
