/*
  Copyright (C) 2005-2011 Vieux Remi, Roigt Julian, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISEIMAGEFEATUREORIENTATION_HPP
#define ISEIMAGEFEATUREORIENTATION_HPP

#include <girl/girlCommon.hpp>
#include <girlTools/girlToolsImage.hpp>
#include <girlTools/girlToolsImageFeature.hpp>

#include "iseFactoryTemplate.hxx"
#include "iseSegmentationContext.hpp"
#include "iseImageFeatureTextureComputer.hpp"

#include <fftw3.h>
#include <opencv2/core/core.hpp>


namespace ise {
  class Boxe;

  class GIRL_API ImageFeatureOrientation : public ise::ImageFeatureTextureComputer {
  public:
    static const size_t NB_FEATURE = 5;
    static const int WINDOW_SIZE = 32;
    static const int PYRAMID_DIVIDER = 2;

    explicit ImageFeatureOrientation(ise::SegmentationContext *segC);
    
    virtual ~ImageFeatureOrientation()
    {
    }

    virtual void compute();
    
    virtual void setParameters(int numberLevel);

    virtual void getParameters(int &numberLevel) const;

    virtual const std::string uri() const;

    virtual const std::string baseUri() const;

    void
    getDimDescription(std::vector<std::string> &dimDesc) const;

    size_t
    getFeatureDim() const { return m_numberLevel * NB_FEATURE; }

    void
    getRequiredComputers(FeatureComputer::FeatureComputerListType &) const
    {
      // TODO enable computation with any image feature
    }


  private:

    void ComputeFeature(cv::Mat *img, int xCenter, int yCenter, int size);

    ImageFeatureOrientation(const ImageFeatureOrientation &);
    ImageFeatureOrientation &operator=(const ImageFeatureOrientation &);
    
    void computePlancherel(fftw_complex* frequency_repr, int width, int height);
    void computeRose(fftw_complex* frequency_repr, int width, int height);
    void computeTransitionFrequency(cv::Mat *img, int xCenter, int yCenter, int size);

    double Variance(double tab[], int size);

    int computeMeanBorder(cv::Mat *img, int xCenter, int yCenter, int size);
    
    void setMeanColRowOnImage(const ise::Boxe &b, girl::shared_ptr<girl::tools::ImageFeature> &grayImage, girl::tools::ImageFeature &result);
    
    void setMeanColRowInBoxe(const ise::Boxe &b, girl::shared_ptr<girl::tools::ImageFeature> &grayImage, girl::tools::ImageFeature &result);

  private:
    
    int m_width;
    int m_height;
    double m_intensity;
    double m_angle;
    double m_std;
    double m_transitionFrequency;
    size_t m_numberLevel;
  };

  typedef FactoryTemplate<ImageFeatureComputer, ImageFeatureOrientation> ImageFeatureOrientationFactory;
}
#endif /* ! ISEIMAGEFEATUREORIENTATION_HPP */
