#ifndef ISEIMAGEFEATUREPIXELCLASSIFICATION_HPP
#define ISEIMAGEFEATUREPIXELCLASSIFICATION_HPP

#include <girl/girlCommon.hpp>
#include <girlTools/girlToolsImageFeature.hpp>

#include "iseFactoryTemplate.hxx"
#include "iseImageFeatureTextureComputer.hpp"
#include "KMeansKnnClassifier.hpp"

namespace ise
{

class GIRL_API ImageFeaturePixelClassification : public ise::ImageFeatureTextureComputer
{
public:
    explicit ImageFeaturePixelClassification(ise::SegmentationContext *segContext);
    ~ImageFeaturePixelClassification();

    const std::string
    baseUri() const;

    const std::string
    uri() const;

    void
    compute();

    size_t
    getFeatureDim() const;

    void
    getDimDescription(std::vector<std::string> &dimDesc) const;

    void
    getRequiredComputers(FeatureComputerListType &computers) const;

    const std::string
    help() const;

    void
    setParameters(const char* model, girl::shared_ptr<ise::ImageFeatureComputer> gabor);

    void
    getParameters(const char* &model, girl::shared_ptr<ise::ImageFeatureComputer> &gabor);


private:
    void
    loadModel();

private:

    girl::shared_ptr<ise::ImageFeatureComputer> m_gaborImage;
    KMeansKnnClassifier m_classifier;
    const char* m_modelName;

};

typedef FactoryTemplate<ImageFeatureComputer, ImageFeaturePixelClassification> ImageFeaturePixelClassificationFactory;

}
#endif // ISEIMAGEFEATUREPIXELCLASSIFICATION_HPP
