#ifndef ISEIMAGEFEATURERELAXATION_HPP
#define ISEIMAGEFEATURERELAXATION_HPP


#include <girl/girlCommon.hpp>
#include <girlTools/girlToolsImageFeature.hpp>

#include "iseFactoryTemplate.hxx"
#include "iseImageFeatureTextureComputer.hpp"
#include "iseSegmentationContext.hpp"

namespace ise
{

class GIRL_API ImageFeatureRelaxation : public ImageFeatureTextureComputer
{
public:
    static const int DEFAULT_CONNEXITY = 4;
    static const bool DEFAULT_UNIFORM_WEIGHT = true;

public:
    explicit ImageFeatureRelaxation(ise::SegmentationContext *segContext);

    ~ImageFeatureRelaxation();

    const std::string
    baseUri() const;

    const std::string
    uri() const;

    void
    compute();

    size_t
    getFeatureDim() const;

    void
    setParameters(girl::shared_ptr<ise::ImageFeatureComputer> classif,
                  int connexity,
		  bool uniformWeight);

    void
    getParameters(girl::shared_ptr<ise::ImageFeatureComputer> classif,
                  int &connexity,
		  bool &uniformWeight);

    const std::string
    help() const;

    void
    getDimDescription(std::vector<std::string> &dimDesc) const;

    void
    getRequiredComputers(FeatureComputerListType &computers) const;


private:

    void
    setNeighbours(std::vector<cv::Point> &ng);

    void
    computeValidNeighbours(const cv::Mat &myMat,
                           const cv::Point &current,
                           const std::vector<cv::Point> &ng,
                           std::vector<cv::Point> &validNg);

    void
      computeWeights(const cv::Point &current,
		     const std::vector<cv::Point> &ng,
		     std::vector<girl::tools::ImageFeature::type> &weights);
    
    size_t
      pixToIndex(const cv::Mat &mat, const cv::Point &pt);
    
 private:

     girl::shared_ptr<ise::ImageFeatureComputer> m_pixelClassif;
     int m_connexity;
     bool m_uniformWeights;
};

typedef FactoryTemplate<ImageFeatureComputer, ImageFeatureRelaxation> ImageFeatureRelaxationFactory;

}
#endif // ISEIMAGEFEATURERELAXATION_HPP
