/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
#ifndef ISEIMAGEFEATURERUNLENGTH_HPP_
#define ISEIMAGEFEATURERUNLENGTH_HPP_

#include <vector>

#include <opencv2/core/core.hpp>

#include "iseImageFeatureComputer.hpp"
#include "iseSegmentationContext.hpp"
/**
 * Class to compute the run length features.
 */


namespace ise {


  class GIRL_API ImageFeatureRunLength : public ImageFeatureComputer
  {
  public:
    // Following Freeman's code
    typedef enum direction {
      NORTH = 2,
      NORTH_EAST = 1,
      EAST = 0,
      SOUTH_EAST = 7,
      SOUTH = 6,
      SOUTH_WEST = 5,
      WEST = 4,
      NORTH_WEST = 3
    } direction;


  public:
    explicit ImageFeatureRunLength(ise::SegmentationContext *segContext);

    virtual ~ImageFeatureRunLength() {}

    virtual void compute();
    /**
     * Will compute the length of the segments of pixels of value @value given the direction @d
     */
    virtual void setParameters(double value , ise::ImageFeatureRunLength::direction d);
    virtual void getParameters(double &value, ise::ImageFeatureRunLength::direction &d) const;

    void
    getDimDescription(std::vector<std::string> &dimDesc) const;

  private:
    ImageFeatureRunLength(const ImageFeatureRunLength&);
    ImageFeatureRunLength& operator=(const ImageFeatureRunLength&);

  private:

    inline void getLeftPoints(std::vector<cv::Point>& pts, int /*width*/, int height) {
      cv::Point pt(0, 0);
      for (int i = 0; i < height; ++i) {
        pt.y = i;
        pts.push_back(pt);
      }
    }

    inline void getRightPoints(std::vector<cv::Point>& pts, int width, int height) {
      cv::Point pt(width - 1, 0);
      for (int i = 0; i < height; ++i) {
        pt.y = i;
        pts.push_back(pt);
      }
    }

    inline void getUpPoints(std::vector<cv::Point>& pts, int width, int /*height*/) {
      cv::Point pt(0, 0);
      for (int i = 0; i < width; ++i) {
        pt.x = i;
        pts.push_back(pt);
      }
    }

    inline void getDownPoints(std::vector<cv::Point>& pts, int width, int height) {
      cv::Point pt(0, height - 1);
      for (int i = 0; i < width; ++i) {
        pt.x = i;
        pts.push_back(pt);
      }
    }

    inline bool
    isInImageLimits(cv::Point pt, int width, int height)
    {
      return pt.x >= 0 && pt.x < width && pt.y >= 0 && pt.y < height;
    }

    virtual inline int getFeatureDim() { return 1; }

    void
    getRequiredComputers(std::vector<std::string> &) const
    {

    }



    private:

    double m_value;
    ise::ImageFeatureRunLength::direction m_direction;

  };

}

#endif /* ISEIMAGEFEATURERUNLENGTH_HPP_ */
