/*
  Templated Image class.
  Copyright (C) 2007-2009 Hugo boujut, Jean-Philippe Domenger, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISEVALUATEDIMAGE_HPP
#define ISEVALUATEDIMAGE_HPP

#include <cassert>
#include <girlTools/girlToolsVector.hpp>

namespace ise
{
  template <class T>
   class GIRL_API ValuatedImage
    {
    public :
      ValuatedImage(int w, int h)
	:
	m_valuation(new T[w*h]),
	m_width(w),
	m_height(h)
	{}

      virtual ~ValuatedImage() 
      {
	delete [] m_valuation;
      };

      virtual ValuatedImage<T> &operator=(const ValuatedImage<T> &valImg)
	{
	  delete [] m_valuation;
	  m_width = valImg.width();
	  m_height = valImg.height();

	  m_valuation = new T[m_width*m_height];
	  
	  for (int y = 0; y < m_height; ++y)
	    for (int x= 0; x < m_width; ++x)
	      setPixel(x, y, valImg.getPixel(x, y));
	  
	  return *this;
	};

      int width() const  {return  m_width;}
      int height() const {return  m_height;}

      const T &getPixel(int x, int y) const 
      {
	assert(x>=0 && x<m_width);
	assert(y>=0 && y<m_height);

	return m_valuation[ y * m_width + x];
      }

      void setPixel(int x, int y, const T &p) 
      { 
	assert(x>=0 && x<m_width);
	assert(y>=0 && y<m_height);

	m_valuation[ y * m_width + x] = p;
      }

      //    private:
      ValuatedImage(const ValuatedImage<T> &valImg);

    protected:
      T *m_valuation;
      int m_width;
      int m_height;
    };
}

#endif
