/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef ISEKERNELCAUCHY_HPP_
#define ISEKERNELCAUCHY_HPP_

#include <girl/girlCommon.hpp>

#include "iseKernelFunction.hpp"


namespace ise
{
  
  /**
   * Implements the cauchy kernel, cf publication
   * K.-L. Wu and M.-S. Yang, 
   * Mean shift-based clustering, 
   * Pattern Recognition, vol. 40, pp 3035-3052, 2007
   */
  
  template <typename T>
  class GIRL_API KernelCauchy : public KernelFunction<T>
  {
    
  public:
    
    KernelCauchy(double beta, double p) 
      : m_beta(beta),
      m_p(p)
      {}
    
    ~KernelCauchy() {}
    
    double operator()(const T* a, const T* b, size_t dim) const;
    
  private:
    
    double m_beta;
    double m_p;
    
  };

  
  template <typename T>
  double
  KernelCauchy<T>::operator()(const T* a, const T* b, size_t dim) const
  {
#if CV_MAJOR_VERSION*100+CV_MINOR_VERSION*10+CV_SUBMINOR_VERSION < 231
    const double edist = cvflann::euclidean_dist(a, a + dim, b);
#else
    const double edist = cvflann::L2<double>()(a, b, dim);
#endif
    const double dist = sqrt(edist);
    const double tmp  = 1/((1+dist)/m_beta);
    return pow(tmp, m_p);
  }
  
}
  
#endif // ISEKERNELCAUCHY_HPP_
