#ifndef ISELINEDETECTOR_HPP
#define ISELINEDETECTOR_HPP

#include <vector>
#include "iseTextLine.hpp"

namespace ise
{

#if 0
/**
  * Interface for algorithm for line detection
  * Pass to constructor the set of connected components
  * the must be grouped into lines, represented by their bounding boxes
  */

typedef std::vector<girl::tools::AABBox> BoxVectorType;

class LineDetector
{
public:
  LineDetector(const BoxVectorType *boxes)
    : m_boxes(boxes)
  {
    std::cout << "Creating LineDetector" << std::endl;
    assert(m_boxes != NULL);
  }

  virtual
  ~LineDetector() {}

  /**
      * Fill @detectedLines with the
      */
  virtual void
  detect(std::vector<TextLine> &detectedLines) = 0;


protected:

  const BoxVectorType *m_boxes;
};






#if 1
/**
  * Interface to a predicate that, given the indices of two connected components
  * must return TRUE is they are in the same group (e.g. same line), false if not or
  * not sure
  */
class SameLinePredicate {

public:

  SameLinePredicate(const BoxVectorType *boxes)
    : m_boxes(boxes)
  {
    assert(m_boxes != NULL);
  }
  virtual
  ~SameLinePredicate() {}

  virtual bool
  operator()(size_t a, size_t b) = 0;
 // {
 //    std::cout << "SameLinePredicate()" << std::endl;
 //    return false;
 //  }


protected:
  const BoxVectorType *m_boxes;
};


class SameLinePredicateYOverlapXDistanceAverageCharacterHeigth
    : public SameLinePredicate
{

public:

  SameLinePredicateYOverlapXDistanceAverageCharacterHeigth(const BoxVectorType *boxes,
                                                           float yOverlap = 0.5,
                                                           float maxXDist = std::numeric_limits<float>::max(),
                                                           float averageHeight = 10.0f) :
    SameLinePredicate(boxes),
    m_yOverlap(yOverlap),
    m_xDist(maxXDist),
    m_meanCharacterHeight(averageHeight)

  {}

  virtual bool
  operator()(size_t i, size_t j)
  {
    std::cout << "SameLinePredicateYOverlapXDistanceAverageCharacterHeigth()" << std::endl;
    assert(i < m_boxes->size());
    assert(j < m_boxes->size());
    const girl::tools::AABBox &a = (*m_boxes)[i];
    const girl::tools::AABBox &b = (*m_boxes)[j];
    float overlap = static_cast<float>(std::min(a.yMax(), b.yMax()) - std::max(a.yMin(), b.yMin())) + 1;
    int d = (std::min(a.xMax(), b.xMax()) - std::max(a.xMin(), b.xMin())) + 1;
    int dist = (d > 0) ? std::numeric_limits<int>::max() : -d;
    bool overlapOk = (overlap/a.height() > m_yOverlap) || (overlap/b.height() > m_yOverlap);
    bool distOk = dist < m_xDist;
    const float SIZETHRESH = 1.7;
    bool aSizeOk = a.height() < SIZETHRESH*m_meanCharacterHeight;
    bool bSizeOK = b.height() < SIZETHRESH*m_meanCharacterHeight;
    bool sizesOk =  aSizeOk && bSizeOK;

    if ((overlapOk && distOk && sizesOk) || (overlapOk && distOk && !aSizeOk && !bSizeOK)) {
      return true;
    }
    return false;
  }
private:

  float m_yOverlap;
  int m_xDist;
  float m_meanCharacterHeight;

};


// class LineDetectorDelegate : public LineDetector
// {

// public:

//   LineDetectorDelegate(BoxVectorType *boxes, SameLinePredicate &pred) :
//     LineDetector(boxes),
//     m_pred(pred)
//   {
//     std::cout << "Creating LineDetectorDelegate " << std::endl;
//   }

//   virtual
//   ~LineDetectorDelegate() {}


//   void
//   detect(std::vector<TextLine> &detectedLines)
//   {
//     assert(m_boxes != NULL);
//     size_t s = m_boxes->size();
//     std::vector<size_t> indices(s);
//     for (size_t i = 0; i < s; ++i) {
//       indices[i] = i;
//     }

//     std::vector<int> labels(s);
//     int nbLines = cv::partition(indices, labels, m_pred);

//     detectedLines.assign(nbLines, TextLine(m_boxes));
//     for (size_t i = 0; i < s; ++i) {
//       detectedLines[labels[i]].addBox(i);
//     }
//   }

// private:
//   SameLinePredicate &m_pred;

// };

#endif


// /**
//   * Implements a line detector that use cv::partition function
//   * which cluster together elements that satisfy a predicate
//   */
// class LineDetectorEquivalencyClasses : public LineDetector
// {

// public:

//   virtual
//   ~LineDetectorEquivalencyClasses() {}

//   virtual bool
//   operator()(size_t a, size_t b) { std::cout << "Pred superclass" << std::endl;
//     return false; }

//   void
//   detect(std::vector<TextLine> &detectedLines)
//   {
//     assert(m_boxes != NULL);
//     //assert(m_predicate != NULL);
//     size_t s = m_boxes->size();
//     std::vector<size_t> indices(s);
//     for (size_t i = 0; i < s; ++i) {
//       indices[i] = i;
//     }
    
//     std::vector<int> labels(s);
//     int nbLines = cv::partition(indices, labels, *this);
    
    
//     detectedLines.assign(nbLines, TextLine(m_boxes));
//     for (size_t i = 0; i < s; ++i) {
//       detectedLines[labels[i]].addBox(i);
//     }
//   }
  
// protected:
//   LineDetectorEquivalencyClasses(BoxVectorType *boxes) :
//     LineDetector(boxes)
//   {
//     std::cout << "Creating LineDetectorEquivalencyClasses " << std::endl;
    
//   }
  
// };

// class LineDetectorYOverlapXDistAverageHeight : public LineDetectorEquivalencyClasses
// {
// public:

//   LineDetectorYOverlapXDistAverageHeight(BoxVectorType *boxes,
//                                          float yOverlap,
//                                          float maxXDist,
//                                          float averageHeight) :
//     LineDetectorEquivalencyClasses(boxes),
//     m_yOverlap(yOverlap),
//     m_xDist(maxXDist),
//     m_meanCharacterHeight(averageHeight)
//   {

//     std::cout << "Creating LineDetectorYOverlapXDistAverageHeight" << std::endl;
// }


//   virtual bool
//   operator()(size_t i, size_t j)
//   {
//     std::cout << "Pred subclass" << std::endl;
//     assert(i < m_boxes->size());
//     assert(j < m_boxes->size());
//     const girl::tools::AABBox &a = (*m_boxes)[i];
//     const girl::tools::AABBox &b = (*m_boxes)[j];
//     float overlap = static_cast<float>(std::min(a.yMax(), b.yMax()) - std::max(a.yMin(), b.yMin())) + 1;
//     int d = (std::min(a.xMax(), b.xMax()) - std::max(a.xMin(), b.xMin())) + 1;
//     int dist = (d > 0) ? std::numeric_limits<int>::max() : -d;
//     bool overlapOk = (overlap/a.height() > m_yOverlap) || (overlap/b.height() > m_yOverlap);
//     bool distOk = dist < m_xDist;
//     const float SIZETHRESH = 1.7;
//     bool aSizeOk = a.height() < SIZETHRESH*m_meanCharacterHeight;
//     bool bSizeOK = b.height() < SIZETHRESH*m_meanCharacterHeight;
//     bool sizesOk =  aSizeOk && bSizeOK;

//     if ((overlapOk && distOk && sizesOk) || (overlapOk && distOk && !aSizeOk && !bSizeOK)) {
//       return true;
//     }
//     return false;
//   }


// private:

//   float m_yOverlap;
//   float m_xDist;
//   float m_meanCharacterHeight;

// };

#endif
}
#endif // ISELINEDETECTOR_HPP
