/*
  Copyright (C) 2005-2011 Remi vieux, Jean-Philippe Domenger, Achille Braquelaire, Jenny Benois-Pineau, Julian Roigt  remi.vieux@labri.fr

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
/*
  This work was (partly) funded by the X-Media project, sponsored by
  the European Commission as part of the Information Society Technologies
  (IST) program under EC grant number IST-FP6-026978.
*/
#ifndef ISEMATRIX_HPP_
#define ISEMATRIX_HPP_

#include <vector>
#include <iostream>
#include <fstream>
#include <sstream>
#include <cassert>
#include <numeric> // std::accumulate

namespace ise {

  template<typename T>
  class matrix : public std::vector<std::vector<T> >
  {
  public:

    matrix() : m_width(0), m_height(0) {}

    matrix(size_t numRow, size_t numCol)
      : std::vector<std::vector<T> >(numRow, std::vector<T>(numCol, 0)),
    m_width(numCol),
    m_height(numRow)
    {}

    virtual ~matrix() {}

    void initVal(T &val) {
     	typename matrix<T>::iterator it = this->begin();
     	const typename matrix<T>::const_iterator itEnd = this->end();
     	for ( ; it != itEnd; ++it) {
     		it->assign(it->size(), val);
     	}
     }

    /*
     * Use to access elements as a feature image.
     * x = horizontal = col
     * y = vertical = row
     * returns the feature (vector<T>) computed at pixel (x, y)
     */
    //template <typename T>
    const std::vector<T>& operator() (size_t x, size_t y) const
    {
      assert(m_width > 0 && m_height > 0);
      assert(m_width*m_height == this->size());
      assert(x < m_width && y < m_height);
      return (*this)[y*m_width + x];
    }

    size_t getWidth() const
    {
      return m_width;
    }

    void setWidth(size_t width)
    {
      m_width = width;
    }

    size_t getHeight() const
    {
      return m_height;
    }

    void setHeight(size_t height)
    {
      m_height = height;
    }

    void getSize(int &width, int &height)
    {
      width = m_width;
      height = m_height;
    }

    void setSize(int width, int height)
    {
      m_width = width;
      m_height = height;
    }

  private:
    size_t m_width;
    size_t m_height;

  };



}

#endif /* ISEMATRIX_HPP_ */
