/*
  3D Histogram implemented with OpenCV.
  Copyright (C) 2005-2009 Jean-Philippe Domenger, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "iseOpenCVHistogram.hpp"	       
#include "iseOpenCVHistogramIterator.hpp"

using namespace ise;


OpenCVHistogram::OpenCVHistogram(unsigned char c1Offset, unsigned char c2Offset, unsigned char c3Offset, CvHistType type)
  : AbstractHistogram(c1Offset, c2Offset, c3Offset), m_histoData(NULL)
{
  int dims[] = {256, 256, 256};
  float rRange[] = {0, 256 >> c1Offset};
  float gRange[] = {0, 256 >> c2Offset};
  float bRange[] = {0, 256 >> c3Offset};
  float *ranges[] = {gRange, bRange, rRange};
  m_histoData = cvCreateHist(3, dims, type, ranges);

}

OpenCVHistogram::~OpenCVHistogram()
{
  
  assert(m_histoData != NULL);
  cvReleaseHist(&m_histoData);
}

void 
OpenCVHistogram::addColor(unsigned char r, 
			  unsigned char g,
			  unsigned char b)
{
  unsigned int c1 = r >> m_offsets[0];
  unsigned int c2 = g >> m_offsets[1];
  unsigned int c3 = b >> m_offsets[2];  
 
  // OpenCV stores rgb channels with gbr order with histograms, so
  // even if we aren't in this color space, we shift the canal order
  float *f = cvGetHistValue_3D(m_histoData, c2, c3, c1);

  if (*f == 0)
    {
      const size_t index = c1 + c2 * (256 >> m_offsets[0]) + c3 * (256 >> m_offsets[0]) * (256 >> m_offsets[1]);
      m_colorVect.push_back(index);
    }

  *f = *f + 1;
}

void 
OpenCVHistogram::clear()
{
  AbstractHistogram::clear();
  cvClearHist(m_histoData);
}

HistogramIterator * 
OpenCVHistogram::getIterator()
{
  return new OpenCVHistogramIterator(this);
}
