#ifndef ISEPARAGRAPH_HPP
#define ISEPARAGRAPH_HPP

#include "iseTextLine.hpp"

namespace ise {
/**
  * A paragraph is a set of ise::TextLines
  * All the paragraphs instances are given a reference to a vector of lines
  * containing all the lines. A paragraph instance maintains the list of lines
  * belonging to itself
  */
class Paragraph
{

public:
  typedef girl::unorderedSet<size_t> ContainerType;
  
  
  
  explicit Paragraph(const std::vector<ise::TextLine> *lines);
  
  ~Paragraph();

    void
    addLine(size_t l);

    void
    removeLine(size_t l);

    girl::POINT_TYPE
    xMin() const;

    girl::POINT_TYPE
    xMax() const;

    girl::POINT_TYPE
    yMin() const;


    girl::POINT_TYPE
    yMax() const;

    size_t
    nbLines() const;

    size_t
    firstLine() const;

    size_t
    lastLine() const;

    /**
      * Check if line l is in the paragraph
      */
    bool
    belongsTo(size_t l) const;

    void
    clear();

    void
    draw(cv::Mat &mat, const cv::Scalar &color, const BitmapRunLength &horiz, const BitmapRunLength &verti) const;

    /**
      * Function to sort the textline in reading order
      */
    bool
    operator()(const size_t &a, const size_t &b) const
    {
      return sortInLatinReadingOrder(a, b);
    }

  const ContainerType&
  getIncludedLines() const
  {
    return m_includedLines; 
  }
protected:

    void
    update(size_t l);

private:

    /**
      * top to bottom, Left to right,  according to the line xMin() and yMin()
      * coordinates
      */

    bool
    sortInLatinReadingOrder(const size_t &a, const size_t &b) const;

private:

    const std::vector<ise::TextLine> *m_lines;
  //  typedef std::priority_queue<size_t, ise::Paragraph> ContainerType;

    ContainerType m_includedLines;

    girl::POINT_TYPE m_xmin;
    girl::POINT_TYPE m_xmax;
    girl::POINT_TYPE m_ymin;
    girl::POINT_TYPE m_ymax;



};


}
#endif // ISEPARAGRAPH_HPP
