#ifndef PATH_HPP
#define PATH_HPP

#define SIGMA 3

#include <girl/girlPoint.hpp>
#include <girl/girlContour.hpp>
#include "iseContourPoint.hpp"

namespace girl {
  class discreteTangent;
}

namespace ise {

  class Path
  {
  public:

    explicit Path(const girl::contour &c);
    Path(const Path &other);
    Path &operator=(const Path &other);
    
    void computeCoordinates(const girl::contour &c);
    void computeTangents(const girl::contour &c);
    void computeCurvatures(double sigma);

    size_t getLength() const { return m_n; }
    girl::point getPMin() const { return girl::point(m_xMin, m_yMin); }
    girl::point getPMax() const { return girl::point(m_xMax, m_yMax); }
    double getContourX(size_t i) const { return m_tabPts[i].getCoordX(); }
    double getContourY(size_t i) const { return m_tabPts[i].getCoordY(); }
    double getContourTX(size_t i) const { return m_tabPts[i].getTX(); }
    double getContourTY(size_t i) const { return m_tabPts[i].getTY(); }
    double getContourCurvature(size_t i) const { return m_tabPts[i].getCurvature(); }
    double getAngle(size_t i) const { return m_tabPts[i].getAngle(); }

  private:

    void computeTangentAngle(const girl::discreteTangent &t, 
			     double &tx, double &ty, double &angle);
      
    int circularIndex(int ind);
    double gaussianDiff(double x, double sigma);
    double localCurvature(int i, double sigma);

  private:
    size_t m_n;
    std::vector<ContourPoint> m_tabPts;
    int m_xMin;
    int m_yMin;
    int m_xMax;
    int m_yMax;
  };

}

#endif /* PATH_HPP */
