#include "isePerimeterEnclosingShapeFeatureVisitor.hpp"

#include "EnclosingBox.hpp"
#include "EnclosingConvexHull.hpp"
#include "EnclosingCircle.hpp"
#include "EnclosingEllipse.hpp"
#include "EnclosingRotatedRect.hpp"

#include <cassert>
#include <cmath>

#ifndef M_PI
//Visual C++ does not define these constants in <cmath>
#define M_PI 3.14159265358979323846
#endif 

using namespace ise;

PerimeterEnclosingShapeFeatureVisitor::PerimeterEnclosingShapeFeatureVisitor()
{
}

double
PerimeterEnclosingShapeFeatureVisitor::visit(EnclosingConvexHull *box)
{
    std::vector<cv::Point> points = box->getPoints();

    if (points.empty()) {
      assert(false);
      throw "not enough points to compute perimeter";
    }

#if 1
    if (points.size() < 3) {
      return 0;
    }

#else

    if (points.size() == 1) {
      //reduced to a single point/pixel
      return ( abs(points[1].y-points[0].y) + abs(points[1].x-points[0].x) );   
    }
    if (points.size() == 2) {
      //reduced to a line
      //we do not necessarily have :  (points[0].x==points[1].x || points[0].y==points[1].y);
      return ???
    }
#endif


    cv::Point p0;
    cv::Point p1;

    double perimeter = 0.0;
    size_t i = 1;
    while (i < points.size()) {

      p0 = points[i-1];
      p1 = points[i];
      
      const int dx = (p0.x - p1.x);
      const int dy = (p0.y - p1.y);

      perimeter += sqrt( double(dx*dx + dy*dy) );

      ++i;
    }
#ifndef _MSC_VER
    assert(std::isfinite(perimeter));
#endif
	assert(perimeter > 0);
    return perimeter;
}

double
PerimeterEnclosingShapeFeatureVisitor::visit(EnclosingBox *box)
{
  const cv::Rect r = box->getRect();
  const double perimeter = (2 * r.width + 2 * r.height);
#ifndef _MSC_VER
  assert(std::isfinite(perimeter));
#endif
  assert(perimeter >= 0);
  return perimeter;
}

double PerimeterEnclosingShapeFeatureVisitor::visit(EnclosingCircle *box)
{
  const double perimeter = 2*M_PI*box->getRadius();
#ifndef _MSC_VER
  assert(std::isfinite(perimeter));
#endif
  assert(perimeter >= 0);
  return perimeter;  
}

double PerimeterEnclosingShapeFeatureVisitor::visit(EnclosingEllipse *box)
{
  const double a = box->getRotatedRect().size.height/2;
  const double b = box->getRotatedRect().size.width/2;
  const double perimeter = M_PI*sqrt(2*(a*a+b*b));
#ifndef _MSC_VER
  assert(std::isfinite(perimeter));
#endif
  assert(perimeter >= 0);
  return perimeter;  
}

double ise::PerimeterEnclosingShapeFeatureVisitor::visit(EnclosingRotatedRect *box)
{
  const double perimeter = 2*box->getRect().size.width + 2*box->getRect().size.height;
#ifndef _MSC_VER
  assert(std::isfinite(perimeter));
#endif
  assert(perimeter >= 0);
  return perimeter;  
}


