#ifndef ISEPERIMETERENCLOSINGSHAPEFEATUREVISITOR_HPP
#define ISEPERIMETERENCLOSINGSHAPEFEATUREVISITOR_HPP

#include <ise/iseEnclosingShapeFeatureVisitor.hpp>
#include <ise/iseEnclosingShapeFeatureVisitorRegistry.hpp>

namespace ise {


/**
  * Computes the perimeter of some EnclosingShapes
  *
  * \code
     EnclosingConvexHull hull(points);
     ise::EnclosingShapeFeatureVisitorRegistry * registry = ise::EnclosingShapeFeatureVisitorRegistry::instance();
     double preimeter = registry->applyEnclosingShapeFeatureVisitor("PerimeterEnclosingShapeFeatureVisitor", hull);
  *
  * \endcode
  * Or :
  * \code
      EnclosingConvexHull hull(points);
      ise::PerimeterEnclosingShapeFeatureVisitor perimeterComputer;
      double area = hull.accept (perimeterComputer) ;

  * \endcode
  * \example tests/testEnclosingShapeFeatureVisitor.cpp
  */
class GIRL_API PerimeterEnclosingShapeFeatureVisitor : public EnclosingShapeFeatureVisitor
{
public:
    PerimeterEnclosingShapeFeatureVisitor();

    /**
      * sums of distance between each points
      */
    virtual double visit(EnclosingConvexHull *box);

    virtual double visit(EnclosingBox *box);

    virtual double visit(EnclosingCircle *box);

    virtual double visit(EnclosingEllipse *box);

    virtual double visit(EnclosingRotatedRect *box);
};

REGISTER_ENCLOSING_SHAPE_VISITOR(PerimeterEnclosingShapeFeatureVisitor)

}

#endif // ISEPERIMETERENCLOSINGSHAPEFEATUREVISITOR_HPP
