/*
  Copyright (C) 2005-2009 Roigt Julian, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

/**
 * @file isePyramid.hpp
 * @brief Pyramid interface
 */

#ifndef ISEPYRAMID_HPP
#define ISEPYRAMID_HPP

#include <vector>

#include <girl/girlCommon.hpp>

#include <girlTools/girlToolsImage.hpp>

#include <opencv2/core/core.hpp>


namespace ise{

  /**
   * @brief Pyramid interface.
   */
  class GIRL_API Pyramid
  {
  public:
    Pyramid() 
      : m_divider()
      , m_level()
      {}

    /**
     * @brief Destructor.
     */ 
    virtual ~Pyramid() {}
    
    /**
     * @brief Sets number of level @a level and the divider @a divider of the pyramid.
     * @param @a divider must be >= 0
     */
    void init( int level, int divider)
    {
      assert( divider >= 0);
      m_level = level;
      m_divider = divider;
    }

    /**
     * @brief Get the number of level of pyramid.
     */
    inline int level() const { return m_level; }

    /**
     * @brief Get the divider between two level of pyramid.
     */
    inline int divider() const { return m_divider; }

    /**
     * @brief Compute the pyramid of Cv::Mat @a image.
     */
    virtual void compute(const cv::Mat &image) = 0;
    
    /**
     * @brief Compute the pyramid in @a res of girl::tools::Image @a image
     */
    virtual void compute(const girl::tools::Image &image, std::vector<girl::tools::Image> &res) = 0;

  protected:
    /**
     * @brief Copy constructo
     */
    Pyramid(const Pyramid &other);

    /**
     * @brief Assignment operator
     */
    Pyramid &operator=(const Pyramid &other);

  protected:
    
    /**
     * @brief The size divider between two level of pyramid
     */
    int m_divider;

    /**
     * @brief The number of level of the pyramid
     */
    int m_level;
  };
}

#endif /* ! ISEPYRAMID_HPP */
