/*
  Quantification CAG class.
  Copyright (C) 2005-2009 Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "iseQuantificationCAG.hpp"

#include "iseClusterSet.hpp"
#include "iseHistogram.hpp"


using namespace ise;

static const double DEFAUTL_THRESHOLD = 0.3;



QuantificationCAG::~QuantificationCAG()
{
  delete m_clusterSet;
}

QuantificationCAG::QuantificationCAG(SegmentationContext *segContext, 
				     Histogram *histo, 
				     int colorSpace)
  :
  AbstractQuantificationHisto(segContext, histo), 
  m_colorSpace(colorSpace),
  m_conversion(ise::conversion::changeSpace(colorSpace)),  
  m_weightedChannelsCoeffs(),
  m_clusterSet(NULL),
  m_threshold(DEFAUTL_THRESHOLD)
{
  //TODO : better initialization !!!
  const size_t numChannels = 3;
  const double inv = 1.0 / static_cast<double>(numChannels);
  m_weightedChannelsCoeffs.resize(numChannels);
  for (size_t i=0; i<numChannels; ++i) 
    m_weightedChannelsCoeffs[i] = inv;

  m_clusterSet = new ClusterSet(m_conversion, m_weightedChannelsCoeffs);
}


bool
QuantificationCAG::areEquivalentColors(const girl::tools::Pixel &RGBcolor1, 
				       const girl::tools::Pixel &RGBcolor2)
{
  return m_clusterSet->areEquivalentColor(RGBcolor1, RGBcolor2);
}

void 
QuantificationCAG::init(const girl::regionDeque &regions, int /*level*/)
{
  m_histo->clear();

  delete m_clusterSet;
  m_clusterSet = new ClusterSet(m_conversion, m_weightedChannelsCoeffs);


  computeHistogramOnRegionSet(regions);
  
  ise::HistogramIterator *iteHisto = m_histo->getIterator();

  HistoInfo info;

  iteHisto->begin();

  while (!iteHisto->atEnd())
    {
      iteHisto->get(&info);
      addColor(*info.getColor(), info.getFrequency());
      iteHisto->next();
    }

  m_clusterSet->reduction(m_threshold);

  delete iteHisto;
}


void 
QuantificationCAG::addColor(const girl::tools::Pixel &RGBcolor, long frequency)
{
  m_clusterSet->addColor(RGBcolor, frequency);
}
	       
void QuantificationCAG::init(girl::region /*r*/, int /*level*/)
{
  /* VIDE.... peut être à Supprimer */
}

void
QuantificationCAG::setParameters(double threshold, int colorSpace, const std::vector<double> &weightedChannelsCoeffs)
{
  m_threshold = threshold;
  m_colorSpace = colorSpace;
  m_conversion = ise::conversion::changeSpace(colorSpace);
  m_weightedChannelsCoeffs = weightedChannelsCoeffs;
}

void
QuantificationCAG::getParameters(double &threshold, int &colorSpace, std::vector<double> &weightedChannelsCoeffs) const
{
  threshold = m_threshold;
  colorSpace = m_colorSpace;
  weightedChannelsCoeffs = m_weightedChannelsCoeffs;
}
