/*
  Quantification CAG class.
  Copyright (C) 2005-2009 Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISEQUANTIFICATIONCAG_HPP
#define ISEQUANTIFICATIONCAG_HPP


#include <girl/girlCommon.hpp>

#include "iseAbstractQuantificationHisto.hpp"
#include "iseConversion.hpp"


namespace ise {

  class ClusterSet;
  class Histogram;

  class GIRL_API QuantificationCAG : public AbstractQuantificationHisto
  {
    
  public:
    
    /**
     * @brief Constructor.
     * 
     * @warning takes ownership of @a histo.
     */
    QuantificationCAG(SegmentationContext *segContext, Histogram *histo, int colorSpace);

    /**
     * @brief Destructor.
     */
    ~QuantificationCAG();
  
    virtual void init(const girl::regionDeque &regions, int level);
    
    virtual void init(girl::region r, int level);
    
    void setThreshold(double t) { m_threshold = t;}
    void getThreshold(double &t) const { t = m_threshold; }

    void setParameters(double threshold, int colorSpaceConversion, const std::vector<double> &weightedChannelsCoeffs); 

    void getParameters(double &threshold, int &colorSpaceConversion, std::vector<double> &weightedChannelsCoeffs) const; 

  protected :
    
    bool areEquivalentColors(const girl::tools::Pixel &RGBcolor1, 
			     const girl::tools::Pixel &RGBcolor2);
    
    void addColor(const girl::tools::Pixel &RGBcolor, long frequency);
    
  private:
    QuantificationCAG(const QuantificationCAG &);
    QuantificationCAG &operator=(const QuantificationCAG &);

  protected:
    int m_colorSpace;
    ise::conversion::PFConversion *m_conversion;
    std::vector<double> m_weightedChannelsCoeffs;
    ClusterSet *m_clusterSet;
    double m_threshold;
   
  };

}

#endif /* ! ISEQUANTIFICATIONCAG_HPP */
