/*
  Quantification Kmeans class.
  Copyright (C) 2005-2009 Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISEQUANTIFICATIONKMEANS_HPP
#define ISEQUANTIFICATIONKMEANS_HPP


#include <girl/girlCommon.hpp>

#include "iseConversion.hpp"
#include "iseQuantification.hpp"
#include <cmath>


namespace ise {

  class GIRL_API QuantificationKMeans : public Quantification
  {
    
  public:
    
    /**
     * @brief Destructor.
     */
    ~QuantificationKMeans();
  
    /**
     * @brief Constructor.
     * 
     * @warning takes ownership of @a histo.
     */
    QuantificationKMeans(SegmentationContext *segContext, 
			 int colorSpace = ISE_RGB_to_K1K2K3,
			 int nbClasses = 4);

    virtual void init(const girl::regionDeque &regions, int level);
    
    virtual void init(girl::region r, int level);
    
    void setParameters(int colorSpace, int nbClasses, const std::vector<double> &weightedChannelsCoeffs) 
    { 
      m_colorSpace = colorSpace;
      m_conversion = ise::conversion::changeSpace(colorSpace);
      m_weightedChannelsCoeffs  = weightedChannelsCoeffs;
      m_nbClasses = nbClasses;
    }

    void getParameters(int &colorSpace, int &nbClasses, std::vector<double> &weightedChannelsCoeffs) const
    { 
      colorSpace = m_colorSpace; 
      weightedChannelsCoeffs = m_weightedChannelsCoeffs;
      nbClasses = m_nbClasses;
    }
   
  protected :
    
    virtual bool areEquivalentColors(const girl::tools::Pixel &RGBcolor1, 
				     const girl::tools::Pixel &RGBcolor2);
    
    
  private:
    QuantificationKMeans(const QuantificationKMeans &);
    QuantificationKMeans &operator=(const QuantificationKMeans &);

  protected:
    ise::conversion::PFConversion *m_conversion;
    int m_colorSpace;
    std::vector<double> m_weightedChannelsCoeffs;
    int m_nbClasses;
  };

}

#endif /* ! ISEQUANTIFICATIONKMEANS_HPP */
