/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
#include "iseQuantificationThresholderDomenger.hpp"

#include <iostream>

#include "iseHistogram1D.hpp"


namespace ise
{
  
  double
  QuantificationThresholderDomenger::getThreshold(const Histogram1D &hist) const
  {
    
    size_t size = hist.size();
    
    if (size < 3) {
      const size_t ThIndex = size / 2;
      return hist.indexToKey(ThIndex);
    }
      
    size_t index = 0;
    while (index < size && hist[index] == 0)
      ++index;

    ++index;


    Histogram1D filteredHisto = hist;
    for (size_t i = index; i < size - 1; ++i) {
      filteredHisto[i] = (0.5*hist[i-1] + hist[i] + 0.5*hist[i+1])/2;
    }

    
    size_t i = 0;
    for (i = 1; i < size; ++i) {
      if ((filteredHisto[i] != 0) && (filteredHisto[i] < filteredHisto[i-1])) 
	break;
    }

    for (size_t j = i+1 ; j < size; ++j) {
      if (filteredHisto[j-1] <  filteredHisto[j]) {
	return hist.indexToKey(j-1);
      }
    }


    return hist.indexToKey(size-1);
  }

  void
  QuantificationThresholderDomenger::getThresholds(const Histogram1D &hist, 
						   std::vector<double> &thresholds)
  {
    thresholds.clear();
    thresholds.push_back(getThreshold(hist));
  }

  size_t
  QuantificationThresholderDomenger::getNumberOfThresholds() const
  {
    return 1;
  }

}
