/*
  Histogram thresholds computer: QIR method.
  Copyright (C) 2005-2011 Remi Vieux  remi.vieux@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISEQUANTIFICATIONTHRESHOLDERDOMENGER_HPP_
#define ISEQUANTIFICATIONTHRESHOLDERDOMENGER_HPP_

#include <girl/girlCommon.hpp>
#include "iseQuantificationThresholder.hpp"

/**
 * Convolve the histogram with (1/2 1 1/2)/2
 * return the first detected concavity (i.e. f(i) - f(i+1) < 0)
 */
namespace ise
{

  class GIRL_API QuantificationThresholderDomenger : public QuantificationThresholder
  {

  public:

    /**
     * @brief Get thresholds @a thresholds computed on histogram @a h.
     */
    virtual void getThresholds(const Histogram1D &h, std::vector<double> &thresholds);

    /**
     * @brief Get number of thresholds that would be computed by this 
     * thresholder, without actually computing them.
     *
     * If number of computed thresholds is not fixed but dependant of 
     * data/histogram, then (size_t)-1 is returned.
     */
    virtual size_t getNumberOfThresholds() const;

    
    double getThreshold(const Histogram1D &h) const;

  };

}

#endif /* ISEQUANTIFICATIONTHRESHOLDERDOMENGER_HPP_ */
