/*
  Histogram thresholds computer: Kapur method.
  Copyright (C) 2005-2009 Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISEQUANTIFICATIONTHRESHOLDERKAPUR2T_HPP
#define ISEQUANTIFICATIONTHRESHOLDERKAPUR2T_HPP

#include <girl/girlCommon.hpp>
#include <ise/iseQuantificationThresholder.hpp>

namespace ise {

  class GIRL_API QuantificationThresholderKapur2T : public QuantificationThresholder
  {
  public:
    QuantificationThresholderKapur2T(double sigma1coeff = 1.0,
				   double sigma2coeff = 1.0);

    void setCoeffs(double sigma1coeff, double sigma2coeff);
    

    /**
     * @brief Get thresholds @a thresholds computed on histogram @a h.
     */
    virtual void getThresholds(const Histogram1D &h, std::vector<double> &thresholds);

    /**
     * @brief Get number of thresholds that would be computed by this 
     * thresholder, without actually computing them.
     *
     * If number of computed thresholds is not fixed but dependant of 
     * data/histogram, then (size_t)-1 is returned.
     */
    virtual size_t getNumberOfThresholds() const;

  protected:
    double m_sigma1coeff;
    double m_sigma2coeff;

  };

}

#endif /* ! ISEQUANTIFICATIONTHRESHOLDERKAPUR2T_HPP */
