/*
  Histogram thresholds computer: Entropy Li thresholding method.
  Copyright (C) 2005-2011 Claire Morand, Jean-Philippe Domenger, Achille Braquelaire, Roigt Julian, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

/**
 * @file iseQuantificationThresholderLi.hpp
 * @brief QuantificationThresholder class.
 */

#ifndef ISEQUANTIFICATIONTHRESHOLDERLI_HPP
#define ISEQUANTIFICATIONTHRESHOLDERLI_HPP

#include <girl/girlCommon.hpp>
#include <ise/iseQuantificationThresholder.hpp>

namespace ise {

  /**
     M. Sezgin and B. Sankur, "Survey over image thresholding techniques 
     and quantitative performance evaluation," Journal of Electronic 
     Imaging, Vol. 13, No. 1. (2004), pp. 146-168.
     
     C. H. Li and C. K. Lee, "Minimum cross-entropy 
     thresholding" Patern Recogn. 26, 617-625(1993)
     
     C.H. Li and P. K. S. Tam, "An iterative algorithm 
     for minimum cross-entropy thresholding," Pattern 
     Recogn, Lett. 19, 771-776(1998)
  */
  class GIRL_API QuantificationThresholderLi : public QuantificationThresholder
  {
  public:

    /**
     * @brief Get thresholds @a thresholds computed on histogram @a h.
     */
    virtual void getThresholds(const Histogram1D &h, std::vector<double> &thresholds);

    /**
     * @brief Get number of thresholds that would be computed by this 
     * thresholder, without actually computing them.
     *
     * If number of computed thresholds is not fixed but dependant of 
     * data/histogram, then (size_t)-1 is returned.
     */
    virtual size_t getNumberOfThresholds() const;

  };

}

#endif /* ! ISEQUANTIFICATIONTHRESHOLDERLI_HPP */
