/*
  Histogram thresholds computer: Otsu method, 1 threshold.
  Copyright (C) 2005-2009 Vicent Rabeux, Claire Morand, Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISEQUANTIFICATIONTHRESHOLDEROTSU1T_HPP
#define ISEQUANTIFICATIONTHRESHOLDEROTSU1T_HPP

#include <girl/girlCommon.hpp>
#include <ise/iseQuantificationThresholder.hpp>

namespace ise {

  class GIRL_API QuantificationThresholderOtsu1T : public QuantificationThresholder
  {
  public:
    QuantificationThresholderOtsu1T();

    
    /**
     * @brief Get thresholds @a thresholds computed on histogram @a h.
     */
    virtual void getThresholds(const Histogram1D &h, std::vector<double> &thresholds);

    /**
     * @brief Get number of thresholds that would be computed by this 
     * thresholder, without actually computing them.
     *
     * If number of computed thresholds is not fixed but dependant of 
     * data/histogram, then (size_t)-1 is returned.
     */
    virtual size_t getNumberOfThresholds() const;
 
    
    double getThreshold(const Histogram1D &h);
    
    /**
     * Get moments of first class computed by last call to getThresholds or getThreshold.
     *
     * If getThresholds has not been previously called, result is undefined.
     *
     * @sa getThresholds.
     */
    void getMoments_c1(double &m0, double &m1, double &m2)
    {
     m0 = m_m0_c1;
     m1 = m_m1_c1;
     m2 = m_m2_c1;
    }
    
   /**
    * Get moments of second class computed by last call to getThresholds or getThreshold.
    *
    * If getThresholds has not been previously called, result is undefined.
    *
    * @sa getThresholds.
    */
    void getMoments_c2(double &m0, double &m1, double &m2)
    {
      m0 = m_m0_c2;
      m1 = m_m1_c2;
      m2 = m_m2_c2;
    }
    
  protected:
    
    double m_m0_c1;
    double m_m1_c1;
    double m_m2_c1;
    double m_m0_c2;
    double m_m1_c2;
    double m_m2_c2;
    
  };
  
}

#endif /* ! ISEQUANTIFICATIONTHRESHOLDEROTSU1T_HPP */
