/*
  Histogram thresholds computer: Otsu method, 2 thresholds.
  Copyright (C) 2005-2011 Claire Morand, Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "iseQuantificationThresholderOtsu2T.hpp"

#include <cmath>
#include "iseHistogram1D.hpp"
#include <iostream>

using namespace ise;

QuantificationThresholderOtsu2T::QuantificationThresholderOtsu2T(double sigma1coeff,
								 double sigma2coeff)
  : m_sigma1coeff(sigma1coeff),
    m_sigma2coeff(sigma2coeff)
{
  
}

void 
QuantificationThresholderOtsu2T::setCoeffs(double sigma1coeff, double sigma2coeff)
{
  m_sigma1coeff = sigma1coeff;
  m_sigma2coeff = sigma2coeff;
}
    
//otsu a 2 seuils
void
QuantificationThresholderOtsu2T::getThresholds(const Histogram1D &h, std::vector<double> &thresholds)
{
  Histogram1D histo = h; //copy
  const size_t sz = histo.size();

  //set min and max values to 0   (//B: was: // set white and black values to 0)
  //  histo[0] = 0;
  //histo[sz-1] = 0;

  double sum_c1 = 0;
  //double sum2_c1 = 0;
  double nbPoints_c1 = 0;
  double sum_c2 = 0;
  //double sum2_c2 = 0;
  double sum_c3 = 0;
  //double sum2_c3 = 0;
  double nbPoints_c3 = 0;

  double nbPoints_c2 = histo[0];
  for (size_t i = 1 ; i < sz ; ++i) {
    sum_c3 += histo[i] * i;
    //sum2_c3 += histo[i] *i * histo[i] * i;
    nbPoints_c3 += histo[i];
  }
  double nbPoints = nbPoints_c2 + nbPoints_c3;

  double p1 = nbPoints_c1 / nbPoints;
  double p2 = nbPoints_c2 / nbPoints;
  double p3 = nbPoints_c3 / nbPoints;

  double average_c1 = (nbPoints_c1 == 0 ? 0 : sum_c1 / nbPoints_c1);
  //double sigma2_c1 = (nbPoints_c1 == 0 ? 0 : sum2_c1 / nbPoints_c1 - average_c1 * average_c1);
  
  double average_c2 = (nbPoints_c2 == 0 ? 0 : sum_c2 / nbPoints_c2);
  //double sigma2_c2 = (nbPoints_c2 == 0 ? 0 : sum2_c2 / nbPoints_c2 - average_c2 * average_c2);
  
  double average_c3 = (nbPoints_c3 == 0 ? 0 : sum_c3 / nbPoints_c3);
  //double sigma2_c3 = (nbPoints_c3 == 0 ? 0 : sum2_c3 / nbPoints_c3 - average_c3 * average_c3);
  
  const double full_average = p1*average_c1 + p2*average_c2 + p3*average_c3;

  double max = p1*(average_c1-full_average)*(average_c1-full_average) +  p2*(average_c2-full_average)*(average_c2-full_average)
    + p3*(average_c3-full_average)*(average_c3-full_average); 

  size_t Th1 = 0;
  size_t Th2 = 1;

  for (size_t i = 0 ; i < sz-1 ; ++i) {//deplacement du seuil Th1
    for (size_t j = i+1 ; j < sz ; ++j) {//deplacement du seuil Th2
      sum_c1 = 0;
      //sum2_c1 = 0;
      nbPoints_c1 = 0;
      sum_c2 = 0;
      //sum2_c2 = 0;
      nbPoints_c2 = 0;
      sum_c3 = 0;
      //sum2_c3 = 0;
      nbPoints_c3 = 0;

      for (size_t k = 0 ; k < sz ; ++k) {
	int value = k * histo[k];
	if (k < i) {
	  sum_c1 += value;
	  //sum2_c1 += value * value;
	  nbPoints_c1 += histo[k];
	}
	else if (k < j) {
	  sum_c2 += value;
	  //sum2_c2 += value * value;
	  nbPoints_c2 += histo[k];
	}
	else {
	  sum_c3 += value;
	  //sum2_c3 += value * value;
	  nbPoints_c3 += histo[k];
	}
      }
      average_c1 = (nbPoints_c1 == 0 ? 0 : sum_c1 / nbPoints_c1);
      //sigma2_c1 = (nbPoints_c1 == 0 ? 0 : sum2_c1 / nbPoints_c1 - average_c1 * average_c1);

      average_c2 = (nbPoints_c2 == 0 ? 0 : sum_c2 / nbPoints_c2);
      //sigma2_c2 = (nbPoints_c2 == 0 ? 0 : sum2_c2 / nbPoints_c2 - average_c2 * average_c2);

      average_c3 = (nbPoints_c3 == 0 ? 0 : sum_c3 / nbPoints_c3);
      //sigma2_c3 = (nbPoints_c3 == 0 ? 0 : sum2_c3 / nbPoints_c3 - average_c3 * average_c3);

      p1 = nbPoints_c1 / nbPoints;
      p2 = nbPoints_c2 / nbPoints; 
      p3 = nbPoints_c3 / nbPoints; 
      const double currentValue = p1*(average_c1-full_average)*(average_c1-full_average) 
	+ p2*(average_c2-full_average)*(average_c2-full_average)
	+ p3*(average_c3-full_average)*(average_c3-full_average); 
      
      if (max < currentValue) {
	max = currentValue;
	Th1 = i;
	Th2 = j;
      } 
    }
  }

  //attention: suppose que les valeurs sont rangees de la plus petite a la plus grande
  thresholds.resize(2);
  thresholds[0] = histo.indexToKey(Th1); 
  thresholds[1] = histo.indexToKey(Th2); 
}

size_t
QuantificationThresholderOtsu2T::getNumberOfThresholds() const
{
  return 2;
}
