/*
  Histogram thresholds computer: Shape Sezan thresolding method.
  Copyright (C) 2005-2009 Claire Morand, Jean-Philippe Domenger, Achille Braquelaire, Roigt Julian, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "iseQuantificationThresholderSezan.hpp"
#include "iseHistogram1D.hpp"
#include "iseHistogram1DTools.hpp"

#include <limits>


using namespace ise;


namespace {
  
  static const double gaussian_sigma = 0.75;

  /**
   *
   * @param[in,out] h 
   */
  size_t
  histoConvolutionAndPeakDetection(ise::Histogram1D &h, size_t kernelSize)
  {
    if (h.size() < 3)
      return 0;

    assert(kernelSize&1); //must be an odd number.

    ise::Histogram1D histo;
  
    ise::applyGaussianFilter(histo, kernelSize, gaussian_sigma, h);

    return ise::histoNumPeakDetection(h);
  }

} //end anonymous namespace




void
QuantificationThresholderSezan::getThresholds(const ise::Histogram1D &h, std::vector<double> &thresholds)
{
  ise::Histogram1D histo = h; // copy
  //std::vector<Histogram1D> lastHisto;
  Histogram1D lastHisto;

  const size_t N = 2;
  size_t n = 0;
  int k = 0;
  size_t size = h.size();

  // See article (3) 
  size_t kernelSize = 5;
  while (1) {
    //lastHisto.push_back(histo);
    lastHisto = histo;
    n = histoConvolutionAndPeakDetection(histo, kernelSize);
    if ( n > N) {
      ++k;
    }
    if (n < N && kernelSize == 3) {
      thresholds.resize(1);
      thresholds[0] = 0;
      return;
    }
    if ( n < N && kernelSize != 3) {
      kernelSize = 3;
      if ( k == 0) {
	thresholds.resize(1);
	thresholds[0] = 0;
	return;
      }
      //histo = lastHisto[--k];
      histo = lastHisto;
    }
 
    if ( n == N) {
      break;
    }
  }

  // valley detection
  int T = 0;
  int max = 0;
  for (size_t i = 1 ; i < size - 1 ; ++i) {
    if ( histo[i-1] < histo[i] && histo[i] > histo[i+1] ) {
      ++max;
    }
    if ( (histo[i] < histo[i-1] && histo[i] < histo[i+1]) || ( histo[i] < histo[i-1] && histo[i] == 0)) {
      if ( max != 0 ) {
	T = i;
	break;
      }
    }
  }

  T = histo.indexToKey(T);

  thresholds.resize(1);
  thresholds[0] = T;
}

size_t
QuantificationThresholderSezan::getNumberOfThresholds() const
{
  return 1;
}
