/*
  Histogram thresholds computer: variance method.
  Copyright (C) 2005-2011 Claire Morand, Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "iseQuantificationThresholderVariance.hpp"

#include <cmath>
#include "iseHistogram1D.hpp"
#include <iostream>

using namespace ise;


void
QuantificationThresholderVariance::getThresholds(const Histogram1D &h, std::vector<double> &thresholds)
{
  Histogram1D histo = h; //copy
  const size_t sz = histo.size();
  
  //calcul des 1eres variances pour T=0
  double sum_c1 = 0;
  double sum2_c1 = 0;
  double nbPoints_c1 = 0;
  double sum_c2 = 0;
  double sum2_c2 = 0;
  double nbPoints_c2 = 0;
  for (size_t i = 0 ; i < sz ; ++i) {
    sum_c2 += histo[i] * i;
    sum2_c2 += histo[i] * i * histo[i] *i;
    nbPoints_c2 += histo[i];
  }
  double average_c1 = (nbPoints_c1 == 0 ? 0 : sum_c1 / nbPoints_c1);
  average_c1 = average_c1 * average_c1;
  double sigma2_c1 = (nbPoints_c1 == 0 ? 0 : sum2_c1 / nbPoints_c1 - average_c1);
  
  double average_c2 = (nbPoints_c2 == 0 ? 0 : sum_c2 / nbPoints_c2);
  average_c2 = average_c2 * average_c2;
  double sigma2_c2 = (nbPoints_c2 == 0 ? 0 : sum2_c2 / nbPoints_c2 - average_c2);
  
  double min = sigma2_c1 + sigma2_c2;
  int Th = 0;

  for (size_t i = 1 ; i < sz ; ++i) {
    int value = histo[i-1] * (i-1);
    sum_c1 += value;
    sum2_c1 += value * value;
    nbPoints_c1 += histo[i-1];
    sum_c2 -= value;
    sum2_c2 -= value * value;
    nbPoints_c2 -= histo[i-1];

    average_c1 = (nbPoints_c1 == 0 ? 0 : sum_c1 / nbPoints_c1);
    average_c1 = average_c1 * average_c1;
    sigma2_c1 = (nbPoints_c1 == 0 ? 0 : sum2_c1 / nbPoints_c1 - average_c1);
    
    average_c2 = (nbPoints_c2 == 0 ? 0 : sum_c2 / nbPoints_c2);
    average_c2 = average_c2 * average_c2;
    sigma2_c2 = (nbPoints_c2 == 0 ? 0 : sum2_c2 / nbPoints_c2 - average_c2);
    
    double currentValue = sigma2_c1 + sigma2_c2;
    if (min > currentValue) {
      min = currentValue;
      Th = i;
    } 
  }

  thresholds.resize(1);
  thresholds[0] = histo.indexToKey(Th); // J
}

size_t
QuantificationThresholderVariance::getNumberOfThresholds() const
{
  return 1;
}
