/*
  Copyright (C) 2005-2011 Remi Vieux remi.vieux@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "iseQuantificationThresholderWhatmough.hpp"


#include <iostream>
#include <fstream>

#include "iseHistogram1D.hpp"
#include "iseSparseHistogram1D.hpp"

#include <opencv2/imgproc/imgproc.hpp>


namespace ise
{

  double getQuantity(int k, int p, int q, const Histogram1D &histo, const SparseHistogram1DD &logHisto)
  {

    double c = static_cast<double>(k - p)/(q - p);
    if (histo[p])
      return histo[p]*exp(c*(logHisto[q] - logHisto[p]));
    else
      return 0;

  }

  double getCvHullQuantity(int k, int p, int q, const Histogram1D &histo, const SparseHistogram1DD &/*logHisto*/) //B: logHisto not used !!!!
  {
    double c = static_cast<double>(k - p)/(q - p);
    return histo[p] + c*(histo[q] - histo[p]);
  }

  struct Toto {
    int index;
    double d;

    Toto(int i, double _d) 
      : index(i), 
	d(_d) 
    {}
    
    bool operator< (const struct Toto &toto) const {
      return d > toto.d;
    }

  };



//  void
//  QuantificationThresholderWhatmough::getThresholds(const Histogram1D &histo, std::vector<double> &thresholds)
//  {
//    size_t size = histo.size();
//    SparseHistogram1DD logHisto;
//    for (size_t i = 0; i < size; ++i) {
//      if (histo[i])
//	logHisto.add(i, std::log(histo[i]));
//      else
//	logHisto.add(i, 0);
//    }
//
//    SparseHistogram1DD expoHull; //B: is a sparsehisto necessary ??????????
//    expoHull.add(0, histo[0]);
//
//    //    std::cout << "Expo Hull" << std::endl;
//    //    std::cout << "0 " << expoHull[0] << std::endl;
//    //R:TODO: REMOVE
//    size_t stop = std::min((size_t)254, size);  //B: why 254 ???????????????!!!!!!
//    for (size_t i = 1; i < stop - 1; ++i) {
//        // TODO:OPTIM cant we use previously computed values?
//        double max = -std::numeric_limits<double>::max();
//        for (size_t p = 0; p < i; ++p) {
//            for (size_t q = i+1; q < stop; ++q) {
//                double val = getQuantity(i, p, q, histo, logHisto);
//                if (val > max)
//                  max = val;
//            }
//        }
//
//        expoHull.add(i, std::max(histo[i], max));
//        out << i << " " << std::max(histo[i], max) << std::endl;
//
//    }
//    out.close();
//
//
//
//
//
//    expoHull.add(stop - 1, histo[size - 1]);
//    // std::cout << size - 1 << " " << expoHull[size - 1] << std::endl;
//
//    std::vector<struct Toto> diff;
//    for (size_t i = 0; i < stop; ++i) {
//        struct Toto toto(i, expoHull[i] - histo[i]);
//        diff.push_back(toto);
//    }
//    std::sort(diff.begin(), diff.end());
//    int nbThresholds = 2;
//    for (int i = 0; i < nbThresholds; ++i)
//      thresholds.push_back(diff[i].index);
//
//
//    {//DEBUG
//      std::cout << "Whatmough thresholds ";
//      for (int i = 0; i < nbThresholds; ++i) {
//	std::cout << diff[i].index << "(" << diff[i].d << ") ";
//      }
//      std::cout << std::endl;
//    }
//
//  }



  void
  QuantificationThresholderWhatmough::getThresholds(const Histogram1D &histo, std::vector<double> &thresholds)
  {
    // V2 OPTIMIZED BUT USING OPENCV CONVEXHULL.. might be better to compute hull of histogram directly
    // using an appropriate method. Or is it?
    size_t size = histo.size();

    std::vector<cv::Point2f> pts(size);
    std::vector<cv::Point2f>::iterator it = pts.begin();
    const std::vector<cv::Point2f>::const_iterator itEnd = pts.end();
//    std::ofstream histoOut("histo.dat");
//    std::ofstream logHistoOut("logHisto.dat");
    for (size_t i = 0; it != itEnd; ++i, ++it) {
        it->x = i;
        it->y = std::log(histo[i]);
//        histoOut << i << " " << histo[i] << std::endl;
//        logHistoOut << i << " " << it->y << std::endl;
    }
//    histoOut.close();
//    logHistoOut.close();

    std::vector<int> hull;
    cv::convexHull(cv::Mat(pts), hull,
#if CV_MAJOR_VERSION*100+CV_MINOR_VERSION*10+CV_SUBMINOR_VERSION < 300
		   CV_COUNTER_CLOCKWISE
#else
		   false
#endif
		   );

    std::vector<int>::const_iterator iter = hull.begin();
    assert(*iter == 0);
    std::vector<int>::const_iterator iterPlus = hull.begin() + 1;
    std::vector<int>::const_iterator iterEnd = hull.end();
    // std::cout << "Expo Hull Interp" << std::endl;
//    std::ofstream interExpOut("expoHullInterp.dat");
    std::vector<float> interpExpoHull(size);
    std::vector<float>::iterator fIt;
    for ( ; (iter != iterEnd) && (*iterPlus > *iter); ++iter, ++iterPlus) {
     //   interExpOut << pts[*iter].x << " " << exp(pts[*iter].y) << std::endl;
        // linear interpolation
        int index0 = *iter;
        float v0 = exp(pts[index0].y);
        int index1 = *iterPlus;
        float v1 = exp(pts[index1].y);
        interpExpoHull[index0] = v0;
        interpExpoHull[index1] = v1;
        int gap = index1 - index0;
        for (int i = 1; i < gap; ++i) {
            float alpha = (gap - i)/static_cast<float>(gap);
            float val = alpha*v0 + (1 - alpha)*v1;
            interpExpoHull[index0 + i] = val;
        }

    }
//    interExpOut.close();


    std::vector<struct Toto> diff;
    for (size_t i = 0; i < size; ++i) {
        struct Toto toto(i, interpExpoHull[i] - histo[i]);
        diff.push_back(toto);
    }
    std::sort(diff.begin(), diff.end());
    int nbLevels = 1;
    std::cout << "Whatmough levels (interp) ";
    for (int i = 0; i < nbLevels; ++i) {
        thresholds.push_back(diff[i].index);
        std::cout << diff[i].index << "(" << diff[i].d << ") ";
    }
    std::cout << std::endl;

//    std::ofstream interExpFull("expoHullFull.dat");
//    for (size_t i = 0; i < size; ++i) {
//        interExpFull << i << " " << interpExpoHull[i] << std::endl;
//    }
//    interExpFull.close();

//    getLevels2(histo, levels);

  }

  size_t
  QuantificationThresholderWhatmough::getNumberOfThresholds() const
  {
    return 1;
  }

} //end namespace ise

