/*
  Copyright (C) 2005-2011 Remi Vieux remi.vieux@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef ISEQUANTIFICATIONTHRESHOLDERWHATMOUGH_HPP_
#define ISEQUANTIFICATIONTHRESHOLDERWHATMOUGH_HPP_

#include <girl/girlCommon.hpp>
#include "iseQuantificationThresholder.hpp"

namespace ise
{
  /**
   * Implements histogram thresholding selection from the paper:
   *
   * @ARTICLE{Whatmough1991,
   * author = {R. J. Whatmough},
   * title = {Automatic threshold selection from a histogram using the “exponential
   *    hull”},
   * journal = {CVGIP: Graphical Models and Image Processing},
   * year = {1991},
   * volume = {53},
   * pages = {592-600},
   * }
   *
   */
  class GIRL_API QuantificationThresholderWhatmough : public QuantificationThresholder
  {
  public:

    /**
     * @brief Get thresholds @a thresholds computed on histogram @a h.
     */
    virtual void getThresholds(const Histogram1D &h, std::vector<double> &thresholds);

    /**
     * @brief Get number of thresholds that would be computed by this 
     * thresholder, without actually computing them.
     *
     * If number of computed thresholds is not fixed but dependant of 
     * data/histogram, then (size_t)-1 is returned.
     */
    virtual size_t getNumberOfThresholds() const;

  };

}

#endif /* ISEQUANTIFICATIONTHRESHOLDERWHATMOUGH_HPP_ */
