/*
  Histogram thresholds computer: Cluster Yanni thresolding method.
  Copyright (C) 2005-2009 Claire Morand, Jean-Philippe Domenger, Achille Braquelaire, Roigt Julian, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include <cmath>

#include "iseHistogram1D.hpp"
#include "iseQuantificationThresholderYanni.hpp"

#include <limits>
#include <iostream>

using namespace ise;


void
QuantificationThresholderYanni::getThresholds(const Histogram1D &h, std::vector<double> &thresholds)
{
  Histogram1D histo = h;
  const size_t sz = histo.size();
  assert(sz > 0);


  double size = 0;
  for (size_t i = 0; i < sz; ++i)
    size += histo[i];
  assert(size > 0);
  const double inv_size = 1.0 / size;

  std::vector<double> histo_norm(sz, 0);
  for (size_t i = 0 ; i < sz ; ++i)
    histo_norm[i] = static_cast<double>(histo[i]) * inv_size;

  const double lmin = histo.getMin();
  const double lmax = histo.getMax();
  //const double absoluteMin = histo.absoluteMin();

  double mid = (lmax + lmin) * 0.5;
  
  double peak1 = 0;
  double peak2 = 0;

  size_t val = 0;
  for (double i = lmin ; i < mid+1 ; ++i) {  //B: WRONG !!!!
    if (histo_norm[histo.keyToIndex(i)] > val) {
      val = histo_norm[histo.keyToIndex(i)];
      peak1 = i;
    }
  }

  val = 0;
  for (double i = mid+1 ; i <= lmax ; ++i) {  //B: WRONG !!!!
    if (histo_norm[histo.keyToIndex(i)] > val) {
      val = histo_norm[histo.keyToIndex(i)];
      peak2 = i;
    }
  }

  mid = (peak1 + peak2) * 0.5;

  double T = 0;
  for (double  i = lmin ; i <= mid ; ++i)   //B: WRONG !!!!
    T += histo_norm[histo.keyToIndex(i)];

  T *= (lmax - lmin);

  thresholds.resize(1);
  thresholds[0] = T;
}

size_t
QuantificationThresholderYanni::getNumberOfThresholds() const
{
  return 1;
}
