/*
  Uniform quantification.
  Copyright (C) 2005-2009 Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "iseQuantificationUniform.hpp"

using namespace ise;

QuantificationUniform::QuantificationUniform(SegmentationContext *segContext, 
					     int colorSpace,
					     unsigned char offset0, 
					     unsigned char offset1, 
					     unsigned char offset2)
  : 
  Quantification(segContext),
  m_conversion(ise::conversion::changeSpace(colorSpace)),
  m_offset0(offset0), 
  m_offset1(offset1),
  m_offset2(offset2),
  m_colorSpace(colorSpace),
  m_weightedChannelsCoeffs()
{
  //TODO : better initialization !!!
  const size_t numChannels = 3;
  const double inv = 1.0 / static_cast<double>(numChannels);
  m_weightedChannelsCoeffs.resize(numChannels);
  for (size_t i=0; i<numChannels; ++i) 
    m_weightedChannelsCoeffs[i] = inv;
}



bool 
QuantificationUniform::areEquivalentColors(const girl::tools::Pixel &RGBcolor1, 
					   const girl::tools::Pixel &RGBcolor2)
{
  int p1[3], p2[3];
  
  for (int i = 0 ; i < 3 ; ++i)
    {
      p1[i] = m_conversion[i](RGBcolor1[0], RGBcolor1[1], RGBcolor1[2]);
      p2[i] = m_conversion[i](RGBcolor2[0], RGBcolor2[1], RGBcolor2[2]);
      p1[i] *= m_weightedChannelsCoeffs[i];
      p2[i] *= m_weightedChannelsCoeffs[i];
    }
  
  
 //  const int v0 = RGBcolor1[0]>> m_offset0;
//   const int v1 = RGBcolor1[1]>> m_offset1;
//   const int v2 = RGBcolor1[2]>> m_offset2;

//   const int w0 = RGBcolor2[0]>> m_offset0;
//   const int w1 = RGBcolor2[1]>> m_offset1;
//   const int w2 = RGBcolor2[2]>> m_offset2;


  const int v0 = p1[0]>> m_offset0;
  const int v1 = p1[1]>> m_offset1;
  const int v2 = p1[2]>> m_offset2;

  const int w0 = p2[0]>> m_offset0;
  const int w1 = p2[1]>> m_offset1;
  const int w2 = p2[2]>> m_offset2;

//   return (m_conversion[0](v0, v1, v2) == m_conversion[0](w0, w1, w2)) &&
//     (m_conversion[1](v0, v1, v2) == m_conversion[1](w0, w1, w2)) &&
//     (m_conversion[2](v0, v1, v2) == m_conversion[2](w0, w1, w2));

  return (v0== w0 && v1 == w1 && v2 == w2);

}


void
QuantificationUniform::setOffsets(unsigned char offset0, unsigned char offset1, unsigned char offset2)
{
  m_offset0 = offset0;
  m_offset1 = offset1;
  m_offset2 = offset2;
}

void
QuantificationUniform::getOffsets(unsigned char &offset0, unsigned char &offset1, unsigned char &offset2) const
{
  offset0 = m_offset0;
  offset1 = m_offset1;
  offset2 = m_offset2;
}

void
QuantificationUniform::setParameters(int colorSpace, std::vector<double> weightedChannelsCoeffs)
{
  m_colorSpace = colorSpace;
  m_conversion = ise::conversion::changeSpace(colorSpace);
  m_weightedChannelsCoeffs = weightedChannelsCoeffs;
}

void
QuantificationUniform::getParameters(int &colorSpace, std::vector<double> &weightedChannelsCoeffs) const
{
  colorSpace = m_colorSpace;
  weightedChannelsCoeffs = m_weightedChannelsCoeffs;
}
