/*
  Uniform quantification.
  Copyright (C) 2005-2009 Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

/**
 * @file iseQuantificationUniform.hpp
 * @brief 
 */
#ifndef ISEQUANTIFICATIONUNIFORM_HPP
#define ISEQUANTIFICATIONUNIFORM_HPP

#include <girl/girlCommon.hpp>
#include "iseQuantification.hpp"
#include "iseConversion.hpp"


namespace ise {

  class GIRL_API QuantificationUniform : public Quantification
  {
  public:
    
  
    QuantificationUniform(SegmentationContext *segContext, 
			  int colorSpace = 0,
			  unsigned char offset1 = 0, unsigned char offset2 = 0, unsigned char offset3 = 0);

    virtual ~QuantificationUniform() {}
  
    virtual void init(const girl::regionDeque  &/* regions */, int /* level */) {} 

    virtual void init(girl::region /* r */, int /* level */) {}

    /**
     * @brief Set offsets for quantification.
     *
     * For RGB, @a offset1, 2, 3 are in [0; 8]
     */
    void setOffsets(unsigned char offset1, unsigned char offset2, unsigned char offset3);

    void getOffsets(unsigned char &offset1, unsigned char &offset2, unsigned char &offset3) const;

    void setParameters(int colorSpace, std::vector<double> weightedChannelsCoeffs);
    void getParameters(int &colorSpace, std::vector<double> &weightedChannelsCoeffs) const;
    
  protected:

    virtual bool areEquivalentColors(const girl::tools::Pixel &RGBcolor1, 
				     const girl::tools::Pixel &RGBcolor2);

  private:
    QuantificationUniform(const QuantificationUniform &);
    QuantificationUniform &operator=(const QuantificationUniform &);

  protected:
    
    ise::conversion::PFConversion *m_conversion;
    unsigned char m_offset0, m_offset1, m_offset2;
    int m_colorSpace;
    std::vector<double> m_weightedChannelsCoeffs;
    
  };

}


#endif  /* ! ISEQUANTIFICATIONUNIFORM_HPP */
