/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
#ifndef ISEREGIONSETBINARYFUNCTIONCHI2DISTANCE_HPP_
#define ISEREGIONSETBINARYFUNCTIONCHI2DISTANCE_HPP_

#include <girl/girlCommon.hpp>


#include "iseFactoryTemplate.hxx"
#include "iseRegionBinaryFunctionWithRegSet.hpp"

/**
 * Return the Chi Square distance between the region feature
 * d(a, b) = \sum_{i}{((a_{i}-b_{i})^2)/(a_{i}+b_{i})}
 *
 */

namespace ise {
  class GIRL_API RegionBinaryFunctionWithRegSetChi2Distance : public RegionBinaryFunctionWithRegSet
  {
  public:

    /**
     * Constructor
     */
    explicit RegionBinaryFunctionWithRegSetChi2Distance(ise::SegmentationContext *segContext)
    : RegionBinaryFunctionWithRegSet(segContext) {}

    virtual
    ~RegionBinaryFunctionWithRegSetChi2Distance() {}


    result_type
    operator()(const first_argument_type & reg1,
    const second_argument_type& reg2);

//    /**
//     * Get parameters associated with the regset
//     */
//    void
//    getParameters(girl::shared_ptr<RegionSetInterface> regSet,
//		  RegionSetParameters &regSetParameters) const
//    {
//      regSet = m_regSet;
//      regSetParameters = m_regSetParameters;
//    }
//
//
//    /**
//     * Set parameters associated with the regset
//     */
//    void
//    setParameters(const girl::shared_ptr<RegionSetInterface> regSet,
//		  const RegionSetParameters& regSetParameters)
//    {
//      assert(regSetParameters.getDimWeights().size() == regSet->getFeatureDim());
//      m_regSet = regSet;
//      m_regSetParameters = regSetParameters;
//    }
//
//    class RegionSetBinaryFunctionChi2DistanceMemento : public RegionSetBinaryFunctionWeighted::RegionSetBinaryFunctionWeightedMemento
//    {
//    public:
//      RegionSetBinaryFunctionChi2DistanceMemento() {}
//
//    protected:
//      virtual void
//      getParameters(const Mementable &m)
//      {
//        const RegionBinaryFunctionWithRegSetChi2Distance& rs
//        = reinterpret_cast<const RegionBinaryFunctionWithRegSetChi2Distance&>(m);
//        RegionSetBinaryFunctionWeightedMemento::getParameters(rs);
//      }
//      virtual void
//      setParameters(Mementable &m)
//      {
//        RegionBinaryFunctionWithRegSetChi2Distance& rs
//        = reinterpret_cast<RegionBinaryFunctionWithRegSetChi2Distance&>(m);
//        RegionSetBinaryFunctionWeightedMemento::setParameters(rs);
//      }
//
//    };
//
//    protected:
//
//    virtual RegionBinaryFunctionMemento* createMemento() const
//    {
//      return new RegionSetBinaryFunctionChi2DistanceMemento;
//    }
//
//    virtual RegionBinaryFunction::result_type operator_aux(const double *wrf1, const double *wrf2, size_t featureDim)
//    {
//      return cvflann::chi_square_dist(wrf1, wrf1 + featureDim, wrf2);
//    }




  };

  typedef FactoryTemplate<RegionBinaryFunction, RegionBinaryFunctionWithRegSetChi2Distance> RegionBinaryFunctionWithRegSetChi2DistanceFactory;

}
#endif /* ISEREGIONSETBINARYFUNCTIONCHI2DISTANCE_HPP_ */
