/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef ISEREGIONBINARYFUNCTIONWITHREGSETHELLINGERDISTANCE_HPP_
#define ISEREGIONBINARYFUNCTIONWITHREGSETHELLINGERDISTANCE_HPP_

#include <girl/girlCommon.hpp>

#include "iseFactoryTemplate.hxx"
#include "iseRegionBinaryFunctionWithRegSet.hpp"

/**
 * Return the Hellinger distance between the region feature
 * d(a, b) = \sum_{i}(\sqrt{a_{i}} - \sqrt{b_{i}})^2
 */
namespace ise {
  class GIRL_API RegionBinaryFunctionWithRegSetHellingerDistance : public RegionBinaryFunctionWithRegSet
  {
  public:

    /**
     * Constructor
     */
    explicit RegionBinaryFunctionWithRegSetHellingerDistance(ise::SegmentationContext *segContext)
    : RegionBinaryFunctionWithRegSet(segContext) {}

    virtual
    ~RegionBinaryFunctionWithRegSetHellingerDistance() {}

    result_type
    operator()(const first_argument_type & reg1,
        const second_argument_type& reg2);

//
//    /**
//     * Get parameters associated with the regset
//     */
//    void
//    getParameters(girl::shared_ptr<RegionSetInterface> regSet,
//		  RegionSetParameters &regSetParameters) const
//    {
//      regSet = m_regSet;
//      regSetParameters = m_regSetParameters;
//    }
//
//
//    /**
//     * Set parameters associated with the regset
//     */
//    void
//    setParameters(const girl::shared_ptr<RegionSetInterface> regSet,
//		  const RegionSetParameters& regSetParameters)
//    {
//      assert(regSetParameters.getDimWeights().size() == regSet->getFeatureDim());
//      m_regSet = regSet;
//      m_regSetParameters = regSetParameters;
//    }
//
//    class RegionSetBinaryFunctionHellingerDistanceMemento : public RegionSetBinaryFunctionWeighted::RegionSetBinaryFunctionWeightedMemento
//    {
//    public:
//      RegionSetBinaryFunctionHellingerDistanceMemento() {}
//
//    protected:
//      virtual void
//      getParameters(const Mementable &m)
//      {
//        const RegionBinaryFunctionWithRegSetHellingerDistance& rs
//        = reinterpret_cast<const RegionBinaryFunctionWithRegSetHellingerDistance&>(m);
//        RegionSetBinaryFunctionWeightedMemento::getParameters(rs);
//      }
//      virtual void
//      setParameters(Mementable &m)
//      {
//        RegionBinaryFunctionWithRegSetHellingerDistance& rs
//        = reinterpret_cast<RegionBinaryFunctionWithRegSetHellingerDistance&>(m);
//        RegionSetBinaryFunctionWeightedMemento::setParameters(rs);
//      }
//
//    };
//
//    protected:
//
//    virtual RegionBinaryFunctionMemento* createMemento() const
//    {
//      return new RegionSetBinaryFunctionHellingerDistanceMemento;
//    }
//
//    virtual RegionBinaryFunction::result_type operator_aux(const double *wrf1, const double *wrf2, size_t featureDim)
//    {
//      return cvflann::hellinger_dist(wrf1, wrf1 + featureDim, wrf2);
//    }

  };

  typedef FactoryTemplate<RegionBinaryFunction, RegionBinaryFunctionWithRegSetHellingerDistance> RegionBinaryFunctionWithRegSetHellingerDistanceFactory;

}
#endif /* ISEREGIONBINARYFUNCTIONWITHREGSETHELLINGERDISTANCE_HPP_ */
