/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
#ifndef ISEREGIONSETBINARYFUNCTIONKLDISTANCE_HPP_
#define ISEREGIONSETBINARYFUNCTIONKLDISTANCE_HPP_

#include <girl/girlCommon.hpp>

#include "iseFactoryTemplate.hxx"
#include  "iseRegionBinaryFunctionWithRegSet.hpp"

/**
 * Return the Kullback-Leibler divergence between the region feature
 * (NOT SYMMETRIC)
 *
 * kl(a, b) = \sum_{i}(a_{i} * log(a_{i}/b_{i})
 */

namespace ise {
  class GIRL_API RegionBinaryFunctionWithRegSetKLDistance : public RegionBinaryFunctionWithRegSet
  {
  public:

    /**
     * Constructor
     */
    explicit RegionBinaryFunctionWithRegSetKLDistance(ise::SegmentationContext *segContext) : RegionBinaryFunctionWithRegSet(segContext) {}

    virtual
    ~RegionBinaryFunctionWithRegSetKLDistance() {}

    result_type
    operator()(const first_argument_type & reg1,
        const second_argument_type& reg2);



    //    /**
    //     * Get parameters associated with the regset
    //     */
    //    void
    //    getParameters(girl::shared_ptr<RegionSetInterface> regSet,
    //        RegionSetParameters &regSetParameters) const
    //    {
    //      regSet = m_regSet;
    //      regSetParameters = m_regSetParameters;
    //    }
    //
    //
    //    /**
    //     * Set parameters associated with the regset
    //     */
    //    void
    //    setParameters(const girl::shared_ptr<RegionSetInterface> regSet,
    //        const RegionSetParameters& regSetParameters)
    //    {
    //      assert(regSetParameters.getDimWeights().size() == regSet->getFeatureDim());
    //      m_regSet = regSet;
    //      m_regSetParameters = regSetParameters;
    //    }
    //
    //    class RegionSetBinaryFunctionKLDistanceMemento : public RegionSetBinaryFunctionWeighted::RegionSetBinaryFunctionWeightedMemento
    //    {
    //    public:
    //      RegionSetBinaryFunctionKLDistanceMemento() {}
    //
    //    protected:
    //      virtual void
    //      getParameters(const Mementable &m)
    //      {
    //        const RegionBinaryFunctionWithRegSetKLDistance& rs
    //        = reinterpret_cast<const RegionBinaryFunctionWithRegSetKLDistance&>(m);
    //        RegionSetBinaryFunctionWeightedMemento::getParameters(rs);
    //      }
    //      virtual void
    //      setParameters(Mementable &m)
    //      {
    //        RegionBinaryFunctionWithRegSetKLDistance& rs
    //        = reinterpret_cast<RegionBinaryFunctionWithRegSetKLDistance&>(m);
    //        RegionSetBinaryFunctionWeightedMemento::setParameters(rs);
    //      }
    //
    //    };
    //
    //    protected:
    //
    //    virtual RegionBinaryFunctionMemento* createMemento() const
    //    {
    //      return new RegionSetBinaryFunctionKLDistanceMemento;
    //    }
    //
    //    virtual RegionBinaryFunction::result_type operator_aux(const double *wrf1, const double *wrf2, size_t featureDim)
    //    {
    //      return cvflann::kl_divergence(wrf1, wrf1 + featureDim, wrf2);
    //    }


  };

  typedef FactoryTemplate<RegionBinaryFunction, RegionBinaryFunctionWithRegSetKLDistance> RegionBinaryFunctionWithRegSetKLDistanceFactory;

}
#endif /* ISEREGIONSETBINARYFUNCTIONKLDISTANCE_HPP_ */
