/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "iseRegionBinaryFunctionWithRegSetMinkowskiDistance.hpp"

#include <opencv2/flann/flann.hpp>


namespace ise
{
  RegionBinaryFunction::result_type
  RegionBinaryFunctionWithRegSetMinkowskiDistance::operator()(const first_argument_type & reg1,
      const second_argument_type& reg2)
  {
    assert(m_regSet);
    RegionFeature rf1(m_memory1);
    RegionFeature rf2(m_memory2);
    m_regSet->getRegionFeatures(reg1, rf1);
    m_regSet->getRegionFeatures(reg2, rf2);
#if CV_MAJOR_VERSION*100+CV_MINOR_VERSION*10+CV_SUBMINOR_VERSION < 231
#if CV_MAJOR_VERSION*100+CV_MINOR_VERSION*10+CV_SUBMINOR_VERSION < 230
    const cvflann::flann_distance_t dt = cvflann::MINKOWSKI;
#else
    const cvflann::flann_distance_t dt = cvflann::FLANN_DIST_MINKOWSKI;
#endif
    const int order = m_order;
    cvflann::set_distance_type(dt, order);
    return cvflann::minkowski_dist(m_memory1, m_memory1 + m_regSet->getFeatureDim(), m_memory2);
#else
    return cvflann::MinkowskiDistance<ise::RegionFeature_Data_Type>(m_order)(m_memory1, m_memory2, m_regSet->getFeatureDim());
#endif
  }
}
