/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*
 * iseRegionFeatureColorimetricMoment.cpp
 *
 *  Created on: Aug 24, 2010
 *      Author: vieux
 */

#include <sstream>

#include "iseColorimetricMomentsComputer.hpp"
#include "iseFlt3Tools.hpp"
#include "iseRegionFeatureColorimetricMoment.hpp"

namespace ise
{

  RegionFeatureColorimetricMoment::RegionFeatureColorimetricMoment(ise::SegmentationContext *segContext, int colorSpace)
   : RegionFeatureComputer(segContext),
     m_colorSpace(colorSpace)
  {
  }

  RegionFeatureColorimetricMoment::~RegionFeatureColorimetricMoment()
  {
  }

  const std::string
  RegionFeatureColorimetricMoment::uri() const
  {
    std::stringstream ss(std::ios::in|std::ios::out);
    ss << __FILE__ << "_ColorSpace_" << m_colorSpace;
    return ss.str();

  }

  RegionFeature*
  RegionFeatureColorimetricMoment::compute(const girl::toger &tog, girl::region id)
  {
    RegionFeature *rf = allocate(COLORIMETRIC_MOMENTS_FEATURE_SIZE);
    assert(rf);
    RegionFeature_Data_Type *result = rf->getData();

    ColorimetricMomentsComputerCS cmp(*(m_segContext->getImage()), m_colorSpace);//, result[0], &(result[1]), &(result[4]));
    tog.iterateOnRegionDomain(id, cmp);
    result[0] = cmp.m_m0;
    RegionFeature_Data_Type* i0 = &(result[1]);
    RegionFeature_Data_Type* i1 = &(result[4]);
    for (int i = 0; i < 3; ++i) {
      i0[i] = cmp.m_m1[i];
      i1[i] = cmp.m_m2[i];
    }
    return rf;
  }

  bool
  RegionFeatureColorimetricMoment::mergeFeatures(RegionFeature *keptFeatures, RegionFeature *delFeatures)
  {
    assert(keptFeatures);
    assert(delFeatures);
    iseDouble_addition(keptFeatures->getData(), delFeatures->getData(), COLORIMETRIC_MOMENTS_FEATURE_SIZE);
    return true;
  }

}
