/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * iseRegionFeatureComputerGravityCenter.hpp
 *
 *  Created on: Oct 15, 2010
 *      Author: vieux
 */

#ifndef ISEREGIONFEATURECOMPUTERGRAVITYCENTER_HPP_
#define ISEREGIONFEATURECOMPUTERGRAVITYCENTER_HPP_

#include <girl/girlCommon.hpp>
#include <girl/girlToger.hpp>

#include "iseRegionFeatureComputer.hpp"
#include "iseRegionSetInterfaceImpl.hpp"

namespace ise
{
  /**
   * Computes the gravity center of the region.
   * The feature vector returns the coordinates of the region and
   * the area.
   * TODO:UNSURE We store here the region area, as well as in the moments
   * Area is useful for merging the features. But we should be able to use or get the area
   * from elsewhere, the information is duplicated here!
   */

  class GIRL_API RegionFeatureComputerGravityCenter : public ise::RegionFeatureComputer,
  public girl::toger::pointIterator
  {
  public:
    explicit RegionFeatureComputerGravityCenter(ise::SegmentationContext* segContext)
    : RegionFeatureComputer(segContext),
      m_x(0),
      m_y(0),
      m_area(0)
    {}

    virtual
    ~RegionFeatureComputerGravityCenter() {}

    inline const std::string
    uri() const { return __FILE__; }

//    inline const std::string
//    regSetType() const { return ise::RegionSetInterfaceImpl::type(); }

    inline size_t
    size() const { return RegionFeatureComputerGravityCenter::FEATURE_SIZE; }

    void
    compute(const girl::region& id, ise::RegionFeature &feature);

    void
    mergeFeatures(RegionFeature&, RegionFeature&, const girl::region &);

    inline void
    getFeatureTitle(std::string &title) const
    {
      title = "Gravity center";
    }

    inline void
    getFeatureDimDescription(std::vector<std::string> &dimDesc) const
    {
      dimDesc.clear();
      dimDesc.push_back("Gravity center X coordinate");
      dimDesc.push_back("Gravity Center Y coordinate");
      dimDesc.push_back("Area of the region");
    }

    // REMI: no parameters need to be defined for this extractor
    // Hence not writing a memento class
    // TODO:UNSURE  Will it cause a problem later in the program
    // (e.g. segfault trying to use the memento)
    RegionFeatureComputerMemento*
    createMemento() const
    {
      return NULL;
    }

    void
    processPoint(const girl::point &);

  private:

    static const size_t FEATURE_SIZE = 3;

    double m_x;
    double m_y;
    double m_area;

  };

}

#endif /* ISEREGIONFEATURECOMPUTERGRAVITYCENTER_HPP_ */
