/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include <sstream>

#include "iseColorimetricMomentsComputer.hpp"
#include "iseFlt3Tools.hpp"
#include "iseRegionFeatureComputerMoment.hpp"
#include "iseSegmentationContext.hpp"
namespace ise
{

  RegionFeatureComputerMoment::RegionFeatureComputerMoment(ise::SegmentationContext *segContext)
  : RegionFeatureComputer(segContext),
    m_imageFeatureUri()
  {
  }

  RegionFeatureComputerMoment::~RegionFeatureComputerMoment()
  {
  }

  const std::string
  RegionFeatureComputerMoment::uri() const
  {
    std::stringstream ss(std::ios::in|std::ios::out);
    ss << __FILE__ << "_" << m_imageFeatureUri;
    return ss.str();
  }

  size_t
  RegionFeatureComputerMoment::size() const
  {
    assert(m_segContext);
    const girl::tools::ImageFeature *imageFeature = m_segContext->getImageFeature(m_imageFeatureUri);
    assert(imageFeature);
    return 2*imageFeature->channels() + 1;
  }


  void
  RegionFeatureComputerMoment::compute(const girl::region &id, RegionFeature &feature)
  {
    assert(m_segContext);
    assert(m_segContext->getToger());
    const girl::tools::ImageFeature *imageFeature = m_segContext->getImageFeature(m_imageFeatureUri);
    assert(imageFeature);
//    RegionFeature *regionFeature = allocate(size());
    RegionFeature_Data_Type *result = feature.getData();
    MomentsComputerIF momentComputer(imageFeature,
				     result,
				     &(result[1]),
				     &(result[imageFeature->channels() + 1]));
    m_segContext->getToger()->iterateOnRegionDomain(id, momentComputer);
    //return regionFeature;
  }

  void
  RegionFeatureComputerMoment::mergeFeatures(RegionFeature &kept, RegionFeature &del, const girl::region&)
  {
    iseDouble_addition(kept.getData(), del.getData(), size());
  }
}
