/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISEREGIONFEATURECOMPUTERMOMENT_HPP_
#define ISEREGIONFEATURECOMPUTERMOMENT_HPP_

//#include <girlTools/girlToolsImageFeature.hpp>
#include "iseRegionFeatureComputer.hpp"
//#include "iseRegionSetStatistical.hpp"

namespace ise
{

  /**
   * Generic class for computing moments of ImageFeatures
   */
  class GIRL_API RegionFeatureComputerMoment : public ise::RegionFeatureComputer
  {
  public:
    explicit RegionFeatureComputerMoment(ise::SegmentationContext *segContext);
    virtual
    ~RegionFeatureComputerMoment();

    const std::string uri() const;

  //  inline const std::string regSetType() const { return ise::RegionSetStatistical::type();}

    /**
     * @return the size of the feature vector = 2*number of channel of image feature(m1, m2) + 1 (m0)
     */
    size_t size() const;

    void compute(const girl::region& id, RegionFeature &feature);

    void mergeFeatures(RegionFeature& keptFeatures, RegionFeature& delFeatures, const girl::region &);

//    inline const girl::tools::ImageFeature& getImageFeature() { return *m_imgFeature; }

    inline virtual void getFeatureTitle(std::string &title) const
    {
      title = "Generic Moment Computer";
    }

    inline virtual void getFeatureDimDescription(std::vector<std::string> &dimDesc) const
    {
      dimDesc.clear();
      dimDesc.assign(size(), "");
      size_t s = (size() - 1)/2;
      dimDesc[0] = "m0";
      for (size_t i = 0; i < s; ++i) {
        dimDesc[i + 1] = "m1";
        dimDesc[s + 1 + i] = "m2";
      }
    }

    inline virtual void setParameters(const std::string &imgFeatureURI)
    {
      m_imageFeatureUri = imgFeatureURI;
    }

    inline virtual void getParameters(std::string &imgFeatureURI) const
    {
      imgFeatureURI = m_imageFeatureUri;
    }

  public:

    class RegionFeatureMomentComputerMemento : public ise::RegionFeatureComputer::RegionFeatureComputerMemento
    {
     friend class RegionFeatureComputerMoment;

    public:
     virtual ~RegionFeatureMomentComputerMemento() {}

     virtual void getParameters(const Mementable &m)
     {
       const RegionFeatureComputerMoment &memento = reinterpret_cast<const RegionFeatureComputerMoment&>(m);
       memento.getParameters(m_uri);
     }
     virtual void setParameters(Mementable &m) const
     {
       RegionFeatureComputerMoment &memento = reinterpret_cast<RegionFeatureComputerMoment&>(m);
       memento.setParameters(m_uri);
     }
    private:
     std::string m_uri;
    };

    virtual RegionFeatureComputerMemento *createMemento() const
    {
      return new RegionFeatureMomentComputerMemento;
    }

  protected:

  //  ise::ImageFeaturesInterface *m_imgFeature;
    std::string m_imageFeatureUri;

  };

}

#endif /* ISEREGIONFEATURECOMPUTERMOMENT_HPP_ */
