/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISEREGIONFEATURENEEDBINARIZATION_HPP_
#define ISEREGIONFEATURENEEDBINARIZATION_HPP_


#include <opencv2/core/core.hpp>


#include "iseRegionFeatureComputer.hpp"
/**
 * Interface for the region feature that rely on a binary image, typically the
 * run length features and apparented. This interface factorize the code
 * for computing the binary image
 */
namespace ise
{

  class GIRL_API RegionFeatureNeedBinarization : public ise::RegionFeatureComputer
  {
  public:

    RegionFeatureNeedBinarization(ise::SegmentationContext *segContext)
    : RegionFeatureComputer(segContext),
      m_binaryImage()
    {}

    virtual
    ~RegionFeatureNeedBinarization() {}

    /**
     * Compute the binary image then launch computeImpl, which has to be redefined by the
     * subclasses.
     */
    RegionFeature *compute(const girl::toger &tog, girl::region id);

    virtual RegionFeature* computeImpl(const girl::toger &tog, girl::region id) = 0;

  protected:

    cv::Mat m_binaryImage;
  };

}

#endif /* ISEREGIONFEATURENEEDBINARIZATION_HPP_ */
