/*
 * iseRegionFeatureComputerNeighbourhoodContext.hpp
 *
 *  Created on: Oct 15, 2010
 *      Author: vieux
 */

#ifndef ISEREGIONFEATURECOMPUTERNEIGHBOURHOODCONTEXT_HPP_
#define ISEREGIONFEATURECOMPUTERNEIGHBOURHOODCONTEXT_HPP_

#include "iseRegionFeatureComputer.hpp"

#include "iseRegionSetInterfaceImpl.hpp"

namespace ise
{

  class SegmentationContext;
  /**
   * Compute "contextual" features, that is features about the neighbors, how
   * they are placed, etc.
   * @see getFeatureDimDescription
   */

  class GIRL_API RegionFeatureComputerNeighbourhoodContext : public RegionFeatureComputer
  {
  public:
    explicit RegionFeatureComputerNeighbourhoodContext(ise::SegmentationContext *segContext)
    : RegionFeatureComputer(segContext),
      // REMI TODO:IMPROVE hard coded default value, but don't want to use
      // #defines and they are used in the memento aswell
      m_numberOfBins(DEFAULT_NUMBER_OF_BINS),
      m_startAngle(DEFAULT_START_ANGLE)
    {}
    virtual
    ~RegionFeatureComputerNeighbourhoodContext() {}

    const std::string uri() const;

    inline const std::string
    regSetType() const { return ise::RegionSetInterfaceImpl::type(); }

    inline size_t
    size() const { return 2*m_numberOfBins + 1; }

    void
    compute(const girl::region& id, RegionFeature &feature);

    /**
     * Cant merge features because when 2 regions are merged, the center
     * of mass of the new region has changed hence the context changes
     */
    void
    mergeFeatures(RegionFeature&, RegionFeature&, const girl::region &keptId);


    inline void
    getFeatureTitle(std::string &title) const
    {
      title =  "Contextual Features";
    }

    void
    getFeatureDimDescription(std::vector<std::string> &dimDesc) const;


    /**
     * Set the algorithm parameters: how to quantize the space
     * @param numberOfBins number of division of the space
     * @param startAngle in degree
     */

    inline void
    setParameters(const size_t &numberOfBins, const double &startAngle)
    {
      m_numberOfBins = numberOfBins;
      m_startAngle = startAngle;
    }

    inline void
    getParameters(size_t &numberOfBins, double &startAngle) const
    {
      numberOfBins = m_numberOfBins;
      startAngle = m_startAngle;
    }

  public:

    class RegionFeatureNeighbourhoodContextMemento :
    public ise::RegionFeatureComputer::RegionFeatureComputerMemento
    {
      friend class RegionFeatureComputerNeighbourhoodContext;

    public:
      RegionFeatureNeighbourhoodContextMemento() :
          m_numberOfBins(RegionFeatureComputerNeighbourhoodContext::DEFAULT_NUMBER_OF_BINS),
          m_startAngle(RegionFeatureComputerNeighbourhoodContext::DEFAULT_START_ANGLE)
          {}
      virtual ~RegionFeatureNeighbourhoodContextMemento() {}

      void
      getParameters(const Mementable &m)
      {
        const RegionFeatureComputerNeighbourhoodContext& rf
        = reinterpret_cast<const RegionFeatureComputerNeighbourhoodContext&>(m);
        rf.getParameters(m_numberOfBins, m_startAngle);
      }

      void
      setParameters(Mementable &m) const
      {
        RegionFeatureComputerNeighbourhoodContext& rf
        = reinterpret_cast<RegionFeatureComputerNeighbourhoodContext&>(m);
        rf.setParameters(m_numberOfBins, m_startAngle);
      }
      private:

      size_t m_numberOfBins;
      double m_startAngle;

    };

    RegionFeatureComputerMemento*
    createMemento() const
    {
      return new RegionFeatureNeighbourhoodContextMemento;
    }

  protected:

    static const size_t DEFAULT_NUMBER_OF_BINS = 8;
    static const double DEFAULT_START_ANGLE;

    size_t m_numberOfBins;
    double m_startAngle;
  };

}

#endif /* ISEREGIONFEATURECOMPUTERNEIGHBOURHOODCONTEXT_HPP_ */
