#include "iseRegionProfileComputer.hpp"

#include <girlTools/girlToolsRegionInfos.hpp>

#include "iseQuantificationThresholderOtsu1T.hpp"
#include "iseImageFeatureThreshold.hpp"
#include "iseSignalProcessingUtils.hpp"

#ifdef VISU
#include <opencv2/highgui/highgui.hpp>
#endif //VISU

namespace ise {

RegionProfileComputer::RegionProfileComputer(SegmentationContext *segContext,
					     int colorSpace) :
  m_segContext(segContext),
  m_imageFeaturePtr(),
  m_profileHoriz(NULL),
  m_profileVerti(NULL),
  m_countHoriz(),
  m_countVerti(),
  m_box(),
  m_colorSpace(colorSpace)
{}

void
displayProfile(const ise::Histogram1D &profile,
	       std::vector<size_t> &counter,
               const std::string &wndName,
               bool horiz,
               double factor)
{

 //std::cout << "Threshold profile " << threhsold << std::endl;

 const double wndSize = 100;
 // ise::Histogram1D::const_iterator max = std::max_element(profile.begin(), profile.end());
 double max = profile.getMaxFrequency();

 cv::Mat mat = cv::Mat::zeros(wndSize, profile.size(), CV_8UC1);

 // std::vector<double> autocor(profile.size(), 0);


 for (size_t i = 0; i < profile.size(); ++i) {
  //   autocor[i] = profile[i];
  double threshold = counter[i]*factor;

  double val = profile[i] / max * wndSize;
  // cv::line(mat,cv::Point(i, wndSize), cv::Point(i, wndSize - val), CV_RGB(255,255,255), 0);
  // cv::line(mat, cv::Point(i, 0), cv::Point(i, val), CV_RGB(255,255,255));
  for (int j = 0; j < val; ++j) {
   mat.at<unsigned char>(wndSize - 1 - j, i) = 255;
  }
  if (val >= threshold) {
   for (int j = val; j < wndSize; ++j) {
    mat.at<unsigned char>(wndSize - 1 - j, i) = 128;
   }
  }
 }

 // std::vector<double> output;
 // ise::autocorrelation(autocor, output);
 // std::sort(autocor.begin(), autocor.end());
 // std::cout << "Autocorrelation:" << std::endl;
 // for (size_t i = 0; i < output.size(); ++i) {
 //   //std::cout << output[i] << std::endl;
 //   std::cout << autocor[i] << std::endl;
 // }
 
#ifdef VISU
 cv::namedWindow(wndName, 0);
 if (horiz) {
  cv::imshow(wndName, mat);
 }
 else {
  cv::Mat flip;
  cv::flip(mat, flip, 0);
  flip = flip.t();
  cv::imshow(wndName, flip);
  
 }
 

 cv::waitKey(0);
#endif //VISU

}

void
RegionProfileComputer::computeProfile(const girl::regionDeque &regs)
{
  // sometimes bug with selection
 if (!regs.size())
  return;

 assert(m_segContext);
 girl::tools::Image *image = m_segContext->getImage();
 assert(image);
 girl::toger *tog = m_segContext->getToger();
 assert(tog);
 // m_profileHoriz.clear();
 // m_profileVerti.clear();
 delete m_profileHoriz;
 delete m_profileVerti;
 
 

 const int colorSpace = ISE_RGB_to_GRAY;
 const int channel    = 0;
 // Compute threshold on the whole image not only the selected regs
 ise::Histogram1D histo(*image, colorSpace, channel);
 ise::QuantificationThresholderOtsu1T otsu;
 std::vector<double> thresholds;
 otsu.getThresholds(histo, thresholds);
 ise::ImageFeatureThreshold imageF(m_segContext);
 std::vector<int> levels;
 for (std::vector<double>::const_iterator it = thresholds.begin();
      it != thresholds.end(); ++it) {
  levels.push_back(static_cast<int>(*it));
 }
 imageF.setParameters(levels, colorSpace, channel);
 imageF.compute();
 m_imageFeaturePtr = imageF.getImageFeature();

 m_box = girl::tools::getAABBox(*tog, regs);
 m_profileHoriz = new ise::Histogram1D(m_box.xMin(), m_box.xMax());
 m_profileVerti = new ise::Histogram1D(m_box.yMin(), m_box.yMax());

 m_countHoriz.assign(m_box.width(), 0);
 m_countVerti.assign(m_box.height(), 0);

 girl::regionDeque::const_iterator it = regs.begin();
 const girl::regionDeque::const_iterator itEnd = regs.end();
 for (; it != itEnd; ++it) {
  tog->iterateOnRegionDomain(*it, *this);
 }

#if 1
 // factor for thresholding
 const double factor = 0.01;
 displayProfile(*m_profileHoriz, m_countHoriz, "Horiz", true, factor);
 displayProfile(*m_profileVerti, m_countVerti, "Verti", false, factor);

#endif


}

void
RegionProfileComputer::processPoint(const girl::point &pt)
{
 assert(m_imageFeaturePtr);
 // int horizIndex = pt.x() - m_box.xMin();
 // int vertiIndex = pt.y() - m_box.yMin();
 // assert(horizIndex >= 0 && horizIndex < m_profileHoriz.size());
 // assert(vertiIndex >= 0 && vertiIndex < m_profileVerti.size());
 //ise::conversion::PFConversion conv = ise::conversion::changeSpace(m_colorSpace)[0];
 girl::tools::PixelFromImage<girl::tools::ImageFeature::type> pix = m_imageFeaturePtr->getPixel(pt.x(), pt.y());
 // const int v = conv(pix[0], pix[1], pix[2]);
 // m_profileHoriz[m_profileHoriz.keyToIndex(pt.x())] += v;
 // m_profileVerti[m_profileVerti.keyToIndex(pt.y())] += v;
 // m_profileHoriz[m_profileHoriz.keyToIndex(pt.x())] += pix[0];
 // m_profileVerti[m_profileVerti.keyToIndex(pt.y())] += pix[0];

 // count only black pixels
 size_t indexH = m_profileHoriz->keyToIndex(pt.x());
 size_t indexV = m_profileVerti->keyToIndex(pt.y());
 if (pix[0] == 0) {
   (*m_profileHoriz)[indexH]++;
    (*m_profileVerti)[indexV]++;
 }
 m_countHoriz[indexH]++;
 m_countVerti[indexV]++;
}


}
