/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*
 * iseRegionSetBinaryFunctionWeighted.hpp
 *
 *  Created on: Oct 18, 2010
 *      Author: vieux
 */

#ifndef ISEREGIONSETBINARYFUNCTIONWEIGHTED_HPP_
#define ISEREGIONSETBINARYFUNCTIONWEIGHTED_HPP_

#include <girl/girlCommon.hpp>

#include "iseRegionSetParameters.hpp"
#include "iseRegionSetBinaryFunction.hpp"

#include <opencv2/flann/flann.hpp>

namespace ise {
  /**
   * Interface for all classes that need to access to region set features AND
   * the weights associated with a region set, i.e. global weight and dimension
   * weights
   */
  class GIRL_API RegionSetBinaryFunctionWeighted : public RegionBinaryFunctionWithRegSet
  {

  public:

    RegionSetBinaryFunctionWeighted() :
      RegionBinaryFunctionWithRegSet(),
      m_regSetParameters(0) {}

    virtual ~RegionSetBinaryFunctionWeighted() {}


    RegionBinaryFunction::result_type
    operator()(const RegionBinaryFunction::first_argument_type &reg1Id,
	       const RegionBinaryFunction::second_argument_type &reg2Id)
    {
      // TODO:IMPROVE how to correctly handle the dimension weights? :/
      size_t featureDim = m_regSet->getFeatureDim();
      double *wrf1 = new double[featureDim];
      double *wrf2 = new double[featureDim];
      const std::vector<double> weights = m_regSetParameters.getDimWeights();
      RegionFeature *rf1 = m_regSet->getRegionFeatures(reg1Id);
      RegionFeature *rf2 = m_regSet->getRegionFeatures(reg2Id);
      assert(rf1 && rf2);
      for (size_t i = 0; i < featureDim; ++i) {
        wrf1[i] = weights[i]*(*rf1)[i];
        wrf2[i] = weights[i]*(*rf2)[i];
      }
      
      RegionBinaryFunction::result_type result = operator_aux(wrf1, wrf2, featureDim);

      delete [] wrf1;
      delete [] wrf2;
      return m_regSetParameters.getWeight()*result;
    }



    class RegionSetBinaryFunctionWeightedMemento : public RegionBinaryFunctionWithRegSet::RegionSetBinaryFunctionMemento
    {
      friend class RegionSetBinaryFunctionWeighted;

    public:
      RegionSetBinaryFunctionWeightedMemento() : m_regSetParameters(0) {}

    protected:
      virtual void getParameters(const Mementable &m)
      {
        const RegionSetBinaryFunctionWeighted& rs = reinterpret_cast<const RegionSetBinaryFunctionWeighted&>(m);
        RegionSetBinaryFunctionMemento::getParameters(rs);
        m_regSetParameters = rs.m_regSetParameters;
      }

      void setParameters(Mementable &m) const
      {
        RegionSetBinaryFunctionWeighted& rs = reinterpret_cast<RegionSetBinaryFunctionWeighted&>(m);
        RegionSetBinaryFunctionMemento::setParameters(rs);
        rs.m_regSetParameters = m_regSetParameters;
      }


    private:

      RegionSetParameters m_regSetParameters;

    };

  protected:

    virtual RegionBinaryFunction::result_type operator_aux(const double *wrf1, const double *wrf2, size_t featureDim) = 0;


    protected:

    RegionSetParameters m_regSetParameters;
  };

}
#endif /* ISEREGIONSETBINARYFUNCTIONWEIGHTED_HPP_ */
