/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*
 * iseRegionSetCollection.cpp
 *
 *  Created on: Aug 23, 2010
 *      Author: vieux
 */

#include <algorithm>

#include "iseRegionSetCollection.hpp"

namespace ise
{

  RegionSetCollection::RegionSetCollection()
  : m_regionSets()
  {
    // TODO Auto-generated constructor stub

  }

  RegionSetCollection::~RegionSetCollection()
  {
    // REMI: Destructor unnecessary after switch to shared_pointers
    //    ContainerType::iterator it = m_regionSets.begin();
    //    const ContainerType::iterator itEnd = m_regionSets.end();
    //    for ( ; it != itEnd; ++it)
    //      delete (*it).second;
  }

  //  void
  //  RegionSetCollection::add(RegionSetInterface *regSet)
  //  {
  //    m_regionSets[regSet->uri()] = regSet;
  //  }
  //
  //  void
  //  RegionSetCollection::remove(const std::string &uri)
  //  {
  //    m_regionSets.erase(uri);
  //  }

  const girl::shared_ptr<RegionSetInterface>
  RegionSetCollection::getRegionSet(const std::string &uri) const
  {
    ContainerType::const_iterator it = m_regionSets.find(uri);
    if (it != m_regionSets.end()) {
      return (*it).second;
    }
    else {
      return girl::shared_ptr<RegionSetInterface>();
    }
  }

  void
  RegionSetCollection::reset()
  {
    ContainerType::iterator it = m_regionSets.begin();
    const ContainerType::iterator itEnd = m_regionSets.end();
    for ( ; it != itEnd; ++it) {
      (*it).second->reset();
    }
  }

  void
  RegionSetCollection::clear()
  {
    reset();
    m_regionSets.clear();
  }

//  size_t
//  RegionSetCollection::nbReg() {
//#ifndef NDEBUG
//    ContainerType::iterator it = m_regionSets.begin();
//    size_t res = it->second->nbReg();
//    const ContainerType::iterator itEnd = m_regionSets.end();
//    for ( ; it != itEnd; ++it) {
//      assert(it->second->nbReg() == res);
//    }
//#endif
//    assert(m_regionSets.size());
//    return m_regionSets.begin()->second->nbReg();
//  }

  //RegionFeature*
  void
  RegionSetCollection::getRegionFeatures(const std::string &uri, const girl::region &id, RegionFeature& rf) const
  {
    ContainerType::const_iterator it = m_regionSets.find(uri);
    assert(it != m_regionSets.end());
    it->second->getRegionFeatures(id, rf);
    //    if (it != m_regionSets.end()) {
    //      return (*it).second->getRegionFeatures(id);
    //    }
    //    else {
    //#ifndef NDEBUG
    //      std::cerr << "Error in file " << __FILE__ << ':' << __LINE__ << std::endl;
    //      std::cerr << "Requesting region feature " << uri << " which is unknown" << std::endl;
    //#endif
    //      return NULL;
    //    }
  }

  //  const RegionFeature*
  //  RegionSetCollection::getRegionFeatures(const std::string &uri, girl::region id) const
  //  {
  //    ContainerType::const_iterator it = m_regionSets.find(uri);
  //    if (it != m_regionSets.end()) {
  //      return (*it).second->getRegionFeatures(id);
  //    }
  //    else {
  //#ifndef NDEBUG
  //      std::cerr << "Error in file " << __FILE__ << ':' << __LINE__ << std::endl;
  //      std::cerr << "Requesting region feature " << uri << " which is unknown" << std::endl;
  //#endif
  //      return NULL;
  //    }
  //  }

  void
  RegionSetCollection::insertRegion(const girl::region& id)
  {
    ContainerType::iterator it = m_regionSets.begin();
    const ContainerType::iterator itEnd = m_regionSets.end();
    for ( ; it != itEnd; ++it) {
      (*it).second->insertRegion(id);
    }
  }

  void
  RegionSetCollection::replaceRegion(const girl::region& id)
  {
    ContainerType::iterator it = m_regionSets.begin();
    const ContainerType::iterator itEnd = m_regionSets.end();
    for ( ; it != itEnd; ++it) {
      (*it).second->replaceRegion(id);
    }
  }

  void
  RegionSetCollection::renameRegion(const girl::region& oldRegId, const girl::region& newRegId)
  {
    ContainerType::iterator it = m_regionSets.begin();
    const ContainerType::iterator itEnd = m_regionSets.end();
    for ( ; it != itEnd; ++it) {
      // REMI: TODO:UNSURE I implement a renameRegion instead of remove region but is it used anywhere else?

      //RegionFeature *features = it->second->removeRegion(oldRegId);
      //assert (features);
      it->second->renameRegion(oldRegId, newRegId);
    }
  }

  void
  RegionSetCollection::mergeRegions(const girl::region& keptRegion, const girl::region& delRegion)
  {
    ContainerType::iterator it = m_regionSets.begin();
    const ContainerType::iterator itEnd = m_regionSets.end();
    for ( ; it != itEnd; ++it) {
      (*it).second->mergeRegions(keptRegion, delRegion);
    }
  }

//  void
//  RegionSetCollection::removeRegion(girl::region id)
//  {
//    ContainerType::iterator it = m_regionSets.begin();
//    const ContainerType::iterator itEnd = m_regionSets.end();
//    for ( ; it != itEnd; ++it) {
//      (*it).second->removeRegion(id);
//    }
//  }

  void
  RegionSetCollection::deleteRegion(girl::region id)
  {
    ContainerType::iterator it = m_regionSets.begin();
    const ContainerType::iterator itEnd = m_regionSets.end();
    for ( ; it != itEnd; ++it) {
      (*it).second->deleteRegion(id);
    }
  }

  void 
  RegionSetCollection::checkAllRegions()
  {
    ContainerType::iterator it = m_regionSets.begin();
    const ContainerType::iterator itEnd = m_regionSets.end();
    for ( ; it != itEnd; ++it) {
      ise::URI title = it->second->getTitle();
      std::cout << "Displaying feature values for " << title << std::endl;
      it->second->checkAllRegions();
    }
  }

  void
  RegionSetCollection::getURIs(std::vector<ise::URI> &list) const
  {
    list.clear();
    ContainerType::const_iterator it = m_regionSets.begin();
    const ContainerType::const_iterator itEnd = m_regionSets.end();
    for ( ; it != itEnd ; ++it)
      list.push_back((*it).first);
  }

  bool
  RegionSetCollection::getFeatureDescription(const std::string &uri, std::string &title, std::vector<std::string>& dimDesc) const
  {
    ContainerType::const_iterator it = m_regionSets.find(uri);
    if (it != m_regionSets.end()) {
      it->second->getDescription(title, dimDesc);
      return true;
    } else {
      return false;
    }
  }

  bool
  RegionSetCollection::getFeatureTitle(const std::string &uri, std::string &title) const
  {
    ContainerType::const_iterator it = m_regionSets.find(uri);
    if (it != m_regionSets.end()) {
      title = it->second->getTitle();
      return true;
    } else {
      return false;
    }
  }

  bool
  RegionSetCollection::getFeatureDimDescription(const std::string &uri, std::vector<std::string> &dimDesc) const
  {
    ContainerType::const_iterator it = m_regionSets.find(uri);
    if (it != m_regionSets.end()) {
      it->second->getDimDescription(dimDesc);
      return true;
    } else {
      return false;
    }
  }

}

