/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*
 * iseRegionSetFactoryHandler.cpp
 *
 *  Created on: Oct 7, 2010
 *      Author: vieux
 */

#include "iseRegionSetFactoryHandler.hpp"

namespace ise
{

  RegionSetFactoryHandler*
  RegionSetFactoryHandler::instance()
  {
    static RegionSetFactoryHandler* fac = NULL;
    if (fac == NULL) {
      fac = new RegionSetFactoryHandler();
    }
    return fac;
  }

  void
  RegionSetFactoryHandler::addFactory(ise::RegionSetAbstractFactory *factory)
  {
    assert(factory);
    m_factories[factory->uri()] = factory;
  }

  void
  RegionSetFactoryHandler::removeFactory(ise::RegionSetAbstractFactory *factory)
  {
    assert(factory);
    FactoryContainer::iterator it = m_factories.find(factory->uri());
    if (it != m_factories.end()) {
      m_factories.erase(it);
    }
  }

  girl::shared_ptr<RegionSetInterface>
  RegionSetFactoryHandler::newRegionSet(girl::shared_ptr<RegionFeatureComputer> computer)
  {
    FactoryContainer::iterator it = m_factories.find(computer->regSetType());
    assert(it != m_factories.end());
    return it->second->newRegionSet(computer);
  }

}
