/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*
 * iseRegionSetInterfaceConnectedComponentsHistogram.hpp
 *
 *  Created on: Mar 15, 2011
 *      Author: vieux
 */

#ifndef ISEREGIONSETINTERFACECONNECTEDCOMPONENTSHISTOGRAM_HPP_
#define ISEREGIONSETINTERFACECONNECTEDCOMPONENTSHISTOGRAM_HPP_

#include "iseRegionSetInterface.hpp"

#include "iseConnectedComponentTools.hpp"
/**
 * Histogram of Connected Components Sizes
 * cf article:
 * author = {Daniel Keysers and Faisal Shafait and Thomas M. Breuel},
 * title = {Document image zone classification - a simple high-performance approach},
 * booktitle = {2nd Int. Conf. on Computer Vision Theory and Applications},
 * year = {2007},
 *
 * Assume that the region we are passing is a "block" region it is binarised
 * using OpenCV's Otsu then features are extracted according to this binarization
 */
namespace ise
{

  class GIRL_API RegionSetInterfaceConnectedComponentsHistogram : public RegionSetInterface
  {

  public:
    static const FeatureComputer::CacheType DEFAULT_CACHE_STRATEGY = ise::FeatureComputer::NONE;
  public:
    RegionSetInterfaceConnectedComponentsHistogram(ise::SegmentationContext *segContext,
        FeatureComputer::CacheType cache = DEFAULT_CACHE_STRATEGY);
    virtual
    ~RegionSetInterfaceConnectedComponentsHistogram();


    const std::string
    uri() const;

    size_t
    getFeatureDim() const;

    void
    getDimDescription(std::vector<std::string> &dimDesc) const;

    void
    setParameters()
    {}

    void
    getRequiredComputers(FeatureComputerListType&) const
    {}

    const std::string
    help() const;

  protected:
    void
    compute(const girl::region &id, ise::RegionFeature &feature);

    void
    mergeFeatures(RegionFeature& kept,
        RegionFeature& del,
        const girl::region& keptId,
        const girl::region& delId);

    void
    computeHistograms(const ConnectedComponentTools::ConnectedComponents &ccs,
        RegionFeature_Data_Type *widthHisto,
        RegionFeature_Data_Type *heightHisto,
        RegionFeature_Data_Type *jointHisto) const;
  };

}

#endif /* ISEREGIONSETINTERFACECONNECTEDCOMPONENTSHISTOGRAM_HPP_ */
