/*
  Copyright (C) 2004-2011 Boris Mansencal  boris.mansencal@labri.fr

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "iseRegionSetInterfaceContourOrientationHistogram.hpp"

#include <girl/girlToger.hpp>

#include "isePath.hpp"
#include "iseSegmentationContext.hpp"

#ifndef M_PI
//Visual C++ does not define these constants in <cmath>
#define M_PI 3.14159265358979323846
#endif 

namespace ise {
RegionSetInterfaceContourOrientationHistogram::RegionSetInterfaceContourOrientationHistogram(ise::SegmentationContext *segContext)
 : RegionSetInterface(segContext),
   m_numBins(360)
{

}

RegionSetInterfaceContourOrientationHistogram::~RegionSetInterfaceContourOrientationHistogram()
{}

const URI
RegionSetInterfaceContourOrientationHistogram::uri() const
{
 std::stringstream ss(std::ios::in|std::ios::out);
 ss << __FILE__ << m_numBins;
 return ss.str();
}


size_t
RegionSetInterfaceContourOrientationHistogram::getFeatureDim() const
{
 return m_numBins;
}


void
RegionSetInterfaceContourOrientationHistogram::getDimDescription(std::vector<std::string> &dimDesc) const
{
 dimDesc.clear();
 for (size_t i = 0; i < getFeatureDim(); ++i) {
  std::stringstream ss(std::ios::in|std::ios::out);
  ss << "Contour histogram bin " << i;
  dimDesc.push_back(std::string(ss.str()));
 }
}

void
RegionSetInterfaceContourOrientationHistogram::getRequiredComputers(FeatureComputerListType &) const
{
}

const std::string
RegionSetInterfaceContourOrientationHistogram::help() const
{
 return "Compute the the tangeant values of the external contour of regions";
}


void
RegionSetInterfaceContourOrientationHistogram::compute(const girl::region &id, ise::RegionFeature &feature)
{
 assert(m_segContext);
 girl::toger *tog = m_segContext->getToger();
 assert(tog);
 memset(feature.getData(), 0, getFeatureDim()*sizeof(RegionFeature_Data_Type));
 double delta = 2*M_PI/m_numBins;
 girl::contour c;
 tog->regionOuterBoundary(id, c);
 ise::Path path(c);
 size_t length = path.getLength();
 double val = 1.0/length;
 for (size_t i = 0; i < length; ++i) {
  double angle = path.getAngle(i) + M_PI;
  int index = angle/delta;
  assert(index >= 0 && index < (int)m_numBins);
  feature[index] += val;
 }
}

void
RegionSetInterfaceContourOrientationHistogram::mergeFeatures(RegionFeature &keptFeatures, RegionFeature &delFeatures, const girl::region &keptId, const girl::region &delId)
{
 compute(keptId, keptFeatures);
}

}
