/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef ISEREGIONSETINTERFACEDISTRIBUTION_HPP_
#define ISEREGIONSETINTERFACEDISTRIBUTION_HPP_

#include <girl/girlCommon.hpp>
#include <girl/girlSharedPtr.hpp>
#include <girl/girlToger.hpp>

#include "iseImageFeatureComputer.hpp"
#include "iseRegionSetInterface.hpp"

#include <opencv2/core/core.hpp>
#include <opencv2/flann/flann.hpp>
/**
 * Region feature which represent the distribution of region pixels with respect
 * to the quantization of an image feature, using kmeans clustering.
 */
namespace ise
{

  class GIRL_API RegionSetInterfaceDistribution : public RegionSetInterface,
  public girl::toger::pointIterator
  {

  public:
    typedef girl::shared_ptr<ise::ImageFeatureComputer> ImageFeaturePtr;
    static const RegionSetInterface::CacheType DEFAULT_CACHE_STRATEGY = RegionSetInterface::LAZY;
    static const int DEFAULT_CODEBOOK_SIZE = 100;
    static const int DEFAULT_CENTER_FLAGS = cv::KMEANS_PP_CENTERS;
    static const bool DEFAULT_NORMALIZE_DISTRIBUTION = true;

    RegionSetInterfaceDistribution(ise::SegmentationContext *segContext,
        RegionSetInterface::CacheType cacheType = DEFAULT_CACHE_STRATEGY,
        ImageFeaturePtr img = ImageFeaturePtr());

    virtual
    ~RegionSetInterfaceDistribution();


    const
    URI uri() const;

    size_t
    getFeatureDim() const;

    void
    getDimDescription(std::vector<std::string>&) const;

    void
    setParameters(int codebookSize, int flags, const ImageFeaturePtr &img);

    void
    getRequiredComputers(FeatureComputerListType &) const;

    const std::string
    help() const;

    cv::Mat getLabelsImage() const
    {
      return m_labels;
    }

  protected:

    void
    compute(const girl::region &id,
        ise::RegionFeature &feature);

    void
    mergeFeatures(RegionFeature &keptFeatures,
        RegionFeature &delFeatures,
        const girl::region &kept,
        const girl::region &del);

    void
    processPoint(const girl::point &pt);


  protected:

    ImageFeaturePtr m_img;

    // Use the index in case of loading an already computed model
    // but at the moment, we don't support this functionnality
    //   cv::flann::Index_<girl::tools::ImageFeature::type> *m_index;

    cv::Mat m_labels;

    RegionFeature_Data_Type *m_memory;

    int m_codebookSize;
    int m_flags;

    bool m_normalizeDistribution;

    bool m_updateIndex;


  private:

    void
    computeClustering();
  };

}

#endif /* ISEREGIONSETINTERFACEDISTRIBUTION_HPP_ */
