/*
  Copyright (C) 2011 Remi Vieux  remi.vieux@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef ISEREGIONSETINTERFACEDOMAINRUNLENGTHHISTOGRAM_HPP_
#define ISEREGIONSETINTERFACEDOMAINRUNLENGTHHISTOGRAM_HPP_

#include <girl/girlCommon.hpp>

#include "iseRegionSetInterface.hpp"

#include "iseRunLengthExtractor.hpp"

/*
 *
 */
namespace ise
{


  /**
   * Gives the histogram of run length sizes but without differentiation black and white pixels
   * inside the region (unlike RegionSetInterfaceRunLengthHistogram): all pixels inside the region
   * domain are considered as foreground, all pixels outside as background
   */
  class GIRL_API RegionSetInterfaceDomainRunLengthHistogram : public RegionSetInterface
  {
  public:
    static const FeatureComputer::CacheType DEFAULT_CACHE_STRATEGY = ise::FeatureComputer::NONE;


  public:
    RegionSetInterfaceDomainRunLengthHistogram(ise::SegmentationContext *segContext,
        FeatureComputer::CacheType cache = DEFAULT_CACHE_STRATEGY);


    virtual
    ~RegionSetInterfaceDomainRunLengthHistogram();


    const std::string
    uri() const;

    size_t
    getFeatureDim() const;

    void
    getDimDescription(std::vector<std::string> &dimDesc) const;

    void
    setParameters()
    {}

    void
    getRequiredComputers(FeatureComputerListType&) const
    {}

    const std::string
    help() const;

  protected:
    void
    compute(const girl::region &id, ise::RegionFeature &feature);

    void
    mergeFeatures(RegionFeature& kept,
        RegionFeature& del,
        const girl::region& keptId,
        const girl::region& delId);

    void
    computeHistogram(RegionFeature_Data_Type *histo, const std::vector<size_t> &sizes) const;


  };

}

#endif /* ISEREGIONSETINTERFACEDOMAINRUNLENGTHHISTOGRAM_HPP_ */
