/*
  Copyright (C) 2010-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "iseRegionSetInterfaceGravityCenter.hpp"

#include "iseSegmentationContext.hpp"

namespace ise
{


  RegionSetInterfaceGravityCenter::RegionSetInterfaceGravityCenter(ise::SegmentationContext* segContext,
      ise::RegionSetInterface::CacheType cache):
      RegionSetInterface(segContext),
      m_area(0),
      m_x(0),
      m_y(0)
  {
    setCacheState(cache);
  }

  RegionSetInterfaceGravityCenter::~RegionSetInterfaceGravityCenter()
  {
  }

  const URI
  RegionSetInterfaceGravityCenter::uri() const
  {
    std::stringstream ss(std::ios::in|std::ios::out);
    ss << __FILE__ << std::endl;
    return ss.str();
  }

  size_t
  RegionSetInterfaceGravityCenter::getFeatureDim() const
  {
    return FEATURE_DIMENSION;
  }

  void
  RegionSetInterfaceGravityCenter::getDimDescription(std::vector<std::string>& dimDesc) const
  {
    dimDesc.clear();
    dimDesc.push_back("Gravity Center x coordinate");
    dimDesc.push_back("Gravity Center y coordinate");
  }


  void
  RegionSetInterfaceGravityCenter::processPoint(const girl::point& pt)
  {
    ++m_area;
    m_x += pt.x();
    m_y += pt.y();
  }

  const std::string
  RegionSetInterfaceGravityCenter::help() const
  {
    return "This computer returns the coordinates of the gravity center of the region";
  }

  void
  RegionSetInterfaceGravityCenter::compute(const girl::region& id, ise::RegionFeature& feature)
  {
    assert(m_segContext);
    girl::toger* tog = m_segContext->getToger();
    assert(tog);
    m_area = m_x = m_y = 0;
    tog->iterateOnRegionDomain(id, *this);
    feature[0] = m_x / m_area;
    feature[1] = m_y / m_area;
  }

  void
  RegionSetInterfaceGravityCenter::mergeFeatures(RegionFeature& kept, RegionFeature& /*del*/,
      const girl::region& keptReg, const girl::region& /*delReg*/)
  {
    compute(keptReg, kept);
  }
}
