/*
  Copyright (C) 2010-2011 Remi Vieux  remi.vieux@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef ISEREGIONSETINTERFACEGRAVITYCENTER_HPP_
#define ISEREGIONSETINTERFACEGRAVITYCENTER_HPP_

#include <girl/girlCommon.hpp>
#include <girl/girlSharedPtr.hpp> //?

#include "iseRegionSetInterface.hpp"



namespace ise {


  /**
   * Compute the x, y coordinates of the gravity center of the region.
   * TODO:IMPROVE we scan the region domain and compute area/grav at the same time
   * could be better to use already computed area in the moments? Or not?
   */
  class GIRL_API RegionSetInterfaceGravityCenter : public RegionSetInterface,
  public girl::toger::pointIterator
  {

  public:

    static const int FEATURE_DIMENSION = 2;
    static const RegionSetInterface::CacheType DEFAULT_CACHE_STRATEGY = RegionSetInterface::LAZY;

  public:
    RegionSetInterfaceGravityCenter(ise::SegmentationContext* segContext,
        ise::RegionSetInterface::CacheType cacheType = DEFAULT_CACHE_STRATEGY);

    virtual
    ~RegionSetInterfaceGravityCenter();

    const URI
    uri() const;

    size_t
    getFeatureDim() const;

    void
    getDimDescription(std::vector<std::string>& dimDesc) const;

    void
    processPoint(const girl::point& pt);

    void
    getRequiredComputers(FeatureComputerListType &) const {}

    const std::string
    help() const;

  protected:

    void
    compute(const girl::region& id, ise::RegionFeature& feature);

    void
    mergeFeatures(RegionFeature& kept, RegionFeature& del,
        const girl::region& keptId, const girl::region& delId);

  private:
    double m_area;
    double m_x;
    double m_y;
  };

}
#endif /* ISEREGIONSETINTERFACEGRAVITYCENTER_HPP_ */


