/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "iseRegionSetInterfaceHoleNumber.hpp"

#include "iseSegmentationContext.hpp"
#include <girlTools/girlToolsRegionInfos.hpp>

namespace ise
{
  RegionSetInterfaceHoleNumber::RegionSetInterfaceHoleNumber(ise::SegmentationContext* segContext,
      ise::RegionSetInterface::CacheType cache)
  : RegionSetInterface(segContext)
  {
    setCacheState(cache);
  }

  RegionSetInterfaceHoleNumber::~RegionSetInterfaceHoleNumber()
  {
  }

  const URI
  RegionSetInterfaceHoleNumber::uri() const
  {
    std::stringstream ss(std::ios::in|std::ios::out);
    ss << __FILE__ << std::endl;
    return ss.str();
  }

  size_t
  RegionSetInterfaceHoleNumber::getFeatureDim() const
  {
    return FEATURE_DIMENSION;
  }

  void
  RegionSetInterfaceHoleNumber::getDimDescription(std::vector<std::string>& dimDesc) const
  {
    dimDesc.clear();
    dimDesc.push_back("Number of holes");
  }

  const std::string
  RegionSetInterfaceHoleNumber::help() const
  {
    return "Compute the number of holes in the current region";
  }


  void
  RegionSetInterfaceHoleNumber::compute(const girl::region& id, ise::RegionFeature& feature)
  {
    assert(m_segContext);
    assert(m_segContext->getToger());

    feature[0] = girl::tools::getNumberOfChildren(*(m_segContext->getToger()), id);
  }

  void
  RegionSetInterfaceHoleNumber::mergeFeatures(RegionFeature& kept, RegionFeature& /*del*/,
      const girl::region& keptReg, const girl::region& /*delReg*/)
  {
    compute(keptReg, kept);
  }
}
