/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISEREGIONSETINTERFACEIMPL_HPP_
#define ISEREGIONSETINTERFACEIMPL_HPP_

#include <iostream>
#include <string>

//////// DEPRECATED??????
#include <girl/girlCommon.hpp>
#include <girl/girlHashFunc.hpp>
#include <girl/girlHashMap.hpp>
#include <girl/girlToger.hpp>

#include "iseRegionFeature.hpp"
#include "iseRegionFeatureComputer.hpp"
#include "iseRegionSetInterface.hpp"

/**
 * This class handles the code for storing, retrieving and triggering
 * the computation of region features. The actual region feature semantics
 * depends on the delegate feature computer object but it is transparent for
 * the region set.
 */
namespace ise {

  class GIRL_API RegionSetInterfaceImpl : public RegionSetInterface {

  public:

    explicit RegionSetInterfaceImpl(girl::shared_ptr<RegionFeatureComputer> featureComputer);

    virtual ~RegionSetInterfaceImpl();

    /**
     * Return the type for this region set
     */
    static inline const std::string type() { return "iseRegionSetBase"; }

    inline const std::string uri() const { return m_featureComputer->uri(); }

    inline size_t nbReg() const { return m_allRegions.size(); }

   // inline const RegionFeatureComputer& getFeatureComputer() const { return m_featureComputer; }

    inline size_t getFeatureDim() const { return m_featureComputer->size() ;}

    inline void getFeatureDescription(std::string &title, std::vector<std::string>& dimDesc) const
    {
      m_featureComputer->getFeatureDescription(title, dimDesc);
    }

    /**
     * @param title is filled with the name of the feature
     */
    inline void getFeatureTitle(std::string &title) const
    {
      m_featureComputer->getFeatureTitle(title);
    }

    /**
     * Return a semantic description of each dimension of the feature vector
     * Default behaviour: return the dimension index
     * @param dimDesc is filled with a description of the semantics of each
     * dimension of the feature vector
     */
    inline void getFeatureDimDescription(std::vector<std::string> &dimDesc) const
    {
      m_featureComputer->getFeatureDimDescription(dimDesc);
    }


    void reset();

    /**
     * Return features of region @id.
     */
    void getRegionFeatures(const girl::region &id, RegionFeature& rf) const;
   // RegionFeature *getRegionFeatures(girl::region id);

    //const RegionFeature *getRegionFeatures(girl::region id) const;

    /**
     * Will compute the features for the region @id in parameter in all the regsets
     * contained in the composite
     */
    void insertRegion(const girl::toger &tog, const girl::region& id);

    /**
     * Will insert the region feature in the representation. Region feature could
     * have been obtained, e.g. from a rename region operation
     */
    void insertRegion(RegionFeature *features, const girl::region& id);

    /**
     * Update the features for the region @a id for all the regsets in the composite
     */
    void replaceRegion(const girl::toger &tog, const girl::region& id);

    /**
      * Remove region features @a reg for region @a id from region set,
      * and return it.
      *
      * @warning @a id must be in regSet
      * @warning it is client responsibility to remove returned region features.
      *
      * @sa deleteRegion.
      */
   // RegionFeature* removeRegion(const girl::region& id);

    /**
     * Rename region @oldId into @newId
     * Delete features if @oldId was already used
     */
    void renameRegion(const girl::region &oldId, const girl::region &newId);

    void deleteRegion(const girl::region& id);

    void mergeRegions(const girl::toger &tog, const girl::region& keptReg, const girl::region& delReg);

    void checkAllRegions();

  protected:

    /**
     * Delegate object which is in charge of the actual computation of descriptor for the region
     */
    girl::shared_ptr<RegionFeatureComputer> m_featureComputer;

    typedef girl::hashMap<girl::region, ise::RegionFeature*> ContainerType;

    ContainerType m_allRegions;

  };
}

#endif /* ISEREGIONSETINTERFACEIMPL_HPP_ */
