/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef ISEREGIONSETINTERFACEMIXEDTEXTIMAGE_HPP
#define ISEREGIONSETINTERFACEMIXEDTEXTIMAGE_HPP

#include <girl/girlCommon.hpp>

#include "iseRegionSetInterface.hpp"

namespace ise{
  class GIRL_API RegionSetInterfaceMixedTextImage : public RegionSetInterface 
  , public girl::toger::pointIterator
  {
    static const int FEATURE_DIMENSION = 4;
    static const RegionSetInterface::CacheType DEFAULT_CACHE_STRATEGY = RegionSetInterface::LAZY;
  public:

    RegionSetInterfaceMixedTextImage(ise::SegmentationContext* segContext, 
        ise::RegionSetInterface::CacheType cacheType = DEFAULT_CACHE_STRATEGY);

    virtual ~RegionSetInterfaceMixedTextImage();

    inline const URI
    uri() const
    {
      std::stringstream ss(std::ios::in|std::ios::out);
      ss << __FILE__ << std::endl;
      return ss.str();
    }

    inline size_t
    getFeatureDim() const
    {
      return FEATURE_DIMENSION;
    }

    inline void
    getDimDescription(std::vector<std::string>& dimDesc) const
    {
      dimDesc.clear();
      dimDesc.push_back("Height");
      dimDesc.push_back("Eccentricity");
      dimDesc.push_back("Ratio: area on bounding box area");
      dimDesc.push_back("Mean horizontal length");
    }

    void
    processPoint(const girl::point &pt);

    void
    setParameters(std::vector<int> levels, int colorSpace, int channel);

    void
    getParameters(std::vector<int> &levels, int &colorSpace, int &channel) const;

    void
    getRequiredComputers(FeatureComputer::FeatureComputerListType &) const {}

    const std::string
    help() const;

  protected:

    void compute(const girl::region& id, ise::RegionFeature& feature);

    void mergeFeatures(RegionFeature& kept, RegionFeature& del,
        const girl::region& keptId, const girl::region& delId);

  private:
    int m_area;
    int m_x1;
    int m_x2;
    int m_y1;
    int m_y2;
    int m_wb; /* white-black transition */
    int m_blackPixel;

    std::vector<int> m_levels;
    int m_colorSpace;
    int m_channel;
  };
}

#endif /* !  ISEREGIONSETINTERFACEMIXEDTEXTIMAGE_HPP */
