/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*
 * iseRegionSetInterfaceNeighboursPosition.hpp
 *
 *  Created on: Dec 8, 2010
 *      Author: vieux
 */

#ifndef ISEREGIONSETINTERFACENEIGHBOURSPOSITION_HPP_
#define ISEREGIONSETINTERFACENEIGHBOURSPOSITION_HPP_

#include <girl/girlCommon.hpp>
#include <girl/girlSharedPtr.hpp>

#include "iseRegionSetInterface.hpp"
#include "iseRegionSetInterfaceGravityCenter.hpp"

namespace ise
{

  /**
   * Given the quantization of the 2D space as specified by numberOfBins and StartAngle,
   * this feature returns, in that order:
   * - number of neighbours
   * - number of neighbours per bins
   * - mean distance of neighbours per bins
   */
  class GIRL_API RegionSetInterfaceNeighboursPosition : public RegionSetInterface
  {

    typedef girl::shared_ptr<ise::RegionSetInterfaceGravityCenter> GravityCenterPtr;

  public:

    static const size_t DEFAULT_NUMBER_OF_BINS = 8;
    static const double DEFAULT_START_ANGLE;

  public:

    RegionSetInterfaceNeighboursPosition(ise::SegmentationContext* segContext,
        RegionSetInterface::CacheType cache = RegionSetInterface::LAZY);


    ~RegionSetInterfaceNeighboursPosition();

    inline const
    URI uri() const;

    inline size_t
    getFeatureDim() const
    {
      return 2*m_numberOfBins + 1;
    }

    void
    getDimDescription(std::vector<std::string> &dimDesc) const;

    void
      setParameters(GravityCenterPtr regSetGravCenter, const size_t &numberOfBins, const double &startAngle)
    {
      m_regSetGravCenter = regSetGravCenter;
      m_numberOfBins = numberOfBins;
      m_startAngle = startAngle;
      if (m_regSetGravCenter)
	allocateMemory();
    }

    //    inline void
    //    getParameters(size_t &numberOfBins, double &startAngle) const
    //    {
    //      numberOfBins = m_numberOfBins;
    //      startAngle = m_startAngle;
    //    }

    void
    getRequiredComputers(FeatureComputerListType &) const;

    const std::string
    help() const;

  protected:

    void
    compute(const girl::region &id, RegionFeature &feature);

    void
    mergeFeatures(RegionFeature &kept,
        RegionFeature &del,
        const girl::region &keptId,
        const girl::region &delId);

    void allocateMemory();

    void freeMemory();

  protected:

    size_t m_numberOfBins;

    double m_startAngle;

    GravityCenterPtr m_regSetGravCenter;

    RegionFeature_Data_Type *m_memory;
  };


}

#endif /* ISEREGIONSETINTERFACENEIGHBOURSPOSITION_HPP_ */
