/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*
 * iseRegionSetInterfaceStrokeCharacteristics.hpp
 *
 *  Created on: Mar 2, 2011
 *      Author: vieux
 */

#ifndef ISEREGIONSETINTERFACESTROKECHARACTERISTICS_HPP_
#define ISEREGIONSETINTERFACESTROKECHARACTERISTICS_HPP_

#include "iseRegionSetInterface.hpp"

/**
 * Compute stroke characteristics, i.e. the minimal and maximal thickness of a region in X coordinates and Y coordinates
 * feature[3] = width of stroke (max) as the max distance of the non-region pixel
 */
namespace ise
{

  class GIRL_API RegionSetInterfaceStrokeCharacteristics : public RegionSetInterface
  {
  public:
    static const RegionSetInterface::CacheType DEFAULT_CACHE_STRATEGY = RegionSetInterface::ALWAYS;
    static const size_t FEATURE_DIM = 4;

  public:
    RegionSetInterfaceStrokeCharacteristics(ise::SegmentationContext *segContext,
        RegionSetInterface::CacheType cacheType = DEFAULT_CACHE_STRATEGY);

    virtual
    ~RegionSetInterfaceStrokeCharacteristics();


    const std::string
    uri() const;


    inline size_t
    getFeatureDim() const
    {
      return FEATURE_DIM;
    }


    void
    getDimDescription(std::vector<std::string> &dimDesc) const;

    inline void
    getRequiredComputers(FeatureComputerListType&) const
    {}

    const std::string
    help() const;


  protected:

    void
    compute(const girl::region &id,
        ise::RegionFeature &feature);

    inline void
    mergeFeatures(RegionFeature &keptFeatures,
        RegionFeature &delFeatures,
        const girl::region&,
        const girl::region&);


    void
    computeDistToOther(const girl::region &id,
        ise::RegionFeature &feature);





  };

}

#endif /* ISEREGIONSETINTERFACESTROKECHARACTERISTICS_HPP_ */
