/*
  Copyright (C) 2011-2012 Jean-Philippe Domenger, Boris Mansencal, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
#ifndef ISEREGIONSETINTERFACETEXTBLOCKCLASSIFIER_HPP
#define ISEREGIONSETINTERFACETEXTBLOCKCLASSIFIER_HPP



/**
 * Classifier trained on Prima dataset to recognize text/non text regions.
 * The classifier used is a dtree classifier, and the features are run length histograms and black and
 * white pixels in horizontal, vertical, right and left diagonal directions, histogram of connected
 * components width and height and joint histo of width and height
 */

#include "iseRegionSetInterface.hpp"
#include <opencv2/ml/ml.hpp>

namespace ise
{


  class GIRL_API RegionSetInterfaceTextBlockClassifier : public RegionSetInterface
  {

  public:

    static const FeatureComputer::CacheType DEFAULT_CACHE_STRATEGY = ise::FeatureComputer::NONE;
   
    enum {TEXT=0,
	  NON_TEXT};

  public:
    /**
     * @param classifierDataFile a file containing the saved classifier, which has been trained externally.
     */
    RegionSetInterfaceTextBlockClassifier(ise::SegmentationContext *segContext,
        const std::string &classifierDataFile);
    virtual
    ~RegionSetInterfaceTextBlockClassifier();

    const std::string
    uri() const;

    size_t
    getFeatureDim() const;

    void
    getDimDescription(std::vector<std::string> &dimDesc) const;

    void
    setParameters()
    {}

    void
    getRequiredComputers(FeatureComputerListType&) const
    {}

    const std::string
    help() const;

  protected:
    void
    compute(const girl::region &id, ise::RegionFeature &feature);

    void
    mergeFeatures(RegionFeature& kept,
        RegionFeature& del,
        const girl::region& keptId,
        const girl::region& delId);

    void
    init(const std::string &filename);

    void
    loadClassifier(const std::string &filename);


  protected:

#if CV_MAJOR_VERSION*100+CV_MINOR_VERSION*10+CV_SUBMINOR_VERSION < 300
    CvBoost m_classifier;
#else
    cv::Ptr<cv::ml::Boost> m_classifier;
#endif
    cv::Mat m_query;
    RegionFeature_Data_Type *m_memory;

  };

}

#endif // ISEREGIONSETINTERFACETEXTBLOCKCLASSIFIER_HPP
