/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*
 * RegionSetInterfaceViewArea.hpp
 *
 *  Created on: Nov 15, 2010
 *      Author: vieux
 */

#ifndef ISEREGIONSETINTERFACEVIEWAREA_HPP_
#define ISEREGIONSETINTERFACEVIEWAREA_HPP_

#include "iseRegionSetInterfaceView.hpp"

/**
 * Return the region area, extracted from the generic moment
 */
namespace ise
{

  class RegionSetInterfaceViewArea : public RegionSetInterfaceView
  {
  public:
    RegionSetInterfaceViewArea(girl::shared_ptr<ise::RegionSetInterface> regSet)
    : RegionSetInterfaceView(regSet) {}


    /**
     * @return uri unique identifier of the region set
     */
    virtual const std::string uri() const = 0;


    /**
     * @return the dimensionality of the feature vector
     */
    inline size_t
    getFeatureDim() const
    {
      return 1;
    }

    /**
     * @param title is filled with the name of the feature
     */
    inline void
    getFeatureTitle(std::string &title) const
    {
      title = "Region Feature Area";
    }

    /**
     * Return a semantic description of each dimension of the feature vector
     * Default behaviour: return the dimension index
     * @param dimDesc is filled with a description of the semantics of each
     * dimension of the feature vector
     */
    inline void
    getFeatureDimDescription(std::vector<std::string> &dimDesc) const
    {
      dimDesc.push_back("Area");
    }

    /**
     * Return a copy of the features of region @id in the pointer @rf
     * Warning: memory must be preallocated, features are copied
     */
    virtual void getRegionFeatures(const girl::region &id, RegionFeature& rf) const = 0;


    /**
     * Debugging function
     */
    virtual void checkAllRegions() = 0;

  };

}

#endif /* ISEREGIONSETINTERFACEVIEWAREA_HPP_ */
