/*
  Copyright (C) 2010-2011 Remi Vieux  remi.vieux@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef ISEREGIONSETSTATISTICAL_HPP_
#define ISEREGIONSETSTATISTICAL_HPP_

#include <girl/girlSharedPtr.hpp>

#include "iseRegionSetInterface.hpp"
/**
 * Class to hold statistical region features, that is the region features
 * are the moments of an image feature
 */
namespace ise
{

  class GIRL_API RegionSetStatistical : public RegionSetInterface
  {
  public:
    explicit RegionSetStatistical(girl::shared_ptr<RegionSetInterface> delegate)
    : m_delegate(delegate) {}

    inline
    ~RegionSetStatistical() {}


    static inline const std::string type() { return "iseRegionSetStatistical"; }

    /**
     * @return the vector dimension of m1, m2
     */
    size_t
    getMomentDim() const;

    /**
     * Return m0 for region @param id
     */
    const RegionFeature_Data_Type* m0(const girl::region id) const;

    /**
     * Return m1 for region @param id
     */
    const RegionFeature_Data_Type* m1(const girl::region id) const;

    /**
     * Return m2 for region @param id
     */
    const RegionFeature_Data_Type* m2(const girl::region id) const;

    /**
     * Return mean for region @param id
     * WARNING: client must free allocated memory
     */
    RegionFeature_Data_Type* mean(const girl::region id) const;


  public:
    // delegated functions
    inline const std::string uri() const { return m_delegate->uri(); }
    inline size_t nbReg() const { return m_delegate->nbReg(); }
    inline size_t getFeatureDim() const { return m_delegate->getFeatureDim(); }
    inline void getFeatureDescription(std::string &title, std::vector<std::string>& dimDesc) const
    {
      return m_delegate->getFeatureDescription(title, dimDesc);
    }
    inline void getFeatureTitle(std::string &title) const { return m_delegate->getFeatureTitle(title); }
    inline void getFeatureDimDescription(std::vector<std::string> &dimDesc) const
    {
      return m_delegate->getFeatureDimDescription(dimDesc);
    }
    inline void reset() { return m_delegate->reset(); }



    inline void getRegionFeatures(const girl::region &id, RegionFeature& rf) const
    {
      m_delegate->getRegionFeatures(id, rf);
    }
//    inline RegionFeature *getRegionFeatures(girl::region id)
//    {
//      return m_delegate->getRegionFeatures(id);
//    }
//    inline const RegionFeature *getRegionFeatures(girl::region id) const
//    {
//      return m_delegate->getRegionFeatures(id);
//    }
    inline void insertRegion(const girl::toger &tog, girl::region id)
    {
      return m_delegate->insertRegion(tog, id);
    }
    inline void insertRegion(RegionFeature *features, girl::region id)
    {
      return m_delegate->insertRegion(features, id);
    }
    inline void replaceRegion(const girl::toger &tog, girl::region id)
    {
      return m_delegate->replaceRegion(tog, id);
    }
//    inline RegionFeature* removeRegion(girl::region id)
//        {
//      return m_delegate->removeRegion(id);
//        }
   inline void deleteRegion(girl::region id)
    {
      return m_delegate->deleteRegion(id);
    }
    inline void mergeRegions(const girl::toger &tog, girl::region keptReg, girl::region delReg)
    {
      return m_delegate->mergeRegions(tog, keptReg, delReg);
    }

    inline void checkAllRegions() { return m_delegate->checkAllRegions(); }


  protected:

    girl::shared_ptr<RegionSetInterface> m_delegate;
  };

}

#endif /* ISEREGIONSETSTATISTICAL_HPP_ */
