/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
#ifndef ISEREGIONSETUNARYFUNCTIONWEIGHTED_HPP_
#define ISEREGIONSETUNARYFUNCTIONWEIGHTED_HPP_

#include <girl/girlCommon.hpp>

#include "iseRegionSetParameters.hpp"
#include "iseRegionSetUnaryFunction.hpp"

#include <opencv2/flann/flann.hpp>

namespace ise {
  /**
   * Interface for all classes that need to access to region set features AND
   * the weights associated with a region set, i.e. global weight and dimension
   * weights
   */
  class GIRL_API RegionSetUnaryFunctionWeighted : public RegionUnaryFunctionWithRegSet
  {

  public:

    RegionSetUnaryFunctionWeighted() :
      RegionUnaryFunctionWithRegSet(),
      m_regSetParameters(0) {}

    virtual ~RegionSetUnaryFunctionWeighted() {}


    class RegionSetUnaryFunctionWeightedMemento : public RegionUnaryFunctionWithRegSet::RegionSetUnaryFunctionMemento
    {
      friend class RegionSetUnaryFunctionWeighted;

    public:
      RegionSetUnaryFunctionWeightedMemento() : m_regSetParameters(0) {}

    protected:
      virtual void getParameters(const Mementable &m)
      {
        const RegionSetUnaryFunctionWeighted& rs = reinterpret_cast<const RegionSetUnaryFunctionWeighted&>(m);
        RegionSetUnaryFunctionMemento::getParameters(rs);
        m_regSetParameters = rs.m_regSetParameters;
      }

      void setParameters(Mementable &m) const
      {
        RegionSetUnaryFunctionWeighted& rs = reinterpret_cast<RegionSetUnaryFunctionWeighted&>(m);
        RegionSetUnaryFunctionMemento::setParameters(rs);
        rs.m_regSetParameters = m_regSetParameters;
      }

    private:

      RegionSetParameters m_regSetParameters;

    };


    protected:

    RegionSetParameters m_regSetParameters;
  };

}
#endif /* ISEREGIONSETUNARYFUNCTIONWEIGHTED_HPP_ */
