/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "iseRegularGridPartitionner.hpp"
#include "iseSegmentationContext.hpp"


using namespace ise;

RegularGridPartitionner::RegularGridPartitionner(SegmentationContext *segContext, int width, int height) 
  : SegmentationClustering(segContext)
  , m_celWidth(width)
  , m_celHeight(height)
{}

void
RegularGridPartitionner::setSegContext(ise::SegmentationContext *s) 
{
  SegmentationClustering::setSegContext(s);
}

bool RegularGridPartitionner::same(const girl::point& p1, girl::direction dir)
{
  girl::point p2 = p1;
  p2.move(dir);

  const int cx1 = p1.x()/m_celWidth;
  const int cy1 = p1.y()/m_celHeight;
  const int cx2 = p2.x()/m_celWidth;
  const int cy2 = p2.y()/m_celHeight;
  
  return ((cx1 == cx2) && (cy1 == cy2));
}

void 
RegularGridPartitionner::setParameters(int width, int height)
{
  m_celWidth = width;
  m_celHeight = height;
} 

void 
RegularGridPartitionner::getParameters(int &width, int &height) const
{
  width = m_celWidth;
  height = m_celHeight;
} 
