/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISEREGULARGRIDPARTITIONNER_HPP
#define ISEREGULARGRIDPARTITIONNER_HPP

#include <girl/girlToger.hpp>
#include "iseSegmentationClustering.hpp"

namespace ise {

  class GIRL_API RegularGridPartitionner :  public SegmentationClustering 
  {
  public:
    RegularGridPartitionner(SegmentationContext *segContext, int width = 10, int height = 10);

    virtual ~RegularGridPartitionner() {}
  
    virtual void setSegContext(ise::SegmentationContext *s);

    virtual void setParameters(int width, int height);

    virtual void getParameters(int &width, int &height) const;

    virtual bool same(const girl::point &p, girl::direction dir);
  
    void setSize(int gridWidth, int gridHeight)
    {
      m_celWidth = gridWidth;
      m_celHeight = gridHeight;
    }
  
    void getSize(int &width, int &height)
    {
      width = m_celWidth;
      height = m_celHeight;
    }
  
    int getCelWidth() const
    {
      return m_celWidth;
    }

    void setCelWidth(int celWidth)
    {
      m_celWidth = celWidth;
    }

    int getCelHeight() const
    {
      return m_celHeight;
    }

    void setCelHeight(int celHeight)
    {
      m_celHeight = celHeight;
    }

    virtual void init(const girl::regionDeque&, int /* level */ =1) {}

    virtual void init(girl::region /* r */, int /* level */) {}

  private:
    RegularGridPartitionner(const RegularGridPartitionner&);
    RegularGridPartitionner& operator=(const RegularGridPartitionner&);

    int m_celWidth;
    int m_celHeight;
  };

}
#endif /* REGULARGRIDPARTITIONNER_HPP_ */
