/*
  Segmentation clustering: binary relaxation.
  Copyright (C) 2005-2009 Claire Morand, Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISESEGMENTATIONCLUSTERINGBINARYRELAXATION_HPP
#define ISESEGMENTATIONCLUSTERINGBINARYRELAXATION_HPP


#include <girl/girlCommon.hpp>

#include "iseSegmentationClustering.hpp"


namespace ise {

  class SegmentationContext;

  class GIRL_API SegmentationClusteringBinaryRelaxation : public SegmentationClustering
  {
    
  public:
    
    /**
     * @brief Destructor.
     */
    ~SegmentationClusteringBinaryRelaxation();
  
    /**
     * @brief Constructor.
     * 
     */
    SegmentationClusteringBinaryRelaxation(SegmentationContext *segContext,
					   double initCoeff = 0.7, 
					   double relaxCoeff1 = 0.2, 
					   double relaxCoeff2 = 0.3);
    
    virtual void init(const girl::regionDeque &regions, int level);
    
    virtual void init(girl::region r, int level);
    
    void setParameters(double initCoeff, double relaxCoeff1, double relaxCoeff2) 
    { 
      m_initCoeff = initCoeff;
      m_relaxCoeff1 = relaxCoeff1;
      m_relaxCoeff2 = relaxCoeff2;
    }

    void getParameters(double &initCoeff, double &relaxCoeff1, double &relaxCoeff2) const 
    { 
      initCoeff   = m_initCoeff;
      relaxCoeff1 = m_relaxCoeff1;
      relaxCoeff2 = m_relaxCoeff2;
    }
   
    virtual bool same(const girl::point &p, girl::direction dir);
    
  private:

    SegmentationClusteringBinaryRelaxation(const SegmentationClusteringBinaryRelaxation &);
    SegmentationClusteringBinaryRelaxation &operator=(const SegmentationClusteringBinaryRelaxation &);

  protected:

    std::vector<double> m_P_C1;
    double m_initCoeff;
    double m_relaxCoeff1;
    double m_relaxCoeff2;
  };

}

#endif /* ! ISESEGMENTATIONCLUSTERINGBINARYRELAXATION_HPP */
