/*
  SegmentationClustering KMeans class.
  Copyright (C) 2005-2009 Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "iseSegmentationClusteringCvKMeans.hpp"

#include "iseSegmentationContext.hpp"

#include <girlTools/girlToolsImage.hpp>
#include <girlTools/girlToolsImageFeature.hpp>

#include <opencv2/core/core.hpp>


using namespace ise;


SegmentationClusteringCvKMeans::~SegmentationClusteringCvKMeans()
{

}

SegmentationClusteringCvKMeans::SegmentationClusteringCvKMeans(SegmentationContext *segContext, 
							       int nbClasses)
  :
  SegmentationClustering(segContext), 
  m_nbClasses(nbClasses),
  m_imageFeatureUri(), 
  m_cvKMeans(nbClasses)
{

}


bool
SegmentationClusteringCvKMeans::same(const girl::point &p1, girl::direction dir)
{
  girl::point p2 = p1;
  p2.move(dir);

  girl::shared_ptr<girl::tools::ImageFeature> imgf = m_segContext->getImageFeature(m_imageFeatureUri);
  int channels = imgf->channels();
  girl::tools::ImageFeature::type *buf1 = imgf->cvMat()->ptr<girl::tools::ImageFeature::type>(p1.y());
  buf1 += channels * p1.x();

  girl::tools::ImageFeature::type *buf2 = imgf->cvMat()->ptr<girl::tools::ImageFeature::type>(p2.y());
  buf2 += channels * p2.x();

  int center1 = m_cvKMeans.getCluster(buf1, channels);
  int center2 = m_cvKMeans.getCluster(buf2, channels);

  return ( center1 == center2);
}

void 
SegmentationClusteringCvKMeans::init(const girl::regionDeque &/*regions*/, int /*level*/)
{ 
  assert(m_segContext);
  assert(!m_segContext->getImageFeature(m_imageFeatureUri)->isNull());
  
  const girl::shared_ptr<girl::tools::ImageFeature> mat = m_segContext->getImageFeature(m_imageFeatureUri);
  const int width = mat->width();
  const int height = mat->height();
  const int channels = mat->channels();

  //m_width = mat->width();

  //cv::Mat data = mat->cvMat()->reshape(1);
  cv::Mat data(height*width, channels,  CV_MAKETYPE(cv::DataDepth<double>::value, 1));

  for (int y = 0 ; y < height ; ++y) {
    girl::tools::ImageFeature::type *buf = mat->scanLine(y);
    for (int x = 0 ; x < width ; ++x) {
      float *bufDest = data.ptr<float>(y*width+x);
      for (int c = 0 ; c < channels ; ++c) {
	bufDest[c] = buf[c];
      }
      buf += mat->formatSize();
    }
  }


  m_cvKMeans.compute(data);
}

void 
SegmentationClusteringCvKMeans::init(girl::region /*r*/, int /*level*/)
{
  
}
		     
