/*
  Quantification Kmeans class.
  Copyright (C) 2005-2009 Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISESEGMENTATIONCLUSTERINGCVKMEANS_HPP
#define ISESEGMENTATIONCLUSTERINGCVKMEANS_HPP


#include <girl/girlCommon.hpp>

#include <girlTools/girlToolsImageFeature.hpp>

#include "iseSegmentationClustering.hpp"
#include "iseDistanceComputer.hpp"
#include "iseCvKMeans.hpp"


namespace ise {

  class GIRL_API SegmentationClusteringCvKMeans : public SegmentationClustering
  {
    
  public:
    
    /**
     * @brief Destructor.
     */
    virtual ~SegmentationClusteringCvKMeans();
  
    /**
     * @brief Constructor.
     * 
     * @warning takes ownership of @a histo.
     */
    SegmentationClusteringCvKMeans(SegmentationContext *segContext, 
				   int nbClasses = 4);

    virtual void init(const girl::regionDeque &regions, int level);
    
    virtual void init(girl::region r, int level);
    
    void setParameters(int nbClasses, std::string imageFeatureUri) 
    { 
      m_nbClasses = nbClasses;
      m_imageFeatureUri = imageFeatureUri;
    }

    void getParameters(int &nbClasses, std::string &imageFeatureUri) const
    { 
      nbClasses = m_nbClasses;
      imageFeatureUri = m_imageFeatureUri;
    }

  protected :
    
    virtual bool same(const girl::point &p1, girl::direction dir);
    
  private:
    SegmentationClusteringCvKMeans(const SegmentationClusteringCvKMeans &);
    SegmentationClusteringCvKMeans &operator=(const SegmentationClusteringCvKMeans &);

  protected:
    int m_nbClasses;
    std::string m_imageFeatureUri;
    ise::CvKMeans m_cvKMeans;
  };

}

#endif /* ! ISESEGMENTATIONCLUSTERINGCVKMEANS_HPP */
