/*
  Copyright (C) 2005-2011 Remi Vieux, Jean-Philippe Domenger, Boris Mansencal  boris.mansencal@labri.fr

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISESEGMENTATIONCLUSTERINGDOCUMENTPRESEG2_HPP_
#define ISESEGMENTATIONCLUSTERINGDOCUMENTPRESEG2_HPP_

#include <girl/girlToger.hpp>
#include <girl/girlHashFunc.hpp>
#include <girl/girlUnorderedSet.hpp>

#include <girlTools/girlToolsImage.hpp>
#include "iseNeighbouringRegions.hpp"
#include "iseSegmentationClustering.hpp"
#include "iseSegmentationContext.hpp"

/*
 *
 */
namespace ise
{

class SegmentationContext;

class GIRL_API SegmentationClusteringDocumentPreseg2 : public SegmentationClustering
{

public:

  static const size_t DEFAULT_AREA_THRESHOLD = 10;
  static const double DEFAULT_RATIO_THRESHOLD;
  static const double DEFAULT_COMPACITY_THRESHOLD;

public:
  explicit SegmentationClusteringDocumentPreseg2(SegmentationContext *segContext);
  virtual
  ~SegmentationClusteringDocumentPreseg2();
  
  // WARNING: R: Set this function for convenience to be able to save images related to the segmentation process
  // do not use it for other than debugging purposes!!
  void
    setParameters(const std::string &imageName);

  void
  getParameters();

  void
  init(const girl::regionDeque &regs, int level = 1);

  void
  init(girl::region, int);

  bool
  same(const girl::point&, girl::direction);

  void
    getRegionsByType(const ise::SegmentationContext *segContext, girl::regionDeque &textRegions, girl::regionDeque &imageRegions, girl::regionDeque &separatorRegions) const;


protected:

  void
  computeTogers();

  void
  computeBinarizationToger();

  void
  computeConnectedComponentsStatistics(const cv::Mat &binaryMask,
                                       cv::Mat &ccLabels,
                                       std::vector<ise::NeighbouringRegions> &spacingsHoriz,
                                       std::vector<ise::NeighbouringRegions> &spacingsVerti,
                                       std::vector<size_t> &ccWidths,
                                       std::vector<size_t> &ccHeights);

  void
  computeEnclosingShapeToger();

  void
  computeImageToger();

  void
  computeSeparatorToger();

  void
  computeWordToger();

  void
  saveToger(const girl::toger& tog, const std::string &name);

  bool
  mustProcessBlock(const girl::region r, const cv::Mat &imageMat, const cv::Mat &ccMat) const;

  void
  processBlock(const girl::region r, girl::regionDeque &regs);

  void
  bitmapSplit(ise::SegmentationContext &segContext, const girl::region r, const cv::Mat &bitmap, girl::regionDeque &createdRegs);

//  void
//  updateRegionNumber(const girl::toger &togOrig,
//                     girl::region &reg,
//                     const girl::toger &togSplitted,
//                     const cv::Mat &ccMat) const;

  // old

  void
  computeParagraphSplit(ise::SegmentationContext &segContext,
                        const girl::regionDeque &regs,
                        girl::regionDeque &createdRegs,
                        int threshHoriz,
                        int threshVerti);
  

protected:


  // info related to binarization
  girl::toger m_binarizationToger;
  cv::Mat     m_binarizationMask;
  girl::regionDeque m_binarizationFgRegions;

  // info to compute block statistics
  cv::Mat m_ccLabels;
  std::vector<ise::NeighbouringRegions> m_spacingsHoriz;
  std::vector<ise::NeighbouringRegions> m_spacingsVerti;
  std::vector<size_t> m_ccWidths;
  std::vector<size_t> m_ccHeights;

  // info related to enclosing shapes
  girl::toger m_enclosingShapeToger;
  girl::regionDeque m_candidateNonTextRegions;


  // info related to separators
  girl::unorderedSet<girl::region> m_separatorRegions; // id of separator regions identified in the binarization toger

  // info related to images
  girl::regionDeque m_imageRegions; // id of regions identified as image in the enclosing shape toger
  cv::Mat m_imageMask;

  // info related to words
  girl::toger m_wordToger;
  cv::Mat m_wordMask;
  cv::Mat m_wordCCLabels;
  std::vector<ise::NeighbouringRegions> m_wordSpacingsHoriz;
  std::vector<ise::NeighbouringRegions> m_wordSpacingsVerti;
  std::vector<size_t> m_wordCcWidths;
  std::vector<size_t> m_wordCcHeights;



  // REMOVE?
  //girl::toger m_imageToger;
  girl::toger m_toger;
  ise::SegmentationContext m_segContext2;
//  girl::tools::Image m_image;
  cv::Mat m_image;

};

}

#endif /* ISESEGMENTATIONCLUSTERINGDOCUMENTPRESEG_HPP_ */
