/*
  Copyright (C) 2005-2011 Remiv Vieux, Roigt Julian, Boris Mansencal  remi.vieux@labri.fr

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
#include "iseSegmentationClusteringEnclosingShape.hpp"

#include <girlTools/girlToolsImage.hpp>

#include "iseConnectedComponentTools.hpp"
#include "iseSegmentationContext.hpp"


//#include <opencv2/highgui/highgui.hpp>


static cv::Mat g_aggregateCCMat;

void
OnTrackBar(int, void*);

namespace ise
{

  SegmentationClusteringEnclosingShape::SegmentationClusteringEnclosingShape(ise::SegmentationContext *segContext) :
                        SegmentationClustering(segContext),
                        m_shapes(),
                        m_shapeFactory(),
                        m_mat(),
                        m_mergeMethod(UNION)
  {

  }

  SegmentationClusteringEnclosingShape::~SegmentationClusteringEnclosingShape()
  {
  }


  void
  SegmentationClusteringEnclosingShape::initExclusive()
  {
    assert(m_segContext->getImage());
    int width  = m_segContext->getImage()->width();
    int height = m_segContext->getImage()->height();
    cv::Scalar BG_COLOR = CV_RGB(BACKGROUND, BACKGROUND, BACKGROUND);
    //cv::Scalar FG_COLOR = CV_RGB(FOREGROUND, FOREGROUND, FOREGROUND);
    cv::Mat tmp = cv::Mat::zeros(height, width, CV_8U);

    m_mat = cv::Mat(height, width, CV_32S, BG_COLOR);


    ShapeList::const_iterator it = m_shapes.begin();
    ShapeList::const_iterator itEnd = m_shapes.end();
    // TODO: OPTIM kind of bourrin comme methode
    int color = 1;
    for (int i = 0; it != itEnd; ++it, ++i) {
      //        std::cout << "cc " << i << std::endl;
        girl::hashMap<int, int> correspondanceMap;

        (*it)->draw(tmp, CV_RGB(1, 1, 1), CV_FILLED);



        //   std::cout << "color=" << color << std::endl;
        //  assert(color < std::numeric_limits<int>::max());

        // OPTIMISED: work on  ROI only
        cv::Rect boundingRect = (*it)->boundingRect();
        boundingRect.x = std::max(0, boundingRect.x);
        boundingRect.y = std::max(0, boundingRect.y);
        boundingRect.width = std::min(boundingRect.width, width - boundingRect.x);
        boundingRect.height = std::min(boundingRect.height, height - boundingRect.y);
        assert(boundingRect.x >= 0);
        assert(boundingRect.y >= 0);
        assert(boundingRect.br().x <= width);
        assert(boundingRect.br().y <= height);
        cv::Mat matRoi = m_mat(boundingRect);
        cv::Mat tmpRoi = tmp(boundingRect);

        cv::MatIterator_<int> matIt = matRoi.begin<int>();
        cv::MatConstIterator_<unsigned char> tmpIt = tmpRoi.begin<unsigned char>();
        cv::MatConstIterator_<unsigned char> tmpItEnd = tmpRoi.end<unsigned char>();
        int label = color;
        for ( ; tmpIt != tmpItEnd; ++tmpIt, ++matIt) {
            if (*tmpIt) {
                // we are on the shape
                if (*matIt) {
                    // another shape was here
                    // check if known
                    girl::hashMap<int, int>::iterator it = correspondanceMap.find(*matIt);
                    if (it == correspondanceMap.end()) {
                        correspondanceMap[*matIt] = ++color;
                        *matIt = color;
                    } else {
                        *matIt = it->second;
                    }
                } else {
                *matIt = label;
		}
            }
        }
        ++color;
//
//        int *matBuf = matRoi.ptr<int>(0);
//        int *tmpBuf = tmpRoi.ptr<int>(0);
//        (*it)->draw(tmp, CV_RGB(1, 1, 1), CV_FILLED);
//        matRoi += tmpRoi;
//        // m_mat += 1;
//        // cv::imshow("Shape", m_mat);
//        // cv::imshow("Tmp", tmp);
//        // cv::waitKey(0);
//        //        double min, max;
//        //        cv::minMaxLoc(matRoi, &min, &max);
//        //        std::cout << "Max = " << max << std::endl;
//        //	double val = color + max + 1;
//        //	assert(val < std::numeric_limits<int>::max());
//        //  color += max + 1;
        tmpRoi.setTo(0);

    }

  }


  void
  SegmentationClusteringEnclosingShape::init(const girl::regionDeque &, int)
  {

    if (m_mergeMethod == EXCLUSIVE) {
        initExclusive();
        return;
    }
    assert(m_segContext->getImage());
    int width  = m_segContext->getImage()->width();
    int height = m_segContext->getImage()->height();
    cv::Scalar BG_COLOR = CV_RGB(BACKGROUND, BACKGROUND, BACKGROUND);
    cv::Scalar FG_COLOR = CV_RGB(FOREGROUND, FOREGROUND, FOREGROUND);

    m_mat = cv::Mat(height, width, CV_8U, BG_COLOR);

    ShapeList::const_iterator it = m_shapes.begin();
    ShapeList::const_iterator itEnd = m_shapes.end();

    int thickness = (m_mergeMethod == UNION) ? CV_FILLED : 0;

    for ( ; it != itEnd; ++it) {
        (*it)->draw(m_mat, FG_COLOR, thickness);
    }

    if (m_mergeMethod) {
        assert(m_shapeFactory);
        aggregateCC(m_mat, m_shapes, *m_shapeFactory, 8);

        m_mat = cv::Mat(height, width, CV_8U, BG_COLOR);
        for (it = m_shapes.begin(); it != m_shapes.end(); ++it) {
            (*it)->draw(m_mat, FG_COLOR, CV_FILLED);
        }
    }

    //    cv::namedWindow("Seg mask", 0);
    //    cv::imshow("Seg mask", m_mat);
    //    cv::waitKey(0);
  }

  bool
  SegmentationClusteringEnclosingShape::same(const girl::point &pt, girl::direction dir)
  {
    //assert(m_mat.type() == CV_32S);
    girl::point ptn(pt);
    ptn.move(dir);
    
    switch(m_mat.type()) {
    case CV_8U: 
      return m_mat.at<unsigned char>(pt.y(), pt.x()) == m_mat.at<unsigned char>(ptn.y(), ptn.x());
      break;
    case CV_32S:
      return m_mat.at<int>(pt.y(), pt.x()) == m_mat.at<int>(ptn.y(), ptn.x());
      break;
    default:
      assert(false);
      break;
    }

	assert(false);
	return false;
  }


  std::vector<EnclosingShapeFactory::EnclosingShapePtr>
  SegmentationClusteringEnclosingShape::aggregateCC(cv::Mat &input, 
      std::vector<EnclosingShapeFactory::EnclosingShapePtr> &cc,
      const EnclosingShapeFactory &factory,
      int connectivity)
  {

    std::vector<EnclosingShapeFactory::EnclosingShapePtr> shapes;
    extractCC(input, shapes, factory, connectivity);
    //std::cout << "Current CC for iteration " << shapes.size() << std::endl;
    if (shapes.size() != cc.size()) {
        g_aggregateCCMat = cv::Mat::zeros(input.rows, input.cols, CV_8U);
        for (unsigned int i = 0; i < shapes.size(); ++i) {
            shapes[i]->draw(g_aggregateCCMat, CV_RGB(255, 255, 255), 0);
            //     delete shapes[i];
            //   shapes[i] = NULL;
        }
        cc = aggregateCC(g_aggregateCCMat, shapes, factory, connectivity);
        return cc;
    }
    else {
        return shapes;
    }
  }

  void
  SegmentationClusteringEnclosingShape::extractCC(cv::Mat &input, 
      std::vector<EnclosingShapeFactory::EnclosingShapePtr> &cc,
      const EnclosingShapeFactory &factory,
      int connectivity)
  {
    //   assert(input.type() == CV_8U);

    for (int i = 0; i < input.rows; ++i) {
        for (int j = 0; j < input.cols; ++j) {
            if (input.at<unsigned char>(i, j) == FOREGROUND) {
                std::vector<cv::Point> points;
                ise::ConnectedComponentTools::extractConnectedComponent(input, cv::Point(j, i), points, connectivity);
                // getCCPoints(input, cv::Point(j, i), points, connectivity);

                cv::Mat pointsMat(points); //convert vector to Mat, O(1) operation.
                pointsMat.reshape(1);      //convert Nx2 1-channel matrix out of Nx1 2-channel, O(1) operation.
                EnclosingShapeFactory::EnclosingShapePtr shape = factory.build(pointsMat);
                cc.push_back(shape);
            }
        }
    }
  }

  //  // the seed point must be moved differently according to the orientation
  //  //
  //  bool
  //  moveSeed(cv::Point &pt, int rows, int cols, ise::RunLengthExtractor::Orientation ori)
  //  {
  //    switch (ori) {
  //    // if I extract horizontally, the seed must move vertically, and so on
  //    case ise::RunLengthExtractor::VERTICAL:
  //      pt.x++;
  //      if (pt.x < cols)
  //        return true;
  //      else
  //        return false;
  //      break;
  //    case ise::RunLengthExtractor::HORIZONTAL:
  //      pt.y++;
  //      if (pt.y < rows)
  //        return true;
  //      else
  //        return false;
  //      break;
  //    case ise::RunLengthExtractor::DIAGO45:
  //      {
  //        // start by shifting toward the left
  //        // then up until (0, 0)
  //        bool moved = false;
  //        if (pt.x > 0) {
  //            pt.x--;
  //            moved = true;
  //        }
  //        if (!moved && pt.y > 0) {
  //            pt.y--;
  //            moved = true;
  //        }
  //        return moved;
  //        break;
  //      }
  //    case ise::RunLengthExtractor::DIAGO135:
  //      {
  //        bool moved = false;
  //        if (pt.x > 0) {
  //            pt.x--;
  //            moved = true;
  //        }
  //        if (pt.y < rows - 1 && !moved) {
  //            pt.y++;
  //            moved = true;
  //        }
  //        return moved;
  //        break;
  //      }
  //    default:
  //      assert(false);
  //      break;
  //    }
  //  }
  //
  //  void
  //  initSeed(cv::Point &pt, int rows, int cols, ise::RunLengthExtractor::Orientation ori)
  //  {
  //    switch(ori)
  //    {
  //    case ise::RunLengthExtractor::VERTICAL:
  //    case ise::RunLengthExtractor::HORIZONTAL:
  //      pt.x = pt.y = 0;
  //      break;
  //    case ise::RunLengthExtractor::DIAGO45:
  //      // start at the bottom right hand corner
  //      pt.x = cols - 1;
  //      pt.y = rows - 1;
  //      break;
  //    case ise::RunLengthExtractor::DIAGO135:
  //      // start at the top right hand corner
  //      pt.x = cols - 1;
  //      pt.y = 0;
  //    }
  //
  //  }

  void
  SegmentationClusteringEnclosingShape::computeRegionDistances(const cv::Mat& input, 
      const ShapeList &shapes,
      ise::RunLengthExtractor::Orientation ori,
      std::vector<ise::NeighbouringRegions> &output)
  {
    assert((int)shapes.size() < std::numeric_limits<int>::max());
    cv::Mat labelMat = cv::Mat::zeros(input.size(), CV_8U);
    ShapeList::const_iterator it = shapes.begin();
    const ShapeList::const_iterator itEnd = shapes.end();
    for (int i = 1; it != itEnd; ++it, ++i) {
        (*it)->draw(labelMat, CV_RGB(i, i, i), CV_FILLED);
    }

    computeRegionDistances(input, labelMat, shapes.size(), ori, output);
  }

  void
  SegmentationClusteringEnclosingShape::computeRegionDistances(const cv::Mat &/*input*/,
							       const cv::Mat &/*labels*/,
							       int /*nbReg*/,
							       ise::RunLengthExtractor::Orientation /*ori*/,
							       std::vector<ise::NeighbouringRegions> &/*output*/)
  {
    //
    //    assert(labels.type() == CV_32S);
    //    assert(input.type() == CV_8U);
    //    output.assign(nbReg, ise::NeighbouringRegions());
    //    cv::Mat tmp = input.clone();
    //    cv::Point seed(0, 0);
    //
    //    initSeed(seed, input.rows, input.cols, ori);
    //
    //    // initialise seed to be sure that it will be positioned exactly at the start of the supporting
    //    // line so that runLengthExtractor will extract the segments with the good order
    //    // for (int i = 0; i < tmp.rows; ++i) {
    //    do {
    //        //std::cout << "Seed(" << seed.x << ',' << seed.y << ")" << std::endl;
    //        RunLengthExtractor xExtractor(tmp, ori, -1, NULL, true, true);
    //        xExtractor.extractRunLength(seed);
    //        // since we started the run length extraction manually, only 1 line is scanned so only 1
    //        // segmentList should be provided, as oppososed to running the extractor on a whole region
    //        // using tog.iterateOnRegionDomain(it, xExtractor)
    //        const ise::RunLengthExtractor::SegmentListContainer &segmentList = xExtractor.getForegroundSegments();
    //        assert(segmentList.size() == 1);
    //        const ise::RunLengthExtractor::SegmentContainer &segments = segmentList[0];
    //        //  std::cout << segments.size() << std::endl;
    //        if (segments.size() > 1) { // no neighbours otherwise
    //            ise::RunLengthExtractor::SegmentContainer::const_iterator segIt = segments.begin();
    //            ise::RunLengthExtractor::SegmentContainer::const_iterator segItNext = segments.begin() + 1;
    //            const ise::RunLengthExtractor::SegmentContainer::const_iterator segItEnd = segments.end();
    //            for ( ; segItNext != segItEnd; ++segIt, ++segItNext) {
    //                girl::point endFirstSegment (segIt->m_end.x, segIt->m_end.y);
    //                girl::point startNextSegment(segItNext->m_start.x, segItNext->m_start.y);
    //                girl::point endNextSegment  (segItNext->m_end.x, segItNext->m_end.y);
    //                int first = labels.at<int>(endFirstSegment.y(), endFirstSegment.x());//tog.findRegion(endFirstSegment);
    //                int next  = labels.at<int>(endNextSegment.y(), endNextSegment.x()); //tog.findRegion(endNextSegment);
    //                // regions are numbered from 1 to N
    //                // so we should not return region 0 (background)
    //                // and we must decrease the reg index to fill in the vector 0, ..., N-1assert(first);
    //                assert(first);
    //                assert(next);
    //		if (first != next) {
    //		  --first;
    //		  --next;
    //
    //		  int space = 0;
    //		  switch(ori) {
    //		  case ise::RunLengthExtractor::HORIZONTAL:
    //		    // in the Run Length article they count the space as the x- differences (see iseRunLengthExtractor)
    //		  case ise::RunLengthExtractor::DIAGO45:
    //		  case ise::RunLengthExtractor::DIAGO135:
    //		    space = startNextSegment.x() - endFirstSegment.x();
    //		    break;
    //		  case ise::RunLengthExtractor::VERTICAL:
    //		    space = startNextSegment.y() - endFirstSegment.y();
    //		    break;
    //		  default:
    //		    break;
    //		  }
    //		  if (space < output[first].distNext) {
    //		    output[first].distNext = space;
    //		    output[first].next = next;
    //		  }
    //		  if (space < output[next].distPrev) {
    //		    // update
    //		    output[next].prev = first;
    //		    output[next].distPrev = space;
    //		  }
    //                }
    //            }
    //        }
    //    } while (moveSeed(seed, input.rows, input.cols, ori));
  }

  /**
    MOVED TO ise::ConnectedComponentsTools
  void
  SegmentationClusteringEnclosingShape::getCCPoints(cv::Mat &input,
      const cv::Point &seed,
      std::vector<cv::Point> &points,
      int connectivity)
  {

    assert(input.type() == CV_8U);
    assert(input.at<unsigned char>(seed.y, seed.x));
    assert(seed.x < input.cols && seed.y < input.rows && seed.x >= 0 && seed.y >= 0);

    std::deque<cv::Point> pointsDeq;
    std::list<cv::Point> theList;
    theList.push_back(seed);
    int cpt = 0;
    while (! theList.empty()) {
        ++cpt;
        cv::Point current = theList.front();
        pointsDeq.push_back(current);
        input.at<unsigned char>(current.y, current.x) = BACKGROUND;
        // enque neighboors
        cv::Point e(current.x + 1, current.y);
        cv::Point w(current.x - 1, current.y);
        cv::Point n(current.x, current.y - 1);
        cv::Point s(current.x, current.y + 1);
        if (n.y >= 0)
          if (input.at<unsigned char>(n.y, n.x)) {
              theList.push_back(n);
              input.at<unsigned char>(n.y, n.x) = BACKGROUND;
          }
        if (e.x < input.cols)
          if (input.at<unsigned char>(e.y, e.x)) {
              theList.push_back(e);
              input.at<unsigned char>(e.y, e.x) = BACKGROUND;
          }
        if (s.y < input.rows)
          if (input.at<unsigned char>(s.y, s.x)) {
              theList.push_back(s);
              input.at<unsigned char>(s.y, s.x) = BACKGROUND;
          }
        if (w.x >= 0)
          if (input.at<unsigned char>(w.y, w.x)) {
              theList.push_back(w);
              input.at<unsigned char>(w.y, w.x) = BACKGROUND;
          }
        if (connectivity == 8) {
            cv::Point ne(current.x + 1, current.y - 1);
            cv::Point no(current.x - 1, current.y - 1);
            cv::Point se(current.x + 1, current.y + 1);
            cv::Point so(current.x - 1, current.y + 1);
            if (ne.x < input.cols && ne.y >= 0)
              if (input.at<unsigned char>(ne.y, ne.x)) {
                  theList.push_back(ne);
                  input.at<unsigned char>(ne.y, ne.x) = BACKGROUND;
              }
            if (se.x < input.cols && se.y < input.rows)
              if (input.at<unsigned char>(se.y, se.x)) {
                  theList.push_back(se);
                  input.at<unsigned char>(se.y, se.x) = BACKGROUND;
              }
            if (so.x >= 0 && so.y < input.rows)
              if (input.at<unsigned char>(so.y, so.x)) {
                  theList.push_back(so);
                  input.at<unsigned char>(so.y, so.x) = BACKGROUND;
              }
            if (no. x >= 0 && no.y >= 0)
              if (input.at<unsigned char>(no.y, no.x)) {
                  theList.push_back(no);
                  input.at<unsigned char>(no.y, no.x) = BACKGROUND;
              }
        }
        theList.pop_front();
    }
    points.assign(pointsDeq.begin(), pointsDeq.end());
  }

   */
}
