/*
  Copyright (C) 2005-2011 Remi Vieux, Roigt Julian, Boris Mansencal  boris.mansencal@labri.fr

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISESEGMENTATIONCLUSTERINGENCLOSINGSHAPE_HPP_
#define ISESEGMENTATIONCLUSTERINGENCLOSINGSHAPE_HPP_

#include <vector>

#include <girl/girlCommon.hpp>

#include "EnclosingShape.hpp"
#include "EnclosingShapeFactory.hpp"
#include "iseNeighbouringRegions.hpp"
#include "iseRunLengthExtractor.hpp"
#include "iseSegmentationClustering.hpp"

#include <opencv2/core/core.hpp>


class SegmentationContext;
/**
 * Refactoring of SegmentationClusteringShapeCC
 * The algorithm takes as parameters a vector of Shapes and perform
 * the iterative merging of those shapes.
 * It is client responsibility to perform any pre-filtering of the set
 * of shapes before giving them to the segClustering.
 * The shapes are merged until there are no overlap between any of them
 */
namespace ise
{


  class GIRL_API SegmentationClusteringEnclosingShape : public SegmentationClustering
  {

  public:

    enum MergeMethod
    {
      UNION = 0,
      ITERATIVE = 1,
      EXCLUSIVE = 2
    };


    typedef std::vector<EnclosingShapeFactory::EnclosingShapePtr> ShapeList;
    typedef girl::shared_ptr<EnclosingShapeFactory> EnclosingShapeFactoryPtr;

  public:

    static const unsigned char FOREGROUND = 255;
    static const unsigned char BACKGROUND = 0;


  public:
    explicit SegmentationClusteringEnclosingShape(SegmentationContext *segContext);

    virtual
    ~SegmentationClusteringEnclosingShape();

    virtual void setParameters(const ShapeList& shapes, EnclosingShapeFactoryPtr factory, MergeMethod method)
    {
      m_shapes = shapes;
      m_shapeFactory = factory;
      m_mergeMethod = method;
    }

    virtual void getParameters(ShapeList &shapes, EnclosingShapeFactoryPtr factory, MergeMethod &method) const
    {
      shapes  = m_shapes;
      factory = m_shapeFactory;
      method  = m_mergeMethod;
    }

    virtual void init(const girl::regionDeque &regions, int level=1);

    virtual void init(girl::region /*r*/, int /*level*/) {}

    virtual bool same(const girl::point &p, girl::direction dir);

    void extractCC(cv::Mat &input, ShapeList &cc, const EnclosingShapeFactory &factory, int connectivity);

    void computeRegionDistances(const cv::Mat &input, const ShapeList &shapes, ise::RunLengthExtractor::Orientation ori, std::vector<ise::NeighbouringRegions> &output);

    void computeRegionDistances(const cv::Mat &input, const cv::Mat &labels, int nbReg, ise::RunLengthExtractor::Orientation ori, std::vector<ise::NeighbouringRegions> &output);

  protected:

    void
    initExclusive();

    std::vector<EnclosingShapeFactory::EnclosingShapePtr> aggregateCC(cv::Mat &input, std::vector<EnclosingShapeFactory::EnclosingShapePtr> &cc, const EnclosingShapeFactory &factory, int connectivity = 8);
  //   void getCCPoints(cv::Mat &input, const cv::Point &seed, std::vector<cv::Point> &points, int connectivity = 4);

  protected:

    ShapeList m_shapes;

    EnclosingShapeFactoryPtr m_shapeFactory;

    cv::Mat m_mat;

    MergeMethod m_mergeMethod;

  };

}

#endif /* ISESEGMENTATIONCLUSTERINGENCLOSINGSHAPE_HPP_ */
