/*
 * iseSegmentationClusteringFelzenswalb.cpp
 *
 *  Created on: Nov 4, 2013
 *      Author: remi
 */

#include "iseSegmentationClusteringFelzenswalb.hpp"

#include "iseSegmentationContext.hpp"

#include <opencv2/flann/flann.hpp>


namespace ise
{

  float SegmentationClusteringFelzenswalb::DEFAULT_SIGMA = 0.8;
  float SegmentationClusteringFelzenswalb::DEFAULT_K = 100;
  int SegmentationClusteringFelzenswalb::DEFAULT_MIN_AREA = 100;
  SegmentationClusteringFelzenswalb::FelzGraphType SegmentationClusteringFelzenswalb::DEFAULT_GRAPH_TYPE = GRID_8_CONNEX;

  SegmentationClusteringFelzenswalb::SegmentationClusteringFelzenswalb(SegmentationContext *segContext)
    : SegmentationClustering(segContext),
      m_sigma(DEFAULT_SIGMA),
      m_k(DEFAULT_K),
      m_minArea(DEFAULT_MIN_AREA),
      m_graphType(DEFAULT_GRAPH_TYPE),
      m_imageFeature(),
      m_edges(),
      m_universe(NULL)
  {
    
  }

  
  SegmentationClusteringFelzenswalb::~SegmentationClusteringFelzenswalb()
  {
  }


  void
  SegmentationClusteringFelzenswalb::init(const girl::regionDeque& /*regions*/, int )
  {
    // TODO: initialiser le graphe uniquement sur les pixels des regions a segmenter.
    // Pour l'instant, on considère le graphe global de l'image.
    doSeg();
  }

  void
  SegmentationClusteringFelzenswalb::init(girl::region /*r*/, int)
  {
  
  }
  
  void
  SegmentationClusteringFelzenswalb::setParameters(float sigma, float k, int minArea, ise::ImageFeatureComputerPtr algo, FelzGraphType graphType)
  {
    m_sigma = sigma;
    m_k = k;
    m_minArea = minArea;
    m_imageFeature = algo;
    m_graphType = graphType;
  }

  void
  SegmentationClusteringFelzenswalb::getParameters(float &sigma, float &k, int &minArea, ise::ImageFeatureComputerPtr &algo, FelzGraphType &graphType) const
  {
    sigma = m_sigma;
    k = m_k;
    minArea = m_minArea;
    algo = m_imageFeature;
    graphType = m_graphType;
  }

  bool
  SegmentationClusteringFelzenswalb::same(const girl::point &p, const girl::direction dir)
  {
    assert(m_universe);
    int width = m_imageFeature->getImageFeature()->width();
    girl::point p2 = p;
    p2.move(dir);
    return m_universe->find(p.y()*width + p.x()) == m_universe->find(p2.y()*width + p2.x());
  }
  



  void
  SegmentationClusteringFelzenswalb::computeGraph()
  {
    assert(m_imageFeature);
    int width = m_segContext->getImage()->width();
    int height = m_segContext->getImage()->height();
    int area = width*height;
    m_edges.clear();
    switch(m_graphType)
      {
      case GRID_4_CONNEX:
      case GRID_8_CONNEX:
	{
	  int factor = (m_graphType == GRID_4_CONNEX) ? 2 : 4;
	  m_edges.reserve(factor*area);
	  cv::flann::L2<RegionFeature_Data_Type> l2;
	  int dim = m_imageFeature->getFeatureDim();
	  for (int y = 0; y < height; ++y) {
	    for (int x = 0; x < width; ++x) {
	      const girl::tools::ImageFeature::type* feature1 = 
		m_imageFeature->getImageFeature()->getPixelPtr(x, y);
	      edge e;
	      if (x < width-1) {
		e.a = y * width + x;
		e.b = y * width + (x+1);
		const girl::tools::ImageFeature::type* feature2 =
		  m_imageFeature->getImageFeature()->getPixelPtr(x+1, y);
		e.w = l2(feature1, feature2, dim);
		m_edges.push_back(e);
	      }
	    if (y < height-1) { // bottom
	      e.a = y * width + x;
	      e.b = (y+1) * width + x;
	      const girl::tools::ImageFeature::type* feature2 = 
		m_imageFeature->getImageFeature()->getPixelPtr(x, y+1);
	      e.w = l2(feature1, feature2, dim);
	      m_edges.push_back(e);
	    }
	    if (m_graphType == GRID_4_CONNEX)
	      continue;
	    if ((x < width-1) && (y < height-1)) { // bottom right
	      e.a = y * width + x;
	      e.b = (y+1) * width + (x+1);
	      const girl::tools::ImageFeature::type* feature2 = 
		m_imageFeature->getImageFeature()->getPixelPtr(x+1, y+1);
	      e.w = l2(feature1, feature2, dim);
	      m_edges.push_back(e);
	    }
	    
	    if ((x < width-1) && (y > 0)) { // top right
	      e.a = y * width + x;
	      e.b = (y-1) * width + (x+1);
	      const girl::tools::ImageFeature::type* feature2 = 
		m_imageFeature->getImageFeature()->getPixelPtr(x+1, y-1);
	      e.w =  l2(feature1, feature2, dim);
	      m_edges.push_back(e);
	    }
	    }
	  }
	}
	break;
      case KNN:
	break;
      default:
	break;
      }
    
  }


  void
  SegmentationClusteringFelzenswalb::doSeg()
  {
    assert(m_imageFeature);
    computeGraph();
    if (m_universe) {
      delete m_universe;
      m_universe = NULL;
    }
    m_universe = segment_graph(m_imageFeature->getImageFeature()->width()*m_imageFeature->getImageFeature()->height(),
			       m_edges.size(),
			       &(m_edges[0]),
			       m_k);
    
    // post process small components
    for (size_t i = 0; i < m_edges.size(); i++) {
      int a = m_universe->find(m_edges[i].a);
      int b = m_universe->find(m_edges[i].b);
      if ((a != b) && ((m_universe->size(a) < m_minArea) || (m_universe->size(b) < m_minArea)))
	m_universe->join(a, b);
    }
  }
} /* namespace ise */
