/*
  SegmentationClustering KMeans class.
  Copyright (C) 2005-2009 Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "iseSegmentationClusteringGenericKMeans.hpp"
//Ajout pour la Def des classes
#include "iseHashHistogram.hpp"
#include "iseHistogram.hpp"
#include "iseSegmentationContext.hpp"

#include <cstdio>
#include <fstream>
#include <cstring>
#include <iostream>
#include <limits>

#include <ise/iseConversion.hpp>

#include <time.h> // R

static clock_t start;

static inline
void startTime()
{ 
  start = clock();
}

static inline
void finishTime()
{
  clock_t finish = clock();
  double duration = (double)(finish - start) / CLOCKS_PER_SEC;
  std::cout<<"durée "<<duration<<std::endl;
}

using namespace ise;

SegmentationClusteringGenericKMeans::~SegmentationClusteringGenericKMeans()
{

}

SegmentationClusteringGenericKMeans::SegmentationClusteringGenericKMeans(SegmentationContext *segContext, 
									 int nbClasses)
  :
  SegmentationClustering(segContext), 
  m_nbClasses(nbClasses),
  m_kmeans(ise::GenericKMeans<double>(nbClasses)),
  m_dim(0)
{

}

bool
SegmentationClusteringGenericKMeans::same(const girl::point &p1, girl::direction dir)
{
  girl::point p2 = p1;
  p2.move(dir);

  assert(m_segContext->getImageFeature(m_imageFeatureUri));

  const girl::shared_ptr<girl::tools::ImageFeature> imgf = m_segContext->getImageFeature(m_imageFeatureUri);

  const girl::tools::ImageFeature::type *pixel1 = imgf->getPixelPtr(p1.x(), p1.y());
  const girl::tools::ImageFeature::type *pixel2 = imgf->getPixelPtr(p2.x(), p2.y());

  // std::vector<double> a(m_dim, 0);
  // std::vector<double> b(m_dim, 0);
  // double dist;

  // for (int i = 0; i < m_dim; ++i) {
  //   a[i] = pixel1[i];
  //   b[i] = pixel2[i];
  // }  


  int cluster1 = m_kmeans.getCluster(pixel1, m_dim);
  int cluster2 = m_kmeans.getCluster(pixel2, m_dim);

  return (cluster1 == cluster2);
}

void 
SegmentationClusteringGenericKMeans::init(const girl::regionDeque &regions, int /*level*/)
{ 
  assert(m_segContext->getImageFeature(m_imageFeatureUri));

  m_dim = m_segContext->getImageFeature(m_imageFeatureUri)->channels();
  
  startTime();

  const int width = m_segContext->getImageFeature(m_imageFeatureUri)->width();
  const int height = m_segContext->getImageFeature(m_imageFeatureUri)->height();
  const int channels = m_segContext->getImageFeature(m_imageFeatureUri)->channels();

  /*assert((*(m_segContext->getImageFeature(m_imageFeatureUri)->cvMat().get())).isContinuous());
  cv::Mat data = (*(m_segContext->getImageFeature(m_imageFeatureUri)->cvMat().get())).reshape(channels);
  */

  cv::Mat data(height*width, channels, CV_MAKETYPE(cv::DataDepth<double>::value, 1));

  for (int y = 0 ; y < height ; ++y) {
    girl::tools::ImageFeature::type *buf = m_segContext->getImageFeature(m_imageFeatureUri)->scanLine(y);
    for (int x = 0 ; x < width ; ++x) {
      double *bufDest = data.ptr<double>(y*width+x);
      for (int c = 0 ; c < channels ; ++c) {
	bufDest[c] = buf[c];
      }
      buf += m_segContext->getImageFeature(m_imageFeatureUri)->formatSize();
    }
  }
  

  finishTime();
  ise::DistanceComputer distComputer;
  distComputer.setDistanceType(ise::EUCLIDEAN);
  startTime();
  std::cout<<"buildCluster start"<<std::endl;
  m_kmeans.setParameters(distComputer, NULL, false);
  m_kmeans.compute(data);
  std::cout<<"buildCluster end"<<std::endl;
  finishTime();
}


void SegmentationClusteringGenericKMeans::init(girl::region /*r*/, int /*level*/)
{
  
}


