/*
  Segmentation clustering: Gonogo0
  Copyright (C) 20013 Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "iseSegmentationClusteringGonogo0.hpp"

#include <cassert>
#include <limits>

#include <ise/iseConversion.hpp>
#include <ise/iseSegmentationContext.hpp>
#include <ise/iseSplitAlgorithmDefault.hpp>
#include <ise/iseSplitDriverDefault.hpp>

#include <ise/iseQuantificationThresholds.hpp>


using namespace ise;

static const int DEFAULT_COLORSPACE = ISE_RGB_to_Lab;
static const int DEFAULT_CHANNEL = 1;
static const double DEFAULT_THRESHOLD = 35;

SegmentationClusteringGonogo0::SegmentationClusteringGonogo0(SegmentationContext *segContext)
  : SegmentationClustering(segContext),
    m_colorSpace(DEFAULT_COLORSPACE),
    m_channel(DEFAULT_CHANNEL),
    m_threshold(DEFAULT_THRESHOLD)
{
  setSegContext(segContext);
}

void
SegmentationClusteringGonogo0::setSegContext(ise::SegmentationContext *s) 
{
  SegmentationClustering::setSegContext(s);
}
void 
SegmentationClusteringGonogo0::setParameters(int colorSpace, int channel, double threshold)
{
  m_colorSpace = colorSpace;
  m_channel = channel;
  m_threshold = threshold;
} 

void 
SegmentationClusteringGonogo0::getParameters(int &colorSpace, int &channel, double &threshold) const
{
  colorSpace = m_colorSpace;
  channel = m_channel;
  threshold = m_threshold;
} 

void 
SegmentationClusteringGonogo0::init(const girl::regionDeque &regions, int /* level */)
{
  assert(m_segContext->getToger());
  assert(m_segContext->getImage());

  m_toger = *(m_segContext->getToger()); //girl::toger(m_segContext->getImage()->width(), m_segContext->getImage()->height()); 

  std::vector<int> levels;
  levels.push_back(static_cast<int>(m_threshold+0.5));

  ise::SegmentationContext segContext(&m_toger, m_segContext->getImage());
  ise::QuantificationThresholds quantif(&segContext);
  quantif.setParams(m_colorSpace, m_channel, levels);

  ise::SplitAlgorithmDefault splitAlgo(&segContext);
  ise::SplitDriverDefault splitDriver(&segContext);
  girl::regionDeque createdRegions;
  splitAlgo.apply(regions, splitDriver, quantif, 0, createdRegions);

}


bool
SegmentationClusteringGonogo0::same(const girl::point &p1, girl::direction dir)
{
  return m_toger.inSameRegion(p1, dir);
}
