/*
  Local thresholds computer base class.
  Copyright (C) 2005-2011 Julian Roigt  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*
  (1) M. Sezgin and B. Sankur, "Survey over image thresholding techniques 
  and quantitative performance evaluation," Journal of Electronic 
  Imaging, Vol. 13, No. 1. (2004), pp. 146-168.

  (2) J. Bernsen, "Dynamic thresholding of gray level images," 
  ICPR'86: Proc. Intl. Conf. Parr. Recog., pp. 1251-1255 (1986)
*/
/**
 * @file iseSegmentationClusteringLocalThresholderBernsen.hpp
 * @brief SegmentationClusteringLocalThresholder class.
 */

#ifndef ISESEGMENTATIONCLUSTERINGLOCALTHRESHOLDERBERNSEN_HPP
#define ISESEGMENTATIONCLUSTERINGLOCALTHRESHOLDERBERNSEN_HPP

#include "iseSegmentationClusteringLocalThresholder.hpp"
#include "iseSegmentationContext.hpp"
#include <girlTools/girlToolsImageFeature.hpp>
#include <girl/girlSharedPtr.hpp>

namespace ise {
  class GIRL_API SegmentationClusteringLocalThresholderBernsen : public SegmentationClusteringLocalThresholder
  {
  public:
    SegmentationClusteringLocalThresholderBernsen();

    virtual bool areEquivalentColor(const girl::point &p1, const girl::point &p2) const;

    virtual void init(ise::SegmentationContext *segC, const girl::regionDeque &);

    virtual void setParameters(size_t windowSize)
    {
      m_windowSize = windowSize;
    }
    
    virtual void getParameters(size_t &windowSize) const
    {
      windowSize = m_windowSize;
    }
    
  private:
    
    size_t m_windowSize;

    girl::shared_ptr<girl::tools::ImageFeature> m_imageF;
    cv::Mat m_imageMin;
    cv::Mat m_imageMax;
  };
}

#endif /* ! ISESEGMENTATIONCLUSTERINGLOCALTHRESHOLDERBERNSEN_HPP */
