/*
  Local thresholds computer base class.
  Copyright (C) 2005-2011 Julian Roigt  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
/*
  (1) M. Sezgin and B. Sankur, "Survey over image thresholding techniques 
  and quantitative performance evaluation," Journal of Electronic 
  Imaging, Vol. 13, No. 1. (2004), pp. 146-168.

  (2) J. Sauvola and M. Pietaksinen, "Adaptive document image binarization," 
  Pattern Recogn, 33, 225-236(2000)
*/
/**
 * @file iseSegmentationClusteringLocalThresholderSauvola.hpp
 * @brief SegmentationClusteringLocalThresholder class.
 */
#ifndef ISESEGMENTATIONCLUSTERINGLOCALTHRESHOLDERSAUVOLA_HPP
#define ISESEGMENTATIONCLUSTERINGLOCALTHRESHOLDERSAUVOLA_HPP

#include <girl/girlCommon.hpp>

#include <opencv2/core/core.hpp>
#include "iseSegmentationClusteringLocalThresholder.hpp"
#include "iseSegmentationContext.hpp"

namespace ise {
  class GIRL_API SegmentationClusteringLocalThresholderSauvola : public SegmentationClusteringLocalThresholder
  {
  public:
    SegmentationClusteringLocalThresholderSauvola();

    virtual bool areEquivalentColor(const girl::point &p1, const girl::point &p2) const;
    
    virtual void init(ise::SegmentationContext *segC, const girl::regionDeque &regions);

    virtual void setParameters(size_t windowSize, float R, float k) 
    { 
      m_windowSize = windowSize;
      m_R = R;
      m_k = k;
    }

    virtual void getParameters(size_t &windowSize, float &R, float &k) const
    {
      windowSize = m_windowSize;
      R = m_R;
      k = m_k;
    }

  private:
    
    size_t m_windowSize;
    float m_R;
    float m_k;

    cv::Mat m_image;
  };
}

#endif /* ! ISESEGMENTATIONCLUSTERINGLOCALTHRESHOLDERSAUVOLA_HPP */
