/*
  Segmentation clustering: Markov.
  Copyright (C) 2005-2009 Claire Morand, Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISESEGMENTATIONCLUSTERINGMARKOV_HPP
#define ISESEGMENTATIONCLUSTERINGMARKOV_HPP


#include <girl/girlCommon.hpp>
#include <girl/girlToger.hpp>

#include "iseSegmentationClustering.hpp"
#include <map>

#include "iseSegmentationClusteringRegionGrowing.hpp"


class regionMarkov
{
private:
  girl::tools::Image::type m_totalR;
  girl::tools::Image::type m_totalG;
  girl::tools::Image::type m_totalB;
  girl::tools::Image::type m_totalR2;
  girl::tools::Image::type m_totalG2;
  girl::tools::Image::type m_totalB2;
  int m_nbPixels;
  double m_sigma2[3];
  double m_average[3];

public:

  regionMarkov()
  {
    m_totalR = 0;
    m_totalG = 0;
    m_totalB = 0;
    m_totalR2 = 0;
    m_totalG2 = 0;
    m_totalB2 = 0;
    m_nbPixels = 0;
    for (int i = 0 ; i < 3; ++i) {
      m_sigma2[i] = 0;
      m_average[i] = 0;
    }
  }  

  explicit regionMarkov(girl::tools::Pixel p)
  {
    m_totalR = p.a();
    m_totalG = p.b();
    m_totalB = p.c();
    m_totalR2 = p.a() * p.a();
    m_totalG2 = p.b() * p.b();
    m_totalB2 = p.c() * p.c();
    m_nbPixels = 1;
    for (int i = 0 ; i < 3; ++i) {
      m_sigma2[i] = 0;
      m_average[i] = 0;
    }
  }

  ~regionMarkov() {};

  void addPixel(girl::tools::Pixel p)
  {
    m_totalR += p.a();
    m_totalG += p.b();
    m_totalB += p.c();
    m_totalR2 += p.a() * p.a();
    m_totalG2 += p.b() * p.b();
    m_totalB2 += p.c() * p.c();
    ++m_nbPixels;
  }

  void computeMoments()
  {
    m_average[0] = static_cast<double>(m_totalR) / static_cast<double>(m_nbPixels);
    m_average[1] = static_cast<double>(m_totalG) / static_cast<double>(m_nbPixels);
    m_average[2] = static_cast<double>(m_totalB) / static_cast<double>(m_nbPixels);
    m_sigma2[0] = static_cast<double>(m_totalR2 )/ static_cast<double>(m_nbPixels);
    m_sigma2[0] = m_sigma2[0] - m_average[0] * m_average[0];
    m_sigma2[1] = static_cast<double>(m_totalG2) / static_cast<double>(m_nbPixels);
    m_sigma2[1] = m_sigma2[1] - m_average[1] * m_average[1];
    m_sigma2[2] = static_cast<double>(m_totalB2) / static_cast<double>(m_nbPixels);
    m_sigma2[2] = m_sigma2[2] - m_average[2] * m_average[2];
  }

  void getSigma(double *tab)
  {
    for (int i = 0 ; i < 3 ; ++i)
      tab[i] = m_sigma2[i];
  }

  void getAverage(double *tab) {
    for (int i = 0 ; i < 3 ; ++i)
      tab[i] = m_average[i];
  }


};

namespace ise {

  class SegmentationContext;

  class GIRL_API SegmentationClusteringMarkov : public SegmentationClustering
  {
    
  public:
    
    /**
     * @brief Destructor.
     */
    ~SegmentationClusteringMarkov();
  
    /**
     * @brief Constructor.
     * 
     * @warning takes ownership of @a histo.
     */
    SegmentationClusteringMarkov(SegmentationContext *segContext,
				 double potential = 15.0,
				 double thresholdSegInit = 15.0);

    virtual void init(const girl::regionDeque &regions, int level);
    
    virtual void init(girl::region r, int level);
    

    void setParameters(double potential, double thresholdSegInit)
    {
      m_potential = potential;
      m_thresholdSegInit = thresholdSegInit;
    }

    void getParameters(double &potential, double &thresholdSegInit) const
    {
      potential = m_potential;
      thresholdSegInit = m_thresholdSegInit;
    }


    virtual bool same(const girl::point &p, girl::direction dir);
    
    void computeRegionCaracteristics(std::map<girl::point, int, 
				     ordrePointRegionGrowing> &Seg, 
				     std::map<int, regionMarkov> &regionCaracteristics);

    double computeU2(int l, int c, int height, int width, int label, 
		     const std::map<girl::point, int, ordrePointRegionGrowing> &Seg);
  
  private:
    SegmentationClusteringMarkov(const SegmentationClusteringMarkov &);
    SegmentationClusteringMarkov &operator=(const SegmentationClusteringMarkov &);

  protected:
    double m_thresholdSegInit;
    double m_potential;
  };

}

#endif /* ! ISESEGMENTATIONCLUSTERINGMARKOV_HPP */
