/*
  Copyright (C) 2005-2009 Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "iseSegmentationClusteringMeanShift.hpp"
#include "iseHashHistogram.hpp"
#include "iseHistogram.hpp"
#include "iseSegmentationContext.hpp"

//#include <cstdio>
//#include <cmath>
//#include <fstream>
//#include <cstring>
//#include <iostream>
#include <limits>
#include <opencv2/core/core.hpp>

#include <ise/iseConversion.hpp>

#include <girlTools/girlToolsImage.hpp>

#include <time.h> // R

static clock_t start;

static inline
void startTime()
{ 
  start = clock();
}

static inline
void finishTime()
{
  clock_t finish = clock();
  double duration = (double)(finish - start) / CLOCKS_PER_SEC;
  std::cout<<"durée "<<duration<<std::endl;
}

using namespace ise;


SegmentationClusteringMeanShift::~SegmentationClusteringMeanShift()
{
  delete[] m_pos;
}


SegmentationClusteringMeanShift::SegmentationClusteringMeanShift(SegmentationContext *segContext) 
  :
  SegmentationClustering(segContext), 
  m_ms(),
  m_index(),
  m_imageFeatureUri(),
  m_dim(0),
  m_pos(NULL)
{
}


bool
SegmentationClusteringMeanShift::same(const girl::point &p1, girl::direction dir)
{
  girl::point p2 = p1;
  p2.move(dir);

  girl::shared_ptr<girl::tools::ImageFeature> imgf = m_segContext->getImageFeature(m_imageFeatureUri);
  int channels = imgf->channels();
  girl::tools::ImageFeature::type *buf1 = imgf->cvMat()->ptr<girl::tools::ImageFeature::type>(p1.y());
  buf1 += channels * p1.x();

  girl::tools::ImageFeature::type *buf2 = imgf->cvMat()->ptr<girl::tools::ImageFeature::type>(p2.y());
  buf2 += channels * p2.x();

  int center1 = m_ms.getCluster(buf1, channels);
  int center2 = m_ms.getCluster(buf2, channels);

  return ( center1 == center2);


  // girl::point p2 = p1;
  // p2.move(dir);

  // //const girl::tools::PixelFromImage<girl::tools::ImageFeature::type> pixel1 = m_segContext->getImageFeature(m_imageFeature)->getPixel(p1.x(), p1.y());
  // //const girl::tools::PixelFromImage<girl::tools::ImageFeature::type> pixel2 = m_segContext->getImageFeature(m_imageFeature)->getPixel(p2.x(), p2.y());

  // //const girl::tools::ImageFeature::type *pixel1 = m_segContext->getImageFeature(m_imageFeature)->getPixelPtr(p1.x(), p1.y());
  // //  const girl::tools::ImageFeature::type *pixel2 = m_segContext->getImageFeature(m_imageFeature)->getPixelPtr(p2.x(), p2.y());

  // /*if (!m_flann) {
  
  //   double a[m_dim], b[m_dim];
  
  //   for (int i = 0 ; i < m_dim ; ++i) {
  //     a[i] = pixel1[i];
  //     b[i] = pixel2[i];
  //   }
  //   return m_ms.same(a, b);
  //   }
  //   else {*/
  //   int width = m_segContext->getImage()->width();
    
  //   int a = m_pos[p1.y()*width+p1.x()];//m_index.find(p1.y()*width+p1.x())->second;
  //   int b = m_pos[p2.y()*width+p2.x()];//m_index.find(p2.y()*width+p2.x())->second;
    
  //   return m_ms.same(a, b);
    /*}*/
}

// namespace ise{
//   class Adder : public girl::toger::pointIterator{
    
//   public:

//     Adder(ise::SegmentationContext &segC, std::string imageFeatureUri, double *pixel,
// 	  std::map<int, int> &index, int dim, int *pos)
//       : m_pixel(pixel),
// 	m_nb(0),
// 	m_index(index),
// 	m_width(0),
// 	m_imageFeature(segC.getImageFeature(imageFeatureUri)),
// 	m_dim(dim),
// 	m_pos(pos)
//     {
//       m_width = segC.getImage()->width();
//       m_index.clear();
//     }
    
//     void processPoint(const girl::point &pt)
//     {
//       const girl::tools::ImageFeature::type *p = m_imageFeature->getPixelPtr(pt.x(), pt.y());
      
//       for (int i = 0 ; i < m_dim ; ++i) {
// 	m_pixel[m_nb*m_dim+i] = p[i];
//       }

//       m_pos[pt.y()*m_width+pt.x()] = m_nb;
//       ++m_nb;
//     }
    
//     int nbPoint() const { return m_nb; }
    
//   protected:
    
//     double *m_pixel;
//     int m_nb;
//     std::map<int, int> &m_index;
//     int m_width;
//     const girl::shared_ptr<girl::tools::ImageFeature> m_imageFeature;
//     int m_dim;
//     int *m_pos;
    
//   };
// }

void 
SegmentationClusteringMeanShift::init(const girl::regionDeque &/*regions*/, int /*level*/)
{
  std::cout<<"debut init"<<std::endl;
  assert(m_segContext);
  assert(!m_segContext->getImageFeature(m_imageFeatureUri)->isNull());
  
  const girl::shared_ptr<girl::tools::ImageFeature> mat = m_segContext->getImageFeature(m_imageFeatureUri);

  const int width = mat->width();
  const int height = mat->height();

  m_dim = mat->channels();

  cv::Mat data(height*width, m_dim,  CV_MAKETYPE(cv::DataDepth<double>::value, 1));

  for (int y = 0 ; y < height ; ++y) {
    girl::tools::ImageFeature::type *buf = mat->scanLine(y);
    for (int x = 0 ; x < width ; ++x) {
      double *bufDest = data.ptr<double>(y*width+x);
      for (int c = 0 ; c < m_dim ; ++c) {
	bufDest[c] = buf[c];
      }
      buf += mat->formatSize();
    }
  }

  m_ms.compute(data);

  /*
  double *pixel = new double[width*height*m_dim];

  m_pos = new int[width*height];
  
  startTime();
  Adder adder(*m_segContext, m_imageFeatureUri, pixel, m_index, m_dim, m_pos);
  girl::regionDeque::const_iterator it = regions.begin();
  const girl::regionDeque::const_iterator itEnd = regions.end();
  for ( ; it != itEnd; ++it) {
    m_segContext->getToger()->iterateOnRegionDomain(*it, adder);
  }
  finishTime();
  
  std::cout<<"debut chargement des points"<<std::endl;
  m_ms.loadPoints(pixel, adder.nbPoint(), m_dim);
  std::cout<<"fin chargement des points"<<std::endl;


  startTime();
  m_ms.segment();
  finishTime();
  delete[] pixel;

  pixel = NULL;

  std::cout<<"fin init"<<std::endl;
  */
}


void 
SegmentationClusteringMeanShift::init(girl::region /*r*/, int /*level*/)
{
  
}

void 
SegmentationClusteringMeanShift::setParameters(std::string imageFeatureUri, float sigmaR)
{ 
  m_ms.setSigmaR(sigmaR);
  m_imageFeatureUri = imageFeatureUri;
}

void 
SegmentationClusteringMeanShift::getParameters(std::string &imageFeatureUri, float &sigmaR) const
{ 
  sigmaR = m_ms.sigmaR();
  imageFeatureUri = m_imageFeatureUri;
}

