/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal, Remi Vieux  remi.vieux@labri.fr

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
#ifndef ISESEGMENTATIONCLUSTERINGPARAGRAPHSPLIT_HPP
#define ISESEGMENTATIONCLUSTERINGPARAGRAPHSPLIT_HPP


#include <vector>
#include <opencv2/core/core.hpp>

#include <girlTools/girlToolsAABBox.hpp>
#include "iseConnectedComponentTools.hpp"
#include "iseSegmentationClustering.hpp"
#include "iseSegmentationContext.hpp"
#include "iseDocumentImageUtils.hpp"
#include "iseParagraph.hpp"


//#define VISU

namespace ise
{
class GIRL_API SegmentationClusteringParagraphSplit : public SegmentationClustering
{
public:
  explicit SegmentationClusteringParagraphSplit(SegmentationContext *segContext);

  virtual
  ~SegmentationClusteringParagraphSplit();

  void
  init(const girl::regionDeque &regions, int level = 1);

  void
  init(girl::region, int);





  bool
  same(const girl::point &p, girl::direction dir);


  /**
    * Pass as parameter mask of binarized image from segContext
    * if it is not set as param, it is computed with openCV threshold otsu
    */
  void
  setParameters(const cv::Mat & ccBinaryMat,
		const cv::Mat & ccLabelsMat,
                const cv::Mat & originalBinar,
                const cv::Mat & wordMat,
                int horizThresh,
                int vertiThresh,
                int meanCharacterWidth,
                int meanCharacterHeight,
		int medianCCHorizSpacings,
		int averageWordsHorizSpacings,
		const std::vector<ise::BitmapRunLength::SpacingInfo>& spacingsHoriz);

private:

  void
  computeMat();

  void
  computeBoxes(int nbCC, cv::Mat &labels, std::vector<girl::tools::AABBox> &ccBoxes);





  void
  detectJustifiedParagraphs(const std::vector<TextLine> &lines,
                            const std::vector<size_t> &linesToCluster,
                            std::vector<int> &paragraphsFound);

  void
  detectAlternateParagraphs(const std::vector<TextLine> &lines,
                            const std::vector<size_t> &linesToCluster,
                            std::vector<int> &paragraphsFound);
  void
  detectLeftParagraphs(const std::vector<TextLine> &lines,
                            const std::vector<size_t> &linesToCluster,
                            std::vector<int> &paragraphsFound);

  void
  detectRightParagraphs(const std::vector<TextLine> &lines,
                            const std::vector<size_t> &linesToCluster,
                            std::vector<int> &paragraphsFound);

  void
  detectCenteredParagraphs(const std::vector<TextLine> &lines,
                            const std::vector<size_t> &linesToCluster,
                            std::vector<int> &paragraphsFound);

  void
  cleanLines(const std::vector<TextLine> &originaLines,
            std::vector<TextLine> &cleanLines,
             const cv::Mat &ccMask,
             const cv::Mat &regionMask
#ifdef VISU
             , cv::Mat &visu,
             cv::Mat &visu2
#endif
             );

  void
    extendLineExtremities(std::vector<TextLine> &lines, int shiftX, int shiftY);


  void
    extendLeft(ise::TextLine& line, int shiftX, int shiftY, float THRESHOLD);

  void
  extendRight(ise::TextLine& line, int shiftX, int shiftY, float THRESHOLD);

  void
  makeParagraphs(const std::vector<TextLine> &lines,
                 std::vector<int> &lineLabels,
                 std::vector<Paragraph> &assignedParagraphs,
                 std::vector<Paragraph> &unassignedParagraphs,
                 std::vector<Paragraph> &isolatedLines);


private:

  cv::Mat m_originalBinar;
  cv::Mat m_mat;
  cv::Mat m_ccLabels;
  cv::Mat m_wordMat;
  std::vector<cv::Mat> m_masks;
  std::vector<girl::tools::AABBox> m_boxes;

  int m_horizThreshold;
  int m_vertiThreshold;
  int m_meanCharacterWidth;
  int m_meanCharacterHeight;
  int m_medianCCHorizSpacings;
  int m_averageWordsHorizSpacings;
  std::vector<ise::BitmapRunLength::SpacingInfo> m_spacingsHoriz;

};

}
#endif // ISESEGMENTATIONCLUSTERINGPARAGRAPHSPLIT_HPP
