/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal, Remi Vieux  remi.vieux@labri.fr

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
#ifndef ISESEGMENTATIONCLUSTERINGPARAGRAPHSPLIT2_HPP
#define ISESEGMENTATIONCLUSTERINGPARAGRAPHSPLIT2_HPP


#include <vector>
#include <opencv2/core/core.hpp>

#include <girlTools/girlToolsAABBox.hpp>
#include "iseConnectedComponentTools.hpp"
#include "iseSegmentationClustering.hpp"
#include "iseSegmentationContext.hpp"


namespace ise
{
class GIRL_API SegmentationClusteringParagraphSplit2 : public SegmentationClustering
{
public:
  explicit SegmentationClusteringParagraphSplit2(SegmentationContext *segContext);

  virtual
  ~SegmentationClusteringParagraphSplit2();

  void
  init(const girl::regionDeque &regions, int level = 1);

  void
  init(girl::region, int);

  void
  init2(girl::region, int);

  bool
  same(const girl::point &p, girl::direction dir);


  /**
    * Pass as parameter mask of binarized image from segContext
    * if it is not set as param, it is computed with openCV threshold otsu
    */
  void
  setParameters(const cv::Mat & mat, int horizThresh, int vertiThresh);

private:

  void
  computeMat();

  void
  computeBoxes(int nbCC, cv::Mat &labels, std::vector<girl::tools::AABBox> &ccBoxes);

  void
  splitParagraphs(const std::vector<girl::tools::AABBox> &ccBoxes,
                  const std::vector<int> &starters,
                  const std::vector<ise::NeighbouringRegions> &spacings,
                  cv::Mat &mask,
                  cv::Mat &visu);

  void
  extractLinkedCC(const std::vector<ise::NeighbouringRegions> &spacings,
                  int startIndex,
                  girl::unorderedSet<int> &linkedCC,
		  const std::vector<girl::tools::AABBox> &ccBoxes);

private:

cv::Mat m_mat;
std::vector<cv::Mat> m_masks;
std::vector<girl::tools::AABBox> m_boxes;

int m_horizThreshold;
int m_vertiThreshold;

};

}
#endif // ISESEGMENTATIONCLUSTERINGPARAGRAPHSPLIT2_HPP
