#include "iseSegmentationClusteringProfile.hpp"

#include <girlTools/girlToolsRegionInfos.hpp>
#include <girlTools/girlToolsPainter.hpp>

#include "iseSegmentationContext.hpp"

#include "iseDocumentImageUtils.hpp"

#include <opencv2/core/core.hpp>
#include <opencv2/imgproc/imgproc.hpp>

#ifdef VISU
#include <opencv2/highgui/highgui.hpp>
#endif //VISU

//#define TIMING

namespace ise
{

  void
  computeFreqHisto(size_t s, const std::vector<size_t> &histo, std::vector<size_t> &output)
  {
    output.assign(s, 0);
    std::vector<size_t>::const_iterator it = histo.begin();
    const std::vector<size_t>::const_iterator itEnd = histo.end();
    for (; it != itEnd; ++it) {
      output[*it]++;
    }
    
    std::cout << "Frequencies " << std::endl;
    it = output.begin();
    for (; it != output.end(); ++it) {
      std::cout << *it << std::endl;
    }
  }


  SegmentationClusteringProfile::SegmentationClusteringProfile(SegmentationContext *segContext) :
    SegmentationClustering(segContext),
    m_histoH(),
    m_histoV(),
    m_countH(),
    m_countV(),
    m_threshH(),
    m_threshV(),
    m_binaryImage(),
    m_box(),
    m_noiseFactor(0.0f),
    m_erodeHorizSize(0),
    m_erodeVertiSize(0),
    m_dilateHorizSize(0),
    m_dilateVertiSize(0)
  {
  }
  
  
  void
  SegmentationClusteringProfile::init(const girl::regionDeque &regions, int)
  {
    if (regions.empty())
      return;

    assert(m_segContext);
    girl::toger *tog = m_segContext->getToger();
    assert(tog);

    // TODO:REMOVE
    //cv::Mat *mat = m_binaryImage->getImageFeature()->cvMat().get();

    m_box = girl::tools::getAABBox(*tog, regions);
    size_t w = m_box.width();
    size_t h = m_box.height();
    if (w == 0 || h == 0)
      return;
    m_histoH.assign(w, 0);
    m_countH.assign(w, 0);
    m_threshH.assign(w, 0);
    m_histoV.assign(h, 0);
    m_countV.assign(h, 0);
    m_threshV.assign(h, 0);

#ifdef TIMING
    {
      Timer t1("ComputeProfile time v1 =");
#endif
      computeProfile(regions);
#ifdef TIMING
    }
 #endif
//  #ifdef TIMING
//    {
//       Timer t1("ComputeProfile time v2 =");
//       computeProfile2(regions);
//     }
// #endif

    //TODO:REMOVE
//    std::cout << " ******** PROFILE HORIZ START*****" << std::endl;
//    for (size_t i = 0; i < m_histoH.size(); ++i) {
//      std::cout << i << " " << m_histoH[i] << std::endl;
//    }
//    std::cout << " ******** END PROFILE HORIZ START*****" << std::endl;
//    std::cout << " ******** PROFILE VERTI START*****" << std::endl;
//    for (size_t i = 0; i < m_histoV.size(); ++i) {
//      std::cout << i << " " << m_histoV[i] << std::endl;
//    }
//    std::cout << " ******** END PROFILE VERTI START*****" << std::endl;

//    girl::regionDeque::const_iterator it = regions.begin();
//    const girl::regionDeque::const_iterator itEnd = regions.end();
//    for (; it != itEnd; ++it) {
//      tog->iterateOnRegionDomain(*it, *this);
//    }


    // no param set, just remove borders
    if (m_noiseFactor == 0 && m_erodeHorizSize == 0 && m_dilateHorizSize == 0 && m_erodeVertiSize == 0 && m_dilateVertiSize == 0) {
      computeThreshold(m_histoH, m_countH, m_threshH, m_noiseFactor);
      computeThreshold(m_histoV, m_countV, m_threshV, m_noiseFactor);
      int startIndexH = 0, endIndexH = m_threshH.size() - 1;
      int startIndexV = 0, endIndexV = m_threshV.size() - 1;
      while (startIndexH < (int)m_threshH.size() &&m_threshH[startIndexH] == 0)
        ++startIndexH;
      while (endIndexH > 0 && m_threshH[endIndexH] == 0)
        --endIndexH;
      while (startIndexV < (int)m_threshV.size() && m_threshV[startIndexV] == 0)
        ++startIndexV;
      while (endIndexV > 0 && m_threshV[endIndexV] == 0)
        --endIndexV;
      if (startIndexH < endIndexH)
        std::fill(m_threshH.begin()+startIndexH, m_threshH.begin()+endIndexH, 255);
      if (startIndexV < endIndexV)
        std::fill(m_threshV.begin()+startIndexV, m_threshV.begin()+endIndexV, 255);
      return;
    }

    computeThreshold(m_histoH, m_countH, m_threshH, m_noiseFactor);
    computeThreshold(m_histoV, m_countV, m_threshV, m_noiseFactor);

    //TODO:REMOVE
//    std::cout << " ******** PROFILE HORIZ THRESH *****" << std::endl;
//    for (size_t i = 0; i < m_threshH.size(); ++i) {
//      std::cout << i << " " << (int)m_threshH[i] << std::endl;
//    }
//    std::cout << " ******** END PROFILE HORIZ THRESH*****" << std::endl;
//    std::cout << " ******** PROFILE VERTI THRESH*****" << std::endl;
//    for (size_t i = 0; i < m_threshV.size(); ++i) {
//      std::cout << i << " " << (int)m_threshV[i] << std::endl;
//    }
//    std::cout << " ******** END PROFILE VERTI THRES*****" << std::endl;


#if 0
    std::vector<size_t> freq;
    std::cout << "Freq histo for profile horizontal" << std::endl;
    computeFreqHisto(h, m_histoH, freq);
    std::cout << "Freq histo for profile Vertical" << std::endl;
    computeFreqHisto(w, m_histoV, freq);
#endif

    if (m_erodeHorizSize && m_dilateHorizSize) {
    //  std::cout << "**** HORIZONTAL****" << std::endl;
      extend(m_threshH, m_histoH);
      filterThreshold(m_threshH, m_erodeHorizSize, m_dilateHorizSize);

    }

    if (m_erodeVertiSize && m_dilateVertiSize) {
      //std::cout << "******** VERTICAL*******" << std::endl;
      extend(m_threshV, m_histoV);
      filterThreshold(m_threshV, m_erodeVertiSize, m_dilateVertiSize);
    }

  }

  void
  SegmentationClusteringProfile::init(girl::region, int)
  {
  }

  bool
  SegmentationClusteringProfile::same(const girl::point &p, girl::direction dir)
  {
    girl::point p2 = p;
    p2.move(dir);
    size_t indexPH  = p.x()  - m_box.xMin();
    size_t indexPV  = p.y()  - m_box.yMin();
    size_t indexP2H = p2.x() - m_box.xMin();
    size_t indexP2V = p2.y() - m_box.yMin();
    return m_threshH[indexPH] == m_threshH[indexP2H] && m_threshV[indexPV] == m_threshV[indexP2V];
  }

  void
  SegmentationClusteringProfile::processPoint(const girl::point &pt)
  {
    size_t indexH = pt.x() - m_box.xMin();
    size_t indexV = pt.y() - m_box.yMin();
    assert(indexH < m_histoH.size());
    assert(indexV < m_histoV.size());
    const girl::shared_ptr<girl::tools::ImageFeature> image = m_binaryImage->getImageFeature();
    assert(image);
    girl::tools::PixelFromImage<girl::tools::ImageFeature::type> pix =
      image->getPixel(pt.x(), pt.y());

    m_countH[indexH]++;
    m_countV[indexV]++;
    // count the number of  black pixels in the binary image
    if (pix[0] == 0) {
      m_histoH[indexH]++;
      m_histoV[indexV]++;
    }
  }


  void
  SegmentationClusteringProfile::computeThreshold(const std::vector<size_t> &histo,
						  const std::vector<size_t> &count,
						  std::vector<unsigned char> &threshold,
						  double factor) const
  {
    assert(histo.size() == count.size() && histo.size() == threshold.size());
    std::vector<size_t>::const_iterator histoIt   = histo.begin();
    std::vector<size_t>::const_iterator countIt   = count.begin();
    std::vector<unsigned char>::iterator threshIt = threshold.begin();
    for (; histoIt != histo.end(); ++histoIt, ++countIt, ++threshIt) {
      double threshold = (*countIt)*factor;
      *threshIt = (*histoIt > threshold) ? 255 : 0;
    }
  }

  void
  SegmentationClusteringProfile::filterThreshold(std::vector<unsigned char> &thresh, size_t erodeSize, size_t dilateSize) 
  {
    cv::Mat mat = cv::Mat(thresh).t();
    assert(mat.type() == CV_8U);
    cv::Mat erodeElem  = cv::getStructuringElement(cv::MORPH_RECT, cv::Size(erodeSize, 1));
    cv::Mat dilateElem = cv::getStructuringElement(cv::MORPH_RECT, cv::Size(dilateSize, 1));

    cv::morphologyEx(mat, mat, cv::MORPH_OPEN, erodeElem);
    //TODO:REMOVE
    //unsigned char* buf = mat.data;
//    std::cout << " ******** PROFILE OPEN *****" << std::endl;
//    for (size_t i = 0; i < thresh.size(); ++i) {
//      std::cout << i << " " << (int)buf[i] << std::endl;
//    }
//    std::cout << " ******** END PROFILE OPEN*****" << std::endl;
    cv::morphologyEx(mat, mat, cv::MORPH_CLOSE, dilateElem);
//    std::cout << " ******** PROFILE CLOSE *****" << std::endl;
//    for (size_t i = 0; i < thresh.size(); ++i) {
//      std::cout << i << " " << (int)buf[i]<< std::endl;
//    }
//    std::cout << " ******** END PROFILE CLOSE*****" << std::endl;

    // no border effect on thresh even when copyData = false, dont know why
    // copy back results
    assert(mat.isContinuous());
    memcpy(&thresh[0], mat.data, thresh.size()*sizeof(unsigned char));

   }

  void
  SegmentationClusteringProfile::extend(std::vector<unsigned char> &thresh, const std::vector<size_t> &histo) 
  {
#if 0
    {//DEBUG
      size_t numBlocks = 0;
      for (size_t i=0; i<thresh.size()-1; ++i) {
	if (thresh[i] != thresh[i+1])
	  ++numBlocks;
      }
      std::cerr<<"numblocks before="<<numBlocks<<std::endl;
    }
#endif

    assert(thresh.size() == histo.size());

    //cv::Mat &mat = *(m_binaryImage->getImageFeature()->cvMat().get());
    if (thresh.size() > 0) {
      for (size_t i=0; i<thresh.size()-1; ++i) {
        if (thresh[i] != thresh[i+1] && thresh[i]==255) { //black->white
          size_t numBlacks = histo[i+1];
          if (numBlacks > 0) {
	    thresh[i+1] = 255;
          }
        }
      }
      for (size_t i=thresh.size() - 1; i>1; --i) {
        if (thresh[i] != thresh[i-1] && thresh[i]==255) { //black->white
          size_t numBlacks = histo[i-1];
          if (numBlacks > 0) {
            thresh[i-1] = 255;
          }
        }
      }
      
    }

#if 0
    {//DEBUG
      size_t numBlocks = 0;
      for (size_t i=0; i<thresh.size()-1; ++i) {
	if (thresh[i] != thresh[i+1])
	  ++numBlocks;
      }
      std::cerr<<"numblocks after="<<numBlocks<<std::endl;
    }
#endif

 
  }

  void
  SegmentationClusteringProfile::computeProfile(const girl::regionDeque &regions)
  {
    assert(m_segContext);
    girl::toger *tog = m_segContext->getToger();
    assert(tog);
    girl::regionDeque::const_iterator it = regions.begin();
    const girl::regionDeque::const_iterator itEnd = regions.end();
    for (; it != itEnd; ++it) {
      tog->iterateOnRegionDomain(*it, *this);
    }
  }

  void
  SegmentationClusteringProfile::computeProfile2(const girl::regionDeque &regions)
  {
    cv::Mat mask = cv::Mat::zeros(m_box.height(), m_box.width(), CV_8U);
    girl::regionDeque::const_iterator it = regions.begin();
    const girl::regionDeque::const_iterator itEnd = regions.end();
    girl::tools::Painter<unsigned char> p(mask, CV_RGB(255,255,255));
    for (; it != itEnd; ++it) {
      m_segContext->getToger()->iterateOnRegionDomain(*it, p);
    }
    const girl::shared_ptr<girl::tools::ImageFeature> image = m_binaryImage->getImageFeature();
    girl::shared_ptr<cv::Mat> toto = image->cvMat();
    cv::Mat roi = (*toto)(cv::Rect(m_box.xMin(), m_box.yMin(), m_box.width(), m_box.height()));
    cv::Mat binary(roi.size(), CV_32F);
    binary = roi > 0;
    cv::Mat binaryMask;
    binary.copyTo(binaryMask, mask);
    binaryMask.convertTo(binary, CV_32F);
    cv::Mat v1 = cv::Mat::ones(roi.cols, 1, CV_32F);
    cv::Mat v2 = cv::Mat::ones(1, roi.rows, CV_32F);
    cv::Mat profileHoriz = binary*v1;
    cv::Mat profileVerti = v2*binary;
  }


  int getSum(const cv::Mat &mat, const girl::point &pt1, const girl::point &pt2)
  {
    assert(mat.type() == CV_32S);
    const int x1 = pt1.y();
    const int y1 = pt1.x();
    const int x2 = pt2.y() + 1;
    const int y2 = pt2.x() + 1;
    return mat.at<int>(x2,y2) - mat.at<int>(x1, y2) - mat.at<int>(x2, y1) + mat.at<int>(x1, y1);
  }
/*
  void
  SegmentationClusteringProfile::computeProfileFast(const girl::regionDeque &regs)
  {
    assert(m_segContext);
    girl::toger *tog = m_segContext->getToger();
    assert(tog);
    assert(!m_integralImage.empty());
    for (girl::regionDeque::const_iterator it = regs.begin();
         it != regs.end(); ++it) {
      std::vector<girl::point> scan;
      girl::point p;
      tog->regionDomain(*it, scan);

      for (size_t i = 0; i < scan.size(); i+=2)
      {
        size_t indexPH  = p.x()  - m_box.xMin();
        size_t indexPV  = p.y()  - m_box.yMin();
        size_t indexP2H = p2.x() - m_box.xMin();
        size_t indexP2V = p2.y() - m_box.yMin();
        int s1 = getSum(m_integralImage, scan[i], scan[i+1]);
        int s2 = getSum(m_integralImage.t(), girl::point(scan[i].y(), scan[i].x()), girl::point(scan[i+1].y(), scan[i+1].x()));

      }




      std::vector<cv::Rect> toRemove;
      girl::tools::AABBox box = girl::tools::getAABBox(*tog, *it);
      const girl::point xMin = box.xMin();
      const girl::point xMax = box.xMax();
      std::vector<girl::point>::const_iterator startIt = scan.begin();
      std::vector<girl::point>::const_iterator endIt = startIt;
      for (int i = 0; i < box.height(); ++i) {
        girl::POINT_TYPE currentY = box.yMin() + i;
        while(endIt->y() == currentY)
          endIt++;
        endIt--;
        if (startIt->x() > xMin) {
          toRemove.push_back(cv::Rect(xMin, currentY, startIt.x() - xMin, 1));
        }
        startIt += 2;
        for (; startIt < endIt; startIt += 2) {
          const girl::point &previous = *(startIt - 1);
          const girl::point &current  = *startIt;
          toRemove.push_back(cv::Rect(previous.x() + 1, currentY, current.x() - previous.x() + 1, 1 ));
        }
        if (endIt->x() < xMax) {
          toRemove.push_back(cv::Rect(endIt->x()+1, currentY, xMax - endIt->x()));
        }
      }
    }

  }
*/

}
