#ifndef ISESEGMENTATIONCLUSTERINGPROFILE_HPP
#define ISESEGMENTATIONCLUSTERINGPROFILE_HPP

#include <girl/girlCommon.hpp>
#include <girl/girlSharedPtr.hpp>

#include <girlTools/girlToolsAABBox.hpp>
#include <girlTools/girlToolsImageFeature.hpp>

#include "iseImageFeatureThreshold.hpp"
#include "iseSegmentationClustering.hpp"

namespace ise
{
class GIRL_API SegmentationClusteringProfile : public SegmentationClustering,
  public girl::toger::pointIterator
{
public:
 explicit SegmentationClusteringProfile(ise::SegmentationContext *segContext);

 virtual
 ~SegmentationClusteringProfile() {}

 void init(const girl::regionDeque &regions, int level);

 void init(girl::region r, int level);

 bool same(const girl::point &p, girl::direction dir);

 void setParameters(size_t erodeHoriz,
                    size_t dilateHoriz,
                    size_t erodeVerti,
                    size_t dilateVerti,
                    double threshold,
                    const ise::FeatureComputerPtr &algo)
 {
  m_erodeHorizSize  = erodeHoriz;
  m_dilateHorizSize = dilateHoriz;
  m_erodeVertiSize  = erodeVerti;
  m_dilateVertiSize = dilateVerti;
  m_noiseFactor = threshold;
  m_binaryImage = girl::static_pointer_cast<ise::ImageFeatureThreshold>(algo);

 }


protected:

 void processPoint(const girl::point &);

 void computeThreshold(const std::vector<size_t> &histo, const std::vector<size_t> &count, std::vector<unsigned char> &threshold, double factor) const;

 void filterThreshold(std::vector<unsigned char> &thresh, size_t erodeSize, size_t dilateSize);

 void extend(std::vector<unsigned char> &thresh, const std::vector<size_t> &histo);

 void computeProfile(const girl::regionDeque &regs);
 void computeProfile2(const girl::regionDeque &regs);
protected:

 std::vector<size_t> m_histoH;
 std::vector<size_t> m_histoV;
 std::vector<size_t> m_countH;
 std::vector<size_t> m_countV;
 std::vector<unsigned char> m_threshH;
 std::vector<unsigned char> m_threshV;

 girl::shared_ptr<ise::ImageFeatureThreshold> m_binaryImage;

 girl::tools::AABBox m_box;

 double m_noiseFactor;

 size_t m_erodeHorizSize;
 size_t m_erodeVertiSize;
 size_t m_dilateHorizSize;
 size_t m_dilateVertiSize;


};

}
#endif // ISESEGMENTATIONCLUSTERINGPROFILE_HPP
