/*
  Segmentation clustering: RLSA
  Copyright (C) 2005-2011 Roigt Julian, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISESEGMENTATIONCLUSTERINGRLSA_HPP
#define ISESEGMENTATIONCLUSTERINGRLSA_HPP

#include <girl/girlToger.hpp>
#include "iseSegmentationClustering.hpp"
#include <opencv2/core/core.hpp>
#include <iostream>

namespace ise {

  class SegmentationContext;
  
  class  GIRL_API SegmentationClusteringRLSA : public SegmentationClustering
  {
  public :

    explicit SegmentationClusteringRLSA(SegmentationContext *segContext);
    
    virtual ~SegmentationClusteringRLSA() {std::cout<<"end"<<std::endl;}

    virtual void setSegContext(ise::SegmentationContext *s);

    virtual void setParameters(int colorSpace, int channel, const std::vector<int> &levels, int C_horizontal, int C_s, int C_vertical);

    virtual void getParameters(int &colorSpace, int &channel, std::vector<int> &levels, int &C_horizontal, int &C_s, int &C_vertical) const;
    
    virtual void init(const girl::regionDeque &regions, int level=1);

    virtual void init(girl::region /*r*/, int /*level*/) {} 
    
    virtual bool same(const girl::point &p, girl::direction dir);

    void compute(cv::Mat &image, int threshold = -1);

  protected:
    
    void computeBestValue(const std::vector<int> &level);

  private:
    int m_colorSpace;
    int m_channel;
    std::vector<int> m_levels;
    int m_C_horizontal;
    int m_C_s;
    int m_C_vertical;
    cv::Mat m_img;
  };

}

#endif /* ! ISESEGMENTATIONCLUSTERINGRLSA_HPP */
