/*
  Segmentation clustering: RXYC
  Copyright (C) 2005-2011 Roigt Julian, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISESEGMENTATIONCLUSTERINGRXYC_HPP
#define ISESEGMENTATIONCLUSTERINGRXYC_HPP

#include <girl/girlToger.hpp>
#include "iseSegmentationClustering.hpp"
#include <girlTools/girlToolsImage.hpp>
#include "iseThresholderData.hpp"

#include <opencv2/core/core.hpp>
#include <limits>

namespace ise {

  class SegmentationContext;
  class Boxe;

  class  GIRL_API SegmentationClusteringRXYC : public SegmentationClustering
  {
  public :

    explicit SegmentationClusteringRXYC(SegmentationContext *segContext);
    
    virtual ~SegmentationClusteringRXYC() {}

    virtual void setSegContext(ise::SegmentationContext *s);

    virtual void setParameters(const ise::ThresholderData &datas, int iteration);

    virtual void getParameters(ise::ThresholderData &datas, int &iteration) const;
    
    virtual void init(const girl::regionDeque &regions, int level=1);

    virtual void init(girl::region , int) {} 
    
    virtual bool same(const girl::point &p, girl::direction dir);

    void compute(const girl::regionDeque &regions, cv::Mat &retImage);

    void rxyc(girl::tools::Image *image, std::deque<ise::Boxe> &boxes, int iterationMax);

    void fillBoxesOnImage(const std::deque<ise::Boxe> &boxes, cv::Mat &img);

  protected:
    void rxyc(girl::tools::Image *image, const std::deque<ise::Boxe> &boxe, std::deque<ise::Boxe> &boxes, int threshold, int iteration = std::numeric_limits<int>::max());

    void yCut(girl::tools::Image *image, const ise::Boxe &boxe, std::deque<ise::Boxe> &boxes, int threshold);
    void xCut(girl::tools::Image *image, const ise::Boxe &boxe, std::deque<ise::Boxe> &boxes, int threshold, int iteration = std::numeric_limits<int>::max());

  private:
    
    ise::ThresholderData m_datas;
    cv::Mat m_img;
    int m_iteration;
  };

}

#endif /* ! ISESEGMENTATIONCLUSTERINGRXYC_HPP */
