/*
  Segmentation clustering: Markov.
  Copyright (C) 2005-2009 Claire Morand, Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "iseSegmentationClusteringRegionGrowing.hpp"

#include "iseSegmentationContext.hpp"
#include <iostream>

using namespace ise;


SegmentationClusteringRegionGrowing::~SegmentationClusteringRegionGrowing()
{
  
}

SegmentationClusteringRegionGrowing::SegmentationClusteringRegionGrowing(SegmentationContext *segContext,
									 double threshold)
  : SegmentationClustering(segContext),
    m_threshold(threshold),
    m_maxLabel(0)
{

}


bool
SegmentationClusteringRegionGrowing::same(const girl::point &p, girl::direction dir)
{ 
  assert(p.x()>=0 && p.x()<m_segContext->getImage()->width()
	 && p.y()>=0 && p.y()<m_segContext->getImage()->height());


  std::map<girl::point, int, ordrePointRegionGrowing>::iterator it;
  it = m_allPoints.find(p);
  int labelp = 0;
  if (it != m_allPoints.end())
    labelp = it->second;
  
  girl::point p2 = p;
  p2.move(dir);
  if (p2.x() < 0 || p2.x() >= m_segContext->getImage()->width()
      || p2.y() < 0 || p2.y() >= m_segContext->getImage()->height())
    return false;
  it = m_allPoints.find(p2);
  int labelp2 = 0;
  if (it != m_allPoints.end())
    labelp2 = it->second;


  if (labelp2 == labelp)
    return true;
  else
    return false;
}

void
SegmentationClusteringRegionGrowing::init(const girl::regionDeque &/*regions*/, int /*level*/)
{
  m_allPoints.clear();
  m_allRegions.clear();
  //m_maxLabel = 0;

  const int width = m_segContext->getImage()->width();
  const int height = m_segContext->getImage()->height();

  const int wh = width*height;
  std::vector<int> control(wh, 0);

  m_maxLabel = 1;
  for (int r = 0 ; r < height ; ++r) {
    for (int c = 0 ; c <width ; ++c) {
      if (control[r*width + c] == 0) {
	control[r*width + c] = 1;
	const girl::point p(c, r);
	m_allPoints.insert(std::pair<girl::point, int>(p, m_maxLabel));
	const girl::tools::Pixel pix = m_segContext->getImage()->getPixel(c, r);
	regionGrowingProperties regionTmp(pix);
	m_allRegions.insert(std::pair<int, regionGrowingProperties>(m_maxLabel, regionTmp));
	this->growRegion(r, c, height, width, m_maxLabel, &control[0]);
	++m_maxLabel;
      }
    }
  }
}

void
SegmentationClusteringRegionGrowing::growRegion(int r, int c, 
						int height, int width,
						int label, int *control)
{
  std::vector<girl::point> pointsToVisit;
  if (r>0) {
    const girl::point p(c, r-1);
    pointsToVisit.insert(pointsToVisit.begin(), p);
  }
  if (r<height-1) {
    const girl::point p(c, r+1);
    pointsToVisit.insert(pointsToVisit.begin(), p);
  }
  if (c>0) {
    const girl::point p(c-1, r);
    pointsToVisit.insert(pointsToVisit.begin(), p);
  }
  if (c<width-1) {
    const girl::point p(c+1, r);
    pointsToVisit.insert(pointsToVisit.begin(), p);
  }

  while (/*pointsToVisit.size()  < 50 &&*/ ! pointsToVisit.empty()) {

    const girl::point currentPoint = pointsToVisit.back();
    const int posX = currentPoint.x();
    const int posY = currentPoint.y();

    girl::tools::Pixel pix = m_segContext->getImage()->getPixel(posX, posY);

    regionGrowingProperties &prop = m_allRegions[label]; 
    const bool isInRegion = prop.compare(pix, static_cast<int>(m_threshold));

    if (isInRegion) {

      control[posY*width+posX] = 1;
      prop.addPixel(pix);
      m_allPoints.insert(std::pair<girl::point, int>(currentPoint, label));

      if (posY>0)
	if (control[(posY-1)*width+posX] == 0) {
	  const girl::point p(posX, posY-1);
	  bool flag = true;
	  for (size_t i = 0 ; i < pointsToVisit.size() ; ++i)
	    if (pointsToVisit[i] == p)
	      flag = false;
	  if (flag)
	    pointsToVisit.insert(pointsToVisit.begin(), p);
	}

      if (posY<height-1)
	if (control[(posY+1)*width+posX] == 0) {
	  girl::point p(posX, posY+1);
	  bool flag = true;
	  for (size_t i = 0 ; i < pointsToVisit.size(); ++i)
	    if (pointsToVisit[i] == p)
	      flag = false;
	  if (flag)
	    pointsToVisit.insert(pointsToVisit.begin(), p);
	}

      if (posX>0)
	if (control[posY*width+posX-1] == 0) {
	  girl::point p(posX-1, posY);
	  bool flag = true;
	  for (size_t i = 0 ; i < pointsToVisit.size(); ++i)
	    if (pointsToVisit[i] == p)
	      flag = false;
	  if (flag)
	    pointsToVisit.insert(pointsToVisit.begin(), p);
	}

      if (posX<width-1)
	if (control[posY*width+posX+1] == 0) {
	  girl::point p(posX+1, posY);
	  bool flag = true;
	  for (size_t i = 0 ; i < pointsToVisit.size(); ++i)
	    if (pointsToVisit[i] == p)
	      flag = false;
	  if (flag)
	    pointsToVisit.insert(pointsToVisit.begin(), p);
	}
    }

    pointsToVisit.pop_back();

  }
}
			    

void
SegmentationClusteringRegionGrowing::init(girl::region /*r*/, int /*level*/)
{

}

