/*
  Segmentation clustering: Markov.
  Copyright (C) 2005-2009 Claire Morand, Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISESEGMENTATIONCLUSTERINGREGIONGROWING_HPP
#define ISESEGMENTATIONCLUSTERINGREGIONGROWING_HPP


#include <girl/girlCommon.hpp>
#include <girl/girlToger.hpp>
#include <girlTools/girlToolsImage.hpp>
#include "iseSegmentationClustering.hpp"
#include <map>
#include <cmath>

#include <iostream>

//structure pour stocker les labels et les points
class ordrePointRegionGrowing
{
public:
  bool operator()(girl::point p1, girl::point p2) const
  {
    if (p1.y() < p2.y())
      return 1;
    if (p1.y() == p2.y())
      if (p1.x() < p2.x())
	return 1;
    return 0;
  }
};

class regionGrowingProperties
{
private:
  girl::tools::Image::type m_r;
  girl::tools::Image::type m_g;
  girl::tools::Image::type m_b;
  int m_nbPixels;

public:

  ~regionGrowingProperties()
  {}

  regionGrowingProperties()
  {
    m_r = 0;
    m_g = 0;
    m_b = 0;
    m_nbPixels = 0;
  }

  explicit regionGrowingProperties(const girl::tools::Pixel &p)
  {
    m_r = p.a();
    m_g = p.b();
    m_b = p.c();
    m_nbPixels = 1;
  }

  regionGrowingProperties(const regionGrowingProperties &source)
  {
    m_r = source.m_r;
    m_g = source.m_g;
    m_b = source.m_b;
    m_nbPixels = source.m_nbPixels;
  }

  void addPixel(const girl::tools::Pixel &p)
  {
    m_r += p.a();
    m_g += p.b();
    m_b += p.c();
    ++m_nbPixels;
  }

  bool compare(const girl::tools::Pixel &p, int threshold) const
  {
    assert(m_nbPixels>0);
    const double inv_nbPixels = 1.0 / static_cast<double>(m_nbPixels);
    const double rAverage = static_cast<double>(m_r) * inv_nbPixels;
    const double gAverage = static_cast<double>(m_g) * inv_nbPixels;
    const double bAverage = static_cast<double>(m_b) * inv_nbPixels;
    const double value = fabs(rAverage - static_cast<double>(p.a())) 
      + fabs(gAverage - static_cast<double>(p.b())) 
      + fabs(bAverage - static_cast<double>(p.c()));

    return ( static_cast<int>(value) < threshold);
  }
};


namespace ise {

  class SegmentationContext;

  class GIRL_API SegmentationClusteringRegionGrowing : public SegmentationClustering
  {
    
  public:
    
    /**
     * @brief Destructor.
     */
    ~SegmentationClusteringRegionGrowing();
  
    /**
     * @brief Constructor.
     * 
     * @warning takes ownership of @a histo.
     */
    SegmentationClusteringRegionGrowing(SegmentationContext *segContext, double threshold = 15.0);

    virtual void init(const girl::regionDeque &regions, int level);
    
    virtual void init(girl::region r, int level);
    
    void setParameters(double threshold) 
    { 
      m_threshold = threshold;
    }

    void getParameters(double &threshold) const 
    { 
      threshold = m_threshold;
    }
    
    std::map<girl::point, int, ordrePointRegionGrowing> &getSegmentation()
    {
      return m_allPoints;
    }
    
    virtual bool same(const girl::point &p, girl::direction dir);

    void growRegion(int l, int c, int height, int width,
		    int label, int *control);
    
  private:
    SegmentationClusteringRegionGrowing(const SegmentationClusteringRegionGrowing &);
    SegmentationClusteringRegionGrowing &operator=(const SegmentationClusteringRegionGrowing &);

  protected:
    std::map<girl::point, int, ordrePointRegionGrowing> m_allPoints;
    std::map<int, regionGrowingProperties> m_allRegions;
    double m_threshold;
    int m_maxLabel;
  };

}

#endif /* ! ISESEGMENTATIONCLUSTERINGREGIONGROWING_HPP */
