/*
  Segmentation clustering: ShapeCC
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
#include <ise/iseSegmentationContext.hpp>
#include <ise/iseConversion.hpp>
#include <girl/girlCommon.hpp>

#include <ise/iseCvSobel.hpp>
#include <ise/iseHistogram1D.hpp>
#include <ise/iseImageFeatureColorSpace.hpp>
#include <ise/iseQuantificationThresholderOtsu.hpp>


#include "iseSegmentationClusteringShapeCC.hpp"
#include "EnclosingShapeFactory.hpp"
#include "EnclosingBoxFactory.hpp"
#include "EnclosingCircleFactory.hpp"
#include "EnclosingEllipseFactory.hpp"
#include "EnclosingRotatedRectFactory.hpp"
#include "EnclosingConvexHullFactory.hpp"
#include <list>

#include <opencv2/imgproc/imgproc.hpp>

#ifdef VISU
#include <opencv2/highgui/highgui.hpp>
#endif //VISU

using namespace ise;

SegmentationClusteringShapeCC::SegmentationClusteringShapeCC(SegmentationContext *segContext)
: SegmentationClustering(segContext)
, m_Shapes("")
, m_image()
{
  setSegContext(segContext);
}

void
SegmentationClusteringShapeCC::setSegContext(ise::SegmentationContext *s) 
{
  SegmentationClustering::setSegContext(s);
}

void 
SegmentationClusteringShapeCC::init(const girl::regionDeque &/*regions*/, int /* level */)
{
  girl::tools::Image *img = m_segContext->getImage();


  //  ise::ImageFeatureColorSpace luvComputer(m_segContext);
  //  luvComputer.setParameters(ISE_RGB_to_Luv);
  //  luvComputer.compute();
  //
  //  girl::shared_ptr<girl::tools::ImageFeature> luvImage =
  //      luvComputer.getImageFeature();
  //
  //
  //  // HARD CODED Roberts Filters for interpixel boundaries
  //  assert(luvImage->channels() == 3);
  //
  //  int width = img->width();
  //  int height = img->height();
  //
  //  girl::tools::ImageFeature horizBoundaries(width, height, 1);
  //  girl::tools::ImageFeature vertiBoundaries(width, height, 1);
  //  girl::tools::ImageFeature interPixelBoundaries(width, 2*height, 1);
  //
  //  // keep track of max gradient module
  //  double maxHoriz = 0;
  //  double maxVerti = 0;
  //
  //  for (int i = 0; i < height - 1; ++i) {
  //      girl::tools::ImageFeature::type *line = luvImage->scanLine(i);
  //      girl::tools::ImageFeature::type *nextLine = luvImage->scanLine(i + 1);
  //      girl::tools::ImageFeature::type *nextPixel = line + luvImage->formatSize();
  //      girl::tools::ImageFeature::type *horizBuf = horizBoundaries.scanLine(i);
  //      girl::tools::ImageFeature::type *vertiBuf = vertiBoundaries.scanLine(i);
  //      girl::tools::ImageFeature::type *interpVertBuf = interPixelBoundaries.scanLine(2*i);
  //      girl::tools::ImageFeature::type *interpHorzBuf = interPixelBoundaries.scanLine(2*i + 1);
  //
  //      for (int j = 0; j < width - 1; ++j) {
  //          //	if (i == 46 && j == 44) {
  //          //	  std::cout<< "break" << std::endl;
  //          //	}
  //          double gradHoriz = sqrt(cvflann::euclidean_dist(line, line + luvImage->formatSize(), nextLine));
  //          double gradVerti = sqrt(cvflann::euclidean_dist(line, line + luvImage->formatSize(), nextPixel));
  //          if (gradHoriz > maxHoriz)
  //            maxHoriz = gradHoriz;
  //          if (gradVerti > maxVerti)
  //            maxVerti = gradVerti;
  //          *horizBuf = gradHoriz;
  //          *vertiBuf = gradVerti;
  //
  //          *interpVertBuf = gradVerti;
  //          *interpHorzBuf = gradHoriz;
  //          interpVertBuf += interPixelBoundaries.formatSize();
  //          interpHorzBuf += interPixelBoundaries.formatSize();
  //
  //          line += luvImage->formatSize();
  //          nextLine += luvImage->formatSize();
  //          nextPixel += luvImage->formatSize();
  //          horizBuf += horizBoundaries.formatSize();
  //          vertiBuf += vertiBoundaries.formatSize();
  //      }
  //  }
  //
  //  double min, max;
  //  cv::minMaxLoc(*horizBoundaries.cvMat(), &min, &max);
  //  cv::Mat displayGradHoriz;
  //  horizBoundaries.cvMat()->convertTo(displayGradHoriz, CV_8U, 255./(max - min), -min);
  //  cv::namedWindow("Horizontal Gradient", 0);
  //  cv::imshow("Horizontal Gradient", displayGradHoriz);
  //  cv::imwrite("Horiz.pgm", displayGradHoriz);
  //  cv::minMaxLoc(*vertiBoundaries.cvMat(), &min, &max);
  //  cv::Mat displayGradVerti;
  //  vertiBoundaries.cvMat()->convertTo(displayGradVerti, CV_8U, 255./(max - min), -min);
  //  cv::namedWindow("Vertical Boundaries", 0);
  //  cv::imshow("Vertical Boundaries", displayGradVerti);
  //
  //  cv::Mat displayInterpix;
  //  cv::minMaxLoc(*interPixelBoundaries.cvMat(), &min, &max);
  //  cv::namedWindow("Interpixel Boundaries", 0);
  //  interPixelBoundaries.cvMat()->convertTo(displayInterpix, CV_8U, 255./(max - min), -min);
  //  cv::imshow("Interpixel Boundaries", displayInterpix);
  //
  //
  //
  //  ise::Histogram1D histoHoriz(0, static_cast<int>(maxHoriz));
  //  ise::Histogram1D histoVerti(0, static_cast<int>(maxVerti));
  //  for (int i = 0; i < horizBoundaries.height(); ++i) {
  //      girl::tools::ImageFeature::type *hBuf = horizBoundaries.scanLine(i);
  //      girl::tools::ImageFeature::type *vBuf = vertiBoundaries.scanLine(i);
  //      for (int i = 0; i < horizBoundaries.width(); ++i) {
  //          histoHoriz[static_cast<int>(*hBuf)]++;
  //          histoVerti[static_cast<int>(*vBuf)]++;
  //          hBuf += horizBoundaries.formatSize();
  //          vBuf += vertiBoundaries.formatSize();
  //      }
  //  }
  //
  //
  //  ise::QuantificationThresholderOtsu otsu;
  //  std::vector<int> levelsHoriz;
  //  otsu.getLevels(histoHoriz, levelsHoriz);
  //  std::cout << "Levels horiz: " << levelsHoriz[0] << " " << levelsHoriz[1] << std::endl;
  //
  //  //  HistogramView histoViewH(histoHoriz, NULL);
  //  //  histoViewH->setVisible(true);
  //  //
  //  //  cv::waitKey(2000);
  //  //
  //  //  HistogramView histoViewV(histoVerti, NULL);
  //  //  histoViewV->setVisible(true);
  //
  //  std::vector<int> levelsVerti;
  //  otsu.getLevels(histoVerti, levelsVerti);
  //  std::cout << "Levels Verti: " << levelsVerti[0] << " " << levelsVerti[1] << std::endl;
  //
  //
  //  cv::Mat toThresh = cv::Mat::zeros(height, width, CV_8U);
  //  unsigned char *data = toThresh.data;
  //  for (int i = 0; i < height; ++i) {
  ////      girl::tools::ImageFeature::type *hBuf = horizBoundaries.scanLine(i);
  ////      girl::tools::ImageFeature::type *vBuf = vertiBoundaries.scanLine(i);
  //      girl::tools::Image::type *imgBuf = img->scanLine(i);
  //      for (int j = 0; j < width; ++j) {
  //          if (!(*imgBuf)) {
  //              *data = 255;
  //          }
  //          ++data;
  //          imgBuf += img->formatSize();
  ////          if (*vBuf > levelsVerti[1]) {
  ////              toThresh.at<unsigned char>(i, j) = 255;
  ////     //         std::cout << "Paint pix vertical " << i << "," << j << std::endl;
  ////          }
  ////          if (*hBuf > levelsHoriz[1]) {
  ////              toThresh.at<unsigned char>(i, j) = 255;
  ////       //       std::cout << "Paint pix horiz" << i << "," << j << std::endl;
  ////          }
  ////          //	++data;
  ////          ++vBuf;
  ////          ++hBuf;
  //      }
  //  }
  //
  //  //  ise::CvSobel sobel(luvImage);
  //  //  girl::shared_ptr<girl::tools::ImageFeature> gradX;
  //  //  girl::shared_ptr<girl::tools::ImageFeature> gradY;
  //  //
  //  //  sobel.compute(gradX, 1, 0);
  //  //  sobel.compute(gradY, 0, 1);
  //  //
  //  //  girl::tools::ImageFeature module(img->width(), img->height(), 1);
  //  //
  //  //
  //  //
  //  //  double max = 0;
  //  //  for (int i = 0; i < img->height(); ++i) {
  //  //    girl::tools::ImageFeature::type *bufX = gradX->scanLine(i);
  //  //    girl::tools::ImageFeature::type *bufY = gradY->scanLine(i);
  //  //
  //  //    girl::tools::ImageFeature::type *dest = module.scanLine(i);
  //  //    for (int j = 0; j < img->width(); ++j) {
  //  //      double modX = 0;
  //  //      double modY = 0;
  //  //      for (int k = 0; k < gradX->channels(); ++k) {
  //  //        modX += (bufX[k]*bufX[k]);
  //  //        modY += (bufY[k]*bufY[k]);
  //  //      }
  //  //      double val = sqrt(modX + modY);
  //  //      //  std::cout << val << std::endl;
  //  //      if (val > max)
  //  //        max = val;
  //  //      *dest = val;
  //  //      bufX += gradX->formatSize();
  //  //      bufY += gradY->formatSize();
  //  //      dest += module.formatSize();
  //  //    }
  //  //  }
  //
  //  //  ise::Histogram1D histo(0, static_cast<int>(max));
  //  //  for (int i = 0; i < module.height(); ++i) {
  //  //      girl::tools::ImageFeature::type *dest = module.scanLine(i);
  //  //      for (int i = 0; i < module.width(); ++i) {
  //  //          histo[static_cast<int>(*dest)]++;
  //  //          dest += module.formatSize();
  //  //      }
  //  //
  //  //  }
  //  //
  //  //  for (int i = 0; i < (int) max; ++i) {
  //  //      std::cout << histo[i] << std::endl;
  //  //  }
  //  //
  //  //  ise::QuantificationThresholderOtsu otsu;
  //  //  std::vector<int> levels;
  //  //  otsu.getLevels(histo, levels);
  //  //  std::cout << "Levels: " << levels[0] << " " << levels[1] << std::endl;
  //  //
  //  //  cv::Mat toThresh = cv::Mat::zeros(module.height(), module.width(), CV_8UC1);
  //  //  unsigned char *data = toThresh.data;
  //  //  for (int i = 0; i < module.height(); ++i) {
  //  //      girl::tools::ImageFeature::type *dest = module.scanLine(i);
  //  //      for (int i = 0; i < module.width(); ++i) {
  //  //          if (*dest > static_cast<double>(levels[0]))
  //  //            *data = 0;//128;
  //  //          if (*dest > static_cast<double>(levels[1]))
  //  //            *data = 255;
  //  //          ++data;
  //  //          dest += module.formatSize();
  //  //      }
  //  //
  //  //  }
  //
  //  cv::namedWindow("Hysteresis me", 0);
  //  cv::imshow("Hysteresis me", toThresh);
  //  cv::waitKey(0);
  //
  //
  //
  //
  //  // cv::Mat *mat = img->cvMat();
  //
  //  // cv::Mat luvImage;
  //  // cv::cvtColor(*mat, luvImage, CV_BGR2Luv);
  //
  //  // cv::Mat sobelX, sobelY;
  //  // cv::Sobel(luvImage, sobelX, CV_32F, 1, 0, 3);
  //  // cv::Sobel(luvImage, sobelY, CV_32F, 0, 1, 3);
  //  // cv::Mat sum = sobelX + sobelY;
  //
  //
  //
  m_image = girl::tools::Image(img->width(), img->height());
  //  //
  //  //  ise::conversion::PFConversion *conv = ise::conversion::changeSpace(ISE_RGB_to_Luv);
  //  //  cv::Mat luvImage(img->height(), img->width(), CV_32FC3);
  //  //  float *luvPtr = luvImage.ptr<float>();
  //  //  for (int i = 0 ; i < img->width(); ++i) {
  //  //    for (int j = 0 ; j < img->height() ; ++j) {
  //  //      const girl::tools::PixelFromImage<girl::tools::Image::type> p = img->getPixel(i, j);
  //  //      float l = static_cast<float>(conv[0](p[0], p[1], p[2]));
  //  //      float u = static_cast<float>(conv[1](p[0], p[1], p[2]));
  //  //      float v = static_cast<float>(conv[2](p[0], p[1], p[2]));
  //  //      *luvPtr++ = l;
  //  //      *luvPtr++ = u;
  //  //      *luvPtr++ = v;
  //  //    }
  //  //  }
  //  //
  //  //  cv::Mat sobelX, sobelY;
  //  //  cv::Sobel(luvImage, sobelX, CV_32F, 1, 0, 3);
  //  //  cv::Sobel(luvImage, sobelY, CV_32F, 0, 1, 3);
  //  //  cv::Mat module;
  //  //  cv::magnitude(sobelX, sobelY, module);
  //  //
  //  //  cv::imshow("gradiantX", module);


  //cv::Mat *mat = img->cvMat();
#ifdef VISU
  cv::Mat toto = cv::imread("/home/vieux/Pictures/verso_retourned_recaled.jpg");
  cv::Mat gray(toto.size(), CV_8U);
  cv::cvtColor(toto, gray, CV_BGR2GRAY);
  cv::Mat canny;
  cv::Canny(gray, canny, 450, 50, 3, true);
  cv::Mat cannyThresh;
  cv::threshold(canny, cannyThresh, 0, 255, cv::THRESH_BINARY|cv::THRESH_OTSU);
  cv::namedWindow("Canny", 0);
  cv::imshow("Canny", cannyThresh);
  cv::waitKey(0);
#else 
  cv::Mat cannyThresh; //TODO:REMI wtf?  cannyThresh is needed by the rest of the code !?
#endif //VISU

  //const std::string names[5] = { "Box", "Circles", "Ellipse", "RotatedRect", "ConvexHull" };
  std::map<std::string, EnclosingShapeFactory*> factories;//(5, (EnclosingShapeFactory*) NULL);
  EnclosingBoxFactory boxFactory;
  EnclosingCircleFactory circleFactory;
  EnclosingEllipseFactory ellipseFactory;
  EnclosingRotatedRectFactory rotatedRectFactory;
  EnclosingConvexHullFactory convexHullFactory;
  factories.insert(std::pair<std::string, EnclosingShapeFactory*>("Box", &boxFactory));
  factories.insert(std::pair<std::string, EnclosingShapeFactory*>("Circles", &circleFactory));
  factories.insert(std::pair<std::string, EnclosingShapeFactory*>("Ellipse", &ellipseFactory));
  factories.insert(std::pair<std::string, EnclosingShapeFactory*>("RotatedRect", &rotatedRectFactory));
  factories.insert(std::pair<std::string, EnclosingShapeFactory*>("ConvexHull", &convexHullFactory));

  cv::Mat cannyThreshCopy = cannyThresh.clone(); // toThresh.clone(); //
  std::vector<EnclosingShapeFactory::EnclosingShapePtr> shapes;
  aggregateCC(cannyThreshCopy, shapes, *(factories[m_Shapes]), 8);
  cv::Mat boxCCs = cv::Mat::zeros(img->height(), img->width(), CV_8U);
  for (unsigned int j = 0; j < shapes.size(); ++j) {
      shapes[j]->draw(boxCCs, CV_RGB(rand()%254+1, rand()%254+1, rand()%254+1), CV_FILLED);
  }
#ifdef VISU
  cv::namedWindow("Shapes", 0);
  cv::imshow("Shapes", boxCCs);
  cv::waitKey(0);
#endif //VISU

  for (int i = 0 ; i < m_image.width(); ++i) {
      for (int j = 0 ; j < m_image.height() ; ++j) {
          std::vector<girl::tools::Image::type> v(4, static_cast<girl::tools::Image::type>(boxCCs.at<uchar>(j, i)));
          girl::tools::PixelStandAlone<girl::tools::Image::type> pixel(v);
          m_image.setPixel(i, j, pixel);
      }
  }

  //  m_image.save("/home/roigt/Bureau/result.jpg");
}

void 
SegmentationClusteringShapeCC::setParameters(std::string Shapes)
{
  m_Shapes = Shapes;
} 

void 
SegmentationClusteringShapeCC::getParameters(std::string &Shapes) const
{
  Shapes = m_Shapes;
} 

bool
SegmentationClusteringShapeCC::same(const girl::point &p1, girl::direction dir)
{
  girl::point p2 = p1;
  p2.move(dir);

  const girl::tools::PixelFromImage<girl::tools::Image::type> pixel1 = m_image.getPixel(p1.x(), p1.y());
  const girl::tools::PixelFromImage<girl::tools::Image::type> pixel2 = m_image.getPixel(p2.x(), p2.y());

  return pixel1[0] == pixel2[0];
}

std::vector<EnclosingShapeFactory::EnclosingShapePtr>
SegmentationClusteringShapeCC::aggregateCC(cv::Mat &input, std::vector<EnclosingShapeFactory::EnclosingShapePtr> &cc, const EnclosingShapeFactory &factory, int connectivity)
{
#ifdef VISU
  cv::namedWindow("Shapes", 0);
  cv::imshow("Shapes", input);
  cv::waitKey(0);
#endif //VISU

  std::vector<EnclosingShapeFactory::EnclosingShapePtr> shapes;
  extractCC(input, shapes, factory, connectivity);
  std::cout << "Current CC for iteration " << shapes.size() << std::endl;
  if (shapes.size() != cc.size()) {
      g_aggregateCCMat = cv::Mat::zeros(input.rows, input.cols, CV_8U);
      for (unsigned int i = 0; i < shapes.size(); ++i) {
          shapes[i]->draw(g_aggregateCCMat, CV_RGB(255, 255, 255), CV_FILLED);
          //     delete shapes[i];
          //   shapes[i] = NULL;
      }
      cc = aggregateCC(g_aggregateCCMat, shapes, factory, connectivity);
      return cc;
  }
  else {
      return shapes;
  }
}

void
SegmentationClusteringShapeCC::extractCC(cv::Mat &input, std::vector<EnclosingShapeFactory::EnclosingShapePtr> &cc, const EnclosingShapeFactory &factory, int connectivity)
{
  assert(input.type() == CV_8U);

  for (int i = 0; i < input.rows; ++i) {
      for (int j = 0; j < input.cols; ++j) {
          if (input.at<unsigned char>(i, j) == 255) {
              std::vector<cv::Point> points;
              getCCPoints(input, cv::Point(j, i), points, connectivity);
              if (points.size() >=6) {
                  cv::Mat pointsMat(points);
                  pointsMat.reshape(1);
                  EnclosingShapeFactory::EnclosingShapePtr shape = factory.build(pointsMat);
                  cc.push_back(shape);
              }
          }
      }
  }
}

void
SegmentationClusteringShapeCC::getCCPoints(cv::Mat &input,
    const cv::Point &seed,
    std::vector<cv::Point> &points,
    int connectivity)
{

  assert(input.type() == CV_8U);
  assert(input.at<unsigned char>(seed.y, seed.x));
  assert(seed.x < input.cols && seed.y < input.rows && seed.x >= 0 && seed.y >= 0);

  std::deque<cv::Point> pointsDeq;
  std::list<cv::Point> theList;
  theList.push_back(seed);
  int cpt = 0;
  while (!theList.empty()) {
      ++cpt;
      cv::Point current = theList.front();
      pointsDeq.push_back(current);
      input.at<unsigned char>(current.y, current.x) = 0;
      // enque neighboors
      cv::Point e(current.x + 1, current.y);
      cv::Point w(current.x - 1, current.y);
      cv::Point n(current.x, current.y - 1);
      cv::Point s(current.x, current.y + 1);
      if (n.y >=0)
        if (input.at<unsigned char>(n.y, n.x)) {
            theList.push_back(n);
            input.at<unsigned char>(n.y, n.x) = 0;
        }
      if (e.x < input.cols)
        if (input.at<unsigned char>(e.y, e.x)) {
            theList.push_back(e);
            input.at<unsigned char>(e.y, e.x) = 0;
        }
      if (s.y < input.rows)
        if (input.at<unsigned char>(s.y, s.x)) {
            theList.push_back(s);
            input.at<unsigned char>(s.y, s.x) = 0;
        }
      if (w.x >= 0)
        if (input.at<unsigned char>(w.y, w.x)) {
            theList.push_back(w);
            input.at<unsigned char>(w.y, w.x) = 0;
        }
      if (connectivity == 8) {
          cv::Point ne(current.x + 1, current.y - 1);
          cv::Point no(current.x - 1, current.y - 1);
          cv::Point se(current.x + 1, current.y + 1);
          cv::Point so(current.x - 1, current.y + 1);
          if (ne.x < input.cols && ne.y >= 0)
            if (input.at<unsigned char>(ne.y, ne.x)) {
                theList.push_back(ne);
                input.at<unsigned char>(ne.y, ne.x) = 0;
            }
          if (se.x < input.cols && se.y < input.rows)
            if (input.at<unsigned char>(se.y, se.x)) {
                theList.push_back(se);
                input.at<unsigned char>(se.y, se.x) = 0;
            }
          if (so.x >= 0 && so.y < input.rows)
            if (input.at<unsigned char>(so.y, so.x)) {
                theList.push_back(so);
                input.at<unsigned char>(so.y, so.x) = 0;
            }
          if (no. x >= 0 && no.y >= 0)
            if (input.at<unsigned char>(no.y, no.x)) {
                theList.push_back(no);
                input.at<unsigned char>(no.y, no.x) = 0;
            }
      }
      theList.pop_front();
  }
  points.assign(pointsDeq.begin(), pointsDeq.end());
}
