#include "iseSegmentationClusteringSpatialMeanShift.hpp"
//Ajout pour la Def des classes
#include "iseHashHistogram.hpp"
#include "iseHistogram.hpp"
#include "iseSegmentationContext.hpp"

#include <cstdio>
#include <fstream>
#include <cstring>
#include <iostream>
#include <limits>

#include <ise/iseConversion.hpp>
using namespace ise;


SegmentationClusteringSpatialMeanShift::~SegmentationClusteringSpatialMeanShift()
{

}

SegmentationClusteringSpatialMeanShift::SegmentationClusteringSpatialMeanShift(SegmentationContext *segContext) 
  : SegmentationClustering(segContext), 
    m_ms(),
    m_dim(0),
    m_algo()
{

}


bool
SegmentationClusteringSpatialMeanShift::same(const girl::point &p1, girl::direction dir)
{
  girl::point p2 = p1;
  p2.move(dir);

  return m_ms.same(p1, p2);
}

class Adder : public girl::toger::pointIterator
{
public:
  Adder(ise::SegmentationContext &segC, double *pixel, ise::ImageFeatureComputerPtr algo, bool *compute, int dim)
    : m_segC(segC),
      m_pixel(pixel),
      m_algo(algo),
      m_compute(compute),
      m_dim(dim)
  {
  }

  
  void processPoint(const girl::point &pt)
  {
    const girl::tools::PixelFromImage<girl::tools::ImageFeature::type> p = m_algo->getImageFeature()->getPixel(pt.x(), pt.y());

    const int width = m_segC.getImage()->width();
    const int s = pt.y()*width + pt.x();
    const int smdim = s*m_dim;
    for (int d = 0 ; d < m_dim ; ++d) {
      m_pixel[smdim + d] = p[d];
    }
    m_compute[s] = true;
  }

protected:
  ise::SegmentationContext &m_segC;
  double *m_pixel;
  ise::ImageFeatureComputerPtr m_algo;
  bool *m_compute;
  int m_dim;
};

static
bool
areAllRegionsSelected(const girl::toger &tog, const girl::regionDeque &regions)
{
  const size_t numRegions = tog.numRegions(); //O(N) but rather fast
  assert(regions.size() <= numRegions);
  return numRegions == regions.size();
}

static
void
copyWholeImageFeature(const cv::Mat &mat, double *pixel)
{

  assert(mat.isContinuous());
  assert(mat.depth() == CV_64F);

  memcpy(pixel, mat.data, mat.rows*mat.cols*mat.channels()*sizeof(double));

  //B:TODO:OPTIM:? this memcpy would be useless if we are sure that ise::SpatialMeanShift do not modify buffer...
}


void 
SegmentationClusteringSpatialMeanShift::init(const girl::regionDeque &regions, int /*level*/)
{
  assert(m_segContext);
  assert(m_segContext->getToger());
  assert(m_algo.get() != NULL);

  m_dim = m_algo->getImageFeature()->channels();
 
  const int width = m_segContext->getImage()->width();
  const int height = m_segContext->getImage()->height();

  double *pixel = new double[width*height*m_dim];
  bool *compute = NULL;

  const girl::toger &tog = *(m_segContext->getToger());
  if (! areAllRegionsSelected(tog, regions)) {

    compute = new bool[width*height];
    memset(compute, false, width*height*sizeof(bool));
  
    Adder a(*m_segContext, pixel, m_algo, compute, m_dim);

    girl::regionDeque::const_iterator it = regions.begin();
    const girl::regionDeque::const_iterator itEnd = regions.end();
    for ( ; it != itEnd; ++it) 
      tog.iterateOnRegionDomain(*it, a);
  }
  else {
    copyWholeImageFeature(*(m_algo->getImageFeature()->cvMat()), pixel);
  }

  m_ms.loadPoints(width, height, m_dim, pixel, compute);
  
  delete [] pixel;
  delete [] compute;
  
  pixel = NULL;
  compute = NULL;

  m_ms.segment();
}


void SegmentationClusteringSpatialMeanShift::init(girl::region /*r*/, int /*level*/)
{
  
}

void SegmentationClusteringSpatialMeanShift::setParameters(float sigmaR, float sigmaS, int minRegion, bool fuseRegion, ise::ImageFeatureComputerPtr algo) 
{ 
  m_ms.setSigmaR(sigmaR);
  m_ms.setSigmaS(sigmaS);
  m_ms.setMinRegion(minRegion);
  m_ms.setFuseRegion(fuseRegion);
  m_algo = algo;
}

void SegmentationClusteringSpatialMeanShift::getParameters(float &sigmaR, float &sigmaS, int &minRegion, bool &fuseRegion, ise::ImageFeatureComputerPtr algo) const
{ 
  sigmaR = m_ms.sigmaR();
  sigmaS = m_ms.sigmaS();
  minRegion = m_ms.minRegion();
  fuseRegion = m_ms.fuseRegion();
  algo = m_algo;
}

