/*
  Segmentation clustering: Watershed.
  Copyright (C) 2005-2009 Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISESEGMENTATIONCLUSTERINGWATERSHED_HPP
#define ISESEGMENTATIONCLUSTERINGWATERSHED_HPP

#include <girl/girlToger.hpp>
#include "iseSegmentationClustering.hpp"
#include "iseImageFeatureComputer.hpp"

#include "iseVector.hpp"

#include <map>

namespace ise {

  class SegmentationContext;
  
  class  GIRL_API SegmentationClusteringWatershed : public SegmentationClustering
  {
  public :

    explicit SegmentationClusteringWatershed(SegmentationContext *segContext);
    
    virtual ~SegmentationClusteringWatershed() {}

    virtual void setSegContext(ise::SegmentationContext *s);

    virtual void setParameters(int nbGradient, ise::ImageFeatureComputerPtr algo,  const std::vector<double> &weights);

    virtual void getParameters(int &nbGradient, ise::ImageFeatureComputerPtr algo, std::vector<double> &weights) const;
    
    virtual void init(const girl::regionDeque &regions, int level=1);

    virtual void init(girl::region /*r*/, int /*level*/) {} /* VIDE.... peut être à Supprimer */
    
    virtual bool same(const girl::point &p, girl::direction dir);

  protected :

    typedef girlExt::vector<int, 1> Vector1i;
    typedef girlExt::vector<int, 2> Vector2i;
    typedef std::deque<Vector2i > Vector2iDeque;
    typedef std::vector<Vector2iDeque > VectorGrad;

    void fillNeighbours(const Vector2i  &pixel, 
			Vector2iDeque &pixelNeigbhourSet);

    void computeGradient();

    int selectLabel(const Vector2i &pixel,
		    const Vector2iDeque &pixelNeigbhourSet,
		    cv::Mat &gradModuleX,
		    cv::Mat &gradModuleY);

    void fillWatershed(Vector2iDeque &queueWatershed,
		       cv::Mat &gradModuleX,
		       cv::Mat &gradModuleY);

    void watershedAlgorithm(const VectorGrad &orderedMap, 
			    cv::Mat &gradModuleX,
			    cv::Mat &gradModuleY);

  protected:

    int m_nbGradient;
    std::vector<double> m_weights;

    ise::ImageFeatureComputerPtr m_algo;

    cv::Mat m_distance;
    cv::Mat m_segmentedImage;

    float m_max;

    bool m_haveToComputeGradient;
  };

}

#endif /* ! ISESEGMENTATIONCLUSTERINGWATERSHED_HPP */
