/*
  Segmentation clustering: Watershed Inter Pixel.  Copyright (C) 2005-2011
  Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISESEGMENTATIONCLUSTERINGWATERSHEDINTERPIXEL_HPP
#define ISESEGMENTATIONCLUSTERINGWATERSHEDINTERPIXEL_HPP

#include <girl/girlToger.hpp>
#include "iseSegmentationClustering.hpp"

#include "iseVector.hpp"

#include <map>

namespace ise {

  class SegmentationContext;
  
  class  GIRL_API SegmentationClusteringWatershedInterPixel : public SegmentationClustering
  {
  public :

    explicit SegmentationClusteringWatershedInterPixel(ise::SegmentationContext *segContext);
    
    virtual ~SegmentationClusteringWatershedInterPixel() {}
    
    
    virtual void setParameters(int nbGradient, int colorSpaceConversion,  const std::vector<double> &normalizedChannelsCoeffs);
    
    virtual void getParameters(int &nbGradient, int &colorSpaceConversion, std::vector<double> &normalizedChannelsCoeffs) const;

        
    virtual void init(const girl::regionDeque &regions, int level=1);

    virtual void init(girl::region /*r*/, int /*level*/) {} /* VIDE.... peut être à Supprimer */
    
    virtual bool same(const girl::point &p, girl::direction dir);

    virtual void setSegContext(ise::SegmentationContext *s);

  protected :
    typedef girlExt::vector<int, 1> Vector1i;
    typedef girlExt::vector<int, 2> Vector2i;
    typedef girlExt::vector<int, 3> Vector3i;

    typedef std::deque<Vector2i > Vector2iDeque;
    typedef std::deque<Vector3i > Vector3iDeque;
    typedef std::vector<Vector3iDeque > VectorGrad;


    void fillNeighbours(const Vector2i &pixel, 
			int index,
			cv::Mat &gradModuleX,
			cv::Mat &gradModuleY,
			Vector2iDeque &pixelNeigbhourSet);
    
    void computeInterPixelGradient();

    void watershedAlgorithm(const VectorGrad &orderedMap, 
			    cv::Mat &gradModuleX,
			    cv::Mat &gradModuleY);


    void insertGradientPoint(int x, int y, 
			     cv::Mat &gradModuleX,
			     cv::Mat &gradModuleY,
			     VectorGrad &ordoredMap, float step);

    float computeInterPixelGradient(int x, int y,
				    cv::Mat &img,
				    cv::Mat &gradModuleX,
				    cv::Mat &gradModuleY);

    void etiquetagePixel(Vector2i &pixel,
			 cv::Mat &gradModuleX,
			 cv::Mat &gradModuleY,
			 int index,
			 Vector2iDeque &queue);

    void computeInterPixelImageGradient();

    void afficherEtape();

  protected:

    int m_conversion;
    int m_nbGradient;
    std::vector<double> m_weightedChannelsCoeffs; //TODO:OPTIM: use float!
    
    cv::Mat m_distance;
    cv::Mat m_segmentedImage;

    float m_max;

    bool m_haveToComputeGradient;
  };

}

#endif /* ! ISESEGMENTATIONCLUSTERINGWATERSHEDINTERPIXEL_HPP */
