/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "iseSegmentationClusteringWhitespaceSeparator.hpp"

#include <iostream>
#include <queue>

#include <girl/girlToger.hpp>
#include <girl/girlHashFunc.hpp>
#include <girl/girlUnorderedSet.hpp>

#include <girl/girlBresenhamDir.hpp>

#include <girlTools/girlToolsAABBox.hpp>
#include <girlTools/girlToolsRegionInfos.hpp>
#include <girlTools/girlToolsPainter.hpp>

#include <ise/iseDocumentImageUtils.hpp>
#include <ise/iseSegmentationContext.hpp>
#include <ise/iseRegionSetInterfaceGeometric.hpp>

#include <opencv2/imgproc/imgproc.hpp>
//#include <opencv2/highgui/highgui.hpp>

#ifndef M_PI
//Visual C++ does not define these constants in <cmath>
#define M_PI 3.14159265358979323846
#endif 


namespace ise
{

SegmentationClusteringWhitespaceSeparator::SegmentationClusteringWhitespaceSeparator(ise::SegmentationContext *segContext)
    : SegmentationClustering(segContext),
      m_mat()
{

}

SegmentationClusteringWhitespaceSeparator::~SegmentationClusteringWhitespaceSeparator()
{
}





void
SegmentationClusteringWhitespaceSeparator::init(const girl::regionDeque &regions, int /*level*/)
{
    girl::toger *tog = m_segContext->getToger();
    assert(tog);

    m_mat = cv::Mat::zeros(tog->height(), tog->width(), CV_8U);

    girl::regionDeque::const_iterator it = regions.begin();
    const girl::regionDeque::const_iterator itEnd = regions.end();
    for (; it != itEnd; ++it)
        init(*it, 0);


//    girl::unorderedSet<girl::region> regionsUset;
//    girl::unorderedSet<girl::region> obstacles;

//    girl::regionDeque::const_iterator it = regions.begin();
//    girl::regionDeque::const_iterator itEnd = regions.end();
//    for ( ; it != itEnd; ++it)
//        regionsUset.insert(*it);

//    for (it = regions.begin() ; it != itEnd; ++it) {

//        girl::regionDeque mothers;
//        tog->regionNeighbourhood(*it, girl::toger::NEIGHBOUR_OUTER, girl::toger::MARK_NONE, mothers);
//        // assert(mothers.size() == 1);
//        if (mothers.empty())
//            continue;
//        obstacles.insert(mothers[0]);

//        girl::regionDeque children;
//        tog->regionNeighbourhood(mothers[0], girl::toger::NEIGHBOUR_INNER, girl::toger::MARK_NONE, children);
//        for (girl::regionDeque::const_iterator rIt = children.begin();
//             rIt != children.end(); ++rIt) {
//            if (regionsUset.find(*rIt) == regionsUset.end())
//                obstacles.insert(*rIt);
//        }
//    }
    
//    cv::Mat labels = cv::Mat::zeros(tog->height(), tog->width(), CV_8U);
//    m_mat = cv::Mat::zeros(tog->height(), tog->width(), CV_8U);

//    girl::unorderedSet<girl::region>::const_iterator oIt = obstacles.begin();
//    const girl::unorderedSet<girl::region>::const_iterator oItEnd = obstacles.end();
//    for ( ; oIt != oItEnd; ++oIt) {
//        girl::contour c;
//        tog->regionOuterBoundary(*oIt, c);

//        girl::point startPoint = c.startPt();
//        girl::point pt = startPoint;
//        cv::Point toDraw;
//        girl::contour::const_dir_iterator cIt = c.begin();
//        const girl::contour::const_dir_iterator cItEnd = c.end();
//        for ( ; cIt != cItEnd; ++cIt) {
//            girl::direction dir = *cIt;
//            switch(dir)
//            {
//            case girl::RIGHT :
//                toDraw.x = pt.x() + 1;
//                toDraw.y = pt.y() + 1;
//                break;
//            case girl::UP :
//                toDraw.x = pt.x() + 1;
//                toDraw.y = pt.y();
//                break;
//            case girl::LEFT :
//                toDraw.x = pt.x();
//                toDraw.y = pt.y();
//                break;
//            case girl::DOWN :
//                toDraw.x = pt.x();
//                toDraw.y = pt.y() + 1;
//                break;
//            default:
//                assert(false);
//            }
//            labels.at<unsigned char>(toDraw.y, toDraw.x) = 255;
//            //cv::circle(labels, toDraw, 0, CV_RGB(255,255,255), CV_FILLED);
//            pt.move(dir);
//        }
//    }

//    // cv::namedWindow("obstacles", 0);
//    // cv::imshow("obstacles", labels);
//    // //   cv::imwrite("obstacles.png", labels);
//    // cv::waitKey(0);

//    for (it = regions.begin() ; it != itEnd; ++it) {
//        dilateSeparator(*it, labels);
//    }
    
}


/*
 Paint pixels of border inside inter-pixel border.
 Only pixles of border are drawn, inside is not filled.
*/ 
static
void
paintBorderInside(girl::toger *tog, girl::region r, cv::Mat &labels, unsigned char value)
{
  girl::contour c;
  tog->regionOuterBoundary(r, c);
  girl::point startPoint = c.startPt();
  girl::point pt = startPoint;
  cv::Point toDraw;
  girl::contour::const_dir_iterator cIt = c.begin();
  const girl::contour::const_dir_iterator cItEnd = c.end();
  for ( ; cIt != cItEnd; ++cIt) {
    girl::direction dir = *cIt;
    switch(dir)
      {
      case girl::RIGHT :
	toDraw.x = pt.x() + 1;
	toDraw.y = pt.y() + 1;
	break;
      case girl::UP :
	toDraw.x = pt.x() + 1;
	toDraw.y = pt.y();
	break;
      case girl::LEFT :
	toDraw.x = pt.x();
	toDraw.y = pt.y();
	break;
      case girl::DOWN :
	toDraw.x = pt.x();
	toDraw.y = pt.y() + 1;
	break;
      default:
	assert(false);
      }
    assert(toDraw.y < labels.rows && toDraw.x < labels.cols);
    labels.at<unsigned char>(toDraw.y, toDraw.x) = value;
    pt.move(dir);
  } 
}


// struct Painter : public girl::toger::pointIterator
// {
//   Painter(cv::Mat &img, unsigned char val)
//     : m_img(img), m_value(val)
//   {}

//   virtual void processPoint(const girl::point &pt)
//   {
//     assert(pt.y() < m_img.rows && pt.x() < m_img.cols);
//     m_img.at<unsigned char>(pt.y(), pt.x()) = m_value;
//   }

//   cv::Mat &m_img;
//   const unsigned char m_value;
// };


/**
 * Paint region @a r of toger @a tog in @a labels with value @a value.
 * All pixels of region @a r are drawn.
 */
//static
//void
//paintDomain(girl::toger *tog, girl::region r, cv::Mat &labels, unsigned char value)
//{
//  Painter p(labels, CV_RGB(value,value,value));
//  tog->iterateOnRegionDomain(r, p);
//}



void
SegmentationClusteringWhitespaceSeparator::init(girl::region r, int)
{
    girl::toger *tog = m_segContext->getToger();
    assert(tog);
    int width = tog->width();
    int height = tog->height();
    cv::Mat labels = cv::Mat::zeros(height, width, CV_8U);
    girl::unorderedSet<girl::region> obstacles;

    girl::regionDeque mothers;
    tog->regionNeighbourhood(r, girl::toger::NEIGHBOUR_OUTER, girl::toger::MARK_NONE, mothers);
    if ( !mothers.empty()) {
        girl::region mother = mothers[0];
        // reverse paint mother contour so that pixels are seen outside the mother region domain
        girl::contour c;
        tog->regionOuterBoundary(mother, c);
        girl::point startPoint = c.startPt();
        girl::point pt = startPoint;
        cv::Point toDraw;
        girl::contour::const_reverse_dir_iterator cIt = c.rbegin();
        const girl::contour::const_reverse_dir_iterator cItEnd = c.rend();
        for ( ; cIt != cItEnd; ++cIt) {
            girl::direction dir = *cIt;
            dir = dir.opposite();
            switch(dir)
            {
            case girl::RIGHT :
                toDraw.x = pt.x() + 1;
                toDraw.y = pt.y() + 1;
                break;
            case girl::UP :
                toDraw.x = pt.x() + 1;
                toDraw.y = pt.y();
                break;
            case girl::LEFT :
                toDraw.x = pt.x();
                toDraw.y = pt.y();
                break;
            case girl::DOWN :
                toDraw.x = pt.x();
                toDraw.y = pt.y() + 1;
                break;
            default:
                assert(false);
            }
	    if (toDraw.x >= 0 && toDraw.x < width &&toDraw.y >= 0 && toDraw.y < height)
	      labels.at<unsigned char>(toDraw.y, toDraw.x) = 255;
            pt.move(dir);
        }
        girl::regionDeque children;
        tog->regionNeighbourhood(mother, girl::toger::NEIGHBOUR_INNER, girl::toger::MARK_NONE, children);
        for (girl::regionDeque::const_iterator rIt = children.begin();
             rIt != children.end(); ++rIt) {
            if (*rIt != r) {
                // paint children obstacles

	      paintBorderInside(tog, *rIt, labels, 255);

            }
        }

    } else 
      return;

    //cv::namedWindow("obstacles", 0);
    //cv::imshow("obstacles", labels);
    // //   cv::imwrite("obstacles.png", labels);
    //cv::waitKey(0);

    dilateSeparator(r, labels);
}


void
SegmentationClusteringWhitespaceSeparator::extendBBox(const cv::Mat &labels, girl::point &tl, girl::point &br, girl::direction d, int width, int height) {


    bool crossObstacles = false;
    while (!crossObstacles) {
	switch(d) {
	case girl::UP: {
            cv::Range rowRange(std::max(0, tl.y() - 1), tl.y());
            cv::Range colRange(tl.x(), br.x());
	    assert(0 <= rowRange.start && rowRange.start <= rowRange.end && rowRange.end <= height);
            assert(0 <= colRange.start && colRange.start <= colRange.end && colRange.end <= width);
            crossObstacles = cv::countNonZero(cv::Mat(labels, rowRange, colRange) > 0) || rowRange.end == 0;
            if (!crossObstacles)
                tl.move(d);
            break;
	}
	case girl::DOWN: {
            cv::Range rowRange(br.y(), std::min(br.y() + 1, height));
            cv::Range colRange(tl.x(), br.x());
	    assert(0 <= rowRange.start && rowRange.start <= rowRange.end && rowRange.end <= height);
            assert(0 <= colRange.start && colRange.start <= colRange.end && colRange.end <= width);
            crossObstacles = cv::countNonZero(cv::Mat(labels, rowRange, colRange) > 0) || rowRange.start == height;
            if (!crossObstacles)
                br.move(d);
            break;
	}
	case girl::LEFT: {
            cv::Range rowRange(tl.y(), br.y());
            cv::Range colRange(std::max(0, tl.x() - 1), tl.x());
	    assert(0 <= rowRange.start && rowRange.start <= rowRange.end && rowRange.end <= height);
            assert(0 <= colRange.start && colRange.start <= colRange.end && colRange.end <= width);
            crossObstacles = cv::countNonZero(cv::Mat(labels, rowRange, colRange) > 0) || colRange.end == 0;
            if (!crossObstacles)
                tl.move(d);
            break;
	}
	case girl::RIGHT: {
            cv::Range rowRange(tl.y(), br.y());
            cv::Range colRange(br.x(), std::min(br.x() + 1, width));
	    assert(0 <= rowRange.start && rowRange.start <= rowRange.end && rowRange.end <= height);
            assert(0 <= colRange.start && colRange.start <= colRange.end && colRange.end <= width);
            crossObstacles = cv::countNonZero(cv::Mat(labels, rowRange, colRange) > 0) || colRange.start == width;
            if (!crossObstacles)
                br.move(d);
            break;
	}
	default:
            assert(false);
	}
	//	std::cout << tl.x() << " " << tl.y() << " " << br.x() << " " << br.y() << " >> " << d << std::endl;
    }
    
}
  
  // @a extremity updated to contact point
  // @a itC : const_iterator on @a directions, on the direction after the contact point
  void
  SegmentationClusteringWhitespaceSeparator::extendLine(girl::point &extremity,
							const std::deque<girl::direction> &directions,
							const cv::Mat &obstacles, 
							std::deque<girl::direction>::const_iterator &itC)
    
  {
    typedef std::deque<girl::direction> DirDq;
    itC = directions.end();
    for (DirDq::const_iterator it = directions.begin();
	 it != directions.end(); ++it) {
      if (obstacles.at<unsigned char>(extremity.y(), extremity.x()) != 0) {
	itC = it;
	break;
      }
      if (extremity.x()>=0 && extremity.x()<m_mat.cols && extremity.y()>=0 && extremity.y()<m_mat.rows) {
	assert(extremity.y() < m_mat.rows && extremity.x() < m_mat.cols);
	m_mat.at<unsigned char>(extremity.y(), extremity.x()) = 255;
      }
      extremity.move(*it);
    }
  }


  bool
  SegmentationClusteringWhitespaceSeparator::extendLineTotally(girl::point p0c,
							       const std::deque<girl::direction> &directions,
							       const cv::Mat &obstacles)
  {

    //p0c is a contact point

    const girl::toger *tog = m_segContext->getToger();
    const int w = tog->width();
    const int h = tog->height();

    bool extensionPossible = true;
    girl::point pt = p0c;
    typedef std::deque<girl::direction> DirDq;
    DirDq::const_iterator it = directions.begin();

    bool passInside = false;
    for ( ;
	 it != directions.end();
	 ++it) {
      if (pt.x()>=0 && pt.x()<w && pt.y()>=0 && pt.y()<h) {
	passInside = true;
	if (obstacles.at<unsigned char>(pt.y(), pt.x()) != 0) {
	  std::cerr<<"contact with obstacle at ("<<pt.x()<<"; "<<pt.y()<<")"<<std::endl;
	  extensionPossible = false;
	  break;
	}
      }
      pt.move(*it);
    }
    if (extensionPossible) {
      assert(it == directions.end());
      if (pt.x()>=0 && pt.x()<w && pt.y()>=0 && pt.y()<h) {
	passInside = true;
	if (obstacles.at<unsigned char>(pt.y(), pt.x()) != 0) {
	  extensionPossible = false;
	}
      }
    }


    if (extensionPossible & passInside) {
      girl::point pt = p0c;
      for (DirDq::const_iterator it2 = directions.begin();
	   it2 != it;
	   ++it2) {      
	if (pt.x()>=0 && pt.x()<w && pt.y()>=0 && pt.y()<h) {
	  assert(pt.y() < m_mat.rows && pt.x() < m_mat.cols);
	  m_mat.at<unsigned char>(pt.y(), pt.x()) = 255;
	}
	assert(it2 != directions.end());
	pt.move(*it2);
      }
      if (pt.x()>=0 && pt.x()<w && pt.y()>=0 && pt.y()<h) {
	assert(pt.y() < m_mat.rows && pt.x() < m_mat.cols);
	m_mat.at<unsigned char>(pt.y(), pt.x()) = 255;
      }

    }
    return extensionPossible & passInside;
  }


struct MaxAbscisseFinder : public girl::toger::directionIterator
{
  //angle in radians
  explicit MaxAbscisseFinder(double angle)
    : m_cosA(cos(angle)),
      m_sinA(sin(angle)),
      m_minTX(std::numeric_limits<double>::max()),
      m_maxTX(-std::numeric_limits<double>::max())
  {

  }

  virtual void processFirstPoint(const girl::point &pt)
  {
    m_currPt = pt;
    m_minPt = pt;
    m_maxPt = pt;
    m_minTX = m_maxTX = convertX(pt);
  }

  virtual void processDirection(girl::direction dir)
  {
    m_currPt.move(dir);
    processPoint(m_currPt);
  }

  // virtual void processLastPoint(const girl::point &pt) 
  // {
    
  // }

  void processPoint(const girl::point &pt)
  {
    const double tx = convertX(pt);
    if (tx < m_minTX) {
      m_minTX = tx;
      m_minPt = pt;
    }
    else if (tx > m_maxTX) {
      m_maxTX = tx;
      m_maxPt = pt;
    }
  }


  double convertX(const girl::point &pt) const
  {
    return pt.x()*m_cosA - pt.y()*m_sinA;
  }

  const double m_cosA, m_sinA;
  girl::point m_currPt;
  girl::point m_minPt; //pt in original coord system with m_minTX as abscisse in rotated coord system
  girl::point m_maxPt; //pt with m_maxTX as abscisse in rotated coord system
  double m_minTX;
  double m_maxTX;
};


  void
  SegmentationClusteringWhitespaceSeparator::dilateSeparator(const girl::region reg, cv::Mat &labels)
  {
    /*
      Algorithm:
      - we first choose two points p0 & p1 as extremities of separator [1]
      - we then find the intersections of the line (p0; p1) with external border of image : pup and pdwon.
         we discritize the line (pup; pdown) and get directions 
      - we then extend the segment [p0; p1] in direction [p0; pup] and in direction [p1; pdown] till we touch an obstacle or the external border. Contact points are then p0c & p1c (they may be outside the image). We correct these points to be the last point before contact.
      - 
      - we have to remove this separator as obstacle
      - paint part of line inside separator
      - grow region on the whole length
      - re-add this separator as obstacle
      - add separator to m_mat
      
     */


    girl::toger *tog = m_segContext->getToger();
    assert(tog);

    ise::RegionSetInterfaceGeometric rsig(m_segContext);
    const size_t rfDim = rsig.getFeatureDim();
    std::vector<double> rfData(rfDim); 
    ise::RegionFeature rf(&rfData[0]);
    rsig.compute(reg, rf);
    const double angleMajorAxisAbscisse = rf[7];
    std::cerr<<"####SegmentationClusteringWhitespaceSeparator: angle major axis / abscisse = "<<rf[7]<<std::endl;
    

    int x0=-1, y0=-1, x1=-1, y1=-1;

#define TRY_TO_AXIS_ALIGN 1

#ifdef TRY_TO_AXIS_ALIGN
    
    //B
    /*
      As we work with binarized/discretized regions, it arrives that horizontal (resp. vertical) separators have an angle different than 0 (resp. 90) degrees. It is not a problem for SegmentationClusteringWhitespaceSeparator itself : they are extended with this angle. 
      But it is a problem when Maximal White Space is used afterwards. Because between these almost horizontal (or almost vertical) separators and maximal white space, we will have very thin regions that remain.
      
      Thus we try to detect almost horizontal and vertical separators and then consider them as horizontal or vertical.
     */

    static const double angle_epsilon = 1.0; //TODO: this angle could be dependant of major axis length: less strict for short lengths, more strict for large lengths

    const bool almostHorizontal = (fabs(angleMajorAxisAbscisse-0.0) < angle_epsilon);
    const bool almostVertical = (fabs(angleMajorAxisAbscisse-90.0) < angle_epsilon);
    
    bool initPtsDone = false;
    if (almostHorizontal || almostVertical) {
      //almost horizontal or vertical

      //std::cerr<<"#####>> angleMajorAxisAbscisse="<<angleMajorAxisAbscisse<<" almostHorizontal="<<almostHorizontal<<" almostVertical="<<almostVertical<<std::endl;

      const girl::tools::AABBox aabb = girl::tools::getAABBox(*tog, reg);
      girl::regionDeque regions;
      girl::tools::getEnclosedRegions(*tog, aabb, regions);
      assert(! regions.empty()); //at least one region: reg.

      //std::cerr<<"#####>> "<<regions.size()<<" regions in AABB"<<std::endl;

      if (regions.size() <= 2) { //region reg and background
	assert(regions.front() == reg || regions.back() == reg);
	//AABB does not intersect any other region than region reg and background
	
	if (almostHorizontal) {
	  x0 = aabb.xMin();
	  x1 = aabb.xMax();
	  y0 = y1 = (aabb.yMin() + aabb.yMax())/2;
	}
	else {
	  assert(almostVertical);
	  x0 = x1 = (aabb.xMin() + aabb.xMax())/2;
	  y0 = aabb.yMin();
	  y1 = aabb.yMax();
	}
	initPtsDone = true;
      }
    }
    if (! initPtsDone) {
#endif //TRY_TO_AXIS_ALIGN

#if 0
    std::vector<girl::point> scan;
    tog->regionDomain(reg, scan);

    const size_t s = scan.size();
    assert(s >= 2);
    if (s == 2) {
      x0 = scan[0].x();
      y0 = scan[0].y();
      x1 = scan[1].x();
      y1 = scan[1].y();
    }
    else {
      const bool horizontal = (fabs(angleMajorAxisAbscisse) <= 0.02);
      if (horizontal) {
	const size_t i = s/2;
	x0 = scan[i].x();
	y0 = scan[i].y();
	x1 = scan[i+1].x();
	y1 = scan[i+1].y();
      }
      else {
	x0 = (scan[0].x()+scan[1].x())/2;
	y0 = scan[0].y();
	x1 = (scan[s-2].x()+scan[s-1].x())/2;
	y1 = scan[s-1].y();
      }
    }
#else

    //we put points of outer region contour in coordinate system rotated of angleMajorAxisAbscisse
    // and we keep coordinates of points with maximum abscisses in this rotated CS.

    const double angle = angleMajorAxisAbscisse * M_PI / 180.0; //in radians
    MaxAbscisseFinder maf(angle);
    tog->iterateOnFaceBoundary(reg, maf);
    
    if (maf.m_minPt.y() <= maf.m_maxPt.y()) {
      x0 = maf.m_minPt.x();
      y0 = maf.m_minPt.y();
      x1 = maf.m_maxPt.x();
      y1 = maf.m_maxPt.y();
    }
    else {
      x1 = maf.m_minPt.x();
      y1 = maf.m_minPt.y();
      x0 = maf.m_maxPt.x();
      y0 = maf.m_maxPt.y();
    }
#endif


#ifdef TRY_TO_AXIS_ALIGN
    }
#endif //TRY_TO_AXIS_ALIGN


    int xup = -2000, yup = -2000; //dummy initialization values on purpose
    int xdown=0, ydown=0;

    if (x0 == x1) {
      xup = xdown = x0;
      yup = -1;
      ydown = tog->height();
    } 
    else if (y0 == y1) {
      xup = -1;
      xdown = tog->width();
      yup = ydown = y0;
    }
    else {
      assert(x1 != x0);

      //const double a = ((tog->height()-y1)-(tog->height()-y0))/(double)(x1-x0);
      //const double b = (tog->height()-y1) - a*x1;
      const double a = (y1-y0)/(double)(x1-x0);
      const double b = y1 - a*x1;

      std::cerr<<"a="<<a<<" b="<<b<<std::endl;

      if (a < 0) {
	
	//pup: on up border or right border
	//pdown: on left border or down border

	int y = -1;
	int x = (y-b)/a;
	if (x >=-1 && x<=tog->width()) {
	  //pup on up border
	  xup = x;
	  yup = y;

	  {

	    int x = -1;
	    int y = a*x + b;
	    if (y >=-1 && y<=tog->height()) {
	      //pdown on left border
	      xdown = x;
	      ydown = y;
	    }
	    else {
	      //pdown on down border
	      ydown = tog->height();
	      xdown = (ydown-b)/a;
	    }
	  }

	}
	else {
	  //pup on right border

	  int x = tog->width();
	  int y = a * x + b;
	  if (y >=-1 && y<=tog->height()) {
	    xup = x;
	    yup = y;
	  }
	
	  {  
	    int x = -1;
	    int y = a*x + b;
	    if (y >=-1 && y<=tog->height()) {
	      //pdown on left border
	      xdown = x;
	      ydown = y;
	    }
	    else {
	      //pdown on down border
	      ydown = tog->height();
	      xdown = (ydown-b)/a;
	    }
	  }
	} 

      }
      else {
	//a > 0
	
	//pup: on left border or up border
	//pdown: on right border or down border


	int y = tog->height(); 
	int x = (y-b)/a;
	if (x >=-1 && x<=tog->width()) {
	  //pdown on down border
	  xdown = x;
	  ydown = y;
	  
	  {
	    int x = -1;
	    int y = a*x + b;
	    if (y >=-1 && y<=tog->height()) {
	      //pup on left border
	      xup = x;
	      yup = y;
	    }
	    else {
	      //pup on up border
	      yup = -1;
	      xup = (yup-b)/a;
	    }
	  }

	}
	else {

	  int x = tog->width();
	  int y =  a*x + b;
	  if (y >=-1 && y<=tog->height()) {
	    //pdown on right border
	    xdown = x;
	    ydown = y;
	  }
	  else {
	    assert(false);
	  }

	  {
	    int x = -1;
	    int y = a*x + b;
	    if (y >=-1 && y<=tog->height()) {
	      //pup on left border
	      xup = x;
	      yup = y;
	    }
	    else {
	      //pup on up border
	      yup = -1;
	      xup = (yup-b)/a;
	    }
	  }
	}


      }

    }
    
    std::cerr<<"x0="<<x0<<" y0="<<y0<<"  x1="<<x1<<" y1="<<y1<<std::endl;
    std::cerr<<"xup="<<xup<<" yup="<<yup<<" xdown="<<xdown<<" ydown="<<ydown<<std::endl;

    assert(xdown>=-1 && xdown<=tog->width());
    assert(xup>=-1 && xup<=tog->width());
    assert(ydown>=-1 && ydown<=tog->height());
    assert(yup>=-1 && yup<=tog->height());

    
    const girl::point p0(x0, y0);
    const girl::point p1(x1, y1);
    typedef std::deque<girl::direction> DirDq;
    DirDq dirsUp;
    girl::BresenhamDirs(p0, girl::point(xup, yup), dirsUp); 
    DirDq::const_iterator itCUp;
    girl::point p0c = p0;
    extendLine(p0c, dirsUp, labels, itCUp);


    DirDq dirsDown;
    girl::BresenhamDirs(p1, girl::point(xdown, ydown), dirsDown); 
    DirDq::const_iterator itCDown;
    girl::point p1c = p1;
    extendLine(p1c, dirsDown, labels, itCDown);

    std::cerr<<"p0c x="<<p0c.x()<<" y="<<p0c.y()<<"    p1c x="<<p1c.x()<<" y="<<p1c.y()<<std::endl;


    //- we have to remove this separator as obstacle
    paintBorderInside(tog, reg, labels, 0);

    //- paint part of line inside separator
    DirDq dirs;
    girl::BresenhamDirs(p0, p1, dirs); 
    girl::point pt = p0;
    for (DirDq::const_iterator it = dirs.begin(); it != dirs.end(); ++it) {
      //labels.at<unsigned char>(pt.y(), pt.x()) = 255;
      assert(pt.y() < m_mat.rows && pt.x() < m_mat.cols);
      m_mat.at<unsigned char>(pt.y(), pt.x()) = 255;
      pt.move(*it);
    }

//    cv::namedWindow("rectangle", 0);
//    cv::imshow("rectangle", m_mat);
//    cv::waitKey(0);

    //- grow region on the whole length

    DirDq dirsC; //all directions from p0c to p1c 
    for (DirDq::const_iterator it = dirsUp.begin(); it != itCUp; ++it) {
      assert(it != dirsUp.end());
      dirsC.push_front(it->opposite());
    }

    /*
    {//DEBUG
      pt = p0c;
      for (DirDq::const_iterator it = dirsC.begin(); it != dirsC.end(); ++it) {
	pt.move(*it);
      }
      std::cerr<<"dirsC  pt=("<<pt.x()<<"; "<<pt.y()<<")   ==?== p0=("<<p0.x()<<"; "<<p0. y()<<")"<<std::endl;
    }
    */

    for (DirDq::const_iterator it = dirs.begin(); it != dirs.end(); ++it) {
      dirsC.push_back(*it);
    }

    /*
    {//DEBUG
      pt = p0c;
      for (DirDq::const_iterator it = dirsC.begin(); it != dirsC.end(); ++it) {
	pt.move(*it);
      }
      std::cerr<<"dirsC  pt=("<<pt.x()<<"; "<<pt.y()<<")   ==?== p1=("<<p1.x()<<"; "<<p1. y()<<")"<<std::endl;
    }
    */

    for (DirDq::const_iterator it = dirsDown.begin(); it != itCDown; ++it) {
      dirsC.push_back(*it);
    }

    /*
    {//DEBUG
      pt = p0c;
      for (DirDq::const_iterator it = dirsC.begin(); it != dirsC.end(); ++it) {
	pt.move(*it);
      }
      std::cerr<<"dirsC  pt=("<<pt.x()<<"; "<<pt.y()<<")   ==?== p1c=("<<p1c.x()<<"; "<<p1c. y()<<")"<<std::endl;
    }
    */

    //as p0c & p1c are contact point, remove first & last direction & move p0c & p1c
    assert(dirsC.size() >= 2);
    girl::direction dirCUp = dirsC.front(); //direction of contact
    p0c.move(dirsC.front());
    dirsC.pop_front();
    p1c.move(dirsC.back().opposite());
    dirsC.pop_back();

    /*
    {//DEBUG
      pt = p0c;
      for (DirDq::const_iterator it = dirsC.begin(); it != dirsC.end(); ++it) {
	pt.move(*it);
      }
      std::cerr<<"dirsC  pt=("<<pt.x()<<"; "<<pt.y()<<")   ==?== p1c=("<<p1c.x()<<"; "<<p1c. y()<<")"<<std::endl;
    }
    */

    std::cerr<<"corrected: p0c x="<<p0c.x()<<" y="<<p0c.y()<<"    p1c x="<<p1c.x()<<" y="<<p1c.y()<<std::endl;


    girl::direction dirPlus = dirCUp.opposite().next();
    girl::direction dirMinus = dirCUp.opposite().previous();

    std::cerr<<"dirCUp="<<dirCUp<<"  dirPlus="<<dirPlus<<"   dirMinus="<<dirMinus<<std::endl;

    bool extendPlus = true;    
    pt = p0c;
    while (extendPlus) {
      pt.move(dirPlus);

      /*
      std::cerr<<"grow plus: p0c=("<<p0c.x()<<"; "<<p0c.y()<<") pt=("<<pt.x()<<"; "<<pt.y()<<")"<<std::endl;

      {//DEBUG
	girl::point p1= p0c, p2=pt;
	for (DirDq::const_iterator it = dirsC.begin(); it != dirsC.end(); ++it) {
	  p1.move(*it);
	  p2.move(*it);
	}
	std::cerr<<"  would end at p0c_end=("<<p1.x()<<"; "<<p1.y()<<")   pt_end=("<<p2.x()<<"; "<<p2. y()<<")"<<std::endl;
      }
      */

      extendPlus = extendLineTotally(pt, dirsC, labels);
    }

    bool extendMinus = true;
    pt = p0c;
    while (extendMinus) {
      pt.move(dirMinus);

      /*
      std::cerr<<"grow minus: p0c=("<<p0c.x()<<"; "<<p0c.y()<<") pt=("<<pt.x()<<"; "<<pt.y()<<")"<<std::endl;

      {//DEBUG
	girl::point p1= p0c, p2=pt;
	for (DirDq::const_iterator it = dirsC.begin(); it != dirsC.end(); ++it) {
	  p1.move(*it);
	  p2.move(*it);
	}
	std::cerr<<"  would end at p0c_end=("<<p1.x()<<"; "<<p1.y()<<")   pt_end=("<<p2.x()<<"; "<<p2. y()<<")"<<std::endl;
      }
      */

      extendMinus = extendLineTotally(pt, dirsC, labels);
    }


    // re-add this separator as obstacle
    paintBorderInside(tog, reg, labels, 255);
    
    //add separator to m_mat
    //paintDomain(tog, reg, m_mat, 255);
    girl::tools::Painter<unsigned char> p(m_mat, CV_RGB(255,255,255));
    tog->iterateOnRegionDomain(reg, p);



   
    //  cv::rectangle(labels, cv::Point(tl.x(), tl.y()), cv::Point(br.x()-1, br.y()-1), CV_RGB(132,132,132), CV_FILLED);
//    cv::namedWindow("rectangle", 0);
//    cv::imshow("rectangle", m_mat);
//    cv::waitKey(0);

    
}



bool
SegmentationClusteringWhitespaceSeparator::same(const girl::point &p, girl::direction dir)
{
    assert(m_mat.type() == CV_8U);
    assert(m_mat.rows && m_mat.cols);
    girl::point ng = p;
    ng.move(dir);
    return m_mat.at<unsigned char>(p.y(), p.x()) == m_mat.at<unsigned char>(ng.y(), ng.x());
}


}
