/*
  Segmentation clustering: Word segmentation
  Copyright (C) 2005-2011 Roigt Julian, Vieux Rémi, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISESEGMENTATIONCLUSTERINGWORD_HPP
#define ISESEGMENTATIONCLUSTERINGWORD_HPP

#include <girl/girlCommon.hpp>
#include <girl/girlToger.hpp>
#include <girl/girlUnorderedSet.hpp>

#include <opencv2/core/core.hpp>

#include "iseForegroundBackgroundClassifier.hpp"
#include "iseNeighbouringRegions.hpp"
#include "iseRunLengthExtractor.hpp"
#include "iseSegmentationClustering.hpp"

namespace ise {

  class SegmentationContext;

  class  GIRL_API SegmentationClusteringWord : public SegmentationClustering
  {
  public :

    static const int WHITE;
    static const int BLACK;
    static const int MIDDLE;

  public:

    explicit SegmentationClusteringWord(SegmentationContext *segContext);
    
    virtual ~SegmentationClusteringWord() {}

    virtual void setSegContext(ise::SegmentationContext *s);

    virtual void setParameters(const int threshold,
        const cv::Mat &mat,
        const std::vector<ise::NeighbouringRegions> &spacings)
    {
      m_threshold = threshold;
      m_ccLabels = mat;
      m_spacings = spacings;
    }

    virtual void getParameters(int &threshold,
        cv::Mat &mat,
        std::vector<ise::NeighbouringRegions> &spacings) const
    {
      threshold = m_threshold;
      mat = m_ccLabels;
      spacings = m_spacings;
    }

    virtual void init(const girl::regionDeque &regions, int level=1);

    virtual void init(girl::region , int) {} 
    
    virtual bool same(const girl::point &p, girl::direction dir);

  private:

    void fillWordOnImage(cv::Mat &image, const girl::unorderedSet<girl::region> &regions);

  private:
    
    cv::Mat m_img;

  private:

    int m_threshold;

    cv::Mat m_ccLabels;

    std::vector<ise::NeighbouringRegions> m_spacings;

  };

}

#endif /* ! ISESEGMENTATIONCLUSTERINGWORD_HPP */
