/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef ISESIGNALPROCESSINGUTILS_HPP_
#define ISESIGNALPROCESSINGUTILS_HPP_

#include <opencv2/core/core.hpp>
#include <girl/girlCommon.hpp>

namespace ise {
/**
 * Define some useful functions for signal processing
 */

/**
 * Compute the autocorrelation of signal @param input and return it
 * in @param output.
 * @param input must be a vector (i.e. input.rows = 1)
 */
  void GIRL_API autocorrelation(cv::Mat &input, cv::Mat &output);

  void GIRL_API autocorrelation(const std::vector<double> &input, std::vector<double> &output);

 // void GIRL_API convolveDFT(const cv::Mat& A, const cv::Mat& B, cv::Mat& C);
  /**
   * Given two points the coordinates of two points a, b in the image plane,
   * return the oriented angle between the horizontal axis and the line going
   * by a, b.
   * @return the angle [0, ..., 359]
   */
  // TODO:IMPROVE Should this function be in this header "Signal PRocessing". Maybe
  // rename the header in iseUtils.hpp ?
  double GIRL_API angle(const double &xa, const double &ya, const double &xb, const double &yb);


  double GIRL_API correlation(const cv::Mat &a, const cv::Mat &b);
}
#endif /* ISESIGNALPROCESSINGUTILS_HPP_ */
