#ifndef MEANSHIFTUTIL_HPP
#define MEANSHIFTUTIL_HPP

#include <cstdio>
#include <cassert>
#include <cstdlib>
#include <iostream>
#include <cstring>

namespace ise {

  class RAList {
  public:
    int label;

    float edgeStrength;
    int edgePixelCount;

    RAList *next;

    RAList():label(-1)
	    , edgeStrength(0)
	    , edgePixelCount(0)
	    , next(NULL)
	    , cur(NULL)
	    , prev(NULL)
	    , exists()
    {
    }

  
    int Insert(RAList *entry) {
      if (!next) {
	next = entry;
	entry->next = NULL;
      
	return 0;
      }
    
      if (next->label > entry->label) {
	entry->next	= next;
	next = entry;
      
	return 0;
      }
    
      exists = 0;
      cur = next;
      while (cur) {
	if (entry->label == cur->label) {
	  //node already exists
	  exists = 1;
	  break;
	}
	else if ((!(cur->next))||(cur->next->label > entry->label)) {
	  //insert entry into the list at this location
	  entry->next = cur->next;
	  cur->next	= entry;
	  break;
	}
      
	//traverse the region adjacency list
	cur = cur->next;
      }
    
      //done. Return exists indicating whether or not a new node was
      //      actually inserted into the region adjacency list.

      return static_cast<int>(exists);
    }

  private:

    RAList *cur, *prev;
    unsigned char exists;

  };

  class Kernel
  {
  public:
    int m_P[2];
    float m_h[2];
    float m_offset[2];
    int m_dim;
    int m_kp;

    explicit Kernel(size_t dim)
      : m_P(), m_h(), m_offset(), m_dim(), m_kp() 
    {
      init(dim);
    };
  
    void init(size_t dim)
    {
      m_kp = 2;  // subspace count
      m_P[0] = 2;
      m_P[1] = static_cast<int>(dim);
      m_dim = 2+static_cast<int>(dim);

      for (int i = 0 ; i < 2 ; ++i) {
	m_h[i] = 1.0;
	m_offset[i] =  1;
      }
    }
  };

  class mode{
  public:
    explicit mode(size_t size)
      : m_data(NULL), m_size(size)
    {
      m_data = new double[size];
      memset(m_data, 0, size*sizeof(double));
    }

    mode(double *data, size_t size)
      : m_data(NULL), m_size(size)
    {
      m_data = new double[size];
      for (size_t i = 0 ; i < size ; ++i)
	m_data[i] = data[i];
    }

    mode(const mode &other)
    {
      m_data = new double[other.m_size];
      m_size = other.m_size;
      for (size_t i = 0 ; i < other.m_size ; ++i)
	m_data[i] = other.m_data[i];
    }

    ~mode()
    { 
      delete[] m_data; 
    }

    mode &operator=(const mode &other) //B
    {
      if (&other != this) {
	m_data = new double[other.m_size];
	m_size = other.m_size;
	for (size_t i = 0 ; i < other.m_size ; ++i)
	  m_data[i] = other.m_data[i];
      }
      return *this;
    }

    size_t size() const { return m_size; }

    const double &operator[](size_t i) const
    {
      assert(i<m_size);
      return m_data[i];
    }

    double &operator[](size_t i)
    {
      assert(i<m_size);
      return m_data[i];
    }

    bool operator==(const mode &other) const
    {
      double el = 0;
      double dist = 0;
      for (size_t i = 0 ; i < m_size ; ++i) {
	el = m_data[i] - other.m_data[i];
	dist += el * el;
      }

      if ( dist < 0.01 )
	return false;

      return true;
    }

    bool operator<(const mode &other) const
    {

      for ( size_t i = 0 ; i < m_size ; ++i) {
	if ( m_data[i] < other.m_data[i] )
	  return true;
	else if ( m_data[i] > other.m_data[i])
	  return false;
      }
      
      return false;
    }
    
  private:
    double *m_data;
    size_t m_size;
  };
  /*
  mode modeCenter(const mode &m1, const mode &m2)
  {
    assert(m1.size() == m2.size());

    mode data(m1.size());

    for (int i = 0 ; i < m1.size() ; ++i) {
      data[i] = ( m1[i] + m2[i] ) / 2.0;
    }
    
    return data;
  }
  */

}

#endif /* MEANSHIFTUTIL_HPP */
