/*
  Breadth First Recursive split algorithm.
  Copyright (C) 2005-2009 Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "iseSplitAlgorithmBFRecursive.hpp"

#include "iseSplitDriverRecursive.hpp"
#include "iseSegmentationClustering.hpp"

using namespace ise;

SplitAlgorithmBFRecursive::SplitAlgorithmBFRecursive(ise::SegmentationContext *segContext)
  :
  SplitAlgorithmDefault(segContext)
{
  
}

void
SplitAlgorithmBFRecursive::apply(const girl::regionDeque &regionsToSplit, 
				 ise::SplitDriver &driver, 
				 ise::SegmentationClustering &clust,
				 int level,
				 girl::regionDeque &createdRegions)
{
  driver.init(regionsToSplit, level);
  clust.init(regionsToSplit, level);
  
  girl::regionDeque l = regionsToSplit; //copy (because splitAux modifies regionsToSplit).
  splitAux(l, driver, clust, level, createdRegions);
}

void
SplitAlgorithmBFRecursive::splitAux(girl::regionDeque &regionsToSplit, 
				    ise::SplitDriver &driver, 
				    ise::SegmentationClustering &clust,
				    int level,
				    girl::regionDeque &createdRegions)
{
  
  girl::regionDeque tmpCreatedRegions;
  while ( ! regionsToSplit.empty() ) {
    girl::region r = regionsToSplit.front();
    regionsToSplit.pop_front();
    if (driver.toSplit(r, level)) {

      girl::regionDeque tmpQueue;
      ise::SplitAlgorithmDefault::splitOneRegion(r, driver, clust, level, tmpQueue);

      if (tmpQueue.size() == 1) {
	assert(tmpQueue.front() == r);
	createdRegions.push_back(r);
      }
      else { 
	girl::regionDeque::const_iterator it = tmpQueue.begin();
	const girl::regionDeque::const_iterator itEnd =  tmpQueue.end();
	for ( ; it!= itEnd; ++it) 
	  tmpCreatedRegions.push_back(*it);
      }
    } 
    else 
      createdRegions.push_back(r);
  }

  if (! tmpCreatedRegions.empty()) {
    initHandleSplitOneRegion(driver, clust, tmpCreatedRegions, level+1);
    splitAux(tmpCreatedRegions, driver, clust, level+1, createdRegions);
  }  
} 


void
SplitAlgorithmBFRecursive::initHandleSplitOneRegion(ise::SplitDriver &driver,
						    ise::SegmentationClustering &clust,
						    girl::regionDeque &regions,
						    int level)
{
  driver.init(regions, level);
  clust.init(regions, level);
}
