/*
  Depth First Recursive split algorithm.
  Copyright (C) 2005-2009 Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "iseSplitAlgorithmDFRecursive.hpp"

#include "iseSplitDriverRecursive.hpp"
#include "iseSegmentationClustering.hpp"

using namespace ise;

SplitAlgorithmDFRecursive::SplitAlgorithmDFRecursive(ise::SegmentationContext *segContext)
  :
  SplitAlgorithmDefault(segContext)
{
  
}

void
SplitAlgorithmDFRecursive::apply(const girl::regionDeque &regions2split, 
				 ise::SplitDriver &driver, 
				 ise::SegmentationClustering &clust,
				 int level,
				 girl::regionDeque &createdRegions)
{
  driver.init(regions2split, level);
  clust.init(regions2split, level);
  
  girl::regionDeque::const_iterator it = regions2split.begin();
  const girl::regionDeque::const_iterator itEnd = regions2split.end();
  for ( ; it != itEnd; ++it) 
    splitAux(*it, driver, clust, level, createdRegions);

}

void
SplitAlgorithmDFRecursive::initHandleSplitOneRegion ( ise::SplitDriver &driver,
						      ise::SegmentationClustering &clust,
						      girl::regionDeque &regions,
						      int level)
{

  driver.init(regions, level);
  clust.init (regions, level);
}



void
SplitAlgorithmDFRecursive::splitAux(girl::region r, 
				    ise::SplitDriver &driver, 
				    ise::SegmentationClustering &clust,
				    int level,
				    girl::regionDeque &createdRegions)
{
 
  girl::regionDeque tmpCreatedRegions;
  ise::SplitAlgorithmDefault::splitOneRegion(r, driver, clust, level, tmpCreatedRegions);

  if (tmpCreatedRegions.size() == 1) {
    createdRegions.push_back(*tmpCreatedRegions.begin());
    return;
  }

  initHandleSplitOneRegion(driver, clust, tmpCreatedRegions, level);

  girl::regionDeque::const_iterator it = tmpCreatedRegions.begin();
  const girl::regionDeque::const_iterator itEnd = tmpCreatedRegions.end();
  
  for ( ; it!= itEnd; ++it) {
    if (driver.toSplit(*it, level)) {
      splitAux(*it, driver, clust, level+1, createdRegions);
    }
    else {
      createdRegions.push_back(*it);
    }
  }
  
}

