/*
  Recursive Split driver default implementation.
  Copyright (C) 2005-2009 Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "iseSplitDriverRecursiveDefault.hpp"

#include "iseSegmentationContext.hpp"
#include "iseRegionTools.hpp"
#include "iseStat.hpp"

using namespace ise;

SplitDriverRecursiveDefault::SplitDriverRecursiveDefault(ise::SegmentationContext *segContext, 
							 double threshold, 
							 int conversion)
  : 
  SplitDriverRecursive(segContext),
  m_homogeneityCriteria(0.0),
  m_threshold(threshold),
  m_conversion(conversion),
  m_toStop(false)
{
  
}


void
SplitDriverRecursiveDefault::init(const girl::regionDeque &regionsToSplit, int level)
{
  if (level == 0) {
    const double variance = iseRegionTools_computeColorimetricVarianceOfRegionSet(*(m_segContext->getToger()), *(m_segContext->getImage()), m_conversion, regionsToSplit);
    m_homogeneityCriteria = variance * m_threshold;
  }

  if (regionsToSplit.size() == 1 && level != 0)
    m_toStop = true;
  else
    m_toStop = false;
}

void
SplitDriverRecursiveDefault::init(girl::region /* r */, int /* level */)
{
  
}




bool
SplitDriverRecursiveDefault::toSplit(girl::region r, int /* level */ ) const
{
  RegionColorimetricFeatures *reg = m_segContext->getRegionSet()->getRegionFeatures(r);


  double m0 = 0.0;
  girl::tools::Flt3 m1(0.0, 0.0, 0.0);
  girl::tools::Flt3 m2(0.0, 0.0, 0.0);

  iseRegionTools_computeColorimetricMoments(*(m_segContext->getToger()), *(m_segContext->getImage()), r, m_conversion, m0, m1, m2);
  const double variance = iseStat_iseComputeVariance(m0, m1, m2);

  if ((reg != NULL) && (variance > m_homogeneityCriteria) && ! m_toStop)
    {
      return true;
    }
  return false;
}

/*
girl::region
SplitDriverRecursiveDefault::select(const girl::regionDeque &regions)
{
  

}

double
SplitDriverRecursiveDefault::evalRegion(const girl::region r)
{

}
*/

void 
SplitDriverRecursiveDefault::setParameters(double threshold)
{
  m_threshold = threshold;
} 

void 
SplitDriverRecursiveDefault::getParameters(double &threshold)
{
  threshold = m_threshold;
} 
