/*
  Split and Merge algorithm default implementation.
  Copyright (C) 2005-2009 Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "iseSplitMergeAlgoExtendSelection.hpp"

#include "iseSplitDriver.hpp"
#include "iseMergeDriver.hpp"
#include "iseSegmentationClustering.hpp"
#include "iseSegmentationContext.hpp"
#include "iseMergeAlgorithm.hpp"
#include "iseSplitAlgorithm.hpp"

#include <girl/girlToger.hpp>
#include <girl/girlHashFunc.hpp>
#include <girlTools/girlToolsRegionInfos.hpp>

using namespace ise;




void 
SplitMergeAlgoExtendSelection::apply(const girl::regionDeque &regionsToSplitMerge,
				     ise::MergeDriver &  /* driverMerge */,
				     ise::SplitDriver &driverSplit,
				     ise::SegmentationClustering &clust,
				     int level,
				     girl::regionDeque &createdRegions)
{
  
  assert(m_splitAlgo);
  assert(m_mergeAlgo);
  assert(m_segContext);

  clust.init(regionsToSplitMerge, 0);

  girl::toger *toger = m_segContext->getToger();

  girl::regionDeque copyQueue;

  girl::unorderedSet<girl::region> regionsToSplit;

  girl::regionDeque::const_iterator it = regionsToSplitMerge.begin();
  const girl::regionDeque::const_iterator itEnd = regionsToSplitMerge.end();
  for ( ; it != itEnd; ++it) {
    if(toger->isRegionValid(*it)) {
      //  because *it could have been merged with his mother
      girl::regionDeque regs;
      regs.push_back(*it); 
      girl::regionDeque motherQueue;
      girl::tools::getMotherRegions(*toger, regs, motherQueue);
      if (motherQueue.empty())
	continue;
      assert(motherQueue.size() == 1);
      girl::region mother = motherQueue.front();
      mother = toger->mergeRegions(mother, *it);
      regionsToSplit.insert(mother);
    }
  }
   
  girl::regionDeque regionsToSplitDq;
  for (girl::unorderedSet<girl::region>::const_iterator it = regionsToSplit.begin();
       it != regionsToSplit.end();
       ++it) {
    if(toger->isRegionValid(*it))
      regionsToSplitDq.push_back(*it);
  }

  m_splitAlgo->apply(regionsToSplitDq, 
		     driverSplit,
		     clust,
		     level,
		     createdRegions);


  //std::swap(copyQueue, createdRegions);

}
