/*
  Split and Merge algorithm base class.
  Copyright (C) 2005-2009 Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISESPLITMERGEALGORITHM_HPP
#define ISESPLITMERGEALGORITHM_HPP

#include <girl/girlToger.hpp>


namespace ise {

  class SegmentationContext;
  class MergeDriver;
  class SplitDriver;
  class SegmentationClustering;
  class SplitAlgorithm;
  class MergeAlgorithm;

  class SplitMergeAlgorithm 
  {

  public:

    SplitMergeAlgorithm(ise::SegmentationContext *segContext, 
			girl::shared_ptr<SplitAlgorithm> splitAlgo, 
			girl::shared_ptr<MergeAlgorithm> mergeAlgo)
      : m_segContext(segContext),
	m_splitAlgo(splitAlgo), 
	m_mergeAlgo(mergeAlgo)
    {}
    
    virtual ~SplitMergeAlgorithm() 
    {}
    

    virtual ise::SegmentationContext *getSegContext() const 
    { 
      return m_segContext; 
    }

    virtual void setSegContext(ise::SegmentationContext *s) 
    { 
      m_segContext = s; 
    }


    virtual void apply(const girl::regionDeque &regionsToMerge,
		       ise::MergeDriver &driverMerge,
		       ise::SplitDriver &driverSplit,
		       ise::SegmentationClustering &clust,
		       int level,
		       girl::regionDeque &keptRegions) = 0;

    virtual void setSplitAlgorithm(girl::shared_ptr<SplitAlgorithm> splitAlgo)
    {
      m_splitAlgo = splitAlgo;
    }
    
    virtual void setMergeAlgorithm(girl::shared_ptr<MergeAlgorithm> mergeAlgo)
    {
      m_mergeAlgo = mergeAlgo;
    }

  private:
    SplitMergeAlgorithm(SplitMergeAlgorithm &c);
    SplitMergeAlgorithm &operator=(SplitMergeAlgorithm &c);


  protected:

    SegmentationContext *m_segContext;
    girl::shared_ptr<SplitAlgorithm> m_splitAlgo;
    girl::shared_ptr<MergeAlgorithm> m_mergeAlgo;

  };
}

#endif /*  ISEMERGEALGORITHM_HPP */
