/*
  Split and Merge recursive algorithm.
  Copyright (C) 2005-2009 Jean-Philippe Domenger, Achille Braquelaire, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "iseSplitMergeAlgorithmRecursive.hpp"

#include "iseSplitDriver.hpp"
#include "iseMergeDriver.hpp"
#include "iseSegmentationClustering.hpp"
#include "iseSegmentationContext.hpp"
#include "iseMergeAlgorithm.hpp"
#include "iseSplitAlgorithm.hpp"

using namespace ise;

void 
SplitMergeAlgorithmRecursive::apply(const girl::regionDeque &regionsToSplitMerge,
				    ise::MergeDriver &mergeDriver,
				    ise::SplitDriver &splitDriver,
				    ise::SegmentationClustering &clust,
				    int level,
				    girl::regionDeque &keptRegions)
{  
  girl::regionDeque regions = regionsToSplitMerge;

  while (const bool c = apply_aux(regions, mergeDriver, splitDriver, clust, level, keptRegions)) {
    if (c) {
      regions = keptRegions;
      keptRegions.clear();
    }
  }
  
}



bool
SplitMergeAlgorithmRecursive::apply_aux(const girl::regionDeque &regionsToSplitMerge,
					ise::MergeDriver &mergeDriver,
					ise::SplitDriver &splitDriver,
					ise::SegmentationClustering &clust,
					int level,
					girl::regionDeque &keptRegions)
{

  girl::regionDeque createdRegions;

  assert(m_splitAlgo);
  m_splitAlgo->apply(regionsToSplitMerge, splitDriver, clust, level, createdRegions);
  const bool noSplit = (regionsToSplitMerge.size() == createdRegions.size());

  assert(m_mergeAlgo); 
  m_mergeAlgo->apply(createdRegions, mergeDriver, level, keptRegions);
  const bool noMerge = ( createdRegions.size() == keptRegions.size());

  if (noSplit || noMerge || regionsToSplitMerge.size() != keptRegions.size()) {
    
    return true;
  }
  else {
    assert(!noSplit && !noMerge && regionsToSplitMerge.size() == keptRegions.size());

    //if we continue we may split/merge infinitely the same region

    return false;


  }

}
