/*
  Segmentation clustering: RLSA
  Copyright (C) 2005-2009 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef ISETHRESHOLDERDATA_HPP
#define ISETHRESHOLDERDATA_HPP

namespace ise
{
  class ThresholderData {
  public:

    ThresholderData(int colorSpace = 0 , int channel = 0, const std::vector<int> &levels = std::vector<int>(1, 0))
      : m_colorSpace(colorSpace)
      , m_channel(channel)
      , m_levels(levels)
    {
    }

    ThresholderData(const ThresholderData &other)
      : m_colorSpace(other.m_colorSpace)
      , m_channel(other.m_channel)
      , m_levels(other.m_levels)
    {
    }

    ThresholderData& operator=(const ThresholderData &other)
    {
      if (this != &other) {
	m_colorSpace = other.m_colorSpace;
	m_channel = other.m_channel;
	m_levels = other.m_levels;
      }
      return *this;
    }

    void getParameters(int &colorSpace , int &channel, std::vector<int> &levels) const
    {
      colorSpace = m_colorSpace;
      channel = m_channel;
      levels = m_levels;
    }

    void setParameters(int colorSpace , int channel, const std::vector<int> &levels) 
    {
      m_colorSpace = colorSpace;
      m_channel = channel;
      m_levels = levels;
    }

    int colorSpace () const { return m_colorSpace; }
    int channel() const { return m_channel; }
    std::vector<int> levels() const { return m_levels; }

    void setColorSpace(int colorSpace ) { m_colorSpace = colorSpace; }
    void setChannel(int channel) { m_channel = channel; }
    void setLevels(const std::vector<int> &levels) { m_levels = levels; }

  private:

    int m_colorSpace;
    int m_channel;
    std::vector<int> m_levels;
  };
}

#endif /* ! ISETHRESHOLDERDATA_HPP */
