/*
  Copyright (C) 2005-2010 Vieux Remi, Roigt Julian, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "iseWeightedImageFeature.hpp"
#include <cassert>

using namespace ise;

//B:this ImageFeature does not store its data ! This is wrong !!!

WeightedImageFeature::WeightedImageFeature(girl::shared_ptr<girl::tools::ImageFeature> imageFeature)
  : girl::tools::ImageFeatureAbstract()
  , m_imageFeature(imageFeature)
  , m_weights()
{
}

WeightedImageFeature::WeightedImageFeature(girl::shared_ptr<girl::tools::ImageFeature> imageFeature,
					   const std::vector<double> &weights)
  : girl::tools::ImageFeatureAbstract()
  , m_imageFeature(imageFeature)
  , m_weights(weights)
{
  assert(weights.size() == static_cast<size_t>(m_imageFeature->channels()));
}

void
WeightedImageFeature::setWeights(const std::vector<double> &weights)
{
  assert(weights.size() == static_cast<size_t>(m_imageFeature->channels()));
  m_weights = weights;
}

void
WeightedImageFeature::weights(std::vector<double> &weights) const
{
  weights = m_weights;
}

int
WeightedImageFeature::width() const
{
  return m_imageFeature->width();
}

int
WeightedImageFeature::height() const
{
  return m_imageFeature->height();
}

int 
WeightedImageFeature::bytesPerLine() const
{
  return m_imageFeature->bytesPerLine();
}

int 
WeightedImageFeature::channels() const
{
  return m_imageFeature->channels();
}

bool
WeightedImageFeature::isNull() const
{
  return m_imageFeature->isNull();
}

girl::tools::ImageFeatureAbstract::type*
WeightedImageFeature::bits()
{
  return m_imageFeature->bits();
}

const girl::tools::ImageFeatureAbstract::type*
WeightedImageFeature::bits() const
{
  return m_imageFeature->bits();
}

girl::tools::PixelFromImage<girl::tools::ImageFeatureAbstract::type>
WeightedImageFeature::getPixel(int x, int y)
{
  girl::tools::PixelFromImage<girl::tools::ImageFeatureAbstract::type> p = m_imageFeature->getPixel(x, y);
  size_t size = m_imageFeature->channels();
  for (size_t i = 0 ; i < size ; ++i) {
    p[i] = p[i] * m_weights[i];
  }
  return p;
}

const girl::tools::PixelFromImage<girl::tools::ImageFeatureAbstract::type>
WeightedImageFeature::getPixel(int x, int y) const
{
  girl::tools::PixelFromImage<girl::tools::ImageFeatureAbstract::type> p = m_imageFeature->getPixel(x, y);
  size_t size = m_imageFeature->channels();
  for (size_t i = 0 ; i < size ; ++i) {
    p[i] = p[i] * m_weights[i];
  }
  return p;
}

