/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "iseWeightedRegionSetInterface.hpp"

using namespace ise;

WeightedRegionSetInterface::WeightedRegionSetInterface(ise::RegionSetInterfacePtr regSet, std::vector<double> weights)
  : ise::RegionSetInterface(regSet->getSegContext())
  , m_regSet(regSet)
  , m_weights(weights)
{
  assert(regSet->getFeatureDim() == weights.size());
}


void 
WeightedRegionSetInterface::getRegionFeatures(const girl::region &id, RegionFeature &rf)
{
  m_regSet->getRegionFeatures(id, rf);
  for (size_t i = 0 ; i < m_weights.size() ; ++i) {
    rf[i] *= m_weights[i];
  }
}

size_t 
WeightedRegionSetInterface::getFeatureDim() const
{
  return m_regSet->getFeatureDim();
}

void 
WeightedRegionSetInterface::getDimDescription(std::vector<std::string> &dimDesc) const
{
  m_regSet->getDimDescription(dimDesc);
}

void 
WeightedRegionSetInterface::getRequiredComputers(FeatureComputerListType &computers) const
{
  m_regSet->getRequiredComputers(computers);
}

void 
WeightedRegionSetInterface::reset()
{
  m_weights.clear();
  m_regSet->reset();
}


const std::string 
WeightedRegionSetInterface::uri() const
{
  return m_regSet->uri();
}
