/*
  Copyright (C) 2005-2011 Remi Vieux  remi.vieux@labri.fr

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "iseWordLinker.hpp"

#include <iostream>
#include <fstream>

#include <girl/girlHashFunc.hpp>
#include <girl/girlHashMap.hpp>
#include <girl/girlUnorderedSet.hpp>

#include <opencv2/core/core.hpp>


#include "iseConnectedComponentTools.hpp"
#include "iseHistogram1D.hpp"
#include "iseNeighbouringRegions.hpp"
#include "iseRunLengthExtractor.hpp"

// #include "iseQuantificationThresholderHuangWang.hpp"
// #include "iseQuantificationThresholderKapur.hpp"
// #include "iseQuantificationThresholderKittler.hpp"
// #include "iseQuantificationThresholderLi.hpp"
// #include "iseQuantificationThresholderLloyd.hpp"
// #include "iseQuantificationThresholderOtsu.hpp"
// #include "iseQuantificationThresholderQIR.hpp"
// #include "iseQuantificationThresholderRamesh.hpp"
// #include "iseQuantificationThresholderRiddler.hpp"
// #include "iseQuantificationThresholderSahoo.hpp"
// #include "iseQuantificationThresholderSezan.hpp"
// #include "iseQuantificationThresholderShanbag.hpp"
// #include "iseQuantificationThresholderVariance.hpp"
// #include "iseQuantificationThresholderYanni.hpp"
// #include "iseQuantificationThresholderYen.hpp"


// TODO: the functions used should not be in EnclosingSHape. Will do for now
#include "iseSegmentationClusteringEnclosingShape.hpp"


#ifdef VISU
#include <opencv2/highgui/highgui.hpp>
#endif //VISU


namespace ise
{



  // TODO:REMOVE
  void
  paintCC(cv::Mat &tmp, const ConnectedComponentTools::ConnectedComponent &cc, cv::Scalar color, int timeout = 1)
  {

    ConnectedComponentTools::ConnectedComponent::const_iterator it = cc.begin();
    const ConnectedComponentTools::ConnectedComponent::const_iterator itEnd = cc.end();
    for ( ; it != itEnd; ++it) {
      cv::circle(tmp, *it, 0, color);
    }

#ifdef VISU
    cv::namedWindow("Word Linkage", 0);
    cv::imshow("Word Linkage", tmp);
    cv::waitKey(timeout);
#endif //VISU
  }



  void print(const girl::hashMap<girl::region, NeighbouringRegions> &map)
  {
    girl::hashMap<girl::region, NeighbouringRegions>::const_iterator it = map.begin();
    const girl::hashMap<girl::region, NeighbouringRegions>::const_iterator itEnd = map.end();
    for ( ; it != itEnd; ++it) {
      std::cout << "Region " << it->first << ": Next " << it->second.next << "(" << it->second.distNext
		<< ") Previous " << it->second.prev << "(" << it->second.distPrev << ")" << std::endl;
    }
  }

  //  void
  //  WordLinker::buildForegroundBackgroundMat(const girl::toger &tog, cv::Mat &mat)
  //  {
  //
  //    assert(m_classifierFactory);
  //    girl::shared_ptr<ise::ForegroundBackgroundClassifier> classifier =
  //        m_classifierFactory->create(m_segContext);
  //    mat = cv::Mat::zeros(tog.height(), tog.width(), CV_8U);//, CV_RGB(255,255,255));
  //    girl::regionDeque regs;
  //    tog.allRegions(regs);
  //
  //    girl::regionDeque fg;
  //    girl::regionDeque bg;
  //    classifier->classify(regs, fg, bg);
  //
  //    girl::regionDeque::const_iterator it = fg.begin();
  //    const girl::regionDeque::const_iterator itEnd = fg.end();
  //    // paint fg regions
  //    for ( ; it != itEnd; ++it) {
  //        tog.iterateOnRegionDomain(*it, *this);
  //    }
  //
  //    //    cv::namedWindow("fg/bg mat", 0);
  //    //    cv::imshow("fg/bg mat", mat);
  //    //    cv::waitKey(100);
  //  }

  void
  WordLinker::linkWords(const girl::toger &tog, girl::hashMap<int, girl::unorderedSet<girl::region> > &words)
  {

    /// PASS THE NEIGHBOOURS FROM THE INTERFACE AS A PARAMETER TO THE WORD IS IT POSSIBLE?
    //    buildForegroundBackgroundMat(tog, m_mat);

    // copy mat because it is modified by extractConnectedComponentsFunction
    cv::Mat tmp = m_mat.clone();

    ConnectedComponentTools::ConnectedComponents ccs;
    ConnectedComponentTools::extractAllConnectedComponents(tmp, ccs, 8);
    cv::Mat labels(m_mat.rows, m_mat.cols, CV_32SC1, cv::Scalar(0, 0, 0, 0));
    ConnectedComponentTools::ConnectedComponents::const_iterator ccsIt = ccs.begin();
    const ConnectedComponentTools::ConnectedComponents::const_iterator ccsItEnd = ccs.end();
    for (int i = 1; ccsIt != ccsItEnd; ++ccsIt, ++i) {
      ConnectedComponentTools::ConnectedComponent::const_iterator ccIt = ccsIt->begin();
      const ConnectedComponentTools::ConnectedComponent::const_iterator ccItEnd = ccsIt->end();
      for ( ; ccIt != ccItEnd; ++ccIt) {
	labels.at<int>(ccIt->y, ccIt->x) = i;
      }
    }



    std::vector<ise::NeighbouringRegions> neighbours;
    m_mat.copyTo(tmp);
    ise::ConnectedComponentTools::computeSpacing(tmp, labels, ccs.size(), neighbours, m_orientation);

    girl::hashMap<int, int> cclabelToWordMap;
    computeWords(neighbours, cclabelToWordMap, m_threshold, ccs);

    girl::hashMap<int, girl::region> ccLabelToRegionMap;
    girl::hashMap<girl::region, int> girlRegToCCLabelMap;
    girl::regionDeque regs;
    tog.allRegions(regs);
    girl::regionDeque::const_iterator regIt = regs.begin();
    const girl::regionDeque::const_iterator regItEnd = regs.end();
    for ( ; regIt != regItEnd; ++regIt) {
      const girl::point pt = tog.anyPoint(*regIt);
      const int ccIndex = labels.at<int>(pt.y(), pt.x());
      if (ccIndex) {
	girlRegToCCLabelMap[*regIt] = ccIndex;
	ccLabelToRegionMap[ccIndex] = *regIt;
      }
    }

    girl::hashMap<girl::region, int>::const_iterator regToCCIt = girlRegToCCLabelMap.begin();
    const girl::hashMap<girl::region, int>::const_iterator regToCCItEnd = girlRegToCCLabelMap.end();
    for ( ; regToCCIt != regToCCItEnd; ++regToCCIt) {
      const girl::region reg = regToCCIt->first;
      const int ccIndex = regToCCIt->second -1;
      girl::hashMap<int, int>::iterator ccLabelToWordLabelIt;
      ccLabelToWordLabelIt = cclabelToWordMap.find(ccIndex);
      assert(ccLabelToWordLabelIt != cclabelToWordMap.end());
      const int wordIndex = ccLabelToWordLabelIt->second;
      girl::hashMap<int, girl::unorderedSet<girl::region> >::iterator wIt;
      if ((wIt = words.find(wordIndex)) != words.end()) {
	wIt->second.insert(reg);
      }
      else {
	words[wordIndex].insert(reg);
      }
    }



    //
    //
    //
    //
    //    girl::hashMap<int, girl::unorderedSet<girl::region> >::const_iterator wIt = words.begin();
    //    const girl::hashMap<int, girl::unorderedSet<girl::region> >::const_iterator wItEnd = words.end();
    //    for (int i = 0; wIt != wItEnd; ++wIt, ++i)  {
    //        std::cout << "Word " << i << ": ";
    //        girl::unorderedSet<girl::region>::const_iterator it = wIt->second.begin();
    //        const girl::unorderedSet<girl::region>::const_iterator itEnd = wIt->second.end();
    //        for ( ; it != itEnd; ++it) {
    //            std::cout << *it << ", ";
    //        }
    //        std::cout << std::endl;
    //    }

  }

  void
  collectRegions(const std::vector<ise::NeighbouringRegions> &neighbours,
		 girl::hashMap<int, int>& reg2wordMap,
		 girl::unorderedSet<int>& addedLabels,
		 int startRegion,
		 int label,
		 int threshold);

  void relabelRegions(girl::hashMap<int, int> &reg2wordMap,
		      girl::unorderedSet<int> &addedLabels,
		      int label)
  {
    girl::unorderedSet<int>::const_iterator it = addedLabels.begin();
    const girl::unorderedSet<int>::const_iterator itEnd = addedLabels.end();
    for ( ; it != itEnd; ++it) {
      const int index = *it;
      assert(reg2wordMap.find(index) != reg2wordMap.end());
      reg2wordMap[index] = label;
    }
  }


  void
  labelRegions(const std::vector<ise::NeighbouringRegions> &neighbours,
	       girl::hashMap<int, int> &reg2wordMap,
	       girl::unorderedSet<int> &addedLabels,
	       int label, int threshold)
  {
    girl::unorderedSet<int>::const_iterator it = addedLabels.begin();
    const girl::unorderedSet<int>::const_iterator itEnd = addedLabels.end();
    for ( ; it != itEnd; ++it) {
      const int index = *it;
      girl::hashMap<int, int>::iterator iter;
      if ((iter = reg2wordMap.find(index)) != reg2wordMap.end()) {
	girl::unorderedSet<int> regions;
	collectRegions(neighbours, reg2wordMap, regions, index, label, threshold);
	relabelRegions(reg2wordMap, regions, label);
	//  collectAndLabelRegions(neighbours, reg2wordMap, index, label, threshold);
      } else
	reg2wordMap[index] = label;
    }

  }


  void
  collectRegions(const std::vector<ise::NeighbouringRegions> &neighbours,
		 girl::hashMap<int, int>& /*reg2wordMap*/,
		 girl::unorderedSet<int>&addedLabels,
		 int startRegion,
		 int /*label*/,
		 int threshold)
  {
    // girl::unorderedSet<int> addedLabels;
    std::deque<int> toProcessLabels;

    addedLabels.insert(startRegion);
    toProcessLabels.push_back(startRegion);

    girl::hashMap<int, int>::iterator iter;

    while (!toProcessLabels.empty()) {
      const int index = toProcessLabels.front();
      const int prev = neighbours[index].prev;
      const int distPrev = neighbours[index].distPrev;
      const int next = neighbours[index].next;
      const int distNext = neighbours[index].distNext;
      if (distPrev < threshold &&
	  addedLabels.find(prev) == addedLabels.end()) {
	// at authorized distance and not added yet
	addedLabels.insert(prev);
	toProcessLabels.push_back(prev);
      }
      if (distNext < threshold &&
	  addedLabels.find(next) == addedLabels.end()) {
	addedLabels.insert(next);
	toProcessLabels.push_back(next);
      }
      toProcessLabels.pop_front();
    }
  }


  int
  WordLinker::computeWords(const std::vector<ise::NeighbouringRegions> &neighbours,
			   girl::hashMap<int, int>& reg2wordMap,
			   int threshold,
			   const ConnectedComponentTools::ConnectedComponents &/*ccs*/)
  {
    size_t wordCount = 1;
    girl::hashMap<int, int>::iterator reg2wordIt;
    std::vector<ise::NeighbouringRegions>::const_iterator nIt = neighbours.begin();
    const std::vector<ise::NeighbouringRegions>::const_iterator nItEnd = neighbours.end();
    //int timeout = 2;
    for (int currentCC = 0; nIt != nItEnd; ++nIt, ++currentCC) {

      //  std::cout << "cc " << currentCC << std::endl;

      if ( reg2wordMap.find(currentCC) == reg2wordMap.end()) {
	girl::unorderedSet<int> addedLabels;
	collectRegions(neighbours, reg2wordMap, addedLabels, currentCC, wordCount, threshold);
	labelRegions(neighbours, reg2wordMap, addedLabels, wordCount++, threshold);
      } else {
	// if (currentCC == 32534)
	//  std::cout << "cc " << currentCC << " deja labelisee: " << reg2wordMap[currentCC] << std::endl;
      }
    }

    girl::unorderedSet<int> count;
    for ( reg2wordIt = reg2wordMap.begin(); reg2wordIt != reg2wordMap.end(); ++reg2wordIt) {
      count.insert(reg2wordIt->second);
    }
    return count.size();

  }

  void
  WordLinker::processPoint(const girl::point &pt)
  {
    m_mat.at<uchar>(pt.y(), pt.x()) = 255;
  }

}
