/*
  Copyright (C) 2005-2011 Remi Vieux  remi.vieux@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISEWORDLINKER_HPP_
#define ISEWORDLINKER_HPP_

#include <vector>

#include <girl/girlCommon.hpp>
#include <girl/girlHashMap.hpp>
#include <girl/girlToger.hpp>
#include <girl/girlUnorderedSet.hpp>

#include <opencv2/core/core.hpp>

#include "iseConnectedComponentTools.hpp"
#include "iseForegroundBackgroundClassifier.hpp"
#include "iseNeighbouringRegions.hpp"
#include "iseRunLengthExtractor.hpp"



/*
 *
 */
namespace ise
{

  class SegmentationContext;

  class GIRL_API WordLinker : public girl::toger::pointIterator
  {

  public:

    explicit WordLinker(ise::SegmentationContext *segContext) :
      m_segContext(segContext),
      m_classifierFactory(NULL),
      m_mat(),
      m_threshold(0),
      m_orientation(ise::RunLengthExtractor::HORIZONTAL)
    {
//      m_classifier = new ise::ForegroundBackgroundClassifierColorArea(segContext);
    }

    virtual ~WordLinker() {
   //   delete m_classifier;
    }
    /**
     * Given a toger, e.g. extracted with shapeCC, which gives a bounding box around each letter
     * return the list of regions which must be merged to form words.
     */

    /**
     * Set horizontal and vertical thresholds
     */
    void
    setParameters(const ise::RunLengthExtractor::Orientation ori, const int threshold, ForegroundBackgroundClassifierFactory *factory)
    {
      m_orientation = ori;
      m_threshold = threshold;
      m_classifierFactory = factory;
    }

    void linkWords(const girl::toger &tog, girl::hashMap<int, girl::unorderedSet<girl::region> > &words);
  //  void linkWords(const girl::toger &tog, )


  protected:

  //  void buildForegroundBackgroundMat(const girl::toger &tog, cv::Mat &mat);

    int computeWords(const std::vector<ise::NeighbouringRegions> &neighbours,
        girl::hashMap<int, int>& reg2wordMap,
        int threshold,
        const ConnectedComponentTools::ConnectedComponents &ccs);

    void
    processPoint(const girl::point &pt);

  protected:

    ise::SegmentationContext *m_segContext;

    ise::ForegroundBackgroundClassifierFactory *m_classifierFactory;

    cv::Mat m_mat;

    int m_threshold;

    ise::RunLengthExtractor::Orientation m_orientation;
  };

}

#endif /* ISEWORDLINKER_HPP_ */
