#ifndef HPRF_HI_PR_HPP
#define HPRF_HI_PR_HPP

#include "types.h"

class MaxFlowAlgorithm
{
public:
  MaxFlowAlgorithm();
  ~MaxFlowAlgorithm();

  inline void setGlobalUpdateFrequency(float freq) { globUpdtFreq = freq; }

  int allocDS();
  void init();
  void stageOne();
  void stageTwo();
  void globalUpdate();

  inline double getFlow() { return flow; }

  void checkSolution();
  void printStat();
  void printFlow();
  void printCut();

  long   n;                    /* number of nodes */
  long   m;                    /* number of arcs */
  node   *nodes;               /* array of nodes */
  arc    *arcs;                /* array of arcs */
  cType  *cap;                 /* array of capacities */
  node   *source;              /* source node pointer */
  node   *sink;                /* sink node pointer */
  long   nMin;                 /* smallest node id */

  node   *sentinelNode;        /* end of the node list marker */

private:
  long   nm;                   /* n + ALPHA * m */
  bucket *buckets;             /* array of buckets */
  long   dMax;                 /* maximum label */
  long   aMax;                 /* maximum actie node label */
  long   aMin;                 /* minimum active node label */
  double flow;                 /* flow value */
  long pushCnt;           /* number of pushes */
  long relabelCnt;       /* number of relabels */
  long updateCnt;       /* number of updates */
  long gapCnt;           /* number of gaps */
  long gNodeCnt;           /* number of nodes after gap */
  //node   *sentinelNode;        /* end of the node list marker */
  arc *stopA;                  /* used in forAllArcs */
  long workSinceUpdate;      /* the number of arc scans since last update */
  float globUpdtFreq;          /* global update frequency */

  void checkMax();
  int gap(bucket *emptyB);
  long relabel(node *i);
  void discharge(node *i);
  void wave();
};

#endif /* HPRF_HI_PR_HPP */
