#ifndef HPRF_PARSER_HPP
#define HPRF_PARSER_HPP

#include "MaxFlowAlgorithm.hpp"
#include "types.h"
#include <istream>
#include <string>

class Parser {
public:
  Parser();
  ~Parser() {}

  void parseStream(MaxFlowAlgorithm &mfa, std::istream &sr);

private:
  void parseLine(std::string in_line);
  void orderArcs();
  void constructLists();
  void assignOutputValues(MaxFlowAlgorithm &mfa);

  long no_lines;             /* no of current input line */
  long no_plines;            /* no of problem-lines */
  long no_nslines;           /* no of node-source-lines */
  long no_nklines;           /* no of node-source-lines */
  long no_alines;            /* no of arc-lines */
  long pos_current;          /* 2*no_alines */

  long    n;                      /* internal number of nodes */
  long    m;                      /* internal number of arcs */

  arc     *arcs;             /* pointer to the arc structure */
  long *arc_first;         /* internal array for holding
				- node degree
				- position of the first outgoing arc */
  long *arc_tail;          /* internal array: tails of the arcs */

  cType    *acap;             /* array of capasities */

  node    *nodes;            /* pointer to the node structure */

  arc     *arc_current;
  cType    cap;                    /* capasity of the current arc */

  long source;               /* no of the source */
  long sink;                 /* no of the sink */

  long    node_min;             /* minimal no of node  */
  long node_max;             /* maximal no of nodes */

};

#endif /* HPRF_PARSER_HPP */
