#include "types.h"
#include "Parser.hpp"
#include "MaxFlowAlgorithm.hpp"
#include <stdio.h>
#include <stdlib.h>
#include <iostream>

int main (int argc, char *argv[]) {
  if (argc > 2) {
    printf("Usage: %s [update frequency]\n", argv[0]);
    exit(1);
  }

  MaxFlowAlgorithm mfa;
  if (argc == 2) mfa.setGlobalUpdateFrequency((float) atof(argv[1]));
 

  printf("c\nc hi_pr version 3.6\n");
  printf("c Copyright C by IG Systems, igsys@eclipse.net\nc\n");

  Parser parser;
  parser.parseStream(mfa, std::cin);

  printf("c nodes:       %10ld\nc arcs:        %10ld\nc\n", mfa.n, mfa.m);

  int cc = mfa.allocDS();
  if ( cc ) { fprintf ( stderr, "Allocation error\n"); exit ( 1 ); }

  mfa.init();
  mfa.stageOne ( );

  printf ("c flow:       %12.01f\n", mfa.getFlow());

#ifndef CUT_ONLY
  mfa.stageTwo ( );

#endif

#ifdef CHECK_SOLUTION
  mfa.checkSolution();
#endif

#ifdef PRINT_STAT
  mfa.printStat();
#endif

#ifdef PRINT_FLOW
  mfa.printFlow();
#endif

#ifdef PRINT_CUT
  mfa.printCut();
#endif

  exit(0);

}
