/*
  Test program for color spaces conversion functions.
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#include "ColorSpacesTest.hpp"

#include <iostream>
#include <limits>
#include <ise/iseConversion.hpp>

// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( ColorSpacesTest );



struct ColorSpaceDef
{
  const char *name;
  int conv;
};

static ColorSpaceDef colorSpaces[] = {{"RGB", ISE_RGB_to_RGB}, {"I1I2I3", ISE_RGB_to_I1I2I3}, {"H1H2H3", ISE_RGB_to_H1H2H3}, {"Luv", ISE_RGB_to_Luv}, {"YIQ", ISE_RGB_to_YIQ}, {"K1K2K3", ISE_RGB_to_K1K2K3}, {"T1T2T3", ISE_RGB_to_T1T2T3}, {"HSV", ISE_RGB_to_HSV}, {"HMMD", ISE_RGB_to_HMMD}, {"YCbCr", ISE_RGB_to_YCbCr}, {"YUV", ISE_RGB_to_YUV}, {"Lab", ISE_RGB_to_Lab}}; 
//{"V1V2V3", ISE_RGB_to_V1V2V3},  //removed because seems totally buggy !


#include <cstdlib>
static
void
dbg()
{
  //int i = 3; //Luv
  int i = 1; //I1I2I3

  int r=0, g=0, b=5;
   
  std::cerr<<"rgb=("<<(int)r<<", "<<(int)g<<", "<<(int)b<<")\n";
 
  int x, y, z;
  {
    int val1[3];
    for (size_t c=0; c<3; ++c)
      val1[c] = (ise::conversion::changeSpace(colorSpaces[i].conv)[c])(r, g, b);
    int val2[3];
    ise::conversion::changeFromRGB(colorSpaces[i].conv, r, g, b, val2[0], val2[1], val2[2]);
    if (! (val1[0] == val2[0] && val1[1] == val2[1] && val1[2] == val2[2])) {
      std::cerr<<"For color space "<<colorSpaces[i].name<<": 3x1: val1=("<<val1[0]<<", "<<val1[1]<<", "<<val1[2]<<") !=  1x3: val2=("<<val2[0]<<", "<<val2[1]<<", "<<val2[2]<<")  for rgb=("<<(int)r<<", "<<(int)g<<", "<<(int)b<<")\n";
      exit(11);
    }
    CPPUNIT_ASSERT(val1[0] == val2[0] && val1[1] == val2[1] && val1[2] == val2[2]);

    x = val1[0]; 
    y = val1[1];
    z = val1[2];
  }
  std::cerr<<"xyz=("<<(int)x<<", "<<(int)y<<", "<<(int)z<<")\n";
  {
    int val1[3];
    for (size_t c=0; c<3; ++c)
      val1[c] = (ise::conversion::restoreSpace(colorSpaces[i].conv)[c])(x, y, z);
    int val2[3];
    ise::conversion::changeToRGB(colorSpaces[i].conv, x, y, z, val2[0], val2[1], val2[2]);
    if (! (val1[0] == val2[0] && val1[1] == val2[1] && val1[2] == val2[2])) {
      std::cerr<<"For color space "<<colorSpaces[i].name<<" to RGB : rgb=("<<(int)r<<", "<<(int)g<<", "<<(int)b<<") -> CS=("<<(int)x<<", "<<(int)x<<", "<<(int)z<<") -> : val1=("<<val1[0]<<", "<<val1[1]<<", "<<val1[2]<<") != val2=("<<val2[0]<<", "<<val2[1]<<", "<<val2[2]<<") \n";
      exit(11);
    }
    CPPUNIT_ASSERT(val1[0] == val2[0] && val1[1] == val2[1] && val1[2] == val2[2]);

    if (! (val1[0] == r && val1[1] == g && val1[2] == b)) {
      std::cerr<<"For color space "<<colorSpaces[i].name<<" to RGB : rgb=("<<(int)r<<", "<<(int)g<<", "<<(int)b<<") -> CS=("<<(int)x<<", "<<(int)y<<", "<<(int)z<<") -> rgb=("<<val1[0]<<", "<<val1[1]<<", "<<val1[2]<<") \n";
      exit(11);
    }
    

  }
  
}


static
void
dbg2()
{
  int i = 15; //V1V2V3

  int r=100, g=200, b=50;
   
  std::cerr<<"rgb=("<<(int)r<<", "<<(int)g<<", "<<(int)b<<")\n";
  {
    int val1[3];
    for (size_t c=0; c<3; ++c)
      val1[c] = (ise::conversion::changeSpace(colorSpaces[i].conv)[c])(r, g, b);
    int val2[3];
    ise::conversion::changeFromRGB(colorSpaces[i].conv, r, g, b, val2[0], val2[1], val2[2]);
    //if (! (val1[0] == val2[0] && val1[1] == val2[1] && val1[2] == val2[2])) {
      std::cerr<<"For color space "<<colorSpaces[i].name<<": 3x1: val1=("<<val1[0]<<", "<<val1[1]<<", "<<val1[2]<<") !=  1x3: val2=("<<val2[0]<<", "<<val2[1]<<", "<<val2[2]<<")  for rgb=("<<(int)r<<", "<<(int)g<<", "<<(int)b<<")\n";
      exit(11);
      //}
    CPPUNIT_ASSERT(val1[0] == val2[0] && val1[1] == val2[1] && val1[2] == val2[2]);
  }
}

static
void
dbg3()
{
  int i = 10; //YUV
  std::cerr<<"nb colorspaces="<<sizeof(colorSpaces)/sizeof(ColorSpaceDef)<<"  colorSpaces[i].name="<<colorSpaces[i].name<<"\n";
  int r=0, g=0, b=3;

  int x, y, z;
  ise::conversion::changeFromRGB(colorSpaces[i].conv, r, g, b, x, y, z);
  int r2, g2, b2;
  ise::conversion::changeToRGB(colorSpaces[i].conv, x, y, z, r2, g2, b2);
  
  //if (! (r ==r2 && g==g2 && b==b2) ) {
    std::cerr<<"For color space "<<colorSpaces[i].name<<": RGB->CS->RGB: rgb=("<<(int)r<<", "<<(int)g<<", "<<(int)b<<") != rgb=("<<(int)r2<<", "<<(int)g2<<", "<<(int)b2<<")\n";
    //}

  exit(10);

}


static
void
printMinMax()
{
  const size_t nbColorSpaces = sizeof(colorSpaces)/sizeof(ColorSpaceDef);
  for (size_t i=0; i<nbColorSpaces; ++i) {
    
    const int Rmin=0, Rmax=255, Gmin=0, Gmax=255, Bmin=0, Bmax=255;
    
    int minX = std::numeric_limits<int>::max();
    int minY = std::numeric_limits<int>::max();
    int minZ = std::numeric_limits<int>::max();
    int maxX = std::numeric_limits<int>::min();
    int maxY = std::numeric_limits<int>::min();
    int maxZ = std::numeric_limits<int>::min();
    
    for (int c0=Rmin; c0<=Rmax; ++c0) {
      for (int c1=Gmin; c1<=Gmax; ++c1) {
	for (int c2=Bmin; c2<=Bmax; ++c2) {
	  
	  int x, y, z;
	  ise::conversion::changeFromRGB(colorSpaces[i].conv, c0, c1, c2, x, y, z);
	  
	  if (x < minX)
	    minX = x;
	  if (x > maxX)
	    maxX = x;
	  
	  if (y < minY)
	    minY = y;
	  if (y > maxY)
	    maxY = y;
	  
	  if (z < minZ)
	    minZ = z;
	  if (z > maxZ)
	    maxZ = z;
	}
      }
    }
    std::cerr<<"For color space "<<colorSpaces[i].name<<"\n";
    std::cerr<<"c[0] in ["<<minX<<"; "<<maxX<<"]\n";
    std::cerr<<"c[1] in ["<<minY<<"; "<<maxY<<"]\n";
    std::cerr<<"c[2] in ["<<minZ<<"; "<<maxZ<<"]\n";
  }
}


//check that the absolute min/max that we get with value in [0; 255]³ are those given by ise::conversion::absoluteMin/Max
void
ColorSpacesTest::testMinMax()
{
  //dbg();
  //dbg2();
  //dbg3();

  //printMinMax();


  const int Rmin=0, Rmax=255, Gmin=0, Gmax=255, Bmin=0, Bmax=255;

  const size_t nbColorSpaces = sizeof(colorSpaces)/sizeof(ColorSpaceDef);
  for (size_t i=0; i<nbColorSpaces; ++i) {
    for (size_t c=0; c<3; ++c) {
      const int setMin = ise::conversion::absoluteMin(colorSpaces[i].conv, c);
      const int setMax = ise::conversion::absoluteMax(colorSpaces[i].conv, c);
      
      int getMin = std::numeric_limits<int>::max();
      int getMax = std::numeric_limits<int>::min();
      for (int c0=Rmin; c0<=Rmax; ++c0) {
	for (int c1=Gmin; c1<=Gmax; ++c1) {
	  for (int c2=Bmin; c2<=Bmax; ++c2) {
	    const int val = (ise::conversion::changeSpace(colorSpaces[i].conv)[c])(c0, c1, c2);
	    if (val < getMin)
	      getMin = val;
	    if (val > getMax)
	      getMax = val;
	    
	    if (val < setMin) 
	      std::cerr<<"For color space "<<colorSpaces[i].name<<": from RGB : conv["<<c<<"]("<<c0<<", "<<c1<<", "<<c2<<")="<<val<<" < setMin="<<setMin<<"\n"; 
	    if (val > setMax) 
	      std::cerr<<"For color space "<<colorSpaces[i].name<<": from RGB : conv["<<c<<"]("<<c0<<", "<<c1<<", "<<c2<<")="<<val<<" > setMax="<<setMax<<"\n"; 
	    

	  }
	}
      }
      
      if (getMin != setMin)
	std::cerr<<"For color space "<<colorSpaces[i].name<<": from RGB : conv["<<c<<"] setMin="<<setMin<<" getMin="<<getMin<<std::endl;
      
      CPPUNIT_ASSERT(getMin == setMin);

      if (getMax != setMax)
	std::cerr<<"For color space "<<colorSpaces[i].name<<": from RGB : conv["<<c<<"] setMax="<<setMax<<" getMax="<<getMax<<std::endl;
      
      CPPUNIT_ASSERT(getMax == setMax);
      
    }
    

  }
  
  
}

//check that values transformed back from color space to RGb are in [0; 255]³
/* NO guarantee that all values in CS map to RGB !!!
void
ColorSpacesTest::testReturnInRGB()
{
  const size_t nbColorSpaces = sizeof(colorSpaces)/sizeof(ColorSpaceDef);
  for (size_t i=0; i<nbColorSpaces; ++i) {
    int csMin[3], csMax[3];
    for (size_t c=0; c<3; ++c) {
      csMin[c] = ise::conversion::absoluteMin(colorSpaces[i].conv, c);
      csMax[c] = ise::conversion::absoluteMax(colorSpaces[i].conv, c);
    }

    for (size_t c=0; c<3; ++c) {
      int getMin = std::numeric_limits<int>::max();
      int getMax = std::numeric_limits<int>::min();
      for (int c0=csMin[0]; c0<=csMax[0]; ++c0) {
	for (int c1=csMin[1]; c1<=csMax[1]; ++c1) {
	  for (int c2=csMin[2]; c2<=csMax[2]; ++c2) {
	    const int val = (ise::conversion::restoreSpace(colorSpaces[i].conv)[c])(c0, c1, c2);
	    if (val < getMin)
	      getMin = val;
	    if (val > getMax)
	      getMax = val;

	    if (val < 0) 
	      std::cerr<<"For color space "<<colorSpaces[i].name<<": to RGB : conv["<<c<<"]("<<c0<<", "<<c1<<", "<<c2<<")="<<val<<" < 0\n"; 
	    if (val > 255) 
	      std::cerr<<"For color space "<<colorSpaces[i].name<<": to RGB : conv["<<c<<"]("<<c0<<", "<<c1<<", "<<c2<<")="<<val<<" > 255\n"; 

	  }
	}
      }
    
      if (getMin <0) {
	std::cerr<<"For color space "<<colorSpaces[i].name<<": to RGB : conv["<<c<<"] getMin="<<getMin<<" < 0 "<<std::endl;
      }
      CPPUNIT_ASSERT(getMin >= 0);
      
      if (getMax > 255) {
	std::cerr<<"For color space "<<colorSpaces[i].name<<": to RGB : conv["<<c<<"] getMax="<<getMax<<" > 255 "<<std::endl;
      }
      CPPUNIT_ASSERT(getMax <= 255);
    }
  }
}
*/

//check that values in RGB transformed to CS then transformed back to RGB are equals
void
ColorSpacesTest::testRGB2CS2RGB()
{
  const int Rmin=0, Rmax=255, Gmin=0, Gmax=255, Bmin=0, Bmax=255;

  bool testPassed = true;

  const size_t nbColorSpaces = sizeof(colorSpaces)/sizeof(ColorSpaceDef);
  for (size_t i=0; i<nbColorSpaces; ++i) {
    for (int r=Rmin; r<=Rmax; ++r) {
      for (int g=Gmin; g<=Gmax; ++g) {
	for (int b=Bmin; b<=Bmax; ++b) {
	  int x, y, z;
	  ise::conversion::changeFromRGB(colorSpaces[i].conv, r, g, b, x, y, z);
	  int r2, g2, b2;
	  ise::conversion::changeToRGB(colorSpaces[i].conv, x, y, z, r2, g2, b2);

	  if (! (r ==r2 && g==g2 && b==b2) ) {
	    std::cerr<<"For color space "<<colorSpaces[i].name<<": RGB->CS->RGB: rgb=("<<(int)r<<", "<<(int)g<<", "<<(int)b<<") != rgb=("<<(int)r2<<", "<<(int)g2<<", "<<(int)b2<<")\n";
	    testPassed = false;
	  }
	  //CPPUNIT_ASSERT(r ==r2 && g==g2 && b==b2);
	}
      }
    }
  }
  CPPUNIT_ASSERT(testPassed);
}


//check that transforming channels individually gives the same result than transforming the channels all at once
// from RGB
void
ColorSpacesTest::test1x3vs3x1FromRGB()
{
 const int Rmin=0, Rmax=255, Gmin=0, Gmax=255, Bmin=0, Bmax=255;

  const size_t nbColorSpaces = sizeof(colorSpaces)/sizeof(ColorSpaceDef);
  for (size_t i=0; i<nbColorSpaces; ++i) {
    for (int r=Rmin; r<=Rmax; ++r) {
      for (int g=Gmin; g<=Gmax; ++g) {
	for (int b=Bmin; b<=Bmax; ++b) {
	  int val1[3];
	  for (size_t c=0; c<3; ++c)
	    val1[c] = (ise::conversion::changeSpace(colorSpaces[i].conv)[c])(r, g, b);
	  int val2[3];
	  ise::conversion::changeFromRGB(colorSpaces[i].conv, r, g, b, val2[0], val2[1], val2[2]);
	  if (! (val1[0] == val2[0] && val1[1] == val2[1] && val1[2] == val2[2])) {
	    std::cerr<<"For color space "<<colorSpaces[i].name<<": 3x1: val1=("<<val1[0]<<", "<<val1[1]<<", "<<val1[2]<<") !=  1x3: val2=("<<val2[0]<<", "<<val2[1]<<", "<<val2[2]<<")  for rgb=("<<(int)r<<", "<<(int)g<<", "<<(int)b<<")\n";
	  }
	  CPPUNIT_ASSERT(val1[0] == val2[0] && val1[1] == val2[1] && val1[2] == val2[2]);
	}
      }
    }
  }
}

//check that transforming channels individually gives the same result than transforming the channels all at once
// to RGB
/*
 NO guarantee that all values in CS map to RGB !!!
void
ColorSpacesTest::test1x3vs3x1ToRGB()
{
  const size_t nbColorSpaces = sizeof(colorSpaces)/sizeof(ColorSpaceDef);
  for (size_t i=0; i<nbColorSpaces; ++i) {
    int csMin[3], csMax[3];
    for (size_t c=0; c<3; ++c) {
      csMin[i] = ise::conversion::absoluteMin(colorSpaces[i].conv, c);
      csMax[i] = ise::conversion::absoluteMax(colorSpaces[i].conv, c);
    }
    
    for (int c0=csMin[0]; c0<=csMax[0]; ++c0) {
      for (int c1=csMin[1]; c1<=csMax[1]; ++c1) {
	for (int c2=csMin[2]; c2<=csMax[2]; ++c2) {

	  int val1[3];
	  for (size_t c=0; c<3; ++c)
	    val1[c] = (ise::conversion::restoreSpace(colorSpaces[i].conv)[c])(c0, c1, c2);
	  int val2[3];
	  ise::conversion::changeToRGB(colorSpaces[i].conv, c0, c1, c2, val2[0], val2[1], val2[2]);
	  if (! (val1[0] == val2[0] && val1[1] == val2[1] && val1[2] == val2[2])) {
	    std::cerr<<"For color space "<<colorSpaces[i].name<<" to RGB : val1=("<<val1[0]<<", "<<val1[1]<<", "<<val1[2]<<") != val2=("<<val2[0]<<", "<<val2[1]<<", "<<val2[2]<<")\n";
	  }
	  CPPUNIT_ASSERT(val1[0] == val2[0] && val1[1] == val2[1] && val1[2] == val2[2]);
	}
      }
    }
  }
}
*/
void
ColorSpacesTest::test1x3vs3x1ToRGB()
{
 const int Rmin=0, Rmax=255, Gmin=0, Gmax=255, Bmin=0, Bmax=255;

  const size_t nbColorSpaces = sizeof(colorSpaces)/sizeof(ColorSpaceDef);
  for (size_t i=0; i<nbColorSpaces; ++i) {
    for (int r=Rmin; r<=Rmax; ++r) {
      for (int g=Gmin; g<=Gmax; ++g) {
	for (int b=Bmin; b<=Bmax; ++b) {

	  int x, y, z;
	  ise::conversion::changeFromRGB(colorSpaces[i].conv, r, g, b, x, y, z);
	  
	  int val1[3];
	  for (size_t c=0; c<3; ++c)
	    val1[c] = (ise::conversion::restoreSpace(colorSpaces[i].conv)[c])(x, y, z);
	  int val2[3];
	  ise::conversion::changeToRGB(colorSpaces[i].conv, x, y, z, val2[0], val2[1], val2[2]);
	  if (! (val1[0] == val2[0] && val1[1] == val2[1] && val1[2] == val2[2])) {
	    std::cerr<<"For color space "<<colorSpaces[i].name<<" to RGB : rgb=("<<(int)r<<", "<<(int)g<<", "<<(int)b<<") -> CS=("<<(int)x<<", "<<(int)y<<", "<<(int)z<<") -> : val1=("<<val1[0]<<", "<<val1[1]<<", "<<val1[2]<<") != val2=("<<val2[0]<<", "<<val2[1]<<", "<<val2[2]<<") \n";
	  }
	  CPPUNIT_ASSERT(val1[0] == val2[0] && val1[1] == val2[1] && val1[2] == val2[2]);
	}
      }
    }
  }
}

