/*
  Test program for 1D Histogram.
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#include "Histogram1DTest.hpp"

#include <cmath>

#include <ise/iseHistogram1D.hpp>

// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( Histogram1DTest );


void 
Histogram1DTest::setUp()
{
}


void 
Histogram1DTest::tearDown()
{
}

static
void
P_testEmpty(int absMin, int absMax)
{
  ise::Histogram1D h(absMin, absMax);
  
  h[0] = 200;
  h.reset(absMin, absMax);

  CPPUNIT_ASSERT(h.absoluteMin() == absMax);
  CPPUNIT_ASSERT(h.absoluteMax() == absMin);
}

void 
Histogram1DTest::testEmpty1()
{
  const int absMin = 0;
  const int absMax = 255;

  P_testEmpty(absMin, absMax);
}

void 
Histogram1DTest::testEmpty2()
{
  const int absMin = -257;
  const int absMax = 257;

  P_testEmpty(absMin, absMax);
}

static
void 
P_testOneValue(int absMin, int absMax, int j)
{
  ise::Histogram1D h(absMin, absMax);    
  CPPUNIT_ASSERT(j>=h.absoluteMin() && j<=h.absoluteMax());
  const int i = j - h.absoluteMin(); //in [0; h.size()[
  CPPUNIT_ASSERT(i>=0 && i<static_cast<int>(h.size()));
  h[i] = 100;

  CPPUNIT_ASSERT(h.getMin() == j);
  CPPUNIT_ASSERT(h.getMax() == j);
  //CPPUNIT_ASSERT(h.getMedian() == j);
  CPPUNIT_ASSERT(h.getMean() == j);
}

void 
Histogram1DTest::testOneValue1()
{
  const int absMin = 0;
  const int absMax = 255;

  P_testOneValue(absMin, absMax, absMin);
  P_testOneValue(absMin, absMax, absMax);
  P_testOneValue(absMin, absMax, (absMin+absMax)/2);
}

void 
Histogram1DTest::testOneValue2()
{
  const int absMin = -345;
  const int absMax = 365;

  P_testOneValue(absMin, absMax, absMin);
  P_testOneValue(absMin, absMax, absMax);
  P_testOneValue(absMin, absMax, (absMin+absMax)/2);
}

static
void 
P_testTwoValues(int absMin, int absMax, int j0, int j1)
{
  CPPUNIT_ASSERT(j0 < j1);
  ise::Histogram1D h(absMin, absMax);    
  CPPUNIT_ASSERT(j0>=h.absoluteMin() && j0<=h.absoluteMax());
  CPPUNIT_ASSERT(j1>=h.absoluteMin() && j1<=h.absoluteMax());

  const int i0 = j0 - h.absoluteMin(); //in [0; h.size()[
  CPPUNIT_ASSERT(i0>=0 && i0<(int)h.size());
  h[i0] = 100;
  const int i1 = j1 - h.absoluteMin(); //in [0; h.size()[
  CPPUNIT_ASSERT(i1>=0 && i1<static_cast<int>(h.size()));
  h[i1] = 100;
  
  CPPUNIT_ASSERT(h.getMin() == j0);
  CPPUNIT_ASSERT(h.getMax() == j1);
  //CPPUNIT_ASSERT(h.getMedian() == j0);
  CPPUNIT_ASSERT(h.getMean() == static_cast<int>(floor((j0+j1)/2.0)));
}

void 
Histogram1DTest::testTwoValues1()
{
  const int absMin = 0;
  const int absMax = 255;

  P_testTwoValues(absMin, absMax, absMin, absMin+1);
  P_testTwoValues(absMin, absMax, absMax-1, absMax);
  P_testTwoValues(absMin, absMax, absMin, absMax);
}

void 
Histogram1DTest::testTwoValues2()
{
  const int absMin = -267;
  const int absMax = 357;

  P_testTwoValues(absMin, absMax, absMin, absMin+1);
  P_testTwoValues(absMin, absMax, absMax-1, absMax);
  P_testTwoValues(absMin, absMax, absMin, absMax);
}

