#include <cstdlib>
#include <iostream>
#include <fstream>
#include <sstream>


#include <ise/iseKernelEpanechnikov.hpp>
#include <ise/iseKernelCauchy.hpp>
#include <ise/iseKernelGaussian.hpp>
#include <ise/iseKernelFunction.hpp>

// g++ -g -Wall TestKernels.cpp -o TestKernels `pkg-config --cflags --libs opencv girl-2.0`

const double zero = 0;

template <typename T>
void
printKernelValues(const std::string &file,
		  ise::KernelFunction<T> &kernel)
{

  std::ofstream outfile(file.c_str());

  double step = 0.1;

  for (double d = -1-step; d < 1+step; d+=step) {
    
    outfile << d << " " << kernel(&zero, &d, 1) << std::endl;
  }
  outfile.close();

}


int
main (int argc, char *argv[])
{

  double ps[7] = {0.1, 0.5, 1,2,3,4,5};

  for (int i = 0; i < 7; ++i) {
    double p = ps[i];

    ise::KernelGaussian<double> g(1.0f, p);
    std::stringstream gss(std::ios::in|std::ios::out);
    gss << "Gaussian" << p << ".dat";
    std::string gs(gss.str());
    printKernelValues(gs, g);

    ise::KernelCauchy<double> c(1.0f, p);
    std::stringstream css(std::ios::in|std::ios::out);
    css << "Cauchy" << p << ".dat";
    std::string cs(css.str());
    printKernelValues(cs, c);

    // ise::KernelEpanechnikov<double> e(1.0f, p);
    // std::stringstream ess(std::ios::in|std::ios::out);
    // ess << "Epanechnikov" << p << ".dat";
    // std::string es(ess.str());
    // printKernelValues(es, e);

    
  }

  return EXIT_SUCCESS;
}
