/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*
 * TestMeanShiftClustering.cpp
 *
 *  Created on: Mar 9, 2011
 *      Author: vieux
 */


// g++ -g -Wall ~/workspace/imageFeatures/ise++/ise/iseSignalProcessingUtils.cpp iseMeanShiftClustering.cpp TestMeanShiftClustering.cpp -o TestMeanShiftClustering -I/home/vieux/workspace/imageFeatures/ise++/ `pkg-config --cflags --libs opencv girl-2.0` -ltbb


#include <cstdlib>

#include <girl/girlSharedPtr.hpp>
#include <girl/girlToger.hpp>
#include <girl/girlUnorderedSet.hpp>

#include <girlTools/girlToolsImage.hpp>
#include <girlTools/girlToolsDiskBackup.hpp>

#include <opencv2/flann/flann.hpp>

#include <ise/iseHistogram1D.hpp>
#include <ise/iseImageFeatureColorSpace.hpp>
#include <ise/iseRegionFeatureSelector.hpp>
#include <ise/iseRegionSetInterfaceMoments.hpp>
#include <ise/iseRegionSetInterfaceMean.hpp>
#include <ise/iseRegionSetInterfaceStrokeCharacteristics.hpp>
#include <ise/iseRegionSetInterfaceGravityCenter.hpp>
#include <ise/iseRegionSetInterfaceGeometric.hpp>
#include <ise/iseRegionSetInterfaceDomainRunLengthHistogram.hpp>
#include <ise/iseSegmentationContext.hpp>
#include <ise/iseQuantificationThresholderOtsu1T.hpp>
#include <ise/iseQuantificationThresholds.hpp>
#include <ise/iseMeanShiftClustering.hpp>


//#include <boost/timer.hpp>

void
usage(const char* c)
{
  std::cerr << "Usage " << c << "<image> <toger> <meanshift sigma>" << std::endl;
  exit(EXIT_FAILURE);
}


// double*
// computeData(size_t nbRegs, const girl::regionDeque &regs,  size_t &featureDim, ise::SegmentationContext &segContext)
// {

//   featureDim = 2;
//   double *data = new double[nbRegs*featureDim];
//   for (size_t i = 0; i < nbRegs; ++i) {
//     double *ptr = &(data[i*featureDim]);
//     for (size_t j = 0; j < featureDim; ++j)
//       ptr[j] = i;

//   }

//   return data;
// }



void
computeData(cv::Mat &data, const girl::regionDeque &regs, girl::shared_ptr<ise::RegionSetInterface> regSegPtr)
{
  size_t nbSamples = regs.size();
  size_t dim = regSegPtr->getFeatureDim();
  data = cv::Mat::zeros(nbSamples, dim, CV_64F);
  for (size_t i = 0; i < nbSamples; ++i) {
      double *buf = data.ptr<double>(i);
      ise::RegionFeature rf(buf);
      regSegPtr->getRegionFeatures(regs[i], rf);
  }

}

void
computeBBData(cv::Mat &data, const girl::regionDeque &regs)
{
  // size_t nbSamples = regs.size();
  // size_t dim = 4;
  // data = cv::Mat::zeros(nbSamples, dim, CV_64F);
  // for (size_t i = 0; i < nbSamples; ++i) {
  //   double *buf = data.ptr<double>(i);
  //   girl::tools::computeBoundingBox(regs[i],
    
  // }


}



double*
computeData(size_t nbRegs, const girl::regionDeque &regs,  size_t &featureDim, ise::SegmentationContext &segContext, std::vector<cv::Range> &ranges)
{

  size_t maxSize = 0;


  girl::shared_ptr<ise::RegionSetInterfaceStrokeCharacteristics> strokesPtr(
      new ise::RegionSetInterfaceStrokeCharacteristics(&segContext, ise::RegionSetInterface::NONE)
  );

  //double *memForStrokes = new double[strokesPtr->getFeatureDim()];
  maxSize = std::max(maxSize, strokesPtr->getFeatureDim());

  girl::shared_ptr<ise::RegionSetInterfaceGravityCenter> gravPtr(
      new ise::RegionSetInterfaceGravityCenter(&segContext, ise::RegionSetInterface::NONE)
  );

  //double *memForGrav = new double[gravPtr->getFeatureDim()];
  maxSize = std::max(maxSize, gravPtr->getFeatureDim());


  girl::shared_ptr<ise::ImageFeatureColorSpace> colorImgPtr(new ise::ImageFeatureColorSpace(&segContext));
  colorImgPtr->setParameters(ISE_RGB_to_Luv);
  colorImgPtr->compute();

  girl::shared_ptr<ise::RegionSetInterfaceMoments> momentsPtr(new ise::RegionSetInterfaceMoments(&segContext, ise::RegionSetInterface::NONE, colorImgPtr));

  girl::shared_ptr<ise::RegionSetInterfaceMean> meanPtr(new ise::RegionSetInterfaceMean(&segContext, ise::RegionSetInterface::NONE));
  meanPtr->setParameters(momentsPtr);
  //double *memForMean = new double[meanPtr->getFeatureDim()];
  maxSize = std::max(maxSize, meanPtr->getFeatureDim());



  girl::shared_ptr<ise::RegionSetInterfaceDomainRunLengthHistogram> drlPtr(new ise::RegionSetInterfaceDomainRunLengthHistogram(&segContext, ise::RegionSetInterface::NONE));
  maxSize = std::max(maxSize, drlPtr->getFeatureDim());



  girl::shared_ptr<ise::RegionSetInterfaceGeometric> geoPtr(new ise::RegionSetInterfaceGeometric(&segContext, ise::RegionSetInterface::NONE));
  maxSize = std::max(maxSize, geoPtr->getFeatureDim());




  //  featureDim = strokesPtr->getFeatureDim();
  featureDim = 3 + drlPtr->getFeatureDim(); //strokesPtr->getFeatureDim() + geoPtr->getFeatureDim() - 2;
  double *data = new double[nbRegs*featureDim];
  double *dataSav = data;

  // domain run length
  ranges.push_back(cv::Range(0 , 8 ));
  ranges.push_back(cv::Range(8 , 16));
  ranges.push_back(cv::Range(16, 24));
  ranges.push_back(cv::Range(24, 32));

  // color
  ranges.push_back(cv::Range(32,35));


  // stroke
  // ranges.push_back(cv::Range(0,2));
  // ranges.push_back(cv::Range(2,3));
  // ranges.push_back(cv::Range(3,4));
  // ranges.push_back(cv::Range(4,6));
  // ranges.push_back(cv::Range(0,1));
  // ranges.push_back(cv::Range(1,2));
  // ranges.push_back(cv::Range(2,3));
  // ranges.push_back(cv::Range(3,4));
  // // geo
  // ranges.push_back(cv::Range(4,5));
  // ranges.push_back(cv::Range(5,6));
  // ranges.push_back(cv::Range(6,7));
  // ranges.push_back(cv::Range(7,9));
  // ranges.push_back(cv::Range(9,11));
  // ranges.push_back(cv::Range(11,12));
  // ranges.push_back(cv::Range(12,13));



  double *buf = new double[maxSize];
  ise::RegionFeature rf(buf);


  int width = segContext.getToger()->width();
  int height = segContext.getToger()->height();

  girl::regionDeque::const_iterator it = regs.begin();
  const girl::regionDeque::const_iterator itEnd = regs.end();
  for (size_t i = 0; it != itEnd; ++it, ++i) {


      drlPtr->getRegionFeatures(*it, rf);
      for (size_t j = 0; j < 4; ++j) {
          double sum = 0;
          for (size_t k = 0; k < 8; ++k) {
              sum += rf[j*8+k];
          }
          for (size_t k = 0; k < 8; ++k) {
              *data++ = rf[j*8+k] /sum;
          }
      }


      //ise::RegionFeature rfStrokes(memForStrokes);
      // strokesPtr->getRegionFeatures(*it, rf);

      // for (size_t i = 0; i < strokesPtr->getFeatureDim(); ++i) {
      // 	*data++ = rf[i];
      // }
      // // stroke width
      // //  *data++ = rf[3];


      // //ise::RegionFeature rfGrav(memForGrav);
      // // gravPtr->getRegionFeatures(*it, rf);



      // geoPtr->getRegionFeatures(*it, rf);
      // // major and minor axis lengths
      // *data++ = rf[5];
      // *data++ = rf[6];

      // //ise::RegionFeature rfMean(memForMean);
      meanPtr->getRegionFeatures(*it, rf);
      *data++ = rf[0];
      *data++ = rf[1];
      *data++ = rf[2];

      // for (size_t i = 0; i < geoPtr->getFeatureDim() - 2; ++i) {
      //      *data++ = rf[i];
      // }


      //      data[i*featureDim] = rfMean[0];
      //      data[i*featureDim+1] = rfMean[1];
      //      data[i*featureDim+2] = rfMean[2];

      // data[i*featureDim] = rfStrokes[3];
      // // data[i*featureDim] = rfGrav[0];///width;
      // data[i*featureDim+1] = rfMean[0];//rfGrav[1];///height;
  }
  //  delete [] memForGrav;
  //  delete [] memForStrokes;
  //  delete [] memForMean;
  delete [] buf;

  return dataSav;
}

void
loadDataFromFile(const std::string &file, cv::Mat &data)
{

}

static int cpt = 0;
double
intervalRand(int low, int high)
{

  // //  return 10;

  // cpt++;
  // int a = rand();
  // int b = rand();
  // if (cpt%4 == 0)
  //   return 10 + (double)(a-b)/RAND_MAX;
  // else
  //   return 50 + (double)(a-b)/RAND_MAX;

  int x = std::max(rand()%high, low) ;
  // x += low;
  // x = x%high;
  //int x = ((rand()%high) + low) %high;
  assert(x >= low);
  assert(x <= high);
  return static_cast<double>(x);
}


void
printMat(const cv::Mat &mat, std::ostream &out)
{
  for (int i = 0; i < mat.rows; ++i) {
      for (int j = 0; j < mat.cols; ++j) {
          out << mat.at<double>(i, j) << " ";
      }
      out << std::endl;
  }
}

void
loadData(cv::Mat &data)
{
  int nbSamples = 500;
  int nbDim = 2;
  data = cv::Mat(nbSamples, nbDim, CV_64F);

  int x1l = 1;
  int x1h = 10;
  int x2l = 30;
  int x2h = 50;
  int y1l = x1l;
  int y1h = x1h;
  int y2l = x2l;
  int y2h = x2h;

  for (int i = 0; i < nbSamples/2; ++i) {
      double *ptr = data.ptr<double>(i);
      ptr[0] = intervalRand(x1l, x1h);
      ptr[1] = intervalRand(y1l, y1h);
      //      std::cout << ptr[0] << " " << ptr[1] << std::endl;
  }

  for (int i = nbSamples/2; i < nbSamples; ++i) {
      double *ptr = data.ptr<double>(i);
      ptr[0] = intervalRand(x2l, x2h);
      ptr[1] = intervalRand(y2l, y2h);
      //       std::cout << ptr[0] << " " << ptr[1] << std::endl;
  }
}


void
segmenteImage(ise::SegmentationContext &segContext, girl::toger &tog, girl::tools::Image &img)
{

  const int colorSpace = ISE_RGB_to_GRAY;
  const int chan = 0;
  ise::Histogram1D histo(img, colorSpace, chan);
  ise::QuantificationThresholderOtsu1T otsu;
  std::vector<double> thresholds;
  otsu.getThresholds(histo, thresholds);
  ise::QuantificationThresholds quantif(&segContext);
  std::vector<int> levels(thresholds.size());
  for (size_t i = 0; i < thresholds.size(); ++i) {
      levels[i] = thresholds[i];
  }
  quantif.setParams(colorSpace, chan, levels);
  girl::regionDeque regs;
  tog.splitRegion(1, quantif, regs); 
}



struct ClusteringPartitionner {

  ClusteringPartitionner(double dist) : m_dist(dist) {}
  ~ClusteringPartitionner() {}

  bool
  operator()(const std::vector<double> &a, const std::vector<double> &b)
  {
#if CV_MAJOR_VERSION*100+CV_MINOR_VERSION*10+CV_SUBMINOR_VERSION < 231
    const double dist = cvflann::euclidean_dist(a.begin(), a.end(), b.begin(), 0);
#else
    const double dist = cvflann::L2<double>()(a.begin(), b.begin(), a.size());
#endif
    return dist < m_dist;
  }

  double m_dist;
};


struct VotePartitionner
{
  VotePartitionner(std::vector<std::vector<double> > &votes, double t)
  : m_votes(votes), m_t(t) {}

  ~VotePartitionner() {}

  bool
  operator()(const size_t i, const size_t j)
  {
    return m_votes[i][j] > m_t;
  }

  std::vector<std::vector<double> > &m_votes;
  double m_t;

};

void
printRegionsPerClusters(int nbClusters, std::vector<int> &labels, girl::regionDeque &regs)
{
  std::vector<std::deque<int> > clusters(nbClusters);
  for (size_t i = 0; i < labels.size(); ++i) {
      clusters[labels[i]].push_back(regs[i]);
  }

  std::cout << std::endl<< std::endl<< std::endl<< std::endl;
  for (int i = 0; i < nbClusters; ++i) {
      std::cout << "Cluster " << i << ": ";
      for (size_t j = 0; j < clusters[i].size(); ++j) {
          std::cout << clusters[i][j] << " ";
      }
      std::cout << std::endl;
  }

}




void
interactiveSelectData(const girl::regionDeque &regs,
    std::vector<girl::shared_ptr<ise::RegionSetInterface> > &regSets,
		      cv::Mat &data,
		      std::vector<cv::Range> &ranges)
{

  std::cout << "******Interactively select features: type 0 or n/N if you don't want the feature, otherwise it"
      " is selected*****";

  std::vector<bool> mask;
  std::vector<std::string> selected;
  
  cv::Range range(0, 0);
  
  for (size_t i = 0; i < regSets.size(); ++i) {
    range.start = range.end;
    std::vector<std::string> dimDesc;
    regSets[i]->getDimDescription(dimDesc);
    for (size_t j = 0; j < dimDesc.size(); ++j) {
      std::cout << dimDesc[j] << std::endl;
      char c;
      std::cin >> c;
      if (c != '0' && c != 'n' && c != 'N') {
	mask.push_back(true);
	selected.push_back(dimDesc[j]);
	range.end++;
      } else {
	mask.push_back(false);
      }
    }
    if (range.end > range.start)
      ranges.push_back(range);
  }
  
  std::cout << "***** END ********" << std::endl
	    << " selected features: " << std::endl;
  for (size_t i = 0; i < selected.size(); ++i)
    std::cout << selected[i] << std::endl;
  
  ise::RegionFeatureSelector::selectFeatures(regs, regSets, mask, data);
  assert((data.cols) == selected.size());
}

int
main(int argc, char *argv[])
{


  for (int i = 1; i < argc; ++i) {

      girl::tools::Image image(argv[i]);
      girl::toger tog(image.width(), image.height());
      ise::SegmentationContext segContext(&tog, &image);
      std::string sav = argv[i];
      sav += "_otsu1.seg";
      try {
          girl::tools::iDiskBackup reader(sav);
          tog.read(reader);
      } catch(std::exception &e) {
          segmenteImage(segContext, tog, image);
          girl::tools::oDiskBackup backup(sav);
          tog.write(backup);
      }

      girl::regionDeque regs;
      tog.allRegions(regs);



      // REGION FEATURES
      girl::shared_ptr<ise::RegionSetInterfaceStrokeCharacteristics> strokesPtr(
          new ise::RegionSetInterfaceStrokeCharacteristics(&segContext, ise::RegionSetInterface::NONE)
      );

      girl::shared_ptr<ise::RegionSetInterfaceGravityCenter> gravPtr(
          new ise::RegionSetInterfaceGravityCenter(&segContext, ise::RegionSetInterface::NONE)
      );

      girl::shared_ptr<ise::ImageFeatureColorSpace> colorImgPtr(new ise::ImageFeatureColorSpace(&segContext));
      colorImgPtr->setParameters(ISE_RGB_to_Luv);
      colorImgPtr->compute();

      girl::shared_ptr<ise::RegionSetInterfaceMoments> momentsPtr(new ise::RegionSetInterfaceMoments(&segContext, ise::RegionSetInterface::NONE, colorImgPtr));

      girl::shared_ptr<ise::RegionSetInterfaceMean> meanPtr(new ise::RegionSetInterfaceMean(&segContext, ise::RegionSetInterface::NONE));
      meanPtr->setParameters(momentsPtr);


      girl::shared_ptr<ise::RegionSetInterfaceDomainRunLengthHistogram> drlPtr(new ise::RegionSetInterfaceDomainRunLengthHistogram(&segContext, ise::RegionSetInterface::NONE));


      girl::shared_ptr<ise::RegionSetInterfaceGeometric> geoPtr(new ise::RegionSetInterfaceGeometric(&segContext, ise::RegionSetInterface::NONE));



      std::vector<girl::shared_ptr<ise::RegionSetInterface> > regSets;
      regSets.push_back(gravPtr);
      regSets.push_back(strokesPtr);
      regSets.push_back(meanPtr);
      // regSets.push_back(drlPtr);
      regSets.push_back(geoPtr);




      std::vector<int> labels(regs.size(), 0);


      cv::Mat interactiveData, interactiveOutput;
      std::vector<cv::Range> ranges;
      interactiveSelectData(regs, regSets, interactiveData, ranges);
      ise::MeanShiftClustering interactiveClustering;
      interactiveClustering.setRanges(ranges);
      interactiveClustering.filter(interactiveData, interactiveOutput);
      //interactiveClustering.doClustering();
      for (int i = 5; i < 50; ++i ) {
             interactiveClustering.computeLabelsHierarchical(i, interactiveOutput, labels);
             printRegionsPerClusters(i, labels, regs);
         }


      return 0;


      const int nbFeature = 3;
      const int nbClusterings = 10;
      const int total = nbFeature*nbClusterings;
      const double ds[nbClusterings] = {0.05, 0.1, 0.15, 0.2, 0.25, 0.3, 0.35, 0.4, 0.45, 0.5};
      std::vector<std::vector<double> > votes(tog.numRegions(), std::vector<double>(tog.numRegions(), 0));

      cv::Mat colorData, colorOutput;
      computeData(colorData, regs, meanPtr);
      ise::MeanShiftClustering colorClustering;
      colorClustering.compute(colorData, colorOutput);
      for (int i = 2; i < 100; ++i ) {
          colorClustering.computeLabelsHierarchical(i, colorOutput, labels);
          printRegionsPerClusters(i, labels, regs);
      }
      //      for (int i = 0; i < nbClusterings; ++i) {
      //          int nbClusters = colorClustering.computeLabels(ds[i], colorOutput, labels);
      //          for (int i = 1; i < tog.numRegions(); ++i) {
      //              for (int j = 0; j < i; ++j) {
      //                  votes[labels[i]][labels[j]] += 1.0/total;
      //              }
      //          }
      //      }

      cv::Mat drlData, drlOutput;
      computeData(drlData, regs, drlPtr);
      ise::MeanShiftClustering drlClustering;
      drlClustering.compute(drlData, drlOutput);
      for (int i = 2; i < 100; ++i ) {
          drlClustering.computeLabelsHierarchical(i, drlOutput, labels);
          printRegionsPerClusters(i, labels, regs);
      }
      //      for (int i = 0; i < nbClusterings; ++i) {
      //          int nbClusters = drlClustering.computeLabels(ds[i], drlOutput, labels);
      //          for (int i = 1; i < tog.numRegions(); ++i) {
      //              for (int j = 0; j < i; ++j) {
      //                  votes[i][j] += 1.0/total;
      //              }
      //          }
      //      }

      cv::Mat strokesData, strokesOutput;
      computeData(strokesData, regs, strokesPtr);
      ise::MeanShiftClustering strokesClustering;
      strokesClustering.compute(strokesData, strokesOutput);
      for (int i = 2; i < 100; ++i ) {
          strokesClustering.computeLabelsHierarchical(i, strokesOutput, labels);
          printRegionsPerClusters(i, labels, regs);
      }
      //      for (int i = 0; i < nbClusterings; ++i) {
      //          int nbClusters = strokesClustering.computeLabels(ds[i], strokesOutput, labels);
      //          for (int i = 1; i < tog.numRegions(); ++i) {
      //              for (int j = 0; j < i; ++j) {
      //                  votes[i][j] += 1.0/total;
      //              }
      //          }
      //      }


      //      std::vector<size_t> toto(tog.numRegions());
      //      for (size_t i = 0; i < tog.numRegions(); ++i)
      //        toto[i] = i;
      //
      //      std::vector<int> finalLabels;
      //      VotePartitionner vp(votes, 0.5);
      //      int nbClusters = cv::partition(toto, finalLabels, vp);
      //      printRegionsPerClusters(nbClusters, finalLabels, regs);







      //      size_t nbRegs = regs.size();
      //      size_t featureDim;
      //      std::vector<cv::Range> ranges;
      //      double *databuf = computeData(nbRegs, regs, featureDim, segContext, ranges);
      //
      //
      //      cv::Mat data(nbRegs, featureDim, CV_64F, databuf);
      //      std::string savData = argv[i];
      //      savData += "THICKNESS_HUE.dat";
      //      std::ofstream dataFile(savData.c_str());
      //      printMat(data, dataFile);
      //      dataFile.close();
      //
      //      cv::Mat output;
      //      ise::MeanShiftClustering meanShift(data, output);
      //      //  meanShift.setRanges(ranges);
      //      meanShift.doClustering();
      //
      //      // std::vector<std::vector<double> > toPartition(data.rows, std::vector<double>(data.cols, 0));
      //      // for (int i = 0; i < output.rows; ++i) {
      //      //   for (int j = 0; j < output.cols; ++j) {
      //      //     toPartition[i][j] = output.at<double>(i, j);
      //      //   }
      //      // }
      //
      //      std::vector<int> labels;
      //
      //      for (double d = 1; d >= 0; d -= 0.1) {
      //          // ClusteringPartitionner p(d);
      //          // int nbClusters = cv::partition(toPartition, labels, p);
      //          int nbClusters = meanShift.computeLabels(d, output, labels);
      //
      //          std::vector<std::deque<int> > clusters(nbClusters);
      //          for (size_t i = 0; i < labels.size(); ++i) {
      //              clusters[labels[i]].push_back(regs[i]);
      //          }
      //
      //          std::cout << std::endl<< std::endl<< std::endl<< std::endl;
      //          for (int i = 0; i < nbClusters; ++i) {
      //              std::cout << "Cluster " << i << ": ";
      //              for (size_t j = 0; j < clusters[i].size(); ++j) {
      //                  std::cout << clusters[i][j] << " ";
      //              }
      //              std::cout << std::endl;
      //          }
      //      }
      //
      //
      //      //      for (int i = 0; i < output.rows; ++i) {
      //      //	for (int j = 0; j < output.cols; ++j) {
      //      //	  std::cout << data.at<double>(i, j) << ",";
      //      //	}
      //      //	std::cout << "-> ";
      //      //	for (int j = 0; j < output.cols; ++j) {
      //      //	  std::cout << output.at<double>(i, j) << ",";
      //      //	}
      //      //	std::cout << std::endl;
      //      //      }
      //
      //
      //      // assert(output.type() == CV_32S);
      //      // assert(output.isContinuous());
      //
      //      // double min, max;
      //      // cv::Point minLoc, maxLoc;
      //      // cv::minMaxLoc(output, &min, &max/*, &minLoc, &maxLoc*/);
      //      // std::vector<std::deque<int> > clusters(max, std::deque<int>());
      //      // int *buf = output.ptr<int>(0);
      //      // for (size_t i = 0; i < max; ++i) {
      //      // 	int index = buf[i];
      //      // 	clusters[index].push_back(regs[i]);
      //      // }
      //
      //      // for (int i = 0; i < max; ++i) {
      //      // 	std::cout << "Cluster " << i << ": ";
      //      // 	for (size_t j = 0; j < clusters[i].size(); ++j) {
      //      // 	  std::cout << clusters[i][j] << " ";
      //      // 	}
      //      // 	std::cout << std::endl;
      //      // }
      //
      //
      //
      //      // std::vector<cv::Range> ranges;
      //      // ranges.push_back(cv::Range(0, 1));
      //      // ranges.push_back(cv::Range(1, 2));
      //      // meanShift.setRanges(ranges);
      //      // meanShift.doClustering();
      //      // for (double p = 0; p < 20; ++p) {
      //      // 	meanShift.setP(0);
      //      // 	meanShift.doClustering();
      //      // 	std::stringstream ss(std::ios::in|std::ios::out);
      //      // 	ss << "output" << p << ".dat";
      //      // 	std::string s(ss.str());
      //      // 	std::ofstream outputFile(s.c_str());
      //      // 	printMat(output, outputFile);
      //      // 	outputFile.close();
      //      // }
      //
      //
      //      delete [] databuf;

  }

  return EXIT_SUCCESS;

  //  if (argc != 4)
  //     usage(argv[0]);

  //   girl::tools::Image image(argv[1]);

  //   girl::tools::iDiskBackup backup(argv[2]);
  //   float sigma = atof(argv[3]);
  //   girl::toger tog;
  //   tog.read(backup);
  //   assert(tog.width() == image.width() && tog.height() == image.height());

  //   ise::SegmentationContext segContext(&tog, &image);



  //   girl::regionDeque regs;
  //   tog.allRegions(regs);

  //   size_t nbRegs = regs.size();
  //   size_t featureDim;
  //   double *databuf = computeData(nbRegs, regs, featureDim, segContext);
  //   cv::Mat data(nbRegs, featureDim, CV_64F, databuf);
  //   cv::Mat output;
  //   std::ofstream dataFile("data.dat");
  //   printMat(data, dataFile);
  //   dataFile.close();



  //   ise::MeanShiftClustering meanShift(data, output);
  //   meanShift.doClustering();
  //   std::ofstream outputFile("output.dat");
  //   printMat(output, outputFile);
  //   outputFile.close();


  //   return EXIT_SUCCESS;
}
