/*
 * iseMeanShiftRange.cpp
 *
 *  Created on: Mar 7, 2011
 *      Author: vieux
 */

#include <iostream>
#include <string>
#include <sstream>
#include <fstream>
#include "iseMeanShiftRange.hpp"


#ifdef WITH_TBB

#include "tbb/parallel_for.h"
#include "tbb/blocked_range.h"

class MeanShifter {

public:
  
  void operator() ( const tbb::blocked_range<size_t>& r ) const {
    for ( size_t i = r.begin(); i != r.end(); ++i ) {
#ifdef VERBOSE
      std::cout << "MS " << i;
#endif
      cv::Mat ms = cv::Mat::zeros(1, input.cols, CV_64F);
      cv::Mat data = input.row(i);
      cv::Mat outData = output.row(i);
      data.copyTo(outData);
      size_t numIter = 0;
      do {
	m_ms.computeMeanShift(outData, ms);
	outData += ms;
	++numIter;
      } while (cv::norm(ms) >= 0.01 && numIter < 10000);
#ifdef VERBOSE
      std::cout << " (" << numIter << ")" <<  std::endl;
#endif
    }
  }

  MeanShifter(const cv::Mat &i, cv::Mat &o, ise::MeanShiftRange &m):
    input(i), output(o), m_ms(m) {}
  const cv::Mat &input;
  cv::Mat &output;
  ise::MeanShiftRange &m_ms;

};

#endif

namespace ise
{

  MeanShiftRange::MeanShiftRange(const cv::Mat &input, const cv::Mat &ranges, cv::Mat &output)
  : m_input(input),
    m_ranges(ranges)
  {

    assert(input.type() == CV_64F);
    assert(ranges.cols == input.cols);
    assert(ranges.type() == CV_64F);
    output = cv::Mat::zeros(input.size(), input.type());

#ifdef WITH_TBB
    tbb::parallel_for(tbb::blocked_range<size_t>(0, input.rows),
              MeanShifter(input, output, *this));
#else
    for (int i = 0; i < input.rows; ++i) {
#ifdef VERBOSE
      std::cout << "MS " << i;
#endif
      std::stringstream ss(std::ios::in|std::ios::out);
      ss << i << ".dat";
      std::string s(ss.str());
      std::ofstream outfile(s.c_str());
      cv::Mat ms = cv::Mat::zeros(1, input.cols, CV_64F);
      cv::Mat data = m_input.row(i);
      cv::Mat outData = output.row(i);
      data.copyTo(outData);
      size_t numIter = 0;
      do {

	for (int i = 0; i < ms.cols; ++i) {
	  outfile <<  outData.at<double>(0, i) << " ";
	}
	outfile << std::endl;
	computeMeanShift(outData, ms);
	outData += ms;
	++numIter;
      } while (cv::norm(ms) >= 0.01 && numIter < 10000);
#ifdef VERBOSE
      std::cout << " (" << numIter << ")" <<  std::endl;
#endif
      outfile.close();
    }
#endif
    
  }

  MeanShiftRange::~MeanShiftRange()
  {
    // TODO Auto-generated destructor stub
  }

  void
  MeanShiftRange::computeMeanShift(const cv::Mat &input, cv::Mat &ms)
  {
    std::vector<int> indexes;
    rangeSearch(input, m_ranges, indexes);
    for (size_t i = 0; i < indexes.size(); ++i) {
        ms += m_input.row(indexes[i]);
    }
    ms /= indexes.size();
    ms -= input;
  }

  void
  MeanShiftRange::rangeSearch(const cv::Mat &input, const cv::Mat &ranges, std::vector<int> &indexes)
  {

    assert(input.rows == ranges.rows);
    assert(input.rows == 1);
    const double *inputPtr = input.ptr<double>(0);
    const double *rangePtr = ranges.ptr<double>(0);
    for (int i = 0; i < m_input.rows; ++i) {
        bool ok = true;
        const double *ptr = m_input.ptr<double>(i);
        for (int j = 0; j < m_input.cols; ++j) {
            ok &= (inputPtr[j] - rangePtr[j] <= ptr[j] && ptr[j] <= inputPtr[j] + rangePtr[j]);
	    if (!ok)
	      break;
        }
        if (ok)
          indexes.push_back(i);
    }
  }

}










//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
///*
// * iseMeanShiftRange.cpp
// *
// *  Created on: Mar 7, 2011
// *      Author: vieux
// */
//
//#include <iostream>
//#include "iseMeanShiftRange.hpp"
//
//
//namespace ise
//{
//
//  MeanShiftRange::MeanShiftRange(const cv::Mat &input, const cv::Mat &ranges, cv::Mat &output)
//  : m_input(input),
//    m_ranges(ranges)
//  {
//
//    assert(input.type() == CV_64F);
//    assert(ranges.cols == input.cols);
//    assert(ranges.type() == CV_64F);
//    output = cv::Mat::zeros(input.size(), input.type());
//
//    for (int i = 0; i < input.rows; ++i) {
//      std::cout << "MS " << i;
//      cv::Mat ms = cv::Mat::zeros(1, input.cols, CV_64F);
//      cv::Mat data = m_input.row(i);
//      cv::Mat outData = output.row(i);
//      data.copyTo(outData);
//      size_t numIter = 0;
//      do {
//	computeMeanShift(outData, ms);
//            outData += ms;
//            ++numIter;
//        } while (cv::norm(ms) >= 0.01 && numIter < 10000);
//      std::cout << " (" << numIter << ")" <<  std::endl;
//    }
//
//
//  }
//
//  MeanShiftRange::~MeanShiftRange()
//  {
//    // TODO Auto-generated destructor stub
//  }
//
//  void
//  MeanShiftRange::computeMeanShift(const cv::Mat &input, cv::Mat &ms)
//  {
//    std::vector<int> indexes;
//    rangeSearch(input, m_ranges, indexes);
//    for (size_t i = 0; i < indexes.size(); ++i) {
//        ms += m_input.row(indexes[i]);
//    }
//    ms /= indexes.size();
//    ms -= input;
//  }
//
//  void
//  MeanShiftRange::rangeSearch(const cv::Mat &input, const cv::Mat &ranges, std::vector<int> &indexes)
//  {
//
//    assert(input.rows == ranges.rows);
//    assert(input.rows == 1);
//    const double *inputPtr = input.ptr<double>(0);
//    const double *rangePtr = ranges.ptr<double>(0);
//    for (int i = 0; i < m_input.rows; ++i) {
//        bool ok = true;
//        const double *ptr = m_input.ptr<double>(i);
//        for (int j = 0; j < m_input.cols; ++j) {
//            ok &= (inputPtr[j] - rangePtr[j] <= ptr[j] && ptr[j] <= inputPtr[j] + rangePtr[j]);
//        }
//        if (ok)
//          indexes.push_back(i);
//    }
//  }
//
//}
