#include <ise/EnclosingBox.hpp>
#include <ise/EnclosingConvexHull.hpp>
#include <ise/iseAreaEnclosingShapeFeatureVisitor.hpp>
#include <ise/isePerimeterEnclosingShapeFeatureVisitor.hpp>
#include <ise/iseCompacityEnclosingShapeFeatureVisitor.hpp>


#include <ise/iseEnclosingShapeFeatureVisitorRegistry.hpp>

#include <opencv2/core/core.hpp>

#include <iostream>

int main ()
{

    /// Element creation
    cv::Rect rect (0, 0, 100, 100);
    EnclosingBox bbox(rect);


    cv::Point p0 (0, 0);
    cv::Point p1 (0, 1);
    cv::Point p3 (1, 1);
    cv::Point p2 (1, 0);

    std::vector<cv::Point> points;
    points.push_back (p0); points.push_back (p1); points.push_back (p3); points.push_back (p2);

    EnclosingConvexHull hull(points);


    /// test Area
    ise::AreaEnclosingShapeFeatureVisitor areaComputer;
    double bboxArea = bbox.accept (areaComputer);
    if ( bboxArea != 100*100 ) {
        std::cout << __LINE__ << std::endl;
        return 1;
    }

    ise::EnclosingShapeFeatureVisitorRegistry * registry = ise::EnclosingShapeFeatureVisitorRegistry::instance();
    double registryBBoxArea = registry->applyEnclosingShapeFeatureVisitor("AreaEnclosingShapeFeatureVisitor", bbox);

    std::cout << "registryBBoxArea :" << registryBBoxArea << std::endl;

    double hullArea = hull.accept (areaComputer);
    if (hullArea != 1) {
        std::cout << __LINE__  << std::endl;
        return 1;
    }

    /// test Perimeter
    ise::PerimeterEnclosingShapeFeatureVisitor perimeterComputer;
    double bboxPerimeter = bbox.accept (perimeterComputer);
    if (bboxPerimeter != 2*100+2*100 ) {
        std::cout << __LINE__ << std::endl;
        return 1;
    }

    double hullPerimeter = hull.accept (perimeterComputer);
    if (hullPerimeter != 4) {
        std::cout << __LINE__  << std::endl;
        return 1;
    }

    /// test Compacity
    ise::CompacityEnclosingShapeFeatureVisitor compacityComputer;
    if (bbox.accept (compacityComputer) != (bboxPerimeter*bboxPerimeter)/bboxArea ) {
        std::cout << __LINE__ << bbox.accept (compacityComputer) <<  std::endl;
        return 1;
    }

    if (hull.accept (compacityComputer) != (hullPerimeter*hullPerimeter)/hullArea) {
        std::cout << __LINE__  << hull.accept (compacityComputer)<<std::endl;
        return 1;
    }



    return 0;
}
