/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <getopt.h>
#include <cstdio>
#include <cstdlib>
#include <girl/girlTimer.hpp>

#include <iostream>
#include <fstream>

#include <girl/girlToger.hpp>
#include <girlTools/girlToolsImage.hpp>
//#include <girlTools/girlToolsPnmCodec.hpp>
//#include <girlTools/girlToolsPngCodec.hpp>
//#include <ise/iseRegionSet.hpp>
#include <ise/iseRegionTools.hpp>
#include <ise/iseSegmentationContext.hpp>
//#include <ise/iseMergeRemoveDetailsAlgorithm.hpp>
//#include <ise/iseMergeDefaultAlgorithm.hpp>
//#include <ise/iseMergeAlgorithmXX.hpp>
//#include <ise/iseMergeRemoveAlgorithm.hpp>
//#include <ise/iseMergeRemoveDetailsAlgorithm.hpp>
#include <ise/iseQuantification.hpp>
#include <ise/iseQuantificationUniform.hpp>
#include <ise/iseQuantificationCAG.hpp>
#include <ise/iseSplitAlgorithmDefault.hpp>
#include <ise/iseSplitAlgorithmBFRecursive.hpp>
#include <ise/iseSplitAlgorithmDFRecursive.hpp>

//#include <ise/iseSplitMergeAlgorithm.hpp>
#include <ise/iseSplitDriverDefault.hpp>
#include <ise/iseSplitDriverRecursiveDefault.hpp>
#include <ise/iseArrayHistogram.hpp>

#include <ise/iseMergeAlgorithmDefault.hpp>
#include <ise/iseMergeDriverSizeFiltering.hpp>

#include <girlTools/girlToolsDiskBackup.hpp>
#include <girlTools/girlToolsMemoryBackup.hpp>
#include <girlTools/girlToolsZBackup.hpp>

static girl::tools::Image img;
static int width;
static int height;

static girl::toger *tog;

static std::string inFilename;
static std::string outFilename = "out.segz";

//static ise::RegionSet *regSet = NULL;


static ise::SegmentationContext *segContext = NULL;
//static iseMergeAlgorithm *mergeAlgo = NULL;
static ise::Quantification *quantif = NULL;
static ise::SplitAlgorithm *splitAlgo = NULL;


int 
main(int argc, char *argv[])
{
  if (argc < 2) {
    std::cerr<<"Usage: "<<argv[0]<<" imageFilename [outputFilename [compressionLevel]]"<<std::endl;
    exit(EXIT_FAILURE);
  }

  if (argc >= 2) 
    inFilename = argv[1];

  if (argc >= 3) 
    outFilename = argv[2];

  int compressionLevel = 6; //in [0; 9]
  if (argc >= 4) 
    compressionLevel = atoi(argv[3]);
  


  /* image loading */
  img = girl::tools::Image(inFilename);
  if (img.isNull()) {
    std::cerr<<"Error: Unable to load image file: "<<inFilename<<std::endl;
    exit(EXIT_FAILURE);
  }
  
  width = img.width();
  height = img.height();

  /* Initialization of representation */
  tog = new girl::toger(width, height);
  //regSet = new ise::RegionSet();
  
  segContext = new ise::SegmentationContext(tog, &img, NULL);


  ise::Histogram *histo = new ise::ArrayHistogram(0, 0, 0);

  quantif = new ise::QuantificationCAG(segContext, histo, 0);
  static_cast<ise::QuantificationCAG *>(quantif)->setThreshold(0.3);

  
  splitAlgo = new ise::SplitAlgorithmBFRecursive(segContext);
  ise::SplitDriver *driver = new ise::SplitDriverDefault(segContext);

  girl::regionDeque regions2split;
  regions2split.push_back(1);
  girl::regionDeque createdRegions;

  girl::Timer t;
  t.start();

  splitAlgo->apply(regions2split, *driver, *quantif, 0, createdRegions);  

  const size_t time0 = t.stop();

  std::cerr<<"split: "<<createdRegions.size()<<" regions"<<std::endl;

  std::cerr<<"time split: "<<time0<<"ms"<<std::endl;

  const size_t numRegions = tog->numRegions();
  if (numRegions != createdRegions.size())
    std::cerr<<"Error: in splitAlgo->apply() ! createdRegions.size()="<<createdRegions.size()<<" != tog->numRegions()="<<numRegions<<std::endl;
  

  t.start();

  tog->shrink();

  const size_t time1 = t.stop();
  std::cerr<<"time shrink: "<<time1<<"ms"<<std::endl;

  if (numRegions != tog->numRegions())
    std::cerr<<"Error: in tog->shrink() ! before: "<<numRegions<<" regions ; after: "<<tog->numRegions()<<" regions"<<std::endl;


  //normal write
  {
    girl::tools::oDiskBackup w(outFilename);
    girl::tools::oZBackup wz(w, compressionLevel);

    t.start();

    bool writeOk = tog->write(wz, wz.flags());
    writeOk &= wz.end();
    gettimeofday(&t1, NULL);
    if (! writeOk) {
      std::cerr<<"Error: unable to write toger !"<<std::endl;
      exit(EXIT_FAILURE);
    }
    std::cerr<<"complete write"<<std::endl;
    const size_t timeWrite = t.stop();
    std::cerr<<" time write: "<<timeWrite<<"ms"<<std::endl;


    girl::tools::iDiskBackup r(outFilename);
    girl::tools::iZBackup rz(r);

    t.start();

    bool readOk = tog->read(rz, rz.flags());

    const size_t timeRead = t.stop();
    if (! readOk) {
      std::cerr<<"Error: unable to read toger !"<<std::endl;
      exit(EXIT_FAILURE);
    }
    if (numRegions != tog->numRegions())
      std::cerr<<"Error: in tog->write()/read() ! before: "<<numRegions<<" regions ; after: "<<tog->numRegions()<<" regions"<<std::endl;
  
    std::cerr<<" time read: "<<timeRead<<"ms"<<std::endl;

    const double timeWriteRead = timeWrite + timeRead;
    std::cerr<<" time write+read: "<<timeWriteRead<<std::endl;

    std::ifstream file(outFilename.c_str(), std::ios_base::in | std::ios_base::binary);
    file.seekg(0, std::ios::end);
    const size_t fileSize = file.tellg();
    std::cerr<<" file size: "<<fileSize<<"octets = "<<fileSize/1024.0<<"Ko = "<<fileSize/1024.0/1024.0<<"Mo"<<std::endl;
  }
  

  //fast write (without taking care of labels/marks)
  {
    girl::tools::oDiskBackup w(outFilename);
    girl::tools::oZBackup wz(w, compressionLevel);

    t.start();

    bool writeOk = tog->write(wz, wz.flags()|girl::io::NO_LABELS_MARKS_CARE);
    writeOk &= wz.end();
    gettimeofday(&t1, NULL);
    if (! writeOk) {
      std::cerr<<"Error: unable to write toger !"<<std::endl;
      exit(EXIT_FAILURE);
    }
    std::cerr<<"write without taking care of labels/marks"<<std::endl;
    const size_t timeWrite = t.stop();
    std::cerr<<" time write: "<<timeWrite<<"ms"<<std::endl;


    girl::tools::iDiskBackup r(outFilename);
    girl::tools::iZBackup rz(r);

    t.start();

    bool readOk = tog->read(rz, rz.flags()|girl::io::NO_LABELS_MARKS_CARE);

    const size_t timeRead = t.stop();
    if (! readOk) {
      std::cerr<<"Error: unable to read toger !"<<std::endl;
      exit(EXIT_FAILURE);
    }
    if (numRegions != tog->numRegions())
      std::cerr<<"Error: in tog->write()/read() ! before: "<<numRegions<<" regions ; after: "<<tog->numRegions()<<" regions"<<std::endl;
  

    std::cerr<<" time read: "<<timeRead<<"ms"<<std::endl;

    const double timeWriteRead = timeWrite + timeRead;
    std::cerr<<" time write+read: "<<timeWriteRead<<std::endl;

    std::ifstream file(outFilename.c_str(), std::ios_base::in | std::ios_base::binary);
    file.seekg(0, std::ios::end);
    const size_t fileSize = file.tellg();
    std::cerr<<" file size: "<<fileSize<<"octets = "<<fileSize/1024.0<<"Ko = "<<fileSize/1024.0/1024.0<<"Mo"<<std::endl;
  }

  delete driver;
  //delete histo;

  delete splitAlgo;
  delete quantif;
  delete segContext;
  
  //delete regSet;
  delete tog;


  return EXIT_SUCCESS;
}

