/*
  Copyright (C) 2013 Boris Mansencal  boris.mansencal@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <cstdio>
#include <cstdlib>
#include <girl/girlTimer.hpp>

#include <iostream>
#include <fstream>

#include <girl/girlToger.hpp>
#include <girlTools/girlToolsImage.hpp>

#include <opencv2/highgui/highgui.hpp>

#include <ise/iseConversion.hpp>
#include <ise/iseImageFeatureColorSpace.hpp>
#include <ise/iseColorimetricMomentsComputer.hpp>
#include <ise/iseRegionSetInterfaceMoments.hpp>


/*
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  TODO: This should be a cpp-unit test !!!

*/




/**/

//TODO : should be available in ise with girl::tools::Image !

class MatSplitter : public girl::toger::sameRegion
{
public:
  MatSplitter(const cv::Mat &img)
    : m_img(img)
  {
    assert(img.type() == CV_8UC3);
  }

  virtual bool same(const girl::point &p, girl::direction dir)
  {
    girl::point p2 = p;
    p2.move(dir);
    return m_img.at<cv::Vec3b>(p.y(), p.x()) == m_img.at<cv::Vec3b>(p2.y(), p2.x()); //whatever BGR or RGB
  }

protected:
  const cv::Mat &m_img;
};

static
void
getTogerFromImageLabel(const cv::Mat &img, girl::toger &tog, girl::regionDeque &regs)
{
  assert(! img.empty());
  MatSplitter sp(img);
  tog = girl::toger(img.cols, img.rows);

  tog.splitRegion(1, sp, regs);
}

/**/



static
void
P_testCC(girl::tools::Image &img, girl::toger &tog, const girl::regionDeque &regs, int colorSpace)
{
  ise::SegmentationContext segContext(&tog, &img);

  girl::shared_ptr<ise::ImageFeatureColorSpace> ifcs(new ise::ImageFeatureColorSpace(&segContext, colorSpace));
  ifcs->compute();
  const girl::shared_ptr<girl::tools::ImageFeature> imgF = ifcs->getImageFeature();

  segContext.addImageFeatureComputer(ifcs);
  

  girl::shared_ptr<ise::RegionSetInterfaceMoments> rsimts(new ise::RegionSetInterfaceMoments(&segContext, ise::RegionSetInterface::ALWAYS, ifcs));
  segContext.addRegionSet(rsimts);

  std::vector<ise::RegionFeature_Data_Type> rfBuffer(rsimts->getFeatureDim(), 0);
  ise::RegionFeature rf(&rfBuffer[0]);



  double moments[7];
  double *m0 = &moments[0];
  double *m1 = &moments[1];
  double *m2 = &moments[4];
  ise::MomentsComputerIF mcIF(imgF.get(), m0, m1, m2);

  double moments_b[7];
  double *m0_b = &moments_b[0];
  double *m1_b = &moments_b[1];
  double *m2_b = &moments_b[4];

  ise::ColorimetricMomentsComputerCS mc(img, colorSpace);
  

  size_t i=0;
  for (girl::regionDeque::const_iterator it = regs.begin(); it!=regs.end(); ++it, ++i) {

    ise::MomentsComputerIF mcIF_b(imgF.get(), m0_b, m1_b, m2_b);
    ise::ColorimetricMomentsComputerCS mc_b(img, colorSpace);

    mcIF.clear();
    mc.clear();

    const girl::region r = *it;
    
    tog.iterateOnRegionDomain(r, mc);
    tog.iterateOnRegionDomain(r, mc_b);

    tog.iterateOnRegionDomain(r, mcIF);
    tog.iterateOnRegionDomain(r, mcIF_b);

    if (mc.m_m0 != mc_b.m_m0 || mc.m_m1[0] != mc_b.m_m1[0] || mc.m_m1[1] != mc_b.m_m1[1] || mc.m_m1[2] != mc_b.m_m1[2] || mc.m_m2[0] != mc_b.m_m2[0] || mc.m_m2[1] != mc_b.m_m2[1] || mc.m_m2[2] != mc_b.m_m2[2]) {

      std::cerr<<"Error: ise::ColorimetricMomentsComputer::clear() is not working (for colorSpace="<<colorSpace<<") !\n";
      exit(EXIT_FAILURE);
    } 

    if (*m0 != *m0_b || m1[0] != m1_b[0] || m1[1] != m1_b[1] || m1[2] != m1_b[2] || m2[0] != m2_b[0] || m2[1] != m2_b[1] || m2[2] != m2_b[2]) {
      std::cerr<<"Error: ise::ColorimetricMomentsComputerIF::clear() is not working (for colorSpace="<<colorSpace<<") !\n";
      exit(EXIT_FAILURE);
    } 

    if (mc.m_m0 != *m0_b || mc.m_m1[0] != m1_b[0] || mc.m_m1[1] != m1_b[1] || mc.m_m1[2] != m1_b[2] || mc.m_m2[0] != m2_b[0] || mc.m_m2[1] != m2_b[1] || mc.m_m2[2] != m2_b[2]) {
      std::cerr<<"Error: difference in computed moments between ise::ColorimetricMomentsComputerCS & ise::ColorimetricMomentsComputerIF for "<<i<<"th region : "<<*it<<" (for colorSpace="<<colorSpace<<")  !\n";
      std::cerr<<mc.m_m0<<" vs "<<*m0_b<<"\n";
      std::cerr<<mc.m_m1[0]<<" vs "<<m1_b[0]<<"\n";
      std::cerr<<mc.m_m1[1]<<" vs "<<m1_b[1]<<"\n";
      std::cerr<<mc.m_m1[2]<<" vs "<<m1_b[2]<<"\n";
      std::cerr<<mc.m_m2[0]<<" vs "<<m2_b[0]<<"\n";
      std::cerr<<mc.m_m2[1]<<" vs "<<m2_b[1]<<"\n";
      std::cerr<<mc.m_m2[2]<<" vs "<<m2_b[2]<<"\n";
      exit(EXIT_FAILURE);
    } 
    


    rsimts->getRegionFeatures(r, rf);

    if (*m0 != rf[0] || m1[0] != rf[1] || m1[1] != rf[2] || m1[2] != rf[3] || m2[0] != rf[4] || m2[1] != rf[5] || m2[2] != rf[6]) {
      std::cerr<<"Error: difference in computed moments between ise::ColorimetricMomentsComputerCS & RegionSetInterfaceMoments for "<<i<<"th region : "<<*it<<"  (for colorSpace="<<colorSpace<<")!\n";
      std::cerr<<rf[0]<<" vs "<<*m0<<"\n";
      std::cerr<<rf[1]<<" vs "<<m1[0]<<"\n";
      std::cerr<<rf[2]<<" vs "<<m1[1]<<"\n";
      std::cerr<<rf[3]<<" vs "<<m1[2]<<"\n";
      std::cerr<<rf[4]<<" vs "<<m2[0]<<"\n";
      std::cerr<<rf[5]<<" vs "<<m2[1]<<"\n";
      std::cerr<<rf[6]<<" vs "<<m2[2]<<"\n";
      exit(EXIT_FAILURE);
    }
    
  }


}


int 
main(int argc, char *argv[])
{

  std::string inFilename;
  std::string inLabelFilename;
  if (argc >= 2) {
    inFilename = argv[1];
    inLabelFilename = inFilename;
    if (argc >= 3) {
      inLabelFilename = argv[2];
    }
  }
  else {
    std::cerr<<"Usage: "<<argv[0]<<" imageFilename [labelImageFilename]"<<std::endl;
    exit(EXIT_FAILURE);
  }


  cv::Mat cvimg2 = cv::imread(inLabelFilename);
  if (cvimg2.empty()) {
    std::cerr<<"Error: Unable to load image file: "<<inLabelFilename<<" via OpenCV."<<std::endl;
    exit(EXIT_FAILURE);
  }
  girl::toger tog; 
  girl::regionDeque regs;
  getTogerFromImageLabel(cvimg2, tog, regs);
  
  girl::tools::Image img = girl::tools::Image(inFilename);
  if (img.isNull()) {
    std::cerr<<"Error: Unable to load image file: "<<inFilename<<std::endl;
    exit(EXIT_FAILURE);
  }
  
  


  ise::SegmentationContext segContext(&tog, &img);


  
  girl::shared_ptr<ise::ImageFeatureColorSpace> ifcs(new ise::ImageFeatureColorSpace(&segContext, ISE_RGB_to_RGB));
  ifcs->compute();
  const girl::shared_ptr<girl::tools::ImageFeature> imgF = ifcs->getImageFeature();

  segContext.addImageFeatureComputer(ifcs);
  

  girl::shared_ptr<ise::RegionSetInterfaceMoments> rsimts(new ise::RegionSetInterfaceMoments(&segContext, ise::RegionSetInterface::ALWAYS, ifcs));
  segContext.addRegionSet(rsimts);

  std::vector<ise::RegionFeature_Data_Type> rfBuffer(rsimts->getFeatureDim(), 0);
  ise::RegionFeature rf(&rfBuffer[0]);



  double moments[7];
  double *m0 = &moments[0];
  double *m1 = &moments[1];
  double *m2 = &moments[4];
  ise::MomentsComputerIF mcIF(imgF.get(), m0, m1, m2);

  double moments_b[7];
  double *m0_b = &moments_b[0];
  double *m1_b = &moments_b[1];
  double *m2_b = &moments_b[4];

  ise::ColorimetricMomentsComputer mc(img);
  

  size_t i=0;
  for (girl::regionDeque::const_iterator it = regs.begin(); it!=regs.end(); ++it, ++i) {

    ise::MomentsComputerIF mcIF_b(imgF.get(), m0_b, m1_b, m2_b);
    ise::ColorimetricMomentsComputer mc_b(img);

    mcIF.clear();
    mc.clear();

    const girl::region r = *it;
    
    tog.iterateOnRegionDomain(r, mc);
    tog.iterateOnRegionDomain(r, mc_b);

    tog.iterateOnRegionDomain(r, mcIF);
    tog.iterateOnRegionDomain(r, mcIF_b);

    if (mc.m_m0 != mc_b.m_m0 || mc.m_m1[0] != mc_b.m_m1[0] || mc.m_m1[1] != mc_b.m_m1[1] || mc.m_m1[2] != mc_b.m_m1[2] || mc.m_m2[0] != mc_b.m_m2[0] || mc.m_m2[1] != mc_b.m_m2[1] || mc.m_m2[2] != mc_b.m_m2[2]) {

      std::cerr<<"Error: ise::ColorimetricMomentsComputer::clear() is not working !\n";
      exit(EXIT_FAILURE);
    } 

    if (*m0 != *m0_b || m1[0] != m1_b[0] || m1[1] != m1_b[1] || m1[2] != m1_b[2] || m2[0] != m2_b[0] || m2[1] != m2_b[1] || m2[2] != m2_b[2]) {
      std::cerr<<"Error: ise::ColorimetricMomentsComputerIF::clear() is not working !\n";
      exit(EXIT_FAILURE);
    } 

    if (mc.m_m0 != *m0_b || mc.m_m1[0] != m1_b[0] || mc.m_m1[1] != m1_b[1] || mc.m_m1[2] != m1_b[2] || mc.m_m2[0] != m2_b[0] || mc.m_m2[1] != m2_b[1] || mc.m_m2[2] != m2_b[2]) {
      std::cerr<<"Error: difference in computed moments between ise::ColorimetricMomentsComputer & ise::ColorimetricMomentsComputerIF for "<<i<<"th region : "<<*it<<"   !\n";
      std::cerr<<mc.m_m0<<" vs "<<*m0_b<<"\n";
      std::cerr<<mc.m_m1[0]<<" vs "<<m1_b[0]<<"\n";
      std::cerr<<mc.m_m1[1]<<" vs "<<m1_b[1]<<"\n";
      std::cerr<<mc.m_m1[2]<<" vs "<<m1_b[2]<<"\n";
      std::cerr<<mc.m_m2[0]<<" vs "<<m2_b[0]<<"\n";
      std::cerr<<mc.m_m2[1]<<" vs "<<m2_b[1]<<"\n";
      std::cerr<<mc.m_m2[2]<<" vs "<<m2_b[2]<<"\n";
      exit(EXIT_FAILURE);
    } 
    


    rsimts->getRegionFeatures(r, rf);

    if (*m0 != rf[0] || m1[0] != rf[1] || m1[1] != rf[2] || m1[2] != rf[3] || m2[0] != rf[4] || m2[1] != rf[5] || m2[2] != rf[6]) {
      std::cerr<<"Error: difference in computed moments between ise::ColorimetricMomentsComputer & RegionSetInterfaceMoments for "<<i<<"th region : "<<*it<<" !\n";
      std::cerr<<rf[0]<<" vs "<<*m0<<"\n";
      std::cerr<<rf[1]<<" vs "<<m1[0]<<"\n";
      std::cerr<<rf[2]<<" vs "<<m1[1]<<"\n";
      std::cerr<<rf[3]<<" vs "<<m1[2]<<"\n";
      std::cerr<<rf[4]<<" vs "<<m2[0]<<"\n";
      std::cerr<<rf[5]<<" vs "<<m2[1]<<"\n";
      std::cerr<<rf[6]<<" vs "<<m2[2]<<"\n";
      exit(EXIT_FAILURE);
    }
    
  }


  P_testCC(img, tog, regs, ISE_RGB_to_RGB);
  P_testCC(img, tog, regs, ISE_RGB_to_Lab);
  P_testCC(img, tog, regs, ISE_RGB_to_Luv);


  std::cerr<<"Ok.\n";

  return 0;
}
