#include <cstdlib>
#include <string>
#include <iostream>


#include <ise/iseImageFeatureTexton.hpp>
#include <ise/iseRegionSetInterfaceDistribution.hpp>
#include <ise/iseRegularGridPartitionner.hpp>
#include <ise/iseSplitAlgorithmDefault.hpp>
#include <ise/iseSplitDriverDefault.hpp>
#include <ise/iseRegionUnaryFunctionWithRegSetIndexOfMax.hpp>

#include <opencv2/highgui/highgui.hpp>


using namespace std;
int
main(int argc, char **argv)
{


  string imageName = argv[1];


  girl::tools::Image image(imageName);
  girl::toger tog(image.width(), image.height());

  ise::SegmentationContext segContext(&tog, &image);

  ise::RegularGridPartitionner grid(&segContext, 1, 1);

  ise::SplitAlgorithmDefault splitAlgo(&segContext);
  ise::SplitDriverDefault splitDriver(&segContext);

  girl::regionDeque toSplit;
  toSplit.push_back(1);
  girl::regionDeque createdRegs;
  splitAlgo.apply(toSplit, splitDriver, grid, 0, createdRegs);

  int numReg = tog.numRegions();
  assert(numReg == (image.width()*image.height()));


  girl::shared_ptr<ise::ImageFeatureTexton> textonPtr(new ise::ImageFeatureTexton(&segContext));
  int wndSize = 3;
  textonPtr->setParameters(wndSize);
  textonPtr->compute();
  
  girl::shared_ptr<ise::RegionSetInterfaceDistribution> distribPtr(new ise::RegionSetInterfaceDistribution(&segContext, ise::RegionSetInterfaceDistribution::DEFAULT_CACHE_STRATEGY, textonPtr));
  distribPtr->setParameters(10, cv::KMEANS_PP_CENTERS, textonPtr);
  distribPtr->setCacheState(ise::RegionSetInterface::ALWAYS);


  std::vector<ise::RegionFeature_Data_Type> v(distribPtr->getFeatureDim());
  ise::RegionFeature rf(&v[0]);
  ise:: RegionUnaryFunctionWithRegSetIndexOfMax indexOfMax(&segContext);
  indexOfMax.setParameters(distribPtr);

  cv::Mat m = cv::Mat::zeros(image.height(), image.width(), CV_8U);
  int gl = 255/distribPtr->getFeatureDim();
  for (girl::regionDeque::const_iterator it = createdRegs.begin();
       it != createdRegs.end(); ++it) {
    distribPtr->getRegionFeatures(*it, rf);
    int idOfMax = indexOfMax(*it);
    std::vector<girl::point> scan;
    tog.regionDomain(*it, scan);
    m.at<unsigned char>(scan[0].y(), scan[0].x()) = idOfMax*gl;
  }

  cv::namedWindow("INDEXOFMAX", 0);
  cv::imshow("INDEXOFMAX", m);
  cv::waitKey(0);
  cv::Mat lbl = distribPtr->getLabelsImage().reshape(1, image.height());
  lbl *= gl;
  cv::Mat lblVisu;
  cv::convertScaleAbs(lbl, lblVisu);
  cv::namedWindow("LABELS", 0);
  cv::imshow("LABELS",lblVisu);
  cv::waitKey(0);


  return EXIT_SUCCESS;
}
