/*
  Copyright (C) 2012 Boris Mansencal  boris.mansencal@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <cstdio>
#include <cstdlib>
#include <girl/girlTimer.hpp>

#include <iostream>
#include <fstream>

#include <girl/girlToger.hpp>
#include <girlTools/girlToolsImage.hpp>

#include <ise/iseConversion.hpp>
#include <ise/iseImageFeatureColorSpace.hpp>



int 
main(int argc, char *argv[])
{

  std::string inFilename;
  if (argc >= 2) 
    inFilename = argv[1];
  else {
    std::cerr<<"Usage: "<<argv[0]<<" imageFilename"<<std::endl;
    exit(EXIT_FAILURE);
  }


  /* image loading */
  girl::tools::Image img = girl::tools::Image(inFilename);
  if (img.isNull()) {
    std::cerr<<"Error: Unable to load image file: "<<inFilename<<std::endl;
    exit(EXIT_FAILURE);
  }
  
  ise::SegmentationContext segContext(NULL, &img);


  {
    ise::ImageFeatureColorSpace ifcs(&segContext, ISE_RGB_to_RGB);

    girl::Timer t;
    t.start();

    ifcs.compute();

    size_t time0 = t.stop();

    std::cerr<<"ImageFeatureColorSpace RGB time to compute: "<<time0<<"ms"<<std::endl;
  }
  {
    ise::ImageFeatureColorSpace ifcs(&segContext, ISE_RGB_to_YUV);

    girl::Timer t;
    t.start();

    ifcs.compute();

    size_t time0 = t.stop();

    std::cerr<<"ImageFeatureColorSpace YUV time to compute: "<<time0<<"ms"<<std::endl;
  }
  {
    ise::ImageFeatureColorSpace ifcs(&segContext, ISE_RGB_to_Lab);

    girl::Timer t;
    t.start();

    ifcs.compute();

    size_t time0 = t.stop();

    std::cerr<<"ImageFeatureColorSpace Lab time to compute: "<<time0<<"ms"<<std::endl;
  }
  {
    ise::ImageFeatureColorSpace ifcs(&segContext, ISE_RGB_to_Luv);

    girl::Timer t;
    t.start();

    ifcs.compute();

    size_t time0 = t.stop();

    std::cerr<<"ImageFeatureColorSpace Luv time to compute: "<<time0<<"ms"<<std::endl;
  }
  {
    ise::ImageFeatureColorSpace ifcs(&segContext, ISE_RGB_to_GRAY);

    girl::Timer t;
    t.start();

    ifcs.compute();

    size_t time0 = t.stop();

    std::cerr<<"ImageFeatureColorSpace GRAY time to compute: "<<time0<<"ms"<<std::endl;
  }
  
  return 0;
}
