/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <getopt.h>
#include <cstdio>
#include <cstdlib>
#include <girl/girlTimer.hpp>

#include <iostream>
#include <fstream>

#include <girl/girlToger.hpp>
#include <girlTools/girlToolsImage.hpp>
//#include <girlTools/girlToolsPnmCodec.hpp>
//#include <girlTools/girlToolsPngCodec.hpp>
//#include <girlTools/girlToolsJpgCodec.hpp>
//#include <ise/iseRegionSet.hpp>
#include <ise/iseRegionTools.hpp>
#include <ise/iseSegmentationContext.hpp>
//#include <ise/iseMergeRemoveDetailsAlgorithm.hpp>
//#include <ise/iseMergeDefaultAlgorithm.hpp>
//#include <ise/iseMergeAlgorithmXX.hpp>
//#include <ise/iseMergeRemoveAlgorithm.hpp>
//#include <ise/iseMergeRemoveDetailsAlgorithm.hpp>
#include <ise/iseQuantification.hpp>
#include <ise/iseQuantificationUniform.hpp>
#include <ise/iseQuantificationCAG.hpp>
#include <ise/iseSplitAlgorithmDefault.hpp>
#include <ise/iseSplitAlgorithmBFRecursive.hpp>
#include <ise/iseSplitAlgorithmDFRecursive.hpp>

//#include <ise/iseSplitMergeAlgorithm.hpp>
#include <ise/iseSplitDriverDefault.hpp>
#include <ise/iseSplitDriverRecursiveDefault.hpp>
#include <ise/iseArrayHistogram.hpp>

#include <ise/iseMergeAlgorithmDefault.hpp>
#include <ise/iseMergeAlgorithmUntil.hpp>
#include <ise/iseMergeDriverSizeFiltering.hpp>

#include <girlTools/girlToolsDiskBackup.hpp>
#include <girlTools/girlToolsMemoryBackup.hpp>
#include <girlTools/girlToolsZBackup.hpp>

static girl::tools::Image img;
static int width;
static int height;

static girl::toger *tog;

static std::string inFilename;

//static ise::RegionSet *regSet = NULL;


static ise::SegmentationContext *segContext = NULL;
//static iseMergeAlgorithm *mergeAlgo = NULL;
static ise::Quantification *quantif = NULL;
static ise::SplitAlgorithm *splitAlgo = NULL;

int 
main(int argc, char *argv[])
{

  if (argc >= 2) 
    inFilename = argv[1];
  else {
    std::cerr<<"Usage: "<<argv[0]<<" imageFilename"<<std::endl;
    exit(EXIT_FAILURE);
  }


  /* image loading */
  img = girl::tools::Image(inFilename);
  if (img.isNull()) {
    std::cerr<<"Error: Unable to load image file: "<<inFilename<<std::endl;
    exit(EXIT_FAILURE);
  }
  
  width = img.width();
  height = img.height();

  /* Initialization of representation */
  tog = new girl::toger(width, height);
  //regSet = new ise::RegionSet();
  
  segContext = new ise::SegmentationContext(tog, &img, NULL);

  /*
  //creer imageFeatureColor (RGB ou autre)
  // ajouter au segcontext
  // TODO
  // creer un regionFeatureComputer moment
  RegionFeatureComputerMoment momentComputer(segContext);
  momentComputer.setParameters(imageFeature.uri());

  girl::shared_ptr<ise::RegionSetInterface> regSet =
  RegionFactoryHandler::instance()->newRegionSet(momentComputer);

  segContext->addRegionSet(regSet);

  mergeAlgo->setParameters(thresholds, regSet->uri());

  */






  ise::Histogram *histo = new ise::ArrayHistogram(0, 0, 0);

  quantif = new ise::QuantificationCAG(segContext, histo, 0);
  static_cast<ise::QuantificationCAG *>(quantif)->setThreshold(0.3);

  
  splitAlgo = new ise::SplitAlgorithmBFRecursive(segContext);
  ise::SplitDriver *driver = new ise::SplitDriverDefault(segContext);

  girl::regionDeque regions2split;
  regions2split.push_back(1);
  girl::regionDeque createdRegions;

  girl::Timer t;
  t.start();

  splitAlgo->apply(regions2split, *driver, *quantif, 0, createdRegions);  

  size_t time0 = t.stop();

  std::cerr<<"after split: "<<createdRegions.size()<<" regions"<<std::endl;

  std::cerr<<"time split: "<<time0<<"ms"<<std::endl;

  const size_t numRegions = tog->numRegions();
  if (numRegions != createdRegions.size())
    std::cerr<<"Error: in splitAlgo->apply() ! createdRegions.size()="<<createdRegions.size()<<" != tog->numRegions()="<<numRegions<<std::endl;
  

  ise::MergeAlgorithm *mergeAlgo = new ise::MergeAlgorithmDefault(segContext); 
  //ise::MergeAlgorithm *mergeAlgo = new ise::MergeAlgorithmUntil(segContext); 


  ise::MergeDriverSizeFiltering *mergeDriver = new ise::MergeDriverSizeFiltering(segContext);


  const int maxSize = 2; //in pixels
  //for (int s=1; s<maxSize; ++s) {
  int s=1;
  {

    girl::toger tog2 = *tog;
    segContext->setToger(&tog2);


    /*
 setParameters(double minAreaThreshold, 
			       double maxAreaThreshold, 
			       double colorDistanceThreshold,
			       const std::vector<std::string> &regSetUri);
     */

    const double minAreaThreshold = s / static_cast<double>(std::max(tog2.width(), tog2.height()));
    const double maxAreaThreshold = maxSize / static_cast<double>(std::max(tog2.width(), tog2.height())) ;
    const double colorDistanceThreshold = 0.5;
    //const int colorSpaceConversion = ISE_RGB_to_RGB;
    //const std::vector<double> channelsCoeffs(3, 1.0/3.0);
    std::vector<std::string> regSetUri;

    //mergeDriver->setParameters(minAreaThreshold, maxAreaThreshold, colorDistanceThreshold, colorSpaceConversion, channelsCoeffs);
    mergeDriver->setParameters(minAreaThreshold, maxAreaThreshold, colorDistanceThreshold, regSetUri);
    
    girl::regionDeque regionsToMerge = createdRegions;
    girl::regionDeque keptRegions;
    t.start();
    mergeAlgo->apply(regionsToMerge, *mergeDriver, 1, keptRegions);
    const size_t time1 = t.stop();

    std::cerr<<"after merge: "<<tog2.numRegions()<<" regions [minSize="<<s<<" threshold="<<minAreaThreshold<<"]"<<std::endl;
    std::cerr<<"time merge: "<<time1<<"ms"<<std::endl;
    
    t.start();
    
    tog2.shrink();

    const size_t time2 = t.stop();
    std::cerr<<"time shrink: "<<time2<<"ms"<<std::endl;
  }
  

  


  delete driver;
  //delete histo;

  delete splitAlgo;
  delete quantif;
  delete segContext;
  
  //delete regSet;
  delete tog;


  return EXIT_SUCCESS;
}

