/*
  Copyright (C) 2010-2011 Julian Roigt  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#ifndef IMAGEFEATURECOLORPROJECTIONPLUGIN_HPP
#define IMAGEFEATURECOLORPROJECTIONPLUGIN_HPP

#include <girl/girlCommon.hpp>

#include <qiseTools/ImageFeatureColorimetricPlugin.hpp>

#include <ise/iseImageFeatureComputer.hpp>

#include "ImageFeatureColorProjectionWidget.hpp"
#include "ImageFeatureColorProjectionSettings.hpp"

#include <QObject>

class GIRL_API ImageFeatureColorProjectionPlugin : public QObject
					    , public ImageFeatureColorimetricPlugin
{

  Q_OBJECT
  Q_INTERFACES(ImageFeatureColorimetricPlugin QIsePlugin)
#if QT_VERSION >= 0x050000
    Q_PLUGIN_METADATA(IID ImageFeatureColorimetricPlugin_iid)
#endif
    
public:
    
  ~ImageFeatureColorProjectionPlugin() {}
  
  ImageFeatureColorProjectionPlugin()
    : ImageFeatureColorimetricPlugin(QString::fromUtf8("ColorProjection"),
			 QIseVersion(1, 0), QIseVersion(1, 0))
  {}
  
  virtual girl::shared_ptr<ise::FeatureComputer> create(ise::SegmentationContext *segContext)
  {
    girl::shared_ptr<ise::FeatureComputer> result
      = ise::ImageFeatureColorProjectionFactory::instance().create(segContext);
    return result;
  }
 
  virtual girl::shared_ptr<FeatureInterfaceSettings> createSettings()
  {
    return girl::shared_ptr<FeatureInterfaceSettings>(new ImageFeatureColorProjectionSettings());
  }

  virtual ImageFeatureWidgetInterface* createWidget(MainView *mainView, FeatureComputerGUIInfoPtr featureComputerGUIInfo, QWidget *parent = NULL)
  {
    return new ImageFeatureColorProjectionWidget(mainView, featureComputerGUIInfo, "ColorProjection", this, parent);
  }
};

#if QT_VERSION < 0x050000
Q_EXPORT_PLUGIN2(QIseImageFeatureColorProjectionPlugin, ImageFeatureColorProjectionPlugin)
#endif

#endif /* ! IMAGEFEATURECOLORPROJECTIONPLUGIN_HPP */
