/*
  Copyright (C) 2013 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef IMAGEFEATURECOLORPROJECTIONSETTINGS_HPP
#define IMAGEFEATURECOLORPROJECTIONSETTINGS_HPP

//#include "ImageFeatureSettingsInterface.hpp"
#include <qiseTools/FeatureInterfaceSettings.hpp>
#include "ImageFeatureColorProjectionWidget.hpp"
#include <ise/iseImageFeatureColorProjection.hpp>

class ImageFeatureColorProjectionSettings : public FeatureInterfaceSettings//ImageFeatureSettingsInterface
{
  
public:

  //B: UGLY: duplication of default parameters !!!!!!!!!!!!!

  ImageFeatureColorProjectionSettings() 
    : FeatureInterfaceSettings()//ImageFeatureSettingsInterface()
    , m_colorSpace(ISE_RGB_to_Lab)
    , m_a00(0.8276)
    , m_a01(-0.5097)
    , m_a02(0.2352)
    , m_a10(-0.1835)
    , m_a11(-0.4189)
    , m_a12(0.8893)
    , m_tx(-28.6515)
    , m_ty(5.6625)
  {
  }

  virtual void configureWidget(FeatureInterfaceWidget *widget) const
  {
    static_cast<ImageFeatureColorProjectionWidget *>(widget)->setParameters(m_colorSpace, m_a00, m_a01, m_a02, m_a10, m_a11, m_a12, m_tx, m_ty);
  }
  
  virtual void getWidgetConfig(const FeatureInterfaceWidget *widget)
  {
    static_cast<const ImageFeatureColorProjectionWidget *>(widget)->getParameters(m_colorSpace, m_a00, m_a01, m_a02, m_a10, m_a11, m_a12, m_tx, m_ty);
  }

  virtual void configureAlgo(girl::shared_ptr<ise::FeatureComputer> algo) const
  {
    assert(algo != NULL);
    girl::shared_ptr<ise::ImageFeatureColorProjection> algorithm = girl::static_pointer_cast<ise::ImageFeatureColorProjection>(algo);
    algorithm->setParameters(m_colorSpace, m_a00, m_a01, m_a02, m_a10, m_a11, m_a12, m_tx, m_ty);
  }

  virtual girl::shared_ptr<FeatureInterfaceSettings> clone() const
  {
    girl::shared_ptr<ImageFeatureColorProjectionSettings> p(new ImageFeatureColorProjectionSettings);

    *p = *this;

    return p;
  }
  
  virtual bool isValid() const
  {
    return m_colorSpace >= 0 && m_colorSpace != ISE_RGB_to_GRAY; //B: ???
  }

private:
  
  int m_colorSpace;
  double m_a00, m_a01, m_a02, m_a10, m_a11, m_a12, m_tx, m_ty;

};

#endif /* ! IMAGEFEATURECOLORPROJECTIONSETTINGS_HPP */
