/*
  Copyright (C) 2013 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "ImageFeatureColorProjectionWidget.hpp"

#include <QString>
#include <QLabel>
#include <QGroupBox>
#include <QGridLayout>
#include <QComboBox>
#include <QDialogButtonBox>
#include <QLineEdit>
#include <QLabel>
#include <QDoubleValidator>

#include <qiseTools/ColorSpaceChoiceWidget.hpp>
#include <qiseTools/ColorSpaceRegistry.hpp>


ImageFeatureColorProjectionWidget::ImageFeatureColorProjectionWidget(MainView *mainView, 
								     FeatureComputerGUIInfoPtr featureComputerGUIInfo, 
								     QString name, 
								     ImageFeaturePlugin *imageFeaturePlugin, 
								     QWidget *parent)
  : ImageFeatureWidgetInterface(mainView, featureComputerGUIInfo, name, imageFeaturePlugin, parent)
  , m_cscw(NULL)
  , m_a00(NULL)
  , m_a01(NULL)
  , m_a02(NULL)
  , m_a10(NULL)
  , m_a11(NULL)
  , m_a12(NULL)
  , m_tx(NULL)
  , m_ty(NULL)
{
  setAttribute(Qt::WA_DeleteOnClose);

  build();
}

void
ImageFeatureColorProjectionWidget::build()
{
  assert(m_cscw == NULL);
  m_cscw = new ColorSpaceChoiceWidget(this);

  m_a00 = new QLineEdit(this);
  m_a00->setValidator(new QDoubleValidator(m_a00));
  m_a01 = new QLineEdit(this);
  m_a01->setValidator(new QDoubleValidator(m_a01));
  m_a02 = new QLineEdit(this);
  m_a02->setValidator(new QDoubleValidator(m_a02));
  m_a10 = new QLineEdit(this);
  m_a10->setValidator(new QDoubleValidator(m_a10));
  m_a11 = new QLineEdit(this);
  m_a11->setValidator(new QDoubleValidator(m_a11));
  m_a12 = new QLineEdit(this);
  m_a12->setValidator(new QDoubleValidator(m_a12));
  m_tx = new QLineEdit(this);
  m_tx->setValidator(new QDoubleValidator(m_tx));
  m_ty = new QLineEdit(this);
  m_ty->setValidator(new QDoubleValidator(m_ty));
  
  QGridLayout *gLayout = new QGridLayout;
  gLayout->addWidget(m_cscw, 0, 0, 1, 3);
  gLayout->addWidget(new QLabel(tr("Transformation:"), this), 1, 0);
  gLayout->addWidget(m_a00, 2, 0);
  gLayout->addWidget(m_a01, 2, 1);
  gLayout->addWidget(m_a02, 2, 2);
  gLayout->addWidget(m_a10, 3, 0);
  gLayout->addWidget(m_a11, 3, 1);
  gLayout->addWidget(m_a12, 3, 2);
  gLayout->addWidget(new QLabel(tr("Translation:"), this), 4, 0);
  gLayout->addWidget(m_tx, 5, 0);
  gLayout->addWidget(m_ty, 5, 1);

  setLayout(gLayout);
}

bool 
ImageFeatureColorProjectionWidget::isValid()
{
  bool b = m_cscw->getColorSpace() >= 0;
  return b;
}

void
ImageFeatureColorProjectionWidget::setParameters(int colorSpace, 
						 double a00, double a01, double a02, 
						 double a10, double a11, double a12, 
						 double tx, double ty)
{
  m_cscw->setColorSpace(colorSpace);
  m_a00->setText(QString::number(a00));
  m_a01->setText(QString::number(a01));
  m_a02->setText(QString::number(a02));
  m_a10->setText(QString::number(a10));
  m_a11->setText(QString::number(a11));
  m_a12->setText(QString::number(a12));
  m_tx->setText(QString::number(tx));
  m_ty->setText(QString::number(ty));
}

void
ImageFeatureColorProjectionWidget::getParameters(int &colorSpace, 
						 double &a00, double &a01, double &a02, 
						 double &a10, double &a11, double &a12, 
						 double &tx, double &ty) const
{
  colorSpace = m_cscw->getColorSpace();
  a00 = m_a00->text().toDouble();
  a01 = m_a01->text().toDouble();
  a02 = m_a02->text().toDouble();
  a10 = m_a10->text().toDouble();
  a11 = m_a11->text().toDouble();
  a12 = m_a12->text().toDouble();
  tx = m_tx->text().toDouble();
  ty = m_ty->text().toDouble();
}	 

