/*
  Copyright (C) 2005-2010 Vieux Remi, Roigt Julian, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISEIMAGEFEATURETHRESHOLDS_HPP
#define ISEIMAGEFEATURETHRESHOLDS_HPP

#include <vector>

#include <girl/girlCommon.hpp>
#include <girlTools/girlToolsImageFeature.hpp>

#include <ise/iseFactoryTemplate.hxx>
#include <ise/iseImageFeatureColorimetricComputer.hpp>
#include <ise/iseSegmentationContext.hpp>


namespace ise {

  class GIRL_API ImageFeatureThresholds : public ise::ImageFeatureColorimetricComputer
  {
    static const int FEATURE_DIMENSION = 1;

  public:
    explicit ImageFeatureThresholds(ise::SegmentationContext *segC);
    
    virtual ~ImageFeatureThresholds()
    {
    }

    virtual void compute();
    
    virtual void setParameters(std::vector<int> levels, int colorSpace, int channel);

    virtual void getParameters(std::vector<int> &levels, int &colorSpace, int &channel) const;

    virtual const std::string uri() const;

    virtual const std::string baseUri() const;

    virtual void getDimDescription(std::vector<std::string> &dimDesc) const;

    virtual size_t getFeatureDim() const { return FEATURE_DIMENSION; }

    virtual void getRequiredComputers(FeatureComputer::FeatureComputerListType &/*req*/) const
    {
    }

  private:
    ImageFeatureThresholds(const ImageFeatureThresholds&);
    ImageFeatureThresholds &operator=(const ImageFeatureThresholds&);
    
  private:
    
    std::vector<int> m_levels;
    int m_colorSpace;
    int m_channel;
  };

  typedef FactoryTemplate<ImageFeatureComputer, ImageFeatureThresholds> ImageFeatureThresholdsFactory;
}
#endif /* ! ISEIMAGEFEATURETHRESHOLDS_HPP */
