/*
  Copyright (C) 2005-2010 Vieux Remi, Roigt Julian, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "ImageFeatureCurvelet.hpp"

using namespace ise;

ImageFeatureCurvelet::ImageFeatureCurvelet(ise::SegmentationContext *segC)
  : ImageFeatureTextureComputer(segC)
  , m_curve()
  , m_scale(8)
  , m_angles(8)
{

}

const std::string
ImageFeatureCurvelet::baseUri() const
{
  return "Curvelet";
}

const std::string
ImageFeatureCurvelet::uri() const 
{
  std::stringstream ss(std::ios::in|std::ios::out);
  ss << baseUri()<< IMAGE_FEATURE_URI_SEPARATOR <<m_scale<< IMAGE_FEATURE_URI_SEPARATOR <<m_angles;
  return ss.str();
}

void
ImageFeatureCurvelet::compute()
{
  assert(m_segContext);
  std::cout<<"compute curvelet"<<std::endl;
  m_curve = Curvelet(m_segContext, m_scale, m_angles);

  m_curve.get_coeff(m_imgF);

  std::cout<<"end compute curvelet"<<std::endl;

}

void
ImageFeatureCurvelet::setParameters(int scale, int angles)
{
  m_scale = scale;
  m_angles = angles;
}

void
ImageFeatureCurvelet::getParameters(int &scale, int &angles) const
{
  scale = m_scale;
  angles = m_angles;
}

void 
ImageFeatureCurvelet::getDimDescription(std::vector<std::string> &dimDesc) const
{
  dimDesc.clear();
  for (size_t i = 0 ; i < getFeatureDim() ; ++i) {
    std::stringstream ss;
    ss<<"Curvelet "<< i;
    dimDesc.push_back(ss.str());
  }
}
