/*
  Copyright (C) 2005-2010 Vieux Remi, Roigt Julian, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISEIMAGEFEATURECURVELET_HPP
#define ISEIMAGEFEATURECURVELET_HPP

#include <girl/girlCommon.hpp>
#include <girlTools/girlToolsImageFeature.hpp>

#include <ise/iseFactoryTemplate.hxx>
#include <ise/iseSegmentationContext.hpp>
#include <ise/iseImageFeatureTextureComputer.hpp>
#include <cassert>
#include "curvelet.hpp"

//inline int pow2(int l) { assert(l>=0); return (1<<l); }

class GIRL_API ImageFeatureCurvelet : public ise::ImageFeatureTextureComputer
{

 public:
  
  explicit ImageFeatureCurvelet(ise::SegmentationContext *segC);
  
  virtual ~ImageFeatureCurvelet()
  {
  }
  
  virtual void compute();
  
  virtual void setParameters(int scale, int angles);

  virtual void getParameters(int &scale, int &angles) const;
  
  virtual const std::string uri() const;
  
  virtual const std::string baseUri() const;
  
  void getDimDescription(std::vector<std::string> &dimDesc) const;
  
  size_t getFeatureDim() const { 
    return m_angles*pow2(int(ceil(double(m_scale-2)/2)));
  }
  
  void getRequiredComputers(FeatureComputer::FeatureComputerListType &) const
  {
    // TODO enable computation with any image feature
  }
  
 private:
  
  ImageFeatureCurvelet(const ImageFeatureCurvelet &);
  ImageFeatureCurvelet &operator=(const ImageFeatureCurvelet &);
  
 private:
  
  Curvelet m_curve;
  int m_scale;
  int m_angles;
};

typedef ise::FactoryTemplate<ise::ImageFeatureComputer, ImageFeatureCurvelet> ImageFeatureCurveletFactory;

#endif /* ! ISEIMAGEFEATURECURVELET_HPP */
