/*
  Copyright (C) 2005-2011 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef IMAGEFEATURECURVELETSETTINGS_HPP
#define IMAGEFEATURECURVELETSETTINGS_HPP

#include <qiseTools/FeatureInterfaceSettings.hpp>
#include "ImageFeatureCurveletWidget.hpp"
#include "ImageFeatureCurvelet.hpp"

class ImageFeatureCurveletSettings : public FeatureInterfaceSettings
{
  
public:
  
  ImageFeatureCurveletSettings() 
    : FeatureInterfaceSettings()
    , m_scale(2)
    , m_level(2)
  {
  }

  virtual void configureWidget(FeatureInterfaceWidget *widget) const
  {
    static_cast<ImageFeatureCurveletWidget *>(widget)->setParameters(m_scale, m_level);
  }
  
  virtual void getWidgetConfig(const FeatureInterfaceWidget *widget)
  {
    static_cast<const ImageFeatureCurveletWidget *>(widget)->getParameters(m_scale, m_level);
  }

  virtual void configureAlgo(girl::shared_ptr<ise::FeatureComputer> algo) const
  {
    girl::shared_ptr<ImageFeatureCurvelet> algorithm = girl::static_pointer_cast<ImageFeatureCurvelet>(algo);
    algorithm->setParameters(m_scale, m_level);
  }

  virtual girl::shared_ptr<FeatureInterfaceSettings> clone() const
  {
    girl::shared_ptr<ImageFeatureCurveletSettings> p(new ImageFeatureCurveletSettings);

    *p = *this;

    return p;
  }

  virtual bool isValid() const
  {
    return (m_scale > 0 && m_level > 0); 
  }
  
private:
  
  int m_scale;
  int m_level;
};

#endif /* ! IMAGEFEATURECURVELETSETTINGS_HPP */
