/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "ImageFeatureCurveletWidget.hpp"

#include <QSpinBox>
#include <QLabel>
#include <QString>

#include <QComboBox>
#include <QDialogButtonBox>
#include <QVBoxLayout>

ImageFeatureCurveletWidget::ImageFeatureCurveletWidget(MainView *mainView, 
						       FeatureComputerGUIInfoPtr featureComputerGUIInfo, 
						       QString name, 
						       ImageFeaturePlugin *imageFeaturePlugin, 
						       QWidget *parent)
  : ImageFeatureWidgetInterface(mainView, featureComputerGUIInfo, name, imageFeaturePlugin, parent)
  , m_scaleSpinBox(NULL)
  , m_anglesSpinBox(NULL)
{
  setAttribute(Qt::WA_DeleteOnClose);

  build();
}

void
ImageFeatureCurveletWidget::build()
{
  assert(m_anglesSpinBox == NULL);
  m_anglesSpinBox = new QSpinBox(this);
  m_scaleSpinBox = new QSpinBox(this);

  QLabel *anglesLabel = new QLabel(tr("Angles : "));

  QLabel *scaleLabel = new QLabel(tr("Scale : "));

  m_anglesSpinBox->setMinimum(1);
  m_scaleSpinBox->setMinimum(1);

  m_anglesSpinBox->setValue(0);
  m_scaleSpinBox->setValue(0);
  
  QVBoxLayout *layout = new QVBoxLayout(this);

  layout->addWidget(anglesLabel);
  layout->addWidget(m_anglesSpinBox);
  layout->addWidget(scaleLabel);
  layout->addWidget(m_scaleSpinBox);

  setLayout(layout);
}

bool
ImageFeatureCurveletWidget::isValid()
{
  return (m_anglesSpinBox->value() >= 1 && m_scaleSpinBox->value() >= 1);
}

void 
ImageFeatureCurveletWidget::setParameters(int scale, int angles)
{
  m_anglesSpinBox->setValue(angles);
  m_scaleSpinBox->setValue(scale);
}

void
ImageFeatureCurveletWidget::getParameters(int &scale, int &angles) const
{
  scale = m_scaleSpinBox->value();
  angles = m_anglesSpinBox->value();
}


