#include "curvelet.hpp"


Curvelet::~Curvelet()
{
  unsigned int i;
  for(i=0;i<c.size();i++)
    c[i].clear();
  c.clear();
  moyennes.clear();
  ecarts_types.clear();
}

int fdct_wrapping_sepangle(double XL1, double XL2, int nbangle, CMatrix& Xhgh, std::vector<CMatrix>& csc)
{
  //WEDGE ORDERING: from -45 degree, counter-clockwise
  typedef pair<int, int> intpair;
  map<intpair, fftw_plan> planmap;

  int nbquadrants = 4;
  int nd = nbangle / 4;
  int wcnt = 0;

  //backup
  CMatrix Xhghb(Xhgh);
  double XL1b = XL1;
  double XL2b = XL2;

  int qvec[] = {2,1,0,3};
  int qi, w, i, j;
  for(qi=0; qi<nbquadrants; qi++)
    {
      int q = qvec[qi];
      //ROTATE data to its right position
      if(q%2==0)
	{
	  XL1 = abs(XL1b);
	  XL2 = abs(XL2b);
	}
      else
	{
	  XL2 = abs(XL1b);
	  XL1 = abs(XL2b);
	}
      fdct_wrapping_rotate_forward(q, Xhghb, Xhgh);
      //figure out XS, XF, XR
      double XW1 = XL1/nd;
      double XW2 = XL2/nd;
      int XS1, XS2;
      int XF1, XF2;
      double XR1, XR2;
      fdct_wrapping_rangecompute(XL1, XL2, XS1, XS2, XF1, XF2, XR1, XR2);
      for(w=nd-1; w>=0; w--)
        {
          double xs = XR1/4 - (XW1/2)/4;
          double xe = XR1;
          double ys = -XR2 + (w-0.5)*XW2;
          double ye = -XR2 + (w+1.5)*XW2; //x range
          int xn = int(ceil(xe-xs));
          int yn = int(ceil(ye-ys));
          //MAKE THEM ODD
          if(xn%2==0) xn++;
          if(yn%2==0) yn++;
          int xf = int(ceil(xs));			 //int yf = int(ceil(ys));
          //theta
          double thts, thtm, thte; //y direction
          if(w==0)
            {
              thts = atan2(-1.0, 1.0-1.0/nd);
              thtm = atan2(-1.0+1.0/nd, 1.0);
              thte = atan2(-1.0+3.0/nd, 1.0);
            }
          else
            if(w==nd-1)
              {
                thts = atan2(-1.0+(2.0*w-1.0)/nd, 1.0);
                thtm = atan2(-1.0+(2.0*w+1.0)/nd, 1.0);
                thte = atan2(1.0, 1.0-1.0/nd);
              }
            else
              {
                thts = atan2(-1.0+(2.0*w-1.0)/nd, 1.0);
                thtm = atan2(-1.0+(2.0*w+1.0)/nd, 1.0);
                thte = atan2(-1.0+(2.0*w+3.0)/nd, 1.0);
              }
          //wrapping
          int xh = xn/2;
          int yh = yn/2; //half length
          double R21 = XR2/XR1;
          //ratio
          CMatrix wpdata(xn, yn, true);
          complex <double>tmp;
          int xcur, ycur, yfm, yto, tmpx, tmpy;
	  double thtcur;
          double wtht;
          double l,r,pou;
          for(xcur=xf; xcur<xe; xcur++)
            { //for each layer
              yfm = (int)ceil( max(-XR2, R21*xcur*tan(thts)) );
              yto = (int)floor( min(XR2, R21*xcur*tan(thte)) );
              for(ycur=yfm; ycur<=yto; ycur++)
                {
                  tmpx = xcur%xn;
                  if(tmpx<-xh)
                    tmpx+=xn;
                  if(tmpx>=-xh+xn)
                    tmpx-=xn;
                  tmpy = ycur%yn;
                  if(tmpy<-yh)
                    tmpy+=yn;
                  if(tmpy>=-yh+yn)
                    tmpy-=yn;
                  wpdata.setr(tmpx, tmpy, Xhgh.getr(xcur, ycur));
                  //partition of unity
                  thtcur = atan2(ycur/XR2, xcur/XR1);
                  if(thtcur<thtm)
                    {
                      fdct_wrapping_window((thtcur-thts)/(thtm-thts), l, r);
                      wtht = l;
                    }
                  else
                    {
                      fdct_wrapping_window((thtcur-thtm)/(thte-thtm), l, r);
                      wtht = r;
                    }
                  pou = wtht;
                  tmp=complex <double>(wpdata.getr(tmpx, tmpy).real()*pou, wpdata.getr(tmpx, tmpy).imag()*pou);
                  wpdata.setr(tmpx, tmpy, tmp);
                }
            }
          //IFFT
          {
            //rotate backward
            CMatrix rpdata;
            fdct_wrapping_rotate_backward(q, wpdata, rpdata);
            //ifftshift
            int xn = rpdata.getRowDimension();
            int yn = rpdata.getColDimension(); //reset xn, yn
            CMatrix tpdata(xn, yn);
            fdct_wrapping_ifftshift(rpdata, tpdata);
            //ifft
            fftw_plan p = NULL;
            map<intpair,fftw_plan>::iterator mit=planmap.find( intpair(xn, yn) );
            if(mit!=planmap.end())
	      {
 		p = fftw_plan_dft_2d(yn, xn, (fftw_complex *)tpdata.data(), (fftw_complex *)tpdata.data(),FFTW_BACKWARD,FFTW_ESTIMATE);;
		(*mit).second=p;
	      }
            else
              {
		p = fftw_plan_dft_2d(yn, xn, (fftw_complex*)tpdata.data(), (fftw_complex*)tpdata.data(),FFTW_BACKWARD, FFTW_ESTIMATE);
                planmap[ intpair(xn, yn) ] = p;
              }
	    fftw_execute(p);
            double sqrtprod = sqrt(double(xn*yn));
            for(j=0; j<yn; j++)
              for(i=0; i<xn; i++)
                {
                  tmp=complex <double>(tpdata.getr(i, j).real()/sqrtprod, tpdata.getr(i, j).imag()/sqrtprod);
                  tpdata.setr(i, j, tmp);
                }
            //store
            csc[wcnt] = tpdata;
          }
          //fdct_wrapping_fftshift(xn, yn, xh, yh, tpdata, wpdata);
          //ROTATION
          //fdct_wrapping_rotate_backward(q, wpdata, csc[wcnt]);
          wcnt++;
        } //end of w loop
    } //end of q loop
  //PUT THE RIGHT DATA BACK
  Xhgh = Xhghb;
  XL1 = XL1b;
  XL2 = XL2b;
  for(map<intpair, fftw_plan>::iterator mit=planmap.begin(); mit!=planmap.end(); ++mit)
    {
      fftw_plan p = (*mit).second;
      fftw_destroy_plan(p);
    }
  return 0;
}

int fdct_wrapping_wavelet(CMatrix& Xhgh, std::vector<CMatrix>& csc)
{
  int i, j;
  int width = Xhgh.getRowDimension();  int height = Xhgh.getColDimension();
  CMatrix T(width, height);
  fdct_wrapping_ifftshift(Xhgh, T);
  fftw_plan p = fftw_plan_dft_2d(height, width, (fftw_complex *)T.data(), (fftw_complex *)T.data(),FFTW_BACKWARD,FFTW_ESTIMATE);;
  fftw_execute(p);
  fftw_destroy_plan(p);
  double sqrtprod = sqrt(double(width*height));
  complex <double> tmp;
  for(j=0; j<height; j++)
    for(i=0; i<width; i++)
      {
        tmp=complex <double>(T.getr(i, j).real()/sqrtprod, T.getr(i, j).imag()/sqrtprod);
        T.setr(i, j, tmp);
      }
  csc[0] = T;
  return 0;
}

//--------------------------------------------------
int fdct_wrapping_param_sepangle(double XL1, double XL2, int nbangle,
				 std::vector<double>& sx, std::vector<double>& sy,
				 std::vector<double>& fx, std::vector<double>& fy,
				 std::vector<int>& nx, std::vector<int>& ny)
{
  fx.resize(nbangle);
  fy.resize(nbangle);
  nx.resize(nbangle);
  ny.resize(nbangle);
  sx.resize(nbangle);
  sy.resize(nbangle);
  
  int nbquadrants = 4;
  int nd = nbangle / 4;  //int nbangles_perquad = nbangles[sc] / 4;
  int wcnt = 0;
  //backup
  double XL1b = XL1;  
  double XL2b = XL2;
  int qvec[] = {2,1,0,3};
  for(int qi=0; qi<nbquadrants; qi++) {
    int q = qvec[qi];
    fdct_wrapping_rotate_forward(q, XL1b, XL2b, XL1, XL2);
    XL1 = abs(XL1);
    XL2 = abs(XL2);
    double XW1 = XL1/nd;
    double XW2 = XL2/nd;
    int XS1, XS2;
    int XF1, XF2;
    double XR1, XR2;
    fdct_wrapping_rangecompute(XL1, XL2, XS1, XS2, XF1, XF2, XR1, XR2);
    for(int w=nd-1; w>=0; w--) {
      //get size
      double xs = XR1/4 - (XW1/2)/4;
      double xe = XR1;
      double ys = -XR2 + (w-0.5)*XW2;
      double ye = -XR2 + (w+1.5)*XW2; //x range
      int xn = int(ceil(xe-xs));
      int yn = int(ceil(ye-ys));
      //MAKE THEM ODD
      if(xn%2==0) xn++;
      if(yn%2==0) yn++;
      
      double tx; double ty;
      //fx,fy
      tx = XR1/2;
      ty = (-XR2 + (w+0.5)*XW2)/2; //center, NOTE: need /2
      fdct_wrapping_rotate_backward(q, tx, ty, fx[wcnt], fy[wcnt]);
      //sx, sy;
      tx = 1.0/xn;
      ty = 1.0/yn;
      fdct_wrapping_rotate_backward(q, xn, yn, nx[wcnt], ny[wcnt]);
      nx[wcnt] = abs(nx[wcnt]);	
      ny[wcnt] = abs(ny[wcnt]);
      fdct_wrapping_rotate_backward(q, tx, ty, sx[wcnt], sy[wcnt]);
      sx[wcnt] = abs(sx[wcnt]);	
      sy[wcnt] = abs(sy[wcnt]);
      wcnt++;
    }
  }
  
  return 0;
}

//--------------------------------------------------
int fdct_wrapping_param_wavelet(int S1, int S2,
				std::vector<double>& sx, std::vector<double>& sy,
				std::vector<double>& fx, std::vector<double>& fy,
				std::vector<int>& nx, std::vector<int>& ny)
{
  fx.resize(1);  fx[0] = 0;
  fy.resize(1);  fy[0] = 0;
  nx.resize(1);  nx[0] = S1;
  ny.resize(1);  ny[0] = S2;
  double dx = 1.0/S1;  double dy = 1.0/S2;
  sx.resize(1);  sx[0] = dx;
  sy.resize(1);  sy[0] = dy;
  return 0;
}

//--------------------------------------------------
int fdct_wrapping_param(int N1, int N2, int nbscales, int nbangles_coarse, int allcurvelets,
			std::vector< std::vector<double> >& sx, std::vector< std::vector<double> >& sy,
			std::vector< std::vector<double> >& fx, std::vector< std::vector<double> >& fy,
			std::vector< std::vector<int> >& nx, std::vector< std::vector<int> >& ny)
{
  //sx, sy, step in spatial domain
  //fx, fy, position in frequency domain
  //nx, ny, size of the grid
  sx.resize(nbscales);
  sy.resize(nbscales);
  fx.resize(nbscales);
  fy.resize(nbscales);
  nx.resize(nbscales);
  ny.resize(nbscales);
    
  std::vector<int> nbangles(nbscales);
  if(allcurvelets==1) {
    //nbangles
    nbangles[0] = 1;
    for(int sc=1; sc<nbscales; sc++)		
      nbangles[sc] = nbangles_coarse * pow2( int(ceil(double(sc-1)/2)) );
    //high freq levels
    double XL1 = 4.0*N1/3.0;  
    double XL2 = 4.0*N2/3.0; //range
    for(int sc=nbscales-1; sc>0; sc--) {
      fdct_wrapping_param_sepangle(XL1, XL2, nbangles[sc], sx[sc], sy[sc], fx[sc], fy[sc], nx[sc], ny[sc]);
      XL1 /= 2;
      XL2 /= 2;
    }
    //coarsest level
    int XS1, XS2;  
    int XF1, XF2;  
    double XR1, XR2;	 
    fdct_wrapping_rangecompute(XL1, XL2, XS1, XS2, XF1, XF2, XR1, XR2);
    fdct_wrapping_param_wavelet(XS1, XS2, sx[0], sy[0], fx[0], fy[0], nx[0], ny[0]);
  } else {
    //nbangles
    nbangles[0] = 1;
    for(int sc=1; sc<nbscales-1; sc++)
      nbangles[sc] = nbangles_coarse * pow2( int(ceil(double(sc-1)/2)) );
    nbangles[nbscales-1] = 1;
    //top level
    fdct_wrapping_param_wavelet(N1, N2, sx[nbscales-1], sy[nbscales-1], fx[nbscales-1], fy[nbscales-1], nx[nbscales-1], ny[nbscales-1]);
    //next levels
    double XL1 = 2.0*N1/3.0;  
    double XL2 = 2.0*N2/3.0; //range
    for(int sc=nbscales-2; sc>0; sc--) {
      fdct_wrapping_param_sepangle(XL1, XL2, nbangles[sc], sx[sc], sy[sc], fx[sc], fy[sc], nx[sc], ny[sc]);
      XL1 /= 2;
      XL2 /= 2;
    }
    //coarsest level
    int XS1, XS2;
    int XF1, XF2;
    double XR1, XR2;
    fdct_wrapping_rangecompute(XL1, XL2, XS1, XS2, XF1, XF2, XR1, XR2);
    fdct_wrapping_param_wavelet(XS1, XS2, sx[0], sy[0], fx[0], fy[0], nx[0], ny[0]);
  }
  
  return 0;
}

/*
  std::vector <std::vector <CMatrix> > c;
  int nb_echelle;
  int nb_angles;
  int width;
  int height;
  std::vector <float> moyennes;
  std::vector <float> ecarts_types;
 */

Curvelet::Curvelet() :
  nb_echelle(0),
  nb_angles(0),
  width(0),
  height(0)
{}

Curvelet::Curvelet(ise::SegmentationContext *segC, int niveau_echelle, int nombre_angles_initiaux, int allcurve)
{
  girl::tools::Image *img = segC->getImage();

  width = img->width();
  height = img->height();

  CMatrix x(width, height, false);

  for (int h = 0 ; h < height ; ++h) {
    girl::tools::Image::type *data = img->scanLine(h);
    for (int w = 0 ; w < width ; ++w) {
      const girl::tools::Image::type val = (data[0]+data[1]+data[2] ) / 3.0 ;
      x.set(w, h, complex<double>(static_cast<double>(val), 0.0) );
      data += img->formatSize();
    }
  }

  nb_echelle=niveau_echelle;
  nb_angles=nombre_angles_initiaux;
  width=x.getRowDimension();
  height=x.getColDimension();
  if(allcurve==1)
    angles(x);
  else
    wavelet(x);
}

int Curvelet::angles(CMatrix x)
{
  int i, j,sc;
  CMatrix T(x);
  fftw_plan p = fftw_plan_dft_2d(height, width, (fftw_complex *)T.data(), (fftw_complex *)T.data(),FFTW_FORWARD,FFTW_ESTIMATE);
  fftw_execute(p);
  fftw_destroy_plan(p);
  double sqrtprod = sqrt(double(width*height));
  complex <double> tmp;
  for(j=0; j<height; j++)
    for(i=0; i<width; i++)
      {
        tmp=complex <double>(T.getr(i, j).real()/sqrtprod, T.getr(i, j).imag()/sqrtprod);
        T.setr(i, j, tmp);
      }

  CMatrix O(width, height, true);
  fdct_wrapping_fftshift(T, O);
  //-----------------------------------------------------------------------------
  std::vector<CMatrix> Xhghs(nb_echelle);
  CMatrix X=0;
  //--------------------------
  double XL1 = 4.0*width/3.0;
  double XL2 = 4.0*height/3.0; //range
  int XS1, XS2;
  int XF1, XF2;
  double XR1, XR2;
  fdct_wrapping_rangecompute(XL1, XL2, XS1, XS2, XF1, XF2, XR1, XR2);
  IntVecteur t1(XS1, true);
  for(i=-XF1; i<-XF1+XS1; i++)
    if(i<-width/2)
      t1.set(i, i+int(width));
    else
      if(i>(width-1)/2)
	t1.set(i, i-int(width));
      else t1.set(i, i);
  IntVecteur t2(XS2, true);
  for(i=-XF2; i<-XF2+XS2; i++)
    if(i<-height/2)
      t2.set(i, i+int(height));
    else
      if(i>(height-1)/2)
	t2.set(i, i-int(height));
      else t2.set(i, i);
  X=CMatrix(XS1, XS2, true);
  for(j=-XF2; j<-XF2+XS2; j++)
    for(i=-XF1; i<-XF1+XS1; i++)
      X.setr(i, j, O.getr(t1.get(i), t2.get(j)));
  
  DblMatrix lowpass(XS1, XS2, true);
  fdct_wrapping_lowpasscompute(XL1, XL2, lowpass); //compute the low pass filter
  for(j=-XF2; j<-XF2+XS2; j++)
    for(i=-XF1; i<-XF1+XS1; i++)
      {
	tmp=complex <double> (X.getr(i, j).real()*lowpass.getr(i, j), X.getr(i, j).imag()*lowpass.getr(i, j));
	X.setr(i, j, tmp);
      }
  //separate
  XL1 = 4.0*width/3.0;
  XL2 = 4.0*height/3.0; //range
  double XL1n, XL2n, XR1n, XR2n;
  int XS1n, XS2n;
  int XF1n, XF2n;
  for(sc=nb_echelle-1; sc>0; sc--)
    {
      XL1n = XL1/2;
      XL2n = XL2/2;
      fdct_wrapping_rangecompute(XL1n, XL2n, XS1n, XS2n, XF1n, XF2n, XR1n, XR2n);
      //computer filter
      DblMatrix lowpass(XS1n, XS2n, true);
      fdct_wrapping_lowpasscompute(XL1n, XL2n, lowpass);
      DblMatrix hghpass(XS1n, XS2n, true);
      for(j=-XF2n; j<-XF2n+XS2n; j++)
        for(i=-XF1n; i<-XF1n+XS1n; i++)
          hghpass.setr(i, j,sqrt(1-lowpass.getr(i, j)*lowpass.getr(i, j)));
      //separate
      CMatrix Xhgh(X);
      for(j=-XF2n; j<-XF2n+XS2n; j++)
        for(i=-XF1n; i<-XF1n+XS1n; i++)
          {
            tmp=complex <double>(Xhgh.getr(i, j).real()*hghpass.getr(i, j), Xhgh.getr(i, j).imag()*hghpass.getr(i, j));
            Xhgh.setr(i, j, tmp);
          }
      CMatrix Xlow(XS1n, XS2n, true);
      for(j=-XF2n; j<-XF2n+XS2n; j++)
        for(i=-XF1n; i<-XF1n+XS1n; i++)
          {
            tmp=complex <double>(X.getr(i, j).real()*lowpass.getr(i, j), X.getr(i, j).imag()*lowpass.getr(i, j));
            Xlow.setr(i, j, tmp);
          }
      //store and prepare for next level
      Xhghs[sc] = Xhgh;
      X = Xlow;
      XL1 = XL1/2;
      XL2 = XL2/2;
    }
  Xhghs[0] = X;   
  //-----------------------------------------------------------------------------
  std::vector<int> nbangles(nb_echelle);
  //nbangles
  nbangles[0] = 1;
  for(sc=1; sc<nb_echelle; sc++)
    nbangles[sc] = nb_angles * pow2( int(ceil(double(sc-1)/2)) );
  //c
  c.resize(nb_echelle);
  for(sc=0; sc<nb_echelle; sc++)
    c[sc].resize( nbangles[sc] );
  
  XL1 = 4.0*width/3.0;
  XL2 = 4.0*height/3.0; //range
  for(sc=nb_echelle-1; sc>0; sc--)
    {
      fdct_wrapping_sepangle(XL1, XL2, nbangles[sc], Xhghs[sc], c[sc]);
      XL1 /= 2;
      XL2 /= 2;
    }
  fdct_wrapping_wavelet(Xhghs[0], c[0]);
  //  sauvegarde_vecteurs(1);
  return 0;
}

int Curvelet::wavelet(CMatrix x)
{
  //  fdct_wrapping(width, height,nb_echelle,nb_angles,0, x,c);
  int i, j,sc;
  // ifft original data
  CMatrix T(x);
  fftw_plan p = fftw_plan_dft_2d(height, width, (fftw_complex *)T.data(), (fftw_complex *)T.data(),FFTW_FORWARD,FFTW_ESTIMATE);
  fftw_execute(p);
  fftw_destroy_plan(p);
  double sqrtprod = sqrt(double(width*height));
  complex <double> tmp;
  for(j=0; j<height; j++)
    for(i=0; i<width; i++)
      {
        tmp=complex <double>(T.getr(i, j).real()/sqrtprod, T.getr(i, j).imag()/sqrtprod);
        T.setr(i, j, tmp);
      }

  CMatrix O(width, height, true);
  fdct_wrapping_fftshift(T, O);
  //-----------------------------------------------------------------------------
  std::vector<CMatrix> Xhghs;
  Xhghs.resize(nb_echelle);
  CMatrix X=O;
  //separate
  double XL1 = 4.0*width/3.0;
  double XL2 = 4.0*height/3.0; //range
  double XL1n, XL2n, XR1n, XR2n;
  int XS1n, XS2n;
  int XF1n, XF2n;
  for(sc=nb_echelle-1; sc>0; sc--)
    {
      XL1n = XL1/2;
      XL2n = XL2/2;
      fdct_wrapping_rangecompute(XL1n, XL2n, XS1n, XS2n, XF1n, XF2n, XR1n, XR2n);
      //computer filter
      DblMatrix lowpass(XS1n, XS2n, true);
      fdct_wrapping_lowpasscompute(XL1n, XL2n, lowpass);
      DblMatrix hghpass(XS1n, XS2n, true);
      for(j=-XF2n; j<-XF2n+XS2n; j++)
        for(i=-XF1n; i<-XF1n+XS1n; i++)
          hghpass.setr(i, j,sqrt(1-lowpass.getr(i, j)*lowpass.getr(i, j)));
      //separate
      CMatrix Xhgh(X);
      for(j=-XF2n; j<-XF2n+XS2n; j++)
        for(i=-XF1n; i<-XF1n+XS1n; i++)
          {
            tmp=complex <double>(Xhgh.getr(i, j).real()*hghpass.getr(i, j), Xhgh.getr(i, j).imag()*hghpass.getr(i, j));
            Xhgh.setr(i, j, tmp);
          }
      CMatrix Xlow(XS1n, XS2n, true);
      for(j=-XF2n; j<-XF2n+XS2n; j++)
        for(i=-XF1n; i<-XF1n+XS1n; i++)
          {
            tmp=complex <double>(X.getr(i, j).real()*lowpass.getr(i, j), X.getr(i, j).imag()*lowpass.getr(i, j));
            Xlow.setr(i, j, tmp);
          }
      //store and prepare for next level
      Xhghs[sc] = Xhgh;
      X = Xlow;
      XL1 = XL1/2;
      XL2 = XL2/2;
    }
  Xhghs[0] = X;   
  //-----------------------------------------------------------------------------
  std::vector<int> nbangles(nb_echelle);
  //nbangles
  nbangles[0] = 1;
  for(sc=1; sc<nb_echelle-1; sc++)
    nbangles[sc] = nb_angles * pow2( int(ceil(double(sc-1)/2)) );
  nbangles[nb_echelle-1] = 1;
  //c
  c.resize(nb_echelle);
  for(sc=0; sc<nb_echelle; sc++)
    c[sc].resize( nbangles[sc] );
  
  fdct_wrapping_wavelet(Xhghs[nb_echelle-1], c[nb_echelle-1]);
  XL1 = 2.0*width/3.0;
  XL2 = 2.0*height/3.0; //range
  for(sc=nb_echelle-2; sc>0; sc--)
    {
      fdct_wrapping_sepangle(XL1, XL2, nbangles[sc], Xhghs[sc], c[sc]);
      XL1 /= 2;
      XL2 /= 2;
    }
  fdct_wrapping_wavelet(Xhghs[0], c[0]);
  //sauvegarde_vecteurs(3);


  return 0;
}

void Curvelet::sauvegarde_vecteurs(int a)
{
  //  cout<<"Sauvegarde des vecteurs de l'echelle "<<a+1<<endl;
  string base("coeff_modifie");
  string filenamestr;
  string ext(".pgm");
  unsigned int b;
  int i, j;
  char *filename;
  double maximum=0.0;
  double minimum=1000000.0;
  double MAX=0.0;
  CMatrix x;

  for(b=0;b<c[a].size();b++)
    {
      ostringstream oss;
      oss<<b;
      filenamestr=base+oss.str();
      filenamestr=filenamestr+ext;
      filename = new char[filenamestr.size() + 1];
      filenamestr.copy(filename,string::npos);
      filename[filenamestr.size()] = 0;

      ofstream coeff(filename);
      delete [] filename;
      x=inversion_angle(a,b);
      MAX=0;
      maximum=0.0;
      minimum=1000000.0;
      for(i=0;i<x.getRowDimension();i++)
        for(j=0;j<x.getColDimension();j++)
          {
            if(real(x.getr(i, j))>maximum)
              maximum=real(x.getr(i, j));
            if(real(x.getr(i, j))<minimum)
              minimum=real(x.getr(i, j));
          }
      if (fabs(maximum)>fabs(MAX))
        MAX=maximum;
      if(fabs(minimum)>fabs(MAX))
        MAX=minimum;

      //x=c[a][b];
      coeff<<"P2"<<endl;
      coeff<<x.getRowDimension()<<" ";
      coeff<<x.getColDimension()<<endl;
      coeff<<255<<endl;
      for(j=0;j<x.getColDimension();j++)
        for(i=0;i<x.getRowDimension();i++)
          //coeff<<(int)(real(x.getr(i, j))/tab_MAX[b]*127.0)+128<<endl;
	  coeff<<(int)(real(x.getr(i, j))/MAX*127.0)+128<<endl;
      coeff.close();
    }
    cout<<"Sauvegarde terminee"<<endl;
}

int fdct_wrapping_invsepangle(double XL1, double XL2, int nbangle, std::vector<CMatrix>& csc, CMatrix& Xhgh)
{
  typedef pair<int, int> intpair;
  map<intpair, fftw_plan> planmap;
  complex <double> tmp;  
  int XS1, XS2;
  int XF1, XF2; 
  double XR1, XR2;
  fdct_wrapping_rangecompute(XL1, XL2, XS1, XS2, XF1, XF2, XR1, XR2);
  Xhgh.set_offsets(true);
  Xhgh.resize(XS1, XS2);
  
  int nbquadrants = 4;
  int nd = nbangle / 4;
  int wcnt = 0;
  
  //backup
  CMatrix Xhghb(Xhgh);
  double XL1b = XL1;
  double XL2b = XL2;

  int qvec[] = {2,1,0,3};
  for(int qi=0; qi<nbquadrants; qi++) 
    {
      int q = qvec[qi];
      //ROTATE data to its right position
      if(q%2==0)
	{
	  XL1 = abs(XL1b);
	  XL2 = abs(XL2b);
	}
      else
	{
	  XL2 = abs(XL1b);
	  XL1 = abs(XL2b);
	}
      fdct_wrapping_rotate_forward(q, Xhghb, Xhgh);
      //figure out XS, XF, XR
      double XW1 = XL1/nd;
      double XW2 = XL2/nd;
      int XS1, XS2;
      int XF1, XF2; 
      double XR1, XR2;
      fdct_wrapping_rangecompute(XL1, XL2, XS1, XS2, XF1, XF2, XR1, XR2);
      for(int w=nd-1; w>=0; w--) 
	{
	  double xs = XR1/4 - (XW1/2)/4;
	  double xe = XR1;
	  double ys = -XR2 + (w-0.5)*XW2;
	  double ye = -XR2 + (w+1.5)*XW2; //x range
	  int xn = int(ceil(xe-xs));
	  int yn = int(ceil(ye-ys));
	  //MAKE THEM ODD
	  if(xn%2==0) xn++;
	  if(yn%2==0) yn++;
	  int xf = int(ceil(xs));
	  //int yf = int(ceil(ys));
	  //theta
	  double thts, thtm, thte; //y direction
	  if(w==0) 
	    {
	      thts = atan2(-1.0, 1.0-1.0/nd);
	      thtm = atan2(-1.0+1.0/nd, 1.0);
	      thte = atan2(-1.0+3.0/nd, 1.0);
	    } 
	  else 
	    if(w==nd-1) 
	      {
		thts = atan2(-1.0+(2.0*w-1.0)/nd, 1.0);
		thtm = atan2(-1.0+(2.0*w+1.0)/nd, 1.0);
		thte = atan2(1.0, 1.0-1.0/nd);
	      }
	    else
	      {
		thts = atan2(-1.0+(2.0*w-1.0)/nd, 1.0);
		thtm = atan2(-1.0+(2.0*w+1.0)/nd, 1.0);
		thte = atan2(-1.0+(2.0*w+3.0)/nd, 1.0);
	      }
	  //load
	  xn = csc[wcnt].getRowDimension();
	  yn = csc[wcnt].getColDimension();
	  int xh = xn/2;
	  int yh = yn/2; //half length
	  CMatrix* wpdata=new CMatrix(xn, yn, true);
	  CMatrix* tpdata=new CMatrix(csc[wcnt]);
	  //fft
	  fftw_plan p = NULL;
	  map<intpair, fftw_plan>::iterator mit=planmap.find( intpair(xn, yn) );
	  if(mit!=planmap.end()) 
	    {
	      p = fftw_plan_dft_2d(yn, xn, (fftw_complex *)tpdata->data(), (fftw_complex *)tpdata->data(),FFTW_FORWARD,FFTW_ESTIMATE);;
	      (*mit).second=p;
	    } 
	  else 
	    {
	      p = fftw_plan_dft_2d(yn, xn, (fftw_complex *)tpdata->data(), (fftw_complex *)tpdata->data(),FFTW_FORWARD,FFTW_ESTIMATE);;
	      planmap[ intpair(xn, yn) ] = p;
	    }
	  fftw_execute(p);
	    
	  double sqrtprod = sqrt(double(xn*yn));
	  
	  for(int i=0; i<xn; i++)
	    for(int j=0; j<yn; j++)
	      {
		tmp=complex <double>(tpdata->getr(i, j).real()/sqrtprod, tpdata->getr(i, j).imag()/sqrtprod);
		tpdata->setr(i, j, tmp);
	      }
	  //fftshift
	  CMatrix * rpdata= new CMatrix(tpdata->getRowDimension(), tpdata->getColDimension(), true);
	  fdct_wrapping_fftshift(*tpdata,*rpdata);
	  //rotate forward
	  fdct_wrapping_rotate_forward(q, *rpdata, *wpdata);
	  xn = wpdata->getRowDimension();
	  yn = wpdata->getColDimension();
          xh = xn/2;
	  yh = yn/2; //half length
		
	  double R21 = XR2/XR1; //ratio
	  for(int xcur=xf; xcur<xe; xcur++)
	    { //for each layer
	      int yfm = (int)ceil( max(-XR2, R21*xcur*tan(thts)) );
	      int yto = (int)floor( min(XR2, R21*xcur*tan(thte)) );
	      for(int ycur=yfm; ycur<=yto; ycur++) 
		{
		  int tmpx = xcur%xn;
		  if(tmpx<-xh)
		    tmpx+=xn;
		  if(tmpx>=-xh+xn)
		    tmpx-=xn;
		  int tmpy = ycur%yn;
		  if(tmpy<-yh)
		    tmpy+=yn;
		  if(tmpy>=-yh+yn)
		    tmpy-=yn;
                  
		  //partition of unity
		  double thtcur = atan2(ycur/XR2, xcur/XR1);
		  double wtht;
		  if(thtcur<thtm) 
		    {
		      double l,r; 
		      fdct_wrapping_window((thtcur-thts)/(thtm-thts), l, r);
		      wtht = l;
		    } 
		  else
		    {
		      double l,r; fdct_wrapping_window((thtcur-thtm)/(thte-thtm), l, r);
		      wtht = r;
		    }
		  double pou = wtht;
		  tmp=complex <double>(wpdata->getr(tmpx, tmpy).real()*pou, wpdata->getr(tmpx, tmpy).imag()*pou);
		  wpdata->setr(tmpx, tmpy, tmp);


		  tmp=complex <double>(Xhgh.getr(xcur, ycur).real()+wpdata->getr(tmpx, tmpy).real(), Xhgh.getr(xcur, ycur).imag()+wpdata->getr(tmpx, tmpy).imag());
		  Xhgh.setr(xcur, ycur, tmp);
		}
	    }
	  wcnt++;
          delete wpdata;
          delete rpdata;
          delete tpdata;
	}//w loop
      fdct_wrapping_rotate_backward(q, Xhgh, Xhghb);
    } //qi loop
  Xhgh = Xhghb;
  XL1 = XL1b;  XL2 = XL2b;

  assert(wcnt==nbangle);

  for(map<intpair, fftw_plan>::iterator mit=planmap.begin(); mit!=planmap.end(); ++mit)
    {
      fftw_plan p = (*mit).second;
      fftw_destroy_plan(p);
    }
  return 0;
}

//---------------------
int fdct_wrapping_invwavelet(std::vector<CMatrix>& csc, CMatrix& Xhgh)
{
  assert(csc.size()==1);
  CMatrix& C = csc[0];
  int N1 = C.getRowDimension();
  int N2 = C.getColDimension();
  
  CMatrix T(C);  //CpxNumMat T(N1, N2);  fdct_wrapping_ifftshift(N1, N2, F1, F2, C, T);
  fftw_plan p = fftw_plan_dft_2d(N2,N1, (fftw_complex *)T.data(), (fftw_complex *)T.data(),FFTW_FORWARD,FFTW_ESTIMATE);
  fftw_execute(p);
  fftw_destroy_plan(p);
  double sqrtprod = sqrt(double(N1*N2));
  complex <double>tmp;
  for(int j=0; j<N2; j++)
    for(int i=0; i<N1; i++)
      {
        tmp=complex <double>(T.getr(i, j).real()/sqrtprod, T.getr(i, j).imag()/sqrtprod);
        T.setr(i, j, tmp);
      }
  
  Xhgh.resize(N1, N2);
  fdct_wrapping_fftshift(T, Xhgh);
  return 0;
}

CMatrix Curvelet::inversion_angle(int niveau, int angle)
{
  int i, j, k, l;
  CMatrix x;
  std::vector <std::vector <CMatrix> > toreverse;
  toreverse=c;
  for(i=0;i<(int)c.size();i++)
    {
      for(j=0;j<(int)c[i].size();j++)
	if((i>niveau)||(i==0))
	  for(k=0;k<c[i][j].getRowDimension();k++)
	    for(l=0;l<c[i][j].getColDimension();l++)
	      toreverse[i][j].setr(k, l,complex<double>(0.0,0.0));
	else
	  if((i==niveau)&&(j>angle))
	    {
	      for(k=0;k<c[i][j].getRowDimension();k++)
		for(l=0;l<c[i][j].getColDimension();l++)
		  toreverse[i][j].setr(k, l,complex<double>(0.0,0.0));
	    }
    }
  invangles(x, toreverse);
  for(i=0;i<(int)toreverse.size();i++)
    toreverse[i].clear();
  toreverse.clear();
  return x;
}

// bitmap * Curvelet::inversion()
// {
//   CMatrix x;
//   invangles(x,c);
//   bitmap * inversee = new bitmap();
//   inversee->set_width(width);
//   inversee->set_height(height);
//   double maxr,minr,MAXr;
//   MAXr=0.0;
//   maxr=-1000000.0;
//   minr=1000000.0;
//   double maxi,mini,MAXi;
//   MAXi=0.0;
//   maxi=-1000000.0;
//   mini=1000000.0;
//   int i, j;
//   for(i=0;i<x.getRowDimension();i++)
//     for(j=0;j<x.getColDimension();j++)
//       {
//         if(real(x.getr(i, j))>maxr)
//           maxr=real(x.getr(i, j));
//         if(real(x.getr(i, j))<minr)
//           minr=real(x.getr(i, j));
//         if(imag(x.getr(i, j))>maxi)
//           maxi=imag(x.getr(i, j));
//         if(imag(x.getr(i, j))<mini)
//           mini=imag(x.getr(i, j));
//       }

//   if(fabs(maxr)>fabs(minr))
//     MAXr=maxr;
//   else
//     MAXr=minr;
//   if(fabs(maxi)>fabs(mini))
//     MAXi=maxi;
//   else
//     MAXi=mini;
//   int r;
//   for(i=0;i<x.getRowDimension();i++)
//     {
//       for(j=0;j<x.getColDimension();j++)
//         {
//           r=((int)((real(x.getr(i, j))/MAXr)*255.0));
//           inversee->set_pixel(i, j,r,r,r);
//         }
//     }
//   return inversee;
// }



int Curvelet::invwavelet(CMatrix& x)
{
  int sc, i, j;
  std::vector<int> nbangles(nb_echelle);
  std::vector<CMatrix> Xhghs;
  Xhghs.resize(nb_echelle);
  nbangles[0] = 1;
  for(sc=1; sc<nb_echelle-1; sc++)
    nbangles[sc] = nb_angles * pow2( int(ceil(double(sc-1)/2)) );
  nbangles[nb_echelle-1] = 1;

  fdct_wrapping_invwavelet(c[nb_echelle-1], Xhghs[nb_echelle-1]);
  double XL1 = 2.0*width/3.0;
  double XL2 = 2.0*height/3.0;
  for(sc=nb_echelle-1;sc>-1;sc--)
    fdct_wrapping_invwavelet(c[sc], Xhghs[sc]);

  //-------------------------------------------xhghs to O
  //combine
  CMatrix X(width, height, true);

  XL1 = 4.0*width/3.0;
  XL2 = 4.0*height/3.0;
  int XS1, XS2;
  int XF1, XF2;
  double XR1, XR2;
  fdct_wrapping_rangecompute(XL1, XL2, XS1, XS2, XF1, XF2, XR1, XR2);
  for(sc=nb_echelle-1; sc>0; sc--)
    {
      double XL1n = XL1/2;
      double XL2n = XL2/2;
      int XS1n, XS2n;
      int XF1n, XF2n;
      double XR1n, XR2n;
      complex <double> tmp;
      fdct_wrapping_rangecompute(XL1n, XL2n, XS1n, XS2n, XF1n, XF2n, XR1n, XR2n);
      DblMatrix lowpass(XS1n, XS2n, true);
      fdct_wrapping_lowpasscompute(XL1n, XL2n, lowpass);
      DblMatrix hghpass(XS1n, XS2n, true);
      for(j=-XF2n; j<-XF2n+XS2n; j++)
        for(i=-XF1n; i<-XF1n+XS1n; i++)
          hghpass.setr(i, j, sqrt(1-lowpass.getr(i, j)*lowpass.getr(i, j)));
      for(j=-XF2n; j<-XF2n+XS2n; j++)
        for(i=-XF1n; i<-XF1n+XS1n; i++)
          {
            tmp=complex <double> (Xhghs[sc].getr(i, j).real()*hghpass.getr(i, j), Xhghs[sc].getr(i, j).imag()*hghpass.getr(i, j));
            Xhghs[sc].setr(i, j, tmp);
          }
      for(j=-XF2n; j<-XF2n+XS2n; j++)
        for(i=-XF1n; i<-XF1n+XS1n; i++)
          {
            tmp=complex <double> (Xhghs[sc-1].getr(i, j).real()*lowpass.getr(i, j), Xhghs[sc-1].getr(i, j).imag()*lowpass.getr(i, j));
            Xhghs[sc-1].setr(i, j, tmp);
          }
      CMatrix& G = Xhghs[sc];
      for(j=G.getOh(); j<G.getOh()+G.getColDimension(); j++)
        for(i=G.getOl(); i<G.getOl()+G.getRowDimension(); i++)
          X.setr(i, j, X.getr(i, j)+G.getr(i, j));
      XL1 = XL1/2;
      XL2 = XL2/2;
      fdct_wrapping_rangecompute(XL1, XL2, XS1, XS2, XF1, XF2, XR1, XR2);
    }
  for(j=-XF2; j<-XF2+XS2; j++)
    for(i=-XF1; i<-XF1+XS1; i++)
      X.setr(i, j, X.getr(i, j)+Xhghs[0].getr(i, j));
  // fold
  CMatrix O(width, height, true);
  O = X;
  CMatrix T(width, height);
  fdct_wrapping_ifftshift(O, T);
  fftw_complex * donnees = (fftw_complex *)T.data();
  fftw_plan p = fftw_plan_dft_2d(height, width,donnees,donnees,FFTW_BACKWARD,FFTW_ESTIMATE);
  fftw_execute(p);
  fftw_destroy_plan(p);
  double sqrtprod = sqrt(double(width*height)); //scale
  complex <double> tmp;
  for(i=0; i<width; i++)
    for(j=0; j<height; j++)
      {
        tmp=complex <double> (T.getr(i, j).real()/sqrtprod,T.getr(i, j).imag()/sqrtprod);      
        T.setr(i, j, tmp);
      }
  x = T;
  return 0;
}


int Curvelet::invangles(CMatrix& x, std::vector <std::vector <CMatrix> >& toreverse)
{
  int sc, i, j;
  //-------------------------------------------angles to Xhgh
  std::vector<int> nbangles(nb_echelle);
  std::vector<CMatrix> Xhghs;  Xhghs.resize(nb_echelle);

  nbangles[0] = 1;
  for(sc=1; sc<nb_echelle; sc++)
    nbangles[sc] = nb_angles * pow2( int(ceil(double(sc-1)/2)) );

  double XL1 = 4.0*width/3.0;
  double XL2 = 4.0*height/3.0;
  for(sc=nb_echelle-1; sc>0; sc--)
    {
      fdct_wrapping_invsepangle(XL1, XL2, nbangles[sc], toreverse[sc], Xhghs[sc]);
      XL1 /= 2;
      XL2 /= 2;
    }
  fdct_wrapping_invwavelet(toreverse[0], Xhghs[0]);
  //-------------------------------------------xhghs to O
  //combine
  XL1 = 4.0*width/3.0;
  XL2 = 4.0*height/3.0; //range
  int XS1, XS2;
  int XF1, XF2;
  double XR1, XR2;
  fdct_wrapping_rangecompute(XL1, XL2, XS1, XS2, XF1, XF2, XR1, XR2);
  CMatrix X(XS1, XS2, true);
  XL1 = 4.0*width/3.0;
  XL2 = 4.0*height/3.0;
  fdct_wrapping_rangecompute(XL1, XL2, XS1, XS2, XF1, XF2, XR1, XR2);
  for(sc=nb_echelle-1; sc>0; sc--)
    {
      double XL1n = XL1/2;
      double XL2n = XL2/2;
      int XS1n, XS2n;
      int XF1n, XF2n;
      double XR1n, XR2n;
      complex <double> tmp;
      fdct_wrapping_rangecompute(XL1n, XL2n, XS1n, XS2n, XF1n, XF2n, XR1n, XR2n);
      DblMatrix lowpass(XS1n, XS2n, true);
      fdct_wrapping_lowpasscompute(XL1n, XL2n, lowpass);
      DblMatrix hghpass(XS1n, XS2n, true);
      for(j=-XF2n; j<-XF2n+XS2n; j++)
        for(i=-XF1n; i<-XF1n+XS1n; i++)
          hghpass.setr(i, j,sqrt(1-lowpass.getr(i, j)*lowpass.getr(i, j)));
      for(j=-XF2n; j<-XF2n+XS2n; j++)
        for(i=-XF1n; i<-XF1n+XS1n; i++)
          {
            tmp=complex <double>(Xhghs[sc].getr(i, j).real()*hghpass.getr(i, j), Xhghs[sc].getr(i, j).imag()*hghpass.getr(i, j));
            Xhghs[sc].setr(i, j, tmp);
          }
      for(j=-XF2n; j<-XF2n+XS2n; j++)
        for(i=-XF1n; i<-XF1n+XS1n; i++)
          {
            tmp=complex <double>(Xhghs[sc-1].getr(i, j).real()*lowpass.getr(i, j), Xhghs[sc-1].getr(i, j).imag()*lowpass.getr(i, j));
            Xhghs[sc-1].setr(i, j, tmp);
          }
      CMatrix& G = Xhghs[sc];
      for(j=G.getOh(); j<G.getOh()+G.getColDimension(); j++)
        for(i=G.getOl(); i<G.getOl()+G.getRowDimension(); i++)
          X.setr(i, j, X.getr(i, j) + G.getr(i, j));
      XL1 = XL1/2;
      XL2 = XL2/2;
      fdct_wrapping_rangecompute(XL1, XL2, XS1, XS2, XF1, XF2, XR1, XR2);
    }
  for(j=-XF2; j<-XF2+XS2; j++)
    for(i=-XF1; i<-XF1+XS1; i++)
      X.setr(i, j, X.getr(i, j) + Xhghs[0].getr(i, j));
  // fold
  CMatrix O(width, height, true);
  XL1 = 4.0*width/3.0;
  XL2 = 4.0*height/3.0;
  fdct_wrapping_rangecompute(XL1, XL2, XS1, XS2, XF1, XF2, XR1, XR2);
  //times pou;
  DblMatrix lowpass(XS1, XS2, true);
  fdct_wrapping_lowpasscompute(XL1, XL2, lowpass);
  complex <double> tmp;
  for(j=-XF2; j<-XF2+XS2; j++)
    for(i=-XF1; i<-XF1+XS1; i++)
      {
        tmp=complex <double>(X.getr(i, j).real()*lowpass.getr(i, j), X.getr(i, j).imag()*lowpass.getr(i, j));
        X.setr(i, j, tmp);
      }
  IntVecteur t1(XS1, true);
  for(i=-XF1; i<-XF1+XS1; i++)
    if(i<-width/2)
      t1.set(i, i+int(width));
    else
      if(i>(width-1)/2)
        t1.set(i, i-int(width));
      else
        t1.set(i, i);
  IntVecteur t2(XS2, true);
  for(i=-XF2; i<-XF2+XS2; i++)
    if(i<-height/2)
      t2.set(i, i+int(height));
    else
      if(i>(height-1)/2)
        t2.set(i, i-int(height));
      else
        t2.set(i, i);
  for(j=-XF2; j<-XF2+XS2; j++)
    for(i=-XF1; i<-XF1+XS1; i++)
      O.setr(t1.get(i), t2.get(j),O.getr(t1.get(i), t2.get(j)) + X.getr(i, j));

  CMatrix T(width, height);
  fdct_wrapping_ifftshift(O, T);
  fftw_plan p = fftw_plan_dft_2d(height, width, (fftw_complex *)T.data(), (fftw_complex *)T.data(),FFTW_BACKWARD,FFTW_ESTIMATE);;
  fftw_execute(p);
  fftw_destroy_plan(p);
  double sqrtprod = sqrt(double(width*height)); //scale
  for(i=0; i<width; i++)
    for(j=0; j<height; j++)
      {
        tmp=complex <double> (T.getr(i, j).real()/sqrtprod, T.getr(i, j).imag()/sqrtprod);      
        T.setr(i, j, tmp);
      }
  x = T;
  return 0;
}


Curvelet Curvelet :: operator= (Curvelet arg)
{
  nb_echelle=arg.getScale();
  nb_angles=arg.getAngles();
  width=arg.getWidth();
  height=arg.getHeight();
  c=arg.getC();
  return (*this);
}

std::vector <std::vector <float> > Curvelet::indices_texture()
{
  for(size_t i=0; i<c.size(); i++)
     for(size_t j=0; j<c[i].size(); j++)//Pour tous les secteurs angulaires
       {
         float somme=0.0; 
	 float nb_elements=0.0; 
	 float somme_carres=0.0;
         for(int k=0; k<c[i][j].getRowDimension(); k++)
	   for(int l=0; l<c[i][j].getColDimension(); l++)
	     {
	       const float module=real(c[i][j].getr(k, l))*real(c[i][j].getr(k, l))+imag(c[i][j].getr(k, l))*imag(c[i][j].getr(k, l));
	       somme+=sqrt(module);
	       somme_carres+=module;
	       nb_elements+=1.0;
	     }
         moyennes.push_back(somme/nb_elements);
         ecarts_types.push_back((somme_carres/nb_elements)-((somme/nb_elements)*(somme/nb_elements)));
       }
  std::vector <std::vector <float> > retour;
  retour.push_back(ecarts_types);
  retour.push_back(moyennes);
  return retour;
}


void
Curvelet::get_coeff(girl::shared_ptr<girl::tools::ImageFeature> &imgF)
{
  //const int size = nb_angles;
  const int scale = getScale()-1;
  const int angles = getAnglesForLevel(scale);

  imgF = girl::shared_ptr<girl::tools::ImageFeature>(new girl::tools::ImageFeature(width, height, angles));

  std::vector< std::vector<double> > sx, sy, fx, fy;
  std::vector< std::vector< int> > nx;
  std::vector< std::vector< int> > ny;

  fdct_wrapping_param(width, height, nb_echelle, nb_angles, 1, sx, sy, fx, fy, nx, ny);

  int error = 0 ;
  for (size_t i = 0 ; i < nx.size() ; ++i)
    for (size_t j = 0 ; j < nx[i].size(); ++j)
      if (nx[i][j] == 0 )
	++error;
  for (size_t i = 0 ; i < ny.size() ; ++i)
    for (size_t j = 0 ; j < ny[i].size(); ++j)
      if (ny[i][j] == 0)
	++error;
  
  if (error != 0) {
    std::cerr<<"erreur Curvelet::get_coeff -> scale ="<<nb_echelle<<" angles= "<<nb_angles<<std::endl;
    return;
  }
  

  for (int h = 0 ; h < height ; ++h ) {
    girl::tools::ImageFeature::type *buf = imgF->scanLine(h);
    for (int w = 0 ; w < width ; ++w ) {
      for (int alpha = 0 ; alpha < angles ; ++alpha) {
	int I = w*nx[scale][alpha]/width;
	int J = h*ny[scale][alpha]/height;
	buf[alpha] = std::norm( get_coeff(scale, alpha, I, J) );
      }
      buf += imgF->formatSize();
    }
  }
}
