#ifndef _CURVELET_INLINE_HPP
#define _CURVELET_INLINE_HPP
#include "vecteur.hpp"

inline int fdct_wrapping_fftshift(CMatrix& T, CMatrix& O)
{
  int width = T.getRowDimension();
  int height = T.getColDimension();
  O.resize(width,height);
  O.set_offsets(true);

  int F1 = width/2;
  int F2 = height/2;
  IntVecteur p1(width,true);
  int i,j;
  for(i=-F1; i<-F1+width; i++)
    p1.set(i,(i+width)%width);
  IntVecteur p2(height,true);
  for(i=-F2; i<-F2+height; i++)
    p2.set(i,(i+height)%height);
  for(j=-F2; j<-F2+height; j++)
    for(i=-F1; i<-F1+width; i++)
      O.setr(i,j, T.getr(p1.get(i),p2.get(j)));
  return 0;
}

inline int fdct_wrapping_ifftshift(CMatrix& O, CMatrix& T)
{
  int width = O.getRowDimension();
  int height = O.getColDimension();
  T.resize(width,height);
  int F1 = width/2;
  int F2 = height/2;
  int i,j;
  IntVecteur p1(width,true);
  for(i=-F1; i<-F1+width; i++)
    p1.set(i, (i+width)%width);
  IntVecteur p2(height,true);
  for(i=-F2; i<-F2+height; i++)
    p2.set(i,(i+height)%height);
  for(j=-F2; j<-F2+height; j++)
    for(i=-F1; i<-F1+width; i++)
      T.setr(p1.get(i),p2.get(j), O.getr(i,j));
  return 0;
}

inline int fdct_wrapping_window(double x, double& l, double& r)
{
  if(x<=0)
    {
      l = 0;
      r = 1;
    }
  else
    if(x>=1)
      {
        l = 1;
        r = 0;
      }
    else
      {
        l = exp(1-1/(1-exp(1-1/(1-x))));
        r = exp(1-1/(1-exp(1-1/x)));
        double norm = sqrt(l*l+r*r);
        l /= norm;
        r /= norm;
      }
  return 0;
}

inline int fdct_wrapping_rangecompute(double XL1, double XL2, int& XS1, int& XS2, int& XF1, int& XF2, double& XR1, double& XR2)
{
  XS1 = 2*int(floor(XL1/2))+1;
  XS2 = 2*int(floor(XL2/2))+1; //number of samples
  XF1 = int(floor(XL1/2));
  XF2 = int(floor(XL2/2)); //offset on either side
  XR1 = XL1/2;
  XR2 = XL2/2;
  return 0;
}

inline int fdct_wrapping_lowpasscompute(double XL1, double XL2, DblMatrix& lowpass)
{
  int XS1 = 2*int(floor(XL1/2))+1;
  int XS2 = 2*int(floor(XL2/2))+1; //number of samples
  int XF1 = int(floor(XL1/2));
  int XF2 = int(floor(XL2/2)); //offset on either side
  double XR1 = XL1/2;
  double XR2 = XL2/2;

  DblVecteur lowpass1(XS1,true);
  lowpass1.setvalue(1.0);
  int i,j;
  double x,l,r;
  for(i=-XF1; i<-XR1/2; i++)
    {
      x = (i+XR1)/(XR1/2);
      fdct_wrapping_window(x, l, r);
      lowpass1.set(i,l);
      lowpass1.set(-i,l);
    }
  DblVecteur lowpass2(XS2,true);
  lowpass2.setvalue(1.0);
  for(i=-XF2; i<-XR2/2; i++)
    {
      x = (i+XR2)/(XR2/2);
      fdct_wrapping_window(x, l, r);
      lowpass2.set(i,l);
      lowpass2.set(-i,l);
    }
  for(i=-XF1; i<-XF1+XS1; i++)
    for(j=-XF2; j<-XF2+XS2; j++)
      lowpass.setr(i,j, lowpass1.get(i) * lowpass2.get(j));
  return 0;
}

template <class F>
inline int fdct_wrapping_rotate_forward(int f, F XL1, F XL2, F& TL1, F& TL2)
{
  if(f==0) { //time 1
    TL1 = XL1; TL2 = XL2;  }
  else
    if(f==1) { //time -i
      TL1 = XL2; TL2 = -XL1;  }
    else
      if(f==2) { //time -1
        TL1 = -XL1; TL2 = -XL2;  }
      else
        if(f==3) { //time i
          TL1 = -XL2; TL2 = XL1;  }
  return 0;
}

template <class F>
inline int fdct_wrapping_rotate_backward(int f, F XL1, F XL2, F& TL1, F& TL2)
{
  if(f==0) 
    {
      TL1 = XL1;
      TL2 = XL2;
    } 
  else 
    if(f==1)
      {
	TL1 = -XL2;
	TL2 = XL1;
      } 
    else
      if(f==2) 
	{
	  TL1 = -XL1;
	  TL2 = -XL2;
	}
      else 
	if(f==3) 
	  {
	    TL1 = XL2;
	    TL2 = -XL1;
	  }
  return 0;
}

template <class F>//inline int fdct_wrapping_rotate_forward(int f, CpxOffMat& X, CpxOffMat& T)
inline int fdct_wrapping_rotate_forward(int f, Matrix<F>& X, Matrix<F>& T)
{
  //rotate face f to the position of face 0
  int XS1 = X.getRowDimension();
  int XS2 = X.getColDimension();
  int XF1 =-X.getOl();
  int XF2 =-X.getOh();
  int TS1, TS2;
  int TF1, TF2;
  int i,j;
  if(f==0)
    {
      TS1 = XS1;
      TS2 = XS2;
      TF1 = XF1;
      TF2 = XF2;	 //TL1 = XL1;	 TL2 = XL2;
      T=Matrix<F>(TS1, TS2,true);
      for(j=-TF2; j<-TF2+TS2; j++)
        for(i=-TF1; i<-TF1+TS1; i++)
          T.setr(i,j, X.getr(i,j));
    }
  else
    if(f==1)
      {
        TS1 = XS2;
        TS2 = XS1;
        TF1 = XF2;
        TF2 = XF1;	 //TL1 = XL2;	 TL2 = XL1;
        T=Matrix<F>(TS1, TS2,true);
        for(j=-TF2; j<-TF2+TS2; j++)
          for(i=-TF1; i<-TF1+TS1; i++)
            T.setr(i,j,X.getr(-j,i));
      }
    else
      if(f==2)
        {
          TS1 = XS1;
          TS2 = XS2;
          TF1 = XF1;
          TF2 = XF2;	 //TL1 = XL1;	 TL2 = XL2;
          T=Matrix<F>(TS1, TS2,true);
          for(j=-TF2; j<-TF2+TS2; j++)
            for(i=-TF1; i<-TF1+TS1; i++)
              T.setr(i,j,X.getr(-i,-j));
        }
      else
        if(f==3)
          {
            TS1 = XS2;
            TS2 = XS1;
            TF1 = XF2;
            TF2 = XF1;	 //TL1 = XL2;	 TL2 = XL1;
            T=Matrix<F>(TS1, TS2,true);
            for(j=-TF2; j<-TF2+TS2; j++)
              for(i=-TF1; i<-TF1+TS1; i++)
                T.setr(i,j,X.getr(j,-i));
          }
  return 0;
}

template <class F>//inline int fdct_wrapping_rotate_backward(int f, CpxOffMat& X, CpxOffMat& T)
inline int fdct_wrapping_rotate_backward(int f, Matrix<F>& X, Matrix<F>& T)
{
  int XS1 = X.getRowDimension();
  int XS2 = X.getColDimension();
  int XF1 =-X.getOl();
  int XF2 =-X.getOh();
  int TS1, TS2;
  int TF1, TF2;
  int i,j;
  if(f==0)
    {
      TS1 = XS1;
      TS2 = XS2;
      TF1 = XF1;
      TF2 = XF2;	 //TL1 = XL1;	 TL2 = XL2;
      T=Matrix<F>(TS1, TS2,true);
      for(j=-TF2; j<-TF2+TS2; j++)
        for(i=-TF1; i<-TF1+TS1; i++)
          T.setr(i,j, X.getr(i,j));
    }
  else
    if(f==1)
      {
        TS1 = XS2;
        TS2 = XS1;
        TF1 = XF2;
        TF2 = XF1;	 //TL1 = XL2;	 TL2 = XL1;
        T=Matrix<F>(TS1, TS2,true);
        for(j=-TF2; j<-TF2+TS2; j++)
          for(i=-TF1; i<-TF1+TS1; i++)
            T.setr(i,j,X.getr(j,-i));
      }
    else
      if(f==2)
        {
          TS1 = XS1;
          TS2 = XS2;
          TF1 = XF1;
          TF2 = XF2;	 //TL1 = XL1;	 TL2 = XL2;
          T=Matrix<F>(TS1, TS2,true);
          for(j=-TF2; j<-TF2+TS2; j++)
            for(i=-TF1; i<-TF1+TS1; i++)
              T.setr(i,j, X.getr(-i,-j));
        }
      else
        if(f==3)
          {
            TS1 = XS2;
            TS2 = XS1;
            TF1 = XF2;
            TF2 = XF1;	 //TL1 = XL2;	 TL2 = XL1;
            T=Matrix<F>(TS1, TS2,true);
            for(j=-TF2; j<-TF2+TS2; j++)
              for(i=-TF1; i<-TF1+TS1; i++)
                T.setr(i,j, X.getr(-j,i));
          }
  return 0;
}

/*inline double energy(CpxOffMat& m)
{
  double val=0;
  cpx* data = m.data();
  int i;
  for(i=0; i<m.m()*m.n(); i++)
    {	 //val += data[i].re*data[i].re + data[i].im*data[i].im;
      val += norm(data[i]);
    }
  return val;
}

inline double energy(DblOffMat& m)
{
  double val=0;
  double* data = m.data();
  int i;
  for(i=0; i<m.m()*m.n(); i++)
    {	 //val += data[i].re*data[i].re + data[i].im*data[i].im;
      val += data[i]*data[i];
    }
  return val;
}

inline double energy(CpxNumMat& m)
{
  double val=0;
  cpx* data = m.data();
  int i;
  for(i=0; i<m.m()*m.n(); i++)
    {	 //val += data[i].re*data[i].re + data[i].im*data[i].im;
      val += norm(data[i]);
    }
  return val;
}

inline double energy(DblNumMat& m)
{
  double val=0;
  double* data = m.data();
  int i;
  for(i=0; i<m.m()*m.n(); i++)
    {	 //val += data[i].re*data[i].re + data[i].im*data[i].im;
      val += data[i]*data[i];
    }
  return val;
  }*/

#endif
