#ifndef _VECTEUR_HPP_
#define _VECTEUR_HPP_
#include <complex>
#include <string.h>
using namespace std;

template <class F>
class Vecteur {
private:
  /**
   * \brief Taille du vecteur
   *
   * Taille du vecteur
   */
  int m;
  /**
   * \brief Offset éventuel
   *
   * Offset éventuel
   */
  int o;
  /**
   * \brief Flag indiquant l'utilisation ou non d'un offset
   *
   * Flag indiquant l'utilisation ou non d'un offset
   */
  bool offset;
  /**
   * \brief Tableau monodimensionnel contenant le vecteur
   *
   * Tableau monodimensionnel contenant le vecteur
   */
  F* donnees;
public:
  /**
   * \brief Constructeur par défaut
   *
   * Constructeur par défaut
   */
  Vecteur(int taille=0, bool setoff=false)
    : m(taille), o(0), offset(setoff), donnees(NULL)
  {
    if(m>0)
      {
	donnees = new F[m];
	memset(donnees, 0, m*sizeof(F));
	if(offset)
	  o=-m/2;
      }
    else
      donnees=NULL;
  }
  /**
   * \brief Constructeur par recopie
   *
   * Constructeur par recopie
   */
  Vecteur(const Vecteur& V)
    : m(V.m), o(V.o), offset(V.offset), donnees(NULL)
  {
    if(m>0)
      {
	donnees = new F[m];
	memset(donnees, 0, m*sizeof(F));
	memcpy(donnees, V.donnees, m*sizeof(F));
      }
    else
      donnees=NULL;    
  }
  /**
   * \brief Destructeur
   *
   * Destructeur
   */
  ~Vecteur()
  {
    if(m>0)
      {
	delete [] donnees;
	donnees=NULL;
      }
  }
  /**
   * \brief Opérateur d'affectation
   *
   * Opérateur d'affectation
   */
  Vecteur& operator=(const Vecteur &V)
  {
    if (&V != this) {
      if(m>0)
	{
	  delete [] donnees;
	  donnees=NULL;
	}
      m=V.m;
      offset=V.offset;
      o=V.o;
      if(m>0)
	{
	  donnees = new F[m];
	  memset(donnees,0,m*sizeof(F));
	  memcpy(donnees, V.donnees,m*sizeof(F));
	}
      else
	donnees=NULL;
    }
    return *this;
  }

  /**
   * \brief Fonction de redimensionnement
   * 
   * Fonction de redimensionnement
   */
  void resize(int taille) const{
  if(taille!=m)
    {
      if(taille>0)
	{
	  if(m>0)
	    {
	      delete [] donnees;
	      donnees=NULL;
	    }
	  m=taille;
	  if(offset)
	    o=-m/2;
	  donnees = new F[m];
	  memset(donnees, 0, m*sizeof(F));
	}
    }
  }
  /**
   * \brief Accesseur en lecture à une valeur du vecteur
   * \param i Index de la valeur à accéder
   *
   * Accesseur en lecture à une valeur du vecteur
   */
  F get(int i) {return donnees[i-o];}
  /**
   * \brief Accesseur en écriture à une valeur du vecteur
   * \param i Index de la valeur à accéder
   * \param v Valeur à écrire
   *
   * Accesseur en écriture à une valeur du vecteur
   */
  void set(int i, F v){donnees[i-o]=v;}
  /**
   * \brief Accesseur en lecture à la taille du vecteur
   *
   * Accesseur en lecture à la taille du vecteur
   */
  int getSize();
  /**
   * \brief Indique si le vecteur utilise un offset ou non
   *
   * Indique si le vecteur utilise un offset ou non
   */
  bool getOffset();
  /**
   * \brief Accès à la valeur de l'offset
   *
   * Accès à la valeur de l'offset
   */
  int getO();
  /**
   * \brief Accès aux valeurs du vecteur
   *
   * Accès aux valeurs du vecteur. A utiliser avec parcimonie
   */
  F* data()const;
  /**
   * \brief Initialisation du vecteur
   * \param val Valeur initialisante
   *
   * Initialisation du vecteur
   */
  void setvalue(F val)
  {
    int i;
    for(i=o;i<o+m;i++)
      donnees[i-o]=val;
  }
};

typedef Vecteur<int> IntVecteur;
typedef Vecteur<float> FltVecteur;
typedef Vecteur<double> DblVecteur;
typedef Vecteur<complex <double> > CVecteur;
#endif //_VECTEUR_HPP_
