#include "ImageFeatureGaborFiltersSettings.hpp"

#include <iostream>
#include <sstream>

#include <ise/iseImageFeatureGaborFilters.hpp>

#include "ImageFeatureGaborFiltersWidget.hpp"

ImageFeatureGaborFiltersSettings::ImageFeatureGaborFiltersSettings()
    : m_sigma(),
      m_theta(),
      m_lambda(),
      m_gamma(),
      m_psi(),
      m_filterSize(),
      m_normalizeImage(),
      m_normalizeKernel(),
      m_normalizeContrast()
{
  ise::ImageFeatureGaborFilters::getDefaultParameters(m_filterSize, m_sigma, m_theta, m_lambda, m_gamma, m_psi, m_normalizeImage, m_normalizeKernel, m_normalizeContrast);

}

void
ImageFeatureGaborFiltersSettings::configureWidget(FeatureInterfaceWidget *widget) const
{
    ImageFeatureGaborFiltersWidget* w = static_cast<ImageFeatureGaborFiltersWidget*>(widget);
    assert(w);
    w->setParameters(ImageFeatureGaborFiltersSettings::toString(m_sigma),
                     ImageFeatureGaborFiltersSettings::toString(m_theta),
                     ImageFeatureGaborFiltersSettings::toString(m_lambda),
                     ImageFeatureGaborFiltersSettings::toString(m_gamma),
                     ImageFeatureGaborFiltersSettings::toString(m_psi),
                     m_normalizeImage,
                     m_normalizeKernel,
                     m_normalizeContrast);
}

void
ImageFeatureGaborFiltersSettings::getWidgetConfig(const FeatureInterfaceWidget *widget)
{
     const ImageFeatureGaborFiltersWidget* w = static_cast<const ImageFeatureGaborFiltersWidget*>(widget);
     std::string sigma, theta, lambda, gamma, psi;
     w->getParameters(sigma, theta, lambda, gamma, psi, m_normalizeImage, m_normalizeKernel, m_normalizeContrast);
     ImageFeatureGaborFiltersSettings::toDoubleVect(sigma, m_sigma);
     ImageFeatureGaborFiltersSettings::toDoubleVect(theta, m_theta);
     ImageFeatureGaborFiltersSettings::toDoubleVect(lambda, m_lambda);
     ImageFeatureGaborFiltersSettings::toDoubleVect(gamma, m_gamma);
     ImageFeatureGaborFiltersSettings::toDoubleVect(psi, m_psi);
}

void
ImageFeatureGaborFiltersSettings::configureAlgo(girl::shared_ptr<ise::FeatureComputer> algo) const
{
    assert(algo != NULL);
    girl::shared_ptr<ise::ImageFeatureGaborFilters> algorithm =
            girl::static_pointer_cast<ise::ImageFeatureGaborFilters>(algo);
    algorithm->setParameters(m_filterSize, m_sigma, m_theta, m_lambda, m_gamma, m_psi, m_normalizeImage, m_normalizeKernel, m_normalizeContrast);
}

girl::shared_ptr<FeatureInterfaceSettings>
ImageFeatureGaborFiltersSettings::clone() const
{
    girl::shared_ptr<ImageFeatureGaborFiltersSettings> p(new ImageFeatureGaborFiltersSettings(*this));
    return p;
}

bool
ImageFeatureGaborFiltersSettings::isValid() const
{
    return !m_sigma.empty() && !m_gamma.empty() && !m_theta.empty() && !m_lambda.empty() && !m_psi.empty();
}

std::string
ImageFeatureGaborFiltersSettings::toString(const std::vector<double> &values)
{
    std::stringstream ss(std::ios::in|std::ios::out);
    std::vector<double>::const_iterator it = values.begin();
    const std::vector<double>::const_iterator itEnd = values.end();
    for (; it != itEnd; ++it) {
        ss << *it << " ";
    }
    std::string s(ss.str());
    return s;
}

void
ImageFeatureGaborFiltersSettings::toDoubleVect(const std::string &s, std::vector<double> &vect)
{
    vect.clear();
    std::stringstream ss(std::ios::in|std::ios::out);
    ss << s;
    double val;
    while (ss >> val)
        vect.push_back(val);
}
