#include "ImageFeaturePixelClassificationSettings.hpp"

#include <ise/iseImageFeaturePixelClassification.hpp>

#include "ImageFeaturePixelClassificationWidget.hpp"

ImageFeaturePixelClassificationSettings::ImageFeaturePixelClassificationSettings()
    : FeatureInterfaceSettings(),
      m_algo(ise::FeatureComputerPtr()),
      m_weights(),
      m_model()
{}

void
ImageFeaturePixelClassificationSettings::configureWidget(FeatureInterfaceWidget *widget) const
{
    static_cast<ImageFeaturePixelClassificationWidget*>(widget)->setParameters(m_algo, m_weights, m_model);
}

void
ImageFeaturePixelClassificationSettings::getWidgetConfig(const FeatureInterfaceWidget *widget)
{
    static_cast<const ImageFeaturePixelClassificationWidget*>(widget)->getParameters(m_algo, m_weights, m_model);
}

void
ImageFeaturePixelClassificationSettings::configureAlgo(girl::shared_ptr<ise::FeatureComputer> algo) const
{
    assert(algo != NULL);
    girl::shared_ptr<ise::ImageFeaturePixelClassification> algorithm =
            girl::static_pointer_cast<ise::ImageFeaturePixelClassification> (algo);

    girl::shared_ptr<ise::ImageFeatureComputer> gaborImage =
            girl::static_pointer_cast<ise::ImageFeatureComputer>(m_algo);
    assert(gaborImage);
    algorithm->setParameters(m_model.c_str(), gaborImage);
}

girl::shared_ptr<FeatureInterfaceSettings>
ImageFeaturePixelClassificationSettings::clone() const
{
    girl::shared_ptr<ImageFeaturePixelClassificationSettings> p(new ImageFeaturePixelClassificationSettings);
    p->m_algo = m_algo;
    p->m_weights = m_weights;
    p->m_model = m_model;
    return p;
}

bool
ImageFeaturePixelClassificationSettings::isValid() const
{
  return m_algo != girl::shared_ptr<ise::ImageFeaturePixelClassification>()  && !m_model.empty();
}
