/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "ImageFeaturePixelClassificationWidget.hpp"

#include <QString>
#include <QLabel>
#include <QGroupBox>
#include <QGridLayout>
#include <QComboBox>
#include <QFileDialog>
#include <QPushButton>

#include <qiseTools/WeightedChannelsImageFeatureWidget.hpp>
#include <qiseTools/ImageFeatureChoiceWidget.hpp>

ImageFeaturePixelClassificationWidget::ImageFeaturePixelClassificationWidget(MainView *mainView,
						       FeatureComputerGUIInfoPtr featureComputerGUIInfo, 
						       QString name,
						       ImageFeaturePlugin *imageFeaturePlugin, 
						       QWidget *parent)
  : ImageFeatureWidgetInterface(mainView, featureComputerGUIInfo, name, imageFeaturePlugin, parent),
    m_wcifw(NULL),
    m_openFileButton(NULL),
    m_modelFile()

{
  setAttribute(Qt::WA_DeleteOnClose);

  build();
}

void
ImageFeaturePixelClassificationWidget::build()
{
  assert(m_mainView->model()->getSegContextPtr());


  QGroupBox *imgfGroup = new QGroupBox(QString("Image Feature"));
  QVBoxLayout *layoutGroup = new QVBoxLayout(imgfGroup);
  m_wcifw = new WeightedChannelsImageFeatureWidget(m_mainView, m_featureComputerGUIInfo, imgfGroup);
  layoutGroup->addWidget(m_wcifw);

  m_openFileButton = new QPushButton(tr("Open model"), this);

  connect(m_openFileButton, SIGNAL(clicked()), this, SLOT(chooseFile()));

  QVBoxLayout *layout = new QVBoxLayout();
  layout->setAlignment(Qt::AlignTop);
  layout->addWidget(m_openFileButton);
  layout->addWidget(imgfGroup);
  setLayout(layout);
}

bool
ImageFeaturePixelClassificationWidget::isValid()
{
  //return (m_wcifw->imageFeatureComputerPtr() != NULL && !m_modelFile.empty());
    return false;
}

void
ImageFeaturePixelClassificationWidget::setParameters(const ise::FeatureComputerPtr &algo,
                                                     const std::vector<double> &weightedChannelsCoeffs,
                                                     const std::string &file)
{
  m_wcifw->setImageFeatureComputerPtr(algo);
  if (weightedChannelsCoeffs.size() > 0)
    m_wcifw->setWeights(weightedChannelsCoeffs);

  m_modelFile = file;
}

void
ImageFeaturePixelClassificationWidget::getParameters(ise::FeatureComputerPtr &algo, std::vector<double> &weightedChannelsCoeffs, std::string &file) const
{
  algo = m_wcifw->imageFeatureComputerPtr();
  m_wcifw->weights(weightedChannelsCoeffs);
  file = m_modelFile;
}	 

void
ImageFeaturePixelClassificationWidget::chooseFile()
{
    QString res = QFileDialog::getOpenFileName(this, tr( "Choose the classification model" ), "~");
    if( !res.isNull() )
    {
        m_modelFile = res.toStdString();
    }
    else
    {
        m_modelFile.clear();
    }
}
