#include "ImageFeatureRelaxationSettings.hpp"

#include <ise/iseImageFeatureRelaxation.hpp>

#include "ImageFeatureRelaxationWidget.hpp"

ImageFeatureRelaxationSettings::ImageFeatureRelaxationSettings()
    : FeatureInterfaceSettings(),
      m_algo(ise::FeatureComputerPtr()),
      m_connexity(ise::ImageFeatureRelaxation::DEFAULT_CONNEXITY)
{}

void
ImageFeatureRelaxationSettings::configureWidget(FeatureInterfaceWidget *widget) const
{
    static_cast<ImageFeatureRelaxationWidget*>(widget)->setParameters(m_algo, m_connexity);
}

void
ImageFeatureRelaxationSettings::getWidgetConfig(const FeatureInterfaceWidget *widget)
{
    static_cast<const ImageFeatureRelaxationWidget*>(widget)->getParameters(m_algo, m_connexity);
}

void
ImageFeatureRelaxationSettings::configureAlgo(girl::shared_ptr<ise::FeatureComputer> algo) const
{
    assert(algo != NULL);
    girl::shared_ptr<ise::ImageFeatureRelaxation> algorithm =
            girl::static_pointer_cast<ise::ImageFeatureRelaxation> (algo);

    girl::shared_ptr<ise::ImageFeatureComputer> gaborImage =
            girl::static_pointer_cast<ise::ImageFeatureComputer>(m_algo);
    assert(gaborImage);
    algorithm->setParameters(gaborImage, m_connexity);
}

girl::shared_ptr<FeatureInterfaceSettings>
ImageFeatureRelaxationSettings::clone() const
{
    girl::shared_ptr<ImageFeatureRelaxationSettings> p(new ImageFeatureRelaxationSettings);
    p->m_algo = m_algo;
    p->m_connexity = m_connexity;
    return p;
}

bool
ImageFeatureRelaxationSettings::isValid() const
{
 return m_algo != NULL && (m_connexity == 4 || m_connexity == 8);
}
