/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "ImageFeatureRelaxationWidget.hpp"

#include <QString>
#include <QLabel>
#include <QGroupBox>
#include <QGridLayout>
#include <QComboBox>
#include <QRadioButton>

#include <qiseTools/WeightedChannelsImageFeatureWidget.hpp>
#include <qiseTools/ImageFeatureChoiceWidget.hpp>

ImageFeatureRelaxationWidget::ImageFeatureRelaxationWidget(MainView *mainView,
                                                           FeatureComputerGUIInfoPtr featureComputerGUIInfo,
                                                           QString name,
                                                           ImageFeaturePlugin *imageFeaturePlugin,
                                                           QWidget *parent)
    : ImageFeatureWidgetInterface(mainView, featureComputerGUIInfo, name, imageFeaturePlugin, parent),
      m_wcifw(NULL),
      m_4connexButton(NULL),
      m_8connexButton(NULL)
{
    setAttribute(Qt::WA_DeleteOnClose);

    build();
}

void
ImageFeatureRelaxationWidget::build()
{
    assert(m_mainView->model()->getSegContextPtr());


    QGroupBox *imgfGroup = new QGroupBox(QString("Image Feature"));
    QVBoxLayout *layoutGroup = new QVBoxLayout(imgfGroup);
    m_wcifw = new WeightedChannelsImageFeatureWidget(m_mainView, m_featureComputerGUIInfo, imgfGroup);
    layoutGroup->addWidget(m_wcifw);

    m_4connexButton = new QRadioButton(tr("4-connex"), this);
    m_8connexButton = new QRadioButton(tr("8-connex"), this);
    m_4connexButton->setChecked(true);
    QVBoxLayout *layout = new QVBoxLayout();
    layout->setAlignment(Qt::AlignTop);
    layout->addWidget(m_4connexButton);
    layout->addWidget(m_8connexButton);
    layout->addWidget(imgfGroup);
    setLayout(layout);
}

bool
ImageFeatureRelaxationWidget::isValid()
{
    return (m_wcifw->imageFeatureComputerPtr() != NULL && (m_4connexButton->isChecked() || m_8connexButton->isChecked()));
}

void
ImageFeatureRelaxationWidget::setParameters(const ise::FeatureComputerPtr &algo, int connexity)
{
    assert(connexity == 4 || 8);
    m_wcifw->setImageFeatureComputerPtr(algo);
    m_4connexButton->setChecked((connexity == 4) ? true : false);
}

void
ImageFeatureRelaxationWidget::getParameters(ise::FeatureComputerPtr &algo,
                                            int &connexity) const
{
    algo = m_wcifw->imageFeatureComputerPtr();
    connexity = m_4connexButton->isChecked() ? 4 : 8;
}	 

