/*
  Copyright (C) 2011 Cedric Aguerre  aguerre@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#ifndef NORMALIZEDCUTMERGEALGORITHMPLUGIN_HPP
#define NORMALIZEDCUTMERGEALGORITHMPLUGIN_HPP

#include <NormalizedCutMergeAlgorithm.hpp>

#include <qiseTools/MergeAlgorithmPlugin.hpp>
#include <MergeAlgorithmNormalizedCutWidget.hpp>
#include <MergeAlgorithmNormalizedCutWidgetSettings.hpp>

#include <QObject>

class NormalizedCutMergeAlgorithmPlugin : public QObject,
					  public MergeAlgorithmPlugin
{

  Q_OBJECT
  Q_INTERFACES(MergeAlgorithmPlugin QIsePlugin)
#if QT_VERSION >= 0x050000
  Q_PLUGIN_METADATA(IID MergeAlgorithmPlugin_iid)
#endif
    
public:
    
  ~NormalizedCutMergeAlgorithmPlugin() {}
  
  NormalizedCutMergeAlgorithmPlugin()
    : MergeAlgorithmPlugin(QString::fromUtf8("NormalizedCut merge algorithm"),
			   QIseVersion(1, 0), QIseVersion(1, 0))
  {}
  

  virtual girl::shared_ptr<ise::MergeAlgorithm>
    create(ise::SegmentationContext *segContext)
  {
    girl::shared_ptr<ise::MergeAlgorithm> result(new ncut::NormalizedCutMergeAlgorithm(segContext));
    return result;
  }
  
  virtual girl::shared_ptr<MergeAlgorithmSettings>
  createSettings(ise::SegmentationContext* /*segContext*/)
  {
    return girl::shared_ptr<MergeAlgorithmSettings>(new MergeAlgorithmNormalizedCutWidgetSettings());
  }

  virtual bool hasSettings() const
  {
    return true;
  }

protected:

  virtual MergeAlgorithmWidgetInterface *
  createWidget(ise::SegmentationContext * /*segContext*/, QWidget * /*parent*/)
  {
    return new MergeAlgorithmNormalizedCutWidget(this);
  }

};

#if QT_VERSION < 0x050000
Q_EXPORT_PLUGIN2(QIseNormalizedCutMergeAlgorithmPlugin, NormalizedCutMergeAlgorithmPlugin)
#endif

#endif /* ! NORMALIZEDCUTMERGEALGORITHMPLUGIN_HPP */
