/*
  Normalized-Cut merge algorithm.
  Copyright (C) 2010 Cédric Aguerre  aguerre@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISENORMALIZEDCUTMERGEALGORITHM_HPP
#define ISENORMALIZEDCUTMERGEALGORITHM_HPP

#include <girl/girlCommon.hpp>
#include <girl/girlToger.hpp>

#include <ise/iseMergeAlgorithmDefault.hpp>

#include <vector>
#include <map>
#include <queue>

namespace ncut {

  class SegmentationContext;

  class GIRL_API NormalizedCutMergeAlgorithm: public ise::MergeAlgorithmDefault
  {

  public:

    typedef enum {NEIGHBOURING_GEOMETRIC, NEIGHBOURING_GRAPH_GEODESIC} NeighbouringType;

    typedef enum {SIMI_GRAY_LEVEL, SIMI_INTERVENING_CONTOUR} SimilarityCriterion;

    explicit NormalizedCutMergeAlgorithm(ise::SegmentationContext *segContext);
    
    virtual ~NormalizedCutMergeAlgorithm();
    
    virtual void apply(girl::regionDeque &regionsToMerge,
		       ise::MergeDriver &driver,
		       int level,
		       girl::regionDeque &keptRegions);

    virtual void setParameters(int nbClusters, int distance, NeighbouringType neighType, SimilarityCriterion criterion, double sampleRate, double edgeVariance)
    {
      m_nbClusters = nbClusters;
      m_distance = distance;
      m_neighType = neighType;
      m_criterion = criterion;
      m_sampleRate = sampleRate;
      m_edgeVariance = edgeVariance;
    }

    virtual void getParameters(int &nbClusters, int &distance, NeighbouringType &neighType, SimilarityCriterion &criterion, double &sampleRate, double &edgeVariance) const
    { 
      nbClusters = m_nbClusters;
      distance = m_distance;
      neighType = m_neighType;
      criterion = m_criterion;
      sampleRate = m_sampleRate;
      edgeVariance = m_edgeVariance;
    }

  protected:
    size_t apply_aux(girl::regionDeque &regionsToMerge,
		     ise::MergeDriver &driver,
		     int level,
		     girl::regionDeque &keptRegions);
    
  private:
    int m_nbClusters;
    int m_distance;
    NeighbouringType m_neighType;
    SimilarityCriterion m_criterion;
    double m_sampleRate;
    double m_edgeVariance;

    double *m_edgMag;
    double *m_edgPha;
    double m_edgMagMax;

    girl::region mergeConnectedRegions(std::queue<long> &regions);
    std::vector<girl::region> mergeRegions(const std::queue<long> &regions);

    double interveningContour(int r1Start, int r1End, int r2Start, int r2End, const std::vector<girl::point> &regPts);

    void electPointsInsideRegion(const girl::region &r, double sampleRate, std::vector<girl::point> &v);

    void addNeighboursGraphGeodesic(std::vector<long> &v, girl::toger::neighbouringModes mode, girl::toger::markFlags mark, const std::vector<long> &allRealRegionIds, std::map<long, long> &nodeUsedId, std::map<long, long> &nodeRealId, int distance, int seed);
    void addNeighboursGeometric(std::vector<long> &v, const std::vector<long> &allRealRegionIds, std::map<long, long> &nodeUsedId, int distance, int seed);

  };
}

#endif /*  ISENORMALIZEDCUTMERGEALGORITHM_HPP */
