/*
  Copyright (C) 2011 Cedric Aguerre  aguerre@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef MERGEALGORITHMNORMALIZEDCUTWIDGETSETTINGS_HPP
#define MERGEALGORITHMNORMALIZEDCUTWIDGETSETTINGS_HPP

#include <qiseTools/MergeAlgorithmPlugin.hpp>

#include "MergeAlgorithmNormalizedCutWidget.hpp"
#include <qiseTools/MergeAlgorithmWidgetSettingsInterface.hpp>
#include <NormalizedCutMergeAlgorithm.hpp>

class MergeAlgorithmNormalizedCutWidgetSettings : public MergeAlgorithmWidgetSettingsInterface {
  
public:
  
  MergeAlgorithmNormalizedCutWidgetSettings()
    : MergeAlgorithmWidgetSettingsInterface(),
      m_nbClusters(8),
      m_distance(3),
      m_neighType(ncut::NormalizedCutMergeAlgorithm::NEIGHBOURING_GEOMETRIC),
      m_criterion(ncut::NormalizedCutMergeAlgorithm::SIMI_GRAY_LEVEL),
      m_sampleRate(0.3),
      m_edgeVariance(0.1)
  {
  }

  virtual void configureWidget(MergeAlgorithmWidgetInterface *widget) const
  {
    static_cast<MergeAlgorithmNormalizedCutWidget*>(widget)->setParameters(m_nbClusters, m_distance, m_neighType, m_criterion, m_sampleRate, m_edgeVariance);
  }
  
  virtual void getWidgetConfig(const MergeAlgorithmWidgetInterface *widget)
  {
    static_cast<const MergeAlgorithmNormalizedCutWidget*>(widget)->getParameters(m_nbClusters, m_distance, m_neighType, m_criterion, m_sampleRate, m_edgeVariance);
  }

  virtual void configureAlgo(girl::shared_ptr<ise::MergeAlgorithm> algo) const
  {
    girl::shared_ptr<ncut::NormalizedCutMergeAlgorithm> algorithm = girl::static_pointer_cast<ncut::NormalizedCutMergeAlgorithm>(algo);
    algorithm->setParameters(m_nbClusters, m_distance, m_neighType, m_criterion, m_sampleRate, m_edgeVariance);
  }

  virtual girl::shared_ptr<MergeAlgorithmWidgetSettingsInterface> clone() const
  {
    girl::shared_ptr<MergeAlgorithmNormalizedCutWidgetSettings> p(new MergeAlgorithmNormalizedCutWidgetSettings);

    *p = *this;

    return p;
  }

  virtual bool isValid() const
  {
    return true; // TODO: test member variables
  }
  
private:

  int m_nbClusters;
  int m_distance;
  ncut::NormalizedCutMergeAlgorithm::NeighbouringType m_neighType;
  ncut::NormalizedCutMergeAlgorithm::SimilarityCriterion m_criterion;
  double m_sampleRate;
  double m_edgeVariance;
};

#endif /* ! MERGEALGORITHMNORMALIZEDCUTWIDGETSETTINGS_HPP */
