/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux boris.mansencal@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "MergeDriverClustererSettings.hpp"

#include "MergeDriverClustererWidget.hpp"



void 
MergeDriverClustererSettings::configureWidget(MergeDriverWidgetInterface *widget) const
{
  MergeDriverClustererWidget *w = static_cast<MergeDriverClustererWidget *>(widget);

  MergeDriverSettings::configureWidget(widget);
  
  w->setParameters(m_clusterer, m_regSet);
}

void
MergeDriverClustererSettings::getWidgetConfig(const MergeDriverWidgetInterface *widget)
{
  const MergeDriverClustererWidget *w = static_cast<const MergeDriverClustererWidget *>(widget);
  
  MergeDriverSettings::getWidgetConfig(widget);
  w->getParameters(m_clusterer, m_regSet);
}

void
MergeDriverClustererSettings::configureAlgo(girl::shared_ptr<ise::MergeDriver> algo) const
{
  girl::shared_ptr<ise::MergeDriverClusterer> p =
    girl::static_pointer_cast<ise::MergeDriverClusterer>(algo);

  MergeDriverSettings::configureAlgo(algo);

  p->setParameters(m_regSet, m_clusterer);
}
 
girl::shared_ptr<MergeDriverSettings> 
MergeDriverClustererSettings::clone() const
{
  girl::shared_ptr<MergeDriverClustererSettings> p(new MergeDriverClustererSettings);

  *p = *this;

  return p;
}

bool
MergeDriverClustererSettings::isValid() const
{
  return ( m_regSet != girl::shared_ptr<ise::RegionSetInterface>() && m_clusterer != girl::shared_ptr<ise::Clusterer>() );
}
