/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux boris.mansencal@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <qiseTools/MergeDriverWidget.hpp>
#include "MergeDriverClustererWidget.hpp"
#include <qiseTools/PluginRegistry.hpp>
#include <qiseTools/RegionFeatureChoiceComboWidget.hpp>

MergeDriverClustererWidget::MergeDriverClustererWidget(MainView *mainView, 
						       MergeDriverPlugin *plugin, 
						       QWidget *parent)
  : MergeDriverWidgetInterface(mainView, plugin, parent)
  , m_mergeDriverWidget(NULL)
{
  buildGUI();
}


void
MergeDriverClustererWidget::buildGUI()
{
  assert(m_mainView);
  assert(m_mainView->model());
  assert(m_mainView->model()->getSegContextPtr());

  m_mergeDriverWidget = new MergeDriverWidget(this);

  m_rfccw = new RegionFeatureChoiceComboWidget(m_mainView, m_mainView->model()->getFeatureComputerGUIInfo(), this, true);

  QIsePluginList<ClustererPlugin *> list = PluginRegistry::getClustererPlugin();

  m_clustererCB = new PluginIndexChoiceWidget(list.getGenericList(), tr("Clusterer"), this);

  {
    QIsePluginList<ClustererPlugin *>::iterator it = list.begin();
    QIsePluginList<ClustererPlugin *>::iterator end = list.end();
    for (; it != end; ++it) {
      m_clusterer[tr((*it)->name().toStdString().c_str())] = (*it)->parameterWidget(m_mainView, this);
    }
  }

  connect(m_clustererCB, SIGNAL(pluginIndexChanged(int)), this, SLOT(showWidget(int)));

  QVBoxLayout *vLayout = new QVBoxLayout;
  vLayout->addWidget(m_mergeDriverWidget);
  vLayout->addWidget(m_rfccw);
  vLayout->addWidget(m_clustererCB);

  std::map<QString, ClustererWidgetInterface* >::const_iterator it = m_clusterer.begin();
  std::map<QString, ClustererWidgetInterface* >::const_iterator itEnd = m_clusterer.end();
  for ( ; it != itEnd ; ++it) {
    vLayout->addWidget( (*it).second );
    (*it).second->hide();
  }

  m_activeClustererWidget = m_clusterer[m_clustererCB->getPluginName(0)];
  if (m_activeClustererWidget != NULL)
    m_activeClustererWidget->show();

  showWidget(0);

  setLayout(vLayout);
}

void
MergeDriverClustererWidget::setParameters(girl::shared_ptr<ise::Clusterer> /*clusterer*/, const girl::shared_ptr<ise::RegionSetInterface> &algo)
{
  m_rfccw->setRegionFeatureComputerPtr(algo);
}

void
MergeDriverClustererWidget::getParameters(girl::shared_ptr<ise::Clusterer> &clusterer, girl::shared_ptr<ise::RegionSetInterface> &algo) const
{
  assert(m_mainView);
  assert(m_mainView->model());
  assert(m_mainView->model()->getSegContextPtr());

  algo = girl::static_pointer_cast<ise::RegionSetInterface>(m_rfccw->regionFeatureComputerPtr());

  std::map<QString, ClustererWidgetInterface* >::const_iterator it = m_clusterer.begin();
  std::map<QString, ClustererWidgetInterface* >::const_iterator itEnd = m_clusterer.end();
  for ( ; it != itEnd ; ++it) {
    (*it).second->saveSettingsInPlugin();
  }

  clusterer = static_cast<ClustererPlugin*>(m_clustererCB->getPlugin())->getNewClusterer(m_mainView->model()->getSegContextPtr());
}

void
MergeDriverClustererWidget::showWidget(int index)
{
  const QString name = m_clustererCB->getPluginName(index);

  if (m_activeClustererWidget != NULL)
    m_activeClustererWidget->hide();

  m_activeClustererWidget = m_clusterer[name];

  if (m_activeClustererWidget != NULL)
    m_activeClustererWidget->show();
}

void
MergeDriverClustererWidget::setNeighbouringModes(girl::toger::neighbouringModes neighbourhoodMode)
{
  m_mergeDriverWidget->setNeighbouringMode(neighbourhoodMode);
}

girl::toger::neighbouringModes 
MergeDriverClustererWidget::getNeighbouringModes() const
{
  return m_mergeDriverWidget->getNeighbouringMode();
}

void
MergeDriverClustererWidget::setMarkFlags(girl::toger::markFlags m)
{
  m_mergeDriverWidget->setMarkFlag(m);
}

girl::toger::markFlags
MergeDriverClustererWidget::getMarkFlags() const
{
  return m_mergeDriverWidget->getMarkFlag();
}
