/*
 * ContextualMergeDriverWidget.cpp
 *
 *  Created on: Oct 22, 2010
 *      Author: vieux
 */

#include "ContextualMergeDriverWidget.hpp"

#include "iseContextualMergeDriver.hpp"

#include <qiseTools/MergeDriverFactory.hpp>
#include <qiseTools/RegionSetParametersWidget.hpp>
#include <qiseTools/SliderSpinBoxWidget.hpp>

#include <QVBoxLayout>

ContextualMergeDriverWidget::ContextualMergeDriverWidget(ise::SegmentationContext *segContext,
    MergeDriverFactory *mergeDriverFactory,
    QWidget *parent)
: MergeDriverWidgetInterface(segContext, mergeDriverFactory, parent),
  m_rspw(NULL),
  m_ssb(NULL)
{
  buildGUI();
}

ContextualMergeDriverWidget::~ContextualMergeDriverWidget()
{
}

void
ContextualMergeDriverWidget::buildGUI()
{
  setGUI();
}

void
ContextualMergeDriverWidget::setGUI()
{
  assert(m_mergeDriverFactory);
  girl::shared_ptr<ise::ContextualMergeDriver> algo
  = girl::static_pointer_cast<ise::ContextualMergeDriver>(m_mergeDriverFactory->getNewMergeDriver());

  m_rspw = new RegionSetParametersWidget(algo->getRegSetContext(), algo->getContextParams(), this);
  m_ssb = new SliderSpinBoxWidget(this);
  m_ssb->setDecimals(0);
  m_ssb->setMinimum(1);
  m_ssb->setMaximum(10000);
  m_ssb->setValue(algo->getThreshold());
  QVBoxLayout *layout = new QVBoxLayout;
  layout->addWidget(m_rspw);
  layout->addWidget(m_ssb);
  setLayout(layout);
}

void
ContextualMergeDriverWidget::getGUI()
{
  girl::shared_ptr<ise::ContextualMergeDriver> algo
    = girl::static_pointer_cast<ise::ContextualMergeDriver>(m_mergeDriverFactory->getNewMergeDriver());
  algo->setThreshold(m_ssb->value());
}
