/*
 * iseContextualMergeDriver.cpp
 *
 *  Created on: Oct 22, 2010
 *      Author: vieux
 */

#include "iseContextualMergeDriver.hpp"

#include <ise/iseRegionFeatureGravityCenter.hpp>
#include <ise/iseRegionFeatureNeighbourhoodContext.hpp>
#include <ise/iseRegionSetBinaryFunctionEuclideanDistance.hpp>
#include <ise/iseRegionSetFactoryHandler.hpp>

namespace ise
{

  ContextualMergeDriver::ContextualMergeDriver(ise::SegmentationContext *segContext)
  : MergeDriverDefault(segContext),
    m_regSetContext(ise::RegionSetFactoryHandler::instance()->newRegionSet(girl::shared_ptr<ise::RegionFeatureComputer>(new ise::RegionFeatureNeighbourhoodContext(segContext)))),
    m_regSetGravityCenter(ise::RegionSetFactoryHandler::instance()->newRegionSet(girl::shared_ptr<ise::RegionFeatureComputer>(new ise::RegionFeatureGravityCenter(segContext)))),
    m_distanceComputer(girl::static_pointer_cast<ise::RegionSetBinaryFunctionEuclideanDistance>(RegionSetBinaryFunctionEuclideanDistanceFactory::instance().create())),
    m_contextParams(m_regSetContext->getFeatureDim()),
    m_threshold(0)
  {
    m_distanceComputer->setParameters(m_regSetContext, m_contextParams);
  }

  ContextualMergeDriver::~ContextualMergeDriver()
  {
  }


  double
  ContextualMergeDriver::distance(girl::region ra, girl::region rb)
  {
    return (*m_distanceComputer)(ra, rb);
  }

  bool
  ContextualMergeDriver::allowed(girl::region ra, girl::region rb, int)
  {
    return (*m_distanceComputer)(ra, rb) < m_threshold;
  }

}
