/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "MergeDriverGradientWidget.hpp"

#include <cassert>

#include <ise/iseMergeDriverGradient.hpp>

#include <QGridLayout>
#include <QHBoxLayout>
#include <QLabel>

#include "MergeDriverPlugin.hpp"
#include "MergeDriverWidget.hpp"
#include "SliderSpinBoxWidget.hpp"
#include "WeightedChannelsColorSpaceWidget.hpp"


//TODO:REMOVE
#include <ise/iseRegionUnaryPredicate.hpp>
#include "PluginIndexChoiceWidget.hpp"
#include "PluginRegistry.hpp"

MergeDriverGradientWidget::MergeDriverGradientWidget(MergeDriverPlugin *mergeDriverPlugin,
						     QWidget *parent)
  :
  MergeDriverWidgetInterface(NULL, mergeDriverPlugin, parent),
  m_mergeDriverWidget(NULL),
  m_wccsw(NULL),
  m_thresholdSB(NULL),
  //TODO:REMOVE
  m_rupcw(NULL)
{
  assert(m_mergeDriverPlugin);
  
  setWindowTitle(tr("MergeDriverGradient parameters..."));
  
  //setAttribute(Qt::WA_DeleteOnClose);
  
  buildGUI();
  
  setGUI();
}

void
MergeDriverGradientWidget::buildGUI()
{
  
  m_mergeDriverWidget = new MergeDriverWidget(this);
  
  m_wccsw = new WeightedChannelsColorSpaceWidget(this);

  QGridLayout *layoutT = new QGridLayout;

  QLabel *thresholdLabel = new QLabel(tr("Threshold: "), this);
  layoutT->addWidget(thresholdLabel, 0, 0);
  m_thresholdSB = new SliderSpinBoxWidget(this);
  m_thresholdSB->setDecimals(10);
  m_thresholdSB->setSingleStep(0.0001);
  m_thresholdSB->setMinimum(0.0);
  m_thresholdSB->setMaximum(1.0);
  layoutT->addWidget(m_thresholdSB, 0, 1);
  
  layoutT->setMargin(0); //because embedded widget
  QWidget *w = new QWidget(this);
  w->setLayout(layoutT);
  
  QFrame *f = new QFrame(this);
  f->setFrameShape(QFrame::HLine);


  // TODO: REMOVE
  QIsePluginList<QIsePlugin*> regionUnaryPredicateList
    = PluginRegistry::getRegionBinaryFunctionPlugin().getGenericList();
  m_rupcw = new PluginIndexChoiceWidget(regionUnaryPredicateList, tr("Region Binary Functions"), this);

  
  QVBoxLayout *vLayout = new QVBoxLayout();

  //TODO: REMOVE
  vLayout->addWidget(m_rupcw);
  vLayout->addWidget(m_mergeDriverWidget);
  vLayout->addWidget(m_wccsw);
  vLayout->addWidget(w);
  vLayout->addWidget(f);

  setLayout(vLayout);


  connect(m_mergeDriverWidget, SIGNAL(parametersChanged()),
	  this, SIGNAL(parametersChanged()));
  connect(m_wccsw, SIGNAL(parametersChanged()),
	  this, SIGNAL(parametersChanged()));
  connect(m_thresholdSB, SIGNAL(valueChanged(double)),
	  this, SIGNAL(parametersChanged()));
}


void
MergeDriverGradientWidget::setGUI()
{
  girl::toger::neighbouringModes mode;
  girl::toger::markFlags m;
  
  girl::shared_ptr<ise::MergeDriverGradient> algo = girl::static_pointer_cast<ise::MergeDriverGradient>(m_mergeDriverPlugin->getNewMergeDriver());
  algo->getNeighbourhood(mode, m);
  
  m_mergeDriverWidget->setNeighbouringMode(mode);
  m_mergeDriverWidget->setMarkFlag(m);

  double threshold;
  int conversion;
  std::vector<double> normalizedWeightsForChannels;

  algo->getParameters(threshold, conversion, normalizedWeightsForChannels);

  m_wccsw->setColorSpace(conversion);
  m_wccsw->setNormalizedWeights(normalizedWeightsForChannels);
  m_thresholdSB->setValue(threshold);
}

void
MergeDriverGradientWidget::getGUI()
{
  assert(m_mergeDriverPlugin);

  /*
  //TODO:remove
  // Get the plugin
  QIsePluginList<RegionUnaryPredicatePlugin *> pluginList =
    PluginRegistry::getRegionUnaryPredicatePlugin();
  int pluginIndex = m_rupcw->getPluginIndex();
  RegionUnaryPredicatePlugin *plugin = pluginList.at(pluginIndex);
  girl::shared_ptr<ise::RegionUnaryPredicate> predicate = plugin->create(m_segContext);

  std::cout << "In MergeDriverGradientWidget::GetGui() the unary predicate says"
      << (*predicate)(1) << std::endl;
  */

  const girl::toger::neighbouringModes mode = m_mergeDriverWidget->getNeighbouringMode();
  const girl::toger::markFlags m = m_mergeDriverWidget->getMarkFlag();
  
  girl::shared_ptr<ise::MergeDriverGradient> algo = girl::static_pointer_cast<ise::MergeDriverGradient>(m_mergeDriverPlugin->getNewMergeDriver());
  algo->setNeighbourhood(mode, m);

  const double threshold = m_thresholdSB->value();
  
  const int conversion = m_wccsw->getColorSpace();

  std::vector<double> normalizedWeightsForChannels;
  m_wccsw->getNormalizedWeights(normalizedWeightsForChannels);

  algo->setParameters(threshold, conversion, normalizedWeightsForChannels);
  
  m_mergeDriverPlugin->saveParameters(algo);
}

