/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux boris.mansencal@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "MergeDriverMetaSettings.hpp"

#include "MergeDriverMetaWidget.hpp"



void 
MergeDriverMetaSettings::configureWidget(MergeDriverWidgetInterface *widget) const
{
  MergeDriverMetaWidget *w = static_cast<MergeDriverMetaWidget *>(widget);

  MergeDriverSettings::configureWidget(widget);

  /*
  assert(m_filterPredicate);
  assert(m_sortOrdering);
  assert(m_allowMergePredicate);
  assert(m_distanceFunction);
  */

  /*
  if (m_filterPredicate) {
    m_filterPredicate->setSettings(m_filterSettings);
    w->setFilterPredicatePlugin(m_filterPredicate);
  }
  */

  if (m_sortOrdering) {
    m_sortOrdering->setSettings(m_sortOrderingSettings);
    w->setStrictWeakOrderingPlugin(m_sortOrdering);
  }

  if (m_allowMergePredicate) {
    m_allowMergePredicate->setSettings(m_allowMergeSettings);
    w->setAllowMergePredicatePlugin(m_allowMergePredicate);
  }

  if (m_distanceFunction) {
    m_distanceFunction->setSettings(m_distanceSettings);
    w->setDistanceFunctionPlugin(m_distanceFunction);
  }
}

void
MergeDriverMetaSettings::getWidgetConfig(const MergeDriverWidgetInterface *widget)
{
  const MergeDriverMetaWidget *w = static_cast<const MergeDriverMetaWidget *>(widget);
  
  MergeDriverSettings::getWidgetConfig(widget);

  /*
  m_filterPredicate = w->getFilterPredicatePlugin();
  if (m_filterPredicate)
  m_filterSettings = m_filterPredicate->settings();
  */

  m_sortOrdering = w->getStrictWeakOrderingPlugin();
  if (m_sortOrdering)
    m_sortOrderingSettings = m_sortOrdering->settings();

  m_allowMergePredicate = w->getAllowMergePredicatePlugin();
  if (m_allowMergePredicate)
    m_allowMergeSettings = m_allowMergePredicate->settings();

  m_distanceFunction = w->getDistanceFunctionPlugin();
  if (m_distanceFunction)
    m_distanceSettings = m_distanceFunction->settings();
}

void
MergeDriverMetaSettings::configureAlgo(girl::shared_ptr<ise::MergeDriver> algo) const
{
  girl::shared_ptr<ise::MergeDriverMeta> p =
    girl::static_pointer_cast<ise::MergeDriverMeta>(algo);
    
  /*
  if (m_filterPredicate) {
    girl::shared_ptr<ise::RegionUnaryPredicate> f = m_filterPredicate->getNewRegionUnaryPredicate(m_filterPredicate->getSegContext());
    if (m_filterSettings)
      m_filterSettings->configureAlgo(f);
    p->setFilterPredicate(f);
  }
  */

  MergeDriverSettings::configureAlgo(algo);

  if (m_sortOrdering) {
    girl::shared_ptr<ise::RegionStrictWeakOrdering> s = m_sortOrdering->getNewRegionStrictWeakOrdering(m_sortOrdering->getSegContext());
    if (m_sortOrderingSettings)
      m_sortOrderingSettings->configureAlgo(s);
    p->setSortPredicate(s);
  }

  if (m_allowMergePredicate) {
    girl::shared_ptr<ise::RegionBinaryPredicate> a = m_allowMergePredicate->getNewRegionBinaryPredicate(m_allowMergePredicate->getSegContext());
    if (m_allowMergeSettings)
      m_allowMergeSettings->configureAlgo(a);
    p->setAllowMergePredicate(a);
  }
  
  if (m_distanceFunction) {
    girl::shared_ptr<ise::RegionBinaryFunction> d = m_distanceFunction->getNewRegionBinaryFunction(m_distanceFunction->getSegContext());
    if (m_distanceSettings) 
      m_distanceSettings->configureAlgo(d);
    p->setDistanceFunction(d);
  }
  
}
 
girl::shared_ptr<MergeDriverSettings> 
MergeDriverMetaSettings::clone() const
{
  girl::shared_ptr<MergeDriverMetaSettings> p(new MergeDriverMetaSettings);

  *p = *this;


  /*
  if (m_filterSettings)
    p->m_filterSettings = m_filterSettings->clone();
  else 
    p->m_filterSettings = girl::shared_ptr<RegionUnaryPredicateSettings>();
  */

  if (m_sortOrderingSettings)
    p->m_sortOrderingSettings = m_sortOrderingSettings->clone();
  else
    p->m_sortOrderingSettings = girl::shared_ptr<RegionStrictWeakOrderingSettings>();

  if (m_allowMergeSettings)
    p->m_allowMergeSettings = m_allowMergeSettings->clone();
  else
    p->m_allowMergeSettings = girl::shared_ptr<RegionBinaryPredicateSettings>();

  if (m_distanceSettings)
    p->m_distanceSettings = m_distanceSettings->clone();
  else
    p->m_distanceSettings = girl::shared_ptr<RegionBinaryFunctionSettings>();

  return p;
}

bool
MergeDriverMetaSettings::isValid() const
{
  //REM: not necessary to have a sortOrdering to be valid.

  return (
	  //m_filterPredicate && (! m_filterPredicate->hasSettings() || (m_filterSettings != NULL && m_filterSettings->isValid()))
	  //&& 
	  //m_sortOrdering && (! m_sortOrdering->hasSettings() || (m_sortOrderingSettings != NULL && m_sortOrderingSettings->isValid()))
	  //&& 
	  m_allowMergePredicate && (! m_allowMergePredicate->hasSettings() || (m_allowMergeSettings != girl::shared_ptr<RegionBinaryPredicateSettings>() && m_allowMergeSettings->isValid()))
	  && 
	  m_distanceFunction && (! m_distanceFunction->hasSettings() || (m_distanceSettings != girl::shared_ptr<RegionBinaryFunctionSettings>() && m_distanceSettings->isValid()))
	  );   
}
