/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux boris.mansencal@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "MergeDriverMetaWidget.hpp"

#include <qiseTools/ComparisonFunctionWidget.hpp>
#include <qiseTools/RegionUnaryFunctionPlugin.hpp>
#include <qiseTools/RegionUnaryFunctionWidget.hpp>

#include <QGridLayout>
#include <QTabWidget>

#include <qiseTools/MergeDriverWidget.hpp>
#include <qiseTools/RegionUnaryPredicatePlugin.hpp>
#include <qiseTools/RegionBinaryPredicatePlugin.hpp>
#include <qiseTools/RegionBinaryFunctionPlugin.hpp>
#include <qiseTools/RegionStrictWeakOrderingPlugin.hpp>

#include <qiseTools/RegionUnaryPredicateComboWidget.hpp>
#include <qiseTools/RegionBinaryPredicateComboWidget.hpp>
#include <qiseTools/RegionBinaryFunctionComboWidget.hpp>
#include <qiseTools/RegionStrictWeakOrderingComboWidget.hpp>
#include <qiseTools/MainView.hpp>

MergeDriverMetaWidget::MergeDriverMetaWidget(MainView *mainView, 
					     MergeDriverPlugin *plugin, 
					     QWidget *parent)
  : MergeDriverWidgetInterface(mainView, plugin, parent),
    m_mergeDriverWidget(NULL),
    //m_filterPage(NULL),
    m_swoPage(NULL),
    m_allowPage(NULL),
    m_distancePage(NULL),
    //m_filterCW(NULL),
    m_swoCW(NULL),
    m_allowCW(NULL),
    m_distanceCW(NULL)
{
  buildGUI();
}


void
MergeDriverMetaWidget::buildGUI()
{
  assert(m_mainView);
  // assert(m_mainView->model());
  // assert(m_mainView->model()->getSegContextPtr());
  
  //ise::SegmentationContext *segContext = m_mainView->model()->getSegContextPtr();
  
  m_mergeDriverWidget = new MergeDriverWidget(this);
  
  /*
  m_filterPage = new QWidget(this);
  {
    m_filterCW = new RegionUnaryPredicateComboWidget(segContext, this);
    QVBoxLayout *l = new QVBoxLayout;
    l->addWidget(m_filterCW);
    m_filterPage->setLayout(l);
  }
  */
    
  m_swoPage = new QWidget(this);
  {
    m_swoCW = new RegionStrictWeakOrderingComboWidget(m_mainView /*segContext*/, this);
    QVBoxLayout *l = new QVBoxLayout;
    l->addWidget(m_swoCW);
    m_swoPage->setLayout(l);
  }

  m_allowPage = new QWidget(this);
  {
    m_allowCW = new RegionBinaryPredicateComboWidget(m_mainView/*segContext*/, this);
    QVBoxLayout *l = new QVBoxLayout;
    l->addWidget(m_allowCW);
    m_allowPage->setLayout(l);
  }
   
  m_distancePage = new QWidget(this);
  {
    m_distanceCW = new RegionBinaryFunctionComboWidget(m_mainView/*segContext*/, this);
    QVBoxLayout *l = new QVBoxLayout;
    l->addWidget(m_distanceCW);
    m_distancePage->setLayout(l);
  }

    
  QTabWidget *tabWidget = new QTabWidget(this);
  //tabWidget->addTab(m_filterPage, tr("Filter"));
  tabWidget->addTab(m_swoPage, tr("Sort"));
  tabWidget->addTab(m_allowPage, tr("Allow"));
  tabWidget->addTab(m_distancePage, tr("Distance"));


  QGridLayout *gGL = new QGridLayout;
  gGL->addWidget(m_mergeDriverWidget, 0, 0);
  gGL->addWidget(tabWidget, 1, 0);
  
  setLayout(gGL);

}


/*
void
MergeDriverMetaWidget::setFilterPredicatePlugin(RegionUnaryPredicatePlugin *p)
{
  m_filterCW->setRegionUnaryPredicatePlugin(p);
}

RegionUnaryPredicatePlugin *
MergeDriverMetaWidget::getFilterPredicatePlugin() const
{
  return m_filterCW->getRegionUnaryPredicatePlugin();
}
*/

void
MergeDriverMetaWidget::setStrictWeakOrderingPlugin(RegionStrictWeakOrderingPlugin *p)
{
  m_swoCW->setRegionStrictWeakOrderingPlugin(p);
}

RegionStrictWeakOrderingPlugin *
MergeDriverMetaWidget::getStrictWeakOrderingPlugin() const
{
  return m_swoCW->getRegionStrictWeakOrderingPlugin();
}

void
MergeDriverMetaWidget::setAllowMergePredicatePlugin(RegionBinaryPredicatePlugin *p)
{
  m_allowCW->setRegionBinaryPredicatePlugin(p);
}

RegionBinaryPredicatePlugin *
MergeDriverMetaWidget::getAllowMergePredicatePlugin() const
{
  return m_allowCW->getRegionBinaryPredicatePlugin();
}

void
MergeDriverMetaWidget::setDistanceFunctionPlugin(RegionBinaryFunctionPlugin *p)
{
  m_distanceCW->setRegionBinaryFunctionPlugin(p);
}

RegionBinaryFunctionPlugin *
MergeDriverMetaWidget::getDistanceFunctionPlugin() const
{
  return m_distanceCW->getRegionBinaryFunctionPlugin();
}

void
MergeDriverMetaWidget::setNeighbouringModes(girl::toger::neighbouringModes neighbourhoodMode)
{
  m_mergeDriverWidget->setNeighbouringMode(neighbourhoodMode);
}

girl::toger::neighbouringModes MergeDriverMetaWidget::getNeighbouringModes() const
{
  return m_mergeDriverWidget->getNeighbouringMode();
}

void
MergeDriverMetaWidget::setMarkFlags(girl::toger::markFlags m)
{
  m_mergeDriverWidget->setMarkFlag(m);
}

girl::toger::markFlags
MergeDriverMetaWidget::getMarkFlags() const
{
  return m_mergeDriverWidget->getMarkFlag();
}
