/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux boris.mansencal@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "MergeDriverMinimalDistanceSettings.hpp"

#include "MergeDriverMinimalDistanceWidget.hpp"



void 
MergeDriverMinimalDistanceSettings::configureWidget(MergeDriverWidgetInterface *widget) const
{
  MergeDriverMinimalDistanceWidget *w = static_cast<MergeDriverMinimalDistanceWidget *>(widget);

  /*
  assert(m_distanceFunction);
  */

  MergeDriverSettings::configureWidget(widget);

  if (m_distanceFunction) {
    m_distanceFunction->setSettings(m_distanceSettings);
    w->setDistanceFunctionPlugin(m_distanceFunction);
  }
}

void
MergeDriverMinimalDistanceSettings::getWidgetConfig(const MergeDriverWidgetInterface *widget)
{
  const MergeDriverMinimalDistanceWidget *w = static_cast<const MergeDriverMinimalDistanceWidget *>(widget);
  
  MergeDriverSettings::getWidgetConfig(widget);

  m_distanceFunction = w->getDistanceFunctionPlugin();
  if (m_distanceFunction)
    m_distanceSettings = m_distanceFunction->settings();
}

void
MergeDriverMinimalDistanceSettings::configureAlgo(girl::shared_ptr<ise::MergeDriver> algo) const
{
  girl::shared_ptr<ise::MergeDriverMinimalDistance> p =
    girl::static_pointer_cast<ise::MergeDriverMinimalDistance>(algo);
 
  MergeDriverSettings::configureAlgo(algo);

   
  if (m_distanceFunction) {
    girl::shared_ptr<ise::RegionBinaryFunction> d = m_distanceFunction->getNewRegionBinaryFunction(m_distanceFunction->getSegContext());
    if (m_distanceSettings) 
      m_distanceSettings->configureAlgo(d);
    p->setDistanceFunction(d);
  }
}

girl::shared_ptr<MergeDriverSettings> 
MergeDriverMinimalDistanceSettings::clone() const
{
  girl::shared_ptr<MergeDriverMinimalDistanceSettings> p(new MergeDriverMinimalDistanceSettings);

  *p = *this;

  if (m_distanceSettings)
    p->m_distanceSettings = m_distanceSettings->clone();
  else
    p->m_distanceSettings = girl::shared_ptr<RegionBinaryFunctionSettings>();

  return p;
}

bool
MergeDriverMinimalDistanceSettings::isValid() const
{
  return (
	  m_distanceFunction != NULL 
	  && (! m_distanceFunction->hasSettings() || (m_distanceSettings != girl::shared_ptr<RegionBinaryFunctionSettings>() && m_distanceSettings->isValid()))
	  );
}
