/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux boris.mansencal@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "MergeDriverMinimalDistanceWidget.hpp"

#include <QVBoxLayout>

#include <qiseTools/MergeDriverWidget.hpp>
#include <qiseTools/RegionBinaryFunctionPlugin.hpp>

//#include <qiseTools/RegionBinaryFunctionWidget.hpp>

#include <qiseTools/RegionBinaryFunctionComboWidget.hpp>
#include <qiseTools/MainView.hpp>

MergeDriverMinimalDistanceWidget::MergeDriverMinimalDistanceWidget(MainView *mainView, 
					     MergeDriverPlugin *plugin, 
					     QWidget *parent)
  : MergeDriverWidgetInterface(mainView, plugin, parent),
    m_rbfcw(NULL)
{
  buildGUI();
}


void
MergeDriverMinimalDistanceWidget::buildGUI()
{
  assert(m_mainView);
  // assert(m_mainView->model());
  // assert(m_mainView->model()->getSegContextPtr());
  
  m_mergeDriverWidget = new MergeDriverWidget(this);

  m_rbfcw = new RegionBinaryFunctionComboWidget(m_mainView, this);
  m_rbfcw->setText(tr("Distance: "));

  QVBoxLayout *vL = new QVBoxLayout;
  vL-> addWidget(m_mergeDriverWidget);
  vL-> addWidget(m_rbfcw);

  setLayout(vL);
}


void
MergeDriverMinimalDistanceWidget::setDistanceFunctionPlugin(RegionBinaryFunctionPlugin *p)
{
  m_rbfcw->setRegionBinaryFunctionPlugin(p);
}

RegionBinaryFunctionPlugin *
MergeDriverMinimalDistanceWidget::getDistanceFunctionPlugin() const
{
  return m_rbfcw->getRegionBinaryFunctionPlugin();
}

void
MergeDriverMinimalDistanceWidget::setNeighbouringModes(girl::toger::neighbouringModes neighbourhoodMode)
{
  m_mergeDriverWidget->setNeighbouringMode(neighbourhoodMode);
}

girl::toger::neighbouringModes MergeDriverMinimalDistanceWidget::getNeighbouringModes() const
{
  return m_mergeDriverWidget->getNeighbouringMode();
}

void
MergeDriverMinimalDistanceWidget::setMarkFlags(girl::toger::markFlags m)
{
  m_mergeDriverWidget->setMarkFlag(m);
}

girl::toger::markFlags
MergeDriverMinimalDistanceWidget::getMarkFlags() const
{
  return m_mergeDriverWidget->getMarkFlag();
}
