/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux boris.mansencal@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "MergeDriverMinimalDistanceWithAllowSettings.hpp"

#include "MergeDriverMinimalDistanceWithAllowWidget.hpp"



void 
MergeDriverMinimalDistanceWithAllowSettings::configureWidget(MergeDriverWidgetInterface *widget) const
{
  MergeDriverMinimalDistanceWithAllowWidget *w = static_cast<MergeDriverMinimalDistanceWithAllowWidget *>(widget);

  /*
  assert(m_distanceFunction);
  */

  MergeDriverSettings::configureWidget(widget);

  if (m_allowMergePredicate) {
    m_allowMergePredicate->setSettings(m_allowMergeSettings);
    w->setAllowMergePredicatePlugin(m_allowMergePredicate);
  }

  if (m_distanceFunction) {
    m_distanceFunction->setSettings(m_distanceSettings);
    w->setDistanceFunctionPlugin(m_distanceFunction);
  }
}

void
MergeDriverMinimalDistanceWithAllowSettings::getWidgetConfig(const MergeDriverWidgetInterface *widget)
{
  const MergeDriverMinimalDistanceWithAllowWidget *w = static_cast<const MergeDriverMinimalDistanceWithAllowWidget *>(widget);
  
  MergeDriverSettings::getWidgetConfig(widget);

  m_allowMergePredicate = w->getAllowMergePredicatePlugin();
  if (m_allowMergePredicate)
    m_allowMergeSettings = m_allowMergePredicate->settings();

  m_distanceFunction = w->getDistanceFunctionPlugin();
  if (m_distanceFunction)
    m_distanceSettings = m_distanceFunction->settings();
}

void
MergeDriverMinimalDistanceWithAllowSettings::configureAlgo(girl::shared_ptr<ise::MergeDriver> algo) const
{
  girl::shared_ptr<ise::MergeDriverMinimalDistanceWithAllow> p =
    girl::static_pointer_cast<ise::MergeDriverMinimalDistanceWithAllow>(algo);
 
  MergeDriverSettings::configureAlgo(algo);

   
  if (m_allowMergePredicate) {
    girl::shared_ptr<ise::RegionBinaryPredicate> a = m_allowMergePredicate->getNewRegionBinaryPredicate(m_allowMergePredicate->getSegContext());
    if (m_allowMergeSettings)
      m_allowMergeSettings->configureAlgo(a);
    p->setAllowMergePredicate(a);
  }
  
  if (m_distanceFunction) {
    girl::shared_ptr<ise::RegionBinaryFunction> d = m_distanceFunction->getNewRegionBinaryFunction(m_distanceFunction->getSegContext());
    if (m_distanceSettings) 
      m_distanceSettings->configureAlgo(d);
    p->setDistanceFunction(d);
  }
  
}

girl::shared_ptr<MergeDriverSettings> 
MergeDriverMinimalDistanceWithAllowSettings::clone() const
{
  girl::shared_ptr<MergeDriverMinimalDistanceWithAllowSettings> p(new MergeDriverMinimalDistanceWithAllowSettings);

  *p = *this;

  if (m_allowMergeSettings)
    p->m_allowMergeSettings = m_allowMergeSettings->clone();
  else
    p->m_allowMergeSettings = girl::shared_ptr<RegionBinaryPredicateSettings>();

  if (m_distanceSettings)
    p->m_distanceSettings = m_distanceSettings->clone();
  else
    p->m_distanceSettings = girl::shared_ptr<RegionBinaryFunctionSettings>();

  return p;
}

bool
MergeDriverMinimalDistanceWithAllowSettings::isValid() const
{
  return (
	  m_allowMergePredicate && (! m_allowMergePredicate->hasSettings() || (m_allowMergeSettings != girl::shared_ptr<RegionBinaryPredicateSettings>() && m_allowMergeSettings->isValid()))
	  && 
	  m_distanceFunction != NULL && (! m_distanceFunction->hasSettings() || (m_distanceSettings != girl::shared_ptr<RegionBinaryFunctionSettings>() && m_distanceSettings->isValid()))
	  );
}
