/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "MergeDriverTangentVarianceWidget.hpp"

#include <cassert>

#include <ise/iseMergeDriverTangentVariance.hpp>

#include <QGridLayout>
#include <QHBoxLayout>
#include <QLabel>

#include "MergeDriverPlugin.hpp"
#include "MergeDriverWidget.hpp"
#include "SliderSpinBoxWidget.hpp"
#include "WeightedChannelsColorSpaceWidget.hpp"

MergeDriverTangentVarianceWidget::MergeDriverTangentVarianceWidget(MergeDriverPlugin *mergeDriverPlugin,
						     QWidget *parent)
  :
  MergeDriverWidgetInterface(NULL, mergeDriverPlugin, parent),
  m_mergeDriverWidget(NULL),
  m_thresholdSB(NULL)
{
  assert(m_mergeDriverPlugin);

  setWindowTitle(tr("MergeDriverTangentVariance parameters..."));

  //setAttribute(Qt::WA_DeleteOnClose);

  buildGUI();
  
  setGUI();
}

void
MergeDriverTangentVarianceWidget::buildGUI()
{

  m_mergeDriverWidget = new MergeDriverWidget(this);

  QGridLayout *layout = new QGridLayout;

  QLabel *thresholdLabel = new QLabel(tr("Tangent variance distance threshold: "), this);
  layout->addWidget(thresholdLabel, 0, 0);
  m_thresholdSB = new SliderSpinBoxWidget(this);
  m_thresholdSB->setDecimals(10);
  m_thresholdSB->setSingleStep(0.0001);
  m_thresholdSB->setMinimum(0.0);
  m_thresholdSB->setMaximum(1.0);
  layout->addWidget(m_thresholdSB, 0, 1);
  
  layout->setMargin(0); //because embedded widget
  QWidget *w = new QWidget(this);
  w->setLayout(layout);
  
  QVBoxLayout *vLayout = new QVBoxLayout();
  vLayout->addWidget(m_mergeDriverWidget);
  vLayout->addWidget(w);
  
  setLayout(vLayout);

  connect(m_mergeDriverWidget, SIGNAL(parametersChanged()),
	  this, SIGNAL(parametersChanged()));
  connect(m_thresholdSB, SIGNAL(valueChanged(double)),
	  this, SIGNAL(parametersChanged()));

}

void
MergeDriverTangentVarianceWidget::setGUI()
{
  assert(m_mergeDriverPlugin);  

  girl::toger::neighbouringModes mode;
  girl::toger::markFlags m;
  
  girl::shared_ptr<ise::MergeDriverTangentVariance> algo = girl::static_pointer_cast<ise::MergeDriverTangentVariance>(m_mergeDriverPlugin->getNewMergeDriver());
  algo->getNeighbourhood(mode, m);
  
  m_mergeDriverWidget->setNeighbouringMode(mode);
  m_mergeDriverWidget->setMarkFlag(m);

  double threshold;
  algo->getParameters(threshold);
  m_thresholdSB->setValue(threshold);
  
}

void
MergeDriverTangentVarianceWidget::getGUI()
{
  assert(m_mergeDriverPlugin);  

  const girl::toger::neighbouringModes mode = m_mergeDriverWidget->getNeighbouringMode();
  const girl::toger::markFlags m = m_mergeDriverWidget->getMarkFlag();
  girl::shared_ptr<ise::MergeDriverTangentVariance> algo = girl::static_pointer_cast<ise::MergeDriverTangentVariance>(m_mergeDriverPlugin->getNewMergeDriver());
  algo->setNeighbourhood(mode, m);

  const double threshold = m_thresholdSB->value();
  
  algo->setParameters(threshold);
  
  m_mergeDriverPlugin->saveParameters(algo);
}

  
