/*
  Copyright (C) 2010-2011 Julian Roigt  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef GAUSSIANPYRAMIDPLUGIN_HPP
#define GAUSSIANPYRAMIDPLUGIN_HPP

#include <ise/iseGaussianPyramid.hpp>

#include <qiseTools/PyramidPlugin.hpp>

#include <qiseTools/GaussianPyramidWidget.hpp>
#include <qiseTools/GaussianPyramidSettings.hpp>

#include <QObject>

class GaussianPyramidPlugin : public QObject,
			 public PyramidPlugin
{

  Q_OBJECT
  Q_INTERFACES(PyramidPlugin QIsePlugin)
#if QT_VERSION >= 0x050000
  Q_PLUGIN_METADATA(IID PyramidPlugin_iid)
#endif

public:

  GaussianPyramidPlugin()
    : PyramidPlugin(QString::fromUtf8("Gaussian Pyramid"),
      QIseVersion(1, 0), QIseVersion(1, 0))
  {}

  ~GaussianPyramidPlugin() {}

  virtual girl::shared_ptr<ise::Pyramid> getNewPyramid()
    {
      girl::shared_ptr<ise::Pyramid> result(new ise::GaussianPyramid());
      if (m_settings)
	m_settings->configureAlgo(result);

      return result;
    }

  virtual girl::shared_ptr<PyramidSettingsInterface> createSettings()
  {
    return girl::shared_ptr<PyramidSettingsInterface>(new GaussianPyramidSettings());
  }

  virtual PyramidWidget* createWidget(MainView *mainView, int numLevel, int divider, QWidget *parent = NULL)
  {
    return new GaussianPyramidWidget(mainView, this, numLevel, divider, parent);
  }
};

#if QT_VERSION < 0x050000
Q_EXPORT_PLUGIN2(QIseGaussianPyramidPlugin, GaussianPyramidPlugin)
#endif

#endif /* ! GAUSSIANPYRAMIDPLUGIN_HPP */
