/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux boris.mansencal@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "RegionBinaryPredicateFromUnaryFunctionSettings.hpp"

#include "RegionBinaryPredicateFromUnaryFunctionWidget.hpp"


void 
RegionBinaryPredicateFromUnaryFunctionSettings::configureWidget(RegionBinaryPredicateWidget *widget) const
{
  RegionBinaryPredicateFromUnaryFunctionWidget *w = static_cast<RegionBinaryPredicateFromUnaryFunctionWidget *>(widget);
  assert(w);
  if (m_unaryFctPlugin) {
    m_unaryFctPlugin->setSettings(m_unaryFctSettings);
    w->setRegionUnaryFunctionPlugin(m_unaryFctPlugin);
  }
  
  w->setComparisonOperator(m_comp);

}

void
RegionBinaryPredicateFromUnaryFunctionSettings::getWidgetConfig(const RegionBinaryPredicateWidget *widget)
{
  const RegionBinaryPredicateFromUnaryFunctionWidget *w = static_cast<const RegionBinaryPredicateFromUnaryFunctionWidget *>(widget);
  
  m_unaryFctPlugin = w->getRegionUnaryFunctionPlugin();
  if (m_unaryFctPlugin)
    m_unaryFctSettings = m_unaryFctPlugin->settings();

  m_comp = w->getComparisonOperator();

}

void
RegionBinaryPredicateFromUnaryFunctionSettings::configureAlgo(girl::shared_ptr<ise::RegionBinaryPredicate> algo) const
{
  girl::shared_ptr<ise::RegionBinaryPredicateFromUnaryFunction> p =
    girl::static_pointer_cast<ise::RegionBinaryPredicateFromUnaryFunction>(algo);
    
  girl::shared_ptr<ise::RegionUnaryFunction> unaryFct;

  if (m_unaryFctPlugin) {
    unaryFct = m_unaryFctPlugin->getNewRegionUnaryFunction(m_unaryFctPlugin->getSegContext());
    if (m_unaryFctSettings)
      m_unaryFctSettings->configureAlgo(unaryFct);
  }

  p->setRegionUnaryFunction(unaryFct);
  p->setComparisonOperator(m_comp);
}

girl::shared_ptr<RegionBinaryPredicateSettings>
RegionBinaryPredicateFromUnaryFunctionSettings::clone() const
{
  girl::shared_ptr<RegionBinaryPredicateFromUnaryFunctionSettings> p(new RegionBinaryPredicateFromUnaryFunctionSettings);

  *p = *this;

  if (m_unaryFctSettings)
    p->m_unaryFctSettings = m_unaryFctSettings->clone();

  return p;
}

bool
RegionBinaryPredicateFromUnaryFunctionSettings::isValid() const
{
  const bool val = ((m_unaryFctPlugin != NULL) 
		    && (! m_unaryFctPlugin->hasSettings() 
			|| (m_unaryFctSettings != girl::shared_ptr<RegionUnaryFunctionSettings>() && m_unaryFctSettings->isValid())));
  return val;
}
