/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux boris.mansencal@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "RegionBinaryPredicateFromUnaryPredicateSettings.hpp"

#include "RegionBinaryPredicateFromUnaryPredicateWidget.hpp"


void 
RegionBinaryPredicateFromUnaryPredicateSettings::configureWidget(RegionBinaryPredicateWidget *widget) const
{
  RegionBinaryPredicateFromUnaryPredicateWidget *w = static_cast<RegionBinaryPredicateFromUnaryPredicateWidget *>(widget);
  assert(w);
  if (m_unaryPredPlugin) {
    m_unaryPredPlugin->setSettings(m_unaryPredSettings);
    w->setRegionUnaryPredicatePlugin(m_unaryPredPlugin);
  }
  
  w->setSelectedRegion(m_selectedRegion);

}

void
RegionBinaryPredicateFromUnaryPredicateSettings::getWidgetConfig(const RegionBinaryPredicateWidget *widget)
{
  const RegionBinaryPredicateFromUnaryPredicateWidget *w = static_cast<const RegionBinaryPredicateFromUnaryPredicateWidget *>(widget);
  
  m_unaryPredPlugin = w->getRegionUnaryPredicatePlugin();
  if (m_unaryPredPlugin)
    m_unaryPredSettings = m_unaryPredPlugin->settings();

  m_selectedRegion = w->getSelectedRegion();

}

void
RegionBinaryPredicateFromUnaryPredicateSettings::configureAlgo(girl::shared_ptr<ise::RegionBinaryPredicate> algo) const
{
  girl::shared_ptr<ise::RegionBinaryPredicateFromUnaryPredicate> p =
    girl::static_pointer_cast<ise::RegionBinaryPredicateFromUnaryPredicate>(algo);
    
  girl::shared_ptr<ise::RegionUnaryPredicate> unaryPred;

  if (m_unaryPredPlugin) {
    unaryPred = m_unaryPredPlugin->getNewRegionUnaryPredicate(m_unaryPredPlugin->getSegContext());
    if (m_unaryPredSettings)
      m_unaryPredSettings->configureAlgo(unaryPred);
  }

  p->setRegionUnaryPredicate(unaryPred);
  p->setSelectedRegion(m_selectedRegion);
}

girl::shared_ptr<RegionBinaryPredicateSettings>
RegionBinaryPredicateFromUnaryPredicateSettings::clone() const
{
  girl::shared_ptr<RegionBinaryPredicateFromUnaryPredicateSettings> p(new RegionBinaryPredicateFromUnaryPredicateSettings);

  *p = *this;

  if (m_unaryPredSettings)
    p->m_unaryPredSettings = m_unaryPredSettings->clone();

  return p;
}

bool
RegionBinaryPredicateFromUnaryPredicateSettings::isValid() const
{
  return ((m_unaryPredPlugin != NULL) 
	  && (! m_unaryPredPlugin->hasSettings() || (m_unaryPredSettings != girl::shared_ptr<RegionUnaryPredicateSettings>() && m_unaryPredSettings->isValid()))
	  && (m_selectedRegion == 0 || m_selectedRegion == 1));
}
