/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux boris.mansencal@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "RegionBinaryPredicateFromUnaryPredicateWidget.hpp"

#include <qiseTools/ComparisonFunctionWidget.hpp>
#include <qiseTools/RegionBinaryFunctionPlugin.hpp>
//#include <qiseTools/RegionBinaryFunctionWidget.hpp>
#include <qiseTools/RegionUnaryPredicateComboWidget.hpp>

#include <QGridLayout>
#include <QLabel>
#include <QRadioButton>




RegionBinaryPredicateFromUnaryPredicateWidget::RegionBinaryPredicateFromUnaryPredicateWidget(MainView *mainView, /*ise::SegmentationContext *segContext, */
											     RegionBinaryPredicatePlugin *plugin, 
											     QWidget *parent)
  : RegionBinaryPredicateWidget(plugin, parent),
    //m_segContext(segContext),
    m_mainView(mainView),
    m_rupcw(NULL),
    m_firstRegionButton(NULL),
    m_secondRegionButton(NULL)
{
  buildGUI();
}


void
RegionBinaryPredicateFromUnaryPredicateWidget::buildGUI()
{
  
  QGridLayout *gL = new QGridLayout;


  m_rupcw = new RegionUnaryPredicateComboWidget(m_mainView/*m_segContext*/, this);

  QLabel *label = new QLabel(tr("Applied to: "), this);
  m_firstRegionButton = new QRadioButton("First region", this);
  m_secondRegionButton = new QRadioButton("Second region", this);

  connect(m_rupcw, SIGNAL(pluginChanged()), this, SIGNAL(pluginChanged()));
  connect(m_firstRegionButton, SIGNAL(toggled(bool)), this, SLOT(signalSelectedRegionChanged()));
  connect(m_secondRegionButton, SIGNAL(toggled(bool)), this, SLOT(signalSelectedRegionChanged()));

  gL->addWidget(m_rupcw, 0, 0, 1, 2);
  gL->addWidget(label, 2, 0);
  gL->addWidget(m_firstRegionButton, 3, 0);
  gL->addWidget(m_secondRegionButton, 3, 1);
  
  setLayout(gL);

  m_firstRegionButton->setChecked(true);

  /*
  connect(m_pw, SIGNAL(pluginIndexChanged(int)),
	  this, SLOT(pluginChanged(int)));
  */
}



RegionUnaryPredicatePlugin *
RegionBinaryPredicateFromUnaryPredicateWidget::getRegionUnaryPredicatePlugin() const
{
  assert(m_rupcw);
  return m_rupcw->getRegionUnaryPredicatePlugin();
}

void
RegionBinaryPredicateFromUnaryPredicateWidget::setRegionUnaryPredicatePlugin(RegionUnaryPredicatePlugin *p)
{
  assert(m_rupcw);
  m_rupcw->setRegionUnaryPredicatePlugin(p);
}

int
RegionBinaryPredicateFromUnaryPredicateWidget::getSelectedRegion() const
{
  assert(m_firstRegionButton);
  assert(m_secondRegionButton);
  if (m_firstRegionButton->isChecked()) {
    return 0;
  }
  else {
    assert(m_secondRegionButton->isChecked());
    return 1;
  }
  
}

void
RegionBinaryPredicateFromUnaryPredicateWidget::setSelectedRegion(int i)
{
  if (i == 0) {
    m_firstRegionButton->setChecked(true);
  }
  else {
    m_secondRegionButton->setChecked(true);
  }
}


void
RegionBinaryPredicateFromUnaryPredicateWidget::signalSelectedRegionChanged()
{
  QRadioButton *rb = qobject_cast<QRadioButton *>(sender());
  assert(rb);
  if (rb->isChecked())
    emit selectedRegionChanged();
  
}

