/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux boris.mansencal@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "RegionBinaryPredicateThresholdSettings.hpp"

#include "RegionBinaryPredicateThresholdWidget.hpp"


void 
RegionBinaryPredicateThresholdSettings::configureWidget(RegionBinaryPredicateWidget *widget) const
{
  RegionBinaryPredicateThresholdWidget *w = static_cast<RegionBinaryPredicateThresholdWidget *>(widget);
  assert(w);
  if (m_binaryFctPlugin) {
    m_binaryFctPlugin->setSettings(m_binaryFctSettings);
    w->setRegionBinaryFunctionPlugin(m_binaryFctPlugin);
  }
  
  w->setComparisonOperator(m_comp);

  w->setThreshold(m_threshold);
}

void
RegionBinaryPredicateThresholdSettings::getWidgetConfig(const RegionBinaryPredicateWidget *widget)
{
  const RegionBinaryPredicateThresholdWidget *w = static_cast<const RegionBinaryPredicateThresholdWidget *>(widget);
  
  m_binaryFctPlugin = w->getRegionBinaryFunctionPlugin();
  if (m_binaryFctPlugin)
    m_binaryFctSettings = m_binaryFctPlugin->settings();

  m_comp = w->getComparisonOperator();

  m_threshold = w->getThreshold();

}

void
RegionBinaryPredicateThresholdSettings::configureAlgo(girl::shared_ptr<ise::RegionBinaryPredicate> algo) const
{
  girl::shared_ptr<ise::RegionBinaryPredicateThreshold> p =
    girl::static_pointer_cast<ise::RegionBinaryPredicateThreshold>(algo);
    
  girl::shared_ptr<ise::RegionBinaryFunction> binaryFct;

  if (m_binaryFctPlugin) {
    binaryFct = m_binaryFctPlugin->getNewRegionBinaryFunction(m_binaryFctPlugin->getSegContext());
    if (m_binaryFctSettings)
      m_binaryFctSettings->configureAlgo(binaryFct);
  }

  p->setRegionBinaryFunction(binaryFct);
  p->setComparisonOperator(m_comp);
  p->setThreshold(m_threshold);
}

girl::shared_ptr<RegionBinaryPredicateSettings>
RegionBinaryPredicateThresholdSettings::clone() const
{
  girl::shared_ptr<RegionBinaryPredicateThresholdSettings> p(new RegionBinaryPredicateThresholdSettings);

  *p = *this;

  if (m_binaryFctSettings)
    p->m_binaryFctSettings = m_binaryFctSettings->clone();

  return p;
}

bool
RegionBinaryPredicateThresholdSettings::isValid() const
{
  return ((m_binaryFctPlugin != NULL) 
	  && (! m_binaryFctPlugin->hasSettings() 
	      || (m_binaryFctSettings != girl::shared_ptr<RegionBinaryFunctionSettings>() && m_binaryFctSettings->isValid())));
}
