/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux boris.mansencal@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "RegionBinaryPredicateThresholdWidget.hpp"

#include <qiseTools/ComparisonFunctionWidget.hpp>
#include <qiseTools/RegionBinaryFunctionPlugin.hpp>
//#include <qiseTools/RegionBinaryFunctionWidget.hpp>
#include <qiseTools/RegionBinaryFunctionComboWidget.hpp>

#include <QGridLayout>
#include <QLabel>
#include <QLineEdit>
#include <QDoubleValidator>




RegionBinaryPredicateThresholdWidget::RegionBinaryPredicateThresholdWidget(MainView *mainView/*ise::SegmentationContext *segContext,*/ ,
									 RegionBinaryPredicatePlugin *plugin, 
									 QWidget *parent)
  : RegionBinaryPredicateWidget(plugin, parent),
    //m_segContext(segContext),
    m_mainView(mainView),
    m_rbfcw(NULL),
    m_cfw(NULL),
    m_threshold(NULL)
{
  buildGUI();
}


void
RegionBinaryPredicateThresholdWidget::buildGUI()
{
  assert(m_mainView);
  // assert(m_mainView->model());
  // assert(m_mainView->model()->getSegContextPtr());

  QGridLayout *gL = new QGridLayout;


  m_rbfcw = new RegionBinaryFunctionComboWidget(m_mainView/*m_segContext*/, this);

  QLabel *cfLabel = new QLabel(tr("Comparison operator: "), this);
  m_cfw = new ComparisonFunctionWidget(this);

  QLabel *thLabel = new QLabel(tr("Threshold: "), this);
  m_threshold = new QLineEdit(this);
  m_threshold->setValidator(new QDoubleValidator(m_threshold));

  connect(m_rbfcw, SIGNAL(pluginChanged()), this, SIGNAL(pluginChanged()));
  connect(m_cfw, SIGNAL(comparisonOperatorChanged()), this, SIGNAL(comparisonOperatorChanged()));
  connect(m_threshold, SIGNAL(textChanged(const QString &)), this, SLOT(signalThresholdChanged()));


  gL->addWidget(m_rbfcw, 0, 0, 1, 2);
  gL->addWidget(cfLabel, 2, 0);
  gL->addWidget(m_cfw, 2, 1);
  gL->addWidget(thLabel, 3, 0);
  gL->addWidget(m_threshold, 3, 1);
  
  setLayout(gL);

  /*
  connect(m_pw, SIGNAL(pluginIndexChanged(int)),
	  this, SLOT(pluginChanged(int)));
  */
}



RegionBinaryFunctionPlugin *
RegionBinaryPredicateThresholdWidget::getRegionBinaryFunctionPlugin() const
{
  assert(m_rbfcw);
  return m_rbfcw->getRegionBinaryFunctionPlugin();
}

void
RegionBinaryPredicateThresholdWidget::setRegionBinaryFunctionPlugin(RegionBinaryFunctionPlugin *p)
{
  assert(m_rbfcw);
  m_rbfcw->setRegionBinaryFunctionPlugin(p);
}



ise::ComparisonOperator
RegionBinaryPredicateThresholdWidget::getComparisonOperator() const
{
  assert(m_cfw);
  return m_cfw->getComparisonOperator();
}

void
RegionBinaryPredicateThresholdWidget::setComparisonOperator(ise::ComparisonOperator c)
{
  assert(m_cfw);
  m_cfw->setComparisonOperator(c);
}

double
RegionBinaryPredicateThresholdWidget::getThreshold() const
{
  assert(m_threshold);
  return m_threshold->text().toDouble();
}    

void
RegionBinaryPredicateThresholdWidget::setThreshold(double th)
{
  m_threshold->setText(QString::number(th));
}

void
RegionBinaryPredicateThresholdWidget::signalThresholdChanged()
{
  emit thresholdChanged(getThreshold());
}

