/*
  Copyright (C) 2010-2011 Remi Vieux  remi.vieux@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef REGIONFEATURECONTEXTUALPLUGIN_HPP_
#define REGIONFEATURECONTEXTUALPLUGIN_HPP_

#include <ise/iseRegionSetFactoryHandler.hpp>
#include <ise/iseRegionFeatureNeighbourhoodContext.hpp>
#include <ise/iseRegionFeatureComputer.hpp>

#include <qiseTools/RegionFeatureComputerPlugin.hpp>
#include <qiseTools/RegionFeatureContextualDialog.hpp>

#include <QObject>

class RegionFeatureContextualPlugin : public QObject,
                                     public RegionFeatureComputerPlugin
{

  Q_OBJECT
  Q_INTERFACES(RegionFeatureComputerPlugin QIsePlugin)
#if QT_VERSION >= 0x050000
  Q_PLUGIN_METADATA(IID RegionFeatureComputerPlugin_iid)
#endif

public:

  RegionFeatureContextualPlugin()
  : RegionFeatureComputerPlugin(QString::fromUtf8("Contextual Region Features"),
      QIseVersion(1, 0), QIseVersion(1, 0)),
      m_memento()
  {}

  ~RegionFeatureContextualPlugin() {}

  virtual girl::shared_ptr<ise::RegionFeatureComputer> getNewRegionFeatureComputer()
    {
      girl::shared_ptr<ise::RegionFeatureComputer> result(new ise::RegionFeatureNeighbourhoodContext(m_segContext));
      if (m_memento == NULL)
        m_memento = result->getMemento();
      else
        result->setMemento(m_memento);
      return result;
    }

  virtual void saveParameters(const girl::shared_ptr<ise::RegionFeatureComputer> &rfc)
  {
    m_memento = rfc->getMemento();
  }

  virtual RegionFeatureComputerDialog* parameterWidget(ise::SegmentationContext *segContext,
                                                       QWidget *parent = NULL)
  {
    m_segContext = segContext;
    return new RegionFeatureContextualDialog(segContext,
					    tr("Contextual Region Features"),
					    getNewRegionFeatureComputer(),
					    parent);
  }


protected:

  girl::shared_ptr<ise::ParametersMemento> m_memento;


};

#if QT_VERSION < 0x050000
Q_EXPORT_PLUGIN2(QIseRegionFeatureContextualPlugin, RegionFeatureContextualPlugin)
#endif

#endif /* REGIONFEATURECONTEXTUALPLUGIN_HPP_ */
