/*
  Copyright (C) 2010-2011 Remi Vieux  remi.vieux@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef REGIONFEATURERUNLENGTHPLUGIN_HPP_
#define REGIONFEATURERUNLENGTHPLUGIN_HPP_

#include <ise/iseRegionSetFactoryHandler.hpp>
#include <ise/iseRegionFeatureRunLength.hpp>
#include <ise/iseRegionFeatureComputer.hpp>

#include <qiseTools/RegionFeatureComputerPlugin.hpp>
#include <qiseTools/RegionFeatureRunLengthDialog.hpp>

#include <QObject>

class RegionFeatureRunLengthPlugin : public QObject,
                                     public RegionFeatureComputerPlugin
{

  Q_OBJECT
  Q_INTERFACES(RegionFeatureComputerPlugin QIsePlugin)

public:

  RegionFeatureRunLengthPlugin()
  : RegionFeatureComputerPlugin(QString::fromUtf8("Run Length Features"),
      QIseVersion(1, 0), QIseVersion(1, 0)),
      m_memento()
  {}

  ~RegionFeatureRunLengthPlugin() {}

  virtual girl::shared_ptr<ise::RegionFeatureComputer> getNewRegionFeatureComputer()
    {
      girl::shared_ptr<ise::RegionFeatureComputer> result(new ise::RegionFeatureRunLength(m_segContext));
      if (m_memento == NULL)
        m_memento = result->getMemento();
      else
        result->setMemento(m_memento);
      return result;
    }

  virtual void saveParameters(const girl::shared_ptr<ise::RegionFeatureComputer> &rfc)
  {
    m_memento = rfc->getMemento();
  }

  virtual RegionFeatureComputerDialog* parameterWidget(ise::SegmentationContext *segContext,
                                                       QWidget *parent = NULL)
  {
    m_segContext = segContext;
    return new RegionFeatureRunLengthDialog(segContext,
					    tr("Run Length Moments"),
					    getNewRegionFeatureComputer(),
					    parent);
  }


protected:

  girl::shared_ptr<ise::ParametersMemento> m_memento;


};

Q_EXPORT_PLUGIN2(QIseRegionFeatureRunLengthPlugin, RegionFeatureRunLengthPlugin)

#endif /* REGIONFEATURERUNLENGTHPLUGIN_HPP_ */
