/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef REGIONSETINTERFACEDISTRIBUTIONPLUGIN_HPP_
#define REGIONSETINTERFACEDISTRIBUTIONPLUGIN_HPP_

#include <ise/iseRegionSetInterfaceDistribution.hpp>

#include <qiseTools/RegionSetInterfacePlugin.hpp>

#include "RegionSetInterfaceSettingsDistribution.hpp"
#include "RegionSetInterfaceWidgetDistribution.hpp"


/**
 * Plugin for Region Pixel Distribution according to a codebook (quantization)
 */
class RegionSetInterfaceDistributionPlugin : public QObject,
public RegionSetInterfacePlugin
{

  Q_OBJECT
  Q_INTERFACES(RegionSetInterfacePlugin QIsePlugin)
#if QT_VERSION >= 0x050000
  Q_PLUGIN_METADATA(IID RegionSetInterfacePlugin_iid)
#endif


public:
  RegionSetInterfaceDistributionPlugin() :
    RegionSetInterfacePlugin(QString::fromUtf8("Pixels Distribution"),
        QIseVersion(1, 0),
        QIseVersion(1, 0))
  {}
  virtual
  ~RegionSetInterfaceDistributionPlugin() {}

protected:

  girl::shared_ptr<ise::FeatureComputer>
  create(ise::SegmentationContext *segContext)
  {
    girl::shared_ptr<ise::FeatureComputer> result(
        new ise::RegionSetInterfaceDistribution(segContext));
    return result;
  }

  RegionSetInterfaceWidget*
  createWidget(MainView *mainView, FeatureComputerGUIInfoPtr featureComputerGUIInfo, QWidget* parent)
  {
    return new RegionSetInterfaceWidgetDistribution(mainView, featureComputerGUIInfo, this, parent);
  }

  virtual girl::shared_ptr<FeatureInterfaceSettings>
  createSettings()
  {
    girl::shared_ptr<FeatureInterfaceSettings> settings
    (new RegionSetInterfaceSettingsDistribution());
    return settings;
  }
};

#if QT_VERSION < 0x050000
Q_EXPORT_PLUGIN2(QIseRegionSetInterfaceDistributionPlugin, RegionSetInterfaceDistributionPlugin)
#endif

#endif /* REGIONSETINTERFACEDISTRIBUTIONPLUGIN_HPP_ */
