/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "RegionSetInterfaceSettingsDistribution.hpp"

#include <ise/iseRegionSetInterfaceDistribution.hpp>
#include <ise/iseSegmentationContext.hpp>

#include "RegionSetInterfaceWidgetDistribution.hpp"


RegionSetInterfaceSettingsDistribution::RegionSetInterfaceSettingsDistribution() :
FeatureInterfaceSettings(),
m_dataPtr(),
m_codebookSize(ise::RegionSetInterfaceDistribution::DEFAULT_CODEBOOK_SIZE),
m_flags(ise::RegionSetInterfaceDistribution::DEFAULT_CENTER_FLAGS)
{

}

RegionSetInterfaceSettingsDistribution::~RegionSetInterfaceSettingsDistribution()
{
}


void
RegionSetInterfaceSettingsDistribution::configureWidget(FeatureInterfaceWidget *widget) const
{
  RegionSetInterfaceWidgetDistribution *distribWidget =
      static_cast<RegionSetInterfaceWidgetDistribution*>(widget);
  assert(distribWidget);
  distribWidget->setParameters(m_dataPtr, m_codebookSize, m_flags);

}

void
RegionSetInterfaceSettingsDistribution::getWidgetConfig(const FeatureInterfaceWidget *widget)
{
  const RegionSetInterfaceWidgetDistribution *distribWidget =
      static_cast<const RegionSetInterfaceWidgetDistribution*>(widget);
  assert(distribWidget);
  distribWidget->getParameters(m_dataPtr, m_codebookSize, m_flags);
}

void
RegionSetInterfaceSettingsDistribution::configureAlgo(girl::shared_ptr<ise::FeatureComputer> algo) const
{
  girl::shared_ptr<ise::RegionSetInterfaceDistribution> distrib =
      girl::static_pointer_cast<ise::RegionSetInterfaceDistribution>(algo);

  assert(algo);
  assert(m_dataPtr);

  distrib->setParameters(m_codebookSize, m_flags, girl::static_pointer_cast<ise::ImageFeatureComputer>(m_dataPtr));
}

girl::shared_ptr<FeatureInterfaceSettings> 
RegionSetInterfaceSettingsDistribution::clone() const
{
  girl::shared_ptr<RegionSetInterfaceSettingsDistribution> p(new RegionSetInterfaceSettingsDistribution());

  p->m_dataPtr = m_dataPtr;
  p->m_codebookSize = m_codebookSize;
  p->m_flags = m_flags;

  return p;
}

bool
RegionSetInterfaceSettingsDistribution::isValid() const
{
  return (m_codebookSize > 0); //B: ???
}
