/*
  Copyright (C) 2005-2010 Remi Vieux, Boris Mansencal remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "RegionSetInterfaceWidgetDistribution.hpp"

#include <limits>

#include <qiseTools/MainView.hpp>

#include <opencv2/core/core.hpp>

#include <QComboBox>
#include <QLabel>
#include <QSpinBox>
#include <QHBoxLayout>


RegionSetInterfaceWidgetDistribution::RegionSetInterfaceWidgetDistribution(MainView *mainView,
									   FeatureComputerGUIInfoPtr featureComputerGUIInfo,
									   RegionSetInterfacePlugin *plugin,
									   QWidget *parent)
  :
  RegionSetInterfaceWidget(mainView, featureComputerGUIInfo, plugin, parent),
  m_segContext(mainView->model()->getSegContextPtr())
{
  buildGUI();
}

RegionSetInterfaceWidgetDistribution::~RegionSetInterfaceWidgetDistribution()
{
}


void
RegionSetInterfaceWidgetDistribution::buildGUI()
{
  QVBoxLayout *mainLayout = new QVBoxLayout;

  m_ifcw = new ImageFeatureChoiceWidget(m_mainView, m_featureComputerGUIInfo, this);
  mainLayout->addWidget(m_ifcw);

  QHBoxLayout *layout = new QHBoxLayout;
  QLabel *codebookLabel = new QLabel(tr("Codebook size:"), this);
  m_codebookSizeSpinBox = new QSpinBox(this);
  m_codebookSizeSpinBox->setRange(2, std::numeric_limits<int>::max());
  layout->addWidget(codebookLabel);
  layout->addWidget(m_codebookSizeSpinBox);
  mainLayout->addLayout(layout);

  QHBoxLayout *layout2 = new QHBoxLayout;
  QLabel *comboLabel = new QLabel(tr("Init method"), this);
  layout2->addWidget(comboLabel);
  m_flagsCombo = new QComboBox(this);
  m_flagsCombo->addItem(tr("Random centers initialisation"), QVariant(cv::KMEANS_RANDOM_CENTERS));
  m_flagsCombo->addItem(tr("KMeans++ centers initialisation"), QVariant(cv::KMEANS_PP_CENTERS));
  layout2->addWidget(m_flagsCombo);
  mainLayout->addLayout(layout2);

  setLayout(mainLayout);
}


void
RegionSetInterfaceWidgetDistribution::getParameters(ise::FeatureComputerPtr &algo, int &codebookSize, int &flags) const
{
  algo = m_ifcw->imageFeatureComputerPtr();
  assert(algo);
  codebookSize = m_codebookSizeSpinBox->value();
  QVariant var = m_flagsCombo->itemData(m_flagsCombo->currentIndex());
  assert(var != QVariant::Invalid);
  flags = var.toInt();
}

void
RegionSetInterfaceWidgetDistribution::setParameters(const ise::FeatureComputerPtr &algo, const int &codebookSize, const int &flags)
{
  m_ifcw->setImageFeatureComputerPtr(algo);
  m_codebookSizeSpinBox->setValue(codebookSize);
  for (int i = 0; i < m_flagsCombo->count(); ++i) {
      QVariant var = m_flagsCombo->itemData(i);
      if (var.toInt() == flags) {
          m_flagsCombo->setCurrentIndex(i);
          break;
      }
  }

}
