/*
  Copyright (C) 2005-2011 Remi Vieux, Boris Mansencal  remi.vieux@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
/*
 * RegionSetInterfaceSettingsMean.cpp
 *
 *  Created on: Dec 3, 2010
 *      Author: vieux
 */

#include "RegionSetInterfaceSettingsMean.hpp"

#include <ise/iseImageFeatureComputerDefault.hpp>
#include <ise/iseRegionSetInterfaceMean.hpp>

#include "RegionSetInterfaceWidgetMean.hpp"

RegionSetInterfaceSettingsMean::RegionSetInterfaceSettingsMean()
: FeatureInterfaceSettings(),
  m_regSetMoment(girl::shared_ptr<ise::RegionSetInterfaceMoments>())
{

}

RegionSetInterfaceSettingsMean::~RegionSetInterfaceSettingsMean()
{
}

void
RegionSetInterfaceSettingsMean::configureWidget(FeatureInterfaceWidget *widget) const
{
  try {
    RegionSetInterfaceWidgetMean *widgetMean =
        static_cast<RegionSetInterfaceWidgetMean *>(widget);
    //TODO:BUG what should be the default parameters here? Because
    // setting the GUI with empty parameters will cause crash
    widgetMean->setParameters(m_regSetMoment);

  } catch (std::bad_cast& bc) {
    std::cerr << bc.what() << std::endl;
    assert(false);
  }
}

void
RegionSetInterfaceSettingsMean::getWidgetConfig(const FeatureInterfaceWidget *widget)
{
  try {
    const RegionSetInterfaceWidgetMean *widgetMean =
        static_cast<const RegionSetInterfaceWidgetMean *>(widget);
    //TODO:BUG what should be the default parameters here? Because
    // setting the GUI with empty parameters will cause crash
    widgetMean->getParameters(m_regSetMoment);

  } catch (std::bad_cast& bc) {
    std::cerr << bc.what() << std::endl;
    assert(false);
  }
}

void
RegionSetInterfaceSettingsMean::configureAlgo(girl::shared_ptr<ise::FeatureComputer> algo) const
{
  assert(algo);
  girl::shared_ptr<ise::RegionSetInterfaceMean> regSetMean =
      girl::static_pointer_cast<ise::RegionSetInterfaceMean>(algo);
  assert(regSetMean);

  // Have to create the object to check if it is in the region set. Create it with no cache
  // so that the object is "simple" to create
  assert(m_regSetMoment);
  regSetMean->setParameters(m_regSetMoment);
}


girl::shared_ptr<FeatureInterfaceSettings>
RegionSetInterfaceSettingsMean::clone() const
{
  girl::shared_ptr<RegionSetInterfaceSettingsMean> p(new RegionSetInterfaceSettingsMean());
  *p = *this;
  return p;
}

bool
RegionSetInterfaceSettingsMean::isValid() const
{
  return (m_regSetMoment != girl::shared_ptr<ise::RegionSetInterfaceMoments>() );
}
