/*
  Copyright (C) 2005-2011 Remi Vieux, Boris Mansencal  remi.vieux@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 */

#include "RegionSetInterfaceSettingsMoments.hpp"

#include <ise/iseImageFeatureComputerDefault.hpp>
#include <ise/iseRegionSetInterfaceMoments.hpp>
#include "RegionSetInterfaceWidgetMoments.hpp"

RegionSetInterfaceSettingsMoments::RegionSetInterfaceSettingsMoments()
: FeatureInterfaceSettings(),
  m_algo()
{
}

RegionSetInterfaceSettingsMoments::~RegionSetInterfaceSettingsMoments()
{
}


void
RegionSetInterfaceSettingsMoments::configureWidget(FeatureInterfaceWidget *widget) const
{
  RegionSetInterfaceWidgetMoments *moments =
      static_cast<RegionSetInterfaceWidgetMoments*>(widget);
  assert(moments);
  moments->setParameters(m_algo);
}

void
RegionSetInterfaceSettingsMoments::getWidgetConfig(const FeatureInterfaceWidget *widget)
{
  const RegionSetInterfaceWidgetMoments *moments =
      static_cast<const RegionSetInterfaceWidgetMoments *>(widget);
  assert(moments);
  moments->getParameters(m_algo);

}

void
RegionSetInterfaceSettingsMoments::configureAlgo(girl::shared_ptr<ise::FeatureComputer> algo) const
{
  girl::shared_ptr<ise::RegionSetInterfaceMoments> regSetMoments =
      girl::static_pointer_cast<ise::RegionSetInterfaceMoments>(algo);

  assert(regSetMoments);

  regSetMoments->setImageFeature(girl::static_pointer_cast<ise::ImageFeatureComputer>(m_algo));

}

girl::shared_ptr<FeatureInterfaceSettings>
RegionSetInterfaceSettingsMoments::clone() const
{
  girl::shared_ptr<RegionSetInterfaceSettingsMoments> p(new RegionSetInterfaceSettingsMoments());
  p->m_algo = m_algo;

  return p;
}

bool
RegionSetInterfaceSettingsMoments::isValid() const
{
  return true; //B: ???
}
