/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "RegionSetInterfaceMomentsGeometric.hpp"

#include <ise/iseConversion.hpp>

RegionSetInterfaceMomentsGeometric::RegionSetInterfaceMomentsGeometric(ise::SegmentationContext* segContext,
								       ise::RegionSetInterface::CacheType cache):
  RegionSetInterface(segContext),
  m_m0(0),
  m_m1x(0),
  m_m1y(0),
  m_m2xy(0),
  m_m2xx(0),
  m_m2yy(0)
{
  setCacheState(cache);
}

RegionSetInterfaceMomentsGeometric::~RegionSetInterfaceMomentsGeometric()
{
}

void
RegionSetInterfaceMomentsGeometric::setParameters()
{
}
  
void
RegionSetInterfaceMomentsGeometric::getParameters() const 
{
}

void
RegionSetInterfaceMomentsGeometric::processPoint(const girl::point& pt)
{
  m_m0++;
  int x = pt.x();
  int y = pt.y();
  m_m1x += x;
  m_m1y += y;
  m_m2xy += x * y;
  m_m2xx += x * x;
  m_m2yy += y * y;
}

void
RegionSetInterfaceMomentsGeometric::compute(const girl::region& id, ise::RegionFeature& feature)
{
  assert(m_segContext);
  assert(m_segContext->getToger());

  m_m0 = 0;
  m_m1x = 0;
  m_m1y = 0;
  m_m2xy = 0;
  m_m2xx = 0;
  m_m2yy = 0;

  girl::toger* tog = m_segContext->getToger();
  assert(tog);
  tog->iterateOnRegionDomain(id, *this);

  feature[0] = m_m0;
  feature[1] = m_m1x;
  feature[2] = m_m1y;
  feature[3] = m_m2xy;
  feature[4] = m_m2xx;
  feature[5] = m_m2yy;
}

void
RegionSetInterfaceMomentsGeometric::mergeFeatures(ise::RegionFeature& kept, ise::RegionFeature& del,
						  const girl::region& /*keptReg*/, const girl::region& /*delReg*/)
{
  for (int i = 0 ; i < FEATURE_DIMENSION ; ++i) {
    kept[i] += del[i];
  }
}


