/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef ISEREGIONSETINTERFACEMOMENTSGEOMETRIC_HPP
#define ISEREGIONSETINTERFACEMOMENTSGEOMETRIC_HPP

#include <girl/girlCommon.hpp>

#include <ise/iseRegionSetInterface.hpp>
#include <ise/iseSegmentationContext.hpp>

class GIRL_API RegionSetInterfaceMomentsGeometric : public ise::RegionSetInterface 
						  , public girl::toger::pointIterator
{
  static const int FEATURE_DIMENSION = 6;
  static const ise::RegionSetInterface::CacheType DEFAULT_CACHE_STRATEGY = ise::RegionSetInterface::LAZY;
 public:

  RegionSetInterfaceMomentsGeometric(ise::SegmentationContext* segContext, 
				     ise::RegionSetInterface::CacheType cacheType = DEFAULT_CACHE_STRATEGY);
    
  virtual ~RegionSetInterfaceMomentsGeometric();

  inline const
  std::string uri() const
  {
    std::stringstream ss(std::ios::in|std::ios::out);
    ss << __FILE__ << std::endl;
    return ss.str();
  }

  inline size_t
  getFeatureDim() const
  {
    return FEATURE_DIMENSION;
  }

  inline void
  getDimDescription(std::vector<std::string>& dimDesc) const
  {
    dimDesc.clear();
    dimDesc.push_back("m0");
    dimDesc.push_back("m1 x");
    dimDesc.push_back("m1 y");
    dimDesc.push_back("m2 xy");
    dimDesc.push_back("m2 xx");
    dimDesc.push_back("m2 yy");
  }

  void
  processPoint(const girl::point &pt);

  void
  setParameters();

  void
  getParameters() const;

  void
  getRequiredComputers(ise::FeatureComputer::FeatureComputerListType&) const {}

  const std::string
  help() const { return "TODO JULIAN"; }

 protected:

  void compute(const girl::region& id, ise::RegionFeature& feature);

  void mergeFeatures(ise::RegionFeature& kept, ise::RegionFeature& del,
		     const girl::region& keptId, const girl::region& delId);

 private:
  double m_m0;
  double m_m1x;
  double m_m1y;
  double m_m2xy;
  double m_m2xx;
  double m_m2yy;
};


#endif /* !  ISEREGIONSETINTERFACEMOMENTSGEOMETRIC_HPP */
