/*
  Copyright (C) 2005-2011 Remi Vieux, Boris Mansencal  remi.vieux@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "RegionSetInterfaceSettingsNeighboursPosition.hpp"

#include <ise/iseRegionSetInterfaceNeighboursPosition.hpp>

#include "RegionSetInterfaceWidgetNeighboursPosition.hpp"

RegionSetInterfaceSettingsNeighboursPosition::RegionSetInterfaceSettingsNeighboursPosition()
  : FeatureInterfaceSettings(),
  m_numberOfBins(ise::RegionSetInterfaceNeighboursPosition::DEFAULT_NUMBER_OF_BINS),
    m_startAngle(ise::RegionSetInterfaceNeighboursPosition::DEFAULT_START_ANGLE),
    m_regSetGravityCenter(girl::shared_ptr<ise::RegionSetInterfaceGravityCenter>())
{
}

RegionSetInterfaceSettingsNeighboursPosition::~RegionSetInterfaceSettingsNeighboursPosition()
{
}

void
RegionSetInterfaceSettingsNeighboursPosition::configureWidget(FeatureInterfaceWidget *genericWidget) const
{
  try {
    RegionSetInterfaceWidgetNeighboursPosition *widget =
        static_cast<RegionSetInterfaceWidgetNeighboursPosition *>(genericWidget);
    widget->setParameters(m_regSetGravityCenter, m_numberOfBins, m_startAngle);
  } catch(std::bad_cast &bc) {
    std::cerr << bc.what() << std::endl;
    assert(false);
  }
}

void
RegionSetInterfaceSettingsNeighboursPosition::getWidgetConfig(const FeatureInterfaceWidget *genericWidget)
{
  try {
    const RegionSetInterfaceWidgetNeighboursPosition *widget =
        static_cast<const RegionSetInterfaceWidgetNeighboursPosition *>(genericWidget);
    widget->getParameters(m_regSetGravityCenter, m_numberOfBins, m_startAngle);
  } catch (std::bad_cast &bc) {
    std::cerr << bc.what() << std::endl;
    assert(false);
  }
}

void
RegionSetInterfaceSettingsNeighboursPosition::configureAlgo(girl::shared_ptr<ise::FeatureComputer> algo) const
{
  girl::shared_ptr<ise::RegionSetInterfaceNeighboursPosition> regSet =
      girl::static_pointer_cast<ise::RegionSetInterfaceNeighboursPosition>(algo);
  assert(regSet);

  regSet->setParameters(m_regSetGravityCenter, m_numberOfBins, m_startAngle);


}

girl::shared_ptr<FeatureInterfaceSettings>
RegionSetInterfaceSettingsNeighboursPosition::clone() const
{
  girl::shared_ptr<RegionSetInterfaceSettingsNeighboursPosition> p(new RegionSetInterfaceSettingsNeighboursPosition());
  *p = *this;
  return p;
}

bool
RegionSetInterfaceSettingsNeighboursPosition::isValid() const
{
  return (m_numberOfBins > 0 && m_regSetGravityCenter); //B: ???
}
