/*
 * RegionSetInterfaceWidgetNeighboursPosition.cpp
 *
 *  Created on: Dec 9, 2010
 *      Author: vieux
 */

#include "RegionSetInterfaceWidgetNeighboursPosition.hpp"

#include <qiseTools/SliderSpinBoxWidget.hpp>
#include <qiseTools/MainView.hpp>
#include <qiseTools/RegionFeatureChoiceGravityCenterWidget.hpp>

#include <QLabel>
#include <QVBoxLayout>

RegionSetInterfaceWidgetNeighboursPosition::RegionSetInterfaceWidgetNeighboursPosition(MainView *mainView,
										       FeatureComputerGUIInfoPtr featureComputerGUIInfo,
    RegionSetInterfacePlugin *plugin, QWidget *parent)
  : RegionSetInterfaceWidget(mainView, featureComputerGUIInfo, plugin, parent),
  m_numBinsSlider(NULL),
    m_startAngleSlider(NULL),
    m_rfccw(NULL)


{
  buildGUI();
}

RegionSetInterfaceWidgetNeighboursPosition::~RegionSetInterfaceWidgetNeighboursPosition()
{
}

void
RegionSetInterfaceWidgetNeighboursPosition::buildGUI()
{
  QLabel *nblabel = new QLabel(tr("Number of bins"), this);
  QLabel *aglabel = new QLabel(tr("Start angle"), this);
  m_numBinsSlider = new SliderSpinBoxWidget(this);
  m_numBinsSlider->setMinimum(1);
  m_numBinsSlider->setMaximum(359);
  m_numBinsSlider->setDecimals(0);
  m_numBinsSlider->setSingleStep(1);
  m_startAngleSlider = new SliderSpinBoxWidget(this);
  m_startAngleSlider->setMinimum(0);
  m_startAngleSlider->setMaximum(359.5);
  m_startAngleSlider->setDecimals(1);
  m_startAngleSlider->setSingleStep(0.5);

  m_rfccw = new RegionFeatureChoiceGravityCenterWidget(m_mainView, m_featureComputerGUIInfo, this);

  QVBoxLayout *layout = new QVBoxLayout;
  layout->addWidget(m_rfccw);
  layout->addWidget(nblabel);
  layout->addWidget(m_numBinsSlider);
  layout->addWidget(aglabel);
  layout->addWidget(m_startAngleSlider);
  layout->addStretch();

  setLayout(layout);
}

void
RegionSetInterfaceWidgetNeighboursPosition::getParameters(girl::shared_ptr<ise::RegionSetInterfaceGravityCenter> &regSetGravityCenter, size_t &numBins, double &angle) const
{
  numBins = static_cast<size_t>(m_numBinsSlider->value());
  angle   = m_startAngleSlider->value();
  regSetGravityCenter = girl::static_pointer_cast<ise::RegionSetInterfaceGravityCenter>(m_rfccw->regionFeatureComputerPtr());
}

void
RegionSetInterfaceWidgetNeighboursPosition::setParameters(const girl::shared_ptr<ise::RegionSetInterfaceGravityCenter> &regSetGravityCenter, const size_t &numBins, const double &angle)
{
  m_numBinsSlider->setValue(static_cast<double>(numBins));
  m_startAngleSlider->setValue(angle);
  m_rfccw->setRegionFeatureComputerPtr(regSetGravityCenter);
}
