/*
  Copyright (C) 2005-2011 Remi Vieux, Boris Mansencal  remi.vieux@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
/*
 * RegionSetInterfaceSettingsRelaxation.cpp
 *
 *  Created on: Dec 3, 2010
 *      Author: vieux
 */

#include "RegionSetInterfaceSettingsRelaxation.hpp"

#include <ise/iseImageFeatureComputerDefault.hpp>
#include <ise/iseRegionSetInterfaceRelaxation.hpp>

#include "RegionSetInterfaceWidgetRelaxation.hpp"

RegionSetInterfaceSettingsRelaxation::RegionSetInterfaceSettingsRelaxation()
: FeatureInterfaceSettings(),
  m_regSetMoment(girl::shared_ptr<ise::RegionSetInterfaceMoments>())
{

}

RegionSetInterfaceSettingsRelaxation::~RegionSetInterfaceSettingsRelaxation()
{
}

void
RegionSetInterfaceSettingsRelaxation::configureWidget(FeatureInterfaceWidget *widget) const
{
  try {
    RegionSetInterfaceWidgetRelaxation *widgetRelaxation =
        static_cast<RegionSetInterfaceWidgetRelaxation *>(widget);
    //TODO:BUG what should be the default parameters here? Because
    // setting the GUI with empty parameters will cause crash
    widgetRelaxation->setParameters(m_regSetMoment);

  } catch (std::bad_cast& bc) {
    std::cerr << bc.what() << std::endl;
    assert(false);
  }
}

void
RegionSetInterfaceSettingsRelaxation::getWidgetConfig(const FeatureInterfaceWidget *widget)
{
  try {
    const RegionSetInterfaceWidgetRelaxation *widgetRelaxation =
        static_cast<const RegionSetInterfaceWidgetRelaxation *>(widget);
    //TODO:BUG what should be the default parameters here? Because
    // setting the GUI with empty parameters will cause crash
    widgetRelaxation->getParameters(m_regSetMoment);

  } catch (std::bad_cast& bc) {
    std::cerr << bc.what() << std::endl;
    assert(false);
  }
}

void
RegionSetInterfaceSettingsRelaxation::configureAlgo(girl::shared_ptr<ise::FeatureComputer> algo) const
{
  assert(algo);
  girl::shared_ptr<ise::RegionSetInterfaceRelaxation> regSetRelaxation =
      girl::static_pointer_cast<ise::RegionSetInterfaceRelaxation>(algo);
  assert(regSetRelaxation);

  // Have to create the object to check if it is in the region set. Create it with no cache
  // so that the object is "simple" to create
  assert(m_regSetMoment);
  regSetRelaxation->setParameters(m_regSetMoment);
}


girl::shared_ptr<FeatureInterfaceSettings>
RegionSetInterfaceSettingsRelaxation::clone() const
{
  girl::shared_ptr<RegionSetInterfaceSettingsRelaxation> p(new RegionSetInterfaceSettingsRelaxation());
  *p = *this;
  return p;
}

bool
RegionSetInterfaceSettingsRelaxation::isValid() const
{
  return (m_regSetMoment != NULL);
}
