/*
  Copyright (C) 2005-2011 Remi Vieux, Boris Mansencal  remi.vieux@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#include "RegionSetInterfaceSettingsVariance.hpp"

#include <ise/iseImageFeatureComputerDefault.hpp>
#include <ise/iseRegionSetInterfaceVariance.hpp>

#include "RegionSetInterfaceWidgetVariance.hpp"

RegionSetInterfaceSettingsVariance::RegionSetInterfaceSettingsVariance()
: FeatureInterfaceSettings(),
  m_regSetMoment(girl::shared_ptr<ise::RegionSetInterfaceMoments>())
{

}

RegionSetInterfaceSettingsVariance::~RegionSetInterfaceSettingsVariance()
{
}

void
RegionSetInterfaceSettingsVariance::configureWidget(FeatureInterfaceWidget *widget) const
{
  try {
    RegionSetInterfaceWidgetVariance *widgetVariance =
        static_cast<RegionSetInterfaceWidgetVariance *>(widget);
    //TODO:BUG what should be the default parameters here? Because
    // setting the GUI with empty parameters will cause crash
    widgetVariance->setParameters(m_regSetMoment);

  } catch (std::bad_cast& bc) {
    std::cerr << bc.what() << std::endl;
    assert(false);
  }
}

void
RegionSetInterfaceSettingsVariance::getWidgetConfig(const FeatureInterfaceWidget *widget)
{
  try {
    const RegionSetInterfaceWidgetVariance *widgetVariance =
        static_cast<const RegionSetInterfaceWidgetVariance *>(widget);
    //TODO:BUG what should be the default parameters here? Because
    // setting the GUI with empty parameters will cause crash
    widgetVariance->getParameters(m_regSetMoment);

  } catch (std::bad_cast& bc) {
    std::cerr << bc.what() << std::endl;
    assert(false);
  }
}

void
RegionSetInterfaceSettingsVariance::configureAlgo(girl::shared_ptr<ise::FeatureComputer> algo) const
{
  assert(algo);
  girl::shared_ptr<ise::RegionSetInterfaceVariance> regSetVariance =
      girl::static_pointer_cast<ise::RegionSetInterfaceVariance>(algo);
  assert(regSetVariance);

  // Have to create the object to check if it is in the region set. Create it with no cache
  // so that the object is "simple" to create
  assert(m_regSetMoment);
  regSetVariance->setParameters(m_regSetMoment);
}


girl::shared_ptr<FeatureInterfaceSettings>
RegionSetInterfaceSettingsVariance::clone() const
{
  girl::shared_ptr<RegionSetInterfaceSettingsVariance> p(new RegionSetInterfaceSettingsVariance());
  *p = *this;
  return p;
}

bool
RegionSetInterfaceSettingsVariance::isValid() const
{
  return (m_regSetMoment != girl::shared_ptr<ise::RegionSetInterfaceMoments>() );
}
