/*
  Copyright (C) 2005-2013 Remi Vieux, Boris Mansencal remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "RegionSetInterfaceWidgetVariance.hpp"

#include <ise/iseSegmentationContext.hpp>

#include <qiseTools/RegionFeatureChoiceMomentsWidget.hpp>
#include <qiseTools/MainView.hpp>
#include <qiseTools/WeightsSetupWidget.hpp>

#include <QVBoxLayout>

RegionSetInterfaceWidgetVariance::RegionSetInterfaceWidgetVariance(MainView *mainView,
							   FeatureComputerGUIInfoPtr featureComputerGUIInfo,
							   RegionSetInterfacePlugin *plugin, QWidget *parent)
  : RegionSetInterfaceWidget(mainView, featureComputerGUIInfo, plugin, parent),
    m_layout(NULL),
    m_rfccw(NULL),
    m_segContext(mainView->model()->getSegContextPtr())
{
  buildGUI();
}

RegionSetInterfaceWidgetVariance::~RegionSetInterfaceWidgetVariance()
{
}


void
RegionSetInterfaceWidgetVariance::buildGUI()
{
  assert(m_rfccw == NULL);
  m_rfccw = new RegionFeatureChoiceMomentsWidget(m_mainView, m_featureComputerGUIInfo, this);

  m_layout = new QVBoxLayout;
  m_layout->addWidget(m_rfccw);
  m_layout->stretch(0);

  setLayout(m_layout);
}

void
RegionSetInterfaceWidgetVariance::getParameters(girl::shared_ptr<ise::RegionSetInterfaceMoments> &regSetMoment) const
{
  regSetMoment = girl::static_pointer_cast<ise::RegionSetInterfaceMoments>(m_rfccw->regionFeatureComputerPtr());
}

void
RegionSetInterfaceWidgetVariance::setParameters(const girl::shared_ptr<ise::RegionSetInterfaceMoments> &regSetMoment)
{
  m_rfccw->setRegionFeatureComputerPtr(regSetMoment);
}

