/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef ISEREGIONUNARYFUNCTIONCOMPACTNESS_HPP
#define ISEREGIONUNARYFUNCTIONCOMPACTNESS_HPP


#include <ise/iseFactoryTemplate.hxx>
#include <girlTools/girlToolsRegionInfos.hpp>
#include <ise/iseRegionUnaryFunction.hpp>
#include <ise/iseSegmentationContext.hpp>

namespace ise
{

  /**
   * Implementation of a RegionUnaryFunction
   * Returns the tangent mean on the region outer boundary
   */
  class GIRL_API RegionUnaryFunctionCompactness : public RegionUnaryFunction
  {

  public:
    /**
     * Constructor
     * @param tog the toger representation
     */
    RegionUnaryFunctionCompactness(ise::SegmentationContext *segContext)
    : RegionUnaryFunction(segContext)
    {}

    /**
     * @return the value of the tangent mean on the region outer boundary
     */
    inline RegionUnaryFunction::result_type
    operator()(const RegionUnaryFunction::argument_type &regId)
    {
      assert(m_segContext);
      assert(m_segContext->getToger());
      
      girl::toger *tog = m_segContext->getToger();
      double area = girl::tools::computeArea(*tog, regId);
      double perimeter = girl::tools::computePerimeter(*tog, regId);
      double perimeter2 = perimeter * perimeter;
      return perimeter2 / area;
      
    }


  };


  typedef FactoryTemplate<RegionUnaryFunction, RegionUnaryFunctionCompactness> RegionUnaryFunctionCompactnessFactory;

}


#endif /* ISEREGIONUNARYFUNCTIONCOMPACTNESS_HPP_ */
