/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef ISEREGIONUNARYFUNCTIONTANGENTMEAN_HPP_
#define ISEREGIONUNARYFUNCTIONTANGENTMEAN_HPP_


#include <ise/iseFactoryTemplate.hxx>
#include <ise/iseRegionTools.hpp>
#include <ise/iseRegionUnaryFunction.hpp>
#include <ise/iseSegmentationContext.hpp>

namespace ise
{

  /**
   * Implementation of a RegionUnaryFunction
   * Returns the tangent mean on the region outer boundary
   */
  class GIRL_API RegionUnaryFunctionTangentMean : public RegionUnaryFunction
  {

  public:
    /**
     * Constructor
     * @param tog the toger representation
     */
    RegionUnaryFunctionTangentMean(ise::SegmentationContext *segContext)
    : RegionUnaryFunction(segContext)
    {}

    /**
     * @return the value of the tangent mean on the region outer boundary
     */
    inline RegionUnaryFunction::result_type
    operator()(const RegionUnaryFunction::argument_type &regId)
    {
      assert(m_segContext);
      return ise::RegionTools_computeTangentMeanOnOuterBoundary(*m_segContext->getToger(), regId);
    }


  };


  typedef FactoryTemplate<RegionUnaryFunction, RegionUnaryFunctionTangentMean> RegionUnaryFunctionTangentMeanFactory;

}


#endif /* ISEREGIONUNARYFUNCTIONTANGENTMEAN_HPP_ */
