/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux boris.mansencal@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "RegionUnaryPredicateThresholdSettings.hpp"

#include <qiseTools/RegionUnaryPredicateThresholdWidget.hpp>


void 
RegionUnaryPredicateThresholdSettings::configureWidget(RegionUnaryPredicateWidget *widget) const
{
  RegionUnaryPredicateThresholdWidget *w = static_cast<RegionUnaryPredicateThresholdWidget *>(widget);

  if (m_unaryFctPlugin) {
    m_unaryFctPlugin->setSettings(m_unaryFctSettings);
    w->setRegionUnaryFunctionPlugin(m_unaryFctPlugin);
  }
  
  w->setComparisonOperator(m_comp);

  w->setThreshold(m_threshold);
}

void
RegionUnaryPredicateThresholdSettings::getWidgetConfig(const RegionUnaryPredicateWidget *widget)
{
  const RegionUnaryPredicateThresholdWidget *w = static_cast<const RegionUnaryPredicateThresholdWidget *>(widget);
  

  m_unaryFctPlugin = w->getRegionUnaryFunctionPlugin();
  if (m_unaryFctPlugin)
    m_unaryFctSettings = m_unaryFctPlugin->settings();

  m_comp = w->getComparisonOperator();

  m_threshold = w->getThreshold();

}

void
RegionUnaryPredicateThresholdSettings::configureAlgo(girl::shared_ptr<ise::RegionUnaryPredicate> algo) const
{
  girl::shared_ptr<ise::RegionUnaryPredicateThreshold> p =
    girl::static_pointer_cast<ise::RegionUnaryPredicateThreshold>(algo);
    

  girl::shared_ptr<ise::RegionUnaryFunction> unaryFct;
  if (m_unaryFctPlugin) {
    unaryFct = m_unaryFctPlugin->getNewRegionUnaryFunction(m_unaryFctPlugin->getSegContext());
    if (m_unaryFctSettings)
      m_unaryFctSettings->configureAlgo(unaryFct);
  }

  p->setRegionUnaryFunction(unaryFct);
  p->setComparisonOperator(m_comp);
  p->setThreshold(m_threshold);
}

girl::shared_ptr<RegionUnaryPredicateSettings>
RegionUnaryPredicateThresholdSettings::clone() const
{
  girl::shared_ptr<RegionUnaryPredicateThresholdSettings> p(new RegionUnaryPredicateThresholdSettings);
  *p = *this;
  if (m_unaryFctSettings)
    p->m_unaryFctSettings = m_unaryFctSettings->clone();
  return p;
}

bool
RegionUnaryPredicateThresholdSettings::isValid() const
{
  return ((m_unaryFctPlugin != NULL) 
	  && (! m_unaryFctPlugin->hasSettings() || (m_unaryFctSettings != girl::shared_ptr<RegionUnaryFunctionSettings>() && m_unaryFctSettings->isValid())));

}

