/*
  Copyright (C) 2005-2011 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#ifndef QUANTIFICATIONCAGPLUGIN_HPP
#define QUANTIFICATIONCAGPLUGIN_HPP

#include <qiseTools/SegmentationClusteringPlugin.hpp>

#include <ise/iseQuantificationCAG.hpp>
#include <ise/iseArrayHistogram.hpp>

#include <qiseTools/QuantificationCAGWidget.hpp>
#include <qiseTools/QuantificationCAGWidgetSettings.hpp>

#include <QObject>

class QuantificationCAGPlugin : public QObject,
				public SegmentationClusteringPlugin
{

  Q_OBJECT
  Q_INTERFACES(SegmentationClusteringPlugin QIsePlugin)
#if QT_VERSION >= 0x050000
  Q_PLUGIN_METADATA(IID SegmentationClusteringPlugin_iid)
#endif
    
public:
    
  ~QuantificationCAGPlugin() {}
  
  QuantificationCAGPlugin()
    : SegmentationClusteringPlugin(QString::fromUtf8("CAG quantification"),
				   QIseVersion(1, 0), QIseVersion(1, 0))
  {}
  
  virtual girl::shared_ptr<ise::SegmentationClustering> create()
  {
    const int histoOffset = 0;
    ise::Histogram *histo = new ise::ArrayHistogram(histoOffset, histoOffset, histoOffset); //TODO: this histogram is never freed !!!
    const int colorSpace = ISE_RGB_to_K1K2K3;
    girl::shared_ptr<ise::SegmentationClustering> result(new ise::QuantificationCAG(m_segContext,
									       histo, 
									       colorSpace));

    return result;
  }
  
  virtual bool inSubMenu(std::string &subMenu)
  {
    subMenu = "General";
    return true;
  }

  virtual girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface> createSettings()
  {
    return girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface>(new QuantificationCAGWidgetSettings());
  }

protected:

  virtual SegmentationClusteringWidgetInterface* createWidget(MainView *mainView, QWidget *parent = NULL)
  {
    return new QuantificationCAGWidget(mainView, this, parent);
  }

};

#if QT_VERSION < 0x050000
Q_EXPORT_PLUGIN2(QIseQuantificationCAGPlugin, QuantificationCAGPlugin)
#endif

#endif /* ! QUANTIFICATIONCAGPLUGIN_HPP */
