/*
  Copyright (C) 2005-2011 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#ifndef QUANTIFICATIONNBCOLORSPLUGIN_HPP
#define QUANTIFICATIONNBCOLORSPLUGIN_HPP

#include <qiseTools/SegmentationClusteringPlugin.hpp>

#include <ise/iseQuantificationNbColors.hpp>

#include <qiseTools/QuantificationNbColorsWidget.hpp>
#include <qiseTools/QuantificationNbColorsWidgetSettings.hpp>

#include <QObject>

class QuantificationNbColorsPlugin : public QObject,
				     public SegmentationClusteringPlugin
{

  Q_OBJECT
  Q_INTERFACES(SegmentationClusteringPlugin QIsePlugin)
#if QT_VERSION >= 0x050000
  Q_PLUGIN_METADATA(IID SegmentationClusteringPlugin_iid)
#endif
    
public:
    
  ~QuantificationNbColorsPlugin() {}
  
  QuantificationNbColorsPlugin()
    : SegmentationClusteringPlugin(QString::fromUtf8("Nb colors quantification"),
				   QIseVersion(1, 0), QIseVersion(1, 0))
  {}
  
  virtual girl::shared_ptr<ise::SegmentationClustering> create()
  {
    const int colorSpace = ISE_RGB_to_K1K2K3;
    const int nbColors = 4;
    girl::shared_ptr<ise::SegmentationClustering> result(new ise::QuantificationNbColors(m_segContext, colorSpace, nbColors));

    return result;
  }
  
  virtual bool inSubMenu(std::string &subMenu)
  {
    subMenu = "General";
    return true;
  }

  virtual girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface> createSettings()
  {
    return girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface>(new QuantificationNbColorsWidgetSettings());
  }

protected:

  virtual SegmentationClusteringWidgetInterface* createWidget(MainView *mainView, QWidget *parent = NULL)
  {
    return new QuantificationNbColorsWidget(mainView, this, parent);
  }

};

#if QT_VERSION < 0x050000
Q_EXPORT_PLUGIN2(QIseQuantificationNbColorsPlugin, QuantificationNbColorsPlugin)
#endif

#endif /* ! QUANTIFICATIONNBCOLORSPLUGIN_HPP */
