/*
  Copyright (C) 2005-2011 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#ifndef QUANTIFICATIONTHRESHOLDSPLUGIN_HPP
#define QUANTIFICATIONTHRESHOLDSPLUGIN_HPP

#include <qiseTools/SegmentationClusteringPlugin.hpp>

#include <ise/iseQuantificationThresholds.hpp>

#include <qiseTools/QIsePluginDependance.hpp>
#include <qiseTools/QuantificationThresholdsWidget.hpp>
#include <qiseTools/QuantificationThresholdsWidgetSettings.hpp>

#include <qiseTools/MainView.hpp>

#include <QObject>

class QuantificationThresholdsPlugin : public QObject,
				       public SegmentationClusteringPlugin
{

  Q_OBJECT
  Q_INTERFACES(SegmentationClusteringPlugin QIsePlugin)
#if QT_VERSION >= 0x050000
  Q_PLUGIN_METADATA(IID SegmentationClusteringPlugin_iid)
#endif
    
public:
    
  ~QuantificationThresholdsPlugin() {}
  
  QuantificationThresholdsPlugin()
    : SegmentationClusteringPlugin(QString::fromUtf8("Thresholds quantification"),
				   QIseVersion(1, 0), QIseVersion(1, 0))
  {/*addDependance(QIsePluginDependance(tr("MergeDriver/Thresholds merge driver"),
     QIseVersion(1, 0)));*/}
  
  virtual girl::shared_ptr<ise::SegmentationClustering> create()
  {
    girl::shared_ptr<ise::SegmentationClustering> result(new ise::QuantificationThresholds(m_segContext));

    return result;
  }
  
  virtual girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface> createSettings()
  {
    return girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface>(new QuantificationThresholdsWidgetSettings());
  }

  bool inSubMenu(std::string &subMenu) {
    subMenu = "General";
    subMenu.append(1, SegmentationClusteringPlugin::SUBMENU_SEPARATOR);
    subMenu += "Thresholds";
    return true;
  }

protected:

  virtual SegmentationClusteringWidgetInterface* createWidget(MainView *mainView, QWidget *parent = NULL)
  {
    if (! mainView->model()->getQImage().isNull()) {
      QuantificationThresholdsWidget *w = new QuantificationThresholdsWidget(mainView, this, parent);
      w->activate();
      return w;
    }
    return NULL;
  }



};

#if QT_VERSION < 0x050000
Q_EXPORT_PLUGIN2(QIseQuantificationThresholdsPlugin, QuantificationThresholdsPlugin)
#endif

#endif /* ! QUANTIFICATIONTHRESHOLDSPLUGIN_HPP */
