/*
  Copyright (C) 2005-2011 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#ifndef QUANTIFICATIONUNIFORMPLUGIN_HPP
#define QUANTIFICATIONUNIFORMPLUGIN_HPP

#include <qiseTools/SegmentationClusteringPlugin.hpp>

#include <ise/iseQuantificationUniform.hpp>

#include <qiseTools/QuantificationUniformWidget.hpp>
#include <qiseTools/QuantificationUniformWidgetSettings.hpp>

#include <QObject>

class QuantificationUniformPlugin : public QObject,
				    public SegmentationClusteringPlugin
{

  Q_OBJECT
  Q_INTERFACES(SegmentationClusteringPlugin QIsePlugin)
#if QT_VERSION >= 0x050000
  Q_PLUGIN_METADATA(IID SegmentationClusteringPlugin_iid)
#endif
    
public:
    
  ~QuantificationUniformPlugin() {}
  
  QuantificationUniformPlugin()
    : SegmentationClusteringPlugin(QString::fromUtf8("Uniform quantification"),
				   QIseVersion(1, 0), QIseVersion(1, 0))
  {}
  
  virtual girl::shared_ptr<ise::SegmentationClustering> create()
  {
    const int colorSpace = ISE_RGB_to_K1K2K3;
    const unsigned char offset = 0;
    girl::shared_ptr<ise::SegmentationClustering> result(new ise::QuantificationUniform(m_segContext, colorSpace, offset, offset, offset));


    return result;
  }
  
  virtual bool inSubMenu(std::string &subMenu)
  {
    subMenu = "General";
    return true;
  }

  virtual girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface> createSettings()
  {
    return girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface>(new QuantificationUniformWidgetSettings());
  }

protected:

  virtual SegmentationClusteringWidgetInterface* createWidget(MainView *mainView, QWidget *parent = NULL)
  {
    return new QuantificationUniformWidget(mainView, this, parent);
  }

};

#if QT_VERSION < 0x050000
Q_EXPORT_PLUGIN2(QIseQuantificationUniformPlugin, QuantificationUniformPlugin)
#endif

#endif /* ! QUANTIFICATIONUNIFORMPLUGIN_HPP */
