/*
  Copyright (C) 2005-2011 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#ifndef SEGMENTATIONCLUSTERINGMARKOVWIDGETSETTINGS_HPP
#define SEGMENTATIONCLUSTERINGMARKOVWIDGETSETTINGS_HPP

#include <qiseTools/SegmentationClusteringWidgetSettingsInterface.hpp>
#include "SegmentationClusteringBlockWidget.hpp"
#include "SegmentationClusteringBlockSettings.hpp"
#include "iseSegmentationClusteringBlock.hpp"

class SegmentationClusteringBlockSettings : public SegmentationClusteringWidgetSettingsInterface {
  
public:
  
  SegmentationClusteringBlockSettings() 
    : SegmentationClusteringWidgetSettingsInterface()
    , m_areaThreshold(ise::SegmentationClusteringBlock::DEFAULT_AREA_THRESHOLD)
    , m_axisRatioThreshold(ise::SegmentationClusteringBlock::DEFAULT_RATIO_THRESHOLD)
    , m_compacityThreshold(ise::SegmentationClusteringBlock::DEFAULT_COMPACITY_THRESHOLD)
  {
  }

  virtual void configureWidget(SegmentationClusteringWidgetInterface *widget) const
  {
    static_cast<SegmentationClusteringBlockWidget*>(widget)->setParameters(m_areaThreshold, m_axisRatioThreshold, m_compacityThreshold);
  }
  
  virtual void getWidgetConfig(const SegmentationClusteringWidgetInterface *widget)
  {
    static_cast<const SegmentationClusteringBlockWidget*>(widget)->getParameters(m_areaThreshold, m_axisRatioThreshold, m_compacityThreshold);
  }

  virtual void configureAlgo(girl::shared_ptr<ise::SegmentationClustering> algo) const
  {
    girl::shared_ptr<ise::SegmentationClusteringBlock> algorithm = girl::static_pointer_cast<ise::SegmentationClusteringBlock>(algo);
    algorithm->setParameters(m_areaThreshold, m_axisRatioThreshold, m_compacityThreshold);
  }

  virtual girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface> clone() const
  {
    girl::shared_ptr<SegmentationClusteringBlockSettings> p(new SegmentationClusteringBlockSettings);

    *p = *this;

    return p;
  }

  virtual bool isValid() const
  {
    return true; //B:???
  }

  
private:
  
  size_t m_areaThreshold;
  double m_axisRatioThreshold;
  double m_compacityThreshold;
};

#endif /* ! SEGMENTATIONCLUSTERINGMARKOVWIDGETSETTINGS_HPP */
