/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#include "SegmentationClusteringBlockWidget.hpp"

#include <limits>
#include <QBoxLayout>
#include <QDoubleSpinBox>
#include <QLabel>
#include <QSpinBox>


#include <qiseTools/RegionFeatureClustererWidget.hpp>

SegmentationClusteringBlockWidget::SegmentationClusteringBlockWidget(MainView *mainView,
				    SegmentationClusteringPlugin *segmentationClustering,
				    QWidget *parent)
  : SegmentationClusteringWidgetInterface(mainView, segmentationClustering, parent)
  , m_areaThresholdSB(NULL)
  , m_axisRatioThresholdSB(NULL)
  , m_compacityThresholdSB(NULL)
{
  buildGUI();
}
  
void
SegmentationClusteringBlockWidget::buildGUI()
{
  QLabel *areaThresholdLabel = new QLabel(tr("Area threshold: "), this);
  areaThresholdLabel->setToolTip(tr("Regions which are is strictly inferiror to this threshold will be merged"));
  m_areaThresholdSB = new QSpinBox(this);
  m_areaThresholdSB->setMinimum(0);
  m_areaThresholdSB->setMaximum(std::numeric_limits<int>::max());

  QLabel *axisRatioThresholdLabel = new QLabel(tr("Axis ratio threshold: "), this);
  axisRatioThresholdLabel->setToolTip(tr("Regions which axis ratio is structly superior to this threshold will be merged"));
  m_axisRatioThresholdSB = new QDoubleSpinBox(this);
  m_axisRatioThresholdSB->setMinimum(0.0);
  m_axisRatioThresholdSB->setMaximum(std::numeric_limits<double>::max());

  QLabel *compacityThresholdLabel = new QLabel(tr("Compacity threshold: "), this);
  compacityThresholdLabel->setToolTip(tr("Regions which compacity is structly superior to this threshold will be merged"));
  m_compacityThresholdSB = new QDoubleSpinBox(this);
  m_compacityThresholdSB->setMinimum(0.0);
  m_compacityThresholdSB->setMaximum(std::numeric_limits<double>::max());

  QGridLayout *gLayout = new QGridLayout;
  gLayout->addWidget(areaThresholdLabel, 0, 0);
  gLayout->addWidget(m_areaThresholdSB, 0, 1);
  gLayout->addWidget(axisRatioThresholdLabel, 1, 0);
  gLayout->addWidget(m_axisRatioThresholdSB, 1, 1);
  gLayout->addWidget(compacityThresholdLabel, 2, 0);
  gLayout->addWidget(m_compacityThresholdSB, 2, 1);

  RegionFeatureClustererWidget *REMOVE_ME = new RegionFeatureClustererWidget(m_mainView, this);
  gLayout->addWidget(REMOVE_ME, 3, 0);

  setLayout(gLayout);
}

void
SegmentationClusteringBlockWidget::setParameters(size_t areaThreshold, double axisRatioThreshold, double compacityThreshold)
{
  assert(m_areaThresholdSB);
  m_areaThresholdSB->setValue(areaThreshold);
  m_axisRatioThresholdSB->setValue(axisRatioThreshold);
  m_compacityThresholdSB->setValue(compacityThreshold);
}

void
SegmentationClusteringBlockWidget::getParameters(size_t &areaThreshold, double &axisRatioThreshold, double &compacityThreshold) const
{
  assert(m_areaThresholdSB);
  areaThreshold = m_areaThresholdSB->value();
  axisRatioThreshold = m_axisRatioThresholdSB->value();
  compacityThreshold = m_compacityThresholdSB->value();
}
