/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef LINEDETECTIONMASK_HPP
#define LINEDETECTIONMASK_HPP

#include <girl/girlCommon.hpp>
#include <cassert>

const int mask[4][3][3] = {

  { {-1, -1, -1}, {2, 2, 2}, {-1, -1, -1} } ,//M0
  { {-1, -1, 2}, {-1, 2, -1}, {2, -1, -1} } ,//M45
  { {-1, 2, -1}, {-1, 2, -1}, {-1, 2, -1} } ,//M90
  { {2, -1, -1}, {-1, 2, -1}, {-1, -1, 2} } //M135
};

namespace ise {
 
  class GIRL_API LineDetectionMask {

  public:

    typedef enum maskOrientation { M0, M45, M90, M135 } MaskOrientation;

    static const int SIZE_MASK = 3;

  public:
    
    explicit LineDetectionMask(MaskOrientation m)
      :	m_maskOrientation(m)
    {
    }

    int get(int x, int y) const { 
      assert(x < SIZE_MASK && y < SIZE_MASK);
      return mask[m_maskOrientation][x][y]; 
    }

    std::vector<std::string> data() {
      std::vector<std::string> tmp;
      tmp.push_back("0°");
      tmp.push_back("45°");
      tmp.push_back("90°");
      tmp.push_back("135°");
      return tmp;
    }

  private:

    MaskOrientation m_maskOrientation;

  };

}

#endif /* ! LINEDETECTIONMASK_HPP */
