/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "iseRegionSetInterfaceDensityFeature.hpp"

#include <opencv2/core/core.hpp>

#include <ise/iseSegmentationContext.hpp>
#include <ise/iseConversion.hpp>
#include <ise/iseImageFeatureColorSpace.hpp>
namespace ise
{
  RegionSetInterfaceDensityFeature::RegionSetInterfaceDensityFeature(ise::SegmentationContext* segContext,
      ise::RegionSetInterface::CacheType cache):
      RegionSetInterface(segContext),
      m_maskOrientation(ise::LineDetectionMask::M0),
      m_area(0), m_val(0)
  {
    setCacheState(cache);

    ise::ImageFeatureColorSpace computer(segContext);
    computer.setParameters(ISE_RGB_to_YUV);

    computer.compute();

    m_image = computer.getImageFeature();
  }

  RegionSetInterfaceDensityFeature::~RegionSetInterfaceDensityFeature()
  {
  }

  void
  RegionSetInterfaceDensityFeature::setParameters(const ise::LineDetectionMask::MaskOrientation &orientation)
  {
    m_maskOrientation = orientation;
  }
  
  void
  RegionSetInterfaceDensityFeature::getParameters(ise::LineDetectionMask::MaskOrientation &orientation) const 
  {
    orientation = m_maskOrientation;
  }

  void
  RegionSetInterfaceDensityFeature::processPoint(const girl::point& pt)
  {
    ++m_area;
    if (isLinePixel(pt))
      m_val++;
  }

  void
  RegionSetInterfaceDensityFeature::compute(const girl::region& id, ise::RegionFeature& feature)
  {
    assert(m_segContext);
    assert(m_segContext->getToger());

    girl::toger* tog = m_segContext->getToger();
    assert(tog);
    m_area = 0;
    m_val = 0;
 

    tog->iterateOnRegionDomain(id, *this);

    feature[0] = m_val / static_cast<double>(m_area);
  }

  void
  RegionSetInterfaceDensityFeature::mergeFeatures(RegionFeature& kept, RegionFeature& /*del*/,
      const girl::region& keptReg, const girl::region& /*delReg*/)
  {
    compute(keptReg, kept);
  }

  const std::string
  RegionSetInterfaceDensityFeature::help() const
  {
    return "TODO JULIAN";
  }

  bool 
  RegionSetInterfaceDensityFeature::isLinePixel(const girl::point &pt)
  {
    cv::Mat *mat = m_image->cvMat().get();

    int x = pt.x();
    int y = pt.y();
    int no=0, n=0, ne=0, e=0, o=0, se=0, s=0, so=0;

    int channels = mat->channels();

    ise::LineDetectionMask mask(m_maskOrientation);

    int current = mask.get(1, 1) * mat->at<girl::tools::ImageFeature::type>(y, x * channels);      
    if (x - 1 >= 0){
      if (y-1 >= 0)
	no = mask.get(0, 0) * mat->at<girl::tools::ImageFeature::type>(y-1, (x-1) * channels);

      o = mask.get(0, 1) * mat->at<girl::tools::ImageFeature::type>(y, (x-1) * channels);

      if (y+1 <mat->rows)
	so = mask.get(0, 2) * mat->at<girl::tools::ImageFeature::type>(y+1, (x-1) * channels);
    }

    if( x+1 < mat->cols ){
      if (y-1 >= 0)
	ne = mask.get(2, 0) * mat->at<girl::tools::ImageFeature::type>(y-1, (x+1) * channels);

      e = mask.get(2, 1) * mat->at<girl::tools::ImageFeature::type>(y, (x+1) * channels);

      if (y+1 <mat->rows)
	se =  mask.get(2, 2) * mat->at<girl::tools::ImageFeature::type>(y+1, (x+1) * channels);
    }

    if (y-1 >= 0)
      n = mask.get(1, 0) * mat->at<girl::tools::ImageFeature::type>(y-1, x * channels);
    if (y+1 <mat->rows)
      s =  mask.get(1, 2) * mat->at<girl::tools::ImageFeature::type>(y+1, x * channels);


    return (no + n + e + s + o + se + ne + so + current ) > 128;
  }
}

