/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef ISEREGIONSETINTERFACEDENSITYFEATURE_HPP
#define ISEREGIONSETINTERFACEDENSITYFEATURE_HPP

#include <girl/girlCommon.hpp>

#include <ise/iseRegionSetInterface.hpp>
#include "iseLineDetectionMask.hpp"
#include <girlTools/girlToolsImageFeature.hpp>


namespace ise{
  class GIRL_API RegionSetInterfaceDensityFeature : public RegionSetInterface 
  , public girl::toger::pointIterator
  {
    static const int FEATURE_DIMENSION = 1;
    static const RegionSetInterface::CacheType DEFAULT_CACHE_STRATEGY = RegionSetInterface::LAZY;
  public:

    RegionSetInterfaceDensityFeature(ise::SegmentationContext* segContext, 
        ise::RegionSetInterface::CacheType cacheType = DEFAULT_CACHE_STRATEGY);

    virtual ~RegionSetInterfaceDensityFeature();

    inline const URI
    uri() const
    {
      std::stringstream ss(std::ios::in|std::ios::out);
      ss << __FILE__ << std::endl;
      return ss.str();
    }

    inline size_t
    getFeatureDim() const
    {
      return FEATURE_DIMENSION;
    }

    inline void
    getDimDescription(std::vector<std::string>& dimDesc) const
    {
      dimDesc.clear();
      dimDesc.push_back("DWB");
    }

    void
    processPoint(const girl::point &pt);

    void
    setParameters(const ise::LineDetectionMask::MaskOrientation &orientation);

    void
    getParameters(ise::LineDetectionMask::MaskOrientation &orientation) const;

    void
    getRequiredComputers(FeatureComputer::FeatureComputerListType &) const {}

    const std::string
    help() const;

  protected:

    void compute(const girl::region& id, ise::RegionFeature& feature);

    void mergeFeatures(RegionFeature& kept, RegionFeature& del,
        const girl::region& keptId, const girl::region& delId);

  private:

    bool isLinePixel(const girl::point &pt);

  private:
    ise::LineDetectionMask::MaskOrientation m_maskOrientation;

    int m_area;
    double m_val;
    
    girl::shared_ptr<girl::tools::ImageFeature> m_image;
  };
}

#endif /* !  ISEREGIONSETINTERFACEDENSITYFEATURE_HPP */
