/*
  Segmentation clustering: DensityFeature
  Copyright (C) 2005-2009 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include <ise/iseSegmentationContext.hpp>
#include <girl/girlCommon.hpp>

#include "iseSegmentationClusteringDensityFeature.hpp"

#include <ise/iseSegmentationClustering.hpp>
#include <ise/iseGenericImage.hpp>
#include <opencv2/core/core.hpp>
#include "iseRegionSetInterfaceDensityFeature.hpp"

using namespace ise;

SegmentationClusteringDensityFeature::SegmentationClusteringDensityFeature(SegmentationContext *segContext)
 : SegmentationClustering(segContext)
 , m_type("")
 , m_image()
 , m_regionsDWB()
 , m_threshold(0)
{
  setSegContext(segContext);
}

void
SegmentationClusteringDensityFeature::setSegContext(ise::SegmentationContext *s) 
{
  SegmentationClustering::setSegContext(s);
}

void 
SegmentationClusteringDensityFeature::init(const girl::regionDeque &regions, int /* level */)
{

  assert(m_segContext);
  assert(m_segContext->getImage());

  girl::tools::Image *img = m_segContext->getImage();

  const int width = img->width();
  const int height = img->height();

  m_image = ImageLabelToger(width, height);

  size_t size = regions.size();

  /* compute density */
  ise::RegionSetInterfaceDensityFeature regSet(m_segContext);
  regSet.setParameters(static_cast<ise::LineDetectionMask::maskOrientation>(1));
  m_regionsDWB.clear();

  for (size_t i = 0 ; i < size ; ++i){
    std::vector<RegionFeature_Data_Type> buff(regSet.getFeatureDim(), 0);
    ise::RegionFeature feature(&buff[0]);
    regSet.getRegionFeatures(regions[i], feature);
    m_regionsDWB.insert(std::pair<int,double>(regions[i], feature[0]));
  }

  /* compute image */
  computeImage();

}

void
SegmentationClusteringDensityFeature::computeImage() 
{
  assert(m_segContext);
  assert(m_segContext->getToger());

  girl::toger *tog = m_segContext->getToger();

  std::map<int, double>::iterator it = m_regionsDWB.begin();
  std::map<int, double>::iterator itEnd = m_regionsDWB.end();

  for ( ; it != itEnd ; ++it) {
    double val = (*it).second;
    
    if( m_type == "text"){
      if( val > m_threshold)
	m_image.setRegId(1);
      else
	m_image.setRegId(-1);
    }
    else{
      if( m_type == "non-text"){
	if (val <= m_threshold)
	  m_image.setRegId(1);
	else
	  m_image.setRegId(-1);
      }
    }

    tog->iterateOnRegionDomain((*it).first, m_image);
  }
}

void 
SegmentationClusteringDensityFeature::setParameters(std::string type, double threshold)
{
   m_type = type;
   m_threshold = threshold;
} 

void 
SegmentationClusteringDensityFeature::getParameters(std::string &type, double &threshold) const
{
   type = m_type;
   threshold = m_threshold;
} 

bool
SegmentationClusteringDensityFeature::same(const girl::point &p1, girl::direction dir)
{
  girl::point p2 = p1;
  p2.move(dir);

  int val1 = m_image.regionId(p1);
  int val2 = m_image.regionId(p2);

  return val1 == val2;
}
