/*
  Copyright (C) 2005-2011 Remi Vieux, Jean-Philippe Domenger, Boris Mansencal  boris.mansencal@labri.fr

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include <iostream>
#include <string>
#include <cstdio>
#include <sstream>
#include <algorithm>
#include <iterator>
#include <numeric>

#include "iseSegmentationClusteringDocumentPreseg5.hpp"

#include <girl/girlHashFunc.hpp>
#include <girl/girlUnorderedSet.hpp>
#include <girl/girlToger.hpp>
#include <girlTools/girlToolsRegionInfos.hpp>
#include <girlTools/girlToolsLabelImagePainter.hpp>
#include <girlTools/girlToolsDiskBackup.hpp>
#include <girlTools/girlToolsPainter.hpp>

#include <ise/iseConversion.hpp>
#include <ise/iseHistogram1D.hpp>
#include <ise/iseHistogram1DTools.hpp>
#include <ise/iseMergeAlgorithmDefault.hpp>
#include <ise/iseMergeAlgorithmUntil.hpp>
#include <ise/iseMergeDriverMinimalDistance.hpp>
#include <ise/iseMergeDriverMinimalDistanceWithAllow.hpp>
#include <ise/iseQuantificationThresholderOtsu1T.hpp>
#include <ise/iseQuantificationThresholds.hpp>
#include <ise/iseRegionBinaryFunctionConstantNull.hpp>
#include <ise/iseRegionSetInterfaceGeometric.hpp>
#include <ise/iseSegmentationContext.hpp>
#include <ise/iseSplitAlgorithmDefault.hpp>
#include <ise/iseSplitDriverDefault.hpp>
#include <ise/iseSegmentationClusteringMaximalWhitespaceRectangle.hpp>
#include <ise/iseSegmentationClusteringLocalThresholderSauvola.hpp>

#include <girl/girlHashMap.hpp>

#include <ise/iseTogerUtils.hpp> //mergeSmallRegions

#include <ise/EnclosingConvexHullFactory.hpp>
#include <ise/EnclosingBoxFactory.hpp>
#include <ise/iseSegmentationClusteringEnclosingShape.hpp>
#include <ise/iseBinarizerThresholder.hpp>
#include <ise/iseSegmentationClusteringWord.hpp>
#include <ise/iseRegionSetInterfaceDocumentZoneClassifier.hpp>
#include <ise/iseRegionSetInterfaceBoundingBox.hpp>
#include <ise/iseSegmentationClusteringWhitespaceSeparator.hpp>
#include <ise/iseConnectedComponentTools.hpp>
#include <ise/iseQuantificationThresholderDomenger.hpp>
#include <ise/iseDeleteSelectionAndSplitAlgorithm.hpp>
#include <ise/iseSplitMergeAlgoExtendSelection.hpp>
#include <ise/iseForegroundBackgroundClassifierColorArea.hpp>
#include <ise/iseForegroundBackgroundClassifierHoles.hpp>
#include <ise/iseSegmentationClusteringBitmap.hpp>
#include <ise/iseSparseHistogram1D.hpp>
#include <ise/iseSegmentationClusteringParagraphSplit.hpp>
#include <ise/iseSegmentationClusteringProfile.hpp>
#include <ise/iseImageFeatureThreshold.hpp>
#include <ise/iseRegionSetInterfaceRunLengthHistogram.hpp>
#include <ise/iseRegionSetInterfaceConnectedComponentsHistogram.hpp>
#include <ise/iseRegionSetInterfaceTextBlockClassifier.hpp>
#include <ise/iseSegmentationClusteringLocalThresholds.hpp>

//#include "dscc.hpp"
#include "Preseg5Config.hpp"

#include <opencv2/core/core.hpp>
#include <opencv2/ml/ml.hpp>

static const double SEPARATOR_RATIO_THRESHOLD = 10;


//#define VISU
//#define TIMING

#ifdef VISU
#include <opencv2/highgui/highgui.hpp>
#endif //VISU



/*
#ifndef NDEBUG
static void
saveToger(const girl::toger &tog, const std::string &name) {
    girl::tools::oDiskBackup backup(name);
    assert(tog.write(backup, girl::io::DEFAULT));
}
#endif
*/

#ifdef VISU

static cv::Mat g_dispImage;
static cv::Mat g_orig;
static cv::Mat g_info;

static void
displayVisu(const std::vector<std::string>& msg, int timeout = 0)
{
    if (! msg.empty()) {
        g_info.setTo(0);
        cv::Point pt(10, 0);
        std::vector<std::string>::const_iterator it = msg.begin();
        std::vector<std::string>::const_iterator itEnd = msg.end();
        for (; it != itEnd; ++it) {
            int bl;
            cv::Size s = cv::getTextSize(*it, cv::FONT_HERSHEY_COMPLEX, 3, 8, &bl);
            pt.y += s.height + 5;
            cv::putText(g_info, *it, pt, cv::FONT_HERSHEY_COMPLEX, 3, CV_RGB(255, 0, 0), 8);

        }
    }
    cv::namedWindow("Visu", 0);
    cv::imshow("Visu", g_dispImage);
    cv::waitKey(timeout);
    //  if (!g_vidWriter) {
    //    g_vidWriter = new cv::VideoWriter("test.avi", CV_FOURCC('Q', '4', '2', '0'), 2, g_dispImage.size(), true);
    //    //   bool ok =  g_vidWriter.open("test.avi", CV_FOURCC('Q', '4', '2', '0'), 2, g_dispImage.size(), true);
    //    assert(g_vidWriter);
    //  }
    //  if (!g_vidWriter->isOpened()) {
    //     std::cerr << "Error with video writer... Not saving video" << std::endl;
    //     char buf[1024];
    //     sprintf(buf, "%s%s%05d%s", g_imageName.c_str(), "img", (int)cpt++, ".ppm");
    //     std::stringstream ss(std::ios::in|std::ios::out);
    //     //    ss << "img" << cpt++ << ".ppm";
    //   //  cv::imwrite(buf, g_dispImage);
    //  } else {
    //    *g_vidWriter << g_dispImage;
    //  }
}



int g_overlap = 50;
int g_dist = std::numeric_limits<int>::max();

void
doPartition(void *data)
{
    std::vector<girl::tools::AABBox> *boxes = static_cast<std::vector<girl::tools::AABBox>*>(data);
    std::vector<int> labels;
    ise::SameLineBoxes same(g_overlap/100.0f, g_dist);
    int nblines = cv::partition(*boxes, labels, same);
    std::cout << "NB LINES= " << nblines << std::endl;
    std::vector<cv::Scalar> colors;
    for (int i = 0; i < nblines; ++i) {
        cv::Scalar col = CV_RGB(rand()%255, rand()%255, rand()%255);
        colors.push_back(col);
    }
    cv::Mat m = cv::Mat::zeros(g_orig.size(), CV_8UC3);
    std::vector<girl::tools::AABBox>::const_iterator it = boxes->begin();
    for (int i = 0; it != boxes->end(); ++it, ++i) {
        cv::rectangle(m, cv::Point(it->xMin(), it->yMin()), cv::Point(it->xMax(), it->yMax()), colors[labels[i]], CV_FILLED);
    }
    cv::imshow("Partition", m);
    cv::waitKey(0);
}

void
onChange1(int val, void* data)
{
    g_overlap = val;
    doPartition(data);
}

void
onChange2(int val, void *data)
{
    g_dist = val;
    doPartition(data);
}




static void
writeHisto(cv::Mat &mat, const std::vector<ise::Histogram1D> &histos, const std::vector<std::string> &titles)
{
    assert(histos.size() == titles.size());
    const int titleSize = 100;
    int subMatHeight = (mat.rows - titleSize) / histos.size();

    for (size_t i = 0; i < histos.size(); ++i) {
        cv::Mat subMat = g_dispImage(cv::Rect(g_orig.cols, titleSize + i*subMatHeight, g_orig.cols, subMatHeight));
        int shift = subMat.cols/2;
        cv::putText(subMat, titles[i], cv::Point(shift, 25), cv::FONT_HERSHEY_COMPLEX, 1, CV_RGB(255, 255, 255));
        const ise::Histogram1D &histo = histos[i];
        double mean = histo.getMean();
        double median = histo.getMedian();
        double max = histo.getKeyOfMaxFrequency();
        char buf[1024];
        sprintf(buf, "%s%f", "Mean=", mean);
        cv::putText(subMat, buf, cv::Point(shift,50), cv::FONT_HERSHEY_COMPLEX, 1, CV_RGB(255, 0, 0));
        sprintf(buf, "%s%f", "Median=", median);
        cv::putText(subMat, buf, cv::Point(shift,75), cv::FONT_HERSHEY_COMPLEX, 1, CV_RGB(0, 255, 0));
        sprintf(buf, "%s%f", "Max=", max);
        cv::Scalar colorMax = CV_RGB(0, 255, 255);
        cv::putText(subMat, buf, cv::Point(shift,100), cv::FONT_HERSHEY_COMPLEX, 1, colorMax);
        //    assert(histo.getMax() < subMat.cols);
        int w = std::min(1.0, subMat.cols/histo.getMax());
        double maxFreq = histo.getMaxFrequency();
        double factor = subMat.rows/maxFreq;
        for (size_t i = 0; i < histo.size(); ++i) {
            //cv::line(subMat, cv::Point(i, subMat.rows), cv::Point(i, subMat.rows - histo[i]*factor), CV_RGB(255, 255, 255));
            cv::rectangle(subMat, cv::Rect(i*w, subMat.rows - histo[i]*factor, w, histo[i]*factor), CV_RGB(255, 255, 255), CV_FILLED);
        }

        //int meanIndex = histo.keyToIndex(mean)*w;
        //int medianIndex = histo.keyToIndex(median)*w;
        //int maxIndex = histo.keyToIndex(max)*w;
        // cv::line(subMat, cv::Point(meanIndex, 0), cv::Point(meanIndex, subMat.rows), CV_RGB(255, 0, 0), 4);
        // cv::line(subMat, cv::Point(medianIndex, 0), cv::Point(medianIndex, subMat.rows), CV_RGB(0, 255, 0), 4);
        // cv::line(subMat, cv::Point(maxIndex, 0), cv::Point(maxIndex, subMat.rows), colorMax, 4);
    }
}


#endif
























class Splitter : public girl::toger::sameRegion {

public:
    explicit Splitter(const cv::Mat &mat) : m_mat(mat) {
        assert(m_mat.type() == CV_8U);
    }
    virtual ~Splitter() {}

    bool same(const girl::point &p, girl::direction dir)
    {
        girl::point p2 = p;
        p2.move(dir);
        return m_mat.at<unsigned char>(p.y(), p.x()) == m_mat.at<unsigned char>(p2.y(), p2.x());
    }

protected:

    const cv::Mat &m_mat;
};


/**
  * Pixels in the same regions are specified by their rowRange and colRange
  */
class RoiSplitter : public girl::toger::sameRegion
{
public:

    RoiSplitter(const cv::Range &rowRange, const cv::Range &colRange)
        :m_rowRange(rowRange), m_colRange(colRange)
    {
    }

    bool
    same(const girl::point &p, girl::direction dir)
    {
        girl::point p2 = p;
        p2.move(dir);
        int pX = isInColRange(p);
        int pY = isInRowRange(p);
        int p2X = isInColRange(p2);
        int p2Y = isInRowRange(p2);
        return (pX == p2X && pY == p2Y);
    }

    int
    isInRowRange(const girl::point &p) const
    {
        if (p.y() < m_rowRange.start)
            return -1;
        else if (p.y() >= m_rowRange.end)
            return 1;
        else
            return 0;
        //  if (p.y() >= m_rowRange.start && p.y() < m_rowRange.end)
        //     return 0;
        //   else
        //     return 1;
    }

    int
    isInColRange(const girl::point &p) const
    {
        if (p.x() < m_colRange.start)
            return -1;
        else if (p.x() >= m_colRange.end)
            return 1;
        else
            return 0;
        // if (p.x() >= m_colRange.start && p.x() < m_colRange.end)
        //   return 0;
        // else
        //   return 1;
    }

private:
    const cv::Range &m_rowRange;
    const cv::Range &m_colRange;
};


class ConnectedComponentCounter : public girl::toger::pointIterator
{

public:
    ConnectedComponentCounter(const cv::Mat &mat,
                              const girl::toger &tog,
                              girl::regionDeque &fgRegs,
                              girl::regionDeque &bgRegs,
                              girl::unorderedSet<size_t> &ccLabels):
        m_mat(mat),
        m_toger(tog),
        m_fgRegs(fgRegs),
        m_bgRegs(bgRegs),
        m_ccLabels(ccLabels)
    {
        assert(mat.type() == CV_32S);
        assert(mat.rows == tog.height() && mat.cols == tog.width());
    }
    ~ConnectedComponentCounter() {}

    void
    processPoint(const girl::point &pt)
    {
        size_t val = static_cast<size_t>(m_mat.at<int>(pt.y(), pt.x()));
        if (val != 0) {
            if (m_ccLabels.find(val - 1) == m_ccLabels.end()) {
                m_ccLabels.insert(val - 1);
                girl::region reg = m_toger.findRegion(pt);
                m_fgRegs.push_back(reg);
            }
        }
    }

private:
    const cv::Mat &m_mat;
    const girl::toger &m_toger;
    girl::regionDeque &m_fgRegs;
    girl::regionDeque &m_bgRegs;
    girl::unorderedSet<size_t> &m_ccLabels;

};


class CCCounterPreseg5 : public girl::toger::pointIterator
{

public:
    explicit CCCounterPreseg5(const std::vector<cv::Mat> &mats) :
        m_mats(mats),
        m_regs(mats.size())
    {
        for (std::vector<cv::Mat>::const_iterator it = m_mats.begin();
             it != m_mats.end(); ++it)
            assert(it->type() == CV_32S);
    }
    virtual
    ~CCCounterPreseg5(){}

    void
    processPoint(const girl::point &p)
    {
        std::vector<cv::Mat>::const_iterator it = m_mats.begin();
        const std::vector<cv::Mat>::const_iterator itEnd = m_mats.end();
        std::vector<girl::unorderedSet<int> >::iterator rIt = m_regs.begin();
        for (; it != itEnd; ++it, ++rIt) {
            int val = it->at<int>(p.y(), p.x());
            if (val)
                rIt->insert(val);
        }
    }

    int
    numCC(size_t i) const
    {
        assert(i < m_regs.size());
        return m_regs[i].size();
    }

    const girl::unorderedSet<int>&
    getFoundRegs(size_t i) const
    {
        assert(i < m_regs.size());
        return m_regs[i];
    }

    void
    restart()
    {
        std::vector<girl::unorderedSet<int> >::iterator rIt = m_regs.begin();
        const  std::vector<girl::unorderedSet<int> >::iterator rItEnd = m_regs.end();
        for (; rIt != rItEnd; ++rIt) {
            rIt->clear();
        }
    }

private:
    const std::vector<cv::Mat> &m_mats;
    std::vector<girl::unorderedSet<int> > m_regs;


};


class MustProcessBlock : public girl::toger::pointIterator
{
public:
    MustProcessBlock(const cv::Mat &binaryMask,
                     const cv::Mat &imageMask) :
        m_binaryMask(binaryMask),
        m_imageMask(imageMask),
        m_imageFound(false),
        m_ccFound(false)
    {
        assert(m_binaryMask.type() == CV_8U);
        assert(m_imageMask.type() == CV_8U);
    }

    void
    processPoint(const girl::point &p) {
        if (!m_imageFound) {
            m_imageFound = (m_imageMask.at<unsigned char>(p.y(), p.x()) != 0);
        }
        if (!m_ccFound) {
            m_ccFound = (m_binaryMask.at<unsigned char>(p.y(), p.x()) != 0);
        }
    }

    bool
    containsImagePixels() const
    {
        return m_imageFound;
    }

    bool
    containsTextPixels() const
    {
        return m_ccFound;
    }
private:
    const cv::Mat m_binaryMask;
    const cv::Mat m_imageMask;
    bool m_imageFound;
    bool m_ccFound;
};



const int TEXT_REGION_INDEX = 100;
const int IMAGE_REGION_INDEX = 50;
const int SEPARATOR_REGION_INDEX = 150;

static std::string g_imageName;

namespace ise
{

SegmentationClusteringDocumentPreseg5::SegmentationClusteringDocumentPreseg5(SegmentationContext *segContext) :
    SegmentationClustering(segContext),
    m_binarizationToger(),
    m_binarizationMask(),
    m_originalBinarization(),
    m_binarizationFgRegions(),
    m_ccLabels(),
    m_spacingsHoriz(),
    m_spacingsVerti(),
    m_ccWidths(),
    m_ccHeights(),
    m_enclosingShapeToger(),
    m_separatorsMask(),
    m_imageRegions(),
    m_imageMask(),
    // m_wordToger(),
    m_wordMask(),
    m_wordCCLabels(),
    m_wordSpacingsHoriz(),
    m_wordSpacingsVerti(),
    m_wordCcWidths(),
    m_wordCcHeights(),
    m_wordCcAreas(),
    m_wordBoxes(),
    m_toger(),
    m_segContext2(),
    m_image()
{
    setSegContext(segContext);
}

SegmentationClusteringDocumentPreseg5::~SegmentationClusteringDocumentPreseg5()
{
}



const int BINARY_IMAGE = 0;
const int GRAYLEVEL_IMAGE = 1;
const int COLOR_IMAGE = 2;

int
SegmentationClusteringDocumentPreseg5::detectImageType() const
{
    const girl::tools::Image *img = m_segContext->getImage();
    Histogram1D rHisto(*img, ISE_RGB_to_RGB, 0);
    Histogram1D gHisto(*img, ISE_RGB_to_RGB, 1);
    Histogram1D bHisto(*img, ISE_RGB_to_RGB, 2);

    for (size_t i = 0; i < rHisto.size(); ++i) {
        if (!(rHisto[i] == gHisto[i] && gHisto[i] == bHisto[i])) {
            std::cout << " ******* COLOR IMAGE ********** " << std::endl;
            return COLOR_IMAGE;
        }
    }
    if (rHisto[0] + rHisto[rHisto.size() - 1] == img->width()*img->height()) {
        std::cout << " ******* BINARY IMAGE ********** " << std::endl;
        return BINARY_IMAGE;
    }

    std::cout << " ******* GRAY LEVEL IMAGE ********** " << std::endl;
    return GRAYLEVEL_IMAGE;
}


void
SegmentationClusteringDocumentPreseg5::setParameters(const std::string &imageName)
{
    g_imageName = imageName;

}

void
SegmentationClusteringDocumentPreseg5::getParameters()
{
}

void
SegmentationClusteringDocumentPreseg5::init(const girl::regionDeque &regs, int)
{
#ifdef TIMING
    Timer total("Total time Preseg5= ");
#endif



    m_segContext2 = *m_segContext;
    m_toger = *m_segContext->getToger();
    m_segContext2.setImageToger(m_segContext->getImage(), &m_toger);

#ifdef VISU
    // VISU
    {
        size_t width = m_toger.width();
        size_t height = m_toger.height();
        g_dispImage = cv::Mat::zeros(height, 2*width, CV_8UC3);
        cv::Rect tl(0, 0, width, height);
        cv::Rect tr(width, 0,  width, height);
        g_orig = g_dispImage(tl);
        g_info = g_dispImage(tr);
    }
#endif

    for (girl::regionDeque::const_iterator it = regs.begin();
         it != regs.end(); ++it) {
#ifdef TIMING
        Timer t("Preprocessing took");
#endif
        preprocessing(*it);
    }





    //  //computeConnectedComponentsStatistics(m_binarizationMask, m_ccLabels, m_spacingsHoriz, m_spacingsVerti, m_ccWidths, m_ccHeights);
    //#ifdef TIMING
    //  {
    //    Timer t("Computing CC stats took");
    //#endif
    //    computeStatsConnectedComponents();
    //#ifdef TIMING
    //  }
    //#endif
    //#ifdef TIMING
    //  {
    //    Timer t("Computing words took");
    //#endif
    //    computeWord();
    //#ifdef TIMING
    //  }
    //#endif
    //  //computeWordToger2();
    //#ifdef TIMING
    //  {
    //    Timer t("Computing Word stats took");
    //#endif
    //    computeStatsWords();
    //#ifdef TIMING
    //  }
    //#endif

    //computeConnectedComponentsStatistics(m_wordMask, m_wordCCLabels, m_wordSpacingsHoriz, m_wordSpacingsVerti, m_wordCcWidths, m_wordCcHeights);
    // start recursive splitting
    girl::regionDeque regsToProcess;
    m_segContext2.getToger()->allRegions(regsToProcess);

    std::vector<cv::Mat> theMats;
    theMats.push_back(m_ccLabels);
    theMats.push_back(m_wordCCLabels);
    CCCounterPreseg5 counter(theMats);
    while(!regsToProcess.empty()) {
        girl::region reg = regsToProcess.front();

        // count the CC and words in the current block
        m_toger.iterateOnRegionDomain(reg, counter);
        const girl::unorderedSet<int> &includedCC = counter.getFoundRegs(0);
        const girl::unorderedSet<int> &includedWords = counter.getFoundRegs(1);
#ifdef VISU
        //    saveToger(m_toger, "togerpreseg4.seg");
#endif
        // assert(includedWords.size() <= includedCC.size());
        // launch processing if block does contain something
        bool geometryOk = true;// blockGeometryOk(reg);
        if (geometryOk) {
            if (includedCC.size() > 0) {
                processBlock(reg, regsToProcess, includedCC, includedWords);
            }
        } else {
            //!geometryOk
            // erase the connected components in the block before discarding the region
            // std::cerr << "Region " << reg << " discarded by geometry" << std::endl;
            // ise::girl::tools::Painter<unsigned char> p(m_binarizationMask, CV_RGB(0, 0, 0));
            // for (girl::unorderedSet<int>::const_iterator it = includedCC.begin();
            // 	   it != includedCC.end(); ++it) {
            // 	m_binarizationToger.iterateOnRegionDomain(*it, p);
            // }
        }
        //    if (mustProcessBlock(reg, m_imageMask, m_binarizationMask)) {
        //     // std::cout << " is beeing processed!! "  << std::endl;
        //      processBlock(reg, regsToProcess);
        //    }
        counter.restart();
        //std::cout << std::endl;
        regsToProcess.pop_front();
    }

#ifdef VISU
    {
        std::vector<std::string> msg;
        msg.push_back("End");
        displayVisu(msg);

    }
#endif

    m_image = cv::Mat::zeros(m_segContext->getImage()->height(), m_segContext->getImage()->width(), CV_8U);


    // paint final mask image for split
    for (girl::regionDeque::const_iterator it = m_imageRegions.begin();
         it != m_imageRegions.end(); ++it) {
        girl::tools::Painter<unsigned char> p(m_image, CV_RGB(IMAGE_REGION_INDEX, IMAGE_REGION_INDEX, IMAGE_REGION_INDEX));
        m_enclosingShapeToger.iterateOnRegionDomain(*it, p);
    }

    // TODO: paint separators?
    m_image += m_separatorsMask;

    regsToProcess.clear();
    m_toger.allRegions(regsToProcess);
    int color = 1;
    for (girl::regionDeque::const_iterator it = regsToProcess.begin();
         it != regsToProcess.end(); ++it) {
        if (mustProcessBlock(*it, m_imageMask, m_binarizationMask)) {
            if (color == SEPARATOR_REGION_INDEX || color == IMAGE_REGION_INDEX)
                color++;
            if (color > 255)
                color = 1;
            girl::tools::Painter<unsigned char> p(m_image, CV_RGB(color, color, color));
            m_toger.iterateOnRegionDomain(*it, p);
            color++;
        }
    }

    return;
}


static float geometryMinWidth = 0;
static float geometryMinHeight = 0;
bool
SegmentationClusteringDocumentPreseg5::blockGeometryOk(girl::region reg)
{
    if (geometryMinHeight == 0 || geometryMinWidth == 0) {
        // geometryMinWidth = std::accumulate(m_ccWidths.begin(), m_ccWidths.end(), 0);
        // geometryMinHeight = std::accumulate(m_ccHeights.begin(), m_ccHeights.end(), 0);
        for (girl::regionDeque::const_iterator it = m_binarizationFgRegions.begin();
             it != m_binarizationFgRegions.end(); ++it) {
            geometryMinWidth += m_ccWidths[*it];
            geometryMinHeight += m_ccHeights[*it];
        }
        if (m_binarizationFgRegions.size() > 0) {
            geometryMinWidth /= m_binarizationFgRegions.size();
            geometryMinHeight /= m_binarizationFgRegions.size();
        }
        geometryMinWidth = std::max(3.0f, geometryMinWidth/3);
        geometryMinHeight = std::max(3.0f, geometryMinHeight/3);
    }
    girl::tools::AABBox box = girl::tools::getAABBox(*m_segContext2.getToger(), reg);
    if (box.width() < geometryMinWidth || box.height() < geometryMinHeight)
        return false;
    else
        return true;
}

void
SegmentationClusteringDocumentPreseg5::computeStatsWords()
{
    const int connectivity = 8;
    //size_t nbCC = 
    ise::ConnectedComponentTools::extractAllConnectedComponents(m_wordMask, m_wordCCLabels, connectivity);//, &m_wordCcWidths, &m_wordCcHeights);
    ise::BitmapRunLength rlh(m_wordCCLabels, ise::RunLengthExtractor::HORIZONTAL);
    rlh.getConnectedComponentsAreas(m_wordCcAreas);
    rlh.getConnectedComponentsAABBox(m_wordBoxes);
    for (size_t i = 0; i < m_wordBoxes.size(); ++i) {
        m_wordCcWidths.push_back(m_wordBoxes[i].width());
        m_wordCcHeights.push_back(m_wordBoxes[i].height());
    }

    rlh.getSpacings(m_wordSpacingsHoriz);

    ise::BitmapRunLength rlv(m_wordCCLabels, ise::RunLengthExtractor::VERTICAL);
    rlv.getSpacings(m_wordSpacingsVerti);
    // ise::ConnectedComponentTools::computeSpacing(m_wordMask, m_wordCCLabels, nbCC, m_wordSpacingsHoriz, ise::RunLengthExtractor::HORIZONTAL);
    // ise::ConnectedComponentTools::computeSpacing(m_wordMask, m_wordCCLabels, nbCC, m_wordSpacingsVerti, ise::RunLengthExtractor::VERTICAL);

}

void
SegmentationClusteringDocumentPreseg5::computeStatsConnectedComponents()
{
    const int connectivity = 8;
    //size_t nbCC = 
    ise::ConnectedComponentTools::extractAllConnectedComponents(m_binarizationMask, m_ccLabels, connectivity, &m_ccWidths, &m_ccHeights);
    //size_t nbCC = m_binarizationToger.numRegions();
    // ise::ConnectedComponentTools::computeSpacing(m_binarizationMask, m_ccLabels, nbCC, m_spacingsHoriz, ise::RunLengthExtractor::HORIZONTAL);
    // ise::ConnectedComponentTools::computeSpacing(m_binarizationMask, m_ccLabels, nbCC, m_spacingsVerti, ise::RunLengthExtractor::VERTICAL);
    ise::BitmapRunLength h(m_ccLabels, ise::RunLengthExtractor::HORIZONTAL);
    ise::BitmapRunLength v(m_ccLabels, ise::RunLengthExtractor::VERTICAL);
    h.getSpacings(m_spacingsHoriz, false);
    v.getSpacings(m_spacingsVerti, false);
}


void
SegmentationClusteringDocumentPreseg5::computeStatsWords2()
{
    // const int connectivity = 8;
    // size_t nbCC = ise::ConnectedComponentTools::extractAllConnectedComponents(m_wordMask, m_wordCCLabels, connectivity, &m_wordCcWidths, &m_wordCcHeights);
    // ise::BitmapRunLength rlh(m_wordCCLabels, ise::RunLengthExtractor::HORIZONTAL);
    // ise::BitmapRunLength rlv(m_wordCCLabels, ise::RunLengthExtractor::VERTICAL);
    // std::vector<ise::BitmapRunLength::SpacingInfo> spacingsH;
    // std::vector<ise::BitmapRunLength::SpacingInfo> spacingsV;
    // rlh.getSpacings(spacingsH, false);
    // m_spacingsHoriz.assign(nbCC, ise::NeighbouringRegions());
    // rlv.getSpacings(spacingsV, false);
    // m_spacingsVerti.assign(nbCC, NeighbouringRegions());
    // for (std::vector<ise::BitmapRunLength::SpacingInfo>::const_iterator it = spacingsH.begin();
    //      it != spacingsH.end(); ++it) {
    //   if (m_spacingsHoriz[it->a - 1].distNext > it->dist) {
    //     m_spacingsHoriz[it->a - 1].distNext = it->dist;
    //     m_spacingsHoriz[it->a - 1].next = it->b;
    //   }
    //   if (m_spacingsHoriz[it->b - 1].distPrev > it->dist) {
    //     m_spacingsHoriz[it->a - 1].distPrev = it->dist;
    //     m_spacingsHoriz[it->a - 1].prev = it->b;
    //   }
    // }

    // for (std::vector<ise::BitmapRunLength::SpacingInfo>::const_iterator it = spacingsV.begin();
    //      it != spacingsV.end(); ++it) {
    //   if (m_spacingsVerti[it->a - 1].distNext > it->dist) {
    //     m_spacingsVerti[it->a - 1].distNext = it->dist;
    //     m_spacingsVerti[it->a - 1].next = it->b;
    //   }
    //   if (m_spacingsVerti[it->b - 1].distPrev > it->dist) {
    //     m_spacingsVerti[it->a - 1].distPrev = it->dist;
    //     m_spacingsVerti[it->a - 1].prev = it->b;
    //   }
    // }

}



//void
//SegmentationClusteringDocumentPreseg5::computeWordToger2()
//{
//  ise::BitmapRunLength rlh(m_ccLabels, ise::RunLengthExtractor::HORIZONTAL);
//  std::vector<ise::BitmapRunLength::SpacingInfo> spacingsH;
//  rlh.getSpacings(spacingsH, false);

//  typedef std::pair<int, int> RegPair;
//  typedef girl::hashMap<RegPair, size_t>  DistType;
//  DistType dists;
//  size_t maxDist = 0;
//  for (std::vector<ise::BitmapRunLength::SpacingInfo>::const_iterator it = spacingsH.begin();
//       it != spacingsH.end(); ++it) {
//    if (it->dist > maxDist) {
//      maxDist = it->dist;
//    }
//    RegPair pair(it->a, it->b);
//    DistType::iterator pIt = dists.find(pair);
//    if (pIt != dists.end()) {
//      pIt->second = std::min(pIt->second, it->dist);
//    } else {
//      dists[pair] = it->dist;
//    }
//  }

//  ise::Histogram1D histo(0, maxDist);
//  for (std::vector<ise::BitmapRunLength::SpacingInfo>::const_iterator it = spacingsH.begin();
//       it != spacingsH.end(); ++it) {
//    histo[it->dist]++;
//  }

//  QuantificationThresholderDomenger jp;
//  std::vector<double> thresholds;
//  jp.getThresholds(histo, thresholds);

//  std::cout << "Thresholds word toger 2=" << thresholds[0] <<  std::endl;

//}


void
SegmentationClusteringDocumentPreseg5::computeDistanceHistogram(const std::vector<ise::BitmapRunLength::SpacingInfo>& spacings, ise::Histogram1D &histo)
{
    typedef std::vector<ise::BitmapRunLength::SpacingInfo> VectorType;
    VectorType::const_iterator it = spacings.begin();
    const VectorType::const_iterator itEnd = spacings.end();
    int min = 0;
    int max = 0;
    for (; it != itEnd; ++it) {
        int val = static_cast<int>(it->dist);
        if (val > max)
            max = val;
    }

    histo.reset(min, max);

    for (it = spacings.begin(); it != itEnd; ++it) {
        histo[it->dist]++;
    }
}


/*
struct WordPartitionner
{
    WordPartitionner(size_t threshold, const cv::Mat &distsMat):
        m_t(threshold), m_mat(distsMat)
    {
        assert(distsMat.type() == CV_32S);
    }
    
    bool
    operator()(int a, int b)
    {
        return m_mat.at<int>(a,b) < m_t;
    }

    size_t m_t;
    const cv::Mat &m_mat;
};
*/

void
SegmentationClusteringDocumentPreseg5::computeWord()
{
    Histogram1D histo;

    // create neighbouringregions info from spacingsInfo
    size_t nbCC = m_ccWidths.size();
    std::vector<ise::NeighbouringRegions> spacingsHoriz(nbCC, ise::NeighbouringRegions());
    for (std::vector<ise::BitmapRunLength::SpacingInfo>::const_iterator it = m_spacingsHoriz.begin();
         it != m_spacingsHoriz.end(); ++it) {
        int dist = it->dist;
        int a = it->a - 1;
        int b = it->b - 1;
        if (dist < spacingsHoriz[a].distNext) {
            spacingsHoriz[a].next = b;
            spacingsHoriz[a].distNext = dist;
        }
        if(dist < spacingsHoriz[b].distPrev) {
            spacingsHoriz[b].prev = a;
            spacingsHoriz[b].distPrev = dist;
        }
    }

    ConnectedComponentTools::computeDistanceHistogram(spacingsHoriz, histo);
    QuantificationThresholderDomenger jp;
    std::vector<double> thresholds;
    jp.getThresholds(histo, thresholds);
    std::cout << "Thresholds word toger 1=" << thresholds[0] <<  std::endl;
    girl::hashMap<int, int> ccToWordMap;
    ise::ConnectedComponentTools::computeLinks_p(spacingsHoriz, ccToWordMap, thresholds[0]);
    //  for (girl::hashMap<int, int>::const_iterator it = ccToWordMap.begin();
    //       it != ccToWordMap.end(); ++it) {
    //    std::cout << it->first << "," << it->second << std::endl;
    //  }


    struct SameWord {
        explicit SameWord(girl::hashMap<int, int> &ccToWordMap) :
            m_ccToWordMap(ccToWordMap)
        {}

        bool operator()(int i, int j) const {
            girl::hashMap<int, int>::const_iterator iti = m_ccToWordMap.find(i);
            girl::hashMap<int, int>::const_iterator itj = m_ccToWordMap.find(j);
            assert(iti != m_ccToWordMap.end());
            assert(itj != m_ccToWordMap.end());
            return iti->second == itj->second;
        }

        girl::hashMap<int, int> &m_ccToWordMap;

    };

    ise::BitmapRunLength rlh(m_ccLabels, ise::RunLengthExtractor::HORIZONTAL);
    BitmapRunLength::RunLengthType horizRunLength;
    BitmapRunLength::RunLengthLabelsType horizLabels;
    rlh.getRunLength(horizRunLength, horizLabels);
    struct SameWord sameW(ccToWordMap);
    m_wordMask = cv::Mat::zeros(m_binarizationMask.size(), CV_8U);
    for (size_t i = 0; i < horizRunLength.size(); ++i) {
        cv::Point start(0, i);
        for (size_t j = 0; j < horizRunLength[i].size(); ++j) {
            cv::Point end = start;
            end.x += horizRunLength[i][j] - 1;
            bool mustDraw = false;
            if (horizLabels[i][j] == 0) {
                // on background. Paint only if linking CC of the same word
                if (j-1 < horizRunLength[i].size() && j+1 < horizRunLength[i].size()) {
                    if (sameW(horizLabels[i][j-1] - 1, horizLabels[i][j+1] - 1)) {
                        mustDraw = true;
                    }
                }
            } else {
                // on foreground, always paint
                mustDraw = true;
            }
            if (mustDraw) {
                cv::line(m_wordMask, start, end, CV_RGB(255, 255, 255));
            }
            start = end;
            start.x++;
        }
    }

#ifdef VISU
    {
        std::vector<ise::Histogram1D> histos;
        histos.push_back(histo);
        std::vector<std::string> histoNames;
        histoNames.push_back("CC spacing histo");
        g_info.setTo(0);
        writeHisto(g_info, histos, histoNames);
        int fromTo[6] = {0, 0, 0, 1, 0, 2};
        cv::mixChannels(&m_wordMask, 1, &g_orig, 1, fromTo, 3);
        std::vector<std::string> msg;
        displayVisu(msg);
        //    cv::namedWindow("Words", 0);
        //    cv::imshow("Words", m_wordMask);
        //    cv::waitKey(0);
    }
#endif
}



void
SegmentationClusteringDocumentPreseg5::init(const girl::region, int)
{

}

bool
SegmentationClusteringDocumentPreseg5::same(const girl::point& p, girl::direction dir)
{
    assert(m_image.type() == CV_8U);
    assert(!m_image.empty());
    girl::point p2(p);
    p2.move(dir);
    return m_image.at<unsigned char>(p.y(), p.x()) == m_image.at<unsigned char>(p2.y(), p2.x());
}

void
SegmentationClusteringDocumentPreseg5::getRegionsByType(const ise::SegmentationContext *segContext, girl::regionDeque &textRegions, girl::regionDeque &imageRegions, girl::regionDeque &separatorRegions) const
{
    assert(!m_image.empty());
    const girl::toger *tog = segContext->getToger();
    textRegions.clear();
    imageRegions.clear();
    separatorRegions.clear();
    girl::regionDeque allRegs;
    tog->allRegions(allRegs);
    girl::regionDeque::const_iterator it = allRegs.begin();
    const girl::regionDeque::const_iterator itEnd = allRegs.end();
    for (; it != itEnd; ++it) {
        girl::point p = tog->anyPoint(*it);
        int val = static_cast<int>(m_image.at<unsigned char>(p.y(), p.x()));
        switch(val) {
        case 0 :
            break;
            //    case TEXT_REGION_INDEX :
            //      textRegions.push_back(*it);
            //      break;
        case IMAGE_REGION_INDEX:
            imageRegions.push_back(*it);
            break;
        case SEPARATOR_REGION_INDEX:
            separatorRegions.push_back(*it);
            break;
        default:
            textRegions.push_back(*it);
            break;
        }
    }
}


//void
//SegmentationClusteringDocumentPreseg5::computeConnectedComponentsStatistics(const cv::Mat &binaryMask,
//                                                                            cv::Mat &ccLabels,
//                                                                            std::vector<ise::NeighbouringRegions> &spacingsHoriz,
//                                                                            std::vector<ise::NeighbouringRegions> &spacingsVerti,
//                                                                            std::vector<size_t> &ccWidths,
//                                                                            std::vector<size_t> &ccHeights
//                                                                            )
//{
//  const int connectivity = 8;
//  size_t nbCC = ise::ConnectedComponentTools::extractAllConnectedComponents(binaryMask, ccLabels, connectivity, &ccWidths, &ccHeights);
//  ise::ConnectedComponentTools::computeSpacing(binaryMask, ccLabels, nbCC, spacingsHoriz, ise::RunLengthExtractor::HORIZONTAL);
//  ise::ConnectedComponentTools::computeSpacing(binaryMask, ccLabels, nbCC, spacingsVerti, ise::RunLengthExtractor::VERTICAL);
//}


void
SegmentationClusteringDocumentPreseg5::removeBorders(const girl::region r, girl::regionDeque &regs)
{
    girl::toger *tog = m_segContext2.getToger();
    girl::tools::AABBox box = girl::tools::getAABBox(*tog, r);
    cv::Rect rect(box.xMin(), box.yMin(), box.width(), box.height());
    cv::Mat roi = m_binarizationMask(rect);
    cv::Mat mask = cv::Mat::zeros(m_binarizationMask.size(), CV_8U);
    girl::tools::Painter<unsigned char> p(mask, CV_RGB(255, 255, 255));
    tog->iterateOnRegionDomain(r, p);
    mask = mask(rect);
    cv::Range rowRange(box.yMin(), box.yMin() + box.height());
    cv::Range colRange(box.xMin(), box.xMin() + box.width());
    for (int i = 0; i < roi.rows; ++i) {
        cv::Mat theRow  = roi.row(i);
        cv::Mat theMask = mask.row(i);
        double norm = cv::norm(theRow, cv::NORM_L1, theMask);
        if (norm == 0) {
            rowRange.start++;
        }
        else
            break;
    }
    for (int i = roi.rows - 1; i >= 0; --i) {
        cv::Mat theRow  = roi.row(i);
        cv::Mat theMask = mask.row(i);
        double norm = cv::norm(theRow, cv::NORM_L1, theMask);
        if (norm == 0) {
            rowRange.end--;
        }
        else
            break;
    }
    for (int i = 0; i < roi.cols; ++i) {
        cv::Mat theRow  = roi.col(i);
        cv::Mat theMask = mask.col(i);
        double norm = cv::norm(theRow, cv::NORM_L1, theMask);
        if (norm == 0) {
            colRange.start++;
        }
        else
            break;
    }
    for (int i = roi.cols - 1; i >= 0; --i) {
        cv::Mat theRow  = roi.col(i);
        cv::Mat theMask = mask.col(i);
        double norm = cv::norm(theRow, cv::NORM_L1, theMask);
        if (norm == 0) {
            colRange.end--;
        }
        else
            break;
    }

    // call split only if borders have changed
    if (rowRange.start > box.yMin() || rowRange.end < box.yMin() + box.height()
            || colRange.start > box.xMin() || colRange.end < box.xMin() + box.width()) {
        RoiSplitter s(rowRange, colRange);
        //  girl::regionDeque regs;
        tog->splitRegion(r, s, regs);
#ifdef ADVANCED_VISU
        {
            std::vector<cv::Mat> planes;
            cv::split(*m_segContext2.getImage()->cvMat(), planes);//convertTo(g_orig, CV_8UC3);
            std::vector<cv::Mat> planes2(3);
            for (size_t i = 0; i < 3; ++i) {
                planes[i].convertTo(planes2[2 - i], CV_8UC1);
            }
            cv::merge(planes2, g_orig);
            girl::tools::paintRegionBorders(g_orig, *tog, regs, CV_RGB(255, 0, 0), 3);
            std::vector<std::string> msg;
            msg.push_back("Removed borders");
            displayVisu(msg);
        }
#endif
        // for (girl::regionDeque::const_iterator it = regs.begin();
        //      it != regs.end(); ++it) {
        //   girl::point pt = tog->anyPoint(*it);
        //      if (pt.x() >= colRange.start && pt.x() < colRange.end && pt.y() >= rowRange.start && pt.y() < rowRange.end)
        //   return *it;
        // }
        // assert(false);


    }

#ifdef ADVANCED_VISU
    else {
        std::vector<std::string> msg;
        msg.push_back("No borders removed");
        displayVisu(msg);
        //    return r;
    }
#endif

}



void
SegmentationClusteringDocumentPreseg5::computeConnectedComponentsStats(const girl::unorderedSet<int> &ccIndexes,
                                                                       Histogram1D &ccWidthsHisto,
                                                                       Histogram1D &ccHeightsHisto,
                                                                       Histogram1D &spacingsCCHorizHisto,
                                                                       Histogram1D &spacingsCCVertiHisto)
{

    // 2.1: cc stats
    int wMax = std::numeric_limits<int>::min();
    int wMin = std::numeric_limits<int>::max();
    int hMax = std::numeric_limits<int>::min();
    int hMin = std::numeric_limits<int>::max();

    // TODO:OPTIM could be done in the loop above which separates fg/bg regions
    for (girl::unorderedSet<int>::const_iterator it = ccIndexes.begin();
         it != ccIndexes.end(); ++it) {
        int w = m_ccWidths[*it/*-1*/];
        int h = m_ccHeights[*it/*-1*/];
        if (w > wMax)
            wMax = w;
        if (w < wMin)
            wMin = w;
        if (h > hMax)
            hMax = h;
        if (h < hMin)
            hMin = h;
    }

    ccWidthsHisto.reset(wMin, wMax);
    ccHeightsHisto.reset(hMin, hMax);
    for (girl::unorderedSet<int>::const_iterator it = ccIndexes.begin();
         it != ccIndexes.end(); ++it) {
        ccWidthsHisto[ccWidthsHisto.keyToIndex(m_ccWidths[*it/* - 1*/])]++;
        ccHeightsHisto[ccHeightsHisto.keyToIndex(m_ccHeights[*it/* - 1*/])]++;
    }


    // histo spacings horiz
    int spacingMax = 0;
    std::vector<int> spacings;
    spacings.reserve(m_spacingsHoriz.size());
    for (std::vector<ise::BitmapRunLength::SpacingInfo>::const_iterator it = m_spacingsHoriz.begin();
         it != m_spacingsHoriz.end(); ++it) {
        if (ccIndexes.find(it->a) != ccIndexes.end() && ccIndexes.find(it->b) != ccIndexes.end()) {
            spacingMax = std::max(spacingMax, static_cast<int>(it->dist));
            spacings.push_back(it->dist);
        }
    }
    spacingsCCHorizHisto.reset(0, spacingMax);
    for (std::vector<int>::const_iterator it = spacings.begin();
         it != spacings.end(); ++it) {
        spacingsCCHorizHisto[spacingsCCHorizHisto.keyToIndex(*it)]++;
    }

    // histo spacings verti
    spacingMax = 0;
    spacings.clear();
    spacings.reserve(m_spacingsVerti.size());
    for (std::vector<ise::BitmapRunLength::SpacingInfo>::const_iterator it = m_spacingsVerti.begin();
         it != m_spacingsVerti.end(); ++it) {
        if (ccIndexes.find(it->a) != ccIndexes.end() && ccIndexes.find(it->b) != ccIndexes.end()) {
            spacingMax = std::max(spacingMax, static_cast<int>(it->dist));
            spacings.push_back(it->dist);
        }
    }
    spacingsCCVertiHisto.reset(0, spacingMax);
    for (std::vector<int>::const_iterator it = spacings.begin();
         it != spacings.end(); ++it) {
        spacingsCCVertiHisto[spacingsCCVertiHisto.keyToIndex(*it)]++;
    }


    // std::vector<ise::NeighbouringRegions> spacingsCCHoriz;
    // std::vector<ise::NeighbouringRegions> spacingsCCVerti;
    // for (girl::unorderedSet<int>::const_iterator it = ccIndexes.begin();
    //      it != ccIndexes.end(); ++it) {
    //   ccWidthsHisto[ccWidthsHisto.keyToIndex(m_ccWidths[*it - 1])]++;
    //   ccHeightsHisto[ccHeightsHisto.keyToIndex(m_ccHeights[*it - 1])]++;
    //   ise::NeighbouringRegions nbgHoriz = m_spacingsHoriz[*it - 1];
    //   ise::NeighbouringRegions nbgVerti = m_spacingsVerti[*it - 1];
    //   // add only cc with neighbours which are actually in the block
    //   if (ccIndexes.find(nbgHoriz.prev) != ccIndexes.end() &&
    //       ccIndexes.find(nbgHoriz.next) != ccIndexes.end()) {
    //     spacingsCCHoriz.push_back(nbgHoriz);
    //   }
    //   if (ccIndexes.find(nbgVerti.prev) != ccIndexes.end() &&
    //       ccIndexes.find(nbgVerti.next) != ccIndexes.end()) {
    //     spacingsCCVerti.push_back(nbgVerti);
    //   }
    // }

    // ise::ConnectedComponentTools::computeDistanceHistogram(spacingsCCHoriz, spacingsCCHorizHisto);
    // ise::ConnectedComponentTools::computeDistanceHistogram(spacingsCCVerti, spacingsCCVertiHisto);


}

void
SegmentationClusteringDocumentPreseg5::computeWordsStats(const girl::unorderedSet<int> &wordsInTheBlock,
                                                         Histogram1D &spacingsWordsHorizHisto,
                                                         Histogram1D &spacingsWordsVertiHisto)
{
    // std::vector<ise::NeighbouringRegions> spacingsWordsHoriz;
    // std::vector<ise::NeighbouringRegions> spacingsWordsVerti;
    // for (girl::unorderedSet<int>::const_iterator it = wordsInTheBlock.begin();
    //      it != wordsInTheBlock.end(); ++it) {
    //   ise::NeighbouringRegions nbg = m_wordSpacingsHoriz[*it - 1];
    //   if (wordsInTheBlock.find(nbg.prev) != wordsInTheBlock.end() &&
    //       wordsInTheBlock.find(nbg.next) != wordsInTheBlock.end()) {
    //     spacingsWordsHoriz.push_back(nbg);
    //   }
    //   ise::NeighbouringRegions nbgVerti = m_wordSpacingsVerti[*it - 1];
    //   if (wordsInTheBlock.find(nbgVerti.prev) != wordsInTheBlock.end() &&
    //       wordsInTheBlock.find(nbgVerti.next) != wordsInTheBlock.end()) {
    //     spacingsWordsVerti.push_back(nbgVerti);
    //   }
    // }
    // ise::ConnectedComponentTools::computeDistanceHistogram(spacingsWordsHoriz, spacingsWordsHorizHisto);
    // ise::ConnectedComponentTools::computeDistanceHistogram(spacingsWordsVerti, spacingsWordsVertiHisto);

    // std::cout << "Done. " << std::endl;

    size_t maxSpacing = 0;
    std::vector<size_t> spacings;
    spacings.reserve(m_wordSpacingsHoriz.size());
    for (std::vector<ise::BitmapRunLength::SpacingInfo>::const_iterator it = m_wordSpacingsHoriz.begin();
         it != m_wordSpacingsHoriz.end(); ++it) {
        if(wordsInTheBlock.find(it->a) != wordsInTheBlock.end() && wordsInTheBlock.find(it->b) != wordsInTheBlock.end()) {
            // valid spacing
            if(maxSpacing < it->dist) {
                maxSpacing = it->dist;
            }
            spacings.push_back(it->dist);
        }
    }
    spacingsWordsHorizHisto.reset(0,maxSpacing);
    for (std::vector<size_t>::const_iterator it = spacings.begin();
         it != spacings.end(); ++it) {
        spacingsWordsHorizHisto[spacingsWordsHorizHisto.keyToIndex(*it)]++;
    }

    spacings.clear();
    maxSpacing = 0;
    for (std::vector<ise::BitmapRunLength::SpacingInfo>::const_iterator it = m_wordSpacingsVerti.begin();
         it != m_wordSpacingsVerti.end(); ++it) {
        if(wordsInTheBlock.find(it->a) != wordsInTheBlock.end() && wordsInTheBlock.find(it->b) != wordsInTheBlock.end()) {
            // valid spacing
            if(maxSpacing < it->dist) {
                maxSpacing = it->dist;
            }
            spacings.push_back(it->dist);
        }
    }
    spacingsWordsVertiHisto.reset(0,maxSpacing);
    for (std::vector<size_t>::const_iterator it = spacings.begin();
         it != spacings.end(); ++it) {
        spacingsWordsVertiHisto[spacingsWordsVertiHisto.keyToIndex(*it)]++;
    }

}

//void
//SegmentationClusteringDocumentPreseg5::computeWhitespace(girl::region r,
//                                                         girl::regionDeque &createdRegs,
//                                                         size_t minWidth,
//                                                         size_t minHeight,
//                                                         size_t minArea)
//{
//  std::cout << "Adding block obstacles..." << std::endl;
//  const girl::toger *originalToger = m_segContext2.getToger();
//  // create toger corresponding to the block with its word regions
//  girl::toger wordToger = *originalToger;
//  SegmentationContext segContextWord(&wordToger, m_segContext2.getImage());
//  girl::regionDeque splitRegsWord;
//  bitmapSplit(segContextWord, r, m_wordMask, splitRegsWord);

//  girl::unorderedSet<size_t> ccIndexes;
//  girl::regionDeque fgRegsCC, bgRegsCC;
//  ConnectedComponentCounter counter(m_ccLabels, m_binarizationToger, fgRegsCC, bgRegsCC, ccIndexes);
//  originalToger->iterateOnRegionDomain(r, counter);

//  // Detect FG/BG word regions in the block and their index for stats
//  girl::unorderedSet<size_t> wordsIndexes;
//  girl::regionDeque fgRegsWord, bgRegsWord;
//  for (girl::regionDeque::const_iterator it = splitRegsWord.begin();
//       it != splitRegsWord.end(); ++it) {
//    girl::point p = wordToger.anyPoint(*it);
//    // TODO:OPTIM memory is wordMask and wordCCLabels both necessary?
//    if (m_wordMask.at<unsigned char>(p.y(), p.x())) {
//      fgRegsWord.push_back(*it);
//      // cc are numbered from 1 to N but their stats are stored in vectors indexed 0...N-1
//      size_t wordIndex = static_cast<size_t>(m_wordCCLabels.at<int>(p.y(), p.x())/* - 1*/);
//      assert(wordIndex < m_wordCcWidths.size());
//      wordsIndexes.insert(wordIndex);
//    } else {
//      bgRegsWord.push_back(*it);
//    } //B:Warning: fgRegs will contain holes in letters !!!
//  }

//  const size_t maxNbRect = 100; // (unused)
//  const double overlapRatio    = 0;
//  const bool mergeWhiteSpaces  = false;
//  const bool useBBForDirectNb  = false;
//  const bool useBBForInnerNb   = true;
//  SegmentationClusteringMaximalWhitespaceRectangle clustering(&segContextWord);
//  clustering.setParameters(minArea, maxNbRect, minWidth, minWidth, minHeight, minHeight, overlapRatio, mergeWhiteSpaces, useBBForDirectNb, useBBForInnerNb);
//  SplitAlgorithmDefault algo(&segContextWord);
//  SplitDriverDefault driver(&segContextWord);
//  girl::regionDeque toSplit, createdRegions;
//  // Adding the obstacle can split original region in several bg regions
//  // launch WS on all bg regions that are not children of fg regions (holes in letters)
//  girl::unorderedSet<girl::region> fgRegsUset;
//  for (girl::regionDeque::const_iterator it = fgRegsWord.begin();
//       it != fgRegsWord.end(); ++it) {
//    fgRegsUset.insert(*it);
//  }
//  for (girl::regionDeque::const_iterator it = bgRegsWord.begin();
//       it != bgRegsWord.end(); ++it) {
//    girl::regionDeque mothers;
//    wordToger.regionNeighbourhood(*it, girl::toger::NEIGHBOUR_OUTER, girl::toger::MARK_NONE, mothers);
//    if (mothers.empty() || fgRegsUset.find(mothers[0]) == fgRegsUset.end()) {
//      //TODO:OPTIM could avoid compute area especially with words... could be checked in WS algo
//      const size_t area = girl::tools::computeArea(wordToger, *it);
//      if (area > minArea) {
//        toSplit.push_back(*it);
//      }
//    }
//  }



//  std::cerr<<"WS: "<<toSplit.size()<<" regions to split"<<std::endl;
//#ifdef BOOST_TIMER
//  boost::timer t;
//#endif
//  algo.apply(toSplit, driver, clustering, 0, createdRegions);
//#ifdef BOOST_TIMER
//  std::cout << "Time elapsed computing WS=" << t.elapsed() << std::endl;
//#endif
//  cv::Mat mat = cv::Mat::zeros(m_binarizationToger.height(), m_binarizationToger.width(), CV_8U);
//  // get a point from each of the newly created rectangles to add only the corresponding regions
//  std::vector<girl::point> createdRectanglesPoints;
//  clustering.getPointsFromResults(createdRectanglesPoints);
//  std::cerr<<"WS: createdRegions.size()="<<createdRegions.size()<<"  createdRectanglesPoints.size()="<<createdRectanglesPoints.size()<<std::endl;
//  for (std::vector<girl::point>::const_iterator it = createdRectanglesPoints.begin();
//       it != createdRectanglesPoints.end(); ++it) {
//    girl::region reg = wordToger.findRegion(*it);
//    Painter p(mat, CV_RGB(rand()%255, 255, 255));
//    wordToger.iterateOnRegionDomain(reg, p);
//  }



//  // finally split the region using the found rectangles
//  bitmapSplit(m_segContext2, r, mat, createdRegs);

//}


void
SegmentationClusteringDocumentPreseg5::computeWhitespace(girl::region r,
                                                         girl::regionDeque &createdRegs,
                                                         const girl::unorderedSet<int> &wordsInTheBlock,
                                                         size_t minWidth,
                                                         size_t minHeight,
                                                         size_t minArea)
{


    girl::toger *tog = m_segContext2.getToger();
    girl::point p0, p1;
    girl::tools::computeBoundingBox(*tog, r, p0, p1);
    const SegmentationClusteringMaximalWhitespaceRectangle::BBox b(p0.x(), p0.y(), p1.x() + 1, p1.y() + 1);

    girl::unorderedSet<int>::const_iterator it = wordsInTheBlock.begin();
    const girl::unorderedSet<int>::const_iterator itEnd = wordsInTheBlock.end();
    std::vector<SegmentationClusteringMaximalWhitespaceRectangle::BBox> obstacles;
    obstacles.reserve(wordsInTheBlock.size());
    for (; it != itEnd; ++it) {
        SegmentationClusteringMaximalWhitespaceRectangle::BBox box(m_wordBoxes[*it].xMin(),
                                                                   m_wordBoxes[*it].yMin(),
                                                                   m_wordBoxes[*it].xMax()+1,
                                                                   m_wordBoxes[*it].yMax()+1);
        obstacles.push_back(box);
    }

    std::vector<size_t> externalObstacles;
    girl::regionDeque obstructingRegs;
    tog->regionNeighbourhood(r, girl::toger::NEIGHBOUR_DIRECT, girl::toger::MARK_NONE, obstructingRegs);
    girl::regionDeque::const_iterator obsIt = obstructingRegs.begin();
    const girl::regionDeque::const_iterator obsItEnd = obstructingRegs.end();
    for (; obsIt != obsItEnd; ++obsIt) {
        std::vector<girl::point> scanlines;
        tog->regionDomain(*obsIt, scanlines);
        for (std::vector<girl::point>::const_iterator itP = scanlines.begin();
             itP != scanlines.end(); itP+=2) {
            assert(itP->y() == (itP+1)->y());
            assert(itP->y() >= 0);
            const SegmentationClusteringMaximalWhitespaceRectangle::BBox bb(itP->x(), itP->y(), (itP+1)->x()+1, itP->y()+1);
            // Ensure overlaps to optimise number of obstacles and make sure eccentricity is positive in function
            // choosePivot
            if (bb.overlaps(b)) {
                obstacles.push_back(bb);
                externalObstacles.push_back(obstacles.size() - 1);
            }
        }
    }
    obstacles.push_back(SegmentationClusteringMaximalWhitespaceRectangle::BBox(b.x0() - 1, b.y0(), b.x0(), b.y1()));
    externalObstacles.push_back(obstacles.size() - 1);
    obstacles.push_back(SegmentationClusteringMaximalWhitespaceRectangle::BBox(b.x1(), b.y0(), b.x1() + 1, b.y1()));
    externalObstacles.push_back(obstacles.size() - 1);
    obstacles.push_back(SegmentationClusteringMaximalWhitespaceRectangle::BBox(b.x0(), b.y0() - 1, b.x1(), b.y0()));
    externalObstacles.push_back(obstacles.size() - 1);
    obstacles.push_back(SegmentationClusteringMaximalWhitespaceRectangle::BBox(b.x0(), b.y1(), b.x1(), b.y1() + 1));
    externalObstacles.push_back(obstacles.size() - 1);


    const size_t maxNbRect = 100; // (unused)
    const double overlapRatio    = 0;
    const bool mergeWhiteSpaces  = false;
    SegmentationClusteringMaximalWhitespaceRectangle clustering(&m_segContext2);
    clustering.setParameters(minArea, maxNbRect, minWidth, minHeight, overlapRatio, mergeWhiteSpaces, obstacles, externalObstacles);
    clustering.computeRectangles(b, r);
    cv::Mat mat = cv::Mat::zeros(tog->height(), tog->width(), CV_8U);
    const std::vector<ise::SegmentationClusteringMaximalWhitespaceRectangle::BBox> &ws =
            clustering.getResults();
    for (std::vector<ise::SegmentationClusteringMaximalWhitespaceRectangle::BBox>::const_iterator it = ws.begin();
         it != ws.end(); ++it) {
        cv::rectangle(mat, cv::Rect(it->x0(), it->y0(), it->width(), it->height()), CV_RGB(255, 255, 255), CV_FILLED);
    }
    createdRegs.clear();
    if (! ws.empty())
      bitmapSplit(m_segContext2, r, mat, createdRegs);

}

void
SegmentationClusteringDocumentPreseg5::computeProfile(const girl::region r,
                                                      girl::regionDeque &createdRegs,
                                                      size_t erodeHoriz,
                                                      size_t dilateHoriz,
                                                      size_t erodeVerti,
                                                      size_t dilateVerti)
{

    ise::SegmentationClusteringProfile clustProfile(&m_segContext2);
    cv::Mat tmp;
    cv::bitwise_not(m_wordMask, tmp);
    girl::shared_ptr<girl::tools::ImageFeature> binaryImagePtr(new girl::tools::ImageFeature(tmp));
    girl::shared_ptr<ImageFeatureBinaryComputer> binaryComputerPtr( new ise::ImageFeatureThreshold(&m_segContext2));
    binaryComputerPtr->setImageFeature(binaryImagePtr);
    clustProfile.setParameters(erodeHoriz, dilateHoriz, erodeVerti, dilateVerti, 0.05, binaryComputerPtr);

    SplitAlgorithmDefault splitAlgo(&m_segContext2);
    SplitDriverDefault splitDriver(&m_segContext2);
    girl::regionDeque toSplit;
    toSplit.push_back(r);
    splitAlgo.apply(toSplit, splitDriver, clustProfile, 1, createdRegs);
}

void
SegmentationClusteringDocumentPreseg5::processBlock(girl::region r,
                                                    girl::regionDeque &regsToProcess,
                                                    const girl::unorderedSet<int> &connectedComponentsInTheBlock,
                                                    const girl::unorderedSet<int> &wordsInTheBlock)
{
    std::cerr<<"processBlock() r="<<r<<" regs.size()="<<regsToProcess.size()<<std::endl;


#ifdef VISU
    {
        girl::regionDeque regs;
        regs.push_back(r);
        std::vector<cv::Mat> planes;
        cv::split(*m_segContext2.getImage()->cvMat(), planes);//convertTo(g_orig, CV_8UC3);
        std::vector<cv::Mat> planes2(3);
        for (size_t i = 0; i < 3; ++i) {
            planes[i].convertTo(planes2[2 - i], CV_8UC1);
        }
        cv::merge(planes2, g_orig);
        girl::tools::paintRegionBorders(g_orig, m_toger, regs, CV_RGB(255, 0, 0), 3);
        std::vector<std::string> msg;
        msg.push_back("Original block");
        displayVisu(msg);
    }
#endif
    {
        // Step 1: remove borders
        girl::regionDeque createdRegs;
        removeBorders(r, createdRegs);
        if (createdRegs.size() > 1) {
            for (girl::regionDeque::const_iterator it = createdRegs.begin();
                 it != createdRegs.end(); ++it) {
                regsToProcess.push_back(*it);
            }
            return;
        }
    }


    // Stop here if the block contains only a single word
    if (wordsInTheBlock.size() == 1)
        return;


    // r = mergeIncorrectIllustrations(r, wordsInTheBlock);



    //  // Step 2: gather block statistics
    // consider, according to statistical theory, that the minimal number of CC
    // inside a block must be at least 30 to be representative
    if (connectedComponentsInTheBlock.size() < 30) {
        std::cerr << "Not enough CC in the block to compute robust statistics! Will keep the block in its current borders" << std::endl;
        return;
    }

    //  // NEW
#ifdef TIMING
    Timer t("CC stats computation");
#endif
    Histogram1D ccWidthsHisto;
    Histogram1D ccHeightsHisto;
    Histogram1D spacingsCCHorizHisto;
    Histogram1D spacingsCCVertiHisto;
    computeConnectedComponentsStats(connectedComponentsInTheBlock, ccWidthsHisto, ccHeightsHisto, spacingsCCHorizHisto,
                                    spacingsCCVertiHisto);
#ifdef TIMING
    t.printElapsed(std::cout);
#endif
    ise::Histogram1D spacingsWordsHorizHisto;
    ise::Histogram1D spacingsWordsVertiHisto;
    computeWordsStats(wordsInTheBlock, spacingsWordsHorizHisto, spacingsWordsVertiHisto);


#ifdef VISU
    {
        std::vector<ise::Histogram1D> histos;
        std::vector<std::string> titles;
        histos.push_back(ccWidthsHisto);
        titles.push_back("Histogram of2 CC widths");
        histos.push_back(ccHeightsHisto);
        titles.push_back("Histogram of CC heights");
        histos.push_back(spacingsCCHorizHisto);
        titles.push_back("Histogram of CC horiz distance");
        histos.push_back(spacingsCCVertiHisto);
        titles.push_back("Histogram of CC vertical distances");
        histos.push_back(spacingsWordsHorizHisto);
        titles.push_back("Histogram of words distances");
        g_info.setTo(0);
        writeHisto(g_info, histos, titles);
        std::vector<std::string> msg;
        displayVisu(msg);
    }
#endif

    // Step 3: Whitespace rectangles


    {
        std::cout << "Computing whitespace"<< std::endl;
        //const size_t maxNbRect = 100; // (unused)
        size_t minWidthLow   = spacingsWordsHorizHisto.getKeyOfMaxFrequency();
        minWidthLow = (minWidthLow == 0) ? ccWidthsHisto.getMean() : minWidthLow;
        size_t minWidthHigh  = 2*minWidthLow + ccWidthsHisto.getMean();
        size_t minHeightLow  = spacingsCCVertiHisto.getKeyOfMaxFrequency();
        minHeightLow = (minHeightLow == 0) ? ccHeightsHisto.getMean() : minHeightLow;
        size_t minHeightHigh = 2*minHeightLow + ccHeightsHisto.getMean();
        //size_t minArea = std::min(minWidthLow*minHeightHigh, minWidthHigh*minHeightLow);

        minWidthLow = minWidthHigh = spacingsWordsHorizHisto.getKeyOfMaxFrequency() + ccWidthsHisto.getMean();
        minHeightLow = minHeightHigh = spacingsCCVertiHisto.getKeyOfMaxFrequency() + ccHeightsHisto.getMean();
        std::cerr << "minWidthLow=" << minWidthLow << " minWidthHigh=" << minWidthHigh << " minHeightLow=" << minHeightLow << " minHeightHigh=" << minHeightHigh << std::endl;
        size_t minArea = std::min(minWidthLow*minHeightHigh, minWidthHigh*minHeightLow);


        //NEW
        girl::regionDeque createdRegs;
        computeWhitespace(r, createdRegs, wordsInTheBlock, minWidthLow, minHeightLow, minArea);

#ifdef VISU
        {
            std::vector<cv::Mat> planes;
            cv::split(*m_segContext2.getImage()->cvMat(), planes);//convertTo(g_orig, CV_8UC3);
            std::vector<cv::Mat> planes2(3);
            for (size_t i = 0; i < 3; ++i) {
                planes[i].convertTo(planes2[2 - i], CV_8UC1);
            }
            cv::merge(planes2, g_orig);
            if (createdRegs.size() > 1) {
                girl::tools::paintRegionBorders(g_orig, m_toger, createdRegs, CV_RGB(255, 0, 0), 3);
                std::vector<std::string> msg;
                msg.push_back("Whitespaces");
                std::stringstream ss(std::ios::in|std::ios::out);
                ss << "minWidth=" << minWidthLow;
                msg.push_back(std::string(ss.str()));
                std::stringstream ss2(std::ios::in|std::ios::out);
                ss2 << "minHeight=" << minHeightLow;
                msg.push_back(std::string(ss2.str()));
                displayVisu(msg);
            } else {
                std::vector<std::string> msg;
                msg.push_back("No WS in the block");
                std::stringstream ss(std::ios::in|std::ios::out);
                ss << "minWidth=" << minWidthLow;
                msg.push_back(std::string(ss.str()));
                std::stringstream ss2(std::ios::in|std::ios::out);
                ss2 << "minHeight=" << minHeightLow;
                msg.push_back(std::string(ss2.str()));
                displayVisu(msg);
            }
        }
#endif
        if (createdRegs.size() > 1) {
            for (girl::regionDeque::const_iterator it = createdRegs.begin();
                 it != createdRegs.end(); ++it) {
                regsToProcess.push_back(*it);
            }
            return;
        }

    }

    // Step 4: Split with profiles
    {

        // do not compute profile for isolated lines
        if (spacingsCCHorizHisto.getMean() == 0 || spacingsCCVertiHisto.getMean() == 0) {
#ifdef VISU
            {
                girl::regionDeque regs;
                regs.push_back(r);
                std::vector<cv::Mat> planes;
                cv::split(*m_segContext2.getImage()->cvMat(), planes);//convertTo(g_orig, CV_8UC3);
                std::vector<cv::Mat> planes2(3);
                for (size_t i = 0; i < 3; ++i) {
                    planes[i].convertTo(planes2[2 - i], CV_8UC1);
                }
                cv::merge(planes2, g_orig);
                girl::tools::paintRegionBorders(g_orig, m_toger, regs, CV_RGB(255, 0, 0), 3);
                std::vector<std::string> msg;
                msg.push_back("Not computing profiles in this block because no spaings horiz or verti CC");
                displayVisu(msg);
            }
#endif
            std::cerr<<"spacingsHorizHisto.getMean()="<<spacingsCCHorizHisto.getMean()<<" or spacingsVertiHisto.getMean()="<<spacingsCCVertiHisto.getMean()<<" is null  => return !"<<std::endl;
            return;
        }

        std::cout << "Computing profiles in region " << r  << std::endl;

        //TODO: detect text orientation to adjust horiz and vertical thresholds
        // remove profile that are tiniest than N*average width of a CC in the block
        size_t erodeHoriz  = 0.5*ccWidthsHisto.getMean();//0.5*hHisto.getMean();
        // connect profile that are are a distance of less than N*estimated inter column distances, that is max of word distances
        size_t dilateHoriz = 1.0*spacingsWordsHorizHisto.getKeyOfMaxFrequency();//1.6*spacingsHorizHisto.getMean(); //spacingsHorizHisto.getMedian(); //B: Mean often greater than Median

        // QuantificationThresholderDomenger jp;
        //  std::vector<double> thresholds;
        //  jp.getThresholds(spacingsCCHorizHisto, thresholds);
        //  dilateHoriz = static_cast<size_t>(thresholds[0]+0.5);

        size_t erodeVerti  = 0.5*ccHeightsHisto.getMean(); //0.5*wHisto.getMean();
        size_t dilateVerti = 1.1*spacingsCCVertiHisto.getKeyOfMaxFrequency();//1.1*spacingsWordsVertiHisto.getKeyOfMaxFrequency(); //
        if (erodeHoriz%2==0)
            erodeHoriz++;
        if (erodeVerti%2==0)
            erodeVerti++;
        if (dilateHoriz%2==0)
            dilateHoriz++;
        if (dilateVerti%2==0)
            dilateVerti++;

        //NEW

        girl::regionDeque createdRegs;
        computeProfile(r, createdRegs, erodeHoriz, dilateHoriz, erodeVerti,dilateVerti);
#ifdef VISU
        {
            std::vector<cv::Mat> planes;
            cv::split(*m_segContext2.getImage()->cvMat(), planes);//convertTo(g_orig, CV_8UC3);
            std::vector<cv::Mat> planes2(3);
            for (size_t i = 0; i < 3; ++i) {
                planes[i].convertTo(planes2[2 - i], CV_8UC1);
            }
            cv::merge(planes2, g_orig);
            if (createdRegs.size() > 1) {
                girl::tools::paintRegionBorders(g_orig, m_toger, createdRegs, CV_RGB(255, 0, 0), 3);
                std::vector<std::string> msg;
                msg.push_back("Profiles");
                char buf[1024];
                sprintf(buf, "%s%li", "erode horiz", erodeHoriz);
                msg.push_back(buf);
                sprintf(buf, "%s%li", "dilate horiz", dilateHoriz);
                msg.push_back(buf);
                sprintf(buf, "%s%li", "erode verti", erodeVerti);
                msg.push_back(buf);
                sprintf(buf, "%s%li", "dilate verti", dilateVerti);
                msg.push_back(buf);
                displayVisu(msg);
            } else {
                std::vector<std::string> msg;
                msg.push_back("No profiles in the block");
                msg.push_back("Profiles");
                char buf[1024];
                sprintf(buf, "%s%li", "erode horiz", erodeHoriz);
                msg.push_back(buf);
                sprintf(buf, "%s%li", "dilate horiz", dilateHoriz);
                msg.push_back(buf);
                sprintf(buf, "%s%li", "erode verti", erodeVerti);
                msg.push_back(buf);
                sprintf(buf, "%s%li", "dilate verti", dilateVerti);
                msg.push_back(buf);
                displayVisu(msg);
            }
        }
#endif
        if (createdRegs.size() > 1) {
            for (girl::regionDeque::const_iterator it = createdRegs.begin();
                 it != createdRegs.end(); ++it) {
                regsToProcess.push_back(*it);
            }
            return;
        }
    }

#if 1

    // Step 5: Paragraph split
    {


        int threshHoriz = spacingsWordsHorizHisto.getKeyOfMaxFrequency();
        if (threshHoriz == 0)
            threshHoriz = 2*spacingsCCHorizHisto.getKeyOfMaxFrequency();
        int threshVerti = ccHeightsHisto.getMean() + spacingsCCVertiHisto.getKeyOfMaxFrequency();
        int characterWidth = static_cast<int>(ccWidthsHisto.getMean() + 0.5);
        int characterHeight = static_cast<int>(ccHeightsHisto.getMean() + 0.5);
        int medianCCHorizSpacing = static_cast<int>(spacingsCCHorizHisto.getMedian());
        int averageWordSpacing = static_cast<int>(spacingsWordsHorizHisto.getMean() + 0.5);
        assert(medianCCHorizSpacing <= averageWordSpacing);
        girl::regionDeque createdRegs;
        computeParagraphSplit(m_segContext2,
                              r,
                              wordsInTheBlock,
                              createdRegs,
                              threshHoriz,
                              threshVerti,
                              characterWidth,
                              characterHeight,
                              medianCCHorizSpacing,
                              averageWordSpacing);
#ifdef VISU
        {
            std::vector<cv::Mat> planes;
            cv::split(*m_segContext2.getImage()->cvMat(), planes);//convertTo(g_orig, CV_8UC3);
            std::vector<cv::Mat> planes2(3);
            for (size_t i = 0; i < 3; ++i) {
                planes[i].convertTo(planes2[2 - i], CV_8UC1);
            }
            cv::merge(planes2, g_orig);
            girl::tools::paintRegionBorders(g_orig, m_toger, createdRegs, CV_RGB(255, 0, 0), 3);
            std::vector<std::string> msg;
            msg.push_back("Text paragraphs found");
            displayVisu(msg);
        }
#endif
        // comment out to not further process regions given by paragraph split
        // if (createdRegs.size() > 1) {
        //   for (girl::regionDeque::const_iterator it = createdRegs.begin();
        //        it != createdRegs.end(); ++it) {
        //     regs.push_back(*it);
        //   }
        //   return;
        // }

    }
#endif
}

void
SegmentationClusteringDocumentPreseg5::binarize(girl::region r, ise::SegmentationContext &segContext, cv::Mat &binaryImage)
{
    girl::tools::Image *image = segContext.getImage();
    int imageType = detectImageType();

    if (imageType == GRAYLEVEL_IMAGE || imageType == COLOR_IMAGE) {

        // Opencv OTSU
        cv::Mat gray, gray2;
        int code = -1;
        if (image->cvMat()->channels() == 4)
	  code = cv::COLOR_RGBA2GRAY;
        else if (image->cvMat()->channels() == 3)
	  code = cv::COLOR_RGB2GRAY; // girl::tools::Image is always RGB, even when read from opencv, conversion from bgr is done in girl::tools::Image constructor
        assert(code != -1);
        cv::cvtColor(*image->cvMat(), gray, code);
        cv::convertScaleAbs(gray, gray2);
        cv::threshold(gray2, binaryImage, 128, 255, cv::THRESH_BINARY_INV|cv::THRESH_OTSU);
    } else {
        binaryImage = cv::Mat::zeros(image->height(), image->width(), CV_8U);
        std::vector<cv::Mat> channels;
        cv::split(*image->cvMat(), channels);
        cv::convertScaleAbs(channels[0], binaryImage);
        binaryImage = ~binaryImage;
    }
    girl::regionDeque regs;
    assert(binaryImage.rows == image->height() && binaryImage.cols == image->width());
    bitmapSplit(segContext, r, binaryImage, regs);


}

void
SegmentationClusteringDocumentPreseg5::detectForegroundBackgroundRegions(const girl::toger &tog,
                                                                         const cv::Mat &binaryMat,
                                                                         girl::regionDeque &fgRegs,
                                                                         girl::regionDeque &bgRegs)
{
    // R: could use ise::ForegroundBackgroundClassifierColorArea but very slow. Assume info is ok in binaryMat
    fgRegs.clear();
    bgRegs.clear();
    girl::regionDeque allRegs;
    tog.allRegions(allRegs);
    girl::regionDeque::const_iterator it = allRegs.begin();
    const girl::regionDeque::const_iterator itEnd = allRegs.end();
    for (; it != itEnd; ++it) {
        girl::point p = tog.anyPoint(*it);
        if (binaryMat.at<unsigned char>(p.y(), p.x()) != 0) {
            fgRegs.push_back(*it);
        } else {
            bgRegs.push_back(*it);
        }
    }
}

void
SegmentationClusteringDocumentPreseg5::removeFgRegionsTouchingBorder(girl::toger &tog, cv::Mat &binaryImage, girl::regionDeque &fgRegs)
{
    girl::regionDeque::const_iterator it = fgRegs.begin();
    const girl::regionDeque::const_iterator itEnd = fgRegs.end();
    girl::regionDeque toRemove;
    for (; it != itEnd; ++it) {
        girl::point p0, p1;
        girl::tools::computeBoundingBox(tog, *it, p0, p1);
        if (p0.x() == 0 || p0.y() == 0 || p0.x() == binaryImage.cols - 1 || p0.y() == binaryImage.rows - 1 ||
                p1.x() == 0 || p1.y() == 0 || p1.x() == binaryImage.cols - 1 || p1.y() == binaryImage.rows - 1) {
            // region touches border
            toRemove.push_back(*it);
            girl::tools::Painter<unsigned char> p(binaryImage, CV_RGB(0, 0, 0));
            tog.iterateOnRegionDomain(*it, p);
        }
    }

    // R: TODO: do the merge in toger

}


void
SegmentationClusteringDocumentPreseg5::computeEnclosingShapeToger(const cv::Mat &binaryImage,
                                                                  SegmentationContext &segContextEnclosingShape)
{

    SegmentationClusteringEnclosingShape enclosingShape(&segContextEnclosingShape);
    SegmentationClusteringEnclosingShape::ShapeList shapes;
    SegmentationClusteringEnclosingShape::EnclosingShapeFactoryPtr factory(new EnclosingConvexHullFactory());
    SegmentationClusteringEnclosingShape::MergeMethod method = ise::SegmentationClusteringEnclosingShape::UNION;
    // IS cloning necessary? Yes because matrix is beeing edited (CC are set to bacground)
    cv::Mat tmp = binaryImage.clone();
    enclosingShape.extractCC(tmp, shapes, *factory, 8);
    enclosingShape.setParameters(shapes, factory, method);
    girl::regionDeque regs;
    girl::regionDeque createdRegs;
    segContextEnclosingShape.getToger()->allRegions(regs);
    SplitAlgorithmDefault algoEnclosingShapes(&segContextEnclosingShape);
    SplitDriverDefault driverEnclosingShapes(&segContextEnclosingShape);
    algoEnclosingShapes.apply(regs, driverEnclosingShapes, enclosingShape, 0, createdRegs);
}


void
SegmentationClusteringDocumentPreseg5::detectIllustrations(ise::SegmentationContext &segContextEnclosingShape,
                                                           const cv::Mat &ccMat,
                                                           girl::regionDeque &illustrationRegs)
{

  assert(ccMat.type() == CV_32S);
  //TODO:BUG
  // PATH is the first one from qise, the second one from ICDAR2011Segmentation executable...
  const char* filename = CC_ILLUSTRATION_CLASSIFIER;
    //"/home/vieux/workspace/imageFeatures/qise/build/src/plugins/RegionSetInterface/boostICDAR2009Text-NonText";
  //const char* filename = "boostICDAR2009Text-NonText";
  //  const char* filename = "/home/domenger/workspace/imageFeatures/qise/build/src/plugins/RegionSetInterface/boostICDAR2009Text-NonText";
  ise::RegionSetInterfaceTextBlockClassifier dzc(&segContextEnclosingShape, filename);
  std::vector<ise::RegionFeature_Data_Type> buffer(dzc.getFeatureDim(), 0);
  ise::RegionFeature rf(&buffer[0]);
  const size_t IMAGE_INDEX = ise::RegionSetInterfaceTextBlockClassifier::TEXT;
  assert(IMAGE_INDEX < dzc.getFeatureDim());
  //m_imageMask = cv::Mat::zeros(height, width, CV_8U);
  girl::regionDeque regs;
  segContextEnclosingShape.getToger()->allRegions(regs);
  std::vector<cv::Mat> mats;
  mats.push_back(ccMat);
  CCCounterPreseg5 counter(mats);
  for (girl::regionDeque::const_iterator it = regs.begin();
       it != regs.end(); ++it) {
    //    CCCounter counter(ccMat);
    segContextEnclosingShape.getToger()->iterateOnRegionDomain(*it, counter);
    if (counter.numCC(0) > 3) {
      dzc.getRegionFeatures(*it, rf);
      if (rf[IMAGE_INDEX] == 1) {
        illustrationRegs.push_back(*it);
//        Painter p(m_imageMask, CV_RGB(IMAGE_REGION_INDEX, IMAGE_REGION_INDEX, IMAGE_REGION_INDEX));
//        m_enclosingShapeToger.iterateOnRegionDomain(*it, p);
//        const girl::unorderedSet<int>& insideregs = counter.getFoundRegs();
//        for (girl::unorderedSet<int>::const_iterator i = insideregs.begin();
//             i != insideregs.end(); ++i) {
//          regsToRemove.insert(*i);
//        }
      }
    }
  }
}


void
SegmentationClusteringDocumentPreseg5::removeSmallRegions(ise::SegmentationContext &segContext,
                                                          const girl::regionDeque &fgRegs,
                                                          cv::Mat &binaryMat)
{
    girl::toger *tog = segContext.getToger();
    girl::regionDeque::const_iterator it = fgRegs.begin();
    const girl::regionDeque::const_iterator itEnd = fgRegs.end();
    girl::hashMap<girl::region, double> areas;
    double m0 = 0, m1 = 0, m2 = 0;
    for (; it != itEnd; ++it) {
        double val = girl::tools::computeArea(*tog, *it);
        areas[*it] = val;
        m0++;
        m1 += (double) val;
        m2 += (double) (val * val);
    }
    const double var = (m2 - m1*m1/m0)/m0;
    const double std = sqrt(var);
    const double threshold = 2.0 * sqrt(std);

    girl::tools::Painter<unsigned char> p(binaryMat, CV_RGB(0, 0, 0));
    for (girl::hashMap<girl::region, double>::const_iterator it = areas.begin(); it != areas.end(); ++it) {
        if (it->second < threshold) {
            tog->iterateOnRegionDomain(it->first, p);
        }
    }

    girl::regionDeque regs = fgRegs;
    ise::mergeSmallRegions(regs, *tog, threshold);
}

void
SegmentationClusteringDocumentPreseg5::extendBlackSeparatorsAndFillMask(ise::SegmentationContext &segContext,
                                                                        const girl::regionDeque &fgRegs,
                                                                        cv::Mat &binaryMat,
                                                                        cv::Mat &separatorMask,
                                                                        girl::regionDeque &separatorRegs)
{




#if 1 // use separator detection based on ratio
    //  m_binarizationMask = cv::Mat::zeros(mat.size(), mat.type());
    //  m_separatorsMask   = cv::Mat::zeros(mat.size(), mat.type());
    girl::toger *tog = segContext.getToger();
    // save point inside each separator to find region index after white space extension
    std::vector<girl::point> separatorPoints;
    ise::RegionSetInterfaceBoundingBox rsiBB(&segContext);
    std::vector<RegionFeature_Data_Type> rsiBuffer(rsiBB.getFeatureDim(), 0);
    RegionFeature rsiRf(&rsiBuffer[0]);
    const size_t RATIO_INDEX = ise::RegionSetInterfaceBoundingBox::RATIO_MAX_MIN;
    for (girl::regionDeque::const_iterator it = fgRegs.begin();
         it != fgRegs.end() ; ++it) {
        assert(tog->isRegionValid(*it));
        rsiBB.getRegionFeatures(*it, rsiRf);
        if (rsiRf[RATIO_INDEX] > SEPARATOR_RATIO_THRESHOLD) {
            separatorPoints.push_back(tog->anyPoint(*it));
            separatorRegs.push_back(*it);
            girl::tools::Painter<unsigned char> p(binaryMat, CV_RGB(0, 0, 0));
            tog->iterateOnRegionDomain(*it, p);
            girl::tools::Painter<unsigned char> p2(separatorMask, CV_RGB(SEPARATOR_REGION_INDEX, SEPARATOR_REGION_INDEX, SEPARATOR_REGION_INDEX));
            tog->iterateOnRegionDomain(*it, p2);
        }
    }


#else
    // USE Line detection code
    girl::toger *tog = segContext.getToger();
    std::vector<girl::point> separatorPoints;
    int width = 0;
    int height = 0;
    girl::regionDeque::const_iterator it = fgRegs.begin();
    const girl::regionDeque::const_iterator itEnd = fgRegs.end();
    for (; it != itEnd; ++it) {
        assert(tog->isRegionValid(*it));
        girl::tools::AABBox box = girl::tools::getAABBox(*tog, *it);
        width += box.width();
        height += box.height();
    }

    width /= fgRegs.size();
    height /= fgRegs.size();
    // const int minLineSize = std::min(width, height);
    DSCC dscc(2*width, 2*height);
    std::list<DSCC> horiz;
    std::list<DSCC> verti;
    dscc.init(binaryMat, horiz, verti);
    girl::unorderedSet<girl::region> singleRegs;

    for (int i = 0; i < horiz.size(); i++){
        DSCC ch = horiz.front();
        horiz.pop_front();
        short ori = ch.getOrientation(ch);
        DSCC_c dscc_ch = ch.getDSCC_c(ch);
        int size_dscc = dscc_ch.size();
        for (int j = 0; j < size_dscc; j++)
        {
            ise::BitmapRunLength::RunLength r = dscc_ch.front();
            dscc_ch.pop_front();
            girl::point pt1(r.m_start.x(), r.m_start.y());
            separatorPoints.push_back(pt1);
            girl::region reg = tog->findRegion(pt1);
            singleRegs.insert(reg);
            girl::tools::Painter<unsigned char> p(binaryMat, CV_RGB(0, 0, 0));
            tog->iterateOnRegionDomain(reg, p);
            girl::tools::Painter<unsigned char> p2(separatorMask, CV_RGB(SEPARATOR_REGION_INDEX, SEPARATOR_REGION_INDEX, SEPARATOR_REGION_INDEX));
            tog->iterateOnRegionDomain(reg, p2);
        }
    }
    for (int i = 0; i < verti.size(); i++){
        DSCC ch = verti.front();
        verti.pop_front();
        short ori = ch.getOrientation(ch);
        DSCC_c dscc_ch = ch.getDSCC_c(ch);
        int size_dscc = dscc_ch.size();
        for (int j = 0; j < size_dscc; j++)
        {
            ise::BitmapRunLength::RunLength r = dscc_ch.front();
            dscc_ch.pop_front();
            girl::point pt1(r.m_start.x(), r.m_start.y());
            separatorPoints.push_back(pt1);
            girl::region reg = tog->findRegion(pt1);
            singleRegs.insert(reg);
            girl::tools::Painter<unsigned char> p(binaryMat, CV_RGB(0, 0, 0));
            tog->iterateOnRegionDomain(reg, p);
            girl::tools::Painter<unsigned char> p2(separatorMask, CV_RGB(SEPARATOR_REGION_INDEX, SEPARATOR_REGION_INDEX, SEPARATOR_REGION_INDEX));
            tog->iterateOnRegionDomain(reg, p2);
        }
    }
    separatorRegs.clear();
    for (girl::unorderedSet<girl::region>::const_iterator it = singleRegs.begin();
         it != singleRegs.end(); ++it){
      separatorRegs.push_back(*it);
    }

#if 1
    cv::namedWindow("Lines", 0);
    cv::imshow("Lines", separatorMask);
    cv::waitKey(0);
#endif

#endif

    ise::SegmentationClusteringWhitespaceSeparator whiteSepExt(&segContext);
    ise::MergeDriverMinimalDistance mergeDriver(&segContext);
    girl::shared_ptr<RegionBinaryFunction> distFunc(new RegionBinaryFunctionConstantNull(&segContext));
    mergeDriver.setDistanceFunction(distFunc);
    SplitDriverDefault splitDriver(&segContext);
    girl::shared_ptr<ise::SplitAlgorithmDefault> splitAlgo(new ise::SplitAlgorithmDefault(&segContext));
    girl::shared_ptr<ise::MergeAlgorithmDefault> mergeAlgo(new ise::MergeAlgorithmDefault(&segContext));
    ise::SplitMergeAlgoExtendSelection smAlgo(&segContext);
    smAlgo.setSplitAlgorithm(splitAlgo);
    smAlgo.setMergeAlgorithm(mergeAlgo);
    girl::regionDeque createdRegions;
    smAlgo.apply(separatorRegs, mergeDriver, splitDriver, whiteSepExt, 0, createdRegions);

    separatorRegs.clear();
    for(std::vector<girl::point>::const_iterator it = separatorPoints.begin();
        it != separatorPoints.end(); ++it) {
        girl::region r = tog->findRegion(*it);
        separatorRegs.push_back(r);
    }

}



void
SegmentationClusteringDocumentPreseg5::preprocessing(girl::region r)
{

    assert(m_segContext);
    girl::tools::Image *image = m_segContext2.getImage();
    assert(image);
    size_t width = image->width();
    size_t height = image->height();
    girl::toger tmpTog(width, height);
    ise::SegmentationContext segContextTmp(&tmpTog, image);


    cv::Mat binaryImage;
    binarize(r, segContextTmp, binaryImage);

#ifdef VISU
    {
        int fromTo[6] = {0, 0, 0, 1, 0, 2};
        cv::mixChannels(&binaryImage, 1, &g_orig, 1, fromTo, 3);
        std::vector<std::string> msg;
        msg.push_back("Start binarization");
        displayVisu(msg);
    }
#endif
    girl::regionDeque fgRegs;
    girl::regionDeque bgRegs;
    detectForegroundBackgroundRegions(tmpTog, binaryImage, fgRegs, bgRegs);

    removeFgRegionsTouchingBorder(tmpTog, binaryImage, fgRegs);
    detectForegroundBackgroundRegions(tmpTog, binaryImage, fgRegs, bgRegs);
    binaryImage.copyTo(m_originalBinarization);


    // create a temporary label image for fgRegs;
    cv::Mat ccMat = cv::Mat::zeros(height, width, CV_32S);
    for (girl::regionDeque::const_iterator it = fgRegs.begin(); it != fgRegs.end(); ++it) {
        girl::tools::Painter<int> p(ccMat, CV_RGB(*it, *it, *it));
        tmpTog.iterateOnRegionDomain(*it, p);
    }

    // end of step 1, tmpTog contains all Otsu binarized regions. m_binarizationFgRegs the FG regions
    // based on global statistics


    // step 2: Non-Text region detection

    m_enclosingShapeToger = girl::toger(width, height);
    SegmentationContext segContextEnclosingShape(&m_enclosingShapeToger, m_segContext2.getImage());
    computeEnclosingShapeToger(binaryImage, segContextEnclosingShape);



    detectIllustrations(segContextEnclosingShape, ccMat, m_imageRegions);

#ifdef VISU
    {

        std::vector<cv::Mat> planes;
        cv::split(*m_segContext2.getImage()->cvMat(), planes);
        std::vector<cv::Mat> planes2(3);
        for (size_t i = 0; i < 3; ++i) {
            planes[i].convertTo(planes2[2 - i], CV_8UC1);
        }
        cv::merge(planes2, g_orig);
        cv::Scalar colorIllustration = CV_RGB(255, 0, 0);
        // girl::tools::paintRegionBorders(visu, m_enclosingShapeToger, m_imageRegions, colorIllustration, 4);
        // std::cerr << "Detected illustrations" << std::endl;
        // cv::waitKey(0);
        // visu.setTo(0);
        //    for (girl::regionDeque::const_iterator it = m_imageRegions.begin();
        //         it != m_imageRegions.end(); ++it) {
        //    }
        girl::tools::paintRegionBorders(g_orig, m_enclosingShapeToger, m_imageRegions, colorIllustration, 4);
        std::vector<std::string> msg;
        msg.push_back("Non-text regions");
        displayVisu(msg);
    }
#endif

    // Remove connected components belonging to illustrations
    for (girl::regionDeque::const_iterator it = m_imageRegions.begin();
         it != m_imageRegions.end(); ++it) {
        girl::tools::Painter<unsigned char> p(binaryImage, CV_RGB(0, 0, 0));
        m_enclosingShapeToger.iterateOnRegionDomain(*it, p);
    }

    detectForegroundBackgroundRegions(tmpTog, binaryImage, fgRegs, bgRegs);

    removeSmallRegions(segContextTmp, fgRegs, binaryImage);
#ifdef VISU
    //  std::cerr << "After removing small regions binarization" << std::endl;
    // // cv::imshow(wname, binaryImage);
    //  cv::waitKey(0);
#endif


    detectForegroundBackgroundRegions(tmpTog, binaryImage, fgRegs, bgRegs);

#ifdef VISU
    {
        int fromTo[6] = {0, 0, 0, 1, 0, 2};
        cv::mixChannels(&binaryImage, 1, &g_orig, 1, fromTo, 3);
        std::vector<std::string> msg;
        msg.push_back("Clean binarization");
        msg.push_back("Removed small regs");
        msg.push_back("Removed regs touching borders");
        msg.push_back("Removed illustrations");
        displayVisu(msg);
    }
#endif







    girl::regionDeque separatorRegs;
    m_separatorsMask = cv::Mat::zeros(height, width, CV_8U);
    extendBlackSeparatorsAndFillMask(segContextTmp, fgRegs, binaryImage, m_separatorsMask, separatorRegs);
    detectForegroundBackgroundRegions(tmpTog, binaryImage, fgRegs, bgRegs);

    // assign members variables with computed informations
    m_binarizationMask = binaryImage;
    //  m_binarizationFgRegions = fgRegs;
    //  m_binarizationToger = tmpTog;
    //  m_ccLabels  = cv::Mat::zeros(height, width, CV_32S);
    //  m_imageMask = cv::Mat::zeros(height, width, CV_8U);
    //  for (girl::regionDeque::const_iterator it = fgRegs.begin();
    //       it != fgRegs.end(); ++it) {
    //    girl::tools::Painter<int> p(m_ccLabels, CV_RGB(*it, *it, *it));
    //    tmpTog.iterateOnRegionDomain(*it, p);
    //  }


    computeStatsConnectedComponents();
    computeWord();
    computeStatsWords();

    // avoid classifier errors
    girl::regionDeque trueIllustrations;
    correctClassificationErrors(trueIllustrations);
#ifdef VISU
    {
        int fromTo[6] = {0, 0, 0, 1, 0, 2};
        cv::mixChannels(&binaryImage, 1, &g_orig, 1, fromTo, 3);
        std::vector<std::string> msg;
        msg.push_back("Restaured false illustrations");
        displayVisu(msg);
    }
#endif

    m_binarizationToger = girl::toger(m_binarizationMask.cols, m_binarizationMask.rows);
    SegmentationContext segContextCC(&m_binarizationToger, NULL);
    girl::regionDeque createdRegs;
    bitmapSplit(segContextCC, 1, m_binarizationMask, createdRegs);
    detectForegroundBackgroundRegions(m_binarizationToger, m_binarizationMask, m_binarizationFgRegions, bgRegs);
    computeStatsConnectedComponents();
    computeWord();
    computeStatsWords();
    m_imageRegions = trueIllustrations;

    cv::Mat mat = cv::Mat::zeros(height, width, CV_8U);
    for (girl::regionDeque::const_iterator it = m_imageRegions.begin();
         it != m_imageRegions.end(); ++it) {
        girl::tools::Painter<unsigned char> p(mat, CV_RGB(IMAGE_REGION_INDEX, IMAGE_REGION_INDEX, IMAGE_REGION_INDEX));
        girl::tools::Painter<unsigned char> p2(m_originalBinarization, CV_RGB(0, 0, 0));
        m_enclosingShapeToger.iterateOnRegionDomain(*it, p);
        m_enclosingShapeToger.iterateOnRegionDomain(*it, p2);
    }
    // copy mask of illustrations
    mat.copyTo(m_imageMask);

    for (girl::regionDeque::const_iterator it = separatorRegs.begin();
         it != separatorRegs.end(); ++it) {
        girl::tools::Painter<unsigned char> p(mat, CV_RGB(SEPARATOR_REGION_INDEX, SEPARATOR_REGION_INDEX, SEPARATOR_REGION_INDEX));
        girl::tools::Painter<unsigned char> p2(m_originalBinarization, CV_RGB(0, 0, 0));
        tmpTog.iterateOnRegionDomain(*it, p);
        tmpTog.iterateOnRegionDomain(*it, p2);
    }

    girl::regionDeque regs;
    bitmapSplit(m_segContext2, r, mat, regs);

#ifdef VISU
    {
        std::vector<cv::Mat> planes;
        cv::split(*m_segContext2.getImage()->cvMat(), planes);
        std::vector<cv::Mat> planes2(3);
        for (size_t i = 0; i < 3; ++i) {
            planes[i].convertTo(planes2[2 - i], CV_8UC1);
        }
        cv::merge(planes2, g_orig);
        cv::Scalar colorIllustration = CV_RGB(255, 0, 0);
        cv::Scalar colorSeparators = CV_RGB(0, 255, 0);
        girl::tools::paintRegionBorders(g_orig, m_enclosingShapeToger, m_imageRegions, colorIllustration, 4);
        girl::tools::paintRegionBorders(g_orig, tmpTog, separatorRegs, colorSeparators, 4);
        girl::regionDeque regs;
        m_segContext2.getToger()->allRegions(regs);
        girl::tools::paintRegionBorders(g_orig, *m_segContext2.getToger(), regs, CV_RGB(0, 0, 255), 2);
        std::vector<std::string> msg;
        msg.push_back("Non-Text Regions (red)");
        msg.push_back("Separators (green)");
        msg.push_back("All regions (blue)");
        msg.push_back("(end of preprocessing)");
        displayVisu(msg);
    }
#endif


    //  detectForegroundBackgroundRegions(m_binarizationToger, m_binarizationMask, m_binarizationFgRegions, bgRegs);

}

void
SegmentationClusteringDocumentPreseg5::correctClassificationErrors(girl::regionDeque &trueIllustrations)
{
    trueIllustrations.clear();
    // start at begin+1 because m_wordCCAreas is the area of the background region
    std::vector<size_t>::iterator maxWordArea = std::max_element(m_wordCcAreas.begin()+1, m_wordCcAreas.end());
    assert(maxWordArea != m_wordCcAreas.end());
    size_t maxArea = *maxWordArea;//*(std::max_element(m_wordCcAreas.begin(), m_wordCcAreas.end()));
    girl::tools::Painter<unsigned char> p(m_binarizationMask, CV_RGB(255, 255, 255));
    for (girl::regionDeque::const_iterator it = m_imageRegions.begin();
         it != m_imageRegions.end(); ++it) {
        size_t area = girl::tools::computeArea(m_enclosingShapeToger, *it);
        if (area < maxArea) {
            // false illustration
            girl::tools::AABBox box = girl::tools::getAABBox(m_enclosingShapeToger, *it);
            float min = static_cast<float>(std::min(box.width(), box.height()));
            float max = static_cast<float>(std::max(box.width(), box.height()));
            if (max/min > SEPARATOR_RATIO_THRESHOLD) {
                // this region is not a illustration but is a separator
                // what to do exactly?
            } else {
                // not an illustration or separator, refeed as text
                cv::Rect roi(box.xMin(), box.yMin(), box.width(), box.height());
                cv::Mat m1 = m_binarizationMask(roi);
                cv::Mat m2 = m_originalBinarization(roi);
                m1 += m2;
            }
        }
        else {
            trueIllustrations.push_back(*it);
        }
    }
}

void
SegmentationClusteringDocumentPreseg5::bitmapSplit(ise::SegmentationContext &segContext,
                                                   const girl::region r,
                                                   const cv::Mat &bitmap,
                                                   girl::regionDeque &createdRegs)
{
    SegmentationClusteringBitmap clustBitmap(&segContext);
    clustBitmap.setParameters(bitmap);
    SplitDriverDefault splitDriver(&segContext);
    SplitAlgorithmDefault splitAlgo(&segContext);
    girl::regionDeque regs2split;
    regs2split.push_back(r);
    splitAlgo.apply(regs2split, splitDriver, clustBitmap, 1, createdRegs);

}


girl::region
SegmentationClusteringDocumentPreseg5::mergeIncorrectIllustrations(girl::region r,
                                                                   const girl::unorderedSet<int>& includedWords)
{
    size_t maxWordArea = std::numeric_limits<size_t>::min();
    for (girl::unorderedSet<int>::const_iterator it = includedWords.begin();
         it != includedWords.end(); ++it) {
        size_t area = m_wordCcAreas[*it];//girl::tools::computeArea(m_wordToger, *it);
        if (area > maxWordArea)
            maxWordArea = area;
    }
    girl::regionDeque possibleIncorrectIllustrations;
    m_segContext2.getToger()->regionNeighbourhood(r, girl::toger::NEIGHBOUR_INNER|girl::toger::NEIGHBOUR_DIRECT, girl::toger::MARK_NONE, possibleIncorrectIllustrations);
    girl::regionDeque incorrectIllustrations;
    for (girl::regionDeque::const_iterator it = possibleIncorrectIllustrations.begin();
         it != possibleIncorrectIllustrations.end(); ++it) {
        if (girl::tools::computeArea(*m_segContext2.getToger(), *it) < maxWordArea) {
            incorrectIllustrations.push_back(*it);
        }
    }

    girl::region blockRegion = r;
    for (girl::regionDeque::const_iterator it = incorrectIllustrations.begin();
         it != incorrectIllustrations.end(); ++it) {
        // paint region in the mask of cc and erase it from image list
        girl::point pt = m_segContext2.getToger()->anyPoint(*it);
        girl::region reg = m_enclosingShapeToger.findRegion(pt);
        girl::regionDeque::iterator regPos = std::find(m_imageRegions.begin(), m_imageRegions.end(), reg);
        // assert(regPos != m_imageRegions.end());
        // region might possibly be a direct neighboor which is not an illustration (e.g. black separator)
        if (regPos != m_imageRegions.end()) {
            girl::tools::Painter<unsigned char> p(m_binarizationMask, CV_RGB(255, 255, 255));
            girl::tools::Painter<unsigned char> p2(m_imageMask, CV_RGB(0, 0, 0));
            m_segContext2.getToger()->iterateOnRegionDomain(*it, p);
            m_segContext2.getToger()->iterateOnRegionDomain(*it, p2);
            m_imageRegions.erase(regPos);
            blockRegion = m_segContext2.getToger()->mergeRegions(blockRegion, *it);
            assert(blockRegion != 0);
        }
    }
    return blockRegion;

}


void
SegmentationClusteringDocumentPreseg5::computeParagraphSplit(ise::SegmentationContext &segContext,
                                                             girl::region r,
                                                             const girl::unorderedSet<int>& includedWords,
                                                             girl::regionDeque &createdRegs,
                                                             int threshHoriz,
                                                             int threshVerti,
                                                             int characterWidth,
                                                             int characterHeight,
                                                             int medianCCHorizSpacing,
                                                             int averageWordSpacing)
{
#if 0
    size_t maxWordArea = std::numeric_limits<size_t>::min();
    for (girl::unorderedSet<int>::const_iterator it = includedWords.begin();
         it != includedWords.end(); ++it) {
        size_t area = m_wordCcAreas[*it];//girl::tools::computeArea(m_wordToger, *it);
        if (area > maxWordArea)
            maxWordArea = area;
    }
    girl::regionDeque possibleIncorrectIllustrations;
    m_segContext2.getToger()->regionNeighbourhood(r, girl::toger::NEIGHBOUR_INNER|girl::toger::NEIGHBOUR_DIRECT, girl::toger::MARK_NONE, possibleIncorrectIllustrations);
    girl::regionDeque incorrectIllustrations;
    for (girl::regionDeque::const_iterator it = possibleIncorrectIllustrations.begin();
         it != possibleIncorrectIllustrations.end(); ++it) {
        if (girl::tools::computeArea(*m_segContext2.getToger(), *it) < maxWordArea) {
            incorrectIllustrations.push_back(*it);
        }
    }

    girl::region blockRegion = r;
    for (girl::regionDeque::const_iterator it = incorrectIllustrations.begin();
         it != incorrectIllustrations.end(); ++it) {
        // paint region in the mask of cc and erase it from image list
        girl::point pt = m_segContext2.getToger()->anyPoint(*it);
        girl::region reg = m_enclosingShapeToger.findRegion(pt);
        girl::regionDeque::iterator regPos = std::find(m_imageRegions.begin(), m_imageRegions.end(), reg);
        // assert(regPos != m_imageRegions.end());
        // region might possibly be a direct neighboor which is not an illustration (e.g. black separator)
        if (regPos != m_imageRegions.end()) {
            girl::tools::Painter<unsigned char> p(m_binarizationMask, CV_RGB(255, 255, 255));
            girl::tools::Painter<unsigned char> p2(m_imageMask, CV_RGB(0, 0, 0));
            m_segContext2.getToger()->iterateOnRegionDomain(*it, p);
            m_segContext2.getToger()->iterateOnRegionDomain(*it, p2);
            m_imageRegions.erase(regPos);
            blockRegion = m_segContext2.getToger()->mergeRegions(blockRegion, *it);
            assert(blockRegion != 0);
        }
    }


    girl::regionDeque currentRegs;
    currentRegs.push_back(blockRegion);
#else
    girl::regionDeque currentRegs;
    currentRegs.push_back(r);
#endif
    createdRegs.clear();
    SegmentationClusteringParagraphSplit splitter(&segContext);
    splitter.setParameters(m_binarizationMask/*m_wordMask*/,
                           m_ccLabels,
                           m_originalBinarization,
                           m_wordMask,
                           threshHoriz,
                           threshVerti,
                           characterWidth,
                           characterHeight,
                           medianCCHorizSpacing,
                           averageWordSpacing,
                           m_spacingsHoriz);

    SplitDriverDefault splitDriver(&segContext);
    SplitAlgorithmDefault splitAlgo(&segContext);
    splitAlgo.apply(currentRegs, splitDriver, splitter, 1, createdRegs);


    // merge regions inside paragraphs
    // first verify that the region is a text paragraph
    // then proceed to remove holes if yes
    girl::toger *tog = segContext.getToger();
    girl::unorderedSet<girl::region> removedRegs;
    for (girl::regionDeque::const_iterator it = createdRegs.begin();
         it != createdRegs.end(); ++it) {
        if (removedRegs.find(*it) == removedRegs.end()) {
            std::vector<cv::Mat> theMats;
            theMats.push_back(m_ccLabels);
            theMats.push_back(m_wordCCLabels);
            CCCounterPreseg5 counter(theMats);
            tog->iterateOnRegionDomain(*it, counter);
            if (counter.getFoundRegs(0).size() > 0) {
                // region contains foreground CC
                girl::regionDeque holes;
                tog->regionNeighbourhood(*it, girl::toger::NEIGHBOUR_INNER, girl::toger::MARK_NONE, holes);
                if (holes.size() > 0) {
                    girl::region r1 = *it;
                    for (girl::regionDeque::const_iterator it2 = holes.begin();
                         it2 != holes.end(); ++it2) {
                        r1 = tog->mergeRegions(r1, *it2);
                        removedRegs.insert((r1 == *it) ? *it2 : *it);
                    }
                }
            }
        }
    }
    girl::regionDeque remainingRegs;
    for (girl::regionDeque::const_iterator it = createdRegs.begin();
         it != createdRegs.end(); ++it) {
        if (removedRegs.find(*it) == removedRegs.end())
            remainingRegs.push_back(*it);
    }
    createdRegs = remainingRegs;
}






bool
SegmentationClusteringDocumentPreseg5::mustProcessBlock(const girl::region r,
                                                        const cv::Mat &imageMat,
                                                        const cv::Mat &ccMat) const
{

    MustProcessBlock must(ccMat, imageMat);
    m_toger.iterateOnRegionDomain(r, must);
    return !must.containsImagePixels() && must.containsTextPixels();

}


}
