/*
  Copyright (C) 2005-2011 Remi Vieux, Jean-Philippe Domenger, Boris Mansencal  boris.mansencal@labri.fr

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISESEGMENTATIONCLUSTERINGDOCUMENTPRESEG5_HPP_
#define ISESEGMENTATIONCLUSTERINGDOCUMENTPRESEG5_HPP_

#include <girl/girlToger.hpp>
#include <girl/girlHashFunc.hpp>
#include <girl/girlUnorderedSet.hpp>

#include <girlTools/girlToolsAABBox.hpp>
#include <girlTools/girlToolsImage.hpp>

#include <ise/iseBitmapRunLength.hpp>
#include <ise/iseNeighbouringRegions.hpp>
#include <ise/iseSegmentationClustering.hpp>
#include <ise/iseSegmentationContext.hpp>
#include <ise/iseHistogram1D.hpp>

/*
 *
 */
namespace ise
{

class SegmentationContext;

class GIRL_API SegmentationClusteringDocumentPreseg5 : public SegmentationClustering
{

public:

  static const size_t DEFAULT_AREA_THRESHOLD = 10;
  static const double DEFAULT_RATIO_THRESHOLD;
  static const double DEFAULT_COMPACITY_THRESHOLD;

public:

  explicit SegmentationClusteringDocumentPreseg5(SegmentationContext *segContext);

  virtual ~SegmentationClusteringDocumentPreseg5();
  
  // WARNING: R: Set this function for convenience to be able to save images related to the segmentation process
  // do not use it for other than debugging purposes!!
  void setParameters(const std::string &imageName);

  void getParameters();

  void init(const girl::regionDeque &regs, int level = 1);

  void init(girl::region, int);

  bool same(const girl::point&, girl::direction);

  void getRegionsByType(const ise::SegmentationContext *segContext, girl::regionDeque &textRegions, girl::regionDeque &imageRegions, girl::regionDeque &separatorRegions) const;


protected:

  void computeStatsWords();

  void computeStatsWords2();

  void computeStatsConnectedComponents();


  /**
  * Fill @regs with regions that are created by border removal. Several non-empty region
  * can be created due to internal obstacles in the toger (black separators/ illustrations)
  */
  void
  removeBorders(const girl::region r, girl::regionDeque &regs);


//  void
//  computeConnectedComponentsStatistics(const cv::Mat &binaryMask,
//                                       cv::Mat &ccLabels,
//                                       std::vector<ise::NeighbouringRegions> &spacingsHoriz,
//                                       std::vector<ise::NeighbouringRegions> &spacingsVerti,
//                                       std::vector<size_t> &ccWidths,
//                                       std::vector<size_t> &ccHeights);


//  void
//  computeWordToger2();

  void
  computeWord();

//  void
//  saveToger(const girl::toger& tog, const std::string &name);

  bool
  mustProcessBlock(const girl::region r, const cv::Mat &imageMat, const cv::Mat &ccMat) const;

  void
  processBlock(girl::region r, girl::regionDeque &regs, const girl::unorderedSet<int> &includedCC, const girl::unorderedSet<int> &includedWords);

  void
  bitmapSplit(ise::SegmentationContext &segContext, const girl::region r, const cv::Mat &bitmap, girl::regionDeque &createdRegs);


  void
  computeParagraphSplit(ise::SegmentationContext &segContext,
                        girl::region r,
                        const girl::unorderedSet<int>& includedWords,
                        girl::regionDeque &createdRegs,
                        int threshHoriz,
                        int threshVerti,
                        int characterWidth,
                        int characterHeight,
			int medianCCHorizSpacing,
			int averageWordSpacing);


  void
  preprocessing(girl::region);

  int
  detectImageType() const;

  void
  binarize(girl::region r, ise::SegmentationContext &segContext, cv::Mat &binaryImage);

  void
  detectForegroundBackgroundRegions(const girl::toger &tog,
                                    const cv::Mat &binaryMat,
                                    girl::regionDeque &fgRegs,
                                    girl::regionDeque &bgRegs);

  /**
    * Will erase(merge) the regions touching image borders from the toger and binaryImage
    * fgRegs will have to be recomputed, because regions could have been renamed during merge
    */
  void
  removeFgRegionsTouchingBorder(girl::toger &tog,
                                cv::Mat &binaryImage,
                                girl::regionDeque &fgRegs);

  void
  computeEnclosingShapeToger(const cv::Mat &binaryImage,
                             ise::SegmentationContext &segContext);

  void
  detectIllustrations(ise::SegmentationContext &segContextEnclosingShapes,
                      const cv::Mat &ccMat,
                      girl::regionDeque &illustrationRegs);
  
  void
    correctClassificationErrors(girl::regionDeque &trueIllustrations);


  void
  removeSmallRegions(ise::SegmentationContext &segContext,
                     const girl::regionDeque &fgRegs,
                     cv::Mat &binaryMat);


  void
  extendBlackSeparatorsAndFillMask(ise::SegmentationContext &segContext,
                                   const girl::regionDeque &fgRegs,
                                   cv::Mat &binaryMat,
                                   cv::Mat &separatorMask,
                                   girl::regionDeque &separatorRegs);


  void
  computeConnectedComponentsStats(const girl::unorderedSet<int> &connectedComponentsInTheBlock,
                                                                         Histogram1D &ccWidthHisto,
                                                                         Histogram1D &ccHeightsHisto,
                                                                         Histogram1D &spacingsCCHorizHisto,
                                                                         Histogram1D &spacingsCCVertiHisto);

  void
  computeWordsStats(const girl::unorderedSet<int> &wordsInTheBlock,
                    //                    Histogram1D &wordsWidthHisto,
                    //                    Histogram1D &wordsHeightsHisto,
                    Histogram1D &spacingsWordsHorizHisto,
                    Histogram1D &spacingsWordsVertiHisto);

//  void
//  computeWhitespace(girl::region r,
//                    girl::regionDeque &createdRegs,
//                    size_t minWidth,
//                    size_t minHeight,
//                    size_t minArea);

  void
  computeWhitespace(girl::region r,
                    girl::regionDeque &createdRegs,
                    const girl::unorderedSet<int> &wordsInTheBlock,
                    size_t minWidth,
                    size_t minHeight,
                    size_t minArea);

  void
  computeProfile(const girl::region r,
                 girl::regionDeque &createdRegs,
                 size_t erodeHoriz,
                 size_t dilateHoriz,
                 size_t erodeVerti,
                 size_t dilateVerti);

  girl::region
  mergeIncorrectIllustrations(girl::region r,
                              const girl::unorderedSet<int>& includedWords);

  bool
    blockGeometryOk(girl::region reg);


  void
    computeDistanceHistogram(const std::vector<ise::BitmapRunLength::SpacingInfo>& spacings, ise::Histogram1D &histo);
  
 protected:


  // info related to binarization
  girl::toger m_binarizationToger;
  cv::Mat     m_binarizationMask;
  cv::Mat     m_originalBinarization;
  girl::regionDeque m_binarizationFgRegions;
  cv::Mat m_ccLabels;
  // ise::BitmapRunLength m_brlCCHoriz;
  // ise::BitmapRunLength m_brlCCVerti;


  // info to compute block statistics
  // std::vector<ise::NeighbouringRegions> m_spacingsHoriz;
  // std::vector<ise::NeighbouringRegions> m_spacingsVerti;
  // ise::BitmapRunLength m_horizCCRunLengths;
  // ise::BitmapRunLength m_vertiCCRunLengths;
  std::vector<ise::BitmapRunLength::SpacingInfo> m_spacingsHoriz;
  std::vector<ise::BitmapRunLength::SpacingInfo> m_spacingsVerti;


  std::vector<size_t> m_ccWidths;
  std::vector<size_t> m_ccHeights;

  // info related to enclosing shapes
  girl::toger m_enclosingShapeToger;
  girl::regionDeque m_candidateNonTextRegions;


  // info related to separators
  cv::Mat m_separatorsMask;

  // info related to images
  girl::regionDeque m_imageRegions; // id of regions identified as image in the enclosing shape toger
  cv::Mat m_imageMask;

  // info related to words
  // girl::toger m_wordToger;
  cv::Mat m_wordMask;
  cv::Mat m_wordCCLabels;
  // std::vector<ise::NeighbouringRegions> m_wordSpacingsHoriz;
  // std::vector<ise::NeighbouringRegions> m_wordSpacingsVerti;
  std::vector<ise::BitmapRunLength::SpacingInfo> m_wordSpacingsHoriz;
  std::vector<ise::BitmapRunLength::SpacingInfo> m_wordSpacingsVerti;
  std::vector<size_t> m_wordCcWidths;
  std::vector<size_t> m_wordCcHeights;
  std::vector<size_t> m_wordCcAreas;
  std::vector<girl::tools::AABBox> m_wordBoxes;



  girl::toger m_toger;
  ise::SegmentationContext m_segContext2;
  cv::Mat m_image;

};

}

#endif /* ISESEGMENTATIONCLUSTERINGDOCUMENTPRESEG5_HPP_ */
