/*
  Copyright (C) 2005-2011 Remi Vieux, Vincent Rabeux, Boris Mansencal  remi.vieux@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef SEGMENTATIONCLUSTERINGENCLOSINGSHAPEWIDGET_HPP
#define SEGMENTATIONCLUSTERINGENCLOSINGSHAPEWIDGET_HPP

#include <vector>

#include <opencv2/core/core.hpp>


#include <girl/girlSharedPtr.hpp>

#include <ise/iseBinarizer.hpp>
#include <ise/iseSegmentationClusteringEnclosingShape.hpp>
#include <ise/EnclosingShapeFactory.hpp>


#include <QWidget>

#include <qiseTools/SegmentationClusteringWidgetInterface.hpp>
#include <ise/iseSegmentationContext.hpp>

class Histogram1DThresholdsWidget;
class OrientationChoiceWidget;

namespace ise {
  class QuantificationThresholder;
}

namespace Ui {
  class SegmentationClusteringEnclosingShapeWidget;
}


class SegmentationClusteringEnclosingShapeWidget : public SegmentationClusteringWidgetInterface
{
  Q_OBJECT

  typedef ise::SegmentationClusteringEnclosingShape::ShapeList ShapeList;
  typedef ise::SegmentationClusteringEnclosingShape::EnclosingShapeFactoryPtr EnclosingShapeFactoryPtr;

public:
  explicit  SegmentationClusteringEnclosingShapeWidget(ise::SegmentationContext *segContext,
						       MainView *mainView,
						       SegmentationClusteringPlugin *segmentationClustering,
						       QWidget *parent = NULL);

  ~SegmentationClusteringEnclosingShapeWidget();

  void setParameters(const ShapeList &, const EnclosingShapeFactoryPtr&, const ise::SegmentationClusteringEnclosingShape::MergeMethod &) {}
  void getParameters(ShapeList &, EnclosingShapeFactoryPtr& , ise::SegmentationClusteringEnclosingShape::MergeMethod &method) const;

protected:

  void buildGUI();

private slots:
  void on_shapeSpacingRunButton_clicked();
  void on_chooseFilterPushButton_clicked();
  void on_chooseBinarizerPushButton_clicked();
  void on_visualizeShapesPushButton_clicked();
  void on_thresholdDoubleSpinBox_valueChanged(double );
    void on_shapeSpacingThresholderCombo_currentIndexChanged(int index);
  void on_chooseFilterComboBox_currentIndexChanged(QString );

  void thresholdDeleted(size_t id);

private :
    void buildFilters();
    void buildShapeSpacingThresholderList();
    void buildBinarizerList();
    void buildShapeFactoriesList();
    void binarize(int binarizerIndex);
    void displayShapes(const std::vector<bool> *filters = NULL);
    void filterShapes(ShapeList &output) const;
    bool launchThresholdersInterface();

private:

  Ui::SegmentationClusteringEnclosingShapeWidget *m_ui;
  Histogram1DThresholdsWidget *m_histoView;
  ise::SegmentationContext *m_segContext;
  size_t m_currentFilterThreshold;


  Histogram1DThresholdsWidget *m_spacingHistoView;
  OrientationChoiceWidget *m_orientationChoiceWidget;
  std::vector<EnclosingShapeFactoryPtr> m_shapeFactories;
  std::vector<ise::Binarizer*> m_binarizers;
  std::vector<girl::shared_ptr<ise::QuantificationThresholder> > m_shapeSpacingThresholders;
  ShapeList m_shapes;
  std::vector<bool> m_filteredShapes;
  cv::Mat m_binaryImage;
};

#endif // SEGMENTATIONCLUSTERINGENCLOSINGSHAPEWIDGET_HPP
