/*
  Copyright (C) 2005-2011 Julian Roigt  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include <qiseTools/ImageFeatureChoiceWidget.hpp>
#include <qiseTools/MainView.hpp>

#include "SegmentationClusteringFelzenswalbWidget.hpp"
#include "SegmentationClusteringSettingsFelzenswalb.hpp"

#include <QLabel>
#include <QLayout>
#include <QDoubleSpinBox>
#include <QSpinBox>


SegmentationClusteringFelzenswalbWidget::SegmentationClusteringFelzenswalbWidget(MainView *mainView, SegmentationClusteringPlugin *segmentationClusteringPlugin, QWidget *parent)
  : SegmentationClusteringWidgetInterface(mainView, segmentationClusteringPlugin, parent),
    m_ifcw(NULL),
    m_sigmaBox(NULL),
    m_kBox(NULL),
    m_areaBox(NULL)
{
  setAttribute(Qt::WA_DeleteOnClose);

  setWindowTitle(tr("SegmentationClusteringFelzenswalb parameters..."));

  buildGUI();
}

void
SegmentationClusteringFelzenswalbWidget::buildGUI()
{
  assert(m_mainView);
  assert(m_mainView->model());
  assert(m_mainView->model()->getSegContextPtr());
  assert(m_mainView->model()->getSegContextPtr()->getImage());
  m_ifcw = new ImageFeatureChoiceWidget(m_mainView, m_mainView->model()->getFeatureComputerGUIInfo(), this);

  m_sigmaBox = new QDoubleSpinBox;
  m_kBox = new QDoubleSpinBox;
  m_areaBox = new QSpinBox;

  QHBoxLayout *sigmaLayout = new QHBoxLayout;
  sigmaLayout->addWidget(new QLabel(tr("sigma")));
  sigmaLayout->addWidget(m_sigmaBox);

  QHBoxLayout *kLayout = new QHBoxLayout;
  kLayout->addWidget(new QLabel(tr("k")));
  kLayout->addWidget(m_kBox);
  m_kBox->setMinimum(0);
  m_kBox->setMaximum(1e6);
  
  QHBoxLayout *minAreaLayout = new QHBoxLayout;
  minAreaLayout->addWidget(new QLabel(tr("minArea")));
  minAreaLayout->addWidget(m_areaBox);
  m_areaBox->setMinimum(1);
  m_areaBox->setMaximum(m_mainView->model()->getSegContextPtr()->getImage()->width()*m_mainView->model()->getSegContextPtr()->getImage()->height());

  QVBoxLayout *vLayout = new QVBoxLayout;
  vLayout->addWidget(m_ifcw);
  vLayout->addLayout(sigmaLayout);
  vLayout->addLayout(kLayout);
  vLayout->addLayout(minAreaLayout);
  setLayout(vLayout);
}


void
SegmentationClusteringFelzenswalbWidget::setParameters(float sigma, float k, int minArea, const ise::FeatureComputerPtr &algo)
{
  m_sigmaBox->setValue(sigma);
  m_kBox->setValue(k);
  m_areaBox->setValue(minArea);
  m_ifcw->setImageFeatureComputerPtr(algo);
}

void
SegmentationClusteringFelzenswalbWidget::getParameters(float &sigma, float &k, int &minArea, ise::FeatureComputerPtr &data) const
{
  assert(m_mainView);
  assert(m_mainView->model());
  assert(m_mainView->model()->getSegContextPtr());

  sigma = m_sigmaBox->value();
  k = m_kBox->value();
  minArea = m_areaBox->value();
  data = m_ifcw->imageFeatureComputerPtr();
}
