/*
  Copyright (C) 2013 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "SegmentationClusteringGonogo1abWidget.hpp"
#include "SegmentationClusteringGonogo1abSettings.hpp"

#include <cassert>
#include <QGridLayout>
#include <QSpinBox>
#include <QLineEdit>
#include <QLabel>
#include <QDoubleValidator>


#include <qiseTools/ColorSpaceChoiceWidget.hpp>


SegmentationClusteringGonogo1abWidget::SegmentationClusteringGonogo1abWidget(MainView *mainView,
									 SegmentationClusteringPlugin *segmentationClusteringPlugin,
									 QWidget *parent)
  : SegmentationClusteringWidgetInterface(mainView, segmentationClusteringPlugin, parent),
    m_cscw(NULL),
    m_nbClassesSB(NULL),
    m_areaSB(NULL),
    m_flashLE(NULL)
{

  setWindowTitle(tr("Gonogo1ab parameters..."));

  //setAttribute(Qt::WA_DeleteOnClose);

  buildGUI();

}

void
SegmentationClusteringGonogo1abWidget::buildGUI()
{
  m_cscw = new ColorSpaceChoiceWidget(this);

  QLabel *nbClassesL = new QLabel(tr("Number of classes: "), this);
  m_nbClassesSB = new QSpinBox(this);
  m_nbClassesSB->setMinimum(2);
  m_nbClassesSB->setMaximum(std::numeric_limits<int>::max());

  QLabel *areaL = new QLabel(tr("Small area threshold: "), this);
  m_areaSB = new QSpinBox(this);
  m_areaSB->setMinimum(1);
  m_areaSB->setMaximum(std::numeric_limits<int>::max());

  
  QLabel *flashLabel = new QLabel(tr("min flash luminance threshold: "), this);
  m_flashLE = new QLineEdit(this);
  m_flashLE->setValidator(new QDoubleValidator(m_flashLE));

  QGridLayout *gLayout = new QGridLayout;
  gLayout->addWidget(m_cscw, 0, 0, 1, 2);
  gLayout->addWidget(nbClassesL, 1, 0);
  gLayout->addWidget(m_nbClassesSB, 1, 1);
  gLayout->addWidget(areaL, 2, 0);
  gLayout->addWidget(m_areaSB, 2, 1);
  gLayout->addWidget(flashLabel, 3, 0);
  gLayout->addWidget(m_flashLE, 3, 1);

  setLayout(gLayout);
}

void
SegmentationClusteringGonogo1abWidget::setParameters(int colorSpace, int nbClasses, int regionArea, double flashLuminanceThreshold)
{
  m_cscw->setColorSpace(colorSpace);
  m_nbClassesSB->setValue(nbClasses);
  m_areaSB->setValue(regionArea);
  m_flashLE->setText(QString::number(flashLuminanceThreshold));
}

void
SegmentationClusteringGonogo1abWidget::getParameters(int &colorSpace, int &nbClasses, int &regionArea, double &flashLuminanceThreshold) const
{
  colorSpace = m_cscw->getColorSpace();
  nbClasses = m_nbClassesSB->value();
  regionArea = m_areaSB->value();
  flashLuminanceThreshold = m_flashLE->text().toDouble();
}


