/*
  Copyright (C) 2013 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef SEGMENTATIONCLUSTERINGGONOGO2SETTINGS_HPP
#define SEGMENTATIONCLUSTERINGGONOGO2SETTINGS_HPP

#include <qiseTools/SegmentationClusteringWidgetSettingsInterface.hpp>
#include "SegmentationClusteringGonogo2Widget.hpp"
#include <ise/iseSegmentationClusteringGonogo2.hpp>

class SegmentationClusteringGonogo2Settings : public SegmentationClusteringWidgetSettingsInterface
{
  
public:
  
  //B: UGLY: duplication of default parameters !!!!!!!!!!!!!
  SegmentationClusteringGonogo2Settings() 
    : SegmentationClusteringWidgetSettingsInterface(),
      m_colorSpace(ISE_RGB_to_Lab),
      m_nbClasses(6),
      m_regionArea(145),
      m_flashLuminanceThreshold(57)
    , m_a00(0.8276)
    , m_a01(-0.5097)
    , m_a02(0.2352)
    , m_a10(-0.1835)
    , m_a11(-0.4189)
    , m_a12(0.8893)
    , m_tx(-28.6515)
    , m_ty(5.6625)
  {
  }

  virtual void configureWidget(SegmentationClusteringWidgetInterface *widget) const
  {
    static_cast<SegmentationClusteringGonogo2Widget*>(widget)->setParameters(m_colorSpace, m_nbClasses, m_regionArea, m_flashLuminanceThreshold, m_a00, m_a01, m_a02, m_a10, m_a11, m_a12, m_tx, m_ty);
  }
  
  virtual void getWidgetConfig(const SegmentationClusteringWidgetInterface *widget)
  {
    static_cast<const SegmentationClusteringGonogo2Widget*>(widget)->getParameters(m_colorSpace, m_nbClasses, m_regionArea, m_flashLuminanceThreshold, m_a00, m_a01, m_a02, m_a10, m_a11, m_a12, m_tx, m_ty);
  }

  virtual void configureAlgo(girl::shared_ptr<ise::SegmentationClustering> algo) const
  {
    girl::shared_ptr<ise::SegmentationClusteringGonogo2> algorithm = girl::static_pointer_cast<ise::SegmentationClusteringGonogo2>(algo);
    algorithm->setParameters(m_colorSpace, m_nbClasses, m_regionArea, m_flashLuminanceThreshold, m_a00, m_a01, m_a02, m_a10, m_a11, m_a12, m_tx, m_ty);
  }
  
  virtual girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface> clone() const
  {
    girl::shared_ptr<SegmentationClusteringGonogo2Settings> p(new SegmentationClusteringGonogo2Settings);
    
    *p = *this;
    
    return p;
  }
  
  virtual bool isValid() const
  {
    return (m_nbClasses > 0 && m_colorSpace >= 0 && m_colorSpace != ISE_RGB_to_GRAY);
  }

private:
  
  int m_colorSpace;
  int m_nbClasses;
  int m_regionArea;
  double m_flashLuminanceThreshold;
  double m_a00, m_a01, m_a02, m_a10, m_a11, m_a12, m_tx, m_ty;
};

#endif /* ! SEGMENTATIONCLUSTERINGGONOG1SETTINGS_HPP */
