/*
  Copyright (C) 2005-2011 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef SEGMENTATIONCLUSTERINGMaximalWhitespaceRectangleWIDGETSETTINGS_HPP
#define SEGMENTATIONCLUSTERINGMaximalWhitespaceRectangleWIDGETSETTINGS_HPP

#include <qiseTools/SegmentationClusteringWidgetSettingsInterface.hpp>

#include <ise/iseSegmentationClusteringMaximalWhitespaceRectangle.hpp>
#include "SegmentationClusteringMaximalWhitespaceRectangleWidget.hpp"

class SegmentationClusteringMaximalWhitespaceRectangleSettings : public SegmentationClusteringWidgetSettingsInterface {
  
public:
  
  SegmentationClusteringMaximalWhitespaceRectangleSettings() 
    : SegmentationClusteringWidgetSettingsInterface(),
      m_minArea(100),
      m_maxNbRect(300),
      m_minWidthLow(10),
      m_minWidthHigh(10),
      m_minHeightLow(10),
      m_minHeightHigh(10),
      m_overlap(0),
      m_mergeWhiteSpaces(false),
      m_useBBoxForDirectNeighbours(false),
      m_useBBoxForInnerNeighbours(true)
  {
  }

  virtual void configureWidget(SegmentationClusteringWidgetInterface *widget) const
  {
    static_cast<SegmentationClusteringMaximalWhitespaceRectangleWidget*>(widget)->setParameters(m_minArea, m_maxNbRect, m_minWidthLow, m_minWidthHigh, m_minHeightLow, m_minHeightHigh, m_overlap, m_mergeWhiteSpaces, m_useBBoxForDirectNeighbours, m_useBBoxForInnerNeighbours);
  }
  
  virtual void getWidgetConfig(const SegmentationClusteringWidgetInterface *widget)
  {
    static_cast<const SegmentationClusteringMaximalWhitespaceRectangleWidget*>(widget)->getParameters(m_minArea, m_maxNbRect, m_minWidthLow, m_minWidthHigh, m_minHeightLow, m_minHeightHigh, m_overlap, m_mergeWhiteSpaces, m_useBBoxForDirectNeighbours, m_useBBoxForInnerNeighbours);
  }

  virtual void configureAlgo(girl::shared_ptr<ise::SegmentationClustering> algo) const
  {
    girl::shared_ptr<ise::SegmentationClusteringMaximalWhitespaceRectangle> algorithm = girl::static_pointer_cast<ise::SegmentationClusteringMaximalWhitespaceRectangle>(algo);
    algorithm->setParameters(m_minArea, m_maxNbRect, m_minWidthLow/*, m_minWidthHigh*/, m_minHeightLow/*, m_minHeightHigh*/, m_overlap, m_mergeWhiteSpaces, m_useBBoxForDirectNeighbours, m_useBBoxForInnerNeighbours);
  }
  
  virtual girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface> clone() const
  {
    girl::shared_ptr<SegmentationClusteringMaximalWhitespaceRectangleSettings> p(new SegmentationClusteringMaximalWhitespaceRectangleSettings);
    
    *p = *this;

    return p;
  }

  virtual bool isValid() const
  {
    // TODO
    return true;
  }
  

private:
  
  size_t m_minArea;
  size_t m_maxNbRect;
  size_t m_minWidthLow;
  size_t m_minWidthHigh;
  size_t m_minHeightLow;
  size_t m_minHeightHigh;
  double m_overlap;
  bool m_mergeWhiteSpaces;
  bool m_useBBoxForDirectNeighbours;
  bool m_useBBoxForInnerNeighbours;
};

#endif /* ! SEGMENTATIONCLUSTERINGMaximalWhitespaceRectangleWIDGETSETTINGS_HPP */
