/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "SegmentationClusteringMaximalWhitespaceRectangleWidget.hpp"
#include "SegmentationClusteringMaximalWhitespaceRectangleSettings.hpp"

#include <cassert>
#include <limits>
#include <QCheckBox>
#include <QLabel>
#include <QGridLayout>
#include <QSpinBox>

#include <ise/iseSegmentationClusteringMaximalWhitespaceRectangle.hpp>

#include <qiseTools/MainView.hpp>
#include <qiseTools/SegmentationClusteringFactory.hpp>




SegmentationClusteringMaximalWhitespaceRectangleWidget::SegmentationClusteringMaximalWhitespaceRectangleWidget(MainView *mainView,
									     SegmentationClusteringPlugin *segmentationClustering,
									     QWidget *parent)
  : SegmentationClusteringWidgetInterface(mainView, segmentationClustering, parent),
    m_minArea(NULL),
    m_maxNbWhiteSpace(NULL),
    m_overlap(NULL)
{

  setWindowTitle(tr("MaximalWhitespaceRectangle parameters..."));

  setAttribute(Qt::WA_DeleteOnClose);

  buildGUI();

}

void
SegmentationClusteringMaximalWhitespaceRectangleWidget::buildGUI()
{
  QGridLayout *layout = new QGridLayout();
  
  m_minArea = new QSpinBox(this);
  m_maxNbWhiteSpace = new QSpinBox(this);
  m_minWidthLow = new QSpinBox(this);
  m_minWidthHigh = new QSpinBox(this);
  m_minHeightLow = new QSpinBox(this);
  m_minHeightHigh = new QSpinBox(this);
  m_overlap = new QDoubleSpinBox(this);
  m_mergeWhitespaceBox = new QCheckBox(tr("Merge whitespaces?"), this);
  m_useBBoxForDirectNeighboursCB = new QCheckBox(tr("Use bounding boxes for direct neighbours?"), this);
  m_useBBoxForInnerNeighboursCB = new QCheckBox(tr("Use bounding boxes for inner neighbours?"), this);
  

  assert(m_mainView->model());
  assert(m_mainView->model()->getSegContextPtr());
  girl::tools::Image *img = m_mainView->model()->getSegContextPtr()->getImage();
  assert(img);
  m_minArea->setRange(0,img->width()*img->height());
  m_maxNbWhiteSpace->setRange(0, std::numeric_limits<int>::max());
  m_overlap->setRange(0, 1);
  m_overlap->setDecimals(2);
  m_overlap->setSingleStep(0.01);
  
  QLabel *l1 = new QLabel(tr("Min Rectangle Area"), this);
  layout->addWidget(l1, 0, 0);
  layout->addWidget(m_minArea, 0, 1);
  QLabel *l2 = new QLabel(tr("Max number of rectangles"), this);
  layout->addWidget(l2, 1, 0);
  layout->addWidget(m_maxNbWhiteSpace, 1, 1);
  QLabel *lwl = new QLabel(tr("Minimal authorized width lower threshold"), this);
  layout->addWidget(lwl, 2, 0);
  layout->addWidget(m_minWidthLow, 2, 1);
  QLabel *lwh = new QLabel(tr("Minimal authorized width higher threshold"), this);
  layout->addWidget(lwh, 3, 0);
  layout->addWidget(m_minWidthHigh, 3, 1);
  QLabel *lhl = new QLabel(tr("Minimal authorized height lower threshold"), this);
  layout->addWidget(lhl, 4, 0);
  layout->addWidget(m_minHeightLow, 4, 1);
  QLabel *lhh = new QLabel(tr("Minimal authorized height higher threshold"), this);
  layout->addWidget(lhh, 5, 0);
  layout->addWidget(m_minHeightHigh, 5, 1);
  QLabel *l3 = new QLabel(tr("Authorized overlap"), this);
  layout->addWidget(l3, 6, 0);
  layout->addWidget(m_overlap, 6, 1);
  layout->addWidget(m_mergeWhitespaceBox, 7, 0);
  layout->addWidget(m_useBBoxForDirectNeighboursCB, 8, 0);
  layout->addWidget(m_useBBoxForInnerNeighboursCB, 9, 0);
  setLayout(layout);
}

void
SegmentationClusteringMaximalWhitespaceRectangleWidget::setParameters(size_t minArea, size_t maxNbRect, size_t minWidthLow, size_t minWidthHigh, size_t minHeightLow, size_t minHeightHigh, double overlap, bool merge, bool useBBoxForDirect, bool useBBoxForInner)
{
  m_minArea->setValue(minArea);
  m_maxNbWhiteSpace->setValue(maxNbRect);
  m_minWidthLow->setValue(minWidthLow);
  m_minWidthHigh->setValue(minWidthHigh);
  m_minHeightLow->setValue(minHeightLow);
  m_minHeightHigh->setValue(minHeightHigh);
  m_overlap->setValue(overlap);
  m_mergeWhitespaceBox->setChecked(merge);
  m_useBBoxForDirectNeighboursCB->setChecked(useBBoxForDirect);
  m_useBBoxForInnerNeighboursCB->setChecked(useBBoxForInner);
}

void
SegmentationClusteringMaximalWhitespaceRectangleWidget::getParameters(size_t &minarea, size_t &maxrect, size_t &minWidthLow, size_t &minWidthHigh, size_t &minHeightLow, size_t &minHeightHigh, double &overlap, bool &merge, bool &useBBoxForDirect, bool &useBBoxForInner) const
{
  minarea = static_cast<size_t>(m_minArea->value());
  maxrect = static_cast<size_t>(m_maxNbWhiteSpace->value());
  minWidthLow =  static_cast<size_t>(m_minWidthLow->value());
  minWidthHigh =  static_cast<size_t>(m_minWidthHigh->value());
  minHeightLow =  static_cast<size_t>(m_minHeightLow->value());
  minHeightHigh =  static_cast<size_t>(m_minHeightHigh->value());
  overlap = m_overlap->value();
  merge = m_mergeWhitespaceBox->isChecked();
  useBBoxForDirect = m_useBBoxForDirectNeighboursCB->isChecked();
  useBBoxForInner = m_useBBoxForInnerNeighboursCB->isChecked();
}


