/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "SegmentationClusteringMixedTextImageWidget.hpp"
#include "SegmentationClusteringMixedTextImageWidgetSettings.hpp"

#include <cassert>
#include <QHBoxLayout>
#include <QVBoxLayout>

#include "iseSegmentationClusteringMixedTextImage.hpp"

#include <qiseTools/SegmentationClusteringPlugin.hpp>

#include <QSpinBox>
#include <QLabel>
#include <QComboBox>
#include <QGroupBox>

SegmentationClusteringMixedTextImageWidget::SegmentationClusteringMixedTextImageWidget(MainView *mainView,
									     SegmentationClusteringPlugin *segmentationClusteringPlugin,
									     QWidget *parent)
  : SegmentationClusteringWidgetInterface(mainView, segmentationClusteringPlugin, parent)
  , m_comboBox(NULL)
{

  setWindowTitle(tr("MixedTextImage parameters..."));

  setAttribute(Qt::WA_DeleteOnClose);

  buildGUI();

  int index = m_comboBox->findText("text");
  m_comboBox->setCurrentIndex(index);
}

SegmentationClusteringMixedTextImageWidget::~SegmentationClusteringMixedTextImageWidget()
{
  m_qtwa->deactivate();
}

void
SegmentationClusteringMixedTextImageWidget::buildGUI()
{
  QVBoxLayout *layout = new QVBoxLayout;

  QHBoxLayout *hLayout = new QHBoxLayout;
  
  QLabel *comboBoxLabel = new QLabel(tr("type :"), this);
  m_comboBox = new QComboBox(this);
  m_comboBox->addItem(tr("text"));
  m_comboBox->addItem(tr("black_line_H"));
  m_comboBox->addItem(tr("black_line_V"));
  m_comboBox->addItem(tr("graphics"));

  hLayout->addWidget(comboBoxLabel);
  hLayout->addWidget(m_comboBox);

  const bool allRegions = false;
  const bool oneChannel = true;
  m_qtwa = new QuantificationThresholdsWidgetAux(m_mainView, allRegions, oneChannel, this);
  m_qtwa->activate();

  layout->addLayout(hLayout);
  layout->addWidget(m_qtwa);
  
  setLayout(layout);
}

void
SegmentationClusteringMixedTextImageWidget::setParameters(const std::vector<int> &levels, int colorSpace, int channel, std::string type)
{
  int pos = m_comboBox->findText(type.c_str());
  m_comboBox->setCurrentIndex(pos);
  m_qtwa->setGUI(colorSpace, channel, levels);
}

void
SegmentationClusteringMixedTextImageWidget::getParameters(std::vector<int> &levels, int &colorSpace, int &channel, std::string &type) const
{
  type = m_comboBox->currentText().toStdString();

  m_qtwa->getGUI(colorSpace, channel, levels);
}
