/*
  Copyright (C) 2011 Cedric Aguerre  aguerre@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef SEGMENTATIONCLUSTERINGNORMALIZEDCUTWIDGETSETTINGS_HPP
#define SEGMENTATIONCLUSTERINGNORMALIZEDCUTWIDGETSETTINGS_HPP

#include <qiseTools/SegmentationClusteringPlugin.hpp>

#include "SegmentationClusteringNormalizedCutWidget.hpp"
#include <SegmentationClusteringNormalizedCut.hpp>

class SegmentationClusteringNormalizedCutWidgetSettings : public SegmentationClusteringWidgetSettingsInterface {
  
public:
  
  SegmentationClusteringNormalizedCutWidgetSettings() 
    : SegmentationClusteringWidgetSettingsInterface()
    , m_nbClusters(8)
  {
  }

  virtual void configureWidget(SegmentationClusteringWidgetInterface *widget) const
  {
    static_cast<SegmentationClusteringNormalizedCutWidget*>(widget)->setParameters(m_nbClusters);
  }
  
  virtual void getWidgetConfig(const SegmentationClusteringWidgetInterface *widget)
  {
    static_cast<const SegmentationClusteringNormalizedCutWidget*>(widget)->getParameters(m_nbClusters);
  }

  virtual void configureAlgo(girl::shared_ptr<ise::SegmentationClustering> algo) const
  {
    girl::shared_ptr<ncut::SegmentationClusteringNormalizedCut> algorithm = girl::static_pointer_cast<ncut::SegmentationClusteringNormalizedCut>(algo);
    algorithm->setParameters(m_nbClusters);
  }

  virtual girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface> clone() const
  {
    girl::shared_ptr<SegmentationClusteringNormalizedCutWidgetSettings> p(new SegmentationClusteringNormalizedCutWidgetSettings);

    *p = *this;

    return p;
  }

  virtual bool isValid() const
  {
    return (m_nbClusters > 0);
  }
  
private:
  
  int m_nbClusters;
};

#endif /* ! SEGMENTATIONCLUSTERINGNORMALIZEDCUTWIDGETSETTINGS_HPP */
