/*
  Copyright (C) 2005-2011 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#ifndef SEGMENTATIONCLUSTERINGPROFILESETTINGS_HPP
#define SEGMENTATIONCLUSTERINGPROFILESETTINGS_HPP

#include <ise/iseSegmentationClusteringProfile.hpp>

#include <qiseTools/SegmentationClusteringWidgetSettingsInterface.hpp>
#include "SegmentationClusteringProfileWidget.hpp"

class SegmentationClusteringProfileSettings : public SegmentationClusteringWidgetSettingsInterface {

public:

 SegmentationClusteringProfileSettings()
  : SegmentationClusteringWidgetSettingsInterface(),
    m_erodeHoriz(0),
    m_dilateHoriz(0),
    m_erodeVerti(0),
    m_dilateVerti(0),
    m_threshold(0.1f),
    m_algo()
 {
 }

 virtual void configureWidget(SegmentationClusteringWidgetInterface *widget) const
 {
   static_cast<SegmentationClusteringProfileWidget*>(widget)->setParameters(m_erodeHoriz, m_dilateHoriz, m_erodeVerti, m_dilateVerti, m_threshold, m_algo);
 }

 virtual void getWidgetConfig(const SegmentationClusteringWidgetInterface *widget)
 {
   static_cast<const SegmentationClusteringProfileWidget*>(widget)->getParameters(m_erodeHoriz, m_dilateHoriz, m_erodeVerti, m_dilateVerti, m_threshold, m_algo);
 }

  virtual void configureAlgo(girl::shared_ptr<ise::SegmentationClustering> algo) const
  {
    girl::shared_ptr<ise::SegmentationClusteringProfile> algorithm = girl::static_pointer_cast<ise::SegmentationClusteringProfile>(algo);
    algorithm->setParameters(m_erodeHoriz, m_dilateHoriz, m_erodeVerti, m_dilateVerti, m_threshold, m_algo);
  }
  
 virtual girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface> clone() const
 {
  girl::shared_ptr<SegmentationClusteringProfileSettings> p(new SegmentationClusteringProfileSettings);
  *p = *this;
  return p;
 }

  virtual bool isValid() const
  {
    return m_algo.get() != NULL;
  }
  
  
private:
  
  size_t m_erodeHoriz;
  size_t m_dilateHoriz;
  size_t m_erodeVerti;
  size_t m_dilateVerti;
  double m_threshold;
  ise::FeatureComputerPtr m_algo;
};

#endif /* ! SEGMENTATIONCLUSTERINGPROFILESETTINGS_HPP */
