/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "SegmentationClusteringTextImageFuzzyCMeanWidget.hpp"
#include "SegmentationClusteringTextImageFuzzyCMeanWidgetSettings.hpp"

#include <cassert>
#include <QVBoxLayout>
#include <QHBoxLayout>

#include "iseSegmentationClusteringTextImageFuzzyCMean.hpp"

#include <qiseTools/SegmentationClusteringPlugin.hpp>

#include <QSpinBox>
#include <QLabel>
#include <QComboBox>
#include <QDoubleSpinBox>

SegmentationClusteringTextImageFuzzyCMeanWidget::SegmentationClusteringTextImageFuzzyCMeanWidget(MainView *mainView,
									     SegmentationClusteringPlugin *segmentationClusteringPlugin,
									     QWidget *parent)
  : SegmentationClusteringWidgetInterface(mainView, segmentationClusteringPlugin, parent)
  , m_comboBox(NULL)
  , m_C_Height(NULL)
  , m_C_Width(NULL)
{

  setWindowTitle(tr("TextImageFuzzyCMean parameters..."));

  setAttribute(Qt::WA_DeleteOnClose);

  buildGUI();

}

void
SegmentationClusteringTextImageFuzzyCMeanWidget::buildGUI()
{
  QVBoxLayout *layout = new QVBoxLayout();
  
  QHBoxLayout *hLayout = new QHBoxLayout;
  
  QLabel *comboBoxLabel = new QLabel(tr("type :"), this);
  m_comboBox = new QComboBox(this);
  m_comboBox->addItem(tr("image"));
  m_comboBox->addItem(tr("text"));
  m_comboBox->addItem(tr("background"));

  hLayout->addWidget(comboBoxLabel);
  hLayout->addWidget(m_comboBox);

  QLabel *C_HeightLabel = new QLabel(tr("block height :"), this);
  m_C_Height = new QSpinBox(this);
  m_C_Height->setRange(-1000,1000);
  m_C_Height->setSingleStep(1);


  QLabel *C_WidthLabel = new QLabel(tr("block width :"), this);
  m_C_Width = new QSpinBox(this);
  m_C_Width->setRange(0,1000);
  m_C_Width->setSingleStep(1);

  QHBoxLayout *hLayoutTolerence = new QHBoxLayout;

  QLabel *tolerenceLabel = new QLabel(tr("cmean tolerence :"), this);
  m_tolerence = new QDoubleSpinBox(this);
  m_tolerence->setDecimals(12);

  hLayoutTolerence->addWidget(tolerenceLabel);
  hLayoutTolerence->addWidget(m_tolerence);

  layout->addLayout(hLayout);
  layout->addLayout(hLayoutTolerence);
  layout->addWidget(C_HeightLabel);
  layout->addWidget(m_C_Height);
  layout->addWidget(C_WidthLabel);
  layout->addWidget(m_C_Width);
  

  setLayout(layout);
}

void
SegmentationClusteringTextImageFuzzyCMeanWidget::setParameters(int height, int width, std::string type, double tolerence)
{
  m_C_Height->setValue(height);
  m_C_Width->setValue(width);
  int pos = m_comboBox->findText(type.c_str());
  m_comboBox->setCurrentIndex(pos);
  m_tolerence->setValue(tolerence);
}

void
SegmentationClusteringTextImageFuzzyCMeanWidget::getParameters(int &height, int &width, std::string &type, double &tolerence) const
{
  width = m_C_Width->value();
  height = m_C_Height->value();
  type = m_comboBox->currentText().toStdString();
  tolerence = m_tolerence->value();
}


