/*
  Copyright (C) 2005-2011 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef SEGMENTATIONCLUSTERINGTEXTIMAGEFUZZYCMEANWIDGETSETTINGS_HPP
#define SEGMENTATIONCLUSTERINGTEXTIMAGEFUZZYCMEANWIDGETSETTINGS_HPP

#include <qiseTools/SegmentationClusteringWidgetSettingsInterface.hpp>
#include "SegmentationClusteringTextImageFuzzyCMeanWidget.hpp"
#include "SegmentationClusteringTextImageFuzzyCMeanWidgetSettings.hpp"
#include "iseSegmentationClusteringTextImageFuzzyCMean.hpp"

class SegmentationClusteringTextImageFuzzyCMeanWidgetSettings : public SegmentationClusteringWidgetSettingsInterface {
  
public:
  
  SegmentationClusteringTextImageFuzzyCMeanWidgetSettings() 
    : SegmentationClusteringWidgetSettingsInterface(),
      m_height(40), 
      m_width(40),
      m_type("image"),
      m_tolerence(0.000001)
  {
  }

  virtual void configureWidget(SegmentationClusteringWidgetInterface *widget) const
  {
    static_cast<SegmentationClusteringTextImageFuzzyCMeanWidget*>(widget)->setParameters(m_height, m_width, m_type, m_tolerence);
  }
  
  virtual void getWidgetConfig(const SegmentationClusteringWidgetInterface *widget)
  {
    static_cast<const SegmentationClusteringTextImageFuzzyCMeanWidget*>(widget)->getParameters(m_height, m_width, m_type, m_tolerence);
  }

  virtual void configureAlgo(girl::shared_ptr<ise::SegmentationClustering> algo) const
  {
    girl::shared_ptr<ise::SegmentationClusteringTextImageFuzzyCMean> algorithm = girl::static_pointer_cast<ise::SegmentationClusteringTextImageFuzzyCMean>(algo);
    algorithm->setParameters(m_height, m_width, m_type, m_tolerence);
  }

  virtual girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface> clone() const
  {
    girl::shared_ptr<SegmentationClusteringTextImageFuzzyCMeanWidgetSettings> p(new SegmentationClusteringTextImageFuzzyCMeanWidgetSettings);
    
    *p = *this;

    return p;
  }

  virtual bool isValid() const
  {
    return ((m_height > 0) && (m_width > 0) && (m_type != "") && (m_tolerence > 0));
  }

  
private:
  
  int m_height;
  int m_width;
  std::string m_type;
  double m_tolerence;
};

#endif /* ! SEGMENTATIONCLUSTERINGTEXTIMAGEFUZZYCMEANWIDGETSETTINGS_HPP */
