/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "SegmentationClusteringTextureBasedLinWidget.hpp"
#include "SegmentationClusteringTextureBasedLinWidgetSettings.hpp"

#include <cassert>
#include <QGridLayout>

#include "iseSegmentationClusteringTextureBasedLin.hpp"

#include <qiseTools/SegmentationClusteringPlugin.hpp>

#include <QSpinBox>
#include <QLabel>
#include <QComboBox>

SegmentationClusteringTextureBasedLinWidget::SegmentationClusteringTextureBasedLinWidget(MainView *mainView,
											 SegmentationClusteringPlugin *segmentationClusteringPlugin,
											 QWidget *parent)
  : SegmentationClusteringWidgetInterface(mainView, segmentationClusteringPlugin, parent)
  , m_Block_height(NULL)
  , m_Block_width(NULL)
  , m_type(NULL)
{

  setWindowTitle(tr("TextureBasedLin parameters..."));

  setAttribute(Qt::WA_DeleteOnClose);

  buildGUI();

}

void
SegmentationClusteringTextureBasedLinWidget::buildGUI()
{
  QGridLayout *layout = new QGridLayout();
  
  QLabel *Block_heightLabel = new QLabel(tr("Block_height :"), this);
  m_Block_height = new QSpinBox(this);
  m_Block_height->setRange(0,10000);
  m_Block_height->setSingleStep(1);
  layout->addWidget(Block_heightLabel, 0, 0);
  layout->addWidget(m_Block_height,0, 1);

  QLabel *Block_widthLabel = new QLabel(tr("Block_width :"), this);
  m_Block_width = new QSpinBox(this);
  m_Block_width->setRange(0,10000);
  m_Block_width->setSingleStep(1);
  layout->addWidget(Block_widthLabel, 1, 0);
  layout->addWidget(m_Block_width,1, 1);

  QLabel *typeLabel = new QLabel(tr("type :"), this);
  m_type = new QComboBox(this);
  m_type->addItem(tr("text"));
  m_type->addItem(tr("image"));
  m_type->addItem(tr("space"));

  layout->addWidget(typeLabel,2,0);
  layout->addWidget(m_type,2, 1);

  setLayout(layout);
}

void
SegmentationClusteringTextureBasedLinWidget::setParameters(int width, int height, const std::string &type)
{
  int pos = m_type->findText(type.c_str());
  m_type->setCurrentIndex(pos);
  m_Block_height->setValue(height);
  m_Block_width->setValue(width);
}

void
SegmentationClusteringTextureBasedLinWidget::getParameters(int &width, int &height, std::string &type) const
{
  type = m_type->currentText().toStdString();
  width = m_Block_width->value();
  height = m_Block_height->value();
}


