/*
  Copyright (C) 2005-2011 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef SEGMENTATIONCLUSTERINGTEXTUREBASEDLINWIDGETSETTINGS_HPP
#define SEGMENTATIONCLUSTERINGTEXTUREBASEDLINWIDGETSETTINGS_HPP

#include <qiseTools/SegmentationClusteringWidgetSettingsInterface.hpp>
#include "SegmentationClusteringTextureBasedLinWidget.hpp"
#include "iseSegmentationClusteringTextureBasedLin.hpp"

class SegmentationClusteringTextureBasedLinWidgetSettings : public SegmentationClusteringWidgetSettingsInterface {
  
public:
  
  SegmentationClusteringTextureBasedLinWidgetSettings() 
    : SegmentationClusteringWidgetSettingsInterface(),
      m_block_height(0), m_block_width(0), m_type()
  {
  }

  virtual void configureWidget(SegmentationClusteringWidgetInterface *widget) const
  {
    static_cast<SegmentationClusteringTextureBasedLinWidget*>(widget)->setParameters(m_block_width, m_block_height, m_type);
  }
  
  virtual void getWidgetConfig(const SegmentationClusteringWidgetInterface *widget)
  {
    static_cast<const SegmentationClusteringTextureBasedLinWidget*>(widget)->getParameters(m_block_width, m_block_height, m_type);
  }

  virtual void configureAlgo(girl::shared_ptr<ise::SegmentationClustering> algo) const
  {
    girl::shared_ptr<ise::SegmentationClusteringTextureBasedLin> algorithm = girl::static_pointer_cast<ise::SegmentationClusteringTextureBasedLin>(algo);
    algorithm->setParameters(m_block_height, m_block_width, m_type);
  }
  
 virtual girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface> clone() const
  {
    girl::shared_ptr<SegmentationClusteringTextureBasedLinWidgetSettings> p(new SegmentationClusteringTextureBasedLinWidgetSettings);

    *p = *this;

    return p;
  }

  virtual bool isValid() const
  {
    return ((m_block_height > 0) && (m_block_width > 0)); //B: ???
  }

private:
  
  int m_block_height;
  int m_block_width;
  std::string m_type;
};

#endif /* ! SEGMENTATIONCLUSTERINGTEXUREBASEDLINWIDGETSETTINGS_HPP */
