/*
  Segmentation clustering: TextureBasedLin
  Copyright (C) 2005-2009 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include <ise/iseSegmentationContext.hpp>
#include <girl/girlCommon.hpp>

#include "iseSegmentationClusteringTextureBasedLin.hpp"

#include <ise/iseGenericKMeans.hpp>
#include <ise/iseMatrix.hpp>
#include <set>
#include <map>

#include <ise/iseGrayLevelCooccurrenceMatrix.hpp>

#include <ise/iseRegularGridPartitionner.hpp>
#include <ise/iseSegmentationClustering.hpp>
#include <ise/iseSplitDriverDefault.hpp>
#include <ise/iseSplitAlgorithmDefault.hpp>

#define NB_GLCM 4
#define NB_FEATURES 5
#define NB_CLASS_KMEANS 3

/*
#include <fstream>
*/

using namespace ise;

enum type{ TEXT, SPACE, IMAGE };

SegmentationClusteringTextureBasedLin::SegmentationClusteringTextureBasedLin(SegmentationContext *segContext)
 : SegmentationClustering(segContext)
 , m_blockHeight(60)
 , m_blockWidth(40)
 , m_type("text")
 , m_nbBlockHeight(0)
 , m_nbBlockWidth(0)
 , m_label()
 , m_regionIdFeature()
 , m_cat()
 , m_imageLabel()
{
  setSegContext(segContext);
}

void
SegmentationClusteringTextureBasedLin::setSegContext(ise::SegmentationContext *s) 
{
  SegmentationClustering::setSegContext(s);
}

void 
SegmentationClusteringTextureBasedLin::init(const girl::regionDeque &/*regions*/, int /* level */)
{
  girl::tools::Image *image = m_segContext->getImage();
  const int width = image->width();
  const int height = image->height();

  m_nbBlockWidth = std::ceil(static_cast<double>(width) / m_blockWidth);
  m_nbBlockHeight = std::ceil(static_cast<double>(height) / m_blockHeight);

  std::cout<<"nb block="<<m_nbBlockWidth*m_nbBlockHeight<<std::endl;

  /* block segmentation */

  girl::toger *tog = new girl::toger(width, height);
  ise::SegmentationContext *segContext = new ise::SegmentationContext(tog, image, NULL);
  ise::SegmentationClustering *clust = new ise::RegularGridPartitionner(segContext, m_blockWidth, m_blockHeight);
  
  ise::SplitAlgorithmDefault *splitAlgo = new ise::SplitAlgorithmDefault(segContext);
  ise::SplitDriver *driver = new ise::SplitDriverDefault(segContext);

  girl::regionDeque regions2split;
  regions2split.push_back(1);
  girl::regionDeque createdRegions;

  splitAlgo->apply(regions2split, *driver, *clust, 0, createdRegions);  

  size_t size = createdRegions.size();

  std::cout<<"grid seg finish"<<std::endl;

  /* GLCMs compute */
  std::vector<double> features(NB_GLCM * NB_FEATURES * createdRegions.size());

  m_regionIdFeature.resize(size+1, 0);
  computeGLCM(segContext, createdRegions, features);
  std::cout<<"GLCM extraction finish"<<std::endl;

  m_imageLabel = ise::ImageLabelToger(width, height);
  girl::regionDeque regs;
  tog->allRegions(regs);
  for (size_t i = 0 ; i < regs.size() ; ++i) {
      m_imageLabel.setRegId(regs[i]);
      tog->iterateOnRegionDomain(regs[i], m_imageLabel);
  }

  /* normalized features */
  cv::Mat normFeatures(size, NB_GLCM * NB_FEATURES, CV_MAKETYPE(cv::DataDepth<double>::value, 1));
       //ise::matrix<double> normFeatures(size, NB_GLCM * NB_FEATURES);
  normalizedFeatures(features, normFeatures, size);
  
  std::cout<<"Normalized finish"<<std::endl;
  

  ise::GenericKMeans<double> kmeans(NB_CLASS_KMEANS);

  //TODO:REMOVE OR FIX UNUSED OUTPUT
  cv::Mat output;
  kmeans.compute(normFeatures, output);
  
  std::cout<<"Kmeans finish"<<std::endl;

  /* boxe labeling */
  m_label = std::vector<unsigned int>(size);
  for (size_t i = 0 ; i < size ; ++i){
    unsigned int label = 0;
    //double d = 0;
    kmeans.getCluster(normFeatures.ptr<double>(i), NB_GLCM * NB_FEATURES);
    m_label[i] = label;
  }

  const cv::Mat centroid = kmeans.getCenters();
  
  /*
  std::ofstream f("/home/roigt/Bureau/1.dat");
  for (size_t i = 0 ; i < centroid[0].size(); ++i){
    f<<i+1<<" ";
    for (size_t j = 0 ; j < centroid.size() ; ++j) {
      f<<centroid[j][i]<<" ";
    }
    f<<std::endl;
  }
  f.close();
  
  system("echo \"plot \'/home/roigt/Bureau/1.dat\' with linespoints; replot \'/home/roigt/Bureau/1.dat\' using 1:3 with linespoints; replot \'/home/roigt/Bureau/1.dat\' using 1:4 with linespoints;\" | gnuplot -persist");
  */

  for(int i = 0 ; i < centroid.rows ; ++i){
    const double *r = centroid.ptr<double>(i);
    for (int j = 0 ; j < centroid.cols; ++j){
      std::cout<<r[j]<<" ";
    }
    std::cout<<std::endl;
  }

  /* scoring */
  ise::matrix<double> score(3, 3);
  double val = 0;
  score.initVal(val);
 
  
  int categoryEnergie[3] = { IMAGE, TEXT, SPACE};
  int categoryEntropy[3] = { SPACE, TEXT, IMAGE};
  int categorySumEntropy[3] = { SPACE, IMAGE, TEXT};
  int categoryDiffEntropy[3] = { SPACE, IMAGE, TEXT};
  int categoryStdd[3] = { IMAGE, TEXT, SPACE};
  
  std::vector<int*> cat;
  cat.push_back(categoryEnergie);
  cat.push_back(categoryEntropy);
  cat.push_back(categorySumEntropy);
  cat.push_back(categoryDiffEntropy);
  cat.push_back(categoryStdd);

  for (int i = 0 ; i < 5 ; ++i) {
    std::set<int> s;
    s.insert(0);
    s.insert(1);
    s.insert(2);
    int j = 0;
    while (! s.empty()) {
      /* argmin_ci(fi(S)) */
      //std::map<std::vector<double> , int> fi;

      std::set<int>::iterator it = s.begin();
      std::set<int>::iterator itEnd = s.end();

      std::map<double,int> norm;
      for ( ; it != itEnd ; ++it){
	double n = 0;
	for( int p = i*4 ; p < i*4+4 ; ++p){
	  const double *r = centroid.ptr<double>(*it);
	  const double val = r[p]; 
	  //n += val * val;
	  n += val /4 ;
	}
	norm.insert(std::pair<double, int>(/*std::sqrt(*/n/*)*/,*it));
      }
      int Ck = norm.begin()->second;
      
      s.erase(Ck);
      
      score[cat[i][j]][Ck]++;
      j++;
    }
  }

  for (int j = 0 ; j < 3 ; ++j){
    std::cout<<score[j][0]<<" "<<score[j][1]<<" "<<score[j][2]<<std::endl;
  }
  
  std::vector<int> res(3, 0);
  //std::vector<int> error;
  //std::vector<int> errorInd;

  for (int i = 0 ; i < 3 ; ++i){
    int max = -1;
    for (int j = 0 ; j < 3 ; ++j){
      if(score[i][j]>max){
	max = score[i][j];
	res[i] = j;
      }
    }
  }
  
  // //erreur detection
  // for (int i = 0 ; i < 3 ; ++i){
  //   int max = 0;
  //   for (int j = 0 ; j < 3 ; ++j){
  //     score[i][j] > max ? max = score[i][j] :0 ;
  //   }
  //   int nb = 0;
  //   for (int j = 0 ; j < 3 ; ++j)
  //     score[i][j] == max ? ++nb : 0 ;
  //   if (nb > 1){
  //     error.push_back(i);
  //   }
  // }
    
  
  // if( error.size() > 2)
  //   std::cout<<"erreur for the scoring"<<std::endl;

  // if (error.size() == 1) {
  //   std::cout<<"erreur ind="<<error[0]<<std::endl;
  //   std::set<int> ind;
  //   ind.insert(0);
  //   ind.insert(1);
  //   ind.insert(2);

  //   for (int i = 0 ;i < 3 ; ++i){
  //     if (i != error[0]){
  // 	ind.erase(res[i]);
  //     }
  //   }
  //   std::cout<<"reste ind="<<(*(ind.begin()))<<std::endl;
  //   if (ind.size() > 0 && ind.size() < 2)
  //     res[error[0]] = *(ind.begin());
    
  // }
  
  std::cout<<"resultat score "<<res[0]<<" "<<res[1]<<" "<<res[2]<<std::endl;
  

  m_cat = std::vector<int>(3,0);
  if (m_type == "text")
    m_cat[res[0]] = 1;
  if (m_type == "space")
    m_cat[res[1]] = 1;
  if (m_type == "image")
    m_cat[res[2]] = 1;
  
  std::cout<<"resultat cat"<<m_cat[0]<<" "<<m_cat[1]<<" "<<m_cat[2]<<std::endl;

  //B: delete all allocated objects //???
  delete splitAlgo;
  delete driver;
  delete clust;
  delete segContext;
  delete tog;
}

void
SegmentationClusteringTextureBasedLin::computeGLCM(ise::SegmentationContext *segC, girl::regionDeque &createdRegions, std::vector<double> &features)
{
  size_t size = createdRegions.size();

  features.clear();
  features.resize(NB_GLCM * NB_FEATURES * size , 0);

  ise::GrayLevelCooccurrenceMatrix vg(segC);
  for (size_t i = 0 ; i < size ; ++i) {
    int id = createdRegions[i];
    int pos = i * NB_GLCM * NB_FEATURES;
    m_regionIdFeature[id] = i;

    for (int j = 0 ; j < NB_GLCM ; ++j) {
      vg.clear();
      vg.compute(id, static_cast<OrientationGLCM>(j));
      features[pos + 0 * NB_GLCM + j] = vg.energy();
      features[pos + 1 * NB_GLCM + j] = vg.entropy();
      features[pos + 2 * NB_GLCM + j] = vg.sumEntropy();
      features[pos + 3 * NB_GLCM + j] = vg.diffEntropy();
      features[pos + 4 * NB_GLCM + j] = vg.stdd();
    }
  }
}

void
  SegmentationClusteringTextureBasedLin::normalizedFeatures(const std::vector<double> &features, cv::Mat &normFeatures, int size)
{
  size_t sizeFeature = NB_GLCM * NB_FEATURES;
  size_t nbFeatures = size;

  std::vector<double> meanFeature(sizeFeature, 0);

  for (size_t i = 0 ; i < nbFeatures  ; ++i) {
    for (size_t j = 0 ; j < sizeFeature ; ++j){
      meanFeature[j] += features[i * sizeFeature +j]/nbFeatures;
    }
  }

  std::vector<double> stddFeature(sizeFeature,0);

  const double inv_nbFeaturesm1 = 1.0/ (nbFeatures-1);
  for (size_t i = 0 ; i < nbFeatures  ; ++i){
    for (size_t j = 0 ; j < sizeFeature ; ++j){
      double val = features[i * sizeFeature + j] - meanFeature[j];
      val = val * val;
      stddFeature[j] += std::sqrt(val * inv_nbFeaturesm1);
    }
  }

  for (size_t i = 0 ; i < nbFeatures ; ++i) {
    for (size_t j = 0 ; j < sizeFeature ; ++j){
      if( stddFeature[j] == 0)
	stddFeature[j] = 1;
      const double inv_stddFeature = 1.0 / stddFeature[j];
      normFeatures.at<double>(i, j) = (features[i * sizeFeature +j] - meanFeature[j]) * inv_stddFeature;
    }
  }
}

void 
SegmentationClusteringTextureBasedLin::setParameters(int blockHeight, int blockWidth, std::string type)
{
   m_blockHeight = blockHeight;
   m_blockWidth = blockWidth;
   m_type = type;
} 

void 
SegmentationClusteringTextureBasedLin::getParameters(int &blockHeight, int &blockWidth, std::string &type) const
{
   blockHeight = m_blockHeight;
   blockWidth = m_blockWidth;
   type = m_type;
} 

bool
SegmentationClusteringTextureBasedLin::same(const girl::point &p1, girl::direction dir)
{
  girl::point p2 = p1;
  p2.move(dir);

  int regionId1 = m_imageLabel.regionId(p1);
  int regionId2 = m_imageLabel.regionId(p2);

  int label1 = m_label[m_regionIdFeature[regionId1]];
  int label2 = m_label[m_regionIdFeature[regionId2]];

  if( m_cat[label1] == m_cat[label2])
    return true;
  else 
    return false;
}
