/*
  Copyright (C) 2005-2011 Roigt Julian, Boris Mansencal  boris.mansencal@labri.fr
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ISESEGMENTATIONCLUSTERINGTEXTUREBASEDLIN_HPP
#define ISESEGMENTATIONCLUSTERINGTEXTUREBASEDLIN_HPP

#include <girl/girlToger.hpp>
#include <ise/iseSegmentationClustering.hpp>
#include <ise/iseBoxe.hpp>
#include <ise/iseGenericImage.hpp>
#include <ise/iseImageLabelToger.hpp>

#include <opencv2/core/core.hpp>


namespace ise {

  class SegmentationContext;
  
  class  GIRL_API SegmentationClusteringTextureBasedLin : public SegmentationClustering
  {
  public :

    explicit SegmentationClusteringTextureBasedLin(SegmentationContext *segContext);
    
    virtual ~SegmentationClusteringTextureBasedLin() {}

    virtual void setSegContext(ise::SegmentationContext *s);

    virtual void setParameters(int nbBlockHeight, int nbBlockWidth, std::string type);

    virtual void getParameters(int &nbBlockHeight, int &nbBlockWidth, std::string &type) const;
    
    virtual void init(const girl::regionDeque &regions, int level=1);

    virtual void init(girl::region /*r*/, int /*level*/) {} 
    
    virtual bool same(const girl::point &p, girl::direction dir);

  protected:
    void normalizedFeatures(const std::vector<double> &features, cv::Mat &normFeatures, int size);
    void computeGLCM(ise::SegmentationContext *segC, girl::regionDeque &createdRegions, std::vector<double> &features);

  private:
    int m_blockHeight;
    int m_blockWidth;
    std::string m_type;
    int m_nbBlockHeight;
    int m_nbBlockWidth;
    std::vector<unsigned int> m_label;
    std::vector<unsigned int> m_regionIdFeature;
    std::vector<int> m_cat;
    ise::ImageLabelToger m_imageLabel;
  };

}

#endif /* ! ISESEGMENTATIONCLUSTERINGTEXTUREBASEDLIN_HPP */
