/*
  Copyright (C) 2005-2011 Remi Vieux, Boris Mansencal  remi.vieux@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#include "SegmentationClusteringWordWidget.hpp"

#include <girl/girlToger.hpp>

#include <ise/iseConnectedComponentTools.hpp>
#include <ise/iseForegroundBackgroundClassifierColorArea.hpp>
#include <ise/iseForegroundBackgroundClassifierHoles.hpp>
#include <ise/iseNeighbouringRegions.hpp>

#include <qiseTools/Histogram1DThresholdsWidget.hpp>
#include <qiseTools/OrientationChoiceWidget.hpp>
#include <qiseTools/PluginRegistry.hpp>

#include <QComboBox>
#include <QHBoxLayout>
#include <QVBoxLayout>

#include <opencv2/core/core.hpp>
#include <opencv2/imgproc/imgproc.hpp>

SegmentationClusteringWordWidget::SegmentationClusteringWordWidget(MainView *mainView,
								   SegmentationClusteringPlugin *factory,
								   QWidget *parent)
: SegmentationClusteringWidgetInterface(mainView, factory, parent),
  m_histoView(NULL),
  m_orientationChoiceWidget(NULL),
  m_thresholdersCombo(NULL),
//  m_classifiersCombo(NULL),
  m_histo(),
 // m_selectedRegs(),
  m_threshold(0),
  m_spacings(),
  m_ccLabels()
{
  buildGUI();
}

SegmentationClusteringWordWidget::~SegmentationClusteringWordWidget()
{
}


void
SegmentationClusteringWordWidget::buildGUI()
{
  m_histoView = new Histogram1DThresholdsWidget(this);
  m_orientationChoiceWidget = new OrientationChoiceWidget(this);
  m_thresholdersCombo = new QComboBox(this);
  m_histo = new ise::Histogram1D();

//  m_classifiersCombo = new QComboBox(this);
//  m_classifiersCombo->addItem(tr("Color/Area"));
//  m_classifiersCombo->addItem(tr("Holes"));
//  m_classifiersCombo->setCurrentIndex(0);
//  computeSelectedRegions(0);

  QIsePluginList<QuantificationThresholderPlugin*> plugins = PluginRegistry::getQuantificationThresholderPlugin();
  QIsePluginList<QuantificationThresholderPlugin* >::const_iterator pluginsIt = plugins.begin();
  const QIsePluginList<QuantificationThresholderPlugin* >::const_iterator pluginsItEnd = plugins.end();
  for (; pluginsIt != pluginsItEnd; ++pluginsIt) {
      m_thresholdersCombo->addItem((*pluginsIt)->name());
  }

  QVBoxLayout *vLayout = new QVBoxLayout;
  QHBoxLayout *hLayout = new QHBoxLayout;
  //hLayout->addWidget(m_classifiersCombo);
  hLayout->addWidget(m_orientationChoiceWidget);
  hLayout->addWidget(m_thresholdersCombo);
  vLayout->addLayout(hLayout);
  vLayout->addWidget(m_histoView);
  vLayout->addStretch(0);

  connect(m_orientationChoiceWidget, SIGNAL(orientationChanged(int)),
      this, SLOT(computeHistogram(int)));
  connect(m_thresholdersCombo, SIGNAL(currentIndexChanged(int)),
      this, SLOT(computeThreshold(int)));
//  connect(m_classifiersCombo, SIGNAL(currentIndexChanged(int)),
//      this, SLOT(computeSelectedRegions(int)));

  m_orientationChoiceWidget->setOrientation(ise::RunLengthExtractor::HORIZONTAL);
  computeHistogram(0);


  this->setLayout(vLayout);

}


//ise::ForegroundBackgroundClassifierFactory*
//SegmentationClusteringWordWidget::getFactory(int index) const
//{
//  switch (index) {
//  case 0:
//    return &(ise::ForegroundBackgroundClassifierColorAreaFactory::instance());
//    break;
//  case 1:
//    return &(ise::ForegroundBackgroundClassifierHolesFactory::instance());
//    break;
//  default:
//    assert(false);
//    return NULL;
//    break;
//  }
//}


void
SegmentationClusteringWordWidget::getParameters(int &threshold, cv::Mat &ccLabels, std::vector<ise::NeighbouringRegions> &neighbours) const
{

  std::vector<double> thresholds;
  m_histoView->thresholds(thresholds);
  if (! thresholds.empty()) {
    threshold = static_cast<int>(thresholds[0]);
  } 
  else {
    threshold = m_threshold;
  }

  ccLabels = m_ccLabels;
  neighbours = m_spacings;
}

void
SegmentationClusteringWordWidget::setParameters(const int /*threshold*/, const cv::Mat &/*ccLabels*/, const std::vector<ise::NeighbouringRegions> &/*neighbours*/)
{
  // TODO: See what is pertinent. Some parameters might be useful to
  // save in the settings while other are useful to pass directly to
  // the algorithm (precomputed) but not reset here
/*  m_orientationChoiceWidget->setOrientation(ori);
  m_threshold = threshold;
  m_histoView->removeAllThresholds();
  m_histoView->addThreshold(static_cast<double>(m_threshold));
  */
}



class Painter : public girl::toger::pointIterator {

public:
  Painter(cv::Mat &mat, cv::Scalar color) :
    m_mat(mat), m_color(color)
  {}
  virtual ~Painter() {}

  void processPoint(const girl::point &pt)
  {
    cv::circle(m_mat, cv::Point(pt.x(), pt.y()), 0, m_color);
  }

private:
  cv::Mat &m_mat;
  cv::Scalar &m_color;
};


//void
//SegmentationClusteringWordWidget::computeSelectedRegions(int classifierIndex)
//{
//  assert(m_mainView);
//  assert(m_mainView->model());
//  assert(m_mainView->model()->getSegContextPtr());
//  assert(m_mainView->model()->getSegContextPtr()->getImage());
//
//  if (classifierIndex == -1)
//    return;
//
//  ise::ForegroundBackgroundClassifierFactory *factory = getFactory(classifierIndex);
//  girl::shared_ptr<ise::ForegroundBackgroundClassifier> classifier = factory->create(m_mainView->model()->getSegContextPtr());
//  girl::toger *tog = m_mainView->model()->getToger();
//  girl::regionDeque regs;
//  tog->allRegions(regs);
//  girl::regionDeque bg;
//  classifier->classify(regs, m_selectedRegs, bg);
//
//  m_mainView->model()->setSelectedRegions(m_selectedRegs);
//}

void
SegmentationClusteringWordWidget::computeHistogram(int/* orientation*/)
{

  //if (m_orientationChoiceWidget->getOrientation() == -1)
  //return;

  // TODO:OPTIM
  // I have to compute twice the same thing, once here for visualization and
  // once during the actual algorithm...
  assert(m_mainView);
  assert(m_mainView->model());
  assert(m_mainView->model()->getSegContextPtr());
  assert(m_mainView->model()->getSegContextPtr()->getImage());

  girl::regionDeque selectedRegs = m_mainView->model()->getSelectedRegions();

  if (!selectedRegs.size()) {
       QMessageBox::critical(this, tr("Error"), tr("No foreground regions selected!"));
       return;
   }

  girl::toger *tog = m_mainView->model()->getToger();
  girl::tools::Image *image = m_mainView->model()->getSegContextPtr()->getImage();

  cv::Mat mat = cv::Mat::zeros(image->height(), image->width(), CV_8U);
  Painter painter(mat, CV_RGB(255,255,255));

  girl::regionDeque::const_iterator it = selectedRegs.begin();
  const girl::regionDeque::const_iterator itEnd = selectedRegs.end();
  for (; it != itEnd; ++it) {
      tog->iterateOnRegionDomain(*it, painter);
  }

  cv::Mat tmp = mat.clone();
  size_t nbCC = ise::ConnectedComponentTools::extractAllConnectedComponents(tmp, m_ccLabels, 8);
  
  if (nbCC > 1) {
    // need at least 2 CC to compute a distance btw them!
    mat.copyTo(tmp);
    ise::ConnectedComponentTools::computeSpacing(tmp, m_ccLabels, nbCC, m_spacings, m_orientationChoiceWidget->getOrientation());
    
    ise::ConnectedComponentTools::computeDistanceHistogram(m_spacings, *m_histo);
    
    m_histoView->setHistogram(*m_histo);
    m_thresholdersCombo->setCurrentIndex(std::max(0, m_thresholdersCombo->currentIndex()));
    computeThreshold(m_thresholdersCombo->currentIndex());
    
  }
}

void
SegmentationClusteringWordWidget::computeThreshold(int thresholderIndex)
{
  if (thresholderIndex == -1)
    return;

  m_histoView->removeAllThresholds();
  QuantificationThresholderPlugin* plugin = PluginRegistry::getQuantificationThresholderPlugin().at(thresholderIndex);
  plugin->setSegContext(m_mainView->model()->getSegContextPtr());
  girl::shared_ptr<ise::QuantificationThresholder> thresholder = plugin->getNewQuantificationThresholder();

  std::vector<double> thresholds;
  thresholder->getThresholds(*m_histo, thresholds);

  std::vector<double>::const_iterator it = thresholds.begin();
  const std::vector<double>::const_iterator itEnd = thresholds.end();
  for (; it != itEnd; ++it) {
      m_histoView->addThreshold(static_cast<double>(*it));
  }

  m_threshold = thresholds[0];
}
