/*
  Copyright (C) 2010-2011 Remi Vieux, Boris Mansencal, J.-P. Domenger  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#ifndef RECURSIVESPLITMERGEALGORITHMPLUGIN_HPP
#define RECURSIVESPLITMERGEALGORITHMPLUGIN_HPP

#include <qiseTools/SplitMergeAlgorithmPlugin.hpp>

#include <qiseTools/SplitAlgorithmPlugin.hpp>
#include <qiseTools/MergeAlgorithmPlugin.hpp>

#include <ise/iseSplitMergeAlgorithmRecursive.hpp>

#include <QObject>

class RecursiveSplitMergeAlgorithmPlugin : public QObject,
					   public SplitMergeAlgorithmPlugin
{
  
  Q_OBJECT
  Q_INTERFACES(SplitMergeAlgorithmPlugin QIsePlugin)
#if QT_VERSION >= 0x050000
  Q_PLUGIN_METADATA(IID SplitMergeAlgorithmPlugin_iid)
#endif
    
 public:
    
  ~RecursiveSplitMergeAlgorithmPlugin() {}
  
  RecursiveSplitMergeAlgorithmPlugin()
    : SplitMergeAlgorithmPlugin(QString::fromUtf8("Recursive splitMerge algorithm"),
				QIseVersion(1, 0), QIseVersion(1, 0)),
      m_splitAlgo(NULL), m_mergeAlgo(NULL)
  {}
  
  virtual girl::shared_ptr<ise::SplitMergeAlgorithm>
    create(ise::SegmentationContext *segContext)
  {
    girl::shared_ptr<ise::SplitMergeAlgorithm> result(new ise::SplitMergeAlgorithmRecursive(segContext));

    if (m_splitAlgo)
      result->setSplitAlgorithm(m_splitAlgo->getNewSplitAlgorithm(segContext));
    if (m_mergeAlgo)
      result->setMergeAlgorithm(m_mergeAlgo->getNewMergeAlgorithm(segContext));
    
    return result;
  }
  
  virtual girl::shared_ptr<SplitMergeAlgorithmSettings>
  createSettings(ise::SegmentationContext * /*segContext*/)
  {
    return girl::shared_ptr<SplitMergeAlgorithmSettings>();//NULL
  }

  virtual bool hasSettings() const
  {
    return false;
  }


  virtual void setSplitAlgorithmPlugin(SplitAlgorithmPlugin* splitAlgo) {
    m_splitAlgo = splitAlgo;
  }
  
  virtual void setMergeAlgorithmPlugin(MergeAlgorithmPlugin* mergeAlgo) {
    m_mergeAlgo = mergeAlgo;
  }
  
protected:

  virtual SplitMergeAlgorithmWidgetInterface *createWidget(ise::SegmentationContext * /*segContext*/, QWidget * /*parent*/)
  {
    return NULL;
  }
  
protected:
  
  SplitAlgorithmPlugin* m_splitAlgo;
  MergeAlgorithmPlugin* m_mergeAlgo;
  
};

#if QT_VERSION < 0x050000
Q_EXPORT_PLUGIN2(QIseRecursiveSplitMergeAlgorithmPlugin, RecursiveSplitMergeAlgorithmPlugin)
#endif

#endif /* ! RECURSIVESPLITMERGEALGORITHMPLUGIN_HPP */
