/*
  Copyright (C) 2010-2011 Cedric Aguerre  aguerre@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#ifndef EIGENSOLVER_HPP
#define EIGENSOLVER_HPP

#include <string>
#include <SparseMatrix.hpp>

#ifdef NCUT_STANDALONE
#define GIRL_API
#else
#include <girl/girlCommon.hpp>
#endif

namespace ncut {

  class GIRL_API EigenSolver
  {
  public:
    virtual ~EigenSolver()
    {}

    virtual const std::string &getName() const = 0;

    /*
     * Solve for eigenvalue problem Av = lv of symetric, sparse matrix mat.
     * The looping procedure and final extraction of eigenvalues and vectors
     * is handled automatically.
     * Compute both the nbEigenvalues largest eigenvalues and corresponding
     * eigenvectors of sparse matrix mat.
     *
     * nbEigenvalues: the number of eigenvalues to be found, starting at the
     *                bottom.  Note that the highest eigenvalues, or some
     *                other choices, can be found.  For now, the choice of
     *                the highest nbEigenvalues eigenvalues is hard-coded.
     * eVals: a one-dimensional array of length nbEigenvalues to hold the
     *        eigenvalues.
     * eVecs: a two-dimensional array of size nbEigenvalues by (matrix order)
     *        to hold the eigenvectors. Note that the vectors are stored
     *        as columns of Evecs, so that the elements of vector i are the
     *        values Evecs[j][i].
     */
    virtual void computeEVV(ncut::SparseMatrix *mat, int nbEigenvalues, double *eVals, double **eVecs) = 0;
  };

}

#endif /*EIGENSOLVER_HPP*/
