/*
  Copyright (C) 2010-2011 Cedric Aguerre  aguerre@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#ifndef NCUTKERNEL_HPP
#define NCUTKERNEL_HPP

#include <NCutManager.hpp>
#include <NCutPlugin.hpp>

#ifdef NCUT_STANDALONE
#define GIRL_API
#else
#include <girl/girlCommon.hpp>
#endif

#include <string>
#include <map>

namespace ncut {

  // The NCut's system core
  class GIRL_API NCutKernel
  {
  public:
    EigenSolverManager &getEigenSolverManager()
    {
      return m_eigenSolverManager;
    }
    SVDSolverManager &getSVDSolverManager()
    {
      return m_SVDSolverManager;
    }
    FourierManager &getFourierManager()
    {
      return m_FourierManager;
    }
    PixelSimilarityManager &getPixelSimilarityManager()
    {
      return m_pixelSimilarityManager;
    }

    // TODO: manage this in a different way (do not use index 0)
    EigenSolver *getEigenSolver()
    {
      return m_eigenSolverManager.getT(0);
    }
    SVDSolver *getSVDSolver()
    {
      return m_SVDSolverManager.getT(0);
    }
    Fourier *getFourier()
    {
      return m_FourierManager.getT(0);
    }
    PixelSimilarity *getPixelSimilarity()
    {
      return m_pixelSimilarityManager.getT(0);
    }

    // Loads a plugin
    void loadPlugin(const std::string &path, const std::string &pluginName);

  private:
    // Map of plugins by their associated file names
    typedef std::map<std::string, NCutPlugin> PluginMap;

    PluginMap m_loadedPlugins;  // All plugins currently loaded
    EigenSolverManager m_eigenSolverManager;
    SVDSolverManager m_SVDSolverManager;
    FourierManager m_FourierManager;
    PixelSimilarityManager m_pixelSimilarityManager;
  };

}

#endif /*NCUTKERNEL_HPP*/
