/*
  Copyright (C) 2010-2011 Cedric Aguerre  aguerre@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#ifndef NCUTMANAGER_HPP
#define NCUTMANAGER_HPP

#include <EigenSolver.hpp>
#include <SVDSolver.hpp>
#include <Fourier.hpp>
#include <PixelSimilarity.hpp>

#ifdef NCUT_STANDALONE
#define GIRL_API
#else
#include <girl/girlCommon.hpp>
#endif

#include <vector>
#include <memory>

namespace ncut {

  template <typename T>
  class GIRL_API NCutManager
  {
  public:
    // Destructor
    ~NCutManager()
      {
      for (reverse_iterator it = m_Ts.rbegin(); it != m_Ts.rend(); ++it)
	delete *it;
    }

    // Allows plugins to add new Ts
    void addT(std::auto_ptr<T> s)
    {
      m_Ts.push_back(s.release());
    }

    // Get the total number of registered Ts
    size_t getDriverCount() const
    {
      return m_Ts.size();
    }

    // Access a T by its index
    T *getT(size_t index)
    {
      return m_Ts.at(index);
    }

  private:
    // A vector of Ts
    typedef std::vector<T *> TVector;
    typedef typename TVector::reverse_iterator reverse_iterator;

    TVector m_Ts; // All available Ts
  };

  typedef NCutManager<EigenSolver> EigenSolverManager;
  typedef NCutManager<SVDSolver> SVDSolverManager;
  typedef NCutManager<Fourier> FourierManager;
  typedef NCutManager<PixelSimilarity> PixelSimilarityManager;

}

#endif /*NCUTMANAGER_HPP*/
