/*
  Copyright (C) 2010-2011 Cedric Aguerre  aguerre@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#ifndef NCUTPLUGIN_HPP
#define NCUTPLUGIN_HPP

#include <string>

#ifdef NCUT_STANDALONE
#define GIRL_API
#else
#include <girl/girlCommon.hpp>
#endif

#if defined _WIN32 || defined __CYGWIN__
#include <windows.h>
#else
#include <dlfcn.h>
#endif

namespace ncut {

  class NCutKernel;

  class GIRL_API NCutPlugin
  {
  public:
    // Initialize and load plugin
    NCutPlugin(const std::string &filename);
    // Copy existing plugin instance
    NCutPlugin(const NCutPlugin &x);
    // Unload a plugin
    ~NCutPlugin();

    NCutPlugin &operator=(const NCutPlugin &x);

    // Query the plugin for its expected version
    int getVersion()
    {
      return m_pfnGetVersion();
    }

    // Register the plugin to a kernel
    void registerPlugin(NCutKernel &k)
    {
      m_pfnRegisterPlugin(k);
    }

  private:

#if defined _WIN32 || defined __CYGWIN__
    typedef HMODULE PHANDLE;
    inline FARPROC extractContents(PHANDLE h, LPCSTR c)
    {
      return GetProcAddress(h, c);
    }
    inline void unload(PHANDLE h)
    {
      FreeLibrary(h);
    }
#else
    typedef void* PHANDLE;
    inline intptr_t extractContents(PHANDLE h, const char *c)
    {
      return reinterpret_cast<intptr_t>(dlsym(h, c));
    }
    inline void unload(PHANDLE h)
    {
      dlclose(h);
    }
#endif

    // Signature for the version query function
    typedef int fnGetVersion();
    // Signature for the plugin's registration function
    typedef void fnRegisterPlugin(NCutKernel &);

    PHANDLE m_hDLL; // Win32 DLL handle
    size_t *m_pDLLRefCount; // Number of references to the DLL
    fnGetVersion *m_pfnGetVersion; // Version query function
    fnRegisterPlugin *m_pfnRegisterPlugin; // Plugin registration function
  };

}

#endif /*NCUTPLUGIN_HPP*/
