/*
  Copyright (C) 2010-2011 Cedric Aguerre  aguerre@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#ifndef NCUTSEGMENTER_HPP
#define NCUTSEGMENTER_HPP

#include <NCutKernel.hpp>
#include <NCutImage.hpp>
#include <SparseMatrix.hpp>
#include <EigenSolver.hpp>
#include <SVDSolver.hpp>
#include <ArrayManager.hpp>
#include <memory>

#ifdef NCUT_STANDALONE
#define GIRL_API
#else
#include <girl/girlCommon.hpp>
#include <ise/iseGenericImage.hpp>
#endif

namespace ncut {

  typedef std::tr1::shared_ptr<ncut::SparseMatrix> MatrixPtr;

  class GIRL_API NCutSegmenter
  {
  public:
    NCutSegmenter();
    ~NCutSegmenter();

    // Return a label map of segmented image in the range [1:nbSegments]
    // The label map is in (row,col) order, matching the dimension of segmented image
    IntPtr segment(std::string filename, int nbSegments);

#ifndef NCUT_STANDALONE
    ncut::NCutImage::ImageEdges extractEdges(const ise::GenericImage<int, 1> &img);
    IntPtr segment(const ise::GenericImage<int, 1> &img, int nbSegments);
#endif

    IntPtr segment(MatrixPtr W, int nbSegments);

  private:
    IntPtr segment(const ncut::NCutImage &img, int nbSegments);
    void ncut(MatrixPtr W, ncut::EigenSolver &eigenSolver, int nbSegments);
    void discretize(int nbSegments, SVDSolver &svdSolver);

  private:
    NCutKernel m_kernel;
    double *m_evals;
    double **m_evecs;
    int m_nc;
    unsigned char **m_evecsDiscrete;
  };

}

#endif /*NCUTSEGMENTER_HPP*/
