/*
  Copyright (C) 2010-2011 Cedric Aguerre  aguerre@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#ifndef SVDSOLVER_HPP
#define SVDSOLVER_HPP

#include <string>

#ifdef NCUT_STANDALONE
#define GIRL_API
#else
#include <girl/girlCommon.hpp>
#endif

namespace ncut {

  class GIRL_API SVDSolver
  {
  public:
    virtual ~SVDSolver()
    {}

    virtual const std::string &getName() const = 0;

    /*
     * Solve for the Singular Value Decomposition (SVD) of a
     * rectangular matrix A.
     *
     * A: the m by n matrix that we are decomposing
     * m: the number of rows in A
     * n: the number of columns in A (generally, n<m)
     * S: a min(m,n) element array to hold the singular values of A
     * U: a [m, min(m,n)] element rectangular array to hold the right
     * singular vectors of A.  These vectors will be the columns of U,
     * so that U[i][j] is the ith element of vector j.
     * VT: a [min(m,n), n] element rectangular array to hold the left
     * singular vectors of A.  These vectors will be the rows of VT
     * (it is a transpose of the vector matrix), so that VT[i][j] is
     * the jth element of vector i.
     *
     * Note that S, U, and VT must be initialized before calling this
     * routine, or there will be an error.  Here is a quick sample piece of
     * code to perform this initialization; in many cases, it can be lifted
     * right from here into your program.
     *
     * S = new double[minmn];
     * U = new double*[m]; for (int i=0; i<m; ++i) U[i] = new double[minmn];
     * VT = new double*[minmn]; for (int i=0; i<minmn; ++i) VT[i] = new double[n];
     */
    virtual void computeSVD(double **A, int m, int n, double *S, double **U, double **VT) = 0;
  };

}

#endif /*SVDSOLVER_HPP*/
