/*
  Copyright (C) 2010-2011 Cedric Aguerre  aguerre@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#ifndef ARRAYMANAGER_HPP
#define ARRAYMANAGER_HPP

#include <tr1/memory>

#ifdef NCUT_STANDALONE
#define GIRL_API
#else
#include <girl/girlCommon.hpp>
#endif

namespace ncut {

  namespace {// anonymous

    // Delete array objects (user-defined deletor used by shared_ptr)
    template <typename ArrayType>
    class ArrayDeleter
    {
    public:
      ArrayDeleter()
      { }

      void operator()(ArrayType *res) {
	delete [] res;
      }
    };

  }

  template <typename ArrayType>
  class GIRL_API ArrayManager
  {
  public:
    typedef std::tr1::shared_ptr<ArrayType> ArrayPtr;

    ArrayManager()
      { }
        
    // return a new Array of size n
    virtual ArrayPtr New(int n) const {
      ArrayPtr r(new ArrayType[n], ArrayDeleter<ArrayType>());
      return r;
    }

  private:
    // Prevents from object copy
    ArrayManager(const ArrayManager &);
    const ArrayManager &operator=(const ArrayManager &);
  };


  // Some useful typedef
  typedef std::tr1::shared_ptr<double> DoublePtr;
  typedef std::tr1::shared_ptr<int> IntPtr;

}

#endif /*ARRAYMANAGER_HPP*/
