/*
  Copyright (C) 2010-2011 Cedric Aguerre  aguerre@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#ifndef FILTERBANK_HPP
#define FILTERBANK_HPP

#include <tr1/memory>
#include <vector>

#ifdef NCUT_STANDALONE
#define GIRL_API
#else
#include <girl/girlCommon.hpp>
#endif

namespace ncut {

  class GIRL_API FilterBank
  {
  public:

    FilterBank(int nFilter, double *filterScales, int nScale, int winSz, bool odd, int enlong = 3);
    FilterBank(int nFilter, double scale, int winSz, bool odd, int enlong = 3);

    ~FilterBank();

    inline int getWinSz() const
    {
      return m_winSz;
    }

    inline double *getFilter(int i) const
    {
      return m_filters->at(i);
    }

    inline int getNbFilters() const
    {
      return m_nFilter * m_nScale;
    }

  private:
    double *doog(double sigma, double r, double theta, int w) const;
    double *gaussian(double *x, int N, double *m, double *c) const;
    void makeFilterBank();

  private:
    std::vector<double *> *m_filters;

#define MAX_N_SCALE 8
    double m_filterScales[MAX_N_SCALE];
    int m_nFilter, m_nScale, m_winSz, m_enlong;
    bool m_odd;
  };

}

#endif /*FILTERBANK_HPP*/
