/*
  Copyright (C) 2010-2011 Cedric Aguerre  aguerre@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#ifndef NCUTIMAGE_HPP
#define NCUTIMAGE_HPP

#include <Fourier.hpp>
#include <FilterBank.hpp>
#include <string>
#include <tr1/memory>
#include <vector>
#include <ArrayManager.hpp>

#ifdef NCUT_STANDALONE
#define GIRL_API
#else
#include <girl/girlCommon.hpp>
#endif

namespace ncut {

  class GIRL_API NCutImage
  {
  public:

    struct ImageEdges {
      DoublePtr mag;
      DoublePtr pha;
    };
    /*
      struct EdgePoint {
      double mag, pha;
      };
    */
    typedef std::vector<std::tr1::shared_ptr<NCutImage> > ImageVector;

    NCutImage();
    NCutImage(std::string filename);
    NCutImage(int width, int height, double *data = NULL);
    ~NCutImage();

    inline double *data() const
    {
      return m_data;
    }
    inline int width() const
    {
      return m_width;
    }
    inline int height() const
    {
      return m_height;
    }

    double getMedianValue() const;

    NCutImage *growWithMirors();

    std::tr1::shared_ptr<ImageVector> FourierFilter(const ncut::Fourier &fourier, const ncut::FilterBank &FB);
    // compute edges magnitude&phase
    // nFilter: number of filter orientations
    // scale: filter scale
    // winSz: filter size
    // enlong: elongation
    ImageEdges computeEdges(const ncut::Fourier &fourier, int nFilter = 4, double scale = 3, int winSz = 21, int enlong = 3) const;

    void save(std::string filename);

    void getMinMax(double &mini, double &maxi) const;

    NCutImage &normalize(); // in [0;255]. Return a reference to self

  private:
    double *m_data;
    int m_width, m_height;

    NCutImage(const NCutImage &);
    NCutImage &operator=(const NCutImage &);
  };

}

#endif /*NCUTIMAGE_HPP*/
