/*
  Copyright (C) 2010-2011 Cedric Aguerre  aguerre@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#include <cstdlib>
#include <NCutImage.hpp>
#include <NCutSegmenter.hpp>

#include <opencv2/core/core.hpp>
#include <opencv2/highgui/highgui.hpp>

#include <iostream>

int
main(int argc, char *argv[])
{
  if (argc != 3) {
    std::cerr << "Usage: " << argv[0] << " <filename> <nbClusters>\n";
    return EXIT_FAILURE;
  }

  // load the image
  std::string filename(argv[1]);
  /*
  // the loaded image is forced to be a 1 channel grayscale image
  ncut::NCutImage img(filename);
  std::cout << "Load " << filename << ": " << img.width() << "x" << img.height() << std::endl;
  */
  // compute the edges imageEdges, the similarity matrix W based on
  // Intervening Contours, the Ncut eigenvectors and discrete segmentation
  int nbSegments = atoi(argv[2]);


  ncut::NCutSegmenter segm;
  ncut::IntPtr segLabel = segm.segment(filename, nbSegments);
  int *seg = segLabel.get(); // seg is in (row,col) order

  // save result
  IplImage *img = cvLoadImage(filename.c_str(), 0);
  CvSize size;
  size.width = img->width;
  size.height = img->height;
  cvReleaseImage(&img);

  IplImage *src = cvCreateImage(size, IPL_DEPTH_8U, 1);
  CvScalar s;
  for (int i = 0, k = 0; i < size.width; ++i)
    for (int j = 0; j < size.height; ++j, ++k) {
      s.val[0] = (unsigned char) seg[k];
      cvSet2D(src, j, i, s);
    }
  IplImage *rescale = cvCreateImage(size, IPL_DEPTH_8U, 1);
  cvConvertScale(src , rescale, 255./(double)nbSegments, 0);
  cvReleaseImage(&src);

  cvSaveImage("SegLabel.png", rescale);
  cvReleaseImage(&rescale);

  return 0;
}
