/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "ClearAllMarkCommand.hpp"

#include <iostream>//DEBUG

#include <QApplication>


ClearAllMarkCommand::ClearAllMarkCommand(SegModel *m, 
		      const girl::toger::markFlags mark)
  : UndoCommand(m)
  , m_memento()
  , m_mark(mark)
{
  setText(tr("Clear mark"));
}

void
ClearAllMarkCommand::redo()
{
  assert(doAble());

  QApplication::setOverrideCursor(Qt::WaitCursor);

  girl::toger *tog = m_model->getToger();
  assert(tog);

  //save model 
  m_memento = m_model->save();

  //- Do the unmark
  tog->unmarkAllDarts(m_mark);
  
  m_model->setModified(true);

  QApplication::restoreOverrideCursor();
}

void
ClearAllMarkCommand::undo()
{
  QApplication::setOverrideCursor(Qt::WaitCursor);

  m_model->restore(m_memento);

  QApplication::restoreOverrideCursor();

}

bool
ClearAllMarkCommand::doAble() const
{
 return (m_model 
	 && m_model->selectable()
	 && m_model->getSelectionType() == SegModel::REGION);

}
