/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "ClearSelectionCommand.hpp"

#include <qiseTools/SegModel.hpp>

ClearSelectionCommand::ClearSelectionCommand(SegModel *m)
  : UndoCommand(m),
    m_previousSelection()
{

}

bool
ClearSelectionCommand::doAble() const
{
  return (m_model != NULL
	  && m_model->selectable()
	  && ( (m_model->getSelectionType() == SegModel::REGION && ! m_model->getSelectedRegions().empty())
	      || (m_model->getSelectionType() == SegModel::SEGMENT && ! m_model->getSelectedDarts().empty()) )
	  );
}


void 
ClearSelectionCommand::redo()
{
  assert(doAble());

  m_previousSelection = m_model->getSelectedRegions(); //copy

  m_model->clearSelection();
  setText(tr("Clear selection"));
}

void
ClearSelectionCommand::undo()
{
  m_model->setSelectedRegions(m_previousSelection);
}

bool
ClearSelectionCommand::mergeWith(const QUndoCommand *c)
{
  //REM: Qt recommends to use ids to check if we are dealing with two same commands, 
  //see http://doc.qt.nokia.com/4.7/qundocommand.html
  if (c->text() != text()) 
    return false;
  
  return true;
}
