/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "DragInteractor.hpp"

#include <QApplication>
#include <QDrag>
#include <QMimeData>
#include <QMouseEvent>

#include "MainWindow.hpp"
#include <qiseTools/MainView.hpp>
#include <qiseTools/SegModel.hpp>
#include "SegModelQIO.hpp"


bool
DragInteractor::eventFilter(QObject *target, QEvent *event)
{
  if (target == m_view) {
  
    if (event->type() == QEvent::MouseButtonPress) {
      QMouseEvent *e = (QMouseEvent *)event;
      
        if (e->button() == Qt::LeftButton) {
	  m_dragStarted = true;
	  m_startDragPosition = e->pos();
	  m_startDragTime.start();

	  return true;
	}

    }
    else if (event->type() == QEvent::MouseMove) {
      
      QMouseEvent *e = (QMouseEvent *)event;

      if (m_dragStarted == true
	  && (e->buttons() & Qt::LeftButton)
	  //&& m_view->hasMovableLayer() == false //ensure that a drop is not currently dealt with
	  && ((e->pos() - m_startDragPosition).manhattanLength() >= QApplication::startDragDistance())
	  && (m_startDragTime.elapsed() >= QApplication::startDragTime())) {


	QMimeData *mimeData = SegModelQIO::copySelectedSegments(m_view->model());

	if (mimeData != NULL) {

	  QDrag *drag = new QDrag(m_view);
	  drag->setMimeData(mimeData);
	  
	  QPixmap pixmap(QString::fromLatin1(":/images/dragPart.png"));
	  drag->setPixmap(pixmap);
	  
	  m_dragStarted = false; //ensure that subsequent moves do not trigger a new drag
	  
#ifndef NDEBUG
	  //Qt::DropAction dropAction = 
#endif
#if QT_VERSION >= 0x040300
	  drag->exec(Qt::CopyAction);
#else
	  drag->start(Qt::CopyAction);
#endif
	  
	  return true;
	}

      }
      
    }
    
  }

  return false;

}
