/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef HISTOGRAMINTERVALSCOLORSPACESDIALOG_HPP
#define HISTOGRAMINTERVALSCOLORSPACESDIALOG_HPP

#include "HistogramIntervalsDialog.hpp"

#include <qiseTools/ColorSpacesChannelsChoiceWidget.hpp>

class QRadioButton;
class QTabWidget;

class HistogramIntervalsWidget;


class HistogramIntervalsColorSpacesDialog : public HistogramIntervalsDialog
{
  Q_OBJECT

public:

  /**
   * @brief Destructor.
   */
  virtual ~HistogramIntervalsColorSpacesDialog();

  /**
   * @brief Constructor.
   */
  explicit HistogramIntervalsColorSpacesDialog(QWidget *parent);

  int getColorSpace() const;

  size_t getNbChannelsForColorSpace() const;

  ColorSpacesChannelsChoiceWidget::ChannelFlags getChannels() const;

  size_t getNbCheckedChannels() const;

  
  typedef enum {COMBINE_AND=0, COMBINE_OR=1} Combination;
  Combination getCombination();

  /**
   * Must be inferiror to getNbChannelsForColorSpace().
   */
  void setCurrentChannel(size_t i);

  /**
   * Set histogram for current channel
   */
  virtual void setHistogram(const ise::GrayScaleHistogram &h);

  /**
   * Get target for current channel
   */
  virtual intervals::Target getTarget() const;

  /**
   * Set target for current channel
   */
  virtual void setTarget(intervals::Target t);

  /**
   * Return kept intervals in sorted increasing order for current channel.
   *
   * We have intervals[i].first<=intervals[i].second<intervals[i+1].first
   */
  virtual void getIntervals(intervals::IntervalCollection &intervals, 
			    intervals::ColorCollection &colors) const;

  /**
   * Set intervals for current channel.
   */
  virtual void setIntervals(const intervals::IntervalCollection &intervals,
			    const intervals::ColorCollection &colors);
  

signals:
   
  //void targetChanged(intervals::Target);

  //void intervalsChanged();

  //void accepted();
  //void rejected();

protected slots:

  void changeColorSpace();

  void changeChannels();

  void changeIntervals();


protected:

  void buildGUI();

  void toggleCombinationWidget();


  HistogramIntervalsWidget *getCurrentHIW();
  
  const HistogramIntervalsWidget *getCurrentHIW() const;

protected:

  ColorSpacesChannelsChoiceWidget *m_csWidget;

  QTabWidget *m_tabWidget;

  QWidget *m_combinationWidget;
  QRadioButton *m_andRB;
  QRadioButton *m_orRB;


  size_t m_currentChannel;

};




#endif /* ! HISTOGRAMINTERVALSCOLORSPACESDIALOG_HPP */
