/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "HistogramIntervalsColorSpacesInteractor.hpp"

#include <algorithm>

#include "HistogramIntervalsColorSpacesDialog.hpp"
#include "HistogramIntervalsCSProcessor.hpp"
#include <qiseTools/MainView.hpp>
#include <qiseTools/SegModel.hpp>


HistogramIntervalsColorSpacesInteractor::~HistogramIntervalsColorSpacesInteractor()
{
  deleteProcessors();
}

HistogramIntervalsColorSpacesInteractor::HistogramIntervalsColorSpacesInteractor(MainWindow *mw, MainView *v)
  : HistogramIntervalsProcessingInteractor(mw, v)
{

}

/*
void
HistogramIntervalsColorSpacesInteractor::activate()
{

}

void
HistogramIntervalsColorSpacesInteractor::deactivate()
{

}
*/


void
HistogramIntervalsColorSpacesInteractor::deleteProcessors()
{
  for (size_t i=0; i<m_processors.size(); ++i)
    delete m_processors[i];
  m_processors.resize(0);
}

void
HistogramIntervalsColorSpacesInteractor::setProcessor(HistogramIntervalsProcessor *p)
{
  HistogramIntervalsProcessingInteractor::setProcessor(p);

  deleteProcessors();
  //do we fill m_processors with clones here ???
}


HistogramIntervalsDialog *
HistogramIntervalsColorSpacesInteractor::buildDialog(QWidget *parent)
{
  return new HistogramIntervalsColorSpacesDialog(parent);
}

  
/*
void
HistogramIntervalsColorSpacesInteractor::updateImage(SegModel *)
{
 
}


void
HistogramIntervalsColorSpacesInteractor::updateToger(SegModel *)
{

}

void
HistogramIntervalsColorSpacesInteractor::updateSelection(SegModel *)
{

}



void
HistogramIntervalsColorSpacesInteractor::accept()
{

}
void
HistogramIntervalsColorSpacesInteractor::reject()
{

}

void
HistogramIntervalsColorSpacesInteractor::changeTarget(intervals::Target t)
{

}

void
HistogramIntervalsColorSpacesInteractor::changeIntervals()
{

}
*/


  
void
HistogramIntervalsColorSpacesInteractor::resetProcessors()
{
  assert(m_processor);

  const size_t prevNbChannels = m_processors.size();

  HistogramIntervalsColorSpacesDialog *d=static_cast<HistogramIntervalsColorSpacesDialog *>(m_dialog);

  const size_t newNbChannels = d->getNbChannelsForColorSpace();

  if (prevNbChannels < newNbChannels) {

    m_processors.resize(newNbChannels, 0); //keep previous values
    for (size_t i=prevNbChannels; i<newNbChannels; ++i) {
      
      m_processors[i] = static_cast<HistogramIntervalsCSProcessor *>(m_processor)->clone();
    }
  }
  else if (prevNbChannels > newNbChannels) {
    for (size_t i=newNbChannels; i<prevNbChannels; ++i) {
      
      delete m_processors[i];
    }
    m_processors.resize(newNbChannels); //keep previous values
  }
  
  assert(m_processors.size() == newNbChannels);

  const int iseCS = d->getColorSpace();

  for (size_t i=0; i<newNbChannels; ++i) {
    
    m_processors[i]->setCSConversion(iseCS, i);
  }

}

void
HistogramIntervalsColorSpacesInteractor::resetHistograms()
{
  HistogramIntervalsColorSpacesDialog *d = static_cast<HistogramIntervalsColorSpacesDialog *>(m_dialog);

  assert(d);	
  assert(d->getNbChannelsForColorSpace() == m_processors.size());
  
  for (size_t i=0; i<m_processors.size(); ++i) {
    
    ise::GrayScaleHistogram h;
    m_processors[i]->getHistogram(h);

    d->setCurrentChannel(i);
    d->setHistogram(h);
  }
 

}

void
HistogramIntervalsColorSpacesInteractor::updateTarget(intervals::Target t)
{
  if (m_processor) {
    assert(m_view);
    assert(m_view->model());
 

    //target has changed: it means that selected regions have changed 
    //OR color space has changed

    //We need to:
    //- reset processors such as we have the same number of processors 
    //  than the number of channels and they all refer to current color space.
    //- update processors with target regions
    //- update dialog with new histograms from processors.

    
    resetProcessors();
 

    const SegModel *m = m_view->model();
    const girl::toger &tog = *(m->getToger());

    if (t == intervals::TARGET_ALL_REGIONS) {
      girl::regionDeque regions;
      tog.allRegions(regions);
      updateTarget(m, regions);
    }
    else if (t == intervals::TARGET_SELECTED_REGIONS) {
      updateTarget(m, m->getSelectedRegions());
    }

    resetHistograms();
  }

}
  
void
HistogramIntervalsColorSpacesInteractor::updateTarget(const SegModel *m, 
						      const girl::regionDeque &regions)
{
  if (m_processor) {
    for (size_t i=0; i<m_processors.size(); ++i) {
      assert(m_processors[i]);
      m_processors[i]->updateTarget(m, regions);
    }
  }

}
  
void
HistogramIntervalsColorSpacesInteractor::getKeptRegions(girl::regionDeque &regions)
{
  
  if (m_processor) {
    
    HistogramIntervalsColorSpacesDialog *d = static_cast<HistogramIntervalsColorSpacesDialog *>(m_dialog);

    const size_t nbChannels = d->getNbChannelsForColorSpace();
    assert(m_processors.size() == nbChannels);

    const size_t nbCheckedChannels = d->getNbCheckedChannels();
    if (nbCheckedChannels > 0) {

      const ColorSpacesChannelsChoiceWidget::ChannelFlags f = d->getChannels();

      //- get first checked channel
      size_t firstCheckedChannel = nbChannels;
      for (size_t i=0; i<nbChannels; ++i) {
	const bool channelChecked = (f & (1<<i));
	if (channelChecked) {
	  firstCheckedChannel = i;
	  break;
	}
      }
      assert(firstCheckedChannel < nbChannels);
      
      //- get regions of first checked channel
      intervals::IntervalCollection intervals;
      intervals::ColorCollection colors;
      
      d->setCurrentChannel(firstCheckedChannel);
      d->getIntervals(intervals, colors);
      
      m_processors[firstCheckedChannel]->getRegionsInIntervals(intervals, regions);
      

      if (nbCheckedChannels > 1) {
	//More than one channel checked
	//we need to test if we want a AND or OR combination of regions
	

	//TODO: we could also use STL algorithms : set_union & set_intersection !!!


	HistogramIntervalsColorSpacesDialog::Combination c = d->getCombination();
	if (c == HistogramIntervalsColorSpacesDialog::COMBINE_AND) {
	  //kept regions are the intersection of each processor kept regions

	  std::sort(regions.begin(), regions.end());
	  
	  for (size_t i=firstCheckedChannel+1; i<nbChannels; ++i) {

	    const bool channelChecked = (f & (1<<i));
	    if (channelChecked) {

	      intervals.clear();
	      colors.clear();
	      
	      d->setCurrentChannel(i);
	      d->getIntervals(intervals, colors);
	      
	      girl::regionDeque regionsT;
	      m_processors[i]->getRegionsInIntervals(intervals, regionsT);
	      
	      std::sort(regionsT.begin(), regionsT.end());

	      //-insert regions both in regions and in regionsT in regionsT2
	      girl::regionDeque regionsT2;
	      
	      girl::regionDeque::const_iterator it1 = regions.begin();
	      const girl::regionDeque::const_iterator itEnd1 = regions.end();
	      girl::regionDeque::const_iterator it2 = regionsT.begin();
	      const girl::regionDeque::const_iterator itEnd2 = regionsT.end();
	      
	      while (it1!=itEnd1 && it2!=itEnd2) {
	      
		if (*it1 < *it2)
		  ++it1;
		else if (*it2 < *it1)
		  ++it2;
		else {
		  //*it1 == *it2
		  
		  regionsT2.push_back(*it2);
		  ++it1;
		  ++it2;
		}
	      }


	      std::swap(regions, regionsT2);
	    }
	  }
	  
	}
	else {
	  assert(c == HistogramIntervalsColorSpacesDialog::COMBINE_OR);
	  //kept regions are the union of each processor kept regions
	  
	  for (size_t i=firstCheckedChannel+1; i<nbChannels; ++i) {

	    const bool channelChecked = (f & (1<<i));
	    if (channelChecked) {

	      intervals.clear();
	      colors.clear();
	      
	      d->setCurrentChannel(i);
	      d->getIntervals(intervals, colors);

	      girl::regionDeque regionsT;
	      m_processors[i]->getRegionsInIntervals(intervals, regionsT);
	    
	      //append
	      girl::regionDeque::const_iterator it = regionsT.begin();
	      const girl::regionDeque::const_iterator itEnd = regionsT.end();
	      for ( ; it != itEnd; ++it)
		regions.push_back(*it);
	    
	    }
	  }

	  //- sort region and unique
	  std::sort(regions.begin(), regions.end());
	  regions.erase(std::unique(regions.begin(), regions.end()), 
			regions.end());

	}



      }


    }

  }

}

/*
void
HistogramIntervalsColorSpacesInteractor::updateDraw()
{

}
*/
