/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "HistogramIntervalsProcessingInteractor.hpp"

#include <ise/iseGrayScaleHistogram.hpp>

#include "MainWindow.hpp"
#include <qiseTools/MainView.hpp>

#include "HistogramIntervalsProcessor.hpp"
#include "RegionsVisualizer.hpp"
#include "RegionsCommander.hpp"
#include "HistogramIntervalsDialog.hpp"

HistogramIntervalsProcessingInteractor::~HistogramIntervalsProcessingInteractor() 
{
  delete m_processor;
  delete m_visualizer;
  delete m_commander;
  
  delete m_dialog;
}

void HistogramIntervalsProcessingInteractor::setProcessor(HistogramIntervalsProcessor *p)
{
  delete m_processor;
  m_processor = p;
}

void
HistogramIntervalsProcessingInteractor::setVisualizer(RegionsVisualizer *v)
{
  delete m_visualizer;
  m_visualizer = v;
}

void
HistogramIntervalsProcessingInteractor::setCommander(RegionsCommander *c)
{
  delete m_commander;
  m_commander = c;
}



void
HistogramIntervalsProcessingInteractor::activate()
{
  if (m_dialog == NULL) {
    
    m_dialog = buildDialog(m_mainWindow);

    connect(m_dialog, SIGNAL(targetChanged(intervals::Target)),
	    this, SLOT(changeTarget(intervals::Target)));
    connect(m_dialog, SIGNAL(intervalsChanged()),
	    this, SLOT(changeIntervals()));

    connect(m_dialog, SIGNAL(accepted()),
	    this, SLOT(accept()));
    connect(m_dialog, SIGNAL(rejected()),
	    this, SLOT(reject()));
  }

  if (m_visualizer)
    m_visualizer->activate();

  updateTarget(m_dialog->getTarget());
    
  updateDraw();
  
  m_dialog->show();
}

void
HistogramIntervalsProcessingInteractor::deactivate()
{
  if (m_visualizer)
    m_visualizer->deactivate();
  
  //if (m_processor)
  //m_processor->deactivate();

  m_dialog->hide();
}


void
HistogramIntervalsProcessingInteractor::updateImage(SegModel * /* m */)
{

}

void
HistogramIntervalsProcessingInteractor::updateToger(SegModel * /* m */)
{
  //change in toger implies change in selection
}

void
HistogramIntervalsProcessingInteractor::updateSelection(SegModel * /* m */)
{
  intervals::Target t = m_dialog->getTarget();
  if (t == intervals::TARGET_SELECTED_REGIONS) {
    changeTarget(t);
  }
}

void
HistogramIntervalsProcessingInteractor::getKeptRegions(girl::regionDeque &regions)
{
  assert(m_dialog);

  intervals::IntervalCollection intervals;
  intervals::ColorCollection colors;
  m_dialog->getIntervals(intervals, colors);
  
  getRegionsInIntervals(intervals, regions);
}


void
HistogramIntervalsProcessingInteractor::accept()
{
  if (m_commander) {
    girl::regionDeque regions;
    getKeptRegions(regions);

    m_commander->execute(m_view->model(), regions);
  }

  deactivate();

}

void
HistogramIntervalsProcessingInteractor::reject()
{
  deactivate();
}


void
HistogramIntervalsProcessingInteractor::changeTarget(intervals::Target t)
{
  updateTarget(t);

  updateDraw();

}

void
HistogramIntervalsProcessingInteractor::changeIntervals()
{
  updateDraw();
}

void
HistogramIntervalsProcessingInteractor::updateTarget(intervals::Target t)
{
  if (m_processor) {
    assert(m_view);
    assert(m_view->model());
  
    const SegModel *m = m_view->model();
    const girl::toger &tog = *(m->getToger());

    if (t == intervals::TARGET_ALL_REGIONS) {
      girl::regionDeque regions;
      tog.allRegions(regions);
      updateTarget(m, regions);
    }
    else if (t == intervals::TARGET_SELECTED_REGIONS) {
      updateTarget(m, m->getSelectedRegions());
    }

    assert(m_processor);
    ise::GrayScaleHistogram h;
    m_processor->getHistogram(h);

    assert(m_dialog);
    m_dialog->setHistogram(h);
  }
}

void
HistogramIntervalsProcessingInteractor::updateTarget(const SegModel *m, 
						     const girl::regionDeque &regions)
{
  if (m_processor) {
    m_processor->updateTarget(m, regions);
  }
}


void
HistogramIntervalsProcessingInteractor::getRegionsInIntervals(const intervals::IntervalCollection &intervals,
							      girl::regionDeque &regions) const
{
  if (m_processor) {
    m_processor->getRegionsInIntervals(intervals, regions);
  }
}

void
HistogramIntervalsProcessingInteractor::updateDraw()
{
  if (m_visualizer) {

    girl::regionDeque regions;
    getKeptRegions(regions);

    m_visualizer->update(m_view->model(), regions);
  }
}
