/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef HISTOGRAMINTERVALSPROCESSINGINTERACTOR_HPP
#define HISTOGRAMINTERVALSPROCESSINGINTERACTOR_HPP

#include <girl/girlToger.hpp>

#include <QObject>

#include "HistogramIntervalsInteractor.hpp"

#include "IntervalsTypes.hpp"

class MainWindow;
class MainView;
class SegModel;

class HistogramIntervalsProcessor;
class RegionsCommander;
class RegionsVisualizer;

class HistogramIntervalsDialog;


class HistogramIntervalsProcessingInteractor : public HistogramIntervalsInteractor
{
  Q_OBJECT

public:
  
  /**
   * @brief Destructor.
   */
  virtual ~HistogramIntervalsProcessingInteractor();

  /**
   * @brief Constructor.
   */
  HistogramIntervalsProcessingInteractor(MainWindow *mw, MainView *v)
    : HistogramIntervalsInteractor(mw, v),
      m_processor(NULL),
      m_visualizer(NULL),
      m_commander(NULL),
      m_dialog(NULL)
  {}

  virtual void activate();

  virtual void deactivate();

  /**
   * @warning takes ownership of @a p.
   */
  virtual void setProcessor(HistogramIntervalsProcessor *p);

  /**
   * @warning takes ownership of @a d.
   */
  virtual void setVisualizer(RegionsVisualizer *d);

  /**
   * @warning takes ownership of @a c.
   */
  virtual void setCommander(RegionsCommander *c);

protected:

  virtual HistogramIntervalsDialog *buildDialog(QWidget *parent) = 0;

  
protected slots:

  virtual void updateImage(SegModel *);
  virtual void updateToger(SegModel *);
  virtual void updateSelection(SegModel *);

  virtual void accept();
  virtual void reject();


  virtual void changeTarget(intervals::Target t);

  virtual void changeIntervals();

protected:

  virtual void getKeptRegions(girl::regionDeque &regions);
  
  virtual void updateTarget(intervals::Target t);
  
  virtual void updateTarget(const SegModel *m, 
			    const girl::regionDeque &regions);
  
  void getRegionsInIntervals(const intervals::IntervalCollection &intervals,
			     girl::regionDeque &regions) const;

  virtual void updateDraw();


protected:

  HistogramIntervalsProcessor *m_processor;
  RegionsVisualizer *m_visualizer;
  RegionsCommander *m_commander;
  
  HistogramIntervalsDialog *m_dialog;
};

#endif /* ! HISTOGRAMINTERVALSPROCESSINGINTERACTOR_HPP */
