/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "HistogramIntervalsSimpleDialog.hpp"

#include <cassert>

#include <QDialogButtonBox>
#include <QVBoxLayout>


HistogramIntervalsSimpleDialog::HistogramIntervalsSimpleDialog(QWidget *parent)
  : HistogramIntervalsDialog(parent), m_hiw(NULL)
{
  buildGUI();
}


void
HistogramIntervalsSimpleDialog::buildGUI()
{
  QVBoxLayout *vLayout = new QVBoxLayout;
  
  m_hiw = new HistogramIntervalsWidget(this);
  
  vLayout->addWidget(m_hiw);

  connect(m_hiw, SIGNAL(targetChanged(intervals::Target)), 
	  this, SIGNAL(targetChanged(intervals::Target)));
  connect(m_hiw, SIGNAL(intervalsChanged()), 
	  this, SIGNAL(intervalsChanged()));


  QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok
						     | QDialogButtonBox::Cancel, Qt::Horizontal, this);
  connect(buttonBox, SIGNAL(accepted()), this, SIGNAL(accepted()));
  connect(buttonBox, SIGNAL(rejected()), this, SIGNAL(rejected()));
  
  vLayout->addWidget(buttonBox);

  setLayout(vLayout);
}


void
HistogramIntervalsSimpleDialog::setHistogram(const ise::GrayScaleHistogram &h)
{
  assert(m_hiw);
  m_hiw->setHistogram(h);
}


intervals::Target
HistogramIntervalsSimpleDialog::getTarget() const
{
  assert(m_hiw);
  return m_hiw->getTarget();
}


void
HistogramIntervalsSimpleDialog::setTarget(intervals::Target t)
{
  assert(m_hiw);
  m_hiw->setTarget(t);
}


void
HistogramIntervalsSimpleDialog::getIntervals(intervals::IntervalCollection &intervals, 
						  intervals::ColorCollection &colors) const
{
  assert(m_hiw);
  m_hiw->getIntervals(intervals, colors);
}

void
HistogramIntervalsSimpleDialog::setIntervals(const intervals::IntervalCollection &intervals,
					     const intervals::ColorCollection &colors)
{
  assert(m_hiw);
  m_hiw->setIntervals(intervals, colors);
}

