/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "HistogramIntervalsWidget.hpp"

#include <cassert>

#include <sstream>
#include <iostream>//DEBUG

//#include <ise/iseGrayScaleHistogram.hpp>

#include <QCheckBox>
#include <QGroupBox>
#include <QLabel>
#include <QLayout>
#include <QPushButton>
#include <QRadioButton>
#include <QDoubleSpinBox>
#include <QVBoxLayout>

#include <qiseTools/Histogram1DThresholdsView.hpp>
#include <qiseTools/Histogram1DThresholdItem.hpp>
#include <qiseTools/ColorButton.hpp>




HistogramIntervalsWidget::HistogramIntervalsWidget(QWidget *parent)
  :  QWidget(parent)
{

  buildGUI();

}


HistogramIntervalsWidget::~HistogramIntervalsWidget()
{

}

void
HistogramIntervalsWidget::buildGUI()
{
  setWindowTitle(tr("Selection from size"));
  
  QVBoxLayout *vLayout = new QVBoxLayout(this);

  m_backgroundColor = m_colorPool.takeFreeColor(); //must be a color not used for other levels

  m_histoView = new Histogram1DThresholdsView(this, m_backgroundColor);
  vLayout->addWidget(m_histoView);

  connect(m_histoView, SIGNAL(levelDeleted(Histogram1DThresholdItem *)),
	  this, SLOT(deleteLevel(Histogram1DThresholdItem *)));


  QGroupBox *mGroupBox = new QGroupBox(this);
  buildTargetChoices(mGroupBox);
  vLayout->addWidget(mGroupBox);
  
  QGroupBox *thGroupBox = new QGroupBox(tr("Thresholds"), this);
  buildThresholdsGroup(thGroupBox);
  vLayout->addWidget(thGroupBox);
  
  setLayout(vLayout);
}

void
HistogramIntervalsWidget::buildTargetChoices(QGroupBox *mGroupBox)
{
  QLabel *l = new QLabel(tr("Apply to: "), this);

  m_targetAllRegionsButton = new QRadioButton(tr("All regions"), this);
  m_targetSelectedRegionsButton = new QRadioButton(tr("Selected regions"), this);
  
  m_targetAllRegionsButton->setChecked(true);

  QHBoxLayout *chbox = new QHBoxLayout;
  chbox->addWidget(l);
  chbox->addWidget(m_targetAllRegionsButton);
  chbox->addWidget(m_targetSelectedRegionsButton);
  mGroupBox->setLayout(chbox);
  
  connect(m_targetAllRegionsButton, SIGNAL(clicked()), 
	  this, SLOT(changeTarget()));
  connect(m_targetSelectedRegionsButton, SIGNAL(clicked()), 
	  this, SLOT(changeTarget()));  
}

void
HistogramIntervalsWidget::buildThresholdsGroup(QGroupBox *thGroupBox)
{
  QPushButton *addThresholdB = new QPushButton(tr("Add"), this);
  connect(addThresholdB, SIGNAL(clicked()), this, SLOT(addThreshold()));
  
  QPushButton *resetButton = new QPushButton(tr("Reset"), this);
  connect(resetButton, SIGNAL(clicked()), this, SLOT(resetThresholds()));

  QHBoxLayout *hLayout = new QHBoxLayout;
  hLayout->addWidget(addThresholdB);
  hLayout->addWidget(resetButton);

  //QGroupBox *addThGroupBox = new QGroupBox(this);
  //m_thresholdsLayout = new QVBoxLayout;
  //addThGroupBox->setLayout(m_thresholdsLayout);

  m_thresholdsLayout = new QVBoxLayout;

  QScrollArea *scroll = new QScrollArea(thGroupBox);
  QWidget *scrollWidget = new QWidget(thGroupBox);
  scrollWidget->setLayout(m_thresholdsLayout);
  scroll->setWidget(scrollWidget);
  scroll->setWidgetResizable(true);
  scroll->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
     
  QVBoxLayout *vbl = new QVBoxLayout;
  vbl->addLayout(hLayout);
  //vbl->addLayout(m_thresholdsLayout);
  vbl->addWidget(scroll);
  thGroupBox->setLayout(vbl);

}
  










//CODE DUPLICATION with QuantificationLevelsDialog !!!!!! 
void
HistogramIntervalsWidget::addThreshold()
{
  const QRgb color = m_colorPool.takeFreeColor(); 

  Histogram1DThresholdItem *level = m_histoView->addThreshold(color);
  
  if (level != NULL) {
    addThreshold(level, color);
  }
  else {
    m_colorPool.restoreFreeColor(color);
  }
}


// some CODE DUPLICATION with QuantificationLevelsDialog !!!!!! 
void
HistogramIntervalsWidget::addThreshold(Histogram1DThresholdItem *level, QRgb color)
{

  QHBoxLayout *layout = new QHBoxLayout();

  ColorButton *colorButton = new ColorButton(color, this);
  colorButton->setEnabled(false);
  layout->addWidget(colorButton);

  QDoubleSpinBox *spinBox = new QDoubleSpinBox(this);   //TODO: use SliderSpinBoxWidget
  spinBox->setMinimum(level->getMin());
  spinBox->setMaximum(level->getMax());
  spinBox->setValue(level->getValue());
  //TODO:specific for colorspaces : setDecimals/setSingleStep/... !!!
  layout->addWidget(spinBox);

  QPushButton *delButton = new QPushButton(this);
  delButton->setIcon(QIcon(QString::fromLatin1(":/images/delButton.png")));
  delButton->setToolTip(tr("delete threshold"));
  layout->addWidget(delButton);

  const int size = 27; //spinBox->height();
  colorButton->setMaximumSize(size, size);
  delButton->setMaximumSize(size, size);

  //DIFFERENCE !
  QCheckBox *disableRangeCB = new QCheckBox(tr("Disable range"), this);
  disableRangeCB->setCheckState(Qt::Unchecked); //unchecked by default !
  layout->addWidget(disableRangeCB);
  

  m_thresholdsList.append(Histogram1DThresholdItemDetails(level, layout, colorButton, spinBox, delButton, color, disableRangeCB)); 
  //Rem: levels are in decreasing order in list (that is the first level has the bigger value)

  m_thresholdsLayout->addLayout(layout);


  connect(level, SIGNAL(valueChanged(double)), 
	  spinBox, SLOT(setValue(double)));

  connect(level, SIGNAL(limitsChanged(Histogram1DThresholdItem *)), 
	  this, SLOT(updateLevelLimits(Histogram1DThresholdItem *)));

  connect(spinBox, SIGNAL(valueChanged(double)), 
	  level, SLOT(setValue(double)));

  connect(level, SIGNAL(valueChanged(double)), 
	  this, SLOT(itemMoved(double)));

  connect(delButton, SIGNAL(clicked()), 
	  this, SLOT(delThreshold()));

  connect(disableRangeCB, SIGNAL(stateChanged(int)), 
	  this, SLOT(rangeToggled(int))); //DIFFERENCE !
  

  
  emit intervalsChanged();

}

// some CODE DUPLICATION with QuantificationLevelsDialog !!!!!! 
void
HistogramIntervalsWidget::delThreshold()
{
  QPushButton *clickedButton = qobject_cast<QPushButton *>(sender());
  for (int i = 0; i< m_thresholdsList.size(); ++i) {
    if (m_thresholdsList.at(i).delButton == clickedButton) {

      delThreshold(i);

      break;
    }
  }
}

void
HistogramIntervalsWidget::deleteLevel(Histogram1DThresholdItem *level)
{
  for (int i = 0; i< m_thresholdsList.size(); ++i) {
    if (m_thresholdsList.at(i).item == level) {
      

      QDoubleSpinBox *spinBox = m_thresholdsList.at(i).spinBox;
      QPushButton *delButton = m_thresholdsList.at(i).delButton;
      QCheckBox *disableRangeCB = m_thresholdsList.at(i).disableRangeCB;//DIFFERENCE !
      
      //cannot do this, level already deleted !
      /*
      disconnect(level, SIGNAL(valueChanged(int)), spinBox, SLOT(setValue(int)));
      disconnect(level, SIGNAL(limitsChanged(Histogram1DThresholdItem *)), this, SLOT(updateLevelLimits(Histogram1DThresholdItem *)));
      disconnect(spinBox, SIGNAL(valueChanged(int)), level, SLOT(setValue(int)));
      disconnect(level, SIGNAL(valueChanged(int)), this, SLOT(itemMoved(int)));
      disconnect(delButton, SIGNAL(clicked()), this, SLOT(delThreshold()));
      disconnect(disableRangeCB, SIGNAL(stateChanged(int)), 
		 this, SLOT(rangeToggled(int))); //DIFFERENCE !
      */

      m_thresholdsLayout->removeItem(m_thresholdsList.at(i).layout);
    
      delete m_thresholdsList.at(i).colorButton;
      delete spinBox;
      delete delButton;
      delete disableRangeCB;//DIFFERENCE !
      delete m_thresholdsList.at(i).layout;
  
      m_colorPool.restoreFreeColor(m_thresholdsList.at(i).color);
  
      m_thresholdsList.removeAt(i);


      //DIFFERENCE !
      emit intervalsChanged();

      return;
    }
  }

  std::cerr<<"################################## HistogramIntervalsWidget::deleteLevel item not found !!!!"<<std::endl;

}


// some CODE DUPLICATION with QuantificationLevelsDialog !!!!!! 
void
HistogramIntervalsWidget::delThreshold(int i)
{
  assert(i>=0 && i<m_thresholdsList.size());

  Histogram1DThresholdItem *th = m_thresholdsList.at(i).item;
  m_histoView->delThreshold(th); //will call deleteLevel(Histogram1DThresholdItem *)

}  

//CODE DUPLICATION with QuantificationLevelsDialog !!!!!! 
void
HistogramIntervalsWidget::delAllThresholds()
{
  while (! m_thresholdsList.isEmpty())
    delThreshold(m_thresholdsList.size()-1); //we delete from the end, to get colors in the original order
  assert(m_thresholdsList.isEmpty());
}

//some CODE DUPLICATION with QuantificationLevelsDialog !!!!!! 
void
HistogramIntervalsWidget::resetThresholds()
{
  delAllThresholds();

  emit intervalsChanged();
}


//some CODE DUPLICATION with QuantificationLevelsDialog !!!!!! 
void
HistogramIntervalsWidget::itemMoved(double)
{
  emit intervalsChanged();
}

//CODE DUPLICATION with QuantificationLevelsDialog !!!!!! 
void
HistogramIntervalsWidget::updateLevelLimits(Histogram1DThresholdItem *level)
{
  for (int i = m_thresholdsList.size() - 1; i >=0; --i) {
    if (m_thresholdsList.at(i).item == level) {
      QDoubleSpinBox *spinBox = m_thresholdsList.at(i).spinBox;
#ifdef VERBOSE_QUANTIFLEVELS
      std::cerr<<"QuantificationLevelsDialog::updateLevelLimits level="<<level<<"  min="<< level->getMin()<<" max="<<level->getMax()<<std::endl;
#endif
      spinBox->setMaximum(level->getMax());
      spinBox->setMinimum(level->getMin());
      break;
    }
  }
}

 //DIFFERENCE !
void
HistogramIntervalsWidget::rangeToggled(int)
{
  //QCheckBox *disableRangeCB = qobject_cast<QAction *>(sender()); //get source of the signal
  
  emit intervalsChanged();
}




void
HistogramIntervalsWidget::setHistogram(const ise::GrayScaleHistogram &h)
{
  m_histoView->setHistogram(h);
}

void
HistogramIntervalsWidget::changeTarget()
{
  emit targetChanged(getTarget());
}

intervals::Target
HistogramIntervalsWidget::getTarget() const 
{
  return m_targetAllRegionsButton->isChecked() ? intervals::TARGET_ALL_REGIONS 
    : intervals::TARGET_SELECTED_REGIONS;
}

void
HistogramIntervalsWidget::setTarget(intervals::Target t)
{
  if (getTarget() != t) {
    if (t == intervals::TARGET_ALL_REGIONS)
      m_targetAllRegionsButton->setChecked(true);
    else {
      assert(t == intervals::TARGET_SELECTED_REGIONS);
      m_targetSelectedRegionsButton->setChecked(true);
    }
    emit targetChanged(t);
  }
}

void
HistogramIntervalsWidget::getIntervals(intervals::IntervalCollection &intervals, 
				       intervals::ColorCollection &colors) const
{
  intervals.clear();
  colors.clear();

  const int minHisto = m_histoView->absoluteMin();
#ifndef NDEBUG
  const int maxHisto = m_histoView->absoluteMax();
  std::cerr<<"HistogramIntervalsWidget::getKeptIntervals minHisto="<<minHisto<<" maxHisto="<<maxHisto<<std::endl;
  assert(minHisto<=maxHisto);
#endif



  //levels are in decreasing order in m_thresholdsList
  int prevV = minHisto;
  for (int i = m_thresholdsList.size() - 1; i >=0; --i) {
    
    const int v = m_thresholdsList[i].item->getValue();

    if (m_thresholdsList[i].disableRangeCB->checkState() == Qt::Unchecked) {
      assert(prevV<=maxHisto);
      const QRgb color = m_thresholdsList[i].color;

      intervals.push_back(intervals::Interval(prevV, v));
      colors.push_back(color);
    }

    prevV = v+1;
  }    


  /*
  {
    //DEBUG
    std::cerr<<"kept intervals: "<<std::endl;
    intervals::IntervalCollection::const_iterator itI = intervals.begin();
    const intervals::IntervalCollection::const_iterator itIEnd = intervals.end();
    for ( ; itI != itIEnd; ++itI)
      std::cerr<<"["<<itI->first<<"; "<<itI->second<<"]"<<std::endl;

  }
  */


}

void
HistogramIntervalsWidget::setIntervals(const intervals::IntervalCollection & /* intervals */,
				       const intervals::ColorCollection & /* colors */)
{
  //TODO !!!
  assert(false);
}



