/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef HISTOGRAMINTERVALSWIDGET_HPP
#define HISTOGRAMINTERVALSWIDGET_HPP


#include <QWidget>

#include "IntervalsTypes.hpp"
#include <qiseTools/ColorPool.hpp>

namespace ise {
  class GrayScaleHistogram;
}

class QCheckBox;
class QGroupBox;
class QLayout; 
class QPushButton;
class QRadioButton;
class QDoubleSpinBox; 
class QVBoxLayout;

class Histogram1DThresholdsView;
class Histogram1DThresholdItem;


class HistogramIntervalsWidget : public QWidget 
{
  Q_OBJECT

public:

  /**
   * @brief Destructor.
   */
  ~HistogramIntervalsWidget();
  

  /**
   * @brief Constructor.
   *
   * @warning take ownership of @a ihp.
   */
  explicit HistogramIntervalsWidget(QWidget *parent);
		     

  void setHistogram(const ise::GrayScaleHistogram &h);

  intervals::Target getTarget() const;

  void setTarget(intervals::Target t);

  /**
   * Return kept intervals in sorted increasing order.
   *
   * We have intervals[i].first<=intervals[i].second<intervals[i+1].first
   */
  void getIntervals(intervals::IntervalCollection &intervals, 
		    intervals::ColorCollection &colors) const;

  void setIntervals(const intervals::IntervalCollection &intervals,
		    const intervals::ColorCollection &colors);

  

signals:
   
  void targetChanged(intervals::Target t);

  void intervalsChanged();


protected:



protected slots:
  

  void changeTarget();

  //CODE DUPLICATION with QuantificationLevelsDialog !!!!!! 

  void deleteLevel(Histogram1DThresholdItem *);

  void resetThresholds();
  void addThreshold();
  void delThreshold();

  void updateLevelLimits(Histogram1DThresholdItem *level);
  void itemMoved(double);

  void rangeToggled(int); //DIFFERENCE !


protected:

  void getKeptIntervals(intervals::IntervalCollection &intervals,
			intervals::ColorCollection &colors) const;


  void updateHistoView();


protected:


  void buildGUI();

  void buildTargetChoices(QGroupBox *mGroupBox);
  void buildThresholdsGroup(QGroupBox *thGroupBox);

  void addThreshold(Histogram1DThresholdItem *level, QRgb color);
  void delThreshold(int i);
  void delAllThresholds();


protected:



  // some CODE DUPLICATION with QuantificationLevelsDialog !!!!!! 
  struct Histogram1DThresholdItemDetails
  {
    Histogram1DThresholdItemDetails(Histogram1DThresholdItem *i, 
		     QLayout *lt, 
		     QPushButton *c, 
		     QDoubleSpinBox *s, 
		     QPushButton *d, 
		     QRgb col, 
		     QCheckBox *drCB)
      : item(i), 
	layout(lt), 
	colorButton(c), 
	spinBox(s), 
	delButton(d), 
	color(col),
	disableRangeCB(drCB)
    {}

    Histogram1DThresholdItem *item;
    QLayout *layout;
    QPushButton *colorButton;
    QDoubleSpinBox *spinBox;
    QPushButton *delButton;
    QRgb color;
    QCheckBox *disableRangeCB;
  };

  QList< Histogram1DThresholdItemDetails > m_thresholdsList;

  QVBoxLayout *m_thresholdsLayout;

  ColorPool m_colorPool;
  QRgb m_backgroundColor;


  QRadioButton *m_targetAllRegionsButton;
  QRadioButton *m_targetSelectedRegionsButton;
 
  Histogram1DThresholdsView *m_histoView;

};


#endif /* ! HISTOGRAMINTERVALSWIDGET_HPP */
