/*
  Copyright (C) 2005-2012 Hugo Boujut, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#ifndef IMAGEANNOTATORDIALOG_HPP
#define IMAGEANNOTATORDIALOG_HPP

#include <QHash>

#include <girl/girlSharedPtr.hpp>

#include <QWidget>

class MainView;
class MainWindow;
class FalseColorImageBuilderThread;
class ImageConcept;
class ImageConceptListWidget;

class GLBlendImageLayer;

class QImage;
class QPushButton;
class QSlider;


class ImageAnnotatorDialog : public QWidget {
  Q_OBJECT

public:  
  ImageAnnotatorDialog(MainView *v, MainWindow *mw);
  ~ImageAnnotatorDialog();

  void activate();
  void deactivate();
  
protected slots:
  void accepted();
  
  //void addImageConcept(const ImageConcept *);
  void editImageConcept(const ImageConcept *);
  void removeImageConcept(const ImageConcept *);  
  //void selectedConceptChanged(const ImageConcept *);

  void selectRegionsForCurrentConcept();
  void addSelectedRegionsToConcept();
  void updateImage(const QImage &image);
  //void importConcepts();
  //void exportConcepts();
  //void exportConceptsRegions();
  void saveImage();
  void loadImage(const QImage &);
  void changeAlpha(int);

  void stats();

protected:
  void build();
  //void syncWidgetItem(ImageConcept &ic, QListWidgetItem &lwi);
  void updateFalseColorImage(bool saveMode);
  void updateFalseColorImage(const QImage &img);
  // void addDefaultImageConcept();
  void writeImage(const QImage &img);
  // void clear();
  void populate(ImageConcept *defaultConcept, const QList<ImageConcept *> &concepts);

  // void exportConcepts(const QString &str, bool exportRegions);

  // bool maybeSave();
  // bool hasModifiedConcepts() const;

  void affectRegionsToConcepts(const QImage &labelImg);

  void setOpacity(float opacity);

protected:

  MainView *m_view;
  MainWindow *m_mw;

  GLBlendImageLayer *m_layer;

  ImageConceptListWidget *m_imageConceptListWidget;

  /*
  QListWidget *m_imageConceptListWidget;
  typedef QHash<QListWidgetItem *, ImageConcept *> HashType;
  HashType m_imageConceptList;

  QListWidgetItem *m_defaultListWidgetItem;
  ImageConcept *m_defaultConcept;

  ImageConceptEditorDialog *m_imgConceptEditor;
  */

  girl::shared_ptr<FalseColorImageBuilderThread> m_fcibThread1;

  /*
  QPushButton *m_addButton;
  QPushButton *m_editButton;
  QPushButton *m_removeButton;
  */
  QSlider *m_alphaSlider;
  QPushButton *m_statsButton;
  QPushButton *m_selectButton;
  QPushButton *m_applyButton;
  QPushButton *m_saveImgButton;

  //bool m_modifiedConcepts;
};

#endif //IMAGEANNOTATORDIALOG_HPP
