#include "ImageConceptEditorDialog.hpp"

#include <QBoxLayout>
#include <QDialogButtonBox>

//REM: default entry color is not displayed, on purpose.
//Because otherwise the whole image would be of this same color,
// and we would not see the regions to annotate.


ImageConceptEditorDialog::ImageConceptEditorDialog(QWidget *parent)
  : QDialog(parent),
    m_imageConcept(NULL)
{
  QVBoxLayout *vLayout = new QVBoxLayout;
  QHBoxLayout *layout = new QHBoxLayout;
  m_colorButton = new ColorButton(this);
  layout->addWidget(m_colorButton);

  m_nameLineEdit = new QLineEdit(this);
  layout->addWidget(m_nameLineEdit);

  QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok |
						     QDialogButtonBox::Cancel, Qt::Horizontal, this);
  connect(buttonBox, SIGNAL(accepted()), this, SLOT(accepted()));
  connect(buttonBox, SIGNAL(rejected()), this, SLOT(rejected()));

  vLayout->addLayout(layout);
  vLayout->addWidget(buttonBox);
  setLayout(vLayout);

  //We want to have the focus in the LineEdit to be able to enter concept name immediately
  setFocusOnTextEdit();
}

void
ImageConceptEditorDialog::setFocusOnTextEdit()
{
  assert(m_nameLineEdit);

  m_nameLineEdit->setFocus(Qt::ActiveWindowFocusReason); 
}


void
ImageConceptEditorDialog::setImageConcept(ImageConcept &imageConcept)
{
  m_imageConcept = &imageConcept;
  assert(m_imageConcept != NULL);

  m_nameLineEdit->setText(m_imageConcept->getName());

  QRgb c = 0;//useless
  m_colorButton->setColor(m_imageConcept->getColor(c));
}

void
ImageConceptEditorDialog::setEnabledColorButton(bool enable)
{
  m_colorButton->setEnabled(enable);
}

void
ImageConceptEditorDialog::accepted()
{
  assert(m_imageConcept != NULL);
  m_imageConcept->setColor(m_colorButton->getColor());
  m_imageConcept->setName(m_nameLineEdit->text());
  emit accept();
}

void
ImageConceptEditorDialog::rejected()
{
  emit reject();
}
