/*
  Copyright (C) 2005-2012 Hugo Boujut, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#ifndef IMAGECONCEPTLISTWIDGET_HPP
#define IMAGECONCEPTLISTWIDGET_HPP

#include <girl/girlToger.hpp>
#include <QHash>
#include <QWidget>

class ImageConcept;
class QListWidget;
class QListWidgetItem;
class ImageConceptEditorDialog;
class QPushButton;


class ImageConceptListWidget : public QWidget
{
  Q_OBJECT

public:

  explicit ImageConceptListWidget(QWidget *parent);

  ~ImageConceptListWidget();

  const ImageConcept *getCurrentConcept() const;
  
  bool hasConcepts() const;

  QList<const ImageConcept *> getConcepts() const;

  /**
   * @brief Clear regions of all existing concepts
   */
  void clearRegions();

  /**
   * @brief Add region @a r to concept with color @a color.
   */
  void addRegion(girl::region r, QRgb color);


  bool maybeSave();

  void addRegionsToCurrentConcept(const girl::regionDeque &regions);

  const ImageConcept *getDefaultConcept() const;

  QRgb getDefaultConceptColor() const;

  bool hasConceptWithColor(QRgb color) const;

  void addConcept(QRgb color, const QString &name = tr(""));

  void writeImage(const QImage &img);

  QString getImageFilename() const;

signals:
  void imageConceptAdded(const ImageConcept *ic);
  void imageConceptEdited(const ImageConcept *ic);
  void imageConceptRemoved(const ImageConcept *ic);
  void imageConceptSelectedChanged(const ImageConcept *ic);
  
  void imageLoaded(const QImage &img);
  void saveImageAs(const QString &filename);
				     
protected slots:
  void addImageConcept();
  void editImageConcept();
  void removeImageConcept();
  void selectedConceptChanged();

  void importConcepts();
  void exportConcepts();
  //void exportConceptsRegions();
  void loadImage();
  void saveImage();
 
protected:
  void build();
  void clear();
  void syncWidgetItem(ImageConcept &ic, QListWidgetItem &lwi);
 
  void populate(ImageConcept *defaultConcept, const QList<ImageConcept *> &concepts);

  void exportConcepts(const QString &str, bool exportRegions);

  bool hasModifiedConcepts() const;

  void addConcept(ImageConcept *ic);
  void addDefaultImageConcept();
 
  ImageConcept *getCurrentCpt();

protected:

  QListWidget *m_imageConceptListWidget;
  typedef QHash<QListWidgetItem *, ImageConcept *> HashType;
  HashType m_imageConceptList;

  QListWidgetItem *m_defaultListWidgetItem;
  ImageConcept *m_defaultConcept;

  ImageConceptEditorDialog *m_imgConceptEditor;

  QPushButton *m_addButton;
  QPushButton *m_editButton;
  QPushButton *m_removeButton;

  QString m_imageFilename;

  bool m_modifiedConcepts;
};

#endif /* ! IMAGECONCEPTLISTWIDGET_HPP */
