/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef IMAGEPYRAMIDBUILDDIALOG_HPP
#define IMAGEPYRAMIDBUILDDIALOG_HPP

#include <QDialog>
#include <QMap>

#include <ise/isePyramid.hpp>
#include <qiseTools/PyramidWidget.hpp>
#include <qiseTools/PyramidChoiceWidget.hpp>
#include <qiseTools/PyramidFactory.hpp>

class QCheckBox;
class QLabel;
class QSpinBox;
class MainView;

class ImagePyramidBuildDialog : public QDialog
{
  Q_OBJECT

public:

  ImagePyramidBuildDialog(MainView *mainView, QWidget *parent = NULL, 
			  int imageWidth = 0, int imageHeight=0);
						
  void setImageSize(int imageWidth, int imageHeight);
	     
  size_t divider() const;

  size_t numLevels() const;

  bool completionEnabled() const;

  girl::shared_ptr<ise::Pyramid> getPyramid() const { return m_pcw->getPyramid(); }

  /**
   * Return base image (larger level of pyramid) size.
   * It will be the size set by constructor or setImageSize if completionEnabled() is false.
   * If completionEnabled() is true, it will be superior or equal.
   */
  void getBaseImageSize(int &baseImageWidth, int &baseImageHeight);

  void setGUI();
	       
  void getGUI() const;

protected slots:

  void accept();
  void reject();

  void dividerChanged(int d);
  void completionChanged(int state);
  void numLevelsChanged(int n);
  
protected:

  void buildGUI();

  void initValues();

protected:

  MainView *m_mainView;
  int m_imageWidth, m_imageHeight;
  int m_baseImageWidth, m_baseImageHeight;
  QLabel *m_imageSizeL;
  QSpinBox *m_dividerSB;
  QCheckBox *m_completionCB;
  QSpinBox *m_numLevelsSB;
  QLabel *m_lastImageSizeL;
  PyramidChoiceWidget *m_pcw;

};


#endif /* ! IMAGEPYRAMIDBUILDDIALOG_HPP */
