/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#include <girl/girlSharedPtr.hpp>
#include "InfoDialog.hpp"

#include <sstream>

#include <QComboBox>
#include <QLabel>
#include <QBoxLayout>
#include <QGridLayout>
#include <QTableWidget>
#include <QHeaderView>

#include <ise/iseHistogram1D.hpp>
#include <ise/iseRegionSetInterface.hpp>



#include <qiseTools/ColorSpaceRegistry.hpp>
#include <qiseTools/Histogram1DWidget.hpp>
#include <qiseTools/RegionFeatureChoiceComboWidget.hpp>
#include <qiseTools/ColorSpaceChoiceWidget.hpp>

#include "InfoInteractor.hpp"
#include <qiseTools/SegView.hpp>
//#include <qiseTools/MainView.hpp>
#include <qiseTools/FeatureComputerGUIInfo.hpp>

InfoDialog::InfoDialog(QWidget *parent, 
		       InfoInteractor *ii,
		       int colorSpace)
  : QDialog(parent),
    m_infoInteractor(ii),
    m_channelsCombo(NULL),
    m_regionLabel(NULL),
    m_perimeterLabel(NULL),
    m_areaLabel(NULL),
    m_cs(NULL),
    m_minLabel(NULL),
    m_maxLabel(NULL),
    m_meanLabel(NULL),
    m_stdDeviationLabel(NULL),
    m_colorSpace(colorSpace),
    m_combo(NULL),
    m_regionFeatureTable(NULL)
{
  setAttribute(Qt::WA_DeleteOnClose);

  buildGUI();
}

void
InfoDialog::populateCombo()
{
  const ColorSpaceDetails *csd = ColorSpaceRegistry::getColorSpace(m_colorSpace);
  assert(csd);
  
  for (size_t i=0; i < csd->getNumChannels(); ++i) {
    const QString &channelName = csd->channelsNames.at(i);
    m_channelsCombo->addItem(channelName);
  }
  m_channelsCombo->setCurrentIndex(0);

  assert(m_channelsCombo->count() == (int)csd->getNumChannels());

  connect(m_channelsCombo, SIGNAL(activated(int)), 
	  this, SLOT(changeChannel(int)));
}

void
InfoDialog::buildGUI()
{
  m_regionLabel = new QLabel(this);


  m_perimeterLabel = new QLabel(this);

  
  m_areaLabel = new QLabel(this);

  QLabel *csLabel = new QLabel(tr("Color space:"), this);
  m_cs = new ColorSpaceChoiceWidget(this);
  m_cs->setColorSpace(m_colorSpace);
  
  QLabel *chanLabel = new QLabel(tr("Channel:"), this);
  m_channelsCombo = new QComboBox(this);
  populateCombo();

  connect(m_cs, SIGNAL(colorSpaceChanged(int)), this, SLOT(changeColorSpace(int)));

  m_minLabel = new QLabel(this);
  m_maxLabel = new QLabel(this);
  
  m_meanLabel = new QLabel(this);
  m_stdDeviationLabel = new QLabel(this);


  m_histoView = new Histogram1DWidget(this);
 
  reset();


  QGridLayout *gLayout = new QGridLayout;
  gLayout->addWidget(m_perimeterLabel, 0, 0);
  gLayout->addWidget(m_areaLabel, 0, 1);

  QHBoxLayout *hLayoutCS = new QHBoxLayout;
  hLayoutCS->addWidget(csLabel);
  hLayoutCS->addWidget(m_cs);
  gLayout->addLayout(hLayoutCS, 1, 0, 1, 2);
  
  QHBoxLayout *hLayout = new QHBoxLayout;
  hLayout->addWidget(chanLabel);
  hLayout->addWidget(m_channelsCombo);
  gLayout->addLayout(hLayout, 2, 0, 1, 2);
  
  gLayout->addWidget(m_minLabel, 3, 0);

  gLayout->addWidget(m_maxLabel, 3, 1);

  gLayout->addWidget(m_meanLabel, 4, 0);
  gLayout->addWidget(m_stdDeviationLabel, 4, 1);
  
  QVBoxLayout *regionFeatureLayout = new QVBoxLayout;
  m_combo = new RegionFeatureChoiceComboWidget(reinterpret_cast<MainView*>(m_infoInteractor->m_view), m_infoInteractor->m_view->model()->getFeatureComputerGUIInfo(), this);
  connect(m_combo, SIGNAL(regionFeatureChanged()), this, SLOT(populateRegionFeatureTable()));

  m_regionFeatureTable = new QTableWidget(this);
  regionFeatureLayout->addWidget(m_combo);
  regionFeatureLayout->addWidget(m_regionFeatureTable);

  QVBoxLayout *vLayout = new QVBoxLayout();
  vLayout->addWidget(m_regionLabel);
  vLayout->addLayout(gLayout);
  vLayout->addWidget(m_histoView);
  vLayout->addLayout(regionFeatureLayout);

  setLayout(vLayout);


  /*
  QHBoxLayout *hLayout0 = new QHBoxLayout;
  hLayout0->addWidget(m_perimeterLabel);
  hLayout0->addStretch();
  hLayout0->addWidget(m_areaLabel);
  hLayout0->addStretch();
  
  QHBoxLayout *hLayout = new QHBoxLayout;
  hLayout->addWidget(chanLabel);
  hLayout->addWidget(m_channelsCombo);
  hLayout->addStretch();

  QHBoxLayout *hLayout2 = new QHBoxLayout;
  hLayout2->addWidget(m_minLabel);
  hLayout2->addStretch();
  hLayout2->addWidget(m_maxLabel);
  hLayout2->addStretch();

  QHBoxLayout *hLayout3 = new QHBoxLayout;
  hLayout3->addWidget(m_meanLabel);
  hLayout3->addStretch();
  hLayout3->addWidget(m_stdDeviationLabel);
  hLayout3->addStretch();

  QVBoxLayout *vLayout = new QVBoxLayout();
  vLayout->addWidget(m_regionLabel);
  vLayout->addLayout(hLayout0);
  vLayout->addLayout(hLayout);
  vLayout->addLayout(hLayout2);
  vLayout->addLayout(hLayout3);
  vLayout->addWidget(m_histoView);

  setLayout(vLayout);
  */
}

void
InfoDialog::reset()
{
  girl::regionDeque regs;
  setSelectedRegions(regs);
  setPerimeter(0);
  setArea(0);
  ise::Histogram1D h;
  setHistogram(h);

}
void
InfoDialog::changeColorSpace(int cs)
{
  m_colorSpace = cs;

  const size_t numChannels = m_cs->getNbChannelsForColorSpace();
  assert(numChannels == (size_t)m_channelsCombo->count());
  for (size_t i=0; i < numChannels; ++i) {
    QString channelName = m_cs->getChannelName(i);
    m_channelsCombo->setItemText(i, channelName);
  }

  updateInfosColor();
}

void
InfoDialog::changeChannel(int /* c */)
{
  updateInfosColor();
}

int
InfoDialog::getColorSpace() const
{
  return m_cs->getColorSpace();
}

int
InfoDialog::getChannel() const
{
  return m_channelsCombo->currentIndex();
}

void
InfoDialog::setSelectedRegions(const girl::regionDeque &selectedRegions)
{
  m_regionLabel->setWordWrap(true);

  QString regionName;
  if (selectedRegions.size() == 1) {
    regionName.setNum((ulong)selectedRegions.front());
    m_regionLabel->setText(tr("Region: %1").arg(regionName));
  }
  else {
    girl::regionDeque::const_iterator it = selectedRegions.begin();
    const girl::regionDeque::const_iterator itEnd = selectedRegions.end();
    if (it != itEnd) {
      std::stringstream ss;
      ss<<*it;
      ++it;
      const size_t maxNb = 8;
      for ( size_t i=0; i<maxNb && it != itEnd; ++it, ++i)
	ss<<", "<<*it;
      if (it != itEnd) {
	ss<<", ...";
      }
      m_regionLabel->setText(tr("Regions: %1").arg(QString::fromStdString(ss.str())));
    }
    else {
      m_regionLabel->setText(tr("..."));
    }
  }
}

void
InfoDialog::setPerimeter(size_t perimeter)
{
  m_perimeterLabel->setText(tr("Perimeter: %1").arg(perimeter));
}

void
InfoDialog::setArea(size_t area)
{
  m_areaLabel->setText(tr("Area: %1").arg(area));
}

void
InfoDialog::setHistogram(const ise::Histogram1D &h)
{
  const int lMin = h.getMin();
  const int lMax = h.getMax();
  const int lMean = h.getMean();
  const float lStdDev = h.getStandardDeviation(lMean);

  m_minLabel->setText(tr("Min: %1").arg(lMin));
  m_maxLabel->setText(tr("Max: %1").arg(lMax));
  m_meanLabel->setText(tr("Mean: %1").arg(lMean));
  m_stdDeviationLabel->setText(tr("Std dev.: %1").arg(lStdDev));

  m_histoView->setHistogram(h);
}

void
InfoDialog::updateInfosColor()
{
  assert(m_infoInteractor);
  ise::Histogram1D h;
  m_infoInteractor->getHistogram(getColorSpace(), getChannel(), h);
  setHistogram(h);
}

void
InfoDialog::updateInfos()
{
  assert(m_infoInteractor);

  setSelectedRegions(m_infoInteractor->getSelectedRegions());

  size_t perimeter;
  m_infoInteractor->getPerimeter(perimeter);
  setPerimeter(perimeter);

  size_t area;
  m_infoInteractor->getArea(area);
  setArea(area);

  updateInfosColor();
  
  populateRegionFeatureTable(m_combo->regionFeatureComputerPtr());

}

void
InfoDialog::populateRegionFeatureTable()
{
  populateRegionFeatureTable(m_combo->regionFeatureComputerPtr());
}

void
InfoDialog::populateRegionFeatureTable(ise::FeatureComputerPtr featureComputer)
{
  assert(m_regionFeatureTable);
  assert(m_infoInteractor);

  m_regionFeatureTable->clear();
  m_regionFeatureTable->verticalHeader()->hide();
  m_regionFeatureTable->horizontalHeader()->resizeSection(0,75);
  
  girl::regionDeque regions = m_infoInteractor->getSelectedRegions();

  m_regionFeatureTable->setColumnCount(regions.size()+1);
  QStringList labels;
  //B:REM: On linux (Fedora 15) with icpc 12.1.0 & qt 4.7.4, using operator<< on QStringLists causes an ICE
  labels.append(tr("Channels"));
  for (size_t i = 0 ; i < regions.size(); ++i)
    labels.append(QString("%1").arg(regions[i]));
  m_regionFeatureTable->setHorizontalHeaderLabels(labels);

  girl::shared_ptr<ise::RegionSetInterface> regSet = girl::static_pointer_cast<ise::RegionSetInterface>(featureComputer);


  if (regSet) {

    size_t dim = regSet->getFeatureDim();
    m_regionFeatureTable->setRowCount(dim);

    std::vector<std::string> dimDesc;
    regSet->getDimDescription(dimDesc);

    int row = m_regionFeatureTable->rowCount();
    assert(row == static_cast<int>(dimDesc.size()));

    for (int i = 0 ; i < row ; ++i) {
      QTableWidgetItem *itemChannel = new QTableWidgetItem(QString(dimDesc[i].c_str()));
      m_regionFeatureTable->setItem(i, 0, itemChannel);
    }

    for (size_t i = 0 ; i < regions.size(); ++i) {
      std::vector<ise::RegionFeature_Data_Type> buff(regSet->getFeatureDim(), 0);
      ise::RegionFeature feature(&buff[0]);
      regSet->getRegionFeatures(regions[i], feature);
      
      for (size_t j = 0 ; j < dim ; ++j) {
	QTableWidgetItem *itemValue = new QTableWidgetItem(tr("%1").arg(buff[j]));
	m_regionFeatureTable->setItem(j, i+1, itemValue);
      }
    }
  }
}
