/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef INFODIALOG_HPP
#define INFODIALOG_HPP

#include <girl/girlToger.hpp>
#include <ise/iseConversion.hpp>
#include <ise/iseFeatureComputer.hpp>

#include <QDialog>

class QLabel;
class QComboBox;
class QTableWidget;

namespace ise {
  class Histogram1D;
}

class Histogram1DWidget;
class InfoInteractor;
class RegionFeatureChoiceComboWidget;
class ColorSpaceChoiceWidget;

class InfoDialog : public QDialog
{

  Q_OBJECT

public:

  InfoDialog(QWidget *parent = NULL, 
	     InfoInteractor *ii = NULL,
	     int colorSpace = ISE_RGB_to_RGB);

  int getColorSpace() const;
  int getChannel() const;

  void updateInfos();

  void populateRegionFeatureTable(ise::FeatureComputerPtr);


protected slots:

  void changeColorSpace(int cs);
  void changeChannel(int c);
  void populateRegionFeatureTable();

protected:

  void buildGUI();
  void populateCombo();
  void reset();

  void setSelectedRegions(const girl::regionDeque &selectedRegions);

  void setPerimeter(size_t perimeter);
  void setArea(size_t area);
  void setHistogram(const ise::Histogram1D &h);

  void updateInfosColor();


protected:

  InfoInteractor *m_infoInteractor;
  QComboBox *m_channelsCombo;
  QLabel *m_regionLabel;
  QLabel *m_perimeterLabel;
  QLabel *m_areaLabel;
  ColorSpaceChoiceWidget *m_cs;
  QLabel *m_minLabel;
  QLabel *m_maxLabel;
  QLabel *m_meanLabel;
  QLabel *m_stdDeviationLabel;
  Histogram1DWidget *m_histoView;
  int m_colorSpace;

  RegionFeatureChoiceComboWidget *m_combo;
  QTableWidget *m_regionFeatureTable;
};


#endif /* ! INFODIALOG_HPP */
