/*
  Copyright (C) 2005-2009 Jerome Charton, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "InteractiveRegularShapeSelectedStatus.hpp"

#include <QPoint>
#include <qiseTools/EditionBox.hpp>
#include "PaintingRegularShape.hpp"

#include "InteractiveShapeResizeCornerStatus.hpp"
#include "InteractiveShapeResizeHeightStatus.hpp"
#include "InteractiveShapeResizeWidthStatus.hpp"
#include "InteractiveShapeRotateStatus.hpp"
#include "InteractiveShapeMovePointStatus.hpp"
#include "InteractiveShapeMoveStatus.hpp"
#include "InteractiveRegularShapeUnSelectedStatus.hpp"

#include <iostream> //DEBUG
#include <cassert>

InteractiveShapeStatus *
InteractiveRegularShapeSelectedStatus::onLeftClick(const QPoint &mouse, PaintingShape *shape)
{
  int width = shape->width();
  int height = shape->height();
  int dimCorner = EditionBox::getDimCorner(m_view, shape);

  assert(dimCorner >= 0);
  
  int upLine = dimCorner + shape->getAreaP1().y();
  int downLine = height - 1 - dimCorner + shape->getAreaP1().y();
  int leftLine = dimCorner + shape->getAreaP1().x();
  int rightLine = width - 1 - dimCorner + shape->getAreaP1().x();

  QPoint source, origin, target;
  
  InteractiveRegularShapeSelectedStatus *backStatus = new InteractiveRegularShapeSelectedStatus(m_view);
  
  if (m_editionBox.isUnderPointHandler(mouse, m_view, shape)) {
    source = shape->getPoints()[m_editionBox.getPointHandledUnder(mouse, m_view, shape)];
    QPoint diffMouseCorner = QPoint(source.x()-mouse.x(), source.y()-mouse.y());
    return new InteractiveShapeMovePointStatus(m_view, backStatus, diffMouseCorner, source, origin,
					       m_editionBox.getPointHandledUnder(mouse, m_view, shape));
  }
  
  if (m_editionBox.isUnderRotationHandler(mouse, m_view, shape)) {
    source = mouse;
    QPoint diffMouseCorner = QPoint(source.x()-mouse.x(), source.y()-mouse.y());
    return new InteractiveShapeRotateStatus(m_view, backStatus, diffMouseCorner, source, origin);
  }
  
  if (!shape->isUnder(mouse)) {
    source = mouse;
    QPoint diffMouseCorner = QPoint(source.x()-mouse.x(), source.y()-mouse.y());
    return new InteractiveShapeMoveStatus(m_view, backStatus, diffMouseCorner, source, origin);
  }
  
  if (mouse.y() < upLine)
    if (mouse.x() < leftLine) {
      origin = shape->getAreaP2();
      source = shape->getAreaP1();
      QPoint diffMouseCorner = QPoint(source.x()-mouse.x(), source.y()-mouse.y());
      return new InteractiveShapeResizeCornerStatus(m_view, backStatus, diffMouseCorner, source, origin);
    }
    else if  (mouse.x() > rightLine) {
      origin = QPoint(shape->getAreaP1().x(), shape->getAreaP2().y());
      source = QPoint(shape->getAreaP2().x(), shape->getAreaP1().y());
      QPoint diffMouseCorner = QPoint(source.x()-mouse.x(), source.y()-mouse.y());
      return new InteractiveShapeResizeCornerStatus(m_view, backStatus, diffMouseCorner, source, origin);
    }
    else {
      origin = shape->getAreaP2();
      source = shape->getAreaP1();
      QPoint diffMouseCorner = QPoint(source.x()-mouse.x(), source.y()-mouse.y());
      return new InteractiveShapeResizeHeightStatus(m_view, backStatus, diffMouseCorner, source, origin);
    }

  else if (mouse.y() > downLine)
    if (mouse.x() < leftLine) {
      origin = QPoint(shape->getAreaP2().x(), shape->getAreaP1().y());
      source = QPoint(shape->getAreaP1().x(), shape->getAreaP2().y());
      QPoint diffMouseCorner = QPoint(source.x()-mouse.x(), source.y()-mouse.y());
      return new InteractiveShapeResizeCornerStatus(m_view, backStatus, diffMouseCorner, source, origin);
    }
    else if (mouse.x() > rightLine) {
      origin = shape->getAreaP1();
      source = shape->getAreaP2();
      QPoint diffMouseCorner = QPoint(source.x()-mouse.x(), source.y()-mouse.y());
      return new InteractiveShapeResizeCornerStatus(m_view, backStatus, diffMouseCorner, source, origin);
    }
    else {
      origin = shape->getAreaP1();
      source = shape->getAreaP2();
      QPoint diffMouseCorner = QPoint(source.x()-mouse.x(), source.y()-mouse.y());
      return new InteractiveShapeResizeHeightStatus(m_view, backStatus, diffMouseCorner, source, origin);
    }
    
  else
    if (mouse.x() < leftLine) {
      origin = shape->getAreaP2();
      source = shape->getAreaP1();
      QPoint diffMouseCorner = QPoint(source.x()-mouse.x(), source.y()-mouse.y());
      return new InteractiveShapeResizeWidthStatus(m_view, backStatus, diffMouseCorner, source, origin);
    }
    else if (mouse.x() > rightLine) {
      origin = shape->getAreaP1();
      source = shape->getAreaP2();
      QPoint diffMouseCorner = QPoint(source.x()-mouse.x(), source.y()-mouse.y());
      return new InteractiveShapeResizeWidthStatus(m_view, backStatus, diffMouseCorner, source, origin);
    }
    else {
      source = shape->getTransformedPoint(mouse);
      QPoint diffMouseCorner = QPoint(0, 0);
      return new InteractiveShapeMoveStatus(m_view, backStatus, diffMouseCorner, source, origin);
    }
}

InteractiveShapeStatus *
InteractiveRegularShapeSelectedStatus::setSelected(bool isSelected)
{
  return isSelected?(InteractiveShapeStatus *)this:(InteractiveShapeStatus *)new InteractiveRegularShapeUnSelectedStatus(m_view);
}
