/*
  Copyright (C) 2005-2009 Jerome Charton, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef INTERACTIVESHAPETRANSFORMSTATUS_HPP
#define INTERACTIVESHAPETRANSFORMSTATUS_HPP

#include <qiseTools/InteractiveShapeStatus.hpp>
#include <QPoint>

class InteractiveShapeTransformStatus : public InteractiveShapeStatus
{
  
public:
  
  InteractiveShapeTransformStatus(GLView *view, InteractiveShapeStatus *previousStatus)
    : InteractiveShapeStatus(view),
      m_previousStatus(previousStatus),
      m_origin(0, 0),
      m_source(0, 0),
      m_diffMouseCorner(0, 0)
  {}

  InteractiveShapeTransformStatus(GLView *view, InteractiveShapeStatus *previousStatus, const QPoint &diffMouseCorner, const QPoint &source)
    : InteractiveShapeStatus(view),
      m_previousStatus(previousStatus),
      m_origin(0, 0),
      m_source(source),
      m_diffMouseCorner(diffMouseCorner)
  {}
  
  InteractiveShapeTransformStatus(GLView *view, InteractiveShapeStatus *previousStatus, const QPoint &diffMouseCorner, const QPoint &source, const QPoint &origin)
    : InteractiveShapeStatus(view),
      m_previousStatus(previousStatus),
      m_origin(origin),
      m_source(source),
      m_diffMouseCorner(diffMouseCorner)
  {}
  
  
  inline virtual InteractiveShapeStatus *onLeftClick(const QPoint &/* mouse */, PaintingShape * /* shape */) {return this;}
  inline virtual InteractiveShapeStatus *onRightClick(const QPoint &/* mouse */, PaintingShape * /* shape */) {return this;}
  
  inline virtual InteractiveShapeStatus *onLeftRelease(const QPoint &/* mouse */, PaintingShape * /* shape */) {return m_previousStatus;}
  inline virtual InteractiveShapeStatus *onRightRelease(const QPoint &/* mouse */, PaintingShape * /* shape */) {return this;}
  
  inline virtual InteractiveShapeStatus *onLeftClickWithCtrl(const QPoint &/* mouse */, PaintingShape * /* shape */) {return this;}
  inline virtual InteractiveShapeStatus *onRightClickWithCtrl(const QPoint &/* mouse */, PaintingShape * /* shape */) {return this;}
  
  inline virtual InteractiveShapeStatus *onMove(const QPoint &/* mouse */, PaintingShape * /* shape */) {return this;}
  
  virtual InteractiveShapeStatus *onMoveWithLeftClick(const QPoint &mouse, PaintingShape *shape) = 0;
  inline virtual InteractiveShapeStatus *onMoveWithRightClick(const QPoint &/* mouse */, PaintingShape * /* shape */) {return this;}
  
  inline virtual InteractiveShapeStatus * setSelected(bool /* isSelected */) {return this;}
  inline virtual bool isSelected() {return true;}

protected:
  
  InteractiveShapeStatus *m_previousStatus;
  QPoint m_origin, m_source, m_diffMouseCorner;

};

#endif /* ! INTERACTIVESHAPETRANSFORMSTATUS_HPP */
