/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "InvertSelectionCommand.hpp"

#include <algorithm>

#include <qiseTools/SegModel.hpp>

InvertSelectionCommand::InvertSelectionCommand(SegModel *m)
  : UndoCommand(m)
{

}

bool
InvertSelectionCommand::doAble() const
{
  return (m_model != NULL
	  && m_model->selectable()
	  && ( (m_model->getSelectionType() == SegModel::REGION && ! m_model->getSelectedRegions().empty())
	      || (m_model->getSelectionType() == SegModel::SEGMENT && ! m_model->getSelectedDarts().empty()) )
	  );
}


void 
InvertSelectionCommand::redo()
{
  assert(doAble());

  if ( (m_model->getSelectionType() == SegModel::REGION)
       || (m_model->getSelectionType() == SegModel::SEGMENT) ) {

	

    girl::regionDeque previousSelection = m_model->getSelectedRegions();//copy
    girl::regionDeque newSelection;    

    if (previousSelection.empty()) {
      m_model->getToger()->allRegions(newSelection);
    }
    else {
      std::sort(previousSelection.begin(), previousSelection.end());
    
      girl::regionDeque regions;
      m_model->getToger()->allRegions(regions);
      girl::regionDeque::const_iterator it = regions.begin();
      const girl::regionDeque::const_iterator itEnd = regions.end();
      for ( ; it != itEnd; ++it) {
	
	const bool found = std::binary_search(previousSelection.begin(), previousSelection.end(), *it);
	if (! found) 
	  newSelection.push_back(*it);
	
	//we could remove found elements from previousSelection
	// cf std::lower_bound to get an iterator on found element (also a binary search)

      }
    }
    m_model->setSelectedRegions(newSelection);

  }
      /*

	//Case (m_model->getSelectionType() == SegModel::SEGMENT)
	//TODO: we cannot have all darts from girl !!!

  else {
    assert(m_model->getSelectionType() == SegModel::SEGMENT);

    //Code duplication

    girl::dartDeque previousSelection = m_model->getSelectedDarts();//copy
    girl::dartDeque newSelection;    

    if (previousSelection.empty()) {
      m_model->getToger()->allDarts(newSelection);
    }
    else {
      std::sort(previousSelection.begin(), previousSelection.end());
    
      girl::dartDeque darts;
      m_model->getToger()->allDarts(darts);
      girl::dartDeque::const_iterator it = darts.begin();
      const girl::dartDeque::const_iterator itEnd = darts.end();
      for ( ; it != itEnd; ++it) {
	
	const bool found = std::binary_search(previousSelection.begin(), previousSelection.end(), *it);
	if (! found) 
	  newSelection.push_back(*it);
	
	//we could remove found elements from previousSelection
	// cf std::lower_bound to get an iterator on found element (also a binary search)

      }
    }
    m_model->setSelectedDarts(newSelection);

  }    
      */


  setText(tr("Invert selection"));
}

void
InvertSelectionCommand::undo()
{
  redo();
}
