/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef MAINWINDOW_HPP
#define MAINWINDOW_HPP

#define HIDDEN_OPTIONS

#include <QMainWindow>

#include <QDialog>
#include <QList>
#include <QMap>

//#include <ise/iseMergeDriverThresholds.hpp>

#include <girl/girlToger.hpp>

//#include "ObjectFactory.h"

#include <qiseTools/QIseVersion.hpp>
#include <qiseTools/QIsePluginList.hpp>
//#include <qiseTools/RegionFeatureChoiceDialog.hpp>

// JULIAN : 
#include <qiseTools/ImageFeatureViewDockWidget.hpp>

namespace ise {
  class SplitAlgorithm;
  class SplitDriver;
  class MergeAlgorithm;
  class SplitMergeAlgorithm;
  class SegmentationClustering;
  class QuantificationThresholds;
  class MergeDriverThresholds;
  class MergeDriverSizeFiltering;
}


class QAction;
class QCloseEvent;
class QKeyEvent;
class QLabel;
class QMenu;
class QTabWidget;
class QToolBar;
class QUndoGroup;
class QUndoView;

class SegDocument;
class SegModel;
class MainView;
class MiniView;
class UndoCommand;

class SegmentationClusteringParametersDialog;
#ifdef WITH_HISTOGRAMINTERVALS
class HistogramIntervalsInteractor;
#endif

class RegionUnaryPredicateThresholdInteractor;

class ImagePyramidBuildDialog;

class Interactor;
class DragInteractor;
class DropInteractor;
class RegionSelectionInteractor;
class InfoInteractor;
class MarkInteractor;
class ShapePaintingInteractor;
class PixelAnnotatorInteractor;
class ViewMover;
class ViewMover2;

class SplitAlgorithmFactory;
class SplitDriverFactory;
class MergeAlgorithmFactory;
class MergeDriverFactory;
class SplitMergeAlgorithmFactory;
class SegmentationClusteringFactory;

class SplitAlgorithmPlugin;
class SplitDriverPlugin;
class MergeAlgorithmPlugin;
class MergeDriverPlugin;
class SplitMergeAlgorithmPlugin;
class SegmentationClusteringPlugin;
class QuantificationThresholderPlugin;
class SegmentationClusteringLocalThresholderPlugin;
class ClustererPlugin;

class SplitDriverWidgetInterface;
class MergeDriverWidgetInterface;
class SegmentationClusteringWidgetInterface;

class ColorConvertWidget;

// REMI:
class ImageAnnotatorDialog;

class MainWindow : public QMainWindow
{
  Q_OBJECT
  
public:

  /**
   * @brief Constructor.
   * 
   * @Warning must be allocated on heap (as Qt::WA_DeleteOnClose attribute is set).
   */
  explicit MainWindow(QWidget *parent = NULL);

  ~MainWindow();

  //void doCommand(UndoCommand *cmd);

  MainView *getMainView()
  {
    return m_mainView;
  }

  void loadFile(const QString &filename);

  void doCommand(UndoCommand *cmd);

  void addSegDocument(SegDocument *doc);

  void removeSegDocument(SegDocument *doc);

  void setCurrentSegDocument(SegDocument *doc);

  SegDocument *currentSegDocument();
  const SegDocument *currentSegDocument() const;

  void installToger(const girl::toger &tog, const QString &fileName = QLatin1String(""));

public slots:

  void setMoveMode();


protected slots:

  void editMenuAboutToShow();

  void about();

  void openImage();
  void openSeg();
  void openImageAsSeg();
  bool save();
  bool saveSegAs();

  void togerChanged(SegModel *);
  void undoStackCleanState();

  void changeSegmentationClustering();
  void changeSplitDriver();
  void changeSplitAlgo();
  void changeMergeDriver();
  void changeMergeAlgo();
  void changeSplitMergeAlgo();

  void changeSplitDriverParameters();
  void changeMergeDriverParameters();
  void changeMergeAlgoParameters();
  void changeSegmentationClusteringParameters();

  void showPluginsLogMessages();
  
  void moveViewLeft();
  void moveViewRight();
  void moveViewUp();
  void moveViewDown();
  void zoomIn();
  void zoomOut();
  void resetZoom();
  void centerView();
  void fitView();

  void invertSelection();
  void clearSelection();
  void selectAllRegions();
  void selectIncludedRegions();
  void selectIncludingRegions();
#ifdef HIDDEN_OPTIONS
  void selectRegionByNumber();
#endif
  void removeAllDetails();
  void removeSelection();
  void mergeAllInsideSelection();
  void mergeAllExceptSelection();
  void split();
  void merge();
  void splitMerge();

  void toggleFalseColorViewMode();

  void toggleImageView();
  void toggleTogerView();
  void toggleSelectionView();

  void setInfoMode();
  void setSelectionMode();

  void setShapesMode();
  void setPixelAnnotatorMode();
  
  //void selectRectangleShape();
  //void selectEllipseShape();
  //void insertShapesAsSegmentation();
  //void insertShapesAsSelection();

  void setDragMode();

  void setMarkMode();

  void changeDrawMode();
  void changeImageSamplingMode();
  
  void updateGuiWithSelection();

  void updateCursorInfo();
  void updateZoomInfo();

  void copy();
  void paste();
  void clipboardDataChanged();
  

  void selectRegionsFromPredicate();
#ifdef WITH_HISTOGRAMINTERVALS
  void selectRegionsFromArea();
  void selectRegionsFromRGBVariance();
  void selectRegionsFromMeanLuminance();
  void selectRegionsFromLuminanceVariance();
  void selectRegionsFromColorSpaces();
#endif
  void selectForegroundRegionsByColorArea();
  void selectForegroundRegionsByHoles();

  void currentSegDocumentChange();

  void setPyramidSettings();
  void finishPyramidDialog(int);
  void projectTogerFromUpperPyramidLevel();

  void setFeaturesChoice();
  //  void setRegionFeatureChoice();

  void openRecentImageFile();
  void openRecentSegFile();
  void clearRecentImageFiles();
  void clearRecentSegFiles();

  // REMI:
  void annotateImage();

protected:

  void newDocument();
  void updateSegDocumentTabs();
  void removeSegDocument_aux(SegDocument *doc, bool addOneIfEmpty);


  void closeEvent(QCloseEvent *event);
  
  void resolvePluginsDependances();
  
  void buildGUI();

  void setupActions();
  void setupMenuBar();
  void setupToolBar();

  void openSegInNewWindow();

  bool maybeSave();
  
  bool loadImage(const QString &filename, bool withGUI = true);

  bool loadSeg(const QString &fileName, bool withGUI = true);
  bool saveSeg(const QString &fileName)  ;

  bool loadImageAsSegmentation(const QString &fileName);

  void setCurrentSegFile(const QString &fileName);
  void updateWindowTitle();

  bool eventFilter(QObject *o, QEvent *e);

  //void wheelEvent( QWheelEvent *event );
  //  void mousePressEvent(QMouseEvent *event);

  //void paintEvent(QPaintEvent *event);

  //void adjustScrollBar(QScrollBar *scrollBar, double factor);

  void setImage(const QImage &image, const QString &fileName = QLatin1String(""));
  void setToger(const girl::toger &tog, const QString &fileName = QLatin1String(""));

  void populateSplitAlgos(QMenu *);
  void destroySplitAlgos();
  void populateSplitDriverAlgos(QMenu *);
  void destroySplitDriverAlgos();

  void populateMergeAlgos(QMenu *);
  void destroyMergeAlgos();
  void populateMergeDriverAlgos(QMenu *);
  void destroyMergeDriverAlgos();

  void populateSplitMergeAlgos(QMenu *);
  void destroySplitMergeAlgos();
  
  void populateSegmentationClusteringAlgos(QMenu *);
  void destroySegmentationClusteringAlgos();

  
  void changeQuantifLevels();


  void changeMode(Interactor *newInteractor, QAction *newAction, const QString &textStatusBar);

  void selectFirstRegionIfNecessary();

  void updateRecentImageFileActions();
  void updateRecentSegFileActions();
  void setCurrentRecentImageFile(const QString &fileName);
  void setCurrentRecentSegFile(const QString &fileName);
  
private:

  static QString ms_applicationName;
  const QString ms_pluginPath;


  QAction *m_openImageAction;
  QAction *m_openSegAction;
  QAction *m_openImageAsSegAction;
  QAction *m_saveAction;
  QAction *m_saveAsAction;
  QAction *m_closeAction;
  QAction *m_exitAction;
  QAction *m_redoAction;
  QAction *m_undoAction;
  QAction *m_invertSelectionAction;
  QAction *m_clearSelectionAction;
  QAction *m_selectAllRegionsAction;
  QAction *m_selectIncludedRegionsAction;
  QAction *m_selectIncludingRegionsAction;
#ifdef HIDDEN_OPTIONS
  QAction *selectRegionByNumberAction;
#endif

  QAction *m_selectRegionsFromPredicateAction;
#ifdef WITH_HISTOGRAMINTERVALS
  QAction *m_selectRegionsFromAreaAction;
  QAction *m_selectRegionsFromRGBVarianceAction;
  QAction *m_selectRegionsFromMeanLuminanceAction;
  QAction *m_selectRegionsFromLuminanceVarianceAction;
  QAction *m_selectRegionsFromColorSpacesAction;
#endif
  QAction *m_selectForegroundRegionsByColorAreaAction;
  QAction *m_selectForegroundRegionsByHolesAction;

  QAction *m_pyramidAction;
  QAction *m_projectTogerFromPyramidAction;

  QAction *m_featuresAction;

  QAction *m_copyAction;
  QAction *m_pasteAction;
  QAction *m_aboutAction;
  QAction *m_pluginLogAction;
  QAction *m_splitAction;
  QAction *m_mergeAction;
  QAction *m_splitMergeAction;
  QAction *m_removeAllDetailsAction;
  QAction *m_removeSelectionAction;
  QAction *m_mergeAllInsideSelectionAction;
  QAction *m_mergeAllExceptSelectionAction;

  QAction *m_moveViewLeftAction;
  QAction *m_moveViewRightAction;
  QAction *m_moveViewUpAction;
  QAction *m_moveViewDownAction;
  QAction *m_zoomInAction;
  QAction *m_zoomOutAction;
  QAction *m_resetZoomAction;
  QAction *m_centerViewAction;
  QAction *m_fitViewAction;

  QAction *m_toggleFalseColorViewModeAction;
  QAction *m_toggleImageViewAction;
  QAction *m_toggleTogerViewAction;
  QAction *m_toggleSelectionViewAction;
  
  QAction *m_selectionModeAction;

  QAction *m_shapesModeAction;
  QAction *m_pixelAnnotatorModeAction;

  QAction *m_moveModeAction;
  QAction *m_infoModeAction;
  QAction *m_markModeAction;
  QAction *m_dragModeAction;
  
  QAction *m_discreteDrawModeAction;
  QAction *m_euclideanDrawModeAction;

  QAction *m_nearestImageSamplingAction;
  QAction *m_linearImageSamplingAction;

  enum{ MaxRecentFiles = 5 };
  QAction *m_recentImageFileActions[MaxRecentFiles];
  QAction *m_recentSegFileActions[MaxRecentFiles];
  QAction *m_recentImageFileNoneAction;
  QAction *m_recentImageFileSepAction;
  QAction *m_recentImageFileClearAction;
  QAction *m_recentSegFileNoneAction;
  QAction *m_recentSegFileSepAction;
  QAction *m_recentSegFileClearAction;


  QToolBar *m_partitionToolBar;

  QMenu *m_viewMenu;

  //QLabel *m_imageLabel;

  MainView *m_mainView;
  MiniView *m_miniView;

  QUndoView *m_undoView;

  ColorConvertWidget *m_colorConvertW;
  QLabel *m_cursorPosLabel;
  QLabel *m_zoomLabel;



  SegmentationClusteringParametersDialog *m_segmentationClusteringParametersDialog;

#ifdef WITH_HISTOGRAMINTERVALS
  HistogramIntervalsInteractor *m_regionAreaSelector;
  HistogramIntervalsInteractor *m_regionRGBVarianceSelector;
  HistogramIntervalsInteractor *m_regionMeanLuminanceSelector;
  HistogramIntervalsInteractor *m_regionLuminanceVarianceSelector;
  HistogramIntervalsInteractor *m_regionColorSpacesSelector;
#endif

  RegionUnaryPredicateThresholdInteractor *m_regionUnaryPredicateThresholdInteractor;


  ImagePyramidBuildDialog *m_imagePyramidBuildDialog;
  int m_divider;

  QList<SegDocument *> m_segDocuments;
  int m_currentDocIndex;
  QUndoGroup *m_undoGroup;
  QTabWidget *m_segDocTabWidget;

  QList<std::pair<plugin_status, QString> > m_pluginsLogMessages;

  typedef QMap<QString, SplitAlgorithmPlugin *> MapActionNameSplitAlgo;

  // REMI:
  ImageAnnotatorDialog *m_imageAnnotatorDialog;

  MapActionNameSplitAlgo m_mapActionNameSplitAlgo;
  SplitAlgorithmPlugin *m_activeSplitAlgo;
  QIsePluginList<SplitAlgorithmPlugin *> m_splitAlgorithmPlugins;


  typedef QMap<QString, SplitDriverPlugin *> MapActionNameSplitDriver;
  MapActionNameSplitDriver m_mapActionNameSplitDriver;
  SplitDriverPlugin *m_activeSplitDriver;
  QIsePluginList<SplitDriverPlugin *> m_splitDriverPlugins;

  //ObjectPlugin<SplitDriverWidgetInterface *(ise::SplitDriver *), ise::SplitDriver *> m_splitDriverWidgetPlugin;

  typedef QMap<QString, MergeAlgorithmPlugin *> MapActionNameMergeAlgo;
  MapActionNameMergeAlgo m_mapActionNameMergeAlgo;
  MergeAlgorithmPlugin *m_activeMergeAlgo;
  QIsePluginList<MergeAlgorithmPlugin *> m_mergeAlgorithmPlugins;

  
  typedef QMap<QString, MergeDriverPlugin *> MapActionNameMergeDriver;
  MapActionNameMergeDriver m_mapActionNameMergeDriver;
  MergeDriverPlugin *m_activeMergeDriver;
  QIsePluginList<MergeDriverPlugin *> m_mergeDriverPlugins;
  
  //ObjectPlugin<MergeDriverWidgetInterface *(ise::MergeDriver *), ise::MergeDriver *> m_mergeDriverWidgetPlugin;

  typedef QMap<QString, SplitMergeAlgorithmPlugin *> MapActionNameSplitMergeAlgo;
  MapActionNameSplitMergeAlgo m_mapActionNameSplitMergeAlgo;
  SplitMergeAlgorithmPlugin *m_activeSplitMergeAlgo;
  QIsePluginList<SplitMergeAlgorithmPlugin *> m_splitMergeAlgorithmPlugins;

  typedef QMap<QString, SegmentationClusteringPlugin *> MapActionNameSegmentationClustering;
  MapActionNameSegmentationClustering m_mapActionNameSegmentationClustering;
  SegmentationClusteringPlugin *m_activeSegmentationClustering;
  QIsePluginList<SegmentationClusteringPlugin *> m_segmentationClusteringPlugins;
  
  //ObjectPlugin<SegmentationClusteringWidgetInterface *(ise::SegmentationClustering *), ise::SegmentationClustering *> m_segmentationClusteringWidgetPlugin;
  //ObjectPlugin<QDialog *(ise::SegmentationClustering *, MainWindow *), ise::SegmentationClustering *> m_segmentationClusteringDialogPlugin; //TODO !!!


  //ise::QuantificationThresholds *m_quantifLevels;

  girl::shared_ptr<ise::MergeDriverSizeFiltering> m_mergeDriverSizeFiltering;
  //girl::shared_ptr<ise::MergeDriverThresholds> m_mergeDriverThresholds;



  RegionSelectionInteractor *m_selector;
  ShapePaintingInteractor *m_shapeInteractor;
  PixelAnnotatorInteractor *m_pixelAnnotatorInteractor;
  ViewMover *m_viewMoverDefault;
  ViewMover *m_viewMover;
  InfoInteractor *m_infoInteractor;
  DragInteractor *m_dragInteractor;

  DropInteractor *m_dropInteractor;
  ViewMover2 *m_miniViewMover;

  MarkInteractor *m_markInteractor;

  QAction *m_lastInteractorAction;
  Interactor *m_lastInteractor;

  QString m_segFilename;
  QString m_imgFilename;
  bool m_isUntitled;//seg not saved

  //JULIAN : 
  ImageFeatureViewDockWidget *m_imgFviewDockWidget;
};


#endif

