/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "MarkInteractorDialog.hpp"

#include <QBoxLayout>
#include <QButtonGroup>
#include <QCheckBox>
//#include <QDialogButtonBox>
#include <QGroupBox>
#include <QLabel>
#include <QPushButton>
#include <QRadioButton>


#include <qiseTools/MarkWidget.hpp>


MarkInteractorDialog::MarkInteractorDialog(QWidget *parent)
  : QDialog(parent)
  , m_frame1(NULL)
  , m_frame2(NULL)
  , m_frame3(NULL)
  , m_frame4(NULL)
  , m_frame5(NULL)
  , m_mw(NULL)
  , m_and(NULL)
  , m_or(NULL)
  , m_allBoundaries(NULL)
  , m_outerBoundaries(NULL)
  , m_clearAllMarks(NULL)
  , m_markSelectedRegions(NULL)

{
  setWindowTitle(tr("Mark..."));

  buildGUI();

}  

void
MarkInteractorDialog::buildGUI()
{
  m_frame1 = new QFrame(this);

  QLabel *l1 = new QLabel(tr("Active marks: "), this);

  m_mw = new MarkWidget(this);

  connect(m_mw, SIGNAL(markChanged()), this, SLOT(changeMark()));

  QLabel *l2 = new QLabel(tr("Combination: "), this);
  //m_and = new QRadioButton(tr("AND"), this);
  m_or = new QRadioButton(tr("OR"), this);
  QButtonGroup *og = new QButtonGroup(this);
  og->addButton(m_or);
  //og->addButton(m_and);
  m_or->setChecked(true); //DEFAULT
  
  //connect(m_and, SIGNAL(clicked()), this, SLOT(changeMode()));
  connect(m_or, SIGNAL(clicked()), this, SLOT(changeMode()));

  QHBoxLayout *hLayoutC = new QHBoxLayout;
  hLayoutC->addWidget(l2);
  hLayoutC->addWidget(m_or);
  //hLayoutC->addWidget(m_and);
  hLayoutC->addStretch();
  
  QVBoxLayout *vLayout1 = new QVBoxLayout;
  vLayout1->addWidget(l1);
  vLayout1->addWidget(m_mw);
  vLayout1->addLayout(hLayoutC);
  
  m_frame1->setLayout(vLayout1);


  m_frame2 = new QFrame(this);
  m_frame2->setFrameShape(QFrame::HLine);
  

  m_frame3 = new QFrame(this);
  
  QLabel *l3 = new QLabel(tr("Mark: "), this);

  QLabel *l4 = new QLabel(tr("Applied to: "), this);
  m_allBoundaries = new QRadioButton(tr("Inner & Outer boundaries"), this);
  m_outerBoundaries = new QRadioButton(tr("Only Outer boundaries"), this);
  QHBoxLayout *hLayout3 = new QHBoxLayout;
  hLayout3->addWidget(l4);
  hLayout3->addWidget(m_allBoundaries);
  hLayout3->addWidget(m_outerBoundaries);
  m_allBoundaries->setChecked(true); //DEFAULT
  
  connect(m_allBoundaries, SIGNAL(clicked()), this, SLOT(changeApply()));
  connect(m_outerBoundaries, SIGNAL(clicked()), this, SLOT(changeApply()));


  m_clearAllMarks = new QPushButton(tr("Clear all marks"), this);
  m_markSelectedRegions = new QPushButton(tr("XOR Mark selected regions"), this);

  connect(m_clearAllMarks, SIGNAL(clicked()), this, SIGNAL(clearAllMarks()));
  connect(m_markSelectedRegions, SIGNAL(clicked()), this, SIGNAL(markSelectedRegions()));

  QHBoxLayout *hLayout4 = new QHBoxLayout;
  hLayout4->addWidget(m_clearAllMarks);
  hLayout4->addWidget(m_markSelectedRegions);

  QVBoxLayout *vLayout2 = new QVBoxLayout;
  vLayout2->addWidget(l3);
  vLayout2->addLayout(hLayout3);
  vLayout2->addLayout(hLayout4);
  m_frame3->setLayout(vLayout2);
  

  m_frame4 = new QFrame(this);
  m_frame4->setFrameShape(QFrame::HLine);

//   QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Close, this);
//   connect(buttonBox, SIGNAL(rejected()), this, SLOT(accept()));

  QVBoxLayout *vLayout = new QVBoxLayout;
  vLayout->addWidget(m_frame1);
  vLayout->addWidget(m_frame2);
  vLayout->addWidget(m_frame3);
  vLayout->addWidget(m_frame4);
  //vLayout->addWidget(buttonBox);
  //vLayout->addWidget(m_frame5);
  setLayout(vLayout);
  

}
 

void
MarkInteractorDialog::activate()
{

}


void
MarkInteractorDialog::deactivate()
{

}


void
MarkInteractorDialog::changeMark()
{
  emit markChanged(getMark());
}

void
MarkInteractorDialog::changeMode()
{
  const bool b = isAND();
  emit modeChanged(b);
}

void
MarkInteractorDialog::changeApply()
{
  const bool applyToall = applyToAllBoundaries();
  emit boundaryChanged(! applyToall);
}

girl::toger::markFlags
MarkInteractorDialog::getMark() const
{
  assert(m_mw);
  return m_mw->getMarkFlag();
}

void
MarkInteractorDialog::setMark(girl::toger::markFlags m)
{
  assert(m_mw);
  m_mw->setMarkFlag(m);
}


bool
MarkInteractorDialog::isAND() const
{
  assert(m_and);
  assert(m_or);
  if (m_and->isChecked())
    return true;
  else {
    assert(m_or->isChecked());
    return false;
  }
}

bool
MarkInteractorDialog::applyToAllBoundaries() const
{
  assert(m_allBoundaries);
  assert(m_outerBoundaries);

  if (m_allBoundaries->isChecked()) {
    assert(! m_outerBoundaries->isChecked());
    return true;
  }
  else {
    assert(m_outerBoundaries->isChecked());
    return false;
  }
}

void
MarkInteractorDialog::setApplyToAllBoundaries(bool a)
{
  m_allBoundaries->setChecked(! a);
  m_outerBoundaries->setChecked(a);
}
