/*
  Copyright (C) 2011 Cedric Aguerre  aguerre@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "MergeAlgorithmParametersDialog.hpp"

#include <QApplication>
#include <QBoxLayout>
#include <QDialogButtonBox>
#include <QMessageBox>

#include <qiseTools/MergeAlgorithmPlugin.hpp>

#include <qiseTools/MergeAlgorithmWidgetInterface.hpp>

#include "MainWindow.hpp"

MergeAlgorithmParametersDialog::MergeAlgorithmParametersDialog(MainWindow *mainWindow,
							       MergeAlgorithmPlugin *mergeAlgorithmPlugin,
							       MergeAlgorithmWidgetInterface *widget)
  : QDialog(mainWindow),
    m_mergeAlgorithmPlugin(mergeAlgorithmPlugin),
    m_widget(widget),
    m_buttonBox(NULL),
    m_mergeAlgorithmSettings()
{
  assert(m_widget);
  setWindowTitle(m_widget->windowTitle());

  m_widget->setMainView(mainWindow->getMainView());

  setAttribute(Qt::WA_DeleteOnClose);

  buildGUI();

  assert(mergeAlgorithmPlugin);
  
  m_mergeAlgorithmSettings = mergeAlgorithmPlugin->getSettings();
}

void
MergeAlgorithmParametersDialog::buildGUI()
{

  m_buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok
				     | QDialogButtonBox::Cancel, Qt::Horizontal, this);

  connect(m_buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
  connect(m_buttonBox, SIGNAL(rejected()), this, SLOT(reject()));

  QVBoxLayout *vLayout = new QVBoxLayout;
  vLayout->addWidget(m_widget);
  vLayout->addWidget(m_buttonBox);

  setLayout(vLayout);
}

void
MergeAlgorithmParametersDialog::accept()
{
  bool saveOk = m_widget->saveSettingsInPlugin();

  if (saveOk) {
    QDialog::accept();

    close(); //will delete it
  }
  else {
    QMessageBox::warning(this, tr("Merge Algorithm Parameters"),
			 tr("Merge Algorithm settings are invalid."),
			 QMessageBox::Close, 
			 QMessageBox::Close);
  }
}

void
MergeAlgorithmParametersDialog::reject()
{
  QDialog::reject();

  close(); //will delete it
}
