/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "MergeAllCommand.hpp"

#include <cassert>

#include <qiseTools/SegModel.hpp>


MergeAllCommand::MergeAllCommand(SegModel *m,
				 MergeAlgorithmPlugin *mergeAlgo,
				 MergeDriverPlugin *mergeDriver)
  :  UndoCommand(m),
     m_selectAllCmd(m),
     m_mergeCmd(m, mergeAlgo, mergeDriver),
     m_clearSelectionCmd(m)
{
  //setText(tr("Merge all regions"));
  setText(tr("Remove all details"));
}

void
MergeAllCommand::redo()
{
  m_selectAllCmd.redo();
  m_mergeCmd.redo();

  //assert(m_model->getToger()->numRegions() == m_model->getSegContextPtr()->getRegionSet()->size());

  m_model->getToger()->shrink();


  m_clearSelectionCmd.redo();
}

void
MergeAllCommand::undo()
{
  m_mergeCmd.undo();
  m_selectAllCmd.undo();
}

bool
MergeAllCommand::doAble() const
{
  return m_selectAllCmd.doAble() 
    && m_model && m_model->mergeable();
}
