/*
  Copyright (C) 2005-2013 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "MergeAllInsideSelectionCommand.hpp"

#include <iostream>//DEBUG

#include <girlTools/girlToolsRegionInfos.hpp>

#include <QApplication>



MergeAllInsideSelectionCommand::MergeAllInsideSelectionCommand(SegModel *m)
  : UndoCommand(m), m_memento()
{
  const girl::regionDeque &selectedRegions = m_model->getSelectedRegions();
  if (selectedRegions.size() > 0) {
    setText(tr("Merge all regions inside selection"));
  }
}

bool
MergeAllInsideSelectionCommand::doAble() const
{
  return (m_model 
	  && m_model->mergeable()
	  && m_model->getSelectionType() == SegModel::REGION
	  && m_model->getSelectedRegions().size() > 0);
}

void
MergeAllInsideSelectionCommand::redo()
{
  assert(doAble());

  QApplication::setOverrideCursor(Qt::WaitCursor);

  assert(m_model->getToger());

  //save model 
  m_memento = m_model->save();


  girl::toger *tog = m_model->getToger();
  assert(tog);

  const girl::regionDeque &previousSelection = m_model->getSelectedRegions();

  girl::tools::mergeAllInsideRegions(*tog, previousSelection);

  m_model->setModified(true);

  //clear selection
  m_model->clearSelection();

  QApplication::restoreOverrideCursor();
}

void MergeAllInsideSelectionCommand::undo()
{
  QApplication::setOverrideCursor(Qt::WaitCursor);

  m_model->restore(m_memento);
  
  QApplication::restoreOverrideCursor();
}
