/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef MERGEDRIVERPARAMETERSDIALOG_HPP
#define MERGEDRIVERPARAMETERSDIALOG_HPP

#include <QDialog>

#include <ise/iseMergeDriver.hpp>

class QDialogButtonBox;
class QLabel;

class MainWindow;
class MergeDriverPlugin;
class MergeDriverSettings;
class SegModel;
class MergeDriverWidgetInterface;
class SelectionSettingsWidget;
class GLSelectionLayer;


class MergeDriverParametersDialog : public QDialog
{
  Q_OBJECT

public:


  MergeDriverParametersDialog(MainWindow *mainWindow,
			      MergeDriverPlugin *mergeDriverPlugin,
			      MergeDriverWidgetInterface *widget);

protected slots:

  void accept();
  void reject();

  void scheduleUpdateSelection();
  void updateSelection();

  //void changeValidity(bool valid);

  void changePreviewColor(QRgb c);
  void changePreviewAlpha(float alpha);
  void changePreviewFill(bool b);

protected:

  void buildGUI();  

  void restoreView();

  void updateSelectionInfo(size_t numSelected,
			   size_t numFiltered,
			   size_t numCandidates);
    
protected:

  MergeDriverPlugin *m_mergeDriverPlugin;
  MergeDriverWidgetInterface *m_widget;
  
  QDialogButtonBox *m_buttonBox;
  QPushButton *m_previewButton;
  SelectionSettingsWidget *m_previewSettingsW;

  QLabel *m_infoW;

  GLSelectionLayer *m_selectionLayer;
  bool m_wasSelectionDisplayed;

  girl::shared_ptr<MergeDriverSettings> m_mergeDriverSettings;

  size_t m_updateSelection;

};


#endif /* ! MERGEDRIVERPARAMETERSDIALOG_HPP */
