/*
  Copyright (C) 2005-2010 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#include "PaintingComplexShape.hpp"


void
PaintingComplexShape::updateAreaPoints()
{
  const int nbPoints = m_points.size();
  for (int i = 0; i < nbPoints; ++i) {
    const QPoint &point = m_points.at(i);

    if (point.x() < m_areaP1.x())
      m_areaP1.setX(point.x());
    else if (point.x() > m_areaP2.x())
      m_areaP2.setX(point.x());
    if (point.y() < m_areaP1.y())
      m_areaP1.setY(point.y());
    else if (point.y() > m_areaP2.y())
      m_areaP2.setY(point.y());
  }
}

void PaintingComplexShape::translate(QPoint &source, const QPoint &target)
{
  const int x = target.x() - source.x();
  const int y = target.y() - source.y();

  const int nbPoints = m_points.size();
  for (int i = 0; i < nbPoints; ++i) {
    translatePoint(m_points[i], x, y);
  }
  translatePoint(m_areaP1, x, y);
  translatePoint(m_areaP2, x, y);
  
  source = target;
  //B: Do this affectation because it is done in PaintingRegularShape::translate()
  //ugly... See comment in PaintingRegularShape::translate()
}

void
PaintingComplexShape::resize2D(QPoint &origin, QPoint &source, const QPoint &target)
{
  //B: code adapted from PaintingRegularShape::resize2D()...

  QPoint oldRealOrigin = getTransformedPoint(origin);
  QPoint newRealOrigin;
  const double xScale = getXScale(origin, source, target);
  const double yScale = getYScale(origin, source, target);

  const int nbPoints = m_points.size();
  for (int i = 0; i < nbPoints; ++i) {
    transformPoint(origin, m_points[i], xScale, yScale);
  }
  updateAreaPoints();
  transformPoint(origin, source, xScale, yScale);//B: Do this because it is done in PaintingRegularShape::resize2D()
  
  newRealOrigin = getTransformedPoint(origin);
  
  translatePoint(source, oldRealOrigin.x()-newRealOrigin.x(), oldRealOrigin.y()-newRealOrigin.y());
  translatePoint(origin, oldRealOrigin.x()-newRealOrigin.x(), oldRealOrigin.y()-newRealOrigin.y());
  translate(newRealOrigin, oldRealOrigin);
}
