/*
  Copyright (C) 2005-2009 Jerome Charton, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef PAINTINGCOMPLEXSHAPE_HPP
#define PAINTINGCOMPLEXSHAPE_HPP

#include <qiseTools/PaintingShape.hpp>

#include <QVector>
#include <QPoint>

class SegView;
class QPaint;

class PaintingComplexShape : public PaintingShape {
  
public:

  PaintingComplexShape(const QVector<QPoint> &points)
    : m_points(points) 
  {}

  //virtual void draw(const SegView *view, QPainter &painter) = 0;

  //virtual void getEnclosedRegions(girl::regionDeque &regionsInRectangle, GLView *view) = 0;
  
  virtual void draw(const GLView *view, QPainter &painter) = 0;

  virtual PaintingShape *clone() = 0;
  
  virtual bool isUnder(const QPoint &point) const = 0;

  //virtual void getSegmentationPath(int w, int h, girl::pathDeque &paths) = 0;

  //virtual void getScanlineSet(int w, int h, ScanlineSet &scanlineSet) = 0;

  virtual void setPoint(unsigned int index, const QPoint &point)
  {
    assert((int)index < m_points.size());
    m_points[index] = point;

    updateAreaPoints();    
  }
  
  virtual QVector<QPoint> getPoints() const 
  {
    return m_points;
  }
  
  virtual void translate(QPoint &source, const QPoint &target);

  virtual void resize2D(QPoint &origin, QPoint &source, const QPoint &target);

protected:

  void updateAreaPoints();

  virtual void getSegments(std::vector<std::vector<girl::point> > &segments, 
			   girl::tools::AABBox &shapeBBox) = 0;

protected:
  
  QVector<QPoint> m_points;

};

#endif /* ! PAINTINGCOMPLEXSHAPE_HPP */
