/*
  Copyright (C) 2005-2010 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "PaintingPolygon.hpp"

#include <girlTools/girlToolsAABBox.hpp>

#include <qiseTools/GLView.hpp>

#include <QPainter>

void
PaintingPolygon::draw(const GLView *view, QPainter &painter)
{
  //B: inspired from PaintingEllipse::draw() !?

  const qreal zoomFactor = view->getZoomFactor();
  const QPoint absolutZero = view->viewPos();
  
  QVector<QPoint> points = m_points; //copy
  const int nbPoints = points.size();
  for (int i=0; i<nbPoints; ++i) {
    QPoint &pt = points[i];
    const int x = static_cast<int>((pt.x() * zoomFactor) + absolutZero.x());
    const int y = static_cast<int>((pt.y() * zoomFactor) + absolutZero.y());
    //B: is it the right way to round these doubles ?...
    pt = QPoint(x, y);
  }
  painter.drawPolygon(points.data(), nbPoints);
  
}

bool
PaintingPolygon::isUnder(const QPoint &point) const
{
  return false; //DEBUG
}


void 
PaintingPolygon::getSegments(std::vector<std::vector<girl::point> > &segments, 
			     girl::tools::AABBox &shapeBBox)
{
  const int nbPoints = m_points.size();

  segments.clear();
  if (nbPoints == 0) // <3 ?? in this case, the contour cannot be closed !?!
    return;


  segments.resize(1);
  std::vector<girl::point> &segments0 = segments[0];

  segments0.clear();
  segments0.reserve(nbPoints+1);

  
  const QPoint &qpt0 = m_points.at(0);
  girl::point pt0(qpt0.x(), qpt0.y());
  pt0 = getTransformedPoint(pt0);
  //transform to inter-pixels !
  pt0 += girl::point(-1, -1);
  segments0.push_back(pt0);
  
  shapeBBox = girl::tools::AABBox(pt0.x(), pt0.y(), pt0.x(), pt0.y());
  assert(shapeBBox.isValid());

  for (int i = 1; i < nbPoints; ++i) { //start from 1
    //    m_points[i]

    const QPoint &qpt = m_points.at(i);
    girl::point pt(qpt.x(), qpt.y());
    pt = getTransformedPoint(pt);
    //transform to inter-pixels !
    pt += girl::point(-1, -1);
    segments0.push_back(pt);

    shapeBBox.add(pt);
    assert(shapeBBox.isValid());
  }

  segments0.push_back(pt0); //to close contour

}
