/*
  Copyright (C) 2005-2009 Jerome Charton, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "PaintingRectangle.hpp"

#include <QPoint>
#include <QPainter>

#include <girl/girlUtils.hpp> //nearByInt
#include <girlTools/girlToolsAABBox.hpp>
#include <qiseTools/GLView.hpp>
#include <qiseTools/SegView.hpp>
#include <qiseTools/ScanlineSet.hpp>

#include <vector>

#include <iostream> // DEBUG

void
PaintingRectangle::draw(const GLView *view, QPainter &painter)
{
  qreal zoomFactor = view->getZoomFactor();
  QPoint absolutZero = view->viewPos();

  const int x = static_cast<int>(girl::nearByInt(m_p1.x() * zoomFactor) + absolutZero.x());
  const int y = static_cast<int>(girl::nearByInt(m_p1.y() * zoomFactor) + absolutZero.y());
  const int w = static_cast<int>(girl::nearByInt(m_p2.x() - m_p1.x()) * zoomFactor);
  const int h = static_cast<int>(girl::nearByInt(m_p2.y() - m_p1.y()) * zoomFactor);
  //B: is it the right way to round these doubles ?...


  painter.drawRect(x, y, w, h);
}


void 
PaintingRectangle::getSegments(std::vector<std::vector<girl::point> > &segments, 
			       girl::tools::AABBox &shapeBBox)
{
  segments.resize(1);
  segments[0].clear();

  segments[0].reserve(4+1);
  
  girl::point p1 = girl::point(m_areaP1.x(), m_areaP1.y());
  girl::point p3 = girl::point(m_areaP2.x() - 1, m_areaP2.y() - 1);
  girl::point p4 = girl::point(p1.x(), p3.y());
  girl::point p2 = girl::point(p3.x(), p1.y());

  p1 = getTransformedPoint(p1);
  p2 = getTransformedPoint(p2);
  p3 = getTransformedPoint(p3);
  p4 = getTransformedPoint(p4);

  std::cerr<<"pixels: p1=("<<p1.x()<<"; "<<p1.y()<<") p3=("<<p3.x()<<"; "<<p3.y()<<")"<<std::endl;

  //transform to inter-pixels !
#if 1
  p1 += girl::point(-1, -1);
  p2 += girl::point(-1, -1);
  p3 += girl::point(-1, -1);
  p4 += girl::point(-1, -1);
#else
  p1 += girl::point(-1, -1);
  p2 += girl::point(0, -1);
  p3 += girl::point(0, 0);
  p4 += girl::point(-1, 0);
#endif

  std::cerr<<"inter-pixels: p1=("<<p1.x()<<"; "<<p1.y()<<") p3=("<<p3.x()<<"; "<<p3.y()<<")"<<std::endl;

  segments[0].push_back(p1);
  segments[0].push_back(p2);
  segments[0].push_back(p3);
  segments[0].push_back(p4);
  segments[0].push_back(p1);


  const int xMin = std::min(std::min(p1.x(), p2.x()), std::min(p3.x(), p4.x()));
  const int yMin = std::min(std::min(p1.y(), p2.y()), std::min(p3.y(), p4.y()));
  const int xMax = std::max(std::max(p1.x(), p2.x()), std::max(p3.x(), p4.x()));
  const int yMax = std::max(std::max(p1.y(), p2.y()), std::max(p3.y(), p4.y()));

  shapeBBox = girl::tools::AABBox(xMin, yMin, xMax, yMax);
  
}

