/*
  Copyright (C) 2005-2009 Jerome Charton, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "PaintingRegularShape.hpp"

bool
PaintingRegularShape::isUnder(const QPoint &point) const
{
  return m_areaP1.x() <= point.x() && m_areaP1.y() <= point.y()
    && m_areaP2.x() >= point.x() && m_areaP2.y() >= point.y();
}

void
PaintingRegularShape::resize2D(QPoint &origin, QPoint &source, const QPoint &target)
{
  QPoint oldRealOrigin = getTransformedPoint(origin);
  QPoint newRealOrigin;
  const double xScale = getXScale(origin, source, target);
  const double yScale = getYScale(origin, source, target);

  transformPoint(origin, m_p1, xScale, yScale);
  transformPoint(origin, m_p2, xScale, yScale);
  updateAreaPoints();
  transformPoint(origin, source, xScale, yScale);
  
  newRealOrigin = getTransformedPoint(origin);
  
  translatePoint(source, oldRealOrigin.x()-newRealOrigin.x(), oldRealOrigin.y()-newRealOrigin.y());
  translatePoint(origin, oldRealOrigin.x()-newRealOrigin.x(), oldRealOrigin.y()-newRealOrigin.y());
  translate(newRealOrigin, oldRealOrigin);
}

void
PaintingRegularShape::updateAreaPoints()
{
  m_areaP1 = QPoint(std::min(m_p1.x(), m_p2.x()), std::min(m_p1.y(), m_p2.y()));
  m_areaP2 = QPoint(std::max(m_p1.x(), m_p2.x()), std::max(m_p1.y(), m_p2.y()));
}

void
PaintingRegularShape::translate(QPoint &source, const QPoint &target)
{
  const int x = target.x() - source.x();
  const int y = target.y() - source.y();

  translatePoint(m_p1, x, y);
  translatePoint(m_p2, x, y);

  //updateAreaPoints();
  translatePoint(m_areaP1, x, y);
  translatePoint(m_areaP2, x, y);

  
  source = target;

  //B:design: source not const because of this affectation ???
  // This affectation should probably be done elsewhere !...
}


