/*
  Copyright (C) 2005-2009 Jerome Charton, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef PAINTINGREGULARSHAPE_HPP
#define PAINTINGREGULARSHAPE_HPP

#include <qiseTools/PaintingShape.hpp>
#include <algorithm>
#include <QPoint>

class GLView;
class QPaint;

/**
 * Regular shape : all shape drawable by its bounding box.
 */ 
class PaintingRegularShape : public PaintingShape
{
  
public:
  
  PaintingRegularShape(const QPoint &p1, const QPoint &p2) 
  : PaintingShape(), 
    m_p1(p1), 
    m_p2(p2) 
  {
    updateAreaPoints();
  }
  
  virtual void draw(const GLView *view, QPainter &painter) = 0;
  
  virtual PaintingShape *clone() = 0;
  
  virtual bool isUnder(const QPoint &point) const;

  //virtual void getSegmentationPath(int w, int h, girl::pathDeque &paths);
  //virtual void getSegmentationPath(girl::pathDeque &paths);
  //virtual void getScanlineSet(int w, int h, ScanlineSet &scanlineSet);

  virtual void resize2D(QPoint &origin, QPoint &source, const QPoint &target);
  
  // Just to try !!!
  virtual void setPoint(unsigned int index, const QPoint &point) {
    assert(index < 2);
    if (index == 0)
      m_p1 = point;
    else
      m_p2 = point;
    updateAreaPoints();
  }
  
  // Just to try !!!
  virtual QVector<QPoint> getPoints() const {
    QVector<QPoint> vector;
    vector.push_back(m_p1);
    vector.push_back(m_p2);
    return vector;
  }
  
  inline const QPoint &getP1() {return m_p1;}
  inline const QPoint &getP2() {return m_p2;}

  inline void setP1(const QPoint &p) {m_p1 = p; updateAreaPoints();}
  inline void setP2(const QPoint &p) {m_p2 = p; updateAreaPoints();}

  virtual void translate(QPoint &source, const QPoint &target);

protected:
  
  void updateAreaPoints();

  virtual void getSegments(std::vector<std::vector<girl::point> > &segments, 
			   girl::tools::AABBox &shapeBBox) = 0;

    
protected:

  //points to draw shape
  //(p1: first click, p2 release click)
  //(only two points to draw rectangle  &ellipse)
  QPoint m_p1;
  QPoint m_p2;
  
};

#endif /* ! PAINTINGREGULARSHAPE_HPP */
