/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "PathsInsertionCommand.hpp"

#include <iostream>
#include <QApplication>



PathsInsertionCommand::PathsInsertionCommand(SegModel *m,
					     girl::pathDeque paths,
					     bool insertOnebyOne)
  : UndoCommand(m),
    m_paths(paths),
    m_insertOneByOne(insertOnebyOne)
{
  assert(m_paths.size() == paths.size());
  const size_t nbPaths = paths.size();
  if (nbPaths > 0) {
    if (nbPaths == 1)
      setText(tr("Insert one path"));
    else
      setText(tr("Insert %1 paths").arg(nbPaths));
  }
}

bool
PathsInsertionCommand::doAble() const
{
  return (m_model 
	  && m_model->selectable()
	  && m_paths.size() > 0);
}

void
PathsInsertionCommand::redo()
{
  assert(doAble());

  QApplication::setOverrideCursor(Qt::WaitCursor);

  assert(m_model->getToger());

  //save model 
  m_memento = m_model->save();

  //do the insertion
  bool result = true;
  if (m_insertOneByOne) {

    girl::pathDeque::const_reverse_iterator it = m_paths.rbegin();
    const girl::pathDeque::const_reverse_iterator itEnd = m_paths.rend();
    for ( ; it != itEnd; ++it) {
      
     girl::pathDeque lPaths;
     lPaths.push_back(*it);

     girl::regionDeque newRegions;
     girl::regionDeque updatedRegions;
     const bool lResult = m_model->getSegContextPtr()->getToger()->insertPaths(lPaths, newRegions, updatedRegions);
     if (lResult == false) {
       result = false;
       break;
     }
    }

  }
  else {
    girl::regionDeque newRegions;
    girl::regionDeque updatedRegions;
    result = m_model->getSegContextPtr()->getToger()->insertPaths(m_paths, newRegions, updatedRegions);
  }
  //B:TODO: We do NOT use produced modified regions !!!
  //we hope their data (in RegSet) is updated with hooks !!!...


  m_model->setModified(result);

  //clear selection
  if (result)
    m_model->clearSelection();

  QApplication::restoreOverrideCursor();
}

void
PathsInsertionCommand::undo()
{
  QApplication::setOverrideCursor(Qt::WaitCursor);

  m_model->restore(m_memento);

  QApplication::restoreOverrideCursor();
}

