/*
  Copyright (C) 2012 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#ifndef PIXELANNOTATORDIALOG_HPP
#define PIXELANNOTATORDIALOG_HPP


#include <QColor>
#include <QDialog>
#include <QHash>

class QSlider;
class QPushButton;
class QCheckBox;

class ImageConcept;
class ImageConceptListWidget;

class PixelAnnotatorInteractor;

class PixelAnnotatorDialog : public QDialog
{
  Q_OBJECT

public:

  PixelAnnotatorDialog(PixelAnnotatorInteractor *interactor, QWidget *parent);


  //void activate();
  //void deactivate();

  void saveImage(const QImage &img);
  
  /**
   * Get scale value in [0; 100]
   */
  int getScale() const;

  /**
   * Set scale value in [0; 100]
   */
  void setScale(int s);

  /**
   * Get opacity value in [0.0f; 1.0f]
   */
  float getOpacity() const;

  /**
   * Set opacity value in [0.0f; 1.0f]
   */
  void setOpacity(float opacity);


  void setFill(bool onoff);

  QRgb getCurrentConceptColor() const;
  QRgb getDefaultConceptColor() const;

  bool getFillOnlyNotAnnotated() const;

  bool maybeSave();

signals:
    
  void labelImageLoaded(const QImage &img);

  void scaleChanged(int s);

  void opacityChanged(float f);

  void currentConceptColorChanged(QRgb color);

  void colorChanged(QRgb prevColor, QRgb newColor);

  void colorRemoved(QRgb color);

  void fillChanged(bool onoff);

  void imageToSave();

protected slots:
  
  void addImageConcept(const ImageConcept *);
  void editImageConcept(const ImageConcept *);
  void removeImageConcept(const ImageConcept *);  
  void changeSelectedConcept(const ImageConcept *);

  //void loadImage(const QImage &);

  // void accepted();
  // void addImageConcept();
  // void editImageConcept();
  // void removeImageConcept();
  // void selectedConceptChanged();
  // void importConcepts();
  // void exportConcepts();
  void changeAlpha(int);

  void stats();

protected:
  void build();
  // void clear();
  // void syncWidgetItem(ImageConcept &ic, QListWidgetItem &lwi);
  // void populate(ImageConcept *defaultConcept, const QList<ImageConcept *> &concepts);
  // void exportConcepts(const QString &str, bool exportRegions);
  // bool maybeSave();
  // bool hasModifiedConcepts() const;
 
protected:

  PixelAnnotatorInteractor *m_interactor;

  ImageConceptListWidget *m_imageConceptListWidget;

  QSlider *m_scaleSlider;
  QSlider *m_alphaSlider;
  QCheckBox *m_fillOnlyNonAnnotatedCB;
  QPushButton *m_fillButton;

  typedef QHash<const ImageConcept *, QRgb> HashType;
  HashType m_concepts2colors;
};

#endif /*! PIXELANNOTATORDIALOG_HPP */
