/*
  Copyright (C) 2012 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#ifndef PIXELANNOTATORINTERACTOR_HPP
#define PIXELANNOTATORINTERACTOR_HPP


#include <qiseTools/Interactor.hpp>

#include <QImage>

class MainView;
class MainWindow;
class GLBlendImageLayer;
//class GLRectangleLayer;
class RectangleSelector;

class PixelAnnotatorDialog;

class PixelAnnotatorInteractor : public Interactor
{
  Q_OBJECT

public:

  PixelAnnotatorInteractor(MainView *v, MainWindow *mw);

  virtual bool eventFilter(QObject *target, QEvent *event);

  virtual void activate();

  virtual void deactivate();
  
protected slots:

  void changeCurrentColor(QRgb color);
  void installImage(const QImage &img);
  void changeScale(int scale);
  void changeOpacity(float opacity);
  void changeColor(QRgb prevColor, QRgb newColor);

  void changeFill(bool onoff);

  void saveImage();

protected:

  void setShapeColor(bool controlPressed);
  void setShapePos(const QPointF &pt);
  void paintShape();

  friend class PixelAnnotatorDialog;
  const QImage &getImage() const { return m_img; }

  void growRegion(const QPointF &ptf);

protected:
  MainView *m_view;
  MainWindow *m_mainWindow;

  PixelAnnotatorDialog *m_dialog;
  GLBlendImageLayer *m_imageLayer;

  //GLRectangleLayer *m_sLayer;
  RectangleSelector *m_selector;

  QImage m_img;
  QRgb m_currentColor;
  int m_scale;
  bool m_pressed;

  bool m_fill;
};

#endif /* ! PIXELANNOTATORINTERACTOR_HPP */
