/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "RegionSelectAllCommand.hpp"

#include <qiseTools/SegModel.hpp>

RegionSelectAllCommand::RegionSelectAllCommand(SegModel *m)
  : UndoCommand(m),
    m_previousSelection()
{

}

bool
RegionSelectAllCommand::doAble() const
{
  return (m_model 
	  && m_model->selectable()
	  &&  m_model->getSelectionType() == SegModel::REGION);
}


void 
RegionSelectAllCommand::redo()
{
  assert(doAble());

  m_previousSelection = m_model->getSelectedRegions(); //copy

  m_model->clearSelection();
  girl::regionDeque regions;
  m_model->getToger()->allRegions(regions);
  m_model->setSelectedRegions(regions);
  setText(tr("Select all %1 regions").arg(regions.size()));
}

void
RegionSelectAllCommand::undo()
{
  m_model->setSelectedRegions(m_previousSelection);
}
