/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "RegionSelectIncludingRegionsCommand.hpp"

#include <girlTools/girlToolsRegionInfos.hpp>
#include <qiseTools/SegModel.hpp>


RegionSelectIncludingRegionsCommand::RegionSelectIncludingRegionsCommand(SegModel *m)
  : UndoCommand(m),
    m_previousSelection()
{

}

bool
RegionSelectIncludingRegionsCommand::doAble() const
{
  return (m_model 
	  && m_model->selectable()
	  &&  m_model->getSelectionType() == SegModel::REGION);
}


void 
RegionSelectIncludingRegionsCommand::redo()
{
  assert(doAble());

  m_previousSelection = m_model->getSelectedRegions(); //copy

  m_model->clearSelection();

  assert(m_model->getToger());
  
  girl::regionDeque includingRegions;

  girl::tools::getIncludingRegions(*(m_model->getToger()), m_previousSelection, includingRegions);

  m_model->setSelectedRegions(includingRegions);

  //code duplication with RegionSelectionCommand.cpp

  if (includingRegions.size() == 0)
    setText(tr("Clear selection"));
  else if (includingRegions.size() == 1)
    setText(tr("Select region %1").arg(includingRegions.front()));
  else
    setText(tr("Select %1 regions").arg(includingRegions.size()));
}

void
RegionSelectIncludingRegionsCommand::undo()
{
  m_model->setSelectedRegions(m_previousSelection);
}
