/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "RegionSelectionCommand.hpp"

#include <qiseTools/SegModel.hpp>

RegionSelectionCommand::RegionSelectionCommand(SegModel *m,
					       const girl::regionDeque &newSelection)
  : UndoCommand(m),
    m_previousSelection(),
    m_newSelection(newSelection)
{
}

bool
RegionSelectionCommand::doAble() const
{
  return (m_model 
	  && m_model->selectable()
	  && m_model->getSelectionType() == SegModel::REGION);
}


void 
RegionSelectionCommand::redo()
{
  assert(doAble());

  m_previousSelection = m_model->getSelectedRegions(); //copy

  m_model->setSelectedRegions(m_newSelection);

  if (m_newSelection.size() == 0)
    setText(tr("Clear selection"));
  else if (m_newSelection.size() == 1)
    setText(tr("Select region %1").arg(m_newSelection.front()));
  else
    setText(tr("Select %1 regions").arg(m_newSelection.size()));
}

void
RegionSelectionCommand::undo()
{
  m_model->setSelectedRegions(m_previousSelection);
}
