/*
  Copyright (C) 2005-2010 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/


#include "RegionSelectionInteractor.hpp"


#include "RegionSelectionCommand.hpp"
#include "ClearSelectionCommand.hpp"
#include "MainWindow.hpp"

#include <qiseTools/MainView.hpp>
#include "SelectionSettingsDockWidget.hpp"


RegionSelectionInteractor::RegionSelectionInteractor(MainView *v, 
						     MainWindow *mw)
  : RegionSelectionBaseInteractor(v),
    m_mainWindow(mw),
    m_selectionSettingsWidget(NULL)
{

}

void
RegionSelectionInteractor::activate()
{
  assert(m_view);

  MainView *v = static_cast<MainView *>(m_view);
  if (m_selectionSettingsWidget == NULL) {
    m_selectionSettingsWidget = new SelectionSettingsDockWidget(m_mainWindow);

    m_selectionSettingsWidget->setColor(v->getSelectionColor());
    m_selectionSettingsWidget->setAlpha(v->getSelectionOpacity());
    m_selectionSettingsWidget->setFillEnabled(v->getSelectionFillEnabled());


    connect(m_selectionSettingsWidget, SIGNAL(colorChanged(QRgb)),
	    this, SLOT(changeColor(QRgb)));
    connect(m_selectionSettingsWidget, SIGNAL(alphaChanged(float)),
	    this, SLOT(changeAlpha(float)));
    connect(m_selectionSettingsWidget, SIGNAL(fillEnabledChanged(bool)),
	    this, SLOT(changeFillEnabled(bool)));
    connect(m_selectionSettingsWidget, SIGNAL(selectorModeChanged(ShapeSelector::SelectorMode)),
	    this, SLOT(changeSelectionMode(ShapeSelector::SelectorMode)));
    connect(m_selectionSettingsWidget, SIGNAL(selectorRegionComputerChanged(ShapeSelector::SelectorRegionComputer)),
	    this, SLOT(changeSelectionRegionComputer(ShapeSelector::SelectorRegionComputer)));
    connect(m_selectionSettingsWidget, SIGNAL(selectorTypeChanged(ShapeSelector::SelectorType)),
	    this, SLOT(changeSelectionType(ShapeSelector::SelectorType)));

    assert(m_mainWindow);
    m_mainWindow->addDockWidget(Qt::LeftDockWidgetArea, m_selectionSettingsWidget);
  }


  changeSelectionType(m_selectionSettingsWidget->getSelectorType());

  changeSelectionMode(m_selectionSettingsWidget->getSelectorMode());
  changeSelectionRegionComputer(m_selectionSettingsWidget->getSelectorRegionComputer());

  m_selector->setColor(v->getSelectionColor());
  m_selector->setOpacity(v->getSelectionOpacity());

  RegionSelectionBaseInteractor::activate();

  assert(m_selectionSettingsWidget != NULL);
}

void
RegionSelectionInteractor::deactivate()
{
  assert(m_view);

  if (m_selectionSettingsWidget != NULL) {
    disconnect(m_selectionSettingsWidget, SIGNAL(colorChanged(QRgb)),
	       this, SLOT(changeColor(QRgb)));
    disconnect(m_selectionSettingsWidget, SIGNAL(alphaChanged(float)),
	       this, SLOT(changeAlpha(float)));
    disconnect(m_selectionSettingsWidget, SIGNAL(fillEnabledChanged(bool)),
	       this, SLOT(changeFillEnabled(bool)));
    disconnect(m_selectionSettingsWidget, SIGNAL(selectorModeChanged(ShapeSelector::SelectorMode)),
	       this, SLOT(changeSelectionMode(ShapeSelector::SelectorMode)));
    disconnect(m_selectionSettingsWidget, SIGNAL(selectorRegionComputerChanged(ShapeSelector::SelectorRegionComputer)),
	       this, SLOT(changeSelectionRegionComputer(ShapeSelector::SelectorRegionComputer)));
    disconnect(m_selectionSettingsWidget, SIGNAL(selectorTypeChanged(ShapeSelector::SelectorType)),
	       this, SLOT(changeSelectionType(ShapeSelector::SelectorType)));

    m_selectionSettingsWidget->hide();
    delete m_selectionSettingsWidget;

    m_selectionSettingsWidget = NULL;
  }

  RegionSelectionBaseInteractor::deactivate();

  assert(m_selectionSettingsWidget == NULL);
}

bool
RegionSelectionInteractor::eventFilter(QObject *target, QEvent *event)
{
  m_selector->setSelectedRegions(m_view->model()->getSelectedRegions()); //copy
  const bool res = RegionSelectionBaseInteractor::eventFilter(target, event);
  if (res && m_selectionChanged) {
    UndoCommand *command = NULL;
    if (m_selector->getSelectedRegions().empty())
      command = new ClearSelectionCommand(m_view->model());
    else
      command = new RegionSelectionCommand(m_view->model(), m_selector->getSelectedRegions());
    assert(m_mainWindow);
    m_mainWindow->doCommand(command);
  }
  return res;
}


