/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#include "RegionUnaryPredicateThresholdDialog.hpp"

#include <QComboBox>
#include <QPushButton>
#include <QRadioButton>
#include <QDialogButtonBox>
#include <QHBoxLayout>
#include <QGridLayout>
#include <QLabel>
#include <qiseTools/Histogram1DThresholdsWidget.hpp>
#include <qiseTools/RegionUnaryPredicateThresholdWidget.hpp>

#include <qiseTools/PluginRegistry.hpp>


RegionUnaryPredicateThresholdDialog::RegionUnaryPredicateThresholdDialog(MainView *mainView/*ise::SegmentationContext *segContext*/, QWidget *parent)
  : QDialog(parent),
    m_mainView(mainView),
    //m_segContext(segContext),
    m_ruptw(NULL),
    m_thresholders(),
    m_thresholdersCB(NULL),
    m_previewButton(NULL),
    m_targetAllRegionsButton(NULL),
    m_targetSelectedRegionsButton(NULL),
    m_histoView(NULL),
    m_threshId(0)
{
  buildGUI();
}

void
RegionUnaryPredicateThresholdDialog::buildThresholders()
{
  //- get QuantificationThresholderPlugins for thresholders giving only 1 threshold.

  QIsePluginList<QuantificationThresholderPlugin*> list = PluginRegistry::getQuantificationThresholderPlugin();

  QList<QuantificationThresholderPlugin *>::iterator it = list.begin();
  QList<QuantificationThresholderPlugin *>::iterator end = list.end();
  for (; it != end; ++it) {
    girl::shared_ptr<ise::QuantificationThresholder> thr = (*it)->getNewQuantificationThresholder();
    assert(thr);
    if (thr->getNumberOfThresholds() == 1)
      m_thresholders[(*it)->name()] = thr;
  }

  //build GUI
  if (! m_thresholders.empty()) {

    m_thresholdersCB = new QComboBox(this);

    MapTresholders::const_iterator it = m_thresholders.begin();
    const MapTresholders::const_iterator itEnd = m_thresholders.end();
    for ( ; it != itEnd; ++it) {
      m_thresholdersCB->addItem(it.key());
    }
    
    connect(m_thresholdersCB, SIGNAL(currentIndexChanged(int)), this, SLOT(applyCurrentThresholder()));
    
  }

}

void
RegionUnaryPredicateThresholdDialog::buildGUI()
{
  m_ruptw = new RegionUnaryPredicateThresholdWidget(m_mainView/*m_segContext*/, NULL, this);
  
  buildThresholders();

  m_previewButton = new QPushButton(tr("Preview"), this);

  QLabel *l = new QLabel(tr("Apply to: "), this);
  m_targetAllRegionsButton = new QRadioButton(tr("All regions"), this);
  m_targetSelectedRegionsButton = new QRadioButton(tr("Selected regions"), this);
  m_targetAllRegionsButton->setChecked(true); //default: all regions

  m_histoView = new Histogram1DThresholdsWidget(this, m_backgroundColor);
  m_threshId = m_histoView->addThreshold();

  connect(m_ruptw, SIGNAL(thresholdChanged(double)), this, SLOT(thresholdChangedInRUPTW(double)));
  //connect(m_histoView, SIGNAL(thresholdChanged(size_t)), this, SLOT(thresholdChangedInHistoView(size_t)));

  QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok
						     | QDialogButtonBox::Cancel, Qt::Horizontal, this);
  connect(buttonBox, SIGNAL(accepted()), this, SIGNAL(accepted()));
  connect(buttonBox, SIGNAL(rejected()), this, SIGNAL(rejected()));
  

  QHBoxLayout *hl = new QHBoxLayout;
  hl->addWidget(l);
  hl->addWidget(m_targetAllRegionsButton);
  hl->addWidget(m_targetSelectedRegionsButton);
  
  QGridLayout *gl = new QGridLayout;
  gl->addWidget(m_ruptw, 0, 0, 1, 3);

  if (! m_thresholders.empty()) {
    assert(m_thresholdersCB);
    QLabel *lth = new QLabel(tr("Initialize threshold: "));    
    QHBoxLayout *hlTh = new QHBoxLayout;
    hlTh->addWidget(lth);
    hlTh->addWidget(m_thresholdersCB);
    
    gl->addLayout(hlTh, 1, 0, 1, 3);
  }

  gl->addWidget(m_previewButton, 2, 0, 1, 1);
  gl->addLayout(hl, 3, 0, 1, 3);
  gl->addWidget(m_histoView, 4, 0, 1, 3);
  gl->addWidget(buttonBox, 5, 0, 1, 3);


  setLayout(gl);
  
  connect(m_ruptw, SIGNAL(pluginChanged()), this, SIGNAL(updatePreview()));
  connect(m_ruptw, SIGNAL(comparisonOperatorChanged()), this, SIGNAL(updatePreview()));
  connect(m_ruptw, SIGNAL(thresholdChanged(double)), this, SIGNAL(updatePreview()));


  /*


  connect(m_targetAllRegionsButton, SIGNAL(clicked()), 
	  this, SLOT(changeTarget()));
  connect(m_targetSelectedRegionsButton, SIGNAL(clicked()), 
	  this, SLOT(changeTarget()));  

  connect(m_thresItem, SIGNAL(valueChanged(double)), 
	  spinBox, SLOT(setValue(double)));
  connect(m_thresItem, SIGNAL(limitsChanged(Histogram1DThresholdItem *)), 
	  this, SLOT(updateLevelLimits(Histogram1DThresholdItem *)));
  connect(m_ruptw, SIGNAL(thresholdChanged(double)), 
	  m_thresItem, SLOT(setValue(double)));
  connect(m_thresItem, SIGNAL(valueChanged(int)), 
	  this, SLOT(itemMoved(int)));
  connect(m_thresItem, SIGNAL(valueChanged(double)), 
	  m_ruptw, SLOT(setThreshold(double)));
  
  
  connect(m_ruptw, SIGNAL(pluginChanged()), this, SLOT(histoToUpdate()));

  connect(m_ruptw, SIGNAL(comparisonOperatorChanged()), this, SLOT(histoToUpdate())); //?????

  
  connect(m_histoView, SIGNAL(levelDeleted(Histogram1DThresholdItem *)),
	  this, SLOT(deleteLevel(Histogram1DThresholdItem *)));
  
  connect(m_previewButton, SIGNAL(clicked()), 
	  this, SLOT(previewToUpdate()));



  */

  connect(m_previewButton, SIGNAL(clicked()), 
	  this, SIGNAL(updatePreview()));


}

void
RegionUnaryPredicateThresholdDialog::thresholdChangedInHistoView(size_t id)
{
  assert(id == m_threshId);
  const double th = m_histoView->value(id);
  m_ruptw->setThreshold(th);
}

void
RegionUnaryPredicateThresholdDialog::thresholdChangedInRUPTW(double th)
{
  m_histoView->setValue(m_threshId, th);
}


void
RegionUnaryPredicateThresholdDialog::applyCurrentThresholder()
{
  assert(m_thresholdersCB);
  const int index = m_thresholdersCB->currentIndex();
  if (index != -1) {
    const QString name = m_thresholdersCB->itemText(index);
    girl::shared_ptr<ise::QuantificationThresholder> thr = m_thresholders[name];
    
    if (thr) {
      emit updateThresholder(thr);
    }
  }

}

RegionUnaryFunctionPlugin *
RegionUnaryPredicateThresholdDialog::getRegionUnaryFunctionPlugin() const
{
  assert(m_ruptw);
  return m_ruptw->getRegionUnaryFunctionPlugin();
}

void
RegionUnaryPredicateThresholdDialog::setRegionUnaryFunctionPlugin(RegionUnaryFunctionPlugin *p)
{
  assert(m_ruptw);
  m_ruptw->setRegionUnaryFunctionPlugin(p);
}

  
ise::ComparisonOperator
RegionUnaryPredicateThresholdDialog::getComparisonOperator() const
{
  assert(m_ruptw);
  return m_ruptw->getComparisonOperator();
}


void
RegionUnaryPredicateThresholdDialog::setComparisonOperator(ise::ComparisonOperator c)
{
  assert(m_ruptw);
  m_ruptw->setComparisonOperator(c);
}


double
RegionUnaryPredicateThresholdDialog::getThreshold() const
{
  assert(m_ruptw);
  return m_ruptw->getThreshold();
}


void
RegionUnaryPredicateThresholdDialog::setThreshold(double th)
{
  assert(m_ruptw);
  m_ruptw->setThreshold(th);
  m_histoView->setValue(m_threshId, th);
}

void
RegionUnaryPredicateThresholdDialog::setHistogram(const ise::SparseHistogram1DD &h)
{
  assert(m_histoView);
  std::vector<double> thresholds;
  m_histoView->thresholds(thresholds);
  
  m_histoView->setHistogram(h); //will remove thresholds

  if (! thresholds.empty())
    m_threshId = m_histoView->addThreshold(thresholds[0]);
  if (m_threshId == 0)
    m_threshId = m_histoView->addThreshold();
}

intervals::Target
RegionUnaryPredicateThresholdDialog::getTarget() const 
{
  assert(m_targetAllRegionsButton);
  return m_targetAllRegionsButton->isChecked() ? intervals::TARGET_ALL_REGIONS 
    : intervals::TARGET_SELECTED_REGIONS;
}

void
RegionUnaryPredicateThresholdDialog::setTarget(intervals::Target t)
{
  if (getTarget() != t) {
    if (t == intervals::TARGET_ALL_REGIONS)
      m_targetAllRegionsButton->setChecked(true);
    else {
      assert(t == intervals::TARGET_SELECTED_REGIONS);
      m_targetSelectedRegionsButton->setChecked(true);
    }
    //emit targetChanged(t);
  }
}

