/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#ifndef REGIONUNARYPREDICATETHRESHOLDDIALOG_HPP
#define REGIONUNARYPREDICATETHRESHOLDDIALOG_HPP

#include <ise/iseRegionFeature.hpp>
#include <ise/iseSparseHistogram1D.hpp>
#include <ise/iseQuantificationThresholder.hpp>
#include <QMap>
#include <QDialog>
#include <qiseTools/ColorPool.hpp>
#include "IntervalsTypes.hpp"

#include <qiseTools/RegionUnaryPredicateThresholdWidget.hpp>
#include <qiseTools/MainView.hpp>

namespace ise {
  class SegmentationContext;
}
class QComboBox;
class QPushButton;
class QRadioButton;
class Histogram1DThresholdsWidget;


class RegionUnaryPredicateThresholdDialog : public QDialog
{
  Q_OBJECT

public:

  RegionUnaryPredicateThresholdDialog(MainView *mainView/*ise::SegmentationContext *segContext*/, QWidget *parent = NULL);



  RegionUnaryFunctionPlugin *getRegionUnaryFunctionPlugin() const;

  void setRegionUnaryFunctionPlugin(RegionUnaryFunctionPlugin *p);
  
  ise::ComparisonOperator getComparisonOperator() const;

  void setComparisonOperator(ise::ComparisonOperator c);

  double getThreshold() const;

  void setThreshold(double th);
  

  intervals::Target getTarget() const;

  void setTarget(intervals::Target t);


  void setHistogram(const ise::SparseHistogram1DD &h);

signals:

  void updatePreview();

  void updateThresholder(girl::shared_ptr<ise::QuantificationThresholder> thr);

protected slots :
  
  void thresholdChangedInHistoView(size_t id);
  void thresholdChangedInRUPTW(double);

  void applyCurrentThresholder();

protected:

  void buildGUI();


  void buildThresholders();


protected:
  //ise::SegmentationContext *m_segContext;
  MainView *m_mainView;

  ColorPool m_colorPool;
  QRgb m_backgroundColor;

  RegionUnaryPredicateThresholdWidget *m_ruptw;

  typedef QMap<QString, girl::shared_ptr<ise::QuantificationThresholder> > MapTresholders;
  MapTresholders m_thresholders;
  QComboBox *m_thresholdersCB;
  
  QPushButton *m_previewButton;
  QRadioButton *m_targetAllRegionsButton;
  QRadioButton *m_targetSelectedRegionsButton;
  Histogram1DThresholdsWidget *m_histoView;
  size_t m_threshId;
  
};

#endif /* ! REGIONUNARYPREDICATETHRESHOLDDIALOG_HPP */
