/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "RegionUnaryPredicateThresholdInteractor.hpp"

#include <ise/iseRegionUnaryFunction.hpp>
#include <ise/iseHistogram1DTools.hpp>

#include <qiseTools/MainView.hpp>
#include <qiseTools/SegModel.hpp>
#include <qiseTools/RegionUnaryFunctionPlugin.hpp>

#include "MainWindow.hpp"
#include "RegionUnaryPredicateThresholdDialog.hpp"

#include "RegionsVisualizerSelection.hpp"
#include "RegionsCommanderSelection.hpp"

#include <iostream>//DEBUG

RegionUnaryPredicateThresholdProcessor::RegionUnaryPredicateThresholdProcessor()
  : RegionsProcessor(),
    m_ruf(),
    m_comp(),
    m_th()
{

}

void
RegionUnaryPredicateThresholdProcessor::getFeaturesHistogram(FeaturesHistogram1D &h)
{
  const RegionsHash &valueRegionsHash = m_regionsHistogram.getUnorderedMap();
  
  RegionsHash::const_iterator it = valueRegionsHash.begin();
  const RegionsHash::const_iterator itEnd = valueRegionsHash.end();
  for ( ; it != itEnd; ++it) {
    const ise::RegionFeature_Data_Type value = it->first;
    const size_t nbRegions = it->second.size();
    h.add(value, nbRegions);
  }

  /*
  //DEBUG
  {
    std::cerr<<"RegionUnaryPredicateThresholdProcessor::getFeaturesHistogram(): "<<std::endl;
    const FeaturesHistogram1D::const_iterator itHEnd = h.end();
    FeaturesHistogram1D::const_iterator itH = h.begin();
    for ( ; itH != itHEnd; ++itH) {
      std::cerr<<itH->first<<"  "<<itH->second<<std::endl;
    }
  }
  */

}

void
RegionUnaryPredicateThresholdProcessor::updateTarget(const SegModel * /*m*/, 
						     const girl::regionDeque &regions)
{
  if (m_ruf) {
    ise::RegionUnaryFunction *f = m_ruf.get();
    assert(f);

    m_regionsHistogram.clear();
    
    girl::regionDeque::const_iterator it = regions.begin();
    const girl::regionDeque::const_iterator itEnd = regions.end();
    for ( ; it != itEnd; ++it) {
      const girl::region r = *it;
      
      const ise::RegionFeature_Data_Type value = (*f)(r);
      
      m_regionsHistogram.add(value, r);
    }

    //std::cerr<<"RegionUnaryPredicateThresholdProcessor::updateTarget regions.size="<<regions.size()<<" histo size="<<m_regionsHistogram.size()<<std::endl;
  }

}

void
RegionUnaryPredicateThresholdProcessor::getRegions(girl::regionDeque &regions)
{
  if (m_ruf) {
    assert(m_comp);

    ise::ComparisonFunction cpt = m_comp;
    const ise::RegionFeature_Data_Type th = m_th;


    const RegionsHash &valueRegionsHash = m_regionsHistogram.getUnorderedMap();
  
    RegionsHash::const_iterator it = valueRegionsHash.begin();
    const RegionsHash::const_iterator itEnd = valueRegionsHash.end();
    for ( ; it != itEnd; ++it) {
      const ise::RegionFeature_Data_Type v = it->first;

      const bool res = cpt(v, th);
      if (res) {
	const RegionCollection &rs = it->second;
	//append/insert rs to regions.
	RegionCollection::const_iterator itR = rs.begin();
	const RegionCollection::const_iterator itREnd = rs.end();
	for ( ; itR != itREnd; ++itR)
	  regions.push_back(*itR);
      }
      //std::cerr<<" comp v="<<v<<" th="<<th<<"  -> res="<<res<<std::endl;
    }

  }

}

void
RegionUnaryPredicateThresholdProcessor::setRegionUnaryFunction(girl::shared_ptr<ise::RegionUnaryFunction> f)
{
  m_ruf = f;
}

void
RegionUnaryPredicateThresholdProcessor::setComparisonFunction(ise::ComparisonFunction c)
{
  m_comp = c;
}

void
RegionUnaryPredicateThresholdProcessor::setThreshold(double th)
{
  m_th = th;
}






RegionUnaryPredicateThresholdInteractor::~RegionUnaryPredicateThresholdInteractor()
{
  delete m_visualizer;
  delete m_commander;
  delete m_processor;
}

RegionUnaryPredicateThresholdInteractor::RegionUnaryPredicateThresholdInteractor(MainWindow *mw, MainView *v)
  : Interactor(),
    m_mainWindow(mw),
    m_view(v),
    m_dialog(NULL),
    m_visualizer(NULL),
    m_commander(NULL)
{
  m_visualizer = new RegionsVisualizerSelection(mw, v);
  m_commander = new RegionsCommanderSelection(mw, v);
  m_processor = new RegionUnaryPredicateThresholdProcessor();
}

void
RegionUnaryPredicateThresholdInteractor::activate()
{
  if (m_dialog == NULL) {
    
    m_dialog = new RegionUnaryPredicateThresholdDialog(m_view/*->model()->getSegContextPtr()*/, m_mainWindow);

    connect(m_dialog, SIGNAL(updatePreview()),
	    this, SLOT(updatePreview()));
    connect(m_dialog, SIGNAL(updateThresholder(girl::shared_ptr<ise::QuantificationThresholder>)),
	    this, SLOT(updateThresholder(girl::shared_ptr<ise::QuantificationThresholder>)));
    

    connect(m_dialog, SIGNAL(accepted()),
	    this, SLOT(accept()));
    connect(m_dialog, SIGNAL(rejected()),
	    this, SLOT(reject()));

  }

  if (m_visualizer)
    m_visualizer->activate();

  updateTarget(m_dialog->getTarget(), girl::shared_ptr<ise::QuantificationThresholder>());
    
  updateDraw();
  
  m_dialog->show();

}


void
RegionUnaryPredicateThresholdInteractor::deactivate()
{
  if (m_visualizer)
    m_visualizer->deactivate();

  assert(m_dialog);
  m_dialog->hide();
}


void
RegionUnaryPredicateThresholdInteractor::updateImage(SegModel *)
{

}

void
RegionUnaryPredicateThresholdInteractor::updateToger(SegModel *)
{
  //change in toger implies change in selection
}

void
RegionUnaryPredicateThresholdInteractor::updateSelection(SegModel *)
{
  /*
  intervals::Target t = m_dialog->getTarget();
  if (t == intervals::TARGET_SELECTED_REGIONS) {
    changeTarget(t);
  }
  */
  
}


void
RegionUnaryPredicateThresholdInteractor::accept()
{
  if (m_commander) {
    girl::regionDeque regions;
    getKeptRegions(regions);

    m_commander->execute(m_view->model(), regions);
  }
  
  deactivate();

}

void
RegionUnaryPredicateThresholdInteractor::reject()
{
  deactivate();
}

void
RegionUnaryPredicateThresholdInteractor::changeTarget(intervals::Target t)
{
  updateTarget(t, girl::shared_ptr<ise::QuantificationThresholder>());

  updateDraw();

}

void
RegionUnaryPredicateThresholdInteractor::updateTarget(intervals::Target t, 
						      girl::shared_ptr<ise::QuantificationThresholder> thr)
{
  if (m_processor) {
    assert(m_view);
    assert(m_view->model());
  
    const SegModel *m = m_view->model();
    const girl::toger &tog = *(m->getToger());

    if (t == intervals::TARGET_ALL_REGIONS) {
      girl::regionDeque regions;
      tog.allRegions(regions);
      updateTarget(m, regions, thr);
    }
    else if (t == intervals::TARGET_SELECTED_REGIONS) {
      updateTarget(m, m->getSelectedRegions(), thr);
    }

    /*
    assert(m_processor);
    ise::SparseHistogram1DD h;
    m_processor->getFeaturesHistogram(h);

    assert(m_dialog);
    m_dialog->setHistogram(h);
    */
  }
}

void
RegionUnaryPredicateThresholdInteractor::updateTarget(const SegModel *m, 
						      const girl::regionDeque &regions,
						      girl::shared_ptr<ise::QuantificationThresholder> thr)
{
  if (m_processor && m_dialog) {

    RegionUnaryFunctionPlugin *rufp = m_dialog->getRegionUnaryFunctionPlugin();

    if (! rufp->hasSettings() || rufp->settings()->isValid()) {

      girl::shared_ptr<ise::RegionUnaryFunction> f = m_dialog->getRegionUnaryFunctionPlugin()->getNewRegionUnaryFunction(const_cast<SegModel*>(m)->getSegContextPtr()); //correctly configured with settings of plugin
      
      m_processor->setRegionUnaryFunction(f);
    
      m_processor->setComparisonFunction(ise::getComparisonFunction(m_dialog->getComparisonOperator()));


      m_processor->updateTarget(m, regions);

      RegionsProcessor::FeaturesHistogram1D h;
      m_processor->getFeaturesHistogram(h);

      double th = 0.0;
      if (thr) {
	assert(thr->getNumberOfThresholds() == 1);
	const size_t size = std::min(h.size(), (size_t)512); //arbitrary
	ise::Histogram1D histo;
	ise::quantizeSparseHistogram1D(h, histo, size);
	std::vector<double> thresholds;
	thr->getThresholds(histo, thresholds);
	assert(thresholds.size() == 1);
	th = thresholds[0];
      }
      else {
	th = m_dialog->getThreshold();
      }

      m_processor->setThreshold(th);    
    
      m_dialog->setHistogram(h);
      m_dialog->setThreshold(th);

    }
    
  }
}

void
RegionUnaryPredicateThresholdInteractor::updateDraw()
{
  if (m_visualizer) {

    girl::regionDeque regions;
    getKeptRegions(regions);

    m_visualizer->update(m_view->model(), regions);
  }
}

void
RegionUnaryPredicateThresholdInteractor::updateThresholder(girl::shared_ptr<ise::QuantificationThresholder> thr)
{
  updateTarget(m_dialog->getTarget(), thr);
    
  updateDraw();
  
}

void
RegionUnaryPredicateThresholdInteractor::updatePreview()
{

  updateTarget(m_dialog->getTarget(), girl::shared_ptr<ise::QuantificationThresholder>());
    
  updateDraw();

}

void
RegionUnaryPredicateThresholdInteractor::getKeptRegions(girl::regionDeque &regions)
{
  //We do not apply the predicate directly as it would entail that we re-compute all functions

  regions.clear();

  if (m_processor) {
    
    m_processor->getRegions(regions);
  }
    
}
