/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef REGIONUNARYPREDICATETHRESHOLDINTERACTOR_HPP
#define REGIONUNARYPREDICATETHRESHOLDINTERACTOR_HPP


#include <ise/iseRegionFeature.hpp>
#include <ise/iseSparseHistogram1D.hpp>
#include <ise/iseRegionsHistogram.hpp>
#include <ise/iseRegionUnaryFunction.hpp>
#include <ise/iseComparisonFunction.hpp>

#include <ise/iseQuantificationThresholder.hpp>

#include <qiseTools/Interactor.hpp>

#include "IntervalsTypes.hpp"


class MainWindow;
class MainView;
class SegModel;

class RegionsVisualizer;
class RegionsCommander;
class RegionUnaryPredicateThresholdDialog;


class RegionsProcessor
{
public:
  virtual ~RegionsProcessor() {}

  RegionsProcessor()
    : m_regionsHistogram()
  {}

  typedef ise::SparseHistogram1DD FeaturesHistogram1D;
  typedef ise::RegionsHistogram<ise::RegionFeature_Data_Type> RegionsHistogram1D;
  typedef RegionsHistogram1D::HashType RegionsHash;
  typedef RegionsHistogram1D::RegionCollection RegionCollection;
  

  virtual void getFeaturesHistogram(FeaturesHistogram1D &h) = 0;

  virtual void updateTarget(const SegModel *m, 
			    const girl::regionDeque &regions) = 0;

  virtual void getRegions(girl::regionDeque &regions) = 0;

protected:

  RegionsHistogram1D m_regionsHistogram;
};



class RegionUnaryPredicateThresholdProcessor : RegionsProcessor
{
public:

  RegionUnaryPredicateThresholdProcessor();

  virtual void updateTarget(const SegModel *m, const girl::regionDeque &regions);

  virtual void getFeaturesHistogram(FeaturesHistogram1D &h);

  virtual void getRegions(girl::regionDeque &regions);

  void setRegionUnaryFunction(girl::shared_ptr<ise::RegionUnaryFunction> f);
  
  void setComparisonFunction(ise::ComparisonFunction c);

  void setThreshold(double th);

protected:
  girl::shared_ptr<ise::RegionUnaryFunction> m_ruf;
  ise::ComparisonFunction m_comp;
  double m_th;

};



class GIRL_API RegionUnaryPredicateThresholdInteractor : public Interactor
{
  Q_OBJECT

 public:

  ~RegionUnaryPredicateThresholdInteractor();

  RegionUnaryPredicateThresholdInteractor(MainWindow *mw, MainView *v);

  //virtual bool eventFilter(QObject * /* target */, QEvent * /* event */) { return false; };

  virtual void activate();

  virtual void deactivate();
  
protected slots:

  virtual void updateImage(SegModel *);
  virtual void updateToger(SegModel *);
  virtual void updateSelection(SegModel *);

  virtual void updatePreview();
  virtual void updateThresholder(girl::shared_ptr<ise::QuantificationThresholder> thr);

  virtual void accept();
  virtual void reject();

 protected:

  void updateTarget(intervals::Target t, 
		    girl::shared_ptr<ise::QuantificationThresholder> thr);
  void changeTarget(intervals::Target t);
  void updateTarget(const SegModel *m, 
		    const girl::regionDeque &regions, 
		    girl::shared_ptr<ise::QuantificationThresholder> thr);
  void updateDraw();
  void getKeptRegions(girl::regionDeque &regions);

protected:

  MainWindow *m_mainWindow;
  MainView *m_view;
  RegionUnaryPredicateThresholdDialog *m_dialog;
  RegionsVisualizer *m_visualizer;
  RegionsCommander *m_commander;
  RegionUnaryPredicateThresholdProcessor *m_processor;
};



#endif /* ! REGIONUNARYPREDICATETHRESHOLDINTERACTOR_HPP */
