/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "RegionsVisualizerSelection.hpp"

#include <qiseTools/MainView.hpp>
#include <qiseTools/GLSelectionLayer.hpp>


RegionsVisualizerSelection::~RegionsVisualizerSelection() 
{
  assert(m_view);
  m_view->removeLayer(m_selectionLayer);
  delete m_selectionLayer;
}

void
RegionsVisualizerSelection::activate()
{
  if (m_selectionLayer == NULL) {
    m_selectionLayer = new GLSelectionLayer(m_view);
  }

  assert(m_view);
  assert(m_selectionLayer);
  m_wasSelectionDisplayed = m_view->getSelectionDisplayed();
  if (m_wasSelectionDisplayed)
    m_view->setSelectionDisplayed(false);
  m_view->addLayer(m_selectionLayer);
  girl::regionDeque regions;
  m_selectionLayer->setDrawModeForRegions(m_view->getDrawMode(), m_view->model(), regions);

  m_view->update();
}

void
RegionsVisualizerSelection::deactivate()
{
  assert(m_view);

  m_view->removeLayer(m_selectionLayer);
  
  m_view->setSelectionDisplayed(m_wasSelectionDisplayed);

}

void
RegionsVisualizerSelection::update(const SegModel *model, 
					  const girl::regionDeque &regions)
{
  assert(m_selectionLayer);
  assert(model);

  m_selectionLayer->updateRegions(model, regions);

  m_view->update();
}
