/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef SEGDOCUMENT_HPP
#define SEGDOCUMENT_HPP

class QUndoWidget;
class QUndoStack;

class SegModel;
class UndoCommand;


class SegDocument
{
public:

  /**
   * @brief Constructor.
   *
   * Build an empty model and a undo stack.
   */
  SegDocument();

  /**
   * @brief Destructor.
   *
   * @warning delete model and undo stack. 
   */
  ~SegDocument();

  /**
   * @brief Get model.
   *
   * @warning this SegDocument keeps ownership of model.
   */
  SegModel *model()
  {
    return m_model;
  }

  /**
   * @brief Get stack.
   *
   * @warning this SegDocument keeps ownership of undo stack.
   */
  QUndoStack *undoStack()
  {
    return m_undoStack;
  }

  const QUndoStack *undoStack() const
  {
    return m_undoStack;
  }

  

protected:

private:
  SegDocument(const SegDocument &);
  SegDocument &operator=(const SegDocument &);


protected:

  SegModel *m_model;
  QUndoStack *m_undoStack;
};


#endif /* ! SEGDOCUMENT_HPP */
