/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef SEGMODELQIO_HPP
#define SEGMODELQIO_HPP

#include <vector>
#include <girl/girlPoint.hpp>
#include <girlTools/girlToolsAABBox.hpp>

class QMimeData;
class BBox;
class SegModel;

#define SEGMENTS_MIMEDATA_FORMAT "application/x-togerdata"


namespace SegModelQIO
{
  
  /**
   * Return a QMimeData pointer, of format SEGMENTS_MIMEDATA_FORMAT, containing copied selected segments of model @a model.
   * It will return NULL if for whatever reason selected segments could not be saved (for example there was no selection). 
   * It is responsibility of calling function to delete returned QMimeData pointer.
   */ 
  QMimeData *copySelectedSegments(SegModel *model);
  
  /**
   * Load @a segments and @a segmentsBBox from @a mimeData.
   * Return true if segments & bbox were loaded correctly, else false.
   */
  bool getSelectedSegments(const QMimeData *mimeData,
			   std::vector< std::vector<girl::point> > &segments, 
			   girl::tools::AABBox &segmentsBBox);

}


#endif /* ! SEGMODELQIO_HPP */
