/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "SegmentationClusteringWatershedInterPixelDialog.hpp"

#include <cassert>
#include <limits>

#include <QDialogButtonBox>
#include <QGridLayout>
#include <QGroupBox>
#include <QHBoxLayout>
#include <QIntValidator>
#include <QLabel>
#include <QSpinBox>

#include <ise/iseSegmentationClusteringWatershedInterPixel.hpp>

#include <qiseTools/WeightedChannelsColorSpaceWidget.hpp>



SegmentationClusteringWatershedInterPixelDialog::SegmentationClusteringWatershedInterPixelDialog(ise::SegmentationClustering *segmentationClusteringWatershed,
									     QWidget *parent)
  :
  QDialog(parent),
  m_segmentationClusteringWatershedInterPixel(static_cast< ise::SegmentationClusteringWatershedInterPixel *> (segmentationClusteringWatershed)),
  m_wccsw(NULL),
  m_nbGradient(NULL)
{

  setWindowTitle(tr("Watershed parameters..."));

  setAttribute(Qt::WA_DeleteOnClose);

  buildGUI();

  setGUI();

}



void
SegmentationClusteringWatershedInterPixelDialog::buildGUI()
{

  m_wccsw = new WeightedChannelsColorSpaceWidget(this);

  QLabel *l = new QLabel(tr("Number of colors: "), this);
  
  m_nbGradient = new QSpinBox(this);
  m_nbGradient->setMinimum(2); //minimum 2 colors !
  m_nbGradient->setMaximum(std::numeric_limits<int>::max());

  QHBoxLayout *hbox = new QHBoxLayout;
  hbox->addWidget(l);
  hbox->addWidget(m_nbGradient);


  QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok
						     | QDialogButtonBox::Cancel, Qt::Horizontal, this);
  
  connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
  connect(buttonBox, SIGNAL(rejected()), this, SLOT(reject()));

  QVBoxLayout *vLayout = new QVBoxLayout();

  vLayout->addLayout(hbox);
  vLayout->addWidget(m_wccsw);
  vLayout->addWidget(buttonBox);
  
  setLayout(vLayout);
}

void
SegmentationClusteringWatershedInterPixelDialog::setGUI()
{
  int nbGradient;
  int conversion;
  std::vector<double> normalizedWeightsForChannels;

  m_segmentationClusteringWatershedInterPixel->getParameters(nbGradient, conversion, normalizedWeightsForChannels);
  m_wccsw->setColorSpace(conversion);
  m_wccsw->setWeights(normalizedWeightsForChannels);

  m_nbGradient->setValue(nbGradient); 
}


void
SegmentationClusteringWatershedInterPixelDialog::getGUI()
{
  const int nbGradient =  m_nbGradient->value();

  const int conversion = m_wccsw->getColorSpace();

  std::vector<double> normalizedWeightsForChannels;
  m_wccsw->getNormalizedWeights(normalizedWeightsForChannels);

  m_segmentationClusteringWatershedInterPixel->setParameters(nbGradient, conversion, normalizedWeightsForChannels);
}


void
SegmentationClusteringWatershedInterPixelDialog::accept()
{
  getGUI();

  QDialog::accept();

  close(); //will delete it
}

void
SegmentationClusteringWatershedInterPixelDialog::reject()
{
  QDialog::reject();

  close(); //will delete it
}
  
