/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "SelectionSettingsDockWidget.hpp"

#include <cassert>

#include <QBoxLayout>

#include <qiseTools/SelectionSettingsWidget.hpp>


SelectionSettingsDockWidget::SelectionSettingsDockWidget(QWidget *parent)
  : QDockWidget(tr("Selection"), parent),
    m_ssw(NULL)
{
  setAllowedAreas(Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);

  buildGUI();
}

SelectionSettingsDockWidget::~SelectionSettingsDockWidget()
{
  disconnect(m_ssw, SIGNAL(colorChanged(QRgb)), this, SIGNAL(colorChanged(QRgb)));
  disconnect(m_ssw, SIGNAL(alphaChanged(float)), this, SIGNAL(alphaChanged(float)));
  disconnect(m_ssw, SIGNAL(fillEnabledChanged(bool)), this, SIGNAL(fillEnabledChanged(bool)));

  disconnect(m_sow, SIGNAL(selectorModeChanged(ShapeSelector::SelectorMode)),
	     this, SIGNAL(selectorModeChanged(ShapeSelector::SelectorMode)));
  disconnect(m_sow, SIGNAL(selectorTypeChanged(ShapeSelector::SelectorType)),
	     this, SIGNAL(selectorTypeChanged(ShapeSelector::SelectorType)));
  disconnect(m_sow, SIGNAL(selectorRegionComputerChanged(ShapeSelector::SelectorRegionComputer)),
	     this, SIGNAL(selectorRegionComputerChanged(ShapeSelector::SelectorRegionComputer)));
}

void
SelectionSettingsDockWidget::buildGUI()
{
  m_sow = new SelectorOptionsWidget(this);

  m_ssw = new SelectionSettingsWidget(this);

  QVBoxLayout *vL = new QVBoxLayout;
  vL->addWidget(m_sow);
  vL->addWidget(m_ssw);

  QWidget *w = new QWidget(this);
  w->setLayout(vL);

  setWidget(w);

  connect(m_ssw, SIGNAL(colorChanged(QRgb)), this, SIGNAL(colorChanged(QRgb)));
  connect(m_ssw, SIGNAL(alphaChanged(float)), this, SIGNAL(alphaChanged(float)));
  connect(m_ssw, SIGNAL(fillEnabledChanged(bool)), this, SIGNAL(fillEnabledChanged(bool)));

  connect(m_sow, SIGNAL(selectorModeChanged(ShapeSelector::SelectorMode)),
	  this, SIGNAL(selectorModeChanged(ShapeSelector::SelectorMode)));
  connect(m_sow, SIGNAL(selectorTypeChanged(ShapeSelector::SelectorType)),
	  this, SIGNAL(selectorTypeChanged(ShapeSelector::SelectorType)));
  connect(m_sow, SIGNAL(selectorRegionComputerChanged(ShapeSelector::SelectorRegionComputer)),
	  this, SIGNAL(selectorRegionComputerChanged(ShapeSelector::SelectorRegionComputer)));
}

bool
SelectionSettingsDockWidget::isFillEnabled() const
{
  assert(m_ssw);
  return m_ssw->isFillEnabled();
}

void
SelectionSettingsDockWidget::setFillEnabled(bool b)
{
  assert(m_ssw);
  m_ssw->setFillEnabled(b);
}

float
SelectionSettingsDockWidget::getAlpha() const
{
  assert(m_ssw);
  return m_ssw->getAlpha();
}

void
SelectionSettingsDockWidget::setAlpha(float v)
{
  assert(m_ssw);
  m_ssw->setAlpha(v);
}

QRgb
SelectionSettingsDockWidget::getColor() const
{
  assert(m_ssw);
  return m_ssw->getColor();
}

void
SelectionSettingsDockWidget::setColor(QRgb c)
{
  assert(m_ssw);
  m_ssw->setColor(c);
}

ShapeSelector::SelectorMode
SelectionSettingsDockWidget::getSelectorMode() const
{
  assert(m_sow);
  return m_sow->getSelectorMode();
}

ShapeSelector::SelectorRegionComputer
SelectionSettingsDockWidget::getSelectorRegionComputer() const 
{
  assert(m_sow);
  return m_sow->getSelectorRegionComputer();
}

ShapeSelector::SelectorType
SelectionSettingsDockWidget::getSelectorType() const
{
  assert(m_sow);
  return m_sow->getSelectorType();
}
