/*
  Copyright (C) 2005-2009 Jerome Charton, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef SHAPEPAINTINGINTERACTOR_HPP
#define SHAPEPAINTINGINTERACTOR_HPP

#include <QPoint>
#include <girl/girlToger.hpp>

#include <qiseTools/Interactor.hpp>
#include <qiseTools/SegView.hpp>
#include <qiseTools/GLInteractiveShape.hpp>

class GLShapesLayer;
class QGraphicsItem;
class MainWindow;
class GLInteractiveShape;
class ScanlineSet;

class QAction;
class QToolBar;


class ShapePaintingInteractor : public Interactor
{
  Q_OBJECT

public:

  ~ShapePaintingInteractor();

  ShapePaintingInteractor(SegView *v, MainWindow *mw);
  
  virtual bool eventFilter(QObject *target, QEvent *event);

  virtual void activate();

  virtual void deactivate();

protected slots:

  void selectRectangleShape();
  void selectEllipseShape();
  void insertShapesAsSegmentation();
  void insertShapesAsSelection();

protected:
  
  void buildGUI();

  void setShape(GLInteractiveShape *shape);

  void checkLayer();
  void checkShapes();
  void addRegionsUnderShapes(const ScanlineSet &shapesScanlineSet, girl::regionDeque &selectedRegions);
  void removeRegionsOnBorder(const ScanlineSet &shapesScanlineSet, girl::regionDeque &selectedRegions);

  inline bool isOverView(girl::point point)
  {
    return  point.x() >= 0 && point.y() >= 0 
      && point.x() < m_view->model()->getToger()->width() && point.y() < m_view->model()->getToger()->height();
  }

protected:
  SegView *m_view;
  MainWindow *m_mainWindow;
  GLInteractiveShape *m_targetedShape;
  GLShapesLayer *m_shapesLayer;
  
  QAction *m_selectEllipseShapeAction;
  QAction *m_selectRectangleShapeAction;
  QAction *m_insertShapesAsSegmentationAction;
  QAction *m_insertShapesAsSelectionAction;
  QToolBar *m_shapesToolBar;
};

#endif /* ! SHAPEPAINTINGINTERACTOR_HPP */
