/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "SplitDriverParametersDialog.hpp"

#include <QBoxLayout>
#include <QDialogButtonBox>
#include <QMessageBox>

#include <ise/iseSplitDriver.hpp>

#include <qiseTools/SplitDriverWidgetInterface.hpp>

#include "MainWindow.hpp"


SplitDriverParametersDialog::SplitDriverParametersDialog(MainWindow *mainWindow,
							 SplitDriverFactory *splitDriverFactory,
							 SplitDriverWidgetInterface *widget)
  : QDialog(mainWindow),
    m_widget(widget),
    m_splitDriverFactory(splitDriverFactory)
{
  
  assert(m_widget);
  setWindowTitle(m_widget->windowTitle());

  setAttribute(Qt::WA_DeleteOnClose);
  
  buildGUI();
  
}

void
SplitDriverParametersDialog::buildGUI()
{

  QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok
						     | QDialogButtonBox::Cancel,
						     Qt::Horizontal, this);
  
  connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
  connect(buttonBox, SIGNAL(rejected()), this, SLOT(reject()));


  QVBoxLayout *vLayout = new QVBoxLayout;
  vLayout->addWidget(m_widget);
  vLayout->addWidget(buttonBox);

  setLayout(vLayout);
}

void
SplitDriverParametersDialog::accept()
{
  bool saveOk = m_widget->saveSettingsInPlugin();
  if (saveOk) {

    QDialog::accept();

    close(); //will delete it
  }
  else {

    QMessageBox::warning(this, tr("Split Driver Parameters"),
			 tr("Split driver settings are invalid."),
			 QMessageBox::Close, 
			 QMessageBox::Close);
  }
}

void
SplitDriverParametersDialog::reject()
{


  QDialog::reject();

  close(); //will delete it

}
