/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef SPLITMERGECOMMAND_HPP
#define SPLITMERGECOMMAND_HPP

#include <QCoreApplication> //for Q_DECLARE_TR_FUNCTIONS

#include "UndoCommand.hpp"
#include <qiseTools/SegModel.hpp>


#include <qiseTools/SplitMergeAlgorithmPlugin.hpp>
#include <qiseTools/SplitDriverPlugin.hpp>
#include <qiseTools/MergeAlgorithmPlugin.hpp>
#include <qiseTools/MergeDriverPlugin.hpp>
#include <qiseTools/SegmentationClusteringPlugin.hpp>



class SplitMergeCommand : public virtual UndoCommand
{
  Q_DECLARE_TR_FUNCTIONS(SplitMergeCommand)

public:

  SplitMergeCommand(SegModel *m,
		    SplitMergeAlgorithmPlugin *splitMergeAlgo,
		    SplitDriverPlugin *splitDriver,
		    MergeDriverPlugin *mergeDriver,
		    SegmentationClusteringPlugin *segClustering);
  
  virtual void redo();
  virtual void undo();

  virtual bool doAble() const;

protected:

  SplitMergeAlgorithmPlugin *m_splitMergeAlgoPlugin;
  SplitDriverPlugin *m_splitDriverPlugin;
  MergeDriverPlugin *m_mergeDriverPlugin;
  SegmentationClusteringPlugin *m_segClusteringPlugin;
  girl::shared_ptr<SplitMergeAlgorithmSettings> m_splitMergeAlgoSettings;
  girl::shared_ptr<SplitDriverSettings> m_splitDriverSettings;
  girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface> m_segClusteringSettings;
  girl::shared_ptr<MergeDriverSettings> m_mergeDriverSettings;
  
  SegModel::Memento m_memento;
};


#endif /* ! SPLITMERGECOMMAND_HPP */
