/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef TOGERQIO_HPP
#define TOGERQIO_HPP

#include <girl/girlToger.hpp>

class QDataStream;

#include <girlTools/girlToolsAABBox.hpp>


namespace TogerQIO
{
  /**
   * @brief Save segments of "positive darts" of all regions of toger @a tog in @a dataStream.
   *
   * @warning this is not the same as calling saveSegmentsOfRegions with
   * all the regions of @a tog. Indeed, it will roughly only save positive darts.
   * Calling saveSegmentsOfRegions on all regions of @a tog would cause lots of 
   * overlapping between segments and because of that could entail a non insertion
   * of segments in toger.
   */
  void saveSegmentsOfToger(QDataStream &dataStream, const girl::toger &tog);
  

  /**
   * @brief Get segments only for "positivie darts" of all regions of @a tog
   * in @a vs, and compute bounding box in @a bbox.
   */
  void getSegmentsForPositiveDartsOfToger(const girl::toger &tog,
					  std::vector< std::vector<girl::point> > &segments, 
					  girl::tools::AABBox &bbox);



  /**
   * @brief Save segments of all darts of regions @a regions of toger @a tog in @a dataStream.
   *
   * @sa saveSegmentsOfToger.
   */
  void saveSegmentsOfRegions(QDataStream &dataStream, const girl::toger &tog, const girl::regionDeque &regions);


  /**
   * @brief Get segments of darts of regions @a regions of toger @a tog
   * in @a vs, and compute bounding box in @a bbox.
   */
  void getSegmentsForDartsOfRegions(const girl::toger &tog, const girl::regionDeque &regions, 
				    std::vector< std::vector<girl::point> > &vs, 
				    girl::tools::AABBox &bbox);


  /**
   * @brief Save segments for darts @a darts of toger @a tog in @a dataStream.
   */
  void saveSegments(QDataStream &dataStream, const girl::toger &tog, const girl::dartDeque &darts);

  
  /**
   * @brief Get segments of darts @a darts of toger @a tog
   * in @a vs, and compute bounding box in @a bbox.
   */
  void getSegmentsForDarts(const girl::toger &tog, const girl::dartDeque &darts, 
			   std::vector< std::vector<girl::point> > &vs,
			   girl::tools::AABBox &bbox);
  

  /**
   * @brief Load segments from @a dataStream in @a segments and bounding box in @a xmin, @a ymin, @a xmax, @a ymax.
   */
  bool load(QDataStream &dataStream, 
	    std::vector< std::vector<girl::point> > &segments,
	    girl::POINT_TYPE &xmin, girl::POINT_TYPE  &ymin, girl::POINT_TYPE  &xmax, girl::POINT_TYPE  &ymax);



  void printSegmentsFromDarts(const girl::toger &tog, const girl::dartDeque &regions);
  void printSegmentsFromRegions(const girl::toger &tog, const girl::regionDeque &darts);
  void printSegmentsFromPoints(const std::vector< std::vector<girl::point> > &segments);


}

#endif /*! TOGERQIO_HPP */
