/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "ViewMover2.hpp"

#include <cassert>
#include <iostream> //DEBUG

#include <QMouseEvent>

#include <qiseTools/GLView.hpp>



ViewMover2::ViewMover2(GLView *view, GLView *dstView)
  : Interactor(),
    m_view(view),
    m_dstView(dstView),
    m_clickPos(),
    m_isMoving(false)    
{

}

void
ViewMover2::activate()
{
  assert(m_view);
  m_view->setCursor(Qt::OpenHandCursor);
}

void
ViewMover2::deactivate()
{
  assert(m_view);
  m_view->unsetCursor();
}


void
ViewMover2::setViews(GLView *view, GLView *dstView)
{
  m_view = view;
  m_dstView = dstView;
}

bool
ViewMover2::eventFilter(QObject *object, QEvent *event)
{
  if (object == m_view) {
    
    if (event->type() == QEvent::MouseButtonPress) {
      QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
      m_clickPos = mouseEvent->pos();
      m_isMoving = true;

      m_savedCursor = m_view->cursor();//see ViewMover.cpp comment.
      m_view->setCursor(Qt::ClosedHandCursor);
     
      return true;
    }
    else if (event->type() == QEvent::MouseMove && m_isMoving) {
     QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);


     /*
     QPoint trans = mouseEvent->pos() - m_clickPos;

     std::cerr<<"ViewMover2::eventFilter  trans x="<<trans.x()<<" y="<<trans.y()<<std::endl;


     assert(m_view);
     QPointF transT;
     m_view->transformFromView(trans, transT);
     trans = QPoint(static_cast<int>(transT.x()+0.5), static_cast<int>(transT.y()+0.5));

     std::cerr<<"ViewMover2::eventFilter  transformed to  x="<<transT.x()<<" y="<<transT.y()<<std::endl;
     std::cerr<<"ViewMover2::eventFilter  manually transformed to  x="<<transT.x()*m_view->getZoomFactor()<<" y="<<transT.y()*m_view->getZoomFactor()<<std::endl;
     */

     QPoint pos = mouseEvent->pos();
     QPointF posT, clickPosT;
     m_view->transformToView(pos, posT);
     m_view->transformToView(m_clickPos, clickPosT);
     QPointF transT = clickPosT - posT;
     QPoint trans = QPoint(static_cast<int>(transT.x()+0.5), static_cast<int>(transT.y()+0.5));
     
     /*
#ifndef NDEBUG
     std::cerr<<"ViewMover2::eventFilter  transT  x="<<transT.x()<<" y="<<transT.y()<<std::endl;
     const QPoint tmp = pos - m_clickPos;
     std::cerr<<"ViewMover2::eventFilter  manually transformed trans x="<<pos.x()*m_view->getZoomFactor()<<" y="<<pos.y()*m_view->getZoomFactor()<<std::endl;
#endif
     */


     assert(m_dstView);
m_dstView->moveView(m_dstView->viewPos() + trans);
     //m_dstView->moveView(m_dstView->viewPos() - trans);

     m_clickPos = mouseEvent->pos();

     return true;
    }
    else if (event->type() == QEvent::MouseButtonRelease && m_isMoving) {
      
      m_isMoving = false;

      //m_view->setCursor(Qt::OpenHandCursor);
      m_view->setCursor(m_savedCursor);//see ViewMover.cpp comment.

      return true;
    }

  }
  return false;
}

