/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "XMarkRegionCommand.hpp"

#include <qiseTools/SegModel.hpp>
#include <ise/iseSegmentationContext.hpp>

XMarkRegionCommand::XMarkRegionCommand(SegModel *m,
				       const girl::regionDeque &regions,
				       const girl::toger::markFlags mark,
				       bool onlyOuterBoundaries)
  : UndoCommand(m)
  , m_regions(regions)
  , m_mark(mark)
  , m_onlyOuterBoundaries(onlyOuterBoundaries)
{
}

XMarkRegionCommand::XMarkRegionCommand(SegModel *m,
				       const girl::region &r,
				       const girl::toger::markFlags mark,
				       bool onlyOuterBoundaries)
  : UndoCommand(m)
  , m_regions()
  , m_mark(mark)
  , m_onlyOuterBoundaries(onlyOuterBoundaries)
{
  m_regions.push_back(r);
}

bool
XMarkRegionCommand::doAble() const
{
  return (m_model 
	  && m_model->selectable()
	  && m_model->getSelectionType() == SegModel::REGION
	  && m_regions.size() > 0);
}


void 
XMarkRegionCommand::redo()
{
  assert(doAble());

  xMark();

  if (m_regions.size() == 1)
    setText(tr("xMark region %1").arg(m_regions.front()));
  else
    setText(tr("xMark %1 regions").arg(m_regions.size()));
}

void
XMarkRegionCommand::undo()
{
  xMark();
}

void
XMarkRegionCommand::xMark()
{
  assert(m_model && m_model->getToger());

  if (m_onlyOuterBoundaries) {
    girl::regionDeque::const_iterator it = m_regions.begin();
    const girl::regionDeque::const_iterator itEnd = m_regions.end();
    for ( ; it != itEnd; ++it) {
      m_model->getToger()->xMarkOuterBoundary(*it, m_mark);
    }  
  }
  else {
    girl::regionDeque::const_iterator it = m_regions.begin();
    const girl::regionDeque::const_iterator itEnd = m_regions.end();
    for ( ; it != itEnd; ++it) {
      m_model->getToger()->xMarkRegionBoundary(*it, m_mark);
    }  
  }

  m_model->setModified(true);
}
