/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "ZoomTogerCommand.hpp"

#include <iostream>//DEBUG

#include <QApplication>




ZoomTogerCommand::ZoomTogerCommand(SegModel *dstModel,
				   SegModel *origModel,
				   int zoomFactor)
  : UndoCommand(dstModel),
    m_origModel(origModel),
    m_zoomFactor(zoomFactor)
{
  setText(tr("Project zoomed toger (x%1)").arg(zoomFactor));
}

bool
ZoomTogerCommand::doAble() const
{
  return (m_model 
	  && m_model->selectable()
	  && m_origModel
	  && m_model->width() <= m_origModel->width()*m_zoomFactor 
	  && m_model->height() <= m_origModel->height()*m_zoomFactor
	  );

}

void
ZoomTogerCommand::redo()
{
  assert(doAble());

  QApplication::setOverrideCursor(Qt::WaitCursor);

  assert(m_model->getToger());

  //save model
  m_memento = m_model->save();

  //do the zoom  
  girl::toger ztog = *(m_origModel->getToger()); //do not copy observers

  //ztog.zoom(m_model->width(), m_model->height());



  const int nw = m_origModel->width()*m_zoomFactor;
  const int nh = m_origModel->height()*m_zoomFactor;
  ztog.zoom(nw, nh);

  assert(ztog.width() == nw);
  assert(ztog.height() == nh);

  const int w = m_model->width();
  const int h = m_model->height();

  if (nw != w || nh != h) {

    assert(w<=nw);
    assert(h<=nh);

    const girl::point p1((nw - w)/2,
			 (nh - h)/2);
    const girl::point p2(p1.x() + w - 1,
			 p1.y() + h - 1);

    ztog.crop(p1, p2);

  }
  
  assert(ztog.width() == w);
  assert(ztog.height() == h);


  m_model->setToger(&ztog); //clear selection !

  m_model->setModified(true);

  //m_model->setSelectedRegions(m_origModel->getSelectedRegions()); //? do we also project selection ???
  //m_model->setSelectedDarts(m_origModel->getSelectedDarts()); //? do we also project selection ???

  QApplication::restoreOverrideCursor();
}

void ZoomTogerCommand::undo()
{
  QApplication::setOverrideCursor(Qt::WaitCursor);

  m_model->restore(m_memento);

  QApplication::restoreOverrideCursor();
}
