
dnl ------------------------------------------------------------------------
dnl Try to find the QT headers and libraries.
dnl $(QT_INCLUDES) will be -Iqt_header_location (if needed)
dnl $(QT_LDFLAGS) will be -Lqt_lib_location (if needed)
#   $(LIB_QT) will be -lQtCore -lQtGui ... 
#
# QTDIR changes search paths.
# You must have VAR_WIN32, VAR_MACOSX & VAR_MACOSX_& defined to use this macro
#
dnl ------------------------------------------------------------------------
AC_DEFUN([AC_PATH_QT],
[
LIB_QT="-lqt-mt"

AC_MSG_CHECKING([for Qt 4])

ac_qt_dir=NO ac_qt_includes=NO ac_qt_libraries=NO 

AC_ARG_WITH(qt-dir,
    [  --with-qt-dir=DIR       where QT is installed. ],
    [  ac_qt_dir="$withval"
    ])

AC_ARG_WITH(qt-includes,
    [  --with-qt-includes=DIR  where the QT includes are. ],
    [  ac_qt_includes="$withval"
    ])

AC_ARG_WITH(qt-libraries,
    [  --with-qt-libraries=DIR where the QT libraries are. ],
    [  ac_qt_libraries="$withval"
    ])


AC_CACHE_VAL(ac_cv_have_qt,
[
if test ! "$ac_qt_dir" = "NO"; then
   QTDIR=$ac_qt_dir
fi

qt_incdirs="$ac_qt_includes $ac_qt_includes/Qt ${QTDIR}/include ${QTDIR}/include/qt ${QTDIR}/include/Qt  /usr/include/qt /usr/lib/qt/include /usr/local/qt/include /usr/include /usr/local/lib/qt/include "
AC_FIND_FILE(QtCore, $qt_incdirs, qt_incdir)
dnl check QT version
if test ! "$qt_incdir" = "NO"; then
  QT_VERSION=4
  QT_PACKAGE=qt4
  qtlib_prefix=libQt

  qt_incdir="$qt_incdir -I$qt_incdir/QtCore -I$qt_incdir/QtGui -I$qt_incdir/QtOpenGL"
else
  AC_FIND_FILE(qgl.h, $qt_incdirs, qt_incdir)
  QT_VERSION=3
  QT_PACKAGE=qt
  qtlib_prefix=libqt
fi
AC_SUBST(QT_VERSION)
AC_SUBST(QT_PACKAGE)
ac_qt_includes="$qt_incdir"

dnl add our own flag QT_REL (QT_VERSION is internally used by Qt)
dnl QT_MINOR_REL is added far below
QT_CPPFLAGS="-DQT_REL=$QT_VERSION"

if test ${QT_VERSION} -eq 4 && test -d $qt_incdir; then
  ac_qt_includes="$qt_incdir -I$qt_incdir/Qt"
dnl we add QT3_SUPPORT compilation flag as indicated 
dnl in http://doc.trolltech.com/4.0/porting4.html
dnl an QT_NO_DEBUG to enable the widgets plugins integration in Qt designer
#Boris: commented out ! No need for QT3 support.
  #QT_CPPFLAGS="$QT_CPPFLAGS -DQT3_SUPPORT  -DQT_NO_DEBUG"
fi



qt_libdirs="$ac_qt_libraries ${QTDIR}/lib /usr/lib/qt/lib /usr/local/lib/qt/lib /usr/lib/ /usr/local/lib/ /usr/lib64"
test -n "${QTDIR}" && qt_libdirs="${QTDIR}/lib ${QTDIR} $qt_libdirs"
if test ! "$ac_qt_libraries" = "NO"; then
  qt_libdirs="$ac_qt_libraries $qt_libdirs"
fi
test=NONE
qt_libdir=NONE
for dir in $qt_libdirs; do
  try="ls -1 $dir/$qtlib_prefix*"
  if test=`eval $try 2> /dev/null`; then qt_libdir=$dir; break; else echo "tried $dir" >&AC_FD_CC ; fi
done
ac_qt_libraries="$qt_libdir"

 qt_libraries="$ac_qt_libraries"
 qt_includes="$ac_qt_includes"
 QT_INCLUDES="-I$ac_qt_includes"
 QT_LDFLAGS="-L$ac_qt_libraries"
])

eval "$ac_cv_have_qt"
if test "$ac_qt_libraries" = NONE; then
  AC_MSG_ERROR([ not found,  put your QTDIR environnement variable to the QT directory  ]);
else
  ac_cv_have_qt="have_qt=yes \
  ac_qt_includes=$ac_qt_includes ac_qt_libraries=$ac_qt_libraries"
  AC_MSG_RESULT([ version ${QT_VERSION}, libraries $ac_qt_libraries, headers $ac_qt_includes ])
  qt_libraries="$ac_qt_libraries"
  qt_includes="$ac_qt_includes"
  QT_INCLUDES="-I$qt_includes"
  QT_LDFLAGS="-L$qt_libraries"
fi
AC_SUBST(qt_libraries)
AC_SUBST(qt_includes)
AC_SUBST(QT_INCLUDES)
AC_SUBST(QT_LDFLAGS)
AC_SUBST(QT_CPPFLAGS)
if test ${QT_VERSION} -eq 4; then
dnl With the version 4 of Qt, its features are splitted in many libs
dnl Qt3Support lib is for Qt 3 compatibility purpose
  if test ${VAR_WIN32} = 1
  then
    LIB_QT_ASSISTANT="-L${QTDIR}/lib -lQtAssistantClient4"
#Boris : no need for -lQt3Support4 -lQtNetwork4 -lQtXml4
#    LIB_QT="-lQtCore4 -lQtGui4 -lQt3Support4 -lQtOpenGL4 -lQtNetwork4 -lQtXml4"
    LIB_QT="-lQtCore4 -lQtGui4 -lQtOpenGL4"
dnl For linking purpose, we need to copy some libs
dnl Copy $QTDIR/lib/Qt*4.dll in $QTDIR/lib/libQt*4.dll if needed
    if test ! -f ${QTDIR}/lib/libQtCore4.dll ; then
      cp ${QTDIR}/lib/QtCore4.dll ${QTDIR}/lib/libQtCore4.dll
    fi
    if test ! -f ${QTDIR}/lib/libQtGui4.dll ; then
      cp ${QTDIR}/lib/QtGui4.dll ${QTDIR}/lib/libQtGui4.dll
    fi
#     if test ! -f ${QTDIR}/lib/libQt3Support4.dll ; then
#       cp ${QTDIR}/lib/Qt3Support4.dll ${QTDIR}/lib/libQt3Support4.dll
#     fi
    if test ! -f ${QTDIR}/lib/libQtOpenGL4.dll ; then
      cp ${QTDIR}/lib/QtOpenGL4.dll ${QTDIR}/lib/libQtOpenGL4.dll
    fi
#     if test ! -f ${QTDIR}/lib/libQtNetwork4.dll ; then
#       cp ${QTDIR}/lib/QtNetwork4.dll ${QTDIR}/lib/libQtNetwork4.dll
#     fi
#     if test ! -f ${QTDIR}/lib/libQtXml4.dll ; then
#       cp ${QTDIR}/lib/QtXml4.dll ${QTDIR}/lib/libQtXml4.dll
#     fi
  else
    if test ${VAR_MACOSX} = 1
    then
      #LIB_QT="-F$ac_qt_libraries -framework QtCore -framework QtGui -framework Qt3Support -framework QtOpenGL -framework QtNetwork"
      LIB_QT="-F$ac_qt_libraries -framework QtCore -framework QtGui -framework QtOpenGL"
      if test -d ${QTDIR}/lib/QtAssistantClient.framework
      then
        LIB_QT_ASSISTANT="-F$ac_qt_libraries -framework QtAssistantClient"
      else
         if test -d ${QTDIR}/lib/QtAssistant.framework
         then
           LIB_QT_ASSISTANT="-F$ac_qt_libraries -framework QtAssistant"
         fi
      fi
    else
      LIB_QT_ASSISTANT="-L${QTDIR}/lib -lQtAssistantClient"
      #LIB_QT="-lQtCore -lQt3Support -lQtGui -lQtOpenGL"
      LIB_QT="-lQtCore -lQtGui -lQtOpenGL"
    fi
  fi
else
  LIB_QT_ASSISTANT="-L${QTDIR}/lib -lqassistantclient"
  if test ${VAR_WIN32} = 1 ; then
dnl Copy $QTDIR/bin/qt-mt*.dll in $QTDIR/lib/libqt-mt.dll if needed
    if test ! -f $QTDIR/lib/libqt-mt.dll ; then
      library=$(ls -1 ${QTDIR}/bin/qt-mt*.dll)
      cp ${library} ${QTDIR}/lib/libqt-mt.dll
    fi
  fi
fi

AC_SUBST(LIB_QT)
AC_SUBST(LIB_QT_ASSISTANT)
AC_SUBST(QTDIR)
])

AC_DEFUN([MOC_ERROR_MESSAGE],
[
    AC_MSG_ERROR([No Qt meta object compiler (moc) found!
Please check whether you installed Qt correctly. 
You need to have a running moc binary.
configure tried to run $ac_cv_path_moc and the test didn't
succeed. If configure shouldn't have tried this one, set
the environment variable MOC to the right one before running
configure.
])
])

AC_DEFUN([UIC_ERROR_MESSAGE],
[
    AC_MSG_ERROR([No Qt user interface compiler (uic) found!
Please check whether you installed Qt correctly. 
You need to have a running uic binary.
configure tried to run $ac_cv_path_uic and the test didn't
succeed. If configure shouldn't have tried this one, set
the environment variable UIC to the right one before running
configure.
])
])


dnl ------------------------------------------------------------------------
dnl Find the meta object compiler in the PATH, in $QTDIR/bin, and some
dnl more usual places
dnl ------------------------------------------------------------------------
dnl
AC_DEFUN([AC_PATH_QT_MOC],
[
  search_dirs="$ac_qt_bindir ${QTDIR}/bin ${QTDIR}/src/moc \
 	    /usr/bin /usr/X11R6/bin /usr/lib/qt/bin \
	    /usr/local/qt/bin"
 
  if test ${QT_VERSION} -eq 4; then
       search_dirs="$search_dirs ${qt_libdir}/qt4/bin"
  fi

   FIND_PATH(moc, MOC, [$search_dirs], [MOC_ERROR_MESSAGE])
 

  if test -z "$MOC"; then
     if test -n "$ac_cv_path_moc"; then
       output=`eval "$ac_cv_path_moc --help 2>&1 | sed -e '1q' | grep Qt"`
     fi
     echo "configure:__oline__: tried to call $ac_cv_path_moc --help 2>&1 | sed -e '1q' | grep Qt" >&AC_FD_CC
     echo "configure:__oline__: moc output: $output" >&AC_FD_CC

     if test -z "$output"; then
       MOC_ERROR_MESSAGE
    fi
   else
     $MOC -v 2>&1 | grep 'Qt 4' > /dev/null
     if test $? -eq 1; then
       qt_bin_dirs="$ac_qt_bindir ${QTDIR}/bin ${QTDIR}/src/moc /usr/bin /usr/X11R6/bin /usr/lib64/qt4/bin /usr/lib/qt4/bin /usr/local/qt/bin ";
       AC_FIND_FILE(moc-qt4, $qt_bin_dirs, qt_bin_dir)
       MOC=$qt_bin_dir/moc-qt4
       if test ! -x $MOC; then
         MOC_ERROR_MESSAGE
       else
         AC_MSG_RESULT([ for Qt 4 found $MOC ])
       fi
     fi
   fi
   
   AC_SUBST(MOC)
])

AC_DEFUN([AC_PATH_QT_UIC],
[
dnl if this is the version 4 of Qt we use uic3
   if test ${QT_VERSION} -eq 4; then
     FIND_PATH(uic3, UIC, [$ac_qt_bindir ${QTDIR}/bin ${QTDIR}/src/uic \
	       /usr/bin /usr/X11R6/bin /usr/lib/qt/bin \
	       /usr/local/qt/bin], [UIC_ERROR_MESSAGE])
   else
     FIND_PATH(uic, UIC, [$ac_qt_bindir ${QTDIR}/bin ${QTDIR}/src/uic \
	       /usr/bin /usr/X11R6/bin /usr/lib/qt/bin \
	       /usr/local/qt/bin], [UIC_ERROR_MESSAGE])
   fi
   if test -z "$UIC"; then
     if test -n "$ac_cv_path_uic"; then
       output=`eval "$ac_cv_path_uic --help 2>&1 | sed -e '1q' | grep Qt"`
     fi
     echo "configure:__oline__: tried to call $ac_cv_path_uic --help 2>&1 | sed -e '1q' | grep Qt" >&AC_FD_CC
     echo "configure:__oline__: uic output: $output" >&AC_FD_CC

     if test -z "$output"; then
       UIC_ERROR_MESSAGE
    fi
   fi
   AC_MSG_CHECKING([whether $UIC runs])
   if test `$UIC -version 2>&1 | grep -c version` -ne 0; then
     AC_MSG_RESULT([yes])
   else
     AC_MSG_RESULT([no])
     UIC_ERROR_MESSAGE
   fi
   AC_SUBST(UIC)
dnl look for Qt minor version and add QT_MINOR_REL
   QT_MINOR_VERSION=`$UIC -version 2>&1 | awk -F'.' '{print $(NF - 1)}'`
   QT_CPPFLAGS="$QT_CPPFLAGS -DQT_MINOR_REL=$QT_MINOR_VERSION"
   AC_SUBST(QT_MINOR_VERSION)
])
