/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef AREAINTERVALSWIDGET_HPP
#define AREAINTERVALSWIDGET_HPP

#include <girl/girlCommon.hpp>//GIRL_API
#include <QWidget>

class QLabel;

class SliderSpinBoxWidget;



class GIRL_API AreaIntervalsWidget : public QWidget
{
  Q_OBJECT

public:

  /**
   * @brief Constructor.
   */
  AreaIntervalsWidget(int imageWidth, int imageHeight, QWidget *parent = NULL);

  /**
   * @briaf Set min and max area percentage values.
   *
   * @warning @a percentageMinArea and @a percentageMaxArea must be in [0.0; 1.0].
   * @warning we must have @a percentageMinArea <= @a percentageMaxArea.
   */
  void setMinMaxAreas(double percentageMinArea, double percentageMaxArea);

  /**
   * @briaf Get min and max area percentage values.
   */
  void getMinMaxAreas(double &percentageMinArea, double &percentageMaxArea);


  /**
   * @brief Set text explaining in-between regions use.
   */
  void setText(const QString &text);

signals:
  
  void parametersChanged();


protected slots:

  void updateMinArea(double);
  void updateMaxArea(double);

  void updateText();

protected:

  void build();

  void setText(double minArea, double maxArea);

protected:

  SliderSpinBoxWidget *m_minAreaSB;
  SliderSpinBoxWidget *m_maxAreaSB;

  QLabel *m_lowArea;
  QLabel *m_highArea;

  QLabel *m_doc;

  int m_imageWidth;
  int m_imageHeight;
};

#endif /* ! AREAINTERVALSWIDGET_HPP */
