/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef BITPYRAMIDRXYCSETTINGS_HPP
#define BITPYRAMIDRXYCSETTINGS_HPP

#include "PyramidSettingsInterface.hpp"
#include "BitPyramidWidget.hpp"
#include <ise/iseBitPyramid.hpp>
#include <ise/iseThresholderData.hpp>

class BitPyramidSettings : public PyramidSettingsInterface {
  
public:
  
  BitPyramidSettings() 
    : PyramidSettingsInterface()
    , m_datas(0, 0, std::vector<int>(1, 128))
    , m_numLevel()
    , m_divider()
  {
  }

  virtual void configureWidget(PyramidWidget *widget) const
  {
    static_cast<BitPyramidWidget*>(widget)->setParameters(m_datas, m_numLevel, m_divider);
  }
  
  virtual void getWidgetConfig(const PyramidWidget *widget)
  {
    static_cast<const BitPyramidWidget*>(widget)->getParameters(m_datas, m_numLevel, m_divider);
  }

  virtual void configureAlgo(girl::shared_ptr<ise::Pyramid> algo) const
  {
    girl::shared_ptr<ise::BitPyramid> algorithm = girl::static_pointer_cast<ise::BitPyramid>(algo);
    algorithm->setParameters(m_datas, m_numLevel, m_divider);
  }

  virtual girl::shared_ptr<PyramidSettingsInterface> clone() const
  {
    girl::shared_ptr<BitPyramidSettings> p(new BitPyramidSettings);

    *p = *this;

    return p;
  }

  virtual bool isValid() const
  {
    return (m_numLevel > 0 && m_divider > 0 && m_datas.levels().size() > 0);
  }
  
private:

  ise::ThresholderData m_datas;
  int m_numLevel;
  int m_divider;
};

#endif /* ! BITPYRAMIDSETTINGS_HPP */
