/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "ButtonThresholder.hpp"

#include <QVBoxLayout>
#include <QGridLayout>
#include <QWidget>
#include <QLabel>
#include <QPushButton>
#include "QuantificationThresholdsWidgetAux.hpp"
#include <QDialog>
#include <QDialogButtonBox>

ButtonThresholder::ButtonThresholder(MainView *mainView, QWidget *parent)
  : QWidget(parent)
  , m_mainView(mainView)
  , m_colorSpaceLabel(NULL)
  , m_colorSpaceValue(NULL)
  , m_channelLabel(NULL)
  , m_channelValue(NULL)
  , m_thresholdLabel(NULL)
  , m_thresholdValue(NULL)
  , m_levels(1, 0)
  , m_colorSpace(ISE_RGB_to_YUV)
  , m_channel(0)
{
  setAttribute(Qt::WA_DeleteOnClose);

  buildGUI();
}

void
ButtonThresholder::buildGUI()
{
  QVBoxLayout *vLayout = new QVBoxLayout;

  m_colorSpaceLabel = new QLabel(tr("ColorSpace :"));
  m_colorSpaceValue = new QLabel(tr("ISE_RGB_to_RGB"));

  m_channelLabel = new QLabel(tr("Channel :"));
  m_channelValue = new QLabel(tr("R"));
  
  m_thresholdLabel = new QLabel(tr("Threshold :"));
  m_thresholdValue = new QLabel(tr("128"));

  QGridLayout *gLayout = new QGridLayout;
  gLayout->addWidget(m_colorSpaceLabel, 0, 0);
  gLayout->addWidget(m_colorSpaceValue, 0, 1, Qt::AlignRight);
  gLayout->addWidget(m_channelLabel, 1, 0);
  gLayout->addWidget(m_channelValue, 1, 1, Qt::AlignRight);
  gLayout->addWidget(m_thresholdLabel, 2, 0);
  gLayout->addWidget(m_thresholdValue, 2, 1, Qt::AlignRight);

  QPushButton *button = new QPushButton(tr("Select thresholder"));
  connect(button, SIGNAL(clicked()), this, SLOT(showThresholder()));

  vLayout->addWidget(button);
  vLayout->addLayout(gLayout);

  setLayout(vLayout);
}

void
ButtonThresholder::showThresholder()
{
  QDialog *dialog = new QDialog(this);
  QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Cancel, Qt::Horizontal, this);
  connect(buttons, SIGNAL(accepted()), dialog, SLOT(accept()));
  connect(buttons, SIGNAL(rejected()), dialog, SLOT(reject()));
  QVBoxLayout *layout = new QVBoxLayout(dialog);
  const bool allRegions = false;
  const bool oneChannel = true;
  QuantificationThresholdsWidgetAux *widget = new QuantificationThresholdsWidgetAux(m_mainView, allRegions, oneChannel, this);
  widget->activate();
  
  widget->setGUI(m_colorSpace, m_channel, m_levels);

  layout->addWidget(widget);
  layout->addWidget(buttons);
  dialog->setLayout(layout);

  int code = dialog->exec();
  assert(widget);
  widget->deactivate();
  if (code == QDialog::Accepted) {

    m_levels.clear();
    widget->getGUI(m_colorSpace, m_channel, m_levels);
    
    m_colorSpaceValue->setText(ColorSpaceRegistry::getColorSpace(m_colorSpace)->name);
    m_channelValue->setText(ColorSpaceRegistry::getColorSpace(m_colorSpace)->channelsNames[m_channel]);
    if (m_levels.size() > 0) 
      m_thresholdValue->setText(QString("%1").arg(m_levels[0]));
    else
      m_thresholdValue->setText(tr(""));
  }
}

void 
ButtonThresholder::setParameters(int colorSpace, int channel, const std::vector<int> &levels)
{
  m_colorSpace = colorSpace;
  m_channel = channel;
  m_levels = levels;
  
  m_colorSpaceValue->setText(ColorSpaceRegistry::getColorSpace(m_colorSpace)->name);
  m_channelValue->setText(ColorSpaceRegistry::getColorSpace(m_colorSpace)->channelsNames[m_channel]);
  if (m_levels.size() > 0) 
    m_thresholdValue->setText(QString("%1").arg(m_levels[0]));
  else
    m_thresholdValue->setText(tr(""));
}

void 
ButtonThresholder::getParameters(int &colorSpace, int &channel, std::vector<int> &levels) const
{
  colorSpace = m_colorSpace;
  channel = m_channel;
  levels = m_levels;
}

void
ButtonThresholder::setParameters(const ise::ThresholderData &datas)
{
  setParameters(datas.colorSpace(), datas.channel(), datas.levels());
}

void
ButtonThresholder::getParameters(ise::ThresholderData &datas) const
{
  int colorSpace;
  int channel;
  std::vector<int> levels;
  getParameters(colorSpace, channel, levels);
  datas.setParameters(colorSpace, channel, levels);
}
