#include "CSVImageConceptReader.hpp"

#include "ImageConcept.hpp"

#include <QTextStream>
#include <QFile>

#define FIELD_SEPARATOR ";"
#define CHARSET "UTF-8"

bool
CSVImageConceptReader::read(const QString &filename,
			    ImageConcept* &defaultConcept,
			    QList<ImageConcept *> &concepts)
{
  QFile file(filename);
  if (! file.open(QIODevice::ReadOnly | QIODevice::Text))
    return false;

  QTextStream inStream(&file);
  inStream.setCodec(CHARSET);

  bool headerFound = false;
  bool readRegions = false;

  QString line;
  do {
    line = inStream.readLine().trimmed();
    QStringList tokens = line.split(QObject::tr(FIELD_SEPARATOR));
    const int nbTokens = tokens.size();
    if (nbTokens != 5 && nbTokens != 6)
      continue;
    
    if (!headerFound) {

      QStringList header;
      /*
      //B: On linux (Fedora 15) with icpc 12.1.0 & qt 4.7.4, the folowing code causes an ICE
      header << QObject::tr("name")
	 << QObject::tr("R")
	 << QObject::tr("G")
	 << QObject::tr("B")
	 << QObject::tr("default");
      */
      header.append(QObject::tr("name"));
      header.append(QObject::tr("R"));
      header.append(QObject::tr("G"));
      header.append(QObject::tr("B"));
      header.append(QObject::tr("default"));
      if (nbTokens == 6) {
	header.append(QObject::tr("regions"));
	readRegions = true;
      }
      if (tokens != header) {
	file.close();
	return false;
      }
      else {
	headerFound = true;
	continue;
      }
    }

    ImageConcept *ic = new ImageConcept;
    createImageConcept(ic, tokens, readRegions);
    if (tokens[4].toInt() == DEFAULT_CONCEPT)
      defaultConcept = ic;
    else
      concepts.append(ic);

  } while (!line.isNull());


  file.close();

  if (!headerFound)
    return false;

  return true;
}

void
CSVImageConceptReader::createImageConcept(ImageConcept *ic, const QStringList &tokens, bool readRegions)
{
  ic->setName(tokens[0]);
  int r = tokens[1].toInt();
  int g = tokens[2].toInt();
  int b = tokens[3].toInt();
  if (readRegions) {
    assert(tokens.size() == 6);
    if (! tokens[5].isEmpty()) {
      QStringList regionsStrs = tokens[5].split(QObject::tr(" "));
      girl::regionDeque regions;
      for (QStringList::const_iterator it = regionsStrs.begin();
	   it != regionsStrs.end();
	   ++it)
	regions.push_back(it->toInt());
      ic->setRegions(regions);
    }
  }

  QRgb color = qRgb(r, g, b);
  ic->setColor(color);
}
