#include "CSVImageConceptWriter.hpp"

#include <QObject>
#include <QFile>

#define FIELD_SEPARATOR ';'
#define FILE_EXTENSION ".csv"
#define CHARSET "UTF-8"

bool
CSVImageConceptWriter::write(QString &filename,
			     ImageConcept* defaultConcept,
			     const QList<ImageConcept *> &concepts,
			     bool writeRegions)
{
  if (! filename.endsWith(QObject::tr(FILE_EXTENSION), Qt::CaseInsensitive))
    filename.append(QObject::tr(FILE_EXTENSION));


  QFile file(filename);
  if (! file.open(QIODevice::WriteOnly | QFile::Truncate | QIODevice::Text))
    return false;

  QTextStream outStream(&file);
  outStream.setCodec(CHARSET);
  outStream << "name" << FIELD_SEPARATOR
	    << "R" << FIELD_SEPARATOR
	    << "G" << FIELD_SEPARATOR
	    << "B" << FIELD_SEPARATOR
	    << "default";
  if (writeRegions)
    outStream << FIELD_SEPARATOR <<"regions";
  outStream << '\n';

  writeConcept(outStream, defaultConcept, DEFAULT_CONCEPT, writeRegions);
  foreach(ImageConcept *concept, concepts)
    writeConcept(outStream, concept, NORMAL_CONCEPT, writeRegions);

  file.close();

  return true;
}

void
CSVImageConceptWriter::writeConcept(QTextStream &outStream,
				    ImageConcept *concept,
				    ImageConceptState state,
				    bool writeRegions)
{
  outStream << concept->getName() << FIELD_SEPARATOR
	    << qRed(concept->getColor(QRgb())) << FIELD_SEPARATOR
	    << qGreen(concept->getColor(QRgb())) << FIELD_SEPARATOR
	    << qBlue(concept->getColor(QRgb())) << FIELD_SEPARATOR
	    << state;
  if (writeRegions) {
    const girl::regionDeque &regions = concept->getRegions();
    outStream << FIELD_SEPARATOR;
    for (girl::regionDeque::const_iterator it = regions.begin();
	 it != regions.end();
	 ++it) 
      outStream << *it << " ";
  }
  outStream << '\n';
}
