/*
  Copyright (C) 2005-2009 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

/**
 * @file CheckLabelSliderSpinBoxWidget.hpp
 * @brief 
 */

#ifndef CHECKLABELSLIDERSPINBOXWIDGET_HPP
#define CHECKLABELSLIDERSPINBOXWIDGET_HPP

#include <girl/girlCommon.hpp>
#include "LabelSliderSpinBoxWidget.hpp"

#include <QWidget>
#include <QHBoxLayout>
#include <QCheckBox>

class GIRL_API CheckLabelSliderSpinBoxWidget : public QWidget
{
  Q_OBJECT
 public:
  /**
   * @brief Constructor
   */
  CheckLabelSliderSpinBoxWidget(const QString &text, QWidget *parent = NULL)
    : QWidget(parent)
    , m_check(NULL)
    , m_lssbw(NULL)
    {
      buildGUI(text);
    }

  inline bool isChecked() const { return m_check->isChecked(); }

  inline QString text() const { return m_lssbw->text(); }

  inline void setText(const QString &text)  { m_lssbw->setText(text); }

  inline double value() const { return m_lssbw->value(); }

  inline void setValue(double val) { m_lssbw->setValue(val); }

  inline void setMinimum(int lmin) { m_lssbw->setMinimum(lmin); }

  inline void setMaximum(int lmax) { m_lssbw->setMaximum(lmax); }

  inline void setDecimals(int dec) { m_lssbw->setDecimals(dec); }

  inline void setSingleStep(double step) { m_lssbw->setSingleStep(step); }

  inline void setChecked(bool c) { m_check->setChecked(c); }

 private slots:

  void activate(int state)
  {
    if (state == Qt::Unchecked) {
      m_lssbw->setEnabled(false);
      emit check(false);
    }
    if (state == Qt::Checked) {
      m_lssbw->setEnabled(true);
      emit check(true);
    }
  }

 signals:

  void check(bool);
  
 protected:
  
  void buildGUI(const QString &text)
  {
    QHBoxLayout *m_layout = new QHBoxLayout;
    m_layout->setMargin(0);

    m_check = new QCheckBox(this);
    m_check->setChecked(true);
    connect(m_check, SIGNAL(stateChanged(int)), this, SLOT(activate(int)));

    m_lssbw = new LabelSliderSpinBoxWidget(text, this);

    m_layout->addWidget(m_check);
    m_layout->addWidget(m_lssbw);

    setLayout(m_layout);
  }

 private:

  QCheckBox *m_check;
  LabelSliderSpinBoxWidget *m_lssbw;
};

#endif /* ! CHECKLABELSLIDERSPINBOXWIDGET_HPP */
