/*
  Copyright (C) 2010-2011 Remi Vieux remi.vieux@labri.fr

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "CheckableSliderSpinBox.hpp"

#include <cassert>

#include <QCheckBox>
#include <QHBoxLayout>

#include "SliderSpinBoxWidget.hpp"

CheckableSliderSpinBox::CheckableSliderSpinBox(const QString &text, QWidget *parent)
: QWidget(parent),
  m_cb(NULL),
  m_ssbw(NULL)
{
  build(text);
}

bool
CheckableSliderSpinBox::isChecked() const
{
  assert(m_cb);
  return m_cb->isChecked();
}

void
CheckableSliderSpinBox::setChecked(const bool b)
{
  assert(m_cb);
  m_cb->setChecked(b);
}

const std::string
CheckableSliderSpinBox::getText() const
{
  assert(m_cb);
  return m_cb->text().toStdString();
}

double
CheckableSliderSpinBox::getWeight() const
{
  assert(m_ssbw);
  return m_ssbw->value();
}

void
CheckableSliderSpinBox::setWeight(const double d)
{
  assert(m_ssbw);
  m_ssbw->setValue(d);
}

void
CheckableSliderSpinBox::build(const QString &text)
{
  m_cb = new QCheckBox(text, this);
  m_ssbw = new SliderSpinBoxWidget(this);
  m_ssbw->setMinimum(0.0f);
  m_ssbw->setMaximum(1.0f);
  m_ssbw->setDecimals(2);
  //  m_ssbw->setSingleStep(0.01f);
  m_ssbw->setValue(1.0f);

  QHBoxLayout *layout = new QHBoxLayout;
  layout->addWidget(m_cb);
  layout->addWidget(m_ssbw);
 // layout->setMargin(0); //because it aims to be an embedded widget.
  setLayout(layout);

}

