/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "ClustererCMeanWidget.hpp"

#include <ise/iseClustererCMean.hpp>
#include <limits>

#include <QBoxLayout>
#include <QLabel>
#include <QSpinBox>

#include "ClustererPlugin.hpp"
#include "MainView.hpp"

ClustererCMeanWidget::ClustererCMeanWidget(MainView *mainView,
						       ClustererPlugin *clustererPlugin,
						       QWidget *parent)
  : ClustererWidgetInterface(mainView, clustererPlugin, parent)
  , m_nbClassesSB(NULL)
  , m_weightingExponent(NULL)
{
  assert(m_clustererPlugin);

  setWindowTitle(tr("ClustererCMean parameters..."));

  setAttribute(Qt::WA_DeleteOnClose);

  buildGUI();
}

void
ClustererCMeanWidget::buildGUI()
{
  QLabel *thresLabel = new QLabel(tr("Number of classes:"), this);
  m_nbClassesSB = new QSpinBox(this);
  m_nbClassesSB->setMinimum(2);
  m_nbClassesSB->setMaximum(std::numeric_limits<int>::max());

  QLabel *weightLabel = new QLabel(tr("weight exponent:"), this);
  m_weightingExponent = new QDoubleSpinBox(this);
  m_weightingExponent->setMaximum(std::numeric_limits<double>::max());

  QLabel *toleranceLabel = new QLabel(tr("tolerance:"), this);
  toleranceLabel->setToolTip(tr("Termination criteria"));
  m_tolerance = new QDoubleSpinBox(this);
  m_tolerance->setMaximum(std::numeric_limits<double>::max());
  m_tolerance->setDecimals(12);


  QGridLayout *gLayout = new QGridLayout;
  gLayout->addWidget(thresLabel, 0, 0);
  gLayout->addWidget(m_nbClassesSB, 0, 1);
  gLayout->addWidget(weightLabel, 1, 0);
  gLayout->addWidget(m_weightingExponent, 1, 1);
  gLayout->addWidget(toleranceLabel, 2, 0);
  gLayout->addWidget(m_tolerance, 2, 1);
  
  setLayout(gLayout);  
}

void
ClustererCMeanWidget::setParameters(int nbClass, double weightingExponent, double tolerance)
{
  m_nbClassesSB->setValue(nbClass);
  m_weightingExponent->setValue(weightingExponent);
  m_tolerance->setValue(tolerance);
}

void
ClustererCMeanWidget::getParameters(int &nbClass, double &weightingExponent, double &tolerance) const
{
  nbClass = m_nbClassesSB->value();
  weightingExponent = m_weightingExponent->value();
  tolerance = m_tolerance->value();
}
