/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "ClustererCvKMeansWidget.hpp"
//#include "ClustererSettingsCvKMeans.hpp"

#include <ise/iseClustererCvKMeans.hpp>
#include <limits>

#include <QBoxLayout>
#include <QLabel>
#include <QSpinBox>

#include "ClustererPlugin.hpp"
#include "MainView.hpp"

ClustererCvKMeansWidget::ClustererCvKMeansWidget(MainView *mainView,
						       ClustererPlugin *clustererPlugin,
						       QWidget *parent)
  : ClustererWidgetInterface(mainView, clustererPlugin, parent)
  , m_nbClassesSB(NULL)
{
  assert(m_clustererPlugin);

  setWindowTitle(tr("ClustererCvKMeans parameters..."));

  setAttribute(Qt::WA_DeleteOnClose);

  buildGUI();
}

void
ClustererCvKMeansWidget::buildGUI()
{
  QLabel *thresLabel = new QLabel(tr("Number of classes:"), this);
  m_nbClassesSB = new QSpinBox(this);
  m_nbClassesSB->setMinimum(2);
  m_nbClassesSB->setMaximum(std::numeric_limits<int>::max());


  QGridLayout *gLayout = new QGridLayout;
  gLayout->addWidget(thresLabel, 0, 0);
  gLayout->addWidget(m_nbClassesSB, 0, 1);
  
  setLayout(gLayout);  
}

void
ClustererCvKMeansWidget::setParameters(int nbClass)
{
  m_nbClassesSB->setValue(nbClass);
}

void
ClustererCvKMeansWidget::getParameters(int &nbClass) const
{
  nbClass = m_nbClassesSB->value();
}
