/*
  Copyright (C) 2005-2011 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#ifndef CLUSTERERPLUGIN_HPP
#define CLUSTERERPLUGIN_HPP

#include <girl/girlSharedPtr.hpp>

#include <ise/iseClusterer.hpp>

#include <qiseTools/ClustererFactory.hpp>
#include <qiseTools/ClustererSettingsInterface.hpp>

#include <qiseTools/QIsePlugin.hpp>
#include <QString>

#include <qiseTools/MainView.hpp>

class ClustererWidgetInterface;

/**
 * Interface for plugins which implements an Clusterer
 */
class GIRL_API ClustererPlugin : public QIsePlugin, public ClustererFactory
{
 public:
  
  virtual ~ClustererPlugin() {}
  
  ClustererWidgetInterface *parameterWidget(MainView *mainView, QWidget *parent)
  {
    ClustererWidgetInterface *w = createWidget(mainView, parent);
    girl::shared_ptr<ClustererSettingsInterface> s = settings();
    if (s)
      s->configureWidget(w);
    return w;
  }

  girl::shared_ptr<ise::Clusterer> getNewClusterer(ise::SegmentationContext *segContext)
  {
    girl::shared_ptr<ise::Clusterer> computer = create(segContext);
    girl::shared_ptr<ClustererSettingsInterface> s = settings();
    if (s)
      s->configureAlgo(computer);
    return computer;
  }
  
  void setSettings(girl::shared_ptr<ClustererSettingsInterface> settings)
  {
    if (settings)
      m_settings = settings->clone();
    else
      m_settings = girl::shared_ptr<ClustererSettingsInterface>();
  }  

  girl::shared_ptr<ClustererSettingsInterface> getSettings()
  {
    if (settings()) 
      return m_settings->clone();
    return girl::shared_ptr<ClustererSettingsInterface>();
  }  

  virtual bool hasSettings() const
  {
    return true;
  }


  inline void setSegContext(ise::SegmentationContext *segContext)
  {
    m_segContext = segContext;
  }
  
  inline ise::SegmentationContext *getSegContext()
  {
    return m_segContext;
  }

  virtual girl::shared_ptr<ClustererSettingsInterface> createSettings() = 0;

  virtual girl::shared_ptr<ise::Clusterer> create(ise::SegmentationContext *segContext) = 0;

 protected:
  
  ClustererPlugin(const QString &name,
		  const QIseVersion &pluginVersion,
		  const QIseVersion &qiseVersion,
		  QString path = QString::fromUtf8(""))
    : QIsePlugin(name, (QString::fromUtf8("Clusterer/") + path),
		 pluginVersion, qiseVersion)
    , m_segContext(NULL)
    , m_settings()
    {}
  
  virtual ClustererWidgetInterface* createWidget(MainView *mainView, QWidget *parent) = 0;

  girl::shared_ptr<ClustererSettingsInterface> settings() 
  { 
    if (!m_settings)
      m_settings = createSettings();
    return m_settings; 
  }

 protected:

  ise::SegmentationContext *m_segContext;
  std::string m_uri;

  girl::shared_ptr<ClustererSettingsInterface> m_settings;
  

};

#define ClustererPlugin_iid "org.girl.qise.ClustererPlugin/1.0"
Q_DECLARE_INTERFACE(ClustererPlugin, ClustererPlugin_iid)

#endif /* ! CLUSTERERPLUGIN_HPP */
