/*
  Copyright (C) 2005-2011 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef CLUSTERERSETTINGSCMEAN_HPP
#define CLUSTERERSETTINGSCMEAN_HPP

#include "ClustererSettingsInterface.hpp"
#include "ClustererCMeanWidget.hpp"
#include <ise/iseClustererCMean.hpp>

class ClustererSettingsCMean : public ClustererSettingsInterface {
  
public:
  
  ClustererSettingsCMean() 
    : ClustererSettingsInterface()
    , m_nbClasses(3)
    , m_weightingExponent(1.25)
    , m_tolerance(ise::ClustererCMean::ERROR_DISS)
  {
  }

  virtual void configureWidget(ClustererWidgetInterface *widget) const
  {
    static_cast<ClustererCMeanWidget*>(widget)->setParameters(m_nbClasses, m_weightingExponent, m_tolerance);
  }
  
  virtual void getWidgetConfig(const ClustererWidgetInterface *widget)
  {
    static_cast<const ClustererCMeanWidget*>(widget)->getParameters(m_nbClasses, m_weightingExponent, m_tolerance);
  }

  virtual void configureAlgo(girl::shared_ptr<ise::Clusterer> algo) const
  {
    algo->setNumberClasses(m_nbClasses);
    girl::static_pointer_cast<ise::ClustererCMean>(algo)->setParameters(m_weightingExponent, m_tolerance);
  }
  
  virtual girl::shared_ptr<ClustererSettingsInterface> clone() const
  {
    girl::shared_ptr<ClustererSettingsCMean> p(new ClustererSettingsCMean);
    
    *p = *this;

    return p;
  }

  virtual bool isValid() const
  {
    return (m_nbClasses > 0); //B:???
  }

private:
  
  int m_nbClasses;
  double m_weightingExponent;
  double m_tolerance;
};

#endif /* ! CLUSTERERCMEANWIDGETSETTINGS_HPP */
