/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef CLUSTERERWIDGETINTERFACE_HPP
#define CLUSTERERWIDGETINTERFACE_HPP

#include <ise/iseClusterer.hpp>

#include "ClustererSettingsInterface.hpp"
#include "ClustererPlugin.hpp"

#include "MainView.hpp"

#include <QWidget>
#include <QString>

class GIRL_API ClustererWidgetInterface : public QWidget
{
  //Q_OBJECT

 public:
  ClustererWidgetInterface(MainView *mainView, 
			   ClustererPlugin *clustererPlugin, 
			   QWidget *parent = NULL)
    : QWidget(parent)
    , m_mainView(mainView)
    , m_clustererPlugin(clustererPlugin)
  {
  }

  virtual ~ClustererWidgetInterface() {}

  void setGUI(const girl::shared_ptr<ClustererSettingsInterface> settings) 
  {
    if (settings)
      settings->configureWidget(this);
  }

  void getGUI(girl::shared_ptr<ClustererSettingsInterface> settings) const
  {
    settings->getWidgetConfig(this);
  }

  ClustererPlugin* plugin() const { return m_clustererPlugin; }

  bool saveSettingsInPlugin()
  {
    if (m_clustererPlugin) {
      if (m_clustererPlugin->hasSettings()) {
	girl::shared_ptr<ClustererSettingsInterface> s = m_clustererPlugin->getSettings();
	getGUI(s);
	assert(s);
	if (s->isValid()) {
	  m_clustererPlugin->setSettings(s);
	  return true;
	}
      }
      else {
	return true;
      }
    }
    return false;
  }

  virtual bool isValid() const
  {
    if (m_clustererPlugin) {
      if (m_clustererPlugin->hasSettings()) {
	girl::shared_ptr<ClustererSettingsInterface> s = m_clustererPlugin->getSettings();
	getGUI(s);
	assert(s);
	if (s->isValid()) {
	  return true;
	}
      }
      else {
	//no settings
	return true;
      }
    }
    return false;
  }

  inline std::string title() const { return m_title; }

  inline void setTitle(const std::string &title) { m_title = title; }

protected:

  virtual void buildGUI() = 0;

protected:

  MainView *m_mainView;
  ClustererPlugin *m_clustererPlugin;
  QString m_name;
  std::string m_title;
};

#endif /* ! CLUSTERERWIDGETINTERFACE_HPP */
