/*
  Copyright (C) 2005-2010 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#include "ColorConvertWidget.hpp"

#include <cassert>

#include <QBoxLayout>
#include <QLabel>

#include "ColorSpaceChoiceWidget.hpp"


ColorConvertWidget::ColorConvertWidget(QWidget *parent,
				       int conversion)
  : QWidget(parent),
    m_cscw(NULL),
    m_channelsValuesL(NULL),
    m_r(0), m_g(0), m_b(0)
{
  build();

  setColorSpace(conversion);    //will not call slot if cs==ISE_RGB_to_RGB which is the default
  changeColorSpace(conversion); //thus we call it
}

void
ColorConvertWidget::build()
{
  m_cscw = new ColorSpaceChoiceWidget(this);
  
  m_channelsValuesL = new QLabel(this);

  QHBoxLayout *hL = new QHBoxLayout;
  hL->addStretch();
  hL->addWidget(m_cscw);
  hL->addWidget(m_channelsValuesL);

  hL->setMargin(0); //because embedded widget

  setLayout(hL);

  connect(m_cscw, SIGNAL(colorSpaceChanged(int)),
	  this, SLOT(changeColorSpace(int)));

  //We don't want the size/geometry of widget to always change 
  // (it's very disturbing, visually)
  //We set max dumb values to fix the size once and for all.
  setValues(9999, 9999, 9999);
  m_channelsValuesL->setMaximumSize(m_channelsValuesL->sizeHint());
  m_channelsValuesL->setMinimumSize(m_channelsValuesL->sizeHint());

  setMaximumSize(sizeHint());
  setMinimumSize(sizeHint());
}

void
ColorConvertWidget::setRGB(int r, int g, int b)
{
  assert(m_cscw);
  const int conversion = m_cscw->getColorSpace();
  
  m_r = r;
  m_g = g;
  m_b = b;

  changeColorSpace(conversion);
}

void
ColorConvertWidget::setColorSpace(int cs)
{
  assert(m_cscw);
  m_cscw->setColorSpace(cs);
}

void
ColorConvertWidget::changeColorSpace(int cs)
{
  int x, y, z;
  ise::conversion::changeFromRGB(cs, m_r, m_g, m_b, x, y, z);
  
  setValues(x, y, z);
}

void
ColorConvertWidget::setValues(int x, int y, int z)
{
  m_channelsValuesL->setText(tr("%1; %2; %3").arg(x).arg(y).arg(z));
}

void
ColorConvertWidget::setEnabledValues(bool enabled)
{
  m_channelsValuesL->setEnabled(enabled);
}
