/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#include "ColorPool.hpp"

#include <cstdlib> //rand
#include <cassert> 



//CODE DUPLICATION 
/**
 * gives a random value in [min; max]
 *
 * @warning call srandom once before this function.
 */
static
int 
P_bounded_rand(int min, int max)
{
  return static_cast<int>(min + (static_cast<float>(rand()) / RAND_MAX * (max - min + 1)));
}

QRgb
ColorPool::getRandomColor()
{
  return qRgba(P_bounded_rand(0, 255), 
	       P_bounded_rand(0, 255), 
	       P_bounded_rand(0, 255), 
	       255);
}


ColorPool::ColorPool(size_t minNbColors)
{
  initColors();

  resize(minNbColors);

  
}

size_t
ColorPool::size() const
{
  return m_colors.size();
}

void
ColorPool::resize(size_t size)
{
  const size_t s = m_colors.size();
  if (s < size) {
    for (size_t i = s; i<size; ++i) {
      QRgb nc;
      do {
	nc = getRandomColor();
      }
      while (m_colors.contains(nc));
      m_colors.append(nc);
    }
  }
  else {
    while ((size_t)m_colors.size() > s)
      m_colors.takeLast();
  }
}

QRgb
ColorPool::getColor(size_t i)
{
  assert(i < size());
  return m_colors[i];
}

QRgb
ColorPool::takeFreeColor()
{
  if (! m_colors.isEmpty()) {
    return m_colors.takeFirst();
  }
  //else
  return getRandomColor();
  
  //TODO : check that this color is not already in use !!!!

}

void
ColorPool::restoreFreeColor(QRgb color)
{
  //TODO : check that this color is not already in use !!!!
  m_colors.push_front(color);
}

void
ColorPool::initColors()
{
  m_colors.append(qRgba(128, 128, 255, 255));
  m_colors.append(qRgba(128, 255, 128, 255));
  m_colors.append(qRgba(255, 128, 128, 255));
  m_colors.append(qRgba(128, 255, 255, 255));
  m_colors.append(qRgba(255, 255, 128, 255));
  m_colors.append(qRgba(255, 128, 255, 255));
  m_colors.append(qRgba(64, 128, 255, 255));
  m_colors.append(qRgba(0, 255, 255, 255));
  m_colors.append(qRgba(255, 0, 255, 255));
  m_colors.append(qRgba(255, 255, 0, 255));
  m_colors.append(qRgba(0, 255, 128, 255));
  m_colors.append(qRgba(255, 0, 128, 255));
  m_colors.append(qRgba(255, 128, 0, 255));
  m_colors.append(qRgba(0, 128, 255, 255));
  m_colors.append(qRgba(128, 0, 255, 255));
  m_colors.append(qRgba(128, 255, 0, 255));
  m_colors.append(qRgba(255, 64, 128, 255));
  m_colors.append(qRgba(64, 128, 64, 255));

}
