/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "ColorSpaceChoiceWidget.hpp"

#include <cassert>

#include <QLabel>
#include <QComboBox>
#include <QHBoxLayout>


ColorSpaceChoiceWidget::ColorSpaceChoiceWidget(QWidget *parent)
  : QWidget(parent),
    m_colorSpaceCombo(NULL)
{

  build();
}

void
ColorSpaceChoiceWidget::build()
{
  m_colorSpaceCombo = new QComboBox(this);


  for (ColorSpaceRegistry::ColorSpaceMap::const_iterator it = ColorSpaceRegistry::begin();
       it != ColorSpaceRegistry::end();
       ++it) {
    m_colorSpaceCombo->addItem(it->second.name);
  }

  //signals combo box : activated/currentIndexChanged

  connect(m_colorSpaceCombo, SIGNAL(currentIndexChanged(int)), 
	  this, SLOT(changeColorSpace(int)));
  

  QHBoxLayout *hbox = new QHBoxLayout;

  hbox->addWidget(m_colorSpaceCombo);
  hbox->addStretch();
  hbox->setMargin(0); //because it aims to be an embedded widget.
  this->setLayout(hbox);
}


const ColorSpaceDetails &
ColorSpaceChoiceWidget::getColorSpaceFromIndex(int index) const
{
  assert(index < (int)ColorSpaceRegistry::getNumColorSpaces());

  ColorSpaceRegistry::ColorSpaceMap::const_iterator it = ColorSpaceRegistry::begin();
  for (int i=0; i<index; ++i)
    ++it;

  return it->second;
}

void
ColorSpaceChoiceWidget::changeColorSpace(int index)
{
  emit colorSpaceChanged(getColorSpaceFromIndex(index).iseColorSpace);
}


void
ColorSpaceChoiceWidget::setColorSpace(int colorSpace)
{
  int index = 0;
  for (ColorSpaceRegistry::ColorSpaceMap::const_iterator it = ColorSpaceRegistry::begin();
       it != ColorSpaceRegistry::end();
       ++it) {
    if (it->second.iseColorSpace == colorSpace)
      break;
    ++index;
  }
  
  m_colorSpaceCombo->setCurrentIndex(index);

}

int
ColorSpaceChoiceWidget::getColorSpace() const
{
  return getColorSpaceFromIndex(m_colorSpaceCombo->currentIndex()).iseColorSpace;
}

size_t
ColorSpaceChoiceWidget::getNbChannelsForColorSpace() const
{
  return getColorSpaceFromIndex(m_colorSpaceCombo->currentIndex()).getNumChannels();
}

  
QString
ColorSpaceChoiceWidget::getChannelName(size_t i) const
{
  return getColorSpaceFromIndex(m_colorSpaceCombo->currentIndex()).channelsNames[i];
}



