/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef COLORSPACECHOICEWIDGET_HPP
#define COLORSPACECHOICEWIDGET_HPP

#include <QWidget>
#include <QVector>

#include <ise/iseConversion.hpp>

#include "ColorSpaceRegistry.hpp"

class QComboBox;
class QLabel;


class GIRL_API ColorSpaceChoiceWidget : public QWidget
{
  Q_OBJECT

public:
  
  /**
   * @brief Constructor.
   */
  explicit ColorSpaceChoiceWidget(QWidget *parent);


  
  /**
   * Set color space according to iseConversion !
   */
  void setColorSpace(int cs);

  /**
   * Get color space according to iseConversion !
   */
  int getColorSpace() const;

  /**
   * Get number of channels for selected color space.
   *
   * TODO: should ba a function of iseConversion ???
   */
  size_t getNbChannelsForColorSpace() const;

  
  /**
   * Get name of ith channel for selected color space.
   *
   * @warning i must be strictly inferior to getNbChannelsForColorSpace().
   */
  QString getChannelName(size_t i) const;
  
signals:

  /**
   * @warning color space is already changed when this signal is emitted.
   */
  void colorSpaceChanged(int conversion);

protected slots:

  void changeColorSpace(int index);



protected:


  void build();


  const ColorSpaceDetails &getColorSpaceFromIndex(int index) const;

protected:

  QComboBox *m_colorSpaceCombo;

};


#endif /* ! COLORSPACECHOICEWIDGET_HPP */
