/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef COLORSPACEREGISTRY_HPP
#define COLORSPACEREGISTRY_HPP

#include <map>

#include <girl/girlCommon.hpp>//GIRL_API

#include <QString>
#include <QVector>

struct GIRL_API ColorSpaceDetails
{
  QString name;
  QVector<QString> channelsNames;
  int iseColorSpace;

  ColorSpaceDetails()
    : iseColorSpace(0)
  {

  }

  /**
   * @brief Constructor for 3-channels color spaces....
   */
  ColorSpaceDetails(const QString &csName,
		    const QString &channelsNames0,
		    const QString &channelsNames1,
		    const QString &channelsNames2,
		    int iseCSConversion)
    : name(csName)
    , iseColorSpace(iseCSConversion)
  {
    const size_t nbColorSpaces = 3;
    channelsNames.reserve(nbColorSpaces);
    channelsNames.push_back(channelsNames0);
    channelsNames.push_back(channelsNames1);
    channelsNames.push_back(channelsNames2);
  }

  size_t getNumChannels() const
  {
    return channelsNames.size();
  }


};




class GIRL_API ColorSpaceRegistry 
{
public:

  typedef std::map<int, ColorSpaceDetails> ColorSpaceMap;

  //B:impl: do not put static methods inline
  // Visual C++ will not be able to find static member variables when linking with produced dll...


  static bool addColorSpace(const ColorSpaceDetails &csd);

  static const ColorSpaceDetails *getColorSpace(int iseColorSpace);

  static size_t getNumColorSpaces();
  
  static ColorSpaceMap::const_iterator begin();

  static ColorSpaceMap::const_iterator end();


protected:

  static ColorSpaceMap ms_colorSpaces;

};





#endif /* ! COLORSPACEREGISTRY_HPP */
