/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef HISTOGRAMINTERVALSCOLORSPACESWIDGET_HPP
#define HISTOGRAMINTERVALSCOLORSPACESWIDGET_HPP


#include <girl/girlCommon.hpp>
#include <girl/girlFlags.hpp>

#include <QWidget>

class QAbstractButton;
class QHBoxLayout;

class ColorSpaceChoiceWidget;

class GIRL_API ColorSpacesChannelsChoiceWidget : public QWidget
{
  Q_OBJECT

public:
    
  /**
   * @brief Constructor.
   *
   * @a exclusive must be true to have only one channel selected at a time, 
   * or false to have zero or more channels
   */
  ColorSpacesChannelsChoiceWidget(QWidget *parent, bool exclusive);


  typedef enum ChannelFlag {
    CHANNEL_NONE = 0,
    CHANNEL_0 = 1,
    CHANNEL_1 = 2,
    CHANNEL_2 = 4,
    CHANNEL_3 = 8,
    CHANNEL_4 = 16,
    CHANNEL_ALL = 255
  } ChannelFlag;
    
  GIRL_DECLARE_FLAGS(ChannelFlags, ChannelFlag)


  /**
   * Get color space according to iseConversion !
   */
  int getColorSpace() const;

  /**
   * Set color space according to iseConversion !
   */
  void setColorSpace();

  /**
   * Get number of channels for selected color space.
   *
   * TODO: should ba a function of iseConversion ???
   */
  size_t getNbChannelsForColorSpace() const;

  
  /**
   * Get name of ith channel for selected color space.
   *
   * @warning i must be strictly inferior to getNbChannelsForColorSpace().
   */
  QString getChannelName(size_t i) const;


  /**
   * Get OR'ed combination of ChannelFlag.
   */
  ChannelFlags getChannels() const;			   

  /**
   * Set OR'ed combination of ChannelFlag.
   */
  void setChannels(ChannelFlags);

  /**
   * Set channel
   */
  void setChannel(int channel);

  /**
   * Set colorSpace
   */
  void setColorSpace(int colorSpace);

  /**
   * Get number of checked channels.
   */
  size_t getNbCheckedChannelsForColorSpace() const;

signals:

  /**
   * @warning channels are already changed when this signal is emitted.
   */
  void channelsChanged();

  /**
   * @warning color space is already changed when this signal is emitted.
   */
  void colorSpaceChanged();

public slots:  

  void changeColorSpace(int);

protected:

  void build();

  void changeColorSpaceChannelsText(int i);

  QAbstractButton *doButton();


protected:

  QHBoxLayout *m_layoutCBs;
  QVector<QAbstractButton *> m_channelsCBs;
  ColorSpaceChoiceWidget *m_cscw;
  bool m_exclusive;
};

GIRL_DECLARE_OPERATORS_FOR_FLAGS(ColorSpacesChannelsChoiceWidget::ChannelFlags)


#endif /* ! HISTOGRAMINTERVALSCOLORSPACESWIDGET_HPP */
