/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux boris.mansencal@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "ComparisonFunctionWidget.hpp"

#include <cassert>

#include <QComboBox>
#include <QHBoxLayout>


ComparisonFunctionWidget::ComparisonFunctionWidget(QWidget *parent)
  : QWidget(parent),
    m_comboBox(NULL)
{

  buildGUI();
}


void
ComparisonFunctionWidget::buildGUI()
{
  
  m_comboBox = new QComboBox(this);
  
  //Warning: add them in same order that they are declared 
  // in isEComparisonFunction.hpp
  m_comboBox->addItem(tr("<"));
  m_comboBox->addItem(tr("<="));
  m_comboBox->addItem(tr(">"));
  m_comboBox->addItem(tr(">="));
  m_comboBox->addItem(tr("=="));
  m_comboBox->addItem(tr("!="));

  m_comboBox->setCurrentIndex(0);

  QHBoxLayout *hbox = new QHBoxLayout;

  hbox->addWidget(m_comboBox);
  hbox->addStretch();
  hbox->setMargin(0); //because it aims to be an embedded widget.
  this->setLayout(hbox);  
  
  connect(m_comboBox, SIGNAL(currentIndexChanged(int)), this, SIGNAL(comparisonOperatorChanged()));

}

ise::ComparisonOperator
ComparisonFunctionWidget::getComparisonOperator() const
{
  assert(m_comboBox->currentIndex()>=0);
  
  return static_cast<ise::ComparisonOperator>(m_comboBox->currentIndex());
}

void
ComparisonFunctionWidget::setComparisonOperator(ise::ComparisonOperator c)
{
  assert(static_cast<int>(c) < m_comboBox->count());
  
  return m_comboBox->setCurrentIndex(static_cast<int>(c));
}
