/*
  Copyright (C) 2005-2009 Jerome Charton, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef EDITIONBOX_HPP
#define EDITIONBOX_HPP


#include <girl/girlCommon.hpp>//GIRL_API

#include <QPoint>

#include "PaintingShape.hpp"
#include "GLView.hpp"

static const int MAX_CORNER_SIZE = 50;

class QPainter;

class GIRL_API EditionBox
{

public:
  void draw(GLView *view, QPainter &painter, const PaintingShape *shape);

  static int getDimCorner(const GLView *view, const PaintingShape *shape)
  {
    return std::min((int)(MAX_CORNER_SIZE / view->getZoomFactor()),
		    (int)((std::min(shape->width(), shape->height()) * 0.2 + 1) / view->getZoomFactor()))+1;
  }

  bool isUnderRotationHandler(const QPoint &point,
			      const GLView *view,
			      const PaintingShape *shape) const;

  bool isUnderPointHandler(const QPoint &point,
			   const GLView *view,
			   const PaintingShape *shape) const;

  int getPointHandledUnder(const QPoint &point,
			   const GLView *view,
			   const PaintingShape *shape) const;

  void setSelectedMode();
  void setUnSelectedMode();
  void setOverUpSideMode();
  void setOverDownSideMode();
  void setOverLeftSideMode();
  void setOverRightSideMode();
  void setOverUpLeftCornerMode();
  void setOverDownLeftCornerMode();
  void setOverUpRightCornerMode();
  void setOverDownRightCornerMode();
  int getDimCorner();

protected:

  void eraseVisibleMode();
  void drawRotationHandler(QPainter &painter);
  void drawPointsInteractors(QPainter &painter, const PaintingShape *shape);
  void drawUpSide(QPainter &painter);
  void drawDownSide(QPainter &painter);
  void drawRightSide(QPainter &painter);
  void drawLeftSide(QPainter &painter);
  void drawUpLeftCorner(QPainter &painter);
  void drawDownLeftCorner(QPainter &painter);
  void drawUpRightCorner(QPainter &painter);
  void drawDownRightCorner(QPainter &painter);

protected:

  bool m_isSetUp, m_isSetDown, m_isSetLeft, m_isSetRight,
    m_isSetUpLeft, m_isSetUpRight, m_isSetDownLeft, m_isSetDownRight,
    m_isSetRotationHandler;
  
  QPoint m_areaP1, m_areaP2;
  int m_width, m_height, m_dimCorner;
  QPoint m_absolutZero;
  qreal m_zoomFactor;
  
};

#endif /* ! EDITIONBOX_HPP */

