/*
  Copyright (C) 2005-2010 Hugo Boujut, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#ifndef FALSECOLORIMAGEBUILDERTHREAD_HPP
#define FALSECOLORIMAGEBUILDERTHREAD_HPP

#include <girl/girlToger.hpp>

#include <QImage>
#include <QList>
#include <QMutex>
#include <QThread>
#include <QWaitCondition>

#include "FalseColorIndex.hpp"

class GIRL_API FalseColorImageBuilderThread : public QThread
{
  Q_OBJECT

public:
  explicit FalseColorImageBuilderThread(QObject *parent = NULL);

  ~FalseColorImageBuilderThread();

  /*
    @a imgIn is the image from which colors are looked-up in @a falseColorIndexes
    @a imgOut is the image onto which we draw with false colors.
    
    Both images may be the same.

   */

  void process(const FalseColorIndexList &falseColorIndexes, 
	       const QImage &imgIn, const QImage &imgOut,
	       int scanLine, int nbLines, float opacity);

  void process(const FalseColorIndexList &falseColorIndexes, 
	       const QImage &imgIn, const QImage &imgOut,
	       const girl::toger &tog, float opacity);


	       
signals:
  
  void processed(const QImage &img);

protected:

  void run();


protected:

  class RegionColorSetter : public girl::toger::pointIterator
  {
  public:
    RegionColorSetter();

    RegionColorSetter(const FalseColorIndex &falseColorIdx, const QImage &img);

    RegionColorSetter(const RegionColorSetter &o);
    RegionColorSetter &operator=(const RegionColorSetter &o);

    void setImage(const QImage &imgIn, const QImage &imgOut);

    void setFalseColorIndex(const FalseColorIndex &falseColorIdx);

    void processPoint(const girl::point &pt);

    void processLine(int y);

  public:
    const FalseColorIndex *m_falseColorIdx;
    QImage m_imgIn;
    QImage m_imgOut;
    uchar *m_dataIn;
    uchar *m_dataOut;
    int m_bytesPerLine;
    int m_width;
  };
   

  QMutex m_mutex;
  QWaitCondition m_condition;

  FalseColorIndexList m_falseColorIndexes;
  RegionColorSetter m_rcs;
  const girl::toger *m_toger;
  int m_scanLine;
  int m_nbLines;
  bool m_doRegions;

  bool m_restart;
  bool m_abort;
};

#endif /* ! FALSECOLORIMAGEBUILDERTHREAD_HPP */
