/* 
 * Copyright (C) 2010-2011 Julian Roigt  boris.mansencal@labri.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation (Inc., 59
 * Temple Place - Suite 330, Boston, MA 02111-1307, USA); either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
#include "FeatureChoiceDialog.hpp"

#include <ise/iseFeatureComputer.hpp>

#include "PluginRegistry.hpp"

#include "ImageFeatureComputerTabWidget.hpp"
#include "RegionFeatureComputerTabWidget.hpp"

#include "FeatureTabWidget.hpp"

#include <QBoxLayout>
#include <QDialogButtonBox>
#include <QString>
#include <QLabel>
#include <QGroupBox>
#include <QTabWidget>
#include <QVBoxLayout>
#include <QApplication>

FeatureChoiceDialog::FeatureChoiceDialog(MainView *mainView, QWidget *parent)
  : QDialog(parent)
  , m_imageFeatureComputerWidget(NULL)
  , m_regionFeatureComputerWidget(NULL)
  , m_iftws()
  , m_mainView(mainView)
  , m_featureChoiceDialogSettings()
  , m_featureComputerGUIInfo()
{
  setWindowTitle(tr("Image Feature choices"));

  setAttribute(Qt::WA_DeleteOnClose);

  assert(mainView);
  assert(mainView->model());
  assert(mainView->model()->getSegContextPtr());

  m_featureComputerGUIInfo = mainView->model()->getFeatureComputerGUIInfo();
  connect(m_featureComputerGUIInfo.get(), SIGNAL(featureChanged()), this, SLOT(changedFeature()));
  buildGUI();
}

void
FeatureChoiceDialog::buildGUI()
{
  QTabWidget *tabFeature = new QTabWidget(this);
  
  //B: use NULL as parents of widgets inserted as tabs 
  //  as it is advised in QTabWidget class documentation
  // http://doc.qt.digia.com/qt/qtabwidget.html#details
  // "Create a QWidget for each of the pages in the tab dialog, but do not specify parent widgets for them."

  m_imageFeatureComputerWidget = new ImageFeatureComputerTabWidget(m_mainView, m_featureComputerGUIInfo, "ImageFeature Computer", NULL);
  //connect(m_imageFeatureComputerWidget, SIGNAL(imageFeatureChanged()), this, SLOT(imageFeatureChanged()));

  m_regionFeatureComputerWidget = new RegionFeatureComputerTabWidget(m_mainView, m_featureComputerGUIInfo, "RegionFeature Computer", NULL);
  //connect(m_regionFeatureComputerWidget, SIGNAL(imageFeatureChanged()), this, SLOT(imageFeatureChanged()));

  {
    QIsePluginList<FeatureTabPlugin*> tabs = PluginRegistry::getFeatureTabPlugin();
    QIsePluginList<FeatureTabPlugin*>::iterator it = tabs.begin();
    const QIsePluginList<FeatureTabPlugin*>::const_iterator itEnd = tabs.end();
    for ( ; it != itEnd ; ++it) {
      m_iftws.push_back((*it)->parameterWidget(m_mainView, m_featureComputerGUIInfo, this));
      //connect(m_iftws[m_iftws.size() - 1], SIGNAL(imageFeatureChanged()), this, SLOT(imageFeatureChanged()));
    }
  }

  size_t sizeIfows = m_iftws.size();

  tabFeature->addTab(m_imageFeatureComputerWidget, tr("ImageFeature Computer"));
  tabFeature->addTab(m_regionFeatureComputerWidget, tr("RegionFeature Computer"));
  for ( size_t  i = 0 ; i < sizeIfows; ++i) {
    tabFeature->addTab(m_iftws[i], m_iftws[i]->getName());
  }

  QGridLayout *layout = new QGridLayout;
  layout->addWidget(tabFeature);

  QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok
						     | QDialogButtonBox::Cancel, Qt::Horizontal, this);
  QWidget::connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
  QWidget::connect(buttonBox, SIGNAL(rejected()), this, SLOT(reject()));
  layout->addWidget(buttonBox);

  setLayout(layout);
}

void
FeatureChoiceDialog::accept()
{
  assert(m_mainView);
  assert(m_mainView->model());
  assert(m_mainView->model()->getSegContextPtr());

  m_featureComputerGUIInfo->applyChanges();

  emit featureChanged();
  
  QDialog::accept();
}

void
FeatureChoiceDialog::reject()
{
  QDialog::reject();
}

void
FeatureChoiceDialog::changedFeature()
{
  m_imageFeatureComputerWidget->initWidget();
  m_regionFeatureComputerWidget->initWidget();
  size_t size = m_iftws.size();
  for (size_t i = 0 ; i < size ; ++i)
    m_iftws[i]->initWidget();
}
