/* 
 * Copyright (C) 2011 Remi Vieux  remi.vieux@labri.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation (Inc., 59
 * Temple Place - Suite 330, Boston, MA 02111-1307, USA); either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "FeatureComputerGUIInfo.hpp"
#include "SegModel.hpp"

#include "PluginRegistry.hpp"
#include "ImageFeatureColorSpaceWidget.hpp"
#include "FeatureInterfaceSettings.hpp"
#include "FeatureData.hpp"
#include <ise/iseRegionSetInterface.hpp>

FeatureComputerGUIInfo::FeatureComputerGUIInfo(SegModel *segModel) 
  : QObject(NULL)
  , m_segModel(segModel)
  , m_titleToAlgo()
  , m_handler()
{
  assert(m_segModel);
  // copy
  m_handler = m_segModel->getSegContextPtr()->getFeatureComputerHandler();


}

FeatureComputerGUIInfo::~FeatureComputerGUIInfo()
{
  // TODO Auto-generated destructor stub
}

// FeatureComputerGUIInfo::FeatureComputerGUIInfo(const FeatureComputerGUIInfo &other)
// {
//   m_segModel = other.m_segModel;
//   m_titleToAlgo = other.m_titleToAlgo;
//   m_handler = other.m_handler;
// }

FeatureComputerGUIInfo&
FeatureComputerGUIInfo::operator=(const FeatureComputerGUIInfo &other)
{
  if ( this != &other) {
    m_segModel = other.m_segModel;
    m_titleToAlgo = other.m_titleToAlgo;
    m_handler = other.m_handler;
  }
  return *this;
}


bool
FeatureComputerGUIInfo::add(const ise::FeatureComputerPtr &algo, const FeatureDataPtr &algoInfos)
{
  const std::string title = algoInfos->title();
  if (m_titleToAlgo.find(title) != m_titleToAlgo.end())
    return false;

  FeatureDataAlgo dataAlgo(algoInfos, algo);
  if (! m_handler.add(algo) )
    return false;
  std::pair<TitleToAlgoContainerType::iterator, bool> insertOk =
      m_titleToAlgo.insert(std::make_pair(title, dataAlgo));
  assert(insertOk.second);

  if (insertOk.second)
    emit featureChanged();

  return insertOk.second;
}

bool
FeatureComputerGUIInfo::rename(const std::string &oldTitle, const std::string &newTitle)
{
  TitleToAlgoContainerType::const_iterator end = m_titleToAlgo.end();
  TitleToAlgoContainerType::iterator itOld;
  if ((itOld = m_titleToAlgo.find(oldTitle)) == end || m_titleToAlgo.find(newTitle) != end)
    return false;

  itOld->second.infos()->setTitle(newTitle);
  std::pair<TitleToAlgoContainerType::iterator, bool> insertOk =
        m_titleToAlgo.insert(std::make_pair(newTitle, itOld->second));
  assert(insertOk.second);
  m_titleToAlgo.erase(itOld);
  
  if (insertOk.second)
    emit featureChanged();

  return insertOk.second;
}

bool
FeatureComputerGUIInfo::remove(const std::string &title, bool removeDependencies)
{
  TitleToAlgoContainerType::iterator it = m_titleToAlgo.find(title);
  if (it == m_titleToAlgo.end())
    return false;

  ise::FeatureComputer::FeatureComputerListType removedComputers;
  bool removeDone = m_handler.remove(it->second.algo()->uri(), removeDependencies, removedComputers);
  if (!removeDone)
    return false;

  std::vector<TitleToAlgoContainerType::const_iterator> toRemove;
  TitleToAlgoContainerType::const_iterator tIt = m_titleToAlgo.begin();
  const TitleToAlgoContainerType::const_iterator tItEnd = m_titleToAlgo.end();
  for (; tIt != tItEnd; ++tIt) {
    ise::FeatureComputerPtr algo = tIt->second.algo();
    ise::FeatureComputer::FeatureComputerListType::const_iterator it =
      removedComputers.find(algo->uri());
    if (it != removedComputers.end()) {
      toRemove.push_back(tIt);
    }
  }
  
  for (size_t i = 0; i < toRemove.size(); ++i) {
    m_titleToAlgo.erase(toRemove[i]);
  }
  emit featureChanged();
  return true;
}

FeatureDataPtr
FeatureComputerGUIInfo::getInfos(const std::string &title) const
{
  TitleToAlgoContainerType::const_iterator it = m_titleToAlgo.find(title);
  assert(it != m_titleToAlgo.end());
  if (it == m_titleToAlgo.end())
    return FeatureDataPtr();
  return it->second.infos();
}

FeatureDataPtr
FeatureComputerGUIInfo::getInfos(const ise::FeatureComputerPtr &algo) const
{
  assert(algo);
  TitleToAlgoContainerType::const_iterator tIt = m_titleToAlgo.begin();
  const TitleToAlgoContainerType::const_iterator tItEnd = m_titleToAlgo.end();
  for (; tIt != tItEnd; ++tIt) {
    ise::FeatureComputerPtr currentComputer = tIt->second.algo();
    if (algo == currentComputer) {
      return tIt->second.infos();
    }
  }
  return FeatureDataPtr();
}

ise::FeatureComputerPtr
FeatureComputerGUIInfo::getAlgo(const std::string &title) const
{
  const TitleToAlgoContainerType::const_iterator it = m_titleToAlgo.find(title);
  //assert(it != m_titleToAlgo.end());
  if (it == m_titleToAlgo.end())
    return ise::FeatureComputerPtr();
  return it->second.algo();
}

void
FeatureComputerGUIInfo::applyChanges() const
{
  //m_segModel->setTitleToAlgoMap(m_titleToAlgo);
  m_segModel->setFeatureComputerGUIInfo(*this);
  // R: TODO check when the computation should actually be launched, check that we don't
  // recomputed already existing features;
  m_segModel->getSegContextPtr()->setFeatureComputerHandler(m_handler);
}

girl::shared_ptr<FeatureComputerGUIInfo>
FeatureComputerGUIInfo::clone() const
{    
  girl::shared_ptr<FeatureComputerGUIInfo> other( new FeatureComputerGUIInfo(m_segModel));
  
  *other = *this;
  
  return other;
}

void
FeatureComputerGUIInfo::loadDefaultFeature()
{
  {
    // default colorSpace RGB
    FeaturePlugin *p = PluginRegistry::getImageFeaturePlugin("ColorSpace");
    if (p) {
      ImageFeatureColorSpaceWidget *w = new ImageFeatureColorSpaceWidget(NULL, girl::shared_ptr<FeatureComputerGUIInfo>(this), "", static_cast<ImageFeaturePlugin*>(p), NULL);
      w->setParameters(ISE_RGB_to_RGB);
      w->saveSettingsInPlugin();
      girl::shared_ptr<FeatureInterfaceSettings> s = p->settings();
      FeatureDataPtr dataColorSpace(new FeatureData(s, p, "ColorSpaceRGB"));
      
      add(dataColorSpace->createAlgo(m_segModel->getSegContextPtr()), dataColorSpace);
    }
  }

  {
    // default colorSpace GRAY
    FeaturePlugin *p = PluginRegistry::getImageFeaturePlugin("ColorSpace");
    if (p) {
      ImageFeatureColorSpaceWidget *w = new ImageFeatureColorSpaceWidget(NULL, girl::shared_ptr<FeatureComputerGUIInfo>(this), "", static_cast<ImageFeaturePlugin*>(p), NULL);
      w->setParameters(ISE_RGB_to_GRAY);
      w->saveSettingsInPlugin();
      girl::shared_ptr<FeatureInterfaceSettings> s = p->settings();
      FeatureDataPtr dataColorSpace(new FeatureData(s, p, "ColorSpaceGRAY"));
      
      add(dataColorSpace->createAlgo(m_segModel->getSegContextPtr()), dataColorSpace);
    }
  }

  // cannot do RegionSetInterfaceMomentWidget , it's in plugin :(
  // {
  //   // default moment GRAY
  //   FeaturePlugin *p = PluginRegistry::getRegionSetPlugin("Moments");
  //   if (p) {
  //     RegionSetInterfaceWidgetMoments *w = new RegionSetInterfaceWidgetMoments(NULL, girl::shared_ptr<FeatureComputerGUIInfo>(this), static_cast<RegionSetInterfacePlugin*>(p), NULL);
  //     w->setParameters(getAlgo("ColorSpaceGRAY"));
  //     w->saveSettingsInPlugin();
  //     girl::shared_ptr<FeatureInterfaceSettings> s = p->settings();
  //     FeatureDataPtr data(new FeatureData(s, p, "MomentsGRAY"));
      
  //     add(data->createAlgo(m_segModel->getSegContextPtr()), data);
  //   }
  // }

  applyChanges();
}
