/*
 * FeatureComputerGUIInfo.hpp
 *
 *  Created on: Mar 4, 2011
 *      Author: vieux
 */

#ifndef FEATURECOMPUTERGUIINFO_HPP_
#define FEATURECOMPUTERGUIINFO_HPP_

#include <string>
#include <vector>

#include <girl/girlCommon.hpp>
#include <girl/girlSharedPtr.hpp>

#include <ise/iseFeatureComputer.hpp>
#include <ise/iseFeatureComputerHandler.hpp>
#include <QObject>

#include "FeatureData.hpp"
#include "FeatureDataAlgo.hpp"
class SegModel;

/**
 * Class to handle FeatureComputers through the GUI, i.e. that contains
 * additional informations to the FeatureComputer such that the user-defined
 * title, cachestate, etc
 */
class GIRL_API FeatureComputerGUIInfo : public QObject
{
  Q_OBJECT
  //typedef SegModel::TitleToAlgoContainerType ContainerType;
  typedef girl::hashMap<std::string, FeatureDataAlgo> TitleToAlgoContainerType;

public:

  explicit FeatureComputerGUIInfo(SegModel *segModel);

  virtual
  ~FeatureComputerGUIInfo();

  FeatureComputerGUIInfo& operator=(const FeatureComputerGUIInfo &other);
    

  bool
  add(const ise::FeatureComputerPtr &algo, const FeatureDataPtr &algoInfos);

  bool
  rename(const std::string &oldTitle, const std::string &newTitle);

  bool
  remove(const std::string &title, bool removeDependencies);

  FeatureDataPtr
  getInfos(const std::string &title) const;

  FeatureDataPtr
  getInfos(const ise::FeatureComputerPtr &algo) const;

  ise::FeatureComputerPtr
  getAlgo(const std::string &title) const;

  template <typename T> void
  getFeatureComputersInfo(std::vector<FeatureDataPtr>& infos) const;

  girl::shared_ptr<FeatureComputerGUIInfo>
  clone() const;

  void
  loadDefaultFeature();

public slots:

  void
  applyChanges() const;

 signals:

  void featureChanged();

protected:

  SegModel *m_segModel;

  TitleToAlgoContainerType m_titleToAlgo;

  ise::FeatureComputerHandler m_handler;


};

typedef girl::shared_ptr<FeatureComputerGUIInfo> FeatureComputerGUIInfoPtr;


template <typename T> void
FeatureComputerGUIInfo::getFeatureComputersInfo(std::vector<FeatureDataPtr> &infos) const
{
  infos.clear();
  ise::FeatureComputer::FeatureComputerListType computerList;
  m_handler.getComputerList<T>(computerList);

  // R: suboptimal method but avoid having a map FeatureCompter->title
  ise::FeatureComputer::FeatureComputerListType::const_iterator it = computerList.begin();
  const ise::FeatureComputer::FeatureComputerListType::const_iterator itEnd = computerList.end();
  for (; it != itEnd; ++it) {
    ise::FeatureComputerPtr theComputer = m_handler.getFeatureComputer(*it);
    assert(theComputer);
    TitleToAlgoContainerType::const_iterator tIt = m_titleToAlgo.begin();
    const TitleToAlgoContainerType::const_iterator tItEnd = m_titleToAlgo.end();
    for (; tIt != tItEnd; ++tIt) {
      ise::FeatureComputerPtr currentComputer = tIt->second.algo();
      if (theComputer == currentComputer) {
              infos.push_back(tIt->second.infos());
              break;
          }
      }
  }
  assert(infos.size() == computerList.size());
}

#endif /* FEATURECOMPUTERGUIINFO_HPP_ */
