/* 
 * Copyright (C) 2010-2011 Remi Vieux  boris.mansencal@labri.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation (Inc., 59
 * Temple Place - Suite 330, Boston, MA 02111-1307, USA); either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
#include "FeatureData.hpp"

#include <qiseTools/FeaturePlugin.hpp>

#include <qiseTools/RegionSetInterfacePlugin.hpp>

FeatureData::FeatureData(girl::shared_ptr<FeatureInterfaceSettings> setting, 
			 FeaturePlugin *plugin, 
			 const std::string &title, 
			 const ise::RegionSetInterface::CacheType &cache)
  : m_setting(setting)
  , m_plugin(plugin)
  , m_title(title)
  , m_cache(cache)
{
}

FeatureData::FeatureData(const FeatureData &other)
{
  m_setting = other.m_setting->clone();
  m_plugin = other.m_plugin;
  m_title = other.m_title;
  m_cache = other.m_cache;
}

FeatureData& 
FeatureData::operator=(const FeatureData &other)
{
  if (this != &other) {
    m_setting = other.m_setting->clone();
    m_plugin = other.m_plugin; 
    m_title = other.m_title;
    m_cache = other.m_cache;
  }
  return *this;
}

void 
FeatureData::setSettings(girl::shared_ptr<FeatureInterfaceSettings> setting)
{
  m_setting = setting;
}

void 
FeatureData::setPlugin(FeaturePlugin *plugin)
{
  m_plugin = plugin;
}

void 
FeatureData::initPlugin() 
{
  m_plugin->setSettings(m_setting);
}

void 
FeatureData::setTitle(const std::string &title)
{
  m_title = title;
}

std::string 
FeatureData::title() const
{
  return m_title; 
}

void 
FeatureData::setCacheState(const ise::RegionSetInterface::CacheType &cache)
{
  m_cache = cache;
}

ise::RegionSetInterface::CacheType 
FeatureData::cacheState() const
{
  return m_cache;
}

girl::shared_ptr<FeatureInterfaceSettings> 
FeatureData::setting() const 
{ 
  return m_setting; 
}

FeaturePlugin* 
FeatureData::plugin() const 
{ 
  return m_plugin; 
}

girl::shared_ptr<ise::FeatureComputer> 
FeatureData::createAlgo(ise::SegmentationContext *segContext) 
{
  m_plugin->setSettings(m_setting);
  girl::shared_ptr<ise::FeatureComputer> algo = m_plugin->getNew(segContext);
  algo->setCacheState(m_cache);
  return algo;
}

girl::shared_ptr<FeatureData> 
FeatureData::clone() const 
{
  girl::shared_ptr<FeatureInterfaceSettings> s = girl::shared_ptr<FeatureInterfaceSettings>();
  if (m_setting) 
    s = girl::static_pointer_cast<FeatureInterfaceSettings>(m_setting->clone() );
  
  girl::shared_ptr<FeatureData> p(new FeatureData(girl::static_pointer_cast<FeatureInterfaceSettings>(s), m_plugin));
  
  *p = *this;
  
  return p;
}
