/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef IMAGEFEATUREDATATEMPLATE_HPP
#define IMAGEFEATUREDATATEMPLATE_HPP

#include <girl/girlCommon.hpp>
#include <girl/girlSharedPtr.hpp>

class FeaturePlugin;
#include <qiseTools/FeatureInterfaceSettings.hpp>

class RegionSetInterfacePlugin;

#include <cstring>

#include <ise/iseRegionSetInterface.hpp>



class GIRL_API FeatureData {
public : 

 FeatureData(girl::shared_ptr<FeatureInterfaceSettings> setting, 
	     FeaturePlugin *plugin, 
	     const std::string &title ="", 
	     const ise::RegionSetInterface::CacheType &cache = ise::RegionSetInterface::NONE);

  FeatureData(const FeatureData &other);

  FeatureData& operator=(const FeatureData &other);

  virtual ~FeatureData() {}

  void setSettings(girl::shared_ptr<FeatureInterfaceSettings> setting);

  void setPlugin(FeaturePlugin *plugin);

  void initPlugin();

  void setTitle(const std::string &title);

  std::string title() const;

  void setCacheState(const ise::RegionSetInterface::CacheType &cache);

  ise::RegionSetInterface::CacheType cacheState() const;

  girl::shared_ptr<FeatureInterfaceSettings> setting() const;

  FeaturePlugin* plugin() const;

  girl::shared_ptr<ise::FeatureComputer> createAlgo(ise::SegmentationContext *segContext);

  girl::shared_ptr<FeatureData> clone() const;
    
  
protected:

  girl::shared_ptr<FeatureInterfaceSettings> m_setting;
  
  FeaturePlugin *m_plugin;

  std::string m_title;

  ise::RegionSetInterface::CacheType m_cache;

};

typedef girl::shared_ptr<FeatureData> FeatureDataPtr;

#endif /* IMAGEFEATUREDATATEMPLATE_HPP */
