/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef FEATUREDIALOG_HPP
#define FEATUREDIALOG_HPP

#include <ise/iseFeatureComputer.hpp>

#include "FeatureInterfaceSettings.hpp"

#include "FeatureInterfaceWidget.hpp"

#include "MainView.hpp"

#include "TitleCacheDialog.hpp"

#include <QString>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QGroupBox>

class GIRL_API FeatureDialog : public TitleCacheDialog
{

public:
  
  FeatureDialog(FeatureInterfaceWidget *widget, FeatureDataPtr data, QWidget *parent = NULL)
    : TitleCacheDialog(data, parent)
    , m_widget(widget)
    , m_settings()
  {
    setAttribute(Qt::WA_DeleteOnClose);
    setWindowTitle("");
    build();

    setSettings(data->setting());
  }

  virtual ~FeatureDialog() {}

  void setSettings(const girl::shared_ptr<FeatureInterfaceSettings> &settings)
  {
    m_settings = settings;
  }

  FeatureInterfaceWidget* widget() const { return m_widget; }

protected:

  void build()
  {
    QVBoxLayout *vLayout = new QVBoxLayout;

    QGroupBox *parameters = new QGroupBox(QString("Parameters"));
    QHBoxLayout *hlayout = new QHBoxLayout;
    
    hlayout->addWidget(m_widget);

    parameters->setLayout(hlayout);

    vLayout->addWidget(parameters);

    TitleCacheDialog::setLayout(vLayout);
  }
  
  virtual void finish() 
  {
    if (m_widget->isEnabled()) {
	m_widget->saveSettingsInPlugin();
	
	assert(m_settings != NULL);
	m_widget->getGUI(m_settings);
    }
  }
    
 protected:
    
  FeatureInterfaceWidget *m_widget;
  girl::shared_ptr<FeatureInterfaceSettings> m_settings;
  
};

#endif /* ! FEATUREDIALOG_HPP */
