/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 */

#include "FeatureListMultiPluginWidget.hpp"

#include <QGridLayout>
#include <QGroupBox>
#include <QCoreApplication>

#include "QIsePluginList.hpp"
#include "PluginRegistry.hpp"
#include "LabelShowButtonWidget.hpp"
#include "FeatureChoiceDialog.hpp"

FeatureListMultiPluginWidget::FeatureListMultiPluginWidget(MainView *mainView,
							   FeatureComputerGUIInfoPtr featureComputerGUIInfo,
							   QString name,
							   QWidget *parent)
  : FeatureListPluginWidget(mainView, featureComputerGUIInfo, name, parent)
  , m_list(NULL)
  , m_plugins()
{
  setAttribute(Qt::WA_DeleteOnClose);

  build();
}


void
FeatureListMultiPluginWidget::build()
{
  QHBoxLayout *layout = new QHBoxLayout;

  QGroupBox *box = new QGroupBox(m_name, this);

  QHBoxLayout *hLayout = new QHBoxLayout;

  m_list = new QListWidget(this);
  // m_list->setMaximumWidth(200);
  //m_list->setMinimumWidth(200);
  // m_list->setMaximumHeight(100);

  connect(m_list, SIGNAL(itemDoubleClicked(QListWidgetItem*)), this, SLOT(addNewFeature()));

  // m_listToCompute->setMaximumWidth(200);
  //m_listToCompute->setMinimumWidth(200);
  // m_listToCompute->setMaximumHeight(100);

  QPushButton *add = new QPushButton(tr("->"), this);
  connect(add, SIGNAL(clicked()), this, SLOT(addNewFeature()));
  QPushButton *del = new QPushButton(tr("<-"), this);
  connect(del, SIGNAL(clicked()), this, SLOT(delFeature()));

  add->setMaximumWidth(25);
  del->setMaximumWidth(25);

  box->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
  m_list->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
  m_listToCompute->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
  setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);


  QVBoxLayout *buttonLayout = new QVBoxLayout;
  buttonLayout->addWidget(add);
  buttonLayout->addWidget(del);


  hLayout->addWidget(m_list, Qt::AlignLeft);
  hLayout->addLayout(buttonLayout, Qt::AlignLeft);
  hLayout->addWidget(m_listToCompute, Qt::AlignLeft);
  //hLayout->addStretch();

  box->setLayout(hLayout);

  layout->addWidget(box);

  setLayout(layout);

}

FeatureDialog*
FeatureListMultiPluginWidget::newCurrentDialog(FeatureDataPtr &data)
{
  assert(m_mainView);

  QListWidgetItem* item = m_list->currentItem();
  FeatureDialog *dialog = NULL;
  if (item != NULL) {
    size_t index = static_cast<size_t>(m_list->currentRow());

    FeatureInterfaceWidget *widget = m_plugins[index]->parameterWidget(m_mainView, m_featureComputerGUIInfo, this);
    girl::shared_ptr<FeatureInterfaceSettings> settings = m_plugins[index]->createSettings();

    data = FeatureDataPtr(new FeatureData(settings, m_plugins[index]));

    dialog = new FeatureDialog(widget, data, this);
    dialog->setSettings(settings);
    dialog->setTitle(m_plugins[index]->name());

    
  }
  return dialog;
}

