/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef FEATURELISTMULTIPLUGINWIDGET_HPP
#define FEATURELISTMULTIPLUGINWIDGET_HPP

#include <girl/girlHashFunc.hpp>
#include <girl/girlHashMap.hpp>

#include <QHBoxLayout>
#include <QLabel>
#include <QListWidget>
#include <QPushButton>
#include <QString>

#include <ise/iseSegmentationContext.hpp>
#include "FeatureDialog.hpp"
#include "FeaturePlugin.hpp"
#include "FeatureData.hpp"
#include "MainView.hpp"

#include "FeatureListPluginWidget.hpp"
#include "FeatureInterfaceSettings.hpp"
#include "FeatureComputerGUIInfo.hpp"

#include <map>

class FeatureListMultiPluginSettings;

class GIRL_API FeatureListMultiPluginWidget : public FeatureListPluginWidget
{
 public:

  virtual ~FeatureListMultiPluginWidget() {}

 protected:

  virtual void build();

  virtual void loadPlugins() = 0;

  void addPlugin(FeaturePlugin *plugin)
  {
    QListWidgetItem *item = new QListWidgetItem(plugin->name());
    m_list->addItem(item);
#ifndef NDEBUG
    std::pair<PluginContainer::iterator, bool> insertOk = 
#endif
      m_plugins.insert(std::make_pair(m_plugins.size(), plugin));
    assert(insertOk.second);
  }

 private:

  virtual FeatureDialog * newCurrentDialog(FeatureDataPtr&);

 protected:
  
  FeatureListMultiPluginWidget(MainView *mainView,
			       FeatureComputerGUIInfoPtr featureComputerGUIInfo, 
			       QString name,
			       QWidget *parent = NULL);

 private:

  QListWidget *m_list;

  typedef girl::hashMap<size_t, FeaturePlugin*> PluginContainer;
  PluginContainer m_plugins;
};

#endif /* ! FEATURELISTMULTIPLUGINWIDGET_HPP */
