/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "FeatureListOnePluginWidget.hpp"
#include "ImageFeatureCompositionWidgetInterface.hpp"
#include "ImageFeatureCompositionPlugin.hpp"
#include "FeatureDialog.hpp"

#include <QPushButton>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QListWidget>
#include <QMessageBox>
#include <QGroupBox>

#include "LabelShowButtonWidget.hpp"

FeatureListOnePluginWidget::FeatureListOnePluginWidget(MainView *mainView, 
						       FeatureComputerGUIInfoPtr featureComputerGUIInfo,
						       ImageFeatureCompositionPlugin *plugin, 
						       QWidget *parent)
  : FeatureListPluginWidget(mainView, featureComputerGUIInfo, "", parent)
  , m_plugin(plugin)
{
  assert(plugin != NULL);

  setAttribute(Qt::WA_DeleteOnClose);

  build();
}

void
FeatureListOnePluginWidget::build()
{
  QHBoxLayout *boxLayout = new QHBoxLayout;

  QVBoxLayout *layout = new QVBoxLayout;
  
  QGroupBox *box = new QGroupBox(m_plugin->name(), this);

  QHBoxLayout *hLayout = new QHBoxLayout;
  QPushButton *add = new QPushButton(tr("Add image feature .."), this);
  connect(add, SIGNAL(clicked()), this, SLOT(addNewFeature())); 

  QPushButton *del = new QPushButton(tr("Del"), this);
  connect(del, SIGNAL(clicked()), this, SLOT(delFeature())); 
  
  hLayout->addWidget(add);
  hLayout->addWidget(del);

  layout->addLayout(hLayout);
  layout->addWidget(m_listToCompute);

  box->setLayout(layout);

  boxLayout->addWidget(box);

  setLayout(boxLayout);

}

FeatureDialog*
FeatureListOnePluginWidget::newCurrentDialog(FeatureDataPtr &data)
{
  girl::shared_ptr<FeatureInterfaceSettings> settings = m_plugin->createSettings();
  m_plugin->setSettings(settings);

  data = FeatureDataPtr(new FeatureData(settings, m_plugin));

  FeatureInterfaceWidget *widget = m_plugin->parameterWidget(m_mainView, m_featureComputerGUIInfo, this);

  FeatureDialog *dialog = new FeatureDialog(widget, data, this);
  dialog->setSettings(settings);
  dialog->setTitle(m_plugin->name());
  
  return dialog;
}

// girl::shared_ptr<FeatureListOnePluginSettings> 
// FeatureListOnePluginWidget::createSettings() const
// {
//   return girl::shared_ptr<FeatureListOnePluginSettings>(new FeatureListOnePluginSettings());
// }
