/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef FEATURELISTPLUGINWIDGET_HPP
#define FEATURELISTPLUGINWIDGET_HPP

#include <girl/girlCommon.hpp>

#include <qiseTools/MainView.hpp>
#include <qiseTools/FeatureData.hpp>
#include <qiseTools/FeatureDialog.hpp>
#include <qiseTools/LabelShowButtonWidget.hpp>

#include <QWidget>
#include <QListWidget>
#include <QMessageBox>

#include <ise/iseFeatureComputerHandler.hpp>
#include "FeatureComputerGUIInfo.hpp"


class GIRL_API FeatureListPluginWidget : public QWidget
{
  Q_OBJECT

 public:

  virtual ~FeatureListPluginWidget() {}

  inline QString getName() const { return m_name; }

  /**
   * @brief Insert an item with title name of @a data in @a m_listToCompute
   * @param exist true if exist in segContext, else false
   */
  void insertFeatureDataToCompute(const FeatureDataPtr &data, bool exist = false);

  void initWidget();

 protected slots:

  /**
   * @brief Open dialog with existed config of current plugin of @a m_listToCompute
   */
  void show(QString);

  /**
   * @brief Open dialog of current plugin and add to compute if title not used
   */
  void addNewFeature();

  void delFeature();

 protected:

  FeatureListPluginWidget(MainView *mainView, 
			  FeatureComputerGUIInfoPtr featureComputerGUIInfo, 
			  QString name, 
			  QWidget *parent);

  virtual void build() = 0;

 private:

  virtual FeatureDialog* newCurrentDialog(FeatureDataPtr&) = 0;

  virtual void getFeatureInfos(std::vector<FeatureDataPtr> &infos) const = 0;
  
 signals:

  void imageFeatureChanged();

 protected:

  MainView *m_mainView;
  QString m_name;
  
  QListWidget *m_listToCompute;

  FeatureComputerGUIInfoPtr m_featureComputerGUIInfo;

};

#endif /* ! FEATURELISTPLUGINWIDGET_HPP */
