/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "FeaturePlugin.hpp"

#include "FeatureInterfaceSettings.hpp"
#include "FeatureInterfaceWidget.hpp"
#include "FeatureComputerGUIInfo.hpp"

FeatureInterfaceWidget *
FeaturePlugin::parameterWidget(MainView *mainView, FeatureComputerGUIInfoPtr featureComputerGUIInfo, QWidget *parent) {
  FeatureInterfaceWidget *w = createWidget(mainView, featureComputerGUIInfo, parent);
    girl::shared_ptr<FeatureInterfaceSettings> s = settings();
    if (s)
      s->configureWidget(w);
    return w;
}

girl::shared_ptr<ise::FeatureComputer> 
FeaturePlugin::getNew(ise::SegmentationContext *segContext) {
  girl::shared_ptr<ise::FeatureComputer> computer = create(segContext);
  girl::shared_ptr<FeatureInterfaceSettings> s = settings();
  if (s)
    s->configureAlgo(computer);
  return computer;
}

void 
FeaturePlugin::setSettings(girl::shared_ptr<FeatureInterfaceSettings> settings) {
  if (settings)
    m_settings = girl::static_pointer_cast<FeatureInterfaceSettings>(settings->clone());
  else
    m_settings = girl::shared_ptr<FeatureInterfaceSettings>();
}

girl::shared_ptr<FeatureInterfaceSettings> 
FeaturePlugin::getSettings() {
  if (settings())
    return girl::static_pointer_cast<FeatureInterfaceSettings>(m_settings->clone());
  return girl::shared_ptr<FeatureInterfaceSettings>();
}

girl::shared_ptr<FeatureInterfaceSettings> 
FeaturePlugin::settings()  {
  if (!m_settings)
    m_settings = createSettings();
  return m_settings;
}
