/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef FEATUREPLUGIN_HPP
#define FEATUREPLUGIN_HPP

#include <girl/girlCommon.hpp>

#include "QIsePlugin.hpp"

#include "FeatureInterfaceSettings.hpp"

#include <ise/iseFeatureComputer.hpp>
class MainView;

class FeatureComputerGUIInfo;

class FeatureInterfaceWidget;

class GIRL_API FeaturePlugin : public QIsePlugin
{
 public:

  virtual ~FeaturePlugin() {}

  FeatureInterfaceWidget *parameterWidget(MainView *mainView, girl::shared_ptr<FeatureComputerGUIInfo> featureComputerGUIInfo, QWidget *parent);

  girl::shared_ptr<ise::FeatureComputer> getNew(ise::SegmentationContext *segContext);

  void setSettings(girl::shared_ptr<FeatureInterfaceSettings> settings);

  girl::shared_ptr<FeatureInterfaceSettings> getSettings();

  virtual bool hasSettings() const
  { 
    return true; 
  }

  void setSegContext(ise::SegmentationContext *segContext)
  {
    m_segContext = segContext;
  }

  girl::shared_ptr<FeatureInterfaceSettings> settings();

  virtual girl::shared_ptr<FeatureInterfaceSettings> createSettings() = 0;

  virtual girl::shared_ptr<ise::FeatureComputer> create(ise::SegmentationContext *segContext) = 0;

 protected:

  FeaturePlugin(const QString &name,
		const QIseVersion &pluginVersion,
		const QIseVersion &qiseVersion,
		QString path = QString::fromUtf8(""))
    : QIsePlugin(name, path,
		 pluginVersion, qiseVersion)
    , m_segContext(NULL)
    , m_settings()
    {}

  virtual FeatureInterfaceWidget* createWidget(MainView *mainView, girl::shared_ptr<FeatureComputerGUIInfo> featureComputerGUIInfo, QWidget *parent) = 0;

 protected:

  ise::SegmentationContext *m_segContext;

  girl::shared_ptr<FeatureInterfaceSettings> m_settings;

};

// Q_DECLARE_INTERFACE(FeaturePlugin,
//  		    "org.girl.qise.FeaturePlugin/1.0")

#endif /* ! FEATUREPLUGIN_HPP */
