/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef FEATURETABWIDGET_HPP 
#define FEATURETABWIDGET_HPP

#include <girl/girlCommon.hpp>

#include <QLayout>
#include <QWidget>

#include <ise/iseSegmentationContext.hpp>

#include "FeatureListPluginWidget.hpp"
#include "FeatureTabPlugin.hpp"
#include "FeatureData.hpp"
#include "MainView.hpp"
#include "FeatureComputerGUIInfo.hpp"

class FeaturesTabSettingsInterface;

class GIRL_API FeatureTabWidget : public QWidget
{
  Q_OBJECT

 public:
  
  FeatureTabWidget(MainView *mainView, FeatureComputerGUIInfoPtr featureComputerGUIInfo, QString name, QWidget *parent = NULL)
    : QWidget(parent)
    , m_mainView(mainView)
    , m_name(name)
    , m_featureComputerGUIInfo(featureComputerGUIInfo)
  {
  }

  virtual ~FeatureTabWidget() {}

  inline QString getName() const { return m_name; }

  void connect(QWidget *widget) {
    QWidget::connect(widget, SIGNAL(imageFeatureChanged()), this, SIGNAL(imageFeatureChanged()));
  }

  virtual void initWidget() = 0;


 signals:

  void imageFeatureChanged();

 protected:

  virtual void build() = 0;

  void setLayout(QLayout *layout)
  {
    QVBoxLayout *l = new QVBoxLayout;

    l->addLayout(layout);
    l->addStretch();

    QWidget::setLayout(l);
  }

 protected:

  MainView *m_mainView;
  QString m_name;

  FeatureComputerGUIInfoPtr m_featureComputerGUIInfo;
};

#endif /* ! FEATURETABWIDGET_HPP */
