/*
  Copyright (C) 2012 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "GLBlendImageLayer.hpp"


static const float DEFAULT_OPACITY = 0.5;


GLBlendImageLayer::GLBlendImageLayer(GLView *v)
  : GLImageLayer(v), m_opacity(DEFAULT_OPACITY)
{

}
  
GLBlendImageLayer::GLBlendImageLayer(GLView *v, const QImage &image)
  : GLImageLayer(v, image), m_opacity(DEFAULT_OPACITY)
{

}

GLBlendImageLayer::GLBlendImageLayer(GLView *v, const QImage &image, float opacity)
  : GLImageLayer(v, image), m_opacity(opacity)
{

}

void
GLBlendImageLayer::draw()
{
  //We want the following behavior:
  //[1] If pixel of image of this layer is transparent, destination pixel (of framebuffer) is not changed.
  //[2] If pixel of this image is not transparent , destination pixel (of framebuffer) is a combination of the two values with m_opacity

  //To implement [1], we use the alpha test : if the alpha value is not equal to 1, fragement is not drawn
  glEnable(GL_ALPHA_TEST);
  glAlphaFunc(GL_EQUAL, 1.0f);

  //To implement [2], we use blending
  glBlendColor(m_opacity, m_opacity, m_opacity, m_opacity);
  glBlendFunc(GL_CONSTANT_COLOR, GL_ONE_MINUS_CONSTANT_COLOR);
  
  //glBlendColor(1.0, 1.0, 1.0, m_opacity);
  //glBlendFunc(GL_CONSTANT_COLOR, GL_ONE);
  //glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  glEnable(GL_BLEND);

  glColor4f(1.0, 1.0, 1.0, m_opacity);
  
  GLImageLayer::draw();
  //m_image.GLDraw();
  
  glDisable(GL_BLEND);
  glBlendColor(0, 0, 0, 0);

  glDisable(GL_ALPHA_TEST);

}
