/*
  Copyright (C) 2010 Cédric Aguerre aguerre@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "GLEllipseLayer.hpp"

#include <cassert>
#include <cmath>

#include "GLUtils.hpp"


GLEllipseLayer::GLEllipseLayer(GLView *v, QRgb color, float opacity)
  : GLSelectorLayer(v, color, opacity),
    m_p1(0, 0),
    m_p2(0, 0),
    m_ellipse()
{
  updateEllipse();
}

GLEllipseLayer::GLEllipseLayer(GLView *v, const QPointF &p1, const QPointF &p2, QRgb color, float opacity)
  : GLSelectorLayer(v, color, opacity),
    m_p1(p1),
    m_p2(p2),
    m_ellipse()
{
  updateEllipse();
}

void
GLEllipseLayer::updateEllipse()
{
  m_ellipse.setPosition((m_p1+m_p2)/2.);
  qreal rH = fabs(m_p1.x() - m_p2.x()) / 2.;
  qreal rV = fabs(m_p1.y() - m_p2.y()) / 2.;
  m_ellipse.setRadii(rH, rV);
}

void
GLEllipseLayer::setEllipse(const QPointF &p1, const QPointF &p2)
{
  m_p1 = p1;
  m_p2 = p2;
  updateEllipse();
}

void
GLEllipseLayer::clear()
{
  m_p1 = m_p2 = QPointF(0, 0);
  updateEllipse();
}

void
GLEllipseLayer::draw()
{
  m_ellipse.setColor(getColor());
  m_ellipse.setFilled(true);
  m_ellipse.draw();
}

void
GLEllipseLayer::boundingRect(QRectF &rect)
{
  rect = QRectF(m_p1.x(), m_p1.y(), m_p2.x()-m_p1.x()+1, m_p2.y()-m_p1.y()+1);
}

bool
GLEllipseLayer::intersects(const QPointF &pt)
{
  qreal px = pt.x();
  qreal py = pt.y();
  qreal rH, rV;
  m_ellipse.getRadii(rH, rV);

  qreal d = (px*px)/(rH*rH) + (py*py)/(rV*rV);
  return (d <= 1.0);
}
