/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "GLImageLayer.hpp"

#include <cassert>
#include <iostream> //DEBUG

#include <QImage>

#include "GLUtils.hpp"
#include "GLView.hpp"


GLImageLayer::GLImageLayer(GLView *v)
  : GLLayer(v), 
    m_image()
{

}
 
GLImageLayer::GLImageLayer(GLView *v, const QImage &image)
  :
  GLLayer(v),
  m_image()
{
  setImage(image);
}

void
GLImageLayer::setImage(const QImage &image)
{
  if (! image.isNull())
    setImage(image, image.width(), image.height());
}

void
GLImageLayer::setImage(const QImage &image, int width, int height)
{
  assert(m_view);

  //TODO: assert initializeGL() has been called !!!!

  m_view->makeCurrent();
  //m_image.GLInit();   //TODO: DO NOT CALL GLInit each time !!!!!
  m_image.setImage(image, width, height);
}

void
GLImageLayer::draw()
{
  glPushMatrix();

  GL_CHECK_ERROR_ALWAYS();

  glPushAttrib(GL_ALL_ATTRIB_BITS);

  m_image.GLDraw();
  
  glPopAttrib();

  GL_CHECK_ERROR_ALWAYS();

  glPopMatrix();
}


/*
void 
GLImageLayer::boundingRect(QRectF &rect)
{
  QPointF ptUL(m_image.x0(), m_image.y0());
  QPointF ptULt;
  QPointF ptUR(m_image.x1(), m_image.y0());
  QPointF ptURt;
  QPointF ptDR(m_image.x1(), m_image.y1());
  QPointF ptDRt;
  QPointF ptDL(m_image.x0(), m_image.y1());
  QPointF ptDLt;

  transformFromLayer(ptUL, ptULt);
  transformFromLayer(ptUR, ptURt);
  transformFromLayer(ptDR, ptDRt);
  transformFromLayer(ptDL, ptDLt);
  
  const qreal xMin = std::min(ptULt.x(), std::min(ptURt.x(), std::min(ptDRt.x(), ptDLt.x())));
  const qreal xMax = std::max(ptULt.x(), std::max(ptURt.x(), std::max(ptDRt.x(), ptDLt.x())));

  const qreal yMin = std::min(ptULt.y(), std::min(ptURt.y(), std::min(ptDRt.y(), ptDLt.y())));
  const qreal yMax = std::max(ptULt.y(), std::max(ptURt.y(), std::max(ptDRt.y(), ptDLt.y())));
  
  rect = QRectF(QPointF(xMin, yMin), QPointF(xMax, yMax));

  //TODO:OPTIM !!!!!
}

bool 
GLImageLayer::intersects(const QPointF &pt)
{
  QPointF ptL;
  transformToLayer(pt, ptL);

  QRectF r(QPoint(m_image.x0(), m_image.y0()), QPoint(m_image.x1(), m_image.y1()));
  return r.contains(ptL);
}
*/

void
GLImageLayer::boundingRect(QRectF &rect)
{
  rect = QRectF(m_image.x0(), m_image.y0(), m_image.x1()-m_image.x0(), m_image.y1()-m_image.y0());
}

void
GLImageLayer::copyPart(const QImage &img, const girl::tools::AABBox &bb)
{
  m_image.copyPart(img, bb);
}
