/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef GLIMAGELAYER_HPP
#define GLIMAGELAYER_HPP

#include "GLLayer.hpp"
#include "GLImage.hpp"

namespace girl {
  namespace tools {
    class AABBox;
  }
}


class GIRL_API GLImageLayer : public GLLayer
{
public:
  /**
   * @brief Constructor.
   */
  explicit GLImageLayer(GLView *v);
  
  /**
   * @brief Constructor.
   */
  GLImageLayer(GLView *v, const QImage &image);

  virtual void boundingRect(QRectF &rect);   

  virtual void draw();

  void setImage(const QImage &image);

  void setImage(const QImage &image, int width, int height);

  void changeImage(const QImage &image);

  /**
   * @brief Copy part of image @a img in bounding box @a bb on layer.
   *
   * @warning @a img must have the same size that the image previously set with setImage().
   * @warning We must have bb.xMin()>=0 && bb.yMin()>=0 && bb.xMax()<img.width() && bb.yMax()<img.height().
   */
  void copyPart(const QImage &img, const girl::tools::AABBox &bb);

  /**
   * Set sampling filter used to display image.
   */
  void setFilter(GLImage::Filter f) { m_image.setFilter(f); }

  /**
   * Get sampling filter used to display image.
   */
  GLImage::Filter filter() const { return m_image.filter(); }


protected:

  
protected:

  GLImage m_image;

};


#endif /* ! GLIMAGELAYER_HPP */
