/*
  Copyright (C) 2005-2009 Jerome Charton, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "GLInteractiveShape.hpp"

#include <iostream> //DEBUG

void
GLInteractiveShape::draw(QPainter &painter)
{
  assert(m_shape);

  GL_CHECK_ERROR_ALWAYS();

  painter.save();

  glPushMatrix();
  QPoint shapeCenter = m_shape->getCenter();
  
  qreal zoomFactor = m_view->getZoomFactor();
  QPoint absolutZero = m_view->viewPos();

  const double tx = -(shapeCenter.x()*zoomFactor+absolutZero.x());
  const double ty = -(shapeCenter.y()*zoomFactor+absolutZero.y());

  /*
  glTranslatef(-tx, -ty, 0);
  glRotatef(m_shape->getRotationInDegree(), 0, 0, 1.0f);
  glTranslatef(tx, ty, 0);
  */
  painter.translate(-tx, -ty);
  painter.rotate(m_shape->getRotationInDegree());
  painter.translate(tx, ty);


  painter.setPen(m_editionColor);
  m_status->draw(painter, m_shape);

  painter.setPen(m_shapeColor);
  m_shape->draw(m_view, painter);


  glPopMatrix();
  
  painter.restore();

  GL_CHECK_ERROR_ALWAYS();
}


void
GLInteractiveShape::checkStatus(InteractiveShapeStatus *newStatus)
{
  if (newStatus != m_status) {
    delete m_status;
    m_status = newStatus;
  }
}
