/*
  Copyright (C) 2005-2009 Jerome Charton, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef GLINTERACTIVESHAPE_HPP
#define GLINTERACTIVESHAPE_HPP

#include <QPainter>

#include "GLView.hpp"
#include "SegView.hpp"
#include "GLColor.hpp"
#include "InteractiveShapeStatus.hpp"
#include "PaintingShape.hpp"

class QPoint;
class MainWindow;
class QGraphicsItem;
class MainWindow;

static const GLColor DEFAULT_SHAPE_COLOR = GLColor(0, 0, 255);
static const GLColor DEFAULT_EDITION_COLOR = GLColor( 63, 127, 127);

class GIRL_API GLInteractiveShape
{

public:

  GLInteractiveShape(GLView *view) 
  : m_view(view), 
    m_status(NULL),
    m_shape(NULL),
    m_virtualMouse(),
    m_shapeColor(DEFAULT_SHAPE_COLOR), 
    m_editionColor(DEFAULT_EDITION_COLOR)
  {}

  virtual ~GLInteractiveShape() {}
  
  void draw(QPainter &painter);
  
  virtual GLInteractiveShape *clone() = 0;
  
  inline void setColor(GLColor &color) 
  {
    m_shapeColor = color;
  }

  inline void setEditionColor(GLColor &color) 
  {
    m_editionColor = color;
  }
  
  inline void onLeftClick(const QPoint &mouse) 
  {
    assert(m_status);
    assert(m_shape);
    
    m_virtualMouse = m_shape->getOriginalPoint(mouse);
    checkStatus(m_status->onLeftClick(m_virtualMouse, m_shape));
  }

  inline void onRightClick(const QPoint &mouse) 
  {
    assert(m_status);
    assert(m_shape);
    
    m_virtualMouse = m_shape->getOriginalPoint(mouse);
    checkStatus(m_status->onRightClick(m_virtualMouse, m_shape));
  }
  
  inline void onLeftRelease(const QPoint &mouse) 
  {
    assert(m_status);
    assert(m_shape);
    
    m_virtualMouse = m_shape->getOriginalPoint(mouse);
    checkStatus(m_status->onLeftRelease(m_virtualMouse, m_shape));
  }

  inline void onRightRelease(const QPoint &mouse)
  {
    m_virtualMouse = m_shape->getOriginalPoint(mouse);
    checkStatus(m_status->onRightRelease(m_virtualMouse, m_shape));
  }
  
  inline void onLeftClickWithCtrl(const QPoint &mouse) 
  {
    assert(m_status);
    assert(m_shape);
    
    m_virtualMouse = m_shape->getOriginalPoint(mouse);
    checkStatus(m_status->onLeftClickWithCtrl(m_virtualMouse, m_shape));
  }

  inline void onRightClickWithCtrl(const QPoint &mouse) 
  {
    assert(m_status);
    assert(m_shape);
    
    m_virtualMouse = m_shape->getOriginalPoint(mouse);
    checkStatus(m_status->onRightClickWithCtrl(m_virtualMouse, m_shape));
  }
  
  inline void onMove(const QPoint &mouse) 
  {
    assert(m_status);
    assert(m_shape);
    
    m_virtualMouse = m_shape->getOriginalPoint(mouse);
    checkStatus(m_status->onMove(m_virtualMouse, m_shape));
  }
  
  inline void onMoveWithLeftClick(const QPoint &mouse) 
  {
    assert(m_status);
    assert(m_shape);
    
    m_virtualMouse = m_shape->getOriginalPoint(mouse);
    checkStatus(m_status->onMoveWithLeftClick(m_virtualMouse, m_shape));
  }
  
  inline void onMoveWithRightClick(const QPoint &mouse) 
  {
    assert(m_status);
    assert(m_shape);
    
    m_virtualMouse = m_shape->getOriginalPoint(mouse);
    checkStatus(m_status->onMoveWithRightClick(m_virtualMouse, m_shape));
  }
  
  inline bool isUnder(const QPoint &point) 
  {
    assert(m_status);
    assert(m_shape);
    
    QPoint transformedPoint = m_shape->getOriginalPoint(point);
    return m_status->isUnder(transformedPoint, m_shape);
  }
  
  inline bool isSelected() 
  {
    assert(m_status);
    
    return m_status->isSelected();
  }

  inline void setSelected(bool isSelected) 
  {
    assert(m_status);
    
    checkStatus(m_status->setSelected(isSelected));
  }

  inline void getSegmentationPath(int w, int h, girl::pathDeque &paths) 
  {
    assert(m_shape);
    
    return m_shape->getSegmentationPath(w, h, paths);
  }

  inline void getScanlineSet(int w, int h, ScanlineSet &scanlineSet) 
  {
    assert(m_shape);
    
    m_shape->getScanlineSet(w, h, scanlineSet);
  }

  inline const PaintingShape *getShape() const
  {
    return m_shape;
  }
  
protected:
  
  void checkStatus(InteractiveShapeStatus *newStatus);

protected:

  GLView *m_view;
  InteractiveShapeStatus *m_status;
  PaintingShape *m_shape;
  QPoint m_virtualMouse;
  GLColor m_shapeColor;
  GLColor m_editionColor;
};

#endif /*! GLINTERACTIVESHAPE_HPP */
