/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "GLMarkLayer.hpp"

#include "SegModel.hpp"
#include "GLUtils.hpp"
#include "GLView.hpp"


static const GLColor DEFAULT_MARK_COLOR = GLColor(180, 0, 255);

static const girl::toger::markFlags DEFAULT_MARK = girl::toger::MARK_NONE;



GLMarkLayer::GLMarkLayer(GLView *v)
  : GLTogerLayer(v)
{
  m_togColor = DEFAULT_MARK_COLOR;

}

GLMarkLayer::GLMarkLayer(GLView *v, const SegModel *model)
  : GLTogerLayer(v, model)
{
  m_togColor = DEFAULT_MARK_COLOR;

}

void
GLMarkLayer::updateToger(const SegModel *model)
{
  assert(model);
  assert(m_view);

  const girl::toger *tog = model->getToger();
  assert(tog != NULL);

  m_width = tog->width();
  m_height = tog->height();

  m_view->makeCurrent();

  GL_CHECK_ERROR();
  
  if (m_togID == 0) {
    m_togID = glGenLists(1);
  }
  if (m_togID != 0) {
    glNewList(m_togID, GL_COMPILE);

    if (m_drawMode == GLTogerPainter::DISCRETE_PATH) {
      GLTogerPainter::drawToger(*tog, m_mark);
    }
    else {
      assert(m_drawMode == GLTogerPainter::EUCLIDEAN_PATH);
      GLTogerPainter::drawToger(*tog, model->getEPS(), m_mark);
    }

    glEndList();
  }

}

void
GLMarkLayer::clear()
{
  assert(m_view);

  if (m_togID != 0) {

    m_view->makeCurrent();

    GL_CHECK_ERROR();
  
    glNewList(m_togID, GL_COMPILE);

    glEndList();
  }

}

