/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "GLMovableSegmentLayer.hpp"

#include <cassert>
#include <iostream> //DEBUG

#include "GLUtils.hpp"
#include "GLView.hpp"

static const GLColor DEFAULT_SEGMENTS_COLOR = GLColor(255, 255, 25);

GLMovableSegmentLayer::GLMovableSegmentLayer(GLView *v)
  : GLLayer(v)
  , m_transformation()
  , m_segmentsColor(DEFAULT_SEGMENTS_COLOR)
  , m_segmentsID(0)
    /*
      , xmin(std::numeric_limit<int>::max())
      , ymin(std::numeric_limit<int>::max())
      , xmax(std::numeric_limit<int>::min())
      , ymax(std::numeric_limit<int>::min())
    */
{

}

GLMovableSegmentLayer::GLMovableSegmentLayer(GLView *v, 
					     const std::vector< std::vector<girl::point> >&segments)
  : GLLayer(v)
  , m_transformation()
  , m_segmentsColor(DEFAULT_SEGMENTS_COLOR)
  , m_segmentsID(0)
    /*
      , xmin(std::numeric_limit<int>::max())
      , ymin(std::numeric_limit<int>::max())
      , xmax(std::numeric_limit<int>::min())
      , ymax(std::numeric_limit<int>::min())
    */
{
  buildDL(segments);
}
  

void
GLMovableSegmentLayer::setSegments(const std::vector< std::vector<girl::point> >&segments)
{
  buildDL(segments);
}

void
GLMovableSegmentLayer::drawSegments(const std::vector< std::vector<girl::point> >&segments)
{
  std::vector< std::vector<girl::point> >::const_iterator it = segments.begin();
  const std::vector< std::vector<girl::point> >::const_iterator itEnd = segments.end();
  for ( ; it != itEnd; ++it) {

    glBegin(GL_LINE_STRIP);

    std::vector<girl::point>::const_iterator itS = it->begin();
    const std::vector<girl::point>::const_iterator itSEnd = it->end();
    for ( ; itS != itSEnd; ++itS) {
      glVertex2i(itS->x()+1, itS->y()+1);
    }
    
    glEnd();
  }

#ifndef NDEBUG
  //DEBUG
  std::cerr<<"GLMovableSegmentLayer::drawSegments segments.size()="<<segments.size()<<std::endl;
  if (segments.size()>0) {
    std::cerr<<"               segments[segments.size()-1].size()="<<segments[segments.size()-1].size()<<std::endl;
    for (size_t i=0; i<segments[segments.size()-1].size(); ++i) {
      std::cerr<<"("<<segments[segments.size()-1][i].x()<<", "<<segments[segments.size()-1][i].y()<<") ";
    }
    std::cerr<<std::endl;
  }
#endif

}

void
GLMovableSegmentLayer::buildDL(const std::vector< std::vector<girl::point> >&segments)
{
  assert(m_view);

  m_view->makeCurrent();

  GL_CHECK_ERROR();

  if (m_segmentsID == 0) {
    m_segmentsID = glGenLists(1);
  }
  if (m_segmentsID != 0) {
    glNewList(m_segmentsID, GL_COMPILE);
    drawSegments(segments);
    glEndList();
  }

}

void
GLMovableSegmentLayer::draw()
{
  if (m_segmentsID) {
#ifndef NDEBUG
    std::cerr<<"GLMovableSegmentLayer::draw()"<<std::endl;
#endif

    GL_CHECK_ERROR();

    glPushMatrix();
  
    m_transformation.apply();
    
    glColor3ub(m_segmentsColor.red(), m_segmentsColor.green(), m_segmentsColor.blue());
  
    glCallList(m_segmentsID);

    glPopMatrix();

    GL_CHECK_ERROR();
  }
}

void 
GLMovableSegmentLayer::setColor(const GLColor &c)
{
  m_segmentsColor = c;
}

GLColor
GLMovableSegmentLayer::getColor() const
{
  return m_segmentsColor;
}
