/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef GLMOVABLESEGMENTLAYER_HPP
#define GLMOVABLESEGMENTLAYER_HPP

#include <vector>

#include <girl/girlPoint.hpp>

#include "GLLayer.hpp"
#include "GLColor.hpp"
#include "GLTransformation.hpp"

class SegLayer;
class SegmentLayer;

class GIRL_API GLMovableSegmentLayer : public GLLayer
{
public:

  /**
   * @brief Constructor.
   */
  explicit GLMovableSegmentLayer(GLView *v);

  /**
   * @brief Constructor.
   */
  GLMovableSegmentLayer(GLView *v, const std::vector< std::vector<girl::point> >&segments);
  
  void setSegments(const std::vector< std::vector<girl::point> > &segments);

  /**
   * @brief Set color used to draw segments.
   *
   */
  void setColor(const GLColor &c);
  
  /**
   * @brief Get color used to draw segments.
   *
   * @sa setColor.
   */
  GLColor getColor() const;
  

  const GLTransformation &getTransformation() const { return m_transformation; }
  void setTransformation(const GLTransformation &t) { m_transformation = t; }


  virtual void draw();

  //virtual void boundingRect(QRectF &rect);
  //virtual bool intersects(const QPointF &pt);

protected:

  void drawSegments(const std::vector< std::vector<girl::point> >&segments);
  void buildDL(const std::vector< std::vector<girl::point> >&segments);


protected:

  GLTransformation m_transformation;
  GLColor m_segmentsColor;
  unsigned int m_segmentsID;
  //int xmin, ymin, xmax, ymax;
};



#endif /* ! GLMOVABLESEGMENTLAYER_HPP */
