/*
  Copyright (C) 2010-2011 Cédric Aguerre  aguerre@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef GLPOLYLINELAYER_HPP
#define GLPOLYLINELAYER_HPP

#include <qiseTools/GLSelectorLayer.hpp>
#include "GLColor.hpp"
#include <QPoint>
#include <deque>
#include <qiseTools/GLSelectorPoint.hpp>


class GIRL_API GLPolylineLayer : public GLSelectorLayer
{
public:

  /**
   * @brief Constructor.
   */
  GLPolylineLayer(GLView *v, QRgb color = 0, float opacity = 1.0f);

  /**
   * @brief Constructor.
   */
  GLPolylineLayer(GLView *v, const std::deque<GLSelectorPoint> &pts, QRgb color = 0, float opacity = 1.0f);

  /**
   * @brief Set selection points.
   */
  void setSelectionPoints(const std::deque<GLSelectorPoint> &pts);

  void setLastSegmentDisplayable(bool b);
  void setPointsDisplayable(bool b);

  virtual void clear();

  virtual void draw();

  virtual void boundingRect(QRectF &rect);
  virtual bool intersects(const QPointF &pt);

protected:

  std::deque<GLSelectorPoint> m_pts;
  bool m_lastSegmentDisplayable;
  bool m_pointsDisplayable;

 private :

  int insidePolygon(const std::deque<GLSelectorPoint> &polygon, const QPointF &p, int bound);

};


#endif /*! GLPOLYLINELAYER_HPP */
