/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "GLRectangleLayer.hpp"

#include <cassert>

#include "GLUtils.hpp"

static const GLColor DEFAULT_SEGMENTS_COLOR = GLColor(255, 0, 0);


GLRectangleLayer::GLRectangleLayer(GLView *v, QRgb color, float opacity)
  : GLSelectorLayer(v, color, opacity),
    m_p1(0, 0), m_p2(0, 0)
{

}

GLRectangleLayer::GLRectangleLayer(GLView *v, const QPointF &p1, const QPointF &p2, QRgb color, float opacity)
  : GLSelectorLayer(v, color, opacity),
    m_p1(p1), m_p2(p2)
{

}
  
void
GLRectangleLayer::setRectangle(const QPointF &p1, const QPointF &p2)
{
  m_p1 = p1;
  m_p2 = p2;
}

void
GLRectangleLayer::clear()
{
  m_p1 = m_p2 = QPointF(0, 0);
}

void
GLRectangleLayer::draw()
{
  glColor4ub(m_color.red(), m_color.green(), m_color.blue(), m_color.alpha());

  //draw background
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  glEnable(GL_BLEND);

  glBegin(GL_TRIANGLE_STRIP);
  glVertex2f(m_p1.x(), m_p1.y());
  glVertex2f(m_p2.x(), m_p1.y());
  glVertex2f(m_p1.x(), m_p2.y());
  glVertex2f(m_p2.x(), m_p2.y());
  glEnd();

  glDisable(GL_BLEND);

  //draw border
  glBegin(GL_LINE_LOOP);
  glVertex2f(m_p1.x(), m_p1.y());
  glVertex2f(m_p1.x(), m_p2.y());
  glVertex2f(m_p2.x(), m_p2.y());
  glVertex2f(m_p2.x(), m_p1.y());
  glEnd();
}

void
GLRectangleLayer::boundingRect(QRectF &rect)
{
  rect = QRectF(m_p1.x(), m_p1.y(), m_p2.x()-m_p1.x()+1, m_p2.y()-m_p1.y()+1);
}

bool
GLRectangleLayer::intersects(const QPointF &pt)
{
  QRectF r;
  boundingRect(r);
  return r.contains(pt);
}
