/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef GLRECTANGLELAYER_HPP
#define GLRECTANGLELAYER_HPP

#include <qiseTools/GLSelectorLayer.hpp>

class GIRL_API GLRectangleLayer : public GLSelectorLayer
{
public:

  /**
   * @brief Constructor.
   */
  GLRectangleLayer(GLView *v, QRgb color = 0, float opacity = 1.0f);

  /**
   * @brief Constructor.
   *
   * @a p1 and @a p2 are two extremity points of rectangle.
   */
  GLRectangleLayer(GLView *v, const QPointF &p1, const QPointF &p2, QRgb color = 0, float opacity = 1.0f);

  /**
   * @brief Set rectangle points.
   *
   * @a p1 and @a p2 are two extremity points of rectangle.
   */
  void setRectangle(const QPointF &p1, const QPointF &p2);

  virtual void clear();

  virtual void draw();

  virtual void boundingRect(QRectF &rect);
  virtual bool intersects(const QPointF &pt);

protected:

  QPointF m_p1;
  QPointF m_p2;
};


#endif /*! GLRECTANGLELAYER_HPP */
