/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "GLSegmentLayer.hpp"

#include <cassert>

#include "GLUtils.hpp"

static const GLColor DEFAULT_SEGMENTS_COLOR = GLColor(255, 255, 0);

GLSegmentLayer::GLSegmentLayer(GLView *v)
  : GLLayer(v)
  , m_segments()
  , m_segmentsColor(DEFAULT_SEGMENTS_COLOR)
{

}

GLSegmentLayer::GLSegmentLayer(GLView *v, const std::deque<girl::point> &pts)
  : GLLayer(v)
  , m_segments()
  , m_segmentsColor(DEFAULT_SEGMENTS_COLOR)
{
  setSegments(pts);
}
  

void
GLSegmentLayer::setSegments(const std::deque<girl::point> &pts)
{
  m_segments = pts;
}

void
GLSegmentLayer::clearSegments()
{
  m_segments.clear();
}

void
GLSegmentLayer::draw()
{
  /*
  GL_CHECK_ERROR();

  glPushMatrix();

  m_transformation.apply();
  */

  glColor3ub(m_segmentsColor.red(), m_segmentsColor.green(), m_segmentsColor.blue());
  

  drawSegments();

  /*
  glPopMatrix();
  */
}

void
GLSegmentLayer::drawSegments()
{ 
  typedef std::deque<girl::point> Collection;

  glBegin(GL_LINES);

  Collection::iterator it = m_segments.begin();
  const Collection::iterator itEnd = m_segments.end();
  for ( ; it != itEnd; ++it) {
    glVertex2i(it->x()+1, it->y()+1);
  }

  glEnd();
}

void 
GLSegmentLayer::setColor(const GLColor &c)
{
  m_segmentsColor = c;
}

GLColor 
GLSegmentLayer::getColor() const
{
  return m_segmentsColor;
}
