/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef GLSEGMENTLAYER_HPP
#define GLSEGMENTLAYER_HPP

#include <deque>

#include <girl/girlPoint.hpp>

#include "GLLayer.hpp"
#include "GLColor.hpp"

class SegLayer;
class SegmentLayer;

class GIRL_API GLSegmentLayer : public GLLayer
{
public:

  /**
   * @brief Constructor.
   */
  explicit GLSegmentLayer(GLView *v);

  /**
   * @brief Constructor.
   */
  GLSegmentLayer(GLView *v, const std::deque<girl::point> &pts);
  
  void clearSegments();

  void setSegments(const std::deque<girl::point> &pts);

  /**
   * @brief Set color used to draw segments.
   *
   */
  void setColor(const GLColor &c);

  /**
   * @brief Get color used to draw segments.
   *
   * @sa setColor.
   */
  GLColor getColor() const;


  virtual void draw();


protected:

  void drawSegments();


protected:

  std::deque<girl::point> m_segments;
  GLColor m_segmentsColor;
};



#endif /* ! GLSEGMENTLAYER_HPP */
