/*
  Copyright (C) 2010 Cédric Aguerre  aguerre@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "GLSelectorLayer.hpp"

#include <cassert>


GLSelectorLayer::GLSelectorLayer(GLView *v, QRgb color, float opacity)
  : GLLayer(v),
    m_color(color),
    m_opacity(opacity)
{

}

void 
GLSelectorLayer::setColor(const GLColor &c, bool setOpacity)
{
  m_color = c;
  if (! setOpacity) 
    m_color.setAlphaF(m_opacity);
}

GLColor
GLSelectorLayer::getColor() const
{
  return m_color;
}

void
GLSelectorLayer::setOpacity(float alpha)
{
  assert(alpha>=0.0 && alpha<=1.0);
  m_opacity = alpha;
  m_color.setAlphaF(alpha);
}

float
GLSelectorLayer::getOpacity() const
{
  return m_opacity;
}
