/*
  Copyright (C) 2010-2011 Cédric Aguerre  aguerre@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef GLSELECTORLAYER_HPP
#define GLSELECTORLAYER_HPP

#include "GLLayer.hpp"
#include "GLColor.hpp"

class GIRL_API GLSelectorLayer : public GLLayer
{
public:

  /**
   * @brief Constructor.
   */
  GLSelectorLayer(GLView *v, QRgb color = 0, float opacity = 1.0f);

  /**
   * @brief Set color used to draw selector.
   *
   * @a setOpacity indicates if opacity of @a c must be taken into account.
   */
  void setColor(const GLColor &c, bool setOpacity = false);

  /**
   * @brief Get color used to draw selection.
   *
   * @sa setColor.
   */
  GLColor getColor() const;
  
  /**
   * @brief Set opacity used to draw selection.
   *
   * @a alpha 0.0f means all transparent, 1.0f means completely opaque.
   */
  void setOpacity(float alpha);

  /**
   * @brief Get opacity used to draw selection.
   *
   * @sa setOpacity.
   */
  float getOpacity() const;


  virtual void clear() = 0;

protected:

  GLColor m_color;
  float m_opacity;
};


#endif /*! GLSELECTORLAYER_HPP */
