/*
  Copyright (C) 2010-2011 Cédric Aguerre  aguerre@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "GLSelectorPoint.hpp"
#include <cmath>
#include <cassert>
#include "GLUtils.hpp"


static const int DEFAULT_RADIUS = 2;
static const bool DEFAULT_VISIBLE = true;


GLSelectorPoint::GLSelectorPoint()
  : GLEllipse(),
    m_visible(DEFAULT_VISIBLE)
{

}

GLSelectorPoint::GLSelectorPoint(const QPointF &pt, QColor color, bool visible)
  : GLEllipse(pt, color),
    m_visible(visible)
{

}

qreal
GLSelectorPoint::x() const
{
  return m_pos.x();
}

qreal
GLSelectorPoint::y() const
{
  return m_pos.y();
}

void
GLSelectorPoint::setRadius(int r)
{
  m_radiusH = m_radiusV = r;
}

bool
GLSelectorPoint::isVisible() const
{
  return m_visible;
}

void
GLSelectorPoint::setVisible(bool b)
{
  m_visible = b;
}

void
GLSelectorPoint::draw()
{
  if (!m_visible)
    return;
  GLEllipse::draw();
}

bool
GLSelectorPoint::operator==(const GLSelectorPoint &pt) const
{
  return (x() == pt.x() && y() == pt.y());
}

bool
GLSelectorPoint::operator!=(const GLSelectorPoint &pt) const
{
  return (x() != pt.x() || y() != pt.y());
}
