/*
  Copyright (C) 2005-2009 Jerome Charton, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "GLShapesLayer.hpp"

//#include <QGLWidget>
#include <cassert>

GLShapesLayer::GLShapesLayer(GLView *v) : GLLayer(v), m_shapes(), m_selectedShapes(), m_currentShape(NULL)
{}

GLShapesLayer::~GLShapesLayer()
{
  //B
  assert(m_currentShape == NULL 
	 || std::find(m_shapes.begin(), m_shapes.end(), m_currentShape) == m_shapes.end());

  deleteAllShapes(); //B
  if (m_currentShape != NULL)
    delete m_currentShape;
}

void
GLShapesLayer::draw()
{
  m_painter.begin((QPaintDevice *)m_view);
  for (shape_set_t::iterator it = m_shapes.begin();
       it != m_shapes.end(); 
       ++it)
    (*it)->draw(m_painter);
  m_painter.end();
}

void
GLShapesLayer::setCurrentShape(GLInteractiveShape *shape)
{
  //B
  assert(m_currentShape == NULL 
	 || std::find(m_shapes.begin(), m_shapes.end(), m_currentShape) == m_shapes.end());


  if (m_currentShape != NULL)
    delete m_currentShape;
  m_currentShape = shape;
}

GLShapesLayer::shape_set_t::iterator
GLShapesLayer::getFirstUnder(QPoint point)
{
  assert(m_currentShape != NULL);

  for (shape_set_t::iterator it = m_shapes.begin();
       it != m_shapes.end(); 
       ++it)
    if ((*it)->isUnder(point))
      return it;
  
  m_shapes.push_front(m_currentShape->clone());
  return m_shapes.begin();
}

void
GLShapesLayer::setFirst(const GLShapesLayer::shape_set_t::iterator &it)
{
  m_shapes.push_front(*it);
  m_shapes.erase(it);
}

void
GLShapesLayer::deleteShape(GLShapesLayer::shape_set_t::iterator &it)
{
  delete (*it);
  it = m_shapes.erase(it);
}

void
GLShapesLayer::deleteAllShapes()
{
  //B
  assert(m_currentShape == NULL
	 || std::find(m_shapes.begin(), m_shapes.end(), m_currentShape) == m_shapes.end());


  const shape_set_t::iterator itEnd = m_shapes.end();
  for (shape_set_t::iterator it = m_shapes.begin();
       it != itEnd; 
       ++it) {

    delete (*it);
  }

  m_shapes.clear();

  assert(m_shapes.empty() == 0);
}

void
GLShapesLayer::deleteSelectedShapes()
{
  for (shape_set_t::iterator it = m_shapes.begin();
       it != m_shapes.end(); 
       ) {

    if ((*it)->isSelected()) {
      delete (*it);
      it = m_shapes.erase(it);
    }
    else
      ++it;
  }

}

