/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "GLTogerLayer.hpp"

#include <iostream>

#include "SegModel.hpp"
#include "GLUtils.hpp"
#include "GLView.hpp"

static const GLColor DEFAULT_TOGER_COLOR = GLColor(0, 255, 0);

static const GLTogerPainter::DrawMode DEFAULT_DRAW_MODE = GLTogerPainter::DISCRETE_PATH;



GLTogerLayer::GLTogerLayer(GLView *v)
  : GLLayer(v)
  , m_togColor(DEFAULT_TOGER_COLOR)
  , m_togID(0)
  , m_width(0)
  , m_height(0)
  , m_drawMode(DEFAULT_DRAW_MODE)
{
  
}

GLTogerLayer::GLTogerLayer(GLView *v, const SegModel *model)
  : GLLayer(v)
  , m_togColor(DEFAULT_TOGER_COLOR)
  , m_togID(0)
  , m_width(0)
  , m_height(0)
  , m_drawMode(DEFAULT_DRAW_MODE)
{
  assert(model);
  update(model);
}


void
GLTogerLayer::update(const SegModel *model)
{
  assert(model);
  assert(m_view);

  const girl::toger *tog = model->getToger();
  assert(tog != NULL);

  m_width = tog->width();
  m_height = tog->height();

  m_view->makeCurrent();

  GL_CHECK_ERROR();
  
  if (m_togID == 0) {
    m_togID = glGenLists(1);
  }
  if (m_togID != 0) {
    glNewList(m_togID, GL_COMPILE);

    if (m_drawMode == GLTogerPainter::DISCRETE_PATH) {
      GLTogerPainter::drawToger(*tog);
    }
    else {
      assert(m_drawMode == GLTogerPainter::EUCLIDEAN_PATH);
      GLTogerPainter::drawToger(*tog, model->getEPS());
    }

    glEndList();
  }
}


void
GLTogerLayer::draw()
{
  if (m_togID) {
    
    GL_CHECK_ERROR();
  
    /*
      glPushMatrix();

      m_transformation.apply();
    */

    GL_CHECK_ERROR_ALWAYS();

    glColor3ub(m_togColor.red(), m_togColor.green(), m_togColor.blue());

    GL_CHECK_ERROR();

    glCallList(m_togID);

    GL_CHECK_ERROR_ALWAYS();

    /*
      glPopMatrix();
    */
  }


}

void 
GLTogerLayer::setColor(const GLColor &c)
{
  m_togColor = c;
}

GLColor
GLTogerLayer::getColor() const
{
  return m_togColor;
}


void
GLTogerLayer::boundingRect(QRectF &rect)
{
  rect = QRectF(0, 0, m_width, m_height);
}

