/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef GLTOGERLAYER_HPP
#define GLTOGERLAYER_HPP

#include <girl/girlToger.hpp>

#include "GLLayer.hpp"
#include "GLColor.hpp"
#include "GLTogerPainter.hpp"

class SegModel;

class GIRL_API GLTogerLayer : public GLLayer
{
public:

  /**
   * @brief Constructor.
   */
  explicit GLTogerLayer(GLView *v);

  /**
   * @brief Constructor.
   */
  GLTogerLayer(GLView *v, const SegModel *model);

  /**
   * @brief Update drawn toger.
   */
  void update(const SegModel *model);

  /**
   * @brief Set color used to draw toger.
   */
  void setColor(const GLColor &c);

  /**
   * @brief Get color used to draw selection.
   *
   * @sa setColor.
   */
  GLColor getColor() const;

  /**
   * @brief Set draw mode.
   */
  void setDrawMode(GLTogerPainter::DrawMode dm, const SegModel *model)
  {
    m_drawMode = dm;
    update(model);
  }

  /**
   * @brief Get draw mode.
   */
  GLTogerPainter::DrawMode getDrawMode() const
  {
    return m_drawMode;
  }

  virtual void boundingRect(QRectF &rect);   

  virtual void draw();

protected:


protected:

  GLColor m_togColor;
  unsigned int m_togID;
  int m_width;
  int m_height;
  GLTogerPainter::DrawMode m_drawMode; 
};

#endif /* ! GLTOGERLAYER_HPP */
