/*
  Copyright (C) 2005-2010 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "GLTogerPainter.hpp"

#include <girlTools/girlToolsRegionInfos.hpp>

#include <girlTools/TogerPainter.hpp>
#include "GLUtils.hpp"


#include <limits> 

#ifdef TIMING_PAINTER
#include <girl/girlTimer.hpp>
#endif
#include <iostream> 


//Some heavy code duplication
//but we want these functions to be the fastest possible.


#ifdef STATS_PAINTER
size_t STATS_number_points = 0;
size_t STATS_number_triangles = 0;
#endif

namespace { //anonymous namespace

  class Contexter
  {
  public:
    
    inline void set()
    {
      glPushMatrix();
      glTranslatef(1, 1, 0.0f);
    }

    inline void unset()
    {
      glPopMatrix();
    }

  };


  class SegmentDrawer : public girl::tools::BoundaryChangePointIterator
  {
  public:

    SegmentDrawer()
      : girl::tools::BoundaryChangePointIterator()
    {

    }

    inline void begin()
    {
      glBegin(GL_LINE_STRIP);
    }

    void processPoint(const girl::point &p)
    {
      glVertex2i(p.x(), p.y());

#ifdef STATS_PAINTER
      ++STATS_number_points;
#endif
    }

    void processLastPoint(const girl::point &p)
    {
      processPoint(p);
    }

    inline void end()
    {
      glEnd();
    }

  protected:

  };


  //---- euclideanPath --------------------------------------------------------


  class SegmentEPDrawer : public girl::tools::BoundaryPointIterator
  {
  public:

    explicit SegmentEPDrawer(const girl::euclideanPathSet &eps)
      : girl::tools::BoundaryPointIterator()
      , m_eps(eps)
    {

    }

    inline void begin()
    {
      glBegin(GL_LINE_STRIP);
    }


    void processPoint(const girl::point &p)
    {
      float xf, yf;
      m_eps.getPoint(p, xf, yf);

      glVertex2f(xf, yf);

#ifdef STATS_PAINTER
      ++STATS_number_points;
#endif
    }

    void processLastPoint(const girl::point &p)
    {
      processPoint(p);
    }

    inline void end()
    {
      glEnd();
    }

  protected:
    const girl::euclideanPathSet &m_eps;

  };

  /*
    static
    void
    P_drawFacePositiveDarts(const girl::toger &tog, girl::face f, 
    const girl::euclideanPathSet &eps)
    {
    SegmentEPDrawer sd(eps);

    const girl::dart canonicD = tog.beta(f);
    girl::dart d = canonicD;
  
    do
    {
    if (! tog.isFaceInfinite(tog.lambda(tog.alpha(d)))) {
    if (d > 0) {
    sd.begin();
    tog.iterateOnSegment(d, sd);
    sd.end();
    }
    }
    else {
    sd.begin();
    tog.iterateOnSegment(d, sd);
    sd.end();
    }
            
    d = tog.phi(d);

    }
    while (d != canonicD);
 
    }

    static
    void
    P_drawFacePositiveDarts(const girl::toger &tog, girl::face f, 
    const girl::euclideanPathSet &eps, 
    girl::toger::markFlags mark)
    {
    SegmentEPDrawer sd(eps);

    const girl::dart canonicD = tog.beta(f);
    girl::dart d = canonicD;
  
    do
    {
    if (tog.isDartMarked(d, mark)) {

    if (! tog.isFaceInfinite(tog.lambda(tog.alpha(d)))) {
    if (d > 0) {
    sd.begin();
    tog.iterateOnSegment(d, sd);
    sd.end();
    }
    }
    else {
    sd.begin();
    tog.iterateOnSegment(d, sd);
    sd.end();
    }
	
    }
    d = tog.phi(d);
      
    }
    while (d != canonicD);
  
    }


    static
    void
    P_drawFace(const girl::toger &tog, girl::face f, 
    const girl::euclideanPathSet &eps)
    {
    SegmentEPDrawer sd(eps);

    sd.begin();
    tog.iterateOnFaceBoundary(f, sd);
    sd.end();
    }

    static
    void
    P_drawFace(const girl::toger &tog, girl::face f, 
    const girl::euclideanPathSet &eps, 
    girl::toger::markFlags mark)
    {
    SegmentEPDrawer sd(eps);

    const girl::dart canonicD = tog.beta(f);
    girl::dart d = canonicD;
  
    do
    {
    if (tog.isDartMarked(d, mark)) {
	
    sd.begin();
    tog.iterateOnSegment(d, sd);
    sd.end();
    }
            
    d = tog.phi(d);

    }
    while (d != canonicD);
    }
  */


  /*
    static
    void
    P_drawSegment(const girl::toger &tog, girl::dart d)
    {
    SegmentDrawer sd;
    drawSegmentT<SegmentDrawer>(sd, tog, d);
    }
  */


  /*
    static
    void
    P_drawSegment(const girl::toger &tog, girl::dart d, 
    girl::toger::markFlags mark)
    {
    if (tog.isDartMarked(d, mark)) {
    P_drawSegment(tog, d);
    }
    }
  */

  /*
    static
    void
    P_drawSegment(const girl::toger &tog, girl::dart d, 
    const girl::euclideanPathSet &eps)
    {
    SegmentEPDrawer sd(eps);

    sd.begin();
    tog.iterateOnSegment(d, sd);
    sd.end();
    }

    static
    void
    P_drawSegment(const girl::toger &tog, girl::dart d, 
    const girl::euclideanPathSet &eps, 
    girl::toger::markFlags mark)
    {
    if (tog.isDartMarked(d, mark)) {
    P_drawSegment(tog, d, eps);
    }
    }
  */




  class polygonStencilDrawer : public girl::tools::BoundaryChangePointIterator
  {
  public:

    polygonStencilDrawer()
      : girl::tools::BoundaryChangePointIterator()
    {
    }

    inline void begin()
    {
      glBegin(GL_TRIANGLE_FAN);
    }

    virtual void processPoint(const girl::point &p)
    {
      glVertex2i(p.x(), p.y());

#ifdef STATS_PAINTER
      ++STATS_number_triangles;
#endif
    }

    inline void end()
    {
      glEnd();

#ifdef STATS_PAINTER
      STATS_number_triangles-=2;
#endif
    }

  };

  class polygonStencilWithBBoxDrawer : public girl::tools::BoundaryChangePointIterator
  {
  public:

    polygonStencilWithBBoxDrawer()
      : girl::tools::BoundaryChangePointIterator(),
	xMin(std::numeric_limits<int>::max()),
	yMin(std::numeric_limits<int>::max()),
	xMax(std::numeric_limits<int>::min()),
	yMax(std::numeric_limits<int>::min())
    {

    }

    inline void begin()
    {
      glBegin(GL_TRIANGLE_FAN);
    }

    virtual void processFirstPoint(const girl::point &p)
    {
      xMin = p.x();
      xMax = p.x();
      yMin = p.y();
      yMax = p.y();

      girl::tools::BoundaryChangePointIterator::processFirstPoint(p);
    }

    virtual void processPoint(const girl::point &p)
    {
      //compute bbox
      if (p.x() < xMin)
	xMin = p.x();
      else if (p.x() > xMax)
	xMax = p.x();
      if (p.y() < yMin)
	yMin = p.y();
      else if (p.y() > yMax)
	yMax = p.y();

      glVertex2i(p.x(), p.y());

#ifdef STATS_PAINTER
      ++STATS_number_triangles;
#endif
    }

    inline void end()
    {
      glEnd();

#ifdef STATS_PAINTER
      STATS_number_triangles-=2;
#endif
    }

  public:
    int xMin;
    int yMin;
    int xMax;
    int yMax;
  };



  //#define COMPUTE_MINIMUM_FILL 1

#ifdef COMPUTE_MINIMUM_FILL

  /*
    With stencil method, for each region we draw triangles fans to 
    all (change) points of the external boundary, and also 
    triangles fans for all holes boundaries.
    On large images, with lots of regions, it can be a lot of triangles with 
    a lot of overlapping.

    However, inside holes we can still have other region that are selected. 
    this is typically the case when all regions are selected.
    For example, suppose we have a region 1 with one hole. Inside this hole we 
    have only one region : region 2.
    If both region 1 and 2 are selected, as region 1 is selected we will draw 
    one triangle fan for region 1 outside boundary, one triangle fan for 
    region 1 hole (that is region 2 outside boundary), then as region 2 is also 
    selected we will draw on triangle fan for region 2 outside boundary. But we
    could have drawn only region 1 outside boundary ! 

    Thus, we can reduce overlapping by computing exactly for selected regions 
    which other regions are inside them and are not selected. We would then only 
    draw sibling top level selected regions.
  
  
    We will build a structure that for each selected region assigns a set of 
    inside unselected regions.
    [We do not draw holes as faces anymore, but as regions].



    ->> Current implementation is very slow !!!
    It seems faster to draw all regions and their respective holes than to 
    first compute our structure.
  
    For example, on 850x1170 image (296_3.png), with 28343 regions, 
    [on Core2Duo 6600, with ATI X1300; Fedora 9]
    - without this computation, first draw takes roughly 
    60ms (for 233596 triangles) 
    - with this computation, first draw takes roughly 
    31100ms (for 104038 triangles) !!!

  
  */

  typedef girl::unorderedSet<girl::region> RegionHash;

  struct regionInfo
  {
    RegionHash holes;
    bool processed : 1;
    bool toDraw : 1;

    regionInfo()
      : holes(), processed(false), toDraw(true)
    {}
  };

  typedef girl::hashMap<girl::region, regionInfo> RegionInfoHash;

  void
  getHoles(const girl::toger &tog, RegionInfoHash &h, RegionInfoHash::iterator itH)
  {
    //std::cerr<<"getHoles "<<itH->first<<" processed="<<itH->second.processed<<std::endl;
    assert(itH->second.processed == false);

    const RegionInfoHash::iterator itHEnd = h.end();

    //process all faces/holes of the region
    const girl::region r = itH->first;
    for (girl::face f = tog.firstChild(r); f!=0; f=tog.nextSibling(f)) {

      const girl::dart canonicD = tog.beta(f);
      girl::dart d = canonicD;
    
      do {
	const girl::region ri = tog.lambda(tog.alpha(d));
	assert(! tog.isFaceInfinite(ri));
	
	RegionInfoHash::iterator itHRi = h.find(ri);
	if (itHRi != itHEnd) {
	  //inside region is also selected

	  if (itHRi->second.processed == false) {
	    //inside region selected but not yet processed, we will dig deeper
	  
	    //std::cerr<<itHRi->first<<" inner "<<itH->first<<std::endl;

	    getHoles(tog, h, itHRi);

	    //process direct neighbours of inner region
	    
	    const girl::dart canonicND = tog.beta(r);
	    girl::dart nd = canonicND;
	    do {
	      const girl::region rn = tog.lambda(tog.alpha(nd));
	      if (! tog.isFaceInfinite(rn)) {
		RegionInfoHash::iterator itHRN = h.find(rn);
		if (itHRN != itHEnd) {
		  //neighbour region (of inner region) also selected
		  if (itHRN->second.processed == false) {
		    //neighbour region (of inner region) not already processed

		    //std::cerr<<itHRN->first<<" direct of "<<itHRi->first<<" inside "<<itH->first<<std::endl;

		    getHoles(tog, h, itHRN);
		  }
		}
		else {
		  //neighour region (of inner region) is not selected
		  itH->second.holes.insert(rn);
		}
	      }

	      nd = tog.phi(nd);
	    }
	    while (nd != canonicND);
	  
	  }
	  else {
	    //inside region selected and processed

	    //transfer holes to parent region
	    RegionHash::const_iterator it = itHRi->second.holes.begin();
	    const RegionHash::const_iterator itEnd = itHRi->second.holes.end();
	    for ( ; it != itEnd; ++it) {
	      itH->second.holes.insert(it->first);
	    }
	    itHRi->second.holes.clear();
	    itHRi->second.toDraw = false;

	  }
	  
	}
	else {
	  //inside region is not selected
	  itH->second.holes.insert(ri);
	}
      
	d = tog.phi(d);	
      }
      while (d != canonicD);
    
    }
  
    itH->second.processed = true;
  }


  void
  getRegionsAndHoles(const girl::toger &tog, const girl::regionDeque &regions,
		     RegionInfoHash &h)
  {

    girl::regionDeque::const_iterator it = regions.begin();
    const girl::regionDeque::const_iterator itEnd = regions.end();
    for ( ; it != itEnd; ++it)
      h[*it] = regionInfo();

    RegionInfoHash::iterator itH = h.begin();
    const RegionInfoHash::iterator itHEnd = h.end();
    for ( ; itH != itHEnd; ++itH) {
      if (! itH->second.processed) {
	getHoles(tog, h, itH);
	assert(itH->second.processed);
      }
    }
  
  }


  template <typename BBDrawer, typename Drawer>
  struct RegionTriangleFanDrawerHash
  {
    RegionTriangleFanDrawerHash(BBDrawer &psdbb, Drawer &psd)
      : m_psdbb(psdbb), m_psd(psd), m_holes(NULL)
    {}

    void setHoles(const RegionHash &holes)
    {
      m_holes = &holes;
    }

    void draw(const girl::toger &tog, girl::region r)
    {
      //first draw triangle fan on outer boundary and compute bounding box 
      m_psdbb.begin();
      tog.iterateOnFaceBoundary(r, m_psdbb);
      m_psdbb.end();
    
      //second, draw triangles fans for holes//inside regions not selected
      assert(m_holes != NULL);
      const RegionHash &holes = *m_holes;
      RegionHash::const_iterator it = holes.begin();
      const RegionHash::const_iterator itEnd = holes.end();
      for ( ; it != itEnd; ++it) {
	m_psd.begin();
	tog.iterateOnFaceBoundary(it->first, m_psd);
	m_psd.end();
      }

    }
    
    float xMin() { return m_psdbb.xMin; }
    float xMax() { return m_psdbb.xMax; }
    float yMin() { return m_psdbb.yMin; }
    float yMax() { return m_psdbb.yMax; }


    BBDrawer &m_psdbb;
    Drawer &m_psd;
    const RegionHash *m_holes;
  };

#endif


  template <typename BBDrawer, typename Drawer>
  struct RegionTriangleFanDrawer
  {
    RegionTriangleFanDrawer(BBDrawer &psdbb, Drawer &psd)
      : m_psdbb(psdbb), m_psd(psd)
    {}

    void draw(const girl::toger &tog, girl::region r)
    {
      //first draw triangle fan on outer boundary and compute bounding box 
      m_psdbb.begin();
      tog.iterateOnFaceBoundary(r, m_psdbb);
      m_psdbb.end();
    
      //second, draw triangles fans for holes//daughters
      for (girl::face f = tog.firstChild(r); f!=0; f=tog.nextSibling(f)) {
	m_psd.begin();
	tog.iterateOnFaceBoundary(f, m_psd);
	m_psd.end();
      }
    }
  
   
    float xMin() { return m_psdbb.xMin; }
    float xMax() { return m_psdbb.xMax; }
    float yMin() { return m_psdbb.yMin; }
    float yMax() { return m_psdbb.yMax; }


    BBDrawer &m_psdbb;
    Drawer &m_psd;
  };



  template <typename Drawer>
  void
  P_drawFilledRegion_aux(const girl::toger &tog, girl::region r, Drawer &dr)
  {
    //- first, draw to stencil 
    //we will have 1 where the polygon is, 0 elsewhere

    glStencilMask(0x01);
    glStencilOp(GL_KEEP, GL_KEEP, GL_INVERT);
    glStencilFunc(GL_ALWAYS, 0, ~0);
    glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);//disable color writes


    dr.draw(tog, r);


    //- second, draw a quad covering polygon.
    // quad will be drawn only where stencil is 1.

    glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);//enable color writes
#if 1
    glStencilFunc(GL_EQUAL, 0x01, 0x01);
    glStencilOp(GL_ZERO, GL_ZERO, GL_ZERO);
#else
    glStencilFunc(GL_NOTEQUAL, 0, 0x01);
#endif

    //B: specify color ??????

    glBegin(GL_QUADS);
    glVertex3f(dr.xMin(), dr.yMin(), 0.0); 
    glVertex3f(dr.xMax(), dr.yMin(), 0.0); 
    glVertex3f(dr.xMax(), dr.yMax(), 0.0); 
    glVertex3f(dr.xMin(), dr.yMax(), 0.0); 
    glEnd();
 
#if 0
    //-third, clear stencil, with the bounding quad 
    glStencilFunc(GL_ALWAYS, 0, 0);
    glStencilOp(GL_ZERO, GL_ZERO, GL_ZERO);
    glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);//disable color writes
  
    glBegin(GL_QUADS);
    glVertex3f(dr.xMin(), dr.yMin(), 0.0); 
    glVertex3f(dr.xMax(), dr.yMin(), 0.0); 
    glVertex3f(dr.xMax(), dr.yMax(), 0.0); 
    glVertex3f(dr.xMin(), dr.yMax(), 0.0); 
    glEnd();
    glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);//enable color writes
#endif  
  }





  //again some CODE DUPLICATION for euclidean paths
  class polygonStencilWithBBoxDrawerEPS : public girl::tools::BoundaryPointIterator
  {
  public:

    explicit polygonStencilWithBBoxDrawerEPS(const girl::euclideanPathSet &eps)
      : girl::tools::BoundaryPointIterator(),
	m_eps(eps),
	xMin(std::numeric_limits<float>::max()),
	yMin(std::numeric_limits<float>::max()),
	xMax(-std::numeric_limits<float>::max()),
	yMax(-std::numeric_limits<float>::max())
    {

    }

    void begin()
    {
      glBegin(GL_TRIANGLE_FAN);
    }

    virtual void processFirstPoint(const girl::point &p)
    {
      float xf;
      float yf;
      m_eps.getPoint(p, xf, yf);

      xMin = xf;
      xMax = xf;
      yMin = yf;
      yMax = yf;

      girl::tools::BoundaryPointIterator::processFirstPoint(p);
    }

    virtual void processPoint(const girl::point &p)
    {
      float xf, yf;
      m_eps.getPoint(p, xf, yf);

      //compute bbox
      if (xf < xMin)
	xMin = xf;
      else if (xf > xMax)
	xMax = xf;
      if (yf < yMin)
	yMin = yf;
      else if (yf > yMax)
	yMax = yf;


      glVertex2f(xf, yf);
    }

    void end()
    {
      glEnd();
    }

  protected:
    const girl::euclideanPathSet &m_eps;

  public:
    float xMin;
    float yMin;
    float xMax;
    float yMax;

  };


  class polygonStencilDrawerEPS : public girl::tools::BoundaryPointIterator
  {
  public:

    explicit polygonStencilDrawerEPS(const girl::euclideanPathSet &eps)
      : girl::tools::BoundaryPointIterator(),
	m_eps(eps)
    {
    }

    void begin()
    {
      glBegin(GL_TRIANGLE_FAN);
    }

    virtual void processPoint(const girl::point &p)
    {
      float xf, yf;
      m_eps.getPoint(p, xf, yf);

      glVertex2f(xf, yf);
    }

    void end()
    {
      glEnd();
    }

  protected:
    const girl::euclideanPathSet &m_eps;

  };


} //end anonymous namespace



  void
  GLTogerPainter::drawFilledRegions(const girl::toger &tog, 
				    const girl::regionDeque &regions, 
				    const girl::euclideanPathSet &eps)
  {
#ifdef STATS_PAINTER
    STATS_number_triangles = 0;
#endif

#ifdef TIMING_PAINTER
    girl::Timer t;
    t.start();
#endif

    //OPTIM:
    //If all regions are selected, we can avoid to draw each region individually:
    // we can draw only a big rectangle !
    // [Even if tog.numRegions() is O(N), this test seems a good optimization.
    //  First tog.numRegions() is O(N) but it is quite fast (no allocation).
    //  Besides, when we do the first split, we often have lots of regions and we will draw all of them filled...]

    if (regions.size() == tog.numRegions()) {
      
      glPushMatrix();
      glTranslatef(1, 1, 0.0f);

      glBegin(GL_QUADS);
      glVertex3f(-1, -1, 0.0); 
      glVertex3f(tog.width()-1, -1, 0.0); 
      glVertex3f(tog.width()-1, tog.height()-1, 0.0); 
      glVertex3f(-1, tog.height()-1, 0.0); 
      glEnd();

      glPopMatrix();
      
    }
    else {

#ifndef COMPUTE_MINIMUM_FILL

      girl::regionDeque::const_iterator it = regions.begin();
      const girl::regionDeque::const_iterator itEnd = regions.end();
      if (it != itEnd) {
	glEnable(GL_STENCIL_TEST);

	glPushMatrix();
	glTranslatef(1, 1, 0.0f);

	polygonStencilWithBBoxDrawerEPS psdbb(eps);
	polygonStencilDrawerEPS psd(eps);
	RegionTriangleFanDrawer<polygonStencilWithBBoxDrawerEPS, polygonStencilDrawerEPS> dr(psdbb, psd);

	for ( ; it != itEnd; ++it)
	  P_drawFilledRegion_aux(tog, *it, dr);

	glPopMatrix();

	glDisable(GL_STENCIL_TEST);
      }

#else

      if (! regions.empty()) {
	glEnable(GL_STENCIL_TEST);

	glPushMatrix();
	glTranslatef(1, 1, 0.0f);

	RegionInfoHash h;
	getRegionsAndHoles(tog, regions, h);

	polygonStencilWithBBoxDrawerEPS psdbb(eps);
	polygonStencilDrawerEPS psd(eps);
	RegionTriangleFanDrawerHash<polygonStencilWithBBoxDrawerEPS, polygonStencilDrawerEPS> dr(psdbb, psd);

	RegionInfoHash::const_iterator it = h.begin();
	const RegionInfoHash::const_iterator itEnd = h.end();
	for ( ; it != itEnd; ++it) {
	  if (it->second.toDraw) {
	    dr.setHoles(it->second.holes);
	    P_drawFilledRegion_aux(tog, it->first, dr);
	  }
	}

	glPopMatrix();
    
	glDisable(GL_STENCIL_TEST);
      }



#endif
    }

#ifdef TIMING_PAINTER
    const size_t time = t.stop();
    std::cerr<<"GLTogerPainter::drawFilledRegions(): time="<<time<<"ms"<<std::endl;
#endif

#ifdef STATS_PAINTER
      std::cerr<<"GLTogerPainter::drawFilledRegions(): "<<STATS_number_triangles
	       <<" triangles"<<std::endl;
#endif
  }

  void
  GLTogerPainter::drawFilledRegion(const girl::toger &tog, girl::region r, const girl::euclideanPathSet &eps)
  {
#ifdef STATS_PAINTER
    STATS_number_triangles = 0;
#endif

#ifdef TIMING_PAINTER
    girl::Timer t;
    t.start();
#endif

    glEnable(GL_STENCIL_TEST);

    glPushMatrix();
    glTranslatef(1, 1, 0.0f);

    polygonStencilWithBBoxDrawerEPS psdbb(eps);
    polygonStencilDrawerEPS psd(eps);
    RegionTriangleFanDrawer<polygonStencilWithBBoxDrawerEPS, polygonStencilDrawerEPS> dr(psdbb, psd);

    P_drawFilledRegion_aux(tog, r, dr);

    glPopMatrix();

    glDisable(GL_STENCIL_TEST);

#ifdef TIMING_PAINTER
    const size_t time = t.stop();
    std::cerr<<"GLTogerPainter::drawFilledRegion(): time="<<time<<"ms"<<std::endl;
#endif

#ifdef STATS_PAINTER
    std::cerr<<"GLTogerPainter::drawFilledRegion(): "<<STATS_number_triangles
	     <<" triangles"<<std::endl;
#endif
  }





  void
  GLTogerPainter::drawToger(const girl::toger &tog)
  {
    SegmentDrawer sd;
    Contexter ctx;
    drawTogerT<SegmentDrawer, Contexter>(sd, ctx, tog);
  }

  void
  GLTogerPainter::drawToger(const girl::toger &tog, girl::toger::markFlags mark)
  {
    SegmentDrawer sd;
    Contexter ctx;
    drawTogerT<SegmentDrawer, Contexter>(sd, ctx, tog, mark);
  }

  void
  GLTogerPainter::drawToger(const girl::toger &tog, const girl::euclideanPathSet &eps)
  {
    SegmentEPDrawer sd(eps);
    Contexter ctx;
    drawTogerT<SegmentEPDrawer, Contexter>(sd, ctx, tog);
  }

  void
  GLTogerPainter::drawToger(const girl::toger &tog, const girl::euclideanPathSet &eps,
			    girl::toger::markFlags mark)
  {
    SegmentEPDrawer sd(eps);
    Contexter ctx;
    drawTogerT<SegmentEPDrawer, Contexter>(sd, ctx, tog, mark);
  }


  void
  GLTogerPainter::drawRegions(const girl::toger &tog, 
			      const girl::regionDeque &regions)
  {
    SegmentDrawer sd;
    Contexter ctx;
    drawRegionsT<SegmentDrawer, Contexter>(sd, ctx, tog, regions);
  }

  void
  GLTogerPainter::drawRegions(const girl::toger &tog, 
			      const girl::regionDeque &regions, 
			      girl::toger::markFlags mark)
  {
    SegmentDrawer sd;
    Contexter ctx;
    drawRegionsT<SegmentDrawer, Contexter>(sd, ctx, tog, regions, mark);

  }

  void
  GLTogerPainter::drawRegions(const girl::toger &tog, 
			      const girl::regionDeque &regions, 
			      const girl::euclideanPathSet &eps)
  {
    SegmentEPDrawer sd(eps);
    Contexter ctx;
    drawRegionsT<SegmentEPDrawer, Contexter>(sd, ctx, tog, regions);
  }

  void
  GLTogerPainter::drawRegions(const girl::toger &tog, 
			      const girl::regionDeque &regions, 
			      const girl::euclideanPathSet &eps, 
			      girl::toger::markFlags mark)
  {
    SegmentEPDrawer sd(eps);
    Contexter ctx;
    drawRegionsT<SegmentEPDrawer, Contexter>(sd, ctx, tog, regions, mark);
  }


  void
  GLTogerPainter::drawDarts(const girl::toger &tog, 
			    const girl::dartDeque &darts)
  {
    SegmentDrawer sd;
    Contexter ctx;
    drawDartsT<SegmentDrawer, Contexter>(sd, ctx, tog, darts);
  }

  void
  GLTogerPainter::drawDarts(const girl::toger &tog, 
			    const girl::dartDeque &darts, 
			    girl::toger::markFlags mark)
  {
    SegmentDrawer sd;
    Contexter ctx;
    drawDartsT<SegmentDrawer, Contexter>(sd, ctx, tog, darts, mark);

  }

  void
  GLTogerPainter::drawDarts(const girl::toger &tog, 
			    const girl::dartDeque &darts, 
			    const girl::euclideanPathSet &eps)
  {
    SegmentEPDrawer sd(eps);
    Contexter ctx;
    drawDartsT<SegmentEPDrawer, Contexter>(sd, ctx, tog, darts);
  }

  void
  GLTogerPainter::drawDarts(const girl::toger &tog, 
			    const girl::dartDeque &darts, 
			    const girl::euclideanPathSet &eps, 
			    girl::toger::markFlags mark)
  {
    SegmentEPDrawer sd(eps);
    Contexter ctx;
    drawDartsT<SegmentEPDrawer, Contexter>(sd, ctx, tog, darts, mark);
  }


  void
  GLTogerPainter::drawFilledRegions(const girl::toger &tog, 
				    const girl::regionDeque &regions)
  {
#ifdef STATS_PAINTER
    STATS_number_triangles = 0;
#endif

#ifdef TIMING_PAINTER
    girl::Timer t;
    t.start();
#endif

    //CODE DUPLICATION

    //OPTIM:
    //If all regions are selected, we can avoid to draw each region individually:
    // we can draw only a big rectangle !
    // [Even if tog.numRegions() is O(N), this test seems a good optimization.
    //  First tog.numRegions() is O(N) but it is quite fast (no allocation).
    //  Besides, when we do the first split, we often have lots of regions and we will draw all of them filled...]

    if (regions.size() == tog.numRegions()) {
      
      glPushMatrix();
      glTranslatef(1, 1, 0.0f);

      glBegin(GL_QUADS);
      glVertex3f(-1, -1, 0.0); 
      glVertex3f(tog.width()-1, -1, 0.0); 
      glVertex3f(tog.width()-1, tog.height()-1, 0.0); 
      glVertex3f(-1, tog.height()-1, 0.0); 
      glEnd();

      glPopMatrix();
      
    }
    else {

#ifndef COMPUTE_MINIMUM_FILL

      girl::regionDeque::const_iterator it = regions.begin();
      const girl::regionDeque::const_iterator itEnd = regions.end();
      if (it != itEnd) {
	glEnable(GL_STENCIL_TEST);

	glPushMatrix();
	glTranslatef(1, 1, 0.0f);

	polygonStencilWithBBoxDrawer psdbb;
	polygonStencilDrawer psd;
	RegionTriangleFanDrawer<polygonStencilWithBBoxDrawer, polygonStencilDrawer> dr(psdbb, psd);

	for ( ; it != itEnd; ++it)
	  P_drawFilledRegion_aux(tog, *it, dr);

	glPopMatrix();
    
	glDisable(GL_STENCIL_TEST);
      }

#else

      if (! regions.empty()) {
	glEnable(GL_STENCIL_TEST);

	glPushMatrix();
	glTranslatef(1, 1, 0.0f);

	RegionInfoHash h;
	getRegionsAndHoles(tog, regions, h);

	polygonStencilWithBBoxDrawer psdbb;
	polygonStencilDrawer psd;
	RegionTriangleFanDrawerHash<polygonStencilWithBBoxDrawer, polygonStencilDrawer> dr(psdbb, psd);

	RegionInfoHash::const_iterator it = h.begin();
	const RegionInfoHash::const_iterator itEnd = h.end();
	for ( ; it != itEnd; ++it) {
	  if (it->second.toDraw) {
	    dr.setHoles(it->second.holes);
	    P_drawFilledRegion_aux(tog, it->first, dr);
	  }
	}

	glPopMatrix();
    
	glDisable(GL_STENCIL_TEST);
      }

#endif
    }

#ifdef TIMING_PAINTER
    const size_t time = t.stop();
    std::cerr<<"GLTogerPainter::drawFilledRegions(): time="<<time<<"ms"<<std::endl;
#endif

#ifdef STATS_PAINTER
    std::cerr<<"GLTogerPainter::drawFilledRegions(): "<<STATS_number_triangles
	     <<" triangles"<<std::endl;
#endif
  }

  void
  GLTogerPainter::drawFilledRegion(const girl::toger &tog, girl::region r)
  {
#ifdef STATS_PAINTER
    STATS_number_triangles = 0;
#endif

#ifdef TIMING_PAINTER
    girl::Timer t;
    t.start();
#endif

    glEnable(GL_STENCIL_TEST);

    glPushMatrix();
    glTranslatef(1, 1, 0.0f);

    polygonStencilWithBBoxDrawer psdbb;
    polygonStencilDrawer psd;
    RegionTriangleFanDrawer<polygonStencilWithBBoxDrawer, polygonStencilDrawer> dr(psdbb, psd);

    P_drawFilledRegion_aux(tog, r, dr);

    glPopMatrix();

    glDisable(GL_STENCIL_TEST);

#ifdef TIMING_PAINTER
    const size_t time = t.stop();
    std::cerr<<"GLTogerPainter::drawFilledRegion(): time="<<time<<"ms"<<std::endl;
#endif

#ifdef STATS_PAINTER
    std::cerr<<"GLTogerPainter::drawFilledRegion(): "<<STATS_number_triangles
	     <<" triangles"<<std::endl;
#endif
  }

