/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef GAUSSIANPYRAMIDRXYCSETTINGS_HPP
#define GAUSSIANPYRAMIDRXYCSETTINGS_HPP

#include "PyramidSettingsInterface.hpp"
#include "GaussianPyramidWidget.hpp"
#include <ise/iseGaussianPyramid.hpp>

class GaussianPyramidSettings : public PyramidSettingsInterface {
  
public:
  
  GaussianPyramidSettings() 
    : PyramidSettingsInterface()
    , m_numLevel()
    , m_divider()
  {
  }

  virtual void configureWidget(PyramidWidget *widget) const
  {
    static_cast<GaussianPyramidWidget *>(widget)->setParameters(m_numLevel, m_divider);
  }
  
  virtual void getWidgetConfig(const PyramidWidget *widget)
  {
    static_cast<const GaussianPyramidWidget *>(widget)->getParameters(m_numLevel, m_divider);
  }

  virtual void configureAlgo(girl::shared_ptr<ise::Pyramid> algo) const
  {
    girl::shared_ptr<ise::GaussianPyramid> algorithm = girl::static_pointer_cast<ise::GaussianPyramid>(algo);
    algorithm->setParameters(m_numLevel, m_divider);
  }
  
  virtual girl::shared_ptr<PyramidSettingsInterface> clone() const 
  {
    girl::shared_ptr<GaussianPyramidSettings> p(new GaussianPyramidSettings);

    *p = *this;

    return p;
  }

  virtual bool isValid() const
  {
    return (m_numLevel > 0 && m_divider > 0);
  }

private:
  
  int m_numLevel;
  int m_divider;
};

#endif /* ! GAUSSIANPYRAMIDSETTINGS_HPP */
