/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "Histogram1DThresholdsWidget.hpp"

#include <cassert>
#include <QBoxLayout>
#include <QLabel>
#include <QPushButton>
#include <QMouseEvent>

#include <ise/iseHistogram1D.hpp>



Histogram1DThresholdsWidget::Histogram1DThresholdsWidget(QWidget *parent, 
							 QRgb firstBackgroundColor)
  : Histogram1DWidgetBase(parent)
{
  buildGUI(firstBackgroundColor);
  
}


Histogram1DThresholdsWidget::Histogram1DThresholdsWidget(const ise::Histogram1D &histo, 
							 QWidget *parent, 
							 QRgb firstBackgroundColor)
  : Histogram1DWidgetBase(parent)
{
  buildGUI(firstBackgroundColor);

  setHistogram(histo);
}

Histogram1DThresholdsWidget::Histogram1DThresholdsWidget(const ise::SparseHistogram1DD &histo, 
							 QWidget *parent, 
							 QRgb firstBackgroundColor)
  : Histogram1DWidgetBase(parent)
{
  buildGUI(firstBackgroundColor);

  setHistogram(histo);

}

void
Histogram1DThresholdsWidget::buildGUI(QRgb firstBackgroundColor)
{
  Histogram1DWidgetBase::buildGUI(new Histogram1DThresholdsView(this, firstBackgroundColor)); 

  connect(static_cast<Histogram1DThresholdsView *>(m_view), SIGNAL(thresholdRemoved(size_t)), this, SIGNAL(thresholdRemoved(size_t)));

  connect(static_cast<Histogram1DThresholdsView *>(m_view), SIGNAL(thresholdChanged(size_t)), this, SIGNAL(thresholdChanged(size_t)));
}



void
Histogram1DThresholdsWidget::setTresholdsConstrainedToHistogram(bool b)
{
  static_cast<Histogram1DThresholdsView *>(m_view)->setTresholdsConstrainedToHistogram(b);
}

bool
Histogram1DThresholdsWidget::tresholdsConstrainedToHistogram() const
{
  return static_cast<Histogram1DThresholdsView *>(m_view)->tresholdsConstrainedToHistogram();
}

void
Histogram1DThresholdsWidget::setTresholdsConstrainedToEachOther(bool b)
{
  static_cast<Histogram1DThresholdsView *>(m_view)->setTresholdsConstrainedToEachOther(b);
}

bool
Histogram1DThresholdsWidget::tresholdsConstrainedToEachOther() const
{
  return static_cast<Histogram1DThresholdsView *>(m_view)->tresholdsConstrainedToEachOther();
}

void
Histogram1DThresholdsWidget::setDisplayBackgrounds(bool b)
{
  static_cast<Histogram1DThresholdsView *>(m_view)->setDisplayBackgrounds(b);
}

bool
Histogram1DThresholdsWidget::displayBackgrounds() const
{
  return static_cast<Histogram1DThresholdsView *>(m_view)->displayBackgrounds();
}

void
Histogram1DThresholdsWidget::setBackgroundsPosition(Histogram1DThresholdsView::BackroundPosition p)
{
  static_cast<Histogram1DThresholdsView *>(m_view)->setBackgroundsPosition(p);
}

Histogram1DThresholdsView::BackroundPosition
Histogram1DThresholdsWidget::backgroundsPosition()
{
  return static_cast<Histogram1DThresholdsView *>(m_view)->backgroundsPosition();
}

void
Histogram1DThresholdsWidget::setFirstBackgroundColor(QRgb color)
{
  static_cast<Histogram1DThresholdsView *>(m_view)->setFirstBackgroundColor(color);
}

QRgb
Histogram1DThresholdsWidget::firstBackgroundColor() const
{
  return static_cast<Histogram1DThresholdsView *>(m_view)->firstBackgroundColor();
}


void
Histogram1DThresholdsWidget::setHistogram(const ise::Histogram1D &histo)
{
  Histogram1DWidgetBase::setHistogram(histo);
}

void
Histogram1DThresholdsWidget::setHistogram(const ise::SparseHistogram1DD &histo)
{
  Histogram1DWidgetBase::setHistogram(histo);
}

void
Histogram1DThresholdsWidget::setMinMax(double min, double max)
{
  static_cast<Histogram1DThresholdsView *>(m_view)->setMinMax(min, max);
}


size_t
Histogram1DThresholdsWidget::addThreshold(double v, QRgb color)
{
  return static_cast<Histogram1DThresholdsView *>(m_view)->addThreshold(v, color);
}

size_t
Histogram1DThresholdsWidget::addThreshold(QRgb color)
{
  return static_cast<Histogram1DThresholdsView *>(m_view)->addThreshold(color);
}

void
Histogram1DThresholdsWidget::removeThreshold(size_t thresholdID)
{
  static_cast<Histogram1DThresholdsView *>(m_view)->removeThreshold(thresholdID);
}

void
Histogram1DThresholdsWidget::removeAllThresholds()
{
  static_cast<Histogram1DThresholdsView *>(m_view)->removeAllThresholds();
}

void
Histogram1DThresholdsWidget::thresholds(std::vector<double> &thresholds) const
{
  static_cast<Histogram1DThresholdsView *>(m_view)->thresholds(thresholds);
}

void
Histogram1DThresholdsWidget::thresholdColors(std::vector<QRgb> &colors) const
{
  static_cast<Histogram1DThresholdsView *>(m_view)->thresholdColors(colors);
}

double
Histogram1DThresholdsWidget::setValue(size_t thresholdID, double k)
{
  return static_cast<Histogram1DThresholdsView *>(m_view)->setValue(thresholdID, k);
}

double
Histogram1DThresholdsWidget::value(size_t thresholdID) const
{
  return static_cast<Histogram1DThresholdsView *>(m_view)->value(thresholdID);
}

void
Histogram1DThresholdsWidget::setColor(size_t thresholdID, QRgb color)
{
  static_cast<Histogram1DThresholdsView *>(m_view)->setColor(thresholdID, color);
}

QRgb
Histogram1DThresholdsWidget::color(size_t thresholdID) const
{
  return static_cast<Histogram1DThresholdsView *>(m_view)->color(thresholdID);
}

void
Histogram1DThresholdsWidget::valueAndBoundaries(size_t thresholdID, double &k, double &kMin, double &kMax)
{
  static_cast<Histogram1DThresholdsView *>(m_view)->valueAndBoundaries(thresholdID, k, kMin, kMax);
}

double
Histogram1DThresholdsWidget::minDiff() const
{
  return static_cast<Histogram1DThresholdsView *>(m_view)->minDiff();
}
