/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef HISTOGRAM1DVIEW_HPP
#define HISTOGRAM1DVIEW_HPP

#include <limits>

#include <girl/girlCommon.hpp>//GIRL_API

#include <ise/iseSparseHistogram1D.hpp>

#include <QWidget>

namespace ise {
  class Histogram1D;
}



class GIRL_API Histogram1DView : public QWidget
{
  Q_OBJECT

public:

  explicit Histogram1DView(QWidget *parent = NULL);

  Histogram1DView(const ise::Histogram1D &histo, QWidget *parent = NULL);
  
  Histogram1DView(const ise::SparseHistogram1DD &histo, QWidget *parent = NULL);
  

  /**
   * Set a new Histogram1D.
   * Will do a setMinMax().
   */
  virtual void setHistogram(const ise::Histogram1D &histo);

  virtual void setHistogram(const ise::SparseHistogram1DD &histo);

  virtual const ise::SparseHistogram1DD &getHistogram() const;
  
  //template <typename K, typename T> void setHistogram(const ise::SparseHistogram1D<K, T> &h);



  /**
   * Change aboslute min & max of histogram view,
   * without changing the underlying Histogram1D.
   * It is useful to move the levels outside the underlying Histogram1D limits
   * [when we change of color space, we can then transform the levels, before setting a new histogram]
   */
  virtual void setMinMax(double min, double max);
  
  inline double absoluteMin() const { return m_kMin; }

  inline double absoluteMax() const { return m_kMax; }
  
  /**
   * @brief Get values (k, f) that would correspond to position @a pos.
   *
   */
  QPointF getValuesFromPos(const QPoint &pos);

  /**
   * @brief zoom view around center of current view.
   *
   * @a f is k times the previous zoom factor.
   */
  //void setZoomFactor(qreal f);

  /**
   * @brief zoom view around point @ pt of current view.
   *
   * @a f is k times the previous zoom factor.
   */
  //void setZoomFactor(qreal f, const QPointF &pt);


public slots:

  void resetView();

protected:

  void build();

  virtual QSize sizeHint() const;

  virtual void paintEvent(QPaintEvent *event);

  virtual void mousePressEvent(QMouseEvent *event); 
  virtual void mouseMoveEvent(QMouseEvent *event); 
  virtual void mouseReleaseEvent(QMouseEvent *event); 

  virtual void keyPressEvent(QKeyEvent *event); 

  virtual void resizeEvent(QResizeEvent * event);

  //virtual void wheelEvent(QWheelEvent *event);

  //void setZoomFactor_aux(qreal f, qreal x, qreal y);


  void drawBar(QPainter &painter, double k, double f);

  void populate();
  
  void updateBoundaries();

  void getTransforms();



  /**
   * Transform key in histogram into x-coord in view.
   */
  double transformK(double k) const;

  /**
   * Transform frequency in histogram into y-coord in view.
   */
  double transformF(double f) const;

  /**
   * Transform x-coord in view into k in histogram.
   */
  double transformFromViewXtoK(double x) const;

  /**
   * Transform y-coord in view into f in histogram.
   */
  double transformFromViewYtoF(double y) const;

  void tryMoveLeft();
  void tryMoveRight();
  void tryMoveDown();
  void tryMoveUp();
  void tryZoomIn();
  void tryZoomOut();

  void updateExtremaView();
  

  //double yMin() const;

  //double yMax() const;

  /*
  template <typename K, typename T> 
    void populate(const ise::SparseHistogram1D<K, T> &h);

  template <typename K, typename T> 
    void getBoundaries(const ise::SparseHistogram1D<K, T> &h, double &kMin, double &kMax, double &fMin, double &fMax);

  template <typename K, typename T> 
    void addBars(const ise::SparseHistogram1D<K, T> &h);
  */
  



protected:  

  ise::SparseHistogram1DD m_histo;

  /*
  qreal m_zoomFactor;
  qreal m_x0;
  qreal m_y0;
  bool m_usePanZoom;

  */

  static const int HEIGHT;
  static const int WIDTH;

  double m_kMin;
  double m_kMax;
  double m_fMin;
  double m_fMax;
  
  double m_kMinView;
  double m_kMaxView;
  double m_fMinView;
  double m_fMaxView;

  double m_xRatio;
  double m_yRatio;
  double m_x0; //correspond to m_kMinView in view
  double m_y0; //correspond to m_fMinView in view
  double m_x1; //correspond to m_kMaxView in view
  double m_y1; //correspond to m_fMaxView in view

  QPoint m_clickPos;
  QPoint m_currPos;
  bool m_isMoving;


  bool m_updateRatios;


};

/*
template <typename K, typename T> 
void Histogram1DView::setHistogram(const ise::SparseHistogram1D<K, T> &h)
{
  populate(h);
}

template <typename K, typename T> 
void Histogram1DView::populate(const ise::SparseHistogram1D<K, T> &h)
{
  m_kMin = 0.0;
  m_kMax = 0.0;
  m_fMax = 0.0;
  
  double kMin, kMax, fMin, fMax;
  getBoundaries(h, kMin, kMax, fMin, fMax);
  
  if (kMax < kMin || fMax < fMin)
    return; //?

  m_kMin = kMin;
  m_kMax = kMax;
  m_fMin = fMin;
  m_fMax = fMax;

  getTransforms(kMin, kMax, fMin, fMax); //set m_xRatio, m_yRatio, 

  buildAxes(kMin, kMax, fMin, fMax); //use m_xRatio, m_yRatio, 

  addBars(h); //use m_xRatio, m_yRatio, 
}

template <typename K, typename T> 
void Histogram1DView::getBoundaries(const ise::SparseHistogram1D<K, T> &h, 
				  double &kMin, double &kMax, double &fMin, double &fMax)
{
  K min_k = std::numeric_limits<K>::max();
  K max_k = std::numeric_limits<K>::min();
  T min_f = std::numeric_limits<T>::max();
  T max_f = std::numeric_limits<T>::min();
  typename ise::SparseHistogram1D<K, T>::const_iterator it = h.begin();
  const typename ise::SparseHistogram1D<K, T>::const_iterator itEnd = h.end();
  for ( ; it != itEnd; ++it) {
    const K k = it->first;
    if (k < min_k)
      min_k = k;
    if (k > max_k)
      max_k = k;

    const T f = it->second;
    if (f < min_f)
      min_f = f;
    if (f > max_f)
      max_f = f;
  }

  kMin = static_cast<double>(min_k);
  kMax = static_cast<double>(max_k);
  fMin = static_cast<double>(min_f);
  fMax = static_cast<double>(max_f);
}

template <typename K, typename T> 
void Histogram1DView::addBars(const ise::SparseHistogram1D<K, T> &h)
{
  typename ise::SparseHistogram1D<K, T>::const_iterator it = h.begin();
  const typename ise::SparseHistogram1D<K, T>::const_iterator itEnd = h.end();
  for ( ; it != itEnd; ++it) {
    addBar(static_cast<double>(it->first), static_cast<double>(it->second));
  }

}
*/

#endif /* ! HISTOGRAM1DVIEW_HPP */
