/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "Histogram1DWidgetBase.hpp"

#include <cassert>
#include <QBoxLayout>
#include <QLabel>
#include <QPushButton>
#include <QMouseEvent>

#include <ise/iseHistogram1D.hpp>

#include "Histogram1DView.hpp"


Histogram1DWidgetBase::Histogram1DWidgetBase(QWidget *parent)
  : QWidget(parent),
    m_view(NULL),
    m_pos(NULL)
{
  
}


void
Histogram1DWidgetBase::buildGUI(Histogram1DView *v)
{

  m_view = v;
  
  m_pos = new QLabel(tr(""), this);
  QPushButton *resetButton = new QPushButton(tr("1"), this);
  resetButton->setMaximumWidth(25);
  resetButton->setToolTip(tr("Reset zoom"));

  QHBoxLayout *hL = new QHBoxLayout;
  hL->addWidget(m_pos);
  hL->addWidget(resetButton);
  
  QVBoxLayout *vL = new QVBoxLayout;
  vL->addWidget(m_view);
  vL->addLayout(hL);

  setLayout(vL);

  vL->setMargin(0);

  connect(resetButton, SIGNAL(released()), m_view, SLOT(resetView()));

  m_view->installEventFilter(this); //to capture mouse moves.

  setMouseTracking(true); //just to have label disabled when not in m_histoView

}

void
Histogram1DWidgetBase::mouseMoveEvent(QMouseEvent *mouseEvent)
{
  const QPoint pos = mouseEvent->pos();
  if (! m_view->frameGeometry().contains(pos)) {
    m_pos->setText(tr(""));
    m_pos->setEnabled(false);
  }
}

bool
Histogram1DWidgetBase::eventFilter(QObject *object, QEvent *event)
{
  if (event->type() == QEvent::MouseMove) {
    if (object == m_view) {
      m_pos->setEnabled(true);
      QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
      const QPointF values = m_view->getValuesFromPos(mouseEvent->pos());
      m_pos->setText(tr("%1, %2").arg(values.x()).arg(values.y()));
    }
    else {
      m_pos->setText(tr(""));
      m_pos->setEnabled(false);
    }
  }
  return false;

}

void
Histogram1DWidgetBase::setHistogram(const ise::Histogram1D &histo)
{
  assert(m_view);
  m_view->setHistogram(histo);
}

void
Histogram1DWidgetBase::setHistogram(const ise::SparseHistogram1DD &histo)
{
  assert(m_view);
  m_view->setHistogram(histo);
}

const ise::SparseHistogram1DD &
Histogram1DWidgetBase::getHistogram() const
{
  assert(m_view);
  return m_view->getHistogram();  
}

