/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "ImageFeatureChoiceWidget.hpp"

#include <cassert>

#include "PluginRegistry.hpp"
#include "MainView.hpp"

#include <QLabel>
#include <QComboBox>
#include <QHBoxLayout>

#include <ise/iseSegmentationContext.hpp>
#include <ise/iseImageFeatureColorimetricComputer.hpp>
#include <ise/iseImageFeatureGeometricComputer.hpp>
#include <ise/iseImageFeatureTextureComputer.hpp>
#include <ise/iseImageFeatureBinaryComputer.hpp>
#include <ise/iseImageFeatureCompositionComputer.hpp>

#include <list>
#include <set>
#include <string>

#include "InstantFeatureDialog.hpp"

ImageFeatureChoiceWidget::ImageFeatureChoiceWidget(MainView *mainView, FeatureComputerGUIInfoPtr featureComputerGUIInfo, QWidget *parent)
  : QWidget(parent)
  , m_mainView(mainView)
  , m_featureComputerGUIInfo(featureComputerGUIInfo)
  , m_imageFeatureCombo(NULL)
  , m_imageFeatureComputerUriCombo(NULL)
{
  build();
}

void
ImageFeatureChoiceWidget::build()
{
  QHBoxLayout *hbox = new QHBoxLayout;
  QLabel *l = new QLabel(tr("Image Feature"), this);
  m_imageFeatureCombo = new QComboBox(this);
  m_imageFeatureComputerUriCombo = new QComboBox(this);

  populateImageFeatureComputerUri();
  m_imageFeatureComputerUriCombo->setCurrentIndex(0);

  populateImageFeatureTitle(m_imageFeatureComputerUriCombo->currentText());



  connect(m_imageFeatureComputerUriCombo, SIGNAL(currentIndexChanged(QString)),
	    this, SLOT(populateImageFeatureTitle(QString)));

  connect(m_imageFeatureCombo, SIGNAL(currentIndexChanged(int)),
	  this, SLOT(changeImageFeature(int)));
  connect(m_imageFeatureCombo, SIGNAL(activated(int)),
	  this, SLOT(changeImageFeature(int)));



  hbox->addWidget(l);
  hbox->addWidget(m_imageFeatureComputerUriCombo);
  hbox->addWidget(m_imageFeatureCombo);
  hbox->addStretch();
  hbox->setMargin(0);
  this->setLayout(hbox);
}

void
ImageFeatureChoiceWidget::populateImageFeatureComputerUri()
{
  // std::vector<FeatureDataPtr> list;
  // m_featureComputerGUIInfo->getFeatureComputersInfo<ise::ImageFeatureComputer>(list);

  // m_imageFeatureComputerUriCombo->clear();

  // std::set<std::string> imageFeatureComputerUri;
  // size_t sizeData = list.size();
  // for (size_t i = 0 ; i < sizeData ; ++i) {
  //   ise::FeatureComputerPtr algo = list[i]->createAlgo(m_mainView->model()->getSegContextPtr());
    
  //   imageFeatureComputerUri.insert(girl::static_pointer_cast<ise::ImageFeatureComputer>(algo)->imageFeatureComputerUri());
  // }

  // std::set<std::string>::iterator it = imageFeatureComputerUri.begin();
  // const std::set<std::string>::const_iterator itEnd = imageFeatureComputerUri.end();
  // for ( ; it != itEnd ; ++it) {
  //   m_imageFeatureComputerUriCombo->addItem((*it).c_str());
  // }
  
  m_imageFeatureComputerUriCombo->blockSignals(true);
  m_imageFeatureComputerUriCombo->clear();
  m_imageFeatureComputerUriCombo->addItem("Colorimetric");
  m_imageFeatureComputerUriCombo->addItem("Colorimetric#Binary");
  m_imageFeatureComputerUriCombo->addItem("Geometric");
  m_imageFeatureComputerUriCombo->addItem("Texture");
  m_imageFeatureComputerUriCombo->addItem("Composition");
  m_imageFeatureComputerUriCombo->blockSignals(false);
}


void
ImageFeatureChoiceWidget::populateImageFeatureTitle(QString imageFeatureComputerUri) 
{
  m_imageFeatureCombo->clear();

  if (imageFeatureComputerUri.isEmpty())
    return;

  std::vector<FeatureDataPtr> list;

  if (imageFeatureComputerUri == "Colorimetric")
    m_featureComputerGUIInfo->getFeatureComputersInfo<ise::ImageFeatureColorimetricComputer>(list);
  
  if (imageFeatureComputerUri == "Texture")
    m_featureComputerGUIInfo->getFeatureComputersInfo<ise::ImageFeatureTextureComputer>(list);

  if (imageFeatureComputerUri == "Geometric")
    m_featureComputerGUIInfo->getFeatureComputersInfo<ise::ImageFeatureGeometricComputer>(list);

  if (imageFeatureComputerUri == "Colorimetric#Binary")
    m_featureComputerGUIInfo->getFeatureComputersInfo<ise::ImageFeatureBinaryComputer>(list);

  if (imageFeatureComputerUri == "Composition")
    m_featureComputerGUIInfo->getFeatureComputersInfo<ise::ImageFeatureCompositionComputer>(list);

  


  std::vector<FeatureDataPtr>::iterator it = list.begin();
  const std::vector<FeatureDataPtr>::iterator itEnd = list.end();
  for ( ; it != itEnd ; ++it) {
    m_imageFeatureCombo->addItem((*it)->title().c_str());
  }

  m_imageFeatureCombo->blockSignals(true);
  m_imageFeatureCombo->addItem("new feature...");
  m_imageFeatureCombo->blockSignals(false);

  if (m_imageFeatureCombo->count() == 1 && m_imageFeatureCombo->currentText() == "new feature...")
    m_imageFeatureCombo->setCurrentIndex(-1);
  else
    m_imageFeatureCombo->setCurrentIndex(0);
}

void
ImageFeatureChoiceWidget::changeImageFeature(int index)
{
  if (m_imageFeatureCombo->currentText() == "new feature...") {
    FeatureDataPtr data;

    QString imageFeatureComputerUri = m_imageFeatureComputerUriCombo->currentText();

    QIsePluginList<FeaturePlugin*> dest;

    if (imageFeatureComputerUri == "Colorimetric") {
      QIsePluginList<ImageFeatureColorimetricPlugin*> list = PluginRegistry::getImageFeatureColorimetricPlugin();
      toFeaturePluginList<ImageFeatureColorimetricPlugin>(list, dest);
    }
    
    if (imageFeatureComputerUri == "Texture") {
      QIsePluginList<ImageFeatureTexturePlugin*> list = PluginRegistry::getImageFeatureTexturePlugin();
      toFeaturePluginList<ImageFeatureTexturePlugin>(list, dest);
    }
    
    if (imageFeatureComputerUri == "Geometric") {
      QIsePluginList<ImageFeatureGeometricPlugin*> list = PluginRegistry::getImageFeatureGeometricPlugin();
      toFeaturePluginList<ImageFeatureGeometricPlugin>(list, dest);
     }
    
    if (imageFeatureComputerUri == "Colorimetric#Binary") {
      QIsePluginList<ImageFeatureBinaryPlugin*> list = PluginRegistry::getImageFeatureBinaryPlugin();
      toFeaturePluginList<ImageFeatureBinaryPlugin>(list, dest);
    }
    
    if (imageFeatureComputerUri == "Composition") {
      QIsePluginList<ImageFeatureCompositionPlugin*> list = PluginRegistry::getImageFeatureCompositionPlugin();
      toFeaturePluginList<ImageFeatureCompositionPlugin>(list, dest);
    }

    InstantFeatureDialog *d = new InstantFeatureDialog(m_mainView, m_featureComputerGUIInfo, dest, data, this);

    int returnCode = d->exec();
    if (returnCode == QDialog::Accepted) {
      rebuild(m_featureComputerGUIInfo);
      setImageFeature(data->title().c_str());
      emit imageFeatureChanged();
    }
    else {
      int pos = -1;
      if (m_imageFeatureCombo->count() > 1)
	pos = 0;
      m_imageFeatureCombo->setCurrentIndex(pos);
      //index = pos;
    }
  }

  if (m_imageFeatureCombo->currentText() != "new feature...") {
    emit imageFeatureChanged();
  }
}

size_t
ImageFeatureChoiceWidget::size() const {
  return m_imageFeatureCombo->count();
}

void
ImageFeatureChoiceWidget::clear() 
{
  m_imageFeatureCombo->clear();
  m_imageFeatureComputerUriCombo->clear();
}

void
ImageFeatureChoiceWidget::setImageFeature(const QString &title)
{
  std::vector<FeatureDataPtr> list;
  m_featureComputerGUIInfo->getFeatureComputersInfo<ise::ImageFeatureComputer>(list);

  const size_t sizeData = list.size();

  for ( size_t i = 0; i < sizeData ; ++i) {
    if (list[i]->title() == title.toStdString()) {
 
      ise::FeatureComputerPtr algo = list[i]->createAlgo(m_mainView->model()->getSegContextPtr());
      std::string imageFeatureComputerUri = girl::static_pointer_cast<ise::ImageFeatureComputer>(algo)->imageFeatureComputerUri();
      
      const int indexComputerUri = m_imageFeatureComputerUriCombo->findText(imageFeatureComputerUri.c_str());
      m_imageFeatureComputerUriCombo->setCurrentIndex(indexComputerUri);
      
      const int indexComputer = m_imageFeatureCombo->findText(title);
      m_imageFeatureCombo->setCurrentIndex(indexComputer);
      break;
    }
  }
}

int 
ImageFeatureChoiceWidget::nbChannelsForImageFeature() const
{
  const std::string title = m_imageFeatureCombo->currentText().toStdString();
  if (title.empty())
    return 0;

  std::vector<FeatureDataPtr> list;
  m_featureComputerGUIInfo->getFeatureComputersInfo<ise::ImageFeatureComputer>(list);

  const size_t sizeData = list.size();
  size_t i = 0;
  for ( ; i < sizeData ; ++i) {
    if (list[i]->title() == title)
      break;
  }
  
  return list[i]->createAlgo(m_mainView->model()->getSegContextPtr())->getFeatureDim();

}

void
ImageFeatureChoiceWidget::featureDimDescription(std::vector<std::string> &dimDesc) const
{
  std::string title = m_imageFeatureCombo->currentText().toStdString();

  std::vector<FeatureDataPtr> list;
  m_featureComputerGUIInfo->getFeatureComputersInfo<ise::ImageFeatureComputer>(list);

  size_t sizeData = list.size();
  size_t i = 0;
  for ( ; i < sizeData ; ++i) {
    if (list[i]->title() == title)
      break;
  }
  
  list[i]->createAlgo(m_mainView->model()->getSegContextPtr())->getDimDescription(dimDesc);

}

void
ImageFeatureChoiceWidget::rebuild(FeatureComputerGUIInfoPtr featureComputerGUIInfo)
{
  m_featureComputerGUIInfo = featureComputerGUIInfo;
  populateImageFeatureComputerUri();
}


void
ImageFeatureChoiceWidget::setImageFeatureComputerPtr(ise::FeatureComputerPtr algo)
{
  if (algo) {
    FeatureDataPtr data = m_featureComputerGUIInfo->getInfos(algo);
   
    if (data) 
      setImageFeature(data->title().c_str());
  }
}

ise::FeatureComputerPtr
ImageFeatureChoiceWidget::imageFeatureComputerPtr() const
{
  std::string title = m_imageFeatureCombo->currentText().toStdString();

  ise::FeatureComputerPtr algo = m_featureComputerGUIInfo->getAlgo(title);

  return algo;
}
