/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#ifndef IMAGEFEATURECHOICEWIDGET_HPP
#define IMAGEFEATURECHOICEWIDGET_HPP

#include <cstring>

#include <girl/girlCommon.hpp>

#include <QVector>
#include <QWidget>

#include <qiseTools/FeatureData.hpp>
#include <qiseTools/FeatureComputerGUIInfo.hpp>

#include "QIsePluginList.hpp"

class MainView;

class QComboBox;
class QLabel;

class GIRL_API ImageFeatureChoiceWidget : public QWidget
{
  Q_OBJECT

public:
  /**
   * @brief Constructor.
   */
  ImageFeatureChoiceWidget(MainView *mainView, FeatureComputerGUIInfoPtr featureComputerGUIInfo, QWidget *parent = NULL);

  void setImageFeature(const QString &title);

  //QString getImageFeature() const;
  
  void setImageFeatureComputerPtr(ise::FeatureComputerPtr algo);

  ise::FeatureComputerPtr imageFeatureComputerPtr() const;

  size_t size() const;

  void clear();

  int nbChannelsForImageFeature() const;

  void featureDimDescription(std::vector<std::string> &dimDesc) const;

  void rebuild(FeatureComputerGUIInfoPtr featureComputerGUIInfo);

signals:
  
  void imageFeatureChanged();

protected slots:

  void changeImageFeature(int index);
  void populateImageFeatureTitle(QString imageFeatureComputerUri);			    
  void populateImageFeatureComputerUri();

protected:

  void build();

 private:
  
  template <typename T>
    void toFeaturePluginList(const QIsePluginList<T*> &src, QIsePluginList<FeaturePlugin*> &dest)
  {
    dest.clear();
    typename QIsePluginList<T*>::const_iterator it = src.begin();
    typename QIsePluginList<T*>::const_iterator itEnd = src.end();
    for ( ; it != itEnd ; ++it)
      dest.append(static_cast<FeaturePlugin*>(*it));
  }

protected:

  MainView *m_mainView;
  FeatureComputerGUIInfoPtr m_featureComputerGUIInfo;

  QComboBox *m_imageFeatureCombo;
  QComboBox *m_imageFeatureComputerUriCombo;
};

#endif /* IMAGEFEATURECHOICEWIDGET_HPP */
