/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "ImageFeatureColorSpaceWidget.hpp"

#include <QString>
#include <QLabel>
#include <QGroupBox>
#include <QGridLayout>
#include <QComboBox>

#include <qiseTools/ColorSpaceChoiceWidget.hpp>
#include <qiseTools/ColorSpaceRegistry.hpp>

ImageFeatureColorSpaceWidget::ImageFeatureColorSpaceWidget(MainView *mainView, FeatureComputerGUIInfoPtr featureComputerGUIInfo, QString name, ImageFeaturePlugin *imageFeaturePlugin, QWidget *parent)
  : ImageFeatureWidgetInterface(mainView, featureComputerGUIInfo, name, imageFeaturePlugin, parent)
  , m_cscw(NULL)
  
{
  setAttribute(Qt::WA_DeleteOnClose);

  build();
}

void
ImageFeatureColorSpaceWidget::build()
{
  assert(m_cscw == NULL);
  m_cscw = new ColorSpaceChoiceWidget(this);

  QVBoxLayout *layout = new QVBoxLayout();
  layout->setAlignment(Qt::AlignTop);

  layout->addWidget(m_cscw);

  setLayout(layout);
}

bool 
ImageFeatureColorSpaceWidget::isValid()
{
  bool b = m_cscw->getColorSpace() >= 0;
  return b;
}

void
ImageFeatureColorSpaceWidget::setParameters(int colorSpace)
{
  m_cscw->setColorSpace(colorSpace);
}

void
ImageFeatureColorSpaceWidget::getParameters(int &colorSpace) const
{
  colorSpace = m_cscw->getColorSpace();
}	 

