/*
  Copyright (C) 2005-2011 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef IMAGEFEATURECOMPOSITIONSETTINGS_HPP
#define IMAGEFEATURECOMPOSITIONSETTINGS_HPP

#include <girl/girlCommon.hpp>
#include "ImageFeatureCompositionWidget.hpp"
#include "SettingsInterfaceTemplate.hxx"
#include <ise/iseImageFeatureComputer.hpp>


typedef SettingsInterfaceTemplate<ImageFeatureCompositionWidget, ise::ImageFeatureComputer> ImageFeatureCompositionSettingsInterface;


class GIRL_API ImageFeatureCompositionSettings : public ImageFeatureCompositionSettingsInterface
{

 public:

  virtual void configureWidget(ImageFeatureCompositionWidget *widget) const
  {
    widget->setGUI(m_settings);
  }

  void getWidgetConfig(const ImageFeatureCompositionWidget *widget)
  {
    widget->getGUI(m_settings);
  }

  void configureAlgo(girl::shared_ptr<ise::ImageFeatureComputer> ) const
  {
  
  }

  virtual girl::shared_ptr<ImageFeatureCompositionSettingsInterface> clone() const
  {
    girl::shared_ptr<ImageFeatureCompositionSettings> p(new ImageFeatureCompositionSettings);

    *p = *this;

    assert(p->m_settings.size() == m_settings.size());
    const size_t s = m_settings.size();
    for (size_t i=0; i<s; ++i) {
      if (m_settings[i])
	p->m_settings[i] = girl::static_pointer_cast<ImageFeatureSettingsInterface>( m_settings[i]->clone() );
      else
	p->m_settings[i] = girl::shared_ptr<ImageFeatureSettingsInterface>();
    }
    return p;
  }
  
  virtual bool isValid() const 
  {
    const size_t s = m_settings.size();
    for (size_t i=0; i<s; ++i)
      if (m_settings[i] == NULL)
	return false;

    return true;    
  }


  void addSettings(girl::shared_ptr<ImageFeatureSettingsInterface> settings)
  {
    m_settings.push_back(settings);
  }

 private:

  std::vector< girl::shared_ptr<ImageFeatureSettingsInterface> > m_settings;

};

#endif /* ! IMAGEFEATURECOMPOSITIONTABSETTINGS_HPP */
