/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "ImageFeatureCompositionTabWidget.hpp"
#include "PluginRegistry.hpp"
#include "QIsePluginList.hpp"
#include "ImageFeatureCompositionGlobalWidget.hpp"
#include "ImageFeatureCompositionLinearCombinationWidget.hpp"

ImageFeatureCompositionTabWidget::ImageFeatureCompositionTabWidget(MainView *mainView,
								   FeatureComputerGUIInfoPtr featureComputerGUIInfo,
								   QString name,
								   QWidget *parent)
  : FeatureTabWidget(mainView, featureComputerGUIInfo, name, parent)
{
  setAttribute(Qt::WA_DeleteOnClose);

  build();
}

void
ImageFeatureCompositionTabWidget::build()
{
  QVBoxLayout *featuresLayout = new QVBoxLayout;
  featuresLayout->setAlignment(Qt::AlignTop);

  QIsePluginList<ImageFeatureCompositionPlugin*>::iterator it = PluginRegistry::getImageFeatureCompositionPlugin().begin();
  const QIsePluginList<ImageFeatureCompositionPlugin*>::iterator itEnd = PluginRegistry::getImageFeatureCompositionPlugin().end();
  for ( ; it != itEnd; ++it) {

    std::string name = (*it)->name().toStdString();

    FeatureListOnePluginWidget *widget = NULL;

    if (name == "Global") 
      widget = new ImageFeatureCompositionGlobalWidget(m_mainView, m_featureComputerGUIInfo, *it, this);
    
    if (name == "LinearCombination")
      widget = new ImageFeatureCompositionLinearCombinationWidget(m_mainView, m_featureComputerGUIInfo, *it, this);

    if (widget) {
      connect(widget);
      m_widgets.push_back(widget);
      featuresLayout->addWidget(m_widgets[m_widgets.size() - 1]);
    }
  }

  setLayout(featuresLayout);
}

void
ImageFeatureCompositionTabWidget::initWidget()
{
  size_t size = m_widgets.size();
  for (size_t i = 0 ; i < size ; ++i) {
    m_widgets[i]->initWidget();
  }
    
}
