/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef IMAGEFEATURECOMPOSITIONWIDGET_HPP
#define IMAGEFEATURECOMPOSITIONWIDGET_HPP

#include <girl/girlCommon.hpp>

#include <girl/girlHashFunc.hpp>
#include <girl/girlHashMap.hpp>

#include <QWidget>
#include <QListWidget>
#include <QString>
#include "MainView.hpp"
#include "ImageFeatureSettingsInterface.hpp"

#include <list>

class ImageFeatureCompositionSettings;

class ImageFeatureCompositionPlugin;

class GIRL_API ImageFeatureCompositionWidget : public QWidget 
{
  Q_OBJECT
 public:
  
  ImageFeatureCompositionWidget(MainView *mainView,  ImageFeatureCompositionPlugin *plugin, QWidget *parent = NULL);

  //void computeImageFeature();
  
  girl::shared_ptr<ImageFeatureCompositionSettings> createSettings() const;
  
  void setGUI(const std::vector< girl::shared_ptr<ImageFeatureSettingsInterface> > &m_settings);

  void getGUI(std::vector< girl::shared_ptr<ImageFeatureSettingsInterface> > &m_settings) const;

  void compute();

 protected:

  void build();

 private:
	      
  void insertItem(const std::string &title, bool exist = false);
  bool listComputeContain(const QString &title);

 public slots:

  void newImageFeature();
  void delImageFeature();

  void addItem(QString);
  void deleteItem(QString);
  void renameItem(QString, QString);

  void show(QString); 

 private:

  //virtual void controlItem(const std::vector<QString> &titles);
  int findInListToRename(const std::string &title) {
    size_t size = m_imageFeatureToRename.size();
    for (size_t i = 0 ; i < size ; ++i) {
      if (m_imageFeatureToRename[i].back() == title)
	return i;
    }
    return -1;
  }

 private:
  
  MainView *m_mainView;

  QListWidget *m_listToCompute;

  std::vector<std::list<QString> > m_titles;

  std::vector<std::string> m_imageFeatureToDelete;
  std::vector<std::list<std::string> > m_imageFeatureToRename;
  ImageFeatureCompositionPlugin *m_plugin;

  girl::hashMap< std::string, girl::shared_ptr<ImageFeatureSettingsInterface> > m_settings;
};

#endif /* ! IMAGEFEATURECOMPOSITIONWIDGET_HPP */
