/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 */

#include "PluginRegistry.hpp"
#include "ImageFeatureComputerTabWidget.hpp"

#include <QVBoxLayout>

#include "ImageFeatureTextureWidget.hpp"
#include "ImageFeatureGeometricWidget.hpp"
#include "ImageFeatureColorimetricWidget.hpp"

ImageFeatureComputerTabWidget::ImageFeatureComputerTabWidget(MainView *mainView,
							     FeatureComputerGUIInfoPtr featureComputerGUIInfo,
							     QString name,
							     QWidget *parent)
  : FeatureComputerTabWidget(mainView, featureComputerGUIInfo, name, parent)
{
  setAttribute(Qt::WA_DeleteOnClose);

  buildGUI();
}

void
ImageFeatureComputerTabWidget::buildGUI()
{
  QVBoxLayout *featuresLayout = new QVBoxLayout;
  featuresLayout->setAlignment(Qt::AlignTop);

  m_c = new ImageFeatureColorimetricWidget(m_mainView, m_featureComputerGUIInfo, "Colorimetric", this);
  m_g = new ImageFeatureGeometricWidget(m_mainView, m_featureComputerGUIInfo, "Geometric", this);
  m_t = new ImageFeatureTextureWidget(m_mainView, m_featureComputerGUIInfo, "Texture", this);

  addWidget(m_c);
  addWidget(m_g);
  addWidget(m_t);

  featuresLayout->addWidget(m_c);
  featuresLayout->addWidget(m_g);
  featuresLayout->addWidget(m_t);

  setLayout(featuresLayout);
}

void
ImageFeatureComputerTabWidget::initWidget()
{
  m_c->initWidget();
  m_g->initWidget();
  m_t->initWidget();
}
