/*
  Copyright (C) 2005-2011 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef IMAGEFEATUREGLOBALSETTINGS_HPP
#define IMAGEFEATUREGLOBALSETTINGS_HPP

#include <girl/girlCommon.hpp>
#include "ImageFeatureGlobalWidget.hpp"
#include "FeatureInterfaceSettings.hpp"
#include <ise/iseImageFeatureGlobal.hpp>

class GIRL_API ImageFeatureGlobalSettings : public FeatureInterfaceSettings
{

 public:

  ImageFeatureGlobalSettings()
    : FeatureInterfaceSettings()
    , m_algos()
    {
    }

  virtual void configureWidget(FeatureInterfaceWidget *widget) const
  {
    static_cast<ImageFeatureGlobalWidget *>(widget)->setParameters(m_algos);
  }

  virtual void getWidgetConfig(const FeatureInterfaceWidget *widget)
  {
    static_cast<const ImageFeatureGlobalWidget *>(widget)->getParameters(m_algos);
  }

  virtual void configureAlgo(girl::shared_ptr<ise::FeatureComputer> algo) const
  {
    girl::static_pointer_cast<ise::ImageFeatureGlobal>(algo)->setParameters(m_algos);
  }

  virtual girl::shared_ptr<FeatureInterfaceSettings> clone() const
  {
    girl::shared_ptr<ImageFeatureGlobalSettings> p(new ImageFeatureGlobalSettings);
    
    *p = *this;

    return p;
  }

  virtual bool isValid() const
  {
    size_t size = m_algos.size();
    for (size_t i = 0 ; i < size ; ++i)
      if (m_algos[i] == ise::ImageFeatureComputerPtr() )
	return false;

    return true;//B: ???
  }

 private:

  std::vector<ise::ImageFeatureComputerPtr > m_algos;
};

#endif /* ! IMAGEFEATUREGLOBALSETTINGS_HPP */
