/*
  Copyright (C) 2005-2011 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef IMAGEFEATUREGRADIENTMODULESETTINGS_HPP
#define IMAGEFEATUREGRADIENTMODULESETTINGS_HPP

#include "FeatureInterfaceSettings.hpp"
#include "ImageFeatureGradientModuleWidget.hpp"
#include <ise/iseImageFeatureGradientModule.hpp>
#include "FeatureData.hpp"

class ImageFeatureGradientModuleSettings : public FeatureInterfaceSettings 
{
  
public:
  
  ImageFeatureGradientModuleSettings() 
    : FeatureInterfaceSettings()
    , m_algo()
    , m_weightedChannelsCoeffs()
  {
  }

  virtual void configureWidget(FeatureInterfaceWidget *widget) const
  {
    static_cast<ImageFeatureGradientModuleWidget *>(widget)->setParameters(m_algo, m_weightedChannelsCoeffs);
  }
  
  virtual void getWidgetConfig(const FeatureInterfaceWidget *widget)
  {
    static_cast<const ImageFeatureGradientModuleWidget *>(widget)->getParameters(m_algo, m_weightedChannelsCoeffs);
  }

  virtual void configureAlgo(girl::shared_ptr<ise::FeatureComputer> algo) const
  {
    assert(algo != NULL);
    girl::shared_ptr<ise::ImageFeatureGradientModule> algorithm = girl::static_pointer_cast<ise::ImageFeatureGradientModule>(algo);
    algorithm->setParameters(girl::static_pointer_cast<ise::ImageFeatureComputer>(m_algo), m_weightedChannelsCoeffs);
  }
  
  virtual girl::shared_ptr<FeatureInterfaceSettings> clone() const
  {
    girl::shared_ptr<ImageFeatureGradientModuleSettings> p(new ImageFeatureGradientModuleSettings);
    p->m_algo = m_algo;
    p->m_weightedChannelsCoeffs = m_weightedChannelsCoeffs;

    return p;
  }

  virtual bool isValid() const
  {
    return ( m_algo != ise::FeatureComputerPtr() );
  }

private:

  ise::FeatureComputerPtr m_algo;
  std::vector<double> m_weightedChannelsCoeffs;
};

#endif /* ! IMAGEFEATUREGRADIENTMODULESETTINGS_HPP */
