/*
  Copyright (C) 2005-2011 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef IMAGEFEATUREGRADIENTSETTINGS_HPP
#define IMAGEFEATUREGRADIENTSETTINGS_HPP

#include "FeatureInterfaceSettings.hpp"
#include "ImageFeatureGradientWidget.hpp"
#include <ise/iseImageFeatureGradient.hpp>
#include "FeatureData.hpp"

class ImageFeatureGradientSettings : public FeatureInterfaceSettings
{
  
public:
  
  ImageFeatureGradientSettings() 
    : FeatureInterfaceSettings()
    , m_algo(ise::FeatureComputerPtr())
    , m_weightedChannelsCoeffs()
    , m_orientation(0)
  {
  }

  virtual void configureWidget(FeatureInterfaceWidget *widget) const
  {
    static_cast<ImageFeatureGradientWidget *>(widget)->setParameters(m_algo, m_weightedChannelsCoeffs, m_orientation);
  }
  
  virtual void getWidgetConfig(const FeatureInterfaceWidget *widget)
  {
    static_cast<const ImageFeatureGradientWidget *>(widget)->getParameters(m_algo, m_weightedChannelsCoeffs, m_orientation);
  }

  virtual void configureAlgo(girl::shared_ptr<ise::FeatureComputer> algo) const
  {
    assert(algo != NULL);
    girl::shared_ptr<ise::ImageFeatureGradient> algorithm = girl::static_pointer_cast<ise::ImageFeatureGradient>(algo);
    algorithm->setParameters(girl::static_pointer_cast<ise::ImageFeatureComputer>(m_algo) , m_weightedChannelsCoeffs, m_orientation);
  }
  
  virtual girl::shared_ptr<FeatureInterfaceSettings> clone() const
  {
    girl::shared_ptr<ImageFeatureGradientSettings> p(new ImageFeatureGradientSettings);
    p->m_algo = m_algo;
    p->m_weightedChannelsCoeffs = m_weightedChannelsCoeffs;
    p->m_orientation = m_orientation;

    return p;
  }

  virtual bool isValid() const
  {
    return ( m_algo != ise::FeatureComputerPtr() );
  }

private:
  
  ise::FeatureComputerPtr m_algo;
  std::vector<double> m_weightedChannelsCoeffs;
  int m_orientation;
};

#endif /* ! IMAGEFEATUREGRADIENTSETTINGS_HPP */
