/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "ImageFeatureGradientWidget.hpp"

#include <QString>
#include <QLabel>
#include <QGroupBox>
#include <QGridLayout>
#include <QComboBox>

#include "WeightedChannelsImageFeatureWidget.hpp"
#include "ImageFeatureChoiceWidget.hpp"

ImageFeatureGradientWidget::ImageFeatureGradientWidget(MainView *mainView, 
						       FeatureComputerGUIInfoPtr featureComputerGUIInfo, 
						       QString name,
						       ImageFeaturePlugin *imageFeaturePlugin, 
						       QWidget *parent)
  : ImageFeatureWidgetInterface(mainView, featureComputerGUIInfo, name, imageFeaturePlugin, parent)
  , m_wcifw(NULL)
  , m_orientation(NULL)
{
  setAttribute(Qt::WA_DeleteOnClose);

  build();
}

void
ImageFeatureGradientWidget::build()
{
  assert(m_mainView->model()->getSegContextPtr());

  QLabel *orientationLabel = new QLabel(QString("Gradient in :"));
  m_orientation = new QComboBox(this);
  m_orientation->addItem("X");
  m_orientation->addItem("Y");

  QHBoxLayout *hLayout = new QHBoxLayout;
  hLayout->addWidget(orientationLabel);
  hLayout->addWidget(m_orientation);

  QGroupBox *imgfGroup = new QGroupBox(QString("Image Feature"));
  QVBoxLayout *layoutGroup = new QVBoxLayout(imgfGroup);
  m_wcifw = new WeightedChannelsImageFeatureWidget(m_mainView, m_featureComputerGUIInfo, imgfGroup);

  layoutGroup->addWidget(m_wcifw);

  QVBoxLayout *layout = new QVBoxLayout();
  layout->setAlignment(Qt::AlignTop);

  layout->addLayout(hLayout);
  layout->addWidget(imgfGroup);

  setLayout(layout);
}

bool
ImageFeatureGradientWidget::isValid()
{
  return (m_wcifw->imageFeatureComputerPtr() != ise::FeatureComputerPtr() );
}

void
ImageFeatureGradientWidget::setParameters(const ise::FeatureComputerPtr &algo, const std::vector<double> &weightedChannelsCoeffs, int orientation)
{
  m_wcifw->setImageFeatureComputerPtr(algo);
  if (weightedChannelsCoeffs.size() > 0)
    m_wcifw->setWeights(weightedChannelsCoeffs);
  m_orientation->setCurrentIndex(orientation);
}

void
ImageFeatureGradientWidget::getParameters(ise::FeatureComputerPtr &algo, std::vector<double> &weightedChannelsCoeffs, int &orientation) const
{
  algo = m_wcifw->imageFeatureComputerPtr();
  m_wcifw->weights(weightedChannelsCoeffs);
  orientation = m_orientation->currentIndex();
}	 

