/*
  Copyright (C) 2005-2011 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef IMAGEFEATURELINEARCOMBINATIONSIMPLESETTINGS_HPP
#define IMAGEFEATURELINEARCOMBINATIONSIMPLESETTINGS_HPP

#include <girl/girlCommon.hpp>
#include "ImageFeatureLinearCombinationSimpleWidget.hpp"
#include "FeatureInterfaceSettings.hpp"
#include <ise/iseImageFeatureLinearCombination.hpp>

class GIRL_API ImageFeatureLinearCombinationSimpleSettings : public FeatureInterfaceSettings{

 public:

  ImageFeatureLinearCombinationSimpleSettings()
    : FeatureInterfaceSettings()
    ,  m_algos()
    ,  m_weights()
    ,  m_dim()
    {}

  virtual void configureWidget(FeatureInterfaceWidget *widget) const
  {
    static_cast<ImageFeatureLinearCombinationSimpleWidget*>(widget)->setParameters(m_algos, m_weights, m_dim);
  }

  virtual void getWidgetConfig(const FeatureInterfaceWidget *widget)
  {
    static_cast<const ImageFeatureLinearCombinationSimpleWidget*>(widget)->getParameters(m_algos, m_weights, m_dim);
  }

  virtual void configureAlgo(girl::shared_ptr<ise::FeatureComputer> algo) const
  {
    girl::static_pointer_cast<ise::ImageFeatureLinearCombination>(algo)->setParameters(m_algos, m_weights, m_dim);
  }

  virtual girl::shared_ptr<FeatureInterfaceSettings> clone() const
  {
    girl::shared_ptr<ImageFeatureLinearCombinationSimpleSettings> p(new ImageFeatureLinearCombinationSimpleSettings);

    *p = *this;

    return p;
  }

  virtual bool isValid() const
  {
    if (m_algos.size() != m_weights.size())
      return false;

    size_t size = m_algos.size();
    for (size_t i = 0 ; i < size ; ++i)
      if (m_algos[i] == ise::ImageFeatureComputerPtr() )
	return false;

    return true; //B: ???   && (m_weights.size() == m_uris.size()) ???
  }

 private:

  std::vector<ise::ImageFeatureComputerPtr> m_algos;
  std::vector<double> m_weights;
  int m_dim;

};

#endif /* ! IMAGEFEATURELINEARCOMBINATIONSIMPLESETTINGS_HPP */
