/* 
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 */

#include "ImageFeatureLinearCombinationSimpleWidget.hpp"
#include "CheckLabelSliderSpinBoxWidget.hpp"

#include <QGroupBox>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QComboBox>
#include <QLabel>

#include <map>

#include <ise/iseImageFeatureLinearCombination.hpp>

ImageFeatureLinearCombinationSimpleWidget::ImageFeatureLinearCombinationSimpleWidget(MainView *mainView,
										     FeatureComputerGUIInfoPtr featureComputerGUIInfo,
										     QString name,
										     ImageFeaturePlugin *plugin,
										     QWidget *parent)
  : ImageFeatureCompositionWidgetInterface(mainView, featureComputerGUIInfo, name, plugin, parent)
  , m_mclssbw(NULL)
  , m_combo(NULL)
  , m_datas()
{
  setAttribute(Qt::WA_DeleteOnClose);

  build();

  initWidget();
}

void
ImageFeatureLinearCombinationSimpleWidget::build()
{
  QVBoxLayout *layout = new QVBoxLayout;

  QHBoxLayout *comboLayout = new QHBoxLayout;

  QLabel *labelCombo = new QLabel(QString("dimension"));
  m_combo = new QComboBox(this);
  connect(m_combo, SIGNAL(currentIndexChanged(int)), this, SLOT(populateMultiCheckLabel()));

  comboLayout->addWidget(labelCombo);
  comboLayout->addWidget(m_combo);

  m_mclssbw = new MultiCheckLabelSliderSpinBoxWidget(this);

  layout->addLayout(comboLayout);
  layout->addWidget(m_mclssbw);
  layout->addStretch();

  setLayout(layout);
}

void
ImageFeatureLinearCombinationSimpleWidget::addData(FeatureDataPtr data)
{
  assert(m_mainView);
  assert(m_mainView->model());
  assert(m_mainView->model()->getSegContextPtr());
  
  int dim = m_featureComputerGUIInfo->getInfos(data->title())->createAlgo(m_mainView->model()->getSegContextPtr())->getFeatureDim();
  QString s = QString("%1").arg(dim);
  m_datas.insert(std::make_pair(s, data));
  if (m_combo->findText(s) == -1) 
    m_combo->addItem(s);

  populateMultiCheckLabel();
}

void
ImageFeatureLinearCombinationSimpleWidget::addItem(FeatureDataPtr data, double weight, bool checked)
{
  m_mclssbw->addItem(data->title().c_str(), weight, checked);
  m_settings[data->title()] = data;
}

void
ImageFeatureLinearCombinationSimpleWidget::populateMultiCheckLabel()
{
  m_mclssbw->clear();

  std::multimap<QString, FeatureDataPtr >::iterator it;
  std::pair< std::multimap<QString, FeatureDataPtr >::iterator , std::multimap<QString, FeatureDataPtr >::iterator> ret = m_datas.equal_range(m_combo->currentText());

  for ( it = ret.first ; it != ret.second ; ++it) {
    addItem(it->second, 1, false);
  }
}

void
ImageFeatureLinearCombinationSimpleWidget::setParameters(const std::vector<ise::ImageFeatureComputerPtr > &algos, const std::vector<double> &weight, int dim)
{
  int index = m_combo->findText(QString("%1").arg(dim));
  m_combo->setCurrentIndex(index);

  for (size_t i = 0 ; i < algos.size() ; ++i) {
    m_mclssbw->addItem(m_featureComputerGUIInfo->getInfos(algos[i])->title().c_str(), weight[i], true);
  }
}

void
ImageFeatureLinearCombinationSimpleWidget::getParameters(std::vector<ise::ImageFeatureComputerPtr > &algos, std::vector<double> &weight, int &dim) const
{
  algos.clear();
  weight.clear();
  dim = m_combo->currentText().toInt();
  int count = m_mclssbw->count();
  for ( int i = 0 ; i < count ; ++i) {
    CheckLabelSliderSpinBoxWidget* item = m_mclssbw->item(i);
    if (item) {
      if (item->isChecked()) {
	algos.push_back( girl::static_pointer_cast<ise::ImageFeatureComputer>(m_featureComputerGUIInfo->getAlgo(m_settings.find(item->text().toStdString())->first)) );
	weight.push_back(item->value());
      }
    }
  }
}


void
ImageFeatureLinearCombinationSimpleWidget::initWidget()
{
  std::vector<FeatureDataPtr> infos;
  m_featureComputerGUIInfo->getFeatureComputersInfo<ise::ImageFeatureComputer>(infos);
  
  size_t size = infos.size();
  
  for (size_t i = 0 ; i < size ; ++i) 
    addData(infos[i]);
}
