/*
  Copyright (C) 2005-2011 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef IMAGEFEATUREORIENTATIONSETTINGS_HPP
#define IMAGEFEATUREORIENTATIONSETTINGS_HPP

#include "FeatureInterfaceSettings.hpp"
#include "ImageFeatureOrientationWidget.hpp"
#include <ise/iseImageFeatureOrientation.hpp>

class ImageFeatureOrientationSettings : public FeatureInterfaceSettings
{
  
public:
  
  ImageFeatureOrientationSettings() 
    : FeatureInterfaceSettings()
    , m_multiresolution(0)
  {
  }

  virtual void configureWidget(FeatureInterfaceWidget *widget) const
  {
    static_cast<ImageFeatureOrientationWidget *>(widget)->setParameters(m_multiresolution);
  }
  
  virtual void getWidgetConfig(const FeatureInterfaceWidget *widget)
  {
    static_cast<const ImageFeatureOrientationWidget *>(widget)->getParameters(m_multiresolution);
  }

  virtual void configureAlgo(girl::shared_ptr<ise::FeatureComputer> algo) const
  {
    girl::shared_ptr<ise::ImageFeatureOrientation> algorithm = girl::static_pointer_cast<ise::ImageFeatureOrientation>(algo);
    algorithm->setParameters(m_multiresolution);
  }

  virtual girl::shared_ptr<FeatureInterfaceSettings> clone() const
  {
    girl::shared_ptr<ImageFeatureOrientationSettings> p(new ImageFeatureOrientationSettings);

    *p = *this;

    return p;
  }

  virtual bool isValid() const
  {
    return true; //B: ???
  }
  
private:
  
  int m_multiresolution;
};

#endif /* ! IMAGEFEATUREORIENTATIONSETTINGS_HPP */
