/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "ImageFeatureOrientationWidget.hpp"

#include <QSpinBox>
#include <QLabel>
#include <QString>

#include <QComboBox>
#include <QDialogButtonBox>
#include <QGridLayout>

ImageFeatureOrientationWidget::ImageFeatureOrientationWidget(MainView *mainView, 
							     FeatureComputerGUIInfoPtr featureComputerGUIInfo, 
							     QString name, 
							     ImageFeaturePlugin *imageFeaturePlugin, 
							     QWidget *parent)
  : ImageFeatureWidgetInterface(mainView, featureComputerGUIInfo, name, imageFeaturePlugin, parent)
  , m_multiresolution(NULL)
{
  setAttribute(Qt::WA_DeleteOnClose);

  build();
}

void
ImageFeatureOrientationWidget::build()
{
  assert(m_multiresolution == NULL);
  m_multiresolution = new QSpinBox(this);
  m_multiresolution->setMinimum(1);
  m_multiresolution->setMaximum(10);

  QLabel *multiLabel = new QLabel(tr("Multiresolution number : "), this);
  
  QGridLayout *layout = new QGridLayout(this);
  layout->setAlignment(Qt::AlignTop);

  layout->addWidget(multiLabel, 0, 0, Qt::AlignTop);
  layout->addWidget(m_multiresolution, 0, 1, Qt::AlignTop);
  
  setLayout(layout);
}

bool
ImageFeatureOrientationWidget::isValid()
{
  return (m_multiresolution->value() >= 1 && m_multiresolution->value() <= 10);
}

void 
ImageFeatureOrientationWidget::setParameters(int multiresolution)
{
  m_multiresolution->setValue(multiresolution);
}

void
ImageFeatureOrientationWidget::getParameters(int &multiresolution) const
{
  multiresolution = m_multiresolution->value();
}


