#ifndef IMAGEFEATUREREGISTRY_HPP
#define IMAGEFEATUREREGISTRY_HPP

#include <qiseTools/ImageFeaturesWidget.hpp>

#include <map>

#include <QString>

struct ImageFeatureDetails
{
  QString name;
  int iseImageFeature;

  ImageFeatureDetails()
    : name(), iseImageFeature(0)
  {

  }

  ImageFeatureDetails(const QString &ifName, int iseIF)
    : name(ifName)
    , iseImageFeature(iseIF)
  {
    
  }

};


class GIRL_API ImageFeatureRegistry
{
public:

  typedef std::map<int, ImageFeatureDetails> ImageFeatureMap;

  static bool addImageFeature(const ImageFeatureDetails &ifd)
  {
    if (m_imageFeatures.find(ifd.iseImageFeature) == m_imageFeatures.end())
      {
	m_imageFeatures[ifd.iseImageFeature] = ifd;
	return true;
      }
    return false;
  }

  static const ImageFeatureDetails *getImageFeature(int iseImageFeature)
  {
    ImageFeatureMap::const_iterator it = m_imageFeatures.find(iseImageFeature);
    if (it != m_imageFeatures.end())
      return &(it->second);
    return NULL;    
  }

  static size_t getNumImageFeatures() 
  {
    return m_imageFeatures.size();
  }

  static int getIseImageFeatureFromName(QString name)
  {
    ImageFeatureMap::const_iterator it = begin();
    ImageFeatureMap::const_iterator itEnd = end();

    for ( ; it != itEnd ; ++it)
      if ( it->second.name == name)
	return it->second.iseImageFeature;
    
    return -1;
  }

  static ImageFeatureMap::const_iterator begin() 
  {
    return m_imageFeatures.begin();
  }

  static ImageFeatureMap::const_iterator end() 
  {
    return m_imageFeatures.end();
  }

protected:

  static ImageFeatureMap m_imageFeatures;
};

#endif /* ! IMAGEFEATUREREGISTRY_HPP */
