/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "ImageFeatureTextonWidget.hpp"

#include <QDoubleSpinBox>
#include <QLabel>
#include <QString>
#include <QGridLayout>

#include <ise/iseSegmentationContext.hpp>

ImageFeatureTextonWidget::ImageFeatureTextonWidget(MainView *mainView,
						   FeatureComputerGUIInfoPtr featureComputerGUIInfo, 
						   QString name,
						   ImageFeaturePlugin *imageFeaturePlugin,
						   QWidget *parent)
  : ImageFeatureWidgetInterface(mainView, featureComputerGUIInfo, name, imageFeaturePlugin, parent)
  , m_windowSize(NULL)
{
  setAttribute(Qt::WA_DeleteOnClose);

  build();
}

void
ImageFeatureTextonWidget::build()
{
  assert(m_windowSize == NULL);
  m_windowSize = new QDoubleSpinBox(this);

  QLabel *windowLabel = new QLabel(tr("Window Size : "), this);

  m_windowSize->setMinimum(3);
  m_windowSize->setDecimals(0);

  m_windowSize->setValue(3);
  m_windowSize->setSingleStep(2);

  QGridLayout *layout = new QGridLayout;

  layout->addWidget(windowLabel, 0, 0);
  layout->addWidget(m_windowSize, 0, 1);

  setLayout(layout);
}

bool
ImageFeatureTextonWidget::isValid()
{
  return (static_cast<int>(m_windowSize->value()) % 2) == 1;
}

void
ImageFeatureTextonWidget::setParameters(int windowSize)
{
  m_windowSize->setValue(windowSize);
}

void
ImageFeatureTextonWidget::getParameters(int &windowSize) const
{
  windowSize = m_windowSize->value();
}
