/*
  Copyright (C) 2005-2011 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef IMAGEFEATURETHRESHOLDSETTINGS_HPP
#define IMAGEFEATURETHRESHOLDSETTINGS_HPP

//#include "ImageFeatureSettingsInterface.hpp"
#include "FeatureInterfaceSettings.hpp"
#include "ImageFeatureThresholdWidget.hpp"
#include <ise/iseImageFeatureThreshold.hpp>

class ImageFeatureThresholdSettings : public FeatureInterfaceSettings//ImageFeatureSettingsInterface 
{
  
public:
  
  ImageFeatureThresholdSettings() 
    : FeatureInterfaceSettings()//ImageFeatureSettingsInterface()
    , m_levels()
    , m_colorSpace(0)
    , m_channel(0)
  {
  }

  virtual void configureWidget(FeatureInterfaceWidget *widget) const
  {
    static_cast<ImageFeatureThresholdWidget *>(widget)->setParameters(m_levels, m_colorSpace, m_channel);
  }
  
  virtual void getWidgetConfig(const FeatureInterfaceWidget *widget)
  {
    static_cast<const ImageFeatureThresholdWidget *>(widget)->getParameters(m_levels, m_colorSpace, m_channel);
  }

  virtual void configureAlgo(girl::shared_ptr<ise::FeatureComputer> algo) const
  {
    girl::shared_ptr<ise::ImageFeatureThreshold> algorithm = girl::static_pointer_cast<ise::ImageFeatureThreshold>(algo);
    algorithm->setParameters(m_levels, m_colorSpace, m_channel);
  }
  
  virtual girl::shared_ptr<FeatureInterfaceSettings> clone() const
  {
    girl::shared_ptr<ImageFeatureThresholdSettings> p(new ImageFeatureThresholdSettings);

    *p = *this;

    return p;
  }

  virtual bool isValid() const
  {
    return (! m_levels.empty()) && (m_channel>=0); //B: ???
  }

private:
  
  std::vector<int> m_levels;
  int m_colorSpace;
  int m_channel;
};

#endif /* ! IMAGEFEATURETHRESHOLDSETTINGS_HPP */
