/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "ImageFeatureThresholdWidget.hpp"

#include <QString>
#include <QLabel>
#include <QGroupBox>
#include <QVBoxLayout>
#include <QComboBox>

#include <qiseTools/QuantificationThresholdsWidgetAux.hpp>

ImageFeatureThresholdWidget::ImageFeatureThresholdWidget(MainView *mainView, 
							 FeatureComputerGUIInfoPtr featureComputerGUIInfo, 
							 QString name, 
							 ImageFeaturePlugin *imageFeaturePlugin, 
							 QWidget *parent)
  : ImageFeatureWidgetInterface(mainView, featureComputerGUIInfo, name, imageFeaturePlugin, parent)
  , m_qtwa(NULL)
{
  setAttribute(Qt::WA_DeleteOnClose);

  build();
}

ImageFeatureThresholdWidget::~ImageFeatureThresholdWidget()
{
  m_qtwa->deactivate();
}

void
ImageFeatureThresholdWidget::build()
{
  const bool allRegions = false;
  const bool oneChannel = true;
  m_qtwa = new QuantificationThresholdsWidgetAux(m_mainView, allRegions, oneChannel, this);
  m_qtwa->activate();
  QVBoxLayout *layout = new QVBoxLayout(this);
  layout->addWidget(m_qtwa);

  setLayout(layout);
}

bool
ImageFeatureThresholdWidget::isValid()
{
  std::vector<int> levels;
  int colorSpace;
  int channel;
  getParameters(levels, colorSpace, channel);
  return (levels.size() == 1 && colorSpace >= 0 && channel < 3);
}

void
ImageFeatureThresholdWidget::setParameters(std::vector<int> levels, int colorSpace, int channel)
{
  m_qtwa->setGUI(colorSpace, channel, levels);
}

void
ImageFeatureThresholdWidget::getParameters(std::vector<int> &levels, int &colorSpace, int &channel) const
{
  m_qtwa->getGUI(colorSpace, channel, levels);
}	 

