/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "ImagePainter.hpp"

#include "ColorGenerator.hpp"



//REM: for performance reasons we do not want to call QImage::setPixel() 
/// (there is a check to call detach() each time)
//  Thus we do raw data direct access.
ImagePainter::RegionColorFiller::RegionColorFiller(QImage &img, QRgb color)
  : RegionColorPainter(img, color),
    m_data(m_img.bits()),
    m_bytesPerLine(m_img.bytesPerLine()),
    m_width(m_img.width())
{
  assert(! m_img.isNull() );
}
  
void 
ImagePainter::RegionColorFiller::processPoint(const girl::point &pt)
{
  uchar *s = m_data + pt.y() * m_bytesPerLine; 
  ((uint *)s)[pt.x()] = m_color;
}



ImagePainter::RegionColorBlender::RegionColorBlender(QImage &img, QRgb color)
  : RegionColorPainter(img, color),
    m_painter(&m_img)
{
  if (! m_img.isNull()) {//TODO:could be faster if image is not in ARGB32 or ARGB32_Premultiplied ???
    QColor c(color);
    c.setAlpha(qAlpha(color));
    const QPen p(c);
    m_painter.setPen(p);
    m_painter.setCompositionMode(QPainter::CompositionMode_SourceOver);
  }

}

void
ImagePainter::RegionColorBlender::processPoint(const girl::point &pt)
{
  m_painter.drawPoint(pt.x(), pt.y());
}


void
ImagePainter::RegionColorBlender::end()
{
  if (m_painter.isActive())
    m_painter.end();
}







void
ImagePainter::fillRegion(const girl::toger &tog, girl::region r,
			 QImage &img, QRgb color)
{
  RegionColorFiller rf(img, color);
  tog.iterateOnRegionDomain(r, rf); 
}

void
ImagePainter::fillRegions(const girl::toger &tog, 
			  const girl::regionDeque &regions,
			  QImage &img, QRgb color)
{
  RegionColorFiller rf(img, color);
  girl::regionDeque::const_iterator it = regions.begin();
  const girl::regionDeque::const_iterator itEnd = regions.end();
  for ( ; it != itEnd; ++it) {
    tog.iterateOnRegionDomain(*it, rf); 
  }
}

void
ImagePainter::blendRegion(const girl::toger &tog, girl::region r,
			  QImage &img, QRgb color)
{
  RegionColorBlender rf(img, color);
  tog.iterateOnRegionDomain(r, rf); 
}

void
ImagePainter::blendRegions(const girl::toger &tog, 
			  const girl::regionDeque &regions,
			  QImage &img, QRgb color)
{
  girl::regionDeque::const_iterator it = regions.begin();
  const girl::regionDeque::const_iterator itEnd = regions.end();
  for ( ; it != itEnd; ++it) {
    fillRegion(tog, *it, img, color);
  }
}

void
ImagePainter::drawTogerInFalseColors(const girl::toger &tog, QImage &img)
{
  ColorGenerator colorGenerator;

  girl::regionDeque regions;
  tog.allRegions(regions);

  girl::regionDeque::const_iterator it = regions.begin();
  const girl::regionDeque::const_iterator itEnd = regions.end();
  for ( ; it != itEnd; ++it) {
    const QRgb color = colorGenerator.getNewColor();
    fillRegion(tog, *it, img, color);
  }
}
