/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef IMAGEPAINTER_HPP
#define IMAGEPAINTER_HPP

#include <girl/girlToger.hpp>

#include <QImage>
#include <QPainter>

namespace ImagePainter
{
  class GIRL_API RegionColorPainter : public girl::toger::pointIterator
  {
  public:

    /**
     * @brief constructor.
     *
     * @warning @a img is NOT copied.
     */
    RegionColorPainter(QImage &img, QRgb color)
      : girl::toger::pointIterator(),
	m_img(img),
	m_color(color)
    {
    }

    //virtual void processPoint(const girl::point &pt) = 0;
    
    QImage &getImage()
    {
      return m_img;
    }

  protected:
    QImage &m_img;
    QRgb m_color;
  };


  class GIRL_API RegionColorFiller : public RegionColorPainter
  {
  public:

    /**
     * @brief constructor.
     *
     * @warning @a img is NOT copied.
     */
    RegionColorFiller(QImage &img, QRgb color);

    virtual void processPoint(const girl::point &pt);
    
  protected:
    uchar *m_data;
    int m_bytesPerLine;
    int m_width;
  };
  
  class GIRL_API RegionColorBlender : public RegionColorPainter
  {
  public:
    
    /**
     * @brief constructor.
     *
     * @warning @a img is NOT copied.
     */
    RegionColorBlender(QImage &img, QRgb color);
  
    virtual void processPoint(const girl::point &pt);
    
    /**
     * @brief Signals that no more point will be processed.
     */
    void end();

  protected:

    QPainter m_painter;
  };



  /**
   * @brief Fill region @r of toger @a tog with color @a c in image @a img.
   */
  GIRL_API void fillRegion(const girl::toger &tog, girl::region r,
			   QImage &img, QRgb color);

  /**
   * @brief Fill regions @regs of toger @a tog with color @a c in image @a img.
   */
  GIRL_API void fillRegions(const girl::toger &tog, const girl::regionDeque &regs,
			    QImage &img, QRgb c);

  /**
   * @brief Blend region @r of toger @a tog with color @a c in image @a img.
   */
  GIRL_API void blendRegion(const girl::toger &tog, girl::region r,
			    QImage &img, QRgb color);

  /**
   * @brief Blend regions @regs of toger @a tog with color @a c in image @a img.
   */
  GIRL_API void blendRegions(const girl::toger &tog, const girl::regionDeque &regs,
			     QImage &img, QRgb c);


  /**
   * @brief Fill with random colors all regions of toger @a tog in image @a img.
   */
  GIRL_API void drawTogerInFalseColors(const girl::toger &tog, QImage &img);

}

#endif /* ! IMAGEPAINTER_HPP */
