/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "InstantFeatureDialog.hpp"

#include <QVBoxLayout>
#include <QDialogButtonBox>
#include "FeatureDialog.hpp"
#include <QLabel>

InstantFeatureDialog::InstantFeatureDialog(MainView *mainView, FeatureComputerGUIInfoPtr featureComputerGUIInfo, QIsePluginList<FeaturePlugin*> &listPlugin, FeatureDataPtr &data, QWidget *parent)
  : QDialog(parent)
  , m_mainView(mainView)
  , m_featureComputerGUIInfo(featureComputerGUIInfo)
  , m_list(NULL)
  , m_listPlugin(listPlugin)
  , m_data(data)
{
  setAttribute(Qt::WA_DeleteOnClose);

  buildGUI();
}

void
InstantFeatureDialog::buildGUI()
{
  QVBoxLayout *layout = new QVBoxLayout;
  QLabel *label = new QLabel("Choose a feature to create:");
  m_list = new QListWidget(this);
  connect( m_list, SIGNAL(itemDoubleClicked(QListWidgetItem*)), this, SLOT(accept()));

  QIsePluginList<FeaturePlugin*>::iterator it = m_listPlugin.begin();
  QIsePluginList<FeaturePlugin*>::const_iterator itEnd = m_listPlugin.end();
  for ( ; it != itEnd ; ++it) {
    m_list->addItem((*it)->name());
  }

  QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok
						     | QDialogButtonBox::Cancel, Qt::Horizontal, this);
  QWidget::connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
  QWidget::connect(buttonBox, SIGNAL(rejected()), this, SLOT(reject()));

  layout->addWidget(label);
  layout->addWidget(m_list);
  layout->addWidget(buttonBox);

  setLayout(layout);
}

void
InstantFeatureDialog::accept()
{
  QListWidgetItem *item = m_list->currentItem();
  if (item == NULL) {
    QDialog::reject();
    return;
  }

  QString name = item->text();
  QIsePluginList<FeaturePlugin*>::iterator it = m_listPlugin.begin();
  QIsePluginList<FeaturePlugin*>::const_iterator itEnd = m_listPlugin.end();
  for ( ; it != itEnd ; ++it) {
    if((*it)->name() == name)
      break;
  }

  FeatureInterfaceWidget *w = (*it)->parameterWidget(m_mainView, m_featureComputerGUIInfo, this);
  girl::shared_ptr<FeatureInterfaceSettings> settings = (*it)->createSettings();
  m_data = FeatureDataPtr(new FeatureData(settings, *it));
  
  FeatureDialog *d = new FeatureDialog(w, m_data, this);
  d->setSettings(settings);
  d->setTitle((*it)->name());
  
  int returnCode = d->exec();

  if (returnCode == QDialog::Accepted) {
    girl::shared_ptr<ise::FeatureComputer> algo = m_data->createAlgo(m_mainView->model()->getSegContextPtr());

    if (! m_featureComputerGUIInfo->add(algo, m_data)) {
      QMessageBox::critical(this, "Error", QString("An image feature %1 with \n same name or same parameters,\n already compute").arg(m_data->title().c_str()));
      QDialog::reject();
      return;
    }
  }
  else {
    std::cerr << "Parameter dialog was canceled! Not adding the feature computer" << std::endl;
    QDialog::reject();
    return;
  }

  QDialog::accept();
}
