/*
  Copyright (C) 2005-2009 Jerome Charton, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef INTERACTIVESHAPESTATUS_HPP
#define INTERACTIVESHAPESTATUS_HPP

#include "EditionBox.hpp"
#include "GLView.hpp"

class QPoint;
class PaintingShape;

class GIRL_API InteractiveShapeStatus
{
public:
  InteractiveShapeStatus(GLView *view) 
  : m_editionBox(), 
    m_view(view) 
  {}

  virtual ~InteractiveShapeStatus() {}

  inline void draw(QPainter &painter, PaintingShape *shape) {if (isSelected()) m_editionBox.draw(m_view, painter, shape);}

  inline bool isUnder(const QPoint &point, PaintingShape *shape) {return m_editionBox.isUnderRotationHandler(point, m_view, shape) || shape->isUnder(point);}
  
  virtual InteractiveShapeStatus *onLeftClick(const QPoint &mouse, PaintingShape *shape) = 0;
  virtual InteractiveShapeStatus *onRightClick(const QPoint &mouse, PaintingShape *shape) = 0;
  
  virtual InteractiveShapeStatus *onLeftRelease(const QPoint &mouse, PaintingShape *shape) = 0;
  virtual InteractiveShapeStatus *onRightRelease(const QPoint &mouse, PaintingShape *shape) = 0;
  
  virtual InteractiveShapeStatus *onLeftClickWithCtrl(const QPoint &mouse, PaintingShape *shape) = 0;
  virtual InteractiveShapeStatus *onRightClickWithCtrl(const QPoint &mouse, PaintingShape *shape) = 0;
  
  virtual InteractiveShapeStatus *onMove(const QPoint &mouse, PaintingShape *shape) = 0;
  
  virtual InteractiveShapeStatus *onMoveWithLeftClick(const QPoint &mouse, PaintingShape *shape) = 0;
  virtual InteractiveShapeStatus *onMoveWithRightClick(const QPoint &mouse, PaintingShape *shape) = 0;

  virtual InteractiveShapeStatus *setSelected(bool isSelected) = 0;
  virtual bool isSelected() = 0;
  
protected:
  EditionBox m_editionBox;
  GLView *m_view;
};

#endif /* ! INTERACTIVESHAPESTATUS_HPP */
