/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 */

#ifndef LABELSHOWBUTTONWIDGET_HPP
#define LABELSHOWBUTTONWIDGET_HPP

#include <girl/girlCommon.hpp>

#include <QHBoxLayout>
#include <QLabel>
#include <QPushButton>
#include <QString>
#include <QVariant>
#include <QWidget>

#include <map>

class GIRL_API LabelShowButtonWidget : public QWidget
{
  Q_OBJECT

 public:

  LabelShowButtonWidget(QString text, QWidget *parent)
    : QWidget(parent)
    , m_text(text)
    , m_datas()
  {
    build();
  }

  inline QString text() const { return m_text; }

  void setText(const QString &text) {
    m_text = text;
    m_label->setText(m_text);
  }

  void setData(const std::string &type, QVariant data)
  {
    m_datas.insert(std::make_pair(type, data));
  }

  QVariant data(const std::string &type) 
  {
    return m_datas[type];
  }

 protected:

  void build()
  {
    QHBoxLayout *layout = new QHBoxLayout;
    layout->setSpacing(0);
    layout->setContentsMargins(0, 0, 0, 0);

    QPushButton *b = new QPushButton(tr("..."));
    b->setMinimumSize(20, 20);
    b->setMaximumSize(20, 20);
    connect(b, SIGNAL(clicked()),  this, SLOT(show()));

    m_label = new QLabel(m_text);

    layout->addWidget(m_label, Qt::AlignLeft);
    layout->addWidget(b, Qt::AlignRight);

    setLayout(layout);
  }

 private slots:
  
  void show() {
    emit show(m_text);
  }

 signals:

  void show(QString);

 private:

  QLabel *m_label;
  QString m_text;
  
  std::map<std::string, QVariant> m_datas;
};

#endif /* ! LABELSHOWBUTTONWIDGET_HPP */
