/*
  Copyright (C) 2005-2009 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

/**
 * @file LabelSliderSpinBoxWidget.hpp
 * @brief 
 */

#ifndef LABELSLIDERSPINBOXWIDGET_HPP
#define LABELSLIDERSPINBOXWIDGET_HPP

#include <girl/girlCommon.hpp>
#include <QWidget>
#include <QString>
#include <QLabel>
#include <QHBoxLayout>
#include "SliderSpinBoxWidget.hpp"

class GIRL_API LabelSliderSpinBoxWidget : public QWidget
{
  Q_OBJECT

 public:
  /**
   * @brief Constructor
   */
  LabelSliderSpinBoxWidget(const QString &text, QWidget *parent = NULL)
    : QWidget(parent)
    , m_label(NULL)
    , m_slider(NULL)
    {
      build(text);
    }

  /**
   * @brief Set label's text
   */
  inline void setText(const QString &text) { m_label->setText(text); }

  /**
   * @brief Get label's text
   */
  inline QString text() const { return m_label->text(); }

  inline double value() const { return m_slider->value(); }

  inline void setValue(double value) { m_slider->setValue(value); }

  inline void setMinimum(int lmin) { m_slider->setMinimum(lmin); }

  inline void setMaximum(int lmax) { m_slider->setMaximum(lmax); }

  inline void setDecimals(int dec) { m_slider->setDecimals(dec); }

  inline void setSingleStep(double step) { m_slider->setSingleStep(step); }

 protected:
  
  void build(const QString &text) 
  {
    QHBoxLayout *m_layout = new QHBoxLayout;
    m_layout->setMargin(0);

    m_label = new QLabel(text);

    m_slider = new SliderSpinBoxWidget(this);

    m_layout->addWidget(m_label);
    m_layout->addWidget(m_slider);

    setLayout(m_layout);
  }

 private:
  
  QLabel *m_label;
  SliderSpinBoxWidget *m_slider;
  

};

#endif /* ! LABELSLIDERSPINBOXWIDGET_HPP */
