/*
  Copyright (C) 2010-2011 Cédric Aguerre  aguerre@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#include "LineSelector.hpp"

#include "GLPolylineLayer.hpp"
#include "SegModel.hpp"
#include "SegView.hpp"
#include <algorithm>
#include <girlTools/girlToolsRegionInfos.hpp>


static const QColor DEFAULT_POINT_COLOR = QColor(255, 0, 0);


LineSelector::LineSelector(SegView *v)
  : ShapeSelector(v),
    m_pts(2)
{
  m_pts[0] = GLSelectorPoint(QPointF(), DEFAULT_POINT_COLOR);
  m_pts[0].setVisible(false);
  m_pts[1] = GLSelectorPoint(QPointF(), DEFAULT_POINT_COLOR);
  m_pts[1].setVisible(false);
}

LineSelector::~LineSelector()
{
  
}

void
LineSelector::addPoint(const QPointF &ptf)
{
  assert(m_layer == NULL);
  m_layer = NULL;

  m_pts[0].setPosition(ptf);
  m_pts[1].setPosition(ptf);
}

bool
LineSelector::visit(const QPointF &/*ptf*/)
{
  return false;
}

void
LineSelector::update(const QPointF &ptf)
{
  if (m_view == NULL) return;
  m_pts[1].setPosition(ptf);

  GLPolylineLayer *lineLayer = static_cast<GLPolylineLayer *>(m_layer);
  if (m_layer == NULL) {
    lineLayer = new GLPolylineLayer(m_view);
    m_layer = lineLayer;
    m_layer->setColor(m_color);
    m_layer->setOpacity(m_opacity);
    m_view->addLayer(m_layer);
  }

  if (m_layer != NULL) {
    lineLayer->setSelectionPoints(m_pts);
    lineLayer->setPointsDisplayable(false);
    m_view->update();
  }
}

bool
LineSelector::finalize(SelectorModifier modifier)
{
  return updateSelectionAndDeleteLayer(modifier);
}

bool
LineSelector::abort() {
  return false;
}

bool
LineSelector::terminate(SelectorModifier /*modifier*/, bool /*closedRegion*/) {
  return false;
}

void
LineSelector::getSegments(std::vector<std::vector<girl::point> > &segments, girl::tools::AABBox &shapeBBox)
{
  assert(m_pts.size() == 2);
  if (m_pts[0] != m_pts[1]) {

    segments.resize(1);
    segments[0].clear();

    segments[0].reserve(2+1);

    girl::point p1 = girl::point(static_cast<int>(m_pts[0].x()), static_cast<int>(m_pts[0].y()));
    girl::point p2 = girl::point(static_cast<int>(m_pts[1].x()), static_cast<int>(m_pts[1].y()));


    segments[0].push_back(p1);
    segments[0].push_back(p2);
    //segments[0].push_back(p1);


    const int xMin = std::min(p1.x(), p2.x());
    const int yMin = std::min(p1.y(), p2.y());
    const int xMax = std::max(p1.x(), p2.x());
    const int yMax = std::max(p1.y(), p2.y());

    shapeBBox = girl::tools::AABBox(xMin, yMin, xMax, yMax);
  }
  else {
    //m_pts[0] == m_pts[1]

    segments.clear();
    shapeBBox = girl::tools::AABBox(m_pts[0].x(), m_pts[0].y(), m_pts[0].x(), m_pts[0].y());
  }
}
