/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef MAINVIEW_HPP
#define MAINVIEW_HPP

#include <map>
#include "SegView.hpp"

#include "GLTogerPainter.hpp" //for DrawMode
#include "GLImageLayer.hpp" //for GLImage::Filter


class QDragEnterEvent;
class QDropEvent;

class GLSelectionLayer;
class GLTogerLayer;




class GIRL_API MainView : public SegView
{
  Q_OBJECT

public:

  MainView(QWidget *parent = NULL, SegModel *model = NULL);
  
  ~MainView();

  /**
   * @warning to nothing to previous set model.
   */
  virtual void setModel(SegModel *model);

  virtual void setImage(const QImage &img);

  virtual void updateView();


  /**
   * @brief Set if region are filled for selection drawing.
   */
  void setSelectionFillEnabled(bool b);

  /**
   * @brief Set if region are filled for selection drawing.
   */
  bool getSelectionFillEnabled() const;

  /**
   * @brief Set color for selection drawing.
   */
  void setSelectionColor(QRgb c);

  /**
   * @brief Get color for selection drawing.
   */
  QRgb getSelectionColor() const;

  /**
   * @brief Set opacity for selection drawing.
   *
   * @warning @a alpha must be in [0.0f; 1.0f].
   */
  void setSelectionOpacity(float alpha);

  /**
   * @brief Get opacity for selection drawing.
   *
   * opacity is in [0.0f; 1.0f].
   */
  float getSelectionOpacity() const;


  void setFalseColorsMode(bool v);
  bool getFalseColorsMode() const;

  void setImageDisplayed(bool v);
  bool getImageDisplayed() const;

  void setTogerDisplayed(bool v);
  bool getTogerDisplayed() const;
  
  void setSelectionDisplayed(bool v);
  bool getSelectionDisplayed() const;

  void setDrawMode(GLTogerPainter::DrawMode dm, const SegModel *model);
  GLTogerPainter::DrawMode getDrawMode() const;

  void setImageSamplingMode(GLImage::Filter f);
  GLImage::Filter getImageSamplingMode() const;

signals:

  void changedRect(const QRectF &modelRectangle, const QRectF &viewRectangleInModel);


protected slots:

  virtual void changeImage(SegModel *);
  virtual void changeToger(SegModel *);
  virtual void changeSelection(SegModel *);
  virtual void changeSelectionType(SegModel *);


  //void acceptInsertion();
  //void cancelInsertion();
  
  


protected:

  void signalChangedRect();

  virtual void resizeGL(int w, int h);

  void setFalseColorImage();

  //void saveSelection(QByteArray &output);
  //void delMovableLayer();

  //virtual void mousePressEvent(QMouseEvent *mouseEvent);
  //virtual void mouseMoveEvent(QMouseEvent *mouseEvent);
  //virtual void dragEnterEvent(QDragEnterEvent *event);
  //virtual void dropEvent(QDropEvent *event);

  //void correctDirs(const girl::point &startPt, const girl::point &endPt, const std::deque<girl::direction> &dirs, girl::pathDeque &paths);


protected:

  GLImageLayer *m_imageLayer;
  GLTogerLayer *m_togerLayer;
  GLSelectionLayer *m_selectionLayer;

  //GLMovableSegmentLayer *m_movableLayer;
  //SegmentInsertionDialog *m_insertionDialog;
  //GLLayerMover *m_layerMover;

  //QPoint m_startDragPosition;
  //QTime m_startDragTime;
  //bool m_dragStarted : 1;

  bool m_falseColorsMode : 1;
  bool m_imageDisplayed : 1;
  bool m_togerDisplayed : 1;
  bool m_selectionDisplayed : 1;
};


#endif /* ! MAINVIEW_HPP */
