/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "MarkWidget.hpp"

#include <QCheckBox>
#include <QHBoxLayout>

static const size_t numMarks = 7; //we reserve the eighth mark for internal use


MarkWidget::MarkWidget(QWidget *parent)
  : QWidget(parent),
    m_marks()
{
  buildGUI();
}
  

void
MarkWidget::buildGUI()
{
  m_marks.resize(numMarks);
  for (size_t i=0; i<numMarks; ++i) {
    m_marks[i] = new QCheckBox(tr("%1").arg(i), this);
    connect(m_marks[i], SIGNAL(stateChanged(int)),
	    this, SIGNAL(markChanged()));
  }

  QHBoxLayout *hbL = new QHBoxLayout();
  for (size_t i=0; i<numMarks; ++i)
    hbL->addWidget(m_marks[i]);
  hbL->addStretch();
  
  setLayout(hbL);

  /*
#if QT_VERSION >= 0x040300
  int left, top, right, bottom;
  getContentsMargins(&left, &top, &right, &bottom);
  setContentsMargins(0, top, 0, bottom);
#endif
  */

}

girl::toger::markFlags
MarkWidget::getMarkFlag() const
{
  girl::toger::markFlags mark = girl::toger::MARK_NONE;
  for (size_t i=0; i<numMarks; ++i) {
    if (m_marks[i]->isChecked())
      mark |= static_cast<girl::toger::markFlag>(1<<i);
  }

  return mark;
}

void
MarkWidget::setMarkFlag(girl::toger::markFlags m)
{
  for (size_t i=0; i<numMarks; ++i) {
    const bool checked = (m & static_cast<girl::toger::markFlag>(1<<i));
    m_marks[i]->setChecked(checked);
  }
}
  
