/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef MARK_WIDGET_HPP
#define MARK_WIDGET_HPP

#include <girl/girlToger.hpp>

#include <QWidget>


class QCheckBox;


class GIRL_API MarkWidget : public QWidget 
{
  Q_OBJECT

public:

  /**
   * @brief Constructor.
   *
   * @a selectionMark is the mark automatically set for selection.
   */
  explicit MarkWidget(QWidget *parent = NULL);
  

  /**
   * @brief Get mark currently set.
   *
   * return OR'ed cobination of marks.
   */
  girl::toger::markFlags getMarkFlag() const;

  /**
   * @brief Set mark.
   *
   * @a markFlags is an OR'ed cobination of marks.
   */
  void setMarkFlag(girl::toger::markFlags);
  
signals:
  
  /**
   * @brief Signal emitted when mark changed.
   */
  void markChanged();

protected:

  void buildGUI();
  
protected:

  std::vector<QCheckBox *>m_marks;

};

#endif /* ! MARK_WIDGET_HPP */
