/*
  Copyright (C) 2010-2011 Cédric Aguerre  aguerre@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "MaxFlowSimilarityWidget.hpp"
#include "WeightedChannelsColorSpaceWidget.hpp"
#include "SliderSpinBoxWidget.hpp"

#include <cassert>

#include <QGroupBox>
#include <QBoxLayout>
#include <QRegExpValidator>
#include <QLabel>
#include <QSpacerItem>


MaxFlowSimilarityWidget::MaxFlowSimilarityWidget(QWidget *parent)
  : QWidget(parent), m_wccsw(NULL), m_threshWidget(NULL), m_thresholdSB(NULL)
{
  buildGUI();
}

void
MaxFlowSimilarityWidget::buildGUI()
{
  QGroupBox *groupBoxCriterion = buildCriterionGUI();
  QGroupBox *groupBoxParameter = buildParameterGUI();

  QFrame *f = new QFrame(this);
  f->setFrameShape(QFrame::VLine);
  
  // create the layout
  QHBoxLayout *layout = new QHBoxLayout;
  //layout->setMargin(0); //because embedded widget
  layout->addWidget(groupBoxCriterion);
  layout->addWidget(f);
  layout->addWidget(groupBoxParameter);
  setLayout(layout);
}

QWidget *
MaxFlowSimilarityWidget::buildThresholdWidget()
{
  QGridLayout *layout2 = new QGridLayout;
  
  //QLabel *distanceLabel = new QLabel(tr("Mean colors distance threshold: "), this);
  m_thresholdLabel = new QLabel(tr("Mean colors distance threshold: "), this);
  m_thresholdSB = new SliderSpinBoxWidget(this);
  m_thresholdSB->setDecimals(10);
  m_thresholdSB->setSingleStep(0.0001);
  m_thresholdSB->setMinimum(0.0);
  m_thresholdSB->setMaximum(1.0);
  //layout2->addWidget(distanceLabel, 2, 0);
  layout2->addWidget(m_thresholdLabel, 2, 0);
  layout2->addWidget(m_thresholdSB, 2, 1);
  
  layout2->setMargin(0); //because embedded widget  
  QWidget *w2 = new QWidget(this);
  w2->setLayout(layout2);

  connect(m_thresholdSB, SIGNAL(valueChanged(double)), this, SIGNAL(parametersChanged()));
  return w2;
}

QGroupBox *
MaxFlowSimilarityWidget::buildParameterGUI()
{
  QGroupBox *groupBoxParameter = new QGroupBox("Parameters");

  m_wccsw = new WeightedChannelsColorSpaceWidget(this);
  m_threshWidget = buildThresholdWidget();

  QVBoxLayout *vboxParameter = new QVBoxLayout;
  vboxParameter->addWidget(m_wccsw);
  vboxParameter->addWidget(m_threshWidget);
  vboxParameter->addStretch(1);

  groupBoxParameter->setLayout(vboxParameter);

  connect(m_wccsw, SIGNAL(parametersChanged()), this, SIGNAL(parametersChanged()));

  return groupBoxParameter;
}

QGroupBox *
MaxFlowSimilarityWidget::buildCriterionGUI()
{
  QGroupBox *groupBoxCriterion = new QGroupBox("Criterion");
  radioSimilarityGrayLevel = new QRadioButton(tr("gray level difference"));
  radioSimilarityColorDistance = new QRadioButton(tr("color similarity (mean)"));
  radioSimilarityColorHomogeneity = new QRadioButton(tr("color homogeneity (variance)"));
  radioSimilarityBhattacharyya = new QRadioButton(tr("Bhattacharyya distance"));
  radioSimilarityHistInter = new QRadioButton(tr("intersection"));
  radioSimilarityHistChiSquare = new QRadioButton(tr("chi-square"));
  radioSimilarityHistExponential = new QRadioButton(tr("exponential"));

  QVBoxLayout *vboxCriterion = new QVBoxLayout;
  vboxCriterion->addWidget(radioSimilarityGrayLevel);
  vboxCriterion->addWidget(radioSimilarityColorDistance);
  vboxCriterion->addWidget(radioSimilarityColorHomogeneity);
  vboxCriterion->addItem(new QSpacerItem(0, 10));
  vboxCriterion->addWidget(new QLabel("Based on RGB color histogram:"));
  vboxCriterion->addWidget(radioSimilarityBhattacharyya);
  vboxCriterion->addWidget(radioSimilarityHistInter);
  vboxCriterion->addWidget(radioSimilarityHistChiSquare);
  vboxCriterion->addWidget(radioSimilarityHistExponential);
  vboxCriterion->addStretch(1);

  groupBoxCriterion->setLayout(vboxCriterion);

  connect(radioSimilarityGrayLevel, SIGNAL(toggled(bool)), this, SLOT(changeSimilarityCriterion(bool)));
  connect(radioSimilarityColorDistance, SIGNAL(toggled(bool)), this, SLOT(changeSimilarityCriterion(bool)));
  connect(radioSimilarityColorHomogeneity, SIGNAL(toggled(bool)), this, SLOT(changeSimilarityCriterion(bool)));
  connect(radioSimilarityBhattacharyya, SIGNAL(toggled(bool)), this, SLOT(changeSimilarityCriterion(bool)));
  connect(radioSimilarityHistInter, SIGNAL(toggled(bool)), this, SLOT(changeSimilarityCriterion(bool)));
  connect(radioSimilarityHistChiSquare, SIGNAL(toggled(bool)), this, SLOT(changeSimilarityCriterion(bool)));
  connect(radioSimilarityHistExponential, SIGNAL(toggled(bool)), this, SLOT(changeSimilarityCriterion(bool)));
  radioSimilarityGrayLevel->setChecked(true);
  radioSimilarityColorDistance->setChecked(false);
  radioSimilarityColorHomogeneity->setChecked(false);
  radioSimilarityBhattacharyya->setChecked(false);
  radioSimilarityHistInter->setChecked(false);
  radioSimilarityHistChiSquare->setChecked(false);
  radioSimilarityHistExponential->setChecked(false);

  return groupBoxCriterion;
}

ise::MinCutMergeAlgorithm::SimilarityCriterion
MaxFlowSimilarityWidget::getSimilarityCriterion()
{
  if (radioSimilarityGrayLevel->isChecked())
    return ise::MinCutMergeAlgorithm::SIMI_GRAY_LEVEL;
  if (radioSimilarityColorDistance->isChecked())
    return ise::MinCutMergeAlgorithm::SIMI_COLOR_SIMILARITY;
  if (radioSimilarityColorHomogeneity->isChecked())
    return ise::MinCutMergeAlgorithm::SIMI_COLOR_HOMOGENEITY;
  if (radioSimilarityBhattacharyya->isChecked())
    return ise::MinCutMergeAlgorithm::SIMI_BHATTACHARYYA;
  if (radioSimilarityHistInter->isChecked())
    return ise::MinCutMergeAlgorithm::SIMI_INTERSECTION;
  if (radioSimilarityHistChiSquare->isChecked())
    return ise::MinCutMergeAlgorithm::SIMI_CHI_SQUARE;
  if (radioSimilarityHistExponential->isChecked())
    return ise::MinCutMergeAlgorithm::SIMI_EXPONENTIAL;

  assert(0);
  return ise::MinCutMergeAlgorithm::SIMI_GRAY_LEVEL;
}

void
MaxFlowSimilarityWidget::setSimilarityCriterion(ise::MinCutMergeAlgorithm::SimilarityCriterion criterion)
{
  if (criterion == ise::MinCutMergeAlgorithm::SIMI_GRAY_LEVEL)
    radioSimilarityGrayLevel->setChecked(true);
  else if (criterion == ise::MinCutMergeAlgorithm::SIMI_COLOR_SIMILARITY)
    radioSimilarityColorDistance->setChecked(true);
  else if (criterion == ise::MinCutMergeAlgorithm::SIMI_COLOR_HOMOGENEITY)
    radioSimilarityColorHomogeneity->setChecked(true);
  else if (criterion == ise::MinCutMergeAlgorithm::SIMI_BHATTACHARYYA)
    radioSimilarityBhattacharyya->setChecked(true);
  else if (criterion == ise::MinCutMergeAlgorithm::SIMI_INTERSECTION)
    radioSimilarityHistInter->setChecked(true);
  else if (criterion == ise::MinCutMergeAlgorithm::SIMI_CHI_SQUARE)
    radioSimilarityHistChiSquare->setChecked(true);
  else if (criterion == ise::MinCutMergeAlgorithm::SIMI_EXPONENTIAL)
    radioSimilarityHistExponential->setChecked(true);
}

int
MaxFlowSimilarityWidget::getColorSpace() const
{
  return m_wccsw->getColorSpace();
}

void
MaxFlowSimilarityWidget::setColorSpace(int cs)
{
  m_wccsw->setColorSpace(cs);
}

void
MaxFlowSimilarityWidget::getNormalizedWeights(std::vector<double> &weights) const
{
  m_wccsw->getNormalizedWeights(weights);
}

void
MaxFlowSimilarityWidget::setNormalizedWeights(const std::vector<double> &weights)
{
  m_wccsw->setNormalizedWeights(weights);
}

double
MaxFlowSimilarityWidget::getThresholdValue() const
{
  return m_thresholdSB->value();
}

void
MaxFlowSimilarityWidget::setThresholdValue(double threshold)
{
  m_thresholdSB->setValue(threshold);
}

/*
 * CALLBACKS
 */

void
MaxFlowSimilarityWidget::changeSimilarityCriterion(bool checked)
{
  if (!checked)
    return;

  if (m_wccsw)
    m_wccsw->setVisible(false);
  if (m_threshWidget)
    m_threshWidget->setVisible(false);

  if (radioSimilarityColorDistance->isChecked()
      || radioSimilarityColorHomogeneity->isChecked()) {
    if (m_wccsw)
      m_wccsw->setVisible(true);
    if (m_threshWidget)
      m_threshWidget->setVisible(true);
    m_thresholdLabel->setText(tr("Mean colors distance threshold: "));
  }
  if (radioSimilarityHistExponential->isChecked()) {
    if (m_threshWidget)
      m_threshWidget->setVisible(true);
    m_thresholdLabel->setText(tr("sigma: "));
  }

  emit(parametersChanged());
}
