/*
  Copyright (C) 2010 Cédric Aguerre  aguerre@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef MAXFLOWSIMILARITYWIDGET_HPP
#define MAXFLOWSIMILARITYWIDGET_HPP

#include <QWidget>
#include <QRadioButton>
#include <QGroupBox>
#include <QLabel>

#include <ise/iseMinCutMergeAlgorithm.hpp>

class WeightedChannelsColorSpaceWidget;
class SliderSpinBoxWidget;

class MaxFlowSimilarityWidget : public QWidget
{
  Q_OBJECT

public:

  explicit MaxFlowSimilarityWidget(QWidget *parent);

  ise::MinCutMergeAlgorithm::SimilarityCriterion getSimilarityCriterion();
  void setSimilarityCriterion(ise::MinCutMergeAlgorithm::SimilarityCriterion criterion);

  int getColorSpace() const;
  void setColorSpace(int cs);

  void getNormalizedWeights(std::vector<double> &weights) const;
  void setNormalizedWeights(const std::vector<double> &weights);

  double getThresholdValue() const;
  void setThresholdValue(double threshold);

signals:
  
  void parametersChanged();

protected slots:

  void changeSimilarityCriterion(bool checked);

protected:
  
  void buildGUI();

private:
  WeightedChannelsColorSpaceWidget *m_wccsw;
  QWidget *m_threshWidget;
  QLabel *m_thresholdLabel;
  SliderSpinBoxWidget *m_thresholdSB;

  QRadioButton *radioSimilarityGrayLevel;
  QRadioButton *radioSimilarityColorDistance;
  QRadioButton *radioSimilarityColorHomogeneity;
  QRadioButton *radioSimilarityBhattacharyya;
  QRadioButton *radioSimilarityHistInter;
  QRadioButton *radioSimilarityHistChiSquare;
  QRadioButton *radioSimilarityHistExponential;

  QWidget *buildThresholdWidget();
  QGroupBox *buildCriterionGUI();
  QGroupBox *buildParameterGUI();
};

#endif /* ! MAXFLOWSIMILARITYWIDGET_HPP */
