/*
  Copyright (C) 2010-2011 Cédric Aguerre  aguerre@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef MAXFLOWSOURCESELECTIONWIDGET_HPP
#define MAXFLOWSOURCESELECTIONWIDGET_HPP

#include <QWidget>
#include <QRadioButton>
#include <QLineEdit>
#include <QGroupBox>

#include <ise/iseMinCutMergeAlgorithm.hpp>
#include "MultiSelectionInteractor.hpp" // TODO: might belong to qise??
#include "MainView.hpp"

#include <qiseTools/SelectionSettingsWidget.hpp>
#include <qiseTools/SelectorOptionsWidget.hpp>
#include <qiseTools/ShapeSelector.hpp>


class MaxFlowSourceSelectionWidget : public QWidget
{
  Q_OBJECT

public:

  explicit MaxFlowSourceSelectionWidget(QWidget *parent);

  void setMainView(MainView *v);

  ise::MinCutMergeAlgorithm::SourceSinkSelectionMethod getMethod();
  void setMethod(ise::MinCutMergeAlgorithm::SourceSinkSelectionMethod method);
 
  void getSourceAndSinkValues(std::vector<long> &sources, std::vector<long> &sinks) const;
  void setSourceAndSinkValues(const std::vector<long> &sources, const std::vector<long> &sinks);

  ise::MinCutMergeAlgorithm::AreaFilter getAreaFilter();
  void setAreaFilter(ise::MinCutMergeAlgorithm::AreaFilter filter);

  long getAreaFilterValue();
  void setAreaFilterValue(long value);

signals:
  
  void parametersChanged();

protected slots:

  void changeMethod(bool checked);
  void editSourceAndSink(const QString &s);
  void editAreaFilter(const QString &s);
  void changeAreaFilter(bool checked);
  void selectParamSourceSink(bool checked);

  void changeSourceColor(QRgb c);
  void changeSourceAlpha(float alpha);
  void changeSourceFill(bool b);

  void changeSinkColor(QRgb c);
  void changeSinkAlpha(float alpha);
  void changeSinkFill(bool b);

  void changeSelectionMode(ShapeSelector::SelectorMode mode);
  void changeSelectionRegionComputer(ShapeSelector::SelectorRegionComputer rc);
  void changeSelectionType(ShapeSelector::SelectorType type);

  void updateSelection();

protected:
  
  void buildGUI();

private:
  MainView *m_view;
  MultiSelectionInteractor *m_interactor;

  SelectorOptionsWidget *m_sow;

  QRadioButton *radioSourceSinkMean;
  QRadioButton *radioSourceSinkManual;
  QRadioButton *radioParamSource;
  QRadioButton *radioParamSink;
  QLineEdit *editSource;
  QLineEdit *editSink;
  SelectionSettingsWidget *sswSource;
  SelectionSettingsWidget *sswSink;
  QWidget *wMethodManual;

  QRadioButton *radioAreaNone;
  QRadioButton *radioAreaAuto;
  QRadioButton *radioAreaManual;
  QLineEdit *editArea;
  QWidget *wAreaManual;

  QGroupBox *buildMethodGUI();
  QGroupBox *buildAreaGUI();
  QGroupBox *buildParameterGUI();
  QWidget *buildSourceSinkManualSettingWidget();
  QWidget *buildAreaFilterManualSettingWidget();
  void showEvent(QShowEvent *event);
  void hideEvent(QHideEvent *event);
};

#endif /* ! MAXFLOWSOURCESELECTIONWIDGET_HPP */
