/*
  Copyright (C) 2010-2011 Cédric Aguerre  aguerre@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "MergeAlgorithmMinCutWidget.hpp"

#include <cassert>

#include <ise/iseMinCutMergeAlgorithm.hpp>

#include <QGridLayout>
#include <QHBoxLayout>
#include <QTabWidget>

#include "MergeAlgorithmPlugin.hpp"
#include "MaxFlowSourceSelectionWidget.hpp"
#include "MaxFlowSimilarityWidget.hpp"


MergeAlgorithmMinCutWidget::MergeAlgorithmMinCutWidget(MergeAlgorithmPlugin *mergeAlgorithmPlugin, QWidget *parent)
  : MergeAlgorithmWidgetInterface(mergeAlgorithmPlugin, parent)
{
  setWindowTitle(tr("MergeAlgorithmMinCut parameters..."));

  //setAttribute(Qt::WA_DeleteOnClose);

  buildGUI();
}

void
MergeAlgorithmMinCutWidget::setMainView(MainView *v)
{
  MergeAlgorithmWidgetInterface::setMainView(v);
  m_sourceSelectionW->setMainView(v);
}

void
MergeAlgorithmMinCutWidget::buildGUI()
{
  m_sourceSelectionW = new MaxFlowSourceSelectionWidget(this);
  m_similarityW = new MaxFlowSimilarityWidget(this);

  // tab widget
  QTabWidget *tabWidget = new QTabWidget(this);
  tabWidget->addTab(m_sourceSelectionW, tr("Source and sink"));
  tabWidget->addTab(m_similarityW, tr("Similarity criterion"));

  // finalize
  QVBoxLayout *vL = new QVBoxLayout;
  vL->addWidget(tabWidget);
  vL->setMargin(0); //because it aims to be an embedded widget.
  setLayout(vL);

  connect(m_sourceSelectionW, SIGNAL(parametersChanged()), this, SIGNAL(parametersChanged()));
  connect(m_similarityW, SIGNAL(parametersChanged()), this, SIGNAL(parametersChanged()));
}

void
MergeAlgorithmMinCutWidget::getParameters(ise::MinCutMergeAlgorithm::SourceSinkSelectionMethod &method, std::vector<long> &sourceIds, std::vector<long> &sinkIds, ise::MinCutMergeAlgorithm::AreaFilter &filter, long &filterValue, ise::MinCutMergeAlgorithm::SimilarityCriterion &criterion, int &conversion, std::vector<double> &weights, double &threshold) const
{
  method =  m_sourceSelectionW->getMethod();
  m_sourceSelectionW->getSourceAndSinkValues(sourceIds, sinkIds);
  filter = m_sourceSelectionW->getAreaFilter();
  filterValue = m_sourceSelectionW->getAreaFilterValue();
  criterion = m_similarityW->getSimilarityCriterion();
  conversion = m_similarityW->getColorSpace();
  m_similarityW->getNormalizedWeights(weights);
  threshold = m_similarityW->getThresholdValue();
}

void
MergeAlgorithmMinCutWidget::setParameters(ise::MinCutMergeAlgorithm::SourceSinkSelectionMethod method, const std::vector<long> &sourceIds, const std::vector<long> &sinkIds, ise::MinCutMergeAlgorithm::AreaFilter filter, long filterValue, ise::MinCutMergeAlgorithm::SimilarityCriterion criterion, int conversion, const std::vector<double> &weights, double threshold)
{
  m_similarityW->setThresholdValue(threshold);
  m_similarityW->setColorSpace(conversion);
  m_similarityW->setNormalizedWeights(weights);
  m_similarityW->setSimilarityCriterion(criterion);
  m_sourceSelectionW->setAreaFilterValue(filterValue);
  m_sourceSelectionW->setAreaFilter(filter);
  m_sourceSelectionW->setMethod(method);
  m_sourceSelectionW->setSourceAndSinkValues(sourceIds, sinkIds);
}
