/*
  Copyright (C) 2011 Cedric Aguerre  aguerre@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef MERGEALGORITHMMINCUTWIDGETSETTINGS_HPP
#define MERGEALGORITHMMINCUTWIDGETSETTINGS_HPP

#include <qiseTools/MergeAlgorithmPlugin.hpp>

#include "MergeAlgorithmMinCutWidget.hpp"
#include <qiseTools/MergeAlgorithmWidgetSettingsInterface.hpp>
#include <ise/iseMinCutMergeAlgorithm.hpp>

static const double DEFAULT_THRESHOLD = 0.5;
static const int DEFAULT_CONVERSION = ISE_RGB_to_K1K2K3;

class MergeAlgorithmMinCutWidgetSettings : public MergeAlgorithmWidgetSettingsInterface {
  
public:
  
  MergeAlgorithmMinCutWidgetSettings()
    : MergeAlgorithmWidgetSettingsInterface(),
      m_method(ise::MinCutMergeAlgorithm::SOURCE_SINK_REGION_MEAN),
      m_sourceIds(),
      m_sinkIds(),
      m_filter(ise::MinCutMergeAlgorithm::AREA_NONE),
      m_filterValue(0),
      m_criterion(ise::MinCutMergeAlgorithm::SIMI_COLOR_SIMILARITY),
      m_conversion(DEFAULT_CONVERSION),
      m_weights(),
      m_threshold(DEFAULT_THRESHOLD)
  {
    m_sourceIds = std::vector<long>(1, -1);
    m_sinkIds = std::vector<long>(1, -1);
    m_weights = std::vector<double>(3, 1.0/3.0);
  }

  virtual void configureWidget(MergeAlgorithmWidgetInterface *widget) const
  {
    static_cast<MergeAlgorithmMinCutWidget*>(widget)->setParameters(m_method, m_sourceIds, m_sinkIds, m_filter, m_filterValue, m_criterion, m_conversion, m_weights, m_threshold);
  }
  
  virtual void getWidgetConfig(const MergeAlgorithmWidgetInterface *widget)
  {
    static_cast<const MergeAlgorithmMinCutWidget*>(widget)->getParameters(m_method, m_sourceIds, m_sinkIds, m_filter, m_filterValue, m_criterion, m_conversion, m_weights, m_threshold);
  }

  virtual void configureAlgo(girl::shared_ptr<ise::MergeAlgorithm> algo) const
  {
    girl::shared_ptr<ise::MinCutMergeAlgorithm> algorithm = girl::static_pointer_cast<ise::MinCutMergeAlgorithm>(algo);
    algorithm->setParameters(m_method, m_sourceIds, m_sinkIds, m_filter, m_filterValue, m_criterion, m_conversion, m_weights, m_threshold);
  }

  virtual girl::shared_ptr<MergeAlgorithmWidgetSettingsInterface> clone() const
  {
    girl::shared_ptr<MergeAlgorithmMinCutWidgetSettings> p(new MergeAlgorithmMinCutWidgetSettings);

    *p = *this;

    return p;
  }

  virtual bool isValid() const
  {
    return true; // TODO: test member variables
  }
  
private:

  ise::MinCutMergeAlgorithm::SourceSinkSelectionMethod m_method;
  std::vector<long> m_sourceIds;
  std::vector<long> m_sinkIds;
  ise::MinCutMergeAlgorithm::AreaFilter m_filter;
  long m_filterValue;
  ise::MinCutMergeAlgorithm::SimilarityCriterion m_criterion;
  int m_conversion;
  std::vector<double> m_weights;
  double m_threshold;
};

#endif /* ! MERGEALGORITHMMINCUTWIDGETSETTINGS_HPP */
