/* 
 * Copyright (C) 2010-2011 Boris Mansencal  boris.mansencal@labri.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation (Inc., 59
 * Temple Place - Suite 330, Boston, MA 02111-1307, USA); either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
#ifndef MERGEALGORITHMPLUGIN_HPP
#define MERGEALGORITHMPLUGIN_HPP

#include "QIsePlugin.hpp"

#include <QString>

#include <ise/iseMergeAlgorithm.hpp>

#include <qiseTools/MergeAlgorithmFactory.hpp>
#include <qiseTools/MergeAlgorithmSettings.hpp>

class MergeAlgorithmWidgetInterface;


class MergeAlgorithmPlugin : public QIsePlugin,
			     public MergeAlgorithmFactory
{
  
public:
  
  virtual ~MergeAlgorithmPlugin()
  {}
  
  virtual girl::shared_ptr<ise::MergeAlgorithm> getNewMergeAlgorithm(ise::SegmentationContext *segContext)
  {
    girl::shared_ptr<ise::MergeAlgorithm> a = create(segContext);
    girl::shared_ptr<MergeAlgorithmSettings> s = settings();
    if (s)
      s->configureAlgo(a);
    return a;

  }
  
  virtual MergeAlgorithmWidgetInterface *parameterWidget(QWidget *parent = NULL)
  {
    MergeAlgorithmWidgetInterface *w = createWidget(m_segContext, parent);
    girl::shared_ptr<MergeAlgorithmSettings> s = settings();
    if (s)
      s->configureWidget(w);
    return w;
  }
  
  void setSettings(girl::shared_ptr<MergeAlgorithmSettings> settings)
  {
    if (settings)
      m_settings = settings->clone();
    else
      m_settings = girl::shared_ptr<MergeAlgorithmSettings>();
  }  

  girl::shared_ptr<MergeAlgorithmSettings> getSettings()
  {
    if (settings()) 
      return m_settings->clone();
    return girl::shared_ptr<MergeAlgorithmSettings>();
  }  

  inline void setSegContext(ise::SegmentationContext *segContext)
  {
    m_segContext = segContext;
  }
  
  inline ise::SegmentationContext *getSegContext()
  {
    return m_segContext;
  }

  virtual girl::shared_ptr<ise::MergeAlgorithm>
    create(ise::SegmentationContext *segContext) = 0;

  virtual girl::shared_ptr<MergeAlgorithmSettings>
    createSettings(ise::SegmentationContext* segContext) = 0;

  virtual bool hasSettings() const
  {
    return true;
  }

protected:
  
  MergeAlgorithmPlugin(const QString &name,
		       const QIseVersion &pluginVersion,
		       const QIseVersion &qiseVersion,
		       QString path = QString::fromUtf8(""))
    : QIsePlugin(name, (QString::fromUtf8("MergeAlgorithm/") + path),
		 pluginVersion, qiseVersion),
      m_segContext(NULL)
  {}
  
   virtual MergeAlgorithmWidgetInterface *
    createWidget(ise::SegmentationContext *segContext, QWidget *parent) = 0;
 
  girl::shared_ptr<MergeAlgorithmSettings> settings()
  {
    if (!m_settings)
      m_settings = createSettings(m_segContext);
    return m_settings;    
  }

private:
  
  MergeAlgorithmPlugin();

protected:

  girl::shared_ptr<MergeAlgorithmSettings> m_settings;
  ise::SegmentationContext *m_segContext;

};

#define MergeAlgorithmPlugin_iid "org.girl.qise.MergeAlgorithmPlugin/1.0"
Q_DECLARE_INTERFACE(MergeAlgorithmPlugin, MergeAlgorithmPlugin_iid)

#endif /* ! MERGEALGORITHMPLUGIN_HPP */
