/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef MERGEALGORITHMWIDGETINTERFACE_HPP
#define MERGEALGORITHMWIDGETINTERFACE_HPP

#include <girl/girlCommon.hpp>//GIRL_API
#include <QWidget>

#include "MergeAlgorithmPlugin.hpp"
#include "MergeAlgorithmSettings.hpp"
class MainView;

class GIRL_API MergeAlgorithmWidgetInterface : public QWidget
{
  Q_OBJECT

public:

  MergeAlgorithmWidgetInterface(MergeAlgorithmPlugin *mergeAlgorithmPlugin = NULL, QWidget *parent = NULL)
    : QWidget(parent),
    m_mergeAlgorithmPlugin(mergeAlgorithmPlugin),
    m_view(NULL)
  {}

  virtual void setMergeAlgorithm(MergeAlgorithmPlugin *mergeAlgorithmPlugin)
  {
    m_mergeAlgorithmPlugin = mergeAlgorithmPlugin;
  }
  
  virtual MergeAlgorithmPlugin *getMergeAlgorithmPlugin()
  {
    return m_mergeAlgorithmPlugin;
  }

  virtual void setMainView(MainView *v)
  {
    m_view = v;
  }

  virtual void getGUI(girl::shared_ptr<MergeAlgorithmSettings> settings) const
  {
    if (settings)
      settings->getWidgetConfig(this);
  }

  virtual void setGUI(const girl::shared_ptr<MergeAlgorithmSettings> settings)
  {
    if (settings)
      settings->configureWidget(this);
  }

  virtual bool saveSettingsInPlugin()
  {
    if (m_mergeAlgorithmPlugin) {
      if (m_mergeAlgorithmPlugin->hasSettings()) {
	girl::shared_ptr<MergeAlgorithmSettings> s = m_mergeAlgorithmPlugin->getSettings();
	getGUI(s);
	assert(s);
	if (s->isValid()) {
	  m_mergeAlgorithmPlugin->setSettings(s);
	  return true;
	}
      }
      else {
	//no settings
	return true;
      }
    }
    return false;
  }

  virtual bool isValid() const
  {
    if (m_mergeAlgorithmPlugin) {
      if (m_mergeAlgorithmPlugin->hasSettings()) {
	girl::shared_ptr<MergeAlgorithmSettings> s = m_mergeAlgorithmPlugin->getSettings();
	getGUI(s);
	assert(s);
	if (s->isValid()) {
	  return true;
	}
      }
      else {
	//no settings
	return true;
      }
    }
    return false;
  }

signals:

  /**
   * emitted when one of the parameters of widget changes.
   */
  void parametersChanged();

protected:
  
  MergeAlgorithmPlugin *m_mergeAlgorithmPlugin;
  MainView *m_view;
};


#endif /* ! MERGEALGORITHMWIDGETINTERFACE_HPP */
