/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "MergeDriverColorHomogeneityWidget.hpp"

#include <cassert>

#include <ise/iseMergeDriverColorHomogeneity.hpp>
#include <ise/iseSegmentationContext.hpp>

#include <QGridLayout>
#include <QHBoxLayout>
#include <QLabel>


#include "MergeDriverFactory.hpp"
#include "MergeDriverWidget.hpp"
#include "RegionFeatureChoiceWidget.hpp"
#include "SliderSpinBoxWidget.hpp"
#include "WeightedChannelsColorSpaceWidget.hpp"


MergeDriverColorHomogeneityWidget::MergeDriverColorHomogeneityWidget(ise::SegmentationContext *segContext,
    MergeDriverFactory *mergeDriverFactory,
    QWidget *parent)
  :
  MergeDriverWidgetInterface(segContext, mergeDriverFactory, parent),
  m_mergeDriverWidget(NULL),
  m_rfcw(NULL),
  m_thresholdSB(NULL)
{
  assert(m_mergeDriverFactory);

  setWindowTitle(tr("MergeDriverColorHomogeneity parameters..."));

  //setAttribute(Qt::WA_DeleteOnClose);

  buildGUI();
  
  setGUI();
}

void
MergeDriverColorHomogeneityWidget::buildGUI()
{

  m_mergeDriverWidget = new MergeDriverWidget(this);

  //m_wccsw = new WeightedChannelsColorSpaceWidget(this);
  m_rfcw = new RegionFeatureChoiceWidget(m_segContext, this);

  QGridLayout *layout2 = new QGridLayout;
  
  QLabel *distanceLabel = new QLabel(tr("Distance threshold: "), this);
  m_thresholdSB = new SliderSpinBoxWidget(this);
  m_thresholdSB->setDecimals(10);
  m_thresholdSB->setSingleStep(0.0001);
  m_thresholdSB->setMinimum(0.0);
  m_thresholdSB->setMaximum(1.0);
  layout2->addWidget(distanceLabel, 2, 0);
  layout2->addWidget(m_thresholdSB, 2, 1);
  
  layout2->setMargin(0); //because embedded widget
  QWidget *w2 = new QWidget(this);
  w2->setLayout(layout2);

  QFrame *f = new QFrame(this);
  f->setFrameShape(QFrame::HLine);
  
  

  QVBoxLayout *vLayout = new QVBoxLayout();
  vLayout->addWidget(m_mergeDriverWidget);
  vLayout->addWidget(m_rfcw);
  vLayout->addWidget(w2);
  vLayout->addWidget(f);

  setLayout(vLayout);


  connect(m_mergeDriverWidget, SIGNAL(parametersChanged()),
	  this, SIGNAL(parametersChanged()));
  connect(m_rfcw, SIGNAL(parametersChanged()),
	  this, SIGNAL(parametersChanged()));
  connect(m_thresholdSB, SIGNAL(valueChanged(double)),
	  this, SIGNAL(parametersChanged()));

}

void
MergeDriverColorHomogeneityWidget::setGUI()
{
  assert(m_mergeDriverFactory);

  girl::toger::neighbouringModes mode;
  girl::toger::markFlags m;
  girl::shared_ptr<ise::MergeDriverColorHomogeneity> algo = girl::static_pointer_cast<ise::MergeDriverColorHomogeneity>(m_mergeDriverFactory->getNewMergeDriver());
  
  algo->getNeighbourhood(mode, m);
  m_mergeDriverWidget->setNeighbouringMode(mode);
  m_mergeDriverWidget->setMarkFlag(m);


  double threshold;
  std::string regSetUri;
  algo->getParameters(threshold, regSetUri);

  m_thresholdSB->setValue(threshold);
  m_rfcw->setRegionFeatureInfo(regSetUri, 1.0);
//  m_wccsw->setColorSpace(conversion);
//  m_wccsw->setNormalizedWeights(normalizedWeightsForChannels);
}

void
MergeDriverColorHomogeneityWidget::getGUI()
{
  assert(m_mergeDriverFactory);

  const girl::toger::neighbouringModes mode = m_mergeDriverWidget->getNeighbouringMode();
  const girl::toger::markFlags m = m_mergeDriverWidget->getMarkFlag();
  girl::shared_ptr<ise::MergeDriverColorHomogeneity> algo = girl::static_pointer_cast<ise::MergeDriverColorHomogeneity>(m_mergeDriverFactory->getNewMergeDriver());
  algo->setNeighbourhood(mode, m);
  
  const double threshold = m_thresholdSB->value();
  std::vector<std::string> uris;
  std::vector<double> weights;
  m_rfcw->getRegionFeaturesInfo(uris, weights);
  if (! uris.empty()) {
    assert(uris.size() == 1);
    assert(weights.size() == 1);
    algo->setParameters(threshold, uris[0]);
    m_mergeDriverFactory->saveParameters(algo);
  }
}
