/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef MERGEDRIVERGEOCOLORWIDGET_HPP
#define MERGEDRIVERGEOCOLORWIDGET_HPP

#include <map>

#include <ise/iseMergeDriverGeoColor.hpp>

#include "MergeDriverWidgetInterface.hpp"

namespace ise {
  class SegmentationContext;
}

class QCheckBox;
class QGridLayout;
class QLabel;
class QSpinBox;
class QString;

class MergeDriverWidget;
class RegionFeatureChoiceWidget;
class SliderSpinBoxWidget;
class WeightedChannelsColorSpaceWidget;
//class AreaIntervalsWidget;

class GLSelectionLayer;


class GIRL_API MergeDriverGeoColorWidget : public MergeDriverWidgetInterface
{
  Q_OBJECT

public:
  MergeDriverGeoColorWidget(ise::SegmentationContext *segContext,
                            MergeDriverFactory *mergeDriver,
			    QWidget *parent = NULL);

  
  virtual void setGUI();
  virtual void getGUI();

protected slots:

  void updateCorrespondingSB(double);

  void sortChangeState(int newValue);
  void sortChangeValue(int newValue);
  void sortChangeValue(double newValue);
  

  static void initMapNames();

protected:

  void buildGUI();

  QWidget *buildProcessGUI(QWidget *parent);
  QWidget *buildColorSpacesGUI(QWidget *parent);
  QWidget *buildSortsGUI(QWidget *parent);

  void buildSortItems();
  void buildSort(size_t numSorts,
		 ise::MergeDriverGeoColor::SortType sortType,
		 const QString &str,
		 double mergeThreshold,
		 double processThreshold,
		 double minValue,
		 double maxValue);

  void restoreView();

  void connectSlots(int i);
  void disconnectSlots(int i);
  void swapSorts(int i1, int i2);


protected:

  typedef std::map<ise::MergeDriverGeoColor::SortType, QString> MapSortTypeName;
  static  MapSortTypeName ms_mapSortTypeName;

  struct SortWidgets
  {
    SortWidgets(ise::MergeDriverGeoColor::SortType sortType,
		QCheckBox *cb = NULL,
		QSpinBox *sb = NULL,
		QLabel *lab = NULL,
		SliderSpinBoxWidget *mT = NULL,
		SliderSpinBoxWidget *pT = NULL)
      : m_sortType(sortType),
	m_enableCB(cb),
	m_rankSB(sb),
	m_label(lab),
	m_mergeThresholdSB(mT),
	m_processThresholdSB(pT)
    {}


    ise::MergeDriverGeoColor::SortType m_sortType;
    QCheckBox *m_enableCB;
    QSpinBox *m_rankSB;
    QLabel *m_label;
    SliderSpinBoxWidget *m_mergeThresholdSB;
    SliderSpinBoxWidget *m_processThresholdSB;
    
  };

  QList< SortWidgets > m_sortWidgetsList;
  

  QGridLayout *m_sortGLayout;
				     

  MergeDriverWidget *m_mergeDriverWidget;

  WeightedChannelsColorSpaceWidget *m_wccswForGradient;
//  WeightedChannelsColorSpaceWidget *m_wccswForColor;
  RegionFeatureChoiceWidget *m_rfcw;


  SliderSpinBoxWidget *m_geoSB;
  SliderSpinBoxWidget *m_colorSB;

  SliderSpinBoxWidget *m_gradientSB;
  SliderSpinBoxWidget *m_curvatureSB;
  
  SliderSpinBoxWidget *m_distanceSB;
  SliderSpinBoxWidget *m_homogeneitySB;

  std::map<SliderSpinBoxWidget *, SliderSpinBoxWidget *> m_sbMap;

  SliderSpinBoxWidget *m_thresholdSB;


  GLSelectionLayer *m_selectionLayer;
  bool m_wasSelectionDisplayed;

};





#endif /* ! MERGEDRIVERGEOCOLORWIDGET_HPP */
