/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux boris.mansencal@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef MERGEDRIVERPLUGIN_HPP
#define MERGEDRIVERPLUGIN_HPP

#include "QIsePlugin.hpp"

#include <QString>

#include <ise/iseMergeDriver.hpp>

//#include <qiseTools/MergeDriverFactory.hpp>
class MergeDriverWidgetInterface;

#include <qiseTools/MergeDriverSettings.hpp>

class MergeDriverWidgetInterface;
class MainView;

class GIRL_API MergeDriverPlugin : public QIsePlugin, public ise::MergeDriverFactory
{
  
public:
  
  virtual ~MergeDriverPlugin()
  {}

  girl::shared_ptr<ise::MergeDriver>
    getNewMergeDriver(ise::SegmentationContext *segContext)
  {
    girl::shared_ptr<ise::MergeDriver> md = create(segContext);
    girl::shared_ptr<MergeDriverSettings> s = settings();
    if (s)
      s->configureAlgo(md);
    return md;
  }

  virtual MergeDriverWidgetInterface *parameterWidget(MainView *mainView, QWidget* parent)
  {
    MergeDriverWidgetInterface *w = createWidget(mainView, parent);
    girl::shared_ptr<MergeDriverSettings> s = settings();
    if (s)
      s->configureWidget(w);
    return w;
  }

  void setSettings(girl::shared_ptr<MergeDriverSettings> settings)
  {
    if (settings)
      m_settings = settings->clone();
    else
      m_settings = girl::shared_ptr<MergeDriverSettings>();
  }  

  girl::shared_ptr<MergeDriverSettings> getSettings()
  {
    if (settings()) 
      return m_settings->clone();
    return girl::shared_ptr<MergeDriverSettings>();
  }  

  void setSegContext(ise::SegmentationContext *segContext)
  {
    m_segContext = segContext;
  }
  
  ise::SegmentationContext *getSegContext()
  {
    return m_segContext;
  }
  
  virtual girl::shared_ptr<ise::MergeDriver>
    create(ise::SegmentationContext *segContext) = 0;

  virtual girl::shared_ptr<MergeDriverSettings>
  createSettings(ise::SegmentationContext* segContext) = 0;

  virtual bool hasSettings() const
  {
    return true;
  }
  
protected:
  
  MergeDriverPlugin(const QString &name,
		    const QIseVersion &pluginVersion,
		    const QIseVersion &qiseVersion,
		    QString path = QString::fromUtf8(""))
    : QIsePlugin(name, (QString::fromUtf8("MergeDriver/") + path),
		 pluginVersion, qiseVersion),
    m_segContext(NULL),
    m_settings()
  {
  }

  virtual MergeDriverWidgetInterface *createWidget(MainView* mainView,
						   QWidget *parent) = 0;
  
  girl::shared_ptr<MergeDriverSettings>
  settings()
  {
    if (!m_settings)
      m_settings = createSettings(m_segContext);
    return m_settings;
  }

  
private:
  
  MergeDriverPlugin();

protected:

  ise::SegmentationContext *m_segContext;

  girl::shared_ptr<MergeDriverSettings> m_settings;
};

#define MergeDriverPlugin_iid "org.girl.qise.MergeDriverPlugin/1.0"
Q_DECLARE_INTERFACE(MergeDriverPlugin, MergeDriverPlugin_iid)

#endif /* ! MERGEDRIVERPLUGIN_HPP */
