/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux boris.mansencal@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef MERGEDRIVERSETTINGS_HPP_
#define MERGEDRIVERSETTINGS_HPP_

#include <ise/iseMergeDriver.hpp>

class MergeDriverWidgetInterface;

class GIRL_API MergeDriverSettings
{
 public:
  
  MergeDriverSettings()
    : m_neighbourhoodMode(girl::toger::NEIGHBOUR_ANY),
    m_neighbourhoodMark(girl::toger::MARK_NONE)
    {}

  virtual ~MergeDriverSettings() {}
  
  virtual void configureWidget(MergeDriverWidgetInterface *widget) const;
  
  virtual void getWidgetConfig(const MergeDriverWidgetInterface *widget);

  virtual void configureAlgo(girl::shared_ptr<ise::MergeDriver> algo) const;

  virtual girl::shared_ptr<MergeDriverSettings> clone() const = 0;

  virtual bool isValid() const = 0;

  
  void setNeighbouringModes(girl::toger::neighbouringModes neighbourhoodMode)
  {
    m_neighbourhoodMode = neighbourhoodMode;
  }

  girl::toger::neighbouringModes getNeighbouringModes()
  {
    return m_neighbourhoodMode;
  }

  void setMarkFlags(girl::toger::markFlags m)
  {
    m_neighbourhoodMark = m;
  }

  girl::toger::markFlags getMarkFlags()
  {
    return m_neighbourhoodMark;
  }
    
 protected:

  girl::toger::neighbouringModes m_neighbourhoodMode;
  girl::toger::markFlags m_neighbourhoodMark;

};

#endif /* MERGEDRIVERSETTINGS_HPP_ */
