/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "MergeDriverSimpleWidget.hpp"

#include <ise/iseMergeDriverDefault.hpp>

#include <QBoxLayout>

#include "MergeDriverPlugin.hpp"
#include "MergeDriverWidget.hpp"


MergeDriverSimpleWidget::MergeDriverSimpleWidget(MainView *mainView,
                                                 MergeDriverPlugin *mergeDriverPlugin,
						 QWidget *parent)
  : MergeDriverWidgetInterface(mainView, mergeDriverPlugin, parent),
    m_mergeDriverWidget(NULL)
{
  assert(m_mergeDriverPlugin);

  setWindowTitle(tr("MergeDriverDefault parameters..."));

  //setAttribute(Qt::WA_DeleteOnClose);

  buildGUI();

  setGUI();
}

void
MergeDriverSimpleWidget::buildGUI()
{
  m_mergeDriverWidget = new MergeDriverWidget(this);
  
  QVBoxLayout *vLayout = new QVBoxLayout();
  vLayout->addWidget(m_mergeDriverWidget);
  
  setLayout(vLayout);  

  connect(m_mergeDriverWidget, SIGNAL(parametersChanged()),
	  this, SIGNAL(parametersChanged()));
}

void
MergeDriverSimpleWidget::setGUI()
{
  assert(m_mergeDriverPlugin);

  girl::toger::neighbouringModes mode;
  girl::toger::markFlags m;
  girl::shared_ptr<ise::MergeDriverDefault> algo = girl::static_pointer_cast<ise::MergeDriverDefault>(m_mergeDriverPlugin->getNewMergeDriver(m_mergeDriverPlugin->getSegContext()));
  algo->getNeighbourhood(mode, m);
  
  m_mergeDriverWidget->setNeighbouringMode(mode);
  m_mergeDriverWidget->setMarkFlag(m);
}

void
MergeDriverSimpleWidget::getGUI()
{
  assert(m_mergeDriverPlugin);

  const girl::toger::neighbouringModes mode = m_mergeDriverWidget->getNeighbouringMode();
  const girl::toger::markFlags m = m_mergeDriverWidget->getMarkFlag();
  
  girl::shared_ptr<ise::MergeDriverDefault> algo = girl::static_pointer_cast<ise::MergeDriverDefault>(m_mergeDriverPlugin->getNewMergeDriver(m_mergeDriverPlugin->getSegContext()));
  algo->setNeighbourhood(mode, m);
  
  //m_mergeDriverPlugin->saveParameters(algo);
}


