/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "MergeDriverThresholdsWidget.hpp"

#include <ise/iseMergeDriverThresholds.hpp>

#include <QBoxLayout>
#include <QLabel>
#include <QSpinBox>

#include "MergeDriverFactory.hpp"
#include "MergeDriverWidget.hpp"


MergeDriverThresholdsWidget::MergeDriverThresholdsWidget(ise::SegmentationContext *segContext,
                                                         MergeDriverFactory *mergeDriverFactory,
							 QWidget *parent)
  : MergeDriverWidgetInterface(segContext, mergeDriverFactory, parent),
    m_mergeDriverWidget(NULL)
{
  assert(m_mergeDriverFactory);

  setWindowTitle(tr("MergeDriverThresholds parameters..."));

  //setAttribute(Qt::WA_DeleteOnClose);

  buildGUI();

  setGUI();
}

void
MergeDriverThresholdsWidget::buildGUI()
{
  //TODO: take a color space
  // (here we have no idea what maximum/minimum of each spinbox is)
  //TODO: take any number of thresholds

  m_mergeDriverWidget = new MergeDriverWidget(this);

  QLabel *thres1Label = new QLabel(tr("Threshold 1: "), this);
  m_thres1SB = new QSpinBox(this);
  QHBoxLayout *hLayout1 = new QHBoxLayout;
  hLayout1->addWidget(thres1Label);
  hLayout1->addWidget(m_thres1SB);
  hLayout1->addStretch();

  QLabel *thres2Label = new QLabel(tr("Threshold 2: "), this);
  m_thres2SB = new QSpinBox(this);
  QHBoxLayout *hLayout2 = new QHBoxLayout;
  hLayout2->addWidget(thres2Label);
  hLayout2->addWidget(m_thres2SB);
  hLayout2->addStretch();

  QVBoxLayout *vLayout1 = new QVBoxLayout();
  vLayout1->addItem(hLayout1);
  vLayout1->addItem(hLayout2);
  QWidget *w = new QWidget(this);
  w->setLayout(vLayout1);

  
  QVBoxLayout *vLayout = new QVBoxLayout();
  vLayout->addWidget(m_mergeDriverWidget);
  vLayout->addWidget(w);
  
  setLayout(vLayout);  


  connect(m_thres1SB, SIGNAL(valueChanged(int)),
	  this, SIGNAL(parametersChanged()));
  connect(m_thres2SB, SIGNAL(valueChanged(int)),
	  this, SIGNAL(parametersChanged()));

}

void
MergeDriverThresholdsWidget::setGUI()
{
  assert(m_mergeDriverFactory);

  girl::toger::neighbouringModes mode;
  girl::toger::markFlags m;
  
  girl::shared_ptr<ise::MergeDriverThresholds> algo = girl::static_pointer_cast<ise::MergeDriverThresholds>(m_mergeDriverFactory->getNewMergeDriver());
  algo->getNeighbourhood(mode, m);
  
  m_mergeDriverWidget->setNeighbouringMode(mode);
  m_mergeDriverWidget->setMarkFlag(m);

  int th1, th2;
  algo->getThresholds(th1, th2);
  m_thres1SB->setValue(th1);
  m_thres2SB->setValue(th2);
}

void
MergeDriverThresholdsWidget::getGUI()
{
  assert(m_mergeDriverFactory);

  const girl::toger::neighbouringModes mode = m_mergeDriverWidget->getNeighbouringMode();
  const girl::toger::markFlags m = m_mergeDriverWidget->getMarkFlag();
  girl::shared_ptr<ise::MergeDriverThresholds> algo = girl::static_pointer_cast<ise::MergeDriverThresholds>(m_mergeDriverFactory->getNewMergeDriver());
  algo->setNeighbourhood(mode, m);

  const int th1 = m_thres1SB->value();
  const int th2 = m_thres2SB->value();
  algo->setThresholds(th1, th2);

  m_mergeDriverFactory->saveParameters(algo);
}

