/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "MergeDriverWidget.hpp"

#include <QBoxLayout>
#include <QCheckBox>
#include <QGroupBox>

#include "MarkWidget.hpp"



MergeDriverWidget::MergeDriverWidget(QWidget *parent)
  :
  QWidget(parent),
  m_direct(NULL),
  m_inner(NULL),
  m_outer(NULL),
  m_mw(NULL)
{

  buildGUI();

}

void
MergeDriverWidget::buildGUI()
{
  QGroupBox *nbox = new QGroupBox(tr("Neighbouring mode"), this);
  m_direct = new QCheckBox(tr("Direct"), this);
  m_inner = new QCheckBox(tr("Inner"), this);
  m_outer = new QCheckBox(tr("Outer"), this);
  QHBoxLayout *hLayout = new QHBoxLayout;
  hLayout->addWidget(m_direct);
  hLayout->addWidget(m_inner);
  hLayout->addWidget(m_outer);
  hLayout->addStretch();
  nbox->setLayout(hLayout);

  QGroupBox *mbox = new QGroupBox(tr("Marks"), this);
  m_mw = new MarkWidget(this);
  QHBoxLayout *hLayout2 = new QHBoxLayout;
  hLayout2->addWidget(m_mw);
  hLayout2->addStretch();
  mbox->setLayout(hLayout2);

  QVBoxLayout *vLayout = new QVBoxLayout;
  vLayout->addWidget(nbox);
  vLayout->addWidget(mbox);
  vLayout->setMargin(0); //because it aims to be an embedded widget.
  setLayout(vLayout);



  connect(m_direct, SIGNAL(stateChanged(int)),
	  this, SIGNAL(parametersChanged()));
  connect(m_inner, SIGNAL(stateChanged(int)),
	  this, SIGNAL(parametersChanged()));
  connect(m_outer, SIGNAL(stateChanged(int)),
	  this, SIGNAL(parametersChanged()));

  connect(m_mw, SIGNAL(markChanged()),
	  this, SIGNAL(parametersChanged()));

  
}

girl::toger::neighbouringModes
MergeDriverWidget::getNeighbouringMode() const
{
  girl::toger::neighbouringModes neighbouringMode = girl::toger::NEIGHBOUR_NONE;
  if (m_direct->isChecked())
    neighbouringMode |= girl::toger::NEIGHBOUR_DIRECT;
  if (m_inner->isChecked())
    neighbouringMode |= girl::toger::NEIGHBOUR_INNER;
  if (m_outer->isChecked())
    neighbouringMode |= girl::toger::NEIGHBOUR_OUTER;
  return neighbouringMode;
}

void
MergeDriverWidget::setNeighbouringMode(girl::toger::neighbouringModes nm)
{
  if (nm & girl::toger::NEIGHBOUR_DIRECT)
    m_direct->setChecked(true);
  else
    m_direct->setChecked(false);

  if (nm & girl::toger::NEIGHBOUR_INNER)
    m_inner->setChecked(true);
  else
    m_inner->setChecked(false);

  if (nm & girl::toger::NEIGHBOUR_OUTER)
    m_outer->setChecked(true);
  else
    m_outer->setChecked(false);
}

girl::toger::markFlags
MergeDriverWidget::getMarkFlag() const
{
  return m_mw->getMarkFlag();
}

void
MergeDriverWidget::setMarkFlag(girl::toger::markFlags m)
{
  m_mw->setMarkFlag(m);
}
