/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef MERGEDRIVERWIDGETINTERFACE_HPP
#define MERGEDRIVERWIDGETINTERFACE_HPP

#include <girl/girlCommon.hpp>//GIRL_API
#include <ise/iseSegmentationContext.hpp>
#include <QWidget>

#include "MergeDriverPlugin.hpp" //class MergeDriverPlugin;
#include "MergeDriverSettings.hpp"
#include "MainView.hpp"

class GIRL_API MergeDriverWidgetInterface : public QWidget
{
#if 0
  Q_OBJECT
#endif

public:

  MergeDriverWidgetInterface(MainView *mainView,
			     MergeDriverPlugin *mergeDriverPlugin = NULL, 
			     QWidget *parent = NULL)
    : QWidget(parent),
      m_mainView(mainView),
      m_mergeDriverPlugin(mergeDriverPlugin)
#if 0
    , m_valid(true)
#endif
  {}

  virtual void setMergeDriverPlugin(MergeDriverPlugin *mergeDriverPlugin)
  {
    m_mergeDriverPlugin = mergeDriverPlugin;
  }
  
  virtual MergeDriverPlugin *getMergeDriverPlugin()
  {
    return m_mergeDriverPlugin;
  }

#if 0
  /**
   * Set validity of widget, i.e., 
   * tells if set parameters are valid for mergeDriver.
   */
  virtual void setValid(bool valid)
  {
    if (valid != m_valid) {
      m_valid = valid;
      emit validityChanged(valid);
    }
  }

  /**
   * Get validity of widget, i.e., 
   * tells if set parameters are valid for mergeDriver.
   */
  virtual bool isValid() const
  {
    return m_valid;
  }
#endif

  
  virtual void getGUI(girl::shared_ptr<MergeDriverSettings> settings) const
  {
    if (settings)
      settings->getWidgetConfig(this);
  }

  virtual void setGUI(const girl::shared_ptr<MergeDriverSettings> settings)
  {
    if (settings)
      settings->configureWidget(this);
  }

  virtual bool saveSettingsInPlugin()
  {
    if (m_mergeDriverPlugin) {
      if (m_mergeDriverPlugin->hasSettings()) {
	girl::shared_ptr<MergeDriverSettings> s = m_mergeDriverPlugin->getSettings();
	getGUI(s);
	assert(s);
	if (s->isValid()) {
	  m_mergeDriverPlugin->setSettings(s);
	  return true;
	}
      }
      else {
	//no settings
	return true;
      }
    }
    return false;
  }

  virtual bool isValid() const
  {
    if (m_mergeDriverPlugin) {
      if (m_mergeDriverPlugin->hasSettings()) {
	girl::shared_ptr<MergeDriverSettings> s = m_mergeDriverPlugin->getSettings();
	getGUI(s);
	assert(s);
	if (s->isValid()) {
	  return true;
	}
      }
      else {
	//no settings
	return true;
      }
    }
    return false;
  }


  virtual void setNeighbouringModes(girl::toger::neighbouringModes neighbourhoodMode) = 0;

  virtual girl::toger::neighbouringModes getNeighbouringModes() const = 0;

  virtual void setMarkFlags(girl::toger::markFlags m) = 0;

  virtual girl::toger::markFlags getMarkFlags() const = 0;


#if 0
signals:

  /**
   * emitted when validity changes.
   */
  void validityChanged(bool valid);

  /**
   * emitted when one of the parameters of widget changes.
   */
  void parametersChanged();
#endif


protected:
  
  MainView *m_mainView;
  MergeDriverPlugin *m_mergeDriverPlugin;
#if 0
  bool m_valid;
#endif
};


#endif /* ! MERGEDRIVERWIDGETINTERFACE_HPP */
