/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "MiniView.hpp"

#include <QWidget>

#include "GLUtils.hpp"
#include "GLImageLayer.hpp"
#include "GLSelectionLayer.hpp"
#include "GLSegmentLayer.hpp"

static const GLColor DEFAULT_TOGER_COLOR = GLColor(0, 255, 0); //CODE DUPLICATION with GLTogerLayer

MiniView::MiniView(QWidget *parent, SegModel *model)
  : SegView(parent, model)
  , m_imageLayer(NULL)
  , m_selectionLayer(NULL)
{
  if (model)
    setModel(model);

}

void
MiniView::setModel(SegModel *model)
{
  assert(model);
  
  //if (m_model) 
  //disconnect(m_model);

  SegView::setModel(model); //do m_model=model;

  if (! m_imageLayer) {
    m_imageLayer = new GLImageLayer(this);
    addLayer(m_imageLayer);
  }
  changeImage(model);

  /*
  m_togerLayer = new GLSegmentLayer(this);
  addLayer(m_togerLayer);
  changeToger(model);
  */

  /*
  m_selectionLayer = new GLSelectionLayer(this);
  addLayer(m_selectionLayer);
  changeSelection(model);
  */	
			     
  connect(model);

}

void 
MiniView::changeImage(SegModel *model)
{
  assert(model);
  
  QImage img = model->getQImage();

  if (! img.isNull()) {
    if (img.width() > 2*minimumSize().width()
	|| img.height() > 2*minimumSize().height()) {
      img = img.scaled(minimumSize(), Qt::KeepAspectRatio, Qt::SmoothTransformation); //FastTransformation);
    }
    assert(m_imageLayer);
    //std::cerr<<" resized to w="<<img.width()<<" h="<<img.height()<<std::endl;
    m_imageLayer->setImage(img, model->getQImage().width(), model->getQImage().height());
  }
  update();

}

void 
MiniView::changeToger(SegModel * /* model */)
{
  /*
  assert(model);
  assert(m_togerLayer);
  if (model->selectable()) {
    const girl::toger *tog = model->getToger();
    if (tog != NULL) {
      const int w = tog->width();
      const int h = tog->height();
      const int x0 = -1;
      const int y0 = -1;
      const int x1 = w-1;
      const int y1 = h-1;
      // (SegmentLayer add offset +1 to coordinates)

      std::deque<girl::point> pts;
      pts.push_back(girl::point(x0, y0));
      pts.push_back(girl::point(x1, y0));
      pts.push_back(girl::point(x1, y0));
      pts.push_back(girl::point(x1, y1));
      pts.push_back(girl::point(x1, y1));
      pts.push_back(girl::point(x0, y1));
      pts.push_back(girl::point(x0, y1));
      pts.push_back(girl::point(x0, y0));
      m_togerLayer->setSegments(pts);
      m_togerLayer->setColor(DEFAULT_TOGER_COLOR);
      //TODO: would be more efficient to have a GLBoxLayer ...
    }
  }
  update();
  */
}

void 
MiniView::changeSelection(SegModel * /*model*/)
{
  /*
  assert(model);
  assert(m_selectionLayer);
 
  m_selectionLayer->update(model);
  update();
  */
}

void
MiniView::changeSelectionType(SegModel *model)
{
  changeSelection(model);
}

#include <iostream>

void
MiniView::changeRects(const QRectF &modelRectangle, const QRectF &observedViewRectangle)
{
  QRectF ur = modelRectangle.united(observedViewRectangle); //Union

  //TODO: some CODE DUPLICATION

  if (ur.width() != 0 && ur.height() !=0) {

  // ~bestFit
  const qreal zw = width()/static_cast<qreal>(ur.width());
  const qreal zh = height()/static_cast<qreal>(ur.height());
  
  m_zoomFactor = zh;
  if (zw*ur.height() < height())
    m_zoomFactor = zw;

  //assert(m_zoomFactor*ur.width()<=width()+0.001);
  //assert(m_zoomFactor*ur.height()<=height()+0.001);

  //std::cerr<<"MiniView   w="<<width()<<" h="<<height()<<std::endl;

  // ~center
  const qreal cx = m_zoomFactor*(ur.topLeft().x()+ur.width()/2.0);
  const qreal cy = m_zoomFactor*(ur.topLeft().y()+ur.height()/2.0);

  m_x0 = - static_cast<int>( cx - width()/2.0 );
  m_y0 = - static_cast<int>( cy - height()/2.0 );

  /*
  std::cerr<<"MiniView   m_x0="<<m_x0<<" m_y0="<<m_y0<<" m_zoomFactor="<<m_zoomFactor<<std::endl;
  QRectF urT;
  transformFromView(ur, urT);
  std::cerr<<"MiniView::changeRects ur x="<< ur.x()<<" y="<< ur.y()<<" w="<< ur.width()<<" h="<< ur.height()<<" transformed to urT x="<< urT.x() <<" y="<< urT.y()<<" w="<< urT.width()<<" h="<< urT.height()<< std::endl;  
  */

  }
}


void
MiniView::updateRect(const QRectF &modelRectangle, const QRectF &observedViewRectangle)
{
  changeRects(modelRectangle, observedViewRectangle);

  m_modelRectangle = modelRectangle;
  
  m_observedViewRectangle = observedViewRectangle;

  update();
}

void
MiniView::resizeGL(int w, int h)
{
  GLView::resizeGL(w, h);

  changeRects(m_modelRectangle, m_observedViewRectangle);
}


void
MiniView::paintGL()
{
  //const int w = width();
  const int h = height();

  GL_CHECK_ERROR();

  glClear(GL_COLOR_BUFFER_BIT); //useless on a zoom in ???

  glLoadIdentity();

  //OpenGL to QT CS
  glRotatef(180.0f, 1.0f, 0.0f, 0.0f);
  glTranslatef(0, -h, 0.0f);


  glTranslatef(m_x0, m_y0, 0.0f);

  glScalef(m_zoomFactor, m_zoomFactor, 1.0f);


  //QRectF ur = m_modelRectangle.united(m_observedViewRectangle); //Union
  QRectF rM = m_modelRectangle;
  QRectF rV = m_observedViewRectangle;

  
  /*
  glColor3f(0.0f, 1.0f, 1.0f);
  glBegin(GL_QUADS);
  glVertex2f(ur.topLeft().x(), ur.topLeft().y());
  glVertex2f(ur.topRight().x(), ur.topRight().y());
  glVertex2f(ur.bottomRight().x(), ur.bottomRight().y());
  glVertex2f(ur.bottomLeft().x(), ur.bottomLeft().y());
  glEnd();
  */

  glColor3f(0.1f, 0.1f, 0.1f);
  glBegin(GL_LINE_LOOP);
  glVertex2f(rM.topLeft().x(), rM.topLeft().y());
  glVertex2f(rM.topRight().x(), rM.topRight().y());
  glVertex2f(rM.bottomRight().x(), rM.bottomRight().y());
  glVertex2f(rM.bottomLeft().x(), rM.bottomLeft().y());
  glEnd();


  GLLayerCollection::iterator it = m_layers.begin();
  const GLLayerCollection::iterator itEnd = m_layers.end();
  for ( ; it != itEnd; ++it) {
    (*it)->draw();
  }

  /*
  InteractorCollection::iterator itI = m_interactors.begin();
  const InteractorCollection::iterator itIEnd = m_interactors.end();
  for ( ; itI != itIEnd; ++itI) {
    (*itI)->draw();
  }
  */
  glColor3f(0.1f, 1.0f, 0.1f);
  glBegin(GL_LINE_LOOP);
  glVertex2f(rV.topLeft().x(), rV.topLeft().y());
  glVertex2f(rV.topRight().x(), rV.topRight().y());
  glVertex2f(rV.bottomRight().x(), rV.bottomRight().y());
  glVertex2f(rV.bottomLeft().x(), rV.bottomLeft().y());
  glEnd();



  /*

  GLView::paintGL();


  QRectF ur = m_modelRectangle.united(m_observedViewRectangle); //Union
  
  glColor3f(0.0f, 1.0f, 1.0f);
  glBegin(GL_QUADS);
  glVertex2f(ur.topLeft().x(), ur.topLeft().y());
  glVertex2f(ur.topRight().x(), ur.topRight().y());
  glVertex2f(ur.bottomRight().x(), ur.bottomRight().y());
  glVertex2f(ur.bottomLeft().x(), ur.bottomLeft().y());
  glEnd();

  QRectF rM = m_modelRectangle;
  glColor3f(1.0f, 1.0f, 0.0f);
  glBegin(GL_QUADS);
  glVertex2f(rM.topLeft().x(), rM.topLeft().y());
  glVertex2f(rM.topRight().x(), rM.topRight().y());
  glVertex2f(rM.bottomRight().x(), rM.bottomRight().y());
  glVertex2f(rM.bottomLeft().x(), rM.bottomLeft().y());
  glEnd();

  QRectF rV = m_observedViewRectangle;
  glColor3f(0.1f, 1.0f, 0.1f);
  glBegin(GL_QUADS);
  glVertex2f(rV.topLeft().x(), rV.topLeft().y());
  glVertex2f(rV.topRight().x(), rV.topRight().y());
  glVertex2f(rV.bottomRight().x(), rV.bottomRight().y());
  glVertex2f(rV.bottomLeft().x(), rV.bottomLeft().y());
  glEnd();
  


  std::cerr<<"MiniView::paintGL ur.topLeft x="<< ur.topLeft().x()<<" y="<<ur.topLeft().y() 
	   <<" bottomRight x="<< ur.bottomRight().x()<<" y="<<ur.bottomRight().y() <<" w="<< ur.width()<<" h="<< ur.height()<<std::endl;
  std::cerr<<"MiniView::paintGL avec scale ur.topLeft x="<< ur.topLeft().x()*m_zoomFactor+m_x0<<" y="<<ur.topLeft().y()*m_zoomFactor+m_y0 
	   <<" bottomRight x="<< ur.bottomRight().x()*m_zoomFactor+m_x0<<" y="<<ur.bottomRight().y()*m_zoomFactor+m_y0 <<" w="<< ur.width()*m_zoomFactor<<" h="<< ur.height()*m_zoomFactor<<std::endl;
  

  //QRect ir = m_modelRectangle.intersects(m_observedViewRectangle); //intersection
*/


}
