/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#include "MultiCheckLabelSliderSpinBoxWidget.hpp"
#include "CheckLabelSliderSpinBoxWidget.hpp"

#include <QLabel>
#include <QGridLayout>
#include <QListWidget>
#include <iostream>
#include <cassert>

MultiCheckLabelSliderSpinBoxWidget::MultiCheckLabelSliderSpinBoxWidget(QWidget *parent)
  : QWidget(parent),
    m_list(NULL)
{

  QGridLayout *layout = new QGridLayout;
  
  m_list = new QListWidget(this);
  m_list->setMinimumWidth(250);
  m_list->setMaximumHeight(300);

  layout->addWidget(m_list);
  
  layout->setMargin(0);

  setLayout(layout);

}

void
MultiCheckLabelSliderSpinBoxWidget::addItem(const QString &text, double value, bool checked)
{
  int index = findText(text);
  if ( index == -1) {
    CheckLabelSliderSpinBoxWidget *slider = new CheckLabelSliderSpinBoxWidget(text, this);
    connect(slider, SIGNAL(check(bool)), this, SLOT(checked(bool)));
    slider->setMinimum(0);
    slider->setMaximum(1);
    slider->setDecimals(2);
    slider->setSingleStep(0.01);
    slider->setValue(value);
    slider->setChecked(checked);

    QListWidgetItem *item = new QListWidgetItem;
    item->setSizeHint(QSize(200, 30));
    m_list->addItem(item);
    m_list->setItemWidget(item, slider);

  }
  else {
    QListWidgetItem *item = m_list->item(index);
    if (item) {
      CheckLabelSliderSpinBoxWidget *slider = static_cast<CheckLabelSliderSpinBoxWidget*>(m_list->itemWidget(item));
      slider->setValue(value);
      slider->setChecked(checked);
    }
  }
}

void
MultiCheckLabelSliderSpinBoxWidget::delItem(const QString &text)
{
  int count = m_list->count();
  for (int i = 0 ; i < count ; ++i) {
    QListWidgetItem *item = m_list->item(i);
    CheckLabelSliderSpinBoxWidget *clssbw = static_cast<CheckLabelSliderSpinBoxWidget*>(m_list->itemWidget(item));
    if (text == clssbw->text()) {
      delete item;
      break;
    }
  }
}

QString
MultiCheckLabelSliderSpinBoxWidget::currentText() 
{
  QListWidgetItem *item = m_list->currentItem();
  if (item != NULL) {
    CheckLabelSliderSpinBoxWidget *clssbw = static_cast<CheckLabelSliderSpinBoxWidget*>(m_list->itemWidget(item));
    return clssbw->text();
  }
  return tr("");
}

double
MultiCheckLabelSliderSpinBoxWidget::currentValue() 
{
  QListWidgetItem *item = m_list->currentItem();
  if (item != NULL) {
    CheckLabelSliderSpinBoxWidget *clssbw = static_cast<CheckLabelSliderSpinBoxWidget*>(m_list->itemWidget(item));
    return clssbw->value();
  }
  return 0;
}

bool
MultiCheckLabelSliderSpinBoxWidget::deleteCurrentItem()
{
  QListWidgetItem *item = m_list->currentItem();
  if (item != NULL) {
    CheckLabelSliderSpinBoxWidget *clssbw = static_cast<CheckLabelSliderSpinBoxWidget*>(m_list->itemWidget(item));

    if (! clssbw->isChecked()) 
      return false;

    QString text = clssbw->text();
    delItem(text);

    return true;
  }
  return false;
}

void
MultiCheckLabelSliderSpinBoxWidget::checked(bool b)
{
  CheckLabelSliderSpinBoxWidget* clssbw = qobject_cast<CheckLabelSliderSpinBoxWidget*>(sender());

  if (b) {
    add(clssbw->text());
  }
  else {
    del(clssbw->text());
  }
}

bool
MultiCheckLabelSliderSpinBoxWidget::contain(const QString &text)
{
  int count = m_list->count();
  for (int i = 0 ; i < count ; ++i) {
    QListWidgetItem *item = m_list->item(i);
    if (item != NULL) {
      CheckLabelSliderSpinBoxWidget *clssbw = static_cast<CheckLabelSliderSpinBoxWidget*>(m_list->itemWidget(item));
      if (text == clssbw->text()) 
	return true;
    }
  }
  return false;
}

int
MultiCheckLabelSliderSpinBoxWidget::findText(const QString &text) const
{
  int count = m_list->count();
  for (int i = 0 ; i < count ; ++i) {
    QListWidgetItem *item = m_list->item(i);
    if (item != NULL) {
      CheckLabelSliderSpinBoxWidget *clssbw = static_cast<CheckLabelSliderSpinBoxWidget*>(m_list->itemWidget(item));
      if (text == clssbw->text()) 
	return i;
    }
  }
  return -1;
}

CheckLabelSliderSpinBoxWidget* 
MultiCheckLabelSliderSpinBoxWidget::item(int index) {
  if (index >= m_list->count())
    return NULL;
  
  QListWidgetItem *item = m_list->item(index);
  return static_cast<CheckLabelSliderSpinBoxWidget*>(m_list->itemWidget(item));
}
