/*
  Copyright (C) 2005-2009 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
/**
 * @file MultiCgeckLabelSliderSpinBoxWidget.hpp
 * @brief Combination of a SliderSpinBoxWidget.
 */
#ifndef MULTICHECKLABELSLIDERSPINBOXWIDGET_HPP
#define MULTICHECKLABELSLIDERSPINBOXWIDGET_HPP

#include <girl/girlCommon.hpp>//GIRL_API
#include <QWidget>
#include <map>
#include <QListWidget>

class QSlider;
class QDoubleSpinBox;
class QLineEdit;

class CheckLabelSliderSpinBoxWidget;


class GIRL_API MultiCheckLabelSliderSpinBoxWidget : public QWidget
{
  Q_OBJECT

 public:

  /**
   * @brief Constructor.
   */
  explicit MultiCheckLabelSliderSpinBoxWidget(QWidget *parent = NULL);

  void addItem(const QString &text, double value = 1.0, bool checked = false);

  void delItem(const QString &text);

  QString currentText();

  double currentValue();
  
  int findText(const QString &text) const;

  /**
   * @brief Delete CurrentItem if is checked
   */
  bool deleteCurrentItem();

  /**
   * @brief return if an item with label @a text it contain in m_list
   */ 
  bool contain(const QString &text);

  /**
   * @brief clear
   */
  void clear() { m_list->clear(); }
  
  int count() { return m_list->count(); }

  CheckLabelSliderSpinBoxWidget* item(int index);

 private slots:

  void checked(bool);

 signals:

  void add(QString);
  void del(QString);

 protected:

  QListWidget *m_list;

};

#endif /* ! MULTICHECKLABELSLIDERSPINBOXWIDGET_HPP */
