/*
  Copyright (C) 2010-2011 Cédric Aguerre  aguerre@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef MULTISELECTIONINTERACTOR_HPP
#define MULTISELECTIONINTERACTOR_HPP

#include <qiseTools/RegionSelectionBaseInteractor.hpp>

#include <qiseTools/MainView.hpp>

#include <qiseTools/GLMultiSelectionLayer.hpp>
#include <list>


typedef std::deque<girl::region> MultiSelection;


class GIRL_API MultiSelectionInteractor : public RegionSelectionBaseInteractor
{
  Q_OBJECT

public:

  explicit MultiSelectionInteractor(MainView *v);

  virtual void activate();

  virtual void deactivate();

  virtual bool eventFilter(QObject *target, QEvent *event);

  void useList(std::string listName);

  void changeColor(std::string listName, QRgb c);
  void changeAlpha(std::string listName, float alpha);
  void changeFill(std::string listName, bool b);

  inline MultiSelection getSelection(std::string listName)
  {
    return m_selections[listName];
  }

  void setSelection(std::string listName, MultiSelection &ms);

signals:
  
  void selectionChanged();

private :
  GLMultiSelectionLayer *m_selectionLayer;
  bool m_wasSelectionDisplayed;
  bool m_wasTogerDisplayed;
  girl::regionDeque m_selectedRegionsInit;
  std::map<std::string, MultiSelection> m_selections;
  MultiSelection *m_activeSelection;

  template <class CONTAINER_TYPE> bool inContainer(girl::region r, CONTAINER_TYPE &ct);

};


#endif /* ! MULTISELECTIONINTERACTOR_HPP */
