/*
  Copyright (C) 2005-2009 Jerome Charton, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef PAINTINGSHAPE_HPP
#define PAINTINGSHAPE_HPP

#include <algorithm>
#include <QPoint>
#include <QVector>
#include <vector>

#include "girl/girlToger.hpp"

#include "ScanlineSet.hpp"
#include "GLUtils.hpp"

#include <qiseTools/Shape.hpp>

class GLView;
class SegView;
class QPainter;

/**
 * All drawable shape.
 */
class GIRL_API PaintingShape : public Shape
{
  
public:
  
  PaintingShape() 
  : m_angle(0.0) 
  {}
  
  virtual ~PaintingShape() {}

  virtual void draw(const GLView *view, QPainter& painter) = 0;

  virtual PaintingShape *clone() = 0;
  
  virtual bool isUnder(const QPoint& point) const = 0;

  virtual void getSegmentationPath(int w, int h, girl::pathDeque& paths);

  virtual void setPoint(unsigned int index, const QPoint& point) = 0;
  virtual QVector<QPoint> getPoints() const = 0;

  inline const QPoint& getAreaP1() const {return m_areaP1;}
  inline const QPoint& getAreaP2() const {return m_areaP2;}
  
  inline QPoint getCenter() const 
  {
    return QPoint(static_cast<int>((m_areaP2.x()+m_areaP1.x())/2.0),
		  static_cast<int>((m_areaP2.y()+m_areaP1.y())/2.0));
  }

  inline void setAreaP1(const QPoint& point) {m_areaP1 = point;}
  inline void setAreaP2(const QPoint& point) {m_areaP2 = point;}

  inline int width() const {return m_areaP2.x() - m_areaP1.x();}
  inline int height() const {return m_areaP2.y() - m_areaP1.y();}
  
  virtual void translate(QPoint &source, const QPoint &target) = 0;

  void translate(const QPoint &vector)
  {QPoint origin(0, 0); translate(origin, vector);}

  void setRotation(double angle) {m_angle = angle;}
  double getRotationInDegree() const {return gradToDeg(m_angle);}
  double getRotationInRadient() const {return m_angle;}
  
  virtual void resize2D(QPoint& origin, QPoint& source, const QPoint& target) = 0;
  
  void resizeX(QPoint& origin, QPoint& source, QPoint& target);
  void resizeY(QPoint& origin, QPoint& source, QPoint& target);
  
  QPoint getTransformedPoint(const QPoint& point) const;
  QPoint getOriginalPoint(const QPoint& point) const;

protected:
  
  girl::point getTransformedPoint(girl::point point);
  girl::point getOriginalPoint(girl::point point);

  double getXScale(const QPoint& origin, const QPoint& source, const QPoint& target);
  double getYScale(const QPoint& origin, const QPoint& source, const QPoint& target);
  
  void transformPoint(const QPoint& origin, QPoint& point, const double xScale, const double yScale);
  void translatePoint(QPoint& point, const int xMove, const int yMove);
  
  void getLinearPerimeterPoints(std::vector<girl::point>& points, std::vector<girl::point>& perimeterPoints);

protected:
  
  //bounding box points
  // m_areaP1=(minX, maxX) m_areaP2=(maxX, maxY)
  QPoint m_areaP1, m_areaP2;
  
  //rotation
  double m_angle;
};

#endif /* ! PAINTINGSHAPE_HPP */
