/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef PARSERIMAGEFEATUREDESCRIPTION_HPP
#define PARSERIMAGEFEATUREDESCRIPTION_HPP

#include <vector>
#include <cstring>
#include "ColorSpaceRegistry.hpp"

class GIRL_API ParserImageFeatureDescription
{
 private:
  static std::vector<std::string> cut(const std::string &s)
  {
    size_t size = s.length();
    std::vector<std::string> ret;
    
    std::string tmp = "";
    for (size_t i = 0 ; i < size ; ++i) {
      if ( s[i] != ' ') {
	tmp += s[i];
      }
      else {
	ret.push_back(tmp);
	tmp = "";
      }
    }
    if (s[size-1] != ' ')
      ret.push_back(tmp);
    
    return ret;
  }
  

 public:
  static std::vector<std::string> parse(const std::vector<std::string> &description)
  {
    size_t size = description.size();

    std::vector<std::string> ret;
  
    for (size_t i = 0 ; i < size ; ++i) {
      std::vector<std::string> c = cut(description[i]);
      if (c[0] == "ColorSpace") {
	int colorSpace = std::atoi(c[1].c_str());
	int channel = std::atoi(c[2].c_str());
      
	ret.push_back(ColorSpaceRegistry::getColorSpace(colorSpace)->channelsNames[channel].toStdString());
      }
      else {
	if (c[0] == "Threshold") {
	  int colorSpace = std::atoi(c[1].c_str());
	  int channel = std::atoi(c[2].c_str());
	  int level = std::atoi(c[3].c_str());
	  
	  std::stringstream ss;
	  ss << ColorSpaceRegistry::getColorSpace(colorSpace)->name.toStdString()<< " "<<ColorSpaceRegistry::getColorSpace(colorSpace)->channelsNames[channel].toStdString()<< " t="<<level;
	  ret.push_back(ss.str());
	}
	else {
	  
	  if (c[0] == "Thresholds") {
	    int colorSpace = std::atoi(c[1].c_str());
	    int channel = std::atoi(c[2].c_str());
	    std::stringstream ss;
	    ss << ColorSpaceRegistry::getColorSpace(colorSpace)->name.toStdString()<< " "<<ColorSpaceRegistry::getColorSpace(colorSpace)->channelsNames[channel].toStdString();
	    ret.push_back(ss.str());
	  }
	  else {
	    ret.push_back(description[i]);
	  }
	}
      }
    }

    return ret;
  }
};
#endif /* ! PARSERIMAGEFEATUREDESCRIPTION_HPP */
