/*
  Copyright (C) 2005-2011 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#include "PluginIndexChoiceWidget.hpp"

#include <QHBoxLayout>
#include <QLabel>


PluginIndexChoiceWidget::PluginIndexChoiceWidget(const QIsePluginList<QIsePlugin*> &plugins,
						 const QString &name,
						 QWidget *parent)
  : QWidget(parent),
    m_combo(NULL),
    m_label(NULL),
    m_plugins()
{
  build(plugins, name);
}

PluginIndexChoiceWidget::~PluginIndexChoiceWidget()
{
}

void
PluginIndexChoiceWidget::build(const QIsePluginList<QIsePlugin*> &plugins,
			       const QString &name)
{
  m_label = new QLabel(name, this);

  m_combo = new QComboBox(this);

  const int numPlugins = m_plugins.size();
  m_plugins.reserve(numPlugins);

  QIsePluginList<QIsePlugin*>::const_iterator it = plugins.begin();
  const QIsePluginList<QIsePlugin*>::const_iterator itEnd = plugins.end();
  for (; it != itEnd; ++it) {
    QIsePlugin *plugin = *it;

    m_combo->addItem(plugin->name());

    m_plugins.push_back(plugin);
  }
  


  // REMI: TODO:UNSURE connect signal/slots ?
  // MUST work only with getPluginIndex, we don't need
  // to use signals/slots do we?
  connect(m_combo, SIGNAL(currentIndexChanged(int)),
          this, SIGNAL(pluginIndexChanged(int)));

  QHBoxLayout *hbox = new QHBoxLayout;
  hbox->addWidget(m_label);
  hbox->addWidget(m_combo);
  hbox->addStretch();
  hbox->setMargin(0); //because it aims to be an embedded widget.
  this->setLayout(hbox);
}

void
PluginIndexChoiceWidget::setText(const QString &name)
{
  assert(m_label);
  m_label->setText(name);
}


void
PluginIndexChoiceWidget::setPluginIndex(int index)
{
  assert(m_combo);
  assert(index < m_combo->count());
  m_combo->setCurrentIndex(index);
}

int
PluginIndexChoiceWidget::getPluginIndex() const
{
  assert(m_combo);
  return m_combo->currentIndex();
}

QIsePlugin *
PluginIndexChoiceWidget::getPlugin()
{
  assert(m_combo);
  if (m_combo->currentIndex()>=0)
    return m_plugins[m_combo->currentIndex()];
  return NULL;
}

void
PluginIndexChoiceWidget::setPlugin(QIsePlugin *p)
{
  const int nbPlugins = m_plugins.size();
  for (int i=0; i<nbPlugins; ++i) {
    if (m_plugins[i] == p) {
      setPluginIndex(i);
      return;
    }
  }
}

QIsePlugin *
PluginIndexChoiceWidget::getPlugin(int i)
{
  assert(m_combo);
  assert(i>=0 && i<m_combo->count());
  return m_plugins[i];
}

QString
PluginIndexChoiceWidget::getPluginName() const
{
  assert(m_combo);
  if (m_combo->currentIndex() >= 0)
    return m_combo->currentText();
  return tr("");
}

QString
PluginIndexChoiceWidget::getPluginName(int index) const
{
  assert(m_combo);
  if (index >= 0 && index < m_combo->count() )
    return m_combo->itemText(index);
  return tr("");
}
