/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef PLUGININDEXCHOICEWIDGET_HPP_
#define PLUGININDEXCHOICEWIDGET_HPP_

#include <girl/girlCommon.hpp>

#include "QIsePluginList.hpp"

#include <QVector>
#include <QWidget>
#include <QComboBox>

class RegionFeatureDistancePlugin;
class QComboBox;
class QLabel;
/**
 * Widget for choosing a single plugin, i.e. using a combo box.
 * The plugins are reffered by their indexes in the plugin registry
 * given in the constructor. Plugins must then be reffered by their
 * indexes in the program when one wants to use them, and query the
 * registry to get the plugin.
 */

class GIRL_API PluginIndexChoiceWidget : public QWidget
{

  Q_OBJECT

public:
  /**
   * @brief Constructor
   * @param plugins: the plugin list (e.g. obtained from the plugin registry)
   * @param name: will be displayed in a label in front of the combo box, use it to
   * name the type of plugin to select
   */
  PluginIndexChoiceWidget(const QIsePluginList<QIsePlugin*> &plugins, const QString& name, QWidget *parent);

  /**
   * @brief Destructor.
   */
  virtual ~PluginIndexChoiceWidget();

  /**
   * @brief Set text displayed in label in front of combo box.
   */
  void setText(const QString &name);
  

  /**
   * Set the index of the plugin to use
   */
  // REMI TODO:UNSURE is this function necessary?
  void setPluginIndex(int index);

  /**
   * return the index of the chosen plugin
   *
   * Return -1 if it was initialized with an empty list.
   */
  int getPluginIndex() const;

  /**
   * return the current plugin
   *
   * Return NULL if it was initialized with an empty list.
   */
  QIsePlugin *getPlugin();

  /**
   * Set current entry to plugin @a p if present.
   */
  void setPlugin(QIsePlugin *p);

  /**
   * return the current plugin
   *
   * Return NULL if it was initialized with an empty list
   * or if i is not in the list size.
   */
  QIsePlugin *getPlugin(int i);

  /**
   * return the current plugin name
   *
   * Return an empty QString if it was initialized with an empty list.
   */
  QString getPluginName() const;

  /**
   * return the plugin name at index @a index
   *
   * Return an empty QString if index not present in @a m_combo
   */
  QString getPluginName(int index) const;

  /**
   * return the index of plugin name @a name
   */
  int findText(QString name) { return m_combo->findText(name); }

  /**
   * return the current text
   */
  QString currentText() const { return m_combo->currentText(); }

  /**
   * set the current index at @a index
   */
  void setCurrentIndex(int index) { 
    m_combo->setCurrentIndex(index == -1 ? 0 : index); 
  }

  /**
   * Return QComboBox's size
   */
  inline int count() const { return m_combo->count(); }

signals:

  /**
   * @warning color space is already changed when this signal is emitted.
   */
 void pluginIndexChanged(int);


//protected slots:
//
//  void changePluginIndex(int index);

protected:

  void build(const QIsePluginList<QIsePlugin*> &plugins, const QString &name);

protected:

  QComboBox *m_combo;
  QLabel *m_label;
  QVector<QIsePlugin*> m_plugins;

};

#endif /* PLUGININDEXCHOICEWIDGET_HPP_ */
