/*
  Copyright (C) 2005-2009 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "PluginRegistry.hpp"

QIsePluginList<QuantificationThresholderPlugin*> PluginRegistry::m_quantificationThresholderPlugin;
QIsePluginList<SegmentationClusteringLocalThresholderPlugin*> PluginRegistry::m_segmentationClusteringLocalThresholderPlugin;
QIsePluginList<RegionSetInterfacePlugin*> PluginRegistry::m_regionSetPlugin;
QIsePluginList<ImageFeatureColorimetricPlugin*> PluginRegistry::m_imageFeatureColorimetricPlugin;
QIsePluginList<ImageFeatureBinaryPlugin*> PluginRegistry::m_imageFeatureBinaryPlugin;
QIsePluginList<ImageFeatureGeometricPlugin*> PluginRegistry::m_imageFeatureGeometricPlugin;
QIsePluginList<ImageFeatureTexturePlugin*> PluginRegistry::m_imageFeatureTexturePlugin;
QIsePluginList<ImageFeatureCompositionPlugin*> PluginRegistry::m_imageFeatureCompositionPlugin;
QIsePluginList<FeatureTabPlugin*> PluginRegistry::m_featureTabPlugin;
QIsePluginList<ImageFeatureWidgetPlugin*> PluginRegistry::m_imageFeatureWidgetPlugin;
QIsePluginList<PyramidPlugin*> PluginRegistry::m_pyramidPlugin;
QIsePluginList<RegionUnaryFunctionPlugin*> PluginRegistry::m_regionUnaryFunctionPlugin;
QIsePluginList<RegionUnaryPredicatePlugin*> PluginRegistry::m_regionUnaryPredicatePlugin;
QIsePluginList<RegionBinaryFunctionPlugin*> PluginRegistry::m_regionBinaryFunctionPlugin;
QIsePluginList<RegionBinaryPredicatePlugin*> PluginRegistry::m_regionBinaryPredicatePlugin;
QIsePluginList<RegionStrictWeakOrderingPlugin*> PluginRegistry::m_regionStrictWeakOrderingPlugin;
QIsePluginList<ClustererPlugin*> PluginRegistry::m_clustererPlugin;


  void
	  PluginRegistry::loadQuantificationThresholderPlugin(const QDir & pluginsDir,
						  QList<std::pair<plugin_status, QString> >& logMessages,
						  bool resolveVersion)
  {
    m_quantificationThresholderPlugin.load(pluginsDir, logMessages, resolveVersion);// = quantif;
  }
  
  QIsePluginList<QuantificationThresholderPlugin*>& 
	  PluginRegistry::getQuantificationThresholderPlugin()
  {
    return m_quantificationThresholderPlugin;
  }
  
  void 
	 PluginRegistry::loadSegmentationClusteringLocalThresholderPlugin(const QDir & pluginsDir,
							       QList<std::pair<plugin_status, QString> >& logMessages,
							       bool resolveVersion)
  {
    m_segmentationClusteringLocalThresholderPlugin.load(pluginsDir, logMessages, resolveVersion);//= segClustering;
  }
  
  QIsePluginList<SegmentationClusteringLocalThresholderPlugin*>& 
	  PluginRegistry::getSegmentationClusteringLocalThresholderPlugin()
  {
    return m_segmentationClusteringLocalThresholderPlugin;
  }
  
  void 
	  PluginRegistry::loadRegionSetPlugin(const QDir &pluginsDir,
                                              QList<std::pair<plugin_status, QString> > &logMessages,
                                              bool resolveVersion)
  {
    m_regionSetPlugin.load(pluginsDir, logMessages, resolveVersion);
  }

  QIsePluginList<RegionSetInterfacePlugin*>& 
	  PluginRegistry::getRegionSetPlugin()
  {
    return m_regionSetPlugin;
  }

  FeaturePlugin *
	  PluginRegistry::getRegionSetPlugin(const std::string &name)
  {
    QIsePluginList<RegionSetInterfacePlugin*> lc = getRegionSetPlugin();
    QIsePluginList<RegionSetInterfacePlugin*>::const_iterator itc = lc.begin();
    QIsePluginList<RegionSetInterfacePlugin*>::const_iterator itcEnd = lc.end();
    for ( ; itc != itcEnd ; ++itc) {
      if ((*itc)->name().toStdString() == name)
	return *itc;
    }
    return NULL;
  }

  // ImageFeaturePlugin

  void 
	  PluginRegistry::loadImageFeaturePlugins(QString pluginPath,
				      QList<std::pair<plugin_status, QString> > &logMessages,
				      bool resolveVersion)
  {
    if (m_imageFeatureGeometricPlugin.isEmpty()) {
      loadImageFeatureGeometricPlugin(QDir(pluginPath+QString("ImageFeature/ImageFeatureGeometric")), logMessages, resolveVersion);
    }

    if (m_imageFeatureTexturePlugin.isEmpty()) {
      loadImageFeatureTexturePlugin(QDir(pluginPath+QString("ImageFeature/ImageFeatureTexture")), logMessages, resolveVersion);
    }

    if (m_imageFeatureColorimetricPlugin.isEmpty()) {
      loadImageFeatureColorimetricPlugin(QDir(pluginPath+QString("ImageFeature/ImageFeatureColorimetric")), logMessages, resolveVersion);
    }
    
    if (m_featureTabPlugin.isEmpty()) {
      loadFeatureTabPlugin(QDir(pluginPath+QString("ImageFeature/ImageFeatureTab")), logMessages, resolveVersion);
    }

    if (m_imageFeatureBinaryPlugin.isEmpty()) {
      loadImageFeatureBinaryPlugin(QDir(pluginPath+QString("ImageFeature/ImageFeatureColorimetric/ImageFeatureBinary")), logMessages, resolveVersion);
    }
    
    if (m_imageFeatureWidgetPlugin.isEmpty()) {
      loadImageFeatureWidgetPlugin(QDir(pluginPath+QString("ImageFeature/ImageFeatureWidget")), logMessages, resolveVersion);
    }

    if (m_imageFeatureCompositionPlugin.isEmpty()) {
      loadImageFeatureCompositionPlugin(QDir(pluginPath+QString("ImageFeature/ImageFeatureComposition")), logMessages, resolveVersion);
    }
  }

  FeaturePlugin *
	  PluginRegistry::getImageFeaturePlugin(const std::string &name)
  {
    QIsePluginList<ImageFeatureColorimetricPlugin*> lc = getImageFeatureColorimetricPlugin();
    QIsePluginList<ImageFeatureColorimetricPlugin*>::const_iterator itc = lc.begin();
    QIsePluginList<ImageFeatureColorimetricPlugin*>::const_iterator itcEnd = lc.end();
    for ( ; itc != itcEnd ; ++itc) {
      if ((*itc)->name().toStdString() == name)
	return *itc;
    }

    QIsePluginList<ImageFeatureBinaryPlugin*> lb = getImageFeatureBinaryPlugin();
    QIsePluginList<ImageFeatureBinaryPlugin*>::const_iterator itb = lb.begin();
    QIsePluginList<ImageFeatureBinaryPlugin*>::const_iterator itbEnd = lb.end();
    for ( ; itb != itbEnd ; ++itb) {
      if ((*itb)->name().toStdString() == name)
	return *itb;
    }

    QIsePluginList<ImageFeatureGeometricPlugin*> lg = getImageFeatureGeometricPlugin();
    QIsePluginList<ImageFeatureGeometricPlugin*>::const_iterator itg = lg.begin();
    QIsePluginList<ImageFeatureGeometricPlugin*>::const_iterator itgEnd = lg.end();
    for ( ; itg != itgEnd ; ++itg) {
      if ((*itg)->name().toStdString() == name)
	return *itg;
    }

    QIsePluginList<ImageFeatureTexturePlugin*> lt = getImageFeatureTexturePlugin();
    QIsePluginList<ImageFeatureTexturePlugin*>::const_iterator itt = lt.begin();
    QIsePluginList<ImageFeatureTexturePlugin*>::const_iterator ittEnd = lt.end();
    for ( ; itt != ittEnd ; ++itt) {
      if ((*itt)->name().toStdString() == name)
	return *itt;
    }

    QIsePluginList<ImageFeatureCompositionPlugin*> lco = getImageFeatureCompositionPlugin();
    QIsePluginList<ImageFeatureCompositionPlugin*>::const_iterator itco = lco.begin();
    QIsePluginList<ImageFeatureCompositionPlugin*>::const_iterator itcoEnd = lco.end();
    for ( ; itco != itcoEnd ; ++itco) {
      if ((*itco)->name().toStdString() == name)
	return (*itco);
    }

    return NULL;
  }

  /* ImageFeatureColorimetric Loader */
  void 
	  PluginRegistry::loadImageFeatureColorimetricPlugin(const QDir &pluginsDir,
  						 QList<std::pair<plugin_status, QString> > &logMessages,
  						 bool resolveVersion)
  {
    m_imageFeatureColorimetricPlugin.load(pluginsDir, logMessages, resolveVersion);
  }

  QIsePluginList<ImageFeatureColorimetricPlugin*>& 
	  PluginRegistry::getImageFeatureColorimetricPlugin()
  {
    return m_imageFeatureColorimetricPlugin;
  }

 /* ImageFeatureBinary Loader */
  void 
	  PluginRegistry::loadImageFeatureBinaryPlugin(const QDir &pluginsDir,
					   QList<std::pair<plugin_status, QString> > &logMessages,
					   bool resolveVersion)
  {
    m_imageFeatureBinaryPlugin.load(pluginsDir, logMessages, resolveVersion);
  }

  QIsePluginList<ImageFeatureBinaryPlugin*>& 
	  PluginRegistry::getImageFeatureBinaryPlugin()
  {
    return m_imageFeatureBinaryPlugin;
  }

  /* ImageFeatureGeometric Loader */
  void 
	  PluginRegistry::loadImageFeatureGeometricPlugin(const QDir &pluginsDir,
  					      QList<std::pair<plugin_status, QString> > &logMessages,
  					      bool resolveVersion)
  {
    m_imageFeatureGeometricPlugin.load(pluginsDir, logMessages, resolveVersion);
  }

  QIsePluginList<ImageFeatureGeometricPlugin*>& 
	  PluginRegistry::getImageFeatureGeometricPlugin()
  {
    return m_imageFeatureGeometricPlugin;
  }

  /* ImageFeatureTexture Loader */
  void 
	  PluginRegistry::loadImageFeatureTexturePlugin(const QDir &pluginsDir,
					      QList<std::pair<plugin_status, QString> > &logMessages,
					      bool resolveVersion)
  {
    m_imageFeatureTexturePlugin.load(pluginsDir, logMessages, resolveVersion);
  }

  QIsePluginList<ImageFeatureTexturePlugin*>& 
	  PluginRegistry::getImageFeatureTexturePlugin()
  {
    return m_imageFeatureTexturePlugin;
  }

  /* ImageFeatureComposition Loader */
  void 
	  PluginRegistry::loadImageFeatureCompositionPlugin(const QDir &pluginsDir,
					      QList<std::pair<plugin_status, QString> > &logMessages,
					      bool resolveVersion)
  {
    m_imageFeatureCompositionPlugin.load(pluginsDir, logMessages, resolveVersion);
  }

  QIsePluginList<ImageFeatureCompositionPlugin*>& 
	  PluginRegistry::getImageFeatureCompositionPlugin()
  {
    return m_imageFeatureCompositionPlugin;
  }

  /* ImageFeature other Loader */
  void 
	  PluginRegistry::loadFeatureTabPlugin(const QDir &pluginsDir,
				   QList<std::pair<plugin_status, QString> > &logMessages,
				   bool resolveVersion)
  {
    m_featureTabPlugin.load(pluginsDir, logMessages, resolveVersion);
  }
  
  QIsePluginList<FeatureTabPlugin*>& 
	  PluginRegistry::getFeatureTabPlugin()
  {
    return m_featureTabPlugin;
  }

  /* ImageFeature widget Loader */
  void 
	  PluginRegistry::loadImageFeatureWidgetPlugin(const QDir &pluginsDir,
					   QList<std::pair<plugin_status, QString> > &logMessages,
					   bool resolveVersion)
  {
    m_imageFeatureWidgetPlugin.load(pluginsDir, logMessages, resolveVersion);
  }
  
  QIsePluginList<ImageFeatureWidgetPlugin*>& 
	  PluginRegistry::getImageFeatureWidgetPlugin()
  {
    return m_imageFeatureWidgetPlugin;
  }


  /* Fin imageFeature */

  void 
	  PluginRegistry::loadPyramidPlugin(const QDir &pluginsDir,
				QList<std::pair<plugin_status, QString> > &logMessages,
				bool resolveVersion)
  {
    m_pyramidPlugin.load(pluginsDir, logMessages, resolveVersion);
  }

  QIsePluginList<PyramidPlugin*>& 
	  PluginRegistry::getPyramidPlugin()
  {
    return m_pyramidPlugin;
  }
  
  void 
	  PluginRegistry::loadRegionUnaryFunctionPlugin(const QDir &pluginsDir,
       QList<std::pair<plugin_status, QString> > &logMessages,
       bool resolveVersion)
   {
     m_regionUnaryFunctionPlugin.load(pluginsDir, logMessages, resolveVersion);
   }

   QIsePluginList<RegionUnaryFunctionPlugin*>&
   PluginRegistry::getRegionUnaryFunctionPlugin()
   {
     return m_regionUnaryFunctionPlugin;
   }

   void 
	   PluginRegistry::loadRegionUnaryPredicatePlugin(const QDir &pluginsDir,
      QList<std::pair<plugin_status, QString> > &logMessages,
      bool resolveVersion)
  {
    m_regionUnaryPredicatePlugin.load(pluginsDir, logMessages, resolveVersion);
  }

  QIsePluginList<RegionUnaryPredicatePlugin*>&
  PluginRegistry::getRegionUnaryPredicatePlugin()
  {
    return m_regionUnaryPredicatePlugin;
  }


  void 
	  PluginRegistry::loadRegionBinaryFunctionPlugin(const QDir &pluginsDir,
      QList<std::pair<plugin_status, QString> > &logMessages,
      bool resolveVersion)
  {
    m_regionBinaryFunctionPlugin.load(pluginsDir, logMessages, resolveVersion);
  }

  QIsePluginList<RegionBinaryFunctionPlugin*>&
  PluginRegistry::getRegionBinaryFunctionPlugin()
  {
    return m_regionBinaryFunctionPlugin;
  }

  void 
	  PluginRegistry::loadRegionBinaryPredicatePlugin(const QDir &pluginsDir,
      QList<std::pair<plugin_status, QString> > &logMessages,
      bool resolveVersion)
  {
    m_regionBinaryPredicatePlugin.load(pluginsDir, logMessages, resolveVersion);
  }

  QIsePluginList<RegionBinaryPredicatePlugin*>&
  PluginRegistry::getRegionBinaryPredicatePlugin()
  {
    return m_regionBinaryPredicatePlugin;
  }

  void 
	  PluginRegistry::loadRegionStrictWeakOrderingPlugin(const QDir &pluginsDir,
						 QList<std::pair<plugin_status, QString> > &logMessages,
						 bool resolveVersion)
  {
    m_regionStrictWeakOrderingPlugin.load(pluginsDir, logMessages, resolveVersion);
  }

  QIsePluginList<RegionStrictWeakOrderingPlugin*>&
    PluginRegistry::getRegionStrictWeakOrderingPlugin()
  {
    return m_regionStrictWeakOrderingPlugin;
  }

  void 
	  PluginRegistry::loadClustererPlugin(const QDir &pluginsDir,
				  QList<std::pair<plugin_status, QString> > &logMessages,
				  bool resolveVersion)
  {
    m_clustererPlugin.load(pluginsDir, logMessages, resolveVersion);
  }

  QIsePluginList<ClustererPlugin*>& 
	  PluginRegistry::getClustererPlugin()
  {
    return m_clustererPlugin;
  }
