/*
  Copyright (C) 2005-2009 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef PLUGINREGISTRY_HPP
#define PLUGINREGISTRY_HPP

#include <map>
#include <set>

#include <girl/girlCommon.hpp> // GIRL_API

#include <QString>
#include <QVector>
#include <QList>
#include <QDir>

#include "QuantificationThresholderPlugin.hpp"
#include "RegionBinaryFunctionPlugin.hpp"
#include "RegionBinaryPredicatePlugin.hpp"
#include "RegionSetInterfacePlugin.hpp"
#include "RegionStrictWeakOrderingPlugin.hpp"
#include "RegionUnaryFunctionPlugin.hpp"
#include "RegionUnaryPredicatePlugin.hpp"
#include "SegmentationClusteringLocalThresholderPlugin.hpp"
#include "SegmentationClusteringPlugin.hpp"
#include "ImageFeaturePlugin.hpp"
#include "FeatureTabPlugin.hpp"
#include "ImageFeatureColorimetricPlugin.hpp"
#include "ImageFeatureGeometricPlugin.hpp"
#include "ImageFeatureTexturePlugin.hpp"
#include "ImageFeatureBinaryPlugin.hpp"
#include "ImageFeatureWidgetPlugin.hpp"
#include "ImageFeatureCompositionPlugin.hpp"
#include "PyramidPlugin.hpp"
#include "ClustererPlugin.hpp"

#include "QIsePluginList.hpp"

#include <string>
class GIRL_API PluginRegistry
{
  //B:Warning:impl: do not put static methods inline
  // Visual C++ will not be able to find static member variables when linking with produced dll...


 public:
  
  static void loadQuantificationThresholderPlugin(const QDir & pluginsDir,
						  QList<std::pair<plugin_status, QString> >& logMessages,
						  bool resolveVersion);
  
  static QIsePluginList<QuantificationThresholderPlugin*>& getQuantificationThresholderPlugin();
  
  static void loadSegmentationClusteringLocalThresholderPlugin(const QDir & pluginsDir,
							       QList<std::pair<plugin_status, QString> >& logMessages,
							       bool resolveVersion);
  
  static QIsePluginList<SegmentationClusteringLocalThresholderPlugin*>& getSegmentationClusteringLocalThresholderPlugin();
  
  static void loadRegionSetPlugin(const QDir &pluginsDir,
                                              QList<std::pair<plugin_status, QString> > &logMessages,
                                              bool resolveVersion);

  static QIsePluginList<RegionSetInterfacePlugin*>& getRegionSetPlugin();

  static FeaturePlugin * getRegionSetPlugin(const std::string &name);

  // ImageFeaturePlugin

  static void loadImageFeaturePlugins(QString pluginPath,
				      QList<std::pair<plugin_status, QString> > &logMessages,
				      bool resolveVersion);

  static FeaturePlugin * getImageFeaturePlugin(const std::string &name);

  /* ImageFeatureColorimetric Loader */
  static void loadImageFeatureColorimetricPlugin(const QDir &pluginsDir,
  						 QList<std::pair<plugin_status, QString> > &logMessages,
  						 bool resolveVersion);

  static QIsePluginList<ImageFeatureColorimetricPlugin*>& getImageFeatureColorimetricPlugin();

 /* ImageFeatureBinary Loader */
  static void loadImageFeatureBinaryPlugin(const QDir &pluginsDir,
					   QList<std::pair<plugin_status, QString> > &logMessages,
					   bool resolveVersion);

  static QIsePluginList<ImageFeatureBinaryPlugin*>& getImageFeatureBinaryPlugin();

  /* ImageFeatureGeometric Loader */
  static void loadImageFeatureGeometricPlugin(const QDir &pluginsDir,
  					      QList<std::pair<plugin_status, QString> > &logMessages,
  					      bool resolveVersion);

  static QIsePluginList<ImageFeatureGeometricPlugin*>& getImageFeatureGeometricPlugin();

  /* ImageFeatureTexture Loader */
  static void loadImageFeatureTexturePlugin(const QDir &pluginsDir,
					      QList<std::pair<plugin_status, QString> > &logMessages,
					      bool resolveVersion);

  static QIsePluginList<ImageFeatureTexturePlugin*>& getImageFeatureTexturePlugin();

  /* ImageFeatureComposition Loader */
  static void loadImageFeatureCompositionPlugin(const QDir &pluginsDir,
					      QList<std::pair<plugin_status, QString> > &logMessages,
					      bool resolveVersion);

  static QIsePluginList<ImageFeatureCompositionPlugin*>& getImageFeatureCompositionPlugin();

  /* ImageFeature other Loader */
  static void loadFeatureTabPlugin(const QDir &pluginsDir,
				   QList<std::pair<plugin_status, QString> > &logMessages,
				   bool resolveVersion);
  
  static QIsePluginList<FeatureTabPlugin*>& getFeatureTabPlugin();

  /* ImageFeature widget Loader */
  static void loadImageFeatureWidgetPlugin(const QDir &pluginsDir,
					   QList<std::pair<plugin_status, QString> > &logMessages,
					   bool resolveVersion);
  
  static QIsePluginList<ImageFeatureWidgetPlugin*>& getImageFeatureWidgetPlugin();


  /* Fin imageFeature */

  static void loadPyramidPlugin(const QDir &pluginsDir,
				QList<std::pair<plugin_status, QString> > &logMessages,
				bool resolveVersion);

  static QIsePluginList<PyramidPlugin*>& getPyramidPlugin();
  
  static void loadRegionUnaryFunctionPlugin(const QDir &pluginsDir,
       QList<std::pair<plugin_status, QString> > &logMessages,
       bool resolveVersion);

   static QIsePluginList<RegionUnaryFunctionPlugin*>&
   getRegionUnaryFunctionPlugin();

   static void loadRegionUnaryPredicatePlugin(const QDir &pluginsDir,
      QList<std::pair<plugin_status, QString> > &logMessages,
      bool resolveVersion);

  static QIsePluginList<RegionUnaryPredicatePlugin*>&
  getRegionUnaryPredicatePlugin();


  static void loadRegionBinaryFunctionPlugin(const QDir &pluginsDir,
      QList<std::pair<plugin_status, QString> > &logMessages,
      bool resolveVersion);

  static QIsePluginList<RegionBinaryFunctionPlugin*>&
  getRegionBinaryFunctionPlugin();

  static void loadRegionBinaryPredicatePlugin(const QDir &pluginsDir,
      QList<std::pair<plugin_status, QString> > &logMessages,
      bool resolveVersion);

  static QIsePluginList<RegionBinaryPredicatePlugin*>&
  getRegionBinaryPredicatePlugin();

  static void loadRegionStrictWeakOrderingPlugin(const QDir &pluginsDir,
						 QList<std::pair<plugin_status, QString> > &logMessages,
						 bool resolveVersion);

  static QIsePluginList<RegionStrictWeakOrderingPlugin*>&
    getRegionStrictWeakOrderingPlugin();

  static void loadClustererPlugin(const QDir &pluginsDir,
				  QList<std::pair<plugin_status, QString> > &logMessages,
				  bool resolveVersion);

  static QIsePluginList<ClustererPlugin*>& getClustererPlugin();

 protected:

  // IMPORTANT: Don't forget to add the plugin list in the .cpp file

  static QIsePluginList<QuantificationThresholderPlugin*> m_quantificationThresholderPlugin;
  static QIsePluginList<SegmentationClusteringLocalThresholderPlugin*> m_segmentationClusteringLocalThresholderPlugin;
  static QIsePluginList<RegionSetInterfacePlugin*> m_regionSetPlugin;

  static QIsePluginList<ImageFeatureColorimetricPlugin*> m_imageFeatureColorimetricPlugin;
  static QIsePluginList<ImageFeatureBinaryPlugin*> m_imageFeatureBinaryPlugin;
  static QIsePluginList<ImageFeatureGeometricPlugin*> m_imageFeatureGeometricPlugin;
  static QIsePluginList<ImageFeatureTexturePlugin*> m_imageFeatureTexturePlugin;
  static QIsePluginList<ImageFeatureCompositionPlugin*> m_imageFeatureCompositionPlugin;
  static QIsePluginList<FeatureTabPlugin*> m_featureTabPlugin;
  static QIsePluginList<ImageFeatureWidgetPlugin*> m_imageFeatureWidgetPlugin;

  static QIsePluginList<PyramidPlugin*> m_pyramidPlugin;
  static QIsePluginList<RegionUnaryFunctionPlugin*> m_regionUnaryFunctionPlugin;
  static QIsePluginList<RegionUnaryPredicatePlugin*> m_regionUnaryPredicatePlugin;
  static QIsePluginList<RegionBinaryFunctionPlugin*> m_regionBinaryFunctionPlugin;
  static QIsePluginList<RegionBinaryPredicatePlugin*> m_regionBinaryPredicatePlugin;
  static QIsePluginList<RegionStrictWeakOrderingPlugin*> m_regionStrictWeakOrderingPlugin;

  static QIsePluginList<ClustererPlugin*> m_clustererPlugin;

};

#endif /*PLUGINREGISTRY_HPP */
