/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "PyramidChoiceWidget.hpp"

#include <cassert>

#include <QLabel>
#include <QVBoxLayout>
#include <string>

#include <QMap>
#include <QGroupBox>

#include "PluginRegistry.hpp"

PyramidChoiceWidget::PyramidChoiceWidget(MainView *mainView, QWidget *parent)
  : QWidget(parent)
  , m_mainView(mainView)
  , m_pyramidCombo(NULL)
  , m_numLevels(0)
  , m_divider(0)
{
  build();
  setGUI();
}

void
PyramidChoiceWidget::init(int numLevels, int divider)
{
  m_numLevels = numLevels;
  m_divider = divider;
}

void
PyramidChoiceWidget::build()
{
  QGroupBox *group = new QGroupBox(tr("Pyramid"), this);
  QVBoxLayout *groupLayout = new QVBoxLayout;
  group->setLayout(groupLayout);

  QIsePluginList<PyramidPlugin *> list = PluginRegistry::getPyramidPlugin();
  m_pyramidCombo = new PluginIndexChoiceWidget(list.getGenericList(), tr("Pyramid"), this);
  connect(m_pyramidCombo, SIGNAL(pluginIndexChanged(int)), this, SLOT(showWidget(int)));
  groupLayout->addWidget(m_pyramidCombo);

  QIsePluginList<PyramidPlugin *>::const_iterator it = list.begin();
  const QIsePluginList<PyramidPlugin *>::const_iterator itEnd = list.end();
  for ( ; it != itEnd ; ++it) {
    PyramidWidget * widget = (*it)->parameterWidget(m_mainView, m_numLevels, m_divider, NULL);
    m_pyramidWidgets[(*it)->name()] = widget;
    m_pyramidWidgets[(*it)->name()]->hide();
    groupLayout->addWidget(m_pyramidWidgets[(*it)->name()], 1, 0);
  }

  m_activePyramidWidget = m_pyramidWidgets[m_pyramidCombo->getPluginName(0)];
  if (m_activePyramidWidget != NULL)
    m_activePyramidWidget->show();

  showWidget(0);

  QVBoxLayout *hbox = new QVBoxLayout;
  hbox->addWidget(group);
  hbox->addStretch();

  hbox->setMargin(0);
  this->setLayout(hbox);
}

void
PyramidChoiceWidget::showWidget(int index)
{
  const QString name = m_pyramidCombo->getPluginName(index);
  
  if (m_activePyramidWidget != NULL)
    m_activePyramidWidget->hide();

  m_activePyramidWidget = m_pyramidWidgets[name];

  if (m_activePyramidWidget != NULL)
    m_activePyramidWidget->show();
}

size_t
PyramidChoiceWidget::size() const {
  return m_pyramidCombo->count();
}

girl::shared_ptr<ise::Pyramid>
PyramidChoiceWidget::getPyramid()
{
  getGUI();
  return m_activePyramid;
}

void
PyramidChoiceWidget::setGUI()
{
  
}

void
PyramidChoiceWidget::getGUI()
{
  const QString name = m_pyramidCombo->getPluginName();

  PyramidWidget* pyramid = m_pyramidWidgets[name];
  if (pyramid != NULL) {
    pyramid->setData(m_numLevels, m_divider);
    pyramid->saveSettingsInPlugin();
    girl::shared_ptr<ise::Pyramid> activePyramid = pyramid->getFactory()->getNewPyramid();

    m_activePyramid = activePyramid;
  }
  else {
    m_activePyramid = girl::shared_ptr<ise::Pyramid>();
  }
}
