/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#ifndef PYRAMIDCHOICEWIDGET_HPP
#define PYRAMIDCHOICEWIDGET_HPP

#include <QVector>
#include <QWidget>
#include <QComboBox>

#include "PluginIndexChoiceWidget.hpp"
#include "PluginRegistry.hpp"
#include <ise/isePyramid.hpp>

#include <girl/girlCommon.hpp>
#include "MainView.hpp"

class QLabel;

class GIRL_API PyramidChoiceWidget : public QWidget
{
  Q_OBJECT

public:
  /**
   * @brief Constructor.
   */
  PyramidChoiceWidget(MainView *mainView, QWidget *parent);

  void init(int numLevels, int divider);

  inline void setNumLevels(int numLevels) { m_numLevels = numLevels;}
  inline void setDivider(int divider) { m_divider = divider;}

  girl::shared_ptr<ise::Pyramid> getPyramid();

  size_t size() const;

  void setGUI();
	      
  void getGUI();

public slots:

  void showWidget(int index);

protected:

  void build();

protected:
  MainView *m_mainView;
  
  PluginIndexChoiceWidget *m_pyramidCombo;

  int m_numLevels;
  int m_divider;

  QMap<QString, PyramidWidget* > m_pyramidWidgets;
  PyramidWidget* m_activePyramidWidget;
  girl::shared_ptr<ise::Pyramid> m_activePyramid;
};

#endif /* PYRAMIDCHOICEWIDGET_HPP */
