/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef PYRAMIDPLUGIN_HPP
#define PYRAMIDPLUGIN_HPP

#include "QIsePlugin.hpp"

#include <QString>

#include <ise/isePyramid.hpp>

#include <qiseTools/PyramidFactory.hpp>

#include <qiseTools/PyramidWidget.hpp>

#include <qiseTools/PyramidSettingsInterface.hpp>

#include "MainView.hpp"

class GIRL_API PyramidPlugin : public QIsePlugin,
			       public PyramidFactory
{
  
 public:
  
  virtual ~PyramidPlugin()
  {}
  
  virtual girl::shared_ptr<ise::Pyramid> getNewPyramid() = 0;
  
  //virtual void saveParameters(const girl::shared_ptr<ise::Pyramid> &sa) = 0;
  
  virtual PyramidWidget *parameterWidget(MainView *mainView, int numLevel, int divider, QWidget *parent = NULL) 
  {
    if (m_settings)
      m_settings = createSettings();

    PyramidWidget *w = createWidget(mainView, numLevel, divider, parent);

    if (w != NULL)
      w->setGUI(m_settings);
    return w;
  }

  virtual girl::shared_ptr<PyramidSettingsInterface> createSettings() = 0;

  virtual PyramidWidget *createWidget(MainView *mainView, int numLevel, int divider, QWidget *parent = NULL ) = 0;

  virtual girl::shared_ptr<PyramidSettingsInterface> settings() { return m_settings; }

  virtual bool hasSettings() const
  {
    return true;
  }


protected:
  
  PyramidPlugin(const QString &name,
		const QIseVersion &pluginVersion,
		const QIseVersion &qiseVersion,
		QString path = QString::fromUtf8(""))
    : QIsePlugin(name, (QString::fromUtf8("Pyramid/") + path),
		 pluginVersion, qiseVersion)
  {}
  
private:
  
  PyramidPlugin();

 protected:

  girl::shared_ptr<PyramidSettingsInterface> m_settings;
};

#define PyramidPlugin_iid "org.girl.qise.PyramidPlugin/1.0"
Q_DECLARE_INTERFACE(PyramidPlugin, PyramidPlugin_iid)

#endif /* ! PYRAMIDPLUGIN_HPP */
