/*
  Copyright (C) 2005-2011 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef PYRAMIDWIDGET_HPP
#define PYRAMIDWIDGET_HPP

#include <QWidget>
#include <girl/girlCommon.hpp>
#include <ise/isePyramid.hpp>
#include "PyramidSettingsInterface.hpp"

#include "PyramidFactory.hpp" 
#include "MainView.hpp"

class PyramidWidget : public QWidget
{
public:
  
  PyramidWidget(MainView *mainView,
		PyramidFactory *pyramidFactory,
		int numLevel,
		int divider,
		QWidget *parent = NULL)
    : QWidget(parent)
    , m_mainView(mainView)
    , m_pyramidFactory(pyramidFactory)
    , m_numLevel(numLevel)
    , m_divider(divider)
  {
    hide();
  }

  virtual ~PyramidWidget() {}
    
  virtual void build() = 0;
  
  void setGUI(const girl::shared_ptr<PyramidSettingsInterface> settings)
  {
    settings->configureWidget(this);
  }

  void getGUI(girl::shared_ptr<PyramidSettingsInterface> settings) const
  {
    settings->getWidgetConfig(this);
  }

  void saveSettingsInPlugin() const
  {
    getGUI(m_pyramidFactory->settings());
  }

  PyramidFactory *getFactory() { return m_pyramidFactory;}

  void getData(int &numLevel, int &divider) const
  {
    numLevel = m_numLevel;
    divider = m_divider;
  }

  void setData(int numLevel, int divider)
  {
    m_numLevel = numLevel;
    m_divider = divider;
  }

protected:

  MainView *m_mainView;
  PyramidFactory *m_pyramidFactory;
  int m_numLevel;
  int m_divider;
};

#endif /* ! PYRAMIDWIDGET_HPP */
