/*
  Copyright (C) 2005-2011 Jerome Charton, Boris Mansencal boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#ifndef QISEPLUGIN_HPP
#define QISEPLUGIN_HPP

#include <QtPlugin>

#include <QString>

#include <utility>

#include "QIseVersion.hpp"
#include "QIsePluginDependance.hpp"

#include <iostream> //DEBUG

class GIRL_API QIsePlugin
{
  
public:
  
  virtual ~QIsePlugin() {}
  
  inline QString name() const {return m_name;}
  inline QString path() const {return m_path;}
  inline QString fullName() const {return m_path+m_name;}
  
  inline QIseVersion version() const {return m_version;}
  inline QIseVersion requiredQIseVersion() const {return m_qiseVersion;}
  
  inline void addDependance(const QIsePluginDependance &dependance)
  {m_dependances.push_back(dependance);}
  
  inline const QList<QIsePluginDependance> getDependances() const
  {return m_dependances;}
  
  inline bool dependsOf(QString fullName) const
  {
    bool result = false;
    QList<QIsePluginDependance>::const_iterator depIterator = getDependances().begin();
    QList<QIsePluginDependance>::const_iterator depEnd = getDependances().end();
    for ( ; depIterator != depEnd; ++depIterator)
      result |= (*depIterator).isRightPlugin(fullName);
    return result;
  }

  inline bool isCompatibleWith(const QIsePlugin &plugin) const
  {
    if (!dependsOf(plugin.fullName()))
      return true;
    
    QList<QIsePluginDependance>::const_iterator depIterator = getDependances().begin();
    QList<QIsePluginDependance>::const_iterator depEnd = getDependances().end();
    for ( ; depIterator != depEnd; ++depIterator)
      if ((*depIterator).isRightPlugin(plugin)
	  && (*depIterator).isCompatible(plugin))
	return true;
    
    return false;
  }
  
protected:
  
  QIsePlugin(QString name,
	     QString path,
	     QIseVersion pluginVersion,
	     QIseVersion qiseVersion)
    : m_name(name),
      m_path(path),
      m_version(pluginVersion),
      m_qiseVersion(qiseVersion),
      m_dependances() {}
  
private:
  
  QString m_name;
  QString m_path;
  QIseVersion m_version;
  QIseVersion m_qiseVersion;
  QList<QIsePluginDependance> m_dependances;
  
};

#define QIsePlugin_iid "org.girl.qise.QIsePlugin/1.0"
Q_DECLARE_INTERFACE(QIsePlugin, QIsePlugin_iid)

#endif /* QISEPLUGIN_HPP */
