/*
  Copyright (C) 2005-2011 Jerome Charton, Boris Mansencal boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#include "QIsePluginDependance.hpp"

#include "QIsePlugin.hpp"

const QIseVersion QIsePluginDependance::MAXIMAL_VERSION(9999, 9999, 00);

const QIseVersion QIsePluginDependance::EQUALS_ON_MAJOR(9999, 9999, 01);
const QIseVersion QIsePluginDependance::EQUALS_ON_MAJOR_AND_MINOR(9999, 9999, 02);
const QIseVersion QIsePluginDependance::EQUALS_STRICT(9999, 9999, 03);
const QIseVersion QIsePluginDependance::EQUALS_AND_GREATHER(9999, 9999, 04);
const QIseVersion QIsePluginDependance::EQUALS_AND_LESSER(9999, 9999, 05);
const QIseVersion QIsePluginDependance::GREATHER_THAN(9999, 9999, 06);
const QIseVersion QIsePluginDependance::LESSER_THAN(9999, 9999, 07);

bool
QIsePluginDependance::isRightPlugin(const QString &fullName) const {
  return (fullName == m_fullName);
}

bool
QIsePluginDependance::isRightPlugin(const QIsePlugin &plugin) const {
  return isRightPlugin(plugin.fullName());
}

bool
QIsePluginDependance::isCompatible(const QIsePlugin &plugin) const {
  return (isRightPlugin(plugin) && isCompatible(plugin.version()));
}
  
bool
QIsePluginDependance::isCompatible(const QIseVersion &version) const {
  bool result = false;
  
  QList<std::pair<QIseVersion, QIseVersion> >::const_iterator it = m_versionIntervals.begin();
  QList<std::pair<QIseVersion, QIseVersion> >::const_iterator end = m_versionIntervals.end();
  
  for ( ; it != end; ++it) {
    QIseVersion referenceVersion = (*it).first;
    QIseVersion extendedVersion = (*it).second;
    if (extendedVersion == EQUALS_ON_MAJOR) {
      result |= (version.major() == referenceVersion.major());
    } else if (extendedVersion == EQUALS_ON_MAJOR_AND_MINOR) {
      result |= ((version.major() == referenceVersion.major())
		 && (version.minor() == referenceVersion.minor()));
    } else if (extendedVersion == EQUALS_STRICT) {
      result |= (version == referenceVersion);
    } else if (extendedVersion == EQUALS_AND_GREATHER) {
      result |= (version >= referenceVersion);
    } else if (extendedVersion == EQUALS_AND_LESSER) {
      result |= (version <= referenceVersion);
    } else if (extendedVersion == GREATHER_THAN) {
      result |= (version > referenceVersion);
    } else if (extendedVersion == LESSER_THAN) {
      result |= (version < referenceVersion);
    } else {
      result |= (((version >= referenceVersion)
		  && (version <= extendedVersion))
		 || ((version <= referenceVersion)
		     && (version >= extendedVersion)));
    }
  
    if (!result)
      std::cerr << fullName().toStdString() << " v"
		<< version.major() << "."
		<< version.minor() << "."
		<< version.micro()
		<< " is not in : "
		<< " v"
		<< referenceVersion.major() << "."
		<< referenceVersion.minor() << "."
		<< referenceVersion.micro()
		<< ":  v"
		<< extendedVersion.major() << "."
		<< extendedVersion.minor() << "."
		<< extendedVersion.micro() << std::endl;
  }
  
  return result;
}
