#ifndef QISEPLUGINDEPENDCANCE_HPP
#define QISEPLUGINDEPENDCANCE_HPP

#include <QObject>
#include <QMetaType>
#include <QString>
#include <QList>

#include <utility>

#include "QIseVersion.hpp"

class QIsePlugin;

class GIRL_API QIsePluginDependance {

public:
  
  QIsePluginDependance(QString fullName = QObject::tr(""),
		       QIseVersion referenceVersion = QIseVersion(1),
		       QIseVersion extendedVersion = QIsePluginDependance::EQUALS_ON_MAJOR)
    : m_fullName(fullName),
      m_versionIntervals()
  {addVersions(referenceVersion, extendedVersion);}
  
  inline void addVersions(QIseVersion referenceVersion,
			  QIseVersion extendedVersion = QIsePluginDependance::EQUALS_ON_MAJOR)
  {m_versionIntervals.push_back(std::pair<QIseVersion, QIseVersion>
				(referenceVersion, extendedVersion));}
  
  inline void setFullName(QString fullName) {m_fullName = fullName;}

  inline void setVersions(const QList<std::pair<QIseVersion, QIseVersion> > &versionIntervals)
  {m_versionIntervals = versionIntervals;}

  inline QString fullName() const {return m_fullName;}
  
  inline const QList<std::pair<QIseVersion, QIseVersion> > &versions() const {return m_versionIntervals;}
  
  bool isRightPlugin(const QString &fullName) const;
  
  bool isRightPlugin(const QIsePlugin &plugin) const;
  
  bool isCompatible(const QIsePlugin &version) const;
  
  bool isCompatible(const QIseVersion &version) const;
  
public:
  
  static const QIseVersion MAXIMAL_VERSION;
  
  static const QIseVersion EQUALS_ON_MAJOR;
  static const QIseVersion EQUALS_ON_MAJOR_AND_MINOR;
  static const QIseVersion EQUALS_STRICT;
  static const QIseVersion EQUALS_AND_GREATHER;
  static const QIseVersion EQUALS_AND_LESSER;
  static const QIseVersion GREATHER_THAN;
  static const QIseVersion LESSER_THAN;

private:
  
  QString m_fullName;
  QList<std::pair<QIseVersion, QIseVersion> > m_versionIntervals;
  
};

Q_DECLARE_METATYPE(QIsePluginDependance)

#endif /* ! QISEPLUGINDEPENDCANCE_HPP */
