/*
  Copyright (C) 2005-2011 Jerome Charton, Boris Mansencal boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#include "QIsePluginDependancesSolver.hpp"

void
QIsePluginDependancesSolver::resolveDependances(QList<std::pair<plugin_status, QString> >& logMessages) {
  /*
  for (int i=0; i != m_pluginsLists.size(); ++i)
    for (int j=0; j != m_pluginsLists[i]->size(); ++j)
      if (!dependancesAreSatisfies(m_pluginsLists[i]->at(j))) {
	m_pluginsLists[i]->removeAt(j);
	i = j = 0;
      }
  
  QVector<QList<const QIsePlugin *> > lists;
  
  for (int i=0; i != m_pluginsLists.size(); ++i)
    for (int j=0; j != m_pluginsLists[i]->size(); ++j)
      lists.push_back(dependancesList(m_pluginsLists[i]->at(j)));
  
  QList<const QIsePlugin *> result = getBestPlugins(lists);
  
  for (int i=0; i != m_pluginsLists.size(); ++i)
    for (int j=0; j != m_pluginsLists[i]->size(); ++j)
      if (result.indexOf(m_pluginsLists[i]->at(j)) == -1) {
	m_pluginsLists[i]->removeAt(j);
	j = 0;
      }
  */
  
  for (int i=0; i != m_pluginsLists.size(); ++i)
    m_pluginsLists[i]->resolveVersioning(logMessages);
  
  QMap<QString, QList<const QIsePlugin*> > pluginMap;
  
  QList<const QIsePlugin *> result;

  fillMap(pluginMap);
  
  
  /*{
    QMapIterator<QString, QList<const QIsePlugin *> > mapIt(pluginMap);
    while (mapIt.hasNext()) {
      const QList<const QIsePlugin *>& samePlugins = mapIt.next().value();
      if ((*samePlugins).size() != 1) {
	(*samePlugins).clear();
      }
    }
    }*/
  
  {
    QMap<QString, QList<const QIsePlugin *> >::iterator mapIt = pluginMap.begin();
    QMap<QString, QList<const QIsePlugin *> >::iterator mapEnd = pluginMap.end();
    for ( ; mapIt != mapEnd; ++mapIt) {
      QList<const QIsePlugin *>& samePlugins = *mapIt;
      if (!dependancesAreSatisfies(samePlugins.front(), logMessages))
	samePlugins.clear();
    }
  }
  
}
/*

QList<const QIsePlugin *>
QIsePluginDependancesSolver::getBestPlugins(QVector<QList<const QIsePlugin *> > lists) const {
  QList<const QIsePlugin *> lastBest;
  
  for (int i=0; i != lists.size(); ++i) {
      QList<QList<const QIsePlugin *> > results;
      tryToAdd(i, lists, results);
      
      QList<QList<const QIsePlugin *> >::const_iterator it = results.begin();
      QList<QList<const QIsePlugin *> >::const_iterator end = results.end();
      for ( ; it != end; ++it)
	lastBest = compare(lastBest, *it);
    }
  
  return lastBest;
}

void
QIsePluginDependancesSolver::tryToAdd(int index,
				      QVector<QList<const QIsePlugin *> > lists,
				      QList<QList<const QIsePlugin *> >& results,
				      QList<const QIsePlugin *> inBuilding) const {
  if (mergeWith(inBuilding, lists[index])) {
    lists.remove(index);
    if (lists.isEmpty())
      results.push_back(inBuilding);
    else
      for (int i=0; i < lists.size(); ++i) {
	tryToAdd(i, lists, results, inBuilding);
    }
  } else {
    results.push_back(inBuilding);
  }
  std::cerr << "Results :" << results.size() << std::endl;
}

bool
QIsePluginDependancesSolver::areCompatible(const QList<const QIsePlugin *>& plugins1,
					   const QList<const QIsePlugin *>& plugins2) const {

  QList<const QIsePlugin *>::const_iterator it1 = plugins1.begin();
  QList<const QIsePlugin *>::const_iterator end1 = plugins1.end();
  QList<const QIsePlugin *>::const_iterator it2 = plugins2.begin();
  QList<const QIsePlugin *>::const_iterator end2 = plugins2.end();
  for ( ; it2 != end2; ++it2)
    for ( ; it1 != end1; ++it1)
      if (!(*it1)->isCompatibleWith(**it2))
	return false;
  return true;
}

*/

void
QIsePluginDependancesSolver::fillMap(QMap<QString, QList<const QIsePlugin*> >& map) const {
  
  QList<const QIsePlugin*> globalList;
    
  QVector<QIsePluginList<QIsePlugin *> *>::const_iterator globalIterator = m_pluginsLists.begin();
  QVector<QIsePluginList<QIsePlugin *> *>::const_iterator globalEnd = m_pluginsLists.end();
  for ( ; globalIterator != globalEnd; ++globalIterator) {
    QIsePluginList<QIsePlugin *>::const_iterator localIterator = (*globalIterator)->begin();
    QIsePluginList<QIsePlugin *>::const_iterator localEnd = (*globalIterator)->end();
    for ( ; localIterator != localEnd; ++localIterator) {
      if (*localIterator != NULL)
	globalList.push_back(*localIterator);
    }
  }
  fillMap(globalList, map);
}

void
QIsePluginDependancesSolver::fillMap(const QList<const QIsePlugin*>& list,
				     QMap<QString, QList<const QIsePlugin*> >& map) const {
  QList<const QIsePlugin *>::const_iterator it = list.begin();
  QList<const QIsePlugin *>::const_iterator end = list.end();
  for ( ; it != end; ++it) {
    if (*it != NULL)
      map[(*it)->fullName()].push_back(*it);
  }
}
/*
bool
QIsePluginDependancesSolver::mergeWith(QList<const QIsePlugin *>& dest,
				       const QList<const QIsePlugin *>& pluginsToAdd) const {
  QMap<QString, QList<const QIsePlugin *> > mapDest;
  QMap<QString, QList<const QIsePlugin *> > mapToAdd;
  
  QList<const QIsePlugin *> result;

  fillMap(dest, mapDest);
  fillMap(pluginsToAdd, mapToAdd);
  
  QMapIterator<QString, QList<const QIsePlugin *> > mapItToAdd(mapToAdd);
  while (mapItToAdd.hasNext()) {
    const QList<const QIsePlugin *>& samePluginsToAdd = mapItToAdd.next().value();
    QList<const QIsePlugin *>::const_iterator itSamePluginsToAdd = samePluginsToAdd.begin();
    QList<const QIsePlugin *>::const_iterator endSamePluginsToAdd = samePluginsToAdd.end();
    bool commonFounded = false;
    for ( ; itSamePluginsToAdd != endSamePluginsToAdd; ++itSamePluginsToAdd) {
      if (mapDest[(*itSamePluginsToAdd)->fullName()].isEmpty()) {
	result.push_back(*itSamePluginsToAdd);
	commonFounded = true;
      } else {
	const QList<const QIsePlugin *>& samePlugins = mapDest[(*itSamePluginsToAdd)->fullName()];
	QList<const QIsePlugin *>::const_iterator itSamePlugins = samePlugins.begin();
	QList<const QIsePlugin *>::const_iterator endSamePlugins = samePlugins.end();
	for ( ; itSamePlugins != endSamePlugins; ++itSamePlugins) {
	  if ((*itSamePlugins)->version() == (*itSamePluginsToAdd)->version()) {
	    result.push_back(*itSamePluginsToAdd);
	    commonFounded = true;
	  }
	}
      }
    }
    if (!commonFounded)
      return false;
  }

  QMapIterator<QString, QList<const QIsePlugin *> > mapItDest(mapDest);
  while (mapItDest.hasNext()) {
    const QList<const QIsePlugin *>& samePluginsDest = mapItDest.next().value();
    QList<const QIsePlugin *>::const_iterator itSamePluginsDest = samePluginsDest.begin();
    QList<const QIsePlugin *>::const_iterator endSamePluginsDest = samePluginsDest.end();
    for ( ; itSamePluginsDest != endSamePluginsDest; ++itSamePluginsDest) {
      if (mapDest[(*itSamePluginsDest)->fullName()].isEmpty())
	result.push_back(*itSamePluginsDest);
    }
  } 

  dest = result;
  
  return !dest.isEmpty();
}

const QList<const QIsePlugin *>&
QIsePluginDependancesSolver::compare(const QList<const QIsePlugin *>& plugins1,
				     const QList<const QIsePlugin *>& plugins2) const {
  std::pair<int, int> score1 = getScore(plugins1);
  std::pair<int, int> score2 = getScore(plugins2);
 
  return (score1.first > score2.first) ? plugins1 :
    ((score1.first < score2.first) ? plugins2 : 
     ((score1.second <= score2.second) ? plugins1 : plugins2));
}


std::pair<int, int>
QIsePluginDependancesSolver::getScore(const QList<const QIsePlugin *>& plugins) const {
  std::pair<int, int> score(0, 0);
  QMap<QString, QList<const QIsePlugin*> > pluginsSortedByName;
 
  QList<const QIsePlugin *>::const_iterator it = plugins.begin();
  QList<const QIsePlugin *>::const_iterator end = plugins.end();
  for ( ; it != end; ++it) {
    const QIsePlugin *plugin = *it;
    if (plugin != NULL) {
      pluginsSortedByName[plugin->fullName()].push_back(plugin);
    }
  }
   
  score.first = pluginsSortedByName.size();
  
  QMapIterator<QString, QList<const QIsePlugin*> > mapPlugIterator(pluginsSortedByName);
  while (mapPlugIterator.hasNext()) {
    QList<const QIsePlugin *> samePlugins = mapPlugIterator.next().value();
    const QIsePlugin *lastVersion = NULL;
    
    while (samePlugins.size() != 0) {
      const QIsePlugin *currentPlugin = samePlugins.takeFirst();
      if (lastVersion == NULL
	  || ((currentPlugin != NULL)
	      && (currentPlugin->version() > lastVersion->version())))
	lastVersion = currentPlugin;
    }
    
    int nbBetter = 0;
    
    for (int i=0; i != m_pluginsLists.size(); ++i)
      for (int j=0; j != m_pluginsLists[i]->size(); ++j)
	if (m_pluginsLists[i]->at(j)->fullName() == lastVersion->fullName())
	  if (m_pluginsLists[i]->at(j)->version() > lastVersion->version())
	    ++nbBetter;
    
    score.second += nbBetter;
  }
  
  return score;
}

QList<const QIsePlugin *>
QIsePluginDependancesSolver::dependancesList(const QIsePlugin *plugin) const {
  QList<const QIsePlugin *> dependances;
  dependances.push_back(plugin);
  
  QList<QIsePluginDependance>::const_iterator depIterator = plugin->getDependances().begin();
  QList<QIsePluginDependance>::const_iterator depEnd = plugin->getDependances().end();
  
  for ( ; depIterator != depEnd; ++depIterator) {
    //bool deptSolved = false;
    
    QVector<QIsePluginList<QIsePlugin *> *>::const_iterator globalIterator = m_pluginsLists.begin();
    QVector<QIsePluginList<QIsePlugin *> *>::const_iterator globalEnd = m_pluginsLists.end();
    
    for ( ; globalIterator != globalEnd; ++globalIterator) {
      
      QIsePluginList<QIsePlugin *>::const_iterator localIterator = (*globalIterator)->begin();
      QIsePluginList<QIsePlugin *>::const_iterator localEnd = (*globalIterator)->end();
      
      for ( ; localIterator != localEnd; ++localIterator)
	if ((*depIterator).isRightPlugin(**localIterator)
	    && (*depIterator).isCompatible(**localIterator))
	  dependances.push_back(*localIterator);
    }
    
  }
  
  return dependances;
}

*/

bool
QIsePluginDependancesSolver::dependancesAreSatisfies(const QIsePlugin *plugin,
						     QList<std::pair<plugin_status, QString> >& logMessages,
						     QList<const QIsePlugin *> parents) const {
  // Avoid Loop
  QList<const QIsePlugin *>::const_iterator it = parents.begin();
  QList<const QIsePlugin *>::const_iterator end = parents.end();
  for ( ; it != end; ++it)
    if (*it == plugin)
      return true;
  
  parents.push_back(plugin);
  
  bool result = true;
  
  QList<QIsePluginDependance>::const_iterator depIterator = plugin->getDependances().begin();
  QList<QIsePluginDependance>::const_iterator depEnd = plugin->getDependances().end();
  
  for ( ; depIterator != depEnd; ++depIterator) {
    bool deptSolved = false;
    
    QVector<QIsePluginList<QIsePlugin *> *>::const_iterator globalIterator = m_pluginsLists.begin();
    QVector<QIsePluginList<QIsePlugin *> *>::const_iterator globalEnd = m_pluginsLists.end();
    
    for ( ; globalIterator != globalEnd; ++globalIterator) {
      
      QIsePluginList<QIsePlugin *>::const_iterator localIterator = (*globalIterator)->begin();
      QIsePluginList<QIsePlugin *>::const_iterator localEnd = (*globalIterator)->end();
      
      for ( ; localIterator != localEnd; ++localIterator) {
	if ((*depIterator).isRightPlugin(**localIterator)) {
	  //if (isLoadable(*localIterator))
	  deptSolved |= dependancesAreSatisfies(*localIterator, logMessages, parents);
	}
      }
    }
    
    if (!deptSolved) {
      QString error;
      error += plugin->fullName() + " v"
		+ plugin->version().major() + '.'
		+ plugin->version().minor() + '.'
		+ plugin->version().micro()
		+ " depends of "
		+ (*depIterator).fullName();
      const QList<std::pair<QIseVersion, QIseVersion> > versions = (*depIterator).versions();
      QList<std::pair<QIseVersion, QIseVersion> >::const_iterator versionsIt= versions.begin();
      QList<std::pair<QIseVersion, QIseVersion> >::const_iterator versionsEnd= versions.end();
      for ( ; versionsIt != versionsEnd; ++versionsIt) {
	error += QString::fromUtf8("[ v")
		  + (*versionsIt).first.major() + '.'
		  + (*versionsIt).first.minor() + '.'
		  + (*versionsIt).first.micro();
	if ((*versionsIt).first != (*versionsIt).second)
	  error += QString::fromUtf8(", v")
		    + (*versionsIt).second.major() + '.'
		    + (*versionsIt).second.minor() + '.'
		    + (*versionsIt).second.micro();
	error += QString::fromUtf8(" ]");
	logMessages.push_back(std::pair<plugin_status, QString>(DEPENDANCES_UNSATISFIED, error));
      }
    }
    result &= deptSolved;
  }
  
  return result;
}
/*
bool
QIsePluginDependancesSolver::isLoadable(const QIsePlugin *plugin) const {

    QVector<QIsePluginList<QIsePlugin *> *>::const_iterator globalIterator = m_pluginsLists.begin();
    QVector<QIsePluginList<QIsePlugin *> *>::const_iterator globalEnd = m_pluginsLists.end();
    
    for ( ; globalIterator != globalEnd; ++globalIterator) {
      QList<QIsePlugin *>::const_iterator localIterator = (*globalIterator)->begin();
      QList<QIsePlugin *>::const_iterator localEnd = (*globalIterator)->end();
      for ( ; localIterator != localEnd; ++localIterator)
	if (!plugin->isCompatibleWith(*(*localIterator)))
	  return false;
    }
    return true;
}

QVector<const QIsePlugin *>
QIsePluginDependancesSolver::getSamePlugins(const QIsePlugin *plugin) const {
  QVector<const QIsePlugin *> result;
  
  QVector<QIsePluginList<QIsePlugin *> *>::const_iterator globalIterator = m_pluginsLists.begin();
  QVector<QIsePluginList<QIsePlugin *> *>::const_iterator globalEnd = m_pluginsLists.end();
  
  for ( ; globalIterator != globalEnd; ++globalIterator) {
    QList<QIsePlugin *>::const_iterator localIterator = (*globalIterator)->begin();
    QList<QIsePlugin *>::const_iterator localEnd = (*globalIterator)->end();
    for ( ; localIterator != localEnd; ++localIterator)
      if ((*localIterator) != plugin
	  && (*localIterator).fullName() == plugin.fullName())
	result.push_back(*localIterator);
  }
  
  return result;
}
*/
