/*
  Copyright (C) 2005-2010 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#ifndef QTOGERPAINTER_HPP
#define QTOGERPAINTER_HPP

#include <girl/girlToger.hpp>

#include <girl/euclideanPathSet.hpp>

class QPainter;

namespace QTogerPainter 
{
  
  typedef enum {DISCRETE_PATH, EUCLIDEAN_PATH} DrawMode;

  /**
   * @brief Draw boundaries of all regions of toger @a tog,
   * i.e., all darts, in discrete draw mode.
   */
  GIRL_API void drawToger(QPainter &p, const girl::toger &tog);

  /**
   * @brief Draw all darts of toger @a tog marked by @a mark 
   * in discrete draw mode.
   */
  GIRL_API void drawToger(QPainter &p, const girl::toger &tog, girl::toger::markFlags mark);

  /**
   * @brief Draw boundaries of all regions of toger @a tog,
   * i.e., all darts, with euclidean paths, defined by points in @a eps.
   */
  GIRL_API void drawToger(QPainter &p, const girl::toger &tog, const girl::euclideanPathSet &eps);

  /**
   * Draw all darts of toger @a tog marked by @a mark in euclidean draw mode.
   */
  GIRL_API void drawToger(QPainter &p, const girl::toger &tog, const girl::euclideanPathSet &eps, 
			  girl::toger::markFlags mark);
  

  
  /**
   * @brief Draw boundaries of regions @a regions of toger @a tog
   * in discrete draw mode.
   */  
  GIRL_API void drawRegions(QPainter &p, const girl::toger &tog, const girl::regionDeque &regions);

  /**
   * @brief Draw boundaries of regions @a regions of toger @a tog 
   * marked by @a mark in discrete draw mode.
   */
  GIRL_API void drawRegions(QPainter &p, const girl::toger &tog, const girl::regionDeque &regions, 
			    girl::toger::markFlags mark);

  /**
   * @brief Draw boundaries of regions @a regions of toger @a tog,
   * with euclidean paths, defined by points in @a eps.
   */
  GIRL_API void drawRegions(QPainter &p, const girl::toger &tog, const girl::regionDeque &regions, 
			    const girl::euclideanPathSet &eps);

  /**
   * @brief Draw boundaries of regions @a regions of toger @a tog,
   * marked by @a mark, with euclidean paths, defined by points in @a eps.
   */
  GIRL_API void drawRegions(QPainter &p, const girl::toger &tog, const girl::regionDeque &regions, 
			    const girl::euclideanPathSet &eps, girl::toger::markFlags mark);

  /**
   * @brief Draw segments of darts @a darts of toger @a tog
   * in discrete draw mode.
   */  
  GIRL_API void drawDarts(QPainter &p, const girl::toger &tog, const girl::dartDeque &darts);

  /**
   * @brief Draw segments of darts @a darts of toger @a tog, marked by @a mark 
   * in discrete draw mode.
   */
  GIRL_API void drawDarts(QPainter &p, const girl::toger &tog, const girl::dartDeque &darts, 
			  girl::toger::markFlags mark);

  /**
   * @brief Draw segments of darts @a darts of toger @a tog
   * with euclidean paths, defined by points in @a eps.
   */  
  GIRL_API void drawDarts(QPainter &p, const girl::toger &tog, const girl::dartDeque &darts, 
			  const girl::euclideanPathSet &eps);

  /**
   * @brief Draw segments of darts @a darts of toger @a tog, marked by @a mark 
   * with euclidean paths, defined by points in @a eps.
   */
  GIRL_API void drawDarts(QPainter &p, const girl::toger &tog, const girl::dartDeque &darts, 
			  const girl::euclideanPathSet &eps, girl::toger::markFlags mark);

#if 0
  /**
   * @brief Draw regions @a regions of toger @a tog.
   */  
  GIRL_API void drawFilledRegions(QPainter &p, const girl::toger &tog, const girl::regionDeque &regions);
  
  /**
   * @brief Draw regions @a regions of toger @a tog,
   * with euclidean paths, defined by points in @a eps.
   */  
  GIRL_API void drawFilledRegions(QPainter &p, const girl::toger &tog, const girl::regionDeque &regions, const girl::euclideanPathSet &eps);
  
  /**
   * @brief Draw region @a r of toger @a tog.
   */  
  GIRL_API void drawFilledRegion(QPainter &p, const girl::toger &tog, girl::region r);

  /**
   * @brief Draw region @a r of toger @a tog,
   * with euclidean paths, defined by points in @a eps.
   */  
  GIRL_API void drawFilledRegion(QPainter &p, const girl::toger &tog, girl::region r, const girl::euclideanPathSet &eps);
#endif


}

#endif /* ! QTOGERPAINTER_HPP */
