/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "QuantificationCAGWidget.hpp"

#include <ise/iseQuantificationCAG.hpp>

#include <QBoxLayout>
#include <QLabel>

#include "SliderSpinBoxWidget.hpp"
#include "WeightedChannelsColorSpaceWidget.hpp"
#include "SegmentationClusteringPlugin.hpp"

QuantificationCAGWidget::QuantificationCAGWidget(MainView *mainView,
						 SegmentationClusteringPlugin *segmentationClusteringPlugin,
						 QWidget *parent)
  : SegmentationClusteringWidgetInterface(mainView, segmentationClusteringPlugin, parent)
  , m_wccsw(NULL)
  , m_thresholdSB(NULL)
{
  assert(m_segmentationClusteringPlugin);

  setWindowTitle(tr("QuantificationCAG parameters..."));

  setAttribute(Qt::WA_DeleteOnClose);

  buildGUI();

}

void
QuantificationCAGWidget::buildGUI()
{
  m_wccsw = new WeightedChannelsColorSpaceWidget(this);

  QLabel *thresLabel = new QLabel(tr("Threshold: "), this);
  m_thresholdSB = new SliderSpinBoxWidget(this);
  
  m_thresholdSB->setDecimals(4);
  m_thresholdSB->setSingleStep(0.0001);
  m_thresholdSB->setMinimum(0.0);
  m_thresholdSB->setMaximum(1.0);

  QHBoxLayout *hbox = new QHBoxLayout;
  hbox->addWidget(thresLabel);
  hbox->addWidget(m_thresholdSB);

  QVBoxLayout *vLayout = new QVBoxLayout();
  vLayout->addLayout(hbox);
  vLayout->addWidget(m_wccsw);
  setLayout(vLayout);  
}

void 
QuantificationCAGWidget::setParameters(double threshold, int colorSpaceConversion, const std::vector<double> &weightedChannelsCoeffs)
{
  m_thresholdSB->setValue(threshold);
  m_wccsw->setColorSpace(colorSpaceConversion);
  m_wccsw->setWeights(weightedChannelsCoeffs);
}
 
void 
QuantificationCAGWidget::getParameters(double &threshold, int &colorSpaceConversion, std::vector<double> &weightedChannelsCoeffs) const
{
  threshold = m_thresholdSB->value();
  colorSpaceConversion = m_wccsw->getColorSpace();
  m_wccsw->getWeights(weightedChannelsCoeffs);
}
