/*
  Copyright (C) 2005-2011 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef QUANTIFICATIONKMEANSWIDGETSETTINGS_HPP
#define QUANTIFICATIONKMEANSWIDGETSETTINGS_HPP

#include "SegmentationClusteringWidgetSettingsInterface.hpp"
#include "QuantificationKMeansWidget.hpp"
#include <ise/iseQuantificationKMeans.hpp>

class QuantificationKMeansWidgetSettings : public SegmentationClusteringWidgetSettingsInterface
{
  
public:
  
  QuantificationKMeansWidgetSettings() 
    : SegmentationClusteringWidgetSettingsInterface()
    , m_nbClasses(4)
    , m_colorSpaceConversion(ISE_RGB_to_K1K2K3)
    , m_weightedChannelsCoeffs(3, 0.33)
  {
  }

  virtual void configureWidget(SegmentationClusteringWidgetInterface *widget) const
  {
    static_cast<QuantificationKMeansWidget*>(widget)->setParameters(m_nbClasses, m_colorSpaceConversion, m_weightedChannelsCoeffs);
  }
  
  virtual void getWidgetConfig(const SegmentationClusteringWidgetInterface *widget)
  {
    static_cast<const QuantificationKMeansWidget*>(widget)->getParameters(m_nbClasses, m_colorSpaceConversion, m_weightedChannelsCoeffs);
  }

  virtual void configureAlgo(girl::shared_ptr<ise::SegmentationClustering> algo) const
  {
    girl::shared_ptr<ise::QuantificationKMeans> algorithm = girl::static_pointer_cast<ise::QuantificationKMeans>(algo);
    algorithm->setParameters(m_nbClasses, m_colorSpaceConversion, m_weightedChannelsCoeffs);
  }

  virtual girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface> clone() const
  {
    girl::shared_ptr<QuantificationKMeansWidgetSettings> p(new QuantificationKMeansWidgetSettings);
    
    *p = *this;

    return p;
  }

  virtual bool isValid() const
  {
    return (m_nbClasses > 0); //B: ???
  }
  
private:
  
  int m_nbClasses;
  int m_colorSpaceConversion;
  std::vector<double> m_weightedChannelsCoeffs;
};

#endif /* ! QUANTIFICATIONKMEANSWIDGETSETTINGS_HPP */
