/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "QuantificationNbColorsWidget.hpp"

#include <limits>

#include <QBoxLayout>
#include <QComboBox>
#include <QGridLayout>
#include <QLabel>
#include <QSpinBox>

#include "ColorSpaceChoiceWidget.hpp"
#include "SegmentationClusteringPlugin.hpp"


QuantificationNbColorsWidget::QuantificationNbColorsWidget(MainView *mainView,
							   SegmentationClusteringPlugin *segmentationClusteringPlugin,
							   QWidget *parent)
  : SegmentationClusteringWidgetInterface(mainView, segmentationClusteringPlugin, parent)
  , m_cscw(NULL)
  , m_nbColorsSB(NULL)
  , m_initCombo(NULL)
  , m_clusterCombo(NULL)
  , m_axisCombo(NULL)
  , m_cutCombo(NULL)
{
  assert(m_segmentationClusteringPlugin);

  setWindowTitle(tr("QuantificationNbColors parameters..."));

  setAttribute(Qt::WA_DeleteOnClose);

  buildGUI();

}


void
QuantificationNbColorsWidget::buildColorSpaceChoice(QLabel *&l, QWidget *&w)
{
  l = new QLabel(tr("Color space: "), this);
  
  m_cscw = new ColorSpaceChoiceWidget(this);
  w = m_cscw;
}

void
QuantificationNbColorsWidget::buildNbColorsChoice(QLabel *&l, QWidget *&w)
{
  l = new QLabel(tr("Number of colors: "), this);
  
  m_nbColorsSB = new QSpinBox(this);
  m_nbColorsSB->setMinimum(2);
  m_nbColorsSB->setMaximum(std::numeric_limits<int>::max());

  w = m_nbColorsSB;
}


void
QuantificationNbColorsWidget::addInitChoice(QComboBox *comboBox,
					    const QString &name,
					    ise::QUANTIF_NB_COLOR_INIT_TYPE t)
{
  comboBox->addItem(name);
  m_initChoice.push_back(t); //m_initChoices[i-th item] = t
  m_initChoiceMap[t] = m_initChoice.size()-1;
}

void
QuantificationNbColorsWidget::addAxisChoice(QComboBox *comboBox,
					    const QString &name,
					    ise::QUANTIF_NB_COLOR_AXIS_CHOICE_TYPE t)
{
  comboBox->addItem(name);
  m_axisChoice.push_back(t); //m_axisChoices[i-th item] = t
  m_axisChoiceMap[t] = m_axisChoice.size()-1;
}

void
QuantificationNbColorsWidget::addClusterChoice(QComboBox *comboBox,
					       const QString &name,
					       ise::QUANTIF_NB_COLOR_CLUSTER_VALUE_TYPE t)
{
  comboBox->addItem(name);
  m_clusterChoice.push_back(t); //m_clusterChoices[i-th item] = t
  m_clusterChoiceMap[t] = m_clusterChoice.size()-1;
}

void
QuantificationNbColorsWidget::addCutChoice(QComboBox *comboBox,
					       const QString &name,
					       ise::QUANTIF_NB_COLOR_CUT_CHOICE_TYPE t)
{
  comboBox->addItem(name);
  m_cutChoice.push_back(t); //m_cutChoices[i-th item] = t
  m_cutChoiceMap[t] = m_cutChoice.size()-1;
}




void
QuantificationNbColorsWidget::buildInitChoice(QLabel *&l, QWidget*&w)
{
  //init choice
  l = new QLabel(tr("Init: "), this);
  m_initCombo = new QComboBox(this);
  addInitChoice(m_initCombo, tr("default"), ise::INIT_DEFAULT);
  addInitChoice(m_initCombo, tr("dangerous"), ise::INIT_DANGEROUS);
  w = m_initCombo;
}

void
QuantificationNbColorsWidget::buildClusterChoice(QLabel *&l, QWidget *&w)
{
  //cluster choice
  l = new QLabel(tr("Cluster: "), this);
  m_clusterCombo = new QComboBox(this);
  addClusterChoice(m_clusterCombo, tr("weighted MSE"), ise::CLUSTER_VALUE_WEIGHTED_MSE);
  addClusterChoice(m_clusterCombo, tr("non weighted MSE"), ise::CLUSTER_VALUE_NON_WEIGHTED_MSE);
  addClusterChoice(m_clusterCombo, tr("cardinal"), ise::CLUSTER_VALUE_CARDINAL);
  w = m_clusterCombo;
}

void
QuantificationNbColorsWidget::buildAxisChoice(QLabel *&l, QWidget *&w)
{
  //axis choice
  l = new QLabel(tr("Axis: "), this);
  m_axisCombo = new QComboBox(this);
  addAxisChoice(m_axisCombo, tr("variance"), ise::AXIS_CHOICE_VARIANCE);
  addAxisChoice(m_axisCombo, tr("length"), ise::AXIS_CHOICE_LENGTH);
  addAxisChoice(m_axisCombo, tr("variance+length"), ise::AXIS_CHOICE_VARIANCE_LENGTH);
  addAxisChoice(m_axisCombo, tr("variance+median"), ise::AXIS_CHOICE_VARIANCE_MEDIAN);
  addAxisChoice(m_axisCombo, tr("MSE"), ise::AXIS_CHOICE_MSE);
  addAxisChoice(m_axisCombo, tr("split variance"), ise::AXIS_CHOICE_SPLIT_VARIANCE);
  w = m_axisCombo;
}

void
QuantificationNbColorsWidget::buildCutChoice(QLabel *&l, QWidget*&w)
{
  //cut choice
  l = new QLabel(tr("Cut: "), this);
  m_cutCombo = new QComboBox(this);
  addCutChoice(m_cutCombo, tr("cut"), ise::CUT_CHOICE_CUT);
  addCutChoice(m_cutCombo, tr("cut+project"), ise::CUT_CHOICE_CUT_PROJECT);
  addCutChoice(m_cutCombo, tr("cut+median"), ise::CUT_CHOICE_CUT_MEDIAN);
  addCutChoice(m_cutCombo, tr("cut+median dangerous"), ise::CUT_CHOICE_CUT_MEDIAN_DANGEROUS);
  w = m_cutCombo;
}

  

void
QuantificationNbColorsWidget::buildGUI()
{
  QLabel *nbColorL;
  QWidget *nbColorW;
  buildNbColorsChoice(nbColorL, nbColorW);

  QLabel *colorSpaceL;
  QWidget *colorSpaceW;
  buildColorSpaceChoice(colorSpaceL, colorSpaceW);

  
  QLabel *initL;
  QWidget *initW;
  buildInitChoice(initL, initW);

  QLabel *clusterL;
  QWidget *clusterW;
  buildClusterChoice(clusterL, clusterW);

  QLabel *axisL;
  QWidget *axisW;  
  buildAxisChoice(axisL, axisW);

  QLabel *cutL;
  QWidget *cutW;  
  buildCutChoice(cutL, cutW);
  
  QGridLayout *gLayout = new QGridLayout;

  gLayout->addWidget(nbColorL, 0, 0);
  gLayout->addWidget(nbColorW, 0, 1);
  gLayout->addWidget(colorSpaceL, 1, 0);
  gLayout->addWidget(colorSpaceW, 1, 1);
  gLayout->addWidget(initL, 2, 0);
  gLayout->addWidget(initW, 2, 1);
  gLayout->addWidget(clusterL, 3, 0);
  gLayout->addWidget(clusterW, 3, 1);
  gLayout->addWidget(axisL, 4, 0);
  gLayout->addWidget(axisW, 4, 1);
  gLayout->addWidget(cutL, 5, 0);
  gLayout->addWidget(cutW, 5, 1);

  QVBoxLayout *vLayout = new QVBoxLayout();
  vLayout->addLayout(gLayout);
  
  setLayout(vLayout);  
}

void 
QuantificationNbColorsWidget::setParameters(ise::QUANTIF_NB_COLOR_AXIS_CHOICE_TYPE act,
					    ise::QUANTIF_NB_COLOR_CLUSTER_VALUE_TYPE cvt,
					    ise::QUANTIF_NB_COLOR_INIT_TYPE it,
					    ise::QUANTIF_NB_COLOR_CUT_CHOICE_TYPE cct,
					    int colorSpace,
					    int nbColors)
{
  m_initCombo->setCurrentIndex(m_initChoiceMap[it]);
  m_clusterCombo->setCurrentIndex(m_clusterChoiceMap[cvt]);
  m_axisCombo->setCurrentIndex(m_axisChoiceMap[act]);
  m_cutCombo->setCurrentIndex(m_cutChoiceMap[cct]);
  m_cscw->setColorSpace(colorSpace);
  m_nbColorsSB->setValue(nbColors);
}

void 
QuantificationNbColorsWidget::getParameters(ise::QUANTIF_NB_COLOR_AXIS_CHOICE_TYPE &act,
					    ise::QUANTIF_NB_COLOR_CLUSTER_VALUE_TYPE &cvt,
					    ise::QUANTIF_NB_COLOR_INIT_TYPE &it,
					    ise::QUANTIF_NB_COLOR_CUT_CHOICE_TYPE &cct,
					    int &colorSpace,
					    int &nbColors) const
{
  it = m_initChoice[m_initCombo->currentIndex()];
  cvt = m_clusterChoice[m_clusterCombo->currentIndex()];
  act = m_axisChoice[m_axisCombo->currentIndex()];
  cct = m_cutChoice[m_cutCombo->currentIndex()];
  colorSpace = m_cscw->getColorSpace();
  nbColors = m_nbColorsSB->value();
}

