/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef QUANTIFICATIONNBCOLORSWIDGET_HPP
#define QUANTIFICATIONNBCOLORSWIDGET_HPP


#include <vector>
#include <map>

#include <ise/iseQuantificationNbColors.hpp>

#include "SegmentationClusteringWidgetInterface.hpp"

class QComboBox;
class QLabel;
class QSpinBox;

class ColorSpaceChoiceWidget;
class SegmentationClusteringPlugin;


class GIRL_API QuantificationNbColorsWidget : public SegmentationClusteringWidgetInterface
{
   Q_OBJECT

public:
  QuantificationNbColorsWidget(MainView *mainView,
			       SegmentationClusteringPlugin *segmentationClustering,
			       QWidget *parent = NULL);
  
  void setParameters(ise::QUANTIF_NB_COLOR_AXIS_CHOICE_TYPE act,
		     ise::QUANTIF_NB_COLOR_CLUSTER_VALUE_TYPE cvt,
		     ise::QUANTIF_NB_COLOR_INIT_TYPE it,
		     ise::QUANTIF_NB_COLOR_CUT_CHOICE_TYPE cct,
		     int colorSpace,
		     int nbColors);
  void getParameters(ise::QUANTIF_NB_COLOR_AXIS_CHOICE_TYPE &act,
		     ise::QUANTIF_NB_COLOR_CLUSTER_VALUE_TYPE &cvt,
		     ise::QUANTIF_NB_COLOR_INIT_TYPE &it,
		     ise::QUANTIF_NB_COLOR_CUT_CHOICE_TYPE &cct,
		     int &colorSpace,
		     int &nbColors) const;
  
protected:

  void buildGUI();

  void buildColorSpaceChoice(QLabel *&l, QWidget *&w);
  void buildNbColorsChoice(QLabel *&l, QWidget *&w);
  void buildInitChoice(QLabel *&l, QWidget *&w);
  void buildClusterChoice(QLabel *&l, QWidget *&w);
  void buildAxisChoice(QLabel *&l, QWidget *&w);
  void buildCutChoice(QLabel *&l, QWidget *&w);


  void addColorSpace(QComboBox *csComboBox,
		     const QString &name,
		     int colorspace);
  void addInitChoice(QComboBox *comboBox,
		     const QString &name,
		     ise::QUANTIF_NB_COLOR_INIT_TYPE t);
  void addAxisChoice(QComboBox *comboBox,
		     const QString &name,
		     ise::QUANTIF_NB_COLOR_AXIS_CHOICE_TYPE t);
  void addClusterChoice(QComboBox *comboBox,
			const QString &name,
			ise::QUANTIF_NB_COLOR_CLUSTER_VALUE_TYPE t);
  void addCutChoice(QComboBox *comboBox,
		    const QString &name,
		    ise::QUANTIF_NB_COLOR_CUT_CHOICE_TYPE t);
  

protected:

  ColorSpaceChoiceWidget *m_cscw;

  QSpinBox *m_nbColorsSB;

  QComboBox *m_initCombo, *m_clusterCombo, *m_axisCombo, *m_cutCombo;
  
  std::vector<ise::QUANTIF_NB_COLOR_INIT_TYPE> m_initChoice;
  std::map<ise::QUANTIF_NB_COLOR_INIT_TYPE, int> m_initChoiceMap;

  std::vector<ise::QUANTIF_NB_COLOR_AXIS_CHOICE_TYPE> m_axisChoice;
  std::map<ise::QUANTIF_NB_COLOR_AXIS_CHOICE_TYPE, int> m_axisChoiceMap;

  std::vector<ise::QUANTIF_NB_COLOR_CLUSTER_VALUE_TYPE> m_clusterChoice;
  std::map<ise::QUANTIF_NB_COLOR_CLUSTER_VALUE_TYPE, int> m_clusterChoiceMap;

  std::vector<ise::QUANTIF_NB_COLOR_CUT_CHOICE_TYPE> m_cutChoice;
  std::map<ise::QUANTIF_NB_COLOR_CUT_CHOICE_TYPE, int> m_cutChoiceMap;



};

#endif /* ! QUANTIFICATIONNBCOLORSWIDGET_HPP */ 
