/*
  Copyright (C) 2005-2011 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef QUANTIFICATIONNBCOLORSWIDGETSETTINGS_HPP
#define QUANTIFICATIONNBCOLORSWIDGETSETTINGS_HPP

#include "SegmentationClusteringWidgetSettingsInterface.hpp"
#include "QuantificationNbColorsWidget.hpp"
#include <ise/iseQuantificationNbColors.hpp>

class QuantificationNbColorsWidgetSettings : public SegmentationClusteringWidgetSettingsInterface
{
  
public:
  
  QuantificationNbColorsWidgetSettings() 
    : SegmentationClusteringWidgetSettingsInterface()
    , m_colorSpace(ISE_RGB_to_RGB)
    , m_nbColors(4)
    , m_axis(static_cast<ise::QUANTIF_NB_COLOR_AXIS_CHOICE_TYPE>(0))
    , m_clusterValueChoice(static_cast<ise::QUANTIF_NB_COLOR_CLUSTER_VALUE_TYPE>(0))
    , m_init(static_cast<ise::QUANTIF_NB_COLOR_INIT_TYPE>(0))
    , m_cutChoice(static_cast<ise::QUANTIF_NB_COLOR_CUT_CHOICE_TYPE>(0))

  {
  }

  virtual void configureWidget(SegmentationClusteringWidgetInterface *widget) const
  {
    static_cast<QuantificationNbColorsWidget*>(widget)->setParameters(m_axis, m_clusterValueChoice, m_init, m_cutChoice, m_colorSpace, m_nbColors);
  }
  
  virtual void getWidgetConfig(const SegmentationClusteringWidgetInterface *widget)
  {
    static_cast<const QuantificationNbColorsWidget*>(widget)->getParameters(m_axis, m_clusterValueChoice, m_init, m_cutChoice, m_colorSpace, m_nbColors);
  }

  virtual void configureAlgo(girl::shared_ptr<ise::SegmentationClustering> algo) const
  {
    girl::shared_ptr<ise::QuantificationNbColors> algorithm = girl::static_pointer_cast<ise::QuantificationNbColors>(algo);
    algorithm->setParameters(m_axis, m_clusterValueChoice, m_init, m_cutChoice, m_colorSpace, m_nbColors);
  }

  virtual girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface> clone() const
  {
    girl::shared_ptr<QuantificationNbColorsWidgetSettings> p(new QuantificationNbColorsWidgetSettings);

    *p = *this;

    return p;
  }
  
  virtual bool isValid() const
  {
    return (m_nbColors > 0); //B: ???
  }


private:
  
  int m_colorSpace;
  int m_nbColors;
  ise::QUANTIF_NB_COLOR_AXIS_CHOICE_TYPE m_axis;
  ise::QUANTIF_NB_COLOR_CLUSTER_VALUE_TYPE m_clusterValueChoice;
  ise::QUANTIF_NB_COLOR_INIT_TYPE m_init;
  ise::QUANTIF_NB_COLOR_CUT_CHOICE_TYPE m_cutChoice;
};

#endif /* ! QUANTIFICATIONNBCOLORSWIDGETSETTINGS_HPP */
