/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include <ise/iseQuantificationThresholds.hpp>
#include "QuantificationThresholdsWidget.hpp"
#include "QuantificationThresholdsWidgetAux.hpp"

#include <QBoxLayout>

#include "MainView.hpp"
#include "SegModel.hpp"

#include "SegmentationClusteringPlugin.hpp"


QuantificationThresholdsWidget::QuantificationThresholdsWidget(MainView *mainView, 
							       SegmentationClusteringPlugin *segmentationClusteringPlugin,
							       QWidget *parent)
  : SegmentationClusteringWidgetInterface(mainView, segmentationClusteringPlugin, parent)
  , m_qtwa(NULL)
{
  setWindowTitle(tr("QuantificationThresholds parameters..."));

  buildGUI();
}


QuantificationThresholdsWidget::~QuantificationThresholdsWidget()
{
  deactivate();
}

void
QuantificationThresholdsWidget::buildGUI()
{
  setWindowTitle(tr("Histogram"));

  QVBoxLayout *vLayout = new QVBoxLayout;

  const bool allRegions = true;
  const bool oneChannel = false;
  m_qtwa = new QuantificationThresholdsWidgetAux(m_mainView, allRegions, oneChannel, this/*parentWidget()*/);
  vLayout->addWidget(m_qtwa);

  setLayout(vLayout);
}

void
QuantificationThresholdsWidget::setParameters(int colorSpaceConversion, 
					      int channel,
					      const std::vector<int> &thresholds)
					      
{
  m_qtwa->setGUI(colorSpaceConversion, channel, thresholds);
}

void
QuantificationThresholdsWidget::getParameters(int &colorSpaceConversion, 
					      int &channel,
					      std::vector<int> &thresholds) const
{
  m_qtwa->getGUI(colorSpaceConversion, channel, thresholds);
}

void
QuantificationThresholdsWidget::setParameters(int colorSpaceConversion, 
					      const std::vector< std::vector<int> > &thresholds)
					      
{
  m_qtwa->setGUI(colorSpaceConversion, thresholds);
}

void
QuantificationThresholdsWidget::getParameters(int &colorSpaceConversion, 
					      std::vector< std::vector<int> > &thresholds) const
{
  m_qtwa->getGUI(colorSpaceConversion, thresholds);
}


void
QuantificationThresholdsWidget::activate() 
{ 
  m_qtwa->activate(); 
}

void
QuantificationThresholdsWidget::deactivate() 
{ 
  m_qtwa->deactivate(); 
}
