/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef QUANTIFICATIONTHRESHOLDSWIDGETAUX_HPP
#define QUANTIFICATIONTHRESHOLDSWIDGETAUX_HPP

#include <girl/girlSharedPtr.hpp>

//#include <ise/iseHashHistogram.hpp>
#include <ise/iseHistogram1D.hpp>
#include <ise/iseQuantificationThresholder.hpp>

#include <QHash>
#include <QImage>
#include <QList>
#include <QMap>
#include <QVector>
#include <QWidget>
#include <iostream>
#include "QuantificationThresholderPlugin.hpp"
#include "SegmentationClusteringWidgetInterface.hpp"
#include "ColorPool.hpp"

#include "ThresholdedColorIndex.hpp"


class QCheckBox;
class QComboBox;
class QGroupBox;
class QPushButton;
class QRadioButton;
class QVBoxLayout;

class Histogram1DThresholdsExtendedWidget;
class MainView;
class SegModel;

class GLSegmentLayer;

class RegionBorderBuilderThread;
class FalseColorImageBuilderThread;

class ColorSpacesChannelsChoiceWidget;


class GIRL_API QuantificationThresholdsWidgetAux : public QWidget
{
  Q_OBJECT

public:

  /**
   * @brief Destructor.
   */
  ~QuantificationThresholdsWidgetAux();

  /**
   * @brief Constructor.
   *
   * @param[in] allRegions sets if the part indicating that quantification 
   *  must be applied to whole image or only selected regions is displayed.
   * @param[in] oneChannel set if the widget is used for only one channel at a time, 
   *  or all channels simultaneously.
   */
  QuantificationThresholdsWidgetAux(MainView *mainView, 
				    bool allRegions,
				    bool oneChannel,
				    QWidget *parent);


  void setGUI(int colorSpace, int channel, const std::vector<int> &thresholds);
  void getGUI(int &colorSpace, int &channel, std::vector<int> &thresholds);
  
  void setGUI(int colorSpace, const std::vector< std::vector<int> > &thresholds);
  void getGUI(int &colorSpace, std::vector< std::vector<int> > &thresholds);


  void activate();
  void deactivate();

protected slots:

  void changeChannel();
  void changeColorSpace();
  void changeThresholdMode();
  void changeViewMode();
  void changeChannelMode();

  void addThreshold(size_t);
  void removeThreshold(size_t);
  void changeThreshold(size_t);
  void changeThresholdColor(size_t, QRgb, QRgb);
  

  void changeImage(SegModel *);
  void changeToger(SegModel *);
  void changeSelection(SegModel *);
  
  void updatePaths(const std::deque<girl::point> &paths);
  void updateImage(const QImage &image);

  //void changeColor(QRgb newColor);

  void viewChannelChanged(int state);

protected:
  
  void buildGUI();

  void buildThresholdModeChoices(QGroupBox *mGroupBox);
  void buildViewModeChoices(QGroupBox *mGroupBox);
  void buildColorSpaceAndChannelsChoices(QGroupBox *cGroupBox);
  void buildOneOrAllChannelsChoices(QGroupBox *ccGroupBox);
  void initThresholders();

  void initColors();
  QRgb getFreeColor();
  void addFreeColor(QRgb);

  
  void setThresholds(const std::vector<int> &thresholds);
  void getThresholds(std::vector<int> &thresholds) const; 

  void setThresholds(const std::vector<double> &thresholds);
  void getThresholds(std::vector<double> &thresholds) const;


  void transformThresholdsToNewColorSpace(int previousCS, int newCS, std::vector<double> &thresholds);
  void transformThresholdsToNewChannel(int previousChan, int newChan, std::vector<double> &thresholds) const;
  void transformThresholdsToNewColorSpaceChannel(int previousCS, int newCS, int previousChan, int newChan, std::vector<double> &thresholds) const;

  void transformThresholds(int min1, int max1, int min2, int max2, std::vector<double> &thresholds) const;

  
  void setChannel(int channel);
  void setColorSpace(int colorSpace);

  int getChannel() const;
  int getColorSpace() const;

  typedef enum {THRESHOLD_IMAGE, THRESHOLD_REGIONS} ThresholdMode;
  ThresholdMode getThresholdMode() const;

  typedef enum {VIEW_FALSECOLORS, VIEW_BORDERS} ViewMode;
  ViewMode getViewMode() const;

  typedef enum {CHANNEL_ONE, CHANNEL_ALL} ChannelMode;
  ChannelMode getChannelMode() const;

  void initHash();
  void initHashImage();
  void initHashRegions();
  void updateHistoView();
  void updateHashTransformedColors();
  void updateImage();
  void updateThresholdedColors();

  void changeThresholdedColor(QRgb oldColor, QRgb newColor);
  void startUpdate();

  void addThresholder(const QString &name, girl::shared_ptr<ise::QuantificationThresholder> thresholder);
  void thresholds(girl::shared_ptr<ise::QuantificationThresholder> thr);

  /**
   * @brief Set opacity for drawing selected face.
   *
   * @a alpha 0.0f means all transparent, 1.0f means completely opaque.
   */
  void setOpacity(float alpha);

  QImage getImageToDisplay();

  void updateDisplayedImageIfNecessary(const QImage &img);

protected:
  MainView *m_mainView;

  ColorSpacesChannelsChoiceWidget *m_csccw;
  QVector<int> m_colorSpaces;

  QCheckBox *m_viewChannelCB;

  QRadioButton *m_thresholdModeImageButton;
  QRadioButton *m_thresholdModeRegionsButton;

  QRadioButton *m_viewModeFalseColorsButton;
  QRadioButton *m_viewModeBordersButton;

  QRadioButton *m_oneChannelButton;
  QRadioButton *m_allChannelsButton;



  int m_channel; //current selected channel
  int m_colorSpace; //current selected color space

  Histogram1DThresholdsExtendedWidget *m_histoView;
  

  MainView *m_view;
  

  float m_opacity;

  friend class RegionColorGetter;
  friend class RegionBorderSetter;
  friend class RegionBorderBuilderThread;

  typedef ThresholdedColorIndex::HashType HashType;

  HashType m_hash;

  ThresholdedColorIndex *m_thresholdedColorIndex;
  FalseColorIndexList m_falseColorIndexList;

  GLSegmentLayer *m_segmentLayer;

  girl::shared_ptr<RegionBorderBuilderThread> m_rbbThread1;
  girl::shared_ptr<FalseColorImageBuilderThread> m_fcibThread1;

  std::vector< std::vector<double> > m_thresholds;
  ColorPool m_colorPool;

  bool m_allRegions;
  bool m_oneChannel;

};

#endif /* ! QUANTIFICATIONTHRESHOLDSWIDGETAUX_HPP */
