/*
  Copyright (C) 2005-2011 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef QUANTIFICATIONTHRESHOLDSWIDGETSETTINGS_HPP
#define QUANTIFICATIONTHRESHOLDSWIDGETSETTINGS_HPP

#include "SegmentationClusteringWidgetSettingsInterface.hpp"
#include "QuantificationThresholdsWidget.hpp"
#include <ise/iseQuantificationThresholds.hpp>

class QuantificationThresholdsWidgetSettings : public SegmentationClusteringWidgetSettingsInterface
{
  
public:
  
  QuantificationThresholdsWidgetSettings() 
    : SegmentationClusteringWidgetSettingsInterface()
    , m_colorSpaceConversion(ISE_RGB_to_K1K2K3)
    , m_thresholds(1)
  {
  }

  virtual void configureWidget(SegmentationClusteringWidgetInterface *widget) const
  {
    static_cast<QuantificationThresholdsWidget*>(widget)->setParameters(m_colorSpaceConversion, m_thresholds);
  }
  
  virtual void getWidgetConfig(const SegmentationClusteringWidgetInterface *widget)
  {
    static_cast<const QuantificationThresholdsWidget*>(widget)->getParameters(m_colorSpaceConversion, m_thresholds);
  }

  virtual void configureAlgo(girl::shared_ptr<ise::SegmentationClustering> algo) const
  {
    girl::shared_ptr<ise::QuantificationThresholds> algorithm = girl::static_pointer_cast<ise::QuantificationThresholds>(algo);
    algorithm->setParams(m_colorSpaceConversion, m_thresholds);
  }
  
  virtual girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface> clone() const
  {
    girl::shared_ptr<QuantificationThresholdsWidgetSettings> p(new QuantificationThresholdsWidgetSettings);

    *p = *this;

    return p;
  }

  virtual bool isValid() const
  {
    //- valid if there is at least one thresholds (on one of the channels)
    if (m_thresholds.empty())
      return false;
    bool valid = false;
    for (size_t i=0; i<m_thresholds.size(); ++i)
      if (! m_thresholds[i].empty()) {
	valid = true;
	break;
      }
    return valid;
  }


private:
  
  int m_colorSpaceConversion;
  std::vector< std::vector<int> > m_thresholds;
};

#endif /* ! QUANTIFICATIONTHRESHOLDSWIDGETSETTINGS_HPP */
