/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "QuantificationUniformWidget.hpp"

#include <ise/iseQuantificationUniform.hpp>

#include <QBoxLayout>
#include <QComboBox>
#include <QSpinBox>
#include <QLabel>

#include "WeightedChannelsColorSpaceWidget.hpp"

#include "SegmentationClusteringPlugin.hpp"

QuantificationUniformWidget::QuantificationUniformWidget(MainView *mainView,
							 SegmentationClusteringPlugin *segmentationClusteringPlugin,
							 QWidget *parent)
  : SegmentationClusteringWidgetInterface(mainView, segmentationClusteringPlugin, parent)
  , m_wccsw(NULL)
{
  assert(m_segmentationClusteringPlugin);

  setWindowTitle(tr("QuantificationUniform parameters..."));

  setAttribute(Qt::WA_DeleteOnClose);

  buildGUI();

}

void
QuantificationUniformWidget::buildGUI()
{
  QLabel *offsetsLabel = new QLabel(tr("Offsets: "), this);
  m_offset1SB = new QSpinBox(this);
  m_offset2SB = new QSpinBox(this);
  m_offset3SB = new QSpinBox(this);
  
  m_offset1SB->setMinimum(0);
  m_offset1SB->setMaximum(8);
  m_offset2SB->setMinimum(0);
  m_offset2SB->setMaximum(8);
  m_offset3SB->setMinimum(0);
  m_offset3SB->setMaximum(8);

  m_wccsw = new WeightedChannelsColorSpaceWidget(this);

  QHBoxLayout *hbox = new QHBoxLayout;
  hbox->addWidget(offsetsLabel);
  hbox->addWidget(m_offset1SB);
  hbox->addWidget(m_offset2SB);
  hbox->addWidget(m_offset3SB);

  QVBoxLayout *vLayout = new QVBoxLayout();
  vLayout->addLayout(hbox);
  vLayout->addWidget(m_wccsw);

  setLayout(vLayout);  
}

void 
QuantificationUniformWidget::setParameters(int colorSpace, std::vector<double> weightedChannelsCoeffs, unsigned char offset1, unsigned char offset2, unsigned char offset3)
{
  m_wccsw->setColorSpace(colorSpace);
  m_wccsw->setWeights(weightedChannelsCoeffs);
  m_offset1SB->setValue(static_cast<int>(offset1));
  m_offset2SB->setValue(static_cast<int>(offset2));
  m_offset3SB->setValue(static_cast<int>(offset3));
}

void 
QuantificationUniformWidget::getParameters(int &colorSpace, 
					   std::vector<double> &weightedChannelsCoeffs, 
					   unsigned char &offset1, unsigned char &offset2, 
					   unsigned char &offset3) const
{
  colorSpace = m_wccsw->getColorSpace();
  //TODO: change min/max of spinboxes accordingly !!! nextPowerOfTwo ?

  m_wccsw->getNormalizedWeights(weightedChannelsCoeffs);
  offset1 = m_offset1SB->value();
  offset2 = m_offset2SB->value();
  offset3 = m_offset3SB->value();
}

