/*
  Copyright (C) 2005-2011 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef QUANTIFICATIONKMEANSWIDGETSETTINGS_HPP
#define QUANTIFICATIONKMEANSWIDGETSETTINGS_HPP

#include "SegmentationClusteringWidgetSettingsInterface.hpp"
#include "QuantificationUniformWidget.hpp"
#include <ise/iseQuantificationUniform.hpp>

class QuantificationUniformWidgetSettings : public SegmentationClusteringWidgetSettingsInterface
{
  
public:
  
  QuantificationUniformWidgetSettings() 
    : SegmentationClusteringWidgetSettingsInterface()
    , m_colorSpace(0)
    , m_weightedChannelsCoeffs(3, 0.33)
    , m_offset1(4)
    , m_offset2(4)
    , m_offset3(4)      
  {
  }

  virtual void configureWidget(SegmentationClusteringWidgetInterface *widget) const
  {
    static_cast<QuantificationUniformWidget*>(widget)->setParameters(m_colorSpace, m_weightedChannelsCoeffs, m_offset1, m_offset2, m_offset3);
  }
  
  virtual void getWidgetConfig(const SegmentationClusteringWidgetInterface *widget)
  {
    static_cast<const QuantificationUniformWidget*>(widget)->getParameters(m_colorSpace, m_weightedChannelsCoeffs, m_offset1, m_offset2, m_offset3);
  }

  virtual void configureAlgo(girl::shared_ptr<ise::SegmentationClustering> algo) const
  {
    girl::shared_ptr<ise::QuantificationUniform> algorithm = girl::static_pointer_cast<ise::QuantificationUniform>(algo);
    algorithm->setParameters(m_colorSpace, m_weightedChannelsCoeffs);
    algorithm->setOffsets(m_offset1, m_offset2, m_offset3);
  }
  
  virtual girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface> clone() const
  {
    girl::shared_ptr<QuantificationUniformWidgetSettings> p(new QuantificationUniformWidgetSettings);

    *p = *this;

    return p;
  }

  virtual bool isValid() const
  {
    return (m_weightedChannelsCoeffs.size() == 3); //B: ???
  }

private:
  
  int m_colorSpace;
  std::vector<double> m_weightedChannelsCoeffs;
  unsigned char m_offset1;
  unsigned char m_offset2;
  unsigned char m_offset3;
};

#endif /* ! QUANTIFICATIONKMEANSWIDGETSETTINGS_HPP */
