/*
  Copyright (C) 2010 Cédric Aguerre  aguerre@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#include "RectangleSelector.hpp"

#include "GLRectangleLayer.hpp"
#include "SegModel.hpp"
#include "SegView.hpp"
#include <algorithm>
#include <girlTools/girlToolsRegionInfos.hpp>

RectangleSelector::RectangleSelector(SegView *v)
  : ShapeSelector(v),
    m_p1(),
    m_p2()
{
  
}

RectangleSelector::~RectangleSelector()
{
  
}

void
RectangleSelector::addPoint(const QPointF &ptf)
{
  assert(m_layer == NULL);
  m_layer = NULL;

  m_p1 = m_p2 = ptf;//QPoint(static_cast<int>(ptf.x()), static_cast<int>(ptf.y()));
}

bool
RectangleSelector::visit(const QPointF &/*ptf*/)
{
  return false;
}

void
RectangleSelector::update(const QPointF &ptf)
{
  if (m_view == NULL)
    return;

  m_p2 = ptf;//QPoint(static_cast<int>(ptf.x()), static_cast<int>(ptf.y()));

  updateDraw();
}

void
RectangleSelector::setPoints(const QPointF &p1, const QPointF &p2)
{
  m_p1 = p1;
  m_p2 = p2;
  
  updateDraw();
}

void
RectangleSelector::getPoints(QPointF &p1, QPointF &p2) const
{
  p1 = m_p1;
  p2 = m_p2;
}

void
RectangleSelector::updateDraw()
{
  assert(m_view);

  GLRectangleLayer *rectangleLayer = static_cast<GLRectangleLayer *>(m_layer);
  if (m_layer == NULL && m_p2.x() != m_p1.x() && m_p2.y() != m_p1.y()) {
    rectangleLayer = new GLRectangleLayer(m_view);
    m_layer = rectangleLayer;
    m_layer->setColor(m_color);
    m_layer->setOpacity(m_opacity);
    m_view->addLayer(m_layer);
  }

  if (m_layer != NULL) {
    rectangleLayer->setRectangle(m_p1, m_p2);
    m_view->update();
  }
}

bool
RectangleSelector::finalize(SelectorModifier modifier)
{
  return updateSelectionAndDeleteLayer(modifier);
}

bool
RectangleSelector::abort() {
  return false;
}

bool
RectangleSelector::terminate(SelectorModifier /*modifier*/, bool /*closedRegion*/) {
  return false;
}

/* CODE BASED ON getSegments() method in qise::PaintingRectangle.cpp */
void
RectangleSelector::getSegments(std::vector<std::vector<girl::point> > &segments, girl::tools::AABBox &shapeBBox)
{
  if (m_p1 != m_p2) {

    segments.resize(1);
    segments[0].clear();

    segments[0].reserve(4+1);
    /*
      girl::point p1 = girl::point(m_areaP1.x(), m_areaP1.y());
      girl::point p3 = girl::point(m_areaP2.x() - 1, m_areaP2.y() - 1);
      girl::point p4 = girl::point(p1.x(), p3.y());
      girl::point p2 = girl::point(p3.x(), p1.y());

      p1 = getTransformedPoint(p1);
      p2 = getTransformedPoint(p2);
      p3 = getTransformedPoint(p3);
      p4 = getTransformedPoint(p4);

      //transform to inter-pixels !
      p1 += girl::point(-1, -1);
      p2 += girl::point(-1, -1);
      p3 += girl::point(-1, -1);
      p4 += girl::point(-1, -1);
    */

    QPoint pt1(static_cast<int>(m_p1.x()), static_cast<int>(m_p1.y()));
    QPoint pt2(static_cast<int>(m_p2.x()), static_cast<int>(m_p2.y()));

    QPoint m_areaP1 = QPoint(std::min(pt1.x(), pt2.x()), std::min(pt1.y(), pt2.y()));
    QPoint m_areaP2 = QPoint(std::max(pt1.x(), pt2.x()), std::max(pt1.y(), pt2.y()));


    girl::point p1 = girl::point(static_cast<int>(m_areaP1.x()), static_cast<int>(m_areaP1.y()));
    girl::point p3 = girl::point(static_cast<int>(m_areaP2.x())-1, static_cast<int>(m_areaP2.y())-1);
    girl::point p4 = girl::point(p1.x(), p3.y());
    girl::point p2 = girl::point(p3.x(), p1.y());


    segments[0].push_back(p1);
    segments[0].push_back(p2);
    segments[0].push_back(p3);
    segments[0].push_back(p4);
    segments[0].push_back(p1);


    const int xMin = std::min(std::min(p1.x(), p2.x()), std::min(p3.x(), p4.x()));
    const int yMin = std::min(std::min(p1.y(), p2.y()), std::min(p3.y(), p4.y()));
    const int xMax = std::max(std::max(p1.x(), p2.x()), std::max(p3.x(), p4.x()));
    const int yMax = std::max(std::max(p1.y(), p2.y()), std::max(p3.y(), p4.y()));

    shapeBBox = girl::tools::AABBox(xMin, yMin, xMax, yMax);
  }
  else {
    //m_p1 == m_p2

    segments.clear();
    shapeBBox = girl::tools::AABBox(m_p1.x(), m_p1.y(), m_p1.x(), m_p1.y());
  }
}
