/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux boris.mansencal@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "RegionBinaryFunctionComboWidget.hpp"

#include <QGridLayout>
#include <QString>

#include "PluginIndexChoiceWidget.hpp"
#include "PluginRegistry.hpp"
#include "RegionBinaryFunctionPlugin.hpp"
#include "RegionBinaryFunctionWidget.hpp"


static const int PLUGIN_WIDGET_ROW = 1;

/*
 Warning: this structure is duplicated in RegionUnaryFunctionComboWidget.

*/

RegionBinaryFunctionComboWidget::RegionBinaryFunctionComboWidget(MainView *mainView/*ise::SegmentationContext *segContext*/, 
							       QWidget *parent)
  : QWidget(parent),
    //m_segContext(segContext)
    m_mainView(mainView)
{
  buildGUI();
}

void
RegionBinaryFunctionComboWidget::buildGUI()
{
  QGridLayout *gL = new QGridLayout(this);

  QIsePluginList<QIsePlugin*> regionBinaryFunctionList
    = PluginRegistry::getRegionBinaryFunctionPlugin().getGenericList();
  m_pcw = new PluginIndexChoiceWidget(regionBinaryFunctionList, tr("Binary function: "), this);
  
  gL->addWidget(m_pcw, 0, 0);

  setLayout(gL);

  gL->setMargin(0); //because embedded widget

  connect(m_pcw, SIGNAL(pluginIndexChanged(int)),
	  this, SLOT(pluginChangedTo(int)));

  //- configure & display GUI for selected plugins
  initPlugin();  
}

void
RegionBinaryFunctionComboWidget::setText(const QString &title)
{
  assert(m_pcw);
  m_pcw->setText(title);
}


void
RegionBinaryFunctionComboWidget::setPlugin(int index)
{
  assert(m_mainView);
  assert(m_mainView->model());
  assert(m_mainView->model()->getSegContextPtr());

  assert(index != -1);
  RegionBinaryFunctionPlugin *pi = reinterpret_cast<RegionBinaryFunctionPlugin*>( m_pcw->getPlugin(index) );

  pi->setSegContext(m_mainView->model()->getSegContextPtr()/*m_segContext*/);
  
  RegionBinaryFunctionWidget *w = pi->parameterWidget(m_mainView, this);
  if (w != NULL) {
    QGridLayout *gL = static_cast<QGridLayout *>(layout());
    assert(gL);
    gL->addWidget(w, PLUGIN_WIDGET_ROW, 0);
  }
  if (index != m_previousIndex)
    emit pluginChanged();

  m_previousIndex = index;
}

void
RegionBinaryFunctionComboWidget::pluginChangedTo(int index)
{
  assert(m_mainView);
  assert(m_mainView->model());
  assert(m_mainView->model()->getSegContextPtr());

  if (m_previousIndex != -1) {

    QGridLayout *gL = static_cast<QGridLayout *>(layout());
    assert(gL);
    QLayoutItem *li = gL->itemAtPosition(PLUGIN_WIDGET_ROW, 0);
    if (li != NULL) {
      RegionBinaryFunctionWidget *w = reinterpret_cast<RegionBinaryFunctionWidget*>(li->widget());
      if (w != NULL) {
	RegionBinaryFunctionPlugin *p = reinterpret_cast<RegionBinaryFunctionPlugin*>( m_pcw->getPlugin(m_previousIndex) );
	girl::shared_ptr<RegionBinaryFunctionSettings> s = p->createSettings(m_mainView->model()->getSegContextPtr()/*m_segContext*/);
	w->getGUI(s);
	p->setSettings(s);
	delete w;
      }
    } 

  }

  setPlugin(index);

}

void
RegionBinaryFunctionComboWidget::initPlugin()
{
  setPlugin(m_pcw->getPluginIndex());
}

RegionBinaryFunctionPlugin *
RegionBinaryFunctionComboWidget::getRegionBinaryFunctionPlugin() const
{
  assert(m_mainView);
  assert(m_mainView->model());
  assert(m_mainView->model()->getSegContextPtr());

  assert(m_pcw);
  RegionBinaryFunctionPlugin *p = static_cast<RegionBinaryFunctionPlugin *>(m_pcw->getPlugin());
  
  QGridLayout *gL = static_cast<QGridLayout *>(layout());
  assert(gL);
  QLayoutItem *li = gL->itemAtPosition(PLUGIN_WIDGET_ROW, 0);
  if (li != NULL) {
    RegionBinaryFunctionWidget *w = reinterpret_cast<RegionBinaryFunctionWidget*>(li->widget());
    if (w != NULL) {
      girl::shared_ptr<RegionBinaryFunctionSettings> s = p->createSettings(m_mainView->model()->getSegContextPtr()/*m_segContext*/);
      w->getGUI(s);
      p->setSettings(s);
    }
  }

  return p;
}

void
RegionBinaryFunctionComboWidget::setRegionBinaryFunctionPlugin(RegionBinaryFunctionPlugin *p)
{
  //m_pcw->setPlugin(p);
  m_pcw->setCurrentIndex(m_pcw->findText(p->name()));
}
