/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef REGIONBINARYFUNCTIONPLUGIN_HPP_
#define REGIONBINARYFUNCTIONPLUGIN_HPP_


#include <ise/iseRegionBinaryFunction.hpp>

#include "QIsePlugin.hpp"
#include "SettingsInterfaceTemplate.hxx"
#include "MainView.hpp"

namespace ise {
  class SegmentationContext;
}

class RegionBinaryFunctionWidget; // TODO


typedef SettingsInterfaceTemplate<RegionBinaryFunctionWidget, ise::RegionBinaryFunction> RegionBinaryFunctionSettings;

/**
 * Interface for plugins which implements a RegionBinaryFunction
 */
class GIRL_API RegionBinaryFunctionPlugin : public QIsePlugin,
					    public ise::RegionBinaryFunctionFactory
{

 public:

  virtual ~RegionBinaryFunctionPlugin() {}

  girl::shared_ptr<ise::RegionBinaryFunction>
    getNewRegionBinaryFunction(ise::SegmentationContext *segContext)
  {
    girl::shared_ptr<ise::RegionBinaryFunction> function =
      create(segContext);
    girl::shared_ptr<RegionBinaryFunctionSettings> s = settings();
    if (s)
      s->configureAlgo(function);
    return function;
  }

  RegionBinaryFunctionWidget *parameterWidget(MainView *mainView, QWidget* parent)
  {
    RegionBinaryFunctionWidget *w = createWidget(mainView/*m_segContext*/, parent);
    girl::shared_ptr<RegionBinaryFunctionSettings> s = settings();
    if (s)
      s->configureWidget(w);
    return w;
  }


  girl::shared_ptr<RegionBinaryFunctionSettings>
    settings()
  {
    if (!m_settings)
      m_settings = createSettings(m_segContext);
    return m_settings;
  }
  
  void
    setSettings(girl::shared_ptr<RegionBinaryFunctionSettings> settings)
  {
    m_settings = settings;
  }
  

  inline void setSegContext(ise::SegmentationContext* segContext)
  {
    m_segContext = segContext;
  }

  inline ise::SegmentationContext *getSegContext()
  {
    return m_segContext;
  }

  virtual girl::shared_ptr<ise::RegionBinaryFunction>
    create(ise::SegmentationContext *segContext) = 0;

  virtual girl::shared_ptr<RegionBinaryFunctionSettings>
    createSettings(ise::SegmentationContext* segContext) = 0;


  virtual bool hasSettings() const
  {
    return true;
  }

 protected:

  RegionBinaryFunctionPlugin(const QString &name,
			     const QIseVersion &pluginVersion,
			     const QIseVersion &qiseVersion,
			     QString path = QString::fromUtf8(""))
    : QIsePlugin(name, (QString::fromUtf8("RegionBinaryFunction/") + path),
		 pluginVersion, qiseVersion),
    m_segContext(NULL)
      {}

  virtual RegionBinaryFunctionWidget *
    createWidget(MainView *mainView/*ise::SegmentationContext *segContext*/, QWidget *parent) = 0;

 protected:

  girl::shared_ptr<RegionBinaryFunctionSettings> m_settings;
  ise::SegmentationContext* m_segContext;

};

#define RegionBinaryFunctionPlugin_iid "org.girl.qise.RegionBinaryFunctionPlugin/1.0"
Q_DECLARE_INTERFACE(RegionBinaryFunctionPlugin, RegionBinaryFunctionPlugin_iid)

#endif /* REGIONUNARYFUNCTIONPLUGIN_HPP_ */




