/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux boris.mansencal@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
#include "RegionBinaryFunctionWithRegSetWeightSettings.hpp"

#include <cassert>
#include <ise/iseRegionBinaryFunctionWithRegSet.hpp>
#include "RegionBinaryFunctionWithRegSetWeightWidget.hpp"

void
RegionBinaryFunctionWithRegSetWeightSettings::configureWidget(RegionBinaryFunctionWidget *w) const
{
  RegionBinaryFunctionWithRegSetWeightWidget *widget =
      static_cast<RegionBinaryFunctionWithRegSetWeightWidget *>(w);
  assert(widget);
  widget->setParameters(m_regSet, m_weight);

}

void
RegionBinaryFunctionWithRegSetWeightSettings::getWidgetConfig(const RegionBinaryFunctionWidget *w)
{
  const RegionBinaryFunctionWithRegSetWeightWidget *widget =
      static_cast<const RegionBinaryFunctionWithRegSetWeightWidget *>(w);
  assert(widget);
  widget->getParameters(m_regSet, m_weight);

}

void
RegionBinaryFunctionWithRegSetWeightSettings::configureAlgo(girl::shared_ptr<ise::RegionBinaryFunction> r) const
{
  girl::shared_ptr<ise::RegionBinaryFunctionWithRegSet> regSet =
      girl::static_pointer_cast<ise::RegionBinaryFunctionWithRegSet>(r);

  assert(regSet);
  regSet->setRegSet(m_regSet);

}

girl::shared_ptr<RegionBinaryFunctionSettings> 
RegionBinaryFunctionWithRegSetWeightSettings::clone() const
{
  girl::shared_ptr<RegionBinaryFunctionWithRegSetWeightSettings> p(new RegionBinaryFunctionWithRegSetWeightSettings);

  *p = *this;

  return p;
}

bool
RegionBinaryFunctionWithRegSetWeightSettings::isValid() const
{
  return ( m_regSet != girl::shared_ptr<ise::RegionSetInterface>() );
}
