/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef REGIONBINARYPREDICATEPLUGIN_HPP_
#define REGIONBINARYPREDICATEPLUGIN_HPP_

#include <ise/iseRegionBinaryPredicate.hpp>
#include <ise/iseSegmentationContext.hpp>

#include "QIsePlugin.hpp"
#include "RegionBinaryPredicateWidget.hpp"
#include "RegionBinaryPredicateSettings.hpp"
#include "MainView.hpp"

/**
 * Interface for plugins which implements a RegionBinaryFunction
 *
 */
class GIRL_API RegionBinaryPredicatePlugin : public QIsePlugin,
					     public ise::RegionBinaryPredicateFactory
{

 public:

  virtual ~RegionBinaryPredicatePlugin() {}

  virtual girl::shared_ptr<ise::RegionBinaryPredicate>
  getNewRegionBinaryPredicate(ise::SegmentationContext *segContext)
  {
    girl::shared_ptr<ise::RegionBinaryPredicate> p = create(segContext);
    girl::shared_ptr<RegionBinaryPredicateSettings> s = settings();
    if (s)
      s->configureAlgo(p);
    return p;
  }

  virtual RegionBinaryPredicateWidget *
    parameterWidget(MainView *mainView, QWidget *parent = NULL)
  {
    RegionBinaryPredicateWidget *w = createWidget(mainView,/*m_segContext,*/ parent);
    girl::shared_ptr<RegionBinaryPredicateSettings> s = settings();
    if (s)
      s->configureWidget(w);
    return w;
  }

  
  girl::shared_ptr<RegionBinaryPredicateSettings>
  settings()
  {
    if (!m_settings)
      m_settings = createSettings(m_segContext);
    return m_settings;
  }

  void
  setSettings(girl::shared_ptr<RegionBinaryPredicateSettings> settings)
  {
    m_settings = settings;
  }
 
  inline void setSegContext(ise::SegmentationContext* segContext)
  {
    m_segContext = segContext;
  }

  inline ise::SegmentationContext *getSegContext()
  {
    return m_segContext;
  }

   virtual girl::shared_ptr<ise::RegionBinaryPredicate> 
     create(ise::SegmentationContext *segContext) = 0;
 
  virtual girl::shared_ptr<RegionBinaryPredicateSettings> 
    createSettings(ise::SegmentationContext *segContext) = 0;  

  virtual bool hasSettings() const
  {
    return true;
  }

 protected:

  RegionBinaryPredicatePlugin(const QString &name,
			      const QIseVersion &pluginVersion,
			      const QIseVersion &qiseVersion,
			      QString path = QString::fromUtf8(""))
    : QIsePlugin(name, (QString::fromUtf8("RegionBinaryPredicate/") + path),
		 pluginVersion, qiseVersion),
    m_segContext(NULL)
      {}

  virtual RegionBinaryPredicateWidget *
    createWidget(MainView *mainView, /*ise::SegmentationContext *segContext,*/ QWidget *parent) = 0;

 protected:
  girl::shared_ptr<RegionBinaryPredicateSettings> m_settings;
  ise::SegmentationContext* m_segContext;
};

#define RegionBinaryPredicatePlugin_iid "org.girl.qise.RegionBinaryPredicatePlugin/1.0"
Q_DECLARE_INTERFACE(RegionBinaryPredicatePlugin, RegionBinaryPredicatePlugin_iid)

#endif /* REGIONBINARYPREDICATEPLUGIN_HPP_ */
