/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "RegionBorderBuilderThread.hpp"

#if defined(TIMING)
#include <girl/girlTimer.hpp>
#endif
#ifdef VERBOSE_THREAD
#include <iostream>
#endif

RegionBorderBuilderThread::RegionBorderSetter::RegionBorderSetter()
  : m_hash(NULL), 
    m_img(NULL),
    m_paths(),
    m_data(NULL),
    m_bytesPerLine(0),
    m_width(0)
{

}

RegionBorderBuilderThread::RegionBorderSetter::RegionBorderSetter(const HashType &hash, const QImage &img)
  : m_hash(&hash), 
    m_img(&img),
    m_paths(),
    m_data(m_img->bits()),
    m_bytesPerLine(m_img->bytesPerLine()),
    m_width(m_img->width())
{
  
}

void
RegionBorderBuilderThread::RegionBorderSetter::processPoint(const girl::point &pt)
{
  //Process pixels inside a region domain.
  //We check two neighbour of pixel (pt.x, pt.y) : UP (pt.x; pt.y-1) and RIGHT (pt.x+1, pt.y)

  //REM: we may access pixels that are out of region domain, but in image.
  
  const HashType &hash = *m_hash;
  const QImage &img = *m_img;

  const uchar *s = m_data + pt.y() * m_bytesPerLine;

  const QRgb thColor = hash[ ((const uint *)s)[pt.x()] ].thresholdedColor;
  if (pt.y()>0) { 

    
    const uchar *sm1 = s - m_bytesPerLine; 

    const QRgb thColorUp = hash[ ((const uint *)sm1)[pt.x()] ].thresholdedColor;
    if (thColor != thColorUp) {
      //add border (x;y), (x+1;y)
      
      m_paths.push_back( girl::point(pt.x()-1, pt.y()-1) );
      m_paths.push_back( girl::point(pt.x(), pt.y()-1) );
    }
  }

  if (pt.x() <img.width()-1) {
    const QRgb thColorR = hash[ ((const uint *)s)[pt.x()+1] ].thresholdedColor;
    if (thColor != thColorR) {
      //add border (x+1;y), (x+1;y+1)
      
      m_paths.push_back( girl::point(pt.x(), pt.y()-1) );
      m_paths.push_back( pt ); //m_paths.push_back( girl::point(pt.x(), pt.y()) );
      
    }
  }

}

void
RegionBorderBuilderThread::RegionBorderSetter::processLine(int y)
{
  /* //processLine0 

  const HashType &hash = *m_hash;
  const QImage &img = *m_img;

  const uchar *s = m_data + y * m_bytesPerLine;

  if (y>0) { 
   const uchar *sm1 = s - m_bytesPerLine; 

   for (int x=0; x<m_width; ++x) {
     const QRgb thColor   = hash[ ((const uint *)s)[x] ].thresholdedColor;
     const QRgb thColorUp = hash[ ((const uint *)sm1)[x] ].thresholdedColor;
     if (thColor != thColorUp) {
       m_paths.push_back( girl::point(x-1, y-1) );
       m_paths.push_back( girl::point(x, y-1) );
     }
   }
  }


  QRgb thColor   = hash[ ((const uint *)s)[0] ].thresholdedColor;
  for (int x=1; x<m_width; ++x) {  
    const QRgb thColorR = hash[ ((const uint *)s)[x] ].thresholdedColor;
    if (thColor != thColorR) {
      m_paths.push_back( girl::point(x-1, y-1) );
      m_paths.push_back( girl::point(x-1, y) ); 
    }
    thColor = thColorR;
  }

  */

  //processLine2

  const HashType &hash = *m_hash;
  //const QImage &img = *m_img;

  const uchar *s = m_data + y * m_bytesPerLine;

  if (y>0) { 
   const uchar *sm1 = s - m_bytesPerLine; 

   QRgb thColor   = hash[ ((const uint *)s)[0] ].thresholdedColor;
   const QRgb thColorUp = hash[ ((const uint *)sm1)[0] ].thresholdedColor;
   if (thColor != thColorUp) {
     m_paths.push_back( girl::point(0-1, y-1) );
     m_paths.push_back( girl::point(0, y-1) );
   }

   for (int x=1; x<m_width; ++x) {
     const QRgb thColorR = hash[ ((const uint *)s)[x] ].thresholdedColor;
     if (thColor != thColorR) {
       m_paths.push_back( girl::point(x-1, y-1) );
       m_paths.push_back( girl::point(x-1, y) ); 
     }

     thColor = thColorR;

     const QRgb thColorUp = hash[ ((const uint *)sm1)[x] ].thresholdedColor;
     if (thColor != thColorUp) {
       m_paths.push_back( girl::point(x-1, y-1) );
       m_paths.push_back( girl::point(x, y-1) );
     }
   }
  
  }     
  else {

    QRgb thColor   = hash[ ((const uint *)s)[0] ].thresholdedColor;
    for (int x=1; x<m_width; ++x) {  
      const QRgb thColorR = hash[ ((const uint *)s)[x] ].thresholdedColor;
      if (thColor != thColorR) {
	m_paths.push_back( girl::point(x-1, y-1) );
	m_paths.push_back( girl::point(x-1, y) );
      }
      thColor = thColorR;
    }

    
  }


}


RegionBorderBuilderThread::RegionBorderBuilderThread(QObject *parent)
  : QThread(parent),
  m_mutex(),
  m_condition(),
  m_rbs(),
  m_toger(NULL),
  m_regions(),
  m_scanLine(0),
  m_nbLines(0),
  m_doRegions(false),
  m_restart(false),
  m_abort(false)
{

}


RegionBorderBuilderThread::~RegionBorderBuilderThread()
{
  m_mutex.lock();
  m_abort = true;
  m_condition.wakeOne();
  m_mutex.unlock();

  wait();
}


void
RegionBorderBuilderThread::process(const QuantificationThresholdsWidgetAux::HashType &hash, 
				   const QImage &img,
				   int scanLine, int nbLines)
{
  QMutexLocker locker(&m_mutex);

  m_scanLine = scanLine;
  m_nbLines = nbLines;
  
  m_rbs = RegionBorderSetter(hash, img);
  m_doRegions = false;

  if (! isRunning()) {
#ifdef VERBOSE_THREAD
    std::cerr<<"start"<<std::endl;
#endif

    start(); //start(LowPriority);
  }
  else {
#ifdef VERBOSE_THREAD
    std::cerr<<"restart"<<std::endl;
#endif

    m_restart = true;
    m_condition.wakeOne();
  }

}

void
RegionBorderBuilderThread::process(const QuantificationThresholdsWidgetAux::HashType &hash, 
				   const QImage &img,
				   const girl::toger &tog, 
				   const girl::regionDeque &regions)
{
  QMutexLocker locker(&m_mutex);
  
  m_toger = &tog;
  m_regions = regions;
  m_rbs = RegionBorderSetter(hash, img);
  m_doRegions = true;
  
  
  if (! isRunning()) {
#ifdef VERBOSE_THREAD
    std::cerr<<"start"<<std::endl;
#endif

    start(); //start(LowPriority);
  }
  else {
#ifdef VERBOSE_THREAD
    std::cerr<<"restart"<<std::endl;
#endif

    m_restart = true;
    m_condition.wakeOne();
  }

}



void
RegionBorderBuilderThread::run()
{
  forever {

#if defined(TIMING)
    girl::Timer t;
    t.start();
#endif

    m_mutex.lock();

    if (m_doRegions && m_regions.empty() && ! m_abort) {
      m_condition.wait(&m_mutex);
    }
    
    if (m_abort) {
      m_mutex.unlock();
      return;
    }
    
    const int scanLine = m_scanLine;
    const int nbLines = m_nbLines;
    RegionBorderSetter rbs = m_rbs;
    const girl::toger &tog = *m_toger;
    const bool doRegions = m_doRegions;
    m_mutex.unlock();
    
    if (! doRegions) {

      assert(rbs.m_img);
      const int h = rbs.m_img->height();
      //const int w = rbs.m_img->width();
      
      const int end = std::min(scanLine + nbLines, h);
      for (int j=scanLine; j<end; ++j) {
	if (m_restart)
	  break;
	if (m_abort)
	  return;
	
	rbs.processLine(j);
	/*
	for (int i=0; i<w; ++i)
	  rbs.processPoint(girl::point(i, j));
	*/
      }
      
    }
    else {
      
      girl::regionDeque::const_iterator it = m_regions.begin();
      const girl::regionDeque::const_iterator itEnd = m_regions.end();
      for ( ; it != itEnd; ++it) {
	if (m_restart)  //TODO:OPTIM?: maybe to do a restart only when a region is done is not enough ???
	  break;
	if (m_abort)
	  return;
	tog.iterateOnRegionDomain(*it, rbs);
      }

    }
    
      
    if (! m_restart) {

#if defined(TIMING)
      const size_t time = t.stop();
      std::cerr<<"FalseColorImageBuilderThread::run  time border builder ="<<time<<"ms"<<std::endl;
#endif
#ifdef VERBOSE_THREAD
      std::cerr<<"will emit for paths.size()="<<rbs.m_paths.size()<<std::endl;
#endif
      emit processed(rbs.m_paths);
    }

    m_mutex.lock();
    if (! m_restart)
      m_condition.wait(&m_mutex);
    m_restart = false;
    m_mutex.unlock();
    
  }

}
