/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef REGIONBORDERBUILDERTHREAD_HPP
#define REGIONBORDERBUILDERTHREAD_HPP

#include <girl/girlToger.hpp>

#include <QMutex>
#include <QThread>
#include <QWaitCondition>

#include "QuantificationThresholdsWidgetAux.hpp" //for HashType

class RegionBorderBuilderThread : public QThread
{
  Q_OBJECT

public:
  explicit RegionBorderBuilderThread(QObject *parent = NULL);

  ~RegionBorderBuilderThread();

  void process(const QuantificationThresholdsWidgetAux::HashType &hash, 
	       const QImage &img,
	       int scanLine, int nbLines);

  void process(const QuantificationThresholdsWidgetAux::HashType &hash, 
	       const QImage &img,
	       const girl::toger &tog, 
	       const girl::regionDeque &regions);

	       
signals:
  
  void processed(const std::deque<girl::point> &paths);

protected:

  void run();


protected:

  typedef QuantificationThresholdsWidgetAux::HashType HashType;

  class RegionBorderSetter : public girl::toger::pointIterator
  {
  public:
    RegionBorderSetter();

    RegionBorderSetter(const HashType &hash, const QImage &img);

    void processPoint(const girl::point &pt);

    void processLine(int y);


  public:
    const HashType *m_hash;
    const QImage *m_img;
    std::deque<girl::point> m_paths;
    const uchar *m_data;
    int m_bytesPerLine;
    int m_width;
  };
   

  QMutex m_mutex;
  QWaitCondition m_condition;
  RegionBorderSetter m_rbs;
  const girl::toger *m_toger;
  girl::regionDeque m_regions;
  int m_scanLine;
  int m_nbLines;
  bool m_doRegions;

  bool m_restart;
  bool m_abort;
};

#endif /* ! REGIONBORDERBUILDERTHREAD_HPP */
