/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 */

#ifndef REGIONFEATURECHOICECOMBOWIDGETTEMPLATE_HPP
#define REGIONFEATURECHOICECOMBOWIDGETTEMPLATE_HPP

#include <vector>
#include <string>

#include <girl/girlCommon.hpp>
#include <qiseTools/FeatureComputerGUIInfo.hpp>
#include "PluginRegistry.hpp"

#include <QWidget>

class QVBoxLayout;
#include <QComboBox>

class MainView;
namespace ise{
  class SegmentationContext;
}

class GIRL_API RegionFeatureChoiceComboWidgetTemplate : public QWidget
{
  Q_OBJECT

public:

  std::string getCurrentRegionFeature() const;

  void setRegionFeature(const std::string &uri);

  void setRegionFeatureComputerPtr(ise::FeatureComputerPtr algo);

  ise::FeatureComputerPtr regionFeatureComputerPtr() const;

  size_t size() const;

  int currentIndex() const { return m_regionFeatureCombo->currentIndex(); }

signals:

  void regionFeatureChanged();
  void currentIndexChanged(int);

protected slots:

  void changeRegionFeature(int index);

protected:

  void build();
  
  RegionFeatureChoiceComboWidgetTemplate(MainView *mainView,
					 FeatureComputerGUIInfoPtr featureComputerGUIInfo,
					 QWidget *parent = NULL,
					 bool applyChange = false);

  template <typename T> void populateRegionFeatureCombo()
  {
    m_regionFeatureCombo->clear();
    std::vector<FeatureDataPtr> list;
    m_featureComputerGUIInfo->getFeatureComputersInfo<T>(list);
    
    size_t sizeData = list.size();
    for (size_t i = 0 ; i < sizeData ; ++i) {
      m_regionFeatureCombo->addItem(list[i]->title().c_str());
    }

    m_regionFeatureCombo->blockSignals(true);
    m_regionFeatureCombo->addItem("new feature...");
    m_regionFeatureCombo->blockSignals(false);

    if (m_regionFeatureCombo->count() == 1 && m_regionFeatureCombo->currentText() == "new feature...")
      m_regionFeatureCombo->setCurrentIndex(-1);
    else
      m_regionFeatureCombo->setCurrentIndex(0);
  }

  void rebuild(FeatureComputerGUIInfoPtr featureComputerGUIInfo);

  virtual void rebuild() = 0;

  void showEvent ( QShowEvent * event );

 private:
  
  template <typename T>
    void toFeaturePluginList(const QIsePluginList<T*> &src, QIsePluginList<FeaturePlugin*> &dest)
  {
    dest.clear();
    typename QIsePluginList<T*>::const_iterator it = src.begin();
    typename QIsePluginList<T*>::const_iterator itEnd = src.end();
    for ( ; it != itEnd ; ++it)
      dest.append(static_cast<FeaturePlugin*>(*it));
  }


protected:
  
  MainView *m_mainView;
  QComboBox *m_regionFeatureCombo;
  FeatureComputerGUIInfoPtr m_featureComputerGUIInfo;
  bool m_applyChange;
};

#endif /* REGIONFEATURECHOICECOMBOWIDGETTEMPLATE_HPP */
