/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 */

#include "RegionFeatureChoiceWidget.hpp"

#include <cassert>

#include <ise/iseSegmentationContext.hpp>

#include <QGroupBox>
#include <QVBoxLayout>

#include "CheckableSliderSpinBox.hpp"


RegionFeatureChoiceWidget::RegionFeatureChoiceWidget(ise::SegmentationContext *segContext,
    QWidget *parent,
    const QString &title)
: QWidget(parent),
  m_checkBoxesMap(),
  m_gb(NULL)
{
  build(segContext, title);
}

void
RegionFeatureChoiceWidget::build(ise::SegmentationContext *segContext, const QString &title)
{
  assert(segContext);
  m_gb = new QGroupBox(title, this);

  ise::FeatureComputer::FeatureComputerListType regSetUris;
  segContext->getRegionSetUris(regSetUris);

  QVBoxLayout *gbLayout = new QVBoxLayout;

  ise::FeatureComputer::FeatureComputerListType::const_iterator it = regSetUris.begin();
  const ise::FeatureComputer::FeatureComputerListType::const_iterator itEnd = regSetUris.end();
  for ( ; it != itEnd; ++it) {
    girl::shared_ptr<ise::RegionSetInterface> regSet =
        segContext->getRegionSet(*it);
    std::string title =  regSet->uri();
    CheckableSliderSpinBox *checkBox =
        new CheckableSliderSpinBox(QString(title.c_str()), this);
    gbLayout->addWidget(checkBox);
    m_checkBoxesMap[*it] = checkBox;
  }

  gbLayout->addStretch();
  m_gb->setLayout(gbLayout);

  QVBoxLayout *layout = new QVBoxLayout;
  layout->addWidget(m_gb);
  this->setLayout(layout);

  //  QLabel *label = new QLabel(tr("Region Feature:"));
  //  m_regionFeatureCombo = new QComboBox(this);
  //
  //  std::list<std::string> list;
  //  segC->getRegionSetUris(list);
  //
  //  std::list<std::string>::iterator it = list.begin();
  //  const std::list<std::string>::iterator itEnd = list.end();
  //  for ( ; it != itEnd ; ++it) {
  //    m_regionFeatureCombo->addItem((*it).c_str());
  //  }
  //
  //  connect(m_regionFeatureCombo, SIGNAL(currentIndexChanged(int)),
  //	  this, SLOT(changeRegionFeature(int)));
  //  connect(m_regionFeatureCombo, SIGNAL(currentIndexChanged(int)),
  //          this, SIGNAL(parametersChanged()));
  //  QHBoxLayout *hbox = new QHBoxLayout;
  //
  //  hbox->addWidget(label);
  //  hbox->addWidget(m_regionFeatureCombo);
  //  hbox->addStretch();
  //  hbox->setMargin(0);
  //  this->setLayout(hbox);
}

//void
//RegionFeatureChoiceWidget::changeRegionFeature(int index)
//{
//  emit regionFeatureChanged(m_regionFeatureCombo->itemText(index).toStdString());
//}
//
//size_t
//RegionFeatureChoiceWidget::size() const {
//  return m_regionFeatureCombo->count();
//}

//void
//RegionFeatureChoiceWidget::setRegionFeature(std::string uri/*int regionFeature*/)
//{
//  m_regionFeatureCombo->setCurrentIndex(m_regionFeatureCombo->findText(uri.c_str()));
//}

void
RegionFeatureChoiceWidget::getRegionFeaturesInfo(std::vector<std::string>& uris,
    std::vector<double> &weights) const
{

  uris.clear();
  weights.clear();

  girl::hashMap<std::string, CheckableSliderSpinBox*>::const_iterator it
  = m_checkBoxesMap.begin();
  const girl::hashMap<std::string, CheckableSliderSpinBox*>::const_iterator itEnd
  = m_checkBoxesMap.end();
  for (; it != itEnd; ++it) {
    CheckableSliderSpinBox* checkBox = it->second;
    assert(checkBox);
    if (checkBox->isChecked()) {
      uris.push_back(checkBox->getText());
      weights.push_back(checkBox->getWeight());
    }
  }
}

void
RegionFeatureChoiceWidget::setRegionFeaturesInfo(const std::vector<std::string> &uris,
    const std::vector<double> &weights)
{
  assert(uris.size() == weights.size());
  std::vector<std::string>::const_iterator uriIt = uris.begin();
  const std::vector<std::string>::const_iterator uriItEnd = uris.end();
  std::vector<double>::const_iterator weightsIt = weights.begin();
  for (; uriIt != uriItEnd; ++uriIt, ++weightsIt) {
    girl::hashMap<std::string, CheckableSliderSpinBox*>::iterator it
    = m_checkBoxesMap.find(*uriIt);
    assert(it != m_checkBoxesMap.end());
    it->second->setChecked(true);
    it->second->setWeight(*weightsIt);
  }
}

void
RegionFeatureChoiceWidget::setRegionFeatureInfo(const std::string &uri, const double &weight)
{
  girl::hashMap<std::string, CheckableSliderSpinBox*>::iterator it
  = m_checkBoxesMap.find(uri);
  // REMI: TODO: URI can be empty(beginning... check if it is empty first, then
  // check if != end ?
  if (it != m_checkBoxesMap.end()) {
    it->second->setChecked(true);
    it->second->setWeight(weight);
  }
}
