/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 */

#ifndef REGIONFEATURECHOICEWIDGET_HPP
#define REGIONFEATURECHOICEWIDGET_HPP

#include <vector>
#include <string>

#include <girl/girlCommon.hpp>
#include <girl/girlHashFunc.hpp>
#include <girl/girlHashMap.hpp>

#include <QGroupBox>
#include <QVBoxLayout>
#include <QWidget>

namespace ise{
  class SegmentationContext;
}

class CheckableSliderSpinBox;

/**
 * Groups CheckableSliderSpinBox widgets
 */

class GIRL_API RegionFeatureChoiceWidget : public QWidget
{
  Q_OBJECT

public:
  /**
   * @brief Constructor.
   */
  RegionFeatureChoiceWidget(ise::SegmentationContext *segContext,
      QWidget *parent = NULL,
      const QString &title = tr("Region Features")
  );

  /**
   * Fill the vector @param uris with the region features that
   * have been checked and their corresponding weights
   */
  void getRegionFeaturesInfo(std::vector<std::string> &uris,
      std::vector<double> &weights) const;

  /**
   * Check the region features given in parameters and their corresponding weights
   */
  void setRegionFeaturesInfo(const std::vector<std::string> &uris,
      const std::vector<double> &weights);

  /**
   * Check the region feature given in parameter and it's corresponding weight
   */
  void setRegionFeatureInfo(const std::string &uri, const double &weight);


  size_t size() const;

  // signals:

  //void regionFeatureChanged(std::string  regionFeatureUri);

  // void parametersChanged();

protected slots:

//  void changeRegionFeature(int index);

protected:

  void build(ise::SegmentationContext *segContext, const QString &title);

protected:

  girl::hashMap<std::string, CheckableSliderSpinBox*> m_checkBoxesMap;
  QGroupBox *m_gb;

};

#endif /* REGIONFEATURECHOICEWIDGET_HPP */
