/* 
 * Copyright (C) 2010-2011 Remi Vieux  remi.vieux@labri.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation (Inc., 59
 * Temple Place - Suite 330, Boston, MA 02111-1307, USA); either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "RegionFeatureContextualDialog.hpp"

#include "SliderSpinBoxWidget.hpp"

#include <QDialogButtonBox>
#include <QGridLayout>
#include <QLabel>

void
RegionFeatureContextualDialog::setGui()
{
  assert(m_featureComputer);
  assert(m_numBinsSlider);
  assert(m_startAngleSlider);

  size_t numBins;
  double startAngle;
  m_featureComputer->getParameters(numBins, startAngle);
  m_numBinsSlider->setValue(static_cast<double>(numBins));
  m_startAngleSlider->setValue(startAngle);
}

void
RegionFeatureContextualDialog::getGui()
{
  assert(m_featureComputer);
  assert(m_numBinsSlider);
  assert(m_startAngleSlider);

  size_t numBins = static_cast<int>(m_numBinsSlider->value());
  double angle   = m_startAngleSlider->value();
  m_featureComputer->setParameters(numBins, angle);
}

void
RegionFeatureContextualDialog::build()
{
  QLabel *nblabel = new QLabel("Number of bins", this);
  QLabel *aglabel = new QLabel("Start angle", this);
  m_numBinsSlider = new SliderSpinBoxWidget(this);
  m_numBinsSlider->setMinimum(1);
  m_numBinsSlider->setMaximum(359);
  m_numBinsSlider->setDecimals(0);
  m_numBinsSlider->setSingleStep(1);
  m_startAngleSlider = new SliderSpinBoxWidget(this);
  m_startAngleSlider->setMinimum(0);
  m_startAngleSlider->setMaximum(359.5);
  m_startAngleSlider->setDecimals(1);
  m_startAngleSlider->setSingleStep(0.5);

  QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok, Qt::Horizontal, this);

  QGridLayout *layout = new QGridLayout;
  layout->addWidget(nblabel, 0, 0);
  layout->addWidget(m_numBinsSlider, 0, 1);
  layout->addWidget(aglabel, 1, 0);
  layout->addWidget(m_startAngleSlider, 1, 1);
  layout->addWidget(buttonBox, 2, 1);
  setLayout(layout);
  connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
}
