/*
  Copyright (C) 2005-2011 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef REGIONFEATURELISTMULTIPLUGINSETTINGS_HPP
#define REGIONFEATURELISTMULTIPLUGINSETTINGS_HPP

#include <girl/girlCommon.hpp>
#include <girl/girlHashFunc.hpp>
#include <girl/girlHashMap.hpp>

#include "SettingsInterfaceTemplate.hxx"
#include "RegionFeatureListMultiPluginWidget.hpp"
#include <ise/iseRegionSetInterface.hpp>

typedef SettingsInterfaceTemplate<RegionFeatureListMultiPluginWidget, ise::RegionSetInterface> RegionFeatureListMultiPluginSettingsInterface;

class GIRL_API RegionFeatureListMultiPluginSettings : public RegionFeatureListMultiPluginSettingsInterface{

 public:

  void configureWidget(RegionFeatureListMultiPluginWidget *widget) const
  {
    widget->setGUI(m_settings);
  }

  void getWidgetConfig(const RegionFeatureListMultiPluginWidget *widget)
  {
    widget->getGUI(m_settings);
  }

  virtual void configureAlgo(girl::shared_ptr<ise::RegionSetInterface> ) const
  {
  
  }

 private:

  typedef girl::hashMap<std::string, RegionFeatureDataPtr > RegionFeatureDataContainer;

 public:

  virtual girl::shared_ptr<RegionFeatureListMultiPluginSettingsInterface> clone() const
  {
    girl::shared_ptr<RegionFeatureListMultiPluginSettings> p(new RegionFeatureListMultiPluginSettings);
    p->m_settings.clear();
    RegionFeatureDataContainer::const_iterator it = m_settings.begin();
    const RegionFeatureDataContainer::const_iterator itEnd = m_settings.end();
    for ( ; it != itEnd; ++it) {
      p->m_settings[it->first] = it->second->clone();
    }
    return p;
  }

  virtual bool isValid() const {
    return true; // TODO : JULIAN
  }

 private:

  RegionFeatureDataContainer m_settings;
};

#endif /* ! REGIONFEATURELISTMULTIPLUGINSETTINGS_HPP */
