/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 */

#include "RegionFeatureListMultiPluginWidget.hpp"
#include "RegionFeatureListMultiPluginSettings.hpp"

#include <QGridLayout>
#include <QGroupBox>
#include <QCoreApplication>

#include <ise/iseRegionFeatureComputer.hpp>

#include "QIsePluginList.hpp"
#include "PluginRegistry.hpp"
#include "LabelShowButtonWidget.hpp"
#include "RegionFeatureChoiceDialog.hpp"

RegionFeatureListMultiPluginWidget::RegionFeatureListMultiPluginWidget(MainView *mainView,
								       FeatureComputerGUIInfo &featureComputerGUIInfo,
								       QString name,
								       QWidget *parent)
  : RegionFeatureListPluginWidget(mainView, featureComputerGUIInfo, name, parent)
  , m_list(NULL)
  , m_plugins()
{
  setAttribute(Qt::WA_DeleteOnClose);

  build();

  loadPlugins();

  initWidget();
}


void
RegionFeatureListMultiPluginWidget::build()
{
  QHBoxLayout *layout = new QHBoxLayout;

  QGroupBox *box = new QGroupBox(m_name, this);

  QHBoxLayout *gridLayout = new QHBoxLayout;

  m_list = new QListWidget(this);
  m_list->setMaximumWidth(200);
  m_list->setMinimumWidth(200);
  m_list->setMaximumHeight(100);

  m_listToCompute->setMaximumWidth(200);
  m_listToCompute->setMinimumWidth(200);
  m_listToCompute->setMaximumHeight(100);

  QPushButton *add = new QPushButton(tr("+"), this);
  connect(add, SIGNAL(clicked()), this, SLOT(addNewRegionFeature()));
  QPushButton *del = new QPushButton(tr("-"), this);
  connect(del, SIGNAL(clicked()), this, SLOT(delRegionFeature()));
  
  add->setMaximumWidth(25);
  del->setMaximumWidth(25);

  QVBoxLayout *buttonLayout = new QVBoxLayout;
  buttonLayout->addWidget(add);
  buttonLayout->addWidget(del);


  gridLayout->addWidget(m_list, Qt::AlignLeft);
  gridLayout->addLayout(buttonLayout, Qt::AlignLeft);
  gridLayout->addWidget(m_listToCompute, Qt::AlignLeft);
  gridLayout->addStretch();

  box->setLayout(gridLayout);

  layout->addWidget(box);

  setLayout(layout);
}

RegionFeatureDialog*
RegionFeatureListMultiPluginWidget::newCurrentDialog(RegionFeatureDataPtr &data)
{
  assert(m_mainView);
  assert(m_mainView->model());
  assert(m_mainView->model()->getSegContextPtr());

  QListWidgetItem* item = m_list->currentItem();
  RegionFeatureDialog *dialog = NULL;
  if (item != NULL) {
    size_t index = static_cast<size_t>(m_list->currentRow());

    RegionSetInterfaceWidget *widget = m_plugins[index]->parameterWidget(m_mainView, this);
    girl::shared_ptr<RegionSetInterfaceSettings> settings = m_plugins[index]->createSettings(m_mainView->model()->getSegContextPtr());

    data = FeatureDataPtr(new FeatureData(settings, m_plugins[index]));

    dialog = new FeatureDialog(widget, data, this);
    dialog->setSettings(settings);
    dialog->setTitle(m_plugins[index]->name());


  }
  return dialog;
}

// girl::shared_ptr<RegionFeatureListMultiPluginSettings> 
// RegionFeatureListMultiPluginWidget::createSettings() const
// {
//   return girl::shared_ptr<RegionFeatureListMultiPluginSettings>(new RegionFeatureListMultiPluginSettings());
// }

void
RegionFeatureListMultiPluginWidget::loadPlugins()
{
  QIsePluginList<RegionSetInterfacePlugin*>::iterator it = PluginRegistry::getRegionSetPlugin().begin();
  const QIsePluginList<RegionSetInterfacePlugin*>::iterator itEnd = PluginRegistry::getRegionSetPlugin().end();
  for (size_t i = 0; it != itEnd; ++it, ++i) {
    QListWidgetItem *item = new QListWidgetItem((*it)->name());
    m_list->addItem(item);
    m_plugins.insert(std::make_pair(i, *it));
  }
}
