/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef REGIONFEATURELISTPLUGINWIDGET_HPP
#define REGIONFEATURELISTPLUGINWIDGET_HPP

#include <girl/girlCommon.hpp>

#include <qiseTools/MainView.hpp>
#include <qiseTools/FeatureData.hpp>
#include <qiseTools/RegionFeatureDialog.hpp>
#include <qiseTools/LabelShowButtonWidget.hpp>

#include <QWidget>
#include <QListWidget>
#include <QMessageBox>

class GIRL_API RegionFeatureListPluginWidget : public QWidget
{
  Q_OBJECT

 public:

  typedef girl::hashMap<std::string, RegionFeatureDataPtr > RegionFeatureDataContainer;

 public:

  virtual ~RegionFeatureListPluginWidget() {}

  inline QString getName() const { return m_name; }

  void insertRegionFeatureDataToCompute(const RegionFeatureDataPtr &data, bool exist = false);

  void compute();

  void setGUI(const RegionFeatureDataContainer &settings);
  
  void getGUI(RegionFeatureDataContainer &settings) const;

  void addExistData(RegionFeatureDataPtr regionFeatureData);
  
 signals:

  void addData(RegionFeatureDataPtr);

 protected slots:

  void show(QString);

  void addNewRegionFeature();

  void delRegionFeature();

 protected:

  RegionFeatureListPluginWidget(MainView *mainView, QString name, QWidget *parent);

  virtual void build() = 0;

 private:

  virtual RegionFeatureDialog* newCurrentDialog(RegionFeatureDataPtr&) = 0;

  virtual void initShowWidget(RegionSetInterfaceWidget *widget, RegionFeatureDataPtr data) = 0;
  
  int findInListToRename(const std::string &title);

  bool existTitleData(const std::string &title);

  void addExistDataInWidget(RegionSetInterfaceWidget *widget);

 protected:

  MainView *m_mainView;
  QString m_name;
  
  QListWidget *m_listToCompute;

  RegionFeatureDataContainer m_settings;

  std::vector<std::string> m_regionFeatureToDelete;
  std::vector<std::list<std::string> > m_regionFeatureToRename;

  girl::hashMap<std::string, RegionFeatureDataPtr> m_allExistData;
};

#endif /* ! REGIONFEATURELISTPLUGINWIDGET_HPP */
