/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
#ifndef REGIONFEATUREMOMENTSDIALOG_HPP_
#define REGIONFEATUREMOMENTSDIALOG_HPP_

#include <ise/iseRegionFeatureMomentComputer.hpp>

#include <QDialog>
#include <QDialogButtonBox>
#include <QString>
#include <QVBoxLayout>

#include "RegionFeatureComputerDialog.hpp"
#include "ImageFeatureChoiceWidget.hpp"

class GIRL_API RegionFeatureMomentsComputerDialog : public RegionFeatureComputerDialog
{
  Q_OBJECT

public:
  RegionFeatureMomentsComputerDialog(ise::SegmentationContext *segContext,
				     QString name,
				     girl::shared_ptr<ise::RegionFeatureComputer> featureComputer,
				     QWidget *parent = NULL) :
    RegionFeatureComputerDialog(segContext, name, parent),
    m_featureComputer(girl::static_pointer_cast<ise::RegionFeatureMomentComputer>(featureComputer)),
    m_ifcw(NULL)
  {
    build();
    setGui();
  }

  virtual void setGui()
  {
    assert(m_featureComputer);
    assert(m_ifcw);
    std::string uri;
    m_featureComputer->getParameters(uri);
    m_ifcw->setImageFeature(uri);

  }
  virtual void getGui()
  {
    assert(m_featureComputer);
    assert(m_ifcw);
    m_featureComputer->setParameters(m_ifcw->getImageFeature());
  }

  virtual inline girl::shared_ptr<ise::RegionFeatureComputer> getFeatureComputer() const
  {
    return m_featureComputer;
  }

protected:

  virtual void build()
  {
    m_ifcw = new ImageFeatureChoiceWidget(m_segContext, this);
    QDialogButtonBox *button = new QDialogButtonBox(QDialogButtonBox::Ok, Qt::Horizontal, this);
    QVBoxLayout *layout = new QVBoxLayout;
    layout->addWidget(m_ifcw);
    layout->addWidget(button);
    setLayout(layout);
    connect(button, SIGNAL(accepted()), this, SLOT(accept()));
  }

private:
  girl::shared_ptr<ise::RegionFeatureMomentComputer> m_featureComputer;
  ImageFeatureChoiceWidget *m_ifcw;

};
#endif /* REGIONFEATUREMOMENTSDIALOG_HPP_ */
