/*
  Copyright (C) 2010-2011 Remi Vieux  remi.vieux@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 */

#ifndef REGIONFEATURERUNLENGTHDIALOG_HPP_
#define REGIONFEATURERUNLENGTHDIALOG_HPP_

#include <QLabel>
#include <QString>
#include <QDialogButtonBox>
#include <QVBoxLayout>
#include <ise/iseRegionFeatureRunLength.hpp>

#include "OrientationChoiceWidget.hpp"
#include "RegionFeatureComputerDialog.hpp"


class GIRL_API RegionFeatureRunLengthDialog : public RegionFeatureComputerDialog
{

  Q_OBJECT

public:

  RegionFeatureRunLengthDialog(ise::SegmentationContext *segContext,
      QString name,
      girl::shared_ptr<ise::RegionFeatureComputer> featureComputer,
      QWidget *parent = NULL) :
        RegionFeatureComputerDialog(segContext, name, parent),
        m_featureComputer(girl::static_pointer_cast<ise::RegionFeatureRunLength>(featureComputer)),
        m_ocw(NULL)
  {
    setWindowTitle(tr("Run Length parameters..."));
    build();
    setGui();
  }

  virtual void setGui()
  {
    assert(m_featureComputer);
    assert(m_ocw);
    ise::RunLengthExtractor::Orientation ori;
    m_featureComputer->getParameters(ori);
    m_ocw->setOrientation(ori);
  }

  virtual void getGui()
  {
    assert(m_featureComputer);
    assert(m_ocw);
    ise::RunLengthExtractor::Orientation ori = m_ocw->getOrientation();
    m_featureComputer->setParameters(ori);
  }

  virtual inline
  girl::shared_ptr<ise::RegionFeatureComputer> getFeatureComputer() const
  {
    return m_featureComputer;
  }

protected:

  virtual void build()
  {
    QLabel *label = new QLabel(this->getLabelText(), this);
    m_ocw = new OrientationChoiceWidget(this);
    QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok, Qt::Horizontal, this);
    QVBoxLayout *layout = new QVBoxLayout;
    layout->addWidget(label);
    layout->addWidget(m_ocw);
    layout->addWidget(buttonBox);
    setLayout(layout);
    connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
  }

  virtual const QString getLabelText() const
  {
    return QString(m_featureComputer->uri().c_str());
  }

private:

  girl::shared_ptr<ise::RegionFeatureRunLength> m_featureComputer;

  OrientationChoiceWidget *m_ocw;

};

#endif /* REGIONFEATURERUNLENGTHDIALOG_HPP_ */
