/*
  Copyright (C) 2010-2011 Remi Vieux  remi.vieux@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 */
#include "RegionFeatureWidget.hpp"

#include <ise/iseRegionSetFactoryHandler.hpp>

#include "RegionSetInterfaceWidget.hpp"

#include <QDialogButtonBox>
#include <QInputDialog>
#include <QListWidget>
#include <QMessageBox>
#include <QGridLayout>
#include <QGroupBox>
#include "PluginRegistry.hpp"

#include "LabelShowButtonWidget.hpp"


static int g_count = 0;

typedef struct TitleUriInfo {

} TitleUriInfo;

RegionFeatureWidget::RegionFeatureWidget(MainView *mainView,
    QString name,
    QWidget *parent)
:QWidget(parent),
 m_mainView(mainView),
 m_segContext(mainView->model()->getSegContextPtr()),
 m_name(name)
{
  setAttribute(Qt::WA_DeleteOnClose);

  build();
}

void
RegionFeatureWidget::build()
{
  QHBoxLayout *layout = new QHBoxLayout;

  QGroupBox *box = new QGroupBox(m_name, this);

  QHBoxLayout *gridLayout = new QHBoxLayout;

  m_list = new QListWidget(this);
  // m_list->setMaximumWidth(200);
  // m_list->setMinimumWidth(200);
  // m_list->setMaximumHeight(100);
  connect(m_list, SIGNAL(itemDoubleClicked(QListWidgetItem*)), this, SLOT(addItem(QListWidgetItem*)));

  m_listToCompute = new QListWidget(this);
  // m_listToCompute->setMaximumWidth(200);
  // m_listToCompute->setMinimumWidth(200);
  // m_listToCompute->setMaximumHeight(100);
  connect(m_listToCompute, SIGNAL(itemDoubleClicked(QListWidgetItem*)), this, SLOT(editRegSetTitle(QListWidgetItem*)));

  QPushButton *add = new QPushButton(tr("->"), this);
  connect(add, SIGNAL(clicked()), this, SLOT(addItem()));
  QPushButton *del = new QPushButton(tr("<-"), this);
  connect(del, SIGNAL(clicked()), this, SLOT(deleteItem()));



  add->setMaximumWidth(25);
  del->setMaximumWidth(25);

  QVBoxLayout *buttonLayout = new QVBoxLayout;
  buttonLayout->addWidget(add);
  buttonLayout->addWidget(del);


  gridLayout->addWidget(m_list, Qt::AlignLeft);
  gridLayout->addLayout(buttonLayout, Qt::AlignLeft);
  gridLayout->addWidget(m_listToCompute, Qt::AlignLeft);
  gridLayout->addStretch();

  box->setLayout(gridLayout);

  layout->addWidget(box);

  setLayout(layout);

  loadPlugins();

}

void
RegionFeatureWidget::loadPlugins()
{
#ifndef NDEBUG
  std::cout << "Loading Region Feature Computer Plugins" << std::endl;
#endif
  QIsePluginList<RegionSetInterfacePlugin*>::iterator it = PluginRegistry::getRegionSetPlugin().begin();
  const QIsePluginList<RegionSetInterfacePlugin*>::iterator itEnd = PluginRegistry::getRegionSetPlugin().end();
  for (size_t i = 0; it != itEnd; ++it, ++i) {
#ifndef NDEBUG
      std::cout << "Loaded: " << (*it)->name().toStdString() << std::endl;
#endif
      QListWidgetItem *item = new QListWidgetItem((*it)->name());
      m_list->addItem(item);
#ifndef NDEBUG
      std::pair<PluginContainer::iterator, bool> insertOk =
#endif
          m_plugins.insert(std::make_pair(i, *it));
      assert(insertOk.second);
  }

  // RegionFeatureComputerPlugin *plugin = *(PluginRegistry::getRegionFeatureComputerPlugin().begin());
  // m_dialog = plugin->parameterWidget(m_segContext, this);

  /*
  m_dialogs.insert(std::pair<QString, RegionFeatureDialog*>(ifuwi->name(), ifuwi));
  QListWidgetItem *item = new QListWidgetItem(ifuwi->name());
  m_list->addItem(item);
   */
}


/*
void
RegionFeatureWidget::loadRegionSets()
{
  assert(m_segContext);
  std::vector<ise::URI> regSetUris;
  m_segContext->getRegionSetUris(regSetUris);
  std::vector<ise::URI>::const_iterator it = regSetUris.begin();
  const std::vector<ise::URI>::const_iterator itEnd = regSetUris.end();
  for (; it != itEnd; ++it) {

  }
}
 */


void
RegionFeatureWidget::buildRegionFeature()
{
  //  assert(m_segContext);
  //  for (int i = 0; i < m_listToCompute->count(); ++i) {
  //    QListWidgetItem *item = m_listToCompute->item(i);
  //    //    RegionFeatureComputerDialog *dialog =
  //    //        static_cast<RegionFeatureComputerDialog*>(m_listToCompute->itemWidget(item));
  //    MyItem *myItem = static_cast<MyItem*>(m_listToCompute->itemWidget(item));
  //    // R: is the following instruction necessary ?
  //    m_dialogs[myItem->name().toStdString()]->getGui();
  //    if (!m_segContext->regionSetExists(myItem->name().toStdString())) {
  //      girl::shared_ptr<ise::RegionFeatureComputer> featureComputer =
  //          m_dialogs[myItem->name().toStdString()]->getFeatureComputer();
  //
  //     // ise::RegionSetInterface *regSet = new ise::RegionSetInterface(*(featureComputer.get()));
  //      girl::shared_ptr<ise::RegionSetInterface> regSet =
  //          ise::RegionSetFactoryHandler::instance()
  //            ->newRegionSet(featureComputer);
  //
  //      m_segContext->addRegionSet(regSet);
  //
  //#ifndef NDEBUG
  //      std::cerr << "Added " << featureComputer->uri() << " Region set to seg context" << std::endl;
  //#endif
  //    }
  //  }
  //#ifndef NDEBUG
  //  m_segContext->checkAllRegions();
  //#endif
  //
  //  //    assert(dialog);
  //  //    girl::shared_ptr<ise::RegionFeatureComputer> featureComputer = dialog->getFeatureComputer();
  //  //    ise::RegionSetInterface *regSet = new ise::RegionSetInterface(*(featureComputer.get()));
  //  //    m_segContext->addRegionSet(regSet);
  //  //#ifndef NDEBUG
  //  //    std::cerr << "Added " << featureComputer->uri() << " Region set to seg context" << std::endl;
  //  //#endif
  //  //  }

}

void
RegionFeatureWidget::init(ise::SegmentationContext *segContext, QString name, QWidget *parent)
{
  m_segContext = segContext;
  m_name = name;
  this->setParent(parent);
}

void
RegionFeatureWidget::addItem()
{
  QListWidgetItem *item = m_list->currentItem();
  if (item)
    addItem(item);
  else
    QMessageBox::warning(this, tr("Erreur fatale pas grave!"), tr("Nothing to add, did you pick a region feature?"));
}

void
RegionFeatureWidget::addItem(QListWidgetItem* item)
{
  size_t index = m_list->row(item);
  assert(index < m_plugins.size());

  // create and display dialog for parameters settings
  QDialog *dialog = new QDialog(this);
  QVBoxLayout *layout = new QVBoxLayout;
  RegionSetInterfacePlugin *plugin = m_plugins[index];
  plugin->setSegContext(m_segContext);
  RegionSetInterfaceWidget *pluginWidgetParameters = plugin->parameterWidget(m_mainView, dialog);
  layout->addWidget(pluginWidgetParameters, 0, Qt::AlignCenter);
  QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok
      | QDialogButtonBox::Cancel);
  connect(buttonBox, SIGNAL(accepted()), dialog, SLOT(accept()));
  connect(buttonBox, SIGNAL(rejected()), dialog, SLOT(reject()));
  layout->addWidget(buttonBox);
  dialog->setLayout(layout);
  int returnCode = dialog->exec();

  if (returnCode == QDialog::Accepted) {
      // be sure to get settings from the widget
    //      girl::shared_ptr<RegionSetInterfaceSettings> settings = plugin->createSettings(m_segContext);
    // R: No border effect with the previous line, the settings used when instantiating the algo
    // are the default settings, because we create a setting but it is not the one stored
    // in the plugin
    girl::shared_ptr<RegionSetInterfaceSettings> settings = plugin->settings();

      pluginWidgetParameters->getGUI(settings);
      girl::shared_ptr<ise::RegionSetInterface> regSet = plugin->getNewRegionSet(m_segContext);
      if (!m_segContext->regionSetExists(regSet->uri())) {
          insertRegionSet(regSet);
          m_segContext->addRegionSet(regSet);
      } else {
          QString msg = (tr("There is already a region set doing the same shit!"));
//          girl::shared_ptr<ise::RegionSetInterface> theRegSet = m_segContext->getRegionSet(regSet->getTitle());
//          std::string theTitle = theRegSet->getTitle();
//          msg.append(theTitle.c_str());
//          msg.append(")(");
//          msg.append(regSet->getTitle().c_str());
//          msg.append(")");
          QMessageBox::warning(this, tr("Erreur fatale pas grave!"), msg);

      }

#if 0
      //#ifndef NDEBUG
      //      m_segContext->checkAllRegions();
#endif
  } else {
      std::cerr << "Parameter dialog was canceled! Not adding the region set" << std::endl;
  }



  //  MyItem *myItem = new MyItem(index, name, this);
  //  connect(myItem, SIGNAL(show(size_t)), this, SLOT(show(size_t)));
  //
  //  m_listToCompute->setItemWidget(item2, myItem);



}

void
RegionFeatureWidget::deleteItem()
{
  QListWidgetItem *item = m_listToCompute->currentItem();
  if (item) {
      QString uri = item->data(Qt::UserRole).toString();
      std::cerr << "Removing " << uri.toStdString() << " from segContext" << std::endl;
      m_segContext->removeRegionSet(uri.toStdString());
      delete item;
      //m_segContext->checkAllRegions();
  } else
    QMessageBox::warning(this, tr("Erreur fatale pas grave!"), tr("Nothing to remove, did you pick a region feature?"));
}


void
RegionFeatureWidget::editRegSetTitle(QListWidgetItem* item)
{

  assert(m_segContext);

  //QString title = QInputDialog::getText(this, tr("Change RegionSet Title"), tr(""), QLineEdit::Normal, item->text());

  //item->setText(title);
  // TODO: change regSet title;
  QString uri = item->data(Qt::UserRole).toString();

//  m_segContext->renameComputer(uri.toStdString(), title.toStdString());
 // item->setData(Qt::UserRole, QString(title));
  girl::shared_ptr<ise::RegionSetInterface> regSet = m_segContext->getRegionSet(uri.toStdString());
  g_count += rand()%3;
  regSet->setCacheState(static_cast<ise::RegionSetInterface::CacheType>(g_count%3));
  std::cout << "Setting cache to " << g_count%3 << std::endl;

  //  regSet->setTitle(title.toStdString());

  //  m_segContext->checkAllRegions();
}
void
RegionFeatureWidget::insertRegionSet(girl::shared_ptr<ise::RegionSetInterface> regSet)
{
  QListWidgetItem *item = new QListWidgetItem();
  // save the URI in the QListWidgetItem for future reuse (delete)
  item->setData(Qt::UserRole, QString(regSet->uri().c_str()));
  std::string title = regSet->uri();
  item->setText(QString(title.c_str()));
  m_listToCompute->addItem(item);
}
