/*
  Copyright (C) 2005-2010 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef REGIONSELECTIONBASEINTERACTOR_HPP
#define REGIONSELECTIONBASEINTERACTOR_HPP


#include <girl/girlToger.hpp>//regionDeque
#include <QColor>
#include <QPoint>
#include <qiseTools/Interactor.hpp>
#include <qiseTools/ShapeSelector.hpp>

class SegView;
class GLRectangleLayer;

class GIRL_API RegionSelectionBaseInteractor : public Interactor
{
  Q_OBJECT

public:

  /**
   * @brief Constructor
   *
   * @param v view to which will be attached drawn layer.
   * @param color color for layer
   * @param opacity opacity for layer
   */
  RegionSelectionBaseInteractor(SegView *v, QRgb color = 0, float opacity = 1.0f);

  /**
   * @brief Destructor
   */
  virtual ~RegionSelectionBaseInteractor();

  virtual void activate();

  virtual void deactivate();

  virtual bool eventFilter(QObject *target, QEvent *event);

  virtual void setColor(const QRgb &c);
  
  virtual void setOpacity(float o);

  void setSelector(ShapeSelector *s);
  
public slots:

  void changeSelectionMode(ShapeSelector::SelectorMode mode);

  void changeSelectionRegionComputer(ShapeSelector::SelectorRegionComputer rc);

  void changeSelectionType(ShapeSelector::SelectorType type);

protected slots:

  void changeColor(const QRgb &c);
  
  void changeAlpha(float a);
 
  void changeFillEnabled(bool b);

protected:

  SegView *m_view;
  bool m_selectionChanged;

  QColor m_color;
  float m_opacity;

  ShapeSelector *m_selector;
};


#endif //REGIONSELECTIONBASEINTERACTOR_HPP
