/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "RegionSetInterfaceChoiceCombo.hpp"

#include <vector>

#include <ise/iseSegmentationContext.hpp>

#include <QComboBox>
#include <QLabel>
#include <QBoxLayout>

RegionSetInterfaceChoiceCombo::RegionSetInterfaceChoiceCombo(ise::SegmentationContext *segContext, QWidget *parent)
 : QWidget(parent),
   m_segContext(segContext)
{
  buildGUI();

}

RegionSetInterfaceChoiceCombo::~RegionSetInterfaceChoiceCombo()
{

}

void
RegionSetInterfaceChoiceCombo::buildGUI()
{
  QLabel *label = new QLabel(tr("Region feature: "), this);
  m_combo = new QComboBox(this);
  QHBoxLayout *hl = new QHBoxLayout;
  hl->addWidget(label);
  hl->addWidget(m_combo);
  hl->setMargin(0); //because embedded widget

  QVBoxLayout *layout = new QVBoxLayout;
  layout->addLayout(hl);
  layout->addStretch(0);
  layout->setMargin(0); //because embedded widget
  this->setLayout(layout);

  assert(m_segContext);
  ise::FeatureComputer::FeatureComputerListType regSets;
  m_segContext->getRegionSetUris(regSets);
  ise::FeatureComputer::FeatureComputerListType::const_iterator it = regSets.begin();
  const ise::FeatureComputer::FeatureComputerListType::const_iterator itEnd = regSets.end();
  for ( ; it != itEnd; ++it) {
    girl::shared_ptr<ise::RegionSetInterface> regSet = m_segContext->getRegionSet(*it);
    std::string title = regSet->uri();
    m_combo->addItem(title.c_str());
  }

  connect(m_combo, SIGNAL(currentIndexChanged(int)),
	  this, SIGNAL(currentIndexChanged(int)));
  
}

void
RegionSetInterfaceChoiceCombo::getGUI(girl::shared_ptr<ise::RegionSetInterface>& regSet) const
{
  assert(m_combo);
  int index = m_combo->currentIndex();
  if (index != -1) {
    // R: TODO: This assume no region set was added in between?
    ise::FeatureComputer::FeatureComputerListType regSets;
    m_segContext->getRegionSetUris(regSets);
    assert(index < (int)regSets.size());
    ise::FeatureComputer::FeatureComputerListType::iterator it = regSets.begin();
    for (int i = 0; i < index; ++i)
      ++it;
    regSet = m_segContext->getRegionSet(*it);
  }
}

void
RegionSetInterfaceChoiceCombo::setGUI(const girl::shared_ptr<ise::RegionSetInterface> regSet)
{
  if (regSet) {//B:instead of assert
    std::string title = regSet->uri();
    QString tmp(title.c_str());
    assert(m_combo);
    int index = m_combo->findText(tmp);
    assert(index != -1);
    m_combo->setCurrentIndex(index);
  }
}

int
RegionSetInterfaceChoiceCombo::currentIndex() const
{
  assert(m_combo);
  return m_combo->currentIndex();
}
