/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
#ifndef REGIONSETINTERFACESETTINGS_HPP_
#define REGIONSETINTERFACESETTINGS_HPP_

#include <ise/iseRegionSetInterface.hpp>
namespace ise {
  class SegmentationContext;
}

#include "SettingsInterfaceTemplate.hxx"

class RegionSetInterfaceWidget;

/**
 * Subclass this class to store and retrieved algo-specific parameters from widget and algorithms.
 */
typedef SettingsInterfaceTemplate<RegionSetInterfaceWidget, ise::RegionSetInterface> RegionSetInterfaceSettingsBase;


class GIRL_API RegionSetInterfaceSettings : public RegionSetInterfaceSettingsBase
{

public:

  virtual ~RegionSetInterfaceSettings() {}

  /**
   * Since all RegionSetInterfaces have in common the fact that we can define the cache strategy, a bit
   * of abstract code is done here. The part which concerns the algorithm-specific code must be implemented
   * in the doXXXX functions which are called from the XXXX functions
   */
  void configureWidget(RegionSetInterfaceWidget *widget) const;

  void getWidgetConfig(const RegionSetInterfaceWidget *widget);

  void configureAlgo(girl::shared_ptr<ise::RegionSetInterface> algo) const;

  girl::shared_ptr<RegionSetInterfaceSettingsBase> clone() const;
  
  bool isValid() const;

protected:

  virtual void
  doConfigureWidget(RegionSetInterfaceWidget *) const = 0;

  virtual void
  doGetWidgetConfig(const RegionSetInterfaceWidget *) = 0;

  virtual void
  doConfigureAlgo(girl::shared_ptr<ise::RegionSetInterface>) const = 0;

  virtual girl::shared_ptr<RegionSetInterfaceSettingsBase> doClone() const = 0;

  virtual bool doIsValid() const = 0;

};


#endif /* REGIONSETINTERFACESETTINGS_HPP_ */
