/*
 * RegionSetParametersWidget.cpp
 *
 *  Created on: Oct 22, 2010
 *      Author: vieux
 */

#include "RegionSetParametersWidget.hpp"

#include "SliderSpinBoxWidget.hpp"

#include <QBoxLayout>
#include <QGridLayout>
#include <QLabel>
#include <QPushButton>
#include <QScrollArea>

RegionSetParametersWidget::RegionSetParametersWidget(girl::shared_ptr<ise::RegionSetInterface> regSet,
						     ise::RegionSetParameters &params,
						     QWidget *parent)
  : QWidget(parent),
    m_regSet(regSet),
    m_params(params),
    m_sliders(),
    m_groupBox(NULL)
{
  buildGUI();
  setGUI();
}

RegionSetParametersWidget::~RegionSetParametersWidget()
{
}

void
RegionSetParametersWidget::buildGUI()
{
  assert(m_regSet.get());

  std::string title;
  std::vector<std::string> dimDesc;
  m_regSet->getFeatureDescription(title, dimDesc);
  QString qTitle(title.c_str());
  m_groupBox = new QGroupBox(qTitle, this);


  QGridLayout *gl = new QGridLayout;
  std::vector<std::string>::const_iterator it = dimDesc.begin();
  const std::vector<std::string>::const_iterator itEnd = dimDesc.end();
  for (int i=0; it != itEnd; ++it, ++i) {
    QLabel * l = new QLabel(QString(it->c_str()), this);    
    SliderSpinBoxWidget *s = new SliderSpinBoxWidget(this);
    s->setMinimum(0.0);
    s->setMaximum(1.0);
    s->setDecimals(2);
    s->setValue(1.0);

    m_sliders.push_back(s);

    gl->addWidget(l, i, 0);
    gl->addWidget(s, i, 1);

  }

  QScrollArea *scroll = new QScrollArea(this);
  QWidget *scrollWidget = new QWidget(this);
  scrollWidget->setLayout(gl);
  scroll->setWidget(scrollWidget);
  scroll->setWidgetResizable(true);
  scroll->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);


  QPushButton *setAll0Button = new QPushButton(tr("Set all to 0"), this);
  connect(setAll0Button, SIGNAL(clicked()),
	  this, SLOT(setAll0()));
  QPushButton *setAll1Button = new QPushButton(tr("Set all to 1"), this);
  connect(setAll1Button, SIGNAL(clicked()),
	  this, SLOT(setAll1()));
  

  QHBoxLayout *lh = new QHBoxLayout;
  lh->addWidget(setAll0Button);
  lh->addWidget(setAll1Button);
  
  QBoxLayout *vl = new QVBoxLayout;
  vl->addLayout(lh);
  vl->addWidget(scroll);
  m_groupBox->setLayout(vl);

  QVBoxLayout *layout = new QVBoxLayout;
  layout->addWidget(m_groupBox);
  this->setLayout(layout);
}

void
RegionSetParametersWidget::setGUI()
{
  const std::vector<double> weights = m_params.getDimWeights();
  assert(weights.size() == m_regSet->getFeatureDim());
  assert(((int)weights.size()) == m_sliders.size());

  std::vector<double>::const_iterator wIt = weights.begin();
  const std::vector<double>::const_iterator wItEnd = weights.end();
  QList<SliderSpinBoxWidget*>::iterator lIt = m_sliders.begin();
  for (size_t i = 0; wIt != wItEnd; ++wIt, ++lIt, ++i) {
    (*lIt)->setValue(*wIt);
  }

}

void
RegionSetParametersWidget::getGUI()
{
  std::vector<double> weights;
  QList<SliderSpinBoxWidget*>::const_iterator lIt = m_sliders.begin();
  const QList<SliderSpinBoxWidget*>::const_iterator lItEnd = m_sliders.end();
  for (size_t i = 0; lIt != lItEnd; ++lIt, ++i) {
    weights.push_back((*lIt)->value());
  }
  m_params.setDimWeights(weights);
}

void
RegionSetParametersWidget::setAll(double v)
{
  QList<SliderSpinBoxWidget*>::const_iterator lIt = m_sliders.begin();
  const QList<SliderSpinBoxWidget*>::const_iterator lItEnd = m_sliders.end();
  for (size_t i = 0; lIt != lItEnd; ++lIt, ++i) {
    (*lIt)->setValue(v);
  }
}

void
RegionSetParametersWidget::setAll0()
{
  setAll(0.0);
}

void
RegionSetParametersWidget::setAll1()
{
  setAll(1.0);
}
