/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef REGIONSTRICTWEAKORDERINGPLUGIN_HPP_
#define REGIONSTRICTWEAKORDERINGPLUGIN_HPP_

#include <ise/iseRegionStrictWeakOrdering.hpp>
#include <ise/iseSegmentationContext.hpp>

#include "QIsePlugin.hpp"
#include "RegionStrictWeakOrderingWidget.hpp"
#include "RegionStrictWeakOrderingSettings.hpp"
#include "MainView.hpp"

/**
 * Interface for plugins which implements a RegionBinaryFunction
 *
 */
class GIRL_API RegionStrictWeakOrderingPlugin : public QIsePlugin,
					     public ise::RegionStrictWeakOrderingFactory
{

 public:

  virtual ~RegionStrictWeakOrderingPlugin() {}

  virtual girl::shared_ptr<ise::RegionStrictWeakOrdering>
  getNewRegionStrictWeakOrdering(ise::SegmentationContext *segContext)
  {
    girl::shared_ptr<ise::RegionStrictWeakOrdering> p = create(segContext);
    girl::shared_ptr<RegionStrictWeakOrderingSettings> s = settings();
    if (s)
      s->configureAlgo(p);
    return p;
  }
  
  virtual RegionStrictWeakOrderingWidget *parameterWidget(MainView *mainView, QWidget *parent = NULL)
  {
    RegionStrictWeakOrderingWidget *w = createWidget(mainView/*m_segContext*/, parent);
    girl::shared_ptr<RegionStrictWeakOrderingSettings> s = settings();
    if (s)
      s->configureWidget(w);
    return w;
  }


  girl::shared_ptr<RegionStrictWeakOrderingSettings>
  settings()
  {
    if (!m_settings)
      m_settings = createSettings(m_segContext);
    return m_settings;
  }

  void
  setSettings(girl::shared_ptr<RegionStrictWeakOrderingSettings> settings)
  {
    m_settings = settings;
  }
 

  inline void setSegContext(ise::SegmentationContext* segContext)
  {
    m_segContext = segContext;
  }

  inline ise::SegmentationContext *getSegContext()
  {
    return m_segContext;
  }

  virtual girl::shared_ptr<ise::RegionStrictWeakOrdering> 
    create(ise::SegmentationContext *segContext) = 0;

  virtual girl::shared_ptr<RegionStrictWeakOrderingSettings> createSettings(ise::SegmentationContext *segContext) = 0;  

  virtual bool hasSettings() const
  {
    return true;
  }


 protected:

  RegionStrictWeakOrderingPlugin(const QString &name,
			      const QIseVersion &pluginVersion,
			      const QIseVersion &qiseVersion,
			      QString path = QString::fromUtf8(""))
    : QIsePlugin(name, (QString::fromUtf8("RegionStrictWeakOrdering/") + path),
		 pluginVersion, qiseVersion),
    m_segContext(NULL)
      {}

  virtual RegionStrictWeakOrderingWidget *
    createWidget(MainView *mainView/*ise::SegmentationContext *segContext*/, QWidget *parent) = 0;
  


 protected:
  girl::shared_ptr<RegionStrictWeakOrderingSettings> m_settings;
  ise::SegmentationContext* m_segContext;
};

#define RegionStrictWeakOrderingPlugin_iid "org.girl.qise.RegionStrictWeakOrderingPlugin/1.0"
Q_DECLARE_INTERFACE(RegionStrictWeakOrderingPlugin, RegionStrictWeakOrderingPlugin_iid)

#endif /* REGIONSTRICTWEAKORDERINGPLUGIN_HPP_ */
