/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux boris.mansencal@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
#include "RegionStrictWeakOrderingWithUnaryFunctionsSettings.hpp"

#include "RegionStrictWeakOrderingWithUnaryFunctionsWidget.hpp"



void
RegionStrictWeakOrderingWithUnaryFunctionsSettings::configureWidget(RegionStrictWeakOrderingWidget *widget) const
{
  RegionStrictWeakOrderingWithUnaryFunctionsWidget *w = static_cast<RegionStrictWeakOrderingWithUnaryFunctionsWidget *>(widget);

  assert(m_plugins.size() == m_settings.size());

  UnaryFctPluginCollection::const_iterator itP = m_plugins.begin();
  const UnaryFctPluginCollection::const_iterator itPEnd = m_plugins.end();
  UnaryFctSettingsCollection::const_iterator itS = m_settings.begin();

  for ( ; itP != itPEnd; ++itP, ++itS) {
    RegionUnaryFunctionPlugin *p = *itP;
    assert(p);
    RegionUnaryFunctionSettingsPtr s = *itS;
    p->setSettings(s);
  }

  w->setRegionUnaryFuntionPlugins(m_plugins);
}


void
RegionStrictWeakOrderingWithUnaryFunctionsSettings::getWidgetConfig(const RegionStrictWeakOrderingWidget *widget)
{
  const RegionStrictWeakOrderingWithUnaryFunctionsWidget *w = static_cast<const RegionStrictWeakOrderingWithUnaryFunctionsWidget *>(widget);

  
  m_plugins.clear();
  m_settings.clear();
  
  w->getRegionUnaryFuntionPlugins(m_plugins);
    
  if (! m_plugins.empty()) {

    ise::SegmentationContext *ctxt = (*m_plugins.begin())->getSegContext();

    UnaryFctPluginCollection::iterator itP = m_plugins.begin();
    const UnaryFctPluginCollection::iterator itPEnd = m_plugins.end(); 
    for ( ; itP != itPEnd; ++itP) {
      RegionUnaryFunctionPlugin *p = *itP;
      assert(p);
      RegionUnaryFunctionSettingsPtr s = p->createSettings(ctxt);
      m_settings.push_back(s);
    }
  }

  assert(m_plugins.size() == m_settings.size());
}


void
RegionStrictWeakOrderingWithUnaryFunctionsSettings::configureAlgo(girl::shared_ptr<ise::RegionStrictWeakOrdering> algo) const
{
  girl::shared_ptr<ise::RegionStrictWeakOrderingWithUnaryFunctions> p =
    girl::static_pointer_cast<ise::RegionStrictWeakOrderingWithUnaryFunctions>(algo);

  typedef ise::RegionStrictWeakOrderingWithUnaryFunctions::UnaryFctPtr UnaryFctPtr;
  typedef ise::RegionStrictWeakOrderingWithUnaryFunctions::UnaryFctCollection UnaryFctCollection;

  UnaryFctCollection unaryFcts;

  assert(m_plugins.size() == m_settings.size());
  
  if (! m_plugins.empty()) {

    UnaryFctPluginCollection::const_iterator itP = m_plugins.begin();
    const UnaryFctPluginCollection::const_iterator itPEnd = m_plugins.end();
    UnaryFctSettingsCollection::const_iterator itS = m_settings.begin();

    ise::SegmentationContext *ctxt = (*m_plugins.begin())->getSegContext();

    for ( ; itP != itPEnd; ++itP, ++itS) {
      RegionUnaryFunctionPlugin *p = *itP;
      assert(p);
      RegionUnaryFunctionSettingsPtr s = *itS;
      p->setSettings(s);

      UnaryFctPtr ufp = p->getNewRegionUnaryFunction(ctxt);
      unaryFcts.push_back(ufp);
    }

  }  

  p->setParameters(unaryFcts);
}

girl::shared_ptr<RegionStrictWeakOrderingSettings>
RegionStrictWeakOrderingWithUnaryFunctionsSettings::clone() const
{
  girl::shared_ptr<RegionStrictWeakOrderingWithUnaryFunctionsSettings> p(new RegionStrictWeakOrderingWithUnaryFunctionsSettings);

  *p = *this;

  p->m_settings.clear();
  UnaryFctSettingsCollection::const_iterator it = m_settings.begin();
  const UnaryFctSettingsCollection::const_iterator itEnd = m_settings.end();
  for ( ; it != itEnd; ++it) {
    if ( (*it) )
      p->m_settings.push_back( (*it)->clone() );
    else
      p->m_settings.push_back( RegionUnaryFunctionSettingsPtr() );
  }
  return p;
}


bool
RegionStrictWeakOrderingWithUnaryFunctionsSettings::isValid() const
{
  UnaryFctPluginCollection::const_iterator itP = m_plugins.begin();
  UnaryFctPluginCollection::const_iterator itPEnd = m_plugins.end();
  for ( ; itP != itPEnd; ++itP) {
    if ( (*itP) == NULL)
      return false;
  }

  UnaryFctSettingsCollection::const_iterator it = m_settings.begin();
  const UnaryFctSettingsCollection::const_iterator itEnd = m_settings.end();
  for ( ; it != itEnd; ++it) {
    if ( (*it) == girl::shared_ptr<RegionUnaryFunctionSettings>() || ! (*it)->isValid() )
      return false;
  }
  
  return true;
}
