/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef REGIONUNARYFUNCTIONPLUGIN_HPP_
#define REGIONUNARYFUNCTIONPLUGIN_HPP_

#include <girl/girlSharedPtr.hpp>
#include <ise/iseRegionUnaryFunction.hpp>

#include "MainView.hpp"

#include "QIsePlugin.hpp"
#include "SettingsInterfaceTemplate.hxx"


namespace ise {
  class SegmentationContext;
}

class RegionUnaryFunctionWidget; 


typedef SettingsInterfaceTemplate<RegionUnaryFunctionWidget, ise::RegionUnaryFunction> RegionUnaryFunctionSettings;

/**
 * Interface for plugins which implements a RegionUnaryFunction
 *
 */
class GIRL_API RegionUnaryFunctionPlugin : public QIsePlugin,
					   public ise::RegionUnaryFunctionFactory
{

 public:

  virtual ~RegionUnaryFunctionPlugin() {}

  girl::shared_ptr<ise::RegionUnaryFunction>
    getNewRegionUnaryFunction(ise::SegmentationContext *segContext)
  {
    girl::shared_ptr<ise::RegionUnaryFunction> function =
      create(segContext);
    girl::shared_ptr<RegionUnaryFunctionSettings> s = settings();
    if (s)
      s->configureAlgo(function);
    return function;
  }

  RegionUnaryFunctionWidget *parameterWidget(MainView *mainView, QWidget *parent = NULL)
  {
    RegionUnaryFunctionWidget *w = createWidget(mainView,  parent);
    girl::shared_ptr<RegionUnaryFunctionSettings> s = settings();
    if (s)
      s->configureWidget(w);
    return w;
  }

  girl::shared_ptr<RegionUnaryFunctionSettings>
    settings()
  {
    if (!m_settings)
      m_settings = createSettings(m_segContext);
    return m_settings;
  }

  void
    setSettings(girl::shared_ptr<RegionUnaryFunctionSettings> settings)
  {
    m_settings = settings;
  }


  inline void setSegContext(ise::SegmentationContext* segContext)
  {
    m_segContext = segContext;
  }

  inline ise::SegmentationContext *getSegContext()
  {
    return m_segContext;
  }

  virtual girl::shared_ptr<ise::RegionUnaryFunction>
    create(ise::SegmentationContext *segContext) = 0;

  virtual girl::shared_ptr<RegionUnaryFunctionSettings>
    createSettings(ise::SegmentationContext* segContext) = 0;

  virtual bool hasSettings() const
  {
    return true;
  }


 protected:

  RegionUnaryFunctionPlugin(const QString &name,
			    const QIseVersion &pluginVersion,
			    const QIseVersion &qiseVersion,
			    QString path = QString::fromUtf8(""))
    : QIsePlugin(name, (QString::fromUtf8("RegionUnaryFunction/") + path),
		 pluginVersion, qiseVersion),
    m_segContext(NULL)
      {}

  virtual RegionUnaryFunctionWidget*
    createWidget(MainView *mainView/*ise::SegmentationContext *segContext*/, QWidget *parent) = 0;

 protected:

  girl::shared_ptr<RegionUnaryFunctionSettings> m_settings;
  ise::SegmentationContext* m_segContext;

};

#define RegionUnaryFunctionPlugin_iid "org.girl.qise.RegionUnaryFunctionPlugin/1.0"
Q_DECLARE_INTERFACE(RegionUnaryFunctionPlugin, RegionUnaryFunctionPlugin_iid)

#endif /* REGIONUNARYFUNCTIONPLUGIN_HPP_ */
