/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
/*
 * RegionUnaryFunctionWithRegSetWeightWidget.hpp
 *
 *  Created on: Jan 4, 2011
 *      Author: vieux
 */

#ifndef REGIONUNARYFUNCTIONWITHREGSETWEIGHTWIDGET_HPP_
#define REGIONUNARYFUNCTIONWITHREGSETWEIGHTWIDGET_HPP_

#include "RegionUnaryFunctionWithRegSetWidget.hpp"
#include "WeightsSetupWidget.hpp"
#include "MainView.hpp"
#include <ise/iseWeightedRegionSetInterface.hpp>

/**
 * Widget to choose a Region Unary Function associated with a RegionSet and set weights
 * to the RegionSet dimensions
 */
class GIRL_API RegionUnaryFunctionWithRegSetWeightWidget : public RegionUnaryFunctionWithRegSetWidget
{

  Q_OBJECT

public:

  RegionUnaryFunctionWithRegSetWeightWidget(MainView *mainView,/*ise::SegmentationContext *segContext,*/ QWidget *parent);

  virtual
  ~RegionUnaryFunctionWithRegSetWeightWidget();

  void
   buildGUI();

   void
   setParameters(const girl::shared_ptr<ise::RegionSetInterface> regSet, const std::vector<double> &weights)
   {
     assert(m_combo);
     if (regSet) {
       ise::RegionSetInterfacePtr r = girl::static_pointer_cast<ise::WeightedRegionSetInterface>(regSet)->getRegSet();
       m_combo->setRegionFeatureComputerPtr(girl::static_pointer_cast<ise::FeatureComputer>(r));
       assert(m_weights);
       m_weights->setWeights(weights);
     }
   }

   void
   getParameters(girl::shared_ptr<ise::RegionSetInterface>& regSet, std::vector<double> &weights) const
   {
     assert(m_combo);
     assert(m_weights);
     m_weights->getWeights(weights);

     ise::RegionSetInterfacePtr r = girl::static_pointer_cast<ise::RegionSetInterface>(m_combo->regionFeatureComputerPtr());
     
     if (r) {
       ise::WeightedRegionSetInterfacePtr w = ise::WeightedRegionSetInterfacePtr(new ise::WeightedRegionSetInterface(r , weights));
     
       regSet = girl::static_pointer_cast<ise::RegionSetInterface>(w);
     }
     else {
       regSet = ise::RegionSetInterfacePtr();
     }
   }

   protected slots:

   void
   changeRegionSet(int);

 protected:

   WeightsSetupWidget *m_weights;
};

#endif /* REGIONUNARYFUNCTIONWITHREGSETWEIGHTWIDGET_HPP_ */
