/*
  Copyright (C) 2005-2010 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux remi.vieux@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef REGIONUNARYPREDICATEPLUGIN_HPP_
#define REGIONUNARYPREDICATEPLUGIN_HPP_

#include <ise/iseRegionUnaryPredicate.hpp>
#include <ise/iseSegmentationContext.hpp>

#include "QIsePlugin.hpp"
#include "RegionUnaryPredicateWidget.hpp"
#include "RegionUnaryPredicateSettings.hpp"
#include "MainView.hpp"


/**
 * Interface for plugins which implements a RegionUnaryFunction
 *
 */
class GIRL_API RegionUnaryPredicatePlugin : public QIsePlugin,
					    public ise::RegionUnaryPredicateFactory
{

public:

  virtual ~RegionUnaryPredicatePlugin() {}

  girl::shared_ptr<ise::RegionUnaryPredicate>
  getNewRegionUnaryPredicate(ise::SegmentationContext *segContext)
  {
    girl::shared_ptr<ise::RegionUnaryPredicate> p = create(segContext);
    girl::shared_ptr<RegionUnaryPredicateSettings> s = settings();
    if (s)
      s->configureAlgo(p);
    return p;
  }
  

  RegionUnaryPredicateWidget *parameterWidget(MainView *mainView, QWidget *parent = NULL)
  {
    RegionUnaryPredicateWidget *w = createWidget(mainView/*m_segContext*/, parent);
    girl::shared_ptr<RegionUnaryPredicateSettings> s = settings();
    if (s)
      s->configureWidget(w);
    return w;
  }

  girl::shared_ptr<RegionUnaryPredicateSettings>
  settings()
  {
    if (!m_settings)
      m_settings = createSettings(m_segContext);
    return m_settings;
  }

  void setSettings(girl::shared_ptr<RegionUnaryPredicateSettings> settings)
  {
    m_settings = settings;
  }
  

  inline void setSegContext(ise::SegmentationContext *segContext)
  {
    m_segContext = segContext;
  }

  inline ise::SegmentationContext *getSegContext()
  {
    return m_segContext;
  }


  virtual girl::shared_ptr<ise::RegionUnaryPredicate> 
    create(ise::SegmentationContext *segContext) = 0;

  virtual girl::shared_ptr<RegionUnaryPredicateSettings> 
    createSettings(ise::SegmentationContext *segContext) = 0;  

  virtual bool hasSettings() const
  {
    return true;
  }


protected:

  RegionUnaryPredicatePlugin(const QString &name,
      const QIseVersion &pluginVersion,
      const QIseVersion &qiseVersion,
      QString path = QString::fromUtf8(""))
  : QIsePlugin(name, (QString::fromUtf8("RegionUnaryPredicate/") + path),
      pluginVersion, qiseVersion),
      m_segContext(NULL)
  {}


  virtual RegionUnaryPredicateWidget *
    createWidget(MainView *mainView/*ise::SegmentationContext *segContext*/, QWidget *parent) = 0;
  

protected:
  girl::shared_ptr<RegionUnaryPredicateSettings> m_settings;
  ise::SegmentationContext *m_segContext;
};

#define RegionUnaryPredicatePlugin_iid "org.girl.qise.RegionUnaryPredicatePlugin/1.0"
Q_DECLARE_INTERFACE(RegionUnaryPredicatePlugin, RegionUnaryPredicatePlugin_iid)

#endif /* REGIONUNARYPREDICATEPLUGIN_HPP_ */
