/*
  Copyright (C) 2005-2011 Jean-Philippe Domenger, Boris Mansencal,
  Julian Roigt, Remi Vieux boris.mansencal@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "RegionUnaryPredicateThresholdWidget.hpp"

#include <qiseTools/ComparisonFunctionWidget.hpp>
#include <qiseTools/RegionUnaryFunctionPlugin.hpp>
//#include <qiseTools/RegionUnaryFunctionWidget.hpp>
#include <qiseTools/RegionUnaryFunctionComboWidget.hpp>

#include <QGridLayout>
#include <QLabel>
#include <QLineEdit>
#include <QDoubleValidator>




RegionUnaryPredicateThresholdWidget::RegionUnaryPredicateThresholdWidget(MainView *mainView/*ise::SegmentationContext *segContext*/, 
									 RegionUnaryPredicatePlugin *plugin, 
									 QWidget *parent)
  : RegionUnaryPredicateWidget(plugin, parent),
    //m_segContext(segContext),
    m_mainView(mainView),
    m_rufcw(NULL),
    m_cfw(NULL),
    m_threshold(NULL)
{
  buildGUI();
}


void
RegionUnaryPredicateThresholdWidget::buildGUI()
{
  
  QGridLayout *gL = new QGridLayout;

  m_rufcw = new RegionUnaryFunctionComboWidget(m_mainView/*m_segContext*/, this);

  QLabel *cfLabel = new QLabel(tr("Comparison operator: "), this);
  m_cfw = new ComparisonFunctionWidget(this);

  QLabel *thLabel = new QLabel(tr("Threshold: "), this);
  m_threshold = new QLineEdit(this);
  m_threshold->setValidator(new QDoubleValidator(m_threshold));

  connect(m_rufcw, SIGNAL(pluginChanged()), this, SIGNAL(pluginChanged()));
  connect(m_cfw, SIGNAL(comparisonOperatorChanged()), this, SIGNAL(comparisonOperatorChanged()));
  connect(m_threshold, SIGNAL(textChanged(const QString &)), this, SLOT(signalThresholdChanged()));


  gL->addWidget(m_rufcw, 0, 0, 1, 2);
  gL->addWidget(cfLabel, 2, 0);
  gL->addWidget(m_cfw, 2, 1);
  gL->addWidget(thLabel, 3, 0);
  gL->addWidget(m_threshold, 3, 1);
  
  setLayout(gL);

  /*
  connect(m_rufcw, SIGNAL(pluginChanged()),
	  this, SLOT(pluginChanged()));
  */
}



RegionUnaryFunctionPlugin *
RegionUnaryPredicateThresholdWidget::getRegionUnaryFunctionPlugin() const
{
  assert(m_rufcw);
  return m_rufcw->getRegionUnaryFunctionPlugin();
}

void
RegionUnaryPredicateThresholdWidget::setRegionUnaryFunctionPlugin(RegionUnaryFunctionPlugin *p)
{
  assert(m_rufcw);
  m_rufcw->setRegionUnaryFunctionPlugin(p);
}



ise::ComparisonOperator
RegionUnaryPredicateThresholdWidget::getComparisonOperator() const
{
  assert(m_cfw);
  return m_cfw->getComparisonOperator();
}

void
RegionUnaryPredicateThresholdWidget::setComparisonOperator(ise::ComparisonOperator c)
{
  assert(m_cfw);
  m_cfw->setComparisonOperator(c);
}

double
RegionUnaryPredicateThresholdWidget::getThreshold() const
{
  assert(m_threshold);
  return m_threshold->text().toDouble();
}    

void
RegionUnaryPredicateThresholdWidget::setThreshold(double th)
{
  const QString text = QString::number(th);
  if (text != m_threshold->text()) {
    m_threshold->setText(text);
  }
}

void
RegionUnaryPredicateThresholdWidget::signalThresholdChanged()
{
  emit thresholdChanged(getThreshold());
}


/*
void
RegionUnaryPredicateThresholdWidget::pluginChanged()
{
  if (m_histoView->visible()) {
    updateHistogram();
  }
  else {
    m_histoToUpdate = true;
  }
}


void
RegionUnaryPredicateThresholdWidget::updateHistogramVisibility(bool visible)
{
  if (visible != m_histoView->visible()) {
    
    if (visible && m_histoToUpdate) {
      
      updateHistogram();
    }
    
    setVisible(visible);
  }


}
 
void
RegionUnaryPredicateThresholdWidget::updateThreshold(const QString &s)
{
  const double newTh = s.toDouble();

  

}
  
void
RegionUnaryPredicateThresholdWidget::updateHistogram()
{
  //TODO: compute histo 

  RegionUnaryFunctionPlugin *p = getRegionUnaryFunctionPlugin();
  if (p != NULL) {
    girl::shared_ptr<ise::RegionUnaryFunction> f = p->getNewRegionUnaryFunction(p->getSegContext());
    
    //need regions !
  

  typedef ise::Histogram1D<ise::RegionFeature_Data_Type, size_t> HistoType;

  HistoType h;
  

  
    updateThreshold(m_threshold->text());

    m_histoToUpdate = false;

}
*/
