/*
  Copyright (C) 2005-2009 Jerome Charton, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "ScanlineSetConstExternalPerimeterIterator.hpp"

#include <cassert>
#include <limits>

#include "ScanlineSet.hpp"

const girl::point&
ScanlineSetConstExternalPerimeterIterator::operator*()
{
  return m_currentPoint;
}

ScanlineSetConstExternalPerimeterIterator&
ScanlineSetConstExternalPerimeterIterator::operator+=(unsigned int n)
{
  for (unsigned int i=0; i<n; ++i)
    ++(*this);
  return *this;
}

ScanlineSetConstExternalPerimeterIterator&
ScanlineSetConstExternalPerimeterIterator::operator++()
{
#ifndef NDEBUG
  const girl::point tmpCurrentPt = m_currentPoint;
#endif

  if (m_scanlineSet->isOverScanline(girl::point(m_currentPoint.x(), m_currentPoint.y()-1))
      || m_scanlineSet->isOverScanline(girl::point(m_currentPoint.x(), m_currentPoint.y()+1))
      || m_scanlineSet->isOverScanline(girl::point(m_currentPoint.x()+1, m_currentPoint.y()-1))
      || m_scanlineSet->isOverScanline(girl::point(m_currentPoint.x()+1, m_currentPoint.y()+1))) { // continue on current segment
    m_currentPoint.setX(m_currentPoint.x() + 1);
    while (m_scanlineSet->isOverScanline(m_currentPoint))
      m_currentPoint.setX(m_scanlineSet->m_data[m_currentPoint.y()-m_scanlineSet->m_offset][m_scanlineSet->getIndiceScanlineUnder(m_currentPoint)+1] + 1);
  }
  else { // go to next scanline
    int pairIndice, lineScanline;
    
    pairIndice = std::numeric_limits<int>::max();
    lineScanline = m_currentPoint.y()-m_scanlineSet->m_offset;
    
    unsigned int indiceUpScanline = m_scanlineSet->getIndiceScanlineAfter(girl::point(m_currentPoint.x(), m_currentPoint.y()-1));
    unsigned int indiceNextScanline = m_scanlineSet->getIndiceScanlineAfter(girl::point(m_currentPoint.x(), m_currentPoint.y()));
    unsigned int indiceDownScanline = m_scanlineSet->getIndiceScanlineAfter(girl::point(m_currentPoint.x(), m_currentPoint.y()+1));
    
    if (m_currentPoint.y()-1 - m_scanlineSet->m_offset > 0
	&& indiceUpScanline != m_scanlineSet->m_data[m_currentPoint.y()-m_scanlineSet->m_offset-1].size())
      if (pairIndice > (int)indiceUpScanline) {
	pairIndice = indiceUpScanline;
	lineScanline = m_currentPoint.y()-m_scanlineSet->m_offset-1;
      }
    
    if (m_currentPoint.y() - m_scanlineSet->m_offset > 0
	&& m_currentPoint.y() - m_scanlineSet->m_offset < (int)m_scanlineSet->m_data.size()
	&& indiceNextScanline != m_scanlineSet->m_data[m_currentPoint.y()-m_scanlineSet->m_offset].size())
      if (pairIndice > (int)indiceNextScanline) {
	pairIndice = indiceNextScanline;
	lineScanline = m_currentPoint.y()-m_scanlineSet->m_offset;
      }
    
    if (m_currentPoint.y()+1 - m_scanlineSet->m_offset < (int)m_scanlineSet->m_data.size()
	&& indiceDownScanline != m_scanlineSet->m_data[m_currentPoint.y()-m_scanlineSet->m_offset+1].size())
      if (pairIndice > (int)indiceDownScanline) {
	pairIndice = indiceDownScanline;
	lineScanline = m_currentPoint.y()-m_scanlineSet->m_offset+1;
      }
    
    if (m_currentPoint.y() == (int)m_scanlineSet->m_data.size()+1+m_scanlineSet->m_offset
	|| (pairIndice == std::numeric_limits<int>::max()
	    && m_currentPoint.y() == (int)m_scanlineSet->m_data.size()+m_scanlineSet->m_offset)) { // end iteration
      m_currentPoint.setY(m_scanlineSet->m_data.size()+1+m_scanlineSet->m_offset);
      m_currentPoint.setX(0);
    }
    else {

      int indexLastNotEmptyLine = lineScanline+1;

      if (pairIndice == std::numeric_limits<int>::max()) { // change line
	pairIndice=0;
	++lineScanline;
	m_currentPoint.setY(m_currentPoint.y()+1);
	
	while (lineScanline < (int)m_scanlineSet->m_data.size()
	      && m_scanlineSet->m_data[lineScanline].size() == 0) {
	  ++lineScanline;
	  m_currentPoint.setY(m_currentPoint.y()+1);
	}

	if (lineScanline < (int)m_scanlineSet->m_data.size()
	      && m_scanlineSet->m_data[lineScanline].size() != 0)
	  indexLastNotEmptyLine = lineScanline;
      }
      
      if (m_currentPoint.y() - m_scanlineSet->m_offset >= (int)m_scanlineSet->m_data.size())
	lineScanline = indexLastNotEmptyLine-1;
      
      assert(lineScanline >= 0 && lineScanline < (int)m_scanlineSet->m_data.size());
      assert(pairIndice >= 0 && pairIndice < (int)m_scanlineSet->m_data[lineScanline].size());

      m_currentPoint.setX(m_scanlineSet->m_data[lineScanline][pairIndice]-1);

      while (m_scanlineSet->isOverScanline(girl::point(m_currentPoint.x(), m_currentPoint.y()-1))
	     || m_scanlineSet->isOverScanline(girl::point(m_currentPoint.x(), m_currentPoint.y()+1)))
	m_currentPoint.setX(m_currentPoint.x()-1);
    }
  }
  
  assert(tmpCurrentPt != m_currentPoint);

  return *this;
}

void
ScanlineSetConstExternalPerimeterIterator::setBegin()
{
  if (m_scanlineSet->m_data.size() != 0) {
    size_t i = 0;
    while (i < m_scanlineSet->m_data.size()
	   && m_scanlineSet->m_data[i].size() == 0)
      ++i;
    
    if (i == m_scanlineSet->m_data.size()) // is empty
      setEnd();
    else
      m_currentPoint = girl::point(m_scanlineSet->m_data[i][0]-1, m_scanlineSet->m_offset-1);
  }
  else {
    m_currentPoint = girl::point(std::numeric_limits<girl::POINT_TYPE>::min(), std::numeric_limits<girl::POINT_TYPE>::min()); //setEnd
  }
}

void
ScanlineSetConstExternalPerimeterIterator::setEnd()
{
  if (m_scanlineSet->m_data.size() != 0) {
    m_currentPoint = girl::point(0, m_scanlineSet->m_data.size()+1+m_scanlineSet->m_offset);
  }
  else {
    m_currentPoint = girl::point(std::numeric_limits<girl::POINT_TYPE>::min(), std::numeric_limits<girl::POINT_TYPE>::min());
  }
}
