/*
  Copyright (C) 2005-2009 Jerome Charton, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "ScanlineSetConstLinesIterator.hpp"
#include <cassert>

#include "ScanlineSet.hpp"

ScanlineSetConstLinesIterator::scanline
ScanlineSetConstLinesIterator::operator*()
{
  const girl::POINT_TYPE y = m_lineIndice + m_scanlineSet->m_offset;
  return scanline(girl::point(*m_pairIterator, y),
		  girl::point(*(m_pairIterator+1), y));
}

ScanlineSetConstLinesIterator
ScanlineSetConstLinesIterator::operator+=(unsigned int n)
{
  for (unsigned int i=0; i<n; ++i)
    ++(*this);
  return *this;
}

ScanlineSetConstLinesIterator&
ScanlineSetConstLinesIterator::operator++()
{
  if (m_lineIndice < m_scanlineSet->m_data.size()) {
    if (m_pairIterator != m_scanlineSet->m_data[m_lineIndice].end()) {
      assert ((m_pairIterator+1) != m_scanlineSet->m_data[m_lineIndice].end());
      m_pairIterator+=2;
    }
    if (m_pairIterator == m_scanlineSet->m_data[m_lineIndice].end()) {
      ++m_lineIndice;
      if (m_lineIndice < m_scanlineSet->m_data.size()) {

	m_pairIterator = m_scanlineSet->m_data[m_lineIndice].begin();

	if (m_pairIterator == m_scanlineSet->m_data[m_lineIndice].end())
	  ++(*this);
      }
    }
  }
  return *this;
}

void
ScanlineSetConstLinesIterator::setBegin()
{
  if (m_scanlineSet->m_data.size() != 0) {
    m_lineIndice = 0;
    while (m_lineIndice < m_scanlineSet->m_data.size()
	   && m_scanlineSet->m_data[m_lineIndice].size() == 0)
      ++m_lineIndice;
    
    if (m_lineIndice == m_scanlineSet->m_data.size()) // is empty
      setEnd();
    else
      m_pairIterator = m_scanlineSet->m_data[m_lineIndice].begin();
  }
  else {
    m_lineIndice = static_cast<unsigned int>(-1); //setEnd();
  }
}

void
ScanlineSetConstLinesIterator::setEnd()
{
  if (m_scanlineSet->m_data.size() != 0) {
    m_lineIndice = m_scanlineSet->m_data.size();
    m_pairIterator = m_scanlineSet->m_data[m_lineIndice-1].end();
  }
  else
    m_lineIndice = static_cast<unsigned int>(-1);
}

bool
GIRL_API
operator==(const ScanlineSetConstLinesIterator &it1, const ScanlineSetConstLinesIterator &it2)
{
  return it1.m_scanlineSet == it2.m_scanlineSet
    && ((it1.m_lineIndice == it2.m_lineIndice
	 && it1.m_pairIterator == it2.m_pairIterator)
	|| (it1.m_lineIndice >= (unsigned int)it1.m_scanlineSet->size()
	    && it2.m_lineIndice >= (unsigned int)it2.m_scanlineSet->size()));
}

