/*
  Copyright (C) 2005-2009 Jerome Charton, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "ScanlineSetConstPerimeterIterator.hpp"

#include <cassert>
#include <limits>
#include <iostream>

#include "ScanlineSet.hpp"

const girl::point&
ScanlineSetConstPerimeterIterator::operator*()
{
  return m_currentPoint;
}

ScanlineSetConstPerimeterIterator&
ScanlineSetConstPerimeterIterator::operator+=(unsigned int n)
{
  for (unsigned int i=0; i<n; ++i)
    ++(*this);
  return *this;
}

ScanlineSetConstPerimeterIterator&
ScanlineSetConstPerimeterIterator::operator++()
{
  if (m_currentPoint.y() == (int)m_scanlineSet->m_data.size() + m_scanlineSet->m_offset) { // end iteration
    m_currentPoint.setY(m_scanlineSet->m_data.size()+m_scanlineSet->m_offset);
    m_currentPoint.setX(0);
    return *this;
  }
  
  m_currentPoint.setX(m_currentPoint.x() + 1);
  
  while (m_scanlineSet->isOverScanline(m_currentPoint)
	 && m_scanlineSet->isOverScanline(girl::point(m_currentPoint.x()+1, m_currentPoint.y()))
	 && m_scanlineSet->isOverScanline(girl::point(m_currentPoint.x(), m_currentPoint.y()-1))
	 && m_scanlineSet->isOverScanline(girl::point(m_currentPoint.x(), m_currentPoint.y()+1))
	 && m_scanlineSet->isOverScanline(girl::point(m_currentPoint.x()-1, m_currentPoint.y()+1))
	 && m_scanlineSet->isOverScanline(girl::point(m_currentPoint.x()-1, m_currentPoint.y()-1))
	 && m_scanlineSet->isOverScanline(girl::point(m_currentPoint.x()+1, m_currentPoint.y()+1))
	 && m_scanlineSet->isOverScanline(girl::point(m_currentPoint.x()+1, m_currentPoint.y()-1)))
    m_currentPoint.setX(m_currentPoint.x() + 1);
  
  if (!m_scanlineSet->isOverScanline(m_currentPoint)) { // go to next scanline
    unsigned int indiceNextScanline = m_scanlineSet->getIndiceScanlineAfter(m_currentPoint);
      
    if (indiceNextScanline != m_scanlineSet->m_data[m_currentPoint.y()-m_scanlineSet->m_offset].size())
      m_currentPoint.setX(m_scanlineSet->m_data[m_currentPoint.y()-m_scanlineSet->m_offset][indiceNextScanline]);
    else { // change line
      m_currentPoint.setY(m_currentPoint.y()+1);
      while (m_currentPoint.y()-m_scanlineSet->m_offset < (int)m_scanlineSet->m_data.size()
	    && m_scanlineSet->m_data[m_currentPoint.y()-m_scanlineSet->m_offset].size() == 0)
	m_currentPoint.setY(m_currentPoint.y()+1);
      
      if (m_currentPoint.y()-m_scanlineSet->m_offset == (int)m_scanlineSet->m_data.size()) { // end iteration
	m_currentPoint.setY(m_scanlineSet->m_data.size() + m_scanlineSet->m_offset);
	m_currentPoint.setX(0);
      }
      else
	m_currentPoint.setX(m_scanlineSet->m_data[m_currentPoint.y()-m_scanlineSet->m_offset][0]);
    }
  }
  
  return *this;
}

void
ScanlineSetConstPerimeterIterator::setBegin()
{
  unsigned int i = 0;
  while (i < m_scanlineSet->m_data.size() && m_scanlineSet->m_data[i].size() == 0)
    ++i;
  if (i != m_scanlineSet->m_data.size())
    m_currentPoint = girl::point(m_scanlineSet->m_data[i][0], m_scanlineSet->m_offset+i);
  else
    setEnd();
}

void
ScanlineSetConstPerimeterIterator::setEnd()
{
  m_currentPoint = girl::point(0, m_scanlineSet->m_data.size() + m_scanlineSet->m_offset);
}
