/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 */

#include "SegModel.hpp"

#include <algorithm>

#include <ise/iseRegionTools.hpp>
#include <ise/iseImageFeatureComputerDefault.hpp>

#include <girlTools/girlToolsMemoryBackup.hpp>
#include <girlTools/girlToolsZBackup.hpp>

#include "FeatureComputerGUIInfo.hpp"
#include "ImageTools.hpp"

#include <iostream> //DEBUG
#include <cassert>


SegModel::SegModel()
: m_segContext(),
  m_image(),
  m_qimage(),
  m_toger(),
  m_eps(1, 1),
  // m_titleToAlgo(),
  //   m_regSet(),
  m_featureComputerGUIInfo(this),
  m_selectionType(REGION),
  m_selectedRegions(),
  m_selectedDarts(),
  m_modified(false),
  m_togerNotNull(false),
  m_epsToUpdate(true)

{
  m_segContext.setImageToger(&m_image, &m_toger);

  m_toger.addObserver(this);
}

int 
SegModel::width() const
{
  return std::max(m_qimage.width(), static_cast<int>(m_toger.width()));
}

int 
SegModel::height() const
{
  return std::max(m_qimage.height(), static_cast<int>(m_toger.height()));
}


const QImage &
SegModel::getQImage() const
{
  //std::cerr<<"SegModel::getQImage() &m_qimage="<<&m_qimage<<std::endl;

  return m_qimage;
}

#include <girl/girlTimer.hpp> //DEBUG!!!

void 
SegModel::setQImage(const QImage &img)
{
  //std::cerr<<"SegModel::setQImage"<<std::endl;
  girl::Timer timer2; //DEBUG!!!
  timer2.start();

  if (! img.isNull()) {
      const QImage::Format f = img.format();
      if (f == QImage::Format_RGB32 || f == QImage::Format_ARGB32) {
        m_qimage = img;
      }
      else {
	std::cerr<<"SegModel::setQImage convertToFormat(QImage::Format_RGB32)"<<std::endl; //DEBUG!!!
        m_qimage = img.convertToFormat(QImage::Format_RGB32);
	
      }
      
      girl::tools::Image gtImg;
      girl::Timer timer; //DEBUG!!!
      timer.start();
      QImage_to_girlToolsImage(m_qimage, gtImg);
      const size_t dbgTime = timer.stop();
      std::cerr<<"SegModel::setQImage QImage_to_girlToolsImage="<<dbgTime<<"ms"<<std::endl;
      m_image = gtImg;

      timer.restart();
      m_featureComputerGUIInfo.loadDefaultFeature();
      const size_t dbgTime2 = timer.stop();
      std::cerr<<"SegModel::setQImage m_featureComputerGUIInfo.loadDefaultFeature()="<<dbgTime2<<"ms"<<std::endl;
  }
  else {
      m_image = girl::tools::Image();
      m_qimage = QImage();
  }
  std::cerr<<"SegModel::setQImage before emit="<<timer2.getElapsedTime()<<"ms"<<std::endl;
  size_t dbgTime3 = timer2.getElapsedTime();

  emit imageChanged(this);

  std::cerr<<"SegModel::setQImage emit="<<timer2.getElapsedTime()-dbgTime3<<"ms"<<std::endl;
  std::cerr<<"SegModel::setQImage after emit="<<timer2.getElapsedTime()<<"ms"<<std::endl;
  dbgTime3 = timer2.getElapsedTime();

  resetToger();

  std::cerr<<"SegModel::setQImage time resetToger()="<<timer2.getElapsedTime()-dbgTime3<<std::endl;

  size_t dbgTime2 = timer2.stop();
  std::cerr<<"SegModel::setQImage total="<<dbgTime2<<"ms"<<std::endl;

}

void
SegModel::setGirlToolsImage(const girl::tools::Image &img)
{
  if (! img.isNull()) {
      QImage qimage;
      girlToolsImage_to_QImage(img, qimage);
      m_image = img;
      m_qimage = qimage;

      m_featureComputerGUIInfo.loadDefaultFeature();
  }
  else {
      m_image = girl::tools::Image();
      m_qimage = QImage();
  }


  emit imageChanged(this);

  resetToger();
}

/*
static 
void 
L_new_region_hook(girl::region regionId, void *data)
{
  assert(data);
  ise::SegmentationContext *segContext = (ise::SegmentationContext *)data;

  ise::RegionColorimetricFeatures *regionF = iseRegionTools_createRegionFeatures(*segContext->getToger(), *segContext->getImage(), regionId);
  segContext->getRegionSet()->replaceRegion(regionF, regionId); //not RegSet::insertRegion() !
}

static 
void 
L_delete_region_hook(girl::region regionId, void * data)
{
  assert(data);
  ise::SegmentationContext *segContext = (ise::SegmentationContext *)data;
  segContext->getRegionSet()->deleteRegion(regionId);
}

static 
void 
L_merge_regions_hook(girl::region keptRegionId, girl::region delRegionId, void * data)
{
  assert(data);
  ise::SegmentationContext *segContext = (ise::SegmentationContext *)data;
  assert(segContext->getRegionSet());

  ise::RegionColorimetricFeatures *keptRegionF = segContext->getRegionSet()->getRegionFeatures(keptRegionId);
  ise::RegionColorimetricFeatures *delRegionF = segContext->getRegionSet()->getRegionFeatures(delRegionId);

  keptRegionF->merge2RegionFeatures(delRegionF);
}

static
void
L_rename_region_hook(girl::region oldRegionId, girl::region newRegionId, void * data)
{
  assert(data);
  ise::SegmentationContext *segContext = (ise::SegmentationContext *)data;
  assert(segContext->getRegionSet());

  assert(segContext->getRegionSet()->getRegionFeatures(newRegionId) == NULL);

  ise::RegionColorimetricFeatures *oldRegionF = segContext->getRegionSet()->getRegionFeatures(oldRegionId);
  assert(oldRegionF != NULL);

  segContext->getRegionSet()->insertRegion(oldRegionF, newRegionId);

  segContext->getRegionSet()->removeRegion(oldRegionId); //remove just id 
}
 */

void 
SegModel::resetToger()
{
  // bool oneRegionToger = false;

  if (! m_image.isNull()) {

      if (m_togerNotNull == false
          || m_toger.width() != m_image.width()
          || m_toger.height() != m_image.height()) {

          m_toger = girl::toger((girl::POINT_TYPE)m_image.width(),
              (girl::POINT_TYPE)m_image.height());
          m_togerNotNull = true;
          //oneRegionToger = true;
      }

  }

  //fillRegSet();
  // R: Remove from SegContext interface
  // m_segContext.setImageRegionSet(&m_image, &m_regSet);
  m_segContext.setImage(&m_image);


  m_selectedRegions.clear();
  m_selectedDarts.clear();


  //if (m_selectionType == REGION && oneRegionToger)
  //  m_selectedRegions.push_back(1); //automatically select region 1 if only one region

  m_epsToUpdate = true;

  m_toger.addObserver(this);

  emit togerChanged(this);
  emit selectionChanged(this);
}

void
SegModel::setToger(const girl::toger *tog)
{
  m_selectedRegions.clear();
  m_selectedDarts.clear();

  //girl::regionDeque regions;


  if (tog != NULL) {

      ///tog->allRegions(regions);
      //std::cerr<<"loaded tog : "<<regions.size()<<" regions"<<std::endl;

      //assert(m_segContext.getToger() == &m_toger);

      m_toger = *tog;

      ///assert(m_segContext.getToger() == &m_toger);
      //regions.clear();
      //m_toger.allRegions(regions);
      //std::cerr<<"set tog : "<<regions.size()<<" regions"<<std::endl;


      m_togerNotNull = true;

      assert(m_image.isNull() || (m_toger.width() == m_image.width() && m_toger.height() == m_image.height()));
  }
  else {

      if (m_image.isNull()) {
          m_toger = girl::toger();
          m_togerNotNull = false;
      }
      else {
          m_toger = girl::toger(m_image.width(), m_image.height());
          m_togerNotNull = true;
      }
  }

  ///assert(m_segContext.getToger() == &m_toger);
  // assert(m_segContext.getRegionSet() == &m_regSet);

  //fillRegSet();
  m_segContext.setToger(&m_toger);

  assert(m_segContext.getToger() == &m_toger);
  //  assert(m_segContext.getRegionSet() == &m_regSet);

  m_toger.addObserver(this);


  m_epsToUpdate = true;

  //regions.clear();
  //m_toger.allRegions(regions);
  //std::cerr<<"set tog : "<<regions.size()<<" regions"<<std::endl;

  emit togerChanged(this);
  emit selectionChanged(this);

}


void
SegModel::fillRegSet()
{
  // R: Remove from SegContext interface
  // m_segContext.setImageRegionSet(&m_image, &m_regSet);
  m_segContext.setImage(&m_image);

  /*

  m_regSet.reset();

  girl::regionDeque regs;
  m_toger.allRegions(regs);

  //std::cerr<<"fillRegSet "<<regs.size()<<" regions"<<std::endl;

  if (! m_image.isNull()) {

    for (girl::regionDeque::const_iterator it = regs.begin();
	 it != regs.end(); ++it)
      L_new_region_hook(*it, &m_segContext);


    //DEBUG
#ifndef NDEBUG
    regs.clear();
    m_toger.allRegions(regs);
    std::cerr<<"fillRegSetB "<<regs.size()<<" regions"<<std::endl;
#endif

    m_toger.setNewRegionHook(L_new_region_hook, &m_segContext);

    //DEBUG
#ifndef NDEBUG
    regs.clear();
    m_toger.allRegions(regs);
    std::cerr<<"fillRegSetC1 "<<regs.size()<<" regions"<<std::endl;
#endif


    m_toger.setDeleteRegionHook(L_delete_region_hook, &m_segContext);
    m_toger.setMergeRegionsHook(L_merge_regions_hook, &m_segContext);
    m_toger.setRenameRegionHook(L_rename_region_hook, &m_segContext);
  }
   */
}

const girl::toger *
SegModel::getToger() const
{
  return m_segContext.getToger();
}

girl::toger *
SegModel::getToger()
{
  return m_segContext.getToger();
}

const girl::tools::Image &
SegModel::getImage() const
{
  return *(m_segContext.getImage());
}


const girl::euclideanPathSet &
SegModel::getEPS() const
{
  if (m_epsToUpdate) {
      m_eps = girl::euclideanPathSet(m_toger.width(), m_toger.height());
      m_eps.initFromAllRegions(m_toger);
      m_epsToUpdate = false;
  }
  return m_eps;
}


/*
ise::SegmentationContext
SegModel::getSegContext() const
{
  return m_segContext;
}
 */

void 
SegModel::setSegContext(const ise::SegmentationContext &segContext)
{
  m_segContext = segContext;

  m_epsToUpdate = true;

  emit togerChanged(this);
}

ise::SegmentationContext *
SegModel::getSegContextPtr()
{
  return &m_segContext;
}


SegModel::SelectionType 
SegModel::getSelectionType() const
{
  return m_selectionType;
}

const girl::regionDeque &
SegModel::getSelectedRegions() const
{
#ifndef NDEBUG
  {
    girl::regionDeque::const_iterator it = m_selectedRegions.begin();
    const girl::regionDeque::const_iterator itEnd = m_selectedRegions.end();
    for ( ; it != itEnd; ++it) {
        assert(m_toger.isRegionValid(*it));
    }
  }
#endif

  return m_selectedRegions;
}

const girl::dartDeque &
SegModel::getSelectedDarts() const
{
  return m_selectedDarts;
}

void SegModel::setSelectionType(SelectionType t)
{
  if (t != m_selectionType) {
      m_selectionType = t;
      emit selectionTypeChanged(this);
  }
}


void
SegModel::setSelectedRegions(const girl::regionDeque &regions)
{
#ifndef NDEBUG
  {
    girl::regionDeque::const_iterator it = regions.begin();
    const girl::regionDeque::const_iterator itEnd = regions.end();
    for ( ; it != itEnd; ++it) {
        assert(m_toger.isRegionValid(*it));
    }
  }
#endif

  m_selectedRegions = regions;

  if (m_selectionType == REGION)
    emit selectionChanged(this);
}

void
SegModel::setSelectedDarts(const girl::dartDeque &darts)
{
  m_selectedDarts = darts;
  if (m_selectionType == SEGMENT)
    emit selectionChanged(this);
}

void
SegModel::clearSelection()
{
  if (m_selectionType == REGION) {
      m_selectedRegions.clear();
  }
  else {
      m_selectedDarts.clear();
  }
  emit selectionChanged(this);
}


bool
SegModel::isRegionSelected(girl::region r) const
{
  if ( std::find(m_selectedRegions.begin(),
      m_selectedRegions.end(),
      r) != m_selectedRegions.end() )
    return true;
  return false;
}

void 
SegModel::addRegionToSelection(girl::region r)
{
  if (! isRegionSelected(r)) {
      m_selectedRegions.push_back(r);

      if (m_selectionType == REGION)
        emit selectionChanged(this);
  }
}

void 
SegModel::removeRegionFromSelection(girl::region r)
{
  girl::regionDeque::iterator it =   std::find(m_selectedRegions.begin(),
      m_selectedRegions.end(),
      r);
  if (it != m_selectedRegions.end()) {
      m_selectedRegions.erase(it);

      if (m_selectionType == REGION)
        emit selectionChanged(this);
  }


}


bool
SegModel::isDartSelected(girl::dart d) const
{
  if ( std::find(m_selectedDarts.begin(),
      m_selectedDarts.end(),
      d) != m_selectedDarts.end() )
    return true;
  return false;
}

void 
SegModel::addDartToSelection(girl::dart d)
{
  m_selectedDarts.push_back(d);
  if (m_selectionType == SEGMENT)
    emit selectionChanged(this);
}

void 
SegModel::removeDartFromSelection(girl::dart d)
{
  girl::dartDeque::iterator it =   std::find(m_selectedDarts.begin(),
      m_selectedDarts.end(),
      d);
  if (it != m_selectedDarts.end()) {
      m_selectedDarts.erase(it);
      if (m_selectionType == SEGMENT)
        emit selectionChanged(this);
  }

}

bool 
SegModel::isModified() const
{
  return m_modified;
}

void 
SegModel::setModified(bool m)
{
  m_modified = m;
  if (m) {
      m_epsToUpdate = true;
      emit togerChanged(this);
  }
}

bool
SegModel::selectable() const
{
  return m_togerNotNull;
}

bool
SegModel::splittable() const
{
  return selectable() && (! m_image.isNull());
}

bool
SegModel::mergeable() const
{
  return splittable() && (m_toger.numRegions() > 1);
}

FeatureComputerGUIInfoPtr
SegModel::getFeatureComputerGUIInfo() const
{
  return m_featureComputerGUIInfo.clone();
  //FeatureComputerGUIInfo *guiInfo = new FeatureComputerGUIInfo(this);
  //return guiInfo;
}

void
SegModel::setFeatureComputerGUIInfo(const FeatureComputerGUIInfo &infos)
{
  m_featureComputerGUIInfo = infos;
}

SegModel::Memento
SegModel::save() const
{
  Memento s;

  girl::tools::oMemoryBackup w;
  girl::tools::oZBackup wz(w);
  const bool writeOk = m_toger.write(wz);
  wz.end();
  s.m_mb = w.get();

  if (! writeOk)
    std::cerr<<"Error: unable to backup toger"<<std::endl;

  s.m_selectionType = m_selectionType;
  s.m_selectedRegions = m_selectedRegions;
  s.m_selectedDarts = m_selectedDarts;
  s.m_modified = m_modified; 

  return s;
}

void
SegModel::restore(SegModel::Memento &s)
{
  girl::tools::iMemoryBackup r(s.m_mb);
  girl::tools::iZBackup rz(r);
  const bool readOk = m_toger.read(rz);

  if (! readOk)
    std::cerr<<"Error: unable to restore toger"<<std::endl;

  setToger(&m_toger);
  setModified(s.m_modified);
  setSelectionType(s.m_selectionType);
  setSelectedRegions(s.m_selectedRegions);
  setSelectedDarts(s.m_selectedDarts);  
}

void
SegModel::renameRegion(girl::region oldRegion, girl::region newRegion)
{
  // because toger::shrink() called when toger is saved
  //  may rename regions in the selection !
  // Warning: same problem may occur when we will have darts selection !!!

  girl::regionDeque::iterator it = std::find(m_selectedRegions.begin(), m_selectedRegions.end(), oldRegion);
  if (it != m_selectedRegions.end()) {
      m_selectedRegions.erase(it);
      m_selectedRegions.push_back(newRegion);
  }

}
