/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef SEGMODEL_HPP
#define SEGMODEL_HPP


#include <girl/girlSharedPtr.hpp>
#include <girl/euclideanPathSet.hpp>

#include <ise/iseSegmentationContext.hpp>

//#include "FeatureDataAlgo.hpp"

#include <QObject>
#include <QImage>

namespace girl {
  namespace tools {
    class MemoryBuffer;
  }
}

#include "FeatureComputerGUIInfo.hpp"

class GIRL_API SegModel : public QObject, public girl::toger::observer
{

  //friend class FeatureComputerGUIInfo;

  Q_OBJECT

public:

  typedef enum {REGION, SEGMENT} SelectionType;

  //typedef girl::hashMap<std::string, FeatureDataAlgo> TitleToAlgoContainerType;

  virtual ~SegModel() {}

  SegModel();

  //SegModel(const QString &filename);

  //SegModel(ise::SegmentationContext &segContext);

  //SegModel(int w, int h);


  const QImage &getQImage() const;

  const girl::toger *getToger() const;

  girl::toger *getToger(); //UGLY !!!

  const girl::tools::Image &getImage() const;

  const girl::euclideanPathSet &getEPS() const;

  /**
   * @warning reset toger and regset
   * @warning clear selections.
   */
  void setQImage(const QImage &);

  /**
   * @warning reset toger and regset
   * @warning clear selections.
   */
  void setGirlToolsImage(const girl::tools::Image &);  

  /**
   * @warning reset regset
   * @warning clear selections.
   */
  void setToger(const girl::toger *tog);

  //ise::SegmentationContext getSegContext() const;   //B:TODO: ugly !!!!!

  void setSegContext(const ise::SegmentationContext &);

  ise::SegmentationContext *getSegContextPtr();   //B:TODO: ugly !!!!!


  int width() const;
  int height() const;

  SelectionType getSelectionType() const;
  const girl::regionDeque &getSelectedRegions() const;
  const girl::dartDeque &getSelectedDarts() const;

  void setSelectionType(SelectionType);
  void setSelectedRegions(const girl::regionDeque &);
  void setSelectedDarts(const girl::dartDeque &);

  void clearSelection();

  bool isRegionSelected(girl::region r) const;
  void addRegionToSelection(girl::region r);
  void removeRegionFromSelection(girl::region r);

  bool isDartSelected(girl::dart d) const;
  void addDartToSelection(girl::dart d);
  void removeDartFromSelection(girl::dart d);

  bool isModified() const;
  void setModified(bool modified);

  /**
   * @brief Tell if a model is selectable,
   * that is it has a toger.
   */
  bool selectable() const;

  /**
   * @brief Tell if a model is splittable,
   * that is it has a toger and an image.
   */
  bool splittable() const;  // == hasCompleteSegContext()

  /**
   * @brief Tell if a model is mergeable,
   * that is it is has a toger and an image, and at least two regions.
   */
  bool mergeable() const;  // == hasCompleteSegContext()

  /**
   * WARNING return a clone!
   */
  girl::shared_ptr<FeatureComputerGUIInfo>
  getFeatureComputerGUIInfo() const;

  void
  setFeatureComputerGUIInfo(const FeatureComputerGUIInfo &info);

  class Memento
  {
    friend class SegModel;

  private:
    girl::shared_ptr<girl::tools::MemoryBuffer> m_mb;
    SelectionType m_selectionType;
    girl::regionDeque m_selectedRegions;
    girl::dartDeque m_selectedDarts;
    bool m_modified;
  };

  Memento save() const;
  void restore(Memento &s);

  //as girl::toger::observer
  virtual bool doRenameRegion() const { return true; }
  virtual void renameRegion(girl::region oldRegion, girl::region newRegion);

signals:

  void selectionTypeChanged(SegModel *);
  void selectionChanged(SegModel *);
  void togerChanged(SegModel *);
  void imageChanged(SegModel *);

protected:

  void resetToger();
  void fillRegSet();

  // const TitleToAlgoContainerType&
  // getTitleToAlgoMap() const
  // {
  //   return m_titleToAlgo;
  // }

  // void
  // setTitleToAlgoMap(const TitleToAlgoContainerType& titleToAlgo)
  // {
  //   m_titleToAlgo = titleToAlgo;
  // }

protected:

  ise::SegmentationContext m_segContext;
  girl::tools::Image m_image;
  QImage m_qimage;
  girl::toger m_toger;
  mutable girl::euclideanPathSet m_eps;

  //TitleToAlgoContainerType m_titleToAlgo;
  FeatureComputerGUIInfo m_featureComputerGUIInfo;


  SelectionType m_selectionType;
  girl::regionDeque m_selectedRegions;
  girl::dartDeque m_selectedDarts;
  bool m_modified;
  bool m_togerNotNull;
  mutable bool m_epsToUpdate;

};


#endif /* ! SEGMODEL_HPP */
