/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "SegView.hpp"


#include <QPainter>

#include <iostream>//DEBUG


SegView::SegView(QWidget *parent, SegModel *model)
  : GLView(parent),
    m_model(model)
{

}


void
SegView::connect(SegModel *model)
{
  assert(model);

  QObject::connect(model, SIGNAL(imageChanged(SegModel *)), this, SLOT(changeImage(SegModel *)));
  QObject::connect(model, SIGNAL(togerChanged(SegModel *)), this, SLOT(changeToger(SegModel *)));
  QObject::connect(model, SIGNAL(selectionChanged(SegModel *)), this, SLOT(changeSelection(SegModel *)));
  QObject::connect(model, SIGNAL(selectionTypeChanged(SegModel *)), this, SLOT(changeSelectionType(SegModel *)));
}

void
SegView::disconnect(SegModel *model)
{
  assert(model);

  QObject::disconnect(model, SIGNAL(imageChanged(SegModel *)), this, SLOT(changeImage(SegModel *)));
  QObject::disconnect(model, SIGNAL(togerChanged(SegModel *)), this, SLOT(changeToger(SegModel *)));
  QObject::disconnect(model, SIGNAL(selectionChanged(SegModel *)), this, SLOT(changeSelection(SegModel *)));
  QObject::disconnect(model, SIGNAL(selectionTypeChanged(SegModel *)), this, SLOT(changeSelectionType(SegModel *)));
}


QPointF
SegView::modelToView(const QPoint &pt)
{
  QPointF out;
  transformFromView(pt, out);
  return out;
}

QPointF
SegView::viewToModel(const QPoint &pt)
{
  QPointF out;
  transformToView(pt, out);
  return out;
}

void
SegView::addInteractor(Interactor *i, bool activate)
{
  assert(i);
  installEventFilter(i);
  if (activate)
    i->activate();
}

void
SegView::removeInteractor(Interactor *i, bool deactivate)
{
  assert(i);
  if (deactivate)
    i->deactivate();
  removeEventFilter(i);
}



