/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef SEGVIEW_HPP
#define SEGVIEW_HPP


#include <QColor>
#include <QPointF>

#include "GLView.hpp"
#include "SegModel.hpp"
#include "Interactor.hpp"



class GIRL_API SegView : public GLView
{
  Q_OBJECT

public:

  SegView(QWidget *parent = NULL, SegModel *model = NULL);

  /**
   *
   * @warning does not take ownership of @a model
   */
  virtual void setModel(SegModel *model)
  {
    m_model = model;
  }

  SegModel *model() 
  {
    return m_model;
  }

  const SegModel *model() const
  {
    return m_model;
  }


  virtual QPointF modelToView(const QPoint &pt);
  virtual QPointF viewToModel(const QPoint &pt);

  virtual void addInteractor(Interactor *i, bool activate = true);
  virtual void removeInteractor(Interactor *i, bool deactivate = true);
					       

protected slots:

  virtual void changeImage(SegModel *) {}
  virtual void changeToger(SegModel *) {}
  virtual void changeSelection(SegModel *) {}
  virtual void changeSelectionType(SegModel *) {}

protected:

  virtual void connect(SegModel *model);
  virtual void disconnect(SegModel *model);

  /*

  void paintAllFaces(QPainter &p, const QColor &color);
  void paintSelection(QPainter &p, 
		      const QColor &outerColor, const QColor &innerColor);

  void paintRegion(QPainter &p, girl::region r,
		     const QColor &outerColor, const QColor &innerColor);
  void paintFace(QPainter &p, girl::face f);

  void paintFacePositiveDarts(QPainter &p, girl::face f);

  void paintDart(QPainter &p, girl::dart d);

  void paintPaths(QPainter &p, 
		  const QColor &color, 
		  const girl::pathDeque &paths);

  void paintPath(QPainter &p, 
		 const girl::path &pth);

  void paintPairs(QPainter &p, 
		  const QColor &color, 
		  const QVector<QPoint> &pairs);

  */

protected:

  SegModel *m_model;


};

#endif /* ! SEGVIEW_HPP */
