/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "SegmentationClusteringBinaryRelaxationWidget.hpp"
#include "SegmentationClusteringBinaryRelaxationWidgetSettings.hpp"

#include <ise/iseSegmentationClusteringBinaryRelaxation.hpp>

#include <qiseTools/SliderSpinBoxWidget.hpp>

#include <QBoxLayout>
#include <QLabel>

#include "SegmentationClusteringPlugin.hpp"

SegmentationClusteringBinaryRelaxationWidget::SegmentationClusteringBinaryRelaxationWidget(MainView *mainView,
											   SegmentationClusteringPlugin *segmentationClusteringPlugin,
											   QWidget *parent)
  : SegmentationClusteringWidgetInterface(mainView, segmentationClusteringPlugin, parent)
  , m_initSB(NULL)
  , m_relax1SB(NULL)
  , m_relax2SB(NULL)
{
  setWindowTitle(tr("Bahnu binary relaxation parameters..."));
  
  setAttribute(Qt::WA_DeleteOnClose);

  buildGUI();
}

void
SegmentationClusteringBinaryRelaxationWidget::buildGUI()
{
  QLabel *initLabel = new QLabel(tr("Initialization coefficient: "), this);
  m_initSB = new SliderSpinBoxWidget(this);
  m_initSB->setDecimals(4);
  m_initSB->setSingleStep(0.0001);
  m_initSB->setMinimum(0.5);
  m_initSB->setMaximum(1.0);
  
  QLabel *relax1Label = new QLabel(tr("Relaxation coefficient 1: "), this);
  m_relax1SB = new SliderSpinBoxWidget(this);
  m_relax1SB->setDecimals(4);
  m_relax1SB->setSingleStep(0.0001);
  m_relax1SB->setMinimum(0.0);
  m_relax1SB->setMaximum(1.0);
  
  QLabel *relax2Label = new QLabel(tr("Relaxation coefficient 2: "), this);
  m_relax2SB = new SliderSpinBoxWidget(this);
  m_relax2SB->setDecimals(4);
  m_relax2SB->setSingleStep(0.0001);
  m_relax2SB->setMinimum(0.0);
  m_relax2SB->setMaximum(1.0);
  

  QGridLayout *gLayout = new QGridLayout;
  gLayout->addWidget(initLabel,   0, 0);
  gLayout->addWidget(m_initSB,    0, 1);
  gLayout->addWidget(relax1Label, 1, 0);
  gLayout->addWidget(m_relax1SB,  1, 1);
  gLayout->addWidget(relax2Label, 2, 0);
  gLayout->addWidget(m_relax2SB,  2, 1);

  setLayout(gLayout);  
}

void
SegmentationClusteringBinaryRelaxationWidget::setParameters(double init, double relax1, double relax2)
{
  m_initSB->setValue(init);
  m_relax1SB->setValue(relax1);
  m_relax2SB->setValue(relax2);  
}

void
SegmentationClusteringBinaryRelaxationWidget::getParameters(double &init, double &relax1, double &relax2) const
{
  init = m_initSB->value();
  relax1 = m_relax1SB->value();
  relax2 = m_relax2SB->value();
}

