/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef SEGMENTATIONCLUSTERINGBINARYRELAXATIONWIDGETSETTINGS_HPP
#define SEGMENTATIONCLUSTERINGBINARYRELAXATIONWIDGETSETTINGS_HPP

#include "SegmentationClusteringWidgetSettingsInterface.hpp"
#include "SegmentationClusteringBinaryRelaxationWidget.hpp"
#include <ise/iseSegmentationClusteringBinaryRelaxation.hpp>

class SegmentationClusteringBinaryRelaxationWidgetSettings : public SegmentationClusteringWidgetSettingsInterface
{
  
public:
  
  SegmentationClusteringBinaryRelaxationWidgetSettings() 
    : SegmentationClusteringWidgetSettingsInterface()
    , m_init(0.7)
    , m_relax1(0.2)
    , m_relax2(0.3)
  {
  }

  virtual void configureWidget(SegmentationClusteringWidgetInterface *widget) const
  {
    static_cast<SegmentationClusteringBinaryRelaxationWidget*>(widget)->setParameters(m_init, m_relax1, m_relax2);
  }
  
  virtual void getWidgetConfig(const SegmentationClusteringWidgetInterface *widget)
  {
    static_cast<const SegmentationClusteringBinaryRelaxationWidget*>(widget)->getParameters(m_init, m_relax1, m_relax2);
  }

  virtual void configureAlgo(girl::shared_ptr<ise::SegmentationClustering> algo) const
  {
    girl::shared_ptr<ise::SegmentationClusteringBinaryRelaxation> algorithm = girl::static_pointer_cast<ise::SegmentationClusteringBinaryRelaxation>(algo);
    algorithm->setParameters(m_init, m_relax1, m_relax2);
  }
  
  virtual girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface> clone() const
  {
    girl::shared_ptr<SegmentationClusteringBinaryRelaxationWidgetSettings> p(new SegmentationClusteringBinaryRelaxationWidgetSettings);

    *p = *this;

    return p;
  }
  
  virtual bool isValid() const
  {
    return true; //B: ???
  }


private:
  
  double m_init;
  double m_relax1;
  double m_relax2;
};

#endif /* ! SEGMENTATIONCLUSTERINGBINARYRELAXATIONWIDGETSETTINGS_HPP */
