/*
  Copyright (C) 2005-2011 Julian Roigt  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "ImageFeatureChoiceWidget.hpp"

#include "SegmentationClusteringImageFeatureClustererWidget.hpp"
#include "SegmentationClusteringSettingsImageFeatureClusterer.hpp"

#include "ClustererWidgetInterface.hpp"
#include "ClustererPlugin.hpp"

#include "PluginRegistry.hpp"
#include "PluginIndexChoiceWidget.hpp"

#include <ise/iseClusterer.hpp>

#include <QLayout>
#include <QComboBox>

SegmentationClusteringImageFeatureClustererWidget::SegmentationClusteringImageFeatureClustererWidget(MainView *mainView, SegmentationClusteringPlugin *segmentationClusteringPlugin, QWidget *parent)
  : SegmentationClusteringWidgetInterface(mainView, segmentationClusteringPlugin, parent)
  , m_ifcw()
  , m_clustererCB(NULL)
  , m_clusterer()
  , m_activeClustererWidget(NULL)
{
  setAttribute(Qt::WA_DeleteOnClose);

  setWindowTitle(tr("SegmentationClusteringImageFeatureClusterer parameters..."));

  buildGUI();
}

void
SegmentationClusteringImageFeatureClustererWidget::buildGUI()
{
  assert(m_mainView);
  assert(m_mainView->model());
  assert(m_mainView->model()->getSegContextPtr());

  m_ifcw = new ImageFeatureChoiceWidget(m_mainView, m_mainView->model()->getFeatureComputerGUIInfo(), this);

  QIsePluginList<ClustererPlugin *> list = PluginRegistry::getClustererPlugin();

  m_clustererCB = new PluginIndexChoiceWidget(list.getGenericList(), tr("Clusterer"), this);

  {
    QIsePluginList<ClustererPlugin *>::iterator it = list.begin();
    QIsePluginList<ClustererPlugin *>::iterator end = list.end();
    for (; it != end; ++it) {
      m_clusterer[tr((*it)->name().toStdString().c_str())] = (*it)->parameterWidget(m_mainView, this);
    }
  }

  connect(m_clustererCB, SIGNAL(pluginIndexChanged(int)), this, SLOT(showWidget(int)));

  QVBoxLayout *vLayout = new QVBoxLayout;
  vLayout->addWidget(m_ifcw);
  vLayout->addWidget(m_clustererCB);

  std::map<QString, ClustererWidgetInterface* >::const_iterator it = m_clusterer.begin();
  std::map<QString, ClustererWidgetInterface* >::const_iterator itEnd = m_clusterer.end();
  for ( ; it != itEnd ; ++it) {
    vLayout->addWidget( (*it).second );
    (*it).second->hide();
  }

  m_activeClustererWidget = m_clusterer[m_clustererCB->getPluginName(0)];
  if (m_activeClustererWidget != NULL)
    m_activeClustererWidget->show();

  showWidget(0);

  setLayout(vLayout);
}

void
SegmentationClusteringImageFeatureClustererWidget::showWidget(int index)
{
  const QString name = m_clustererCB->getPluginName(index);

  if (m_activeClustererWidget != NULL)
    m_activeClustererWidget->hide();

  m_activeClustererWidget = m_clusterer[name];

  if (m_activeClustererWidget != NULL)
    m_activeClustererWidget->show();
}

void
SegmentationClusteringImageFeatureClustererWidget::setParameters(girl::shared_ptr<ise::Clusterer> /*clusterer*/, const ise::FeatureComputerPtr &algo)
{
  m_ifcw->setImageFeatureComputerPtr(algo);
}

void
SegmentationClusteringImageFeatureClustererWidget::getParameters(girl::shared_ptr<ise::Clusterer> &clusterer, ise::FeatureComputerPtr &algo) const
{
  assert(m_mainView);
  assert(m_mainView->model());
  assert(m_mainView->model()->getSegContextPtr());

  algo = m_ifcw->imageFeatureComputerPtr();

  std::map<QString, ClustererWidgetInterface* >::const_iterator it = m_clusterer.begin();
  std::map<QString, ClustererWidgetInterface* >::const_iterator itEnd = m_clusterer.end();
  for ( ; it != itEnd ; ++it) {
    (*it).second->saveSettingsInPlugin();
  }

  clusterer = static_cast<ClustererPlugin*>(m_clustererCB->getPlugin())->getNewClusterer(m_mainView->model()->getSegContextPtr());
}

