/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include <qiseTools/SegmentationClusteringLocalThresholderNiblackWidget.hpp>
#include <qiseTools/SegmentationClusteringLocalThresholderNiblackWidgetSettings.hpp>

#include <ise/iseSegmentationContext.hpp>

#include <QDoubleSpinBox>
#include <QLabel>
#include <QSpinBox>
#include <QGridLayout>

SegmentationClusteringLocalThresholderNiblackWidget::SegmentationClusteringLocalThresholderNiblackWidget(SegmentationClusteringLocalThresholderPlugin *segmentationClusteringLocalThresholderPlugin, QWidget *parent)
  : SegmentationClusteringLocalThresholderWidget(segmentationClusteringLocalThresholderPlugin, parent)
  , m_windowSize(NULL)
  , m_k(NULL)
{
  setAttribute(Qt::WA_DeleteOnClose);

  build();
}

void
SegmentationClusteringLocalThresholderNiblackWidget::build()
{
  QGridLayout *layout = new QGridLayout;

  QLabel *windowSizeLabel = new QLabel(tr("Window size:"), this);
  m_windowSize = new QSpinBox(this);

  m_windowSize->setSingleStep(2);
  m_windowSize->setMinimum(3);
  m_windowSize->setMaximum(1000);

  QLabel *kLabel = new QLabel(tr("k :"), this);
  m_k = new QDoubleSpinBox(this);

  m_k->setMinimum(-1000);
  m_k->setMaximum(1000);

  layout->addWidget(windowSizeLabel, 0, 0);
  layout->addWidget(m_windowSize, 0, 1);
  layout->addWidget(kLabel, 1, 0);
  layout->addWidget(m_k, 1, 1);

  setLayout(layout);
}

void
SegmentationClusteringLocalThresholderNiblackWidget::setParameters(size_t windowSize, double k)
{
  m_windowSize->setValue(windowSize);
  m_k->setValue(k);
}

void
SegmentationClusteringLocalThresholderNiblackWidget::getParameters(size_t &windowSize, double &k) const
{
  windowSize = m_windowSize->value();
  k = m_k->value();
}
