/* 
 * Copyright (C) 2010-2011 Julian Roigt  boris.mansencal@labri.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation (Inc., 59
 * Temple Place - Suite 330, Boston, MA 02111-1307, USA); either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
#ifndef SEGMENTATIONCLUSTERINGLOCALTHRESHOLDERPLUGIN_HPP
#define SEGMENTATIONCLUSTERINGLOCALTHRESHOLDERPLUGIN_HPP

#include "QIsePlugin.hpp"

#include <QString>

#include <ise/iseSegmentationClusteringLocalThresholder.hpp>
#include "SegmentationClusteringLocalThresholderWidgetSettingsInterface.hpp"
#include "qiseTools/SegmentationClusteringLocalThresholderFactory.hpp"
#include <ise/iseSegmentationContext.hpp>

class SegmentationClusteringLocalThresholderWidget;

class GIRL_API SegmentationClusteringLocalThresholderPlugin : public QIsePlugin,
							      public SegmentationClusteringLocalThresholderFactory
{
  
public:
  
  virtual ~SegmentationClusteringLocalThresholderPlugin()
  {}
  
  virtual girl::shared_ptr<ise::SegmentationClusteringLocalThresholder> getNewSegmentationClusteringLocalThresholder()
  {
    girl::shared_ptr<ise::SegmentationClusteringLocalThresholder> a = create();
    girl::shared_ptr<SegmentationClusteringLocalThresholderWidgetSettingsInterface> s = settings();
    if (s)
      s->configureAlgo(a);
    return a;
  }
  
  virtual SegmentationClusteringLocalThresholderWidget *parameterWidget(QWidget *parent = NULL)
  {
    SegmentationClusteringLocalThresholderWidget * w = createWidget(parent);
    girl::shared_ptr<SegmentationClusteringLocalThresholderWidgetSettingsInterface> s = settings();
    if (s)
      s->configureWidget(w);
    return w;
  }

  void setSettings(girl::shared_ptr<SegmentationClusteringLocalThresholderWidgetSettingsInterface> settings)
  {
    if (settings)
      m_settings = settings->clone();
    else
      m_settings = girl::shared_ptr<SegmentationClusteringLocalThresholderWidgetSettingsInterface>();
  }  

  girl::shared_ptr<SegmentationClusteringLocalThresholderWidgetSettingsInterface> getSettings()
  {
    if (settings()) 
      return m_settings->clone();
    return girl::shared_ptr<SegmentationClusteringLocalThresholderWidgetSettingsInterface>();
  }  


  virtual void saveParameters(const girl::shared_ptr<SegmentationClusteringLocalThresholderWidgetSettingsInterface> &sd)
  {
    m_settings = sd;
  }

  inline void setSegContext(ise::SegmentationContext *segContext)
  {
    m_segContext = segContext;
  }
  

  virtual girl::shared_ptr<ise::SegmentationClusteringLocalThresholder> create()  = 0;

  virtual girl::shared_ptr<SegmentationClusteringLocalThresholderWidgetSettingsInterface> createSettings()  = 0;
   
  virtual bool hasSettings() const
  {
    return true;
  }


protected:
  
  SegmentationClusteringLocalThresholderPlugin(const QString &name,
				  const QIseVersion &pluginVersion,
				  const QIseVersion &qiseVersion,
				  QString path = QString::fromUtf8(""))
    : QIsePlugin(name, (QString::fromUtf8("SegmentationClusteringLocalThresholder/") + path),
		 pluginVersion, qiseVersion)
    , m_segContext(NULL)
    , m_settings()
  {}
  
  virtual girl::shared_ptr<SegmentationClusteringLocalThresholderWidgetSettingsInterface> settings()
  { 
    if (!m_settings)
      m_settings = createSettings(); //(m_segContext);
    return m_settings; 
  }

 virtual SegmentationClusteringLocalThresholderWidget* createWidget(QWidget *parent = NULL) = 0;


private:
  
  SegmentationClusteringLocalThresholderPlugin();

protected:

  ise::SegmentationContext *m_segContext;

  girl::shared_ptr<SegmentationClusteringLocalThresholderWidgetSettingsInterface> m_settings;
};

#define SegmentationClusteringLocalThresholderPlugin_iid "org.girl.qise.SegmentationClusteringLocalThresholderPlugin/1.0"
Q_DECLARE_INTERFACE(SegmentationClusteringLocalThresholderPlugin, SegmentationClusteringLocalThresholderPlugin_iid)

#endif /* ! SEGMENTATIONCLUSTERINGLOCALTHRESHOLDERPLUGIN_HPP */
