/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "SegmentationClusteringLocalThresholderSauvolaWidget.hpp"
#include "SegmentationClusteringLocalThresholderSauvolaWidgetSettings.hpp"

#include <QLabel>
#include <QDoubleSpinBox>
#include <QSpinBox>
#include <QGridLayout>

SegmentationClusteringLocalThresholderSauvolaWidget::SegmentationClusteringLocalThresholderSauvolaWidget(SegmentationClusteringLocalThresholderPlugin *segmentationClusteringLocalThresholderPlugin,
													 QWidget *parent)
  : SegmentationClusteringLocalThresholderWidget(segmentationClusteringLocalThresholderPlugin, parent)
  , m_windowSize(NULL)
  , m_R(NULL)
  , m_k(NULL)
{
  setAttribute(Qt::WA_DeleteOnClose);

  build();
}

void
SegmentationClusteringLocalThresholderSauvolaWidget::build()
{
  QGridLayout *layout = new QGridLayout;
  layout->setAlignment(Qt::AlignTop);

  QLabel *windowSizeLabel = new QLabel(tr("Window size : "), this);
  m_windowSize = new QSpinBox(this);

  QLabel *RLabel = new QLabel(tr("R : "), this);
  m_R = new QDoubleSpinBox(this);

  QLabel *kLabel = new QLabel(tr("k : "), this);
  m_k = new QDoubleSpinBox(this);

  m_windowSize->setSingleStep(2);
  m_windowSize->setMinimum(3);
  m_windowSize->setMaximum(1000);

  m_k->setSingleStep(0.1);
  m_k->setMaximum(1000);
  m_R->setSingleStep(0.1);
  m_R->setMaximum(1000);

  layout->addWidget(windowSizeLabel, 0, 0);
  layout->addWidget(m_windowSize, 0, 1);
  layout->addWidget(RLabel, 1, 0);
  layout->addWidget(m_R, 1, 1);
  layout->addWidget(kLabel, 2, 0);
  layout->addWidget(m_k, 2, 1);

  setLayout(layout);
}

void
SegmentationClusteringLocalThresholderSauvolaWidget::setParameters(size_t windowSize, float R, float k)
{
  m_windowSize->setValue(windowSize);
  m_R->setValue(R);
  m_k->setValue(k);
}

void
SegmentationClusteringLocalThresholderSauvolaWidget::getParameters(size_t &windowSize, float &R, float &k) const
{
  windowSize = m_windowSize->value();
  R = m_R->value();
  k = m_k->value();
}

