/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef SEGMENTATIONCLUSTERINGLOCALTHRESHOLDERBERNSENWIDGETSETTINGS_HPP
#define SEGMENTATIONCLUSTERINGLOCALTHRESHOLDERBERNSENWIDGETSETTINGS_HPP

#include "SegmentationClusteringLocalThresholderWidgetSettingsInterface.hpp"
#include "SegmentationClusteringLocalThresholderSauvolaWidget.hpp"
#include <ise/iseSegmentationClusteringLocalThresholderSauvola.hpp>

class SegmentationClusteringLocalThresholderSauvolaWidgetSettings : public SegmentationClusteringLocalThresholderWidgetSettingsInterface {
  
public:
  
  SegmentationClusteringLocalThresholderSauvolaWidgetSettings() 
    : SegmentationClusteringLocalThresholderWidgetSettingsInterface()
    , m_windowSize(15)
    , m_R(128)
    , m_k(0.5)
  {
  }

  virtual void configureWidget(SegmentationClusteringLocalThresholderWidget *widget) const
  {
    static_cast<SegmentationClusteringLocalThresholderSauvolaWidget *>(widget)->setParameters(m_windowSize, m_R, m_k);
  }
  
  virtual void getWidgetConfig(const SegmentationClusteringLocalThresholderWidget *widget)
  {
    static_cast<const SegmentationClusteringLocalThresholderSauvolaWidget *>(widget)->getParameters(m_windowSize, m_R, m_k);
  }

  virtual void configureAlgo(girl::shared_ptr<ise::SegmentationClusteringLocalThresholder> algo) const
  {
    girl::shared_ptr<ise::SegmentationClusteringLocalThresholderSauvola> algorithm = girl::static_pointer_cast<ise::SegmentationClusteringLocalThresholderSauvola>(algo);
    algorithm->setParameters(m_windowSize, m_R, m_k);
  }
  
  virtual girl::shared_ptr<SegmentationClusteringLocalThresholderWidgetSettingsInterface> clone() const
  {
    girl::shared_ptr<SegmentationClusteringLocalThresholderSauvolaWidgetSettings> p(new SegmentationClusteringLocalThresholderSauvolaWidgetSettings);

    *p = *this;

    return p;
  }

  virtual bool isValid() const
  {
    return (m_windowSize > 0);//B:???
  }

private:
  
  size_t m_windowSize;
  float m_R;
  float m_k;
};

#endif /* ! SEGMENTATIONCLUSTERINGLOCALTHRESHOLDERBERNSENWIDGETSETTINGS_HPP */
