/*
  Copyright (C) 2005-2011 Julian Roigt boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "SegmentationClusteringLocalThresholdsWidget.hpp"
#include "SegmentationClusteringLocalThresholdsWidgetSettings.hpp"

#include <iostream>

#include <ise/iseSegmentationClusteringLocalThresholds.hpp>
#include <ise/iseSegmentationClusteringLocalThresholder.hpp>

#include <ise/iseSegmentationClusteringLocalThresholderBernsen.hpp>
#include <ise/iseSegmentationClusteringLocalThresholderSauvola.hpp>
#include <ise/iseSegmentationClusteringLocalThresholderWhite.hpp>

#include "MainView.hpp"
#include "SegModel.hpp"

#include "ColorSpaceRegistry.hpp"
#include "SegmentationClusteringPlugin.hpp"
#include "PluginRegistry.hpp"

#include <QLayout>
#include <QComboBox>
#include <QString>
#include <QApplication>

SegmentationClusteringLocalThresholdsWidget::SegmentationClusteringLocalThresholdsWidget(MainView *mainView, 
							       SegmentationClusteringPlugin *segmentationClusteringPlugin,
							       QWidget *parent)
  : SegmentationClusteringWidgetInterface(mainView, segmentationClusteringPlugin, parent)
  , m_thresholdersCB(NULL)
  , m_thresholders()
  , m_activeThresholderWidget(NULL)
{
  setAttribute(Qt::WA_DeleteOnClose);

  setWindowTitle(tr("SegmentationClusteringLocalThresholds parameters..."));
  
  buildGUI();
}


void
SegmentationClusteringLocalThresholdsWidget::buildGUI()
{
  QIsePluginList<SegmentationClusteringLocalThresholderPlugin *> list = PluginRegistry::getSegmentationClusteringLocalThresholderPlugin();


  m_thresholdersCB = new PluginIndexChoiceWidget(list.getGenericList(), tr("Thresholder"), this);

  {
    QIsePluginList<SegmentationClusteringLocalThresholderPlugin *>::iterator it = list.begin();
    QIsePluginList<SegmentationClusteringLocalThresholderPlugin *>::iterator end = list.end();
    for ( ; it != end; ++it) {
      m_thresholders[tr((*it)->name().toStdString().c_str())] = (*it)->parameterWidget(this);
    }
  }

  connect(m_thresholdersCB, SIGNAL(pluginIndexChanged(int)), this, SLOT(showWidget(int)));

  QVBoxLayout *vLayout = new QVBoxLayout;
  vLayout->addWidget(m_thresholdersCB);

  std::map<QString, SegmentationClusteringLocalThresholderWidget* >::const_iterator it = m_thresholders.begin();
  std::map<QString, SegmentationClusteringLocalThresholderWidget* >::const_iterator itEnd = m_thresholders.end();
  for ( ; it != itEnd ; ++it)
    vLayout->addWidget( (*it).second );

  m_activeThresholderWidget = m_thresholders[m_thresholdersCB->getPluginName(0)];
  if (m_activeThresholderWidget != NULL)
    m_activeThresholderWidget->show();

  showWidget(0);

  setLayout(vLayout);
}

void
SegmentationClusteringLocalThresholdsWidget::showWidget(int index)
{
  const QString name = m_thresholdersCB->getPluginName(index);

  if (m_activeThresholderWidget != NULL)
    m_activeThresholderWidget->hide();

  m_activeThresholderWidget = m_thresholders[name];

  if (m_activeThresholderWidget != NULL)
    m_activeThresholderWidget->show();
}

void
SegmentationClusteringLocalThresholdsWidget::setParameters(std::string m_name)
{
  int index = m_thresholdersCB->findText(m_name.c_str());
  m_thresholdersCB->setCurrentIndex(index);
}

void
SegmentationClusteringLocalThresholdsWidget::getParameters(girl::shared_ptr<ise::SegmentationClusteringLocalThresholder> &activeThresholder, std::string &m_name) const
{
  std::map<QString, SegmentationClusteringLocalThresholderWidget* >::const_iterator it = m_thresholders.begin();
  std::map<QString, SegmentationClusteringLocalThresholderWidget* >::const_iterator itEnd = m_thresholders.end();
  for ( ; it != itEnd ; ++it) {
    (*it).second->saveSettingsInPlugin();
  }

  activeThresholder = m_activeThresholderWidget->getPlugin()->getNewSegmentationClusteringLocalThresholder();
  m_name = m_thresholdersCB->currentText().toStdString();
}


