/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "SegmentationClusteringManualWidget.hpp"

#include <cassert>
#include <QGridLayout>

#include <ise/iseSegmentationClusteringManual.hpp>

#include "SegmentationClusteringFactory.hpp"
#include "SegmentationClusteringParametersDialog.hpp"

#include <QComboBox>
#include <QGroupBox>
#include <QList>
#include <QPushButton>
#include "PluginRegistry.hpp"

SegmentationClusteringManualWidget::SegmentationClusteringManualWidget(MainView *mainView,
								       SegmentationClusteringFactory *segmentationClusteringFactory,
								       QWidget *parent)
  : SegmentationClusteringWidgetInterface(mainView, segmentationClusteringFactory, parent)
  , m_thresholdType(NULL)
  , m_threshold(NULL)
  , m_segmentation(NULL)
  , m_localThresholders()
  , m_segmentationClusteringPlugin()
{

  setWindowTitle(tr("Manual parameters..."));

  setAttribute(Qt::WA_DeleteOnClose);

  buildGUI();

  setGUI();

}

void
SegmentationClusteringManualWidget::addLocalThresholder(const QString& name, SegmentationClusteringLocalThresholderWidget *thresholderWidget)
{
  assert(m_threshold);

  m_localThresholders[name] = thresholderWidget;
  m_threshold->addItem(name);  
}

void
SegmentationClusteringManualWidget::addQuantificationThresholder(const QString &name)
{
  assert(m_threshold);
  
  m_threshold->addItem(name);
}

void
SegmentationClusteringManualWidget::populateThreshold()
{
  m_threshold->clear();
  m_localThresholders.clear();

  if ( m_thresholdType->currentText() == "Local Threshold") {
    QList<SegmentationClusteringLocalThresholderPlugin *> list = PluginRegistry::getSegmentationClusteringLocalThresholderPlugin();
    QList<SegmentationClusteringLocalThresholderPlugin *>::iterator it = list.begin();
    QList<SegmentationClusteringLocalThresholderPlugin *>::iterator end = list.end();
    for ( ; it != end; ++it) {
      addLocalThresholder(tr((*it)->name().toStdString().c_str()), (*it)->parameterWidget(this));
    }
  }
  if ( m_thresholdType->currentText() == "Quantification Threshold") {
    QList<QuantificationThresholderPlugin *> list = PluginRegistry::getQuantificationThresholderPlugin();
    QList<QuantificationThresholderPlugin *>::iterator it = list.begin();
    QList<QuantificationThresholderPlugin *>::iterator end = list.end();
    for ( ; it != end; ++it) {
      addQuantificationThresholder(tr((*it)->name().toStdString().c_str()));
    }

  }
}

void
SegmentationClusteringManualWidget::addSegmentationClustering(const QString &name, SegmentationClusteringPlugin *segPlugin)
{
  assert(m_threshold);
  
  m_segmentationClusteringPlugin[name] = segPlugin;
  m_segmentation->addItem(name);
}

void
SegmentationClusteringManualWidget::populateSegmentation()
{
  /*QList<SegmentationClusteringPlugin *> list = PluginRegistry::getSegmentationClusteringPlugin();
  QList<SegmentationClusteringPlugin *>::iterator it = list.begin();
  QList<SegmentationClusteringPlugin *>::iterator end = list.end();
  for ( ; it != end; ++it) {
    if ((*it)->name() != "Manual")
      addSegmentationClustering(tr((*it)->name().toStdString().c_str()), (*it));
  }
  */
}

void
SegmentationClusteringManualWidget::buildGUI()
{
  QGridLayout *layout = new QGridLayout();
  
  QGroupBox *threshold = new QGroupBox(tr("Thresholds"), this);
  QGridLayout *layoutThreshold = new QGridLayout;
  m_thresholdType = new QComboBox(this);
  m_threshold = new QComboBox(this);
  
  m_thresholdType->addItem(tr("Quantification Threshold"));
  m_thresholdType->addItem(tr("Local Threshold"));
  m_thresholdType->setCurrentIndex(0);

  populateThreshold();

  layoutThreshold->addWidget(m_thresholdType, 0, 0);
  layoutThreshold->addWidget(m_threshold, 0, 1);

  threshold->setLayout(layoutThreshold);

  QGroupBox *segmentation = new QGroupBox(tr("Segmentation:"), this);
  QGridLayout *layoutSegmentation = new QGridLayout;
  m_segmentation = new QComboBox(this);
  QPushButton *segButton = new QPushButton(tr("Parameters.."), this);
  connect(segButton, SIGNAL(clicked()), this, SLOT(showDialogSeg()));

  populateSegmentation();

  layoutSegmentation->addWidget(m_segmentation, 0, 0);
  layoutSegmentation->addWidget(segButton, 0, 1);

  segmentation->setLayout(layoutSegmentation);
  
  layout->addWidget(threshold, 0, 0);
  layout->addWidget(segmentation, 1, 0);

  setLayout(layout);
}

void
SegmentationClusteringManualWidget::showDialogSeg()
{
  SegmentationClusteringWidgetInterface *w = m_segmentationClusteringPlugin[m_segmentation->currentText()]->parameterWidget(m_mainView);
  
  if (w != NULL) {
    QDialog *d = new SegmentationClusteringParametersDialog(w, this); 
    d->show();
  }
}

void
SegmentationClusteringManualWidget::setGUI()
{


  girl::shared_ptr<ise::SegmentationClusteringManual> algo = girl::static_pointer_cast<ise::SegmentationClusteringManual>(m_segmentationClusteringFactory->getNewSegmentationClustering());
  algo->getParameters();


}

void
SegmentationClusteringManualWidget::getGUI()
{



  girl::shared_ptr<ise::SegmentationClusteringManual> algo = girl::static_pointer_cast<ise::SegmentationClusteringManual>(m_segmentationClusteringFactory->getNewSegmentationClustering());
  algo->setParameters();  m_segmentationClusteringFactory->saveParameters(algo);
}
