/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "SegmentationClusteringMarkovWidget.hpp"

#include <QBoxLayout>
#include <QDoubleSpinBox>
#include <QLabel>
#include <QSpinBox>



SegmentationClusteringMarkovWidget::SegmentationClusteringMarkovWidget(MainView *mainView,
								       SegmentationClusteringPlugin *segmentationClusteringPlugin,
								       QWidget *parent)
  : SegmentationClusteringWidgetInterface(mainView, segmentationClusteringPlugin, parent)
  , m_potentialSB(NULL)
  , m_thresholdSegInitSB(NULL)
{
  setWindowTitle(tr("Markov Random Field parameters..."));

  setAttribute(Qt::WA_DeleteOnClose);

  buildGUI();
}

void
SegmentationClusteringMarkovWidget::buildGUI()
{
  QLabel *potentialLabel = new QLabel(tr("Ponderation: "), this);
  m_potentialSB = new QDoubleSpinBox(this);
  m_potentialSB->setMinimum(0.0);

  QLabel *thresholdSegInitLabel = new QLabel(tr("Initial Segmentation Threshold: "), this);
  m_thresholdSegInitSB = new QSpinBox(this);
  m_thresholdSegInitSB->setMinimum(0);

  QGridLayout *gLayout = new QGridLayout;
  gLayout->addWidget(potentialLabel, 0, 0);
  gLayout->addWidget(m_potentialSB, 0, 1);
  gLayout->addWidget(thresholdSegInitLabel, 1, 0);
  gLayout->addWidget(m_thresholdSegInitSB, 1, 1);

  setLayout(gLayout);  
}

void
SegmentationClusteringMarkovWidget::setParameters(double potential, double thresholdSegInit)
{
  m_potentialSB->setValue(potential);
  m_thresholdSegInitSB->setValue(thresholdSegInit);
}

void
SegmentationClusteringMarkovWidget::getParameters(double &potential, double &thresholdSegInit) const
{
  potential = m_potentialSB->value();
  thresholdSegInit = m_thresholdSegInitSB->value();
}


