/*
  Copyright (C) 2005-2011 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef SEGMENTATIONCLUSTERINGMARKOVWIDGETSETTINGS_HPP
#define SEGMENTATIONCLUSTERINGMARKOVWIDGETSETTINGS_HPP

#include "SegmentationClusteringWidgetSettingsInterface.hpp"
#include "SegmentationClusteringMarkovWidget.hpp"
#include <ise/iseSegmentationClusteringMarkov.hpp>

class SegmentationClusteringMarkovWidgetSettings : public SegmentationClusteringWidgetSettingsInterface {
  
public:
  
  SegmentationClusteringMarkovWidgetSettings() 
    : SegmentationClusteringWidgetSettingsInterface()
    , m_potential(15.0)
    , m_thresholdSegInit(15.0)
  {
  }

  virtual void configureWidget(SegmentationClusteringWidgetInterface *widget) const
  {
    static_cast<SegmentationClusteringMarkovWidget*>(widget)->setParameters(m_potential, m_thresholdSegInit);
  }
  
  virtual void getWidgetConfig(const SegmentationClusteringWidgetInterface *widget)
  {
    static_cast<const SegmentationClusteringMarkovWidget*>(widget)->getParameters(m_potential, m_thresholdSegInit);
  }

  virtual void configureAlgo(girl::shared_ptr<ise::SegmentationClustering> algo) const
  {
    girl::shared_ptr<ise::SegmentationClusteringMarkov> algorithm = girl::static_pointer_cast<ise::SegmentationClusteringMarkov>(algo);
    algorithm->setParameters(m_potential, m_thresholdSegInit);
  }

  virtual girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface> clone() const
  {
    girl::shared_ptr<SegmentationClusteringMarkovWidgetSettings> p(new SegmentationClusteringMarkovWidgetSettings);

    *p = *this;

    return p;
  }

  virtual bool isValid() const
  {
    return true; //B:???
  }

  
private:
  
  double m_potential;
  double m_thresholdSegInit;
};

#endif /* ! SEGMENTATIONCLUSTERINGMARKOVWIDGETSETTINGS_HPP */
