#ifndef SEGMENTATIONCLUSTERINGPLUGIN_HPP
#define SEGMENTATIONCLUSTERINGPLUGIN_HPP

#include "QIsePlugin.hpp"

#include <QString>

#include <qiseTools/SegmentationClusteringFactory.hpp>
#include "SegmentationClusteringWidgetSettingsInterface.hpp"
//#include <qiseTools/SegmentationClusteringWidgetInterface.hpp>

class SegmentationClusteringWidgetInterface;


class GIRL_API SegmentationClusteringPlugin : public QIsePlugin,
					      public SegmentationClusteringFactory
{
  
public:
  
  static const char SUBMENU_SEPARATOR = '#';

  virtual ~SegmentationClusteringPlugin()
  {}
  
  virtual girl::shared_ptr<ise::SegmentationClustering> getNewSegmentationClustering() //ise::SegmentationContext *segContext
  {
    girl::shared_ptr<ise::SegmentationClustering> a = create(); //(m_segContext);
    girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface> s = settings();
    if (s)
      s->configureAlgo(a);
    return a;    
  }
  
  virtual SegmentationClusteringWidgetInterface *parameterWidget(MainView *mainView, QWidget *parent = NULL) 
  {
    SegmentationClusteringWidgetInterface *w = createWidget(mainView, parent);
    girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface> s = settings();
    if (s)
      s->configureWidget(w);
    return w;
  }
  
  void setSettings(girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface> settings)
  {
    if (settings)
      m_settings = settings->clone();
    else
      m_settings = girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface>();
  }  

  girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface> getSettings()
  {
    if (settings()) 
      return m_settings->clone();
    return girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface>();
  }  


  inline ise::SegmentationContext *getSegContext()
  {
    return m_segContext;
  }
  
  inline void setSegContext(ise::SegmentationContext *segContext)
  {
    m_segContext = segContext;
  }
  
  virtual bool inSubMenu(std::string &subMenu) = 0;

  virtual girl::shared_ptr<ise::SegmentationClustering> create() = 0; //ise::SegmentationContext *segContext
  
  virtual girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface> createSettings() = 0;

  virtual bool hasSettings() const
  {
    return true;
  }

protected:
  
  SegmentationClusteringPlugin(const QString &name,
		       const QIseVersion &pluginVersion,
		       const QIseVersion &qiseVersion,
		       QString path = QString::fromUtf8(""))
    : QIsePlugin(name, (QString::fromUtf8("SegmentationClustering/") + path),
		 pluginVersion, qiseVersion),
      m_segContext(NULL),
      m_settings()
  {}
  

  virtual SegmentationClusteringWidgetInterface* createWidget(MainView *mainView, QWidget *parent = NULL) = 0;


  virtual girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface> settings()  
  { 
    if (!m_settings)
      m_settings = createSettings(); //(m_segContext);
    return m_settings; 
  }


private:
  
  SegmentationClusteringPlugin();

protected:

  ise::SegmentationContext *m_segContext;

  girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface> m_settings;
};

#define SegmentationClusteringPlugin_iid "org.girl.qise.SegmentationClusteringPlugin/1.0"
Q_DECLARE_INTERFACE(SegmentationClusteringPlugin, SegmentationClusteringPlugin_iid)

#endif /* ! SEGMENTATIONCLUSTERINGPLUGIN_HPP */
