/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "SegmentationClusteringRLSAWidget.hpp"
#include "SegmentationClusteringRLSAWidgetSettings.hpp"

#include <cassert>
#include <QVBoxLayout>

#include <ise/iseSegmentationClusteringRLSA.hpp>

#include "SegmentationClusteringFactory.hpp"

#include <QSpinBox>
#include <QLabel>
#include <QGroupBox>

SegmentationClusteringRLSAWidget::SegmentationClusteringRLSAWidget(MainView *mainView,
									     SegmentationClusteringPlugin *segmentationClusteringPlugin,
									     QWidget *parent)
  : SegmentationClusteringWidgetInterface(mainView, segmentationClusteringPlugin, parent)
  , m_C_horizontal(NULL)
  , m_C_s(NULL)
  , m_C_vertical(NULL)
  , m_group(NULL)
{

  setWindowTitle(tr("RLSA parameters..."));

  setAttribute(Qt::WA_DeleteOnClose);

  buildGUI();
}

void
SegmentationClusteringRLSAWidget::buildGUI()
{
  QVBoxLayout *layout = new QVBoxLayout;
  
  m_thresholder = new ButtonThresholder(m_mainView, this);

  m_group = new QGroupBox(tr("configure"), this);
  m_group->setCheckable(true);

  QGridLayout *layoutGroup = new QGridLayout;

  QLabel *C_horizontalLabel = new QLabel(tr("C_horizontal :"), this);
  m_C_horizontal = new QSpinBox(this);
  m_C_horizontal->setRange(0, 10000);
  m_C_horizontal->setSingleStep(1);
  layoutGroup->addWidget(C_horizontalLabel, 0, 0);
  layoutGroup->addWidget(m_C_horizontal, 0, 1);

  QLabel *C_sLabel = new QLabel(tr("C_s :"), this);
  m_C_s = new QSpinBox(this);
  m_C_s->setRange(0, 10000);
  m_C_s->setSingleStep(1);
  layoutGroup->addWidget(C_sLabel, 1, 0);
  layoutGroup->addWidget(m_C_s, 1, 1);

  QLabel *C_verticalLabel = new QLabel(tr("C_vertical :"), this);
  m_C_vertical = new QSpinBox(this);
  m_C_vertical->setRange(0, 10000);
  m_C_vertical->setSingleStep(1);
  layoutGroup->addWidget(C_verticalLabel, 2, 0);
  layoutGroup->addWidget(m_C_vertical, 2, 1);

  m_group->setLayout(layoutGroup);

  layout->addWidget(m_thresholder);
  layout->addWidget(m_group, 0, 0);

  setLayout(layout);
}

void
SegmentationClusteringRLSAWidget::setParameters(int colorSpace, int channel, const std::vector<int> &levels, int horizontal, int s, int vertical)
{
  m_thresholder->setParameters(colorSpace, channel, levels);

  if (horizontal == -1 && s == -1 && vertical == -1)
    m_group->setChecked(false);
  m_C_horizontal->setValue(horizontal);
  m_C_s->setValue(s);
  m_C_vertical->setValue(vertical);
}

void
SegmentationClusteringRLSAWidget::getParameters(int &colorSpace, int &channel, std::vector<int> &levels, int &horizontal, int &s, int &vertical) const
{
  m_thresholder->getParameters(colorSpace, channel, levels);

  horizontal = m_C_horizontal->value();
  s = m_C_s->value();
  vertical = m_C_vertical->value();

  if ( !m_group->isChecked()) {
    horizontal = -1;
    vertical = -1;
    s = -1;
  }
}

