/*
  Copyright (C) 2005-2011 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef SEGMENTATIONCLUSTERINGRLSAWIDGETSETTINGS_HPP
#define SEGMENTATIONCLUSTERINGRLSAWIDGETSETTINGS_HPP

#include "SegmentationClusteringWidgetSettingsInterface.hpp"
#include "SegmentationClusteringRLSAWidget.hpp"
#include <ise/iseSegmentationClusteringRLSA.hpp>

class SegmentationClusteringRLSAWidgetSettings : public SegmentationClusteringWidgetSettingsInterface {
  
public:
  
  SegmentationClusteringRLSAWidgetSettings() 
    : SegmentationClusteringWidgetSettingsInterface()
    , m_colorSpace(ISE_RGB_to_RGB)
    , m_channel(0)
    , m_levels(1, 128)
    , m_horizontal(-1)
    , m_s(-1)
    , m_vertical(-1)
  {
  }

  virtual void configureWidget(SegmentationClusteringWidgetInterface *widget) const
  {
    static_cast<SegmentationClusteringRLSAWidget*>(widget)->setParameters(m_colorSpace, m_channel, m_levels, m_horizontal, m_s, m_vertical);
  }
  
  virtual void getWidgetConfig(const SegmentationClusteringWidgetInterface *widget)
  {
    static_cast<const SegmentationClusteringRLSAWidget*>(widget)->getParameters(m_colorSpace, m_channel, m_levels, m_horizontal, m_s, m_vertical);
  }

  virtual void configureAlgo(girl::shared_ptr<ise::SegmentationClustering> algo) const
  {
    girl::shared_ptr<ise::SegmentationClusteringRLSA> algorithm = girl::static_pointer_cast<ise::SegmentationClusteringRLSA>(algo);
    algorithm->setParameters(m_colorSpace, m_channel, m_levels, m_horizontal, m_s, m_vertical);
  }
  
  virtual girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface> clone() const
  {
    girl::shared_ptr<SegmentationClusteringRLSAWidgetSettings> p(new SegmentationClusteringRLSAWidgetSettings);

    *p = *this;

    return p;
  }
  
  virtual bool isValid() const
  {
    return ( (! m_levels.empty()) && m_horizontal >= -1 && m_s >= -1 && m_vertical >= -1);
  }

private:
  
  int m_colorSpace;
  int m_channel;
  std::vector<int> m_levels;
  int m_horizontal;
  int m_s;
  int m_vertical;
};

#endif /* ! SEGMENTATIONCLUSTERINGRLSAWIDGETSETTINGS_HPP */
