/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "SegmentationClusteringRXYCWidget.hpp"
#include "SegmentationClusteringRXYCWidgetSettings.hpp"

#include <cassert>
#include <QVBoxLayout>
#include <QSpinBox>
#include <QLabel>

#include <ise/iseSegmentationClusteringRXYC.hpp>

#include "SegmentationClusteringPlugin.hpp"



SegmentationClusteringRXYCWidget::SegmentationClusteringRXYCWidget(MainView *mainView,
									     SegmentationClusteringPlugin *segmentationClusteringPlugin,
									     QWidget *parent)
  : SegmentationClusteringWidgetInterface(mainView, segmentationClusteringPlugin, parent)
  , m_iteration(NULL)
{

  setWindowTitle(tr("RXYC parameters..."));

  setAttribute(Qt::WA_DeleteOnClose);

  buildGUI();

}

void
SegmentationClusteringRXYCWidget::buildGUI()
{
  QVBoxLayout *layout = new QVBoxLayout;
  
  m_thresholder = new ButtonThresholder(m_mainView, this);

  QLabel *iterationLabel = new QLabel("number iteration :");
  m_iteration = new QSpinBox(this);
  m_iteration->setRange(-1, 1000);

  QHBoxLayout *hLayout = new QHBoxLayout;
  hLayout->addWidget(iterationLabel);
  hLayout->addWidget(m_iteration);

  layout->addWidget(m_thresholder);
  layout->addLayout(hLayout);

  setLayout(layout);
}

void
SegmentationClusteringRXYCWidget::setParameters(const ise::ThresholderData &data, int iteration)
{
  m_thresholder->setParameters(data);
  m_iteration->setValue(iteration);
}

void
SegmentationClusteringRXYCWidget::getParameters(ise::ThresholderData &data, int &iteration) const
{
  m_thresholder->getParameters(data);
  iteration = m_iteration->value();
}


