/*
  Copyright (C) 2005-2011 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef SEGMENTATIONCLUSTERINGRXYCWIDGETSETTINGS_HPP
#define SEGMENTATIONCLUSTERINGRXYCWIDGETSETTINGS_HPP

#include "SegmentationClusteringWidgetSettingsInterface.hpp"
#include "SegmentationClusteringRXYCWidget.hpp"
#include <ise/iseSegmentationClusteringRXYC.hpp>
#include <ise/iseThresholderData.hpp>

class SegmentationClusteringRXYCWidgetSettings : public SegmentationClusteringWidgetSettingsInterface {
  
public:
  
  SegmentationClusteringRXYCWidgetSettings() 
    : SegmentationClusteringWidgetSettingsInterface()
    , m_datas(0, 0, std::vector<int>(1, 128))
    , m_iteration(-1)
  {
  }

  virtual void configureWidget(SegmentationClusteringWidgetInterface *widget) const
  {
    static_cast<SegmentationClusteringRXYCWidget*>(widget)->setParameters(m_datas, m_iteration);
  }
  
  virtual void getWidgetConfig(const SegmentationClusteringWidgetInterface *widget)
  {
    static_cast<const SegmentationClusteringRXYCWidget*>(widget)->getParameters(m_datas, m_iteration);
  }

  virtual void configureAlgo(girl::shared_ptr<ise::SegmentationClustering> algo) const
  {
    girl::shared_ptr<ise::SegmentationClusteringRXYC> algorithm = girl::static_pointer_cast<ise::SegmentationClusteringRXYC>(algo);
    algorithm->setParameters(m_datas, m_iteration);
  }
  
  virtual girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface> clone() const
  {
    girl::shared_ptr<SegmentationClusteringRXYCWidgetSettings> p(new SegmentationClusteringRXYCWidgetSettings);

    *p = *this;

    return p;
  }

  virtual bool isValid() const
  {
    return (m_iteration >= -1); //B:???
  }
  

private:
  
  ise::ThresholderData m_datas;
  int m_iteration;
};

#endif /* ! SEGMENTATIONCLUSTERINGRXYCWIDGETSETTINGS_HPP */
