/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "SegmentationClusteringRegularGridWidget.hpp"

#include <ise/iseRegularGridPartitionner.hpp>
#include <ise/iseSegmentationContext.hpp>

#include <qiseTools/MainView.hpp>

#include <QBoxLayout>
#include <QSpinBox>
#include <QLabel>
#include <QTextStream>

#include "SegmentationClusteringPlugin.hpp"

SegmentationClusteringRegularGridWidget::SegmentationClusteringRegularGridWidget(MainView *mainView,
									     SegmentationClusteringPlugin *segmentationClusteringPlugin,
									     QWidget *parent)
  : SegmentationClusteringWidgetInterface(mainView, segmentationClusteringPlugin, parent)
  , m_segClust(NULL)
  , m_widthEntry(NULL)
  , m_heightEntry(NULL)
{

  setWindowTitle(tr("Regular grid partitionner parameters..."));

  setAttribute(Qt::WA_DeleteOnClose);

  buildGUI();
}

void
SegmentationClusteringRegularGridWidget::buildGUI()
{
  QLabel *widthLabel = new QLabel(tr("Grid width:"), this);
  m_widthEntry = new QSpinBox(this);
  QLabel *heightLabel = new QLabel(tr("Grid height:"), this);
  m_heightEntry = new QSpinBox(this);

  // TODO: Will crash if null pointer somewhere, shall we check here?
  const int w = m_mainView->model()->getSegContextPtr()->getImage()->width();
  const int h = m_mainView->model()->getSegContextPtr()->getImage()->height();
  m_widthEntry->setRange(1, w);
  m_heightEntry->setRange(1, h);

  QHBoxLayout *hbox1 = new QHBoxLayout;
  hbox1->addWidget(widthLabel);
  hbox1->addWidget(m_widthEntry);
  hbox1->addWidget(heightLabel);
  hbox1->addWidget(m_heightEntry);

  QVBoxLayout *vLayout = new QVBoxLayout();
  vLayout->addLayout(hbox1);

  setLayout(vLayout);
}

void
SegmentationClusteringRegularGridWidget::setParameters(int width, int height)
{
  m_widthEntry->setValue(width);
  m_heightEntry->setValue(height);  
}

void
SegmentationClusteringRegularGridWidget::getParameters(int &width, int &height) const
{
   width = m_widthEntry->value();
   height = m_heightEntry->value();
}
