/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#include "SegmentationClusteringSpatialMeanShiftWidget.hpp"
#include "SegmentationClusteringSpatialMeanShiftWidgetSettings.hpp"

#include <limits>

#include <ise/iseSegmentationClusteringSpatialMeanShift.hpp>

#include <QBoxLayout>
#include <QDialogButtonBox>
#include <QLabel>
#include <QSpinBox>
#include <QCheckBox>

#include "SegmentationClusteringPlugin.hpp"
#include "MainView.hpp"

SegmentationClusteringSpatialMeanShiftWidget::SegmentationClusteringSpatialMeanShiftWidget(MainView *mainView,
											   SegmentationClusteringPlugin *segmentationClusteringPlugin,
											   QWidget *parent)
  : SegmentationClusteringWidgetInterface(mainView, segmentationClusteringPlugin, parent)
  , m_sigmaR(NULL)
  , m_sigmaS(NULL)
  , m_minRegion(NULL)
  , m_fuseRegion(NULL)
  , m_ifcw(NULL)
{
  setWindowTitle(tr("SegmentationClusteringSpatialMeanShift parameters..."));

  setAttribute(Qt::WA_DeleteOnClose);

  buildGUI();
}

void
SegmentationClusteringSpatialMeanShiftWidget::buildGUI()
{
  QGridLayout *gLayout = new QGridLayout;


  m_sigmaR = new QDoubleSpinBox(this);
  m_sigmaS = new QDoubleSpinBox(this);
  m_minRegion = new QDoubleSpinBox(this);
  QLabel *labelSigmaR = new QLabel(tr("SigmaR"), this);
  QLabel *labelSigmaS = new QLabel(tr("SigmaS"), this);
  QLabel *labelMinRegion = new QLabel(tr("Min. Region Area"), this);

  m_ifcw = new ImageFeatureChoiceWidget(m_mainView, m_mainView->model()->getFeatureComputerGUIInfo(), this);
  QLabel *labelIfwc = new QLabel(tr("Image Feature"), this);

  m_fuseRegion = new QCheckBox(this);
  QLabel *labelFuseRegion = new QLabel(tr("MeanShift FuseRegion"), this);

  m_sigmaR->setValue(7);
  m_sigmaS->setValue(6.5);
  m_minRegion->setMaximum(100000);
  m_minRegion->setValue(100);

  gLayout->addWidget(labelIfwc, 0, 0);
  gLayout->addWidget(m_ifcw, 0, 1);
  gLayout->addWidget(labelSigmaR, 1, 0);
  gLayout->addWidget(m_sigmaR, 1, 1);
  gLayout->addWidget(labelSigmaS, 2, 0);
  gLayout->addWidget(m_sigmaS, 2, 1);
  gLayout->addWidget(labelMinRegion, 3, 0);
  gLayout->addWidget(m_minRegion, 3, 1);
  gLayout->addWidget(labelFuseRegion, 4, 0);
  gLayout->addWidget(m_fuseRegion, 4, 1);

  QVBoxLayout *vLayout = new QVBoxLayout();
  vLayout->addLayout(gLayout);

  setLayout(vLayout);
}

void
SegmentationClusteringSpatialMeanShiftWidget::setParameters(float sigmaR, float sigmaS, int minRegion, bool fuseRegion, const ise::FeatureComputerPtr &algo)
{
  m_sigmaR->setValue(sigmaR);
  m_sigmaS->setValue(sigmaS);
  m_minRegion->setValue(minRegion);
  m_fuseRegion->setChecked(fuseRegion);
  m_ifcw->setImageFeatureComputerPtr(algo);
}

void
SegmentationClusteringSpatialMeanShiftWidget::getParameters(float &sigmaR, float &sigmaS, int &minRegion, bool &fuseRegion, ise::FeatureComputerPtr &algo) const
{
  sigmaR = m_sigmaR->value();
  sigmaS = m_sigmaS->value();
  minRegion = m_minRegion->value();
  fuseRegion = m_fuseRegion->isChecked();
  algo = m_ifcw->imageFeatureComputerPtr();
}
