/*
  Copyright (C) 2005-2011 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef SEGMENTATIONCLUSTERINGSPATIALMEANSHIFTWIDGETSETTINGS_HPP
#define SEGMENTATIONCLUSTERINGSPATIALMEANSHIFTWIDGETSETTINGS_HPP

#include "SegmentationClusteringWidgetSettingsInterface.hpp"
#include "SegmentationClusteringSpatialMeanShiftWidget.hpp"
#include <ise/iseSegmentationClusteringSpatialMeanShift.hpp>

class SegmentationClusteringSpatialMeanShiftWidgetSettings : public SegmentationClusteringWidgetSettingsInterface
{
  
public:
  
  SegmentationClusteringSpatialMeanShiftWidgetSettings() 
    : SegmentationClusteringWidgetSettingsInterface()
    , m_sigmaR(7)
    , m_sigmaS(6.5)
    , m_minRegion(100)
    , m_fuseRegion(false)
    , m_algo(ise::FeatureComputerPtr())
  {
  }

  virtual void configureWidget(SegmentationClusteringWidgetInterface *widget) const
  {
    static_cast<SegmentationClusteringSpatialMeanShiftWidget*>(widget)->setParameters(m_sigmaR, m_sigmaS, m_minRegion, m_fuseRegion, m_algo);
  }
  
  virtual void getWidgetConfig(const SegmentationClusteringWidgetInterface *widget)
  {
    static_cast<const SegmentationClusteringSpatialMeanShiftWidget*>(widget)->getParameters(m_sigmaR, m_sigmaS, m_minRegion, m_fuseRegion, m_algo);
  }

  virtual void configureAlgo(girl::shared_ptr<ise::SegmentationClustering> algo) const
  {
    girl::shared_ptr<ise::SegmentationClusteringSpatialMeanShift> algorithm = girl::static_pointer_cast<ise::SegmentationClusteringSpatialMeanShift>(algo);
    algorithm->setParameters(m_sigmaR, m_sigmaS, m_minRegion, m_fuseRegion, girl::static_pointer_cast<ise::ImageFeatureComputer>(m_algo));
  }
  
  virtual girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface> clone() const
  {
    girl::shared_ptr<SegmentationClusteringSpatialMeanShiftWidgetSettings> p(new SegmentationClusteringSpatialMeanShiftWidgetSettings);

    *p = *this;

    return p;
  }

  virtual bool isValid() const
  {
    return true; //B:???
  }

private:
  
  float m_sigmaR;
  float m_sigmaS;
  int m_minRegion;
  bool m_fuseRegion;
  ise::FeatureComputerPtr m_algo;
};

#endif /* ! SEGMENTATIONCLUSTERINGSPATIALMEANSHIFTWIDGETSETTINGS_HPP */
