/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "SegmentationClusteringWatershedWidget.hpp"
#include "SegmentationClusteringWatershedWidgetSettings.hpp"

#include <cassert>
#include <limits>

#include <QGridLayout>
#include <QGroupBox>
#include <QHBoxLayout>
#include <QIntValidator>
#include <QLabel>
#include <QSpinBox>

#include <ise/iseSegmentationClusteringWatershed.hpp>

#include <qiseTools/WeightedChannelsImageFeatureWidget.hpp>

#include <qiseTools/ImageFeatureChoiceWidget.hpp>
#include <qiseTools/MainView.hpp>
#include "SegmentationClusteringPlugin.hpp"

#include "ImageFeatureChoiceWidget.hpp"

SegmentationClusteringWatershedWidget::SegmentationClusteringWatershedWidget(MainView *mainView,
									     SegmentationClusteringPlugin *segmentationClusteringPlugin,
									     QWidget *parent)
  :
  SegmentationClusteringWidgetInterface(mainView, segmentationClusteringPlugin, parent),
  m_nbGradient(NULL),
  m_wcifw(NULL)
{

  setWindowTitle(tr("Watershed parameters..."));

  setAttribute(Qt::WA_DeleteOnClose);

  buildGUI();

}



void
SegmentationClusteringWatershedWidget::buildGUI()
{
  assert(m_mainView);
  assert(m_mainView->model());

  QLabel *l = new QLabel(tr("Number of colors: "), this);
  
  m_nbGradient = new QSpinBox(this);
  m_nbGradient->setMinimum(2); //minimum 2 colors !
  m_nbGradient->setMaximum(std::numeric_limits<int>::max());

  QHBoxLayout *hbox = new QHBoxLayout;
  hbox->addWidget(l);
  hbox->addWidget(m_nbGradient);

  m_wcifw = new WeightedChannelsImageFeatureWidget(m_mainView, m_mainView->model()->getFeatureComputerGUIInfo(), this);

  QVBoxLayout *vLayout = new QVBoxLayout();

  vLayout->addLayout(hbox);
  vLayout->addWidget(m_wcifw);
  
  setLayout(vLayout);
}

void
SegmentationClusteringWatershedWidget:: setParameters(int nbGradient, const ise::FeatureComputerPtr &algo, const std::vector<double> &weights)
{
  m_wcifw->setImageFeatureComputerPtr(algo);
  m_wcifw->setWeights(weights);
  m_nbGradient->setValue(nbGradient); 
}

void
SegmentationClusteringWatershedWidget::getParameters(int &nbGradient, ise::FeatureComputerPtr &algo, std::vector<double> &weights) const
{
  nbGradient =  m_nbGradient->value();
  algo = m_wcifw->imageFeatureComputerPtr();
  m_wcifw->weights(weights);
}
