/*
  Copyright (C) 2005-2011 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef SEGMENTATIONCLUSTERINGWATERSHEDWIDGETSETTINGS_HPP
#define SEGMENTATIONCLUSTERINGWATERSHEDWIDGETSETTINGS_HPP

#include "SegmentationClusteringPlugin.hpp"

#include "SegmentationClusteringWatershedWidget.hpp"
#include <ise/iseSegmentationClusteringWatershed.hpp>

class SegmentationClusteringWatershedWidgetSettings : public SegmentationClusteringWidgetSettingsInterface {
  
public:
  
  SegmentationClusteringWatershedWidgetSettings() 
    : SegmentationClusteringWidgetSettingsInterface()
    , m_nbGradient(255)
    , m_algo(ise::FeatureComputerPtr())
    , m_weights()
  {
  }

  virtual void configureWidget(SegmentationClusteringWidgetInterface *widget) const
  {
    static_cast<SegmentationClusteringWatershedWidget*>(widget)->setParameters(m_nbGradient, m_algo, m_weights);
  }
  
  virtual void getWidgetConfig(const SegmentationClusteringWidgetInterface *widget)
  {
    static_cast<const SegmentationClusteringWatershedWidget*>(widget)->getParameters(m_nbGradient, m_algo, m_weights);
  }

  virtual void configureAlgo(girl::shared_ptr<ise::SegmentationClustering> algo) const
  {
    girl::shared_ptr<ise::SegmentationClusteringWatershed> algorithm = girl::static_pointer_cast<ise::SegmentationClusteringWatershed>(algo);
    algorithm->setParameters(m_nbGradient, girl::static_pointer_cast<ise::ImageFeatureComputer>(m_algo), m_weights);
  }

  virtual girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface> clone() const
  {
    girl::shared_ptr<SegmentationClusteringWatershedWidgetSettings> p(new SegmentationClusteringWatershedWidgetSettings);

    *p = *this;

    return p;
  }

  virtual bool isValid() const
  {
    return (m_algo && girl::static_pointer_cast<ise::ImageFeatureComputer>(m_algo)->getFeatureDim() == m_weights.size());
  }
  
private:
  
  int m_nbGradient;
  ise::FeatureComputerPtr m_algo;
  std::vector<double> m_weights;
};

#endif /* ! SEGMENTATIONCLUSTERINGWATERSHEDWIDGETSETTINGS_HPP */
