/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef SEGMENTATIONCLUSTERINGWIDGETINTERFACE_HPP
#define SEGMENTATIONCLUSTERINGWIDGETINTERFACE_HPP

#include <girl/girlCommon.hpp>//GIRL_API
#include <QWidget>
#include "SegmentationClusteringPlugin.hpp"


class MainView;


class GIRL_API SegmentationClusteringWidgetInterface : public QWidget
{
public:

  SegmentationClusteringWidgetInterface(MainView *mainView,
					SegmentationClusteringPlugin *segmentationClusteringPlugin,
					QWidget *parent = NULL)
    : QWidget(parent),
    m_mainView(mainView),
    m_segmentationClusteringPlugin(segmentationClusteringPlugin)
    {

    }

  virtual void setSegmentationClustering(SegmentationClusteringPlugin *segmentationClusteringPlugin)
  {
    m_segmentationClusteringPlugin = segmentationClusteringPlugin;
  }
  
  virtual SegmentationClusteringPlugin *getSegmentationClusteringMemento()
  {
    return m_segmentationClusteringPlugin;
  }

  virtual void setGUI(const girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface> settings)
  {
    if (settings)
      settings->configureWidget(this);
  }
  
  virtual void getGUI(girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface> settings) const
  {
    if (settings)
      settings->getWidgetConfig(this);
  }

  virtual bool saveSettingsInPlugin() const
  {
    if (m_segmentationClusteringPlugin) {
      if (m_segmentationClusteringPlugin->hasSettings()) {
	girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface> s = m_segmentationClusteringPlugin->getSettings();
	getGUI(s);
	assert(s);
	if (s->isValid()) {
	  m_segmentationClusteringPlugin->setSettings(s);
	  return true;
	}
      }
      else {
	//no settings
	return true;
      }
    }
    return false;
  }
 
  virtual bool isValid() const
  {
    if (m_segmentationClusteringPlugin) {
      if (m_segmentationClusteringPlugin->hasSettings()) {
	girl::shared_ptr<SegmentationClusteringWidgetSettingsInterface> s = m_segmentationClusteringPlugin->getSettings();
	getGUI(s);
	assert(s);
	if (s->isValid()) {
	  return true;
	}
      }
      else {
	//no settings
	return true;
      }
    }
    return false;
  }
 
 protected:

  MainView *m_mainView;
  SegmentationClusteringPlugin *m_segmentationClusteringPlugin;

};


#endif /* ! SEGMENTATIONCLUSTERINGWIDGETINTERFACE_HPP */
