/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef SELECTIONSETTINGSWIDGET_HPP
#define SELECTIONSETTINGSWIDGET_HPP

#include <girl/girlCommon.hpp>
#include <qiseTools/ShapeSelector.hpp>

#include <QWidget>
#include <QAction>

class QCheckBox;
class QSlider;

class ColorButton;


class GIRL_API SelectionSettingsWidget : public QWidget
{
  Q_OBJECT

public:

  /**
   * @brief Constructor.
   */
  explicit SelectionSettingsWidget(QWidget *parent);

  /**
   * @brief Get if region filling is enabled
   */
  bool isFillEnabled() const;

  /**
   * @brief Set if region filling is enabled
   */
  void setFillEnabled(bool );
  

  /**
   * @brief Get opacity in [0; 1].
   */
  float getAlpha() const;

  /**
   * @brief Set opacity.
   *
   * @warning opacity must be in [0; 1].
   */
  void setAlpha(float v);

  /**
   * @brief Get color.
   */
  QRgb getColor() const;

  /**
   * @brief Set color.
   *
   * will emit colorChanged(QRgb) if color different from previous one.
   */
  void setColor(QRgb c);

signals:

  void fillEnabledChanged(bool b);
  void alphaChanged(float alpha);
  void colorChanged(QRgb color);

protected slots:

  void changeFillEnabled(int b);
  void changeColor(QRgb c);
  void changeAlpha(int v);

protected:
  
  void buildGUI();

protected:

  ColorButton *m_colorB;
  QSlider *m_alphaSlider;
  QCheckBox *m_fillCB;
};


#endif /* ! SELECTIONSETTINGSWIDGET_HPP */
