/*
  Copyright (C) 2010-2011 Cédric Aguerre  aguerre@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef SHAPE_HPP
#define SHAPE_HPP

#include <girl/girlToger.hpp>
#include <qiseTools/ScanlineSet.hpp>

class GIRL_API Shape
{
public:

  Shape();

  virtual ~Shape();

  static void addRegionsUnderShapes(const ScanlineSet& shapesScanlineSet, girl::regionDeque& selectedRegions, const girl::toger *tog);
  static void removeRegionsOnBorder(const ScanlineSet& shapesScanlineSet, girl::regionDeque& selectedRegions, const girl::toger *tog);

  /**
   * @brief Get scanlineSet of shape clamped to AABBox(0, 0, imageWidth-1, imageHeight-1).
   */
  void getScanlineSet(int imageWidth, int imageHeight, ScanlineSet& scanlineSet);

protected:

  virtual void getSegments(std::vector<std::vector<girl::point> > &segments, girl::tools::AABBox &shapeBBox) = 0;

  inline static bool isOverView(girl::point point, const girl::toger *tog)
  {
    return  point.x() >= 0 && point.y() >= 0 
      && point.x() < tog->width() && point.y() < tog->height();
  }

  void getSegmentationPath(girl::pathDeque& paths);

  static void addRegionsOnBorder(const ScanlineSet& shapesScanlineSet, girl::regionDeque& selectedRegions, const girl::toger *tog);

};

#endif /* ! SHAPE_HPP */
