/*
  Copyright (C) 2010-2011 Cédric Aguerre  aguerre@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef SHAPESELECTOR_HPP
#define SHAPESELECTOR_HPP

#include <girl/girlToger.hpp>//regionDeque
#include <QObject>
#include <QColor>
#include <QPoint>
#include <qiseTools/GLSelectorLayer.hpp>
#include <qiseTools/Shape.hpp>
#include <qiseTools/SegView.hpp>


class GIRL_API ShapeSelector : public Shape
{
public:

  typedef enum {SELECTOR_MODE_REPLACE, SELECTOR_MODE_ADD, SELECTOR_MODE_SUBTRACT, SELECTOR_MODE_INTERSECT} SelectorMode;

  typedef enum {SELECTOR_RC_ENCLOSED, SELECTOR_RC_INTERSECTED, SELECTOR_RC_BOTH} SelectorRegionComputer;

  typedef enum {SELECTOR_TYPE_RECTANGLE, SELECTOR_TYPE_ELLIPSE, SELECTOR_TYPE_FREEHAND, SELECTOR_TYPE_LINE} SelectorType;

  typedef enum {SELECTOR_MODIFIER_NONE, SELECTOR_MODIFIER_INVERT} SelectorModifier;

  ShapeSelector(SegView *v, QRgb color = 0, float opacity = 1.0f);

  virtual ~ShapeSelector();

  void setView(SegView *v);


  const QColor getColor() const;
  void setColor(const QColor &c);
  
  float getOpacity() const;
  void setOpacity(float o);

  SelectorMode getMode() const;
  void setMode(SelectorMode m);
  
  SelectorRegionComputer getRegionComputer() const;
  void setRegionComputer(SelectorRegionComputer rc);
  
  const girl::regionDeque &getSelectedRegions() const;

  void setSelectedRegions(const girl::regionDeque &s);

  virtual void addPoint(const QPointF &ptf) = 0;

  // return true is selector is modified
  virtual bool visit(const QPointF &ptf) = 0;

  virtual void update(const QPointF &ptf) = 0;

  // return true if selection has changed
  virtual bool finalize(SelectorModifier modifier) = 0;

  // return true if abort() has an effect (something changes)
  virtual bool abort() = 0;

  // return true if terminate() has an effect (something changes)
  virtual bool terminate(SelectorModifier modifier, bool closedRegion) = 0;

protected:

  // return true if selection changes
  bool updateSelectionAndDeleteLayer(SelectorModifier modifier);

  void deleteLayer();

protected:

  SegView *m_view;
  QColor m_color;
  float m_opacity;
  SelectorMode m_mode;
  SelectorRegionComputer m_rc;
  GLSelectorLayer *m_layer;

private:

  // update m_selectedRegions with s regarding to m_mode. Return true if selection changes.
  bool updateSelectionWith(girl::regionDeque &s, SelectorModifier modifier);

  // identify selected regions regarding to m_rc
  void identifySelectedRegions(girl::regionDeque &regions);

private :

  // derived classes are not allowed to modify m_selectedRegions directly
  girl::regionDeque m_selectedRegions;
};


#endif /* ! SHAPESELECTOR_HPP */
