/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#ifndef SLIDER_SPINBOX_WIDGET_HPP
#define SLIDER_SPINBOX_WIDGET_HPP

#include <girl/girlCommon.hpp>//GIRL_API
#include <QWidget>

class QSlider;
class QDoubleSpinBox;

/**
 * Combination of a slider and a spinBox for 'double' values.
 *
 */
class GIRL_API SliderSpinBoxWidget : public QWidget
{
  Q_OBJECT

public:

  /**
   * @brief Constructor.
   */
  explicit SliderSpinBoxWidget(QWidget *parent = NULL);

  /**
   * @brief Set maximum.
   *
   * @sa QDoubleSpinBox::setMaximum.
   */
  void setMaximum(double maximum);

  /**
   * @brief Get maximum.
   *
   * @sa QDoubleSpinBox::maximum.
   */
  double maximum() const;

  /**
   * Set minimum;
   *
   * @sa QDoubleSpinBox::setMinimum.
   */
  void setMinimum(double minimum);
  
  /**
   * Get minimum;
   *
   * @sa QDoubleSpinBox::minimum.
   */
  double minimum() const;
  
  /**
   * @brief Set single step.
   *
   * Will also set decimals if @a step can not be represented with current number of decimals.
   *
   * @sa QDoubleSpinBox::setSingleStep.
   */
  void setSingleStep(double step);

  /**
   * @brief Set decimals.
   *
   * @sa QDoubleSpinBox::setDecimals.
   */
  void setDecimals(int decimals);

  /**
   * @brief Set value.
   *
   * @sa QDoubleSpinBox::setValue.
   */
  void setValue(double value);

  /**
   * @brief Get value.
   *
   * @sa QDoubleSpinBox::value.
   */
  double value() const;

  void setEditable(bool b);

signals:
  void valueChanged(double);

protected slots:

  void sliderValueChanged(int value);
  void spinBoxValueChanged(double value);
  
protected:

  QSlider *m_slider;
  QDoubleSpinBox *m_spinBox;

};

#endif /* SLIDER_SPINBOX_WIDGET_HPP */


