/* 
 * Copyright (C) 2010-2011 Boris Mansencal  boris.mansencal@labri.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation (Inc., 59
 * Temple Place - Suite 330, Boston, MA 02111-1307, USA); either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
#ifndef SPLITALGORITHMPLUGIN_HPP
#define SPLITALGORITHMPLUGIN_HPP

#include "QIsePlugin.hpp"

#include <QString>

#include <ise/iseSplitAlgorithm.hpp>

#include <qiseTools/SplitAlgorithmFactory.hpp>
#include <qiseTools/SplitAlgorithmWidgetInterface.hpp>
#include "SettingsInterfaceTemplate.hxx"


typedef SettingsInterfaceTemplate<SplitAlgorithmWidgetInterface, ise::SplitAlgorithm> SplitAlgorithmSettings;


class SplitAlgorithmPlugin : public QIsePlugin,
			     public SplitAlgorithmFactory
{
  
public:
  
  virtual ~SplitAlgorithmPlugin()
  {}
  
  girl::shared_ptr<ise::SplitAlgorithm> getNewSplitAlgorithm(ise::SegmentationContext *segContext)
  {
    girl::shared_ptr<ise::SplitAlgorithm> a = create(segContext);
    girl::shared_ptr<SplitAlgorithmSettings> s = settings();
    if (s)
      s->configureAlgo(a);
    return a;
  }
  
  SplitAlgorithmWidgetInterface *parameterWidget(QWidget *parent = NULL)
  {
    SplitAlgorithmWidgetInterface *w = createWidget(m_segContext, parent);
    girl::shared_ptr<SplitAlgorithmSettings> s = settings();
    if (s)
      s->configureWidget(w);
    return w;    
  }
  
  void setSettings(girl::shared_ptr<SplitAlgorithmSettings> settings)
  {
    if (settings)
      m_settings = settings->clone();
    else
      m_settings = girl::shared_ptr<SplitAlgorithmSettings>();
  }  

  girl::shared_ptr<SplitAlgorithmSettings> getSettings()
  {
    if (settings()) 
      return m_settings->clone();
    return girl::shared_ptr<SplitAlgorithmSettings>();
  }  

  inline void setSegContext(ise::SegmentationContext *segContext)
  {
    m_segContext = segContext;
  }
  
  inline ise::SegmentationContext *getSegContext()
  {
    return m_segContext;
  }  


  virtual girl::shared_ptr<ise::SplitAlgorithm>
    create(ise::SegmentationContext *segContext) = 0;

  virtual girl::shared_ptr<SplitAlgorithmSettings>
    createSettings(ise::SegmentationContext* segContext) = 0;

  virtual bool hasSettings() const
  {
    return true;
  }

protected:
  
  SplitAlgorithmPlugin(const QString &name,
		       const QIseVersion &pluginVersion,
		       const QIseVersion &qiseVersion,
		       QString path = QString::fromUtf8(""))
    : QIsePlugin(name, (QString::fromUtf8("SplitAlgorithm/") + path),
		 pluginVersion, qiseVersion),
      m_segContext(NULL)
  {}
  
  virtual SplitAlgorithmWidgetInterface *
    createWidget(ise::SegmentationContext *segContext, QWidget *parent) = 0;

  girl::shared_ptr<SplitAlgorithmSettings> settings()
  {
    if (!m_settings)
      m_settings = createSettings(m_segContext);
    return m_settings;    
  }

private:
  
  SplitAlgorithmPlugin();

protected:

  girl::shared_ptr<SplitAlgorithmSettings> m_settings;
  ise::SegmentationContext *m_segContext;

};

#define SplitAlgorithmPlugin_iid "org.girl.qise.SplitAlgorithmPlugin/1.0"
Q_DECLARE_INTERFACE(SplitAlgorithmPlugin, SplitAlgorithmPlugin_iid)

#endif /* ! SPLITALGORITHMPLUGIN_HPP */
