/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef SPLITALGORITHMWIDGETINTERFACE_HPP
#define SPLITALGORITHMWIDGETINTERFACE_HPP

#include <girl/girlCommon.hpp>//GIRL_API
#include <QWidget>

class SplitAlgorithmFactory;


class GIRL_API SplitAlgorithmWidgetInterface : public QWidget
{
  Q_OBJECT

public:

  SplitAlgorithmWidgetInterface(SplitAlgorithmFactory *splitAlgorithmFactory = NULL, QWidget *parent = NULL)
    : QWidget(parent)
    , m_splitAlgorithmFactory(splitAlgorithmFactory),
      m_valid(true)
  {}

  virtual void setSplitAlgorithmFactory(SplitAlgorithmFactory *splitAlgorithmFactory)
  {
    m_splitAlgorithmFactory = splitAlgorithmFactory;
  }
  
  virtual SplitAlgorithmFactory *getSplitAlgorithmFactory()
  {
    return m_splitAlgorithmFactory;
  }

  /**
   * Set validity of widget, i.e., 
   * tells if set parameters are valid for splitAlgorithmFactory.
   */
  virtual void setValid(bool valid)
  {
    if (valid != m_valid) {
      m_valid = valid;
      emit validityChanged(valid);
    }
  }

  /**
   * Get validity of widget, i.e., 
   * tells if set parameters are valid for splitAlgorithmFactory.
   */
  virtual bool isValid() const
  {
    return m_valid;
  }


  virtual void setGUI() = 0;
  
  virtual void getGUI() = 0;

signals:

  /**
   * emitted when validity changes.
   */
  void validityChanged(bool valid);

  /**
   * emitted when one of the parameters of widget changes.
   */
  void parametersChanged();

protected:
  
  SplitAlgorithmFactory *m_splitAlgorithmFactory;
  bool m_valid;
};


#endif /* ! SPLITALGORITHMWIDGETINTERFACE_HPP */
