/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#ifndef SPLITDRIVERPLUGIN_HPP
#define SPLITDRIVERPLUGIN_HPP

#include "QIsePlugin.hpp"

#include <QString>

#include <ise/iseSplitDriver.hpp>

#include <qiseTools/SplitDriverFactory.hpp>
//#include <qiseTools/SplitDriverWidgetInterface.hpp>
#include "SettingsInterfaceTemplate.hxx"

class SplitDriverWidgetInterface;


typedef SettingsInterfaceTemplate<SplitDriverWidgetInterface, ise::SplitDriver> SplitDriverSettings;


class GIRL_API SplitDriverPlugin : public QIsePlugin,
				   public SplitDriverFactory
{
  
public:
  
  virtual ~SplitDriverPlugin()
  {}
  
  girl::shared_ptr<ise::SplitDriver> getNewSplitDriver(ise::SegmentationContext *segContext)
  {
    girl::shared_ptr<ise::SplitDriver> a = create(segContext);
    girl::shared_ptr<SplitDriverSettings> s = settings();
    if (s)
      s->configureAlgo(a);
    return a;
  }
  
  SplitDriverWidgetInterface *parameterWidget(QWidget *parent = NULL)
  {
    SplitDriverWidgetInterface *w = createWidget(m_segContext, parent);
    girl::shared_ptr<SplitDriverSettings> s = settings();
    if (s)
      s->configureWidget(w);
    return w;
  }
  
  void setSettings(girl::shared_ptr<SplitDriverSettings> settings)
  {
    if (settings)
      m_settings = settings->clone();
    else
      m_settings = girl::shared_ptr<SplitDriverSettings>();
  }

  girl::shared_ptr<SplitDriverSettings> getSettings()
  {
    if (settings()) 
      return m_settings->clone();
    return girl::shared_ptr<SplitDriverSettings>();
  }  


  inline void setSegContext(ise::SegmentationContext *segContext)
  {
    m_segContext = segContext;
  }
  
  ise::SegmentationContext *getSegContext()
  {
    return m_segContext;
  }
  
  virtual girl::shared_ptr<ise::SplitDriver>
    create(ise::SegmentationContext *segContext) = 0;

  virtual girl::shared_ptr<SplitDriverSettings>
    createSettings(ise::SegmentationContext* segContext) = 0;

  virtual bool hasSettings() const
  {
    return true;
  }


protected:
  
  SplitDriverPlugin(const QString &name,
		       const QIseVersion &pluginVersion,
		       const QIseVersion &qiseVersion,
		       QString path = QString::fromUtf8(""))
    : QIsePlugin(name, (QString::fromUtf8("SplitDriver/") + path),
		 pluginVersion, qiseVersion),
      m_segContext(NULL)
  {}
  
  virtual SplitDriverWidgetInterface *
    createWidget(ise::SegmentationContext *segContext, QWidget *parent) = 0;

  girl::shared_ptr<SplitDriverSettings> settings()
  {
    if (!m_settings)
      m_settings = createSettings(m_segContext);
    return m_settings;    
  }

private:
  
  SplitDriverPlugin();

protected:

  girl::shared_ptr<SplitDriverSettings> m_settings;
  ise::SegmentationContext *m_segContext;

};

#define SplitDriverPlugin_iid "org.girl.qise.SplitDriverPlugin/1.0"
Q_DECLARE_INTERFACE(SplitDriverPlugin, SplitDriverPlugin_iid)

#endif /* ! SPLITDRIVERPLUGIN_HPP */
