/*
  Copyright (C) 2005-2011 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef SPLITDRIVERWIDGETINTERFACE_HPP
#define SPLITDRIVERWIDGETINTERFACE_HPP

#include <girl/girlCommon.hpp>//GIRL_API
#include <QWidget>

#include "SplitDriverPlugin.hpp"


class GIRL_API SplitDriverWidgetInterface : public QWidget
{
public:

  SplitDriverWidgetInterface(SplitDriverPlugin *splitDriverPlugin, QWidget *parent = NULL)
    : QWidget(parent),
      m_splitDriverPlugin(splitDriverPlugin)
  {}


  virtual void setSplitDriver(SplitDriverPlugin *splitDriverPlugin)
  {
    m_splitDriverPlugin = splitDriverPlugin;
  }
  
  virtual SplitDriverPlugin *getSplitDriver()
  {
    return m_splitDriverPlugin;
  }

  virtual void setGUI(const girl::shared_ptr<SplitDriverSettings> settings)
  {
    if (settings)
      settings->configureWidget(this);
   }
  
  virtual void getGUI(girl::shared_ptr<SplitDriverSettings> settings) const
  {
    if (settings)
      settings->getWidgetConfig(this);
  }


  virtual bool saveSettingsInPlugin()
  {
    if (m_splitDriverPlugin) {
      if (m_splitDriverPlugin->hasSettings()) {
	girl::shared_ptr<SplitDriverSettings> s = m_splitDriverPlugin->getSettings();
	getGUI(s);
	assert(s);
	if (s->isValid()) {
	  m_splitDriverPlugin->setSettings(s);
	  return true;
	}
      }
      else {
	//no settings
	return true;
      }
    }
    return false;
  }

  virtual bool isValid() const
  {
    if (m_splitDriverPlugin) {
      if (m_splitDriverPlugin->hasSettings()) {
	girl::shared_ptr<SplitDriverSettings> s = m_splitDriverPlugin->getSettings();
	getGUI(s);
	assert(s);
	if (s->isValid()) {
	  return true;
	}
      }
      else {
	//no settings
	return true;
      }
    }
    return false;
  }


protected:

  SplitDriverPlugin *m_splitDriverPlugin;

};


#endif /* ! SPLITDRIVERWIDGETINTERFACE_HPP */
