#ifndef SPLITMERGEALGORITHMPLUGIN_HPP
#define SPLITMERGEALGORITHMPLUGIN_HPP

#include "QIsePlugin.hpp"

#include <QString>

#include <ise/iseSplitMergeAlgorithm.hpp>

#include <qiseTools/SplitMergeAlgorithmFactory.hpp>
#include <qiseTools/SplitMergeAlgorithmWidgetInterface.hpp>
#include "SettingsInterfaceTemplate.hxx"

class SplitAlgorithmPlugin;
class MergeAlgorithmPlugin;

typedef SettingsInterfaceTemplate<SplitMergeAlgorithmWidgetInterface, ise::SplitMergeAlgorithm> SplitMergeAlgorithmSettings;


class SplitMergeAlgorithmPlugin : public QIsePlugin,
				  public SplitMergeAlgorithmFactory
{
  
public:
  
  virtual ~SplitMergeAlgorithmPlugin()
  {}
  
  girl::shared_ptr<ise::SplitMergeAlgorithm> getNewSplitMergeAlgorithm(ise::SegmentationContext *segContext)
  {
    girl::shared_ptr<ise::SplitMergeAlgorithm> a = create(segContext);
    girl::shared_ptr<SplitMergeAlgorithmSettings> s = settings();
    if (s)
      s->configureAlgo(a);
    return a;
  }  
  
  virtual SplitMergeAlgorithmWidgetInterface *parameterWidget(QWidget *parent = NULL)
  {
    SplitMergeAlgorithmWidgetInterface *w = createWidget(m_segContext, parent);
    girl::shared_ptr<SplitMergeAlgorithmSettings> s = settings();
    if (s)
      s->configureWidget(w);
    return w;
  }
  
  void setSettings(girl::shared_ptr<SplitMergeAlgorithmSettings> settings)
  {
    if (settings)
      m_settings = settings->clone();
    else
      m_settings = girl::shared_ptr<SplitMergeAlgorithmSettings>();
  }  

  girl::shared_ptr<SplitMergeAlgorithmSettings> getSettings()
  {
    if (settings()) 
      return m_settings->clone();
    return girl::shared_ptr<SplitMergeAlgorithmSettings>();
  }  

  inline void setSegContext(ise::SegmentationContext *segContext)
  {
    m_segContext = segContext;
  }
  
  ise::SegmentationContext *getSegContext()
  {
    return m_segContext;
  }
  
  virtual girl::shared_ptr<ise::SplitMergeAlgorithm>
    create(ise::SegmentationContext *segContext) = 0;

  virtual girl::shared_ptr<SplitMergeAlgorithmSettings>
    createSettings(ise::SegmentationContext* segContext) = 0;

  virtual bool hasSettings() const
  {
    return true;
  }


  virtual void setSplitAlgorithmPlugin(SplitAlgorithmPlugin *splitAlgo)= 0;
  
  virtual void setMergeAlgorithmPlugin(MergeAlgorithmPlugin *mergeAlgo) = 0;

protected:
  
  SplitMergeAlgorithmPlugin(const QString &name,
		       const QIseVersion &pluginVersion,
		       const QIseVersion &qiseVersion,
		       QString path = QString::fromUtf8(""))
    : QIsePlugin(name, (QString::fromUtf8("SplitMergeAlgorithm/") + path),
		 pluginVersion, qiseVersion),
      m_segContext(NULL)
  {}

   virtual SplitMergeAlgorithmWidgetInterface *
    createWidget(ise::SegmentationContext *segContext, QWidget *parent) = 0;
 
  girl::shared_ptr<SplitMergeAlgorithmSettings> settings()
  {
    if (!m_settings)
      m_settings = createSettings(m_segContext);
    return m_settings;    
  }

private:
  
  SplitMergeAlgorithmPlugin();

protected:

  girl::shared_ptr<SplitMergeAlgorithmSettings> m_settings;
  ise::SegmentationContext *m_segContext;

};

#define SplitMergeAlgorithmPlugin_iid "org.girl.qise.SplitMergeAlgorithmPlugin/1.0"
Q_DECLARE_INTERFACE(SplitMergeAlgorithmPlugin, SplitMergeAlgorithmPlugin_iid)
                    

#endif /* ! SPLITMERGEALGORITHMPLUGIN_HPP */
