/* 
 * Copyright (C) 2010-2011 Boris Mansencal, Hugo Boujut, Remi Vieux  boris.mansencal@labri.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation (Inc., 59
 * Temple Place - Suite 330, Boston, MA 02111-1307, USA); either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
#ifndef THRESHOLDEDCOLORINDEX_HPP
#define THRESHOLDEDCOLORINDEX_HPP

#include <QHash>
#include "FalseColorIndex.hpp"

struct Pixel
{
  Pixel(int A=0, int B=0, int C=0)
  {
    a[0] = A;
    a[1] = B;
    a[2] = C;
  }
  
  bool operator==(const Pixel &p) const
  {
    return a[0] == p.a[0] && a[1] == p.a[1] && a[2] == p.a[2];
  }
  
  int operator[](int i) const
  {
    assert(i>=0 && i<3);
    return a[i];
  }
  
  int &operator[](int i)
  {
    assert(i>=0 && i<3);
    return a[i];
  }
  
  int a[3];
};


struct ColorEntry
{
  ColorEntry(Pixel tc = Pixel(), 
	     size_t f = 0,
	     QRgb thc = 0)
    : transformedColor(tc),
      frequency(f),
      thresholdedColor(thc)
  {}
  
  Pixel transformedColor; //color in second color space
  size_t frequency;
  QRgb thresholdedColor; //color of level 
};


class ThresholdedColorIndex : public FalseColorIndex {
public:
  typedef QHash<QRgb, ColorEntry> HashType;

  ThresholdedColorIndex();

  ThresholdedColorIndex(HashType &hash,
			const girl::regionDeque regions,
			const girl::toger &tog);
  

  const QRgb& getColor(const QRgb &color) const;
  void setHash(HashType &hash) { m_hash = &hash; }
  
protected:
  HashType *m_hash;
};

#endif //THRESHOLDEDCOLORINDEX_HPP
