/*
  Copyright (C) 2005-2010 Julian Roigt, Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef TITLECACHEDIALOG_HPP
#define TITLECACHEDIALOG_HPP

#include <girl/girlCommon.hpp>

#include <QComboBox>
#include <QDialog>
#include <QWidget>
#include <QGroupBox>
#include <QLineEdit>
#include <QHBoxLayout>
#include <QDialogButtonBox>
#include <QMessageBox>

#include "FeatureData.hpp"
#include <ise/iseRegionSetInterface.hpp>

class GIRL_API TitleCacheDialog : public QDialog
{
  Q_OBJECT

 public:
  
  TitleCacheDialog(FeatureDataPtr data, QWidget *parent = NULL)
    : QDialog(parent)
    , m_title(NULL)
    , m_cacheCombo(NULL)
    , m_data(data)
  {
    setAttribute(Qt::WA_DeleteOnClose);
    buildGUI();
    
    setTitle(data->title().c_str());
    setCacheState(data->cacheState());
  }

  virtual ~TitleCacheDialog() {}
  
  QString title() const { return m_title->text(); }

  void setTitle(const QString &title) { m_title->setText(title); }
  
  void setCacheState(const ise::RegionSetInterface::CacheType &cache) 
  {
    assert(m_cacheCombo);
    int index = (cache >= 0 ? cache : 0);
    m_cacheCombo->setCurrentIndex(index);
  }
  
  ise::RegionSetInterface::CacheType cacheState() const
  {
    assert(m_cacheCombo);
    return static_cast<ise::RegionSetInterface::CacheType>(m_cacheCombo->currentIndex());
  }

 protected:

  void setLayout(QLayout* layout)
  {
    m_layout->addLayout(layout, 2, 0);
  }

  void buildGUI()
  {
    QGroupBox *titleGroup = new QGroupBox(QString("Title"));
    QHBoxLayout *titleLayout = new QHBoxLayout;
    
    m_title = new QLineEdit(this);
    titleLayout->addWidget(m_title);
    
    titleGroup->setLayout(titleLayout);

    QGroupBox *cacheGroup = new QGroupBox(QString("Cache Strategy"));
    QVBoxLayout *cacheLayout = new QVBoxLayout;

    m_cacheCombo = new QComboBox(this);
    //TODO:OPTIM Hard coded combo box, is it possible to build it from enum
    // in case new state are defined?
    m_cacheCombo->addItem(tr("NONE"));
    m_cacheCombo->addItem(tr("LAZY"));
    m_cacheCombo->addItem(tr("ALWAYS"));

    cacheLayout->addWidget(m_cacheCombo);
    cacheGroup->setLayout(cacheLayout);

    m_layout = new QGridLayout;
    m_layout->setAlignment(Qt::AlignTop);

    m_layout->addWidget(titleGroup);
    m_layout->addWidget(cacheGroup);

    QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel
						       , Qt::Horizontal, this);

    connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
    connect(buttonBox, SIGNAL(rejected()), this, SLOT(reject()));

    m_layout->addWidget(buttonBox, 3, 0, 2, 0);

    QDialog::setLayout(m_layout);
  }

  bool isValid(QString &message) const
  {
    if (m_title->text().isEmpty()) {
      message += "Title will not be empty !\n";
      return false;
    }
    return true;
  }

  virtual void finish() = 0;

 public slots:
  
  void accept()
  {
    QString message;
    if (isValid(message)) {
      finish();
      m_data->setTitle(m_title->text().toStdString());
      m_data->setCacheState(cacheState());
      QDialog::accept();
    }
    else {
      QMessageBox::critical(this, "Error", message);
    }
  }

 protected:

  QLineEdit *m_title;
  QComboBox *m_cacheCombo;

  QGridLayout *m_layout;

  FeatureDataPtr m_data;
};

#endif /* ! TITLECACHEDIALOG_HPP */
