/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef TRANSFORMATION_HPP
#define TRANSFORMATION_HPP

#include <girl/girlCommon.hpp>//GIRL_API

#include <QPointF>

/**
 * Represents T(R(S(p))) transformation.
 */
class GIRL_API Transformation
{
public:

  Transformation();

  Transformation(qreal tx, qreal ty, 
		 qreal alphaInRad, 
		 qreal sx, qreal sy);
  

  void setTranslation(qreal x, qreal y);
  void setRotationAngle(qreal alphaInRad);
  void setRotationCenter(qreal cx, qreal cy);
  void setScale(qreal sx, qreal sy);

  void getTranslation(qreal &x, qreal &y) const;
  void getRotationAngle(qreal &alphaInRad) const;
  void getRotationCenter(qreal &cx, qreal &cy) const;
  void getScale(qreal &sx, qreal &sy) const;
  
  /**
   * @return T(R(S(pt)))
   */
  QPointF transformFrom(const QPointF &pt) const;

  /**
   * @return S-1(R-1(T-1(pt)))
   */
  QPointF transformTo(const QPointF &pt) const;


protected:
  
  qreal m_tx, m_ty;
  qreal m_alphaInRad;
  qreal m_cosAlpha, m_sinAlpha;
  qreal m_cx, m_cy;
  qreal m_sX, m_sY;
  qreal m_invSX, m_invSY;

};

#endif /* ! TRANSFORMATION_HPP */
