/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef WEIGHTEDCHANNELSCOLORSPACEWIDGET_HPP
#define WEIGHTEDCHANNELSCOLORSPACEWIDGET_HPP

#include <vector>

#include <girl/girlCommon.hpp>//GIRL_API

#include <QVector>
#include <QWidget>

class QGridLayout;
class QLabel;

class ColorSpaceChoiceWidget;
class SliderSpinBoxWidget;


class GIRL_API WeightedChannelsColorSpaceWidget : public QWidget
{
  Q_OBJECT

public:

  /**
   * @brief Constructor.
   */
  explicit WeightedChannelsColorSpaceWidget(QWidget *parent = NULL);


  /**
   * Set text for label of widget.
   */
  void setText(const QString &text);


  /**
   * @brief Set color space according to iseConversion !
   */
  void setColorSpace(int cs);

  /**
   * @brief Get color space according to iseConversion !
   */
  int getColorSpace() const;

  /**
   * @brief Get number of channels for selected color space.
   *
   * TODO: should be a function of iseConversion ???
   */
  size_t getNbChannelsForColorSpace() const;

  
  /**
   * @brief Get name of ith channel for selected color space.
   *
   * @warning i must be strictly inferior to getNbChannelsForColorSpace().
   */
  QString getChannelName(size_t i) const;


  /**
   * @brief Get weights affected to all channels of current color space.
   *
   * Each weight is in [0.0; 1.0], thus sum(weights) is in [0.0;  getNbChannelsForColorSpace()]
   */
  void getWeights(std::vector<double> &weights) const;

  /**
   * @brief Set weights affected to all channels of current color space.
   *
   * Each weight is in [0.0; 1.0], thus sum(weights) is in [0.0;  getNbChannelsForColorSpace()]
   */
  void setWeights(const std::vector<double> &weights);

  /**
   * @brief Get weights affected to all channels of current color space normalized by number of channels.
   *
   * Each weight is in [0.0; 1.0/getNbChannelsForColorSpace()], thus sum(weights) is in [0.0; 1.0]
   */
  void getNormalizedWeights(std::vector<double> &weights) const;

  /**
   * @brief Set weights affected to all channels of current color space normalized by number of channels.
   *
   * Each weight is in [0.0; 1.0/getNbChannelsForColorSpace()], thus sum(weights) is in [0.0; 1.0]
   */
  void setNormalizedWeights(const std::vector<double> &weights);


signals:

  void parametersChanged();

protected slots:

  void changeColorSpace(int index);


protected:

  void build();

protected:
  QLabel *m_csl;
  ColorSpaceChoiceWidget *m_cscw;

  QGridLayout *m_channelsLayout;
  QVector<SliderSpinBoxWidget *> m_channelsSpinBoxes;
  QVector<QLabel *> m_channelsLabels;
  QVector<QLabel *> m_labelsPlus;

};


#endif /* ! WEIGHTEDCHANNELSCOLORSPACEWIDGET_HPP */
