/*
  Copyright (C) 2005-2009 Boris Mansencal  boris.mansencal@labri.fr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef WEIGHTEDCHANNELSIMAGEFEATUREWIDGET_HPP
#define WEIGHTEDCHANNELSIMAGEFEATUREWIDGET_HPP

#include <vector>

#include <girl/girlCommon.hpp>//GIRL_API

#include <QVector>
#include <QWidget>
#include "FeatureData.hpp"
#include "FeatureComputerGUIInfo.hpp"

class QGridLayout;
class QLabel;

class SliderSpinBoxWidget;
class ImageFeatureChoiceWidget;
class MainView;

class GIRL_API WeightedChannelsImageFeatureWidget : public QWidget
{
  Q_OBJECT

public:

  /**
   * @brief Constructor.
   */
  WeightedChannelsImageFeatureWidget(MainView *mainView, FeatureComputerGUIInfoPtr, QWidget *parent = NULL);

  /**
   * Set imageFeature uri
   */
  void setImageFeatureUri(const QString &uri);

  void setImageFeatureComputerPtr(const ise::FeatureComputerPtr &data);

  /**
   * Get imageFeature uri
   */
  //QString imageFeatureUri() const;

  ise::FeatureComputerPtr imageFeatureComputerPtr() const;

  /**
   * @brief Get weights affected to all channels of current color space.
   *
   * Each weight is in [0.0; 1.0], thus sum(weights) is in [0.0;  getNbChannelsFor()]
   */
  void weights(std::vector<double> &weights) const;

  /**
   * @brief Set weights affected to all channels of current color space.
   *
   * Each weight is in [0.0; 1.0], thus sum(weights) is in [0.0;  getNbChannelsFor()]
   */
  void setWeights(const std::vector<double> &weights);

  /**
   * @brief Get weights affected to all channels of current color space normalized by number of channels.
   *
   * Each weight is in [0.0; 1.0/nbChannelsFor()], thus sum(weights) is in [0.0; 1.0]
   */
  void normalizedWeights(std::vector<double> &weights) const;

  /**
   * @brief Set weights affected to all channels of current color space normalized by number of channels.
   *
   * Each weight is in [0.0; 1.0/nbChannelsFor()], thus sum(weights) is in [0.0; 1.0]
   */
  void setNormalizedWeights(const std::vector<double> &weights);

  /**
   * Get nbChannel for an imageFeature
   */
  int nbChannelsForImageFeature() const;

  ImageFeatureChoiceWidget *imageFeatureChoiceWidget() const { return m_ifcw; }
  
  void setEnabled(bool b);

signals:

  void parametersChanged();

protected slots:

  void changeImageFeature();


protected:

  void build();

protected:

  ImageFeatureChoiceWidget *m_ifcw;
  
  MainView * m_mainView;
  QGridLayout *m_channelsLayout;
  QVector<SliderSpinBoxWidget *> m_channelsSpinBoxes;
  QVector<QLabel *> m_channelsLabels;
  QVector<QLabel *> m_labelsPlus;

  FeatureComputerGUIInfoPtr m_featureComputerGUIInfo;
};


#endif /* ! WEIGHTEDCHANNELSIMAGEFEATUREWIDGET_HPP */
