/*
 * WeightsSetupWidget.hpp
 *
 *  Created on: Dec 3, 2010
 *      Author: vieux
 */

#ifndef WEIGHTSSETUPWIDGET_HPP_
#define WEIGHTSSETUPWIDGET_HPP_

#include <vector>

#include <girl/girlCommon.hpp>
#include <girl/girlSharedPtr.hpp>

#include <ise/iseRegionSetInterface.hpp>

#include <QList>
#include <QWidget>


class QLabel;
class QPushButton;
class QGridLayout;
class QGroupBox;
class SliderSpinBoxWidget;


/**
 * This widget defines a set of [0, 1] normalized sliders that
 * can be independently set
 */
class GIRL_API WeightsSetupWidget : public QWidget
{

  Q_OBJECT

public:
  /**
   * @param size number of sliders desired
   */
  explicit WeightsSetupWidget(QWidget *parent = NULL);

  virtual ~WeightsSetupWidget();

public:

  void
  setWeights(const std::vector<double>& weights);

  void
  getWeights(std::vector<double>& weights) const;

  void
  setRegionSet(girl::shared_ptr<ise::RegionSetInterface> regSet);


protected slots:

  void setAll0();
  void setAll1();

protected:

  void
  buildDimDescription();

  void
  buildGUI();

  void
  setAll(double v);

private:

  girl::shared_ptr<ise::RegionSetInterface> m_regSet;

  QList<SliderSpinBoxWidget*> m_sliders;
  QList<QLabel*> m_labels;
  QGroupBox* m_groupBox;

  QPushButton* m_zeroButton;
  QPushButton* m_oneButton;

  QGridLayout *m_gridLayout;
};

#endif /* WEIGHTSSETUPWIDGET_HPP_ */
