/*
  Copyright (C) 2013 Boris Mansencal  boris.mansencal@labri.fr
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <cstdio>
#include <cstdlib>
#include <girl/girlTimer.hpp>

#include <iostream>
#include <fstream>

#include <girl/girlToger.hpp>
#include <girlTools/girlToolsImage.hpp>

#include <opencv2/highgui/highgui.hpp>

#include <ise/iseConversion.hpp>
#include <ise/iseImageFeatureColorSpace.hpp>

#include <qiseTools/ImageTools.hpp>

/*
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  TODO: This should be a cpp-unit test !!!

*/


static
void
P_testCS(const cv::Mat &cvimg, girl::tools::Image &img, 
	 int colorSpace)
{
  ise::SegmentationContext segContext(NULL, &img);

  {
    ise::ImageFeatureColorSpace ifcs(&segContext, colorSpace);
    ifcs.compute();

    const girl::shared_ptr<girl::tools::ImageFeature> imgF = ifcs.getImageFeature();


    if (img.channels() != imgF->channels()) {
      std::cerr<<"ERROR: images do not have the same number of channels : "<<img.channels()<<" vs "<<imgF->channels()<<"\n";
      exit(EXIT_FAILURE);
    }

    assert(cvimg.type() == CV_8UC3);

    for (int y=0; y<img.height(); ++y) {
      for (int x=0; x<img.width(); ++x) {

	cv::Vec3b pix0 = cvimg.at<cv::Vec3b>(y, x);
	std::swap(pix0[0], pix0[2]); //BGR to RGB

	int a0[3];
	ise::conversion::changeFromRGB(colorSpace, pix0[0], pix0[1], pix0[2], a0[0], a0[1], a0[2]);


	const girl::tools::PixelFromImage<girl::tools::Image::type> pix1 = img.getPixel(x, y);
	assert(img.channels()>=3);
	int a1[3];
	ise::conversion::changeFromRGB(colorSpace, pix1[0], pix1[1], pix1[2], a1[0], a1[1], a1[2]);
	
	const girl::tools::PixelFromImage<girl::tools::ImageFeature::type> pix2 = imgF->getPixel(x, y);
	assert(imgF->channels() >= 3);
	int a2[3];
	a2[0] = static_cast<int>(pix2[0]);
	a2[1] = static_cast<int>(pix2[1]);
	a2[2] = static_cast<int>(pix2[2]);

	for (int i = 0 ; i < 3 ; ++i) {
	  const int v0 = a0[i];
	  const int v1 = a1[i];
	  const int v2 = a2[i];
	  if (abs(v1-v0) > 0) {
	    std::cerr<<"ERROR: pixel ("<<x<<"; "<<y<<") are different at channel "<<i<<" between cv::Mat & girlToolsImage for colorSpace="<<colorSpace<<"!\n";
	    std::cerr<<(int)pix0[i]<<"->"<<v0<<" vs "<<(int)pix1[i]<<"->"<<v1<<"\n";
	    exit(EXIT_FAILURE);
	  }
	  if (abs(v1-v2) > 0) {
	    std::cerr<<"ERROR: pixel ("<<x<<"; "<<y<<") are different at channel "<<i<<" between girlToolsImage & ImageFeature for colorSpace="<<colorSpace<<" !\n";
	    std::cerr<<(int)pix1[i]<<"->"<<v1<<" vs "<<pix2[i]<<"->"<<v2<<"\n";
	    exit(EXIT_FAILURE);
	  }

	}
	
      }
    }


    

  }

}

int 
main(int argc, char *argv[])
{

  std::string inFilename;
  if (argc >= 2) 
    inFilename = argv[1];
  else {
    std::cerr<<"Usage: "<<argv[0]<<" imageFilename"<<std::endl;
    exit(EXIT_FAILURE);
  }

  cv::Mat cvimg = cv::imread(inFilename);
  if (cvimg.empty()) {
    std::cerr<<"Error: Unable to load image file: "<<inFilename<<" via OpenCV."<<std::endl;
    exit(EXIT_FAILURE);
  }
  
  /* image loading */
  girl::tools::Image img = girl::tools::Image(inFilename);
  if (img.isNull()) {
    std::cerr<<"Error: Unable to load image file : "<<inFilename<<std::endl;
    exit(EXIT_FAILURE);
  }
  
  QImage qimg(inFilename.c_str());
  if (qimg.isNull()) {
    std::cerr<<"Error: Unable to load image file : "<<inFilename<<" via Qt"<<std::endl;
    exit(EXIT_FAILURE);
  }
  girl::tools::Image img2;
  QImage_to_girlToolsImage(qimg, img2);
  


  ise::SegmentationContext segContext(NULL, &img);


  {
    ise::ImageFeatureColorSpace ifcs(&segContext, ISE_RGB_to_RGB);
    ifcs.compute();

    const girl::shared_ptr<girl::tools::ImageFeature> imgF = ifcs.getImageFeature();


    if (img.channels() != imgF->channels()) {
      std::cerr<<"ERROR: images do not have the same number of channels : "<<img.channels()<<" vs "<<imgF->channels()<<"\n";
      exit(EXIT_FAILURE);
    }

    assert(cvimg.type() == CV_8UC3);

    for (int y=0; y<img.height(); ++y) {
      for (int x=0; x<img.width(); ++x) {

	cv::Vec3b pix0 = cvimg.at<cv::Vec3b>(y, x);
	std::swap(pix0[0], pix0[2]); //BGR to RGB

	const girl::tools::PixelFromImage<girl::tools::Image::type> pix1 = img.getPixel(x, y);
	const girl::tools::PixelFromImage<girl::tools::ImageFeature::type> pix2 = imgF->getPixel(x, y);

	QRgb qpix = qimg.pixel(x, y);
	cv::Vec3b pix3;
	pix3[0] = qRed(qpix);
	pix3[1] = qGreen(qpix);
	pix3[2] = qBlue(qpix);

	const girl::tools::PixelFromImage<girl::tools::Image::type> pix4 = img2.getPixel(x, y);


	for (int i = 0 ; i < imgF->channels() ; ++i) {
	  const unsigned char v0 = pix0[i];
	  const unsigned short v1 = static_cast<unsigned short>(pix1[i]);
	  const double v2 = static_cast<double>(pix2[i]);
	  const unsigned char v3 = pix3[i];
	  const unsigned short v4 = static_cast<unsigned short>(pix4[i]);

	  if (abs(v1-v0) > 0) {
	    std::cerr<<"ERROR: pixel ("<<x<<"; "<<y<<") are different at channel "<<i<<" between cv::Mat & girlToolsImage !\n";
	    std::cerr<<(int)v0<<" vs "<<(int)v1<<"\n";
	    exit(EXIT_FAILURE);
	  }
	  if (fabs(v1-v2) > 0.0001) {
	    std::cerr<<"ERROR: pixel ("<<x<<"; "<<y<<") are different at channel "<<i<<" between girlToolsImage & ImageFeature !\n";
	    std::cerr<<(double)v1<<" vs "<<v2<<"\n";
	    exit(EXIT_FAILURE);
	  }
	  if (abs(v3-v0) > 0) {
	    std::cerr<<"ERROR: pixel ("<<x<<"; "<<y<<") are different at channel "<<i<<" between cv::Mat & QImage !\n";
	    std::cerr<<(int)v0<<" vs "<<(int)v3<<"\n";
	    exit(EXIT_FAILURE);
	  }
	  if (abs(v1-v4) > 0) {
	    std::cerr<<"ERROR: pixel ("<<x<<"; "<<y<<") are different at channel "<<i<<" between girlToolsImage & girlToolsImage obtained from QImage!\n";
	    std::cerr<<(int)v1<<" vs "<<(int)v4<<"\n";
	    exit(EXIT_FAILURE);
	  }
	  


	}
	
      }
    }



  }

  P_testCS(cvimg, img, ISE_RGB_to_RGB);
  P_testCS(cvimg, img, ISE_RGB_to_Lab);
  P_testCS(cvimg, img, ISE_RGB_to_Luv);
  
  std::cerr<<"Ok\n";

  return 0;
}
