#!/bin/bash

## -----------------------------------------------------------------
## Change the following variables to better suit your configuration.


#Absolute path/prefix where tools (binaries & libraries) will be installed
INSTALL_PATH=${HOME}/tools

#Path where source must be checkout and compiled
COMPILATION_PATH=.

#OpenCV root directory
#OPENCV_ROOT_DIR=/usr
#OPENCV_ROOT_DIR=${HOME}/tools/share/opencv
OPENCV_ROOT_DIR=${HOME}/tools/share/OpenCV


#Number of compilation threads
#An optimal value is often the number of processors+1
NB_PROCS=1
unamestr=`uname`
case "$unamestr" in
    "Linux")
	NB_PROCS=`grep -c 'model name' /proc/cpuinfo 2>/dev/null`
	;;
    "Darwin")
	NB_PROCS=`sysctl -n hw.ncpu`
	;;
    *)
	echo "Unknown system"
	;;
esac
NB_THREADS=`expr $NB_PROCS + 1`
#NB_THREADS=3 #uncomment this line to force number of compilation threads


#change this to compile in 32 or 64 bits.
EXTRA_FLAGS=""
case "$OSTYPE" in
    "darwin10.0")
#	EXTRA_FLAGS="-DCMAKE_OSX_ARCHITECTURES=x86_64 -DCMAKE_MACOSX_DEPLOYMENT_TARGET=10.6"
#	EXTRA_FLAGS="-DCMAKE_OSX_ARCHITECTURES=i386 -DCMAKE_MACOSX_DEPLOYMENT_TARGET=10.5"
	;;
esac

ON_MACOSX=0
LIB_EXT=".so"
if [[ "$unamestr" == 'Darwin' ]]; then
    LIB_EXT=".dylib"
    ON_MACOSX=1
fi

LIB_DIR="lib"
unamemstr=`uname -m`
case "$unamemstr" in
    "x86_64")
	LIB_DIR="lib64"
	;;
esac



## -----------------------------------------------------------------
## You should not modify this script after this line.

######################################################
######################################################
######################################################
######################################################

SCRIPT_FILENAME=$0

## Check that basic commands are available

check_file()
{
    FILE=$1

    if [ ! -f ${FILE} ]
    then
	echo "Error: ${FILE} not found"
	echo "  edit and correct ${SCRIPT_FILENAME} "
	exit 1
    fi
}
    


check_command()
{
    CMD=$1

    ($CMD --version) >/dev/null 2>&1
    
    RES=$?

    if [ $RES -ne 0 ]
    then
	echo "Error: $CMD not found in \$PATH"
	echo "Maybe the corresponding package is not installed for your system."
	echo ""
    fi
    return $RES
}

MUST_EXIT=0
echo ""
for c in g++ cmake
do 
    
    check_command $c
    let "MUST_EXIT |= $?"
done

if [ ! -d ${INSTALL_PATH} ]
then
    mkdir ${INSTALL_PATH}
fi
if [ ! -d ${INSTALL_PATH} ] || [ ! -w ${INSTALL_PATH} ] || [ ! -x ${INSTALL_PATH} ]
then
    echo "We do not have permissions to install sources in INSTALL_PATH=$INSTALL_PATH"
    let "MUST_EXIT |= 1"
fi



if [ $MUST_EXIT -ne 0 ]
then
    echo "Exiting..."
    exit
fi


######################################################
# Parse script arguments

doClobber=0
doClean=0
doUninstall=0
doConfigure=0
doMake=0
doInstall=0
doBuildComplete=1 #do a complete build except if one of the action is specified.

doAllLibs=1 #doAll except if any of the libs is specified individually
doGirl=0
doGirlTools=0
doIse=0
doQiseTools=0
doPlugins=0
doQise=0

doOptim=0
doDebug=0
doRelWithDebInfo=0
doVerbose=0

processArgs ()
{
    if [ ! $# -gt 0 ]
    then
	#no args : will do everything

        doBuildComplete=1
	doAllLibs=1

    else
	#some args 

	for arg in $@
	do
	    unknownOption="$arg"

	    case "$arg" in
		"configure")
		    doConfigure=1
		    doBuildComplete=0
		    ;;
		"CMake")
		    #idem "configure"
		    doConfigure=1
		    doBuildComplete=0
		    ;;
		"cmake")
		    #idem "configure"
		    doConfigure=1
		    doBuildComplete=0
		    ;;
		"clean")
		    doClean=1 
		    doBuildComplete=0
		    ;;
		"clobber")
		    doClobber=1 
		    doBuildComplete=0
		    ;;
		"make")
		    doMake=1 
		    doBuildComplete=0
		    ;;
		"install")
		    doInstall=1 
		    doBuildComplete=0
		    ;;
		"uninstall")
		    doUninstall=1
		    doBuildComplete=0
		    ;;		    
		"girl")
		    doGirl=1
		    doAllLibs=0
		    ;;
		"girl++")
		    #idem "girl"
		    doGirl=1
		    doAllLibs=0
		    ;;
		"girlTools")
		    doGirlTools=1
		    doAllLibs=0
		    ;;
		"ise")
		    doIse=1
		    doAllLibs=0
		    ;;
		"ise++")
		    #idem "ise"
		    doIse=1
		    doAllLibs=0
		    ;;
		"qiseTools")
		    doQiseTools=1
		    doAllLibs=0
		    ;;
		"qise")
		    doQise=1
		    doAllLibs=0
		    ;;
		"plugins")
		    doPlugins=1
		    doAllLibs=0
		    ;;
		"optim")
		    doOptim=1
		    ;;
		"Release")
		    #idem "optim"
		    doOptim=1
		    ;;
		"RelWithDebInfo")
		    doRelWithDebInfo=1
		    ;;
		"debug")
		    doDebug=1
		    ;;
		"Debug")
		    #idem "debug"
		    doDebug=1
		    ;;
		"verbose")
		    doVerbose=1
		    ;;
		*)
		  echo "Error: unknown option: $unknownOption"
		  exit 11
		  ;;
	    esac
	    
	done

    fi

    if [ "$doAllLibs" -ne 0 ]
    then
	doGirl=1
	doGirlTools=1
	doIse=1
	doQiseTools=1
	doQise=1
	doPlugins=1
    fi    

    if 	[ "$doBuildComplete" -ne 0 ]
    then
	doUninstall=1
	doConfigure=1
	#doClean=1
	doMake=1
	doInstall=1
    fi


    ## check that optim & debug are not specified at the same time                     
    if [ "$doOptim" -ne 0 ] ; then
        if [ "$doDebug" -ne 0 ] ; then
            echo "Error: cannot have both optim & debug !"
            exit 9
        fi
    fi
    #TODO: check Optim & RelWithDebInfo
    #TODO: check Debug & RelWithDebInfo

    OPTIM_FLAGS=""
    if [ "$doOptim" -ne 0 ] ; then
	OPTIM_FLAGS="-DCMAKE_BUILD_TYPE=Release"
	doConfigure=1 #in case of build type change
    fi
    if [ "$doRelWithDebInfo" -ne 0 ] ; then
	OPTIM_FLAGS="-DCMAKE_BUILD_TYPE=RelWithDebInfo"
	doConfigure=1 #in case of build type change
    fi
    if [ "$doDebug" -ne 0 ] ; then
	OPTIM_FLAGS="-DCMAKE_BUILD_TYPE=Debug"
	doConfigure=1 #in case of build type change
    fi


   # if both configure & (make or install) are specified, we also do clean            
    if [ "$doConfigure" -ne 0 ] ; then
        if [ "$doMake" -ne 0 ] ; then
            doClean=1
        fi
        if [ "$doInstall" -ne 0 ] ; then
            doClean=1
        fi
    fi

   #if clean then uninstall
   # if [ "$doClean" -ne 0 ] ; then
   #	doUninstall=1
   # fi
    

}


processArgs $@

######################################################

## Compile project

cd ${COMPILATION_PATH}

export PATH=${INSTALL_PATH}/bin:$PATH
export LD_LIBRARY_PATH=${INSTALL_PATH}/lib:$LD_LIBRARY_PATH
export PKG_CONFIG_PATH=${INSTALL_PATH}/lib/pkgconfig:${PKG_CONFIG_PATH}


VERBOSEFLAG=""
if [ $doVerbose -ne 0 ] ; then
    VERBOSEFLAG="VERBOSE=1"
fi




#compilations
#------------

BUILD_DIR="build"




compile()
{
    #TODO: check number of arguments ?
    #TODO: check first argument is "lib" or "app"

    kind="$1" 
    dir="$2"
    shift 2
    deps="$*"

    echo "### ${dir} ###"

    CMD="cd ${dir}"
    ${CMD}
    RES=$? 
    if [ $RES -ne 0 ]
    then
	echo "Error: unable to execute \"${CMD}\""
	return $RES
    fi

    if [ $doInstall -ne 0 ] ; then
	if [ -d ${BUILD_DIR} ] ; then
	    cd ${BUILD_DIR}
	    RES=$? 
	    if [ $RES -eq 0 ] ; then
         	#If "install" specified, we do a "make uninstall" before any update/configure
		CMD="make uninstall"
		${CMD} >/dev/null 2>/dev/null
		cd ..
	    fi
	fi
    fi

	

    if [ $doClobber -ne 0 ] ; then
	CMD="rm -rf ${BUILD_DIR}"
	${CMD}
    fi

    mkdir -p ${BUILD_DIR}
    CMD="cd ${BUILD_DIR}"
    ${CMD}
    RES=$? 
    if [ $RES -ne 0 ]
    then
	echo "Error: unable to execute \"${CMD}\""
	return $RES
    fi
    
    CMD="cmake .. ${EXTRA_FLAGS} ${deps}"
    if [ $doConfigure -ne 0 ] ; then
	echo "PWD=`pwd`"
	echo "${CMD}"
	${CMD}
	RES=$? 
	if [ $RES -ne 0 ]
	then
	    echo "Error: unable to execute \"${CMD}\""
	    return $RES
	fi
    fi


    if [ $doClean -ne 0 ] ; then
	if [ ! -f ./Makefile ] ; then
	    echo "Error: Makefile not created"
	    return 1
	fi
	CMD="make clean "
	echo "**** $CMD"
	if [ $doVerbose -ne 0 ] ; then
	    ${CMD} 
	else
	    ${CMD} >/dev/null	    
	fi
	RES=$? 
	if [ $RES -ne 0 ]
	then
	    echo "Error: unable to execute \"${CMD}\""
	    return $RES
	fi
    fi

    if [ $doUninstall -ne 0 ] ; then
	if [ ! -f ./Makefile ] ; then
	    echo "Error: Makefile not created"
	    return 1
	fi
	CMD="make uninstall"
	echo "**** $CMD"
	if [ $doVerbose -ne 0 ] ; then
	    ${CMD} 2>/dev/null
	else
	    ${CMD} 2>/dev/null >/dev/null	    
	fi
	RES=$? 
	if [ $RES -ne 0 ]
	then
	#    echo "Error: unable to execute \"${CMD}\""
	#    return $RES
	    echo "Warning: unable to execute \"${CMD}\""
	    ### We do only a warning because if there was no "make install" previously, "make uninstall" will fail.
	    ### this is also why we redirect standard error output to /dev/null
	fi
    fi


    if [ $doMake -ne 0 ] ; then
	if [ ! -f ./Makefile ] ; then
	    echo "Error: Makefile not created"
	    return 1
	fi
	CMD="make -j ${NB_THREADS} ${VERBOSEFLAG}"
	echo "**** $CMD"
	if [ $doVerbose -ne 0 ] ; then
	    ${CMD} 
	else
	    ${CMD} >/dev/null	    
	fi
	RES=$? 
	if [ $RES -ne 0 ]
	then
	    echo "Error: unable to execute \"${CMD}\""
	    return $RES
	fi
    fi

    if [ $doInstall -ne 0 ] ; then
	if [ ! -f ./Makefile ] ; then
	    echo "Error: Makefile not created"
	    return 1
	fi
	if [ "${ON_MACOSX}" -eq 0 ] || [ "${kind}" = "lib" ] ; then #do not install programs on MacOSX as we build bundles
	    CMD="make install -j ${NB_THREADS} ${VERBOSEFLAG}"
	    echo "**** $CMD"
	    if [ $doVerbose -ne 0 ] ; then
		${CMD} 
	    else
		${CMD} >/dev/null	    
	    fi
	    RES=$? 
	    if [ $RES -ne 0 ]
	    then
		echo "Error: unable to execute \"${CMD}\""
		return $RES
	    fi
	else
	    # programs on MacOSX are built as bundles
	    cp -r src/${dir}.app ${INSTALL_PATH}/bin
	fi
    fi

    CMD="cd ../.."
    ${CMD}
    RES=$? 
    if [ $RES -ne 0 ]
    then
	echo "Error: unable to execute \"${CMD}\""
	return $RES
    fi
    return 0
}


COMPILATION_PATH=`pwd`  #get absolute path


if [ $doGirl -ne 0 ] ; then
    compile lib girl++ -DCMAKE_INSTALL_PREFIX=${INSTALL_PATH} ${OPTIM_FLAGS}
    RES=$? 
    if [ $RES -ne 0 ]
    then
	echo "Exiting..."
	exit $RES
    fi
fi

GIRL_FIND_PATH=${COMPILATION_PATH}/girl++/cmake/Modules
GIRLTOOLS_FLAGS="-DGIRL_INCLUDE_DIR=${INSTALL_PATH}/include/girl-2.0 -DGIRL_LIBRARY=${INSTALL_PATH}/lib/libgirl-2.0${LIB_EXT} -DOpenCV_DIR=${OPENCV_ROOT_DIR}"

if [ $doGirlTools -ne 0 ] ; then
    compile lib girlTools -DCMAKE_INSTALL_PREFIX=${INSTALL_PATH} -DCMAKE_MODULE_PATH=${GIRL_FIND_PATH} ${GIRLTOOLS_FLAGS} ${OPTIM_FLAGS}
    RES=$? 
    if [ $RES -ne 0 ]
    then
	echo "Exiting..."
	exit $RES
    fi
fi

GIRLTOOLS_FIND_PATH=${COMPILATION_PATH}/girlTools/cmake/Modules
ISE_FLAGS="${GIRLTOOLS_FLAGS} -DGIRLTOOLS_INCLUDE_DIR=${INSTALL_PATH}/include/girlTools-1.0 -DGIRLTOOLS_LIBRARY=${INSTALL_PATH}/lib/libgirlTools-1.0${LIB_EXT}"
if [ $doIse -ne 0 ] ; then
    compile lib "ise++" -DCMAKE_INSTALL_PREFIX=${INSTALL_PATH} -DCMAKE_MODULE_PATH=${GIRL_FIND_PATH}\;${GIRLTOOLS_FIND_PATH} ${ISE_FLAGS}  ${OPTIM_FLAGS}
    RES=$? 
    if [ $RES -ne 0 ]
    then
	echo "Exiting..."
	exit $RES
    fi
fi

ISE_FIND_PATH=${COMPILATION_PATH}/ise++/cmake/Modules
QISETOOLS_FLAGS="${ISE_FLAGS} -DISE_INCLUDE_DIR=${INSTALL_PATH}/include/ise-0.2 -DISE_LIBRARY=${INSTALL_PATH}/lib/libise-0.2${LIB_EXT}"
if [ $doQiseTools -ne 0 ] ; then
    compile lib qiseTools -DCMAKE_INSTALL_PREFIX=${INSTALL_PATH} -DCMAKE_MODULE_PATH=${GIRL_FIND_PATH}\;${GIRLTOOLS_FIND_PATH}\;${ISE_FIND_PATH} ${QISETOOLS_FLAGS} ${OPTIM_FLAGS}
    RES=$? 
    if [ $RES -ne 0 ]
    then
	echo "Exiting..."
	exit $RES
    fi
fi

PLUGINS_INSTALL_DIR="${COMPILATION_PATH}/qise/${BUILD_DIR}/src/plugins"
if [ "${ON_MACOSX}" -eq 1 ] ; then
    PLUGINS_INSTALL_DIR="${COMPILATION_PATH}/qise/${BUILD_DIR}/src/qise.app/Contents/MacOS/plugins"
fi


QISETOOLS_FIND_PATH=${COMPILATION_PATH}/qiseTools/cmake/Modules
QISE_FLAGS="${QISETOOLS_FLAGS} -DQISETOOLS_INCLUDE_DIR=${INSTALL_PATH}/include/qiseTools-1.0 -DQISETOOLS_LIBRARY=${INSTALL_PATH}/lib/libqiseTools-1.0${LIB_EXT}"
if [ $doPlugins -ne 0 ] ; then
    compile lib plugins -DCMAKE_MODULE_PATH=${GIRL_FIND_PATH}\;${GIRLTOOLS_FIND_PATH}\;${ISE_FIND_PATH}\;${QISETOOLS_FIND_PATH} ${QISE_FLAGS}  -DCMAKE_INSTALL_PREFIX="${PLUGINS_INSTALL_DIR}"  ${OPTIM_FLAGS}
    RES=$? 
    if [ $RES -ne 0 ]
    then
	echo "Exiting..."
	exit $RES
    fi
fi

if [ $doQise -ne 0 ] ; then
    compile app qise -DCMAKE_INSTALL_PREFIX=${INSTALL_PATH} -DCMAKE_MODULE_PATH=${GIRL_FIND_PATH}\;${GIRLTOOLS_FIND_PATH}\;${ISE_FIND_PATH}\;${QISETOOLS_FIND_PATH} ${QISE_FLAGS} ${OPTIM_FLAGS}
    RES=$? 
    if [ $RES -ne 0 ]
    then
	echo "Exiting..."
	exit $RES
    fi
fi






